Package: summata
Type: Package
Title: Publication-Ready Summary Tables and Forest Plots
Version: 0.12.0
Authors@R: c(
    person(
        given = c("Paul", "Hsin-ti"), 
        family = "McClelland", 
        role = c("aut", "cre", "cph"), 
        email = "PaulHMcClelland@protonmail.com",
        comment = c(ORCID = "0000-0002-3119-6531")
    ))
Description: A comprehensive framework for descriptive statistics and regression
    analysis that produces publication-ready tables and forest plots. Provides a
    unified interface from descriptive statistics through multivariable modeling,
    with support for linear models, generalized linear models, Cox proportional
    hazards, and mixed-effects models. Also includes univariable screening, 
    multivariate regression, model comparison, and export to multiple formats
    including PDF, DOCX, PPTX, 'LaTeX', HTML, and RTF. Built on 'data.table'
    for computational efficiency.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
URL: https://phmcc.codeberg.page/summata/,
        https://codeberg.org/phmcc/summata,
        https://github.com/phmcc/summata
BugReports: https://github.com/phmcc/summata/issues
Depends: R (>= 4.2.0)
Imports: data.table, survival, ggplot2, stats, grDevices, parallel,
        tools, utils
Suggests: MASS, coxme, flextable, grid, knitr, lme4, MuMIn, officer,
        pROC, ragg, ResourceSelection, reformulas, rmarkdown, stringr,
        systemfonts, testthat, tinytex, withr, xtable
SystemRequirements: All optional. A 'LaTeX' distribution providing
        pdflatex or xelatex, and kpsewhich, for PDF output from
        table2pdf(); pdfcrop for auto-sized PDF output; Ghostscript for
        the optional font embedding in forestsave(). Each feature
        reports the absence of its tool and the remainder of the
        package is unaffected.
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-21 00:23:17 UTC; paul
Author: Paul Hsin-ti McClelland [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3119-6531>)
Maintainer: Paul Hsin-ti McClelland <PaulHMcClelland@protonmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 06:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(print,compfit_result)
S3method(print,fit_result)
S3method(print,fullfit_result)
S3method(print,multifit_result)
S3method(print,survtable)
S3method(print,table2docx_result)
S3method(print,table2pptx_result)
S3method(print,table2rtf_result)
S3method(print,uniscreen_result)
export(autoforest)
export(autotable)
export(compfit)
export(coxforest)
export(desctable)
export(fit)
export(fullfit)
export(glmforest)
export(lmforest)
export(m2dt)
export(multifit)
export(multiforest)
export(survtable)
export(table2docx)
export(table2html)
export(table2pdf)
export(table2pptx)
export(table2rtf)
export(table2tex)
export(uniforest)
export(uniscreen)
import(data.table)
importFrom(grDevices,axisTicks)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,IQR)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,cor)
importFrom(stats,family)
importFrom(stats,fisher.test)
importFrom(stats,fitted)
importFrom(stats,fivenum)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,wilcox.test)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,str)
importFrom(utils,tail)

# <span class="pkg-name">summata</span> <a href="https://phmcc.codeberg.page/summata/"><img src="man/figures/summata.png" align="right" height="139" alt="summata website" /></a>

<!-- badges: start -->
[![R-CMD-check](https://github.com/phmcc/summata/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/phmcc/summata/actions/workflows/R-CMD-check.yaml)
[![test-coverage](https://github.com/phmcc/summata/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/phmcc/summata/actions/workflows/test-coverage.yaml)
[![lint](https://github.com/phmcc/summata/actions/workflows/lint.yaml/badge.svg)](https://github.com/phmcc/summata/actions/workflows/lint.yaml)
[![CRAN status](https://www.r-pkg.org/badges/version/summata)](https://CRAN.R-project.org/package=summata)
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
<!-- badges: end -->

> ***summata*** | /suːˈmɑːtə/ | *Latin, n. pl. of* summātum*, perfect passive participle of* summāre*: those that have been summarized*
>
> Complete, publication-ready statistical summaries.

## Overview

The `summata` package provides a comprehensive framework for generating summary tables and visualizations from statistical analyses. Built on `data.table` for computational efficiency, it streamlines the workflow from descriptive statistics, through regression modeling, to final output—all using a unified interface with standardized, presentation-ready results.

For a more comprehensive description of this package and its features, see the [full documentation and vignettes](https://phmcc.codeberg.page/summata/).

<img src="man/figures/README_hero.png" alt="Cox regression forest plot" width="100%">

## Installation

The stable release of this package can be installed from CRAN.

```r
install.packages("summata")
```

Alternatively, install it directly from GitHub (stable) or Codeberg (development):

```r
# Stable release
devtools::install_github("phmcc/summata")

# Development version
devtools::install_git("https://codeberg.org/phmcc/summata.git")
```

## Package Composition

### Design Principles

The architecture of `summata` reflects three guiding principles:

1. **Consistent syntax.** All modeling functions share a common signature: data first, followed by the outcome or variable of interest, then constituent/modifying variables. This convention facilitates both pipe-based workflows and pedagogical clarity.

2. **Transparent computation.** Functions attach their underlying model objects and raw numerical results as attributes, permitting verification of computations or extension of analyses beyond the formatted output.
   
3. **Separation of concerns.** Analysis, formatting, and export constitute distinct operations, allowing each stage to be modified independently.

These principles manifest in the standard calling convention:

``` r
result <- fit(data, variable, c("var1", "var2", ..., "varN"), ...)
```

where `data` is the dataset, `variable` is the variable of interest (dependent variable, endpoint/outcome, stratification/grouping variable, *etc.*), and `c("var1", "var2", ..., "varN")`is a vector of constituent/modifying variables (independent variables, model covariates, compared/explanatory factors, *etc.*). The result is a formatted table for export, with readily accessible attributes for further analysis.

``` r
# The formatted table
print(result)

# The underlying model object
model <- attr(result, "model")

# Raw numerical results
raw <- attr(result, "raw_data")
```

### Functional Reference

This package provides a variety of functions for different stages of statistical analysis:

#### Descriptive analysis

Tables to provide quick summary data and comparison tests between different groups.

| Function | Purpose |
|:---------|:--------|
| `desctable()` | Descriptive statistics with stratification and hypothesis testing |
| `survtable()` | Survival probability estimates at specified time points |

#### Predictive analysis

Fitted univariable and multivariable regression results for predictive modeling.

| Function | Purpose |
|:---------|:--------|
| `uniscreen()` | Systematic univariable analysis across multiple predictors |
| `fit()` | Single regression model with formatted coefficient extraction |
| `fullfit()` | Integrated univariable screening with multivariable regression |
| `compfit()` | Nested model comparison with composite scoring |
| `multifit()` | Multivariate regression analysis with a single predictor evaluated against multiple outcomes |

#### Table export

Export of finalized tables to various commonly used formats.

| Function | Format | Dependencies |
|:---------|:-------|:-----------|
| `tablesave()` | Format from file extension | Varies |
| `table2pdf()` | PDF | `xtable`, LaTeX distribution |
| `table2tex()` | LaTeX source | `xtable` |
| `table2html()` | HTML | `xtable` |
| `table2docx()` | Microsoft Word | `flextable`, `officer` |
| `table2pptx()` | Microsoft PowerPoint | `flextable`, `officer` |
| `table2rtf()` | Rich Text Format | `flextable`, `officer` |

#### Data visualization

Generation of publication-ready forest plot graphics to summarize regression models.

| Function | Application |
|:---------|:------------|
| `autoforest()` | Automatic model class detection |
| `lmforest()` | Linear models |
| `glmforest()` | Generalized linear models |
| `coxforest()` | Proportional hazards models |
| `uniforest()` | Univariable screening results |
| `multiforest()` | Multivariate regression analysis results |
| `forestsave()` | Save plots with recommended dimensions |
| `recdims()` | Recommended figure dimensions |

### Supported Model Classes

The following regression models are currently supported by `summata`. Specify the model using the `model_type` parameter in the appropriate regression function (`uniscreen()`, `fit()`, `fullfit()`, `compfit()`, or `multifit()`):

| Model Class | `model_type` | Function | Effect Measure |
|:------------|:----------|:---------|:---------------|
| Linear regression | `lm` | `stats::lm()` | *β* coefficient |
| Logistic regression | `glm`, `family = "binomial"` | `stats::glm()` | Odds ratio |
| Logistic (overdispersed) | `glm`, `family = "quasibinomial"` | `stats::glm()` | Odds ratio |
| Poisson regression | `glm`, `family = "poisson"` | `stats::glm()` | Rate ratio |
| Poisson (overdispersed) | `glm`, `family = "quasipoisson"` | `stats::glm()` | Rate ratio |
| Gaussian (via GLM) | `glm`, `family = "gaussian"` | `stats::glm()` | *β* coefficient |
| Gamma regression | `glm`, `family = "Gamma"` | `stats::glm()` | Ratio* |
| Inverse Gaussian | `glm`, `family = "inverse.gaussian"` | `stats::glm()` | Ratio* |
| Cox proportional hazards | `coxph` | `survival::coxph()` | Hazard ratio |
| Conditional logistic | `clogit` | `survival::clogit()` | Odds ratio |
| Negative binomial | `negbin` | `MASS::glm.nb()` | Rate ratio |
| Linear mixed effects | `lmer` | `lme4::lmer()` | *β* coefficient |
| Generalized linear mixed effects | `glmer` | `lme4::glmer()` | Odds/rate ratio |
| Cox mixed effects | `coxme` | `coxme::coxme()` | Hazard ratio |

*with log link; coefficient with identity link

## Comparison with Related Packages

The R ecosystem includes several established packages for regression table generation. The following comparison identifies areas of overlap and distinction:

| Capability | summata | gtsummary | finalfit | arsenal |
|:-----------|:-------:|:---------:|:--------:|:-------:|
| Descriptive statistics | ✓ | ✓ | ✓ | ✓ |
| Survival summaries | ✓ | ✓ | ◐ | — |
| Univariable screening | ✓ | ✓ | ✓ | ✓ |
| Multivariable regression | ✓ | ✓ | ✓ | ✓ |
| Multi-format export | ✓ | ✓ | ✓ | ✓ |
| Integrated forest plots | ✓ | ◐ | ✓ | — |
| Model comparison | ✓ | ✓ | ◐ | — |
| Mixed-effects models | ✓ | ◐ | ◐ | — |
| Multivariate regression analysis | ✓ | — | — | — |

<sub>✓ Full support | ◐ Partial support | — Not available</sub>

A detailed feature comparison is available in the [package documentation](https://phmcc.codeberg.page/summata/articles/feature_comparison.html).

## Illustrative Example

The `clintrial` dataset included with this package provides simulated clinical trial data comprising patient identifiers, baseline characteristics, therapeutic interventions, short-term outcomes, and long-term survival statistics. The following example demonstrates how `summata` functions can be used to analyze perioperative factors affecting 30-day hospital readmission.

### **Step 0:** Data Preparation

Prior to analysis, load the dataset, apply labels, and define predictors:

``` r
library(summata)

# Load example data
data("clintrial")
data("clintrial_labels")

# Define candidate predictors
predictors <- c("age", "sex", "race", "ethnicity", "bmi", "smoking",
                "hypertension", "diabetes", "ecog", "creatinine",
                "hemoglobin", "biomarker_x", "biomarker_y", "grade",
                "stage", "treatment", "surgery")
```

### **Step 1:** Descriptive Statistics

Use the `desctable()` function to generate summary statistics with stratification by a grouping variable (in this case, 30-day readmission):

``` r
table1 <- desctable(
    data = clintrial,
    by = "readmission_30d",
    variables = predictors,
    labels = clintrial_labels
)

tablesave(table1, "table1.pdf",
          caption = "\\textbf{Table 1} - Baseline characteristics by 30-day readmission status",
          paper = "auto",
          condense_table = TRUE,
          dark_header = TRUE,
          zebra_stripes = TRUE
)
```

<img src="man/figures/README_desctable.png" alt="Descriptive statistics table" width="100%">

### **Step 2:** Regression Analysis

Perform an integrated univariable-to-multivariable regression workflow using the `fullfit()` function:

``` r
table2 <- fullfit(
    data = clintrial,
    outcome = "readmission_30d",
    predictors = predictors,
    method = "screen",
    p_threshold = 0.05,
    model_type = "glm",
    labels = clintrial_labels
)

tablesave(table2, "table2.pdf",
          caption = "\\textbf{Table 2} - Predictors of 30-day readmission, multivariable Cox regression with univariable screen",
          paper = "auto",
          condense_table = TRUE,
          dark_header = TRUE,
          zebra_stripes = TRUE
)
```

<img src="man/figures/README_fullfit.png" alt="Regression table" width="100%">

### **Step 3:** Forest Plot

Finally, generate a forest plot to provide a graphical representation of effect estimates using the `glmforest()` function:

``` r
forest_30d <- glmforest(table2,
                        title = "Factors Affecting 30-Day Readmission",
                        labels = clintrial_labels,
                        indent_groups = TRUE
                        )

forestsave(forest_30d, "forest_30d.pdf")
```

<img src="man/figures/README_glmforest.png" alt="Forest plot" width="100%">

## Development

### Repository

- **Primary development**: [codeberg.org/phmcc/summata](https://codeberg.org/phmcc/summata)
- **GitHub releases**: [github.com/phmcc/summata](https://github.com/phmcc/summata)

### Contributing

Bug reports and feature requests may be submitted via the issue tracker ([Codeberg](https://codeberg.org/phmcc/summata/issues) or [Github](https://github.com/phmcc/summata/issues)). Contributions are welcome; please consult the contributing guidelines prior to submitting pull requests.

## Acknowledgments

The design of `summata` draws inspiration from several existing packages:

- **finalfit** (Harrison) — Regression workflow concepts  
- **gtsummary** (Sjoberg et al.) — Table generation architecture  
- **arsenal** (Heinzen et al.) — Descriptive statistics methodology  
- **data.table** (Dowle & Srinivasan) — High-performance data operations

## License

GPL (>= 3.0)

## Citation

``` r
citation("summata")

To cite summata in publications, use:

  McClelland PH (2026). _summata: Publication-Ready Summary Tables and Forest Plots_. R package version 0.11.5, <https://phmcc.codeberg.page/summata/>.

A BibTeX entry for LaTeX users is

  @Manual{,
    title = {summata: Publication-Ready Summary Tables and Forest Plots},
    author = {Paul Hsin-ti McClelland},
    year = {2026},
    note = {R package version 0.12.0},
    url = {https://phmcc.codeberg.page/summata/},
  }
```

## Further Resources

- **Function documentation**: `?function_name` or the [reference index](https://phmcc.codeberg.page/summata/reference/index.html)
- **Vignettes**: `vignette("summata")` or [online articles](https://phmcc.codeberg.page/summata/articles/index.html)
- **Issue tracker**: [Codeberg Issues](https://codeberg.org/phmcc/summata/issues), [GitHub Issues](https://github.com/phmcc/summata/issues)

---

<sub>The `summata` package is under active development. Any breaking changes to the API will be reported in the changelog.</sub>

#' Recommended Figure Dimensions
#'
#' Returns the figure dimensions recommended for a forest plot, computed from
#' the plot's own content by the function that produced it.
#'
#' @param x A forest plot produced by \code{lmforest()}, \code{glmforest()},
#'   \code{coxforest()}, \code{uniforest()}, \code{multiforest()}, or
#'   \code{autoforest()}.
#' @param units Character string giving the units the dimensions are returned
#'   in: \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Defaults to the
#'   units the plot was created in, so that the dimensions pass through
#'   unconverted unless a change is requested.
#'
#' @return A named numeric vector with elements \code{width} and \code{height},
#'   expressed in \code{units}. The units are recorded on the result as a
#'   \code{"units"} attribute, so that a value carried between functions
#'   remains self-describing.
#'
#' @details
#' Forest plots are sized by their content rather than by the page: a plot with
#' more variables, longer labels, or more factor levels requires a larger
#' canvas. Each plotting function computes a suitable size and attaches it to
#' the object it returns, and \code{recdims()} reports it.
#'
#' The values returned are those \code{forestsave()} applies, and are reported
#' exactly as computed rather than rounded, so that a figure written with
#' \code{forestsave()} and one written with \code{ggplot2::ggsave()} at these
#' dimensions are identical. Explicit use is therefore needed only where the
#' dimensions themselves are wanted, as when sizing a \pkg{knitr} chunk or
#' arranging several plots on one page.
#'
#' Where \code{units} differs from the units the plot was created in, the
#' dimensions are converted. A plot produced before the units were recorded
#' carries none, and is treated as having been created in inches.
#'
#' @seealso \code{\link{forestsave}} for saving at these dimensions,
#'   \code{\link{autoforest}} for producing the plot
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' model <- stats::glm(readmission_30d ~ age + sex + stage,
#'                     data = clintrial, family = stats::binomial)
#'
#' p <- glmforest(model, data = clintrial, labels = clintrial_labels)
#'
#' # Example 1: Dimensions in the units the plot was created in
#' recdims(p)
#'
#' # Example 2: Journals commonly specify figure widths in millimeters
#' recdims(p, units = "mm")
#'
#' # Example 3: Sizing an output canvas from the individual elements
#' dims <- recdims(p)
#' dims[["width"]]
#' dims[["height"]]
#' attr(dims, "units")
#'
#' @family visualization functions
#' @export
recdims <- function(x, units = NULL) {

    if (!inherits(x, "ggplot")) {
        stop("'x' must be a forest plot produced by one of the forest plot ",
             "functions.", call. = FALSE)
    }

    dims <- attr(x, "rec_dims")

    if (is.null(dims) || is.null(dims$width) || is.null(dims$height)) {
        stop("'x' carries no recommended dimensions. These are attached by ",
             "the forest plot functions, so a plot from another source has ",
             "none.", call. = FALSE)
    }

    valid_units <- c("in", "cm", "mm", "px", "pt")

    ## Plots produced before the units were recorded carry none, and the
    ## dimensions were expressed in inches at that time
    from_units <- if (!is.null(dims$units) && dims$units %in% valid_units) {
                      dims$units
                  } else {
                      "in"
                  }

    ## Output units default to those of the recommendation, so the dimensions
    ## pass through unconverted in the common case
    if (is.null(units)) {
        units <- from_units
    }

    if (!is.character(units) || length(units) != 1 ||
        !units %in% valid_units) {
        stop("'units' must be one of: ",
             paste(sQuote(valid_units), collapse = ", "), ".",
             call. = FALSE)
    }

    result <- c(
        width  = unname(convert_units(dims$width,
                                      from = from_units, to = units)),
        height = unname(convert_units(dims$height,
                                      from = from_units, to = units))
    )

    ## Recorded so that a value carried into a saving function cannot be
    ## mis-sized by the conversion factor
    attr(result, "units") <- units

    return(result)
}

## recdims() reports the dimensions the plotting function computed, and must
## agree with what forestsave() applies. A discrepancy between the two would
## mean a figure sized one way and reported another.

recdims_fixture <- function(units = "in") {
    dt <- data.table::as.data.table(fresh_clintrial())
    model <- stats::glm(readmission_30d ~ age + sex + stage,
                        data = dt, family = stats::binomial)
    suppressMessages(suppressWarnings(
        glmforest(model, data = dt, units = units)))
}


test_that("recdims reports the attached dimensions unchanged", {

    p <- recdims_fixture()
    dims <- attr(p, "rec_dims")

    result <- recdims(p)

    expect_named(result, c("width", "height"))
    expect_equal(unname(result[["width"]]), dims$width)
    expect_equal(unname(result[["height"]]), dims$height)
    expect_identical(attr(result, "units"), dims$units)
})


test_that("recdims converts where the requested units differ", {

    p <- recdims_fixture(units = "in")
    inches <- recdims(p)

    mm <- recdims(p, units = "mm")

    expect_identical(attr(mm, "units"), "mm")
    expect_equal(unname(mm[["width"]]),
                 unname(inches[["width"]]) * 25.4,
                 tolerance = 1e-6)

    ## A round trip returns the original values
    back <- recdims(p, units = "in")
    expect_equal(unname(back[["width"]]), unname(inches[["width"]]))
})


test_that("recdims defaults to the units the plot was created in", {

    p_cm <- recdims_fixture(units = "cm")

    expect_identical(attr(recdims(p_cm), "units"), "cm")

    ## Requesting inches from a centimetre plot converts rather than relabels
    in_dims <- recdims(p_cm, units = "in")
    expect_equal(unname(in_dims[["width"]]),
                 unname(recdims(p_cm)[["width"]]) / 2.54,
                 tolerance = 1e-6)
})


test_that("recdims and forestsave agree on the dimensions used", {

    p <- recdims_fixture()
    dims <- recdims(p)

    out <- tempfile(fileext = ".pdf")

    ## forestsave() reports the dimensions it applies, which must be the
    ## figures recdims() returns
    expect_message(forestsave(p, out),
                   sprintf("width = %.1f", dims[["width"]]))
})


test_that("recdims validates its arguments", {

    p <- recdims_fixture()

    expect_error(recdims("not a plot"), "must be a forest plot")
    expect_error(recdims(p, units = "furlongs"), "'units' must be one of")

    ## A plot from another source carries no recommendation
    plain <- ggplot2::ggplot(data.frame(x = 1, y = 1), ggplot2::aes(x, y)) +
        ggplot2::geom_point()
    expect_error(recdims(plain), "no recommended dimensions")
})

### * Global setting

#' Number Formatting Utilities
#'
#' Internal utilities for locale-aware number formatting across all summata
#' output functions. Supports preset locales (US, European, SI/ISO, plain) and
#' fully custom separator definitions.
#'
#' @section Global Option:
#' The default number format can be set once per session:
#' \preformatted{
#'   options(summata.number_format = "eu")
#' }
#' This avoids passing \code{number_format} to every function call.
#'
#' @name number_format
#' @keywords internal
NULL


### * Format validation

#' Validate number_format parameter
#'
#' Checks that a \code{number_format} value is valid before use. Called early
#' in top-level functions to fail fast with a clear error message.
#'
#' @param number_format Value to validate.
#' @return Invisibly returns \code{TRUE} if valid.
#' @keywords internal
validate_number_format <- function(number_format) {
    if (is.null(number_format)) return(invisible(TRUE))

    if (!is.character(number_format)) {
        stop("'number_format' must be a character string or character vector.",
             call. = FALSE)
    }

    if (length(number_format) == 1L) {
        valid_presets <- c("us", "eu", "space", "none")
        if (!number_format %in% valid_presets) {
            stop("Unknown number_format preset: '", number_format,
                 "'. Valid presets are: ",
                 paste(paste0("'", valid_presets, "'"), collapse = ", "),
                 ". Or use a two-element vector c(big.mark, decimal.mark).",
                 call. = FALSE)
        }
    } else if (length(number_format) == 2L) {
        if (number_format[1] == number_format[2] && nchar(number_format[1]) > 0) {
            stop("big.mark and decimal.mark cannot be the same non-empty character.",
                 call. = FALSE)
        }
    } else {
        stop("Custom 'number_format' must be a two-element character vector ",
             "c(big.mark, decimal.mark).", call. = FALSE)
    }

    invisible(TRUE)
}


### * Marks formatting

#' Resolve number format marks
#'
#' Converts a \code{number_format} specification into a list of \code{big.mark}
#' and \code{decimal.mark} values used by all downstream formatting functions.
#' Supports named presets, custom two-element vectors, and the global
#' \code{summata.number_format} option.
#'
#' @param number_format Character string specifying a named preset, a
#'   two-element character vector \code{c(big.mark, decimal.mark)}, or
#'   \code{NULL} to use the global option (falling back to \code{"us"}).
#'
#'   Named presets:
#'   \describe{
#'     \item{\code{"us"}}{Comma thousands, period decimal: 1,234.56}
#'     \item{\code{"eu"}}{Period thousands, comma decimal: 1.234,56}
#'     \item{\code{"space"}}{Thin-space thousands, period decimal: 1 234.56
#'       (SI/ISO 31-0 standard)}
#'     \item{\code{"none"}}{No thousands separator, period decimal: 1234.56}
#'   }
#'
#'   Custom vector: \code{c(",", ".")} or \code{c(".", ",")} \emph{etc.} The first
#'   element is \code{big.mark}, the second is \code{decimal.mark}.
#'
#' @return A list with components:
#'   \describe{
#'     \item{\code{big.mark}}{Character string for thousands separator.}
#'     \item{\code{decimal.mark}}{Character string for decimal separator.}
#'   }
#' 
#' @keywords internal
resolve_number_marks <- function(number_format = NULL) {

    ## Fall back to global option, then to "us"
    if (is.null(number_format)) {
        number_format <- getOption("summata.number_format", "us")
    }

    ## Custom vector: c(big.mark, decimal.mark)
    if (is.character(number_format) && length(number_format) == 2L) {
        return(list(big.mark = number_format[1L], decimal.mark = number_format[2L]))
    }

    ## Named presets (use thin space U+202F for SI style)
    if (is.character(number_format) && length(number_format) == 1L) {
        return(switch(number_format,
                      "us"    = list(big.mark = ",",      decimal.mark = "."),
                      "eu"  = list(big.mark = ".",      decimal.mark = ","),
                      "space" = list(big.mark = "\u202F", decimal.mark = "."),
                      "none"  = list(big.mark = "",       decimal.mark = "."),
                      stop("Unknown number_format preset: '", number_format,
                           "'. Use 'us', 'eu', 'space', 'none', or a ",
                           "two-element character vector c(big.mark, decimal.mark).",
                           call. = FALSE)
                      ))
    }

    stop("'number_format' must be a character string preset or a ",
         "two-element character vector.", call. = FALSE)
}


#' Resolve the CI or range separator
#'
#' Determines the appropriate separator character between two numeric bounds
#' (\emph{e.g.,} CI lower-upper, range min-max) based on whether either bound is
#' negative and the current locale's decimal mark. This avoids ambiguous
#' output like \code{"(-5--3)"} or \code{"(1,2-3,4)"} with European commas.
#'
#' Rules:
#' \enumerate{
#'   \item If either bound is negative, use \code{" to "} to avoid
#'     double-hyphen ambiguity (\emph{e.g.,} \code{"-5 to -3"} not \code{"-5--3"}).
#'   \item If the decimal mark is a comma (EU locale), use \code{"\u2013"}
#'     (en-dash) to avoid confusion between decimal commas and separating
#'     commas (\emph{e.g.,} \code{"1,2\u20133,4"} not \code{"1,2-3,4"}).
#'   \item Otherwise, use a plain hyphen \code{"-"}.
#' }
#'
#' @param lower Numeric value of the lower bound (or minimum).
#' @param upper Numeric value of the upper bound (or maximum).
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string separator.
#' @keywords internal
resolve_separator <- function(lower, upper, marks) {
    has_negative <- (!is.na(lower) && lower < 0) ||
        (!is.na(upper) && upper < 0)
    if (has_negative) return(" to ")
    if (marks$decimal.mark == ",") return("\u2013")
    "-"
}


#' Apply locale decimal mark to a sprintf-formatted string
#'
#' Replaces the period decimal mark in a \code{sprintf}-formatted string with
#' the locale-appropriate decimal mark, and fixes negative-zero artefacts.
#'
#' @param x Character string (already formatted with \code{sprintf}).
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string with corrected decimal marks and no negative zeros.
#' @keywords internal
apply_decimal_mark <- function(x, marks) {
    if (marks$decimal.mark != ".") {
        x <- gsub(".", marks$decimal.mark, x, fixed = TRUE)
    }
    ## Fix negative zero
    dec_esc <- gsub("([.|()\\^{}+$*?\\[\\]])", "\\\\\\1", marks$decimal.mark)
    pattern <- paste0("(?<![0-9])-0(", dec_esc, "0+)(?![0-9])")
    gsub(pattern, "0\\1", x, perl = TRUE)
}


### * Number formatting

#' Format a numeric value with locale-aware separators
#'
#' General-purpose number formatter used by all display functions. For values
#' \eqn{\ge} 1000 (in absolute value), inserts the appropriate
#' thousands separator. Fixes negative-zero display artefacts.
#'
#' @param x Numeric value to format.
#' @param fmt_str Character string \code{sprintf} format specification
#'   (\emph{e.g.,} \code{"\%.1f"}). Used only for |x| < 1000.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string with the formatted number.
#' @keywords internal
format_num <- function(x, fmt_str, marks) {
    if (is.na(x)) return("")
    if (abs(x) >= 1000) {
        formatted <- format(round(x, 1), big.mark = marks$big.mark,
                            scientific = FALSE)
        ## Replace decimal mark if non-standard
        if (marks$decimal.mark != ".") {
            formatted <- sub(".", marks$decimal.mark, formatted, fixed = TRUE)
        }
        trimws(formatted)
    } else {
        apply_decimal_mark(sprintf(fmt_str, x), marks)
    }
}


#' Format integer counts with locale-aware thousands separator
#'
#' Formats integer values with a thousands separator for display in tables.
#' Values below 1000 are returned as plain character strings, since a grouping
#' mark only applies from four digits.
#'
#' Vectorized over \code{n}. Each element is formatted independently, so counts
#' of differing magnitude are not padded to a common width.
#'
#' @param n Numeric vector of counts.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}. Resolved from the global option
#'   when not supplied, so that callers without a locale of their own still
#'   produce output consistent with the rest of the package.
#' @param na Character string substituted for missing values. Contexts differ in
#'   what reads correctly: tables conventionally show a dash, plot annotations
#'   are better left blank, and the default preserves the missing value for
#'   callers doing their own substitution.
#' @return Character vector of formatted counts, the same length as \code{n}.
#' @keywords internal
format_count <- function(n, marks = NULL, na = NA_character_) {

    if (is.null(marks)) {
        marks <- resolve_number_marks(NULL)
    }

    out <- as.character(n)
    out[is.na(n)] <- na

    big <- which(!is.na(n) & abs(n) >= 1000)

    if (length(big) > 0) {
        ## trim = TRUE prevents format() padding the elements to a common
        ## width, and scientific = FALSE prevents it choosing exponential
        ## notation, which it otherwise does from six digits upward
        out[big] <- trimws(format(n[big],
                                  big.mark = marks$big.mark,
                                  decimal.mark = marks$decimal.mark,
                                  scientific = FALSE,
                                  trim = TRUE))
    }

    return(out)
}


#' Format a categorical statistic for display
#'
#' Converts frequency counts into formatted display strings following
#' standard conventions (n, n (\%), \% only) with locale-aware decimal marks.
#'
#' @param n Integer count for the category.
#' @param total Integer total count for percentage calculation.
#' @param stat_type Character string: \code{"n"}, \code{"n_percent"}, or
#'   \code{"percent"}.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string with the formatted statistic.
#' @keywords internal
format_categorical_stat <- function(n, total, stat_type, marks) {
    if (length(stat_type) > 1) stat_type <- stat_type[1]

    n_formatted <- format_count(n, marks)
    pct <- 100 * n / total

    switch(stat_type,
           "n" = n_formatted,
           "percent" = {
               pct_str <- sprintf("%.1f", pct)
               if (marks$decimal.mark != ".") {
                   pct_str <- sub(".", marks$decimal.mark, pct_str, fixed = TRUE)
               }
               paste0(pct_str, "%")
           },
           "n_percent" = {
               pct_str <- sprintf("%.1f", pct)
               if (marks$decimal.mark != ".") {
                   pct_str <- sub(".", marks$decimal.mark, pct_str, fixed = TRUE)
               }
               paste0(n_formatted, " (", pct_str, "%)")
           },
           n_formatted
    )
}


#' Format survival median with CI for display
#'
#' Formats a survival median estimate with confidence interval using
#' locale-aware decimal marks and safe CI separators. Used by
#' \code{process_survival} in descriptive tables.
#'
#' @param median Numeric median survival time.
#' @param lower Numeric lower CI bound.
#' @param upper Numeric upper CI bound.
#' @param fmt_str Character string \code{sprintf} format specification.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string with formatted "median (lower-upper)".
#' @keywords internal
format_survival_ci <- function(median, lower, upper, fmt_str, marks) {
    sep <- resolve_separator(lower, upper, marks)
    fmt_one <- function(x) {
        result <- sprintf(fmt_str, x)
        apply_decimal_mark(result, marks)
    }
    paste0(fmt_one(median), " (", fmt_one(lower), sep, fmt_one(upper), ")")
}


#' Format a \emph{p}-value with locale-aware decimal mark
#'
#' Converts a numeric \emph{p}-value to a display string with the correct
#' decimal separator and threshold notation (\emph{e.g.,} "< 0.001" or "< 0,001").
#'
#' @param p Numeric \emph{p}-value.
#' @param digits Integer number of decimal places.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string with the formatted \emph{p}-value.
#' @keywords internal
format_pvalue <- function(p, digits, marks) {
    if (is.na(p)) return("")

    threshold <- 10^(-digits)
    if (p < threshold) {
        threshold_str <- paste0("< 0", marks$decimal.mark,
                                strrep("0", digits - 1), "1")
        return(threshold_str)
    }

    fmt_str <- paste0("%.", digits, "f")
    result <- sprintf(fmt_str, p)
    if (marks$decimal.mark != ".") {
        result <- sub(".", marks$decimal.mark, result, fixed = TRUE)
    }
    result
}

## format_count() is the package's single count formatter. It is vectorized,
## applies the grouping mark only from four digits, and must not fall back to
## exponential notation for large counts.

test_that("format_count is vectorized and formats each element independently", {

    marks <- summata:::resolve_number_marks("us")

    result <- summata:::format_count(c(5, 999, 1000, 20000), marks)

    expect_length(result, 4L)
    ## No padding to a common width
    expect_identical(result, c("5", "999", "1,000", "20,000"))
})


test_that("format_count does not fall back to exponential notation", {

    marks <- summata:::resolve_number_marks("us")

    expect_identical(summata:::format_count(100000, marks), "100,000")
    expect_identical(summata:::format_count(1000000, marks), "1,000,000")
})


test_that("format_count follows the locale and resolves it when absent", {

    expect_identical(
        summata:::format_count(20000, summata:::resolve_number_marks("eu")),
        "20.000")

    withr::local_options(summata.number_format = "eu")
    expect_identical(summata:::format_count(20000), "20.000")
})


test_that("format_count preserves missing values", {

    marks <- summata:::resolve_number_marks("us")

    result <- summata:::format_count(c(1000, NA, 5), marks)

    expect_true(is.na(result[2]))
    expect_identical(result[c(1, 3)], c("1,000", "5"))
})


test_that("format_count emits no marks warning under any locale", {

    for (fmt in c("us", "eu")) {
        marks <- summata:::resolve_number_marks(fmt)
        expect_no_warning(summata:::format_count(c(999, 1000, 250000), marks))
    }
})


test_that("format_count substitutes missing values per its caller's convention", {

    marks <- summata:::resolve_number_marks("us")
    counts <- c(1000, NA, 5)

    ## Tables show a dash, plot annotations are left blank, and the default
    ## preserves the missing value
    expect_identical(summata:::format_count(counts, marks, na = "-"),
                     c("1,000", "-", "5"))
    expect_identical(summata:::format_count(counts, marks, na = ""),
                     c("1,000", "", "5"))
    expect_true(is.na(summata:::format_count(counts, marks)[2]))
})

#' Test Suite for International Number Formatting
#'
#' Tests for locale-aware number formatting across all summata functions:
#' desctable, survtable, fit, fullfit, uniscreen, compfit, multifit,
#' and forest plot functions. Tests exercise the public API; a small
#' section uses \code{:::} for edge cases that are difficult to trigger
#' through exported functions alone.
#'
#' @details Run with testthat::test_file("tests/testthat/test_number_format.R")

library(data.table)

## ============================================================================
## Setup: Create test data
## ============================================================================


## Small dataset for fast regression tests
set.seed(42)
test_dt <- data.table(
    age = rnorm(200, 55, 15),
    bmi = rnorm(200, 27, 5),
    sbp = rnorm(200, 130, 20),
    treatment = factor(sample(c("Drug", "Placebo"), 200, replace = TRUE)),
    sex = factor(sample(c("Male", "Female"), 200, replace = TRUE)),
    outcome = rbinom(200, 1, 0.3),
    surv_time = rexp(200, 0.02),
    surv_status = rbinom(200, 1, 0.4),
    site = factor(sample(paste0("Site", 1:5), 200, replace = TRUE))
)


## ============================================================================
## Section 1: Validation — invalid number_format rejected everywhere
## ============================================================================

test_that("desctable rejects invalid number_format", {
    expect_error(desctable(clintrial, variables = "age",
                           number_format = "french"))
    expect_error(desctable(clintrial, variables = "age",
                           number_format = 123))
})

test_that("fit rejects invalid number_format", {
    expect_error(fit(test_dt, outcome = "outcome", predictors = "age",
                     number_format = "invalid"))
})

test_that("survtable rejects invalid number_format", {
    expect_error(survtable(clintrial,
                           outcome = "Surv(surv_time, surv_status)",
                           times = 12, number_format = "invalid"))
})

test_that("compfit rejects invalid number_format", {
    expect_error(compfit(test_dt, outcome = "outcome",
                         model_list = list("M1" = "age"),
                         number_format = c(",", ",")))
})

test_that("multifit rejects invalid number_format", {
    expect_error(multifit(test_dt, outcomes = "outcome",
                          predictor = "treatment",
                          number_format = c("x")))
})


## ============================================================================
## Section 2: desctable() with number_format
## ============================================================================

test_that("desctable number_format=NULL runs without error", {
    result <- desctable(clintrial, variables = "age", number_format = NULL)
    expect_s3_class(result, "data.table")
})

test_that("desctable eu continuous stats use comma decimal", {
    result <- desctable(clintrial, variables = c("age", "bmi"),
                        number_format = "eu")
    expect_s3_class(result, "data.table")

    ## Stat rows should contain comma (eu decimal mark)
    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    non_empty <- all_cells[nchar(all_cells) > 0]
    expect_true(any(grepl(",", non_empty)))
})

test_that("desctable eu categorical percentages use comma decimal", {
    result <- desctable(clintrial, variables = "sex",
                        number_format = "eu")

    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    pct_cells <- all_cells[grepl("%", all_cells)]
    if (length(pct_cells) > 0) {
        expect_true(any(grepl(",", pct_cells)))
    }
})

test_that("desctable eu p-values use comma decimal", {
    result <- desctable(clintrial, by = "treatment",
                        variables = "age", test = TRUE,
                        number_format = "eu")

    if ("p-value" %in% names(result)) {
        p_vals <- result[`p-value` != "" & `p-value` != "-", `p-value`]
        if (length(p_vals) > 0) {
            ## No bare "0.xxx" — should be "0,xxx" or "< 0,xxx"
            expect_false(any(grepl("^0\\.", p_vals)))
        }
    }
})

test_that("desctable eu N row uses dot thousands separator", {
    big_dt <- rbindlist(replicate(10, clintrial, simplify = FALSE))
    result <- desctable(big_dt, by = "treatment", variables = "age",
                        number_format = "eu")

    n_row <- result[Variable == "N"]
    all_n <- unlist(n_row[, -c("Variable", "Group"), with = FALSE])
    numeric_n <- all_n[all_n != ""]
    ## If N >= 1000, thousands separator should be dot (not comma)
    if (any(nchar(gsub("[^0-9]", "", numeric_n)) >= 4)) {
        expect_true(any(grepl("\\.", numeric_n)))
    }
})

test_that("desctable space format runs without error", {
    result <- desctable(clintrial, variables = "age",
                        number_format = "space")
    expect_s3_class(result, "data.table")
})

test_that("desctable custom format runs without error", {
    result <- desctable(clintrial, variables = "age",
                        number_format = c("'", "."))
    expect_s3_class(result, "data.table")
})

test_that("desctable global option summata.number_format works", {
    withr::local_options(summata.number_format = "eu")

    result <- desctable(clintrial, variables = "age")
    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    expect_true(any(grepl(",", all_cells)))
})

test_that("desctable US and eu produce same structure", {
    result_us <- desctable(clintrial, by = "treatment",
                           variables = c("age", "sex"),
                           number_format = "us")
    result_eu <- desctable(clintrial, by = "treatment",
                             variables = c("age", "sex"),
                             number_format = "eu")

    expect_equal(nrow(result_us), nrow(result_eu))
    expect_equal(ncol(result_us), ncol(result_eu))
    expect_equal(names(result_us), names(result_eu))
    expect_equal(result_us$Variable, result_eu$Variable)
    expect_equal(result_us$Group, result_eu$Group)
})


## ============================================================================
## Section 3: desctable() — negative values and range separators
## ============================================================================

test_that("desctable negative range uses 'to' separator", {
    dt_neg <- data.table(
        x = c(-10, -5, 0, 5, 10, 15),
        grp = factor(c("A", "A", "A", "B", "B", "B"))
    )

    result <- desctable(dt_neg, variables = "x",
                        stats_continuous = "range")

    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    range_cells <- all_cells[nchar(all_cells) > 3]
    if (length(range_cells) > 0) {
        expect_true(any(grepl(" to ", range_cells)))
    }
})

test_that("desctable eu positive range uses en-dash", {
    dt_pos <- data.table(x = c(1, 5, 10, 15, 20))

    result <- desctable(dt_pos, variables = "x",
                        stats_continuous = "range",
                        number_format = "eu")

    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    range_cells <- all_cells[nchar(all_cells) > 3]
    if (length(range_cells) > 0) {
        expect_true(any(grepl("\u2013", range_cells)))
    }
})

test_that("desctable median_iqr with negatives avoids ambiguous output", {
    set.seed(2024)
    dt_neg <- data.table(x = rnorm(100, -5, 3))

    result <- desctable(dt_neg, variables = "x",
                        stats_continuous = "median_iqr")

    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    iqr_cells <- all_cells[grepl("\\[", all_cells)]
    if (length(iqr_cells) > 0) {
        ## Should never have double-dash like "[-8.2--3.1]"
        expect_false(any(grepl("--", iqr_cells, fixed = TRUE)))
    }
})


## ============================================================================
## Section 4: survtable() with number_format
## ============================================================================

test_that("survtable works with eu number_format", {
    result <- survtable(clintrial,
                        outcome = "Surv(os_months, os_status)",
                        times = c(12, 24), number_format = "eu")
    expect_s3_class(result, "data.table")
})

test_that("survtable works with space number_format", {
    result <- survtable(clintrial,
                        outcome = "Surv(os_months, os_status)",
                        times = c(12, 24), number_format = "space")
    expect_s3_class(result, "data.table")
})


## ============================================================================
## Section 5: fit() with number_format
## ============================================================================

test_that("fit eu effect column uses comma decimal", {
    result <- fit(test_dt, outcome = "outcome",
                  predictors = c("age", "sex"),
                  number_format = "eu")
    expect_s3_class(result, "data.table")

    effect_col <- grep("OR|HR|RR|Coeff", names(result), value = TRUE)
    if (length(effect_col) > 0) {
        vals <- result[[effect_col[1]]]
        non_ref <- vals[vals != "" & vals != "reference"]
        if (length(non_ref) > 0) {
            expect_true(any(grepl(",", non_ref)))
        }
    }
})

test_that("fit eu p-values use comma decimal", {
    result <- fit(test_dt, outcome = "outcome",
                  predictors = "age", number_format = "eu")

    if ("p-value" %in% names(result)) {
        p_vals <- result[`p-value` != "" & `p-value` != "-", `p-value`]
        if (length(p_vals) > 0) {
            expect_false(any(grepl("^0\\.", p_vals)))
        }
    }
})

test_that("fit linear model never produces double-dash CIs", {
    result <- fit(test_dt, outcome = "sbp",
                  predictors = c("age", "treatment"),
                  family = "gaussian",
                  number_format = "eu")

    effect_col <- grep("Coeff", names(result), value = TRUE)
    if (length(effect_col) > 0) {
        vals <- result[[effect_col[1]]]
        non_ref <- vals[vals != "" & vals != "reference"]
        expect_false(any(grepl("--", non_ref, fixed = TRUE)))
    }
})

test_that("fit space format runs without error", {
    result <- fit(test_dt, outcome = "outcome",
                  predictors = c("age", "sex"),
                  number_format = "space")
    expect_s3_class(result, "data.table")
})

test_that("fit with pre-fitted model accepts number_format", {
    mod <- glm(outcome ~ age + sex, data = test_dt, family = binomial)
    result <- fit(test_dt, model = mod, number_format = "eu")
    expect_s3_class(result, "data.table")
})

test_that("fit US and eu produce same structure", {
    result_us <- fit(test_dt, outcome = "outcome",
                     predictors = c("age", "sex"),
                     number_format = "us")
    result_eu <- fit(test_dt, outcome = "outcome",
                       predictors = c("age", "sex"),
                       number_format = "eu")

    expect_equal(nrow(result_us), nrow(result_eu))
    expect_equal(ncol(result_us), ncol(result_eu))
})


## ============================================================================
## Section 6: uniscreen() with number_format
## ============================================================================

test_that("uniscreen eu runs without error", {
    result <- uniscreen(test_dt, outcome = "outcome",
                        predictors = c("age", "sex"),
                        number_format = "eu")
    expect_s3_class(result, "data.table")
})

test_that("uniscreen eu effect column uses comma", {
    result <- uniscreen(test_dt, outcome = "outcome",
                        predictors = c("age", "sex"),
                        number_format = "eu")

    effect_col <- grep("OR|HR|RR|Coeff", names(result), value = TRUE)
    if (length(effect_col) > 0) {
        vals <- result[[effect_col[1]]]
        non_ref <- vals[vals != "" & vals != "reference"]
        if (length(non_ref) > 0) {
            expect_true(any(grepl(",", non_ref)))
        }
    }
})


## ============================================================================
## Section 7: fullfit() with number_format
## ============================================================================

test_that("fullfit eu runs without error", {
    result <- fullfit(test_dt, outcome = "outcome",
                      predictors = c("age", "sex"),
                      method = "all", number_format = "eu")
    expect_s3_class(result, "data.table")
})

test_that("fullfit eu output contains comma decimals", {
    result <- fullfit(test_dt, outcome = "outcome",
                      predictors = c("age", "sex"),
                      method = "all", columns = "both",
                      number_format = "eu")

    all_cells <- unlist(result[, -c("Variable", "Group"), with = FALSE])
    non_empty <- all_cells[all_cells != "" & all_cells != "reference" &
                           all_cells != "-"]
    if (length(non_empty) > 0) {
        expect_true(any(grepl(",", non_empty)))
    }
})


## ============================================================================
## Section 8: compfit() with number_format
## ============================================================================

test_that("compfit eu runs without error", {
    result <- compfit(test_dt, outcome = "outcome",
                      model_list = list("M1" = "age",
                                        "M2" = c("age", "sex")),
                      number_format = "eu")
    expect_s3_class(result, "data.table")
})

test_that("compfit eu Global p uses comma decimal", {
    result <- compfit(test_dt, outcome = "outcome",
                      model_list = list("M1" = "age",
                                        "M2" = c("age", "sex")),
                      number_format = "eu")

    if ("Global p" %in% names(result)) {
        gp <- result$`Global p`
        gp_valid <- gp[!is.na(gp) & gp != "-"]
        if (length(gp_valid) > 0) {
            expect_false(any(grepl("^0\\.", gp_valid)))
        }
    }
})


## ============================================================================
## Section 9: multifit() with number_format
## ============================================================================

test_that("multifit eu runs without error", {
    result <- multifit(test_dt, outcomes = "outcome",
                       predictor = "treatment",
                       covariates = "age", columns = "adjusted",
                       number_format = "eu")
    expect_s3_class(result, "data.table")
})

test_that("multifit eu effect column uses comma", {
    result <- multifit(test_dt, outcomes = "outcome",
                       predictor = "treatment",
                       covariates = "age", columns = "adjusted",
                       number_format = "eu")

    effect_col <- grep("OR|HR|RR|Coeff", names(result), value = TRUE)
    if (length(effect_col) > 0) {
        vals <- result[[effect_col[1]]]
        non_empty <- vals[vals != "" & vals != "-"]
        if (length(non_empty) > 0) {
            expect_true(any(grepl(",", non_empty)))
        }
    }
})


## ============================================================================
## Section 10: Forest plots with number_format
## ============================================================================

test_that("glmforest eu runs without error", {
    skip_if_not_installed("ggplot2")
    mod <- glm(outcome ~ age + sex, data = test_dt, family = binomial)
    p <- glmforest(mod, data = test_dt, number_format = "eu")
    expect_true(inherits(p, "gg") || inherits(p, "ggplot"))
})

test_that("lmforest eu runs without error", {
    skip_if_not_installed("ggplot2")
    mod <- lm(sbp ~ age + sex, data = test_dt)
    p <- lmforest(mod, data = test_dt, number_format = "eu")
    expect_true(inherits(p, "gg") || inherits(p, "ggplot"))
})

test_that("coxforest eu runs without error", {
    skip_if_not_installed("ggplot2")
    mod <- survival::coxph(
        survival::Surv(surv_time, surv_status) ~ age + sex,
        data = test_dt)
    p <- coxforest(mod, data = test_dt, number_format = "eu")
    expect_true(inherits(p, "gg") || inherits(p, "ggplot"))
})

test_that("autoforest passes number_format through", {
    skip_if_not_installed("ggplot2")
    mod <- glm(outcome ~ age + sex, data = test_dt, family = binomial)
    p <- autoforest(mod, data = test_dt, number_format = "eu")
    expect_true(inherits(p, "gg") || inherits(p, "ggplot"))
})

test_that("uniforest eu runs without error", {
    skip_if_not_installed("ggplot2")
    uni_result <- uniscreen(test_dt, outcome = "outcome",
                            predictors = c("age", "sex"))
    p <- uniforest(uni_result, number_format = "eu")
    expect_true(inherits(p, "gg") || inherits(p, "ggplot"))
})

test_that("multiforest eu runs without error", {
    skip_if_not_installed("ggplot2")
    mf_result <- multifit(test_dt, outcomes = "outcome",
                          predictor = "treatment",
                          covariates = "age", columns = "adjusted")
    p <- multiforest(mf_result, number_format = "eu")
    expect_true(inherits(p, "gg") || inherits(p, "ggplot"))
})


## ============================================================================
## Section 11: Edge cases via ::: (hard to trigger through public API)
## ============================================================================

## These internal functions have tricky logic that is difficult to exercise
## reliably through the public API with controlled inputs.

test_that("negative zero is fixed for all presets", {
    apply_dm <- summata:::apply_decimal_mark
    for (preset in c("us", "eu", "space", "none")) {
        m <- summata:::resolve_number_marks(preset)
        result <- apply_dm("-0.00", m)
        expect_false(startsWith(result, "-"),
                     info = paste("negative zero not fixed for:", preset))
    }
})

test_that("resolve_separator: negatives always produce ' to '", {
    resolve_sep <- summata:::resolve_separator
    for (preset in c("us", "eu", "space", "none")) {
        m <- summata:::resolve_number_marks(preset)
        expect_equal(resolve_sep(-5, 3, m), " to ",
                     info = paste("failed for:", preset))
        expect_equal(resolve_sep(-5, -3, m), " to ",
                     info = paste("both-negative failed for:", preset))
    }
})

test_that("resolve_separator: eu positive uses en-dash", {
    resolve_sep <- summata:::resolve_separator
    m <- summata:::resolve_number_marks("eu")
    expect_equal(resolve_sep(1, 3, m), "\u2013")
})

test_that("resolve_separator: US positive uses hyphen", {
    resolve_sep <- summata:::resolve_separator
    m <- summata:::resolve_number_marks("us")
    expect_equal(resolve_sep(1, 3, m), "-")
})

test_that("validate_number_format rejects identical custom separators", {
    validate <- summata:::validate_number_format
    expect_error(validate(c(",", ",")), "cannot be the same")
})

test_that("validate_number_format rejects wrong-length vector", {
    validate <- summata:::validate_number_format
    expect_error(validate(c(",", ".", ";")))
    expect_error(validate(c(",")))
})

test_that("format_pvalues_fit: eu threshold string uses comma", {
    fmt_p <- summata:::format_pvalues_fit
    m <- summata:::resolve_number_marks("eu")
    result <- fmt_p(c(0.042, 0.0001, NA), 3, m)
    expect_equal(result[1], "0,042")
    expect_equal(result[2], "< 0,001")
    expect_equal(result[3], "-")
})

test_that("format_pvalues_fit: NULL marks falls back to US", {
    fmt_p <- summata:::format_pvalues_fit
    result <- fmt_p(c(0.042, 0.0001, NA), 3)
    expect_equal(result[1], "0.042")
    expect_equal(result[2], "< 0.001")
    expect_equal(result[3], "-")
})

test_that("forest_ci_separator: eu positive uses en-dash", {
    fcs <- summata:::forest_ci_separator
    m <- summata:::resolve_number_marks("eu")
    expect_equal(fcs(FALSE, m), "\u2013")
})

test_that("forest_ci_separator: negatives always use comma-space", {
    fcs <- summata:::forest_ci_separator
    for (preset in c("us", "eu", "space")) {
        m <- summata:::resolve_number_marks(preset)
        expect_equal(fcs(TRUE, m), ", ",
                     info = paste("failed for:", preset))
    }
})

test_that("forest_ci_separator: NULL marks defaults to US", {
    fcs <- summata:::forest_ci_separator
    expect_equal(fcs(FALSE, NULL), "-")
    expect_equal(fcs(TRUE, NULL), ", ")
})

test_that("global option fallback works when number_format is NULL", {
    resolve <- summata:::resolve_number_marks

    withr::local_options(summata.number_format = "eu")

    m <- resolve(NULL)
    expect_equal(m$decimal.mark, ",")

    withr::local_options(summata.number_format = NULL)
    m <- resolve(NULL)
    expect_equal(m$decimal.mark, ".")
})

### * Extraction functions

#' Get factor levels from model (works with S3 and S4)
#' 
#' Extracts factor level information from fitted model objects. Handles both
#' S3 models (glm, lm, coxph) via xlevels slot and S4 models (lme4) via
#' the model frame.
#' 
#' @param model Fitted model object (S3 or S4).
#' @return Named list of factor levels, or \code{NULL} if no factors present.
#' @keywords internal
get_model_xlevels <- function(model) {
    
    ## For regular S3 models (glm, lm, coxph, clogit)
    if (!isS4(model)) {
        ## Special handling for coxme - it doesn't store xlevels
        if (inherits(model, "coxme")) {
            return(NULL)  # Will be handled separately
        }
        return(model$xlevels)
    }
    
    ## For S4 lme4 models (glmer, lmer)
    if (inherits(model, "merMod")) {
        ## Extract from the model frame
        frame_data <- model@frame
        
        xlevels <- list()
        for (var_name in names(frame_data)) {
            if (is.factor(frame_data[[var_name]])) {
                xlevels[[var_name]] <- levels(frame_data[[var_name]])
            }
        }
        
        return(if (length(xlevels) > 0) xlevels else NULL)
    }
    
    ## Default: return NULL
    return(NULL)
}


#' Get data from model object (works with S3 and S4)
#' 
#' Retrieves the original data used to fit a model. Checks multiple locations
#' including model attributes, $data slot, $model slot, and @frame for S4.
#' 
#' @param model Fitted model object (S3 or S4).
#' @return Data frame or data.table used to fit the model, or \code{NULL} if unavailable.
#' @keywords internal
get_model_data <- function(model) {
    
    ## Try attribute first
    data_attr <- attr(model, "data")
    if (!is.null(data_attr)) {
        return(data_attr)
    }
    
    ## For S3 objects, try model$data
    if (!isS4(model)) {
        if (!is.null(model$data)) {
            return(model$data)
        }
        if (!is.null(model$model)) {
            return(model$model)
        }
    }
    
    ## For S4 lme4 objects, use model frame
    if (inherits(model, "merMod")) {
        return(model@frame)
    }
    
    ## Default
    return(NULL)
}


#' Identify the variables required by a fitted model
#'
#' Returns the names of every variable referenced by a model formula, including
#' the response, stratification factors, offsets and random-effect grouping
#' factors. Terms are expanded where possible, so a model specified with a dot
#' on the right-hand side (\emph{e.g.,} \code{y ~ .}) returns the complete set
#' of variables rather than the literal dot.
#'
#' @param model Fitted model object.
#' @param model_class Character string of the model class.
#' @return Character vector of variable names, or \code{character(0)} when the
#'   formula cannot be recovered.
#' @keywords internal
get_model_variables <- function(model, model_class) {

    ## Mixed-effects and coxme formulas carry the grouping factors, which must
    ## also be non-missing for an observation to enter the fit. The formula is
    ## therefore preferred over the terms object for these classes, because the
    ## terms object describes the fixed effects only.
    if (identical(model_class, "coxme") || inherits(model, "merMod")) {
        return(unique(tryCatch(all.vars(stats::formula(model)),
                               error = function(e) character(0))))
    }

    ## The terms object holds the expanded formula, which matters when the
    ## model was specified with a dot on the right-hand side.
    vars <- tryCatch(all.vars(stats::terms(model)),
                     error = function(e) NULL)

    if (is.null(vars)) {
        vars <- tryCatch(all.vars(stats::formula(model)),
                         error = function(e) character(0))
    }

    return(unique(vars))
}


#' Number of observations used to fit a model
#'
#' Provides a single accessor for the fitted sample size across the model
#' classes supported by \pkg{summata}. Survival models record this quantity in
#' class-specific components rather than through \code{stats::nobs()}.
#'
#' @param model Fitted model object.
#' @param model_class Character string of the model class.
#' @return Numeric scalar giving the number of observations used in fitting, or
#'   \code{NA_real_} when it cannot be determined.
#' @keywords internal
get_model_nobs <- function(model, model_class) {

    n_used <- NA_real_

    if (identical(model_class, "coxme")) {
        ## coxme stores c(events, observations)
        if (!is.null(model$n) && length(model$n) >= 2L) {
            n_used <- as.numeric(model$n[2])
        }
    } else if (model_class %in% c("coxph", "clogit")) {
        if (!is.null(model$n)) {
            n_used <- as.numeric(model$n[1])
        }
    }

    if (is.na(n_used)) {
        n_used <- tryCatch(as.numeric(stats::nobs(model)),
                           error = function(e) NA_real_)
    }

    return(n_used)
}


#' Restrict data to the observations used in model fitting
#'
#' Group-level sample sizes must be counted over the observations that entered
#' the model, not over every row supplied by the user. Several fitted objects
#' carry no model frame from which those observations can be read directly:
#' \code{survival::coxph()} does not store one by default, \pkg{coxme} models
#' never store one, and the memory-conserving fits performed internally by
#' \code{uniscreen()} and \code{multifit()} pass \code{model = FALSE}. In these
#' cases the supplied data still contains the observations dropped during
#' fitting, and counting them inflates group sizes whenever the response or a
#' covariate is missing.
#'
#' @param model Fitted model object.
#' @param model_class Character string of the model class.
#' @param data Data frame or data.table supplied to the model call.
#' @return Data.table restricted to the observations used in fitting. The input
#'   is returned unchanged when the analysis rows cannot be determined.
#' @keywords internal
get_analysis_data <- function(model, model_class, data) {

    if (is.null(data) || !inherits(data, "data.frame")) {
        return(data)
    }

    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }

    n_used <- get_model_nobs(model, model_class)

    ## No observations were dropped, so no restriction is required
    if (!is.na(n_used) && nrow(data) == n_used) {
        return(data)
    }

    model_vars <- get_model_variables(model, model_class)
    model_vars <- model_vars[model_vars %in% names(data)]

    if (length(model_vars) == 0L) {
        return(data)
    }

    keep <- stats::complete.cases(data[, model_vars, with = FALSE])

    return(data[keep])
}


#' Detect if model is univariable or multivariable
#' 
#' Determines whether a model contains one predictor (univariable) or multiple
#' predictors (multivariable) by analyzing coefficient names and factor structure.
#' Handles interactions and random effects appropriately.
#' 
#' @param model Fitted model object.
#' @return Character string: "Univariable" or "Multivariable".
#' @keywords internal
detect_model_type <- function(model) {
    ## For glmer/glmerMod/lmer/lmerMod models, handle random effects specially
    if (inherits(model, c("glmerMod", "lmerMod", "merMod"))) {
        ## Get the fixed effects formula (excluding random effects)
        if (!is.null(model@call$formula)) {
            formula_str <- as.character(model@call$formula)[3]  # RHS of formula
            ## Remove random effects terms (anything with |)
            formula_str <- gsub("\\([^|]*\\|[^)]*\\)", "", formula_str)
            ## Clean up extra spaces and plus signs
            formula_str <- gsub("\\s*\\+\\s*\\+", " +", formula_str)
            formula_str <- gsub("^\\s*\\+\\s*", "", formula_str)
            formula_str <- gsub("\\s*\\+\\s*$", "", formula_str)
            
            ## Split by + to get terms
            terms <- trimws(strsplit(formula_str, "\\+")[[1]])
            terms <- terms[terms != ""]
            
            ## Count unique base variables (handle interactions)
            unique_vars <- character()
            for (term in terms) {
                if (grepl(":", term)) {
                    ## Interaction term - split and add components
                    components <- trimws(strsplit(term, ":")[[1]])
                    unique_vars <- c(unique_vars, components)
                } else {
                    unique_vars <- c(unique_vars, term)
                }
            }
            unique_vars <- unique(unique_vars)
            
            return(data.table::fifelse(length(unique_vars) == 1, "Univariable", "Multivariable"))
        }
    }
    
    ## Original code for other model types
    ## Get coefficient names once
    coef_names <- names(stats::coef(model))
    
    ## Remove intercept from count
    if ("(Intercept)" %in% coef_names) {
        term_names <- coef_names[coef_names != "(Intercept)"]
    } else {
        term_names <- coef_names
    }
    
    ## Identify interaction terms (contain ":")
    is_interaction <- grepl(":", term_names, fixed = TRUE)
    interaction_terms <- term_names[is_interaction]
    main_terms <- term_names[!is_interaction]
    
    ## For models without factor variables, count unique base variables from main effects
    xlevels <- get_model_xlevels(model)
    
    ## Special handling for coxme - reconstruct factor info
    if (inherits(model, "coxme") && is.null(xlevels)) {
        data_source <- get_model_data(model)
        if (!is.null(data_source)) {
            xlevels <- list()
            formula_vars <- all.vars(stats::formula(model))
            if (length(formula_vars) >= 2) {
                predictor_vars <- formula_vars[-(1:2)]  # Remove Surv components
                predictor_vars <- predictor_vars[!grepl("\\|", predictor_vars)]  # Remove random effects
                
                for (var in predictor_vars) {
                    if (var %in% names(data_source) && is.factor(data_source[[var]])) {
                        xlevels[[var]] <- levels(data_source[[var]])
                    }
                }
            }
            if (length(xlevels) == 0) xlevels <- NULL
        }
    }
    
    if (is.null(xlevels) || length(xlevels) == 0) {
        ## Count main effect terms
        n_terms <- length(main_terms)
        
        ## Where interactions are present, extract their base variables as well
        if (length(interaction_terms) > 0) {
            ## Extract variables from interactions (split on ":")
            interaction_vars <- unique(unlist(strsplit(interaction_terms, ":", fixed = TRUE)))
            ## Count any interaction variables not already in main effects
            new_vars <- setdiff(interaction_vars, main_terms)
            n_terms <- n_terms + length(new_vars)
        }
        
        return(data.table::fifelse(n_terms == 1, "Univariable", "Multivariable"))
    }
    
    ## For models with factor variables, count unique base variables
    ##
    ## Each coefficient is assigned to the longest factor name it begins with.
    ## Returning the first name in the list would collapse "sexualityHetero"
    ## onto "sex" whenever both variables appear in the model, understating the
    ## variable count and misreporting a multivariable model as univariable.
    ## Names are compared literally, so a variable carrying an inline
    ## transformation is matched on its own characters.
    xlevel_names <- names(xlevels)
    xlevel_names <- xlevel_names[order(nchar(xlevel_names), decreasing = TRUE)]
    
    matched_var <- vapply(main_terms, function(term) {
        hits <- xlevel_names[startsWith(term, xlevel_names)]
        if (length(hits) == 0L) NA_character_ else hits[1]
    }, character(1), USE.NAMES = FALSE)
    
    factor_vars_present <- unique(matched_var[!is.na(matched_var)])
    n_vars <- length(factor_vars_present)
    
    ## Count unique continuous variables IN MAIN EFFECTS
    continuous_terms <- main_terms[is.na(matched_var)]
    n_continuous <- length(unique(continuous_terms))
    
    ## Total unique variables (from main effects only)
    ## Interactions are excluded from variable count
    n_terms <- n_vars + n_continuous
    
    return(data.table::fifelse(n_terms == 1, "Univariable", "Multivariable"))
}


#' Get readable model type name
#' 
#' Converts model class names to human-readable descriptions. For GLMs,
#' uses the family to provide specific names (\emph{e.g.,} "Logistic", "Poisson").
#' 
#' @param model Fitted model object.
#' @return Character string with readable model type name.
#' @keywords internal
get_model_type_name <- function(model) {
    model_class <- class(model)[1]
    
    ## Remove wrapper classes
    if (model_class == "mmodel") {
        model_class <- class(model)[2]
    }
    
    ## For GLM, be more specific based on family
    if (model_class == "glm") {
        family <- model$family$family
        
        ## Use switch instead of multiple ifelse()
        return(switch(family,
                      binomial = "Logistic",
                      poisson = "Poisson",
                      gaussian = "Linear (GLM)",
                      Gamma = "Gamma",
                      quasibinomial = "Quasi-Binomial",
                      quasipoisson = "Quasi-Poisson",
                      paste0(stringr::str_to_title(family), " GLM")
                      ))
    }
    
    ## Map to readable names for other model types
    return(switch(model_class,
                  lm = "Linear",
                  coxph = "Cox PH",
                  clogit = "Conditional Logistic",
                  coxme = "Mixed Effects Cox",
                  negbin = "Negative Binomial",
                  glmer = "glmerMod",  # Keep full name for clarity
                  glmerMod = "glmerMod",
                  lmer = "Linear Mixed",
                  lmerMod = "Linear Mixed",
                  model_class  # default
                  ))
}


#' Parse term into variable and group
#' 
#' Splits coefficient term names into base variable names and factor levels.
#' For example, "sexMale" becomes variable="sex" and group="Male".
#' Handles interaction terms and continuous variables appropriately.
#' 
#' @param terms Character vector of coefficient term names.
#' @param xlevels Named list of factor levels from the model.
#' @param model Optional model object for extracting factor info from coxme models.
#' @return Data.table with 'variable' and 'group' columns.
#' @keywords internal
parse_term <- function(terms, xlevels = NULL, model = NULL) {
    n_terms <- length(terms)
    
    ## Initialize result vectors
    variable <- character(n_terms)
    group <- character(n_terms)
    
    ## Interactions should not be parsed as factor variables
    is_interaction <- grepl(":", terms, fixed = TRUE)

    ## Special handling for coxme models - reconstruct xlevels if needed
    if (!is.null(model) && inherits(model, "coxme") && is.null(xlevels)) {
        data_source <- attr(model, "data")
        if (is.null(data_source)) {
            data_source <- get_model_data(model)
        }
        
        if (!is.null(data_source)) {
            xlevels <- list()
            
            ## Extract factor structure from coefficient names
            coef_names <- names(coxme::fixef(model))
            
            ## Check each column in the data
            for (col_name in names(data_source)) {
                ## Check if any coefficient starts with this column name
                if (any(grepl(paste0("^", col_name), coef_names))) {
                    ## This is a factor variable - get its levels
                    if (is.factor(data_source[[col_name]])) {
                        xlevels[[col_name]] <- levels(data_source[[col_name]])
                    } else if (is.character(data_source[[col_name]])) {
                        xlevels[[col_name]] <- sort(unique(data_source[[col_name]]))
                    }
                }
            }
            
            if (length(xlevels) == 0) xlevels <- NULL
        }
    }
    
    if (!is.null(xlevels) && length(xlevels) > 0) {
        ## Match each coefficient to the factor name it begins with.
        ##
        ## Names are compared literally rather than as regular expressions, so
        ## that variables carrying an inline transformation (\emph{e.g.,}
        ## \code{factor(stage)}) are matched on their own characters rather than
        ## on the grouping and alternation those characters would denote.
        ##
        ## Names are processed from shortest to longest so that the longest
        ## match is the one retained. Processing in formula order would assign
        ## the coefficient "sexualityHetero" to "sex" whenever both variables
        ## appear in the model and "sex" is written second.
        xlevel_names <- names(xlevels)
        xlevel_names <- xlevel_names[order(nchar(xlevel_names))]
        
        ## For each factor variable, find all matching terms at once, skip interaction terms
        for (var in xlevel_names) {
            ## Find which terms start with this variable name
            matches <- startsWith(terms, var) & !is_interaction  # Skip interactions
            
            if (any(matches)) {
                ## Extract levels for all matching terms at once
                variable[matches] <- var
                group[matches] <- substring(terms[matches], nchar(var) + 1L)
            }
        }
        
        ## Any remaining unmatched terms (including interactions) are treated as-is
        unmatched <- variable == ""
        if (any(unmatched)) {
            variable[unmatched] <- terms[unmatched]
            ## group already initialized to "" for these
        }
    } else {
        ## No factor variables - all continuous (including interactions)
        variable <- terms
        ## group already initialized to ""
    }
    
    return(data.table::data.table(variable = variable, group = group))
}


#' Extract event variable from survival model
#' 
#' Parses the Surv() expression in survival model formulas to extract
#' the event/status variable name. Works with coxph, clogit, and coxme models.
#' 
#' @param model Fitted survival model object.
#' @param model_class Character string of the model class.
#' @return Character string naming the event variable, or \code{NULL} if not found.
#' @keywords internal
get_event_variable <- function(model, model_class) {
    event_var <- NULL
    
    if (model_class %in% c("coxph", "clogit", "coxme")) {
        
        ## Get the formula string differently for each model type
        outcome_str <- NULL
        
        if (model_class == "coxme") {
            ## For coxme, use formulaList$fixed
            if (!is.null(model$formulaList$fixed)) {
                outcome_str <- tryCatch({
                    as.character(model$formulaList$fixed)[2]
                }, error = function(e) NULL)
            }
        } else {
            ## For coxph and clogit, use standard formula
            outcome_str <- tryCatch({
                as.character(stats::formula(model))[2]
            }, error = function(e) NULL)
        }
        
        ## Extract event variable from Surv()
        if (!is.null(outcome_str) && !is.na(outcome_str)) {
            if (grepl("Surv\\(", outcome_str)) {
                ## Remove "Surv(" from beginning and ")" from end
                surv_expr <- gsub("^Surv\\(", "", outcome_str)
                surv_expr <- gsub("\\)$", "", surv_expr)
                
                ## Split by comma to get time and event
                surv_parts <- trimws(strsplit(surv_expr, ",")[[1]])
                if (length(surv_parts) >= 2) {
                    event_var <- surv_parts[2]
                }
            }
        }
    }
    
    return(event_var)
}


#' Identify the response variables of a fitted model
#'
#' Returns the variables appearing on the left-hand side of a model formula. A
#' survival response contributes both the time and status variables.
#'
#' @param model Fitted model object.
#' @return Character vector of response variable names, or \code{character(0)}
#'   when the formula cannot be recovered.
#' @keywords internal
get_outcome_variables <- function(model) {

    fml <- tryCatch(stats::formula(model), error = function(e) NULL)

    if (is.null(fml) || length(fml) < 3L) {
        return(character(0))
    }

    return(tryCatch(all.vars(fml[[2]]), error = function(e) character(0)))
}


#' Count the observations available to an analysis
#'
#' Summarizes how many of the supplied observations could enter a model, and
#' separates those excluded for a missing response from those excluded for a
#' missing covariate. The distinction matters: a missing response and a missing
#' covariate raise different questions about whether a complete-case analysis is
#' appropriate.
#'
#' Several models may be described at once by passing a list of predictor sets,
#' as when a univariable screen fits one model per predictor. The analyzed count
#' is then returned as one element per set.
#'
#' @param data Data frame or data.table supplied to the analysis.
#' @param outcome_vars Character vector of response variable names.
#' @param predictor_vars Character vector of predictor variable names, or a list
#'   of such vectors when several models are described.
#' @param event_var Character name of the event indicator, or \code{NULL} for
#'   models that carry no event count. Factors are coded as the modeling
#'   functions code them, with any level beyond the first counting as an event.
#' @return Named list with \code{n_supplied}, \code{n_analyzed},
#'   \code{n_missing_outcome} and \code{n_missing_predictor}, and, where an
#'   event indicator was supplied, \code{events_supplied} and
#'   \code{events_analyzed}. \code{NULL} when the counts cannot be determined.
#' @keywords internal
get_analysis_counts <- function(data, outcome_vars, predictor_vars = NULL,
                                event_var = NULL) {

    if (is.null(data) || !inherits(data, "data.frame") || nrow(data) == 0) {
        return(NULL)
    }

    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }

    outcome_vars <- unique(outcome_vars[outcome_vars %in% names(data)])

    if (length(outcome_vars) == 0) {
        return(NULL)
    }

    if (!is.list(predictor_vars)) {
        predictor_vars <- list(predictor_vars)
    }

    n_supplied <- nrow(data)
    outcome_ok <- stats::complete.cases(data[, outcome_vars, with = FALSE])

    ## Event indicator, coded as the modeling functions code it. A count
    ## response contributes its counts rather than a binary flag, which
    ## mirrors the total that m2dt() reports for Poisson and negative
    ## binomial models.
    event_indicator <- NULL
    if (!is.null(event_var) && length(event_var) == 1 &&
        event_var %in% names(data)) {
        y <- data[[event_var]]
        event_indicator <- if (is.factor(y)) {
                               as.integer(y) > 1L
                           } else if (is.numeric(y) || is.logical(y)) {
                               y
                           } else {
                               NULL
                           }
    }

    counts <- lapply(predictor_vars, function(vars) {
        vars <- unique(vars[vars %in% names(data) & !vars %in% outcome_vars])

        predictor_ok <- if (length(vars) > 0) {
                            stats::complete.cases(data[, vars, with = FALSE])
                        } else {
                            rep(TRUE, n_supplied)
                        }

        analyzed <- outcome_ok & predictor_ok

        c(n_analyzed = sum(analyzed),
          n_missing_predictor = sum(outcome_ok & !predictor_ok),
          events_analyzed = if (is.null(event_indicator)) {
                                NA_real_
                            } else {
                                sum(event_indicator[analyzed], na.rm = TRUE)
                            })
    })

    result <- list(
        n_supplied = n_supplied,
        n_analyzed = vapply(counts, function(x) x[["n_analyzed"]], numeric(1)),
        n_missing_outcome = sum(!outcome_ok),
        n_missing_predictor = vapply(counts, function(x) x[["n_missing_predictor"]], numeric(1))
    )

    if (!is.null(event_indicator)) {
        result$events_supplied <- sum(event_indicator, na.rm = TRUE)
        result$events_analyzed <- vapply(counts, function(x) x[["events_analyzed"]],
                                         numeric(1))
    }

    return(result)
}


#' Count the observations available to a fitted model
#'
#' Convenience wrapper around \code{get_analysis_counts()} that derives the
#' response and predictor variables from the model itself.
#'
#' @param model Fitted model object.
#' @param model_class Character string of the model class.
#' @param data Data frame or data.table supplied to the model call.
#' @return See \code{get_analysis_counts()}.
#' @keywords internal
get_model_analysis_counts <- function(model, model_class, data) {

    outcome_vars <- get_outcome_variables(model)

    if (length(outcome_vars) == 0) {
        return(NULL)
    }

    predictor_vars <- setdiff(get_model_variables(model, model_class), outcome_vars)

    ## stats::family() resolves for glm, lm and merMod objects and errors for
    ## survival fits, which are identified by their class instead
    family_name <- tryCatch(stats::family(model)$family, error = function(e) NULL)
    event_var <- get_event_variable_for_counts(outcome_vars, model_class, family_name)

    return(get_analysis_counts(data, outcome_vars, predictor_vars, event_var))
}


#' Identify the event indicator among a model's response variables
#'
#' Determines which response variable carries the event count, and whether the
#' model has one at all. Linear and Gaussian models do not; the families that
#' count events are those for which \code{m2dt()} reports an events figure, so
#' that a denominator is only offered where a numerator exists.
#'
#' @param outcome_vars Character vector of response variable names.
#' @param model_type Character string of the model type or class.
#' @param family Model family, where applicable. Accepted as a name, a family
#'   object, or a generator function, since callers resolve the family at
#'   different points: \code{fit()} passes the name it was given, while
#'   \code{uniscreen()} may already have resolved \code{"Gamma"} to
#'   \code{Gamma(link = "log")}.
#' @return Character name of the event indicator, or \code{NULL} when the model
#'   carries no event count.
#' @keywords internal
get_event_variable_for_counts <- function(outcome_vars, model_type = NULL,
                                          family = NULL) {

    if (length(outcome_vars) == 0) {
        return(NULL)
    }

    ## The family may arrive as a name, a family object, or the generator
    ## function itself. Comparing an unresolved family object against a
    ## character vector yields one value per list element rather than one
    ## logical, so it is reduced to its name first.
    family_name <- if (inherits(family, "family")) {
                       family$family
                   } else if (is.function(family)) {
                       tryCatch(family()$family, error = function(e) NULL)
                   } else if (is.character(family)) {
                       family[1]
                   } else {
                       NULL
                   }

    survival_types <- c("coxph", "clogit", "coxme")
    count_types <- c("negbin", "glm.nb")
    glm_types <- c("glm", "glmer", "glmerMod")
    event_families <- c("binomial", "quasibinomial", "poisson", "quasipoisson")

    survival_response <- if (length(outcome_vars) >= 2) {
                             outcome_vars[2]
                         } else {
                             outcome_vars[1]
                         }

    ## Without a usable model type, a two-variable response is survival
    if (length(model_type) != 1 || is.na(model_type)) {
        return(survival_response)
    }

    ## A survival response carries the time variable first and the status
    ## variable second; the event indicator is the status variable
    if (model_type %in% survival_types) {
        return(survival_response)
    }

    if (model_type %in% count_types) {
        return(outcome_vars[1])
    }

    if (model_type %in% glm_types &&
        length(family_name) == 1 && family_name %in% event_families) {
        return(outcome_vars[1])
    }

    return(NULL)
}


### * Formatting functions

#' Describe the analyzed sample for console output
#'
#' Renders the counts produced by \code{get_analysis_counts()} as a single line
#' for the \code{print()} methods. The line is produced whenever the counts are
#' available, including when every supplied observation entered the analysis:
#' a complete sample is itself worth stating, and reporting it only on loss
#' would leave the reader to infer the difference between no exclusions and no
#' disclosure.
#'
#' @param counts List as returned by \code{get_analysis_counts()}.
#' @param label Character label preceding the counts.
#' @param marks List of number marks as returned by
#'   \code{resolve_number_marks()}. Counts and percentages are separated as
#'   the accompanying table separates them, so that a locale setting applies
#'   to the whole of the output rather than to the table alone. Resolved from
#'   the global option when not supplied.
#' @return Character string, or \code{NULL} when the counts are unavailable.
#' @keywords internal
format_analysis_counts <- function(counts, label = "Observations analyzed",
                                   marks = NULL) {

    if (is.null(counts) || length(counts$n_analyzed) == 0) {
        return(NULL)
    }

    n_supplied <- counts$n_supplied
    lo <- min(counts$n_analyzed)
    hi <- max(counts$n_analyzed)

    ## The denominator may be absent, as when a total is unavailable for the
    ## quantity being described. Nothing is reported rather than a bare
    ## numerator dressed as a proportion.
    if (length(n_supplied) == 0 || !is.finite(n_supplied) || n_supplied == 0) {
        return(NULL)
    }

    if (!is.finite(lo) || !is.finite(hi)) {
        return(NULL)
    }

    if (is.null(marks)) {
        marks <- tryCatch(resolve_number_marks(NULL),
                          error = function(e) list(big.mark = "",
                                                   decimal.mark = "."))
    }

    ## Counts and percentages are rendered by the same helpers the tables use,
    ## so that a figure in this line and the same figure in the n column below
    ## cannot be formatted differently
    count_str <- function(x) format_count(x, marks)

    pct_str <- function(x) apply_decimal_mark(sprintf("%.1f", x), marks)

    if (lo == hi) {
        return(sprintf("%s: %s of %s (%s%%)",
                       label, count_str(lo), count_str(n_supplied),
                       pct_str(100 * lo / n_supplied)))
    }

    return(sprintf("%s: %s-%s of %s (%s-%s%%)",
                   label, count_str(lo), count_str(hi), count_str(n_supplied),
                   pct_str(100 * lo / n_supplied),
                   pct_str(100 * hi / n_supplied)))
}


#' Describe the analyzed events for console output
#'
#' Renders the event counts held in \code{get_analysis_counts()} using the same
#' formatter as the observation counts, so that the two lines cannot drift apart
#' in rounding, separators or range handling.
#'
#' @param counts List as returned by \code{get_analysis_counts()}.
#' @param label Character label preceding the counts.
#' @param marks List of number marks as returned by
#'   \code{resolve_number_marks()}.
#' @return Character string, or \code{NULL} when the model carries no events.
#' @keywords internal
format_event_counts <- function(counts, label = "Events analyzed",
                                marks = NULL) {

    if (is.null(counts) || is.null(counts$events_analyzed)) {
        return(NULL)
    }

    return(format_analysis_counts(
        list(n_supplied = counts$events_supplied,
             n_analyzed = counts$events_analyzed),
        label = label,
        marks = marks
    ))
}

## Coefficient names are matched against variable names by prefix. Matching in
## formula order allowed a short name to claim a coefficient belonging to a
## longer name that starts with the same characters, and matching by regular
## expression mishandled names containing metacharacters.

test_that("coefficients are assigned to the longest matching variable name", {

    ## Declared with the longer name first, which is the order in which the
    ## shorter name would previously have overwritten the correct assignment.
    xlevels <- list(
        sexuality = c("Hetero", "Other"),
        sex = c("Female", "Male")
    )

    parsed <- summata:::parse_term(c("sexualityOther", "sexMale"), xlevels)

    expect_identical(parsed$variable, c("sexuality", "sex"))
    expect_identical(parsed$group, c("Other", "Male"))
})


test_that("variable names containing regular expression metacharacters parse", {

    xlevels <- list(`factor(stage)` = c("I", "II", "III"))

    parsed <- summata:::parse_term(c("factor(stage)II", "factor(stage)III"),
                                   xlevels)

    expect_identical(parsed$variable, rep("factor(stage)", 2L))
    expect_identical(parsed$group, c("II", "III"))
})


test_that("interaction terms are left unparsed regardless of name overlap", {

    xlevels <- list(
        sex = c("Female", "Male"),
        sexuality = c("Hetero", "Other")
    )

    parsed <- summata:::parse_term("sexMale:sexualityOther", xlevels)

    expect_identical(parsed$variable, "sexMale:sexualityOther")
    expect_identical(parsed$group, "")
})


test_that("variable counting distinguishes names that share a prefix", {

    ## The shorter name is written first, which is the order in which it would
    ## previously have absorbed the coefficients of the longer name and
    ## understated the number of variables in the model.
    set.seed(20240101)
    df <- data.frame(
        y = stats::rbinom(200, 1, 0.5),
        sex = factor(sample(c("Female", "Male"), 200, replace = TRUE)),
        sexuality = factor(sample(c("Hetero", "Other"), 200, replace = TRUE))
    )

    model <- stats::glm(y ~ sex + sexuality, data = df,
                        family = stats::binomial)

    expect_identical(summata:::detect_model_type(model), "Multivariable")
})


test_that("prefix overlap does not disturb group sizes", {

    set.seed(20240102)
    df <- data.frame(
        y = stats::rbinom(200, 1, 0.5),
        sex = factor(sample(c("Female", "Male"), 200, replace = TRUE)),
        sexuality = factor(sample(c("Hetero", "Other"), 200, replace = TRUE))
    )

    result <- fit(
        data = df,
        outcome = "y",
        predictors = c("sex", "sexuality"),
        model_type = "glm",
        family = "binomial"
    )

    raw <- attr(result, "raw_data")

    for (pred in c("sex", "sexuality")) {
        sizes <- raw[["n"]][raw[["variable"]] == pred]
        expect_length(sizes, 2L)
        expect_equal(sum(sizes), nrow(df))
    }
})

#' Test Suite for fit()
#' 
#' Comprehensive tests covering all model types, output formatting,
#' interactions, stratification, clustering, weights, and edge cases.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-fit.R")

library(data.table)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Create binary outcome for logistic regression tests
## IMPORTANT: Add to BOTH datasets
clintrial$response <- as.integer(clintrial$os_status == 1 & clintrial$os_months < 24)
clintrial_complete$response <- as.integer(clintrial_complete$os_status == 1 & clintrial_complete$os_months < 24)

## Create integer count outcome for Poisson/quasipoisson/negative binomial tests
## IMPORTANT: Add to BOTH datasets
set.seed(123)
clintrial$count_outcome <- as.integer(ceiling(clintrial$los_days))
clintrial_complete$count_outcome <- as.integer(ceiling(clintrial_complete$los_days))

## Create a weights column for weighted regression tests
set.seed(42)
clintrial$weight <- runif(nrow(clintrial), 0.5, 2.0)
clintrial_complete$weight <- runif(nrow(clintrial_complete), 0.5, 2.0)

## Helper function to check fit_result structure
expect_fit_result <- function(result) {
    expect_s3_class(result, "fit_result")
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("Variable" %in% names(result))
    expect_true("Group" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "model")))
    expect_true(!is.null(attr(result, "raw_data")))
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "predictors")))
    expect_true(!is.null(attr(result, "formula_str")))
    expect_true(!is.null(attr(result, "model_scope")))
    expect_true(!is.null(attr(result, "model_type")))
}



## Helper to verify reference rows
expect_reference_rows <- function(result, vars_with_refs) {
    raw <- attr(result, "raw_data")
    for (v in vars_with_refs) {
        var_rows <- raw[variable == v]
        if (nrow(var_rows) > 1) {
            ## Factor variables should have a reference row
            has_ref <- any(var_rows$reference == "reference", na.rm = TRUE)
            expect_true(has_ref, 
                        info = paste("Expected reference row for factor:", v))
        }
    }
}


## ============================================================================
## SECTION 1: Basic GLM (Logistic Regression) Tests
## ============================================================================

test_that("fit works with basic logistic regression - univariable", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        family = "binomial"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Univariable")
    expect_effect_column(result, "OR")
    expect_pvalue_column(result)
    
    ## Should have single predictor
    expect_equal(length(attr(result, "predictors")), 1)
    
    ## Effect column should say "OR (95% CI)" for univariable
    col_names <- names(result)
    expect_true(any(grepl("^OR \\(95% CI\\)$", col_names)))
})


test_that("fit works with basic logistic regression - multivariable", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking"),
        model_type = "glm",
        family = "binomial"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Multivariable")
    expect_effect_column(result, "aOR")
    expect_pvalue_column(result)
    
    ## Should have multiple predictors
    expect_equal(length(attr(result, "predictors")), 3)
    
    ## Effect column should say "aOR (95% CI)" for multivariable
    col_names <- names(result)
    expect_true(any(grepl("^aOR \\(95% CI\\)$", col_names)))
})


test_that("fit handles factor variables correctly in GLM", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "stage"),
        model_type = "glm",
        family = "binomial",
        reference_rows = TRUE
    )
    
    expect_fit_result(result)
    
    ## Stage is a factor - should have multiple rows
    raw <- attr(result, "raw_data")
    stage_rows <- raw[variable == "stage"]
    expect_true(nrow(stage_rows) >= 2)  # At least reference + one level
    
    ## Check reference row exists
    has_ref <- any(stage_rows$reference == "reference", na.rm = TRUE)
    expect_true(has_ref)
})


test_that("fit reference_rows = FALSE excludes reference rows", {
    
    result_with_ref <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "stage"),
        model_type = "glm",
        reference_rows = TRUE
    )
    
    result_no_ref <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "stage"),
        model_type = "glm",
        reference_rows = FALSE
    )
    
    ## Without reference rows should have fewer rows
    expect_true(nrow(result_no_ref) < nrow(result_with_ref))
    
    ## Check no reference markers in raw data
    raw_no_ref <- attr(result_no_ref, "raw_data")
    if ("reference" %in% names(raw_no_ref)) {
        has_refs <- sum(raw_no_ref$reference == "reference", na.rm = TRUE)
        expect_equal(has_refs, 0)
    }
})


test_that("fit shows n and events columns correctly for GLM", {
    
    ## With n and events
    result_full <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        show_n = TRUE,
        show_events = TRUE
    )
    expect_true("n" %in% names(result_full))
    expect_true("Events" %in% names(result_full))
    
    ## Without n
    result_no_n <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        show_n = FALSE,
        show_events = TRUE
    )
    expect_false("n" %in% names(result_no_n))
    expect_true("Events" %in% names(result_no_n))
    
    ## Without events
    result_no_events <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        show_n = TRUE,
        show_events = FALSE
    )
    expect_true("n" %in% names(result_no_events))
    expect_false("Events" %in% names(result_no_events))
})


test_that("fit respects digits parameter for GLM", {
    
    result_2dig <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        digits = 2
    )
    
    result_3dig <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        digits = 3
    )
    
    ## Extract effect columns
    effect_col_2 <- grep("OR.*CI", names(result_2dig), value = TRUE)[1]
    effect_col_3 <- grep("OR.*CI", names(result_3dig), value = TRUE)[1]
    
    ## 2-digit result should have pattern like "1.02 (0.99-1.05)"
    ## 3-digit result should have pattern like "1.023 (0.993-1.053)"
    val_2 <- result_2dig[[effect_col_2]][1]
    val_3 <- result_3dig[[effect_col_3]][1]
    
    ## Count decimal places in the first number
    first_num_2 <- sub(" .*", "", val_2)
    first_num_3 <- sub(" .*", "", val_3)
    
    decimals_2 <- nchar(sub(".*\\.", "", first_num_2))
    decimals_3 <- nchar(sub(".*\\.", "", first_num_3))
    
    expect_equal(decimals_2, 2)
    expect_equal(decimals_3, 3)
})


test_that("fit respects p_digits parameter", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        p_digits = 4
    )
    
    ## P-values should have up to 4 decimal places
    pvals <- result$`p-value`
    numeric_pvals <- pvals[grepl("^0\\.", pvals)]
    
    if (length(numeric_pvals) > 0) {
        ## Check that numeric p-values have 4 decimal places
        decimals <- sapply(numeric_pvals, function(p) {
            nchar(sub("0\\.", "", p))
        })
        expect_true(all(decimals == 4))
    }
})


## ============================================================================
## SECTION 2: Linear Model Tests
## ============================================================================

test_that("fit works with linear regression - univariable", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = "age",
        model_type = "lm"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Univariable")
    expect_equal(attr(result, "model_type"), "Linear")
    
    ## Linear models use Coefficient, not OR
    expect_effect_column(result, "Coefficient")
    
    ## Should NOT have Events column
    expect_false("Events" %in% names(result))
})


test_that("fit works with linear regression - multivariable", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "stage"),
        model_type = "lm"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Multivariable")
    expect_effect_column(result, "Coefficient")
})


test_that("fit linear model show_events is ignored", {
    
    ## Even if show_events = TRUE, linear models shouldn't show events
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "lm",
        show_events = TRUE
    )
    
    expect_false("Events" %in% names(result))
})


test_that("fit linear model QC stats are captured", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "stage"),
        model_type = "lm",
        keep_qc_stats = TRUE
    )
    
    raw <- attr(result, "raw_data")
    
    ## Linear models should have R-squared and other stats
    expect_true("AIC" %in% names(raw))
    expect_true("BIC" %in% names(raw))
})


## ============================================================================
## SECTION 3: Cox Proportional Hazards Tests
## ============================================================================

test_that("fit works with Cox regression - univariable", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = "age",
        model_type = "coxph"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Univariable")
    expect_effect_column(result, "HR")
    
    ## Should have Events column
    expect_true("Events" %in% names(result))
})


test_that("fit works with Cox regression - multivariable", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "stage", "treatment"),
        model_type = "coxph"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Multivariable")
    
    ## Multivariable Cox should show aHR
    col_names <- names(result)
    expect_true(any(grepl("aHR", col_names)))
})


test_that("fit Cox model with stratification", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "treatment"),
        model_type = "coxph",
        strata = "sex"
    )
    
    expect_fit_result(result)
    
    ## Check strata is stored in attributes
    expect_equal(attr(result, "strata"), "sex")
    
    ## Formula should include strata()
    formula_str <- attr(result, "formula_str")
    expect_true(grepl("strata\\(.*sex.*\\)", formula_str))
    
    ## Sex should NOT appear as a predictor in the output
    ## (it's used for stratification, not estimation)
    raw <- attr(result, "raw_data")
    expect_false("sex" %in% raw$variable)
})


test_that("fit Cox model with clustering", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "treatment"),
        model_type = "coxph",
        cluster = "site"
    )
    
    expect_fit_result(result)
    
    ## Check cluster is stored in attributes
    expect_equal(attr(result, "cluster"), "site")
    
    ## Model should have robust standard errors
    model <- attr(result, "model")
    expect_true(!is.null(model$naive.var) || !is.null(attr(model, "cluster")))
})


test_that("fit Cox model with both strata and cluster", {
    
    ## Create a second grouping variable for stratification
    clintrial_test <- data.table::as.data.table(clintrial)
    set.seed(2024)
    clintrial_test[, region := sample(c("North", "South"), .N, replace = TRUE)]
    
    result <- fit(
        data = clintrial_test,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "treatment"),
        model_type = "coxph",
        strata = "region",
        cluster = "site"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "strata"), "region")
    expect_equal(attr(result, "cluster"), "site")
})


## ============================================================================
## SECTION 4: Poisson Regression Tests
## ============================================================================

test_that("fit works with Poisson regression", {
    
    ## Create count outcome
    clintrial_test <- data.table::as.data.table(clintrial)
    set.seed(2024)
    clintrial_test[, complications := rpois(.N, lambda = 2)]
    
    result <- fit(
        data = clintrial_test,
        outcome = "complications",
        predictors = c("age", "sex", "stage"),
        model_type = "glm",
        family = "poisson"
    )
    
    expect_fit_result(result)
    
    ## Poisson models should show RR (rate ratio)
    expect_effect_column(result, "RR")
})


## ============================================================================
## SECTION 5: Interaction Terms Tests
## ============================================================================

test_that("fit handles basic interaction terms", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        interactions = c("age:sex"),
        model_type = "glm"
    )
    
    expect_fit_result(result)
    
    ## Check interactions stored in attributes
    expect_equal(attr(result, "interactions"), c("age:sex"))
    
    ## Formula should include interaction
    formula_str <- attr(result, "formula_str")
    expect_true(grepl("age:sex", formula_str))
    
    ## Raw data should have interaction term rows
    raw <- attr(result, "raw_data")
    interaction_rows <- raw[grepl(":", variable)]
    expect_true(nrow(interaction_rows) >= 1)
})


test_that("fit handles multiple interaction terms", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "treatment"),
        interactions = c("age:sex", "age:treatment"),
        model_type = "glm"
    )
    
    expect_fit_result(result)
    
    formula_str <- attr(result, "formula_str")
    expect_true(grepl("age:sex", formula_str))
    expect_true(grepl("age:treatment", formula_str))
})

## ============================================================================
## SECTION 6: Weighted Regression Tests
## ============================================================================

test_that("fit handles weights in GLM", {
    
    ## Suppress expected "non-integer #successes" warning from weighted binomial
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        weights = "weight"
    ))
    
    expect_fit_result(result)
    expect_equal(attr(result, "weights"), "weight")
    
    ## Model should have weights
    model <- attr(result, "model")
    expect_true(!is.null(model$prior.weights))
})


test_that("fit handles weights in linear model", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "lm",
        weights = "weight"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "weights"), "weight")
})


test_that("weighted vs unweighted models produce different results", {
    
    result_unweighted <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    ## Suppress expected "non-integer #successes" warning from weighted binomial
    result_weighted <- suppressWarnings(fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        weights = "weight"
    ))
    
    ## Get coefficients from models
    coef_unweighted <- coef(attr(result_unweighted, "model"))
    coef_weighted <- coef(attr(result_weighted, "model"))
    
    ## Coefficients should be different (not identical)
    expect_false(all(abs(coef_unweighted - coef_weighted) < 1e-10))
})


## ============================================================================
## SECTION 7: Custom Labels Tests
## ============================================================================

test_that("fit applies custom labels to variables", {
    
    custom_labels <- c(
        age = "Age (years)",
        sex = "Sex",
        stage = "Cancer Stage"
    )
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        model_type = "glm",
        labels = custom_labels
    )
    
    ## Check that labels are applied
    ## Variable column should contain labeled names
    variables <- unique(result$Variable[result$Variable != ""])
    
    expect_true("Age (years)" %in% variables || any(grepl("Age", variables)))
    expect_true("Sex" %in% variables || any(grepl("Sex", variables)))
    expect_true("Cancer Stage" %in% variables || any(grepl("Stage", variables)))
})


test_that("fit applies labels to interaction terms", {
    
    custom_labels <- c(
        age = "Age",
        sex = "Sex"
    )
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        interactions = c("age:sex"),
        model_type = "glm",
        labels = custom_labels
    )
    
    ## Interaction should show labeled variables with " * " separator
    variables <- result$Variable
    
    ## Should contain interaction with labeled names
    has_labeled_interaction <- any(grepl("Age.*Sex|Sex.*Age", variables))
    expect_true(has_labeled_interaction)
})


test_that("fit labels work with clintrial_labels dataset", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage", "treatment"),
        model_type = "glm",
        labels = clintrial_labels
    )
    
    expect_fit_result(result)
    
    ## Labels should be applied
    variables <- result$Variable[result$Variable != ""]
    
    ## Check some expected labels exist
    expect_true(length(variables) > 0)
})


## ============================================================================
## SECTION 8: Confidence Level Tests
## ============================================================================

test_that("fit respects conf_level parameter", {
    
    result_95 <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        conf_level = 0.95
    )
    
    result_90 <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        conf_level = 0.90
    )
    
    ## Get raw data to compare CI widths
    raw_95 <- attr(result_95, "raw_data")
    raw_90 <- attr(result_90, "raw_data")
    
    ## 90% CI should be narrower than 95% CI
    ci_width_95 <- raw_95$exp_upper[1] - raw_95$exp_lower[1]
    ci_width_90 <- raw_90$exp_upper[1] - raw_90$exp_lower[1]
    
    expect_true(ci_width_90 < ci_width_95)
})


test_that("fit works with 99% confidence interval", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        conf_level = 0.99
    )
    
    expect_fit_result(result)
    
    ## 99% CI column should be named appropriately or at least exist
    col_names <- names(result)
    expect_true(any(grepl("CI", col_names)))
})


## ============================================================================
## SECTION 9: Exponentiate Parameter Tests
## ============================================================================

test_that("fit auto-exponentiates for logistic regression", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        family = "binomial"
    )
    
    ## Should show OR, not raw coefficients
    col_names <- names(result)
    expect_true(any(grepl("OR", col_names)))
})


test_that("fit exponentiate = FALSE shows raw coefficients", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = "age",
        model_type = "glm",
        family = "binomial",
        exponentiate = FALSE
    )
    
    ## Should show Coefficient, not OR
    col_names <- names(result)
    expect_true(any(grepl("Coefficient", col_names)))
    expect_false(any(grepl("\\bOR\\b", col_names)))
})


test_that("fit exponentiate = TRUE forces OR even for linear model", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = "age",
        model_type = "lm",
        exponentiate = TRUE
    )
    
    ## Should show exponentiated values
    raw <- attr(result, "raw_data")
    
    ## exp_coef should exist in raw data
    expect_true("exp_coef" %in% names(raw))
})


## ============================================================================
## SECTION 10: Mixed Effects Models Tests
## ============================================================================

test_that("fit works with lmer (linear mixed effects)", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    
    gc()  # Force garbage collection before lme4
    
    ## Use smaller dataset for stability
    test_data <- clintrial_complete[1:200, ]
    
    result <- fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex", "(1|site)"),
        model_type = "lmer"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_type"), "Linear Mixed")
    
    ## Model should be lmerMod
    model <- attr(result, "model")
    expect_true(inherits(model, "lmerMod"))
})


test_that("fit works with glmer (generalized linear mixed effects)", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()
    
    gc()
    
    test_data <- clintrial_complete[1:200, ]
    
    result <- fit(
        data = test_data,
        outcome = "response",
        predictors = c("age", "sex", "(1|site)"),
        model_type = "glmer",
        family = "binomial"
    )
    
    expect_fit_result(result)
    
    ## Model should be glmerMod
    model <- attr(result, "model")
    expect_true(inherits(model, "glmerMod"))
})


test_that("fit works with coxme (mixed effects Cox)", {
    
    skip_if_not_installed("coxme")
    skip_on_cran()
    
    gc()
    
    test_data <- clintrial_complete[1:200, ]
    
    result <- fit(
        data = test_data,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "(1|site)"),
        model_type = "coxme"
    )
    
    expect_fit_result(result)
    
    ## Model should be coxme
    model <- attr(result, "model")
    expect_true(inherits(model, "coxme"))
})


## ============================================================================
## SECTION 11: Conditional Logistic Regression Tests
## ============================================================================

test_that("fit works with conditional logistic regression", {
    
    ## Create matched case-control data
    set.seed(123)
    n_strata <- 50
    matched_data <- data.table(
        stratum = rep(1:n_strata, each = 3),
        case = rep(c(1, 0, 0), n_strata),
        age = rnorm(n_strata * 3, 60, 10),
        exposure = rbinom(n_strata * 3, 1, 0.3)
    )
    
    result <- fit(
        data = matched_data,
        outcome = "case",
        predictors = c("age", "exposure"),
        model_type = "clogit",
        strata = "stratum"
    )
    
    expect_fit_result(result)
    expect_equal(attr(result, "strata"), "stratum")
    
    ## Conditional logistic shows effect estimates (could be OR or HR depending on implementation)
    expect_effect_column(result)
})


## ============================================================================
## SECTION 12: QC Stats and Raw Data Tests
## ============================================================================

test_that("fit keeps QC stats when requested", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        model_type = "glm",
        keep_qc_stats = TRUE
    )
    
    raw <- attr(result, "raw_data")
    
    ## Should have QC statistics
    expect_true("AIC" %in% names(raw))
    expect_true("BIC" %in% names(raw))
    expect_true("n" %in% names(raw))
    expect_true("events" %in% names(raw))
})


test_that("fit raw_data has correct structure", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    raw <- attr(result, "raw_data")
    
    ## Check required columns in raw data
    expect_true("variable" %in% names(raw))
    expect_true("coef" %in% names(raw))
    expect_true("se" %in% names(raw))
    expect_true("p_value" %in% names(raw))
    expect_true("exp_coef" %in% names(raw))  # For GLM
})


test_that("fit model object is accessible and valid", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    model <- attr(result, "model")
    
    ## Model should be a valid glm object
    expect_s3_class(model, "glm")
    
    ## Should be able to extract coefficients
    expect_true(length(coef(model)) > 0)
    
    ## Should be able to get predictions
    expect_true(length(predict(model)) > 0)
})


## ============================================================================
## SECTION 13: Edge Cases and Error Handling
## ============================================================================

test_that("fit handles missing data appropriately", {
    
    ## clintrial has some missing values
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking"),  # smoking may have NAs
        model_type = "glm"
    )
    
    expect_fit_result(result)
    
    ## N should reflect complete cases
    raw <- attr(result, "raw_data")
    expect_true(raw$n[1] > 0)
    expect_true(raw$n[1] <= nrow(clintrial))
})


test_that("fit works with single-level factors appropriately", {
    
    ## Create data with a near-single-level factor
    test_data <- data.table::as.data.table(clintrial)
    test_data[, constant_var := factor("A")]
    
    ## This should either work or give an informative error
    ## depending on how the model handles it
    result_or_error <- tryCatch({
        fit(
            data = test_data,
            outcome = "response",
            predictors = c("age", "constant_var"),
            model_type = "glm"
        )
    }, error = function(e) e)
    
    ## Either succeeds or gives an error (both are acceptable)
    expect_true(inherits(result_or_error, "fit_result") || 
                inherits(result_or_error, "error"))
})


test_that("fit errors on invalid model_type", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = "age",
            model_type = "invalid_model_type"
        ),
        regexp = "[Uu]nsupported model type"
    )
})


test_that("fit errors when required package missing for coxph", {
    
    ## This test verifies the error message when survival isn't loaded
    ## but since it's typically available, we just verify the check exists
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = "age",
        model_type = "coxph"
    )
    
    expect_fit_result(result)
})


test_that("fit handles empty predictors appropriately", {
    
    ## Empty predictors should cause an error
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = character(0),
            model_type = "glm"
        )
    )
})


## ============================================================================
## SECTION 13B: Input Validation Tests
## ============================================================================

test_that("fit auto-corrects Surv outcome with GLM model_type", {
    
    ## Should auto-switch to coxph and emit a message
    expect_message(
        result <- fit(
            data = clintrial,
            outcome = "Surv(os_months, os_status)",
            predictors = c("age", "sex"),
            model_type = "glm"
        ),
        regexp = "[Ss]witch|[Ss]urvival"
    )
    
    expect_fit_result(result)
    ## Model type attribute stores display name ("Cox PH"), not input parameter
    expect_true(grepl("[Cc]ox", attr(result, "model_type")))
})


test_that("fit auto-corrects Surv outcome with lm model_type", {
    
    ## Should auto-switch to coxph and emit a message
    expect_message(
        result <- fit(
            data = clintrial,
            outcome = "Surv(os_months, os_status)",
            predictors = c("age", "sex"),
            model_type = "lm"
        ),
        regexp = "[Ss]witch|[Ss]urvival"
    )
    
    expect_fit_result(result)
    expect_true(grepl("[Cc]ox", attr(result, "model_type")))
})


test_that("fit errors when coxph used without Surv outcome", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "coxph"
        ),
        regexp = "[Ss]urv|[Ss]urvival"
    )
})


test_that("fit errors when coxme used without Surv outcome", {
    
    skip_if_not_installed("coxme")
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex", "(1|site)"),
            model_type = "coxme"
        ),
        regexp = "[Ss]urv|[Ss]urvival"
    )
})


test_that("fit errors with continuous outcome and binomial family", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "los_days",
            predictors = c("age", "sex"),
            model_type = "glm",
            family = "binomial"
        ),
        regexp = "[Cc]ontinuous|binomial"
    )
})


test_that("fit warns with binary outcome and gaussian family", {
    
    expect_warning(
        result <- fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "glm",
            family = "gaussian"
        ),
        regexp = "[Bb]inary|binomial|gaussian"
    )
    
    expect_fit_result(result)
})


test_that("fit warns with binary outcome and lm model_type", {
    
    expect_warning(
        result <- fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "lm"
        ),
        regexp = "[Bb]inary|logistic|binomial"
    )
    
    expect_fit_result(result)
})


test_that("fit errors when outcome variable not found", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "nonexistent_variable",
            predictors = c("age", "sex"),
            model_type = "glm"
        ),
        regexp = "not found|[Oo]utcome"
    )
})


test_that("fit errors when Surv variables not found", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "Surv(nonexistent_time, nonexistent_status)",
            predictors = c("age", "sex"),
            model_type = "coxph"
        ),
        regexp = "not found|[Ss]urvival"
    )
})


test_that("fit errors when predictor variable not found", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "nonexistent_predictor"),
            model_type = "glm"
        ),
        regexp = "not found|[Pp]redictor"
    )
})


test_that("fit errors with invalid conf_level", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "glm",
            conf_level = 1.5
        ),
        regexp = "conf_level|between 0 and 1"
    )
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "glm",
            conf_level = 0
        ),
        regexp = "conf_level|between 0 and 1"
    )
})


test_that("fit errors with invalid digits parameter", {
    
    expect_error(
        fit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "glm",
            digits = -1
        ),
        regexp = "digits|non-negative"
    )
})


test_that("fit errors with empty data", {
    
    empty_data <- clintrial[0, ]
    
    expect_error(
        fit(
            data = empty_data,
            outcome = "response",
            predictors = c("age", "sex"),
            model_type = "glm"
        ),
        regexp = "empty|no rows|non-empty"
    )
})


test_that("fit handles very long predictor lists", {
    
    ## Test with many predictors
    many_predictors <- c("age", "sex", "smoking", "diabetes", "hypertension",
                         "stage", "grade", "ecog", "treatment", "surgery")
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = many_predictors,
        model_type = "glm"
    )
    
    expect_fit_result(result)
    expect_equal(length(attr(result, "predictors")), length(many_predictors))
})


## ============================================================================
## SECTION 14: Print Method Tests
## ============================================================================

test_that("print.fit_result produces output", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    ## Capture print output
    output <- capture.output(print(result))
    
    ## Should contain model information. The sample size is carried by the
    ## "Observations analyzed" line, which states it with its denominator
    ## rather than as a bare count.
    expect_true(any(grepl("Multivariable|Univariable", output)))
    expect_true(any(grepl("Formula", output)))
    expect_true(any(grepl("Observations analyzed:", output, fixed = TRUE)))
})


test_that("print.fit_result shows events for survival models", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex"),
        model_type = "coxph"
    )
    
    output <- capture.output(print(result))
    
    ## Should show Events
    expect_true(any(grepl("Events", output)))
})


## ============================================================================
## SECTION 15: Integration and Consistency Tests
## ============================================================================

test_that("fit produces consistent results across runs", {
    
    result1 <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    result2 <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    ## Results should be identical
    expect_equal(attr(result1, "raw_data")$coef, attr(result2, "raw_data")$coef)
    expect_equal(attr(result1, "raw_data")$p_value, attr(result2, "raw_data")$p_value)
})


test_that("fit model coefficients match direct model fitting", {
    
    ## Fit using fit()
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial"
    )
    
    ## Fit directly
    direct_model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("fit works with data.frame input (not just data.table)", {
    
    df <- as.data.frame(clintrial)
    
    result <- fit(
        data = df,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    expect_fit_result(result)
})


test_that("fit preserves factor level ordering", {
    
    ## Create data with ordered factor
    test_data <- data.table::as.data.table(clintrial)
    test_data[, stage_ordered := factor(stage, levels = c("I", "II", "III", "IV"), 
                                        ordered = TRUE)]
    
    result <- fit(
        data = test_data,
        outcome = "response",
        predictors = c("age", "stage_ordered"),
        model_type = "glm"
    )
    
    expect_fit_result(result)
    
    ## Check that stage levels appear in correct order in output
    raw <- attr(result, "raw_data")
    stage_rows <- raw[variable == "stage_ordered"]
    
    if (nrow(stage_rows) > 1) {
        ## Groups should be in factor order
        groups <- stage_rows$group[!is.na(stage_rows$group) & stage_rows$group != ""]
        
        ## Reference should be first level (I), so we should see II, III, IV
        expect_true(length(groups) >= 1)
    }
})


## ============================================================================
## SECTION 16: Different GLM Families Tests
## ============================================================================

## ----------------------------------------------------------------------------
## 16a: Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("fit works with gaussian family", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "gaussian"
    )
    
    expect_fit_result(result)
    
    ## Should produce coefficients (not exponentiated)
    col_names <- names(result)
    expect_true(any(grepl("Coefficient.*CI|Estimate.*CI", col_names)),
                info = paste("Expected Coefficient column for gaussian, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fit gaussian coefficients match direct glm", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "gaussian"
    )
    
    ## Fit directly
    direct_model <- glm(los_days ~ age + sex, data = clintrial, family = gaussian)
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("fit gaussian matches lm results", {
    
    result_glm <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "gaussian"
    )
    
    result_lm <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "lm"
    )
    
    ## Coefficients should be essentially identical
    coefs_glm <- coef(attr(result_glm, "model"))
    coefs_lm <- coef(attr(result_lm, "model"))
    
    expect_equal(coefs_glm, coefs_lm, tolerance = 1e-10)
})


## ----------------------------------------------------------------------------
## 16b: Quasibinomial Family Tests (overdispersed binary)
## ----------------------------------------------------------------------------

test_that("fit works with quasibinomial family", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        family = "quasibinomial"
    )
    
    expect_fit_result(result)
    
    ## Should produce odds ratios like binomial
    col_names <- names(result)
    expect_true(any(grepl("OR.*CI", col_names)),
                info = paste("Expected OR column for quasibinomial, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fit quasibinomial coefficients match direct glm", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "quasibinomial"
    )
    
    ## Fit directly
    direct_model <- glm(response ~ age + sex, data = clintrial, family = quasibinomial)
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("fit quasibinomial point estimates match binomial", {
    
    result_quasi <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "quasibinomial"
    )
    
    result_binom <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial"
    )
    
    ## Point estimates should be identical (SEs differ due to dispersion)
    coefs_quasi <- coef(attr(result_quasi, "model"))
    coefs_binom <- coef(attr(result_binom, "model"))
    
    expect_equal(coefs_quasi, coefs_binom, tolerance = 1e-10)
})


test_that("fit quasibinomial handles overdispersion", {
    
    ## Quasibinomial allows dispersion != 1
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "glm",
        family = "quasibinomial"
    )
    
    model <- attr(result, "model")
    
    ## Dispersion parameter should be estimated
    summ <- summary(model)
    expect_true(!is.null(summ$dispersion))
})


## ----------------------------------------------------------------------------
## 16c: Quasipoisson Family Tests (overdispersed counts)
## ----------------------------------------------------------------------------

test_that("fit works with quasipoisson family", {
    
    result <- fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        family = "quasipoisson"
    )
    
    expect_fit_result(result)
    
    ## Should produce rate ratios like poisson
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for quasipoisson, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fit quasipoisson coefficients match direct glm", {
    
    result <- fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "quasipoisson"
    )
    
    ## Fit directly
    direct_model <- glm(count_outcome ~ age + sex, data = clintrial, family = quasipoisson)
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("fit quasipoisson point estimates match poisson", {
    
    result_quasi <- fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "quasipoisson"
    )
    
    result_pois <- fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "poisson"
    )
    
    ## Point estimates should be identical (SEs differ due to dispersion)
    coefs_quasi <- coef(attr(result_quasi, "model"))
    coefs_pois <- coef(attr(result_pois, "model"))
    
    expect_equal(coefs_quasi, coefs_pois, tolerance = 1e-10)
})


test_that("fit quasipoisson handles overdispersion", {
    
    result <- fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "glm",
        family = "quasipoisson"
    )
    
    model <- attr(result, "model")
    
    ## Dispersion parameter should be estimated
    summ <- summary(model)
    expect_true(!is.null(summ$dispersion))
    expect_true(summ$dispersion != 1)  ## Should differ from 1 for overdispersed data
})


## ----------------------------------------------------------------------------
## 16d: Gamma Family Tests (positive continuous)
## ----------------------------------------------------------------------------

test_that("fit works with Gamma family (log link)", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = Gamma(link = "log")
    ))
    
    expect_fit_result(result)
    
    ## With log link, should produce ratios (exponentiated)
    col_names <- names(result)
    expect_true(any(grepl("Ratio.*CI|Coefficient.*CI", col_names)),
                info = paste("Expected Ratio or Coefficient column for Gamma, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fit works with Gamma family (inverse link - default)", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = Gamma()  ## Default inverse link
    ))
    
    expect_fit_result(result)
})


test_that("fit Gamma coefficients match direct glm", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = Gamma(link = "log")
    ))
    
    ## Fit directly
    direct_model <- glm(los_days ~ age + sex, data = test_data, family = Gamma(link = "log"))
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("fit Gamma handles factor variables", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "stage"),
        model_type = "glm",
        family = Gamma(link = "log"),
        reference_rows = TRUE
    ))
    
    expect_fit_result(result)
    
    ## Stage should have reference row
    raw <- attr(result, "raw_data")
    stage_rows <- raw[variable == "stage"]
    expect_true(nrow(stage_rows) >= 2)
})


## ----------------------------------------------------------------------------
## 16e: Inverse Gaussian Family Tests (positive, right-skewed)
## ----------------------------------------------------------------------------

test_that("fit works with inverse.gaussian family", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = inverse.gaussian()
    ))
    
    expect_fit_result(result)
})


test_that("fit inverse.gaussian with log link works", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = inverse.gaussian(link = "log")
    ))
    
    expect_fit_result(result)
    
    ## With log link, should produce ratios
    col_names <- names(result)
    expect_true(any(grepl("Ratio.*CI|Coefficient.*CI", col_names)),
                info = paste("Expected Ratio or Coefficient column, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fit inverse.gaussian coefficients match direct glm", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = inverse.gaussian(link = "log")
    ))
    
    ## Fit directly
    direct_model <- glm(los_days ~ age + sex, data = test_data, 
                        family = inverse.gaussian(link = "log"))
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


## ----------------------------------------------------------------------------
## 16f: Negative Binomial (negbin) Tests
## ----------------------------------------------------------------------------

test_that("fit works with negative binomial regression (negbin)", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "negbin"
    ))
    
    expect_fit_result(result)
    
    ## Should produce rate ratios
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for negative binomial, found:", 
                             paste(col_names, collapse = ", ")))
    
    ## Model should be of class negbin
    model <- attr(result, "model")
    expect_true(inherits(model, "negbin"))
})


test_that("fit negbin produces correct effect estimates", {
    
    skip_if_not_installed("MASS")
    
    ## Fit using fit()
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex"),
        model_type = "negbin"
    ))
    
    ## Fit directly with MASS
    direct_model <- suppressWarnings(MASS::glm.nb(count_outcome ~ age + sex, data = clintrial))
    
    ## Coefficients should match
    fit_coefs <- coef(attr(result, "model"))
    direct_coefs <- coef(direct_model)
    
    expect_equal(fit_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("fit negbin works with multivariable models", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "negbin"
    ))
    
    expect_fit_result(result)
    expect_equal(attr(result, "model_scope"), "Multivariable")
    
    ## Should have aRR column for multivariable
    col_names <- names(result)
    expect_true(any(grepl("aRR.*CI", col_names)),
                info = paste("Expected aRR column for multivariable negbin, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fit negbin handles factor variables correctly", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "stage"),
        model_type = "negbin",
        reference_rows = TRUE
    ))
    
    expect_fit_result(result)
    
    ## Stage is a factor - should have multiple rows with reference
    raw <- attr(result, "raw_data")
    stage_rows <- raw[variable == "stage"]
    expect_true(nrow(stage_rows) >= 2)
    
    ## Check reference row exists
    has_ref <- any(stage_rows$reference == "reference", na.rm = TRUE)
    expect_true(has_ref)
})


test_that("fit negbin respects formatting parameters", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex"),
        model_type = "negbin",
        digits = 3,
        p_digits = 4,
        show_n = TRUE,
        show_events = FALSE
    ))
    
    expect_fit_result(result)
    expect_true("n" %in% names(result))
    expect_false("Events" %in% names(result))
})


test_that("fit negbin works with labels", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "negbin",
        labels = clintrial_labels
    ))
    
    expect_fit_result(result)
    
    ## Labels should be applied
    expect_true(any(grepl("Age|Sex|Treatment", result$Variable)))
})


test_that("fit negbin works with interactions", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        interactions = c("age:sex"),
        model_type = "negbin"
    ))
    
    expect_fit_result(result)
    
    ## Interaction should be in formula
    formula_str <- attr(result, "formula_str")
    expect_true(grepl("age:sex", formula_str))
})


test_that("fit with pre-fitted model works for negbin", {
    
    skip_if_not_installed("MASS")
    
    ## Fit model directly with MASS
    nb_model <- suppressWarnings(MASS::glm.nb(count_outcome ~ age + sex + treatment, data = clintrial))
    
    ## Pass to fit()
    result <- fit(
        model = nb_model,
        data = clintrial,
        labels = clintrial_labels
    )
    
    ## Check basic structure (without outcome/predictors which aren't available for pre-fitted models)
    expect_s3_class(result, "fit_result")
    expect_s3_class(result, "data.table")
    expect_true("Variable" %in% names(result))
    expect_true("Group" %in% names(result))
    expect_true(!is.null(attr(result, "model")))
    expect_true(!is.null(attr(result, "raw_data")))
    
    ## Should recognize it as negative binomial
    model_type <- attr(result, "model_type")
    expect_true(grepl("Negative Binomial", model_type))
})


## ============================================================================
## SECTION 17: Model Attribute Preservation Tests
## ============================================================================

test_that("fit preserves all specified attributes", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        interactions = c("age:sex"),
        model_type = "glm",
        family = "binomial"
    )
    
    ## Check all attributes are preserved
    expect_equal(attr(result, "outcome"), "response")
    expect_equal(attr(result, "predictors"), c("age", "sex", "stage"))
    expect_equal(attr(result, "interactions"), c("age:sex"))
    
    ## Formula should be correct
    formula_str <- attr(result, "formula_str")
    expect_true(grepl("response", formula_str))
    expect_true(grepl("age", formula_str))
    expect_true(grepl("sex", formula_str))
    expect_true(grepl("stage", formula_str))
    expect_true(grepl("age:sex", formula_str))
})


test_that("fit stores data in model object", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    model <- attr(result, "model")
    
    ## Data should be accessible from model
    model_data <- attr(model, "data")
    expect_true(!is.null(model_data) || !is.null(model$data))
})


## ============================================================================
## SECTION 18: Performance and Large Data Tests
## ============================================================================

test_that("fit handles moderately large datasets", {
    
    ## Create larger dataset by replicating
    large_data <- rbindlist(rep(list(clintrial), 10))
    large_data[, id := .I]  # Add unique ID
    
    result <- fit(
        data = large_data,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    expect_fit_result(result)
    
    ## N should reflect larger dataset
    raw <- attr(result, "raw_data")
    expect_true(raw$n[1] > nrow(clintrial))
})


## ============================================================================
## SECTION 19: Numeric Precision Tests
## ============================================================================

test_that("fit handles extreme coefficient values", {
    
    ## Create data that might produce extreme coefficients
    test_data <- data.table::as.data.table(clintrial)
    test_data[, tiny_var := age / 10000]  # Very small scale
    
    result <- fit(
        data = test_data,
        outcome = "response",
        predictors = c("tiny_var", "sex"),
        model_type = "glm"
    )
    
    expect_fit_result(result)
    
    ## Should still produce valid output
    raw <- attr(result, "raw_data")
    expect_true(all(!is.na(raw$coef)))
})


test_that("fit handles perfect separation gracefully", {
    ## Create data with near-perfect separation
    set.seed(2024)
    test_data <- data.table(
        outcome = c(rep(0, 50), rep(1, 50)),
        predictor = c(rep(0, 50), rep(1, 50)),
        noise = rnorm(100)
    )
    
    ## This might produce warnings about convergence
    result <- suppressWarnings(
        fit(
            data = test_data,
            outcome = "outcome",
            predictors = c("predictor", "noise"),
            model_type = "glm",
            family = "binomial"
        )
    )
    
    ## Should still return a valid result
    expect_fit_result(result)
})


## ============================================================================
## SECTION 20: Formula String Validation Tests
## ============================================================================

test_that("fit formula_str is valid R formula", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        interactions = c("age:sex"),
        model_type = "glm"
    )
    
    formula_str <- attr(result, "formula_str")
    
    ## Should be parseable as formula
    parsed_formula <- as.formula(formula_str)
    expect_s3_class(parsed_formula, "formula")
})


test_that("fit formula includes all components correctly", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "treatment"),
        model_type = "coxph",
        strata = "sex"
    )
    
    formula_str <- attr(result, "formula_str")
    
    ## Check all components present
    expect_true(grepl("Surv\\(os_months, os_status\\)", formula_str))
    expect_true(grepl("age", formula_str))
    expect_true(grepl("treatment", formula_str))
    expect_true(grepl("strata\\(.*sex.*\\)", formula_str))
})


## ============================================================================
## SECTION 22: Profile Likelihood CI and Caching
## ============================================================================

test_that("fit propagates cached confint to model attribute", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    model <- attr(result, "model")
    
    ## Model should carry cached confint from m2dt
    cached <- attr(model, "cached_confint")
    expect_true(!is.null(cached),
                info = "fit() should propagate cached_confint to model object")
    expect_true(is.matrix(cached))
    expect_equal(attr(model, "cached_confint_level"), 0.95)
})


test_that("fit cached confint matches stats::confint output", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    model <- attr(result, "model")
    cached <- attr(model, "cached_confint")
    
    ## Compute confint directly for comparison
    direct_ci <- suppressMessages(suppressWarnings(
        stats::confint(model, level = 0.95)
    ))
    
    ## Cached values should match direct computation
    ## (non-NA rows; intercept row may be NA if skipped)
    shared_rows <- intersect(rownames(cached)[!is.na(cached[, 1])],
                             rownames(direct_ci))
    expect_true(length(shared_rows) > 0)
    
    for (rn in shared_rows) {
        expect_equal(cached[rn, 1], direct_ci[rn, 1], tolerance = 1e-10,
                     info = paste("Cached CI lower mismatch for", rn))
        expect_equal(cached[rn, 2], direct_ci[rn, 2], tolerance = 1e-10,
                     info = paste("Cached CI upper mismatch for", rn))
    }
})


test_that("fit GLM profile CIs differ from Wald", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        model_type = "glm"
    )
    
    raw <- attr(result, "raw_data")
    non_ref <- raw[is.na(reference) | reference == ""]
    
    ## Compute Wald CIs
    model <- attr(result, "model")
    coefs <- coef(model)[-1]
    ses <- summary(model)$coefficients[-1, "Std. Error"]
    z <- qnorm(0.975)
    wald_lower <- coefs - z * ses
    
    ## Profile should differ from Wald for at least one term
    any_differs <- any(abs(non_ref$coef_lower - wald_lower) > 1e-6)
    expect_true(any_differs,
                info = "fit() GLM CIs should use profile likelihood, not Wald")
})


test_that("fit lm CIs use exact t-distribution", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "lm"
    )
    
    raw <- attr(result, "raw_data")
    non_ref <- raw[is.na(reference) | reference == ""]
    
    ## Exact t CIs should match confint.lm() output
    model <- attr(result, "model")
    direct_ci <- confint(model, level = 0.95)
    ## Remove intercept
    direct_ci <- direct_ci[rownames(direct_ci) != "(Intercept)", , drop = FALSE]
    
    expect_equal(unname(non_ref$coef_lower), unname(direct_ci[, 1]), tolerance = 1e-10,
                 info = "lm CIs should match confint.lm()")
})

## The print methods disclose how much of the supplied data reached the model.
## The line is unconditional: a complete sample is stated rather than left to be
## distinguished from an absent disclosure.

test_that("fit() records the analyzed sample and the reason for exclusions", {

    dt <- with_missing_outcome("readmission_30d", 100L)

    result <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial"
    )

    counts <- attr(result, "analysis_counts")

    expect_equal(counts$n_supplied, nrow(dt))
    expect_equal(counts$n_analyzed, as.numeric(stats::nobs(attr(result, "model"))))
    expect_equal(counts$n_missing_outcome, 100)
    expect_equal(counts$n_analyzed + counts$n_missing_outcome +
                 counts$n_missing_predictor, counts$n_supplied)
})


test_that("print methods report the analyzed sample when observations are excluded", {

    dt <- with_missing_outcome("readmission_30d", 100L)

    result <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial"
    )

    counts <- attr(result, "analysis_counts")
    expected <- sprintf("Observations analyzed: %d of %d",
                        as.integer(counts$n_analyzed),
                        as.integer(counts$n_supplied))

    expect_output(print(result), expected, fixed = TRUE)
})


test_that("a complete sample is reported rather than left unstated", {

    ## Reporting only on loss would leave the reader unable to distinguish a
    ## complete sample from an absent disclosure, so the line is unconditional.
    df <- fresh_clintrial()
    keep <- c("readmission_30d", "age", "sex")
    dt <- data.table::as.data.table(df[stats::complete.cases(df[, keep]), keep])

    result <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial"
    )

    counts <- attr(result, "analysis_counts")
    expect_equal(counts$n_analyzed, counts$n_supplied)

    expect_output(print(result),
                  sprintf("Observations analyzed: %d of %d (100.0%%)",
                          as.integer(counts$n_analyzed),
                          as.integer(counts$n_supplied)),
                  fixed = TRUE)
})


test_that("uniscreen reports a range when predictors differ in missingness", {

    df <- fresh_clintrial()
    df$readmission_30d[seq_len(100)] <- NA
    df$bmi[seq_len(150)] <- NA
    dt <- data.table::as.data.table(df)

    result <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "bmi"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    counts <- attr(result, "analysis_counts")

    ## One analyzed count per predictor, and bmi contributes the smaller one
    expect_length(counts$n_analyzed, 2L)
    expect_gt(max(counts$n_analyzed), min(counts$n_analyzed))

    expect_output(print(result), "Observations analyzed: [0-9]+-[0-9]+ of ")
})


test_that("a blank line separates the header from the descriptors", {

    dt <- data.table::as.data.table(fresh_clintrial())

    screened <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    out <- utils::capture.output(print(screened))

    expect_equal(out[2], "Univariable Screening Results")
    expect_equal(out[3], "")
    expect_match(out[4], "^Outcome: ")
})


test_that("fit() print reports observations and events on the same footing", {

    dt <- with_missing_outcome("readmission_30d", 100L)

    result <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial"
    )

    counts <- attr(result, "analysis_counts")
    raw <- attr(result, "raw_data")

    ## The events denominator counts events in the supplied data using the same
    ## coding as the model, so it is never smaller than the events analyzed
    expect_true(!is.null(counts$events_supplied))
    expect_gte(counts$events_supplied, counts$events_analyzed)
    expect_equal(counts$events_analyzed, as.numeric(raw$events[1]))

    out <- utils::capture.output(print(result))

    expect_true(any(grepl(sprintf("Observations analyzed: %d of %d",
                                  as.integer(counts$n_analyzed),
                                  as.integer(counts$n_supplied)),
                          out, fixed = TRUE)))
    expect_true(any(grepl(sprintf("Events analyzed: %d of %d",
                                  as.integer(counts$events_analyzed),
                                  as.integer(counts$events_supplied)),
                          out, fixed = TRUE)))

    ## The bare "n = " line is gone, its figure now carried by the first line
    expect_false(any(grepl("^n = ", out)))
})


test_that("linear models report no events line", {

    dt <- data.table::as.data.table(fresh_clintrial())

    result <- fit(
        data = dt,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "lm"
    )

    out <- utils::capture.output(print(result))

    expect_true(any(grepl("Observations analyzed:", out, fixed = TRUE)))
    expect_false(any(grepl("Events analyzed:", out, fixed = TRUE)))
})


test_that("screening workflows report events alongside observations", {

    df <- fresh_clintrial()
    df$readmission_30d[seq_len(100)] <- NA
    df$bmi[seq_len(150)] <- NA
    dt <- data.table::as.data.table(df)

    screened <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "bmi"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    counts <- attr(screened, "analysis_counts")

    ## One value per screened predictor, with a shared denominator
    expect_length(counts$events_analyzed, 2L)
    expect_length(counts$events_supplied, 1L)
    expect_true(all(counts$events_analyzed <= counts$events_supplied))

    out <- utils::capture.output(print(screened))
    expect_true(any(grepl("Observations analyzed:", out, fixed = TRUE)))
    expect_true(any(grepl("Events analyzed:", out, fixed = TRUE)))
})


test_that("multifit reports observations only", {

    df <- fresh_clintrial()
    df$bmi[seq_len(100)] <- NA
    dt <- data.table::as.data.table(df)

    result <- multifit(
        data = dt,
        outcomes = c("readmission_30d", "os_status"),
        predictor = "sex",
        covariates = c("age", "bmi"),
        model_type = "glm",
        family = "binomial"
    )

    counts <- attr(result, "analysis_counts")

    ## Each outcome has its own event total, so no single figure could describe
    ## them all and events are deliberately omitted
    expect_null(counts$events_analyzed)

    out <- utils::capture.output(print(result))
    expect_true(any(grepl("Observations analyzed:", out, fixed = TRUE)))
    expect_false(any(grepl("Events analyzed:", out, fixed = TRUE)))
})


test_that("the event resolver accepts a family in any of its forms", {

    outcome_vars <- "readmission_30d"

    ## A name, a resolved family object, and the generator function itself
    expect_equal(summata:::get_event_variable_for_counts(
        outcome_vars, "glm", "binomial"), "readmission_30d")
    expect_equal(summata:::get_event_variable_for_counts(
        outcome_vars, "glm", stats::binomial()), "readmission_30d")
    expect_equal(summata:::get_event_variable_for_counts(
        outcome_vars, "glm", stats::binomial), "readmission_30d")

    ## Families that carry no event count, including one already resolved to an
    ## object by the caller
    expect_null(summata:::get_event_variable_for_counts(
        outcome_vars, "glm", "gaussian"))
    expect_null(summata:::get_event_variable_for_counts(
        outcome_vars, "glm", stats::Gamma(link = "log")))
    expect_null(summata:::get_event_variable_for_counts(
        outcome_vars, "lm", NULL))
})


test_that("Gamma and inverse Gaussian screens report no events line", {

    dt <- data.table::as.data.table(fresh_clintrial())

    result <- suppressWarnings(uniscreen(
        data = dt,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "Gamma",
        parallel = FALSE
    ))

    counts <- attr(result, "analysis_counts")
    expect_null(counts$events_analyzed)

    out <- utils::capture.output(print(result))
    expect_true(any(grepl("Observations analyzed:", out, fixed = TRUE)))
    expect_false(any(grepl("Events analyzed:", out, fixed = TRUE)))
})


test_that("printed counts follow the number_format setting", {

    ## A dataset large enough for a thousands separator to appear
    df <- fresh_clintrial()
    df <- do.call(rbind, replicate(3, df, simplify = FALSE))
    df$readmission_30d[seq_len(200)] <- NA
    dt <- data.table::as.data.table(df)

    us <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        number_format = "us"
    )

    eu <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        number_format = "eu"
    )

    ## format() warns when big.mark matches the decimal mark it would default
    ## to, so both marks must be supplied together
    expect_no_warning(us_out <- utils::capture.output(print(us)))
    expect_no_warning(eu_out <- utils::capture.output(print(eu)))

    us_line <- us_out[grepl("Observations analyzed", us_out, fixed = TRUE)]
    eu_line <- eu_out[grepl("Observations analyzed", eu_out, fixed = TRUE)]

    ## Counts carry the thousands separator and percentages the decimal mark
    expect_match(us_line, "[0-9],[0-9]{3}")
    expect_match(us_line, "\\.[0-9]%")
    expect_match(eu_line, "[0-9]\\.[0-9]{3}")
    expect_match(eu_line, ",[0-9]%")
})

#' Test Suite for fullfit()
#' 
#' Comprehensive tests covering variable selection methods, output formats,
#' model types, metrics options, return types, and edge cases.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-fullfit.R")

library(data.table)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Create binary outcome for logistic regression tests
## IMPORTANT: Add to BOTH datasets
clintrial$response <- as.integer(clintrial$os_status == 1 & clintrial$os_months < 24)
clintrial_complete$response <- as.integer(clintrial_complete$os_status == 1 & clintrial_complete$os_months < 24)

## Create integer count outcome for Poisson/quasipoisson/negative binomial tests
## IMPORTANT: Add to BOTH datasets
set.seed(123)
clintrial$count_outcome <- as.integer(ceiling(clintrial$los_days))
clintrial_complete$count_outcome <- as.integer(ceiling(clintrial_complete$los_days))

## Helper function to check fullfit_result structure
expect_fullfit_result <- function(result) {
    expect_s3_class(result, "fullfit_result")
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("Variable" %in% names(result))
    expect_true("Group" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "model_type")))
    expect_true(!is.null(attr(result, "method")))
}

## Helper to check for univariable columns
expect_uni_columns <- function(result, metrics = c("effect", "p")) {
    col_names <- names(result)
    
    if ("effect" %in% metrics) {
        ## Column format is "OR (95% CI)" for univariable (not "Univariable OR...")
        has_uni_effect <- any(grepl("^(OR|HR|RR|Coefficient|Estimate)\\s*\\(.*CI", col_names))
        expect_true(has_uni_effect, 
                    info = paste("Expected univariable effect column, found:", 
                                 paste(col_names, collapse = ", ")))
    }
    
    if ("p" %in% metrics) {
        expect_true("Uni p" %in% col_names,
                    info = paste("Expected 'Uni p' column, found:", 
                                 paste(col_names, collapse = ", ")))
    }
}

## Helper to check for multivariable columns
expect_multi_columns <- function(result, metrics = c("effect", "p")) {
    col_names <- names(result)
    
    if ("effect" %in% metrics) {
        ## Column format is "aOR (95% CI)" for multivariable (not "Multivariable aOR...")
        has_multi_effect <- any(grepl("^(aOR|aHR|aRR|Adj\\. Coefficient)\\s*\\(.*CI", col_names))
        expect_true(has_multi_effect, 
                    info = paste("Expected multivariable effect column, found:", 
                                 paste(col_names, collapse = ", ")))
    }
    
    if ("p" %in% metrics) {
        expect_true("Multi p" %in% col_names,
                    info = paste("Expected 'Multi p' column, found:", 
                                 paste(col_names, collapse = ", ")))
    }
}


## ============================================================================
## SECTION 1: Basic Functionality - Method = "screen"
## ============================================================================

test_that("fullfit works with method='screen' (default)", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "stage"),
        method = "screen",
        p_threshold = 0.05,
        model_type = "glm"
    )
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "method"), "screen")
    
    ## Should have both uni and multi columns by default
    expect_uni_columns(result)
    expect_multi_columns(result)
})


test_that("fullfit screen method respects p_threshold", {
    
    ## Strict threshold - fewer variables in multi
    result_strict <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "diabetes", "stage", "grade"),
        method = "screen",
        p_threshold = 0.01,
        model_type = "glm"
    )
    
    ## Liberal threshold - more variables in multi
    result_liberal <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "diabetes", "stage", "grade"),
        method = "screen",
        p_threshold = 0.50,
        model_type = "glm"
    )
    
    ## Liberal should include at least as many predictors as strict
    n_strict <- attr(result_strict, "n_multi")
    n_liberal <- attr(result_liberal, "n_multi")
    
    ## Handle case where strict might have 0 or NULL
    if (!is.null(n_strict) && !is.null(n_liberal)) {
        expect_true(n_liberal >= n_strict)
    }
})


test_that("fullfit screen with very strict threshold handles edge cases", {
    
    ## Very strict threshold - may or may not select variables depending on data
    ## This tests that the function handles the edge case gracefully
    result <- tryCatch({
        suppressWarnings(
            fullfit(
                data = clintrial,
                outcome = "response",
                predictors = c("age", "sex"),
                method = "screen",
                p_threshold = 0.0001,  # Very strict
                model_type = "glm"
            )
        )
    }, error = function(e) NULL)
    
    ## Should either return a valid result or NULL (if no vars selected)
    if (!is.null(result)) {
        expect_fullfit_result(result)
    }
    
    ## Either way, the function should not error unexpectedly
    expect_true(TRUE)
})


## ============================================================================
## SECTION 2: Method = "all"
## ============================================================================

test_that("fullfit works with method='all'", {
    
    predictors <- c("age", "sex", "smoking", "stage")
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = predictors,
        method = "all",
        model_type = "glm"
    )
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "method"), "all")
    
    ## All predictors should be in multivariable model
    n_multi <- attr(result, "n_multi")
    expect_equal(n_multi, length(predictors))
})


test_that("fullfit method='all' includes all variables in multi columns", {
    
    predictors <- c("age", "sex", "stage")
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = predictors,
        method = "all",
        model_type = "glm",
        columns = "both"
    )
    
    ## Multi columns should exist (aOR/aHR/aRR format)
    multi_effect_col <- grep("^(aOR|aHR|aRR|Adj\\.)", names(result), value = TRUE)[1]
    expect_true(!is.na(multi_effect_col))
    
    ## Get unique non-empty variable names
    vars_in_result <- unique(result$Variable[result$Variable != ""])
    
    ## Should have entries for all predictors (or their labels)
    expect_true(length(vars_in_result) >= length(predictors))
})


## ============================================================================
## SECTION 3: Method = "custom"
## ============================================================================

test_that("fullfit works with method='custom'", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "diabetes", "stage"),
        method = "custom",
        multi_predictors = c("age", "stage"),  # Only include these in multi
        model_type = "glm"
    )
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "method"), "custom")
    
    ## Only custom predictors in multi
    n_multi <- attr(result, "n_multi")
    expect_equal(n_multi, 2)
})


test_that("fullfit method='custom' requires multi_predictors", {
    
    expect_error(
        fullfit(
            data = clintrial,
            outcome = "response",
            predictors = c("age", "sex"),
            method = "custom"
            ## multi_predictors not specified - should error
        ),
        regexp = "multi_predictors must be specified"
    )
})


test_that("fullfit method='custom' shows uni for all, multi for selected", {
    
    all_predictors <- c("age", "sex", "smoking", "stage")
    selected <- c("age", "stage")
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = all_predictors,
        method = "custom",
        multi_predictors = selected,
        model_type = "glm",
        columns = "both"
    )
    
    ## All predictors should appear in the table
    vars_in_result <- unique(result$Variable[result$Variable != ""])
    
    ## Variables not in multi should show "-" in multi columns
    multi_effect_col <- grep("^(aOR|aHR|aRR|Adj\\.)", names(result), value = TRUE)[1]
    
    if (!is.na(multi_effect_col)) {
        ## Check that non-selected variables have "-" in multi column
        ## (This is complex due to factor levels, so just verify structure)
        expect_true(any(result[[multi_effect_col]] == "-") || 
                    length(selected) == length(all_predictors))
    } else {
        ## If no multi column found, that's also acceptable for certain column configs
        expect_true(TRUE)
    }
})


## ============================================================================
## SECTION 4: Columns Parameter
## ============================================================================

test_that("fullfit columns='both' shows uni and multi", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        columns = "both",
        model_type = "glm"
    )
    
    expect_uni_columns(result)
    expect_multi_columns(result)
})


test_that("fullfit columns='uni' shows only univariable", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        method = "all",
        columns = "uni",
        model_type = "glm"
    )
    
    col_names <- names(result)
    
    ## Should have univariable columns (OR/HR/RR without 'a' prefix)
    expect_true(any(grepl("^(OR|HR|RR|Coefficient)\\s*\\(", col_names)))
    
    ## Should NOT have multivariable columns (aOR/aHR/aRR)
    expect_false(any(grepl("^(aOR|aHR|aRR|Adj\\.)", col_names)))
    expect_false("Multi p" %in% col_names)
})


test_that("fullfit columns='multi' shows only multivariable", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        method = "all",
        columns = "multi",
        model_type = "glm"
    )
    
    col_names <- names(result)
    
    ## Should have multivariable columns (aOR/aHR/aRR)
    expect_true(any(grepl("^(aOR|aHR|aRR|Adj\\.)", col_names)))
    
    ## Should NOT have univariable columns (OR without 'a' prefix, Uni p)
    expect_false("Uni p" %in% col_names)
})


## ============================================================================
## SECTION 5: Metrics Parameter
## ============================================================================

test_that("fullfit metrics='both' shows effect and p-value", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        metrics = "both",
        model_type = "glm"
    )
    
    col_names <- names(result)
    
    ## Should have effect columns
    expect_true(any(grepl("(OR|HR|RR|Estimate).*CI", col_names)))
    
    ## Should have p-value columns
    expect_true(any(grepl("p$", col_names)))
})


test_that("fullfit metrics='effect' shows only effect estimates", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        metrics = "effect",
        model_type = "glm"
    )
    
    col_names <- names(result)
    
    ## Should have effect columns
    expect_true(any(grepl("(OR|HR|RR|Estimate).*CI", col_names)))
    
    ## Should NOT have p-value columns
    expect_false("Uni p" %in% col_names)
    expect_false("Multi p" %in% col_names)
})


test_that("fullfit metrics='p' shows only p-values", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        metrics = "p",
        model_type = "glm"
    )
    
    col_names <- names(result)
    
    ## Should have p-value columns
    expect_true(any(grepl("p$", col_names)))
    
    ## Should NOT have effect columns (CI columns)
    expect_false(any(grepl("95% CI", col_names)))
})


test_that("fullfit metrics as vector works", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        metrics = c("effect", "p"),  # Vector form
        model_type = "glm"
    )
    
    col_names <- names(result)
    
    ## Should have both
    expect_true(any(grepl("CI", col_names)))
    expect_true(any(grepl("p$", col_names)))
})


## ============================================================================
## SECTION 6: Return Type Parameter
## ============================================================================

test_that("fullfit return_type='table' returns fullfit_result", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        return_type = "table",
        model_type = "glm"
    )
    
    expect_fullfit_result(result)
})


test_that("fullfit return_type='model' returns model object", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        return_type = "model",
        model_type = "glm"
    )
    
    ## Should return a glm object, not a data.table
    expect_s3_class(result, "glm")
    expect_false(inherits(result, "data.table"))
    
    ## Should be usable as a model
    expect_true(length(coef(result)) > 0)
})


test_that("fullfit return_type='both' returns list with table and model", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        method = "all",
        return_type = "both",
        model_type = "glm"
    )
    
    ## Should be a list
    expect_type(result, "list")
    expect_true("table" %in% names(result))
    expect_true("model" %in% names(result))
    
    ## Table should be fullfit_result
    expect_s3_class(result$table, "fullfit_result")
    
    ## Model should be glm
    expect_s3_class(result$model, "glm")
})


## ============================================================================
## SECTION 7: Model Types
## ============================================================================

test_that("fullfit works with GLM (logistic)", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        model_type = "glm",
        family = "binomial",
        method = "all"
    )
    
    expect_fullfit_result(result)
    
    ## Should show OR
    col_names <- names(result)
    expect_true(any(grepl("OR", col_names)))
})


test_that("fullfit works with linear model", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "stage"),
        model_type = "lm",
        method = "all"
    )
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "model_type"), "lm")
    
    ## Linear models should show Estimate, not OR
    col_names <- names(result)
    expect_true(any(grepl("Estimate", col_names)) || any(grepl("Coefficient", col_names)))
})


test_that("fullfit works with Cox regression", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "stage", "treatment"),
        model_type = "coxph",
        method = "all"
    )
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "model_type"), "coxph")
    
    ## Should show HR
    col_names <- names(result)
    expect_true(any(grepl("HR", col_names)))
})


test_that("fullfit works with Poisson regression", {
    
    ## Create proper count data for Poisson
    set.seed(123)
    test_data <- data.table::as.data.table(clintrial)
    test_data[, count_outcome := as.integer(round(los_days))]
    
    result <- suppressWarnings(
        fullfit(
            data = test_data,
            outcome = "count_outcome",
            predictors = c("age", "sex", "stage"),
            model_type = "glm",
            family = "poisson",
            method = "all"
        )
    )
    
    expect_fullfit_result(result)
    
    ## Should show RR
    col_names <- names(result)
    expect_true(any(grepl("RR", col_names)))
})


## ============================================================================
## SECTION 8: Display Options
## ============================================================================

test_that("fullfit respects show_n parameter", {
    
    result_with_n <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        show_n = TRUE
    )
    
    result_no_n <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        show_n = FALSE
    )
    
    expect_true("n" %in% names(result_with_n))
    expect_false("n" %in% names(result_no_n))
})


test_that("fullfit respects show_events parameter", {
    
    result_with_events <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        show_events = TRUE
    )
    
    result_no_events <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        show_events = FALSE
    )
    
    expect_true("Events" %in% names(result_with_events))
    expect_false("Events" %in% names(result_no_events))
})


test_that("fullfit respects digits parameter", {
    
    result_2dig <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age"),
        model_type = "glm",
        method = "all",
        digits = 2
    )
    
    result_3dig <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age"),
        model_type = "glm",
        method = "all",
        digits = 3
    )
    
    ## Extract effect columns and check decimal places
    effect_col_2 <- grep("OR.*CI", names(result_2dig), value = TRUE)[1]
    effect_col_3 <- grep("OR.*CI", names(result_3dig), value = TRUE)[1]
    
    if (!is.na(effect_col_2) && !is.na(effect_col_3)) {
        val_2 <- result_2dig[[effect_col_2]][1]
        val_3 <- result_3dig[[effect_col_3]][1]
        
        ## Check decimal places differ
        if (!is.na(val_2) && !is.na(val_3) && val_2 != "-" && val_3 != "-") {
            first_num_2 <- sub(" .*", "", val_2)
            first_num_3 <- sub(" .*", "", val_3)
            
            decimals_2 <- nchar(sub(".*\\.", "", first_num_2))
            decimals_3 <- nchar(sub(".*\\.", "", first_num_3))
            
            expect_equal(decimals_2, 2)
            expect_equal(decimals_3, 3)
        }
    }
})


test_that("fullfit respects reference_rows parameter", {
    
    result_with_ref <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "stage"),
        model_type = "glm",
        method = "all",
        reference_rows = TRUE,
        parallel = FALSE
    )
    
    result_no_ref <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "stage"),
        model_type = "glm",
        method = "all",
        reference_rows = FALSE,
        parallel = FALSE
    )
    
    ## With reference rows should have more rows
    expect_true(nrow(result_with_ref) >= nrow(result_no_ref))
})


## ============================================================================
## SECTION 9: Labels Parameter
## ============================================================================

test_that("fullfit applies custom labels", {
    
    custom_labels <- c(
        age = "Age (years)",
        sex = "Sex",
        stage = "Cancer Stage"
    )
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        model_type = "glm",
        method = "all",
        labels = custom_labels
    )
    
    ## Check labels are applied
    variables <- result$Variable[result$Variable != ""]
    
    ## At least one custom label should appear
    has_label <- any(grepl("Age|Sex|Cancer Stage", variables))
    expect_true(has_label)
})


test_that("fullfit works with clintrial_labels", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage", "treatment"),
        model_type = "glm",
        method = "all",
        labels = clintrial_labels
    )
    
    expect_fullfit_result(result)
})


## ============================================================================
## SECTION 10: keep_models Parameter
## ============================================================================

test_that("fullfit keep_models=FALSE does not store uni models", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        keep_models = FALSE
    )
    
    ## When keep_models is FALSE, uni_results might exist but models should not be stored
    ## Just verify the result is valid
    expect_fullfit_result(result)
})


test_that("fullfit keep_models=TRUE stores uni models", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        keep_models = TRUE
    )
    
    expect_fullfit_result(result)
    
    ## When keep_models=TRUE, models should be accessible somehow
    ## The exact attribute structure may vary, so just check result is valid
    ## and has the expected attributes
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "model")))
})


## ============================================================================
## SECTION 11: Confidence Level
## ============================================================================

test_that("fullfit respects conf_level parameter", {
    
    result_95 <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age"),
        model_type = "glm",
        method = "all",
        conf_level = 0.95
    )
    
    result_90 <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age"),
        model_type = "glm",
        method = "all",
        conf_level = 0.90
    )
    
    ## Both should work - checking structure
    expect_fullfit_result(result_95)
    expect_fullfit_result(result_90)
    
    ## Results should be different (narrower CI for 90%)
    ## This is hard to test directly from formatted output
})


## ============================================================================
## SECTION 12: Exponentiate Parameter
## ============================================================================

test_that("fullfit auto-exponentiates for logistic", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        method = "all"
    )
    
    ## Should show OR by default
    col_names <- names(result)
    expect_true(any(grepl("OR", col_names)))
})


test_that("fullfit exponentiate=FALSE shows coefficients", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        method = "all",
        exponentiate = FALSE
    )
    
    ## Should show Coefficient or Estimate, not OR
    col_names <- names(result)
    expect_true(any(grepl("Coefficient|Estimate", col_names)))
})


## ============================================================================
## SECTION 13: Factor Variables
## ============================================================================

test_that("fullfit handles factor variables correctly", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "stage"),  # stage is a factor
        model_type = "glm",
        method = "all",
        reference_rows = TRUE,
        parallel = FALSE
    )
    
    expect_fullfit_result(result)
    
    ## Stage should have multiple rows (reference + levels)
    ## Find rows where Variable contains "stage" or follows it
    stage_idx <- which(result$Variable == "stage" | 
                       grepl("[Ss]tage", result$Variable))
    
    ## Should have multiple rows for factor
    if (length(stage_idx) > 0) {
        expect_true(length(stage_idx) >= 1)
    }
})


test_that("fullfit handles multiple factor variables", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("sex", "stage", "grade"),  # Multiple factors
        model_type = "glm",
        method = "all"
    )
    
    expect_fullfit_result(result)
    
    ## Should have rows for all factors
    expect_true(nrow(result) > 3)  # More than just 3 single rows
})


## ============================================================================
## SECTION 14: Attributes Preservation
## ============================================================================

test_that("fullfit preserves all expected attributes", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage"),
        model_type = "glm",
        method = "screen",
        p_threshold = 0.10
    )
    
    ## Check all expected attributes
    expect_equal(attr(result, "outcome"), "response")
    expect_equal(attr(result, "model_type"), "glm")
    expect_equal(attr(result, "method"), "screen")
    expect_equal(attr(result, "columns"), "both")
    
    ## Should have model if multivariable was fit
    model <- attr(result, "model")
    if (!is.null(attr(result, "n_multi")) && attr(result, "n_multi") > 0) {
        expect_true(!is.null(model))
    }
})


test_that("fullfit stores expected attributes", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        columns = "both"  # Need uni for this
    )
    
    ## Should have key attributes
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "model_type")))
    expect_true(!is.null(attr(result, "method")))
    
    ## Model should be stored when multivariable is fit
    expect_true(!is.null(attr(result, "model")))
})


test_that("fullfit n_multi attribute is correct", {
    
    predictors <- c("age", "sex", "stage")
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = predictors,
        model_type = "glm",
        method = "all"
    )
    
    n_multi <- attr(result, "n_multi")
    expect_equal(n_multi, length(predictors))
})


## ============================================================================
## SECTION 15: Print Method
## ============================================================================

test_that("print.fullfit_result produces output", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all"
    )
    
    ## Capture print output
    output <- capture.output(print(result))
    
    ## Should contain key information
    expect_true(any(grepl("Fullfit|Analysis", output, ignore.case = TRUE)))
    expect_true(any(grepl("Outcome", output)))
    expect_true(any(grepl("Method", output)))
})


## ============================================================================
## SECTION 16: Edge Cases
## ============================================================================

test_that("fullfit handles single predictor", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = "age",  # Single predictor
        model_type = "glm",
        method = "all"
    )
    
    expect_fullfit_result(result)
})


test_that("fullfit handles many predictors", {
    
    many_predictors <- c("age", "sex", "smoking", "diabetes", "hypertension",
                         "stage", "grade", "ecog", "treatment", "surgery")
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = many_predictors,
        model_type = "glm",
        method = "all"
    )
    
    expect_fullfit_result(result)
})


test_that("fullfit handles missing data appropriately", {
    
    ## clintrial has some missing values
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking"),  # smoking may have NAs
        model_type = "glm",
        method = "all"
    )
    
    expect_fullfit_result(result)
})


test_that("fullfit works with data.frame input", {
    
    df <- as.data.frame(clintrial)
    
    result <- fullfit(
        data = df,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all"
    )
    
    expect_fullfit_result(result)
})


## ============================================================================
## SECTION 16B: Input Validation Tests
## ============================================================================

test_that("fullfit errors or auto-corrects Surv outcome with GLM model_type", {
    
    ## Should either auto-correct (with message) or error
    result_or_error <- tryCatch({
        suppressMessages(suppressWarnings(
            fullfit(
                data = clintrial,
                outcome = "Surv(os_months, os_status)",
                predictors = c("age", "sex"),
                model_type = "glm",
                method = "all"
            )
        ))
    }, error = function(e) e)
    
    ## Either produces valid result or an error (both acceptable)
    expect_true(
        inherits(result_or_error, "fullfit_result") ||
        inherits(result_or_error, "error")
    )
})


test_that("fullfit errors when coxph used without Surv outcome", {
    
    ## Should error - either from validation or from coxph itself
    expect_error(
        suppressWarnings(
            fullfit(
                data = clintrial,
                outcome = "response",
                predictors = c("age", "sex"),
                model_type = "coxph",
                method = "all"
            )
        )
    )
})


test_that("fullfit errors with continuous outcome and binomial family", {
    
    ## Should error - either from validation or from glm itself
    expect_error(
        suppressWarnings(
            fullfit(
                data = clintrial,
                outcome = "los_days",
                predictors = c("age", "sex"),
                model_type = "glm",
                family = "binomial",
                method = "all"
            )
        )
    )
})


test_that("fullfit errors when outcome variable not found", {
    
    expect_error(
        suppressWarnings(
            fullfit(
                data = clintrial,
                outcome = "nonexistent_variable",
                predictors = c("age", "sex"),
                model_type = "glm",
                method = "all"
            )
        )
    )
})


test_that("fullfit errors when predictor variable not found", {
    
    expect_error(
        suppressWarnings(
            fullfit(
                data = clintrial,
                outcome = "response",
                predictors = c("age", "nonexistent_predictor"),
                model_type = "glm",
                method = "all"
            )
        )
    )
})


test_that("fullfit handles edge case p_threshold values", {
    
    ## p_threshold = 1 should include all predictors
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "screen",
        p_threshold = 1.0
    )
    expect_fullfit_result(result)
})


test_that("fullfit errors with empty data", {
    
    empty_data <- clintrial[0, ]
    
    expect_error(
        suppressWarnings(
            fullfit(
                data = empty_data,
                outcome = "response",
                predictors = c("age", "sex"),
                model_type = "glm",
                method = "all"
            )
        )
    )
})


## ============================================================================
## SECTION 17: Integration with Other Functions
## ============================================================================

test_that("fullfit model output works with summary()", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        return_type = "model"
    )
    
    ## Should be able to summarize
    summ <- summary(result)
    expect_true(!is.null(summ))
})


test_that("fullfit model output works with predict()", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all",
        return_type = "model"
    )
    
    ## Should be able to predict
    preds <- predict(result, type = "response")
    expect_true(length(preds) > 0)
})


test_that("fullfit produces consistent results", {
    
    result1 <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all"
    )
    
    result2 <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm",
        method = "all"
    )
    
    ## Results should be identical
    expect_equal(result1, result2)
})


## ============================================================================
## SECTION 18: Cox Regression Specific Tests
## ============================================================================

test_that("fullfit Cox shows Events column", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "stage"),
        model_type = "coxph",
        method = "all",
        show_events = TRUE
    )
    
    expect_true("Events" %in% names(result))
})


test_that("fullfit Cox screening works correctly", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "smoking", "stage", "grade"),
        model_type = "coxph",
        method = "screen",
        p_threshold = 0.10
    )
    
    expect_fullfit_result(result)
    
    ## Should have HR columns
    col_names <- names(result)
    expect_true(any(grepl("HR", col_names)))
})


## ============================================================================
## SECTION 19: Linear Model Specific Tests
## ============================================================================

test_that("fullfit linear model doesn't show Events", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex"),
        model_type = "lm",
        method = "all",
        show_events = TRUE  # Should be ignored for LM
    )
    
    ## Linear models shouldn't have Events
    expect_false("Events" %in% names(result))
})


test_that("fullfit linear model screening works", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "stage", "treatment"),
        model_type = "lm",
        method = "screen",
        p_threshold = 0.20
    )
    
    expect_fullfit_result(result)
})


## ============================================================================
## SECTION 19b: Additional GLM Family Tests
## ============================================================================

## ----------------------------------------------------------------------------
## Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("fullfit works with gaussian family via glm", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "glm",
        family = "gaussian",
        method = "all"
    )
    
    expect_fullfit_result(result)
    
    ## Should have Coefficient columns (not OR/RR)
    col_names <- names(result)
    expect_true(any(grepl("Coefficient.*CI|Estimate.*CI", col_names)),
                info = paste("Expected Coefficient column for gaussian, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fullfit gaussian screening works", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment", "stage", "grade"),
        model_type = "glm",
        family = "gaussian",
        method = "screen",
        p_threshold = 0.20
    )
    
    expect_fullfit_result(result)
})


## ----------------------------------------------------------------------------
## Quasibinomial Family Tests
## ----------------------------------------------------------------------------

test_that("fullfit works with quasibinomial family", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "glm",
        family = "quasibinomial",
        method = "all"
    )
    
    expect_fullfit_result(result)
    
    ## Should have OR columns like binomial
    col_names <- names(result)
    expect_true(any(grepl("OR.*CI", col_names)),
                info = paste("Expected OR column for quasibinomial, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fullfit quasibinomial screening works", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "treatment", "stage", "grade"),
        model_type = "glm",
        family = "quasibinomial",
        method = "screen",
        p_threshold = 0.20
    )
    
    expect_fullfit_result(result)
})


## ----------------------------------------------------------------------------
## Quasipoisson Family Tests
## ----------------------------------------------------------------------------

test_that("fullfit works with quasipoisson family", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "glm",
        family = "quasipoisson",
        method = "all"
    )
    
    expect_fullfit_result(result)
    
    ## Should have RR columns like poisson
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for quasipoisson, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fullfit quasipoisson screening works", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage", "grade"),
        model_type = "glm",
        family = "quasipoisson",
        method = "screen",
        p_threshold = 0.20
    )
    
    expect_fullfit_result(result)
})


## ----------------------------------------------------------------------------
## Gamma Family Tests
## ----------------------------------------------------------------------------

test_that("fullfit works with Gamma family (log link)", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fullfit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        family = Gamma(link = "log"),
        method = "all"
    ))
    
    expect_fullfit_result(result)
})


test_that("fullfit Gamma screening works", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fullfit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "glm",
        family = Gamma(link = "log"),
        method = "screen",
        p_threshold = 0.20
    ))
    
    expect_fullfit_result(result)
})


## ----------------------------------------------------------------------------
## Inverse Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("fullfit works with inverse.gaussian family", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(fullfit(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        family = inverse.gaussian(link = "log"),
        method = "all"
    ))
    
    expect_fullfit_result(result)
})


## ----------------------------------------------------------------------------
## Negative Binomial (negbin) Tests
## ----------------------------------------------------------------------------

test_that("fullfit works with negative binomial regression (negbin)", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "negbin",
        method = "all"
    ))
    
    expect_fullfit_result(result)
    
    ## Should have RR columns for count data
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for negbin, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("fullfit negbin screening works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage", "grade"),
        model_type = "negbin",
        method = "screen",
        p_threshold = 0.20
    ))
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "method"), "screen")
})


test_that("fullfit negbin method='custom' works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "negbin",
        method = "custom",
        multi_predictors = c("age", "treatment")
    ))
    
    expect_fullfit_result(result)
    expect_equal(attr(result, "method"), "custom")
    expect_equal(attr(result, "n_multi"), 2)
})


test_that("fullfit negbin with labels works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "negbin",
        method = "all",
        labels = clintrial_labels
    ))
    
    expect_fullfit_result(result)
    
    ## Labels should be applied
    expect_true(any(grepl("Age|Sex|Treatment", result$Variable)))
})


test_that("fullfit negbin return_type='model' works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "negbin",
        method = "all",
        return_type = "model"
    ))
    
    ## Should return a negbin model object
    expect_true(inherits(result, "negbin"))
})


test_that("fullfit negbin return_type='both' works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(fullfit(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "negbin",
        method = "all",
        return_type = "both"
    ))
    
    expect_type(result, "list")
    expect_s3_class(result$table, "fullfit_result")
    expect_true(inherits(result$model, "negbin"))
})


## ============================================================================
## SECTION 20: Complex Combinations
## ============================================================================

test_that("fullfit handles complex parameter combinations", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "smoking", "stage", "grade", "treatment"),
        model_type = "coxph",
        method = "screen",
        p_threshold = 0.15,
        columns = "both",
        metrics = "both",
        show_n = TRUE,
        show_events = TRUE,
        reference_rows = TRUE,
        digits = 2,
        p_digits = 3,
        labels = clintrial_labels,
        parallel = FALSE
    )
    
    expect_fullfit_result(result)
    
    ## Should have all the expected columns
    col_names <- names(result)
    expect_true("Variable" %in% col_names)
    expect_true("Group" %in% col_names)
    expect_true("n" %in% col_names)
    expect_true("Events" %in% col_names)
    ## Check for uni columns (HR format) and multi columns (aHR format)
    expect_true(any(grepl("^(HR|OR|RR)\\s*\\(", col_names)))
    expect_true(any(grepl("^(aHR|aOR|aRR)", col_names)))
    expect_true(any(grepl("p$", col_names)))
})


test_that("fullfit custom method with columns='multi' works", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "stage"),
        model_type = "glm",
        method = "custom",
        multi_predictors = c("age", "stage"),
        columns = "multi"  # Only show multi columns
    )
    
    expect_fullfit_result(result)
    
    ## Should have only multi columns (aOR/aHR/aRR format)
    col_names <- names(result)
    expect_true(any(grepl("^(aOR|aHR|aRR|Adj\\.)", col_names)))
    expect_false("Uni p" %in% col_names)
})


test_that("fullfit with return_type='both' and method='screen'", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "stage", "treatment"),
        model_type = "glm",
        method = "screen",
        p_threshold = 0.20,
        return_type = "both"
    )
    
    expect_type(result, "list")
    expect_s3_class(result$table, "fullfit_result")
    
    ## Model might be NULL if nothing passed screening
    if (!is.null(result$model)) {
        expect_s3_class(result$model, "glm")
    }
})


## ============================================================================
## SECTION 21: Complete-Case N/Events in Multivariable Rows
## ============================================================================

test_that("fullfit multivariable n matches model nobs", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "bmi", "smoking", "stage"),
        method = "all",
        model_type = "glm",
        show_n = TRUE,
        columns = "both"
    )
    
    ## Get the fitted model
    model <- attr(result, "model")
    expected_n <- stats::nobs(model)
    
    ## Only continuous predictor rows (Group == "-") should show
    ## the model-level n; factor level rows show group-specific n
    multi_effect_col <- grep("^aOR", names(result), value = TRUE)[1]
    if (!is.na(multi_effect_col)) {
        has_multi <- result[[multi_effect_col]] != "-"
        is_continuous <- result$Group == "-"
        continuous_multi <- has_multi & is_continuous
        multi_n_values <- as.integer(result$n[continuous_multi])
        multi_n_values <- multi_n_values[!is.na(multi_n_values)]
        
        if (length(multi_n_values) > 0) {
            expect_true(all(multi_n_values == expected_n),
                        info = paste("Continuous predictor n should be", expected_n,
                                     "but found:", paste(unique(multi_n_values), collapse = ", ")))
        }
    }
})


test_that("fullfit screen method shows different n for uni-only vs multi rows", {
    
    ## Use predictors with varying missingness to create a gap
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                       "stage", "grade", "treatment"),
        method = "screen",
        p_threshold = 0.05,
        model_type = "glm",
        show_n = TRUE,
        columns = "both"
    )
    
    ## The function should produce a valid result regardless
    expect_fullfit_result(result)
    
    ## If some variables were screened out, their n may differ from
    ## multivariable n (due to complete-case differences)
    model <- attr(result, "model")
    if (!is.null(model)) {
        model_n <- stats::nobs(model)
        
        ## Continuous predictor multi rows should have consistent n
        multi_effect_col <- grep("^aOR", names(result), value = TRUE)[1]
        if (!is.na(multi_effect_col)) {
            has_multi <- result[[multi_effect_col]] != "-"
            is_continuous <- result$Group == "-"
            multi_n <- as.integer(result$n[has_multi & is_continuous])
            multi_n <- multi_n[!is.na(multi_n)]
            
            if (length(multi_n) > 0) {
                expect_true(all(multi_n == model_n),
                            info = "All continuous multivariable rows should have same n")
            }
        }
    }
})


test_that("fullfit multivariable Events match model events", {
    
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "stage"),
        method = "all",
        model_type = "glm",
        show_events = TRUE,
        columns = "both"
    )
    
    ## Get expected events from fitted model
    model <- attr(result, "model")
    if (!is.null(model) && !is.null(model$y)) {
        expected_events <- sum(model$y, na.rm = TRUE)
        
        ## Continuous predictor rows should show model-level events
        ## (factor group rows show group-specific events)
        age_row <- result[Variable != "" & grepl("age|Age", Variable, ignore.case = TRUE)]
        if (nrow(age_row) > 0) {
            row_events <- as.integer(age_row$Events[1])
            if (!is.na(row_events)) {
                expect_equal(row_events, expected_events,
                             info = "Continuous predictor Events should match model events")
            }
        }
    }
})


test_that("fullfit columns='uni' does not apply complete-case n", {
    
    ## When showing only univariable results, n should be per-variable
    result <- fullfit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "bmi"),
        method = "all",
        model_type = "glm",
        show_n = TRUE,
        columns = "uni"
    )
    
    expect_fullfit_result(result)
    
    ## Per-variable n values may differ due to different missingness patterns
    n_values <- as.integer(result$n)
    n_values <- n_values[!is.na(n_values)]
    
    ## Should have valid n values
    expect_true(all(n_values > 0))
})

### * Main function

#' Compare Multiple Regression Models
#'
#' Fits multiple regression models and provides a comprehensive comparison table
#' with model quality metrics, convergence diagnostics, and selection guidance.
#' Computes a composite score combining multiple quality metrics to facilitate 
#' rapid model comparison and selection.
#'
#' @param data Data frame or data.table containing the dataset.
#' 
#' @param outcome Character string specifying the outcome variable. For survival
#'   analysis, use \code{Surv()} syntax (\emph{e.g.,} \code{"Surv(time, status)"}).
#' 
#' @param model_list List of character vectors, each containing predictor names
#'   for one model. Can also be a single character vector to auto-generate nested models.
#' 
#' @param model_names Character vector of names for each model. If \code{NULL}, uses
#'   "Model 1", "Model 2", \emph{etc.} Default is \code{NULL}.
#' 
#' @param interactions_list List of character vectors specifying interaction
#'   terms for each model. Each element corresponds to one model in model_list.
#'   Use \code{NULL} for models without interactions. Use colon notation for interactions
#'   (\emph{e.g.,} \code{c("age:treatment")}). If \code{NULL}, no interactions are added to
#'   any model. Default is \code{NULL}.
#' 
#' @param random Character string specifying the random-effects formula for 
#'   mixed-effects models (\code{glmer}, \code{lmer}, \code{coxme}). Use standard
#'   \code{lme4}/\code{coxme} syntax, \emph{e.g.,} \code{"(1|site)"} for random
#'   intercepts by site. This random effects formula is applied to all models in the
#'   comparison. Alternatively, random effects can be included directly in the predictor 
#'   vectors within \code{model_list} using the same syntax, which allows
#'   different random effects structures across models. Default is \code{NULL}.
#' 
#' @param model_type Character string specifying model type. If \code{"auto"}, detects
#'   based on outcome. Options include:
#'   \itemize{
#'     \item \code{"auto"} - Automatically detect based on outcome type (default)
#'     \item \code{"glm"} - Generalized linear model. Supports logistic, Poisson, 
#'       Gamma, Gaussian via \code{family} parameter.
#'     \item \code{"lm"} - Linear regression for continuous outcomes
#'     \item \code{"coxph"} - Cox proportional hazards for survival analysis
#'     \item \code{"negbin"} - Negative binomial regression for overdispersed counts 
#'       (requires MASS package)
#'     \item \code{"lmer"} - Mixed-effects linear regression for clustered continuous
#'       outcomes
#'     \item \code{"glmer"} - Mixed-effects logistic regression for clustered
#'       categorical outcomes
#'     \item \code{"coxme"} - Mixed-effects Cox regression for clustered time-to-event
#'       outcomes
#'   }
#' 
#' @param family For GLM and GLMER models, specifies the error distribution and link function.
#'   Common options include:
#'   \itemize{
#'     \item \code{"binomial"} - Logistic regression for binary outcomes (default)
#'     \item \code{"poisson"} - Poisson regression for count data
#'     \item \code{"quasibinomial"} - Logistic with overdispersion
#'     \item \code{"quasipoisson"} - Poisson with overdispersion
#'     \item \code{"gaussian"} - Normal distribution (linear regression via GLM)
#'     \item \code{"Gamma"} - Gamma for positive continuous data
#'     \item \code{"inverse.gaussian"} - For positive, highly skewed data
#'   }
#'   For negative binomial, use \code{model_type = "negbin"} instead.
#'   See \code{\link[stats]{family}} for all options.
#' 
#' @param conf_level Numeric confidence level for intervals. Default is 0.95.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param include_coefficients Logical. If TRUE, includes a second table with
#'   coefficient estimates. Default is FALSE.
#' 
#' @param scoring_weights Named list of scoring weights. Each weight should be
#'   between 0 and 1, and they should sum to 1. Available metrics depend on model
#'   type. If \code{NULL}, uses sensible defaults. See Details for available metrics.
#' 
#' @param labels Named character vector providing custom display labels for
#'   variables. Default is \code{NULL}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param verbose Logical. If \code{TRUE}, displays model fitting warnings
#'   (\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
#'   routine fitting messages are suppressed while unexpected warnings are
#'   preserved. When \code{NULL}, uses
#'   \code{getOption("summata.verbose", FALSE)}.
#'
#' @param ... Additional arguments passed to model fitting functions.
#'
#' @return A data.table with class "compfit_result" containing:
#'   \describe{
#'     \item{Model}{Model name/identifier}
#'     \item{CMS}{Composite Model Score for model selection (higher is better)}
#'     \item{N}{Sample size}
#'     \item{Events}{Number of events (for survival/logistic)}
#'     \item{Predictors}{Number of predictors}
#'     \item{Converged}{Whether model converged properly}
#'     \item{AIC}{Akaike Information Criterion}
#'     \item{BIC}{Bayesian Information Criterion}
#'     \item{\emph{R}\eqn{^2} / Pseudo-\emph{R}\eqn{^2}}{McFadden pseudo-R-squared (GLM)}
#'     \item{Concordance}{C-statistic (logistic/survival)}
#'     \item{Brier Score}{Brier accuracy score (logistic)}
#'     \item{Global \emph{p}}{Overall model \emph{p}-value}
#'   }
#'   
#'   Attributes include:
#'   \describe{
#'     \item{models}{List of fitted model objects}
#'     \item{coefficients}{Coefficient comparison table (if requested)}
#'     \item{best_model}{Name of recommended model}
#'   }
#'
#' @details
#' This function fits all specified models and computes comprehensive quality
#' metrics for comparison. It generates a Composite Model Score (CMS) that 
#' combines multiple metrics: lower AIC/BIC (information criteria), higher 
#' concordance (discrimination), and model convergence status.
#' 
#' For GLMs, McFadden's pseudo-R-squared is calculated as 1 - (logLik/logLik_null).
#' For survival models, the global \emph{p}-value comes from the log-rank test.
#' 
#' Models that fail to converge are flagged and penalized in the composite score.
#' 
#' \strong{Interaction Terms:}
#' 
#' When \code{interactions_list} is provided, each element specifies the
#' interaction terms for the corresponding model in \code{model_list}. This is
#' particularly useful for testing whether adding interactions improves model fit:
#' \itemize{
#'   \item Use \code{NULL} for models without interactions
#'   \item Specify interactions using colon notation: \code{c("age:treatment", "sex:stage")}
#'   \item Main effects for all variables in interactions must be in the predictor list
#'   \item Common pattern: Compare main effects model vs model with interactions
#' }
#' 
#' Scoring weights can be customized based on model type:
#' \itemize{
#'   \item GLM: \code{"convergence"}, \code{"aic"}, \code{"concordance"}, \code{"pseudo_r2"}, \code{"brier"} 
#'   \item Cox: \code{"convergence"}, \code{"aic"}, \code{"concordance"}, \code{"global_p"}
#'   \item Linear: \code{"convergence"}, \code{"aic"}, \code{"pseudo_r2"}, \code{"rmse"}
#' }
#' Default weights emphasize discrimination (concordance) and model fit (AIC).
#'
#' The composite score is designed as a tool to quickly rank models by their 
#' quality metrics. It should be used alongside traditional model selection 
#' criteria rather than as a definitive model selection method.
#'
#' @seealso
#' \code{\link{fit}} for individual model fitting,
#' \code{\link{fullfit}} for automated variable selection,
#' \code{\link{tablesave}} for exporting tables
#'
#' @examples
#' # Load example data
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Information criteria assume a common sample, so the candidate predictors
#' # are restricted to complete cases before any comparison
#' comparison_data <- na.omit(
#'     clintrial[, c("readmission_30d", "os_months", "os_status", "age", "sex",
#'                   "smoking", "diabetes", "stage", "ecog", "grade",
#'                   "treatment")]
#' )
#'
#' # Example 1: Compare nested logistic regression models
#' models <- list(
#'     base = c("age", "sex"),
#'     clinical = c("age", "sex", "smoking", "diabetes"),
#'     full = c("age", "sex", "smoking", "diabetes", "stage", "ecog")
#' )
#'
#' comparison <- compfit(
#'     data = comparison_data,
#'     outcome = "readmission_30d",
#'     model_list = models,
#'     model_names = c("Base", "Clinical", "Full")
#' )
#' comparison
#'
#' \donttest{
#'
#' # Example 2: Compare Cox survival models
#' surv_models <- list(
#'     simple = c("age", "sex"),
#'     clinical = c("age", "sex", "stage", "grade")
#' )
#'
#' surv_comparison <- compfit(
#'     data = comparison_data,
#'     outcome = "Surv(os_months, os_status)",
#'     model_list = surv_models,
#'     model_type = "coxph"
#' )
#' surv_comparison
#'
#' # Example 3: Test the effect of adding an interaction term.
#' # Treatment efficacy differs by disease stage in these data, so the
#' # interaction is expected to improve the fit.
#' interaction_models <- list(
#'     main = c("age", "treatment", "stage"),
#'     interact = c("age", "treatment", "stage")
#' )
#'
#' interaction_comp <- compfit(
#'     data = comparison_data,
#'     outcome = "Surv(os_months, os_status)",
#'     model_list = interaction_models,
#'     model_type = "coxph",
#'     model_names = c("Main Effects", "With Interaction"),
#'     interactions_list = list(
#'         NULL,
#'         c("treatment:stage")
#'     )
#' )
#' interaction_comp
#'
#' # Example 4: Include coefficient comparison table
#' detailed <- compfit(
#'     data = comparison_data,
#'     outcome = "readmission_30d",
#'     model_list = models,
#'     include_coefficients = TRUE,
#'     labels = clintrial_labels
#' )
#'
#' # Access coefficient table
#' coef_table <- attr(detailed, "coefficients")
#' coef_table
#'
#' # Example 5: Access fitted model objects
#' fitted_models <- attr(comparison, "models")
#' names(fitted_models)
#'
#' # Example 6: Get best model recommendation
#' best <- attr(comparison, "best_model")
#' cat("Recommended model:", best, "\n")
#'
#' }
#'
#' @family regression functions
#' @export
compfit <- function(data,
                    outcome,
                    model_list,
                    model_names = NULL,
                    interactions_list = NULL,
                    random = NULL,
                    model_type = "auto",
                    family = "binomial",
                    conf_level = 0.95,
                    p_digits = 3,
                    include_coefficients = FALSE,
                    scoring_weights = NULL,
                    labels = NULL,
                    number_format = NULL,
                    verbose = NULL,
                    ...) {
    
    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }

    ## Resolve verbose setting
    verbose <- if (is.null(verbose)) getOption("summata.verbose", FALSE) else verbose

    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)

    ## Check if any model has random effects (for auto-detection)
    ## Random effects are specified with | syntax, e.g., "(1|site)"
    ## Check both model_list and the random parameter
    has_random_in_models <- any(sapply(model_list, function(preds) {
        any(grepl("\\|", preds))
    }))
    has_any_random_effects <- !is.null(random) || has_random_in_models
    
    ## Note if random effects specified in both places
    if (!is.null(random) && has_random_in_models) {
        message("Note: Random effects specified in both 'random' parameter and 'model_list'. ",
                "The 'random' parameter (", random, ") will be applied to all models, ",
                "combined with any model-specific random effects in the predictor lists.")
    }

    ## Auto-detect model type if requested
    if (model_type == "auto") {
        if (grepl("^Surv\\(", outcome)) {
            if (has_any_random_effects) {
                model_type <- "coxme"
                message("Auto-detected survival outcome with random effects, using coxme")
            } else {
                model_type <- "coxph"
                message("Auto-detected survival outcome, using Cox regression")
            }
        } else if (is.factor(data[[outcome]]) || length(unique(data[[outcome]])) == 2) {
            if (has_any_random_effects) {
                model_type <- "glmer"
                family <- "binomial"
                message("Auto-detected binary outcome with random effects, using glmer")
            } else {
                model_type <- "glm"
                family <- "binomial"
                message("Auto-detected binary outcome, using logistic regression")
            }
        } else if (is.numeric(data[[outcome]])) {
            if (has_any_random_effects) {
                model_type <- "lmer"
                message("Auto-detected continuous outcome with random effects, using lmer")
            } else {
                model_type <- "lm"
                message("Auto-detected continuous outcome, using linear regression")
            }
        } else {
            stop("Cannot auto-detect model type for outcome: ", outcome)
        }
    }
    
    ## Validate random parameter for non-mixed-effects models
    mixed_types <- c("glmer", "lmer", "coxme")
    if (!is.null(random) && !model_type %in% mixed_types) {
        warning("'random' parameter is ignored for non-mixed-effects models (", model_type, ").")
        random <- NULL
    }

    ## Set model names if not provided
    if (is.null(model_names)) {
        ## Use list names if available, otherwise generate defaults
        if (!is.null(names(model_list)) && all(nzchar(names(model_list)))) {
            model_names <- names(model_list)
        } else {
            model_names <- paste("Model", seq_along(model_list))
        }
    }
    
    ## Validate interactions_list if provided
    if (!is.null(interactions_list)) {
        if (!is.list(interactions_list)) {
            stop("interactions_list must be a list")
        }
        if (length(interactions_list) != length(model_list)) {
            stop("interactions_list must have the same length as model_list (", 
                 length(model_list), " models)")
        }
        ## Check each element is either NULL or character vector
        for (i in seq_along(interactions_list)) {
            if (!is.null(interactions_list[[i]])) {
                if (!is.character(interactions_list[[i]])) {
                    stop("Each element of interactions_list must be NULL or a character vector")
                }
            }
        }
    }

    ## Initialize results storage
    n_models <- length(model_list)
    comparison_list <- vector("list", n_models)
    models <- vector("list", n_models)
    names(models) <- model_names
    coef_results <- list()

    ## Fit each model
    for (i in seq_along(model_list)) {
        ## Get interaction count for message
        n_interact <- if (!is.null(interactions_list) && !is.null(interactions_list[[i]])) length(interactions_list[[i]]) else 0
        if (n_interact > 0) {
            message(sprintf("Fitting %s with %d predictors + %d interaction%s...", model_names[i], length(model_list[[i]]), n_interact, if(n_interact > 1) "s" else ""))
        } else {
            message(sprintf("Fitting %s with %d predictors...", model_names[i], length(model_list[[i]])))
        }
        
        ## Extract interaction terms for this model BEFORE tryCatch
        current_interactions <- if (!is.null(interactions_list)) {
                                    interactions_list[[i]]
                                } else {
                                    NULL
                                }
        
        ## Fit model using fit() for consistency
        model_result <- tryCatch({
            fit(data = data,
                outcome = outcome,
                interactions = current_interactions,
                predictors = model_list[[i]],
                model_type = model_type,
                family = family,
                random = random,
                conf_level = conf_level,
                labels = labels,
                keep_qc_stats = TRUE,
                verbose = verbose,
                ...)
        }, error = function(e) {
            message("  Warning - model failed to fit: ", e$message)
            NULL
        })

        ## Build comparison row - conditional on model type
        if (is.null(model_result)) {
            ## Model failed to fit
            row <- data.table::data.table(
                                   Model = model_names[i],
                                   N = nrow(data),
                                   Events = NA_integer_,
                                   Predictors = length(model_list[[i]]),
                                   Converged = "Failed",
                                   AIC = NA_real_,
                                   BIC = NA_real_,
                                   `Pseudo-R^2` = NA_real_,
                                   Concordance = NA_real_,
                                   `Global p` = NA_real_
                               )
            ## Add model-type specific columns for failed models
            if (model_type %in% c("glm", "negbin")) {
                row$`Brier Score` <- NA_real_
            } else if (model_type %in% c("lmer", "glmer", "coxme")) {
                row$Groups <- NA_integer_
                row$`Marginal R^2` <- NA_real_
                row$`Conditional R^2` <- NA_real_
                row$ICC <- NA_real_
                if (model_type == "glmer") {
                    row$`Brier Score` <- NA_real_
                }
            }
        } else {

            ## Extract model and raw data
            model <- attr(model_result, "model")
            raw_data <- attr(model_result, "raw_data")
            models[[model_names[i]]] <- model
            
            ## Store coefficients if requested
            if (include_coefficients) {
                coef_results[[model_names[i]]] <- model_result
            }
            
            ## Check convergence
            converged <- check_convergence(model)
            
            ## Extract metrics
            metrics <- extract_model_metrics(model, raw_data, model_type)

            ## Build comparison row
            row <- data.table::data.table(
                                   Model = model_names[i],
                                   N = metrics$n,
                                   Events = metrics$events,
                                   Predictors = length(model_list[[i]]),
                                   Converged = converged,
                                   AIC = if (!is.null(metrics$aic) && !is.na(metrics$aic)) round(metrics$aic, 1) else NA_real_,
                                   BIC = if (!is.null(metrics$bic) && !is.na(metrics$bic)) round(metrics$bic, 1) else NA_real_,
                                   `Pseudo-R^2` = if (!is.null(metrics$pseudo_r2) && !is.na(metrics$pseudo_r2)) round(metrics$pseudo_r2, 3) else NA_real_,
                                   Concordance = if (!is.null(metrics$concordance) && !is.na(metrics$concordance)) round(metrics$concordance, 3) else NA_real_,
                                   `Global p` = if (!is.null(metrics$global_p) && !is.na(metrics$global_p)) format_pvalues_fit(metrics$global_p, p_digits, marks) else NA_character_
                               )
            
            ## Add model-type specific columns
            if (model_type %in% c("glm", "negbin")) {
                row$`Brier Score` <- if (!is.null(metrics$brier_score)) round(metrics$brier_score, 3) else NA_real_
            } else if (model_type %in% c("lmer", "glmer", "coxme")) {
                ## Add mixed-effects specific columns
                row$Groups <- if (!is.null(metrics$n_groups)) as.integer(metrics$n_groups) else NA_integer_
                row$`Marginal R^2` <- if (!is.null(metrics$marginal_r2) && !is.na(metrics$marginal_r2)) round(metrics$marginal_r2, 3) else NA_real_
                row$`Conditional R^2` <- if (!is.null(metrics$conditional_r2) && !is.na(metrics$conditional_r2)) round(metrics$conditional_r2, 3) else NA_real_
                row$ICC <- if (!is.null(metrics$icc) && !is.na(metrics$icc)) round(metrics$icc, 3) else NA_real_
                
                if (model_type == "glmer") {
                    row$`Brier Score` <- if (!is.null(metrics$brier_score)) round(metrics$brier_score, 3) else NA_real_
                }
            }
        }
        
        ## Store row in pre-allocated list
        comparison_list[[i]] <- row
    }

    ## Combine all rows using rbindlist
    comparison <- data.table::rbindlist(comparison_list, fill = TRUE)

    ## Information criteria and likelihood-based comparisons are only valid
    ## across models fitted to the same observations. Each model is fitted to
    ## complete cases for its own predictor set, so a predictor carrying missing
    ## values silently reduces the sample for every model that contains it, and
    ## the resulting AIC and BIC are not comparable with those of the models
    ## that omit it.
    ##
    ## This is reported rather than corrected. Restricting all candidates to a
    ## common complete-case sample changes the estimates themselves, and the
    ## choice between that, imputation, or dropping the predictor belongs to the
    ## analyst.
    if ("N" %chin% names(comparison)) {
        fitted_rows <- !is.na(comparison$N)
        
        if ("Converged" %chin% names(comparison)) {
            fitted_rows <- fitted_rows & comparison$Converged != "Failed"
        }
        
        sample_sizes <- unique(comparison$N[fitted_rows])
        
        if (length(sample_sizes) > 1) {
            warning("Models were fitted to different numbers of observations (",
                    paste(sort(sample_sizes), collapse = ", "), "). ",
                    "AIC, BIC and other likelihood-based comparisons assume a ",
                    "common sample and are not valid across models fitted to ",
                    "different observations. Consider restricting the data to ",
                    "complete cases across all candidate predictors before ",
                    "comparing.",
                    call. = FALSE)
        }
    }

    ## Calculate scores and sort
    comparison <- calculate_model_scores(comparison, model_type, scoring_weights)

    ## Rename columns from ASCII to Unicode superscripts for display
    col_renames <- c(
        "Marginal R^2" = "Marginal R\u00b2",
        "Conditional R^2" = "Conditional R\u00b2",
        "Adjusted R^2" = "Adjusted R\u00b2"
    )
    ## LM models use R² (actual R-squared); all others use Pseudo-R²
    if (model_type == "lm") {
        col_renames[["Pseudo-R^2"]] <- "R\u00b2"
    } else {
        col_renames[["Pseudo-R^2"]] <- "Pseudo-R\u00b2"
    }
    for (old_name in names(col_renames)) {
        if (old_name %in% names(comparison)) {
            data.table::setnames(comparison, old_name, col_renames[[old_name]])
        }
    }

    ## Format for display (ensure proper column order with Score always last)
    ## Get all current column names
    all_cols <- names(comparison)
    
    ## Define the preferred order for known columns (Score will be moved to end)
    if (model_type %in% c("glm", "negbin")) {
        preferred_order <- c("Model", "N", "Events", "Predictors", "Converged",
                             "AIC", "BIC", "Pseudo-R\u00b2", "Concordance", "Brier Score",
                             "Global p")
    } else if (model_type %in% c("lmer", "glmer")) {
        preferred_order <- c("Model", "N", "Events", "Predictors", "Groups", "Converged",
                             "AIC", "BIC", "Pseudo-R\u00b2", "Marginal R\u00b2", "Conditional R\u00b2",
                             "ICC", "Concordance", "Global p")
    } else if (model_type == "coxme") {
        preferred_order <- c("Model", "N", "Events", "Predictors", "Groups", "Converged",
                             "AIC", "BIC", "Concordance", "Global p")
    } else if (model_type == "lm") {
        preferred_order <- c("Model", "N", "Predictors", "Converged",
                             "AIC", "BIC", "R\u00b2", "RMSE", "Global p")
    } else {
        preferred_order <- c("Model", "N", "Events", "Predictors", "Converged",
                             "AIC", "BIC", "Pseudo-R\u00b2", "Concordance", "Global p")
    }
    
    ## Build final column order: Model, Score, then preferred columns, then extras
    existing_preferred <- intersect(preferred_order, all_cols)
    existing_preferred <- setdiff(existing_preferred, "Model")
    other_cols <- setdiff(all_cols, c(preferred_order, "CMS"))
    final_order <- c("Model", "CMS", existing_preferred, other_cols)
    
    ## Only reorder columns that actually exist
    final_order <- intersect(final_order, all_cols)
    data.table::setcolorder(comparison, final_order)
    
    ## Attach attributes
    data.table::setattr(comparison, "models", models)
    data.table::setattr(comparison, "model_type", model_type)
    data.table::setattr(comparison, "outcome", outcome)
    
    if (!is.null(random)) {
        data.table::setattr(comparison, "random", random)
    }

    if (include_coefficients && length(coef_results) > 0) {
        coef_table <- combine_coefficient_tables(coef_results, model_names)
        data.table::setattr(comparison, "coefficients", coef_table)
    }

    ## Identify best model (first one after sorting)
    if (nrow(comparison) > 0) {
        data.table::setattr(comparison, "best_model", comparison$Model[1])
    }

    class(comparison) <- c("compfit_result", class(comparison))

    return(comparison)
}


### * Print function

#' Print method showing scoring methodology
#'
#' @param x Object of class \code{compfit_result}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @family regression functions
#' @export
print.compfit_result <- function(x, ...) {
    cat("\nModel Comparison Results\n")
    cat("Outcome: ", attr(x, "outcome"), "\n", sep = "")
    cat("Model Type: ", attr(x, "model_type"), "\n", sep = "")
    
    ## Show scoring weights
    weights <- attr(x, "weights")
    if (!is.null(weights)) {
        cat("\nCMS Weights:\n")
        for (metric in names(weights)) {
            ## Formatting for metric names
            display_name <- switch(metric,
                                   "convergence" = "Convergence",
                                   "aic" = "AIC",
                                   "bic" = "BIC",
                                   "concordance" = "Concordance",
                                   "c_stat" = "C-statistic",
                                   "c_index" = "C-index",
                                   "pseudo_r2" = "Pseudo-R\u00b2",
                                   "global_p" = "Global p-value",
                                   "rmse" = "RMSE",
                                   "brier" = "Brier score",
                                   "adj_r2" = "Adjusted R\u00b2",
                                   "marginal_r2" = "Marginal R\u00b2",
                                   "conditional_r2" = "Conditional R\u00b2",
                                   "icc" = "ICC",
                                   metric  # Generic fallback
                                   )
            cat(sprintf("  %s: %.0f%%\n", display_name, weights[[metric]] * 100))
        }
    }
    
    ## Identify best model
    if (nrow(x) > 0) {
        best_model <- x$Model[1]  # Already sorted by score
        cat("\nRecommended Model: ", best_model, " (CMS: ", x$CMS[1], ")\n", sep = "")
    }
    
    cat("\nModels ranked by selection score:\n")
    NextMethod("print", x)

    cat("\nCMS interpretation: 85+ Excellent, 75-84 Very Good, 65-74 Good, 55-64 Fair, < 55 Poor\n")
    
    if (!is.null(attr(x, "coefficients"))) {
        cat("\nNote: Coefficient comparison available via attr(result, 'coefficients')\n")
    }
    
    invisible(x)
}

#' Test Suite for compfit()
#' 
#' Comprehensive tests covering standard models, mixed-effects models,
#' interactions, edge cases, and output validation.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-compfit.R")

library(data.table)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Create binary outcome for logistic regression tests
## IMPORTANT: Add to BOTH datasets
clintrial$response <- as.integer(clintrial$os_status == 1 & clintrial$os_months < 24)
clintrial_complete$response <- as.integer(clintrial_complete$os_status == 1 & clintrial_complete$os_months < 24)

## Helper function to check compfit_result structure
expect_compfit_result <- function(result) {
    expect_s3_class(result, "compfit_result")
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("Model" %in% names(result))
    expect_true("N" %in% names(result))
    expect_true("Converged" %in% names(result))
    expect_true("AIC" %in% names(result))
    expect_true("CMS" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "models")))
    expect_true(!is.null(attr(result, "model_type")))
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "best_model")))
}

## Helper to check scores are valid
expect_valid_scores <- function(result) {
    scores <- result$`CMS`
    expect_true(all(scores >= 0 | is.na(scores)))
    expect_true(all(scores <= 100 | is.na(scores)))
    ## Should be sorted descending
    non_na_scores <- scores[!is.na(scores)]
    if (length(non_na_scores) > 1) {
        expect_true(all(diff(non_na_scores) <= 0))
    }
}


## ============================================================================
## SECTION 1: Basic Functionality Tests
## ============================================================================

test_that("compfit works with basic GLM models", {
    
    models <- list(
        base = c("age", "sex"),
        clinical = c("age", "sex", "smoking", "diabetes"),
        full = c("age", "sex", "smoking", "diabetes", "stage", "grade")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm",
        family = "binomial"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 3)
    expect_valid_scores(result)
    
    ## Check that all models have valid N
    expect_true(all(result$N > 0))
    
    ## Check concordance is between 0.5 and 1
    conc <- result$Concordance[!is.na(result$Concordance)]
    expect_true(all(conc >= 0.5 & conc <= 1))
})


test_that("compfit works with linear models", {
    
    models <- list(
        simple = c("age", "sex"),
        moderate = c("age", "sex", "stage", "ecog"),
        complex = c("age", "sex", "stage", "ecog", "treatment", "surgery")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "los_days",
        model_list = models,
        model_type = "lm"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 3)
    expect_equal(attr(result, "model_type"), "lm")
    
    ## Linear models should have R²
    expect_true("R\u00b2" %in% names(result))
})


test_that("compfit works with Cox models", {
    
    models <- list(
        base = c("age", "sex"),
        clinical = c("age", "sex", "stage", "grade"),
        full = c("age", "sex", "stage", "grade", "treatment", "surgery")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "Surv(os_months, os_status)",
        model_list = models,
        model_type = "coxph"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 3)
    expect_equal(attr(result, "model_type"), "coxph")
    
    ## Cox models should have Events column
    expect_true("Events" %in% names(result))
    expect_true(all(result$Events > 0))
})


test_that("compfit auto-detects model type correctly", {
    
    ## Binary outcome -> GLM
    result_binary <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = list(simple = c("age", "sex")),
        model_type = "auto"
    )
    expect_equal(attr(result_binary, "model_type"), "glm")
    
    ## Continuous outcome -> LM
    result_cont <- compfit(
        data = clintrial,
        outcome = "los_days",
        model_list = list(simple = c("age", "sex")),
        model_type = "auto"
    )
    expect_equal(attr(result_cont, "model_type"), "lm")
    
    ## Survival outcome -> coxph
    result_surv <- compfit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        model_list = list(simple = c("age", "sex")),
        model_type = "auto"
    )
    expect_equal(attr(result_surv, "model_type"), "coxph")
})


## ============================================================================
## SECTION 2: Mixed-Effects Model Tests
## ============================================================================

## Note: lme4 can cause segfaults when models are fitted within certain 
## evaluation contexts (testthat, tryCatch, etc.) due to C++ memory management
## issues. We use skip_on_cran() and careful error handling to mitigate this.

test_that("compfit works with lmer models", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    
    ## Force garbage collection before lme4 operations
    gc()
    
    models <- list(
        simple = c("age", "sex", "(1|site)")
    )
    
    ## Use a smaller dataset to reduce memory pressure
    test_data <- clintrial_complete[1:200, ]
    
    result <- compfit(
        data = test_data,
        outcome = "los_days",
        model_list = models,
        model_type = "lmer",
        REML = FALSE  # Use ML for valid AIC comparison
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 1)
    
    ## lmer-specific columns
    expect_true("Groups" %in% names(result))
    expect_true("Marginal R2" %in% names(result) || "Pseudo-R²" %in% names(result))
    expect_true("ICC" %in% names(result))
    
    ## Clean up

    gc()
})


test_that("compfit works with glmer models", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    
    gc()
    
    models <- list(
        simple = c("age", "sex", "(1|site)")
    )
    
    ## Use smaller dataset - need response column
    test_data <- clintrial_complete[1:200, ]
    
    result <- compfit(
        data = test_data,
        outcome = "response",
        model_list = models,
        model_type = "glmer",
        family = "binomial"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 1)
    
    ## glmer-specific columns
    expect_true("Groups" %in% names(result))
    expect_true("Concordance" %in% names(result))
    
    gc()
})


test_that("compfit works with coxme models", {
    
    skip_if_not_installed("coxme")
    skip_on_cran()  # Mixed models can segfault in test environments
    
    gc()
    
    models <- list(
        simple = c("age", "sex", "(1|site)")
    )
    
    ## Use smaller dataset
    test_data <- clintrial_complete[1:200, ]
    
    result <- compfit(
        data = test_data,
        outcome = "Surv(os_months, os_status)",
        model_list = models,
        model_type = "coxme"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 1)
    expect_equal(attr(result, "model_type"), "coxme")
    
    ## Should have Events and Groups
    expect_true("Events" %in% names(result))
    expect_true("Groups" %in% names(result))
    
    gc()
})


test_that("compfit auto-detects mixed models from random effects in predictors", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    
    gc()
    
    ## Use smaller dataset - response column is already in clintrial_complete
    test_data <- clintrial_complete[1:150, ]
    
    ## Continuous + random effects -> lmer
    result_lmer <- compfit(
        data = test_data,
        outcome = "los_days",
        model_list = list(mixed = c("age", "sex", "(1|site)")),
        model_type = "auto",
        REML = FALSE
    )
    expect_equal(attr(result_lmer, "model_type"), "lmer")
    
    gc()
    
    ## Binary + random effects -> glmer
    result_glmer <- compfit(
        data = test_data,
        outcome = "response",
        model_list = list(mixed = c("age", "sex", "(1|site)")),
        model_type = "auto"
    )
    expect_equal(attr(result_glmer, "model_type"), "glmer")
    
    gc()
})


test_that("compfit handles different random effects structures", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    skip("Random slope models are prone to convergence issues and segfaults")
    
    gc()
    
    models <- list(
        intercept_only = c("age", "treatment", "(1|site)"),
        random_slope = c("age", "treatment", "(1 + age|site)")
    )
    
    ## This may produce convergence warnings for complex random effects
    result <- suppressWarnings(compfit(
        data = clintrial_complete,
        outcome = "los_days",
        model_list = models,
        model_type = "lmer",
        REML = FALSE
    ))
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 2)
    
    gc()
})


## ============================================================================
## SECTION 3: Interaction Term Tests
## ============================================================================

test_that("compfit works with interactions_list parameter", {
    
    models <- list(
        base = c("age", "treatment"),
        interaction = c("age", "treatment")
    )
    
    interactions <- list(
        NULL,  # No interactions for base model
        c("age:treatment")  # Interaction for second model
    )
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = models,
        interactions_list = interactions,
        model_type = "glm"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 2)
    
    ## Interaction model should have more predictors listed
    ## (Note: exact count depends on implementation)
})


test_that("compfit handles multiple interactions per model", {
    
    models <- list(
        simple = c("age", "treatment", "stage"),
        complex = c("age", "treatment", "stage")
    )
    
    interactions <- list(
        NULL,
        c("age:treatment", "treatment:stage")
    )
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = models,
        interactions_list = interactions,
        model_type = "glm"
    )
    
    expect_compfit_result(result)
})


test_that("compfit validates interactions_list length matches model_list", {
    
    models <- list(
        a = c("age"),
        b = c("age", "sex")
    )
    
    ## Wrong number of interactions
    interactions <- list(NULL)  # Only 1 instead of 2
    
    expect_error(
        compfit(
            data = clintrial,
            outcome = "response",
            model_list = models,
            interactions_list = interactions,
            model_type = "glm"
        ),
        regexp = "same length"
    )
})


## ============================================================================
## SECTION 4: Model Naming Tests
## ============================================================================

test_that("compfit uses provided model_names", {
    
    models <- list(
        c("age", "sex"),
        c("age", "sex", "stage")
    )
    
    custom_names <- c("Baseline", "Extended")
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_names = custom_names,
        model_type = "glm"
    )
    
    expect_true(all(custom_names %in% result$Model))
})


test_that("compfit uses list names as model names when no model_names provided", {
    
    models <- list(
        base = c("age", "sex"),
        full = c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    expect_true(all(c("base", "full") %in% result$Model))
})


test_that("compfit generates default model names for unnamed list", {
    
    models <- list(
        c("age", "sex"),
        c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    ## Should have some model names (Model 1, Model 2 or similar)
    expect_true(all(!is.na(result$Model)))
    expect_equal(length(unique(result$Model)), 2)
})


test_that("compfit passes labels to underlying fit function", {
    
    models <- list(simple = c("age", "sex"))
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = models,
        model_type = "glm",
        labels = clintrial_labels,
        include_coefficients = TRUE
    )
    
    coef_table <- attr(result, "coefficients")
    
    ## If labels were applied, coefficient table should use them
    ## (exact check depends on implementation)
    expect_true(!is.null(coef_table))
})


## ============================================================================
## SECTION 5: Scoring and Ranking Tests
## ============================================================================

test_that("compfit ranks models by CMS in descending order", {
    
    models <- list(
        minimal = c("age"),
        base = c("age", "sex"),
        full = c("age", "sex", "stage", "grade", "treatment")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "Surv(os_months, os_status)",
        model_list = models,
        model_type = "coxph"
    )
    
    scores <- result$`CMS`
    
    ## Scores should be in descending order (best first)
    expect_true(all(diff(scores) <= 0))
})


test_that("compfit identifies best model correctly", {
    
    models <- list(
        a = c("age"),
        b = c("age", "sex"),
        c = c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    best_model <- attr(result, "best_model")
    
    ## Best model should be the first row (highest score)
    expect_equal(best_model, result$Model[1])
})


test_that("compfit accepts custom scoring weights", {
    
    custom_weights <- list(
        convergence = 0.10,
        aic = 0.30,
        concordance = 0.40,
        pseudo_r2 = 0.20
    )
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = list(simple = c("age", "sex")),
        model_type = "glm",
        scoring_weights = custom_weights
    )
    
    expect_compfit_result(result)
    
    ## Weights should be stored in attributes
    expect_true(!is.null(attr(result, "weights")))
})


## ============================================================================
## SECTION 6: Coefficient Table Tests
## ============================================================================

test_that("compfit returns coefficient table when requested", {
    
    models <- list(
        base = c("age", "sex"),
        full = c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm",
        include_coefficients = TRUE
    )
    
    coef_table <- attr(result, "coefficients")
    
    expect_true(!is.null(coef_table))
    expect_s3_class(coef_table, "data.table")
    expect_true("Model" %in% names(coef_table))
    
    ## Should have rows for both models - check model names are present
    ## Models might be renamed to "Model 1", "Model 2" or keep original names
    model_names_in_coef <- unique(coef_table$Model)
    expect_equal(length(model_names_in_coef), 2)
})


test_that("compfit returns NULL coefficients when not requested", {
    
    models <- list(simple = c("age", "sex"))
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = models,
        model_type = "glm",
        include_coefficients = FALSE
    )
    
    expect_null(attr(result, "coefficients"))
})


## ============================================================================
## SECTION 7: Edge Cases and Error Handling
## ============================================================================

test_that("compfit handles single model", {
    
    models <- list(only = c("age", "sex", "stage"))
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 1)
})


test_that("compfit continues when one model fails", {
    
    ## This test checks that compfit handles model failures gracefully
    ## We can't easily create a guaranteed failure, so we just verify
    ## the function handles edge cases
    
    models <- list(
        good = c("age", "sex"),
        also_good = c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    expect_compfit_result(result)
})


test_that("compfit validates model_list is not empty", {
    
    expect_error(
        compfit(
            data = clintrial,
            outcome = "response",
            model_list = list(),
            model_type = "glm"
        )
    )
})


test_that("compfit handles missing data appropriately", {
    
    models <- list(
        has_missing = c("age", "bmi", "smoking"),  # bmi and smoking have NAs
        complete = c("age", "sex")  # No NAs in these
    )
    
    ## The candidates are deliberately fitted to different samples, which
    ## compfit() reports.
    result <- expect_warning(
        compfit(
            data = clintrial,
            outcome = "response",
            model_list = models,
            model_type = "glm"
        ),
        "different numbers of observations"
    )
    
    expect_compfit_result(result)
    
    ## Get N values - note: results may be reordered by CMS
    ## Use which() to find the correct rows
    idx_missing <- which(result$Model == "has_missing")
    idx_complete <- which(result$Model == "complete")
    
    ## Both models should be present
    expect_true(length(idx_missing) == 1)
    expect_true(length(idx_complete) == 1)
    
    n_missing <- result$N[idx_missing]
    n_complete <- result$N[idx_complete]
    
    ## Model with complete predictors should have >= N as model with missing
    expect_true(n_complete >= n_missing)
})


test_that("compfit handles invalid model_type gracefully", {
    
    models <- list(simple = c("age", "sex"))
    
    ## This should either error or fall back to auto
    ## The exact behavior depends on implementation
    error_occurred <- FALSE
    result <- tryCatch({
        compfit(
            data = clintrial,
            outcome = "response",
            model_list = models,
            model_type = "invalid_type"
        )
    }, error = function(e) {
        error_occurred <<- TRUE
        NULL
    })
    
    ## Should either error or produce a result
    expect_true(error_occurred || inherits(result, "compfit_result"))
})


## ============================================================================
## SECTION 8: Output Format Tests
## ============================================================================

test_that("CMS is always the second column", {
    
    ## Test with GLM
    result_glm <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = list(simple = c("age", "sex")),
        model_type = "glm"
    )
    expect_equal(names(result_glm)[2], "CMS")
    
    ## Test with LM
    result_lm <- compfit(
        data = clintrial,
        outcome = "los_days",
        model_list = list(simple = c("age", "sex")),
        model_type = "lm"
    )
    expect_equal(names(result_lm)[2], "CMS")
    
    ## Test with Cox
    result_cox <- compfit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        model_list = list(simple = c("age", "sex")),
        model_type = "coxph"
    )
    expect_equal(names(result_cox)[2], "CMS")
})


test_that("CMS is second column for mixed models", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()
    
    gc()
    
    test_data <- clintrial_complete[1:150, ]
    
    result <- compfit(
        data = test_data,
        outcome = "los_days",
        model_list = list(simple = c("age", "sex", "(1|site)")),
        model_type = "lmer",
        REML = FALSE
    )
    
    expect_equal(names(result)[2], "CMS")
    
    gc()
})


test_that("compfit column order is correct for GLM", {
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = list(simple = c("age", "sex")),
        model_type = "glm"
    )
    
    expected_order <- c("Model", "CMS", "N", "Events", "Predictors", "Converged",
                       "AIC", "BIC", "Pseudo-R²", "Concordance", "Brier Score",
                       "Global p")
    
    actual_cols <- names(result)
    
    ## Check that columns present in both lists appear in the expected relative order
    common_cols <- intersect(expected_order, actual_cols)
    actual_positions <- match(common_cols, actual_cols)
    expected_positions <- match(common_cols, expected_order)
    
    ## The relative ordering should be preserved (both should be monotonically increasing)
    expect_true(all(diff(actual_positions) > 0), 
                info = "Actual columns should maintain expected relative order")
    expect_true(all(diff(expected_positions) > 0),
                info = "Expected columns should be in order")
})


test_that("compfit column order is correct for lmer", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    
    gc()
    
    ## Use smaller dataset
    test_data <- clintrial_complete[1:150, ]
    
    result <- compfit(
        data = test_data,
        outcome = "los_days",
        model_list = list(mixed = c("age", "sex", "(1|site)")),
        model_type = "lmer",
        REML = FALSE
    )
    
    ## Should have lmer-specific columns
    expect_true("Groups" %in% names(result))
    expect_true("ICC" %in% names(result))
    
    gc()
})


test_that("print.compfit_result works without error", {
    
    models <- list(
        base = c("age", "sex"),
        full = c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    ## Should print without error
    expect_output(print(result), "Model Comparison Results")
    expect_output(print(result), "CMS")
    expect_output(print(result), "Recommended Model")
})


## ============================================================================
## SECTION 9: Convergence Detection Tests
## ============================================================================

test_that("compfit detects convergence status", {
    
    models <- list(
        simple = c("age", "sex"),
        complex = c("age", "sex", "stage", "grade", "treatment", "surgery", "ecog")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    ## Converged column should exist and have valid values
    expect_true("Converged" %in% names(result))
    expect_true(all(result$Converged %in% c("Yes", "No", "Suspect", "Failed")))
})


test_that("compfit detects singular fits in mixed models", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    skip("Singular fit detection requires careful setup and is prone to segfaults")
    
    gc()
    
    ## Create a scenario likely to produce singular fit
    ## (very few groups or variance near zero)
    small_data <- clintrial_complete[1:100, ]
    
    models <- list(
        simple = c("age", "(1|site)")
    )
    
    ## May produce singular fit warning
    result <- suppressWarnings(compfit(
        data = small_data,
        outcome = "los_days",
        model_list = models,
        model_type = "lmer",
        REML = FALSE
    ))
    
    expect_compfit_result(result)
    ## Convergence status should reflect any issues
    expect_true(result$Converged[1] %in% c("Yes", "Suspect"))
    
    gc()
})


## ============================================================================
## SECTION 10: Attribute Preservation Tests
## ============================================================================

test_that("compfit preserves fitted models in attributes", {
    
    models <- list(
        base = c("age", "sex"),
        full = c("age", "sex", "stage")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    
    stored_models <- attr(result, "models")
    
    expect_true(is.list(stored_models))
    expect_equal(length(stored_models), 2)
    
    ## Models should be stored - names might be original or display names
    ## Just check that we have 2 models that are glm objects
    expect_true(all(sapply(stored_models, function(m) inherits(m, "glm") || is.null(m))))
})


test_that("compfit stores weights in attributes", {
    
    custom_weights <- list(
        convergence = 0.10,
        aic = 0.30,
        concordance = 0.40,
        pseudo_r2 = 0.20
    )
    
    result <- compfit(
        data = clintrial,
        outcome = "response",
        model_list = list(simple = c("age", "sex")),
        model_type = "glm",
        scoring_weights = custom_weights
    )
    
    stored_weights <- attr(result, "weights")
    expect_true(!is.null(stored_weights))
})


## ============================================================================
## SECTION 11: Integration Tests (Complex Workflows)
## ============================================================================

test_that("compfit supports full clinical trial analysis workflow", {
    
    ## Define clinically meaningful model hierarchy
    models <- list(
        demographics = c("age", "sex", "race"),
        clinical = c("age", "sex", "race", "ecog", "smoking", "diabetes"),
        disease = c("age", "sex", "race", "ecog", "smoking", "diabetes", 
                    "stage", "grade"),
        full = c("age", "sex", "race", "ecog", "smoking", "diabetes",
                 "stage", "grade", "treatment", "surgery")
    )
    
    result <- compfit(
        data = clintrial_complete,
        outcome = "Surv(os_months, os_status)",
        model_list = models,
        model_names = c("Demographics", "Clinical", "Disease", "Full"),
        model_type = "coxph",
        labels = clintrial_labels,
        include_coefficients = TRUE
    )
    
    expect_compfit_result(result)
    expect_equal(nrow(result), 4)
    expect_true(!is.null(attr(result, "coefficients")))
    
    ## Verify meaningful discrimination improvement
    ## (fuller models should generally have better concordance)
    ## Note: Not strictly guaranteed, but expected with this data
})


test_that("compfit supports mixed model comparison workflow", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()  # lme4 can segfault in test environments
    skip("Comparing fixed vs mixed models in auto mode is complex and prone to issues")
    
    gc()
    
    ## Compare fixed vs mixed effects
    models <- list(
        fixed = c("age", "treatment", "stage"),
        mixed_intercept = c("age", "treatment", "stage", "(1|site)"),
        mixed_slope = c("age", "treatment", "stage", "(1 + treatment|site)")
    )
    
    ## mixed_slope may have convergence issues
    result <- suppressWarnings(compfit(
        data = clintrial_complete,
        outcome = "los_days",
        model_list = models,
        model_type = "auto",
        REML = FALSE
    ))
    
    expect_compfit_result(result)
    
    ## The fixed-only model will be fit as lm, others as lmer
    ## This tests the auto-detection per model
    
    gc()
})


test_that("compfit works with PFS outcome", {
    
    models <- list(
        base = c("age", "sex", "ecog"),
        full = c("age", "sex", "ecog", "stage", "treatment")
    )
    
    result <- compfit(
        data = clintrial,
        outcome = "Surv(pfs_months, pfs_status)",
        model_list = models,
        model_type = "coxph"
    )
    
    expect_compfit_result(result)
    expect_true(all(result$Events > 0))
})


## ============================================================================
## SECTION 12: Performance and Scalability Tests
## ============================================================================

test_that("compfit handles many models efficiently", {
    
    ## Create 10 nested models
    base_vars <- c("age", "sex")
    add_vars <- c("stage", "grade", "ecog", "smoking", "diabetes", 
                  "hypertension", "treatment", "surgery")
    
    models <- list()
    for (i in 0:length(add_vars)) {
        if (i == 0) {
            models[[paste0("model_", i)]] <- base_vars
        } else {
            models[[paste0("model_", i)]] <- c(base_vars, add_vars[1:i])
        }
    }
    
    ## Should complete in reasonable time
    time_start <- Sys.time()
    result <- compfit(
        data = clintrial_complete,
        outcome = "response",
        model_list = models,
        model_type = "glm"
    )
    time_end <- Sys.time()
    
    expect_compfit_result(result)
    expect_equal(nrow(result), length(models))
    
    ## Should complete in under 30 seconds for 10 models
    expect_true(difftime(time_end, time_start, units = "secs") < 30)
})

## Information criteria are comparable only across models fitted to the same
## observations. Each candidate is fitted to complete cases for its own
## predictor set, so a predictor carrying missing values reduces the sample for
## the models that contain it.

## clintrial carries missing values of its own, so tests start from a
## complete-case base and introduce missingness only where one is needed.
compfit_base <- function() {
    keep <- c("readmission_30d", "age", "sex", "bmi")
    df <- fresh_clintrial()
    df[stats::complete.cases(df[, keep]), keep]
}

compfit_candidates <- list(base = c("age", "sex"),
                           extended = c("age", "sex", "bmi"))


test_that("compfit warns when candidate models are fitted to different samples", {

    df <- compfit_base()
    df$bmi[seq_len(100)] <- NA

    expect_warning(
        compfit(
            data = df,
            outcome = "readmission_30d",
            model_list = compfit_candidates,
            model_type = "glm",
            family = "binomial"
        ),
        "different numbers of observations"
    )
})


test_that("compfit is silent when candidate models share a sample", {

    expect_no_warning(
        compfit(
            data = compfit_base(),
            outcome = "readmission_30d",
            model_list = compfit_candidates,
            model_type = "glm",
            family = "binomial"
        ),
        message = "different numbers of observations"
    )
})


test_that("the documented comparison workflow is quiet and well behaved", {

    ## Mirrors the compfit() examples: a single complete-case dataset shared by
    ## every comparison, so information criteria describe a common sample.
    df <- fresh_clintrial()
    keep <- c("readmission_30d", "os_months", "os_status", "age", "sex",
              "smoking", "diabetes", "stage", "ecog", "grade", "treatment")
    dat <- stats::na.omit(df[, keep])

    models <- list(
        base = c("age", "sex"),
        clinical = c("age", "sex", "smoking", "diabetes"),
        full = c("age", "sex", "smoking", "diabetes", "stage", "ecog")
    )

    result <- expect_no_warning(
        suppressMessages(compfit(
            data = dat,
            outcome = "readmission_30d",
            model_list = models,
            model_names = c("Base", "Clinical", "Full")
        )),
        message = "different numbers of observations"
    )

    ## Every model uses the same observations
    expect_length(unique(result$N), 1L)

    ## No model reports a suspect fit. os_status produced complete separation at
    ## ECOG 3, where every patient had the event; readmission_30d does not.
    expect_false(any(result$Converged == "Suspect"))
})


test_that("coefficient estimates from the documented models are finite", {

    skip_if_not_installed("xtable")

    df <- fresh_clintrial()
    keep <- c("readmission_30d", "age", "sex", "smoking", "diabetes",
              "stage", "ecog")
    dat <- stats::na.omit(df[, keep])

    detailed <- suppressMessages(compfit(
        data = dat,
        outcome = "readmission_30d",
        model_list = list(base = c("age", "sex"),
                          full = c("age", "sex", "smoking", "diabetes",
                                   "stage", "ecog")),
        include_coefficients = TRUE
    ))

    coefs <- attr(detailed, "coefficients")
    effect_col <- grep("CI", names(coefs), value = TRUE)[1]
    skip_if(is.na(effect_col), "no effect column found")

    ## Complete separation renders as an implausibly large estimate rather than
    ## as NA, so the check is on magnitude
    numeric_part <- suppressWarnings(
        as.numeric(sub(" .*", "", coefs[[effect_col]])))
    expect_true(all(is.na(numeric_part) | numeric_part < 1000))
})

### * Main function

#' Create Forest Plot for Linear Models
#'
#' Generates a publication-ready forest plot that combines a formatted data table 
#' with a graphical representation of regression coefficients from a linear model. 
#' The plot integrates variable names, group levels, sample sizes, coefficients 
#' with confidence intervals, \emph{p}-values, and model diagnostics (\emph{R}\eqn{^2},
#' \emph{F}-statistic, AIC) in a single comprehensive visualization designed for
#' manuscripts and presentations.
#'
#' @param x Either a fitted linear model object (class \code{lm} or \code{lmerMod}), 
#'   a \code{fit_result} object from \code{fit()}, or a \code{fullfit_result}
#'   object from \code{fullfit()}. When a \code{fit_result} or \code{fullfit_result}
#'   is provided, the model, data, and labels are automatically extracted.
#'   
#' @param data Data frame or data.table containing the original data used to 
#'   fit the model. If \code{NULL} (default) and \code{x} is a model, the function 
#'   attempts to extract data from the model object. If \code{x} is a \code{fit_result},
#'   data is extracted automatically. Providing data explicitly is recommended when
#'   passing a model directly.
#'   
#' @param title Character string specifying the plot title displayed at the top. 
#'   Default is \code{"Linear Model"}.
#'   
#' @param effect_label Character string for the effect measure label on the 
#'   forest plot axis. Default is \code{"Coefficient"}.
#'   
#' @param digits Integer specifying the number of decimal places for coefficients 
#'   and confidence intervals. Default is 2.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param conf_level Numeric confidence level for confidence intervals. Must be
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
#'   percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
#'   when \code{conf_level = 0.90}).
#'   
#' @param font_size Numeric multiplier controlling the base font size for all 
#'   text elements. Default is 1.0.
#'   
#' @param annot_size Numeric value controlling the relative font size for data 
#'   annotations. Default is 3.88.
#'   
#' @param header_size Numeric value controlling the relative font size for column 
#'   headers. Default is 5.82.
#'   
#' @param title_size Numeric value controlling the relative font size for the 
#'   main plot title. Default is 23.28.
#'   
#' @param table_width Numeric value between 0 and 1 specifying the proportion of 
#'   total plot width allocated to the data table. Default is 0.6.
#'   
#' @param plot_width Numeric value specifying the intended output width in 
#'   specified \code{units}. Default is \code{NULL} (automatic).
#'   
#' @param plot_height Numeric value specifying the intended output height in 
#'   specified \code{units}. Default is \code{NULL} (automatic).
#'   
#' @param show_n Logical. If \code{TRUE}, includes a column showing group-specific 
#'   sample sizes.
#'   Counts describe the observations used in fitting rather than every row
#'   supplied. Default is \code{TRUE}.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels under 
#'   their parent variable name, creating hierarchical structure. The "Group" 
#'   column is hidden when \code{TRUE}. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses binary categorical 
#'   variables into single rows. Automatically sets \code{indent_groups = TRUE}. 
#'   Default is \code{FALSE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. If \code{FALSE} (default), variable names are displayed in plain
#'   text.
#'   
#' @param center_padding Numeric value specifying horizontal spacing between 
#'   table and forest plot. Default is 4.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating gray 
#'   background shading to different variables. Default is \code{TRUE}.
#'   
#' @param ref_label Character string to display for reference categories of 
#'   factor variables. Default is \code{"reference"}.
#'   
#' @param labels Named character vector providing custom display labels for 
#'   variables. Example: \code{c(age = "Age (years)", height = "Height (cm)")}. 
#'   Default is \code{NULL}, in which case the labels attached by the function
#'   that produced \code{x} are used, where \code{x} is a \pkg{summata} result
#'   rather than a model object. Original variable names are used where neither
#'   is available.
#' @param units Character string specifying units for plot dimensions: 
#'   \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.
#'   
#' @param color Character string specifying the color for coefficient point 
#'   estimates in the forest plot. Default is \code{"#5A8F5A"} (green). Use 
#'   hex codes or R color names.
#'
#' @param qc_footer Logical. If \code{TRUE}, displays model quality control
#'   statistics in the footer (observations analyzed, \emph{R}\eqn{^2}, adjusted
#'   \emph{R}\eqn{^2}, \emph{F}-statistic, AIC). Default is \code{TRUE}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @return A \code{ggplot} object containing the complete forest plot. The plot 
#'   can be:
#'   \itemize{
#'     \item Displayed directly: \code{print(plot)}
#'     \item Saved to file: \code{forestsave(plot, "forest.pdf")}
#'     \item Further customized with \pkg{ggplot2} functions
#'   }
#'   
#'   The returned object includes an attribute \code{"rec_dims"} 
#'   accessible via \code{attr(plot, "rec_dims")}, which is a list 
#'   containing:
#'   \describe{
#'     \item{width}{Numeric. Recommended plot width in specified units}
#'     \item{height}{Numeric. Recommended plot height in specified units}
#'   }
#'   
#'   These recommendations are automatically calculated based on the number of 
#'   variables, text sizes, and layout parameters, and are printed to console 
#'   if \code{plot_width} or \code{plot_height} are not specified. The list also 
#'   carries a \code{units} element recording the units the dimensions are 
#'   expressed in, matching the \code{units} argument. \code{forestsave()} reads 
#'   all three and requires no further handling.
#'
#'   The returned object also includes an attribute \code{"table_data"}
#'   accessible via \code{attr(plot, "table_data")}, a data.table holding the
#'   values drawn in the plot: one row per term in model order, with the
#'   variable, factor level, sample size, estimate, confidence bounds and
#'   p-value. Sample sizes describe the observations used in fitting rather
#'   than every row supplied, so they sum to the model sample size within each
#'   variable.
#'
#' @details
#' \strong{Linear Model-Specific Features:}
#' 
#' The linear model forest plot differs from logistic and Cox plots in several ways:
#' \itemize{
#'   \item \strong{Coefficients}: Raw regression coefficients shown (not exponentiated)
#'   \item \strong{Reference line}: At coefficient = 0 (not at 1)
#'   \item \strong{Linear scale}: Forest plot uses linear scale (not log scale)
#'   \item \strong{No events column}: Only sample sizes shown (no event counts)
#'   \item \strong{\emph{R}\eqn{^2} statistics}: Model fit assessed by \emph{R}\eqn{^2} and adjusted \emph{R}\eqn{^2}
#'   \item \strong{\emph{F}-test}: Overall model significance from \emph{F}-statistic
#' }
#' 
#' \strong{Plot Components:}
#' 
#' \enumerate{
#'   \item \strong{Title}: Centered at top
#'   \item \strong{Data Table} (left): Contains:
#'     \itemize{
#'       \item Variable: Predictor names
#'       \item Group: Factor levels (if applicable)
#'       \item \emph{n}: Sample sizes by group
#'       \item Coefficient (95\% CI); \emph{p}-value: Raw coefficients with CIs and \emph{p}-values
#'     }
#'   \item \strong{Forest Plot} (right):
#'     \itemize{
#'       \item Point estimates (squares sized by sample size)
#'       \item 95\% confidence intervals (error bars)
#'       \item Reference line at coefficient = 0
#'       \item Linear scale
#'     }
#'   \item \strong{Model Statistics} (footer):
#'     \itemize{
#'       \item Observations analyzed (with total observations and percentage)
#'       \item \emph{R}\eqn{^2} and adjusted \emph{R}\eqn{^2}
#'       \item \emph{F}-statistic with degrees of freedom and \emph{p}-value
#'       \item AIC
#'     }
#' }
#' 
#' \strong{Interpreting Coefficients:}
#' 
#' Linear regression coefficients represent the change in the outcome variable 
#' for a one-unit change in the predictor:
#' \itemize{
#'   \item \strong{Continuous predictors}: Coefficient = change in Y per unit of X
#'   \item \strong{Binary predictors}: Coefficient = difference in Y between groups
#'   \item \strong{Factor predictors}: Coefficients = differences from reference 
#'     category
#'   \item \strong{Sign matters}: Positive = increase in Y, Negative = decrease in Y
#'   \item \strong{Zero crossing}: CI crossing zero suggests no significant effect
#' }
#' 
#' Example: If the coefficient for "age" is 0.50 when predicting BMI, 
#' BMI increases by 0.50 kg/m\eqn{^2} for each additional year of age.
#' 
#' \strong{Model Fit Statistics:}
#' 
#' The footer displays key diagnostics:
#' \itemize{
#'   \item \strong{\emph{R}\eqn{^2}}: Proportion of variance explained (0 to 1)
#'     \itemize{
#'       \item 0.0-0.3: Weak explanatory power
#'       \item 0.3-0.5: Moderate
#'       \item 0.5-0.7: Good
#'       \item > 0.7: Strong (rare in social/biological sciences)
#'     }
#'   \item \strong{Adjusted \emph{R}\eqn{^2}}: \emph{R}\eqn{^2} penalized for number of predictors
#'     \itemize{
#'       \item Always \eqn{\le} \emph{R}\eqn{^2}
#'       \item Preferred for model comparison
#'       \item Accounts for model complexity
#'     }
#'   \item \strong{\emph{F}-statistic}: Tests null hypothesis that all coefficients = 0
#'     \itemize{
#'       \item Degrees of freedom: df1 = # predictors, df2 = # observations - # predictors - 1
#'       \item Significant \emph{p}-value indicates model explains variance better than intercept-only
#'     }
#'   \item \strong{AIC}: For model comparison (lower is better)
#' }
#' 
#' \strong{Assumptions:}
#' 
#' Linear regression assumes:
#' \enumerate{
#'   \item Linearity of relationships
#'   \item Independence of observations
#'   \item Homoscedasticity (constant variance)
#'   \item Normality of residuals
#'   \item No multicollinearity
#' }
#' 
#' Check assumptions using:
#' \itemize{
#'   \item \code{plot(model)} for diagnostic plots
#'   \item \code{car::vif(model)} for multicollinearity
#'   \item \code{lmtest::bptest(model)} for heteroscedasticity
#'   \item \code{shapiro.test(residuals(model))} for normality
#' }
#' 
#' \strong{Reference Categories:}
#' 
#' For factor variables:
#' \itemize{
#'   \item First level is the reference (coefficient = 0)
#'   \item Other levels show difference from reference
#'   \item Reference displayed with \code{ref_label}
#'   \item Relevel factors before modeling if needed: 
#'     \code{factor(x, levels = c("desired_ref", ...))}
#' }
#' 
#' \strong{Sample Size Reporting:}
#' 
#' The "\emph{n}" column shows:
#' \itemize{
#'   \item For continuous variables: Total observations with non-missing data
#'   \item For factor variables: Number of observations in each category
#'   \item Footer shows observations used in fitting, the total supplied, and 
#'     the percentage retained (accounting for missing values)
#' }
#'
#' @seealso 
#' \code{\link{autoforest}} for automatic model detection,
#' \code{\link{glmforest}} for logistic/GLM forest plots,
#' \code{\link{coxforest}} for Cox model forest plots,
#' \code{\link{uniforest}} for univariable screening forest plots,
#' \code{\link{multiforest}} for multi-outcome forest plots,
#' \code{\link[stats]{lm}} for fitting linear models,
#' \code{\link{fit}} for regression modeling,
#'   \code{\link{forestsave}} for saving with recommended dimensions
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example model
#' model1 <- lm(bmi ~ age + sex + smoking, data = clintrial)
#'
#' # Example 1: Basic linear model forest plot
#' p <- lmforest(model1, data = clintrial)
#'
#' \donttest{
#' 
#' old_width <- options(width = 180)
#' 
#' # Example 2: With custom labels and title
#' plot2 <- lmforest(
#'     x = model1,
#'     data = clintrial,
#'     title = "Predictors of Body Mass Index",
#'     effect_label = "Change in BMI (kg/m^2)",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 3: Comprehensive model with indented layout
#' model3 <- lm(
#'     bmi ~ age + sex + smoking + hypertension + diabetes + creatinine,
#'     data = clintrial
#' )
#' 
#' plot3 <- lmforest(
#'     x = model3,
#'     data = clintrial,
#'     labels = clintrial_labels,
#'     indent_groups = TRUE,
#'     zebra_stripes = TRUE
#' )
#' 
#' # Example 4: Condensed layout
#' plot4 <- lmforest(
#'     x = model3,
#'     data = clintrial,
#'     condense_table = TRUE,
#'     labels = clintrial_labels
#' )
#' 
#' # Example 5: Different outcome (hemoglobin)
#' model5 <- lm(
#'     hemoglobin ~ age + sex + bmi + smoking + creatinine,
#'     data = clintrial
#' )
#' 
#' plot5 <- lmforest(
#'     x = model5,
#'     data = clintrial,
#'     title = "Predictors of Baseline Hemoglobin",
#'     effect_label = "Change in Hemoglobin (g/dL)",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 6: Save with recommended dimensions
#' forestsave(plot5, file.path(tempdir(), "linear_forest.pdf"))
#'
#' options(old_width)
#' 
#' }
#' @family visualization functions
#' @export
lmforest <- function(x, data = NULL,
                     title = "Linear Model",
                     effect_label = "Coefficient",
                     digits = 2,
                     p_digits = 3,
                     conf_level = 0.95,
                     font_size = 1.0,
                     annot_size = 3.88,
                     header_size = 5.82,
                     title_size = 23.28,
                     plot_width = NULL,
                     plot_height = NULL,
                     table_width = 0.6,
                     show_n = TRUE,
                     indent_groups = FALSE,
                     condense_table = FALSE,
                     bold_variables = FALSE,
                     center_padding = 4,
                     zebra_stripes = TRUE,
                     ref_label = "reference",
                     labels = NULL,
                     units = "in",
                     color = "#5A8F5A",
                     qc_footer = TRUE,
                     number_format = NULL) {
    
    ## Check for required packages
    if (!requireNamespace("data.table", quietly = TRUE)) {
        stop("Package 'data.table' is required but not installed.")
    }
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required but not installed.")
    }
    if (!requireNamespace("grid", quietly = TRUE)) {
        stop("Package 'grid' is required but not installed.")
    }
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    ## Handle input: accept either a model object or a fit_result/fullfit_result from fit()/fullfit()
    if (inherits(x, "fit_result") || inherits(x, "fullfit_result")) {
        ## Extract model, data, and labels from fit_result or fullfit_result
        model <- attr(x, "model")
        
        if (is.null(model)) {
            stop("The fit_result/fullfit_result does not contain a model.\n",
                 "This may occur if fullfit() was run with columns='uni' only.")
        }
        
        ## Use data from fit_result if not provided
        if (is.null(data)) {
            ## Try to get data from model
            if (inherits(model, c("lmerMod", "lmerModLmerTest", "merMod"))) {
                data <- model@frame
            } else if (!is.null(model$data)) {
                data <- model$data
            } else if (!is.null(model$model)) {
                data <- model$model
            } else {
                tryCatch({
                    data <- eval(model$call$data)
                }, error = function(e) {
                    data <- NULL
                })
            }
        }
        
        ## Use labels from fit_result if not provided
        if (is.null(labels)) {
            labels <- attr(x, "labels")
        }
        
        ## Validate that the model is a linear model (not GLM)
        is_glm <- inherits(model, "glm")
        is_valid_lm <- inherits(model, c("lm", "lmerMod", "lmerModLmerTest", "merMod")) && !is_glm
        
        if (!is_valid_lm) {
            stop("fit_result does not contain a linear model (lm or lmerMod).\n",
                 "Model type: ", class(model)[1], "\n",
                 "Use glmforest() for GLM models or coxforest() for Cox models.")
        }
    } else if (inherits(x, c("lm", "lmerMod", "lmerModLmerTest", "merMod")) && !inherits(x, "glm")) {
        ## Direct model object (lm but not glm)
        model <- x
    } else {
        stop("x must be either:\n",
             "  - An lm or lmerMod model object (not glm), or\n",
             "  - A fit_result from fit(), or\n",
             "  - A fullfit_result from fullfit()\n",
             "
Received class: ", paste(class(x), collapse = ", "))
    }
    
    ## Check model class - support both LM and LMER
    is_lme4 <- inherits(model, c("lmerMod", "lmerModLmerTest", "merMod"))
    is_lm <- inherits(model, "lm") && !inherits(model, "glm")

    ## Internally work in inches
    if (!is.null(plot_width) && units != "in") {
        plot_width <- convert_units(plot_width, from = units, to = "in")
    }
    
    ## Get model data
    if (is_lme4) {
        ## For lme4 models, the model frame contains the complete cases
        model_data <- model@frame
    } else {
        ## For regular lm, use model$model which contains complete cases
        model_data <- model$model
    }
    
    ## Get the original data for reference (if provided)
    if(is.null(data)){
        warning("The `data` argument is not provided. Data will be extracted from model fit.")
        if (is_lme4) {
            data <- model@frame
        } else {
            data <- model$data
            if (is.null(data))
                data <- model$model
        }
        if (is.null(data))
            stop("The `data` argument should be provided either to lmforest or lm/lmer.")
    }
    
    ## A model fitted with model = FALSE retains no model frame. Group counts
    ## must still describe the observations used in fitting, so the supplied
    ## data is restricted to complete cases across the model variables rather
    ## than counted in full.
    if (is.null(model_data) && !is.null(data)) {
        model_data <- get_analysis_data(model, class(model)[1], data)
    }
    
    ## Convert to data.table if not already
    if(!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    if(!data.table::is.data.table(model_data)) {
        model_data <- data.table::as.data.table(model_data)
    }
    
    ## Extract terms based on model type
    if (is_lme4) {
        ## For lme4 models, only the fixed effects are used
        ## Get the fixed effects formula (removing random effects)
        fixed_formula <- lme4::nobars(stats::formula(model))
        
        ## Get variable names from the fixed formula (excluding outcome)
        var_names <- all.vars(fixed_formula)[-1]  # Remove outcome variable
        
        ## Get the model frame for determining variable types
        model_frame <- model@frame
        
        ## Create terms object similar to LM structure
        terms <- sapply(var_names, function(v) {
            if (v %in% names(model_frame)) {
                if (is.factor(model_frame[[v]])) "factor"
                else if (is.character(model_frame[[v]])) "character"
                else "numeric"
            } else if (v %in% names(data)) {
                if (is.factor(data[[v]])) "factor"
                else if (is.character(data[[v]])) "character"
                else "numeric"
            } else "numeric"
        }, USE.NAMES = TRUE)
        
    } else {
        ## For regular lm
        terms <- attr(model$terms, "dataClasses")[-1]
    }
    
    ## Filter out interaction terms (contain ":") 
    terms <- terms[!grepl(":", names(terms), fixed = TRUE)]
    
    ## Extract coefficients and confidence intervals
    if (is_lme4) {
        ## For lme4 models
        coef_summary <- summary(model)$coefficients
        
        ## Calculate confidence intervals using Wald method with specified conf_level
        coef_vals <- lme4::fixef(model)
        se_vals <- sqrt(diag(as.matrix(vcov(model))))
        t_crit <- stats::qt((1 + conf_level) / 2, df = summary(model)$devcomp$dims["n"] - 
                                                      summary(model)$devcomp$dims["p"])
        
        ## Use profile CI if available, else fall back to Wald.
        ##
        ## The value is returned from both branches rather than assigned with
        ## <<-. Where confint() raises an error the assignment in the first
        ## branch never runs, so a cascading assignment from the handler would
        ## find no binding in this frame and create one in the global
        ## environment.
        conf_int <- tryCatch({
            ci <- confint(model, method = "Wald", level = conf_level)
            ## Remove random effect rows if present
            fixed_names <- names(coef_vals)
            if (any(rownames(ci) %in% fixed_names)) {
                ci <- ci[fixed_names, , drop = FALSE]
            }
            ci
        }, error = function(e) {
            ci_pct_low <- paste0(round((1 - conf_level) / 2 * 100, 1), " %")
            ci_pct_high <- paste0(round((1 + conf_level) / 2 * 100, 1), " %")
            ci <- cbind(
                coef_vals - t_crit * se_vals,
                coef_vals + t_crit * se_vals
            )
            colnames(ci) <- c(ci_pct_low, ci_pct_high)
            rownames(ci) <- names(coef_vals)
            ci
        })
        
        ## Extract t values and p values
        ## Note: lme4 may or may not provide \emph{p}-values depending on the package
        if ("Pr(>|t|)" %in% colnames(coef_summary)) {
            t_values <- coef_summary[, "t value"]
            p_values <- coef_summary[, "Pr(>|t|)"]
        } else {
            ## Calculate \emph{p}-values using normal approximation if not provided
            t_values <- coef_summary[, "t value"]
            ## Approximate df for \emph{p}-value calculation
            df_resid <- summary(model)$devcomp$dims["n"] - 
                                     summary(model)$devcomp$dims["p"]
            p_values <- 2 * stats::pt(abs(t_values), df = df_resid, lower.tail = FALSE)
        }
        
    } else {
        ## For regular lm
        coef_summary <- summary(model)$coefficients
        conf_int <- get_cached_confint(model, conf_level)
        t_values <- coef_summary[, "t value"]
        p_values <- coef_summary[, "Pr(>|t|)"]
    }
    
    ## Remove intercept if present
    if("(Intercept)" %in% rownames(coef_summary)) {
        intercept_idx <- which(rownames(coef_summary) == "(Intercept)")
        coef_summary <- coef_summary[-intercept_idx, , drop = FALSE]
        conf_int <- conf_int[-intercept_idx, , drop = FALSE]
        t_values <- t_values[-intercept_idx]
        p_values <- p_values[-intercept_idx]
    }
    
    ## Create coefficient data table
    coef <- data.table::data.table(
                            term = rownames(coef_summary),
                            estimate = coef_summary[, "Estimate"],
                            std_error = coef_summary[, "Std. Error"],
                            statistic = t_values,
                            p_value = p_values,
                            conf_low = conf_int[, 1],
                            conf_high = conf_int[, 2]
                        )
    
    ## Get model statistics
    if (is_lme4) {
        ## For lme4 models
        model_summary <- summary(model)
        
        gmodel <- list(
            nobs = nrow(model@frame),
            ## lme4 doesn't have traditional R-squared; use conditional/marginal if available
            r_squared = NA,
            adj_r_squared = NA,
            f_statistic = NA,
            f_df1 = NA,
            f_df2 = NA,
            AIC = stats::AIC(model),
            ## Store info about random effects structure
            n_groups = sapply(model_summary$ngrps, identity)
        )
        
        ## Try to get pseudo R-squared using MuMIn if available
        tryCatch({
            if (requireNamespace("MuMIn", quietly = TRUE)) {
                r2_vals <- MuMIn::r.squaredGLMM(model)
                gmodel$r_squared <- r2_vals[1, "R2m"]  # Marginal R^2
                gmodel$conditional_r2 <- r2_vals[1, "R2c"]  # Conditional R^2
            }
        }, error = function(e) {
            ## Keep as NA
        })
        
        ## F-test not directly available for lmer
        gmodel$f_pvalue <- NA
        
    } else {
        ## For regular lm
        model_summary <- summary(model)
        gmodel <- list(
            nobs = nobs(model),
            r_squared = model_summary$r.squared,
            adj_r_squared = model_summary$adj.r.squared,
            f_statistic = model_summary$fstatistic[1],
            f_df1 = model_summary$fstatistic[2],
            f_df2 = model_summary$fstatistic[3],
            AIC = stats::AIC(model)
        )
        
        ## Calculate F-test \emph{p}-value
        gmodel$f_pvalue <- pf(gmodel$f_statistic, gmodel$f_df1, gmodel$f_df2, 
                              lower.tail = FALSE)
    }
    
    ## Calculate total observations and percentage analyzed
    total_obs <- nrow(data)
    
    ## Rendered by the formatter the print methods use, so that a figure and
    ## the table it accompanies disclose in identical terms. Linear models
    ## carry no event count, so only the observations line appears.
    gmodel$counts_block <- format_analysis_counts(
        list(n_supplied = total_obs, n_analyzed = gmodel$nobs),
        label = "Observations analyzed", marks = marks)
    aic_val <- format(round(gmodel$AIC, 2), big.mark = marks$big.mark, decimal.mark = marks$decimal.mark, scientific = FALSE, nsmall = 2)
    gmodel$AIC_formatted <- trimws(aic_val)
    
    ## Extract xlevels (factor levels) based on model type
    if (is_lme4) {
        ## For lme4 models, extract from the model frame
        frame_data <- model@frame
        xlevels <- list()
        
        for (var_name in names(terms)) {
            if (terms[var_name] %in% c("factor", "character")) {
                if (var_name %in% names(frame_data) && is.factor(frame_data[[var_name]])) {
                    xlevels[[var_name]] <- levels(frame_data[[var_name]])
                } else if (var_name %in% names(model_data) && is.factor(model_data[[var_name]])) {
                    xlevels[[var_name]] <- levels(model_data[[var_name]])
                }
            }
        }
        
        if (length(xlevels) == 0) xlevels <- NULL
        
    } else {
        ## For regular lm
        xlevels <- model$xlevels
    }

    ## Extract statistics for every variable, preserving order
    all_terms <- lapply(seq_along(terms), function(i){
        var <- names(terms)[i]
        
        if (terms[i] %in% c("factor", "character")) {
            ## Get the factor levels from xlevels (proper order)
            if(!is.null(xlevels) && var %in% names(xlevels)) {
                factor_levels <- xlevels[[var]]
                
                ## Create data table with proper levels from model data
                level_counts <- model_data[!is.na(get(var)), .N, by = var]
                data.table::setnames(level_counts, c("level", "Freq"))
                
                ## Ensure all levels are present
                all_levels_dt <- data.table::data.table(
                                                 level = factor_levels,
                                                 Freq = 0
                                             )
                
                ## Update with actual counts
                all_levels_dt[level_counts, Freq := i.Freq, on = "level"]
                
                all_levels_dt[, var := var]
                all_levels_dt[, pos := .I]
                all_levels_dt[, var_order := i]
                return(all_levels_dt[, .(var, level, Freq, pos, var_order)])
            } else {
                ## Fallback for variables not in xlevels
                adf <- model_data[!is.na(get(var)), .N, by = var]
                data.table::setnames(adf, old = c(var, "N"), new = c("level", "Freq"))
                adf[, var := var]
                adf[, pos := .I]
                adf[, var_order := i]
                return(adf[, .(var, level, Freq, pos, var_order)])
            }
        }
        else if (terms[i] == "numeric") {
            ## For numeric variables, return a single row
            return(data.table::data.table(
                                   var = var, 
                                   level = "-", 
                                   Freq = sum(!is.na(model_data[[var]])), 
                                   pos = 1, 
                                   var_order = i
                               ))
        }
        else {
            ## Other cases
            vars = grep(paste0("^", var, "*."), coef$term, value=TRUE)
            return(data.table::data.table(
                                   var = vars, 
                                   level = "", 
                                   Freq = nrow(model_data),  # Use model_data row count
                                   pos = seq_along(vars), 
                                   var_order = i
                               ))
        }
    })
    
    all_terms_df <- data.table::rbindlist(all_terms)
    data.table::setnames(all_terms_df, c("var", "level", "N", "pos", "var_order"))

    ## Determine interaction terms via ":" strings
    interaction_terms <- coef$term[grepl(":", coef$term, fixed = TRUE)]
    
    if (length(interaction_terms) > 0) {
        ## Get the next var_order value
        next_var_order <- max(all_terms_df$var_order) + 1L
        
        ## Process each interaction term
        interaction_rows <- lapply(seq_along(interaction_terms), function(idx) {
            int_term <- interaction_terms[idx]
            
            ## Parse the interaction term to create a readable label
            int_parts <- strsplit(int_term, ":", fixed = TRUE)[[1]]
            
            ## Build readable variable name from parts
            display_parts <- sapply(int_parts, function(part) {
                ## Try to match against known factor variables to extract var and level
                matched_result <- NULL
                for (var_name in names(terms)) {
                    if (startsWith(part, var_name)) {
                        level_str <- sub(paste0("^", var_name), "", part)
                        if (nchar(level_str) > 0) {
                            ## Apply label if available
                            var_label <- if (!is.null(labels) && var_name %in% names(labels)) {
                                             labels[[var_name]]
                                         } else {
                                             var_name
                                         }
                            matched_result <- paste0(var_label, " (", level_str, ")")
                            break
                        }
                    }
                }
                ## If no match found (continuous variable in interaction or exact match)
                if (is.null(matched_result)) {
                    var_label <- if (!is.null(labels) && part %in% names(labels)) {
                                     labels[[part]]
                                 } else {
                                     part
                                 }
                    matched_result <- var_label
                }
                matched_result
            })
            
            display_name <- paste(display_parts, collapse = " \u00d7 ")
            
            ## Calculate N for this interaction (observations where both conditions are met)
            int_n <- tryCatch({
                n_count <- NA_integer_
                
                ## Try to extract the subset count from the model data
                ## For each part of the interaction, build a condition
                conditions <- list()
                for (j in seq_along(int_parts)) {
                    part <- int_parts[j]
                    for (var_name in names(terms)) {
                        if (startsWith(part, var_name) && terms[var_name] %in% c("factor", "character")) {
                            level_str <- sub(paste0("^", var_name), "", part)
                            if (nchar(level_str) > 0 && var_name %in% names(model_data)) {
                                conditions[[var_name]] <- level_str
                            }
                            break
                        }
                    }
                }
                
                ## If conditions found, count matching rows
                if (length(conditions) > 0) {
                    ## Build subset expression safely
                    subset_dt <- data.table::copy(model_data)
                    for (cond_var in names(conditions)) {
                        subset_dt <- subset_dt[get(cond_var) == conditions[[cond_var]]]
                    }
                    n_count <- nrow(subset_dt)
                }
                
                n_count
            }, error = function(e) NA_integer_)
            
            data.table::data.table(
                            var = display_name,
                            level = "-",
                            N = int_n,
                            pos = 1L,
                            var_order = next_var_order + idx - 1L,
                            .int_term = int_term  # Store raw term for later inds assignment
                        )
        })
        
        interaction_df <- data.table::rbindlist(interaction_rows)
        
        ## Bind interaction rows to main terms
        all_terms_df <- data.table::rbindlist(list(all_terms_df, interaction_df), 
                                              use.names = TRUE, fill = TRUE)
    }

    ## Apply condensing and indenting if requested
    if (condense_table || indent_groups) {
        if (condense_table) {
            indent_groups <- TRUE
        }
        
        ## For interaction rows, use the stored raw term; for others use var+level pattern
        if (".int_term" %in% names(all_terms_df)) {
            all_terms_df[, inds := data.table::fifelse(
                                                   !is.na(.int_term), 
                                                   .int_term,
                                                   data.table::fifelse(level == "-", var, paste0(var, level))
                                               )]
        } else {
            all_terms_df[, inds := data.table::fifelse(level == "-", var, paste0(var, level))]
        }
        orig_inds_map <- data.table::copy(all_terms_df[, .(var, level, inds, N)])
        
        processed_rows <- list()
        row_counter <- 1
        unique_vars <- unique(all_terms_df[, var])
        
        for (v in unique_vars) {
            var_rows <- all_terms_df[var == v]
            
            if (nrow(var_rows) == 1) {
                ## Single-level variable (continuous)
                processed_rows[[row_counter]] <- var_rows
                row_counter <- row_counter + 1
            } else {
                ## Multi-level variable (categorical)
                is_binary <- nrow(var_rows) == 2
                
                if (condense_table && is_binary) {
                    ## Detect reference row by checking if level is first in xlevels
                    ## (first level is always reference in R factor contrasts)
                    ref_level <- NULL
                    if (!is.null(xlevels) && v %in% names(xlevels)) {
                        ref_level <- xlevels[[v]][1]
                    }
                    
                    ## Find non-reference row
                    non_ref_idx <- NULL
                    if (!is.null(ref_level)) {
                        ref_idx <- which(var_rows$level == ref_level)
                        if (length(ref_idx) == 1) {
                            non_ref_idx <- setdiff(1:2, ref_idx)
                        }
                    }
                    
                    ## Fallback to estimate-based detection if available
                    if (is.null(non_ref_idx) && "estimate" %in% names(var_rows)) {
                        non_ref_idx <- find_non_reference_row(var_rows, "estimate")
                    }
                    
                    ## Final fallback: assume row 2 is non-reference
                    if (is.null(non_ref_idx) || length(non_ref_idx) != 1) {
                        non_ref_idx <- 2L
                        ref_idx <- 1L
                    } else {
                        ref_idx <- setdiff(1:2, non_ref_idx)
                    }
                    
                    non_ref_row <- var_rows[non_ref_idx]
                    ref_row <- var_rows[ref_idx]
                    
                    condensed_row <- data.table::copy(non_ref_row)
                    non_ref_category <- condensed_row$level
                    ref_category <- ref_row$level
                    
                    ## Look up label for smarter condensing detection
                    var_label <- if (!is.null(labels) && v %in% names(labels)) {
                                     labels[[v]]
                                 } else if (v %in% names(model_data) && 
                                            !is.null(attr(model_data[[v]], "label"))) {
                                     attr(model_data[[v]], "label")
                                 } else {
                                     v
                                 }
                    
                    ## Use greedy approach: condense if EITHER category is recognized
                    if (should_condense_binary(ref_category, non_ref_category, var_label)) {
                        condensed_row[, var := v]
                    } else {
                        condensed_row[, var := paste0(v, " (", non_ref_category, ")")]
                    }
                    condensed_row[, level := "-"]
                    processed_rows[[row_counter]] <- condensed_row
                    row_counter <- row_counter + 1
                    
                } else if (indent_groups) {
                    ## Add header for any multi-level variable when indenting
                    header_row <- data.table::data.table(
                                                  var = v,
                                                  level = "-",
                                                  N = NA_integer_,
                                                  pos = var_rows$pos[1],
                                                  var_order = var_rows$var_order[1],
                                                  inds = NA_character_
                                              )
                    processed_rows[[row_counter]] <- header_row
                    row_counter <- row_counter + 1
                    
                    ## Add indented levels
                    for (i in seq_len(nrow(var_rows))) {
                        group_row <- data.table::copy(var_rows[i])
                        group_row[, var := paste0("    ", level)]
                        group_row[, level := "-"]
                        processed_rows[[row_counter]] <- group_row
                        row_counter <- row_counter + 1
                    }
                } else {
                    ## Standard layout
                    for (i in seq_len(nrow(var_rows))) {
                        processed_rows[[row_counter]] <- var_rows[i]
                        row_counter <- row_counter + 1
                    }
                }
            }
        }
        
        all_terms_df <- data.table::rbindlist(processed_rows, fill = TRUE)
        
        for (i in seq_len(nrow(all_terms_df))) {
            current_var <- all_terms_df$var[i]
            
            if (is.na(all_terms_df$inds[i]) && !grepl("^    ", current_var) && !grepl("\\(", current_var)) {
                next
            }
            
            if (grepl("\\(", current_var)) {
                orig_var <- gsub(" \\(.*\\)", "", current_var)
                orig_level <- gsub(".*\\((.*)\\)", "\\1", current_var)
                
                matching <- orig_inds_map[var == orig_var & level == orig_level]
                if (nrow(matching) > 0) {
                    all_terms_df[i, `:=`(inds = matching$inds[1],
                                         N = matching$N[1])]
                }
            }
            else if (grepl("^    ", current_var)) {
                clean_level <- gsub("^    ", "", current_var)
                
                parent_var <- NA_character_
                for (j in (i-1):1) {
                    if (!grepl("^    ", all_terms_df$var[j]) && all_terms_df$var[j] != "") {
                        parent_var <- gsub(" \\(.*\\)", "", all_terms_df$var[j])
                        break
                    }
                }
                
                if (!is.na(parent_var)) {
                    matching <- orig_inds_map[var == parent_var & level == clean_level]
                    if (nrow(matching) > 0) {
                        all_terms_df[i, `:=`(inds = matching$inds[1],
                                             N = matching$N[1])]
                    }
                }
            }
        }
    } else {
        ## For non-condensed/non-indented display, set inds for coefficient matching
        ## For interaction rows, use the stored raw term; for others use var+level pattern
        if (".int_term" %in% names(all_terms_df)) {
            all_terms_df[, inds := data.table::fifelse(
                                                   !is.na(.int_term), 
                                                   .int_term,
                                                   data.table::fifelse(level == "-", var, paste0(var, level))
                                               )]
        } else {
            all_terms_df[, inds := data.table::fifelse(level == "-", var, paste0(var, level))]
        }
    }
    
    ## Clean up temporary column if it exists
    if (".int_term" %in% names(all_terms_df)) {
        all_terms_df[, .int_term := NULL]
    }
    
    ## Process coefficients
    coef[, term := gsub(term, pattern = "`", replacement = "")]
    coef[, inds := term]
    
    ## Merge data
    to_show <- merge(all_terms_df, coef, by.x = "inds", by.y = "inds", all.x = TRUE, sort = FALSE)
    
    ## Sort by variable order first, then position within variable
    data.table::setorder(to_show, var_order, pos)
    
    ## Add variable-based shading indicator (zebra stripes)
    if (zebra_stripes) {
        to_show[, shade_group := var_order %% 2]
        shade_colors <- c("#FFFFFF", "#EEEEEE")
    } else {
        to_show[, shade_group := 0]
        shade_colors <- c("#FFFFFF", "#FFFFFF")
    }
    
    ## Select columns
    to_show <- to_show[, .(var, level, N, p_value, estimate, conf_low, conf_high, pos, var_order, shade_group)]

    ## Format the values
    to_show_exp_clean <- data.table::copy(to_show)
    
    ## Create formatted columns for display
    to_show_exp_clean[, effect := estimate]
    to_show_exp_clean[, effect_formatted := data.table::fifelse(is.na(N) & is.na(estimate),
                                                                "",
                                                                data.table::fifelse(is.na(estimate), 
                                                                                    ref_label,
                                                                                    format_number(estimate, digits, marks)))]
    to_show_exp_clean[, conf_low_formatted := data.table::fifelse(is.na(conf_low), 
                                                                  NA_character_,
                                                                  format_number(conf_low, digits, marks))]
    to_show_exp_clean[, conf_high_formatted := data.table::fifelse(is.na(conf_high), 
                                                                   NA_character_,
                                                                   format_number(conf_high, digits, marks))]
    
    ## Format CI percentage for display in headers
    ci_pct <- round(conf_level * 100)

    ## Format \emph{p}-values using p_digits parameter
    p_threshold <- 10^(-p_digits)
    p_threshold_str <- if (!is.null(marks)) {
                           paste0("< 0", marks$decimal.mark, strrep("0", p_digits - 1), "1")
                       } else {
                           paste0("< ", format(p_threshold, scientific = FALSE))
                       }
    
    to_show_exp_clean[, p_formatted := data.table::fifelse(is.na(p_value), 
                                                           NA_character_,
                                                           data.table::fifelse(p_value < p_threshold, 
                                                                               p_threshold_str,
                                                                               format_number(p_value, p_digits, marks)))]
    
    ## Determine if ANY coefficient or CI bound is negative - if so, use comma notation throughout
    ## This ensures consistent formatting across all rows
    use_comma_notation <- any(to_show_exp_clean$conf_low < 0 | to_show_exp_clean$conf_high < 0, na.rm = TRUE)
    ci_separator <- forest_ci_separator(use_comma_notation, marks)
    
    ## Create the combined effect string with expression for italic p
    to_show_exp_clean[, effect_string_expr := data.table::fifelse(
                                                              is.na(N) & is.na(estimate),
                                                              "''",
                                                              data.table::fcase(
                                                                  is.na(estimate), paste0("'", ref_label, "'"),
                                                                  
                                                                  p_value < p_threshold,
                                                                  paste0("'", effect_formatted, " (", conf_low_formatted, ci_separator, 
                                                                         conf_high_formatted, "); '*~italic(p)~'", p_threshold_str, "'"),
                                                                  
                                                                  default = paste0("'", effect_formatted, " (", conf_low_formatted, ci_separator, 
                                                                                   conf_high_formatted, "); '*~italic(p)~'= ", p_formatted, "'")
                                                              )
                                                          )]
    
    ## Format N with thousands separator
    to_show_exp_clean[, n_formatted := data.table::fifelse(is.na(N), "",
        format_count(N, marks, na = ""))]
    
    ## Clean up variable names for display
    to_show_exp_clean[, var_display := as.character(var)]
    
    if (indent_groups || condense_table) {
        to_show_exp_clean[, var_display := var]
        
        for (v in unique(to_show_exp_clean$var)) {
            if (v != "" && !grepl("^    ", v)) {
                ## Skip interaction terms (already formatted) - identified by " × " in the name
                if (grepl(" \u00d7 ", v)) next
                
                clean_v <- gsub(" \\(.*\\)", "", v)

                label <- if (!is.null(labels) && clean_v %in% names(labels)) {
                             labels[clean_v]
                         } else if (clean_v %in% names(data) && !is.null(attr(data[[clean_v]], "label"))) {
                             attr(data[[clean_v]], "label")
                         } else {
                             NULL
                         }
                
                if (!is.null(label)) {
                    if (grepl("\\(", v)) {
                        category <- gsub(".*\\((.*)\\)", "\\1", v)
                        if (is_affirmative_category(category, label)) {
                            to_show_exp_clean[var == v, var_display := label]
                        } else {
                            to_show_exp_clean[var == v, var_display := paste0(label, " (", category, ")")]
                        }
                    } else {
                        to_show_exp_clean[var == v, var_display := label]
                    }
                }
            }
        }
        
        to_show_exp_clean[, level := ""]
    } else {
        for(v in unique(to_show_exp_clean$var)) {
            if(v %in% to_show_exp_clean$var) {
                ## Skip interaction terms (already formatted) - identified by " × " in the name
                if (grepl(" \u00d7 ", v)) next
                
                if(!is.null(labels) && v %in% names(labels)) {
                    to_show_exp_clean[var == v, var_display := labels[v]]
                }
                else if(v %in% names(data) && !is.null(attr(data[[v]], "label"))) {
                    to_show_exp_clean[var == v, var_display := attr(data[[v]], "label")]
                }
            }
        }
    }

    if (!indent_groups) {
        to_show_exp_clean[duplicated(var), var_display := ""]
    }
    
    ## Handle missing estimates for plotting
    to_show_exp_clean[is.na(estimate), estimate := 0]
    
    ## Reorder (flip), but maintain the variable grouping
    ## Retain the assembled table for return as an attribute. The copy is taken
    ## before the row order is reversed for plotting, so that the returned table
    ## follows the order of the model terms rather than the drawing order.
    table_data <- data.table::copy(to_show_exp_clean)
    
    to_show_exp_clean <- to_show_exp_clean[order(rev(seq_len(nrow(to_show_exp_clean))))]
    to_show_exp_clean[, x_pos := .I]
    
    ## Add shade_group based on var_order for alternating by variable
    to_show_exp_clean[, shade_group := var_order %% 2]
    
    ## Number of rows for later adjustments
    n_rows <- nrow(to_show_exp_clean)
    
    ## Calculate plot ranges
    rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
    breaks <- pretty(rangeb, n = 7)
    breaks <- breaks[breaks >= rangeb[1] & breaks <= rangeb[2]]
    
    ## Check for one-sided case
    is_one_sided <- (min(rangeb) > 0) || (max(rangeb) < 0)
    
    ## Ensure 0 is included for coefficients
    if(!0 %in% breaks) {
        breaks <- sort(unique(c(breaks, 0)))
    }
    
    ## Extend range if needed
    if(min(rangeb) > 0) {
        rangeb[1] <- -0.1 * abs(max(rangeb))
    } else if(max(rangeb) < 0) {
        rangeb[2] <- 0.1 * abs(min(rangeb))
    }
    
    reference_value <- 0

    ## Calculate plot ranges
    rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
    breaks <- pretty(rangeb, n = 7)
    breaks <- breaks[breaks >= rangeb[1] & breaks <= rangeb[2]]

    ## Check for one-sided case
    is_one_sided <- (min(rangeb) > 0) || (max(rangeb) < 0)

    ## Ensure 0 is included for coefficients
    if(!0 %in% breaks) {
        breaks <- sort(unique(c(breaks, 0)))
    }

    ## Extend range if needed
    if(min(rangeb) > 0) {
        rangeb[1] <- -0.1 * abs(max(rangeb))
    } else if(max(rangeb) < 0) {
        rangeb[2] <- 0.1 * abs(min(rangeb))
    }

    reference_value <- 0

    ## Calculate layout using helper function
    layout <- calculate_forest_layout(
        to_show_exp_clean = to_show_exp_clean,
        show_n = show_n,
        show_events = FALSE,  # linear models do not have events
        indent_groups = indent_groups,
        condense_table = condense_table,
        effect_label = effect_label,
        ref_label = ref_label,
        font_size = font_size,
        table_width = data.table::fifelse(is.null(table_width), 0.6, table_width),
        rangeb = rangeb,
        center_padding = center_padding
    )

    ## Set up the extended range for plotting
    rangeplot <- c(layout$rangeplot_start, rangeb[2] + diff(rangeb) * 0.05)

    ## Extract positions
    y_variable <- layout$positions$var
    if (!(indent_groups || condense_table)) {
        y_level <- layout$positions$level
    }
    if (show_n) {
        y_n <- layout$positions$n
    }
    y_coef <- layout$positions$effect

    ## Use the effect abbreviation from layout
    effect_abbrev <- layout$effect_abbrev

    ## Calculate recommended dimensions
    rec_height <- max(5, min(20, 3 + nrow(to_show_exp_clean) * 0.25))

    if(!is.null(plot_width)) {
        rec_width <- plot_width
        if(!is.null(plot_height)) {
            rec_height <- plot_height
        }
    } else {
        ## Use the calculated total width
        rec_width <- layout$total_width + 1.0  # Add margins
        
        ## Apply reasonable bounds based on layout
        if (condense_table || indent_groups) {
            if (!show_n) {
                rec_width <- max(7.5, min(11, rec_width))
            } else {
                rec_width <- max(9, min(14, rec_width))
            }
        } else {
            if (!show_n) {
                rec_width <- max(12, min(15, rec_width))
            } else {
                rec_width <- max(13, min(16, rec_width))
            }
        }
    }

    ## Font sizes
    annot_font <- font_size * annot_size
    header_font <- font_size * header_size

    ## Custom ticks data
    ticks_df <- data.frame(
        x = -0.5,
        xend = -0.7,
        y = breaks,
        yend = breaks
    )
    
    ## Create the plot
    p <- ggplot2::ggplot(to_show_exp_clean, ggplot2::aes(x_pos, estimate)) +
        
        ## Shading rectangles
        ggplot2::geom_rect(ggplot2::aes(xmin = x_pos - .5, xmax = x_pos + .5,
                                        ymin = rangeplot[1], ymax = rangeplot[2],
                                        fill = ordered(shade_group))) +
        ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.10, 0.05))) +
        ggplot2::scale_size_area(max_size = 6, guide = "none") +
        ggplot2::scale_fill_manual(values = shade_colors, guide = "none") +
        
        ## Forest plot elements
        ggplot2::geom_point(ggplot2::aes(size = N), pch = 22, color = "#000000", fill = color, na.rm = TRUE) +
        ggplot2::geom_errorbar(ggplot2::aes(ymin = conf_low, ymax = conf_high), width = 0.15) +
        
        ## Y-axis for forest plot
        ggplot2::annotate(geom = "segment",
                          x = -0.5, xend = -0.5,
                          y = rangeb[1],
                          yend = rangeb[2],
                          color = "#000000", linewidth = 1) +
        
        ## Reference line at 0
        ggplot2::annotate(geom = "segment", 
                          x = -0.5, xend = max(to_show_exp_clean$x_pos) + 0.5, 
                          y = reference_value, yend = reference_value, 
                          linetype = "longdash") +
        
        ## Ticks
        ggplot2::geom_segment(data = ticks_df,
                              ggplot2::aes(x = x, xend = xend, y = y, yend = yend),
                              inherit.aes = FALSE,
                              color = "#000000",
                              linewidth = 1) +
        ggplot2::geom_text(data = ticks_df,
                           ggplot2::aes(x = xend - 0.05, y = y, label = sprintf("%g", y)),
                           inherit.aes = FALSE,
                           hjust = 0.5,
                           vjust = 1.3,
                           size = annot_font * 1.5) +
        ## Set coordinate system
        ggplot2::coord_flip(ylim = rangeplot) +
        ggplot2::ggtitle(title) +
        ggplot2::scale_y_continuous(name = effect_label,
                                    labels = sprintf("%g", breaks),
                                    expand = c(0.02, 0.02),
                                    breaks = breaks) +
        ggplot2::theme_light(base_family = detect_plot_font()) +
        ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 0, b = 0, l = 0),
                       panel.grid.minor.y = ggplot2::element_blank(),
                       panel.grid.minor.x = ggplot2::element_blank(),
                       panel.grid.major.y = ggplot2::element_blank(),
                       panel.grid.major.x = ggplot2::element_blank(),
                       legend.position = "none",
                       panel.border = ggplot2::element_blank(),
                       axis.title.y = ggplot2::element_blank(),
                       axis.text.y = ggplot2::element_blank(),
                       axis.ticks.y = ggplot2::element_blank(),
                       axis.title.x = ggplot2::element_blank(),
                       axis.ticks.x = ggplot2::element_blank(),
                       axis.text.x = ggplot2::element_blank(),
                       plot.title = ggplot2::element_text(size = font_size * title_size, face = "bold", hjust = 0.5)) +
        ggplot2::xlab("") +
        
        ## Variable column
        ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_variable,
                          label = "Variable", fontface = "bold", hjust = 0,
                          size = header_font) +
        
        {if (indent_groups || condense_table) {
             fontfaces <- if (bold_variables) {
                 data.table::fifelse(grepl("^    ", to_show_exp_clean$var_display), "plain", "bold")
             } else {
                 rep("plain", nrow(to_show_exp_clean))
             }
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_variable,
                               label = to_show_exp_clean$var_display, 
                               fontface = fontfaces, 
                               hjust = 0,
                               size = annot_font)
         } else {
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_variable,
                               label = to_show_exp_clean$var_display, 
                               fontface = if (bold_variables) "bold" else "plain", 
                               hjust = 0,
                               size = annot_font)
         }} +
        
        ## Group/level column
        {if (!(indent_groups || condense_table)) {
             list(
                 ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_level,
                                   label = "Group", fontface = "bold", hjust = 0,
                                   size = header_font),
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_level,
                                   label = to_show_exp_clean$level, hjust = 0,
                                   size = annot_font)
             )
         }} +
        
        ## N column (conditional)
        {if (show_n) {
             list(
                 ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_n,
                                   label = "n", fontface = "bold.italic", hjust = 0.5,
                                   size = header_font),
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_n,
                                   label = to_show_exp_clean$n_formatted, hjust = 0.5,
                                   size = annot_font)
             )
         }} +
        
        ## Coefficient column - use dynamic CI percentage
        ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.4, y = y_coef,
                          label = paste0("bold('", effect_abbrev, " (", ci_pct, "% CI); '*bolditalic(p)*'-value')"),
                          hjust = 0, size = header_font, parse = TRUE) +
        
        ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_coef,
                          label = to_show_exp_clean$effect_string_expr, hjust = 0,
                          size = annot_font, parse = TRUE) +
        
        ## X-axis label
        ggplot2::annotate(geom = "text", x = -1.5, y = reference_value,
                          label = effect_label, fontface = "bold",
                          hjust = 0.5, vjust = 2, size = annot_font * 1.5) +
        
        ## Model statistics at bottom (conditional)
        {if (qc_footer) {
             r2_str <- format_number(gmodel$r_squared, 3, marks)
             adj_r2_str <- format_number(gmodel$adj_r_squared, 3, marks)
             ## Build footer - F-statistic may be NA for mixed models
             if (!is.na(gmodel$f_statistic) && !is.na(gmodel$f_pvalue)) {
                 f_stat_str <- format_number(gmodel$f_statistic, 2, marks)
                 f_p_str <- if (gmodel$f_pvalue < p_threshold) p_threshold_str else paste0("= ", format_number(gmodel$f_pvalue, p_digits, marks))
                 f_line <- paste0("\nF-statistic: ", f_stat_str,
                                  " (df1 = ", gmodel$f_df1, ", df2 = ", gmodel$f_df2,
                                  "); p ", f_p_str)
             } else {
                 f_line <- ""
             }
             ggplot2::annotate(geom = "text", x = 0.5, y = y_variable,
                               label = paste0(gmodel$counts_block,
                                              "\nR\u00b2: ", r2_str,
                                              " (Adjusted: ", adj_r2_str, ")",
                                              f_line,
                                              "\nAIC: ", gmodel$AIC_formatted),
                               size = annot_font * 0.65, hjust = 0, vjust = 1.2, fontface = "italic")
        }}

    ## Convert units back for output if needed
    if (units != "in") {
        rec_width <- convert_units(rec_width, from = "in", to = units)
        rec_height <- convert_units(rec_height, from = "in", to = units)
    }

    ## Provide dimension recommendations
    if(is.null(plot_width) || is.null(plot_height)) {
        message(sprintf("Recommended plot dimensions: width = %.1f %s, height = %.1f %s",
                        rec_width, units, rec_height, units))
    }
    
    ## Add recommended dimensions as an attribute
    attr(p, "rec_dims") <- list(width = rec_width, height = rec_height, units = units)

    ## Add the assembled table as an attribute
    attr(p, "table_data") <- table_data

    ## Return the plot
    return(p)
}

#' Test Suite for Forest Plot Functions
#' 
#' Comprehensive tests covering glmforest, lmforest, coxforest, autoforest,
#' uniforest, and multiforest. Tests output structure, parameters, visual
#' elements, mixed-effects models, and interaction terms.
#' 
#' @details Run with testthat::test_file("tests/testthat/test_forest.R")

library(data.table)
library(ggplot2)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create complete-case subset for mixed-effects tests
clintrial_complete <- na.omit(clintrial)

## Create binary response for logistic tests
clintrial$response <- as.integer(clintrial$os_status == 1 & clintrial$os_months < 24)
clintrial_complete$response <- as.integer(clintrial_complete$os_status == 1 & 
                                          clintrial_complete$os_months < 24)

## ============================================================================
## Helper Functions
## ============================================================================

## Helper to check forest plot output structure
expect_forest_plot <- function(p, has_dims = TRUE) {
    expect_true(inherits(p, "ggplot"), 
                info = "Forest plot should be a ggplot object")
    
    if (has_dims) {
        dims <- attr(p, "rec_dims")
        expect_true(!is.null(dims), 
                    info = "Forest plot should have rec_dims attribute")
        expect_true("width" %in% names(dims), 
                    info = "rec_dims should have width")
        expect_true("height" %in% names(dims), 
                    info = "rec_dims should have height")
        expect_true(dims$width > 0, info = "Width should be positive")
        expect_true(dims$height > 0, info = "Height should be positive")
    }
}

## Helper to check plot layers
expect_has_layers <- function(p, min_layers = 5) {
    n_layers <- length(p$layers)
    expect_true(n_layers >= min_layers,
                info = paste("Expected at least", min_layers, "layers, got", n_layers))
}

## ============================================================================
## SECTION 1: glmforest() Basic Tests
## ============================================================================

test_that("glmforest works with basic GLM", {
    
    model <- glm(response ~ age + sex + smoking, 
                 data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial, 
                                    title = "Test GLM Forest"))
    
    expect_forest_plot(p)
    expect_has_layers(p)
})


test_that("glmforest works with fit_result input", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking"),
        model_type = "glm"
    )
    
    p <- suppressMessages(glmforest(result, title = "GLM from fit()"))
    
    expect_forest_plot(p)
})


test_that("glmforest respects digits parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p1 <- suppressMessages(glmforest(model, data = clintrial, digits = 1))
    p2 <- suppressMessages(glmforest(model, data = clintrial, digits = 4))
    
    expect_forest_plot(p1)
    expect_forest_plot(p2)
})


test_that("glmforest respects p_digits parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p1 <- suppressMessages(glmforest(model, data = clintrial, p_digits = 2))
    p2 <- suppressMessages(glmforest(model, data = clintrial, p_digits = 4))
    
    expect_forest_plot(p1)
    expect_forest_plot(p2)
})


test_that("glmforest respects conf_level parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    ## Test various confidence levels
    p_90 <- suppressMessages(glmforest(model, data = clintrial, conf_level = 0.90))
    p_95 <- suppressMessages(glmforest(model, data = clintrial, conf_level = 0.95))
    p_99 <- suppressMessages(glmforest(model, data = clintrial, conf_level = 0.99))
    
    expect_forest_plot(p_90)
    expect_forest_plot(p_95)
    expect_forest_plot(p_99)
})


test_that("glmforest respects show_n parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_with_n <- suppressMessages(glmforest(model, data = clintrial, show_n = TRUE))
    p_without_n <- suppressMessages(glmforest(model, data = clintrial, show_n = FALSE))
    
    expect_forest_plot(p_with_n)
    expect_forest_plot(p_without_n)
})


test_that("glmforest respects show_events parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_with_events <- suppressMessages(glmforest(model, data = clintrial, show_events = TRUE))
    p_without_events <- suppressMessages(glmforest(model, data = clintrial, show_events = FALSE))
    
    expect_forest_plot(p_with_events)
    expect_forest_plot(p_without_events)
})


test_that("glmforest respects indent_groups parameter", {
    
    model <- glm(response ~ age + stage, data = clintrial, family = binomial)
    
    p_flat <- suppressMessages(glmforest(model, data = clintrial, indent_groups = FALSE))
    p_indented <- suppressMessages(glmforest(model, data = clintrial, indent_groups = TRUE))
    
    expect_forest_plot(p_flat)
    expect_forest_plot(p_indented)
})


test_that("glmforest respects condense_table parameter", {
    
    model <- glm(response ~ age + sex + smoking, data = clintrial, family = binomial)
    
    p_full <- suppressMessages(glmforest(model, data = clintrial, condense_table = FALSE))
    p_condensed <- suppressMessages(glmforest(model, data = clintrial, condense_table = TRUE))
    
    expect_forest_plot(p_full)
    expect_forest_plot(p_condensed)
})


test_that("glmforest respects zebra_stripes parameter", {
    
    model <- glm(response ~ age + sex + smoking, data = clintrial, family = binomial)
    
    p_with_stripes <- suppressMessages(glmforest(model, data = clintrial, zebra_stripes = TRUE))
    p_without_stripes <- suppressMessages(glmforest(model, data = clintrial, zebra_stripes = FALSE))
    
    expect_forest_plot(p_with_stripes)
    expect_forest_plot(p_without_stripes)
})


test_that("glmforest respects qc_footer parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_with_footer <- suppressMessages(glmforest(model, data = clintrial, qc_footer = TRUE))
    p_without_footer <- suppressMessages(glmforest(model, data = clintrial, qc_footer = FALSE))
    
    expect_forest_plot(p_with_footer)
    expect_forest_plot(p_without_footer)
})


test_that("glmforest respects labels parameter", {
    
    model <- glm(response ~ age + sex + smoking, data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial, 
                                    labels = clintrial_labels))
    
    expect_forest_plot(p)
})


test_that("glmforest respects color parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_default <- suppressMessages(glmforest(model, data = clintrial))
    p_red <- suppressMessages(glmforest(model, data = clintrial, color = "#E74C3C"))
    p_blue <- suppressMessages(glmforest(model, data = clintrial, color = "blue"))
    
    expect_forest_plot(p_default)
    expect_forest_plot(p_red)
    expect_forest_plot(p_blue)
})


test_that("glmforest respects bold_variables parameter", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    p_plain <- suppressMessages(glmforest(model, data = clintrial, bold_variables = FALSE))
    p_bold <- suppressMessages(glmforest(model, data = clintrial, bold_variables = TRUE))
    
    expect_forest_plot(p_plain)
    expect_forest_plot(p_bold)
})


test_that("glmforest handles Poisson regression", {
    
    ## Create count outcome
    set.seed(2024)
    clintrial$event_count <- rpois(nrow(clintrial), lambda = 3)
    
    model <- glm(event_count ~ age + sex + smoking, 
                 data = clintrial, family = poisson)
    
    p <- suppressMessages(glmforest(model, data = clintrial, 
                                    title = "Poisson Model"))
    
    expect_forest_plot(p)
})


test_that("glmforest handles interaction terms", {
    
    model <- glm(response ~ age + sex + treatment + sex:treatment,
                 data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial,
                                    title = "GLM with Interactions",
                                    labels = clintrial_labels))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 2: lmforest() Basic Tests
## ============================================================================

test_that("lmforest works with basic LM", {
    
    model <- lm(los_days ~ age + sex + smoking, data = clintrial)
    
    p <- suppressMessages(lmforest(model, data = clintrial,
                                   title = "Test LM Forest"))
    
    expect_forest_plot(p)
    expect_has_layers(p)
})


test_that("lmforest works with fit_result input", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "smoking"),
        model_type = "lm"
    )
    
    p <- suppressMessages(lmforest(result, title = "LM from fit()"))
    
    expect_forest_plot(p)
})


test_that("lmforest respects key parameters", {
    
    model <- lm(los_days ~ age + sex + stage, data = clintrial)
    
    ## Test multiple parameters at once
    p <- suppressMessages(lmforest(model, data = clintrial,
                                   digits = 3,
                                   p_digits = 4,
                                   conf_level = 0.90,
                                   show_n = TRUE,
                                   indent_groups = TRUE,
                                   zebra_stripes = TRUE,
                                   bold_variables = TRUE,
                                   labels = clintrial_labels))
    
    expect_forest_plot(p)
})


test_that("lmforest handles interaction terms", {
    
    model <- lm(los_days ~ age + sex + treatment + age:treatment,
                data = clintrial)
    
    p <- suppressMessages(lmforest(model, data = clintrial,
                                   title = "LM with Interactions"))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 3: coxforest() Basic Tests
## ============================================================================

test_that("coxforest works with basic coxph", {
    
    model <- survival::coxph(survival::Surv(os_months, os_status) ~ age + sex + smoking,
                             data = clintrial)
    
    p <- suppressMessages(coxforest(model, data = clintrial,
                                    title = "Test Cox Forest"))
    
    expect_forest_plot(p)
    expect_has_layers(p)
})


test_that("coxforest works with fit_result input", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "smoking"),
        model_type = "coxph"
    )
    
    p <- suppressMessages(coxforest(result, title = "Cox from fit()"))
    
    expect_forest_plot(p)
})


test_that("coxforest respects key parameters", {
    
    model <- survival::coxph(survival::Surv(os_months, os_status) ~ age + sex + stage,
                             data = clintrial)
    
    p <- suppressMessages(coxforest(model, data = clintrial,
                                    digits = 3,
                                    p_digits = 4,
                                    conf_level = 0.90,
                                    show_n = TRUE,
                                    show_events = TRUE,
                                    indent_groups = TRUE,
                                    labels = clintrial_labels))
    
    expect_forest_plot(p)
})


test_that("coxforest handles interaction terms", {
    
    model <- survival::coxph(survival::Surv(os_months, os_status) ~ age + sex + treatment + 
                                 treatment:stage,
                             data = clintrial)
    
    p <- suppressMessages(coxforest(model, data = clintrial,
                                    title = "Cox with Interactions"))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 4: autoforest() Tests
## ============================================================================

test_that("autoforest correctly routes GLM", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p <- suppressMessages(autoforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("autoforest correctly routes LM", {
    
    model <- lm(los_days ~ age + sex, data = clintrial)
    
    p <- suppressMessages(autoforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("autoforest correctly routes coxph", {
    
    model <- survival::coxph(survival::Surv(os_months, os_status) ~ age + sex,
                             data = clintrial)
    
    p <- suppressMessages(autoforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("autoforest correctly routes fit_result model", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    ## Extract model from fit_result and pass to autoforest
    model <- attr(result, "model")
    p <- suppressMessages(autoforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("autoforest passes through parameters", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    p <- suppressMessages(autoforest(model, data = clintrial,
                                     title = "Custom Title",
                                     digits = 3,
                                     p_digits = 4,
                                     conf_level = 0.90,
                                     labels = clintrial_labels))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 5: uniforest() Tests
## ============================================================================

test_that("uniforest works with uniscreen result", {
    
    uni_result <- uniscreen(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "stage"),
        model_type = "glm",
        parallel = FALSE
    )
    
    p <- suppressMessages(uniforest(uni_result, 
                                    title = "Univariable Screening"))
    
    expect_forest_plot(p)
})


test_that("uniforest respects key parameters", {
    
    uni_result <- uniscreen(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking"),
        model_type = "glm",
        parallel = FALSE
    )
    
    p <- suppressMessages(uniforest(uni_result,
                                    digits = 3,
                                    p_digits = 4,
                                    conf_level = 0.90,
                                    show_n = TRUE,
                                    show_events = TRUE,
                                    indent_groups = TRUE,
                                    zebra_stripes = TRUE,
                                    labels = clintrial_labels))
    
    expect_forest_plot(p)
})


test_that("uniforest works with Cox uniscreen", {
    
    uni_result <- uniscreen(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "smoking"),
        model_type = "coxph",
        parallel = FALSE
    )
    
    p <- suppressMessages(uniforest(uni_result,
                                    title = "Univariable Cox"))
    
    expect_forest_plot(p)
})


test_that("uniforest works with LM uniscreen", {
    
    uni_result <- uniscreen(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "smoking"),
        model_type = "lm",
        parallel = FALSE
    )
    
    p <- suppressMessages(uniforest(uni_result,
                                    title = "Univariable Linear"))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 6: multiforest() Tests
## ============================================================================

test_that("multiforest works with multifit result", {
    
    multi_result <- multifit(
        data = clintrial,
        outcomes = c("response", "os_status"),
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )
    
    p <- suppressMessages(multiforest(multi_result,
                                      title = "Multi-Outcome Analysis"))
    
    expect_forest_plot(p)
})


test_that("multiforest respects key parameters", {
    
    multi_result <- multifit(
        data = clintrial,
        outcomes = c("response", "os_status"),
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )
    
    p <- suppressMessages(multiforest(multi_result,
                                      digits = 3,
                                      p_digits = 4,
                                      conf_level = 0.90,
                                      show_n = TRUE,
                                      show_events = TRUE,
                                      zebra_stripes = TRUE,
                                      labels = clintrial_labels))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 7: Mixed-Effects Model Tests
## ============================================================================

test_that("glmforest works with glmer models", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()
    
    gc()
    
    test_data <- clintrial_complete[1:200, ]
    
    model <- suppressWarnings(
        lme4::glmer(os_status ~ age + sex + smoking + (1|site),
                    data = test_data, family = binomial)
    )
    
    ## Suppress nobars deprecation warning from lme4
    p <- suppressMessages(suppressWarnings(
        glmforest(model, data = test_data, title = "Mixed-Effects Logistic")
    ))
    
    expect_forest_plot(p)
    
    gc()
})


test_that("lmforest works with lmer models", {
    
    skip_if_not_installed("lme4")
    skip_on_cran()
    
    gc()
    
    test_data <- clintrial_complete[1:200, ]
    
    model <- lme4::lmer(los_days ~ age + sex + smoking + (1|site),
                        data = test_data)
    
    p <- suppressMessages(lmforest(model, data = test_data,
                                   title = "Mixed-Effects Linear"))
    
    expect_forest_plot(p)
    
    gc()
})


test_that("coxforest works with coxme models", {
    
    skip_if_not_installed("coxme")
    skip_on_cran()
    
    gc()
    
    test_data <- clintrial_complete[1:200, ]
    
    model <- coxme::coxme(survival::Surv(os_months, os_status) ~ age + sex + (1|site),
                          data = test_data)
    
    p <- suppressMessages(coxforest(model, data = test_data,
                                    title = "Mixed-Effects Cox"))
    
    expect_forest_plot(p)
    
    gc()
})


## ============================================================================
## SECTION 8: Edge Cases and Error Handling
## ============================================================================

test_that("glmforest handles single predictor", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("glmforest handles many predictors", {
    
    model <- glm(response ~ age + sex + smoking + stage + treatment + 
                     diabetes + hypertension + surgery,
                 data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("glmforest handles all continuous predictors", {
    
    model <- glm(response ~ age + bmi + biomarker_x,
                 data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("glmforest handles all categorical predictors", {
    
    model <- glm(response ~ sex + smoking + stage + treatment,
                 data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial))
    
    expect_forest_plot(p)
})


test_that("forest functions handle extreme confidence levels", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    ## Very narrow CI
    p_80 <- suppressMessages(glmforest(model, data = clintrial, conf_level = 0.80))
    expect_forest_plot(p_80)
    
    ## Very wide CI
    p_99 <- suppressMessages(glmforest(model, data = clintrial, conf_level = 0.99))
    expect_forest_plot(p_99)
})


test_that("forest functions handle custom ref_label", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial,
                                    ref_label = "ref"))
    
    expect_forest_plot(p)
})


test_that("forest functions handle unit conversions", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_in <- suppressMessages(glmforest(model, data = clintrial, units = "in"))
    p_cm <- suppressMessages(glmforest(model, data = clintrial, units = "cm"))
    
    expect_forest_plot(p_in)
    expect_forest_plot(p_cm)
    
    ## Check that dimensions are appropriately scaled
    dims_in <- attr(p_in, "rec_dims")
    dims_cm <- attr(p_cm, "rec_dims")
    
    ## cm should be larger numbers (2.54x inches)
    expect_true(dims_cm$width > dims_in$width)
})


## ============================================================================
## SECTION 9: Plot Dimension Tests
## ============================================================================

test_that("forest plots provide reasonable dimension recommendations", {
    
    ## Small model
    model_small <- glm(response ~ age + sex, data = clintrial, family = binomial)
    p_small <- suppressMessages(glmforest(model_small, data = clintrial))
    dims_small <- attr(p_small, "rec_dims")
    
    ## Larger model
    model_large <- glm(response ~ age + sex + smoking + stage + treatment + 
                           diabetes + hypertension,
                       data = clintrial, family = binomial)
    p_large <- suppressMessages(glmforest(model_large, data = clintrial))
    dims_large <- attr(p_large, "rec_dims")
    
    ## Larger model should have greater height
    expect_true(dims_large$height > dims_small$height,
                info = "Larger model should have taller recommended height")
    
    ## Both should have reasonable widths (8-16 inches is typical)
    expect_true(dims_small$width >= 7 && dims_small$width <= 20)
    expect_true(dims_large$width >= 7 && dims_large$width <= 20)
})


test_that("forest plots respect explicit dimensions", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial,
                                    plot_width = 10,
                                    plot_height = 6))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 10: table_width Parameter Tests
## ============================================================================

test_that("forest plots respect table_width parameter", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    ## Default (0.6)
    p_default <- suppressMessages(glmforest(model, data = clintrial))
    
    ## Narrow table (0.4)
    p_narrow <- suppressMessages(glmforest(model, data = clintrial, table_width = 0.4))
    
    ## Wide table (0.7)
    p_wide <- suppressMessages(glmforest(model, data = clintrial, table_width = 0.7))
    
    expect_forest_plot(p_default)
    expect_forest_plot(p_narrow)
    expect_forest_plot(p_wide)
})


## ============================================================================
## SECTION 11: Integration with fit() workflow
## ============================================================================

test_that("complete fit() to forest workflow works for GLM", {
    
    result <- fit(
        data = clintrial,
        outcome = "response",
        predictors = c("age", "sex", "smoking", "stage"),
        model_type = "glm",
        labels = clintrial_labels
    )
    
    p <- suppressMessages(glmforest(result,
                                    title = "Predictors of Response",
                                    conf_level = 0.95,
                                    indent_groups = TRUE,
                                    qc_footer = TRUE))
    
    expect_forest_plot(p)
})


test_that("complete fit() to forest workflow works for Cox", {
    
    result <- fit(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "coxph",
        labels = clintrial_labels
    )
    
    p <- suppressMessages(coxforest(result,
                                    title = "Survival Analysis",
                                    conf_level = 0.95,
                                    indent_groups = TRUE))
    
    expect_forest_plot(p)
})


test_that("complete fit() to forest workflow works for LM", {
    
    result <- fit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "surgery", "diabetes"),
        model_type = "lm",
        labels = clintrial_labels
    )
    
    p <- suppressMessages(lmforest(result,
                                   title = "Length of Stay Predictors",
                                   conf_level = 0.95))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 12: Font Size Parameter Tests
## ============================================================================

test_that("forest plots respect font_size parameter", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_small <- suppressMessages(glmforest(model, data = clintrial, font_size = 0.8))
    p_normal <- suppressMessages(glmforest(model, data = clintrial, font_size = 1.0))
    p_large <- suppressMessages(glmforest(model, data = clintrial, font_size = 1.2))
    
    expect_forest_plot(p_small)
    expect_forest_plot(p_normal)
    expect_forest_plot(p_large)
})


test_that("forest plots respect annot_size and header_size", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p <- suppressMessages(glmforest(model, data = clintrial,
                                    annot_size = 4.0,
                                    header_size = 6.0,
                                    title_size = 24))
    
    expect_forest_plot(p)
})


## ============================================================================
## SECTION 13: exponentiate Parameter Tests
## ============================================================================

test_that("glmforest respects exponentiate = FALSE", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_exp <- suppressMessages(glmforest(model, data = clintrial, exponentiate = TRUE))
    p_raw <- suppressMessages(glmforest(model, data = clintrial, exponentiate = FALSE))
    
    expect_forest_plot(p_exp)
    expect_forest_plot(p_raw)
})


## ============================================================================
## SECTION 14: Consistency Tests
## ============================================================================

test_that("autoforest produces same result as direct function call", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    p_direct <- suppressMessages(glmforest(model, data = clintrial, 
                                           title = "Test",
                                           conf_level = 0.90))
    p_auto <- suppressMessages(autoforest(model, data = clintrial,
                                          title = "Test",
                                          conf_level = 0.90))
    
    ## Both should be valid forest plots
    expect_forest_plot(p_direct)
    expect_forest_plot(p_auto)
    
    ## Both should have similar dimensions
    dims_direct <- attr(p_direct, "rec_dims")
    dims_auto <- attr(p_auto, "rec_dims")
    
    expect_equal(dims_direct$width, dims_auto$width, tolerance = 0.1)
    expect_equal(dims_direct$height, dims_auto$height, tolerance = 0.1)
})

## Forest plots report the same sample sizes as the tables they accompany.
## Counts are read from the "table_data" attribute, which holds the values drawn
## in the plot in model term order.

forest_n <- function(p, variable) {
    tbl <- attr(p, "table_data")
    tbl[["N"]][tbl[["var"]] == variable]
}


test_that("coxforest counts only the observations used in fitting", {

    df <- fresh_clintrial()
    df$os_status[seq_len(100)] <- NA
    dt <- data.table::as.data.table(df)

    model <- survival::coxph(
        survival::Surv(os_months, os_status) ~ age + sex + race,
        data = dt
    )

    p <- suppressMessages(suppressWarnings(coxforest(model, data = dt)))

    ## Group sizes partition the fitted sample size
    for (pred in c("sex", "race")) {
        expect_equal(sum(forest_n(p, pred)), model$n)
    }

    ## Continuous rows report the fitted sample size, not every supplied row
    expect_equal(unique(forest_n(p, "age")), model$n)
    expect_true(all(attr(p, "table_data")[["N"]] <= nrow(dt)))
})


test_that("glmforest counts correctly when no model frame was retained", {

    df <- fresh_clintrial()
    df$readmission_30d[seq_len(100)] <- NA
    dt <- data.table::as.data.table(df)

    ## model = FALSE leaves the fit without a model frame, so the counting frame
    ## must be reconstructed from the supplied data.
    model <- stats::glm(readmission_30d ~ age + sex + race,
                        data = dt, family = stats::binomial,
                        model = FALSE, y = TRUE)

    p <- suppressMessages(suppressWarnings(glmforest(model, data = dt)))

    n_model <- stats::nobs(model)

    for (pred in c("sex", "race")) {
        expect_equal(sum(forest_n(p, pred)), n_model)
    }
    expect_equal(unique(forest_n(p, "age")), n_model)
})


test_that("forest plots expose the assembled table", {

    df <- fresh_clintrial()
    dt <- data.table::as.data.table(df)

    model <- stats::glm(readmission_30d ~ age + sex,
                        data = dt, family = stats::binomial)

    p <- suppressMessages(suppressWarnings(glmforest(model, data = dt)))
    tbl <- attr(p, "table_data")

    expect_s3_class(tbl, "data.table")
    expect_true(all(c("var", "level", "N", "estimate") %in% names(tbl)))
    expect_gt(nrow(tbl), 0L)
})


test_that("forest footers state the denominator alongside the percentage", {

    df <- fresh_clintrial()
    df$readmission_30d[seq_len(100)] <- NA
    dt <- data.table::as.data.table(df)

    model <- stats::glm(readmission_30d ~ age + sex,
                        data = dt, family = stats::binomial)

    p <- suppressMessages(suppressWarnings(glmforest(model, data = dt)))

    ## The footer text is carried on an annotation layer
    labels <- unlist(lapply(p$layers, function(lyr) {
        as.character(lyr$aes_params$label)
    }))
    footer <- labels[grepl("Observations analyzed", labels, fixed = TRUE)]

    skip_if(length(footer) == 0, "footer annotation not found")

    expect_match(footer[1],
                 sprintf("Observations analyzed: %d of %d",
                         as.integer(stats::nobs(model)), nrow(dt)),
                 fixed = TRUE)
})


test_that("forest footers report observations and events on stacked lines", {

    df <- fresh_clintrial()
    df$readmission_30d[seq_len(100)] <- NA
    dt <- data.table::as.data.table(df)

    model <- stats::glm(readmission_30d ~ age + sex,
                        data = dt, family = stats::binomial)

    p <- suppressMessages(suppressWarnings(glmforest(model, data = dt)))

    labels <- unlist(lapply(p$layers, function(lyr) {
        as.character(lyr$aes_params$label)
    }))
    footer <- labels[grepl("Observations analyzed", labels, fixed = TRUE)]

    skip_if(length(footer) == 0, "footer annotation not found")

    ## Both lines present, and the events line follows the observations line
    expect_match(footer[1], "Observations analyzed: [0-9]+ of [0-9]+")
    expect_match(footer[1], "Events analyzed: [0-9]+ of [0-9]+")
    expect_lt(regexpr("Observations analyzed", footer[1], fixed = TRUE),
              regexpr("Events analyzed", footer[1], fixed = TRUE))
})


test_that("linear model footers carry no events line", {

    dt <- data.table::as.data.table(fresh_clintrial())

    model <- stats::lm(los_days ~ age + sex, data = dt)

    p <- suppressMessages(suppressWarnings(lmforest(model, data = dt)))

    labels <- unlist(lapply(p$layers, function(lyr) {
        as.character(lyr$aes_params$label)
    }))
    footer <- labels[grepl("Observations analyzed", labels, fixed = TRUE)]

    skip_if(length(footer) == 0, "footer annotation not found")

    expect_false(grepl("Events analyzed", footer[1], fixed = TRUE))
})

### * Main function

#' Export Table to Microsoft Word Format (DOCX)
#'
#' Converts a data frame, data.table, or matrix to a fully editable Microsoft Word 
#' document (\code{.docx}) using the \pkg{flextable} and \pkg{officer} packages.
#' Creates publication-ready tables with extensive formatting options including
#' typography, alignment, colors, and page layout. Tables can be further edited in
#' Microsoft Word after creation.
#'
#' @param table Data frame, data.table, or matrix to export. Can be output from 
#'   \code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
#'   \code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.
#'   
#' @param file Character string specifying the output DOCX filename. Must have 
#'   \code{.docx} extension. Example: \code{"results.docx"}, \code{"Table1.docx"}.
#'   
#' @param caption Character string. Optional caption displayed above the table 
#'   in the Word document. Default is \code{NULL}.
#'   
#' @param font_size Numeric. Base font size in points for table content. 
#'   Default is 8. Typical range: 8-12 points. Headers use slightly larger size.
#'   
#' @param font_family Character string. Font family name for the table. Must be 
#'   a font installed on the system. Default is \code{"Arial"}. Common options: 
#'   \code{"Times New Roman"}, \code{"Calibri"}, \code{"Helvetica"}.
#'   
#' @param format_headers Logical. If \code{TRUE}, formats column headers by 
#'   italicizing statistical notation ("\emph{n}", "\emph{p}"), converting underscores 
#'   to spaces, and improving readability. Default is \code{TRUE}.
#'   
#' @param bold_significant Logical. If \code{TRUE}, applies bold formatting to 
#'   \emph{p}-values below the significance threshold. Makes significant results stand 
#'   out. Default is \code{TRUE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. Default is \code{FALSE}.
#'   
#' @param p_threshold Numeric. Threshold for bold \emph{p}-value formatting. Only 
#'   used when \code{bold_significant = TRUE}. Default is 0.05.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels under 
#'   their parent variable using horizontal spacing, creating hierarchical display. 
#'   Useful for categorical variables in regression tables. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses table by showing 
#'   only essential rows (single row for continuous, non-reference for binary). 
#'   Automatically sets \code{indent_groups = TRUE}. Significantly reduces table 
#'   height. Default is \code{FALSE}.
#'   
#' @param condense_quantitative Logical. If \code{TRUE}, condenses continuous 
#'   and survival variables into single rows while preserving all categorical 
#'   variable rows (including binary). Only applies to descriptive tables from 
#'   \code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
#'   \code{condense_table}, this does not collapse binary categorical variables. 
#'   Default is \code{FALSE}.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating row shading 
#'   to different variables (not individual rows) for visual grouping. 
#'   Default is \code{FALSE}.
#'   
#' @param dark_header Logical. If \code{TRUE}, creates a dark background with 
#'   light text for the header row, providing strong visual contrast. 
#'   Default is \code{FALSE}.
#'   
#' @param paper Character string specifying paper size:
#'   \itemize{
#'     \item \code{"letter"} - US Letter (8.5" × 11") [default]
#'     \item \code{"a4"} - A4 (210 mm × 297 mm)
#'     \item \code{"legal"} - US Legal (8.5" × 14")
#'   }
#'   
#' @param orientation Character string specifying page orientation:
#'   \itemize{
#'     \item \code{"portrait"} - Vertical [default]
#'     \item \code{"landscape"} - Horizontal (for wide tables)
#'   }
#'   
#' @param width Numeric. Table width in inches. If \code{NULL} (default), 
#'   automatically fits to content and page width. Specify to control exactly.
#'   
#' @param align Character vector specifying column alignment for each column. 
#'   Options: \code{"left"}, \code{"center"}, or \code{"right"}. If \code{NULL} 
#'   (default), automatically determines based on content (text left, numbers right).
#'   Example: \code{c("left", "left", "center", "right", "right")}.
#'   
#' @param return_ft Logical. If \code{TRUE}, returns the \pkg{flextable} object 
#'   directly for further customization. If \code{FALSE} (default), returns 
#'   invisibly with \pkg{flextable} object as attribute. See Details for usage. 
#'   Default is \code{FALSE}.
#'   
#' @param ... Additional arguments passed to \code{\link[officer]{read_docx}} 
#'   for document initialization.
#'
#' @param quiet Logical. Suppress progress and confirmation messages, such
#'   as the notice reporting the file written. Errors and warnings are
#'   unaffected. Default is \code{FALSE}.
#'
#' @return Behavior depends on \code{return_ft}:
#'   \describe{
#'     \item{\code{return_ft = FALSE}}{Invisibly returns a list with components:
#'       \itemize{
#'         \item \code{file} - Path to created file
#'         \item \code{caption} - Caption text (if provided)
#'       }
#'       The \pkg{flextable} object is accessible via \code{attr(result, "flextable")}
#'     }
#'     \item{\code{return_ft = TRUE}}{Directly returns the \pkg{flextable} object for 
#'       immediate further customization}
#'   }
#'   
#'   In both cases, creates a \code{.docx} file at the specified location.
#'
#' @details
#' \strong{Package Requirements:}
#' 
#' This function requires:
#' \itemize{
#'   \item \strong{\pkg{flextable}} - For creating formatted tables
#'   \item \strong{\pkg{officer}} - For Word document manipulation
#' }
#' 
#' Install if needed:
#' \preformatted{
#' install.packages(c("flextable", "officer"))
#' }
#' 
#' \strong{Output Features:}
#' 
#' The generated Word document contains:
#' \itemize{
#'   \item Fully editable table (native Word table, not image)
#'   \item Professional typography and spacing
#'   \item Proper page setup (size, orientation, margins)
#'   \item Caption (if provided) as separate paragraph above table
#'   \item All formatting preserved but editable
#'   \item Compatible with Word 2007 and later
#' }
#' 
#' \strong{Further Customization:}
#' 
#' For programmatic customization beyond the built-in options, access the 
#' \code{flextable} object:
#' 
#' \emph{Method 1: Via attribute (default)}
#'
#' \preformatted{
#' result <- table2docx(table, "output.docx")
#' ft <- attr(result, "flextable")
#' 
#' # Customize flextable
#' ft <- flextable::bold(ft, i = 1, j = 1, part = "body")
#' ft <- flextable::color(ft, i = 2, j = 3, color = "red")
#' 
#' # Re-save if needed
#' doc <- officer::read_docx()
#' doc <- flextable::body_add_flextable(doc, ft)
#' print(doc, target = "customized.docx")
#' }
#' 
#' \emph{Method 2: Direct return}
#'
#' \preformatted{
#' ft <- table2docx(table, "output.docx", return_ft = TRUE)
#' 
#' # Customize immediately
#' ft <- flextable::bg(ft, bg = "yellow", part = "header")
#' ft <- flextable::autofit(ft)
#' 
#' # Save to new document
#' doc <- officer::read_docx()
#' doc <- flextable::body_add_flextable(doc, ft)
#' print(doc, target = "custom.docx")
#' }
#' 
#' \strong{Page Layout:}
#' 
#' The function automatically sets up the Word document with:
#' \itemize{
#'   \item Specified paper size and orientation
#'   \item Standard margins (1 inch by default)
#'   \item Continuous section (no page breaks before table)
#'   \item Left-aligned table placement
#' }
#' 
#' For landscape orientation:
#' \itemize{
#'   \item Automatically swaps page width and height
#'   \item Applies landscape property to section
#'   \item Useful for wide tables with many columns
#' }
#' 
#' \strong{Table Width Management:}
#' 
#' Width behavior:
#' \itemize{
#'   \item \code{width = NULL} - Auto-fits to content and page width
#'   \item \code{width = 6} - Exactly 6 inches wide
#'   \item Width distributed evenly across columns by default
#'   \item Can adjust individual column widths in Word after creation
#' }
#' 
#' For very wide tables:
#' \enumerate{
#'   \item Use \code{orientation = "landscape"}
#'   \item Use \code{paper = "legal"} for extra width
#'   \item Reduce \code{font_size}
#'   \item Use \code{condense_table = TRUE}
#'   \item Consider breaking across multiple tables
#' }
#' 
#' \strong{Typography:}
#' 
#' The function applies professional typography:
#' \itemize{
#'   \item Column headers: Bold, slightly larger font
#'   \item Body text: Regular weight, specified font size
#'   \item Numbers: Right-aligned for easy comparison
#'   \item Text: Left-aligned for readability
#'   \item Consistent spacing: Adequate padding in cells
#' }
#' 
#' Font family must be installed on the system where Word opens the document. 
#' Common cross-platform choices:
#' \itemize{
#'   \item Arial - Sans-serif, highly readable
#'   \item Times New Roman - Serif, traditional
#'   \item Calibri - Microsoft default, modern
#'   \item Helvetica - Sans-serif, professional
#' }
#' 
#' \strong{Zebra Striping:}
#' 
#' When \code{zebra_stripes = TRUE}:
#' \itemize{
#'   \item Alternating variables receive light gray background
#'   \item All rows of same variable share same shading
#'   \item Improves visual grouping
#'   \item Particularly useful for tables with many factor variables
#'   \item Color can be changed in Word after creation
#' }
#' 
#' \strong{Dark Header:}
#' 
#' When \code{dark_header = TRUE}:
#' \itemize{
#'   \item Header row: Dark gray/black background
#'   \item Header text: White for high contrast
#'   \item Modern, professional appearance
#'   \item Draws attention to column names
#' }
#' 
#' \strong{Integration with R Markdown/Quarto:}
#' 
#' For R Markdown/Quarto Word output:
#'
#' \preformatted{
#' # Create flextable for inline display
#' ft <- table2docx(results, "temp.docx", return_ft = TRUE)
#' 
#' # Display in R Markdown chunk
#' ft  # Renders in Word output
#' }
#' 
#' Or use \code{flextable} directly in chunks:
#' 
#' \preformatted{
#' flextable::flextable(results)
#' }
#' 
#' @seealso
#' \code{\link{tablesave}} for saving in the format given by the file extension,
#' \code{\link{table2pptx}} for PowerPoint slides,
#' \code{\link{table2pdf}} for PDF output,
#' \code{\link{table2html}} for HTML tables,
#' \code{\link{table2rtf}} for Rich Text Format,
#' \code{\link{table2tex}} for LaTeX output,
#' \code{\link[flextable]{flextable}} for the underlying table object,
#' \code{\link[officer]{read_docx}} for Word document manipulation
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example table
#' results <- fit(
#'    data = clintrial,
#'    outcome = "os_status",
#'    predictors = c("age", "sex", "treatment", "stage"),
#'    labels = clintrial_labels
#' )
#'
#' # Example 1: Basic Word export
#' if (requireNamespace("flextable", quietly = TRUE) &&
#'     requireNamespace("officer", quietly = TRUE)) {
#'   table2docx(results, file.path(tempdir(), "results.docx"))
#' }
#'
#' \donttest{
#' old_width <- options(width = 180)
#' # Example 2: With caption
#' table2docx(results, file.path(tempdir(), "captioned.docx"),
#'         caption = "Table 1: Multivariable Logistic Regression Results")
#' 
#' # Example 3: Landscape orientation for wide tables
#' table2docx(results, file.path(tempdir(), "wide.docx"),
#'         orientation = "landscape")
#' 
#' # Example 4: Custom font and size
#' table2docx(results, file.path(tempdir(), "custom_font.docx"),
#'         font_family = "Times New Roman",
#'         font_size = 11)
#' 
#' # Example 5: Hierarchical display
#' table2docx(results, file.path(tempdir(), "indented.docx"),
#'         indent_groups = TRUE)
#' 
#' # Example 6: Condensed table
#' table2docx(results, file.path(tempdir(), "condensed.docx"),
#'         condense_table = TRUE)
#' 
#' # Example 7: With zebra stripes
#' table2docx(results, file.path(tempdir(), "striped.docx"),
#'         zebra_stripes = TRUE)
#' 
#' # Example 8: Dark header style
#' table2docx(results, file.path(tempdir(), "dark.docx"),
#'         dark_header = TRUE)
#' 
#' # Example 9: A4 paper for international journals
#' table2docx(results, file.path(tempdir(), "a4.docx"),
#'         paper = "a4")
#' 
#' # Example 10: Get flextable for customization
#' result <- table2docx(results, file.path(tempdir(), "base.docx"))
#' ft <- attr(result, "flextable")
#' 
#' # Customize the flextable
#' ft <- flextable::bold(ft, i = 1, part = "body")
#' ft <- flextable::color(ft, j = "p-value", color = "blue")
#' 
#' # Example 11: Direct flextable return
#' ft <- table2docx(results, file.path(tempdir(), "direct.docx"), return_ft = TRUE)
#' ft <- flextable::bg(ft, bg = "yellow", part = "header")
#' 
#' # Example 12: Publication-ready table
#' table2docx(results, file.path(tempdir(), "publication.docx"),
#'         caption = "Table 2: Adjusted Odds Ratios for Mortality",
#'         font_family = "Times New Roman",
#'         font_size = 10,
#'         indent_groups = TRUE,
#'         zebra_stripes = FALSE,
#'         bold_significant = TRUE)
#' 
#' # Example 13: Custom column alignment
#' table2docx(results, file.path(tempdir(), "aligned.docx"),
#'         align = c("left", "left", "center", "right", "right"))
#' 
#' # Example 14: Disable significance bolding
#' table2docx(results, file.path(tempdir(), "no_bold.docx"),
#'         bold_significant = FALSE)
#' 
#' # Example 15: Stricter significance threshold
#' table2docx(results, file.path(tempdir(), "strict.docx"),
#'         bold_significant = TRUE,
#'         p_threshold = 0.01)
#'
#' options(old_width)
#' }
#' @family export functions
#' @export
table2docx <- function(table,
                       file,
                     caption = NULL,
                     font_size = 8,
                     font_family = "Arial",
                     format_headers = TRUE,
                     bold_significant = TRUE,
                     bold_variables = FALSE,
                     p_threshold = 0.05,
                     indent_groups = FALSE,
                     condense_table = FALSE,
                     condense_quantitative = FALSE,
                     zebra_stripes = FALSE,
                     dark_header = FALSE,
                     paper = "letter",
                     orientation = "portrait",
                     width = NULL,
                     align = NULL,
                     return_ft = FALSE,
                     quiet = FALSE,
                     ...) {
    
    if (!requireNamespace("flextable", quietly = TRUE)) {
        stop("Package 'flextable' required. Install with: install.packages('flextable')")
    }
    if (!requireNamespace("officer", quietly = TRUE)) {
        stop("Package 'officer' required. Install with: install.packages('officer')")
    }
    
    if (!grepl("\\.docx$", tolower(file))) {
        stop("File must have .docx extension")
    }
    
    ## Validate paper and orientation
    paper <- match.arg(paper, c("letter", "a4", "legal"))
    orientation <- match.arg(orientation, c("portrait", "landscape"))
    
    ## Process table using shared function
    result <- process_table_for_flextable(
        table = table,
        caption = caption,
        font_size = font_size,
        font_family = font_family,
        format_headers = format_headers,
        bold_significant = bold_significant,
        p_threshold = p_threshold,
        indent_groups = indent_groups,
        condense_table = condense_table,
        condense_quantitative = condense_quantitative,
        zebra_stripes = zebra_stripes,
        dark_header = dark_header,
        bold_variables = bold_variables,
        paper = paper,
        orientation = orientation,
        width = width,
        align = align
    )
    
    ## Extract flextable for easy access
    ft <- result$ft
    
    ## Create Word document
    doc <- officer::read_docx(...)
    
    ## Set page size and orientation using prop_section
    page_width <- if(paper == "a4") 8.27 else 8.5
    page_height <- if(paper == "a4") 11.69 else if(paper == "legal") 14 else 11
    
    if (orientation == "landscape") {
        temp <- page_width
        page_width <- page_height
        page_height <- temp
    }
    
    sect_properties <- officer::prop_section(
                                    page_size = officer::page_size(width = page_width, 
                                                                   height = page_height, 
                                                                   orient = orientation),
                                    type = "continuous",
                                    page_margins = officer::page_mar()
                                )
    
    doc <- officer::body_set_default_section(doc, value = sect_properties)
    
    ## Add caption if provided
    if (!is.null(result$caption)) {
        doc <- officer::body_add_par(doc, result$caption, 
                                     style = "Normal", pos = "after")
        doc <- officer::body_add_par(doc, "", style = "Normal")
    }
    
    ## Add table
    doc <- flextable::body_add_flextable(doc, ft, align = "left")
    
    ## Save document
    print(doc, target = file)
    
    if (!quiet) message(sprintf("Table saved to %s", file))
    
    ## Return based on user preference
    if (return_ft) {
        ## Return flextable directly for easy access
        return(ft)
    } else {
        ## Return invisibly with flextable as attribute for backward compatibility
        result_obj <- list(
            file = file,
            caption = result$caption
        )
        attr(result_obj, "flextable") <- ft
        class(result_obj) <- c("table2docx_result", "list")
        return(invisible(result_obj))
    }
}


### * Print function

#' Print method for table2docx results
#'
#' @param x Object of class \code{table2docx_result}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @export
print.table2docx_result <- function(x, ...) {
    cat("Table saved to:", x$file, "\n")
    if (!is.null(x$caption)) {
        cat("Caption:", x$caption, "\n")
    }
    cat("Flextable object available via: attr(result, 'flextable')\n")
    invisible(x)
}

#' Test Suite for table2* Export Functions
#' 
#' Tests for table export functions: table2docx, table2pptx, table2html, 
#' table2pdf, table2tex, table2rtf. Focuses on input validation, 
#' file creation, and integration with summata output.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-table2export.R")

library(data.table)

## ============================================================================
## Setup: Create test data
## ============================================================================


## Create sample tables for testing
clintrial$response <- as.integer(clintrial$os_status == 1 & clintrial$os_months < 24)

## Create a simple fit result for testing
test_table <- fit(
    data = clintrial,
    outcome = "response",
    predictors = c("age", "sex", "stage"),
    model_type = "glm"
)

## Create a simple data.frame for basic tests
simple_table <- data.frame(
    Variable = c("Age", "Sex", "", ""),
    Group = c("", "Male", "Female", ""),
    Total = c("55.2 ± 12.3", "150 (48.5%)", "159 (51.5%)", ""),
    `p-value` = c("0.042", "", "", "0.823"),
    check.names = FALSE
)


## ============================================================================
## SECTION 1: table2docx Input Validation
## ============================================================================

test_that("table2docx requires .docx extension", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    
    expect_error(
        table2docx(simple_table, "output.txt"),
        regexp = "\\.docx"
    )
})


test_that("table2docx validates paper parameter", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    
    ## match.arg throws error mentioning the valid options
    expect_error(
        table2docx(simple_table, tempfile(fileext = ".docx"), paper = "invalid")
    )
})


test_that("table2docx validates orientation parameter", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    
    ## match.arg throws error mentioning the valid options
    expect_error(
        table2docx(simple_table, tempfile(fileext = ".docx"), orientation = "invalid")
    )
})


## ============================================================================
## SECTION 9: table2pptx Input Validation
## ============================================================================

test_that("table2pptx requires .pptx extension", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    
    expect_error(
        table2pptx(simple_table, "output.txt"),
        regexp = "\\.pptx"
    )
})


## ============================================================================
## SECTION 10: table2html Input Validation
## ============================================================================

test_that("table2html requires .html extension", {
    
    skip_if_not_installed("kableExtra")
    
    expect_error(
        table2html(simple_table, "output.txt"),
        regexp = "\\.html"
    )
})


## ============================================================================
## SECTION 11: table2pdf Input Validation
## ============================================================================

test_that("table2pdf requires .pdf extension", {
    
    skip_if_not_installed("xtable")
    skip_if_not_installed("tinytex")
    
    expect_error(
        table2pdf(simple_table, "output.txt"),
        regexp = "\\.pdf"
    )
})


## ============================================================================
## SECTION 12: table2tex Input Validation
## ============================================================================

test_that("table2tex requires .tex extension", {
    
    skip_if_not_installed("xtable")
    
    expect_error(
        table2tex(simple_table, "output.txt"),
        regexp = "\\.tex"
    )
})


## ============================================================================
## SECTION 13: table2rtf Input Validation
## ============================================================================

test_that("table2rtf requires .rtf extension", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    
    expect_error(
        table2rtf(simple_table, "output.txt"),
        regexp = "\\.rtf"
    )
})


## ============================================================================
## SECTION 14: File Creation Tests (Integration)
## ============================================================================

test_that("table2docx creates file", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(simple_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx returns correct structure", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(simple_table, temp_file, caption = "Test Caption")
    
    expect_s3_class(result, "table2docx_result")
    expect_equal(result$file, temp_file)
    expect_equal(result$caption, "Test Caption")
    
    ## Should have flextable attribute
    ft <- attr(result, "flextable")
    expect_true(!is.null(ft))
    expect_s3_class(ft, "flextable")
})


test_that("table2docx return_ft=TRUE returns flextable", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(simple_table, temp_file, return_ft = TRUE)
    
    expect_s3_class(result, "flextable")
})


test_that("table2pptx creates file", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".pptx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2pptx(simple_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


test_that("table2html creates file", {
    
    skip_if_not_installed("kableExtra")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".html")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2html(simple_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


test_that("table2tex creates file", {
    
    skip_if_not_installed("xtable")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".tex")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2tex(simple_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


## ============================================================================
## SECTION 15: Package Requirement Tests
## ============================================================================

test_that("table2docx checks for flextable package", {
    
    ## This test verifies the error message format
    ## We can't actually test the missing package scenario easily
    ## but we can verify the function exists and has proper structure
    
    expect_true(is.function(table2docx))
})


test_that("table2html checks for kableExtra package", {
    
    expect_true(is.function(table2html))
})


test_that("table2pdf checks for xtable package", {
    
    expect_true(is.function(table2pdf))
})


## ============================================================================
## SECTION 16: Option Processing Tests
## ============================================================================

test_that("table2docx processes indent_groups option", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    ## Should not error with indent_groups
    result <- table2docx(test_table, temp_file, indent_groups = TRUE)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx processes condense_table option", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    ## Should not error with condense_table
    result <- table2docx(test_table, temp_file, condense_table = TRUE)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx processes zebra_stripes option", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    ## Should not error with zebra_stripes
    result <- table2docx(test_table, temp_file, zebra_stripes = TRUE)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx processes dark_header option", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    ## Should not error with dark_header
    result <- table2docx(test_table, temp_file, dark_header = TRUE)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx processes font options", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    ## Should not error with custom font options
    result <- table2docx(test_table, temp_file, 
                         font_size = 10, 
                         font_family = "Times New Roman")
    
    expect_true(file.exists(temp_file))
})


## ============================================================================
## SECTION 17: Integration with summata Output
## ============================================================================

test_that("table2docx works with fit() output", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(test_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx works with desctable() output", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    desc_table <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex")
    )
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(desc_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


test_that("table2html works with fit() output", {
    
    skip_if_not_installed("kableExtra")
    skip_on_cran()
    
    temp_file <- tempfile(fileext = ".html")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2html(test_table, temp_file)
    
    expect_true(file.exists(temp_file))
})


## ============================================================================
## SECTION 18: Edge Cases
## ============================================================================

test_that("table2docx handles empty table gracefully", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    empty_table <- data.frame(
        Variable = character(0),
        Value = character(0)
    )
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    ## Should either work or give informative error
    result <- tryCatch(
        table2docx(empty_table, temp_file),
        error = function(e) e
    )
    
    ## Either file created or error caught
    expect_true(file.exists(temp_file) || inherits(result, "error"))
})


test_that("table2docx handles single row table", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    single_row <- data.frame(
        Variable = "Age",
        Value = "55.2"
    )
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(single_row, temp_file)
    
    expect_true(file.exists(temp_file))
})


test_that("table2docx handles special characters", {
    
    skip_if_not_installed("flextable")
    skip_if_not_installed("officer")
    skip_on_cran()
    
    special_table <- data.frame(
        Variable = c("α", "β", "γ"),
        Value = c("< 0.001", "≥ 5%", "95% CI")
    )
    
    temp_file <- tempfile(fileext = ".docx")
    on.exit(unlink(temp_file), add = TRUE)
    
    result <- table2docx(special_table, temp_file)
    
    expect_true(file.exists(temp_file))
})

### * Main function

#' Export Table to LaTeX Format
#'
#' Converts a data frame, data.table, or matrix to LaTeX source code suitable for 
#' inclusion in LaTeX documents. Generates publication-quality table markup with 
#' extensive formatting options including booktabs styling, color schemes, and 
#' hierarchical displays. Output can be directly \code{\\input{}} or \code{\\include{}} 
#' into LaTeX manuscripts. Requires \pkg{xtable} for export.
#'
#' @param table Data frame, data.table, or matrix to export. Can be output from 
#'   \code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
#'   \code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.
#'   
#' @param file Character string specifying the output \code{.tex} filename. Must have 
#'   \code{.tex} extension. Example: \code{"results.tex"}, \code{"table1.tex"}.
#'   
#' @param caption Character string. Table caption for LaTeX caption command. 
#'   Supports multi-line captions using double backslash. Default is \code{NULL}.
#'
#' @param caption_size Numeric. Caption font size in points. If \code{NULL} (default), 
#'   caption will use the document's default caption size (typically slightly smaller 
#'   than body text). Set to a specific value (\emph{e.g.,} 6, 7, 8, 9) to control caption 
#'   size explicitly. This generates a LaTeX comment for use when wrapping 
#'   the table. Typical range: 6-10 points.
#'   
#' @param format_headers Logical. If \code{TRUE}, formats column headers by 
#'   converting underscores to spaces, italicizing statistical notation ("\emph{n}", 
#'   "\emph{p}"), and applying title case. Default is \code{TRUE}.
#'   
#' @param variable_padding Logical. If \code{TRUE}, adds vertical spacing around 
#'   variable groups using \code{\\addlinespace} for improved readability. 
#'   Default is \code{FALSE}.
#'   
#' @param cell_padding Character string or numeric. Vertical padding within cells:
#'   \itemize{
#'     \item \code{"none"} - No extra padding
#'     \item \code{"normal"} - Standard padding [default]
#'     \item \code{"relaxed"} - Increased padding
#'     \item \code{"loose"} - Maximum padding
#'     \item Numeric - Custom \code{\\arraystretch} value
#'   }
#'   
#' @param bold_significant Logical. If \code{TRUE}, wraps significant \emph{p}-values 
#'   in \code{textbf} commands for bold display. Default is \code{TRUE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. Default is \code{FALSE}.
#'   
#' @param p_threshold Numeric. Threshold for bold \emph{p}-value formatting. Only 
#'   used when \code{bold_significant = TRUE}. Default is 0.05.
#'   
#' @param align Character string or vector specifying column alignment:
#'   \itemize{
#'     \item \code{"l"} - Left
#'     \item \code{"c"} - Center  
#'     \item \code{"r"} - Right
#'     \item Paragraph column with specified width (p-type)
#'   }
#'   If \code{NULL}, automatically determines based on content. Can specify 
#'   per-column as vector. Default is \code{NULL}.
#'   
#' @param indent_groups Logical. If \code{TRUE}, uses hspace to 
#'   indent grouped rows, creating hierarchical display. Useful for factor 
#'   variables in regression tables. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses table by showing 
#'   only essential rows (single row for continuous, non-reference for binary). 
#'   Automatically sets \code{indent_groups = TRUE}. Default is \code{FALSE}.
#'   
#' @param condense_quantitative Logical. If \code{TRUE}, condenses continuous 
#'   and survival variables into single rows while preserving all categorical 
#'   variable rows (including binary). Only applies to descriptive tables from 
#'   \code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
#'   \code{condense_table}, this does not collapse binary categorical variables. 
#'   Default is \code{FALSE}.
#'   
#' @param booktabs Logical. If \code{TRUE}, uses booktabs package commands 
#'   (toprule, midrule, bottomrule) for professional 
#'   table rules. Requires booktabs package in LaTeX preamble. 
#'   Default is \code{FALSE}.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, adds alternating row colors 
#'   for variable groups using rowcolor command. Requires 
#'   \code{xcolor} package with table option in preamble. Default is \code{FALSE}.
#'   
#' @param stripe_color Character string. LaTeX color specification for zebra 
#'   stripes (\emph{e.g.,} \code{"gray!20"}, \code{"blue!10"}). Only used when 
#'   \code{zebra_stripes = TRUE}. Default is \code{"gray!20"}.
#'   
#' @param dark_header Logical. If \code{TRUE}, creates white text on black 
#'   background for header row. Requires \code{xcolor} package with \code{table}
#'   option. Default is \code{FALSE}.
#'   
#' @param label Character string. LaTeX label for cross-references. 
#'   Example: \code{"tab:regression"}. Default is \code{NULL}.
#'
#' @param show_logs Logical. If \code{TRUE}, displays informational messages 
#'   about required LaTeX packages and formatting options applied. If 
#'   \code{FALSE}, suppresses these messages. Default is \code{FALSE}.
#'   
#' @param ... Additional arguments passed to \code{\link[xtable]{xtable}}.
#'
#' @param quiet Logical. Suppress progress and confirmation messages, such
#'   as the notice reporting the file written. Errors and warnings are
#'   unaffected. Default is \code{FALSE}.
#'
#' @return Invisibly returns the file path, matching \code{tablesave()} and
#'   \code{forestsave()}. Called for its side effect of creating a \code{.tex}
#'   file at the specified location containing a LaTeX tabular environment.
#'
#' @details
#' \strong{Output Format:}
#' 
#' The function generates a standalone LaTeX tabular environment that can be:
#' \enumerate{
#'   \item Included in documents with \code{\\input} command
#'   \item Embedded in table/figure environments
#'   \item Used in manuscript classes (\code{article}, \code{report}, \emph{etc.})
#' }
#' 
#' The output includes:
#' \itemize{
#'   \item Complete tabular environment with proper alignment
#'   \item Horizontal rules (\code{\\hline} or \code{booktabs} rules)
#'   \item Column headers with optional formatting
#'   \item Data rows with automatic escaping of special characters
#'   \item Optional caption and label commands
#' }
#' 
#' \strong{Required LaTeX Packages:}
#' 
#' The following are required in the LaTeX document preamble:
#' 
#' \emph{Always required:}
#' \preformatted{
#' \\usepackage[T1]{fontenc}
#' \\usepackage[utf8]{inputenc}
#' \\usepackage{array}
#' \\usepackage{graphicx}  % If using resizebox
#' }
#' 
#' \emph{Optional (based on parameters):}
#' \preformatted{
#' \\usepackage{booktabs}  % For booktabs = TRUE
#' \\usepackage[table]{xcolor}  % For zebra_stripes or dark_header
#' }
#' 
#' \strong{Booktabs Style:}
#' 
#' When \code{booktabs = TRUE}, the table uses publication-quality rules:
#' \itemize{
#'   \item \code{\\toprule} - Heavy rule at top
#'   \item \code{\\midrule} - Medium rule below headers
#'   \item \code{\\bottomrule} - Heavy rule at bottom
#'   \item No vertical rules (\code{booktabs} style)
#'   \item Better spacing around rules
#' }
#' 
#' This is the preferred style for most academic journals.
#' 
#' \strong{Color Features:}
#' 
#' \emph{Zebra Stripes:}
#' Creates alternating background colors for visual grouping:
#' \preformatted{
#' zebra_stripes = TRUE
#' stripe_color = "gray!20"  # 20\% gray
#' stripe_color = "blue!10"  # 10\% blue  
#' }
#' 
#' \emph{Dark Header:}
#' Creates high-contrast header row:
#' \preformatted{
#' dark_header = TRUE  # Black background, white text
#' }
#' 
#' Both require the xcolor package with the table option in the document preamble.
#' 
#' \strong{Integration with LaTeX Documents:}
#' 
#' \emph{Basic inclusion:}
#' \preformatted{
#' \\begin{table}[htbp]
#'   \\centering
#'   \\caption{Regression Results}
#'   \\label{tab:regression}
#'   \\input{results.tex}
#' \\end{table}
#' }
#' 
#' \emph{With resizing:}
#' \preformatted{
#' \\begin{table}[htbp]
#'   \\centering
#'   \\caption{Results}
#'   \\resizebox{\\textwidth}{!}{\\input{results.tex}}
#' \\end{table}
#' }
#' 
#' \emph{Landscape orientation:}
#' \preformatted{
#' \\usepackage{pdflscape}
#' \\begin{landscape}
#'   \\begin{table}[htbp]
#'     \\centering
#'     \\input{wide_results.tex}
#'   \\end{table}
#' \\end{landscape}
#' }
#' 
#' \strong{Caption Formatting:}
#' 
#' Captions in the \code{caption} parameter are written as LaTeX comments in 
#' the output file for reference. For actual LaTeX captions, wrap the table 
#' in a table environment (see examples above).
#' 
#' \strong{Special Characters:}
#' 
#' The function automatically escapes LaTeX special characters in the data:
#' \itemize{
#'   \item Ampersand, percent, dollar sign, hash, underscore
#'   \item Left and right braces
#'   \item Tilde and caret (using \code{textasciitilde} and \code{textasciicircum})
#' }
#' 
#' Variable names and labels should not include these characters unless 
#' intentionally using LaTeX commands.
#' 
#' @seealso
#' \code{\link{tablesave}} for saving in the format given by the file extension,
#' \code{\link{table2pdf}} for direct PDF output,
#' \code{\link{table2html}} for HTML tables,
#' \code{\link{table2docx}} for Word documents,
#' \code{\link{table2pptx}} for PowerPoint,
#' \code{\link{table2rtf}} for Rich Text Format,
#' \code{\link{fit}} for regression tables,
#' \code{\link{desctable}} for descriptive tables
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example table
#' results <- fit(
#'    data = clintrial,
#'    outcome = "os_status",
#'    predictors = c("age", "sex", "treatment", "stage"),
#'    labels = clintrial_labels
#' )
#'
#' # Example 1: Basic LaTeX export
#' if (requireNamespace("xtable", quietly = TRUE)) {
#'   table2tex(results, file.path(tempdir(), "basic.tex"))
#' }
#'
#' \donttest{
#' # Example 2: With booktabs for publication
#' table2tex(results, file.path(tempdir(), "publication.tex"),
#'        booktabs = TRUE,
#'        caption = "Multivariable logistic regression results",
#'        label = "tab:regression")
#' 
#' # Example 3: Multi-line caption with abbreviations
#' table2tex(results, file.path(tempdir(), "detailed.tex"),
#'        booktabs = TRUE,
#'        caption = "Table 1: Risk Factors for Mortality\\\\
#'                  aOR = adjusted odds ratio; CI = confidence interval\\\\
#'                  Model adjusted for age, sex, treatment, and disease stage",
#'        label = "tab:mortality")
#' 
#' # Example 4: Hierarchical display with indentation
#' table2tex(results, file.path(tempdir(), "indented.tex"),
#'        indent_groups = TRUE,
#'        booktabs = TRUE)
#' 
#' # Example 5: Condensed table (reduced height)
#' table2tex(results, file.path(tempdir(), "condensed.tex"),
#'        condense_table = TRUE,
#'        booktabs = TRUE)
#' 
#' # Example 6: With zebra stripes
#' table2tex(results, file.path(tempdir(), "striped.tex"),
#'        zebra_stripes = TRUE,
#'        stripe_color = "gray!15",
#'        booktabs = TRUE)
#' # Remember to add \usepackage[table]{xcolor} to the LaTeX document
#' 
#' # Example 7: Dark header style
#' table2tex(results, file.path(tempdir(), "dark_header.tex"),
#'        dark_header = TRUE,
#'        booktabs = TRUE)
#' # Requires \usepackage[table]{xcolor}
#' 
#' # Example 8: Custom cell padding
#' table2tex(results, file.path(tempdir(), "relaxed.tex"),
#'        cell_padding = "relaxed",
#'        booktabs = TRUE)
#' 
#' # Example 9: Custom column alignment (auto-detected by default)
#' table2tex(results, file.path(tempdir(), "custom_align.tex"),
#'        align = c("c", "c", "c", "c", "c", "c", "c"))
#' 
#' # Example 10: No header formatting (keep original names)
#' table2tex(results, file.path(tempdir(), "raw_headers.tex"),
#'        format_headers = FALSE)
#' 
#' # Example 11: Disable significance bolding
#' table2tex(results, file.path(tempdir(), "no_bold.tex"),
#'        bold_significant = FALSE,
#'        booktabs = TRUE)
#' 
#' # Example 12: Stricter significance threshold
#' table2tex(results, file.path(tempdir(), "strict_sig.tex"),
#'        bold_significant = TRUE,
#'        p_threshold = 0.01,  # Bold only if p < 0.01
#'        booktabs = TRUE)
#'
#' # Example 13: With caption size control
#' table2tex(results, file.path(tempdir(), "caption_size.tex"),
#'        caption_size = 6,
#'        caption = "Table 1 - Results with Compact Caption\\\\
#'                  Smaller caption fits better on constrained pages")
#' 
#' # Example 14: Complete publication-ready table
#' table2tex(results, file.path(tempdir(), "final_table1.tex"),
#'        booktabs = TRUE,
#'        caption = "Table 1: Multivariable Analysis of Mortality Risk Factors",
#'        label = "tab:main_results",
#'        indent_groups = TRUE,
#'        zebra_stripes = FALSE,  # Many journals prefer no stripes
#'        bold_significant = TRUE,
#'        cell_padding = "normal")
#' 
#' # Example 15: Descriptive statistics table
#' desc_table <- desctable(clintrial, by = "treatment",
#'    variables = c("age", "sex", "bmi"), labels = clintrial_labels)
#' 
#' table2tex(desc_table, file.path(tempdir(), "table1_descriptive.tex"),
#'        booktabs = TRUE,
#'        caption = "Table 1: Baseline Characteristics",
#'        label = "tab:baseline")
#' 
#' # Example 16: Model comparison table
#' models <- list(
#'    base = c("age", "sex"),
#'    full = c("age", "sex", "treatment", "stage")
#' )
#' 
#' # Information criteria assume a common sample, so the candidate
#' # predictors are restricted to complete cases before comparison
#' comparison_data <- na.omit(clintrial[, c("os_status", "age", "sex", "treatment", "stage")])
#' 
#' comparison <- compfit(
#'    data = comparison_data,
#'    outcome = "os_status",
#'    model_list = models
#' )
#' 
#' table2tex(comparison, file.path(tempdir(), "model_comparison.tex"),
#'        booktabs = TRUE,
#'        caption = "Model Comparison Statistics",
#'        label = "tab:models")
#' 
#' }
#' @family export functions
#' @export
table2tex <- function (table,
                       file,
                     format_headers = TRUE,
                     variable_padding = FALSE, 
                     cell_padding = "normal",
                     bold_significant = TRUE,
                     bold_variables = FALSE,
                     p_threshold = 0.05,
                     align = NULL, 
                     indent_groups = FALSE,
                     condense_table = FALSE,
                     condense_quantitative = FALSE,
                     booktabs = FALSE,
                     zebra_stripes = FALSE,
                     stripe_color = "gray!20",
                     dark_header = FALSE,
                     caption = NULL,
                     caption_size = NULL,
                     label = NULL,
                     show_logs = FALSE,
                     quiet = FALSE,
                     ...) {
    
    if (!requireNamespace("xtable", quietly = TRUE)) {
        stop("Package 'xtable' required. Install with: install.packages('xtable')")
    }
    
    if (!grepl("\\.tex$", tolower(file))) {
        stop("File must have .tex extension")
    }

    ## Validate caption_size if provided
    if (!is.null(caption_size)) {
        if (!is.numeric(caption_size) || caption_size <= 0) {
            stop("caption_size must be a positive numeric value")
        }
    }
    
    df <- as.data.frame(table)

    ## Detect variable groups before any processing
    var_groups <- NULL
    if (zebra_stripes) {
        if ("Variable" %in% names(df)) {
            var_starts <- which(df$Variable != "" & !is.na(df$Variable))
            if (length(var_starts) > 0) {
                var_groups <- integer(nrow(df))
                for (i in seq_along(var_starts)) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   var_starts[i + 1] - 1
                               } else {
                                   nrow(df)
                               }
                    var_groups[start_idx:end_idx] <- i
                }
            }
        }
    }

    has_n_row <- FALSE
    n_row_data <- NULL
    if (nrow(df) > 0 && "Variable" %in% names(df) && df$Variable[1] == "N") {
        has_n_row <- TRUE
        n_row_data <- df[1, ]
        df <- df[-1, ]  # Remove N row from data
        ## Adjust var_groups if N row was removed
        if (!is.null(var_groups) && length(var_groups) > 1) {
            var_groups <- var_groups[-1]
        }
    }

    if (condense_table) {
        indent_groups <- TRUE
        df <- condense_table_rows(df, indent_groups = indent_groups)
        ## Re-detect groups after condensing
        if (zebra_stripes && "Variable" %in% names(df)) {
            var_starts <- which(df$Variable != "" & !is.na(df$Variable))
            if (length(var_starts) > 0) {
                var_groups <- integer(nrow(df))
                for (i in seq_along(var_starts)) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   var_starts[i + 1] - 1
                               } else {
                                   nrow(df)
                               }
                    var_groups[start_idx:end_idx] <- i
                }
            }
        }
        df <- format_indented_groups(df, indent_string = "\\hspace{1em}")
    } else if (condense_quantitative) {
        indent_groups <- TRUE
        df <- condense_quantitative_rows(df, indent_groups = indent_groups)
        ## Re-detect groups after condensing
        if (zebra_stripes && "Variable" %in% names(df)) {
            var_starts <- which(df$Variable != "" & !is.na(df$Variable))
            if (length(var_starts) > 0) {
                var_groups <- integer(nrow(df))
                for (i in seq_along(var_starts)) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   var_starts[i + 1] - 1
                               } else {
                                   nrow(df)
                               }
                    var_groups[start_idx:end_idx] <- i
                }
            }
        }
        df <- format_indented_groups(df, indent_string = "\\hspace{1em}")
    } else if (indent_groups) {
        df <- format_indented_groups(df, indent_string = "\\hspace{1em}")
    }

    ## Bold variable names (non-indented rows in Variable column)
    if (bold_variables && "Variable" %in% names(df)) {
        ## Find rows where Variable is not empty and not indented (doesn't start with \hspace)
        var_rows <- which(!is.na(df$Variable) & 
                          df$Variable != "" & 
                          df$Variable != "-" &
                          !grepl("^\\\\hspace", df$Variable))
        if (length(var_rows) > 0) {
            df$Variable[var_rows] <- paste0("\\textbf{", df$Variable[var_rows], "}")
        }
    }

    if (bold_significant) {
        df <- format_pvalues_export_tex(df, p_threshold)
    }
    
    ## Track padding rows if adding padding
    if (variable_padding && ("Variable" %in% names(df) || "variable" %in% names(df))) {
        ## Before adding padding, adjust var_groups if necessary
        if (!is.null(var_groups)) {
            ## Find where padding rows will be inserted
            padding_positions <- which(diff(var_groups) != 0)
            ## Adjust var_groups for padding
            new_var_groups <- integer(nrow(df) + length(padding_positions))
            current_pos <- 1
            for (i in seq_len(nrow(df))) {
                new_var_groups[current_pos] <- var_groups[i]
                current_pos <- current_pos + 1
                if (i %in% padding_positions) {
                    ## Padding row gets group 0 (no shading)
                    new_var_groups[current_pos] <- 0
                    current_pos <- current_pos + 1
                }
            }
            var_groups <- new_var_groups
        }
        df <- add_variable_padding(df)
    }
    
    ## Set up add.to.row for zebra stripes
    add.to.row <- NULL
    if (zebra_stripes && "Variable" %in% names(df)) {
        ## Check if table has been indented
        is_indented <- indent_groups || condense_table || condense_quantitative
        
        ## Identify padding rows (all cells empty) - these should not be striped
        is_padding_row <- apply(df, 1, function(row) {
            all(is.na(row) | row == "" | row == " ")
        })
        
        if (is_indented) {
            ## For indented tables - look for rows where Variable is not empty and not indented
            var_starts <- which(!grepl("\\\\hspace", df$Variable) & 
                                df$Variable != "" & 
                                !is.na(df$Variable) &
                                !is_padding_row)
            
            commands <- character()
            positions <- numeric()
            
            for (i in seq_along(var_starts)) {
                if (i %% 2 != 0) {  # Odd variable groups get shading
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   if (variable_padding) {
                                       var_starts[i + 1] - 2
                                   } else {
                                       var_starts[i + 1] - 1
                                   }
                               } else {
                                   nrow(df)
                               }
                    
                    for (row in start_idx:end_idx) {
                        ## Skip padding rows
                        if (!is.na(df$Variable[row]) && !is_padding_row[row]) {
                            commands <- c(commands, paste0("\\rowcolor{", stripe_color, "}"))
                            positions <- c(positions, row - 1)
                        }
                    }
                }
            }
            
            if (length(commands) > 0) {
                add.to.row <- list(pos = as.list(positions), command = commands)
            }
        } else if ("Group" %in% names(df)) {
            ## For non-indented tables with Group column
            var_starts <- which(df$Variable != "" & !is.na(df$Variable) & !is_padding_row)
            
            commands <- character()
            positions <- numeric()
            
            for (i in seq_along(var_starts)) {
                if (i %% 2 != 0) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   if (variable_padding) {
                                       var_starts[i + 1] - 2
                                   } else {
                                       var_starts[i + 1] - 1
                                   }
                               } else {
                                   nrow(df)
                               }
                    
                    for (row in start_idx:end_idx) {
                        ## Skip padding rows
                        if (!is.na(df$Variable[row]) && !is_padding_row[row]) {
                            commands <- c(commands, paste0("\\rowcolor{", stripe_color, "}"))
                            positions <- c(positions, row - 1)
                        }
                    }
                }
            }
            
            if (length(commands) > 0) {
                add.to.row <- list(pos = as.list(positions), command = commands)
            }
        } else if (!is.null(var_groups)) {
            ## Use var_groups for simpler tables
            commands <- character()
            positions <- numeric()
            
            for (i in seq_len(nrow(df))) {
                ## Skip padding rows (var_groups == 0 or is_padding_row)
                if (var_groups[i] %% 2 != 0 && var_groups[i] > 0 && !is_padding_row[i]) {
                    commands <- c(commands, paste0("\\rowcolor{", stripe_color, "}"))
                    positions <- c(positions, i - 1)
                }
            }
            
            if (length(commands) > 0) {
                add.to.row <- list(pos = as.list(positions), command = commands)
            }
        }
    }
    
    ## Add cell padding command if requested
    ## Determine array stretch value for cell padding
    array_stretch <- switch(as.character(cell_padding),
                            "none" = NULL,
                            "normal" = "1.3",
                            "relaxed" = "1.5",
                            "loose" = "1.8",
                            {
                                ## If numeric value provided
                                if (is.numeric(cell_padding)) {
                                    as.character(cell_padding)
                                } else {
                                    "1.3"  # Default to normal
                                }
                            })
    
    ## Add arraystretch command at the beginning of the table
    if (!is.null(array_stretch)) {
        arraystretch_command <- sprintf("\\renewcommand{\\arraystretch}{%s}", array_stretch)
        
        if (!is.null(add.to.row) && length(add.to.row$pos) > 0) {
            ## Check if position -1 already exists
            if (-1 %in% unlist(add.to.row$pos)) {
                ## Find the index and prepend to that command
                idx <- which(unlist(add.to.row$pos) == -1)
                add.to.row$command[idx] <- paste0(arraystretch_command, " ", add.to.row$command[idx])
            } else {
                ## Add as first command
                add.to.row$pos <- c(list(-1), add.to.row$pos)
                add.to.row$command <- c(arraystretch_command, add.to.row$command)
            }
        } else {
            ## Create new add.to.row with arraystretch
            add.to.row <- list(
                pos = list(-1),
                command = arraystretch_command
            )
        }
    }
    
    if (is.null(align)) {
        align <- determine_alignment(df)
    }
    
    ## Store original column names before formatting
    original_col_names <- names(df)
    
    if (format_headers) {
        if (has_n_row) {
            names(df) <- format_column_headers_with_n_tex(names(df), n_row_data)
        } else {
            names(df) <- format_column_headers(names(df))
        }
    }
    
    if (dark_header) {
        ## Wrap each column name with the white color command
        col_names <- names(df)
        for (i in seq_along(col_names)) {
            col_names[i] <- paste0("\\color{white}", col_names[i])
        }
        names(df) <- col_names
        
        ## Add the rowcolor command (without the color command)
        header_command <- "\\rowcolor{black}"
        
        if (!is.null(add.to.row) && length(add.to.row$pos) > 0) {
            ## Determine whether to combine with arraystretch at position -1
            if (-1 %in% unlist(add.to.row$pos)) {
                ## Find the index and append to that command
                idx <- which(unlist(add.to.row$pos) == -1)
                add.to.row$command[idx] <- paste0(add.to.row$command[idx], " ", header_command)
            } else {
                ## Prepend header command
                add.to.row$pos <- c(list(-1), add.to.row$pos)
                add.to.row$command <- c(header_command, add.to.row$command)
            }
        } else {
            ## Create new add.to.row
            add.to.row <- list(
                pos = list(-1),
                command = header_command
            )
        }
    }
    
    xt <- xtable::xtable(df,
                         align = align,
                         caption = caption, 
                         label = label, ...)
    
    ## Prepare file output with necessary package declarations if needed
    file_conn <- file(file, "w")
    
    ## Write comment about required packages if using special features
    if (dark_header || zebra_stripes) {
        writeLines("% This table requires \\usepackage[table]{xcolor} in the LaTeX preamble", file_conn)
    }
    if (!is.null(array_stretch)) {
        writeLines(sprintf("%% Cell padding applied with \\arraystretch{%s}", array_stretch), file_conn)
    }
    if (!is.null(caption)) {
        writeLines(sprintf("%% Caption: %s", caption), file_conn)
    }
    if (!is.null(caption_size)) {
        baseline_skip <- ceiling(caption_size * 1.2)
        writeLines(sprintf("%% To apply caption font size, wrap caption in: {\\fontsize{%d}{%d}\\selectfont ...}", 
                           caption_size, baseline_skip), file_conn)
        writeLines(sprintf("%% Example: {\\fontsize{%d}{%d}\\selectfont\\caption{%s}}", 
                           caption_size, baseline_skip, 
                           if(!is.null(caption)) caption else "Your caption here"), file_conn)
    }
    if (!is.null(label)) {
        writeLines(sprintf("%% Label: %s", label), file_conn)
    }
    
    close(file_conn)
    
    ## Print with or without add.to.row commands
    if (!is.null(add.to.row) && length(add.to.row$pos) > 0) {
        print(xt,
              file = file,
              append = TRUE,  # Append after the preceding comments
              include.rownames = FALSE,
              booktabs = booktabs, 
              floating = FALSE,
              add.to.row = add.to.row,
              sanitize.text.function = sanitize_for_latex, 
              sanitize.rownames.function = sanitize_for_latex,
              sanitize.colnames.function = function(x) x,
              hline.after = c(-1, 0, nrow(xt)))
    } else {
        print(xt,
              file = file,
              append = TRUE,  # Append after the preceding comments
              include.rownames = FALSE,
              booktabs = booktabs, 
              floating = FALSE,
              sanitize.text.function = sanitize_for_latex, 
              sanitize.rownames.function = sanitize_for_latex,
              sanitize.colnames.function = function(x) x,
              hline.after = c(-1, 0, nrow(xt)))
    }
    
    ## Add a note about required packages if using special features
    if (show_logs) {
        if (dark_header || zebra_stripes) {
            if (!quiet) message("Note: This table requires \\usepackage[table]{xcolor} in the LaTeX preamble")
        }
        if (!is.null(array_stretch) && cell_padding != "none") {
            if (!quiet) message(sprintf("Note: Cell padding applied with \\arraystretch{%s}", array_stretch))
        }
        if (!is.null(caption_size)) {
            baseline_skip <- ceiling(caption_size * 1.2)
            if (!quiet) message(sprintf("Note: To use %dpt caption, wrap caption in: {\\fontsize{%d}{%d}\\selectfont ...}", 
                            caption_size, caption_size, baseline_skip))
        }
    }
    
    if (!quiet) message(sprintf("Table saved to %s", file))
    
    invisible(file)
}

### * Variable sorting

#' Process variable wrapper
#' 
#' Routes variable processing to appropriate handler based on variable type
#' (continuous, categorical, or survival). Returns both formatted display
#' strings and raw numeric values.
#' 
#' @param data Data.table containing the variable.
#' @param var Character string naming the variable to process.
#' @param group_var Optional character string naming the grouping variable.
#' @param stats_continuous Character vector of statistics for continuous variables.
#' @param stats_categorical Character vector of statistics for categorical variables.
#' @param digits Integer number of decimal places for continuous statistics.
#' @param conf_level Numeric confidence level for survival confidence intervals.
#' @param na_include Logical whether to include missing values as a category.
#' @param na_label Character string label for missing values.
#' @param test Logical whether to perform statistical tests.
#' @param test_continuous Character string specifying test type for continuous variables.
#' @param test_categorical Character string specifying test type for categorical variables.
#' @param total Logical or character controlling total column display.
#' @param total_label Character string label for total column.
#' @param labels Named character vector of variable labels.
#' @param na_percent Logical whether to include NA in percentage denominators.
#' @param p_per_stat Logical whether to show separate \emph{p}-values per statistic for
#'   continuous variables. Default \code{FALSE} for better performance.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @param ... Additional arguments passed to test functions.
#' @return List with 'formatted' and 'raw' data.table components.
#' @keywords internal
process_variable <- function(data, var, group_var = NULL, 
                             stats_continuous, stats_categorical,
                             digits, conf_level = 0.95, na_include, na_label,
                             test, test_continuous, test_categorical,
                             total, total_label, labels, na_percent, 
                             p_per_stat = FALSE, marks = NULL, ...) {
    
    ## Get variable label
    var_label <- if (!is.null(labels) && var %chin% names(labels)) {
                     labels[var]
                 } else {
                     var
                 }
    
    ## Determine variable type and process accordingly
    if (grepl("^Surv\\(", var)) {
        return(process_survival(
            data = data,
            var = var,
            var_label = var_label,
            group_var = group_var,
            digits = digits,
            conf_level = conf_level,
            na_include = na_include,
            na_label = na_label,
            test = test,
            total = total,
            total_label = total_label,
            marks = marks,
            ...
        ))
    } else if (is.numeric(data[[var]])) {
        return(process_continuous(
            data = data,
            var = var,
            var_label = var_label,
            group_var = group_var,
            stats = stats_continuous,
            digits = digits,
            na_include = na_include,
            na_label = na_label,
            test = test,
            test_type = test_continuous,
            total = total,
            total_label = total_label,
            p_per_stat = p_per_stat,
            marks = marks,
            ...
        ))
    } else {
        return(process_categorical(
            data = data,
            var = var,
            var_label = var_label,
            group_var = group_var,
            stats = stats_categorical,
            na_include = na_include,
            na_label = na_label,
            test = test,
            test_type = test_categorical,
            total = total,
            total_label = total_label,
            na_percent = na_percent,
            marks = marks,
            ...
        ))
    }
}


### * Continuous variables

#' Process continuous variable
#' 
#' Calculates descriptive statistics for continuous numeric variables, with
#' optional grouping and statistical testing. Supports multiple summary
#' statistics (mean \eqn{\pm} SD, median [IQR], range) and various hypothesis tests.
#' 
#' @param data Data.table containing the variable.
#' @param var Character string naming the variable to process.
#' @param var_label Character string label for display.
#' @param group_var Optional character string naming the grouping variable.
#' @param stats Character vector of statistics to calculate.
#' @param digits Integer number of decimal places.
#' @param na_include Logical whether to include missing values.
#' @param na_label Character string label for missing values.
#' @param test Logical whether to perform statistical tests.
#' @param test_type Character string specifying test type.
#' @param total Logical or character controlling total column display.
#' @param total_label Character string label for total column.
#' @param p_per_stat Logical. If TRUE, calculate separate \emph{p}-values for each 
#'   statistic type (\emph{e.g.,} t-test for means, Wilcoxon for medians). If \code{FALSE}
#'   (default), calculate a single \emph{p}-value based on the first statistic type.
#' @param ... Additional arguments passed to test functions.
#' @return List with 'formatted' and 'raw' data.table components.
#' @keywords internal
process_continuous <- function(data, var, var_label, group_var, stats, digits,
                               na_include, na_label, test, test_type,
                               total, total_label, p_per_stat = FALSE,
                               marks = NULL, ...) {
    
    ## Pre-extract the variable column
    var_vec <- data[[var]]
    not_na <- !is.na(var_vec)
    total_vals <- var_vec[not_na]
    total_n <- length(total_vals)
    
    ## Calculate total statistics
    if (total_n > 0) {
        total_fivenum <- fivenum(total_vals)
        total_stats <- list(
            mean = mean(total_vals),
            sd = sd(total_vals),
            median = total_fivenum[3],
            q1 = total_fivenum[2],
            q3 = total_fivenum[4],
            min = total_fivenum[1],
            max = total_fivenum[5],
            n = total_n
        )
    } else {
        total_stats <- list(mean = NA_real_, sd = NA_real_, median = NA_real_,
                            q1 = NA_real_, q3 = NA_real_, min = NA_real_, 
                            max = NA_real_, n = 0L)
    }
    
    ## Calculate group statistics (if grouped)
    group_stats_list <- NULL
    groups <- NULL
    
    if (!is.null(group_var)) {
        grp_vec <- data[[group_var]]
        if (is.factor(grp_vec)) {
            groups <- levels(grp_vec)
        } else {
            groups <- unique(grp_vec)
            groups <- groups[!is.na(groups)]
        }
        
        ## Compute statistics for each group
        group_stats_list <- lapply(groups, function(g) {
            idx <- which(grp_vec == g & not_na)
            if (length(idx) == 0) {
                return(list(mean = NA_real_, sd = NA_real_, median = NA_real_,
                           q1 = NA_real_, q3 = NA_real_, min = NA_real_, 
                           max = NA_real_, n = 0L))
            }
            vals <- var_vec[idx]
            fn <- fivenum(vals)
            list(
                mean = mean(vals),
                sd = sd(vals),
                median = fn[3],
                q1 = fn[2],
                q3 = fn[4],
                min = fn[1],
                max = fn[5],
                n = length(vals)
            )
        })
        names(group_stats_list) <- as.character(groups)
    }
    
    ## Calculate p-values
    p_values <- list()
    p_value_single <- NULL
    if (test && !is.null(group_var) && length(groups) >= 2) {
        grp_vec <- data[[group_var]]
        
        if (p_per_stat) {
            ## Calculate separate p-value for each statistic type
            for (stat_type in stats) {
                if (stat_type == "range") {
                    p_values[[stat_type]] <- NULL
                } else {
                    p_values[[stat_type]] <- perform_continuous_test(
                        var_vec, grp_vec, test_type, stat_type)
                }
            }
        } else {
            ## Calculate single p-value based on first non-range statistic
            first_stat <- stats[stats != "range"][1]
            if (is.na(first_stat)) first_stat <- "median_iqr"
            p_value_single <- perform_continuous_test(
                var_vec, grp_vec, test_type, first_stat)
        }
    }
    
    ## Build output
    n_stats <- length(stats)
    has_missing <- na_include && any(!not_na)
    n_rows <- n_stats + if (has_missing) 1L else 0L
    
    ## Compute format string
    fmt_str <- paste0("%.", digits, "f")
    
    formatted_list <- vector("list", n_rows)
    raw_list <- vector("list", n_rows)
    
    for (i in seq_along(stats)) {
        stat_type <- stats[i]
        
        formatted_row <- list(
            variable = if (i == 1L) var_label else "",
            level = get_stat_label(stat_type)
        )
        
        raw_row <- list(
            variable = if (i == 1L) var_label else "",
            level = stat_type,
            stat_type = stat_type
        )
        
        ## Add total column
        if (!isFALSE(total)) {
            formatted_row[[total_label]] <- format_continuous_stat(
                total_stats, stat_type, fmt_str, marks)
            raw_row <- add_raw_stats(raw_row, total_label, total_stats, stat_type)
        }
        
        ## Add group columns
        if (!is.null(group_var)) {
            for (g in groups) {
                grp_col <- as.character(g)
                grp_stats <- group_stats_list[[grp_col]]
                formatted_row[[grp_col]] <- format_continuous_stat(
                    grp_stats, stat_type, fmt_str, marks)
                raw_row <- add_raw_stats(raw_row, grp_col, grp_stats, stat_type)
            }
            
            ## Add p-value
            if (test && stat_type != "range") {
                if (p_per_stat && !is.null(p_values[[stat_type]])) {
                    ## Per-statistic p-values
                    formatted_row[["p_value"]] <- p_values[[stat_type]]
                    raw_row[["p_value"]] <- p_values[[stat_type]]
                } else if (!p_per_stat && i == 1L && !is.null(p_value_single)) {
                    ## Single p-value on first row only
                    formatted_row[["p_value"]] <- p_value_single
                    raw_row[["p_value"]] <- p_value_single
                }
            }
        }
        
        formatted_list[[i]] <- data.table::as.data.table(formatted_row)
        raw_list[[i]] <- data.table::as.data.table(raw_row)
    }
    
    ## Missing row
    if (has_missing) {
        n_miss_total <- sum(!not_na)
        miss_formatted <- list(variable = "", level = na_label)
        miss_raw <- list(variable = "", level = na_label, stat_type = "missing")
        
        if (!isFALSE(total)) {
            miss_formatted[[total_label]] <- format_count(n_miss_total, marks)
            miss_raw[[total_label]] <- n_miss_total
        }
        
        if (!is.null(group_var)) {
            grp_vec <- data[[group_var]]
            for (g in groups) {
                grp_col <- as.character(g)
                n_miss <- sum(!not_na & grp_vec == g, na.rm = TRUE)
                miss_formatted[[grp_col]] <- format_count(n_miss, marks)
                miss_raw[[grp_col]] <- n_miss
            }
        }
        
        formatted_list[[n_rows]] <- data.table::as.data.table(miss_formatted)
        raw_list[[n_rows]] <- data.table::as.data.table(miss_raw)
    }
    
    list(
        formatted = data.table::rbindlist(formatted_list, fill = TRUE),
        raw = data.table::rbindlist(raw_list, fill = TRUE)
    )
}


#' Add raw statistics to row
#' 
#' Appends raw numeric statistics to a data.table row for downstream processing.
#' Used to preserve underlying values alongside formatted display strings.
#' 
#' @param row Data.table row to modify.
#' @param col Character string column name for the statistics.
#' @param stats Named list of numeric statistics (mean, sd, median, \emph{etc.}).
#' @param stat_type Character string indicating which statistic is displayed.
#' @return Modified row (by reference).
#' @keywords internal
add_raw_stats <- function(row, col, stats, stat_type) {
    if (stat_type == "mean_sd") {
        row[[col]] <- stats$mean
        row[[paste0(col, "_sd")]] <- stats$sd
        row[[paste0(col, "_n")]] <- stats$n
    } else if (stat_type == "median_iqr") {
        row[[col]] <- stats$median
        row[[paste0(col, "_q1")]] <- stats$q1
        row[[paste0(col, "_q3")]] <- stats$q3
        row[[paste0(col, "_n")]] <- stats$n
    } else if (stat_type == "median_range") {
        row[[col]] <- stats$median
        row[[paste0(col, "_min")]] <- stats$min
        row[[paste0(col, "_max")]] <- stats$max
        row[[paste0(col, "_n")]] <- stats$n
    } else if (stat_type == "range") {
        row[[col]] <- stats$min
        row[[paste0(col, "_max")]] <- stats$max
        row[[paste0(col, "_n")]] <- stats$n
    }
    row  # Return the modified row
}


#' Perform statistical tests for continuous variables
#' 
#' Conducts hypothesis tests comparing continuous variables across groups.
#' Supports t-tests, Wilcoxon tests, ANOVA, and Kruskal-Wallis tests with
#' automatic selection based on number of groups.
#' 
#' @param var_vec Numeric vector of the continuous variable.
#' @param grp_vec Factor or character vector defining groups.
#' @param test_type Character string: "parametric", "nonparametric", or "auto".
#' @param stat_type Character string indicating primary statistic being tested.
#' @return Numeric \emph{p}-value from the hypothesis test.
#' @keywords internal
perform_continuous_test <- function(var_vec, grp_vec, test_type, stat_type) {
    ## Remove NAs
    valid <- !is.na(var_vec) & !is.na(grp_vec)
    if (sum(valid) < 3) return(NA_real_)
    
    x <- var_vec[valid]
    g <- grp_vec[valid]
    
    groups <- unique(g)
    n_groups <- length(groups)
    if (n_groups < 2) return(NA_real_)
    
    ## Auto-select test
    if (test_type == "auto") {
        if (grepl("mean", stat_type)) {
            test_type <- if (n_groups == 2) "t" else "aov"
        } else {
            test_type <- if (n_groups == 2) "wrs" else "kwt"
        }
    }
    
    tryCatch({
        switch(test_type,
            "t" = t.test(x ~ g)$p.value,
            "wrs" = wilcox.test(x ~ g)$p.value,
            "aov" = {
                fit <- aov(x ~ g)
                summary(fit)[[1]][["Pr(>F)"]][1]
            },
            "kwt" = kruskal.test(x ~ g)$p.value,
            NA_real_
        )
    }, error = function(e) NA_real_)
}


#' Format continuous statistic for display
#' 
#' Converts numeric summary statistics into formatted display strings following
#' standard conventions (mean \eqn{\pm} SD, median [IQR], range, \emph{etc.}).
#' 
#' @param stats Named list of numeric statistics (mean, sd, median, q1, q3, min, max).
#' @param stat_type Character string: "mean_sd", "median_iqr", "median_range", or "range".
#' @param fmt_str Character string format specification for sprintf.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character string with formatted statistic.
#' @keywords internal
format_continuous_stat <- function(stats, stat_type, fmt_str, marks) {
    if (stats$n == 0) return("")
    
    switch(stat_type,
        "mean_sd" = paste0(
            format_num(stats$mean, fmt_str, marks), " \u00B1 ",
            format_num(stats$sd, fmt_str, marks)
        ),
        "median_iqr" = {
            sep <- resolve_separator(stats$q1, stats$q3, marks)
            paste0(
                format_num(stats$median, fmt_str, marks), " [",
                format_num(stats$q1, fmt_str, marks), sep,
                format_num(stats$q3, fmt_str, marks), "]"
            )
        },
        "median_range" = {
            sep <- resolve_separator(stats$min, stats$max, marks)
            paste0(
                format_num(stats$median, fmt_str, marks), " (",
                format_num(stats$min, fmt_str, marks), sep,
                format_num(stats$max, fmt_str, marks), ")"
            )
        },
        "range" = {
            sep <- resolve_separator(stats$min, stats$max, marks)
            paste0(format_num(stats$min, fmt_str, marks), sep, 
                   format_num(stats$max, fmt_str, marks))
        },
        ""
    )
}


#' Get display label for statistic type
#' 
#' Converts internal statistic type codes to formatted display labels for
#' table column headers.
#' 
#' @param stat_type Character string: "mean_sd", "median_iqr", "median_range",
#'   "range", "n_miss", or custom type.
#' @return Character string with formatted label (\emph{e.g.,} "Mean \eqn{\pm} SD").
#' @keywords internal
get_stat_label <- function(stat_type) {
    switch(stat_type,
           "mean_sd" = "Mean \u00B1 SD",
           "median_iqr" = "Median [IQR]",
           "median_range" = "Median (Range)",
           "range" = "Range",
           "n_miss" = "Missing",
           stat_type
    )
}


### * Categorical variables

#' Process categorical variable
#' 
#' Calculates frequency and percentage statistics for categorical variables,
#' with optional grouping and chi-square/Fisher's exact testing. Handles
#' factor levels, missing values, and custom labeling.
#' 
#' @param data Data.table containing the variable.
#' @param var Character string naming the variable to process.
#' @param var_label Character string label for display.
#' @param group_var Optional character string naming the grouping variable.
#' @param stats Character vector of statistics to calculate.
#' @param na_include Logical whether to include missing values as a category.
#' @param na_label Character string label for missing values.
#' @param test Logical whether to perform statistical tests.
#' @param test_type Character string specifying test type.
#' @param total Logical or character controlling total column display.
#' @param total_label Character string label for total column.
#' @param na_percent Logical whether to include NA in percentage denominators.
#' @param ... Additional arguments passed to test functions.
#' @return List with 'formatted' and 'raw' data.table components.
#' @keywords internal
process_categorical <- function(data, var, var_label, group_var, stats,
                                na_include, na_label, test, test_type,
                                total, total_label, na_percent,
                                marks = NULL, ...) {
    
    ## Pre-extract vectors
    var_vec <- data[[var]]
    
    ## Get levels
    if (is.factor(var_vec)) {
        levels_to_show <- levels(var_vec)
    } else {
        levels_to_show <- unique(var_vec)
        levels_to_show <- levels_to_show[!is.na(levels_to_show)]
    }
    
    ## Add NA level if requested
    has_na <- anyNA(var_vec)
    if (na_include && has_na) {
        levels_to_show <- c(levels_to_show, NA)
    }
    
    n_levels <- length(levels_to_show)
    
    ## Pre-compute all counts with single table() call
    use_na <- if (na_include) "ifany" else "no"
    
    if (!is.null(group_var)) {
        grp_vec <- data[[group_var]]
        
        if (is.factor(grp_vec)) {
            groups <- levels(grp_vec)
        } else {
            groups <- unique(grp_vec)
            groups <- groups[!is.na(groups)]
        }
        
        ## Single table call for cross-tabulation
        tab <- table(var_vec, grp_vec, useNA = use_na)
        total_tab <- rowSums(tab)
        grp_totals <- colSums(tab)
        
        ## Calculate denominators
        if (na_percent) {
            total_denom <- sum(tab)
            grp_denoms <- grp_totals
        } else {
            ## Exclude NA row from denominators
            non_na_rows <- !is.na(rownames(tab))
            total_denom <- sum(tab[non_na_rows, , drop = FALSE])
            grp_denoms <- colSums(tab[non_na_rows, , drop = FALSE])
        }
        
        ## Calculate p-value once
        p_value <- if (test) {
            perform_categorical_test(tab, test_type)
                   } else {
            NULL
        }
        
        ## Build output
        formatted_list <- vector("list", n_levels)
        raw_list <- vector("list", n_levels)
        
        for (i in seq_along(levels_to_show)) {
            lvl <- levels_to_show[i]
            lvl_label <- if (is.na(lvl)) na_label else as.character(lvl)
            lvl_char <- if (is.na(lvl)) NA_character_ else as.character(lvl)
            
            level_formatted <- list(
                variable = if (i == 1L) var_label else "",
                level = lvl_label
            )
            
            level_raw <- list(
                variable = if (i == 1L) var_label else "",
                level = lvl_label,
                stat_type = "category"
            )
            
            ## Get row from table
            if (is.na(lvl)) {
                tab_row <- tab[is.na(rownames(tab)), , drop = FALSE]
                n_total <- sum(tab_row)
            } else {
                n_total <- total_tab[lvl_char]
                if (is.na(n_total)) n_total <- 0L
            }
            
            ## Add total column
            if (!isFALSE(total)) {
                ## NA row: use total denom if na_percent, otherwise count only
                if (is.na(lvl)) {
                    denom <- if (na_percent) total_denom else sum(tab)
                    na_stat <- if (na_percent) stats else "n"
                } else {
                    denom <- total_denom
                    na_stat <- stats
                }
                level_formatted[[total_label]] <- format_categorical_stat(
                    as.integer(n_total), as.integer(denom), na_stat, marks)
                level_raw[[total_label]] <- as.integer(n_total)
                level_raw[[paste0(total_label, "_total")]] <- as.integer(denom)
            }
            
            ## Add group columns
            for (g in groups) {
                grp_col <- as.character(g)
                
                if (is.na(lvl)) {
                    n <- sum(tab[is.na(rownames(tab)), grp_col])
                    denom <- if (na_percent) grp_denoms[grp_col] else grp_totals[grp_col]
                    na_stat <- if (na_percent) stats else "n"
                } else {
                    n <- tab[lvl_char, grp_col]
                    if (is.na(n)) n <- 0L
                    denom <- grp_denoms[grp_col]
                    na_stat <- stats
                }
                
                level_formatted[[grp_col]] <- format_categorical_stat(
                    as.integer(n), as.integer(denom), na_stat, marks)
                level_raw[[grp_col]] <- as.integer(n)
                level_raw[[paste0(grp_col, "_total")]] <- as.integer(denom)
            }
            
            ## Add p-value to first level only
            if (i == 1L && !is.null(p_value)) {
                level_formatted[["p_value"]] <- p_value
                level_raw[["p_value"]] <- p_value
            }
            
            formatted_list[[i]] <- data.table::as.data.table(level_formatted)
            raw_list[[i]] <- data.table::as.data.table(level_raw)
        }
        
    } else {
        ## Ungrouped version
        total_tab <- table(var_vec, useNA = use_na)
        total_denom <- if (na_percent) sum(total_tab) else sum(total_tab[!is.na(names(total_tab))])
        
        formatted_list <- vector("list", n_levels)
        raw_list <- vector("list", n_levels)
        
        for (i in seq_along(levels_to_show)) {
            lvl <- levels_to_show[i]
            lvl_label <- if (is.na(lvl)) na_label else as.character(lvl)
            
            level_formatted <- list(
                variable = if (i == 1L) var_label else "",
                level = lvl_label
            )
            
            level_raw <- list(
                variable = if (i == 1L) var_label else "",
                level = lvl_label,
                stat_type = "category"
            )
            
            if (!isFALSE(total)) {
                if (is.na(lvl)) {
                    n <- sum(is.na(var_vec))
                    na_stat <- if (na_percent) stats else "n"
                } else {
                    n <- total_tab[as.character(lvl)]
                    if (is.na(n)) n <- 0L
                    na_stat <- stats
                }
                
                level_formatted[[total_label]] <- format_categorical_stat(
                    as.integer(n), as.integer(total_denom), na_stat, marks)
                level_raw[[total_label]] <- as.integer(n)
                level_raw[[paste0(total_label, "_total")]] <- as.integer(total_denom)
            }
            
            formatted_list[[i]] <- data.table::as.data.table(level_formatted)
            raw_list[[i]] <- data.table::as.data.table(level_raw)
        }
    }
    
    list(
        formatted = data.table::rbindlist(formatted_list, fill = TRUE),
        raw = data.table::rbindlist(raw_list, fill = TRUE)
    )
}


#' Perform statistical tests for categorical variables
#' 
#' Conducts chi-square or Fisher's exact tests for categorical variables
#' across groups. Automatically selects Fisher's exact test for small
#' expected frequencies.
#' 
#' @param tab Contingency table (matrix or table object).
#' @param test_type Character string: "chisq" for chi-square, "fisher" for
#'   Fisher's exact, or "auto" for automatic selection.
#' @return Numeric \emph{p}-value from the hypothesis test.
#' @keywords internal
perform_categorical_test <- function(tab, test_type) {
    ## Remove NA rows for testing
    non_na_rows <- !is.na(rownames(tab))
    tab_test <- tab[non_na_rows, , drop = FALSE]
    
    if (any(dim(tab_test) < 2)) return(NA_real_)
    
    ## Auto-select test
    if (test_type == "auto") {
        expected <- suppressWarnings(chisq.test(tab_test)$expected)
        test_type <- if (any(expected < 5)) "fisher" else "chisq"
    }
    
    tryCatch({
        switch(test_type,
            "fisher" = ,
            "fisher.test" = fisher.test(tab_test, workspace = 2e5)$p.value,
            "chisq" = ,
            "chisq.test" = chisq.test(tab_test)$p.value,
            NA_real_
        )
    }, error = function(e) NA_real_)
}


### * Time-to-event variables

#' Process time-to-event variable
#' 
#' Calculates survival statistics including median survival times with
#' confidence intervals, with optional grouping and log-rank testing. Parses
#' Surv() expressions and uses \pkg{survival} package functions.
#' 
#' @param data Data.table containing the survival variables.
#' @param var Character string with Surv() expression (\emph{e.g.,} "Surv(time, status)").
#' @param var_label Character string label for display.
#' @param group_var Optional character string naming the grouping variable.
#' @param digits Integer number of decimal places.
#' @param conf_level Numeric confidence level for confidence intervals.
#' @param na_include Logical whether to include missing values.
#' @param na_label Character string label for missing values.
#' @param test Logical whether to perform log-rank test.
#' @param total Logical or character controlling total column display.
#' @param total_label Character string label for total column.
#' @param ... Additional arguments (currently unused).
#' @return List with 'formatted' and 'raw' data.table components.
#' @keywords internal
process_survival <- function(data, var, var_label, group_var, digits,
                             conf_level = 0.95, na_include, na_label, 
                             test, total, total_label, marks = NULL, ...) {
    
    ## Parse Surv() expression
    surv_match <- regexec("Surv\\(([^,]+),\\s*([^)]+)\\)", var)
    surv_parts <- regmatches(var, surv_match)[[1]]
    
    if (length(surv_parts) < 3) {
        stop("Invalid Surv() syntax: ", var)
    }
    
    time_var <- trimws(surv_parts[2])
    status_var <- trimws(surv_parts[3])
    
    if (!requireNamespace("survival", quietly = TRUE)) {
        stop("Package 'survival' required for survival analysis")
    }
    
    ## Build CI label based on conf_level
    ci_pct <- round(conf_level * 100)
    ci_label <- paste0("Median (", ci_pct, "% CI)")
    
    ## Build CI column name suffixes for extracting from survfit summary
    ## survfit names CI columns as e.g. "0.95LCL" and "0.95UCL" using conf.int value
    ci_lower_name <- paste0(format(conf_level, nsmall = 2), "LCL")
    ci_upper_name <- paste0(format(conf_level, nsmall = 2), "UCL")
    
    fmt_str <- paste0("%.", digits, "f")
    
    if (!is.null(group_var)) {
        grp_vec <- data[[group_var]]
        if (is.factor(grp_vec)) {
            groups <- levels(grp_vec)
        } else {
            groups <- unique(grp_vec)
            groups <- groups[!is.na(groups)]
        }
        
        ## Calculate p-value (log-rank test)
        p_value <- if (test) {
            tryCatch({
                surv_obj <- survival::Surv(data[[time_var]], data[[status_var]])
                survival::survdiff(surv_obj ~ grp_vec)$pvalue
            }, error = function(e) NA_real_)
        } else {
            NULL
        }
        
        formatted_row <- list(
            variable = var_label,
            level = ci_label
        )
        
        raw_row <- list(
            variable = var_label,
            level = "median",
            stat_type = "survival"
        )
        
        ## Add total column
        if (!isFALSE(total)) {
            surv_obj <- survival::Surv(data[[time_var]], data[[status_var]])
            fit <- survival::survfit(surv_obj ~ 1, conf.int = conf_level)
            table <- summary(fit)$table
            
            formatted_row[[total_label]] <- format_survival_ci(
                table["median"], table[ci_lower_name], table[ci_upper_name],
                fmt_str, marks
            )
            
            raw_row[[total_label]] <- table["median"]
            raw_row[[paste0(total_label, "_ci_lower")]] <- table[ci_lower_name]
            raw_row[[paste0(total_label, "_ci_upper")]] <- table[ci_upper_name]
        }
        
        ## Add group columns
        for (g in groups) {
            grp_col <- as.character(g)
            grp_idx <- which(grp_vec == g)
            grp_time <- data[[time_var]][grp_idx]
            grp_status <- data[[status_var]][grp_idx]
            
            surv_obj <- survival::Surv(grp_time, grp_status)
            fit <- survival::survfit(surv_obj ~ 1, conf.int = conf_level)
            table <- summary(fit)$table
            
            formatted_row[[grp_col]] <- format_survival_ci(
                table["median"], table[ci_lower_name], table[ci_upper_name],
                fmt_str, marks
            )
            
            raw_row[[grp_col]] <- table["median"]
            raw_row[[paste0(grp_col, "_ci_lower")]] <- table[ci_lower_name]
            raw_row[[paste0(grp_col, "_ci_upper")]] <- table[ci_upper_name]
        }
        
        if (test && !is.null(p_value)) {
            formatted_row[["p_value"]] <- p_value
            raw_row[["p_value"]] <- p_value
        }
        
        formatted_result <- data.table::as.data.table(formatted_row)
        raw_result <- data.table::as.data.table(raw_row)
        
    } else {
        formatted_row <- list(
            variable = var_label,
            level = ci_label
        )
        
        raw_row <- list(
            variable = var_label,
            level = "median",
            stat_type = "survival"
        )
        
        if (!isFALSE(total)) {
            surv_obj <- survival::Surv(data[[time_var]], data[[status_var]])
            fit <- survival::survfit(surv_obj ~ 1, conf.int = conf_level)
            table <- summary(fit)$table
            
            formatted_row[[total_label]] <- format_survival_ci(
                table["median"], table[ci_lower_name], table[ci_upper_name],
                fmt_str, marks
            )
            
            raw_row[[total_label]] <- table["median"]
            raw_row[[paste0(total_label, "_ci_lower")]] <- table[ci_lower_name]
            raw_row[[paste0(total_label, "_ci_upper")]] <- table[ci_upper_name]
        }
        
        formatted_result <- data.table::as.data.table(formatted_row)
        raw_result <- data.table::as.data.table(raw_row)
    }
    
    list(formatted = formatted_result, raw = raw_result)
}


### * Table formatting

#' Format \emph{p}-values for descriptive tables
#' 
#' Converts numeric p-values to formatted strings with appropriate precision.
#' Handles very small p-values with threshold notation (\emph{e.g.,} "< 0.001").
#' 
#' @param result Data.table with 'p_value' column to format.
#' @param p_digits Integer number of decimal places for p-values.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Modified data.table with 'p_value' column (formatted strings).
#' @keywords internal
format_pvalues_desctable <- function(result, p_digits, marks) {
    if ("p_value" %chin% names(result)) {
        threshold <- 10^(-p_digits)
        threshold_str <- paste0("< 0", marks$decimal.mark,
                                strrep("0", p_digits - 1), "1")
        fmt_str <- paste0("%.", p_digits, "f")
        
        ## Format p-values vectorized
        p_str <- sprintf(fmt_str, result$p_value)
        if (marks$decimal.mark != ".") {
            p_str <- sub(".", marks$decimal.mark, p_str, fixed = TRUE)
        }
        
        result[, p_formatted := data.table::fifelse(
            is.na(p_value) | Variable == "N", 
            "",
            data.table::fifelse(
                p_value < threshold,
                threshold_str,
                p_str
            )
        )]
        
        ## Remove old column, rename new
        result[, p_value := NULL]
        data.table::setnames(result, "p_formatted", "p-value")
    }
    result
}


#' Reorder columns to position total column
#' 
#' Rearranges data.table columns to place the total column in the specified
#' position (first, last, or default). Ensures proper ordering of Variable,
#' Group, total, group columns, and \emph{p}-value.
#' 
#' @param result Data.table with columns to reorder.
#' @param total Logical or character: \code{TRUE}/"first" (total first), "last" (total last).
#' @param total_label Character string name of the total column.
#' @return Modified data.table with reordered columns.
#' @keywords internal
reorder_total_column <- function(result, total, total_label) {
    if (total_label %chin% names(result)) {
        cols <- names(result)
        
        base_cols <- c("Variable", "Group")
        group_cols <- cols[!cols %chin% c(base_cols, total_label, "p-value")]
        p_col <- if ("p-value" %chin% cols) "p-value" else NULL
        
        if (isTRUE(total) || identical(total, "first")) {
            new_order <- c(base_cols, total_label, group_cols, p_col)
        } else if (identical(total, "last")) {
            new_order <- c(base_cols, group_cols, total_label, p_col)
        } else {
            new_order <- c(base_cols, total_label, group_cols, p_col)
        }
        
        new_order <- new_order[new_order %chin% cols]
        data.table::setcolorder(result, new_order)
    }
    result
}

### * Main function

#' Export Table to PDF Format
#'
#' Converts a data frame, data.table, or matrix to a professionally formatted PDF 
#' document using LaTeX as an intermediate format. Provides extensive control over 
#' page layout, typography, and formatting for publication-ready output. Particularly 
#' well-suited for tables from regression analyses, descriptive statistics, and 
#' model comparisons. Requires \pkg{xtable} for export.
#'
#' @param table Data frame, data.table, or matrix to export. Can be output from 
#'   \code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
#'   \code{fullfit()}, \code{compfit()}, or any tabular data structure.
#'   
#' @param file Character string specifying the output PDF filename. Must have 
#'   \code{.pdf} extension. Example: \code{"results.pdf"}, \code{"Table1.pdf"}.
#'   
#' @param orientation Character string specifying page orientation: 
#'   \itemize{
#'     \item \code{"portrait"} - Vertical orientation [default]
#'     \item \code{"landscape"} - Horizontal orientation (recommended for wide tables)
#'   }
#'   
#' @param paper Character string specifying paper size:
#'   \itemize{
#'     \item \code{"letter"} - US Letter (8.5" x 11") [default]
#'     \item \code{"a4"} - A4 (210 mm x 297 mm)
#'     \item \code{"auto"} - Auto-size to content (no margins, crops to fit)
#'   }
#'   
#' @param margins Numeric vector of length 4 specifying margins in inches as 
#'   \code{c(top, right, bottom, left)}. Default is \code{c(1, 1, 1, 1)}. 
#'   Ignored when \code{paper = "auto"}.
#'   
#' @param fit_to_page Logical. If \code{TRUE}, scales table to fit within the 
#'   text width (respects margins). Useful for wide tables that would otherwise 
#'   overflow. Default is \code{TRUE}.
#'   
#' @param font_size Numeric. Base font size in points. Default is 8. Smaller 
#'   values accommodate more content; larger values improve readability. 
#'   Typical range: 6-12 points.
#'   
#' @param caption Character string. Optional caption displayed below the table. 
#'   Supports LaTeX formatting for multi-line captions, superscripts, italics, \emph{etc.} 
#'   See Details for formatting guidance. Default is \code{NULL}.
#'
#' @param caption_size Numeric. Caption font size in points. If \code{NULL} (default), 
#'   uses the base \code{font_size}. Set to a specific value (\emph{e.g.,} 6, 7, 8) to 
#'   control caption size independently of table font size. Useful for fitting 
#'   captions on constrained page sizes. Typical range: 6-10 points.
#' 
#' @param format_headers Logical. If \code{TRUE}, applies automatic formatting 
#'   to column headers: converts underscores to spaces, italicizes statistical 
#'   notation ("\emph{n}", "\emph{p}"), and improves readability. Default is \code{TRUE}.
#'   
#' @param variable_padding Logical. If \code{TRUE}, adds vertical spacing between 
#'   different variables in the table, creating visual grouping. Particularly 
#'   useful for regression tables with multiple predictors. Default is \code{FALSE}.
#'   
#' @param cell_padding Character string or numeric specifying vertical padding 
#'   within table cells:
#'   \itemize{
#'     \item \code{"none"} - No extra padding (most compact)
#'     \item \code{"normal"} - Standard padding [default]
#'     \item \code{"relaxed"} - Increased padding
#'     \item \code{"loose"} - Maximum padding
#'     \item Numeric value - Custom multiplier (\emph{e.g.,} \code{1.5})
#'   }
#'   Adjusts \code{\\arraystretch} in LaTeX.
#'   
#' @param bold_significant Logical. If \code{TRUE}, applies bold formatting to 
#'   p-values below the significance threshold, making significant results stand 
#'   out visually. Default is \code{TRUE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. Default is \code{FALSE}.
#'   
#' @param p_threshold Numeric. Threshold for bold \emph{p}-value formatting. Only 
#'   used when \code{bold_significant = TRUE}. Default is 0.05.
#'   
#' @param align Character string or vector specifying column alignment. Options:
#'   \itemize{
#'     \item \code{"l"} - Left aligned
#'     \item \code{"c"} - Center aligned
#'     \item \code{"r"} - Right aligned
#'   }
#'   If \code{NULL} (default), automatically determines alignment based on content 
#'   (text left, numbers right). Can specify per-column: \code{c("l", "l", "r", "r")}.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels/groups 
#'   under their parent variable using horizontal space, creating a hierarchical 
#'   display. Useful for factor variables in regression tables. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses the table by:
#'   \itemize{
#'     \item Showing only one row per continuous variable (estimate + CI)
#'     \item Showing only non-reference categories for binary factors
#'     \item Automatically setting \code{indent_groups = TRUE}
#'   }
#'   Significantly reduces table height. Default is \code{FALSE}.
#'   
#' @param condense_quantitative Logical. If \code{TRUE}, condenses continuous 
#'   and survival variables into single rows while preserving all categorical 
#'   variable rows (including binary). Only applies to descriptive tables from 
#'   \code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
#'   \code{condense_table}, this does not collapse binary categorical variables. 
#'   Default is \code{FALSE}.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating gray 
#'   background shading to different variables (not individual rows) for improved 
#'   visual grouping and readability. Default is \code{FALSE}.
#'   
#' @param stripe_color Character string. LaTeX color specification for zebra 
#'   stripes. Default is \code{"gray!20"} (20\% gray). Can use other colors like 
#'   \code{"blue!10"}, \code{"red!15"}. Requires \code{zebra_stripes = TRUE}.
#'   
#' @param dark_header Logical. If \code{TRUE}, creates a dark (black) background 
#'   with white text for the header row. Provides strong visual contrast. 
#'   Default is \code{FALSE}.
#'   
#' @param show_logs Logical. If \code{TRUE}, retains LaTeX log and auxiliary 
#'   files after PDF compilation for troubleshooting. If \code{FALSE}, deletes 
#'   these files. Default is \code{FALSE}.
#'   
#' @param ... Additional arguments passed to \code{\link[xtable]{xtable}} for 
#'   advanced LaTeX table customization.
#'
#' @param quiet Logical. Suppress progress and confirmation messages, such
#'   as the notice reporting the file written. Errors and warnings are
#'   unaffected. Default is \code{FALSE}.
#'
#' @return Invisibly returns the file path, matching \code{tablesave()} and
#'   \code{forestsave()}. Called for its side effect of creating a PDF file at
#'   the specified location. If compilation fails, check the \code{.log} file
#'   (if \code{show_logs = TRUE}) for error details.
#'
#' @details
#' \strong{LaTeX Requirements:}
#' 
#' This function requires a working LaTeX installation. The function checks for 
#' LaTeX availability and provides installation guidance if missing.
#' 
#' \emph{Recommended LaTeX distributions:}
#' \itemize{
#'   \item \strong{TinyTeX} (lightweight, R-integrated): Install via 
#'     \code{tinytex::install_tinytex()}
#'   \item \strong{TeX Live} (comprehensive, cross-platform)
#'   \item \strong{MiKTeX} (Windows)
#'   \item \strong{MacTeX} (macOS)
#' }
#' 
#' \emph{Required LaTeX packages} (auto-installed with most distributions):
#' \itemize{
#'   \item \code{fontenc}, \code{inputenc} - Character encoding
#'   \item \code{array}, \code{booktabs}, \code{longtable} - Table formatting
#'   \item \code{graphicx} - Scaling tables
#'   \item \code{geometry} - Page layout
#'   \item \code{pdflscape}, \code{lscape} - Landscape orientation
#'   \item \code{helvet} - Sans-serif fonts
#'   \item \code{standalone}, \code{varwidth} - Auto-sizing (for \code{paper = "auto"})
#'   \item \code{float}, \code{caption} - Floats and captions
#'   \item \code{xcolor}, \code{colortable} - Colors (for \code{zebra_stripes} or \code{dark_header})
#' }
#' 
#' \strong{Caption Formatting:}
#' 
#' Captions support LaTeX commands for rich formatting:
#' \preformatted{
#' # Multi-line caption with line breaks
#' caption = "Table 1: Multivariable Analysis\\\\
#'           OR = odds ratio; CI = confidence interval"
#' 
#' # With superscripts (using LaTeX syntax)
#' caption = "Table 1: Results\\\\
#'           Adjusted for age and sex\\\\
#'           p-values from Wald tests"
#' 
#' # With special characters (must escape percent signs)
#' caption = "Results for income (in thousands)"
#' }
#' 
#' \strong{Auto-Sizing (\code{paper = "auto"}):}
#' 
#' When \code{paper = "auto"}, the function attempts to create a minimal PDF 
#' sized exactly to the table content:
#' \enumerate{
#'   \item Using the \code{standalone} LaTeX class (cleanest output)
#'   \item Fallback to \code{pdfcrop} utility if standalone unavailable
#'   \item Fallback to minimal margins if neither available
#' }
#' 
#' \strong{Table Width Management:}
#' 
#' For wide tables that don't fit on the page:
#' \enumerate{
#'   \item Use \code{orientation = "landscape"}
#'   \item Use \code{fit_to_page = TRUE} (default) to auto-scale
#'   \item Reduce \code{font_size} (\emph{e.g.,} 7 or 6)
#'   \item Consider \code{paper = "auto"} for maximum flexibility
#' }
#'
#' \strong{Troubleshooting:}
#' 
#' If PDF compilation fails:
#' \enumerate{
#'   \item Check that LaTeX is installed: Run \code{Sys.which("pdflatex")}
#'   \item Set \code{show_logs = TRUE} and examine the .log file
#'   \item Common issues:
#'     \itemize{
#'       \item Missing LaTeX packages: Install via package manager
#'       \item Special characters in text: Escape properly
#'       \item Very wide tables: Use landscape or reduce font size
#'       \item Caption formatting: Check LaTeX syntax
#'     }
#' }
#'
#' @seealso
#' \code{\link{tablesave}} for saving in the format given by the file extension,
#' \code{\link{table2tex}} for LaTeX source files,
#' \code{\link{table2html}} for HTML output,
#' \code{\link{table2docx}} for Microsoft Word,
#' \code{\link{table2pptx}} for PowerPoint,
#' \code{\link{table2rtf}} for Rich Text Format,
#' \code{\link{desctable}} for descriptive tables,
#' \code{\link{fit}} for regression tables
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example table
#' results <- fit(
#'     data = clintrial,
#'     outcome = "readmission_30d",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     labels = clintrial_labels
#' )
#'
#' # Test that LaTeX can compile (needed for all PDF examples)
#' has_latex <- local({
#'   if (!nzchar(Sys.which("pdflatex"))) return(FALSE)
#'   test_tex <- file.path(tempdir(), "summata_latex_test.tex")
#'   writeLines(c("\\documentclass{article}", "\\usepackage{booktabs}",
#'                "\\begin{document}", "test", "\\end{document}"), test_tex)
#'   tryCatch(
#'     system2("pdflatex", c("-interaction=nonstopmode",
#'             paste0("-output-directory=", tempdir()), test_tex),
#'             stdout = FALSE, stderr = FALSE),
#'     error = function(e) 1L) == 0L
#' })
#'
#' # Example 1: Basic PDF export
#' if(has_latex){
#'   table2pdf(results, file.path(tempdir(), "basic_results.pdf"))
#' }
#'
#' \donttest{
#'
#' # Example 2: Landscape orientation for wide tables
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "wide_results.pdf"),
#'            orientation = "landscape")
#' }
#'
#' # Example 3: With caption
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "captioned.pdf"),
#'            caption = "Table 1: Multivariable logistic regression results")
#' }
#'
#' # Example 4: Multi-line caption with formatting
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "formatted_caption.pdf"),
#'            caption = "Table 1: Risk Factors for Mortality\\\\
#'                      aOR = adjusted odds ratio; CI = confidence interval")
#' }
#'
#' # Example 5: Auto-sized PDF (no fixed page dimensions)
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "autosize.pdf"),
#'            paper = "auto")
#' }
#'
#' # Example 6: A4 paper with custom margins
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "a4_custom.pdf"),
#'            paper = "a4",
#'            margins = c(0.75, 0.75, 0.75, 0.75))
#' }
#'
#' # Example 7: Larger font for readability
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "large_font.pdf"),
#'            font_size = 11)
#' }
#'
#' # Example 8: Indented hierarchical display
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "indented.pdf"),
#'            indent_groups = TRUE)
#' }
#'
#' # Example 9: Condensed table (reduced height)
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "condensed.pdf"),
#'            condense_table = TRUE)
#' }
#'
#' # Example 10: With zebra stripes
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "striped.pdf"),
#'            zebra_stripes = TRUE,
#'            stripe_color = "gray!15")
#' }
#'
#' # Example 11: Dark header style
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "dark_header.pdf"),
#'            dark_header = TRUE)
#' }
#'
#' # Example 12: Combination of formatting options
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "publication_ready.pdf"),
#'            orientation = "portrait",
#'            paper = "letter",
#'            font_size = 9,
#'            caption = "Table 2: Multivariable Analysis\\\\
#'                      Model adjusted for age, sex, and clinical factors",
#'            indent_groups = TRUE,
#'            zebra_stripes = TRUE,
#'            bold_significant = TRUE,
#'            p_threshold = 0.05)
#' }
#'
#' # Example 13: Adjust cell padding
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "relaxed_padding.pdf"),
#'            cell_padding = "relaxed")  # More spacious
#' }
#'
#' # Example 14: No scaling (natural table width)
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "no_scale.pdf"),
#'            fit_to_page = FALSE,
#'            font_size = 10)
#' }
#'
#' # Example 15: Hide significance bolding
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "no_bold.pdf"),
#'            bold_significant = FALSE)
#' }
#'
#' # Example 16: Custom column alignment
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "custom_align.pdf"),
#'            align = c("c", "c", "c", "c", "c", "c", "c"))
#' }
#'
#' # Example 17: Descriptive statistics table
#' if (has_latex) {
#'   desc_table <- desctable(clintrial, by = "treatment",
#'        variables = c("age", "sex", "bmi", "stage"), labels = clintrial_labels)
#'   
#'   table2pdf(desc_table, file.path(tempdir(), "descriptive.pdf"),
#'            caption = "Table 1: Baseline Characteristics by Treatment Group",
#'            orientation = "landscape")
#' }
#'
#' # Example 18: Model comparison table
#' if (has_latex) {
#'   models <- list(
#'        base = c("age", "sex"),
#'        full = c("age", "sex", "bmi", "treatment")
#'   )
#'   
#'   # Information criteria assume a common sample, so the candidate
#'   # predictors are restricted to complete cases before comparison
#'   comparison_data <- na.omit(clintrial[, c("os_status", "age", "sex", "bmi", "treatment")])
#'   
#'   comparison <- compfit(
#'        data = comparison_data,
#'        outcome = "os_status",
#'        model_list = models,
#'        verbose = FALSE
#'   )
#'   
#'   table2pdf(comparison, file.path(tempdir(), "model_comparison.pdf"),
#'            caption = "Table 3: Model Comparison Statistics")
#' }
#'
#' # Example 19: Very wide table with aggressive fitting
#' if (has_latex) {
#'   wide_model <- fit(
#'        data = clintrial,
#'        outcome = "os_status",
#'        predictors = c("age", "sex", "race", "bmi", "smoking", 
#'                      "hypertension", "diabetes", "treatment", "stage")
#'   )
#'   
#'   table2pdf(wide_model, file.path(tempdir(), "very_wide.pdf"),
#'            orientation = "landscape",
#'            font_size = 7,
#'            fit_to_page = TRUE,
#'            condense_table = TRUE)
#' }
#'
#' # Example 20: With caption size control
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "caption_size.pdf"),
#'            font_size = 8,
#'            caption_size = 6,
#'            caption = "Table 4: Results with Compact Caption\\\\
#'                      Smaller caption fits better on constrained pages")
#' }
#'
#' # Example 21: Troubleshooting - keep logs
#' if (has_latex) {
#'   table2pdf(results, file.path(tempdir(), "debug.pdf"),
#'            show_logs = TRUE)
#'   # If it fails, check debug.log for error messages
#' }
#' }
#' 
#' @family export functions
#' @export
table2pdf <- function (table,
                       file,
                     orientation = "portrait",
                     paper = "letter", 
                     margins = NULL,
                     fit_to_page = TRUE,
                     font_size = 8,
                     caption = NULL,
                     caption_size = NULL, 
                     format_headers = TRUE,
                     variable_padding = FALSE,
                     cell_padding = "normal",
                     bold_significant = TRUE,
                     bold_variables = FALSE,
                     p_threshold = 0.05,
                     align = NULL,
                     indent_groups = FALSE,
                     condense_table = FALSE,
                     condense_quantitative = FALSE,
                     zebra_stripes = FALSE,
                     stripe_color = "gray!20",
                     dark_header = FALSE,
                     show_logs = FALSE,
                     quiet = FALSE,
                     ...) {
    
    if (!requireNamespace("xtable", quietly = TRUE)) {
        stop("Package 'xtable' required. Install with: install.packages('xtable')")
    }
    
    if (!grepl("\\.pdf$", tolower(file))) {
        stop("File must have .pdf extension")
    }
    
    if (!check_latex()) {
        if (requireNamespace("tinytex", quietly = TRUE) && tinytex::is_tinytex()) {
        } else {
            stop("PDF compilation requires LaTeX. Install TinyTeX with: tinytex::install_tinytex()")
        }
    }

    ## Set caption_size to font_size if not specified
    if (is.null(caption_size)) {
        caption_size <- font_size
    }
    
    ## Validate caption_size
    if (!is.numeric(caption_size) || caption_size <= 0) {
        stop("caption_size must be a positive numeric value")
    }
    
    orientation <- match.arg(orientation, c("portrait", "landscape"))
    paper_settings <- get_paper_settings(paper, margins)
    df <- as.data.frame(table)

    ## Detect variable groups BEFORE any processing
    var_groups <- NULL
    if (zebra_stripes) {
        if ("Variable" %in% names(df)) {
            var_starts <- which(df$Variable != "" & !is.na(df$Variable))
            if (length(var_starts) > 0) {
                var_groups <- integer(nrow(df))
                for (i in seq_along(var_starts)) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   var_starts[i + 1] - 1
                               } else {
                                   nrow(df)
                               }
                    var_groups[start_idx:end_idx] <- i
                }
            }
        }
    }
    
    has_n_row <- FALSE
    n_row_data <- NULL
    if (nrow(df) > 0 && "Variable" %in% names(df) && df$Variable[1] == "N") {
        has_n_row <- TRUE
        n_row_data <- df[1, ]
        df <- df[-1, ]
        if (!is.null(var_groups) && length(var_groups) > 1) {
            var_groups <- var_groups[-1]
        }
    }

    if (bold_significant) {
        df <- format_pvalues_export_tex(df, p_threshold)
    }
    
    original_nrow <- nrow(df)

    if (condense_table) {
        indent_groups <- TRUE
        df <- condense_table_rows(df, indent_groups = indent_groups)
        if (zebra_stripes && "Variable" %in% names(df)) {
            var_starts <- which(df$Variable != "" & !is.na(df$Variable))
            if (length(var_starts) > 0) {
                var_groups <- integer(nrow(df))
                for (i in seq_along(var_starts)) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   var_starts[i + 1] - 1
                               } else {
                                   nrow(df)
                               }
                    var_groups[start_idx:end_idx] <- i
                }
            }
        }
        df <- format_indented_groups(df, indent_string = "\\hspace{1em}")
    } else if (condense_quantitative) {
        ## Only condense continuous/survival variables (not categorical)
        ## Also set indent_groups = TRUE to avoid awkward empty Group column
        indent_groups <- TRUE
        df <- condense_quantitative_rows(df, indent_groups = indent_groups)
        if (zebra_stripes && "Variable" %in% names(df)) {
            var_starts <- which(df$Variable != "" & !is.na(df$Variable))
            if (length(var_starts) > 0) {
                var_groups <- integer(nrow(df))
                for (i in seq_along(var_starts)) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   var_starts[i + 1] - 1
                               } else {
                                   nrow(df)
                               }
                    var_groups[start_idx:end_idx] <- i
                }
            }
        }
        df <- format_indented_groups(df, indent_string = "\\hspace{1em}")
    } else if (indent_groups) {
        df <- format_indented_groups(df, indent_string = "\\hspace{1em}")
    }
    
    ## Bold variable names (non-indented rows in Variable column)
    if (bold_variables && "Variable" %in% names(df)) {
        ## Find rows where Variable is not empty and not indented (doesn't start with \hspace)
        var_rows <- which(!is.na(df$Variable) & 
                          df$Variable != "" & 
                          df$Variable != "-" &
                          !grepl("^\\\\hspace", df$Variable))
        if (length(var_rows) > 0) {
            df$Variable[var_rows] <- paste0("\\textbf{", df$Variable[var_rows], "}")
        }
    }
    
    padding_rows <- NULL
    if (variable_padding && ("Variable" %in% names(df) || "variable" %in% names(df))) {
        if (!is.null(var_groups)) {
            padding_positions <- which(diff(var_groups) != 0)
            new_var_groups <- integer(nrow(df) + length(padding_positions))
            current_pos <- 1
            for (i in seq_len(nrow(df))) {
                new_var_groups[current_pos] <- var_groups[i]
                current_pos <- current_pos + 1
                if (i %in% padding_positions) {
                    new_var_groups[current_pos] <- 0
                    current_pos <- current_pos + 1
                }
            }
            var_groups <- new_var_groups
        }
        df <- add_variable_padding(df)
    }
    
    add.to.row <- NULL
    if (zebra_stripes && "Variable" %in% names(df)) {
        is_indented <- indent_groups || condense_table || condense_quantitative
        
        ## Identify padding rows (all cells empty) - these should not be striped
        is_padding_row <- apply(df, 1, function(row) {
            all(is.na(row) | row == "" | row == " ")
        })
        
        if (is_indented) {
            var_starts <- which(!grepl("\\\\hspace", df$Variable) & 
                                df$Variable != "" & 
                                !is.na(df$Variable) &
                                !is_padding_row)
            
            commands <- character()
            positions <- numeric()
            
            for (i in seq_along(var_starts)) {
                if (i %% 2 != 0) {
                    start_idx <- var_starts[i]
                    end_idx <- if (i < length(var_starts)) {
                                   if (variable_padding) {
                                       var_starts[i + 1] - 2
                                   } else {
                                       var_starts[i + 1] - 1
                                   }
                               } else {
                                   nrow(df)
                               }
                    
                    for (row in start_idx:end_idx) {
                        ## Skip padding rows
                        if (!is.na(df$Variable[row]) && !is_padding_row[row]) {
                            commands <- c(commands, paste0("\\rowcolor{", stripe_color, "}"))
                            positions <- c(positions, row - 1)
                        }
                    }
                }
            }
            
            if (length(commands) > 0) {
                add.to.row <- list(pos = as.list(positions), command = commands)
            }
        } else {
            if ("Group" %in% names(df)) {
                var_starts <- which(df$Variable != "" & !is.na(df$Variable) & !is_padding_row)
                
                commands <- character()
                positions <- numeric()
                
                for (i in seq_along(var_starts)) {
                    if (i %% 2 != 0) {
                        start_idx <- var_starts[i]
                        end_idx <- if (i < length(var_starts)) {
                                       if (variable_padding) {
                                           var_starts[i + 1] - 2
                                       } else {
                                           var_starts[i + 1] - 1
                                       }
                                   } else {
                                       nrow(df)
                                   }
                        
                        for (row in start_idx:end_idx) {
                            ## Skip padding rows
                            if (!is.na(df$Variable[row]) && !is_padding_row[row]) {
                                commands <- c(commands, paste0("\\rowcolor{", stripe_color, "}"))
                                positions <- c(positions, row - 1)
                            }
                        }
                    }
                }
                
                if (length(commands) > 0) {
                    add.to.row <- list(pos = as.list(positions), command = commands)
                }
            } else if (!is.null(var_groups)) {
                commands <- character()
                positions <- numeric()
                
                for (i in seq_len(nrow(df))) {
                    ## Skip padding rows (var_groups == 0 or is_padding_row)
                    if (var_groups[i] %% 2 != 0 && var_groups[i] > 0 && !is_padding_row[i]) {
                        commands <- c(commands, paste0("\\rowcolor{", stripe_color, "}"))
                        positions <- c(positions, i - 1)
                    }
                }
                
                if (length(commands) > 0) {
                    add.to.row <- list(pos = as.list(positions), command = commands)
                }
            }
        }
    }

    if (is.null(align)) {
        align <- determine_alignment(df)
    }
    
    original_col_names <- names(df)
    
    if (format_headers) {
        if (has_n_row) {
            names(df) <- format_column_headers_with_n_tex(names(df), n_row_data)
        } else {
            names(df) <- format_column_headers(names(df))
        }
    }

    if (dark_header) {
        col_names <- names(df)
        for (i in seq_along(col_names)) {
            col_names[i] <- paste0("\\color{white}", col_names[i])
        }
        names(df) <- col_names
        
        header_command <- "\\rowcolor{black}"
        
        if (!is.null(add.to.row) && length(add.to.row$pos) > 0) {
            add.to.row$pos <- c(list(-1), add.to.row$pos)
            add.to.row$command <- c(header_command, add.to.row$command)
        } else {
            add.to.row <- list(
                pos = list(-1),
                command = header_command
            )
        }
    }
    
    ## Create xtable object
    xt <- xtable::xtable(df, align = align, ...)
    
    file_base <- tools::file_path_sans_ext(file)
    tex_file <- paste0(file_base, ".tex")
    use_standalone <- FALSE
    
    ## Determine array stretch value for cell padding
    array_stretch <- switch(as.character(cell_padding),
                            "none" = NULL,
                            "normal" = "1.3",
                            "relaxed" = "1.5",
                            "loose" = "1.8",
                            {
                                ## If numeric value provided
                                if (is.numeric(cell_padding)) {
                                    as.character(cell_padding)
                                } else {
                                    "1.3"  # Default to normal
                                }
                            })
    
    if (is.null(paper_settings$width)) {
        standalone_check <- system2("kpsewhich", args = "standalone.cls", 
                                    stdout = TRUE, stderr = FALSE)
        use_standalone <- length(standalone_check) > 0 && standalone_check != ""
        
        if (use_standalone) {
            if (!quiet) message("Using standalone class for auto-sized output")
            xcolor_line <- if (zebra_stripes || dark_header) "\\usepackage[table]{xcolor}\n" else ""
            ## Add arraystretch command if padding requested
            array_stretch_line <- if (!is.null(array_stretch)) 
                                      sprintf("\\renewcommand{\\arraystretch}{%s}\n", array_stretch) else ""
            
            cat(sprintf("\\documentclass[%dpt,border=10pt,varwidth=\\maxdimen]{standalone}\n
                         \\usepackage[T1]{fontenc}\n
                         \\usepackage[utf8]{inputenc}\n
                         %s\\usepackage{helvet}\n
                         \\renewcommand{\\familydefault}{\\sfdefault}\n
                         \\usepackage{array}\n
                         \\usepackage{graphicx}\n
                         \\usepackage{varwidth}\n
                         %s\\begin{document}\n
                         \\begin{varwidth}{\\linewidth}\n", 
                        font_size, xcolor_line, array_stretch_line), file = tex_file)
            
        } else {
            
            has_pdfcrop <- Sys.which("pdfcrop") != ""
            
            if (has_pdfcrop) {
                if (!quiet) message("Standalone class not found. Will use pdfcrop for auto-sizing")
            } else {
                warning("Auto-sizing requested but neither standalone class nor pdfcrop available.\n", 
                        "Install standalone with: tlmgr install standalone\n", 
                        "Using minimal margins instead.")
            }
            
            xcolor_line <- if (zebra_stripes || dark_header) "\\usepackage[table]{xcolor}\n" else ""
            array_stretch_line <- if (!is.null(array_stretch)) 
                                      sprintf("\\renewcommand{\\arraystretch}{%s}\n", array_stretch) else ""
            
            cat(sprintf("\\documentclass[%dpt]{article}\n
                         \\usepackage[T1]{fontenc}\n
                         \\usepackage[utf8]{inputenc}\n
                         \\usepackage[paperwidth=15in,paperheight=15in,margin=0.5in]{geometry}\n
                         %s\\usepackage{helvet}\n
                         \\renewcommand{\\familydefault}{\\sfdefault}\n
                         \\usepackage{array}\n
                         \\usepackage{graphicx}\n
                         \\pagestyle{empty}\n
                         %s\\begin{document}\n
                         \\noindent\n", 
                        font_size, xcolor_line, array_stretch_line), file = tex_file)
        }
        
    } else {
        
        margin_str <- sprintf("top=%.1fin,right=%.1fin,bottom=%.1fin,left=%.1fin", 
                              paper_settings$margins[1], paper_settings$margins[2], 
                              paper_settings$margins[3], paper_settings$margins[4])
        xcolor_line <- if (zebra_stripes || dark_header) "\\usepackage[table]{xcolor}\n" else ""
        array_stretch_line <- if (!is.null(array_stretch)) 
                                  sprintf("\\renewcommand{\\arraystretch}{%s}\n", array_stretch) else ""
        
        cat(sprintf("\\documentclass[%dpt]{article}\n
                     \\usepackage[T1]{fontenc}\n
                     \\usepackage[utf8]{inputenc}\n
                     \\usepackage[%s,%s,%s]{geometry}\n
                     %s\\usepackage{helvet}\n
                     \\renewcommand{\\familydefault}{\\sfdefault}\n
                     \\usepackage{array}\n
                     \\usepackage{graphicx}\n
                     \\pagestyle{empty}\n
                     %s\\begin{document}\n", 
                    font_size, paper_settings$latex_paper, orientation, 
                    margin_str, xcolor_line, array_stretch_line), file = tex_file)
    }
    
    if (fit_to_page && !is.null(paper_settings$width)) {
        cat("\n\\noindent\\resizebox{\\textwidth}{!}{%\n", file = tex_file, 
            append = TRUE)
    }

    if ((zebra_stripes || dark_header) && length(add.to.row$pos) > 0) {
        print(xt,
              file = tex_file,
              append = TRUE,
              include.rownames = FALSE, 
              booktabs = FALSE,
              floating = FALSE,
              tabular.environment = "tabular", 
              hline.after = c(-1, 0, nrow(xt)),
              add.to.row = add.to.row,
              sanitize.text.function = sanitize_for_latex, 
              sanitize.rownames.function = sanitize_for_latex,
              sanitize.colnames.function = function(x) x)
    } else {
        print(xt,
              file = tex_file,
              append = TRUE,
              include.rownames = FALSE,
              booktabs = FALSE,
              floating = FALSE,
              tabular.environment = "tabular",
              hline.after = c(-1, 0, nrow(xt)),
              sanitize.text.function = sanitize_for_latex,
              sanitize.rownames.function = sanitize_for_latex,
              sanitize.colnames.function = function(x) x)
    }
    
    if (fit_to_page && !is.null(paper_settings$width)) {
        cat("}\n", file = tex_file, append = TRUE)
    }
    
    if (!is.null(caption)) {
        cat(sprintf("\n\n{\\fontsize{%d}{%d}\\selectfont\\vspace{1em}\\noindent %s}", 
                    caption_size, ceiling(caption_size * 1.2), caption), 
            file = tex_file, append = TRUE)
    }

    if (use_standalone) {
        cat("\n\\end{varwidth}\n", file = tex_file, append = TRUE)
    }
    
    cat("\n\\end{document}", file = tex_file, append = TRUE)

    if (!quiet) message("Compiling PDF...")
    
    result <- system2("pdflatex",
                      args = c("-interaction=nonstopmode",
                               paste0("-output-directory=", dirname(tex_file)),
                               tex_file),
                      stdout = FALSE,
                      stderr = FALSE)
    
    pdf_file <- paste0(file_base, ".pdf")
    
    if (is.null(paper_settings$width) && !use_standalone && file.exists(pdf_file) && Sys.which("pdfcrop") != "") {
        if (!quiet) message("Cropping PDF to content...")
        temp_pdf <- paste0(file_base, "_temp.pdf")
        file.rename(pdf_file, temp_pdf)
        system2("pdfcrop", args = c("--margins", "10", temp_pdf, pdf_file), stdout = FALSE, stderr = FALSE)
        file.remove(temp_pdf)
    }
    
    if (!file.exists(pdf_file)) {
        warning("PDF compilation failed. Ensure show_logs = TRUE and check ", file_base, 
                ".log for errors")
    }
    
    aux_files <- paste0(file_base, c(".aux", ".log", ".tex"))

    if (!show_logs) {
        for (f in aux_files) {
            if (file.exists(f))
                file.remove(f)
        }
    }
    
    if (!quiet) message(sprintf("Table saved to %s", file))
    
    invisible(file)
}

### * Unicode conversions

## Column name constants for Unicode superscripts
## Defined here to ensure consistent column naming throughout
.pseudo_r2_col <- "Pseudo-R\u00b2"
.marginal_r2_col <- "Marginal R\u00b2"
.conditional_r2_col <- "Conditional R\u00b2"
.adjusted_r2_col <- "Adjusted R\u00b2"


### * Validation functions

#' Check model convergence
#' 
#' Checks convergence status for various model types including standard
#' regression models and mixed-effects models.
#' 
#' @param model Fitted model object.
#' @return Character string: "Yes", "No", "Suspect", or "Failed"
#' @keywords internal
check_convergence <- function(model) {
    
    if (inherits(model, "glm")) {
        if (!model$converged) return("No")
        if (any(abs(coef(model)) > 10)) return("Suspect")
        return("Yes")
        
    } else if (inherits(model, c("coxph", "coxme"))) {
        
        ## Cox models do not always have an iter field
        ## Check for actual convergence issues
        if (!is.null(model$info) && !is.null(model$info$convergence)) {
            if (!model$info$convergence) return("No")
        }
        
        ## Check for extreme coefficients as a proxy for issues
        if (any(abs(coef(model)) > 10, na.rm = TRUE)) return("Suspect")
        
        ## Check if model failed (no coefficients)
        if (length(coef(model)) == 0) return("Failed")
        
        return("Yes")
        
    } else if (inherits(model, c("lmerMod", "glmerMod", "merMod"))) {
        ## Mixed-effects models from lme4
        
        ## Check for convergence warnings in the model object
        ## lme4 stores convergence info in the optinfo slot
        if (!is.null(model@optinfo$conv$lme4)) {
            conv_info <- model@optinfo$conv$lme4
            if (!is.null(conv_info$code) && conv_info$code != 0) {
                return("No")
            }
            if (!is.null(conv_info$messages) && length(conv_info$messages) > 0) {
                ## Check for specific convergence warnings
                msgs <- tolower(paste(conv_info$messages, collapse = " "))
                if (grepl("failed to converge|singular|gradient|hessian", msgs)) {
                    return("Suspect")
                }
            }
        }
        
        ## Check for singular fit (boundary estimates)
        if (lme4::isSingular(model)) {
            return("Suspect")
        }
        
        ## Check for extreme fixed effects coefficients
        fixed_coefs <- lme4::fixef(model)
        if (any(abs(fixed_coefs) > 10, na.rm = TRUE)) {
            return("Suspect")
        }
        
        ## Check that random effects variance is estimable
        var_comps <- lme4::VarCorr(model)
        for (i in seq_along(var_comps)) {
            var_vals <- diag(as.matrix(var_comps[[i]]))
            if (any(var_vals < 1e-10) || any(is.na(var_vals))) {
                return("Suspect")
            }
        }
        
        return("Yes")
        
    } else {
        
        return("Yes")
        
    }
}

#' Auto-detect model type based on outcome and random effects
#' 
#' Determines the appropriate model type based on outcome variable
#' characteristics and presence of random effects.
#' 
#' @param data Data.frame or data.table containing the outcome variable.
#' @param outcome Character string specifying the outcome variable or Surv() expression.
#' @param has_random_effects Logical indicating if random effects are specified.
#' @param family Character string for GLM family (default "binomial").
#' @return Character string indicating detected model type.
#' @keywords internal
detect_model_type_auto <- function(data, outcome, has_random_effects, family = "binomial") {
    
    is_survival <- grepl("^Surv\\(", outcome)
    
    if (is_survival) {
        if (has_random_effects) {
            message("Auto-detected survival outcome with random effects, using coxme")
            return("coxme")
        } else {
            message("Auto-detected survival outcome, using Cox regression")
            return("coxph")
        }
    }
    
    ## Check outcome type
    is_binary <- is.factor(data[[outcome]]) || 
        (is.numeric(data[[outcome]]) && length(unique(data[[outcome]])) == 2)
    is_continuous <- is.numeric(data[[outcome]]) && length(unique(data[[outcome]])) > 2
    
    if (is_binary) {
        if (has_random_effects) {
            message("Auto-detected binary outcome with random effects, using glmer")
            return("glmer")
        } else {
            message("Auto-detected binary outcome, using logistic regression")
            return("glm")
        }
    } else if (is_continuous) {
        if (has_random_effects) {
            message("Auto-detected continuous outcome with random effects, using lmer")
            return("lmer")
        } else {
            message("Auto-detected continuous outcome, using linear regression")
            return("lm")
        }
    } else {
        stop("Cannot auto-detect model type for outcome: ", outcome)
    }
}

#' Normalize model type names
#' 
#' Converts model class names to standardized model type strings.
#' 
#' @param model_type Character string of model type or class name.
#' @return Normalized character string (\emph{e.g.,} "lmerMod" becomes "lmer").
#' @keywords internal
normalize_model_type <- function(model_type) {
    switch(model_type,
           "lmerMod" = "lmer",
           "glmerMod" = "glmer",
           model_type)
}

#' Check required packages for model type
#' 
#' Verifies that necessary packages are installed for the specified model type.
#' Stops with informative error if required packages are missing.
#' 
#' @param model_type Character string indicating model type.
#' @return \code{NULL} (invisibly). Stops execution if packages missing.
#' @keywords internal
check_required_packages <- function(model_type) {
    
    if (model_type %in% c("lmer", "glmer")) {
        if (!requireNamespace("lme4", quietly = TRUE)) {
            stop("Package 'lme4' is required for mixed-effects models. ",
                 "Install with: install.packages('lme4')")
        }
    }
    
    if (model_type == "coxme") {
        if (!requireNamespace("coxme", quietly = TRUE)) {
            stop("Package 'coxme' is required for mixed-effects Cox models. ",
                 "Install with: install.packages('coxme')")
        }
    }
    
    if (model_type %in% c("coxph", "clogit")) {
        if (!requireNamespace("survival", quietly = TRUE)) {
            stop("Package 'survival' is required for Cox models. ",
                 "Install with: install.packages('survival')")
        }
    }
}


### * Extraction functions

#' Extract comprehensive model metrics based on academic consensus
#' 
#' Extracts quality control metrics from fitted models for comparison.
#' Supports GLM, Cox, linear, and mixed-effects models.
#' 
#' @param model Fitted model object.
#' @param raw_data Data.table with raw model information.
#' @param model_type Character string indicating model type.
#' @return Named list of metrics.
#' @keywords internal
extract_model_metrics <- function(model, raw_data, model_type) {
    
    metrics <- list(
        n = raw_data$n[1],
        events = if ("events" %chin% names(raw_data)) raw_data$events[1] else NA_integer_,
        predictors = length(coef(model)),
        aic = if ("AIC" %chin% names(raw_data)) raw_data$AIC[1] else AIC(model),
        bic = if ("BIC" %chin% names(raw_data)) raw_data$BIC[1] else BIC(model),
        ## Initialize all possible metrics as NA to avoid missing list elements
        pseudo_r2 = NA_real_,
        concordance = NA_real_,
        global_p = NA_real_,
        null_deviance = NA_real_,
        residual_deviance = NA_real_,
        deviance_ratio = NA_real_,
        mcfadden_r2 = NA_real_,
        nagelkerke_r2 = NA_real_,
        tjur_r2 = NA_real_,
        c_statistic = NA_real_,
        brier_score = NA_real_,
        hoslem_p = NA_real_,
        ## Mixed-effects-specific metrics
        marginal_r2 = NA_real_,
        conditional_r2 = NA_real_,
        icc = NA_real_,
        n_groups = NA_integer_,
        sigma = NA_real_
    )
    
    ## GLM-specific metrics
    if (model_type == "glm") {
        ## Deviance metrics
        metrics$null_deviance <- model$null.deviance
        metrics$residual_deviance <- model$deviance
        metrics$deviance_ratio <- 1 - (model$deviance / model$null.deviance)
        
        if (model$family$family == "binomial") {
            ## Multiple pseudo R-squared measures
            n <- length(model$y)
            
            ## McFadden R^2
            null_model <- stats::glm(model$y ~ 1, family = model$family)
            metrics$mcfadden_r2 <- as.numeric(1 - (logLik(model)/logLik(null_model)))
            metrics$pseudo_r2 <- metrics$mcfadden_r2  # Set generic pseudo_r2
            
            ## Nagelkerke R^2 (Cox-Snell adjusted)
            cox_snell <- 1 - exp((model$deviance - model$null.deviance)/n)
            metrics$nagelkerke_r2 <- cox_snell / (1 - exp(-model$null.deviance/n))
            
            ## Tjur R^2 (coefficient of discrimination)
            pred_probs <- fitted(model)
            metrics$tjur_r2 <- mean(pred_probs[model$y == 1]) - mean(pred_probs[model$y == 0])
            
            ## C-statistic (concordance)
            if (requireNamespace("pROC", quietly = TRUE)) {
                roc_obj <- pROC::roc(model$y, fitted(model), quiet = TRUE)
                metrics$c_statistic <- as.numeric(pROC::auc(roc_obj))
                metrics$concordance <- metrics$c_statistic  # Set generic concordance
                
                ## Optimal threshold metrics
                coords <- pROC::coords(roc_obj, "best", ret = c("threshold", "sensitivity", "specificity"))
                metrics$optimal_threshold <- coords$threshold
                metrics$sensitivity <- coords$sensitivity
                metrics$specificity <- coords$specificity
            }
            
            ## Brier score
            metrics$brier_score <- mean((fitted(model) - model$y)^2)
            
            ## Hosmer-Lemeshow test (if ResourceSelection available)
            if (requireNamespace("ResourceSelection", quietly = TRUE) && n >= 40) {
                tryCatch({
                    hl <- ResourceSelection::hoslem.test(model$y, fitted(model), g = 10)
                    metrics$hoslem_p <- hl$p.value
                }, error = function(e) {
                    metrics$hoslem_p <- NA_real_
                })
            }
            
            ## Global LR test
            lr_stat <- model$null.deviance - model$deviance
            df <- model$df.null - model$df.residual
            metrics$global_p <- pchisq(lr_stat, df, lower.tail = FALSE)
        }
        
    } else if (model_type == "coxph") {
        
        ## Initialize Cox-specific metrics
        metrics$c_index <- NA_real_
        metrics$rsq <- NA_real_
        metrics$lr_test_p <- NA_real_
        
        ## Cache summary to avoid repeated calls
        summ <- summary(model)
        
        ## Concordance
        if (!is.null(model$concordance)) {
            metrics$c_index <- model$concordance["concordance"]
            metrics$concordance <- metrics$c_index
            
        } else if (!is.null(summ$concordance)) {
            metrics$c_index <- summ$concordance["C"]
            metrics$concordance <- metrics$c_index
        }
        
        ## R-squared (different location in Cox models)
        if (!is.null(summ$rsq)) {
            metrics$rsq <- summ$rsq[1]
            metrics$pseudo_r2 <- metrics$rsq
        }
        
        ## Global tests
        if (!is.null(summ$logtest)) {
            metrics$lr_test_p <- summ$logtest["pvalue"]
            metrics$global_p <- metrics$lr_test_p
            
        } else if (!is.null(summ$waldtest)) {
            metrics$global_p <- summ$waldtest["pvalue"]
        }
        
    } else if (model_type == "lm") {
        
        ## Linear model metrics
        summ <- summary(model)
        metrics$pseudo_r2 <- summ$r.squared
        metrics$adj_r2 <- summ$adj.r.squared
        metrics$sigma <- summ$sigma
        metrics$rmse <- sqrt(mean(residuals(model)^2))
        
        ## F-test for global significance
        if (!is.null(summ$fstatistic)) {
            f_stat <- summ$fstatistic
            metrics$global_p <- pf(f_stat[1], f_stat[2], f_stat[3], lower.tail = FALSE)
        }
        
    } else if (model_type == "lmer" || inherits(model, "lmerMod")) {
        
        ## Linear mixed-effects model metrics
        metrics <- extract_lmer_metrics(model, raw_data, metrics)
        
    } else if (model_type == "glmer" || inherits(model, "glmerMod")) {
        
        ## Generalized linear mixed-effects model metrics
        metrics <- extract_glmer_metrics(model, raw_data, metrics)
        
    } else if (model_type == "coxme") {
        
        ## Mixed-effects Cox model metrics
        metrics <- extract_coxme_metrics(model, raw_data, metrics)
    }
    
    return(metrics)
}


#' Extract metrics for linear mixed-effects models (lmer)
#' 
#' Extracts quality metrics specific to linear mixed-effects models including
#' R-squared measures, ICC, and global significance tests.
#' 
#' @param model Fitted lmerMod object from lme4.
#' @param raw_data Data.table with raw model information.
#' @param metrics Named list of initialized metrics to populate.
#' @return Updated metrics list with lmer-specific values.
#' @keywords internal
extract_lmer_metrics <- function(model, raw_data, metrics) {
    
    ## Number of observations and groups
    metrics$n <- nobs(model)
    n_grps <- lme4::ngrps(model)
    metrics$n_groups <- sum(n_grps)
    
    ## Fixed effects count (excluding intercept for predictor count)
    fixed_coefs <- lme4::fixef(model)
    metrics$predictors <- length(fixed_coefs) - 1  # Exclude intercept
    
    ## Residual standard deviation
    metrics$sigma <- sigma(model)
    
    ## RMSE (same as sigma for linear mixed models)
    metrics$rmse <- metrics$sigma
    
    ## R-squared measures using MuMIn if available
    if (requireNamespace("MuMIn", quietly = TRUE)) {
        ## MuMIn refits a null model and warns that the result is only correct
        ## when that model uses the same variables. The condition holds here, as
        ## the null is derived from the fitted model itself, and the advisory
        ## would otherwise surface on every mixed-effects comparison.
        r2_vals <- tryCatch({
            suppressWarnings(MuMIn::r.squaredGLMM(model))
        }, error = function(e) NULL)
        
        if (!is.null(r2_vals) && nrow(r2_vals) > 0) {
            metrics$marginal_r2 <- r2_vals[1, "R2m"]
            metrics$conditional_r2 <- r2_vals[1, "R2c"]
            metrics$pseudo_r2 <- metrics$marginal_r2  # Use marginal for comparison
        }
    }
    
    ## Alternative R-squared calculation if MuMIn not available
    if (is.na(metrics$pseudo_r2)) {
        ## Simple variance-based R^2
        var_fixed <- var(predict(model, re.form = NA))  # Fixed effects only
        var_total <- var(model@resp$y)
        metrics$pseudo_r2 <- var_fixed / var_total
        metrics$marginal_r2 <- metrics$pseudo_r2
    }
    
    ## ICC (Intraclass Correlation Coefficient)
    ## ICC = variance of random effects / total variance
    var_comps <- as.data.frame(lme4::VarCorr(model))
    random_var <- sum(var_comps$vcov[var_comps$grp != "Residual"])
    residual_var <- var_comps$vcov[var_comps$grp == "Residual"]
    if (!is.na(residual_var) && (random_var + residual_var) > 0) {
        metrics$icc <- random_var / (random_var + residual_var)
    }
    
    ## Global test using likelihood ratio test
    ## Compare to null model (intercept + random effects only)
    null_formula <- stats::update(stats::formula(model), . ~ 1 + (1 | .))
    
    ## Try to fit null model and compute LR test
    tryCatch({
        ## Get random effects structure
        ## lme4 has moved findbars() to the reformulas package and warns when
        ## called through the old location. Prefer reformulas where it is
        ## available and fall back to lme4 otherwise.
        re_terms <- if (requireNamespace("reformulas", quietly = TRUE)) {
                        reformulas::findbars(stats::formula(model))
                    } else {
                        suppressWarnings(lme4::findbars(stats::formula(model)))
                    }
        if (length(re_terms) > 0) {
            ## Build null formula with random effects
            re_string <- paste(sapply(re_terms, deparse), collapse = " + ")
            null_form <- stats::as.formula(paste(". ~ 1 +", re_string))
            null_model <- stats::update(model, formula = null_form)
            
            ## LR test
            lr_stat <- as.numeric(-2 * (logLik(null_model) - logLik(model)))
            df_diff <- attr(logLik(model), "df") - attr(logLik(null_model), "df")
            if (df_diff > 0) {
                metrics$global_p <- pchisq(lr_stat, df_diff, lower.tail = FALSE)
            }
        }
    }, error = function(e) {
        ## If null model fails, use Wald test approximation
        summ <- summary(model)
        coef_tab <- coef(summ)
        if (nrow(coef_tab) > 1) {
            ## Use minimum p-value as conservative estimate
            ## (excluding intercept)
            if (ncol(coef_tab) >= 5) {
                p_vals <- coef_tab[-1, ncol(coef_tab)]
                metrics$global_p <- min(p_vals, na.rm = TRUE)
            }
        }
    })
    
    return(metrics)
}


#' Extract metrics for generalized linear mixed-effects models (glmer)
#' 
#' Extracts quality metrics specific to generalized linear mixed-effects models
#' including concordance, R-squared measures, ICC, and Brier score for binomial.
#' 
#' @param model Fitted glmerMod object from lme4.
#' @param raw_data Data.table with raw model information.
#' @param metrics Named list of initialized metrics to populate.
#' @return Updated metrics list with glmer-specific values.
#' @keywords internal
extract_glmer_metrics <- function(model, raw_data, metrics) {
    
    summ <- summary(model)
    
    ## Number of observations and groups
    metrics$n <- nobs(model)
    n_grps <- lme4::ngrps(model)
    metrics$n_groups <- sum(n_grps)
    
    ## Fixed effects count
    fixed_coefs <- lme4::fixef(model)
    metrics$predictors <- length(fixed_coefs) - 1  # Exclude intercept
    
    ## For binomial models, get events
    if (summ$family == "binomial") {
        response_var <- model@resp$y
        if (!is.null(response_var)) {
            metrics$events <- sum(response_var, na.rm = TRUE)
        }
    }
    
    ## R-squared measures using MuMIn if available
    if (requireNamespace("MuMIn", quietly = TRUE)) {
        ## MuMIn refits a null model and warns that the result is only correct
        ## when that model uses the same variables. The condition holds here, as
        ## the null is derived from the fitted model itself, and the advisory
        ## would otherwise surface on every mixed-effects comparison.
        r2_vals <- tryCatch({
            suppressWarnings(MuMIn::r.squaredGLMM(model))
        }, error = function(e) NULL)
        
        if (!is.null(r2_vals) && nrow(r2_vals) > 0) {
            metrics$marginal_r2 <- r2_vals[1, "R2m"]
            metrics$conditional_r2 <- r2_vals[1, "R2c"]
            metrics$pseudo_r2 <- metrics$marginal_r2
        }
    }
    
    ## C-statistic for binomial models
    if (summ$family == "binomial") {
        if (requireNamespace("pROC", quietly = TRUE)) {
            c_stat <- tryCatch({
                fitted_vals <- fitted(model)
                response_vals <- model@resp$y
                roc_obj <- pROC::roc(response_vals, fitted_vals, quiet = TRUE)
                as.numeric(pROC::auc(roc_obj))
            }, error = function(e) NA_real_)
            
            if (!is.na(c_stat)) {
                metrics$c_statistic <- c_stat
                metrics$concordance <- c_stat
            }
        }
        
        ## Brier score
        fitted_vals <- fitted(model)
        response_vals <- model@resp$y
        metrics$brier_score <- mean((fitted_vals - response_vals)^2)
    }
    
    ## ICC for random effects
    var_comps <- as.data.frame(lme4::VarCorr(model))
    random_var <- sum(var_comps$vcov[var_comps$grp != "Residual"])
    
    ## For binomial, use pi^2/3 as residual variance on latent scale
    if (summ$family == "binomial") {
        latent_var <- pi^2 / 3
        metrics$icc <- random_var / (random_var + latent_var)
    }
    
    ## Global test using LR test
    tryCatch({
        ## lme4 has moved findbars() to the reformulas package and warns when
        ## called through the old location. Prefer reformulas where it is
        ## available and fall back to lme4 otherwise.
        re_terms <- if (requireNamespace("reformulas", quietly = TRUE)) {
                        reformulas::findbars(stats::formula(model))
                    } else {
                        suppressWarnings(lme4::findbars(stats::formula(model)))
                    }
        if (length(re_terms) > 0) {
            re_string <- paste(sapply(re_terms, deparse), collapse = " + ")
            null_form <- stats::as.formula(paste(". ~ 1 +", re_string))
            null_model <- stats::update(model, formula = null_form)
            
            lr_stat <- as.numeric(-2 * (logLik(null_model) - logLik(model)))
            df_diff <- attr(logLik(model), "df") - attr(logLik(null_model), "df")
            if (df_diff > 0) {
                metrics$global_p <- pchisq(lr_stat, df_diff, lower.tail = FALSE)
            }
        }
    }, error = function(e) {
        ## Fallback to Wald test
        coef_tab <- coef(summ)
        if (nrow(coef_tab) > 1 && ncol(coef_tab) >= 4) {
            p_vals <- coef_tab[-1, ncol(coef_tab)]
            metrics$global_p <- min(p_vals, na.rm = TRUE)
        }
    })
    
    return(metrics)
}


#' Extract metrics for mixed-effects Cox models (coxme)
#' 
#' Extracts quality metrics specific to mixed-effects Cox proportional hazards
#' models including concordance, pseudo-R-squared, and ICC.
#' 
#' @param model Fitted coxme object from coxme package.
#' @param raw_data Data.table with raw model information.
#' @param metrics Named list of initialized metrics to populate.
#' @return Updated metrics list with coxme-specific values.
#' @keywords internal
extract_coxme_metrics <- function(model, raw_data, metrics) {
    
    ## Sample size and events
    if (!is.null(model$n)) {
        metrics$events <- model$n[1]
        metrics$n <- model$n[2]
    }
    
    ## Number of groups
    if (!is.null(model$n)) {
        ## coxme stores n as c(events, n, ngroups)
        if (length(model$n) >= 3) {
            metrics$n_groups <- model$n[3]
        }
    }
    
    ## Fixed effects count
    fixed_coefs <- coxme::fixef(model)
    metrics$predictors <- length(fixed_coefs)
    
    ## Concordance/C-statistic
    if (requireNamespace("survival", quietly = TRUE)) {
        c_stat <- tryCatch({
            if (!is.null(model$y) && !is.null(model$linear.predictor)) {
                conc <- survival::concordance(model$y ~ model$linear.predictor)
                conc$concordance
            } else {
                NA_real_
            }
        }, error = function(e) NA_real_)
        
        if (!is.na(c_stat)) {
            metrics$concordance <- c_stat
            metrics$c_statistic <- c_stat
        }
    }
    
    ## Log-likelihood based pseudo-R^2
    loglik <- model$loglik
    if (!is.null(loglik) && length(loglik) >= 2) {
        ## McFadden-style pseudo-R^2
        metrics$pseudo_r2 <- 1 - (loglik[length(loglik)] / loglik[1])
    }
    
    ## Global LR test
    if (!is.null(loglik) && length(loglik) >= 2) {
        lr_stat <- 2 * (loglik[length(loglik)] - loglik[1])
        df <- length(fixed_coefs)
        if (df > 0) {
            metrics$global_p <- pchisq(lr_stat, df, lower.tail = FALSE)
        }
    }
    
    ## ICC from random effects variance
    var_comps <- coxme::VarCorr(model)
    if (!is.null(var_comps)) {
        ## Extract variance components
        total_var <- 0
        for (vc in var_comps) {
            if (is.matrix(vc)) {
                total_var <- total_var + sum(diag(vc))
            } else {
                total_var <- total_var + sum(vc)
            }
        }
        ## For Cox models, there's no residual variance, so ICC is relative
        ## to assumed baseline variance (often pi^2/6 for proportional hazards)
        baseline_var <- pi^2 / 6
        metrics$icc <- total_var / (total_var + baseline_var)
    }
    
    return(metrics)
}


### * Formatting functions

#' Format model comparison table
#' 
#' Rounds numeric columns to appropriate precision based on metric type.
#' 
#' @param comparison Data.table with comparison metrics.
#' @return Data.table with properly rounded numeric columns.
#' @keywords internal
format_model_comparison <- function(comparison) {
    
    ## Round numeric columns to appropriate precision
    numeric_cols <- names(comparison)[sapply(comparison, is.numeric)]
    
    for (col in numeric_cols) {
        if (col %chin% c("AIC", "BIC")) {
            comparison[, (col) := round(get(col), 1)]
        } else if (col %chin% c("C-statistic", "C-index", "Concordance", 
                                "McFadden R\u00b2", "Nagelkerke R\u00b2", "R\u00b2", "Adj R\u00b2",
                                "Marginal R\u00b2", "Conditional R\u00b2", "ICC",
                                "Brier Score", "RMSE", "Pseudo-R\u00b2")) {
            comparison[, (col) := round(get(col), 3)]
        }
    }
    
    return(comparison)
}


#' Combine coefficient tables from multiple models
#' 
#' Merges coefficient tables from multiple fitted models into a single
#' data.table with a Model identifier column.
#' 
#' @param coef_list List of data.tables containing coefficient information.
#' @param model_names Character vector of model names corresponding to coef_list.
#' @return Combined data.table with Model column, or \code{NULL} if empty.
#' @keywords internal
combine_coefficient_tables <- function(coef_list, model_names) {
    if (length(coef_list) == 0) return(NULL)
    
    ## Add model identifier to each table
    for (i in seq_along(coef_list)) {
        if (!is.null(coef_list[[i]])) {
            coef_list[[i]]$Model <- model_names[i]
        }
    }
    
    ## Combine all tables
    combined <- data.table::rbindlist(coef_list, fill = TRUE)
    
    ## Reorder columns to put Model first
    if ("Model" %in% names(combined)) {
        cols <- names(combined)
        new_order <- c("Model", setdiff(cols, "Model"))
        combined <- combined[, ..new_order]
    }
    
    return(combined)
}


#' Build row for failed model
#' 
#' Creates a comparison table row with appropriate NA values for a model
#' that failed to fit.
#' 
#' @param model_name Character string name of the model.
#' @param n Integer sample size.
#' @param n_predictors Integer number of predictors attempted.
#' @param model_type Character string indicating model type.
#' @return Data.table with single row of NA metrics and "Failed" convergence.
#' @keywords internal
build_failed_model_row <- function(model_name, n, n_predictors, model_type) {
    
    row <- data.table::data.table(
                           Model = model_name,
                           N = n,
                           Events = NA_integer_,
                           Predictors = n_predictors,
                           Converged = "Failed",
                           AIC = NA_real_,
                           BIC = NA_real_,
                           `Pseudo-R^2` = NA_real_,
                           Concordance = NA_real_,
                           `Global p` = NA_real_
                       )
    
    ## Add model-type specific columns
    if (model_type == "glm") {
        row$`Brier Score` <- NA_real_
    } else if (model_type %in% c("lmer", "glmer", "coxme")) {
        row$Groups <- NA_integer_
        row$`Marginal R^2` <- NA_real_
        row$`Conditional R^2` <- NA_real_
        row$ICC <- NA_real_
        if (model_type == "glmer") {
            row$`Brier Score` <- NA_real_
        }
        if (model_type == "lmer") {
            row$RMSE <- NA_real_
        }
    }
    
    return(row)
}


#' Build comparison row for successfully fitted model
#' 
#' Creates a comparison table row with extracted metrics for a successfully
#' fitted model.
#' 
#' @param model_name Character string name of the model.
#' @param n_predictors Integer number of predictors in the model.
#' @param converged Character string convergence status.
#' @param metrics Named list of extracted model metrics.
#' @param model_type Character string indicating model type.
#' @return Data.table with single row of formatted metrics.
#' @keywords internal
build_model_row <- function(model_name, n_predictors, converged, metrics, model_type, marks = NULL) {
    
    ## Base columns for all models
    row <- data.table::data.table(
                           Model = model_name,
                           N = metrics$n,
                           Events = metrics$events,
                           Predictors = n_predictors,
                           Converged = converged,
                           AIC = safe_round(metrics$aic, 1),
                           BIC = safe_round(metrics$bic, 1),
                           `Pseudo-R^2` = safe_round(metrics$pseudo_r2, 3),
                           Concordance = safe_round(metrics$concordance, 3),
                           `Global p` = if (!is.null(metrics$global_p) && !is.na(metrics$global_p)) {
                                            format_pvalues_fit(metrics$global_p, 3, marks)
                                        } else {
                                            NA_character_
                                        }
                       )
    
    ## Add GLM-specific columns
    if (model_type == "glm") {
        row$`Brier Score` <- safe_round(metrics$brier_score, 3)
    }
    
    ## Add mixed-effects specific columns
    if (model_type %in% c("lmer", "glmer", "coxme")) {
        row$Groups <- metrics$n_groups
        row$`Marginal R^2` <- safe_round(metrics$marginal_r2, 3)
        row$`Conditional R^2` <- safe_round(metrics$conditional_r2, 3)
        row$ICC <- safe_round(metrics$icc, 3)
        
        if (model_type == "glmer") {
            row$`Brier Score` <- safe_round(metrics$brier_score, 3)
        }
        if (model_type == "lmer") {
            row$RMSE <- safe_round(metrics$rmse, 3)
        }
    }
    
    return(row)
}


#' Safe rounding that handles \code{NULL} and NA
#' 
#' Rounds numeric values while gracefully handling \code{NULL}, empty, and NA inputs.
#' 
#' @param x Numeric value to round.
#' @param digits Integer number of decimal places.
#' @return Rounded numeric value, or NA_real_ if input is \code{NULL}/NA/empty.
#' @keywords internal
safe_round <- function(x, digits) {
    if (is.null(x) || length(x) == 0) return(NA_real_)
    if (is.na(x)) return(NA_real_)
    round(x, digits)
}


#' Order comparison columns based on model type
#' 
#' Reorders columns in the comparison table to follow a logical sequence
#' appropriate for the model type.
#' 
#' @param comparison Data.table with comparison metrics.
#' @param model_type Character string indicating model type.
#' @return Data.table with reordered columns.
#' @keywords internal
order_comparison_columns <- function(comparison, model_type) {
    
    col_order <- switch(model_type,
                        "glm" = c("Model", "N", "Events", "Predictors", "Converged",
                                  "AIC", "BIC", "Pseudo-R\u00b2", "Concordance", "Brier Score",
                                  "Global p", "CMS"),
                        "coxph" = c("Model", "N", "Events", "Predictors", "Converged",
                                    "AIC", "BIC", "Pseudo-R\u00b2", "Concordance",
                                    "Global p", "CMS"),
                        "lm" = c("Model", "N", "Predictors", "Converged",
                                 "AIC", "BIC", "Pseudo-R\u00b2", "Global p", "CMS"),
                        "lmer" = c("Model", "N", "Groups", "Predictors", "Converged",
                                   "AIC", "BIC", "Marginal R\u00b2", "Conditional R\u00b2", "ICC",
                                   "RMSE", "Global p", "CMS"),
                        "glmer" = c("Model", "N", "Events", "Groups", "Predictors", "Converged",
                                    "AIC", "BIC", "Concordance", "Marginal R\u00b2", "Conditional R\u00b2",
                                    "ICC", "Brier Score", "Global p", "CMS"),
                        "coxme" = c("Model", "N", "Events", "Groups", "Predictors", "Converged",
                                    "AIC", "BIC", "Concordance", "Pseudo-R\u00b2", "ICC",
                                    "Global p", "CMS"),
                        ## Default fallback
                        c("Model", "N", "Events", "Predictors", "Converged",
                          "AIC", "BIC", "Pseudo-R\u00b2", "Concordance",
                          "Global p", "CMS")
                        )
    
    existing_cols <- intersect(col_order, names(comparison))
    extra_cols <- setdiff(names(comparison), col_order)
    
    data.table::setcolorder(comparison, c(existing_cols, extra_cols))
    
    return(comparison)
}


### * Calculation/CMS functions

#' Calculate Composite Mean Scores (CMS) for model comparison
#' 
#' Computes composite Score based on weighted combination of model
#' quality metrics. Weights vary by model type to reflect academic consensus
#' on important metrics for each model class.
#' 
#' @param comparison Data.table with model comparison metrics.
#' @param model_type Character string indicating model type.
#' @param scoring_weights Optional named list of custom weights. If \code{NULL},
#'   uses default weights for the model type.
#' @return Data.table with CMS column added, sorted by score.
#' @keywords internal
calculate_model_scores <- function(comparison, model_type, scoring_weights = NULL) {
    
    ## Define default weights based on model type
    default_weights <- list(
        glm = list(convergence = 0.15, aic = 0.25, concordance = 0.40, 
                   pseudo_r2 = 0.15, brier = 0.05),
        coxph = list(convergence = 0.15, aic = 0.30, concordance = 0.40, 
                     global_p = 0.15),
        lm = list(convergence = 0.15, aic = 0.25, pseudo_r2 = 0.45, 
                  rmse = 0.15),
        ## Mixed-effects models
        lmer = list(convergence = 0.20, aic = 0.25, marginal_r2 = 0.25,
                    conditional_r2 = 0.15, icc = 0.15),
        glmer = list(convergence = 0.15, aic = 0.25, concordance = 0.30,
                     marginal_r2 = 0.15, icc = 0.15),
        coxme = list(convergence = 0.15, aic = 0.30, concordance = 0.35,
                     pseudo_r2 = 0.10, icc = 0.10)
    )
    
    ## Use provided weights or defaults
    if (is.null(scoring_weights)) {
        weights <- default_weights[[model_type]]
        if (is.null(weights)) {
            weights <- list(convergence = 0.20, aic = 0.40, bic = 0.40)
        }
    } else {
        weights <- scoring_weights
    }
    
    ## Validate weights sum to 1
    weight_sum <- sum(unlist(weights))
    if (abs(weight_sum - 1) > 0.01) {
        warning("Scoring weights do not sum to 1, normalizing...")
        weights <- lapply(weights, function(x) x / weight_sum)
    }
    
    ## Initialize scores with pre-allocation
    n_models <- nrow(comparison)
    scores <- list(
        conv_score = numeric(n_models),
        aic_score = numeric(n_models),
        concordance_score = numeric(n_models),
        pseudo_r2_score = numeric(n_models),
        brier_score = numeric(n_models),
        global_score = numeric(n_models),
        rmse_score = numeric(n_models),
        bic_score = numeric(n_models),
        marginal_r2_score = numeric(n_models),
        conditional_r2_score = numeric(n_models),
        icc_score = numeric(n_models),
        total = numeric(n_models)
    )
    
    ## Convergence score (universal)
    scores$conv_score <- data.table::fcase(
                                         comparison$Converged == "Yes", 100,
                                         comparison$Converged == "Suspect", 70,
                                         comparison$Converged == "No", 30,
                                         default = 0
                                     )
    
    ## AIC score (universal, lower is better)
    if (!all(is.na(comparison$AIC))) {
        aic_values <- comparison$AIC[!is.na(comparison$AIC)]
        if (length(aic_values) > 1) {
            aic_best <- min(aic_values)
            aic_worst <- max(aic_values)
            if (aic_worst - aic_best > 0) {
                scores$aic_score <- data.table::fifelse(
                                                    is.na(comparison$AIC), 0,
                                                    100 * (1 - (comparison$AIC - aic_best)/(aic_worst - aic_best))
                                                )
            } else {
                scores$aic_score <- rep(100, n_models)
            }
        } else {
            scores$aic_score <- data.table::fifelse(is.na(comparison$AIC), 0, 100)
        }
        scores$aic_score[is.na(scores$aic_score)] <- 0
    } else {
        scores$aic_score <- rep(50, n_models)
    }
    
    ## Model-specific scores
    if (model_type == "glm") {
        ## Concordance/C-statistic
        if ("Concordance" %chin% names(comparison) && !all(is.na(comparison$Concordance))) {
            scores$concordance_score <- data.table::fcase(
                                                        is.na(comparison$Concordance), 0,
                                                        comparison$Concordance <= 0.5, 0,
                                                        comparison$Concordance <= 0.6, 40 * (comparison$Concordance - 0.5)/0.1,
                                                        comparison$Concordance <= 0.7, 40 + 20 * (comparison$Concordance - 0.6)/0.1,
                                                        comparison$Concordance <= 0.8, 60 + 20 * (comparison$Concordance - 0.7)/0.1,
                                                        comparison$Concordance <= 0.9, 80 + 10 * (comparison$Concordance - 0.8)/0.1,
                                                        default = 90 + 10 * (comparison$Concordance - 0.9)/0.1
                                                    )
            scores$concordance_score <- pmin(100, scores$concordance_score)
        } else {
            scores$concordance_score <- rep(50, n_models)
        }
        
        ## Pseudo-R^2
        if ("Pseudo-R^2" %chin% names(comparison) && !all(is.na(comparison[["Pseudo-R^2"]]))) {
            ## McFadden's R^2 rarely exceeds 0.4 for good models
            scores$pseudo_r2_score <- data.table::fifelse(
                                                      is.na(comparison[["Pseudo-R^2"]]), 0,
                                                      pmin(100, comparison[["Pseudo-R^2"]] * 250)  # 0.4 -> 100 points
                                                  )
        } else {
            scores$pseudo_r2_score <- rep(50, n_models)
        }

        ## Brier score - only if column exists and weight is non-zero
        if ("Brier Score" %chin% names(comparison) && "brier" %chin% names(weights) && weights$brier > 0) {
            ## Lower is better (0 = perfect, 0.25 = no skill)
            scores$brier_score <- data.table::fifelse(
                                                  is.na(comparison$`Brier Score`), 50,
                                                  100 * (1 - comparison$`Brier Score`/0.25)
                                              )
        } else {
            scores$brier_score <- rep(50, n_models)
            weights$brier <- 0  # Set weight to 0 if not using
        }
        
        ## Normalize weights if Brier is excluded
        if (weights$brier == 0) {
            remaining_weights <- weights[names(weights) != "brier"]
            weight_sum <- sum(unlist(remaining_weights))
            remaining_weights <- lapply(remaining_weights, function(x) x / weight_sum)
            weights[names(remaining_weights)] <- remaining_weights
        }
        
        ## Calculate weighted total
        scores$total <- scores$conv_score * weights$convergence +
            scores$aic_score * weights$aic +
            scores$concordance_score * weights$concordance +
            scores$pseudo_r2_score * weights$pseudo_r2
        
    } else if (model_type == "coxph") {
        ## Similar adjustments for Cox models
        if ("Concordance" %chin% names(comparison) && !all(is.na(comparison$Concordance))) {
            scores$concordance_score <- data.table::fcase(
                                                        is.na(comparison$Concordance), 0,
                                                        comparison$Concordance <= 0.5, 0,
                                                        default = pmin(100, 200 * (comparison$Concordance - 0.5))
                                                    )
        } else {
            scores$concordance_score <- rep(50, n_models)
        }
        
        ## Global p-value score
        if ("Global p" %chin% names(comparison)) {
            global_numeric <- suppressWarnings(as.numeric(gsub("< ", "", comparison$`Global p`)))
            scores$global_score <- data.table::fcase(
                                                   is.na(global_numeric), 50,
                                                   global_numeric > 0.05, 50,
                                                   default = 50 + 50 * (0.05 - global_numeric)/0.05
                                               )
        } else {
            scores$global_score <- rep(50, n_models)
        }
        
        scores$total <- scores$conv_score * weights$convergence +
            scores$aic_score * weights$aic +
            scores$concordance_score * weights$concordance +
            scores$global_score * weights$global_p
        
    } else if (model_type == "lm") {
        ## R^2 scoring
        if ("Pseudo-R^2" %chin% names(comparison) && !all(is.na(comparison[["Pseudo-R^2"]]))) {
            scores$pseudo_r2_score <- comparison[["Pseudo-R^2"]] * 100
            scores$pseudo_r2_score[is.na(scores$pseudo_r2_score)] <- 0
        } else {
            scores$pseudo_r2_score <- rep(50, n_models)
        }
        
        ## RMSE scoring would go here if column exists
        scores$rmse_score <- rep(50, n_models)  # Placeholder
        
        scores$total <- scores$conv_score * weights$convergence +
            scores$aic_score * weights$aic +
            scores$pseudo_r2_score * weights$pseudo_r2 +
            scores$rmse_score * weights$rmse
        
    } else if (model_type == "lmer") {
        ## Linear mixed-effects models
        scores <- calculate_lmer_scores(comparison, weights, scores, n_models)
        
    } else if (model_type == "glmer") {
        ## Generalized linear mixed-effects models
        scores <- calculate_glmer_scores(comparison, weights, scores, n_models)
        
    } else if (model_type == "coxme") {
        ## Mixed-effects Cox models
        scores <- calculate_coxme_scores(comparison, weights, scores, n_models)
        
    } else {
        ## Generic fallback
        scores$bic_score <- rep(50, n_models)
        if (!all(is.na(comparison$BIC))) {
            bic_values <- comparison$BIC[!is.na(comparison$BIC)]
            if (length(bic_values) > 1) {
                bic_best <- min(bic_values)
                bic_worst <- max(bic_values)
                if (bic_worst - bic_best > 0) {
                    scores$bic_score <- 100 * (1 - (comparison$BIC - bic_best)/(bic_worst - bic_best))
                } else {
                    scores$bic_score <- rep(100, n_models)
                }
            } else {
                scores$bic_score <- rep(100, n_models)
            }
            scores$bic_score[is.na(scores$bic_score)] <- 0
        }
        
        scores$total <- scores$conv_score * weights$convergence +
            scores$aic_score * weights$aic +
            scores$bic_score * weights$bic
    }
    
    ## Add final score to comparison table using := for efficiency
    comparison[, CMS := round(scores$total, 1)]
    
    ## Sort by score (highest first)
    data.table::setorder(comparison, -CMS)
    
    ## Store detailed scores as attribute
    data.table::setattr(comparison, "detailed_scores", scores)
    data.table::setattr(comparison, "weights", weights)
    
    return(comparison)
}


#' Calculate scores for linear mixed-effects models
#' 
#' Computes component scores for lmer models based on marginal R-squared,
#' conditional R-squared, and ICC metrics.
#' 
#' @param comparison Data.table with model comparison metrics.
#' @param weights Named list of scoring weights.
#' @param scores List of initialized score vectors.
#' @param n_models Integer number of models being compared.
#' @return Updated scores list with calculated values and total.
#' @keywords internal
calculate_lmer_scores <- function(comparison, weights, scores, n_models) {
    
    ## Marginal R^2 score (variance explained by fixed effects)
    ## Higher is better, typically ranges 0-0.8 for good models
    marg_r2_col <- if ("Marginal R^2" %chin% names(comparison)) "Marginal R^2" else "Pseudo-R^2"
    if (marg_r2_col %chin% names(comparison) && !all(is.na(comparison[[marg_r2_col]]))) {
        scores$marginal_r2_score <- data.table::fifelse(
                                                    is.na(comparison[[marg_r2_col]]), 0,
                                                    pmin(100, comparison[[marg_r2_col]] * 125)  # 0.8 -> 100 points
                                                )
    } else {
        scores$marginal_r2_score <- rep(50, n_models)
    }
    
    ## Conditional R^2 score (variance explained by full model)
    if ("Conditional R^2" %chin% names(comparison) && !all(is.na(comparison[["Conditional R^2"]]))) {
        scores$conditional_r2_score <- data.table::fifelse(
                                                       is.na(comparison[["Conditional R^2"]]), 0,
                                                       pmin(100, comparison[["Conditional R^2"]] * 110)  # 0.9 -> ~100 points
                                                   )
    } else {
        scores$conditional_r2_score <- rep(50, n_models)
    }
    
    ## ICC score - moderate ICC (0.05-0.5) is typically good
    ## Too low suggests random effects unnecessary, too high suggests problems
    if ("ICC" %chin% names(comparison) && !all(is.na(comparison$ICC))) {
        scores$icc_score <- data.table::fcase(
                                            is.na(comparison$ICC), 50,
                                            comparison$ICC < 0.01, 30,    # Very low - random effects may not be needed
                                            comparison$ICC < 0.05, 60,    # Low but present
                                            comparison$ICC < 0.25, 100,   # Ideal range
                                            comparison$ICC < 0.50, 90,    # Still good
                                            comparison$ICC < 0.75, 70,    # High - check for model issues
                                            default = 50                  # Very high - potential problems
                                        )
    } else {
        scores$icc_score <- rep(50, n_models)
    }
    
    ## Calculate weighted total
    scores$total <- scores$conv_score * weights$convergence +
        scores$aic_score * weights$aic +
        scores$marginal_r2_score * weights$marginal_r2 +
        scores$conditional_r2_score * weights$conditional_r2 +
        scores$icc_score * weights$icc
    
    return(scores)
}


#' Calculate scores for generalized linear mixed-effects models
#' 
#' Computes component scores for glmer models based on concordance,
#' marginal R-squared, and ICC metrics.
#' 
#' @param comparison Data.table with model comparison metrics.
#' @param weights Named list of scoring weights.
#' @param scores List of initialized score vectors.
#' @param n_models Integer number of models being compared.
#' @return Updated scores list with calculated values and total.
#' @keywords internal
calculate_glmer_scores <- function(comparison, weights, scores, n_models) {
    
    ## Concordance score (same as GLM)
    if ("Concordance" %chin% names(comparison) && !all(is.na(comparison$Concordance))) {
        scores$concordance_score <- data.table::fcase(
                                                    is.na(comparison$Concordance), 0,
                                                    comparison$Concordance <= 0.5, 0,
                                                    comparison$Concordance <= 0.6, 40 * (comparison$Concordance - 0.5)/0.1,
                                                    comparison$Concordance <= 0.7, 40 + 20 * (comparison$Concordance - 0.6)/0.1,
                                                    comparison$Concordance <= 0.8, 60 + 20 * (comparison$Concordance - 0.7)/0.1,
                                                    comparison$Concordance <= 0.9, 80 + 10 * (comparison$Concordance - 0.8)/0.1,
                                                    default = 90 + 10 * (comparison$Concordance - 0.9)/0.1
                                                )
        scores$concordance_score <- pmin(100, scores$concordance_score)
    } else {
        scores$concordance_score <- rep(50, n_models)
    }
    
    ## Marginal R^2 score
    marg_r2_col <- if ("Marginal R^2" %chin% names(comparison)) "Marginal R^2" else "Pseudo-R^2"
    if (marg_r2_col %chin% names(comparison) && !all(is.na(comparison[[marg_r2_col]]))) {
        ## For GLMMs, McFadden-style R^2 is lower, scale accordingly
        scores$marginal_r2_score <- data.table::fifelse(
                                                    is.na(comparison[[marg_r2_col]]), 0,
                                                    pmin(100, comparison[[marg_r2_col]] * 200)  # 0.5 -> 100 points
                                                )
    } else {
        scores$marginal_r2_score <- rep(50, n_models)
    }
    
    ## ICC score (same as lmer)
    if ("ICC" %chin% names(comparison) && !all(is.na(comparison$ICC))) {
        scores$icc_score <- data.table::fcase(
                                            is.na(comparison$ICC), 50,
                                            comparison$ICC < 0.01, 30,
                                            comparison$ICC < 0.05, 60,
                                            comparison$ICC < 0.25, 100,
                                            comparison$ICC < 0.50, 90,
                                            comparison$ICC < 0.75, 70,
                                            default = 50
                                        )
    } else {
        scores$icc_score <- rep(50, n_models)
    }
    
    ## Calculate weighted total
    scores$total <- scores$conv_score * weights$convergence +
        scores$aic_score * weights$aic +
        scores$concordance_score * weights$concordance +
        scores$marginal_r2_score * weights$marginal_r2 +
        scores$icc_score * weights$icc
    
    return(scores)
}


#' Calculate scores for mixed-effects Cox models
#' 
#' Computes component scores for coxme models based on concordance,
#' pseudo-R-squared, and ICC metrics.
#' 
#' @param comparison Data.table with model comparison metrics.
#' @param weights Named list of scoring weights.
#' @param scores List of initialized score vectors.
#' @param n_models Integer number of models being compared.
#' @return Updated scores list with calculated values and total.
#' @keywords internal
calculate_coxme_scores <- function(comparison, weights, scores, n_models) {
    
    ## Concordance score
    if ("Concordance" %chin% names(comparison) && !all(is.na(comparison$Concordance))) {
        scores$concordance_score <- data.table::fcase(
                                                    is.na(comparison$Concordance), 0,
                                                    comparison$Concordance <= 0.5, 0,
                                                    default = pmin(100, 200 * (comparison$Concordance - 0.5))
                                                )
    } else {
        scores$concordance_score <- rep(50, n_models)
    }
    
    ## Pseudo-R^2 score
    if ("Pseudo-R^2" %chin% names(comparison) && !all(is.na(comparison[["Pseudo-R^2"]]))) {
        scores$pseudo_r2_score <- data.table::fifelse(
                                                  is.na(comparison[["Pseudo-R^2"]]), 0,
                                                  pmin(100, comparison[["Pseudo-R^2"]] * 250)
                                              )
    } else {
        scores$pseudo_r2_score <- rep(50, n_models)
    }
    
    ## ICC score
    if ("ICC" %chin% names(comparison) && !all(is.na(comparison$ICC))) {
        scores$icc_score <- data.table::fcase(
                                            is.na(comparison$ICC), 50,
                                            comparison$ICC < 0.01, 30,
                                            comparison$ICC < 0.05, 60,
                                            comparison$ICC < 0.25, 100,
                                            comparison$ICC < 0.50, 90,
                                            comparison$ICC < 0.75, 70,
                                            default = 50
                                        )
    } else {
        scores$icc_score <- rep(50, n_models)
    }
    
    ## Calculate weighted total
    scores$total <- scores$conv_score * weights$convergence +
        scores$aic_score * weights$aic +
        scores$concordance_score * weights$concordance +
        scores$pseudo_r2_score * weights$pseudo_r2 +
        scores$icc_score * weights$icc
    
    return(scores)
}

#' Package Imports
#'
#' Declares the imports that are called without a namespace prefix, so that
#' R CMD check does not report them as undefined globals. Functions called as
#' \code{package::function()} elsewhere in the package need no entry here, but
#' their package must still appear under Imports in DESCRIPTION.
#'
#' @name summata-imports
#' @import data.table
#' @importFrom stats sd var median quantile IQR
#' @importFrom stats logLik AIC BIC coef confint vcov
#' @importFrom stats chisq.test fisher.test t.test wilcox.test kruskal.test
#' @importFrom stats lm glm anova
#' @importFrom stats family binomial gaussian poisson
#' @importFrom stats predict fitted residuals
#' @importFrom stats formula terms model.frame model.matrix
#' @importFrom stats na.omit complete.cases
#' @importFrom stats cor pnorm qnorm
#' @importFrom stats nobs pchisq pf
#' @importFrom grDevices axisTicks
#' @importFrom utils head tail str capture.output
#' @importFrom survival Surv strata cluster coxph
#' @keywords internal
NULL

#' @noRd
`%||%` <- function(x, y) if (is.null(x)) y else x

### * Main function

#' Create Forest Plot for Univariable Screening
#'
#' Generates a publication-ready forest plot from a \code{uniscreen()} output
#' object. The plot displays effect estimates (OR, HR, RR, or coefficients) with
#' confidence intervals for each predictor tested in univariable analysis against
#' a single outcome.
#'
#' @param x Univariable screen result object (data.table with class attributes
#'   from \code{uniscreen()}).
#'   
#' @param title Character string specifying the plot title. Default is 
#'   \code{"Univariable Screening"}. Use descriptive titles for publication.
#'   
#' @param effect_label Character string for the effect measure label on the 
#'   forest plot axis. Default is \code{NULL}, which auto-detects based on 
#'   model type (\emph{e.g.,} "Odds Ratio", "Hazard Ratio", "Rate Ratio", "Coefficient").
#'   
#' @param digits Integer specifying the number of decimal places for effect 
#'   estimates and confidence intervals. Default is 2.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param conf_level Numeric confidence level for confidence intervals. Must be
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
#'   percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
#'   when \code{conf_level = 0.90}). Note: This parameter affects display only;
#'   the underlying CIs come from the uniscreen result.
#'   
#' @param font_size Numeric multiplier controlling the base font size for all 
#'   text elements. Default is 1.0.
#'   
#' @param annot_size Numeric value controlling the relative font size for data 
#'   annotations. Default is 3.88.
#'   
#' @param header_size Numeric value controlling the relative font size for column 
#'   headers. Default is 5.82.
#'   
#' @param title_size Numeric value controlling the relative font size for the 
#'   main plot title. Default is 23.28.
#'   
#' @param table_width Numeric value between 0 and 1 specifying the proportion of 
#'   total plot width allocated to the data table. Default is 0.6 (60\% table, 
#'   40\% forest plot).
#'   
#' @param plot_width Numeric value specifying the intended output width in 
#'   specified \code{units}. Used for optimizing layout. Default is \code{NULL} 
#'   (automatic). Recommended: 10-16 inches.
#'   
#' @param plot_height Numeric value specifying the intended output height in 
#'   specified \code{units}. Default is \code{NULL} (automatic based on number 
#'   of rows).
#'   
#' @param show_n Logical. If \code{TRUE}, includes a column showing sample sizes.
#'   Counts describe the observations used in fitting rather than every row
#'   supplied. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes a column showing the 
#'   number of events for each row. Default is \code{NULL}, which auto-detects
#'   based on model type (\code{TRUE} for binomial/survival, \code{FALSE} for linear).
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels
#'   under their parent variable name, creating a hierarchical display. If 
#'   \code{FALSE} (default), shows variable and level in separate columns.
#'
#' @param condense_table Logical. If \code{TRUE}, condenses binary categorical 
#'   variables into single rows by showing only the non-reference category. 
#'   Automatically sets \code{indent_groups = TRUE}. Useful for tables with 
#'   many binary variables. Default is \code{FALSE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. If \code{FALSE} (default), variable names are displayed in plain
#'   text.
#'   
#' @param center_padding Numeric value specifying horizontal spacing between 
#'   table and forest plot. Default is 4.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating gray 
#'   background shading to different variables for improved readability. 
#'   Default is \code{TRUE}.
#'   
#' @param color Character string specifying the color for point estimates in 
#'   the forest plot. Default is \code{NULL}, which auto-selects based on
#'   effect type: purple (\code{"#8A61D8"}) for hazard ratios (Cox), teal 
#'   (\code{"#4BA6B6"}) for odds ratios (logistic), blue (\code{"#3F87EE"}) for 
#'   rate/risk ratios (Poisson, Gamma, \emph{etc.}), and green (\code{"#5A8F5A"}) for 
#'   coefficients (linear models).
#'   Use hex codes or R color names for custom colors.
#'   
#' @param null_line Numeric value for the reference line position. Default is 
#'   \code{NULL}, which uses 1 for ratio measures (OR, HR, RR) and 0 for 
#'   coefficients.
#'   
#' @param log_scale Logical. If \code{TRUE}, uses log scale for the x-axis.
#'   Default is \code{NULL}, which auto-detects (\code{TRUE} for OR/HR/RR, \code{FALSE} for
#'   coefficients).
#'   
#' @param labels Named character vector providing custom display labels for 
#'   variables. Applied to predictor names in the plot.
#'   Default is \code{NULL}, in which case the labels attached by the function
#'   that produced \code{x} are used where \code{x} is a \pkg{summata} result
#'   rather than a model object, so that labels supplied once are carried
#'   through the workflow. Original variable names are used where neither is
#'   available.
#'
#' @param show_footer Logical. If \code{TRUE}, displays a footer with the
#'   outcome variable name. Default is \code{TRUE}.
#'   
#' @param units Character string specifying units for plot dimensions: 
#'   \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @return A \code{ggplot} object containing the complete forest plot. The plot 
#'   can be:
#'   \itemize{
#'     \item Displayed directly: \code{print(plot)}
#'     \item Saved to file: \code{forestsave(plot, "forest.pdf")}
#'     \item Further customized with \pkg{ggplot2} functions
#'   }
#'   
#'   The returned object includes an attribute \code{"rec_dims"} 
#'   accessible via \code{attr(plot, "rec_dims")}, which is a list 
#'   containing:
#'   \describe{
#'     \item{width}{Numeric. Recommended plot width in specified units}
#'     \item{height}{Numeric. Recommended plot height in specified units}
#'     \item{units}{Character. The units the dimensions are expressed in,
#'       matching the \code{units} argument}
#'   }
#'   
#'   These recommendations are automatically calculated based on the number of 
#'   variables, text sizes, and layout parameters, and are printed to console 
#'   if \code{plot_width} or \code{plot_height} are not specified.
#'   \code{forestsave()} reads all three and requires no further handling.
#'
#'   The returned object also includes an attribute \code{"table_data"}
#'   accessible via \code{attr(plot, "table_data")}, a data.table holding the
#'   values drawn in the plot, in the order of the model terms. Sample sizes
#'   and event counts describe the observations used in fitting rather than
#'   every row supplied.
#'
#' @details
#' The forest plot displays univariable (unadjusted) associations between each
#' predictor and the outcome. This is useful for:
#' \itemize{
#'   \item Visualizing results of initial variable screening
#'   \item Identifying potential predictors for multivariable modeling
#'   \item Presenting crude associations alongside adjusted results
#'   \item Quick visual assessment of effect sizes and significance
#' }
#'
#' The plot automatically handles:
#' \itemize{
#'   \item Different effect types (OR, HR, RR, coefficients) with appropriate
#'     axis scaling (log vs linear)
#'   \item Factor variables with multiple levels (grouped under variable name)
#'   \item Continuous variables (single row per predictor)
#'   \item Reference categories for categorical variables
#' }
#'
#' @seealso 
#' \code{\link{autoforest}} for automatic model detection,
#' \code{\link{uniscreen}} for generating univariable screening results,
#' \code{\link{multiforest}} for multi-outcome forest plots,
#' \code{\link{coxforest}}, \code{\link{glmforest}}, \code{\link{lmforest}} for
#' single-model forest plots,
#'   \code{\link{forestsave}} for saving with recommended dimensions
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example uniscreen result
#' uni_results <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "smoking", "treatment", "stage"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#'
#' # Example 1: Basic univariable forest plot
#' p <- uniforest(uni_results, title = "Univariable Associations with Mortality")
#'
#' \donttest{
#' 
#' old_width <- options(width = 180)
#' 
#' # Example 2: Survival analysis
#' library(survival)
#' surv_results <- uniscreen(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     model_type = "coxph",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' 
#' # Labels attached by uniscreen() are applied automatically
#' p2 <- uniforest(surv_results, title = "Univariable Survival Analysis")
#' 
#' # Example 3: Linear regression
#' lm_results <- uniscreen(
#'     data = clintrial,
#'     outcome = "los_days",
#'     predictors = c("age", "sex", "surgery", "diabetes"),
#'     model_type = "lm",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' 
#' p3 <- uniforest(lm_results, title = "Predictors of Length of Stay")
#' 
#' # Example 4: Customize appearance
#' p4 <- uniforest(
#'     uni_results,
#'     title = "Crude Associations with Mortality",
#'     color = "#E74C3C",
#'     indent_groups = TRUE,
#'     zebra_stripes = TRUE,
#'     bold_variables = TRUE
#' )
#' 
#' # Example 5: Save with recommended dimensions
#' forestsave(p4, file.path(tempdir(), "univariable_forest.pdf"))
#'
#' options(old_width)
#' 
#' }
#' @family visualization functions
#' @export
uniforest <- function(x,
                      title = "Univariable Screening",
                      effect_label = NULL,
                      digits = 2,
                      p_digits = 3,
                      conf_level = 0.95,
                      font_size = 1.0,
                      annot_size = 3.88,
                      header_size = 5.82,
                      title_size = 23.28,
                      plot_width = NULL,
                      plot_height = NULL,
                      table_width = 0.6,
                      show_n = TRUE,
                      show_events = NULL,
                      indent_groups = FALSE,
                      condense_table = FALSE,
                      bold_variables = FALSE,
                      center_padding = 4,
                      zebra_stripes = TRUE,
                      color = NULL,
                      null_line = NULL,
                      log_scale = NULL,
                      labels = NULL,
                      show_footer = TRUE,
                      units = "in",
                      number_format = NULL) {
    
    ## Input validation
    if (!requireNamespace("data.table", quietly = TRUE)) {
        stop("Package 'data.table' is required but not installed.")
    }
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required but not installed.")
    }
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    ## Check if user passed correct input type
    if (!inherits(x, "uniscreen_result")) {
        if (inherits(x, c("coxph", "glm", "lm", "glmerMod", "lmerMod", "coxme"))) {
            stop("uniforest() expects a uniscreen() result, not a model object.\n",
                 "\nFor single-model forest plots, use:\n",
                 "  - coxforest() for Cox models\n",
                 "  - glmforest() for GLM models\n",
                 "  - lmforest() for linear models\n",
                 "\nFor univariable screening visualization, first run uniscreen(), then pass the result to uniforest().")
        }
        if (inherits(x, "fit_result") || inherits(x, "fullfit_result")) {
            stop("uniforest() expects a uniscreen() result, not a fit()/fullfit() result.\n",
                 "\nFor fit_result/fullfit_result objects, use the appropriate forest function:\n",
                 "  - coxforest() for Cox models\n",
                 "  - glmforest() for GLM models\n",
                 "  - lmforest() for linear models")
        }
        stop("uniforest() expects a uniscreen() result.\n",
             "Input is of class: ", paste(class(x), collapse = ", "))
    }
    
    ## Extract raw data from uscreen result
    ## Labels attached by the producing function are used when none are given
    ## here, so that a labeled result does not need them supplied twice. An
    ## explicit argument takes precedence.
    if (is.null(labels)) {
        labels <- attr(x, "labels")
    }

    raw_data <- attr(x, "raw_data")
    
    if (is.null(raw_data) || !data.table::is.data.table(raw_data)) {
        stop("Input does not appear to be a valid uniscreen() result (missing raw_data attribute).")
    }
    
    ## Get attributes from uscreen result
    us_outcome <- attr(x, "outcome")
    us_model_type <- attr(x, "model_type")
    
    ## Determine effect type from raw data column names
    ## m2dt creates columns named OR, HR, RR, or Estimate
    if ("OR" %in% names(raw_data)) {
        effect_type <- "OR"
    } else if ("HR" %in% names(raw_data)) {
        effect_type <- "HR"
    } else if ("RR" %in% names(raw_data)) {
        effect_type <- "RR"
    } else {
        effect_type <- "Estimate"
    }
    
    ## Set defaults based on effect type
    if (is.null(null_line)) {
        null_line <- if (effect_type %in% c("OR", "HR", "RR")) 1 else 0
    }
    
    if (is.null(log_scale)) {
        log_scale <- effect_type %in% c("OR", "HR", "RR")
    }
    
    ## Set effect label based on effect type
    if (is.null(effect_label)) {
        effect_label <- switch(effect_type,
                               "OR" = "Odds Ratio",
                               "HR" = "Hazard Ratio",
                               "RR" = "Rate Ratio",
                               "Coefficient")
    }
    
    ## Set effect abbreviation for column header
    ## For univariable screening, use non-adjusted abbreviations
    effect_abbrev <- switch(effect_type,
                            "OR" = "OR",
                            "HR" = "HR",
                            "RR" = "RR",
                            "Coefficient")
    
    ## Format CI percentage for display in headers
    ci_pct <- round(conf_level * 100)
    
    ## Create header expression with italic p
    effect_header_expr <- paste0("bold('", effect_abbrev, " (", ci_pct, "% CI); '*bolditalic(p)*'-value')")
    
    ## Set default show_events based on model type
    if (is.null(show_events)) {
        show_events <- effect_type %in% c("OR", "HR", "RR")
    }
    
    ## Set default color based on effect type
    if (is.null(color)) {
        color <- switch(effect_type,
                        "HR" = "#8A61D8",
                        "OR" = "#4BA6B6",
                        "RR" = "#3F87EE",
                        "#5A8F5A")
    }
    
    ## Internally work in inches (uses convert_units from forest_utils.R)
    if (!is.null(plot_width) && units != "in") {
        plot_width <- convert_units(plot_width, from = units, to = "in")
    }
    
    ## Prepare data for plotting
    to_show <- data.table::copy(raw_data)
    
    ## Handle estimate column
    if ("coefficient" %in% names(to_show) && !"estimate" %in% names(to_show)) {
        to_show[, estimate := coefficient]
    }
    
    ## Handle CI columns - m2dt() uses ci_lower/ci_upper (exponentiated) or coef_lower/coef_upper (raw)
    ## Forest plots require the raw (log scale) values for log_scale models
    ## and raw values for linear models
    if (!"conf_low" %in% names(to_show)) {
        if ("coef_lower" %in% names(to_show)) {
            ## Use raw coefficient bounds if available (these are on log scale for GLM/Cox)
            to_show[, conf_low := coef_lower]
            to_show[, conf_high := coef_upper]
        } else if ("ci_lower" %in% names(to_show)) {
            ## ci_lower/ci_upper are exponentiated for GLM/Cox, raw for LM
            ## Use the effect_type detected earlier
            if (effect_type %in% c("OR", "HR", "RR")) {
                ## Log scale model - need to log the CIs back
                to_show[, conf_low := log(ci_lower)]
                to_show[, conf_high := log(ci_upper)]
            } else {
                ## Linear model - CIs are already on raw scale
                to_show[, conf_low := ci_lower]
                to_show[, conf_high := ci_upper]
            }
        }
    }
    
    ## Handle level column - m2dt might use 'group' instead
    if ("group" %in% names(to_show) && !"level" %in% names(to_show)) {
        to_show[, level := group]
    }
    
    ## Handle n columns - m2dt uses n_group/events_group for categorical, n/events for continuous
    ## First try n_group, then fall back to n for continuous variables
    if (!"n_obs" %in% names(to_show)) {
        if ("n_group" %in% names(to_show) && "n" %in% names(to_show)) {
            ## Use n_group where available, fall back to n where n_group is NA
            to_show[, n_obs := data.table::fifelse(is.na(n_group), n, n_group)]
        } else if ("n_group" %in% names(to_show)) {
            to_show[, n_obs := n_group]
        } else if ("n" %in% names(to_show)) {
            to_show[, n_obs := n]
        }
    }
    if (!"n_events" %in% names(to_show)) {
        if ("events_group" %in% names(to_show) && "events" %in% names(to_show)) {
            ## Use events_group where available, fall back to events where events_group is NA
            to_show[, n_events := data.table::fifelse(is.na(events_group), events, events_group)]
        } else if ("events_group" %in% names(to_show)) {
            to_show[, n_events := events_group]
        } else if ("events" %in% names(to_show)) {
            to_show[, n_events := events]
        }
    }
    
    ## Handle is_reference - m2dt uses 'reference' column with text, not boolean
    if ("reference" %in% names(to_show) && !"is_reference" %in% names(to_show)) {
        to_show[, is_reference := (reference != "" & !is.na(reference))]
    }
    
    ## Keep only relevant columns that exist
    keep_cols <- c("predictor", "term", "level", "estimate", "conf_low", "conf_high", 
                   "p_value", "n_obs", "n_events", "is_reference", "effect_type")
    keep_cols <- keep_cols[keep_cols %in% names(to_show)]
    to_show <- to_show[, ..keep_cols]
    
    ## Ensure required columns exist even if empty
    if (!"level" %in% names(to_show)) {
        to_show[, level := NA_character_]
    }
    if (!"n_obs" %in% names(to_show)) {
        to_show[, n_obs := NA_integer_]
    }
    if (!"n_events" %in% names(to_show)) {
        to_show[, n_events := NA_integer_]
    }
    if (!"estimate" %in% names(to_show)) {
        to_show[, estimate := NA_real_]
    }
    if (!"conf_low" %in% names(to_show)) {
        to_show[, conf_low := NA_real_]
    }
    if (!"conf_high" %in% names(to_show)) {
        to_show[, conf_high := NA_real_]
    }
    if (!"is_reference" %in% names(to_show)) {
        to_show[, is_reference := FALSE]
    }
    
    ## Apply labels to predictor names
    to_show[, var_display := predictor]
    if (!is.null(labels)) {
        for (pred in unique(to_show$predictor)) {
            if (pred %in% names(labels)) {
                to_show[predictor == pred, var_display := labels[[pred]]]
            }
        }
    }
    
    ## Create display columns - handle case where level might be missing or NA
    to_show[, level_display := data.table::fifelse(
                                               is.na(level) | level == "" | level == "-", 
                                               "-", 
                                               as.character(level)
                                           )]
    
    ## Assign variable order for grouping and shading
    predictor_order <- unique(to_show$predictor)
    to_show[, var_order := match(predictor, predictor_order)]
    
    ## Calculate N and events formatting
    to_show[, N := n_obs]
    to_show[, n_formatted := format_count(n_obs, marks, na = "")]
    to_show[, events_formatted := format_count(n_events, marks, na = "")]
    
    ## Format the values for display based on log_scale
    to_show_exp_clean <- data.table::copy(to_show)
    
    if (log_scale) {
        ## Log scale (OR, HR, RR): exponentiate values
        to_show_exp_clean[, hr := data.table::fifelse(is.na(estimate), NA_real_, exp(estimate))]
        to_show_exp_clean[, hr_formatted := data.table::fifelse(
                                                            is.na(estimate), "",
                                                            format_number(exp(estimate), digits, marks)
                                                        )]
        to_show_exp_clean[, conf_low_formatted := data.table::fifelse(
                                                                  is.na(conf_low), "",
                                                                  format_number(exp(conf_low), digits, marks)
                                                              )]
        to_show_exp_clean[, conf_high_formatted := data.table::fifelse(
                                                                   is.na(conf_high), "",
                                                                   format_number(exp(conf_high), digits, marks)
                                                               )]
    } else {
        ## Linear scale: use raw coefficients
        to_show_exp_clean[, hr := data.table::fifelse(is.na(estimate), NA_real_, estimate)]
        to_show_exp_clean[, hr_formatted := data.table::fifelse(
                                                            is.na(estimate), "",
                                                            format_number(estimate, digits, marks)
                                                        )]
        to_show_exp_clean[, conf_low_formatted := data.table::fifelse(
                                                                  is.na(conf_low), "",
                                                                  format_number(conf_low, digits, marks)
                                                              )]
        to_show_exp_clean[, conf_high_formatted := data.table::fifelse(
                                                                   is.na(conf_high), "",
                                                                   format_number(conf_high, digits, marks)
                                                               )]
    }
    
    ## Format \emph{p}-values
    p_threshold <- 10^(-p_digits)
    p_threshold_str <- if (!is.null(marks)) {
                           paste0("< 0", marks$decimal.mark, strrep("0", p_digits - 1), "1")
                       } else {
                           paste0("< ", format(p_threshold, scientific = FALSE))
                       }
    
    to_show_exp_clean[, p_formatted := data.table::fifelse(
                                                       is.na(p_value), "",
                                                       data.table::fifelse(p_value < p_threshold, p_threshold_str,
                                                                           format_number(p_value, p_digits, marks))
                                                   )]
    
    ## Determine if ANY coefficient or CI bound is negative (only relevant for linear scale)
    ## If so, use comma notation throughout for consistency
    use_comma_notation <- !log_scale && any(to_show_exp_clean$conf_low < 0 | to_show_exp_clean$conf_high < 0, na.rm = TRUE)
    ci_separator <- forest_ci_separator(use_comma_notation, marks)
    
    ## Create effect string with italic p
    to_show_exp_clean[, hr_string_expr := data.table::fifelse(
                                                          is.na(estimate) | is_reference == TRUE,
                                                          "''",
                                                          data.table::fcase(
                                                              p_value < p_threshold,
                                                              paste0("'", hr_formatted, " (", conf_low_formatted, ci_separator, conf_high_formatted, 
                                                                     "); '*~italic(p)~'", p_threshold_str, "'"),
                                                              
                                                              default = paste0("'", hr_formatted, " (", conf_low_formatted, ci_separator, conf_high_formatted, 
                                                                               "); '*~italic(p)~'= ", p_formatted, "'")
                                                          )
                                                      )]
    
    ## Handle reference rows
    to_show_exp_clean[is_reference == TRUE, hr_string_expr := "'reference'"]
    
    ## Handle condense_table: collapse binary variables to single rows
    if (condense_table) {
        indent_groups <- TRUE  # condense_table implies indent_groups
        
        ## Identify binary predictors (exactly 2 rows)
        predictor_counts <- to_show_exp_clean[, .N, by = predictor]
        binary_predictors <- predictor_counts[N == 2]$predictor
        
        if (length(binary_predictors) > 0) {
            ## Process each binary predictor
            rows_to_remove <- integer()
            
            for (pred in binary_predictors) {
                pred_rows <- to_show_exp_clean[predictor == pred]
                
                ## Find reference and non-reference rows
                ref_idx <- which(pred_rows$is_reference == TRUE)
                non_ref_idx <- which(pred_rows$is_reference == FALSE)
                
                if (length(ref_idx) == 1 && length(non_ref_idx) == 1) {
                    ref_category <- pred_rows$level[ref_idx]
                    non_ref_category <- pred_rows$level[non_ref_idx]
                    
                    ## Get label for this predictor
                    var_label <- pred_rows$var_display[1]
                    
                    ## Use a greedy approach to determine whether to condense
                    if (should_condense_binary(ref_category, non_ref_category, var_label)) {
                        ## Keep the non-reference row but update display
                        to_show_exp_clean[predictor == pred & is_reference == FALSE, 
                                          var_display := var_label]
                    } else {
                        ## Append category name
                        to_show_exp_clean[predictor == pred & is_reference == FALSE, 
                                          var_display := paste0(var_label, " (", non_ref_category, ")")]
                    }
                    
                    ## Clear the level for condensed row
                    to_show_exp_clean[predictor == pred & is_reference == FALSE, level := "-"]
                    to_show_exp_clean[predictor == pred & is_reference == FALSE, level_display := "-"]
                    
                    ## Mark reference row for removal
                    ref_row_idx <- which(to_show_exp_clean$predictor == pred & 
                                         to_show_exp_clean$is_reference == TRUE)
                    rows_to_remove <- c(rows_to_remove, ref_row_idx)
                }
            }
            
            ## Remove reference rows of binary predictors
            if (length(rows_to_remove) > 0) {
                to_show_exp_clean <- to_show_exp_clean[-rows_to_remove]
            }
        }
    }
    
    ## Zebra stripe shading by variable
    if (zebra_stripes) {
        shade_colors <- c("0" = "#EEEEEE", "1" = "#FFFFFF")
        to_show_exp_clean[, shade_group := (var_order + 1) %% 2]
    } else {
        shade_colors <- c("0" = "#FFFFFF", "1" = "#FFFFFF")
        to_show_exp_clean[, shade_group := 0]
    }
    
    ## Handle indented display vs separate columns
    if (indent_groups || condense_table) {
        ## Create combined display with variable name as header, levels indented
        ## First, identify header rows (first occurrence of each predictor)
        to_show_exp_clean[, is_header := !duplicated(predictor)]
        
        ## Determine if each row is for a continuous variable (level is "-" or NA)
        to_show_exp_clean[, is_continuous := level_display == "-" | is.na(level_display)]
        
        ## For header rows of multi-level variables, show variable name (bold)
        ## For subsequent rows of multi-level variables, indent the level
        ## For continuous variables, show just the variable name
        to_show_exp_clean[, display_text := data.table::fifelse(
                                                            is_continuous,
                                                            var_display,  # Continuous variable: just name
                                                            data.table::fifelse(
                                                                            is_header,
                                                                            var_display,  # First row of categorical: variable name
                                                                            paste0("    ", level_display)  # Subsequent rows: indented level
                                                                        )
                                                        )]
        
        ## For multi-level variables, insert a header row if needed
        ## Check which predictors have multiple rows (categorical variables)
        multi_level_predictors <- to_show_exp_clean[, .N, by = predictor][N > 1]$predictor
        
        ## Record the incoming row order. Rows arrive in the order of the
        ## model terms, which follows the factor level order, and that order
        ## must survive the interleaving of header rows below.
        to_show_exp_clean[, level_order := .I]
        
        ## Create new data for header rows
        if (length(multi_level_predictors) > 0) {
            header_rows <- to_show_exp_clean[predictor %in% multi_level_predictors & is_header == TRUE]
            header_rows <- data.table::copy(header_rows)
            header_rows[, `:=`(
                display_text = var_display,
                estimate = NA_real_,
                conf_low = NA_real_,
                conf_high = NA_real_,
                p_value = NA_real_,
                hr = NA_real_,
                hr_string_expr = "''",
                n_formatted = "",
                events_formatted = "",
                is_header = TRUE,
                is_continuous = FALSE,
                row_type = "header"
            )]
            
            ## Mark data rows
            to_show_exp_clean[, row_type := "data"]
            to_show_exp_clean[predictor %in% multi_level_predictors, display_text := paste0("    ", level_display)]
            to_show_exp_clean[predictor %in% multi_level_predictors, is_continuous := FALSE]
            
            ## Combine and sort
            to_show_exp_clean <- rbind(header_rows, to_show_exp_clean, fill = TRUE)
            
            ## Sort by variable, then header before its levels, then the
            ## incoming order within each variable. Sorting on
            ## level_display here would order the levels alphabetically and
            ## discard the factor level order established by the model.
            to_show_exp_clean[, sort_key := data.table::fifelse(row_type == "header", 0, 1)]
            data.table::setorder(to_show_exp_clean, var_order, sort_key, level_order)
        } else {
            to_show_exp_clean[, row_type := "data"]
        }
        
        ## Determine which rows should be bold: headers and continuous variables
        to_show_exp_clean[, should_bold := (row_type == "header") | (is_continuous == TRUE)]
    } else {
        ## Non-indented display: variable and level in separate columns
        to_show_exp_clean[, display_text := var_display]
        to_show_exp_clean[duplicated(predictor), display_text := ""]
        to_show_exp_clean[, is_header := FALSE]
        to_show_exp_clean[, is_continuous := FALSE]
        to_show_exp_clean[, should_bold := TRUE]  # All variable names bold in non-indented mode
        to_show_exp_clean[, row_type := "data"]
    }
    
    ## Handle missing estimates for plotting
    to_show_exp_clean[is.na(estimate), estimate := 0]
    
    ## Reorder (flip) - maintain variable grouping
    ## Retain the assembled table for return as an attribute. The copy is taken
    ## before the row order is reversed for plotting, so that the returned table
    ## follows the order of the model terms rather than the drawing order.
    table_data <- data.table::copy(to_show_exp_clean)
    
    to_show_exp_clean <- to_show_exp_clean[order(rev(seq_len(nrow(to_show_exp_clean))))]
    to_show_exp_clean[, x_pos := .I]
    
    ## Calculate plot ranges
    if (log_scale) {
        rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
        min_ci <- min(to_show$conf_low, na.rm = TRUE)
        max_ci <- max(to_show$conf_high, na.rm = TRUE)
        
        ## Intelligent tick selection for log scale
        range_magnitude <- diff(rangeb)
        
        if (exp(min_ci) < 0.01 && exp(max_ci) > 2) {
            breaks <- c(0.01, 0.1, 0.5, 1, 2, 5)
        } else if (range_magnitude > 3) {
            all_breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
            if (length(all_breaks) > 7) {
                important <- c(1)
                if (min(all_breaks) < 0.5) important <- c(min(all_breaks), important)
                if (max(all_breaks) > 2) important <- c(important, max(all_breaks))
                other_breaks <- setdiff(all_breaks, important)
                if (length(other_breaks) > 3) {
                    keep_idx <- round(seq(1, length(other_breaks), length.out = 3))
                    other_breaks <- other_breaks[keep_idx]
                }
                breaks <- sort(unique(c(important, other_breaks)))
            } else {
                breaks <- all_breaks
            }
        } else {
            breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
        }
        
        if (!1 %in% breaks) breaks <- sort(unique(c(breaks, 1)))
        
        if (min_ci > 0) {
            rangeb[1] <- log(0.9)
        } else if (max_ci < 0) {
            rangeb[2] <- log(1.1)
        }
        
        breaks <- breaks[breaks >= exp(rangeb[1]) & breaks <= exp(rangeb[2])]
        reference_value <- 1
    } else {
        ## Linear scale
        rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
        range_width <- diff(rangeb)
        rangeb[1] <- rangeb[1] - range_width * 0.05
        rangeb[2] <- rangeb[2] + range_width * 0.05
        
        if (rangeb[1] > 0) rangeb[1] <- -0.1 * abs(rangeb[2])
        if (rangeb[2] < 0) rangeb[2] <- 0.1 * abs(rangeb[1])
        
        breaks <- pretty(rangeb, n = 7)
        breaks <- breaks[breaks >= rangeb[1] & breaks <= rangeb[2]]
        
        if (!0 %in% breaks) breaks <- sort(unique(c(breaks, 0)))
        
        reference_value <- 0
    }
    
    ## Calculate layout (note: condense_table implies indent_groups)
    layout <- calculate_uniforest_layout(
        to_show_exp_clean = to_show_exp_clean,
        show_n = show_n,
        show_events = show_events,
        indent_groups = (indent_groups || condense_table),
        table_width = table_width,
        center_padding = center_padding,
        effect_abbrev = effect_abbrev,
        font_size = font_size,
        log_scale = log_scale,
        rangeb = rangeb,
        ci_pct = ci_pct
    )

    ## Set up column positions
    y_variable <- layout$positions$variable
    if (!(indent_groups || condense_table)) {
        y_level <- layout$positions$level
    }
    if (show_n) {
        y_n <- layout$positions$n
    }
    if (show_events) {
        y_events <- layout$positions$events
    }
    y_effect <- layout$positions$effect

    rangeplot <- c(layout$rangeplot_start, rangeb[2] + diff(rangeb) * 0.05)

    ## Calculate recommended dimensions
    rec_height <- max(5, min(20, 3 + nrow(to_show_exp_clean) * 0.25))

    if (!is.null(plot_width)) {
        rec_width <- plot_width
        if (!is.null(plot_height)) {
            rec_height <- plot_height
        }
    } else {
        rec_width <- layout$total_width + 1.0
        rec_width <- max(10, min(20, rec_width))
    }

    ## Font sizes
    annot_font <- font_size * annot_size
    header_font <- font_size * header_size

    ## Custom ticks - for log scale, breaks are on exponentiated scale (0.5, 1, 2)
    ## For linear scale, breaks are on raw scale (-2, -1, 0, 1, 2)
    ticks_df <- data.frame(
        x = -0.5,
        xend = -0.7,
        y = breaks,  # Position on the plot (already transformed for log scale)
        yend = breaks,
        label = sprintf("%g", breaks)
    )

    ## Create filtered data for forest plot elements (exclude header rows)
    plot_data <- if ("row_type" %in% names(to_show_exp_clean)) {
                     to_show_exp_clean[row_type == "data" | is.na(row_type)]
                 } else {
                     to_show_exp_clean
                 }

    ## Define coordinate transformation
    tfm <- if (log_scale) exp else identity

    ## Create the plot
    p <- ggplot2::ggplot(to_show_exp_clean, ggplot2::aes(x_pos, tfm(estimate))) +
        
        ## Shading rectangles
        ggplot2::geom_rect(ggplot2::aes(xmin = x_pos - .5, xmax = x_pos + .5,
                                        ymin = tfm(rangeplot[1]), ymax = tfm(rangeplot[2]),
                                        fill = ordered(shade_group))) +
        ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.10, 0.05))) +
        ggplot2::scale_size_area(max_size = 6, guide = "none") +
        ggplot2::scale_fill_manual(values = shade_colors, guide = "none") +
        
        ## Forest plot elements
        ggplot2::geom_point(data = plot_data[!is.na(hr) & is_reference != TRUE], 
                            ggplot2::aes(size = N), pch = 22, color = "#000000", fill = color, na.rm = TRUE) +
        ggplot2::geom_errorbar(data = plot_data[!is.na(hr) & is_reference != TRUE], 
                               ggplot2::aes(ymin = tfm(conf_low), ymax = tfm(conf_high)), 
                               width = 0.15, na.rm = TRUE) +
        
        ## Y-axis for forest plot
        ggplot2::annotate(geom = "segment",
                          x = -0.5, xend = -0.5,
                          y = tfm(rangeb[1]), yend = tfm(rangeb[2]),
                          color = "#000000", linewidth = 1) +
        
        ## Reference line
        ggplot2::annotate(geom = "segment",
                          x = -0.5, xend = max(to_show_exp_clean$x_pos) + 0.5,
                          y = reference_value, yend = reference_value,
                          linetype = "longdash") +
        
        ## Ticks
        ggplot2::geom_segment(data = ticks_df,
                              ggplot2::aes(x = x, xend = xend, y = y, yend = yend),
                              inherit.aes = FALSE, color = "#000000", linewidth = 1) +
        ggplot2::geom_text(data = ticks_df,
                           ggplot2::aes(x = xend - 0.05, y = y, label = label),
                           inherit.aes = FALSE, hjust = 0.5, vjust = 1.3,
                           size = annot_font * 1.5) +
        
        ## Coordinate system
        ggplot2::coord_flip(ylim = tfm(rangeplot)) +
        ggplot2::ggtitle(title) +
        
        ## Y-axis scale
        {if (log_scale) {
             ggplot2::scale_y_log10(name = effect_label,
                                    labels = sprintf("%g", breaks),
                                    expand = c(0.02, 0.02),
                                    breaks = breaks)
         } else {
             ggplot2::scale_y_continuous(name = effect_label,
                                         labels = sprintf("%g", breaks),
                                         expand = c(0.02, 0.02),
                                         breaks = breaks)
         }} +
        
        ggplot2::theme_light(base_family = detect_plot_font()) +
        ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 0, b = 0, l = 0),
                       panel.grid.minor.y = ggplot2::element_blank(),
                       panel.grid.minor.x = ggplot2::element_blank(),
                       panel.grid.major.y = ggplot2::element_blank(),
                       panel.grid.major.x = ggplot2::element_blank(),
                       legend.position = "none",
                       panel.border = ggplot2::element_blank(),
                       axis.title.y = ggplot2::element_blank(),
                       axis.text.y = ggplot2::element_blank(),
                       axis.ticks.y = ggplot2::element_blank(),
                       axis.title.x = ggplot2::element_blank(),
                       axis.ticks.x = ggplot2::element_blank(),
                       axis.text.x = ggplot2::element_blank(),
                       plot.title = ggplot2::element_text(size = font_size * title_size, 
                                                          face = "bold", hjust = 0.5)) +
        ggplot2::xlab("") +
        
        ## Variable column header
        ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_variable),
                          label = "Variable", fontface = "bold", hjust = 0, size = header_font) +
        
        ## Variable column content
        {if (indent_groups) {
             ## Use should_bold column, but respect bold_variables parameter
             fontfaces <- if (bold_variables) {
                              data.table::fifelse(to_show_exp_clean$should_bold, "bold", "plain")
                          } else {
                              rep("plain", nrow(to_show_exp_clean))
                          }
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_variable),
                               label = to_show_exp_clean$display_text,
                               fontface = fontfaces,
                               hjust = 0, size = annot_font)
         } else {
             ## Non-indented: bold all variable names if bold_variables is TRUE
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_variable),
                               label = to_show_exp_clean$display_text, 
                               fontface = if (bold_variables) "bold" else "plain",
                               hjust = 0, size = annot_font)
         }} +
        
        ## Level column (only when not indented or condensed)
        {if (!(indent_groups || condense_table)) {
             list(
                 ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_level),
                                   label = "Group", fontface = "bold", hjust = 0, size = header_font),
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_level),
                                   label = to_show_exp_clean$level_display, hjust = 0, size = annot_font)
             )
         }} +
        
        ## N column
        {if (show_n) {
             list(
                 ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_n),
                                   label = "n", fontface = "bold.italic", hjust = 0.5, size = header_font),
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_n),
                                   label = to_show_exp_clean$n_formatted, hjust = 0.5, size = annot_font)
             )
         }} +
        
        ## Events column
        {if (show_events) {
             list(
                 ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_events),
                                   label = "Events", fontface = "bold", hjust = 0.5, size = header_font),
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_events),
                                   label = to_show_exp_clean$events_formatted, hjust = 0.5, size = annot_font)
             )
         }} +
        
        ## Effect column header (with italic p)
        ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_effect),
                          label = effect_header_expr, hjust = 0, size = header_font, parse = TRUE) +
        
        ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_effect),
                          label = to_show_exp_clean$hr_string_expr, hjust = 0,
                          size = annot_font, parse = TRUE) +
        
        ## X-axis label
        ggplot2::annotate(geom = "text", x = -1.5, y = reference_value,
                          label = effect_label, fontface = "bold",
                          hjust = 0.5, vjust = 2, size = annot_font * 1.5) +
        
        ## Outcome footer (conditional)
        {if (show_footer && !is.null(us_outcome)) {
             ## Apply label to outcome if available
             outcome_display <- us_outcome
             if (!is.null(labels)) {
                 ## For simple outcomes, check directly
                 if (us_outcome %in% names(labels)) {
                     outcome_display <- labels[[us_outcome]]
                 } else {
                     ## For Surv() outcomes, try to extract and label component variables
                     ## \emph{e.g.,}, "Surv(os_months, os_status)" -> check for os_months, os_status labels
                     if (grepl("^Surv\\(", us_outcome)) {
                         ## Extract variable names from Surv()
                         surv_vars <- gsub("^Surv\\(|\\)$", "", us_outcome)
                         surv_vars <- trimws(strsplit(surv_vars, ",")[[1]])
                         
                         ## Apply labels to each component
                         labeled_vars <- sapply(surv_vars, function(v) {
                             if (v %in% names(labels)) labels[[v]] else v
                         })
                         outcome_display <- paste0("Surv(", paste(labeled_vars, collapse = ", "), ")")
                     }
                 }
             }
             outcome_text <- paste0("Outcome: ", outcome_display)
             ggplot2::annotate(geom = "text", x = 0.25, y = tfm(y_variable),
                               label = outcome_text,
                               size = annot_font, hjust = 0, vjust = 1.2, fontface = "italic")
         }}

    ## Convert units back for output if needed
    if (units != "in") {
        rec_width <- convert_units(rec_width, from = "in", to = units)
        rec_height <- convert_units(rec_height, from = "in", to = units)
    }

    ## Provide dimension recommendations
    if (is.null(plot_width) || is.null(plot_height)) {
        message(sprintf("Recommended plot dimensions: width = %.1f %s, height = %.1f %s",
                        rec_width, units, rec_height, units))
    }

    ## Add recommended dimensions as attribute
    ## The units are recorded alongside the dimensions. Both are converted to
    ## the requested units above, so a consumer that assumed inches would
    ## otherwise mis-size the output by the conversion factor.
    attr(p, "rec_dims") <- list(width = rec_width, height = rec_height,
                                units = units)

    ## Add the assembled table as an attribute
    attr(p, "table_data") <- table_data

    return(p)
}


### * Layout formatting

#' Calculate table layout for uniforest plots
#' 
#' Internal function to determine column positions and widths for forest plot
#' table section. Positions are calculated in the same units as the data
#' (log scale for OR/HR/RR, linear for coefficients).
#' 
#' @param to_show_exp_clean Data.table with formatted data for plotting.
#' @param show_n Logical whether to include n column.
#' @param show_events Logical whether to include events column.
#' @param indent_groups Logical whether levels are indented.
#' @param table_width Proportion of width for table.
#' @param center_padding Padding between table and forest.
#' @param effect_abbrev Effect type abbreviation.
#' @param font_size Font size multiplier.
#' @param log_scale Logical whether using log scale.
#' @param rangeb Numeric vector with plot range bounds (in data units).
#' 
#' @return List with column positions, widths, and layout parameters.
#' 
#' @keywords internal
calculate_uniforest_layout <- function(to_show_exp_clean,
                                       show_n,
                                       show_events,
                                       indent_groups,
                                       table_width,
                                       center_padding,
                                       effect_abbrev,
                                       font_size,
                                       log_scale,
                                       rangeb,
                                       ci_pct = 95) {
    
    ## Character to inch conversion (approximate)
    char_to_inch <- 0.08 * font_size
    margin <- 0.15  # Increased margin for better separation between columns
    
    ## Calculate column widths in inches
    columns <- list()
    
    ## Variable column width
    var_width_chars <- max(nchar(as.character(to_show_exp_clean$display_text)), nchar("Variable"), na.rm = TRUE)
    columns$variable <- var_width_chars * char_to_inch
    
    ## Level column (only if not indented)
    if (!indent_groups) {
        level_width_chars <- max(nchar(as.character(to_show_exp_clean$level_display)), nchar("Group"), na.rm = TRUE)
        columns$level <- level_width_chars * char_to_inch
    }
    
    ## N column
    if (show_n) {
        n_width_chars <- max(nchar(to_show_exp_clean$n_formatted), nchar("n"), na.rm = TRUE)
        columns$n <- n_width_chars * char_to_inch
    }
    
    ## Events column
    if (show_events) {
        events_width_chars <- max(nchar(to_show_exp_clean$events_formatted), nchar("Events"), na.rm = TRUE)
        columns$events <- events_width_chars * char_to_inch
    }
    
    ## Effect column - use dynamic CI percentage
    effect_header <- paste0(effect_abbrev, " (", ci_pct, "% CI); p-value")
    effect_lengths <- nchar(paste0(
        to_show_exp_clean$hr_formatted, " (",
        to_show_exp_clean$conf_low_formatted, "-",
        to_show_exp_clean$conf_high_formatted, "); p = ",
        to_show_exp_clean$p_formatted
    ))
    effect_width_chars <- max(effect_lengths, nchar(effect_header), na.rm = TRUE) + center_padding
    columns$effect <- effect_width_chars * char_to_inch
    
    ## Calculate total table width in inches
    calc_table_width <- sum(unlist(columns)) + length(columns) * margin * 2
    
    ## Calculate forest width based on table_width proportion
    calc_forest_width <- calc_table_width * ((1 - table_width) / table_width)
    
    ## Convert table width to data scale units
    ## rangeb is in data units (log scale for OR/HR/RR, linear for coefficients)
    calc_range_width <- diff(rangeb)
    calc_table_width_data_units <- (calc_table_width / calc_forest_width) * calc_range_width
    
    ## Calculate positions in data scale units
    ## Start from the left edge (rangeb[1] minus table width)
    rangeplot_start <- rangeb[1] - calc_table_width_data_units
    
    ## Convert inch measurements to data scale units
    inch_to_data <- calc_range_width / calc_forest_width
    
    positions <- list()
    current_pos <- rangeplot_start
    
    for (name in names(columns)) {
        if (name == "events") {
            current_pos <- current_pos + margin * inch_to_data * 3.5
        } else {
            current_pos <- current_pos + margin * inch_to_data
        }
        positions[[name]] <- current_pos
        current_pos <- current_pos + columns[[name]] * inch_to_data + margin * inch_to_data
    }
    
    return(list(
        calc_table_width = calc_table_width,
        calc_forest_width = calc_forest_width,
        positions = positions,
        rangeplot_start = rangeplot_start,
        total_width = calc_table_width + calc_forest_width
    ))
}

## Labels supplied to a fitting function travel with its result, so that the
## forest plot functions apply the same labels without being given them again.

label_fixture <- function() {
    env <- new.env(parent = emptyenv())
    utils::data("clintrial_labels", package = "summata", envir = env)
    env$clintrial_labels
}


test_that("fitting functions attach the labels they were given", {

    dt <- data.table::as.data.table(fresh_clintrial())
    labs <- label_fixture()

    fitted <- fit(dt, "readmission_30d", c("age", "sex"),
                  model_type = "glm", family = "binomial", labels = labs)
    screened <- uniscreen(dt, "readmission_30d", c("age", "sex"),
                          model_type = "glm", family = "binomial",
                          labels = labs, parallel = FALSE)

    expect_identical(attr(fitted, "labels"), labs)
    expect_identical(attr(screened, "labels"), labs)

    ## A result built without labels carries none
    plain <- fit(dt, "readmission_30d", c("age", "sex"),
                 model_type = "glm", family = "binomial")
    expect_null(attr(plain, "labels"))
})


test_that("forest plots inherit labels from the result they are given", {

    dt <- data.table::as.data.table(fresh_clintrial())
    labs <- label_fixture()

    screened <- uniscreen(dt, "readmission_30d", c("age", "sex"),
                          model_type = "glm", family = "binomial",
                          labels = labs, parallel = FALSE)

    inherited <- suppressMessages(suppressWarnings(uniforest(screened)))
    explicit <- suppressMessages(suppressWarnings(
        uniforest(screened, labels = labs)))

    ## The two are equivalent, which is the point of the inheritance
    expect_identical(attr(inherited, "table_data"),
                     attr(explicit, "table_data"))
})


test_that("an explicit labels argument overrides the inherited one", {

    dt <- data.table::as.data.table(fresh_clintrial())
    labs <- label_fixture()
    override <- c(age = "AGE OVERRIDE", sex = "SEX OVERRIDE")

    screened <- uniscreen(dt, "readmission_30d", c("age", "sex"),
                          model_type = "glm", family = "binomial",
                          labels = labs, parallel = FALSE)

    p <- suppressMessages(suppressWarnings(
        uniforest(screened, labels = override)))

    tbl <- attr(p, "table_data")
    skip_if(is.null(tbl), "uniforest does not expose table_data")

    expect_true(any(grepl("AGE OVERRIDE", unlist(tbl), fixed = TRUE)))
})

## Factor levels are drawn in the order the model established, not in
## alphabetical order. The distinction is visible only where the two differ,
## so the fixtures below use levels that are deliberately not alphabetical.

order_fixture <- function() {
    dt <- data.table::as.data.table(fresh_clintrial())
    ## "Well" < "Moderately" < "Poorly" clinically, but alphabetically the
    ## order is Moderately, Poorly, Well
    dt[]
}

plot_levels <- function(p, variable) {
    tbl <- attr(p, "table_data")
    rows <- trimws(tbl[["display_text"]])
    ## Levels follow their variable's header row
    start <- which(rows == variable)
    if (length(start) == 0) return(character(0))
    after <- rows[(start[1] + 1):length(rows)]
    headers <- trimws(tbl[["display_text"]])[tbl[["row_type"]] == "header"]
    keep <- character(0)
    for (r in after) {
        if (r %in% headers) break
        keep <- c(keep, r)
    }
    return(keep)
}


test_that("uniforest preserves factor level order when indenting", {

    dt <- order_fixture()
    skip_if(!"grade" %in% names(dt), "grade not present")

    screened <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "grade"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    p <- suppressMessages(suppressWarnings(
        uniforest(screened, indent_groups = TRUE)))

    drawn <- plot_levels(p, "grade")
    skip_if(length(drawn) < 2, "grade levels not found in table_data")

    ## The drawn order matches the factor levels, not sort()
    expect_identical(drawn, levels(dt$grade))
    expect_false(identical(drawn, sort(levels(dt$grade))))
})


test_that("indenting does not change which levels are drawn", {

    dt <- order_fixture()

    screened <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "grade"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    flat <- suppressMessages(suppressWarnings(
        uniforest(screened, indent_groups = FALSE)))
    indented <- suppressMessages(suppressWarnings(
        uniforest(screened, indent_groups = TRUE)))

    flat_n <- attr(flat, "table_data")[["n_formatted"]]
    ind_tbl <- attr(indented, "table_data")
    ind_n <- ind_tbl[["n_formatted"]][ind_tbl[["row_type"]] != "header"]

    ## Header rows aside, the same rows appear in the same order
    expect_identical(flat_n, ind_n)
})

### * Main function

#' Convert Model to Data Table
#'
#' Extracts coefficients, confidence intervals, and comprehensive model statistics 
#' from fitted regression models and converts them to a standardized data.table 
#' format suitable for further analysis or publication. This is a core utility 
#' function frequently used internally by other \pkg{summata} regression functions,
#' although it can be used as a standalone function as well.
#'
#' @param data Data frame or data.table containing the dataset used to fit the
#'   model. Required for computing group-level sample sizes and event counts.
#'   Where the fitted object retains no model frame, the supplied data is
#'   restricted to complete cases across the model variables so that group-level
#'   sample sizes describe the observations actually used in fitting. Group
#'   sample sizes therefore sum to the model-level \code{n}, and both exclude
#'   observations with a missing response.
#'
#' @param model Fitted model object. Supported classes include:
#'   \itemize{
#'     \item \code{glm} - Generalized linear models (logistic, Poisson, \emph{etc.})
#'     \item \code{lm} - Linear models
#'     \item \code{coxph} - Cox proportional hazards models
#'     \item \code{clogit} - Conditional logistic regression
#'     \item \code{coxme} - Mixed effects Cox models
#'     \item \code{lmerMod} - Linear mixed effects models
#'     \item \code{glmerMod} - Generalized linear mixed effects models
#'   }
#'   
#' @param conf_level Numeric confidence level for confidence intervals. Must be
#'   between 0 and 1. Default is 0.95 (95\% CI).
#'   
#' @param keep_qc_stats Logical. If \code{TRUE}, includes model quality statistics 
#'   such as AIC, BIC, \emph{R}\eqn{^2}, concordance, and model fit tests. These appear 
#'   as additional columns in the output. Default is \code{TRUE}.
#'   
#' @param include_intercept Logical. If \code{TRUE}, includes the model intercept 
#'   in output. If \code{FALSE}, removes the intercept row from results. Useful 
#'   for creating cleaner presentation tables. Default is \code{TRUE}.
#'   
#' @param terms_to_exclude Character vector of term names to exclude from output.
#'   Useful for removing specific unwanted parameters (\emph{e.g.,} nuisance variables,
#'   spline terms). Default is \code{NULL}. Note: If \code{include_intercept = FALSE}, 
#'   "(Intercept)" is automatically added to this list.
#'   
#' @param reference_rows Logical. If \code{TRUE}, adds rows for reference 
#'   categories of factor variables with appropriate labels and baseline values 
#'   (OR/HR = 1, Coefficient = 0). This makes tables more complete and easier to 
#'   interpret. Default is \code{TRUE}.
#'   
#' @param reference_label Character string used to label reference category rows
#'   in the output. Appears in the \code{reference} column. Default is \code{"reference"}.
#'
#' @param skip_counts Logical. If \code{TRUE}, skips computation of group-level
#'   sample sizes and event counts (faster but less informative). Default is 
#'   \code{FALSE}.
#'
#' @param conf_method Character string controlling the confidence interval method.
#'   If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
#'   \itemize{
#'     \item \code{"profile"} - Profile likelihood intervals for GLM and negative
#'       binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
#'       intervals for linear models. Falls back to Wald on profiling failure.
#'       Quasi-likelihood families always use Wald (no true likelihood).
#'     \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
#'       \eqn{\times} SE) for all model types. Faster but less accurate near
#'       boundary conditions or with small subgroups.
#'   }
#'   Cox and mixed-effects models use Wald intervals regardless of this setting.
#'   Set globally with \code{options(summata.conf_method = "wald")} to use Wald
#'   throughout a session.
#'
#' @return A \code{data.table} containing extracted model information with the 
#'   following standard columns:
#'   \describe{
#'     \item{model_scope}{Character. Either "Univariable" (unadjusted model with 
#'       single predictor) or "Multivariable" (adjusted model with multiple predictors)}
#'     \item{model_type}{Character. Type of regression (\emph{e.g.,} "Logistic", "Linear", 
#'       "Cox PH", "Poisson", \emph{etc.})}
#'     \item{variable}{Character. Variable name (for factor variables, the base 
#'       variable name without the level)}
#'     \item{group}{Character. Group/level name for factor variables; empty string 
#'       for continuous variables}
#'     \item{n}{Integer. Number of observations used in fitting the model, that
#'       is complete cases across the model variables}
#'     \item{n_group}{Integer. Number of those observations at this variable
#'       level (factor variables only). Group sizes sum to \code{n} within
#'       each variable}
#'     \item{events}{Integer. Number of events among the observations used in
#'       fitting (for survival and logistic models)}
#'     \item{events_group}{Integer. Number of events at this variable level
#'       (for survival and logistic models with factor variables). Group counts
#'       sum to \code{events} within each variable}
#'     \item{coefficient}{Numeric. Raw regression coefficient (log odds, log hazard, 
#'       \emph{etc.})}
#'     \item{se}{Numeric. Standard error of the coefficient}
#'     \item{OR/HR/RR/Coefficient}{Numeric. Effect estimate - column name depends on 
#'       model type:
#'       \itemize{
#'         \item \code{OR} for logistic regression (odds ratio)
#'         \item \code{HR} for Cox models (hazard ratio)
#'         \item \code{RR} for Poisson regression (rate/risk ratio)
#'         \item \code{Coefficient} for linear models or other GLMs
#'       }}
#'     \item{ci_lower}{Numeric. Lower bound of confidence interval for effect estimate}
#'     \item{ci_upper}{Numeric. Upper bound of confidence interval for effect estimate}
#'     \item{statistic}{Numeric. Test statistic (z-value for GLM/Cox, t-value for LM)}
#'     \item{p_value}{Numeric. \emph{p}-value for coefficient test}
#'     \item{sig}{Character. Significance markers: \code{***} (p < 0.001), \code{**}
#'       (\emph{p} < 0.01), \code{*} (\emph{p} < 0.05), \code{.} (\emph{p} < 0.10).}
#'     \item{sig_binary}{Logical. Binary indicator: \code{TRUE} if \emph{p} < 0.05, 
#'       \code{FALSE} otherwise}
#'     \item{reference}{Character. Contains \code{reference_label} for reference 
#'       category rows when \code{reference_rows = TRUE}, empty string otherwise}
#'   }
#'
#' @details
#' This function is the core extraction utility used by \code{fit()} and other
#' regression functions. It handles the complexities of different model classes
#' and provides a consistent output format suitable for tables and forest plots.
#'
#' \strong{Model Type Detection:}
#' The function automatically detects model type and applies appropriate:
#' \itemize{
#'   \item Effect measure naming (OR, HR, RR, Coefficient)
#'   \item Confidence interval calculation (see below)
#'   \item Event counting for binary/survival outcomes
#' }
#'
#' \strong{Confidence Interval Methods:}
#' The CI method is selected per model class using \code{stats::confint()}
#' dispatch:
#' \itemize{
#'   \item \strong{GLM/negative binomial}: Profile likelihood via
#'     \code{MASS::confint.glm()}, except quasi-families which use Wald
#'   \item \strong{Linear models}: Exact \emph{t}-distribution via
#'     \code{confint.lm()}
#'   \item \strong{Cox PH}: Wald intervals (coefficient \eqn{\pm}
#'     \emph{z} \eqn{\times} SE)
#'   \item \strong{Mixed-effects models}: Wald intervals
#' }
#' Falls back to Wald on profiling failure.
#'
#' \strong{Mixed Effects Models:}
#' For \pkg{lme4} models (glmer, lmer), the function extracts fixed effects
#' only. Random effects variance components are not included in the output
#' table, as they represent clustering structure rather than predictor effects.
#'
#' @seealso
#' \code{\link{fit}} for the main regression interface,
#' \code{\link{glmforest}}, \code{\link{coxforest}}, \code{\link{lmforest}} for
#' forest plot visualization
#'
#' @examples
#' # Load example data
#' data(clintrial)
#' 
#' # Example 1: Extract from logistic regression
#' glm_model <- glm(os_status ~ age + sex + treatment, 
#'                  data = clintrial, family = binomial)
#' 
#' glm_result <- m2dt(clintrial, glm_model)
#' glm_result
#'
#' \donttest{
#' 
#' # Example 2: Extract from linear model
#' lm_model <- lm(los_days ~ age + sex + surgery, data = clintrial)
#' 
#' lm_result <- m2dt(clintrial, lm_model)
#' lm_result
#' 
#' # Example 3: Cox proportional hazards model
#' library(survival)
#' cox_model <- coxph(Surv(os_months, os_status) ~ age + sex + stage,
#'                    data = clintrial)
#' 
#' cox_result <- m2dt(clintrial, cox_model)
#' cox_result
#' 
#' # Example 4: Exclude intercept for cleaner tables
#' clean_result <- m2dt(clintrial, glm_model, include_intercept = FALSE)
#' clean_result
#' 
#' # Example 5: Change confidence level
#' result_90ci <- m2dt(clintrial, glm_model, conf_level = 0.90)
#' result_90ci
#'
#' }
#'
#' @family utility functions
#' @export
m2dt <- function(data,
                 model,
                 conf_level = 0.95,
                 keep_qc_stats = TRUE,
                 include_intercept = TRUE,
                 terms_to_exclude = NULL,
                 reference_rows = TRUE,
                 reference_label = "reference",
                 skip_counts = FALSE,
                 conf_method = NULL) {

    ## Validate inputs
    if (missing(data)) {
        stop("data parameter is required. Usage: m2dt(data, model)")
    }
    
    if (!inherits(data, c("data.frame", "data.table"))) {
        stop("data must be a data.frame or data.table")
    }
    
    ## Store data for use throughout function
    model_data <- if (data.table::is.data.table(data)) data else data.table::as.data.table(data)
    
    ## Store as attribute for helper functions
    attr(model, "data") <- model_data

    ## Set model class - comprehensive detection for all model types
    model_classes <- class(model)
    
    ## Priority order for mixed models (check specific classes first)
    if ("lmerMod" %in% model_classes || inherits(model, "lmerMod")) {
        model_class <- "lmerMod"
    } else if ("glmerMod" %in% model_classes || inherits(model, "glmerMod")) {
        model_class <- "glmerMod"
    } else if ("lmerTest" %in% model_classes) {
        ## lmerTest package wraps lmer models
        model_class <- "lmerMod"
    } else if (inherits(model, "merMod")) {
        ## Generic merMod - determine if linear or generalized
        if ("lmer" %in% model_classes) {
            model_class <- "lmerMod"
        } else if ("glmer" %in% model_classes) {
            model_class <- "glmerMod"
        } else {
            ## Fallback for generic merMod
            model_class <- "lmerMod"  # Assume linear if not specified
        }
    } else if ("negbin" %in% model_classes) {
        ## MASS::glm.nb produces negbin class - treat as glm for extraction
        ## but mark it for special handling (rate ratios)
        model_class <- "negbin"
    } else {
        ## Standard detection for other models
        model_class <- class(model)[1]
    }
    
    ## Null operator for handling missing values
    `%||%` <- function(a, b) if (is.null(a)) b else a
    
    ## Add intercept to exclusion list if requested
    if (!include_intercept) {
        terms_to_exclude <- unique(c(terms_to_exclude, "(Intercept)"))
    }
    
    ## Get model type (Univariable vs Multivariable)
    model_scope <- detect_model_type(model)
    
    ## Get readable model type name
    model_type_name <- get_model_type_name(model)
    
    ## Variable to hold confint result for downstream caching
    .confint_cache <- NULL
    .confint_cache_level <- NULL

    ## Resolve CI method: "profile" (default) or "wald"
    conf_method <- if (is.null(conf_method)) {
        getOption("summata.conf_method", "profile")
    } else {
        match.arg(conf_method, c("profile", "wald"))
    }

    ## Extract results based on model class
    if (model_class %in% c("glm", "lm", "negbin")) {
        
        coef_summary <- summary(model)$coefficients

        ## Compute confidence intervals.
        ## conf_method = "profile": profile likelihood for GLM/negbin, exact t
        ##   for lm, Wald for quasi-families. Falls back to Wald on failure.
        ## conf_method = "wald": Wald (normal approximation) for all models.
        ##   Faster but less accurate near boundary conditions.
        is_quasi <- inherits(model, "glm") &&
            grepl("^quasi", model$family$family)

        ## Skip profiling the intercept when it will be excluded from output.
        all_terms <- rownames(coef_summary)
        skip_intercept <- !include_intercept &&
            "(Intercept)" %in% all_terms

        parm <- if (skip_intercept) {
            setdiff(all_terms, "(Intercept)")
        } else {
            all_terms
        }

        use_wald <- conf_method == "wald" || is_quasi

        conf_int_raw <- if (use_wald) {
            stats::confint.default(model, parm = parm, level = conf_level)
        } else {
            tryCatch(
                suppressMessages(suppressWarnings(
                    stats::confint(model, parm = parm, level = conf_level)
                )),
                error = function(e) {
                    stats::confint.default(model, parm = parm, level = conf_level)
                }
            )
        }

        ## Ensure result is always a matrix (confint may return a named
        ## vector when parm specifies a single parameter)
        if (is.null(dim(conf_int_raw))) {
            conf_int_raw <- matrix(conf_int_raw, nrow = 1,
                                   dimnames = list(parm, names(conf_int_raw)))
        }

        ## Reconstruct full matrix aligned with coef_summary rows
        ## (intercept row gets NA if it was skipped)
        if (skip_intercept && nrow(conf_int_raw) < length(all_terms)) {
            conf_int <- matrix(NA_real_, nrow = length(all_terms), ncol = 2,
                               dimnames = list(all_terms, colnames(conf_int_raw)))
            conf_int[rownames(conf_int_raw), ] <- conf_int_raw
        } else {
            conf_int <- conf_int_raw
        }

        ## Save confint for caching at end of function
        .confint_cache <- conf_int
        .confint_cache_level <- conf_level
        
        dt <- data.table::data.table(
                              model_scope = model_scope,
                              model_type = model_type_name,
                              term = rownames(coef_summary),
                              n = stats::nobs(model),
                              events = NA_real_,
                              coefficient = coef_summary[, "Estimate"],
                              se = coef_summary[, "Std. Error"]
                          )
        
        ## Use confint() output for CI bounds (profile likelihood for
        ## GLM/negbin, exact t-distribution for lm, Wald for quasi-families)
        dt[, `:=`(
            coef = coefficient,
            coef_lower = conf_int[, 1],
            coef_upper = conf_int[, 2]
        )]
        
        ## Special handling for logistic regression (including quasibinomial)
        if (model_class == "glm" && !isS4(model)) {
            if (model$family$family %in% c("binomial", "quasibinomial", "poisson", "quasipoisson")) {
                if (!is.null(model$y)) {
                    dt[, events := sum(model$y, na.rm = TRUE)]
                } else if (!is.null(model$model)) {
                    outcome_col <- model$model[[1]]
                    if (is.factor(outcome_col)) {
                        dt[, events := sum(as.numeric(outcome_col) == 2, na.rm = TRUE)]
                    } else {
                        dt[, events := sum(outcome_col, na.rm = TRUE)]
                    }
                }
            }
        }
        
        ## Handle negative binomial models (MASS::glm.nb)
        if (model_class == "negbin") {
            if (!is.null(model$y)) {
                dt[, events := sum(model$y, na.rm = TRUE)]
            } else if (!is.null(model$model)) {
                outcome_col <- model$model[[1]]
                dt[, events := sum(outcome_col, na.rm = TRUE)]
            }
        }
        
        ## Determine if should exponentiate
        should_exp <- FALSE
        is_logistic <- FALSE
        is_poisson <- FALSE
        is_negbin <- model_class == "negbin"
        
        if (model_class == "glm" && !isS4(model)) {
            family_name <- model$family$family
            link_name <- model$family$link
            
            ## binomial and quasibinomial both produce odds ratios
            is_logistic <- family_name %in% c("binomial", "quasibinomial")
            is_poisson <- family_name == "poisson" || 
                (family_name == "quasipoisson")
            
            should_exp <- is_logistic || is_poisson || link_name == "log"
        }
        
        ## Negative binomial uses log link - exponentiate for rate ratios
        if (is_negbin) {
            should_exp <- TRUE
        }
        
        ## Add exponentiated coefficients
        dt[, `:=`(
            exp_coef = if (should_exp) exp(coefficient) else coefficient,
            exp_lower = if (should_exp) exp(coef_lower) else coef_lower,
            exp_upper = if (should_exp) exp(coef_upper) else coef_upper
        )]
        
        if (is_logistic) {
            dt[, `:=`(
                OR = exp_coef,
                ci_lower = exp_lower,
                ci_upper = exp_upper
            )]
        } else if (is_poisson || is_negbin) {
            ## Poisson and negative binomial both produce rate ratios
            dt[, `:=`(
                RR = exp_coef,
                ci_lower = exp_lower,
                ci_upper = exp_upper
            )]
        } else if (should_exp) {
            ## Other log-link models
            dt[, `:=`(
                Coefficient = exp_coef,
                ci_lower = exp_lower,
                ci_upper = exp_upper
            )]
        } else {
            ## Linear models - use raw coefficients
            dt[, `:=`(
                Coefficient = coef,
                ci_lower = coef_lower,
                ci_upper = coef_upper
            )]
        }
        
        ## Add test statistics
        stat_col <- if ("z value" %in% colnames(coef_summary)) "z value" else "t value"
        dt[, `:=`(
            statistic = coef_summary[, stat_col],
            p_value = coef_summary[, ncol(coef_summary)]
        )]
        
        ## Add QC stats if requested
        if (keep_qc_stats) {
            if (model_class %in% c("glm", "negbin")) {
                dt[, `:=`(
                    AIC = stats::AIC(model),
                    BIC = stats::BIC(model),
                    deviance = stats::deviance(model),
                    null_deviance = if (!isS4(model)) model$null.deviance else NA,
                    df_residual = stats::df.residual(model)
                )]
                
                ## Add R-squared for non-binomial GLMs
                if (model_class == "glm" && !isS4(model) && model$family$family != "binomial") {
                    dt[, R2 := 1 - (deviance / null_deviance)]
                }
                
                ## Add theta for negative binomial
                if (model_class == "negbin") {
                    dt[, theta := model$theta]
                }
                
                ## For binomial, add discrimination/calibration metrics
                if (is_logistic && keep_qc_stats) {
                    ## C-statistic (if pROC available)
                    if (requireNamespace("pROC", quietly = TRUE)) {
                        if (!isS4(model) && !is.null(model$y)) {
                            roc_obj <- pROC::roc(model$y, stats::fitted(model), quiet = TRUE)
                            dt[, c_statistic := as.numeric(pROC::auc(roc_obj))]
                        }
                    }
                    
                    ## Hosmer-Lemeshow test (if ResourceSelection available)
                    ##
                    ## The test partitions the fitted values into bins of equal
                    ## size. A model whose predictors are all categorical yields
                    ## few distinct covariate patterns, and the requested bins
                    ## cannot be formed. The test is not applicable in that
                    ## case, so it is omitted rather than reported against an
                    ## unstated number of bins; reporting it would also emit a
                    ## warning on every call for an entirely ordinary model.
                    if (requireNamespace("ResourceSelection", quietly = TRUE)) {
                        if (!isS4(model) && !is.null(model$y)) {
                            
                            hl_bins <- 10L
                            fitted_vals <- stats::fitted(model)
                            
                            ## Mirrors the binning performed by hoslem.test()
                            hl_breaks <- unique(stats::quantile(
                                fitted_vals,
                                probs = seq(0, 1, 1 / hl_bins),
                                na.rm = TRUE
                            ))
                            
                            if (length(hl_breaks) > hl_bins) {
                                hl <- tryCatch(
                                    ResourceSelection::hoslem.test(
                                        model$y, fitted_vals, g = hl_bins
                                    ),
                                    warning = function(w) NULL,
                                    error = function(e) NULL
                                )
                                
                                if (!is.null(hl)) {
                                    dt[, `:=`(
                                        hoslem_chi2 = hl$statistic,
                                        hoslem_p = hl$p.value
                                    )]
                                }
                            }
                        }
                    }
                }
            } else if (model_class == "lm") {
                summ <- summary(model)
                dt[, `:=`(
                    R2 = summ$r.squared,
                    adj_R2 = summ$adj.r.squared,
                    AIC = stats::AIC(model),
                    BIC = stats::BIC(model),
                    sigma = summ$sigma,
                    df_residual = stats::df.residual(model)
                )]
            }
        }
        
    } else if (model_class %in% c("coxph", "clogit")) {
        
        if (!requireNamespace("survival", quietly = TRUE))
            stop("Package 'survival' required")
        
        summ <- summary(model)
        coef_summary <- summ$coefficients
        conf_int <- stats::confint(model, level = conf_level)
        
        dt <- data.table::data.table(
                              model_scope = model_scope %||% "Multivariable",
                              model_type = model_type_name,
                              term = rownames(coef_summary),
                              n = if (!is.null(model$n)) model$n[1] else summ$n,
                              events = if (!is.null(model$nevent)) model$nevent 
                                       else if (!is.null(model$n)) model$n[2] 
                                       else summ$nevent,
                              coefficient = coef_summary[, "coef"],
                              se = coef_summary[, "se(coef)"],
                              ## Store both versions
                              coef = coef_summary[, "coef"],
                              coef_lower = conf_int[, 1],
                              coef_upper = conf_int[, 2],
                              exp_coef = coef_summary[, "exp(coef)"],
                              exp_lower = exp(conf_int[, 1]),
                              exp_upper = exp(conf_int[, 2]),
                              ## Primary display columns
                              HR = coef_summary[, "exp(coef)"],
                              ci_lower = exp(conf_int[, 1]),
                              ci_upper = exp(conf_int[, 2]),
                              statistic = coef_summary[, "z"],
                              p_value = coef_summary[, "Pr(>|z|)"]
                          )

        ## Add QC stats
        if (keep_qc_stats) {
            dt[, `:=`(
                concordance = summ$concordance[1],
                concordance_se = summ$concordance[2],
                rsq = summ$rsq[1],
                rsq_max = summ$rsq[2],
                likelihood_ratio_test = summ$logtest[1],
                likelihood_ratio_df = summ$logtest[2],
                likelihood_ratio_p = summ$logtest[3],
                wald_test = summ$waldtest[1],
                wald_df = summ$waldtest[2],
                wald_p = summ$waldtest[3],
                score_test = summ$sctest[1],
                score_df = summ$sctest[2],
                score_p = summ$sctest[3]
            )]
        }

    } else if (model_class %in% c("coxme", "lme", "lmer", "lmerMod", "glmer", "glmerMod", "merMod") || 
               inherits(model, c("lmerMod", "glmerMod", "merMod", "lmer", "glmer"))) {
        
        ## Mixed effects models
        summ <- summary(model)
        
        if (model_class == "coxme") {
            coef_vec <- coxme::fixef(model)
            vcov_mat <- as.matrix(stats::vcov(model))
            se_vec <- sqrt(diag(vcov_mat))
            z_vec <- coef_vec / se_vec
            p_vec <- 2 * (1 - stats::pnorm(abs(z_vec)))
            
            dt <- data.table::data.table(
                                  model_scope = model_scope %||% "Multivariable",
                                  model_type = model_type_name,
                                  term = names(coef_vec),
                                  n = model$n[2],
                                  events = model$n[1],
                                  n_group = NA_real_,
                                  events_group = NA_real_, 
                                  coefficient = coef_vec,
                                  se = se_vec,
                                  coef = coef_vec,
                                  coef_lower = coef_vec - stats::qnorm((1 + conf_level) / 2) * se_vec,
                                  coef_upper = coef_vec + stats::qnorm((1 + conf_level) / 2) * se_vec,
                                  exp_coef = exp(coef_vec),
                                  exp_lower = exp(coef_vec - stats::qnorm((1 + conf_level) / 2) * se_vec),
                                  exp_upper = exp(coef_vec + stats::qnorm((1 + conf_level) / 2) * se_vec),
                                  HR = exp(coef_vec),
                                  ci_lower = exp(coef_vec - stats::qnorm((1 + conf_level) / 2) * se_vec),
                                  ci_upper = exp(coef_vec + stats::qnorm((1 + conf_level) / 2) * se_vec),
                                  statistic = z_vec,
                                  p_value = p_vec
                              )
            
            ## Add QC stats for coxme
            if (keep_qc_stats) {
                ## AIC and BIC
                dt[, `:=`(
                    AIC = stats::AIC(model),
                    BIC = stats::BIC(model)
                )]
                
                ## Log-likelihood
                loglik <- model$loglik
                if (!is.null(loglik) && length(loglik) >= 2) {
                    dt[, `:=`(
                        loglik_null = loglik[1],
                        loglik_model = loglik[length(loglik)]
                    )]
                }
                
                ## Integrated log-likelihood (penalized)
                if (!is.null(model$loglik)) {
                    dt[, integrated_loglik := model$loglik[length(model$loglik)]]
                }
                
                ## Initialize c_statistic with NA
                dt[, c_statistic := NA_real_]
                
                ## Concordance/C-statistic for coxme
                ## coxme stores both the survival object (y) and linear predictor directly
                if (requireNamespace("survival", quietly = TRUE)) {
                    c_stat <- tryCatch({
                        ## Both y (Surv object) and linear.predictor are stored in coxme models
                        if (!is.null(model$y) && !is.null(model$linear.predictor)) {
                            conc <- survival::concordance(model$y ~ model$linear.predictor)
                            conc$concordance
                        } else {
                            NA_real_
                        }
                    }, error = function(e) NA_real_)
                    
                    if (!is.na(c_stat)) {
                        dt[, c_statistic := c_stat]
                    }
                }
            }
            
        } else {
            
            ## lmer/glmer from lme4
            if (!requireNamespace("lme4", quietly = TRUE))
                stop("Package 'lme4' required")
            
            coef_summary <- stats::coef(summ)
            
            ## Determine if should exponentiate
            if (model_class %in% c("lmer", "lmerMod")) {
                should_exp <- FALSE  ## Linear mixed models: no exponentiation
            } else if (model_class %in% c("glmer", "glmerMod")) {
                should_exp <- (summ$family == "binomial" || summ$link == "log")
            } else {
                should_exp <- FALSE  # Default for other mixed models
            }
            
            dt <- data.table::data.table(
                                  model_scope = model_scope %||% "Multivariable",
                                  model_type = model_type_name,
                                  term = rownames(coef_summary),
                                  n = stats::nobs(model),
                                  events = NA_real_,
                                  coefficient = coef_summary[, "Estimate"],
                                  se = coef_summary[, "Std. Error"]
                              )
            
            ## For glmer binomial, calculate total events from the response
            if (model_class %in% c("glmer", "glmerMod") && inherits(model, "merMod")) {
                if (summ$family == "binomial") {
                    response_var <- model@resp$y  
                    if (!is.null(response_var)) {
                        dt[, events := sum(response_var, na.rm = TRUE)]
                    }
                }
            }
            
            z_score <- stats::qnorm((1 + conf_level) / 2)
            dt[, `:=`(
                coef = coefficient,
                coef_lower = coefficient - z_score * se,
                coef_upper = coefficient + z_score * se,
                exp_coef = if (should_exp) exp(coefficient) else coefficient,
                exp_lower = if (should_exp) exp(coefficient - z_score * se) else coefficient - z_score * se,
                exp_upper = if (should_exp) exp(coefficient + z_score * se) else coefficient + z_score * se
            )]
            
            ## Add appropriate effect column
            if (model_class %in% c("glmer", "glmerMod") && summ$family == "binomial") {
                dt[, `:=`(
                    OR = exp_coef,
                    ci_lower = exp_lower,
                    ci_upper = exp_upper
                )]
            } else if (should_exp) {
                dt[, `:=`(
                    RR = exp_coef,
                    ci_lower = exp_lower,
                    ci_upper = exp_upper
                )]
            } else {
                dt[, `:=`(
                    Coefficient = coef,
                    ci_lower = coef_lower,
                    ci_upper = coef_upper
                )]
            }
            
            ## Add test statistics
            if (ncol(coef_summary) >= 3) {
                ## Use z-values if available
                stat_col <- if ("z value" %in% colnames(coef_summary)) {
                                "z value"
                            } else if ("t value" %in% colnames(coef_summary)) {
                                "t value"
                            } else {
                                NULL
                            }
                
                if (!is.null(stat_col)) {
                    dt[, statistic := coef_summary[, stat_col]]
                } else {
                    ## Calculate z-statistics manually
                    dt[, statistic := coefficient / se]
                }
                
                ## Check if \emph{p}-values are provided
                p_col <- grep("^Pr\\(", colnames(coef_summary))
                if (length(p_col) > 0) {
                    dt[, p_value := coef_summary[, p_col[1]]]
                } else {
                    ## Calculate \emph{p}-values from z/t statistics
                    dt[, p_value := 2 * (1 - stats::pnorm(abs(statistic)))]
                }
            } else {
                ## No test statistics - calculate manually
                dt[, `:=`(
                    statistic = coefficient / se,
                    p_value = 2 * (1 - stats::pnorm(abs(coefficient / se)))
                )]
            }
            
            ## Add QC stats for lmer/glmer
            if (keep_qc_stats) {
                ## AIC and BIC
                dt[, `:=`(
                    AIC = stats::AIC(model),
                    BIC = stats::BIC(model)
                )]
                
                ## Log-likelihood
                loglik_val <- as.numeric(stats::logLik(model))
                dt[, loglik := loglik_val]
                
                ## Deviance - handle REML vs ML fits
                dev_val <- tryCatch({
                    if (model_class %in% c("lmer", "lmerMod") && lme4::isREML(model)) {
                        ## For REML fits, use REMLcrit or deviance with REML=FALSE
                        stats::deviance(model, REML = FALSE)
                    } else {
                        stats::deviance(model)
                    }
                }, error = function(e) NA_real_)
                dt[, deviance := dev_val]
                
                ## For lmer models, add R-squared measures
                if (model_class %in% c("lmer", "lmerMod")) {
                    ## Initialize R-squared columns with NA
                    dt[, `:=`(
                        r_squared = NA_real_,
                        r_squared_marginal = NA_real_,
                        r_squared_conditional = NA_real_
                    )]
                    
                    ## Try to get R-squared using MuMIn if available
                    if (requireNamespace("MuMIn", quietly = TRUE)) {
                        r2_vals <- tryCatch({
                            MuMIn::r.squaredGLMM(model)
                        }, error = function(e) NULL)
                        
                        if (!is.null(r2_vals)) {
                            dt[, `:=`(
                                r_squared = r2_vals[1, "R2m"],
                                r_squared_marginal = r2_vals[1, "R2m"],
                                r_squared_conditional = r2_vals[1, "R2c"]
                            )]
                        }
                    }
                    
                    ## Residual standard deviation (sigma)
                    dt[, sigma := stats::sigma(model)]
                }
                
                ## For glmer binomial, try to compute C-statistic
                if (model_class %in% c("glmer", "glmerMod") && inherits(model, "merMod")) {
                    ## Initialize c_statistic with NA
                    dt[, c_statistic := NA_real_]
                    
                    if (summ$family == "binomial") {
                        if (requireNamespace("pROC", quietly = TRUE)) {
                            c_stat <- tryCatch({
                                fitted_vals <- stats::fitted(model)
                                response_vals <- model@resp$y
                                roc_obj <- pROC::roc(response_vals, fitted_vals, quiet = TRUE)
                                as.numeric(pROC::auc(roc_obj))
                            }, error = function(e) NA_real_)
                            
                            if (!is.na(c_stat)) {
                                dt[, c_statistic := c_stat]
                            }
                        }
                    }
                }
            }
        }
        
    } else {
        stop("Unsupported model class: ", model_class)
    }
    
    ## Parse terms into variable and group AFTER creating dt
    xlevels <- get_model_xlevels(model)
    
    ## Parse terms - pass model for coxme special handling
    parsed <- parse_term(dt$term, xlevels, model)
    dt[, `:=`(variable = parsed$variable, group = parsed$group)]
    
    ## Initialize n_group and events_group columns if absent
    if (!"n_group" %in% names(dt)) {
        dt[, n_group := NA_real_]
    }
    if (!"events_group" %in% names(dt)) {
        dt[, events_group := NA_real_]
    }

    ## Group counts calculation
    all_counts <- NULL
    data_dt <- NULL
    event_var <- NULL
    outcome_var <- NULL
    factor_vars <- character(0)

    if (!skip_counts) {
        
        ## Prepare data for group counting
        ## Use model's internal data (complete cases only) for accurate counts
        if (!is.null(xlevels) || model_class == "coxme") {
            ## Try to get the model's data (complete cases used in fitting)
            data_source <- NULL
            
            if (model_class %in% c("glm", "lm", "negbin")) {
                ## For glm/lm/negbin, model$model contains complete cases
                if (!is.null(model$model)) {
                    data_source <- model$model
                }
            } else if (inherits(model, "merMod")) {
                ## For lme4 models, use the frame
                data_source <- model@frame
            }
            
            ## Survival models are handled through the supplied data below. A
            ## coxph model frame stores the response as a Surv object under a
            ## deparsed name (for example, "Surv(os_months, os_status)"), from
            ## which the event variable cannot be resolved, so event counts are
            ## taken from the supplied columns instead.
            
            ## Fallback to the supplied data when the model retains no model
            ## frame. This occurs for fits created with model = FALSE (the
            ## memory-conserving default used internally by uniscreen() and
            ## multifit()), for survival::coxph(), which does not store a model
            ## frame by default, and for coxme and clogit models.
            ##
            ## The supplied data still contains the observations dropped during
            ## fitting, so it must be restricted to the analysis rows before
            ## group counts are taken. Without this restriction, categorical
            ## variables report totals over every supplied row while the
            ## model-level n reports complete cases only, and the two disagree
            ## whenever the response or a covariate is missing.
            reconstruct_rows <- is.null(data_source)
            if (reconstruct_rows) {
                data_source <- model_data
            }
            
            data_dt <- if (data.table::is.data.table(data_source)) data_source else data.table::as.data.table(data_source)
            
            if (reconstruct_rows) {
                data_dt <- get_analysis_data(model, model_class, data_dt)
            }
            
            ## For coxme, reconstruct xlevels from the data if needed
            if (is.null(xlevels) && model_class == "coxme") {
                ## Use formulaList$fixed for coxme
                formula_to_use <- model$formulaList$fixed
                
                ## Get variables
                all_formula_vars <- all.vars(formula_to_use)
                term_vars <- all_formula_vars[-1]  # Exclude response
                
                xlevels <- list()
                
                for (var_name in term_vars) {
                    if (var_name %in% names(data_dt) && is.factor(data_dt[[var_name]])) {
                        xlevels[[var_name]] <- levels(data_dt[[var_name]])
                    }
                }
            }
            
            ## Determine outcome and event variables
            ## Events only make sense for binomial, poisson, quasipoisson, quasibinomial, negbin
            ## NOT for gaussian, Gamma, inverse.gaussian (continuous outcomes)
            outcome_var <- NULL
            event_var <- NULL
            
            ## Define families that should have events calculated
            event_families <- c("binomial", "quasibinomial", "poisson", "quasipoisson")
            
            if (model_class == "negbin" && !isS4(model)) {
                ## Negative binomial always has events (count data)
                if (!is.null(model$formula)) {
                    outcome_var <- all.vars(model$formula)[1]
                } else {
                    outcome_var <- all.vars(stats::formula(model))[1]
                }
            } else if (model_class == "glm" && !isS4(model)) {
                ## Only set outcome_var for GLM families with meaningful events
                family_name <- model$family$family
                if (family_name %in% event_families) {
                    if (!is.null(model$formula)) {
                        outcome_var <- all.vars(model$formula)[1]
                    } else {
                        outcome_var <- all.vars(stats::formula(model))[1]
                    }
                }
            } else if (model_class %in% c("lmer", "lmerMod", "glmer", "glmerMod") && inherits(model, "merMod")) {
                ## For mixed models, only calculate events for binomial/poisson families
                fam <- tryCatch(family(model)$family, error = function(e) "gaussian")
                if (fam %in% event_families) {
                    formula_obj <- model@call$formula
                    if (!is.null(formula_obj)) {
                        outcome_var <- all.vars(formula_obj)[1]
                    }
                }
            } else if (model_class %in% c("coxph", "clogit", "coxme")) {
                ## For all survival models, use the unified helper function
                event_var <- get_event_variable(model, model_class)
            }
            
            ## Calculate n_group and events_group for all factor variables at once
            if (length(xlevels) > 0 && !is.null(data_dt)) {
                
                ## Get factor variables that exist in the data
                factor_vars <- names(xlevels)[names(xlevels) %in% names(data_dt)]
                
                if (length(factor_vars) > 0) {
                    
                    ## Use melt for efficient long-format conversion (faster than lapply+rbindlist)
                    ## Select only the required columns, to minimize memory usage
                    cols_needed <- factor_vars
                    if (!is.null(event_var) && event_var %in% names(data_dt)) {
                        cols_needed <- c(cols_needed, event_var)
                    } else if (!is.null(outcome_var) && outcome_var %in% names(data_dt)) {
                        cols_needed <- c(cols_needed, outcome_var)
                    }
                    
                    ## Melt to long format - much faster than lapply for multiple variables
                    long_dt <- data.table::melt(
                                               data_dt[, ..cols_needed],
                                               measure.vars = factor_vars,
                                               variable.name = "variable",
                                               value.name = "group",
                                               na.rm = TRUE,
                                               variable.factor = FALSE
                                           )
                    long_dt[, group := as.character(group)]
                    
                    if (!is.null(event_var) && event_var %in% names(data_dt)) {
                        ## For survival models
                        all_counts <- long_dt[, .(
                            n_group = .N,
                            events_group = sum(get(event_var), na.rm = TRUE)
                        ), by = .(variable, group)]
                        
                        ## Single join to update all counts at once
                        dt[all_counts, `:=`(
                                           n_group = i.n_group,
                                           events_group = i.events_group
                                       ), on = .(variable, group)]
                        
                    } else if (!is.null(outcome_var) && outcome_var %in% names(long_dt)) {
                        ## For GLM/GLMER/negbin models - calculate events from outcome
                        ## Use get() to handle column name as string
                        outcome_col <- long_dt[[outcome_var]]
                        if (is.factor(outcome_col)) {
                            long_dt[, .events_calc := as.numeric(outcome_col) == 2]
                        } else {
                            long_dt[, .events_calc := get(outcome_var)]
                        }
                        
                        all_counts <- long_dt[, .(
                            n_group = .N,
                            events_group = sum(.events_calc, na.rm = TRUE)
                        ), by = .(variable, group)]
                        
                        ## Single join to update all counts
                        dt[all_counts, `:=`(
                                           n_group = i.n_group,
                                           events_group = i.events_group
                                       ), on = .(variable, group)]
                        
                    } else {
                        ## No outcome/event variable: just count n
                        all_counts <- long_dt[, .(n_group = .N), by = .(variable, group)]
                        
                        ## Single join to update counts
                        dt[all_counts, n_group := i.n_group, on = .(variable, group)]
                    }
                    
                    ## A factor level with no observations among the analysis
                    ## rows never appears in all_counts, so its count is left
                    ## missing and the row would otherwise inherit the
                    ## model-level total. Such a level is empty by definition
                    ## and reports zero, which preserves the identity between
                    ## the group sizes and the fitted sample size.
                    ##
                    ## The fill is keyed on the declared levels of the counted
                    ## variables, so that it applies only where zero is known to
                    ## be the correct answer. Continuous and interaction terms
                    ## never form such a key; terms carrying an inline
                    ## transformation (\emph{e.g.,} factor(stage)) are absent
                    ## from the counting data; and coefficients produced by
                    ## non-treatment contrasts (\emph{e.g.,} "stage.L" from an
                    ## ordered factor) carry a suffix that is not a level. All
                    ## of these retain the model-level total, their group counts
                    ## being unknown rather than zero.
                    known_levels <- unlist(
                        lapply(factor_vars, function(var) {
                            paste(var, xlevels[[var]], sep = "\r")
                        }),
                        use.names = FALSE
                    )
                    
                    empty_levels <- is.na(dt$n_group) &
                        paste(dt$variable, dt$group, sep = "\r") %in% known_levels
                    
                    if (any(empty_levels)) {
                        dt[empty_levels, n_group := 0]
                        
                        if ("events_group" %in% names(all_counts)) {
                            dt[empty_levels, events_group := 0]
                        }
                    }
                }
            }
        }
        
        ## Calculate n_group and events_group for interaction terms
        ## Interactions are identified by ":" in the variable name
        interaction_rows <- grep(":", dt$variable, fixed = TRUE)
        
        if (length(interaction_rows) > 0 && !is.null(model_data)) {
            ## data_dt is populated above whenever the model contains factor
            ## terms. A model whose only non-continuous structure is an
            ## interaction reaches this block with data_dt unset, so it is
            ## derived here from the supplied data and restricted to the
            ## analysis rows in the same way.
            if (is.null(data_dt)) {
                data_dt <- get_analysis_data(model, model_class, model_data)
            } else if (!data.table::is.data.table(data_dt)) {
                data_dt <- data.table::as.data.table(data_dt)
            }
            
            ## Get xlevels if not already available
            if (is.null(xlevels)) {
                xlevels <- get_model_xlevels(model)
            }
            
            ## Determine event variable for survival models
            ## Events only make sense for binomial, poisson, quasipoisson, quasibinomial, negbin
            event_var <- NULL
            outcome_var <- NULL
            
            ## Define families that should have events calculated
            event_families <- c("binomial", "quasibinomial", "poisson", "quasipoisson")
            
            if (model_class %in% c("coxph", "clogit", "coxme")) {
                event_var <- get_event_variable(model, model_class)
            } else if (model_class == "negbin" && !isS4(model)) {
                ## Negative binomial always has events (count data)
                if (!is.null(model$formula)) {
                    outcome_var <- all.vars(model$formula)[1]
                } else {
                    outcome_var <- all.vars(stats::formula(model))[1]
                }
            } else if (model_class == "glm" && !isS4(model)) {
                ## Only set outcome_var for GLM families with meaningful events
                family_name <- model$family$family
                if (family_name %in% event_families) {
                    if (!is.null(model$formula)) {
                        outcome_var <- all.vars(model$formula)[1]
                    } else {
                        outcome_var <- all.vars(stats::formula(model))[1]
                    }
                }
            } else if (model_class %in% c("lmer", "lmerMod", "glmer", "glmerMod") && inherits(model, "merMod")) {
                ## For mixed models, only calculate events for binomial/poisson families
                fam <- tryCatch(family(model)$family, error = function(e) "gaussian")
                if (fam %in% event_families) {
                    formula_obj <- model@call$formula
                    if (!is.null(formula_obj)) {
                        outcome_var <- all.vars(formula_obj)[1]
                    }
                }
            }
            
            ## Calculate counts for each interaction term
            for (row_idx in interaction_rows) {
                term_name <- dt$term[row_idx]
                
                ## Split interaction term into components (\emph{e.g.,} "treatmentDrug A:stageII" -> c("treatmentDrug A", "stageII"))
                components <- strsplit(term_name, ":", fixed = TRUE)[[1]]
                
                ## Parse each component into variable and level
                conditions <- list()
                valid_interaction <- TRUE
                
                for (comp in components) {
                    ## Try to match against xlevels to find variable name and level
                    found <- FALSE
                    
                    if (!is.null(xlevels)) {
                        for (var_name in names(xlevels)) {
                            if (startsWith(comp, var_name)) {
                                level_val <- substring(comp, nchar(var_name) + 1)
                                if (nchar(level_val) > 0 && var_name %in% names(data_dt)) {
                                    conditions[[length(conditions) + 1]] <- list(var = var_name, level = level_val)
                                    found <- TRUE
                                    break
                                }
                            }
                        }
                    }
                    
                    ## If not found in xlevels, treat as continuous variable
                    if (!found) {
                        ## Check if the component is a known variable name (continuous)
                        if (comp %in% names(data_dt)) {
                            ## For continuous variables, cannot filter by level
                            ## Just mark it as continuous (no filtering needed for count)
                            conditions[[length(conditions) + 1]] <- list(var = comp, level = NULL, continuous = TRUE)
                            found <- TRUE
                        }
                    }
                    
                    if (!found) {
                        valid_interaction <- FALSE
                        break
                    }
                }
                
                ## Calculate counts if all components were parsed successfully
                if (valid_interaction && length(conditions) > 0) {
                    ## Build filter expression for all factor conditions
                    filter_expr_parts <- character()
                    
                    for (cond in conditions) {
                        if (!isTRUE(cond$continuous) && !is.null(cond$level)) {
                            ## Factor variable - filter by level
                            filter_expr_parts <- c(filter_expr_parts, 
                                                   sprintf("get('%s') == '%s'", cond$var, cond$level))
                        }
                    }
                    
                    if (length(filter_expr_parts) > 0) {
                        ## Create combined filter expression
                        filter_expr <- paste(filter_expr_parts, collapse = " & ")
                        
                        ## Calculate n_group
                        n_val <- tryCatch({
                            filtered_dt <- data_dt[eval(parse(text = filter_expr))]
                            nrow(filtered_dt)
                        }, error = function(e) NA_real_)
                        
                        ## Calculate events_group if applicable
                        events_val <- NA_real_
                        
                        if (!is.null(event_var) && event_var %in% names(data_dt)) {
                            events_val <- tryCatch({
                                filtered_dt <- data_dt[eval(parse(text = filter_expr))]
                                sum(filtered_dt[[event_var]], na.rm = TRUE)
                            }, error = function(e) NA_real_)
                        } else if (!is.null(outcome_var) && outcome_var %in% names(data_dt)) {
                            events_val <- tryCatch({
                                filtered_dt <- data_dt[eval(parse(text = filter_expr))]
                                outcome_col <- filtered_dt[[outcome_var]]
                                if (is.factor(outcome_col)) {
                                    sum(as.numeric(outcome_col) == 2, na.rm = TRUE)
                                } else {
                                    sum(outcome_col, na.rm = TRUE)
                                }
                            }, error = function(e) NA_real_)
                        }
                        
                        ## Update the dt row
                        if (!is.na(n_val)) {
                            dt[row_idx, n_group := n_val]
                        }
                        if (!is.na(events_val)) {
                            dt[row_idx, events_group := events_val]
                        }
                    }
                }
            }
        }
    }
    
    ## Add reference rows for factor variables while maintaining original order
    ## Create all reference rows at once (vectorized approach)
    xlevels_ref <- xlevels
    
    if (!is.null(xlevels_ref) && reference_rows && length(xlevels_ref) > 0) {

        ## Add reference column to existing data
        dt[, reference := ""]
        
        ## Build reference counts data.table from all_counts if available
        ref_vars <- names(xlevels_ref)
        ref_levels <- vapply(xlevels_ref, `[`, character(1), 1)
        
        ## Create a lookup table for reference level counts
        ## Check all_counts more carefully - it may not exist or may be NULL
        have_all_counts <- !is.null(all_counts) && 
            inherits(all_counts, "data.table") && 
            nrow(all_counts) > 0 &&
            all(c("variable", "group") %in% names(all_counts))
        
        if (have_all_counts) {
            ## Build reference lookup from all_counts in one operation
            ref_lookup <- data.table::data.table(
                                          variable = ref_vars,
                                          group = ref_levels
                                      )
            ref_lookup <- all_counts[ref_lookup, on = .(variable, group)]
            
            ## A reference level absent from all_counts has no observations
            ## among the analysis rows and reports zero rather than inheriting
            ## the model-level total. Variables that were not counted at all are
            ## left missing, as their counts are unknown rather than zero.
            ref_counted <- ref_lookup$variable %in% factor_vars
            
            if (any(ref_counted) && "n_group" %in% names(ref_lookup)) {
                ref_lookup[ref_counted & is.na(n_group), n_group := 0]
                
                if ("events_group" %in% names(ref_lookup)) {
                    ref_lookup[ref_counted & is.na(events_group), events_group := 0]
                }
            }
        } else if (!skip_counts && !is.null(data_dt) && inherits(data_dt, "data.table")) {
            ## Fallback: calculate reference counts directly
            ref_counts_list <- lapply(seq_along(ref_vars), function(i) {
                var <- ref_vars[i]
                ref_level <- ref_levels[i]
                if (var %in% names(data_dt)) {
                    if (!is.null(event_var) && event_var %in% names(data_dt)) {
                        ## Survival models - use event_var
                        data_dt[get(var) == ref_level & !is.na(get(var)), .(
                                                                              variable = var,
                                                                              group = ref_level,
                                                                              n_group = .N,
                                                                              events_group = sum(get(event_var), na.rm = TRUE)
                                                                          )]
                    } else if (!is.null(outcome_var) && outcome_var %in% names(data_dt)) {
                        ## GLM/negbin models - use outcome_var
                        outcome_col <- data_dt[[outcome_var]]
                        if (is.factor(outcome_col)) {
                            events_sum <- sum((as.numeric(outcome_col) == 2)[data_dt[[var]] == ref_level & !is.na(data_dt[[var]])], na.rm = TRUE)
                        } else {
                            events_sum <- sum(outcome_col[data_dt[[var]] == ref_level & !is.na(data_dt[[var]])], na.rm = TRUE)
                        }
                        data_dt[get(var) == ref_level & !is.na(get(var)), .(
                                                                              variable = var,
                                                                              group = ref_level,
                                                                              n_group = .N,
                                                                              events_group = events_sum
                                                                          )]
                    } else {
                        data_dt[get(var) == ref_level & !is.na(get(var)), .(
                                                                              variable = var,
                                                                              group = ref_level,
                                                                              n_group = .N
                                                                          )]
                    }
                } else {
                    NULL
                }
            })
            ref_lookup <- data.table::rbindlist(ref_counts_list[!vapply(ref_counts_list, is.null, logical(1))], fill = TRUE)
        } else {
            ref_lookup <- data.table::data.table(
                                          variable = ref_vars,
                                          group = ref_levels,
                                          n_group = NA_real_
                                      )
        }
        
        ## Ensure ref_lookup has required columns
        if (!inherits(ref_lookup, "data.table") || nrow(ref_lookup) == 0) {
            ref_lookup <- data.table::data.table(
                                          variable = ref_vars,
                                          group = ref_levels,
                                          n_group = NA_real_
                                      )
        }
        
        ## Get the column structure from dt
        dt_cols <- names(dt)
        
        ## Find which variables have matching rows in dt
        vars_in_dt <- ref_vars[ref_vars %in% unique(dt$variable)]
        
        if (length(vars_in_dt) > 0 && nrow(dt) > 0) {
            ## Get template values from first matching row of each variable
            template_dt <- dt[, .SD[1], by = variable][variable %in% vars_in_dt]
            
            ## Build reference rows data.table directly
            ref_rows_dt <- data.table::data.table(
                                           variable = vars_in_dt,
                                           group = ref_levels[match(vars_in_dt, ref_vars)],
                                           term = paste0(vars_in_dt, ref_levels[match(vars_in_dt, ref_vars)]),
                                           coefficient = 0,
                                           se = NA_real_,
                                           coef = 0,
                                           coef_lower = NA_real_,
                                           coef_upper = NA_real_,
                                           exp_coef = 1,
                                           exp_lower = NA_real_,
                                           exp_upper = NA_real_,
                                           statistic = NA_real_,
                                           p_value = NA_real_,
                                           reference = reference_label
                                       )
            
            ## Add counts from ref_lookup
            if (nrow(ref_lookup) > 0 && "n_group" %in% names(ref_lookup)) {
                ref_rows_dt <- ref_lookup[ref_rows_dt, on = .(variable, group)]
            } else {
                ref_rows_dt[, n_group := NA_real_]
                if ("events_group" %in% dt_cols) {
                    ref_rows_dt[, events_group := NA_real_]
                }
            }
            
            ## Copy metadata columns from template using a proper join
            ## This avoids issues with match() returning wrong-length vectors
            meta_cols <- c("model_scope", "model_type", "n", "events")
            meta_cols_to_add <- meta_cols[meta_cols %in% dt_cols & !meta_cols %in% names(ref_rows_dt)]
            
            if (length(meta_cols_to_add) > 0) {
                ## Select only the required columns from the template
                template_subset <- template_dt[, c("variable", meta_cols_to_add), with = FALSE]
                ## Ensure no duplicates in template
                template_subset <- unique(template_subset, by = "variable")
                ## Join to add metadata columns
                ref_rows_dt <- template_subset[ref_rows_dt, on = "variable"]
            }
            
            ## Update n and events with group-specific values where available
            if ("n_group" %in% names(ref_rows_dt) && "n" %in% names(ref_rows_dt)) {
                ref_rows_dt[!is.na(n_group), n := n_group]
            }
            if ("events_group" %in% names(ref_rows_dt) && "events" %in% names(ref_rows_dt)) {
                ref_rows_dt[!is.na(events_group), events := events_group]
            }
            
            ## Add model-specific effect columns
            if ("HR" %in% dt_cols) {
                ref_rows_dt[, `:=`(HR = 1, ci_lower = NA_real_, ci_upper = NA_real_)]
            }
            if ("OR" %in% dt_cols) {
                ref_rows_dt[, `:=`(OR = 1, ci_lower = NA_real_, ci_upper = NA_real_)]
            }
            if ("RR" %in% dt_cols) {
                ref_rows_dt[, `:=`(RR = 1, ci_lower = NA_real_, ci_upper = NA_real_)]
            }
            if ("Coefficient" %in% dt_cols) {
                ref_rows_dt[, `:=`(Coefficient = 0, ci_lower = NA_real_, ci_upper = NA_real_)]
            }
            
            ## Add any QC stat columns that exist in dt
            qc_cols <- intersect(dt_cols, c("AIC", "BIC", "deviance", "null_deviance", 
                                            "loglik", "c_statistic", "rsq", "rsq_adj"))
            for (col in qc_cols) {
                if (!(col %in% names(ref_rows_dt))) {
                    ref_rows_dt[, (col) := template_dt[[col]][match(variable, template_dt$variable)]]
                }
            }
            
            ## Combine main dt and reference rows
            dt <- data.table::rbindlist(list(dt, ref_rows_dt), use.names = TRUE, fill = TRUE)
            
            ## Sort to put reference rows in correct positions
            ## Within each variable, reference row comes first, then levels in original order
            dt[, .is_ref := reference == reference_label]
            
            ## Create group order based on original factor levels from xlevels
            ## This ensures levels appear in their defined order, not alphabetically
            dt[, .group_order := {
                order_val <- rep(NA_integer_, .N)
                for (var in names(xlevels_ref)) {
                    var_mask <- variable == var
                    if (any(var_mask)) {
                        lvls <- xlevels_ref[[var]]
                        order_val[var_mask] <- match(group[var_mask], lvls)
                    }
                }
                ## For non-factor variables (NA order), use 0 to sort first
                order_val[is.na(order_val)] <- 0L
                order_val
            }]
            
            data.table::setorder(dt, variable, -.is_ref, .group_order)
            dt[, c(".is_ref", ".group_order") := NULL]
        }
    }

    ## Update n and events with group-specific counts where available  
    dt[!is.na(n_group), n := n_group]
    if ("events_group" %in% names(dt)) {
        dt[!is.na(events_group), events := events_group]
    }
    
    ## Filter excluded terms
    if (!is.null(terms_to_exclude)) {
        dt <- dt[!term %in% terms_to_exclude]
    }
    
    ## Add significance indicators
    dt[, `:=`(
        sig = data.table::fcase(
                              is.na(p_value), "",
                              p_value < 0.001, "***",
                              p_value < 0.01, "**",
                              p_value < 0.05, "*",
                              p_value < 0.1, ".",
                              default = ""
                          ),
        sig_binary = !is.na(p_value) & p_value < 0.05
    )]

    ## Reorder variables to match original predictor order if available
    predictors_order <- attr(model, "predictors")
    if (!is.null(predictors_order) && "variable" %in% names(dt)) {
        
        ## Remove random effects notation - vectorized
        clean_predictors <- predictors_order[!grepl("\\|", predictors_order)]
        
        ## Get unique variables in dt
        dt_vars <- unique(dt$variable)
        
        ## Separate interaction terms from main effects
        interaction_vars <- dt_vars[grepl(":", dt_vars, fixed = TRUE)]
        main_effect_vars <- setdiff(dt_vars, interaction_vars)
        
        ## Build ordered_vars using vectorized matching
        ## First, direct matches
        direct_matches <- clean_predictors[clean_predictors %in% main_effect_vars]
        
        ## Then, factor variable matches (where dt_vars start with predictor name)
        remaining_predictors <- setdiff(clean_predictors, direct_matches)
        factor_matches <- character()
        if (length(remaining_predictors) > 0) {
            ## For each remaining predictor, find variables that start with it
            for (pred in remaining_predictors) {
                matches <- main_effect_vars[startsWith(main_effect_vars, pred)]
                factor_matches <- c(factor_matches, setdiff(matches, c(direct_matches, factor_matches)))
            }
        }
        
        ## Combine: direct matches first, then factor matches, preserving order
        ordered_vars <- c(direct_matches, factor_matches)
        
        ## Get main effects that aren't already ordered
        unordered_main <- setdiff(main_effect_vars, ordered_vars)
        
        ## Final order: ordered variables, unordered main effects, interactions
        final_order <- unique(c(ordered_vars, unordered_main, interaction_vars))
        
        ## Use factor levels for efficient sorting
        dt[, variable := factor(variable, levels = final_order)]
        data.table::setkey(dt, variable)
        dt[, variable := as.character(variable)]
        
        ## If reference rows exist, ensure they come first within each variable
        ## and levels appear in their original order (not alphabetically)
        if ("reference" %in% names(dt)) {
            dt[, .temp_var_order := match(variable, final_order)]
            dt[, .is_ref := reference != ""]
            
            ## Create group order based on original factor levels from xlevels
            if (!is.null(xlevels) && length(xlevels) > 0) {
                dt[, .group_order := {
                    order_val <- rep(NA_integer_, .N)
                    for (var in names(xlevels)) {
                        var_mask <- variable == var
                        if (any(var_mask)) {
                            lvls <- xlevels[[var]]
                            order_val[var_mask] <- match(group[var_mask], lvls)
                        }
                    }
                    ## For non-factor variables (NA order), use 0 to sort first
                    order_val[is.na(order_val)] <- 0L
                    order_val
                }]
                data.table::setorder(dt, .temp_var_order, -.is_ref, .group_order)
                dt[, c(".temp_var_order", ".is_ref", ".group_order") := NULL]
            } else {
                data.table::setorder(dt, .temp_var_order, -.is_ref, group)
                dt[, c(".temp_var_order", ".is_ref") := NULL]
            }
        }
    }
    
    ## Add attributes
    data.table::setattr(dt, "model_class", model_class)
    data.table::setattr(dt, "formula_str", deparse(stats::formula(model)))
    
    if (model_class == "glm") {
        data.table::setattr(dt, "model_family", model$family$family)
        data.table::setattr(dt, "model_link", model$family$link)
    }
    
    if (model_class == "negbin") {
        data.table::setattr(dt, "model_family", "Negative Binomial")
        data.table::setattr(dt, "model_link", "log")
        data.table::setattr(dt, "theta", model$theta)
    }
    
    ## Cache confint result for downstream reuse by fit() and forest functions
    if (!is.null(.confint_cache)) {
        data.table::setattr(dt, "cached_confint", .confint_cache)
        data.table::setattr(dt, "cached_confint_level", .confint_cache_level)
    }

    dt[]
    return(dt)
}

## Shared expectation helpers.
##
## Helpers used by more than one test file live here so that a single definition
## governs the contract. Helpers specific to one file remain in that file.
##
## Objects defined here are visible to every test file. A test file that assigns
## to the same name shadows this definition for that file only.


## Check that a formatted result carries an effect column, optionally of a
## specific type.
##
## Applies to the display tables produced by fit(), fullfit(), uniscreen() and
## multifit(), whose effect columns combine the estimate with its confidence
## interval (for example, "OR (95% CI)"). For the unformatted columns returned
## by m2dt(), see expect_raw_effect_column() in test-m2dt.R.
expect_effect_column <- function(result, effect_type = NULL) {
    col_names <- names(result)

    ## Find effect column (contains OR, HR, RR, Coefficient, or Estimate with CI)
    effect_cols <- grep("(OR|HR|RR|Coefficient|Estimate).*CI", col_names, value = TRUE)
    expect_true(length(effect_cols) >= 1,
                info = paste("Expected effect column, found:", paste(col_names, collapse = ", ")))

    if (!is.null(effect_type)) {
        ## For linear models, both "Coefficient" and "Estimate" are valid
        if (effect_type == "Coefficient") {
            expect_true(any(grepl("Coefficient|Estimate", effect_cols)),
                        info = paste("Expected Coefficient or Estimate in columns, found:",
                                     paste(effect_cols, collapse = ", ")))
        } else {
            expect_true(any(grepl(effect_type, effect_cols)),
                        info = paste("Expected", effect_type, "in columns, found:",
                                     paste(effect_cols, collapse = ", ")))
        }
    }
}


## Check that a formatted result carries a p-value column and that its entries
## are formatted as expected.
##
## Accepted forms are a decimal value, a threshold below the reporting precision,
## an em-dash placeholder for reference rows, and an empty string for rows that
## carry no test. The threshold pattern is deliberately written as 0\\.0*1 rather
## than 0\\.001, because the reporting precision is configurable through
## p_digits and the threshold moves with it.
expect_pvalue_column <- function(result) {
    expect_true("p-value" %in% names(result))

    pvals <- result$`p-value`
    valid_pvals <- grepl("^[0-9]\\.[0-9]+$|^< 0\\.0*1$|^-$|^$", pvals)
    expect_true(all(valid_pvals),
                info = paste("Invalid p-values found:",
                             paste(pvals[!valid_pvals], collapse = ", ")))
}


## Load a pristine copy of the packaged dataset.
##
## Reading the object directly picks up whatever copy is nearest in scope, which
## may be a test file's own augmented version or, in an interactive session, a
## modified copy in the global environment. Tests that depend on the dataset
## being unmodified should use this instead.
fresh_clintrial <- function() {
    env <- new.env(parent = emptyenv())
    utils::data("clintrial", package = "summata", envir = env)
    as.data.frame(env$clintrial)
}


## Return clintrial with the first n_missing values of the named outcome set to
## NA, as a data.table. Used by the tests covering complete-case counting.
with_missing_outcome <- function(outcome, n_missing = 100L) {
    df <- fresh_clintrial()
    df[[outcome]][seq_len(n_missing)] <- NA
    data.table::as.data.table(df)
}

#' Test Suite for m2dt()
#' 
#' Comprehensive tests covering model-to-data.table conversion for
#' GLM, LM, Cox, mixed effects models, with QC stats, reference rows,
#' and various output options.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-m2dt.R")

library(data.table)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Create binary outcome for logistic regression tests
clintrial$response <- as.integer(clintrial$os_status == 1 & clintrial$os_months < 24)
clintrial_complete$response <- as.integer(clintrial_complete$os_status == 1 & clintrial_complete$os_months < 24)

## Helper function to check m2dt result structure
expect_m2dt_result <- function(result) {
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("model_scope" %in% names(result))
    expect_true("model_type" %in% names(result))
    expect_true("term" %in% names(result))
    expect_true("coefficient" %in% names(result))
    expect_true("se" %in% names(result))
    expect_true("p_value" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "model_class")))
    expect_true(!is.null(attr(result, "formula_str")))
}

## Helper to check for a raw effect column. m2dt() returns unformatted
## columns (OR, HR, coef), unlike the display columns tested elsewhere.
expect_raw_effect_column <- function(result, effect_type) {
    expect_true(effect_type %in% names(result),
                info = paste("Expected", effect_type, "column, found:", 
                             paste(names(result), collapse = ", ")))
}

## Helper to check for QC stats
expect_qc_stats <- function(result, stats) {
    for (stat in stats) {
        expect_true(stat %in% names(result),
                    info = paste("Expected QC stat", stat, "not found"))
    }
}


## ============================================================================
## SECTION 1: Basic GLM - Logistic Regression
## ============================================================================

test_that("m2dt works with logistic regression", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "OR")
    expect_equal(attr(result, "model_class"), "glm")
})


test_that("m2dt logistic shows correct model type", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_equal(unique(result$model_type), "Logistic")
})


test_that("m2dt logistic detects univariable vs multivariable", {
    
    ## Univariable
    model_uni <- glm(response ~ age, data = clintrial, family = binomial)
    result_uni <- m2dt(data = clintrial, model = model_uni)
    expect_equal(unique(result_uni$model_scope), "Univariable")
    
    ## Multivariable
    model_multi <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    result_multi <- m2dt(data = clintrial, model = model_multi)
    expect_equal(unique(result_multi$model_scope), "Multivariable")
})


test_that("m2dt logistic includes events count", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_true("events" %in% names(result))
    expect_true(all(!is.na(result$events)))
})


## ============================================================================
## SECTION 2: Linear Model
## ============================================================================

test_that("m2dt works with linear model", {
    
    model <- lm(los_days ~ age + sex, data = clintrial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "Coefficient")
    expect_equal(attr(result, "model_class"), "lm")
})


test_that("m2dt linear shows correct model type", {
    
    model <- lm(los_days ~ age, data = clintrial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_equal(unique(result$model_type), "Linear")
})


test_that("m2dt linear does not exponentiate", {
    
    model <- lm(los_days ~ age, data = clintrial)
    
    result <- m2dt(data = clintrial, model = model)
    
    ## Coefficient should equal coefficient (not exponentiated)
    expect_equal(result$Coefficient, result$coefficient)
})


## ============================================================================
## SECTION 3: Poisson Regression
## ============================================================================

test_that("m2dt works with Poisson regression", {
    
    ## Create integer count data
    test_data <- data.table::as.data.table(clintrial)
    test_data[, count_outcome := as.integer(round(los_days))]
    
    model <- suppressWarnings(
        glm(count_outcome ~ age + sex, data = test_data, family = poisson)
    )
    
    result <- m2dt(data = test_data, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "RR")
})


test_that("m2dt Poisson shows correct model type", {
    
    test_data <- data.table::as.data.table(clintrial)
    test_data[, count_outcome := as.integer(round(los_days))]
    
    model <- suppressWarnings(
        glm(count_outcome ~ age, data = test_data, family = poisson)
    )
    
    result <- m2dt(data = test_data, model = model)
    
    expect_equal(unique(result$model_type), "Poisson")
})


## ============================================================================
## SECTION 4: Cox Proportional Hazards
## ============================================================================

test_that("m2dt works with Cox regression", {
    
    model <- survival::coxph(
                           survival::Surv(os_months, os_status) ~ age + sex + stage,
                           data = clintrial
                       )
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "HR")
    expect_equal(attr(result, "model_class"), "coxph")
})


test_that("m2dt Cox shows correct model type", {
    
    model <- survival::coxph(
                           survival::Surv(os_months, os_status) ~ age,
                           data = clintrial
                       )
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_equal(unique(result$model_type), "Cox PH")
})


test_that("m2dt Cox includes events count", {
    
    model <- survival::coxph(
                           survival::Surv(os_months, os_status) ~ age + sex,
                           data = clintrial
                       )
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_true("events" %in% names(result))
    expect_true(all(!is.na(result$events)))
})


## ============================================================================
## SECTION 5: Conditional Logistic Regression
## ============================================================================

test_that("m2dt works with conditional logistic regression", {
    
    ## Create strata variable
    test_data <- data.table::as.data.table(clintrial)
    test_data[, strata_var := rep(1:100, length.out = .N)]
    
    model <- survival::clogit(
                           response ~ age + sex + survival::strata(strata_var),
                           data = test_data
                       )
    
    result <- m2dt(data = test_data, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "HR")  # clogit uses HR
    expect_equal(attr(result, "model_class"), "clogit")
})


## ============================================================================
## SECTION 6: Mixed Effects Models - lmer
## ============================================================================

test_that("m2dt works with lmer", {
    
    skip_if_not_installed("lme4")
    
    model <- lme4::lmer(los_days ~ age + sex + (1|site), data = clintrial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "Coefficient")
    expect_true(attr(result, "model_class") %in% c("lmerMod", "lmer"))
})


test_that("m2dt lmer shows correct model type", {
    
    skip_if_not_installed("lme4")
    
    model <- lme4::lmer(los_days ~ age + (1|site), data = clintrial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_true(grepl("Mixed|lmer", unique(result$model_type), ignore.case = TRUE))
})


## ============================================================================
## SECTION 7: Mixed Effects Models - glmer
## ============================================================================

test_that("m2dt works with glmer (binomial)", {
    
    skip_if_not_installed("lme4")
    
    model <- suppressWarnings(
        lme4::glmer(response ~ age + sex + (1|site), 
                    data = clintrial, family = binomial)
    )
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "OR")
    expect_true(attr(result, "model_class") %in% c("glmerMod", "glmer"))
})


## ============================================================================
## SECTION 8: Mixed Effects Cox - coxme
## ============================================================================

test_that("m2dt works with coxme", {
    
    skip_if_not_installed("coxme")
    model <- coxme::coxme(
                        survival::Surv(os_months, os_status) ~ age + sex + (1|site),
                        data = clintrial
                    )
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    expect_raw_effect_column(result, "HR")
    expect_equal(attr(result, "model_class"), "coxme")
})


## ============================================================================
## SECTION 9: QC Statistics - keep_qc_stats Parameter
## ============================================================================

test_that("m2dt keep_qc_stats=TRUE includes QC statistics for GLM", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, keep_qc_stats = TRUE)
    
    expect_qc_stats(result, c("AIC", "BIC"))
})


test_that("m2dt keep_qc_stats=FALSE excludes QC statistics", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, keep_qc_stats = FALSE)
    
    expect_false("AIC" %in% names(result))
    expect_false("BIC" %in% names(result))
})


test_that("m2dt includes QC stats for linear model", {
    
    model <- lm(los_days ~ age + sex, data = clintrial)
    
    result <- m2dt(data = clintrial, model = model, keep_qc_stats = TRUE)
    
    expect_qc_stats(result, c("R2", "adj_R2", "AIC", "BIC"))
})


test_that("m2dt includes QC stats for Cox model", {
    
    model <- survival::coxph(
                           survival::Surv(os_months, os_status) ~ age + sex,
                           data = clintrial
                       )
    
    result <- m2dt(data = clintrial, model = model, keep_qc_stats = TRUE)
    
    expect_qc_stats(result, c("concordance", "likelihood_ratio_test"))
})


## ============================================================================
## SECTION 10: Intercept Handling
## ============================================================================

test_that("m2dt include_intercept=TRUE includes intercept", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = TRUE)
    
    expect_true("(Intercept)" %in% result$term)
})


test_that("m2dt include_intercept=FALSE excludes intercept", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    expect_false("(Intercept)" %in% result$term)
})


## ============================================================================
## SECTION 11: Reference Rows for Factors
## ============================================================================

test_that("m2dt reference_rows=TRUE adds reference rows", {
    
    model <- glm(response ~ age + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, reference_rows = TRUE)
    
    ## Should have a reference row
    expect_true("reference" %in% names(result))
    expect_true(any(result$reference == "reference"))
})


test_that("m2dt reference_rows=FALSE excludes reference rows", {
    
    model <- glm(response ~ age + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, reference_rows = FALSE)
    
    ## Result should be valid
    expect_m2dt_result(result)
    
    ## Should NOT have reference rows with "reference" label
    if ("reference" %in% names(result)) {
        expect_true(all(result$reference == "" | is.na(result$reference)))
    } else {
        ## If no reference column at all, that's also correct
        expect_true(TRUE)
    }
})


test_that("m2dt reference rows have correct values", {
    
    model <- glm(response ~ stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, reference_rows = TRUE)
    
    ## Reference row should have OR = 1, coefficient = 0
    ref_rows <- result[reference == "reference"]
    
    if (nrow(ref_rows) > 0) {
        expect_true(all(ref_rows$OR == 1))
        expect_true(all(ref_rows$coefficient == 0))
    }
})


test_that("m2dt reference_label customizes reference label", {
    
    model <- glm(response ~ stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, 
                   reference_rows = TRUE,
                   reference_label = "ref_level")
    
    expect_true(any(result$reference == "ref_level"))
})


## ============================================================================
## SECTION 12: Term Exclusion
## ============================================================================

test_that("m2dt terms_to_exclude removes specified terms", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, 
                   terms_to_exclude = c("sexMale"))
    
    expect_false("sexMale" %in% result$term)
})


test_that("m2dt terms_to_exclude with multiple terms", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, 
                   terms_to_exclude = c("age", "sexMale"))
    
    expect_false("age" %in% result$term)
    expect_false("sexMale" %in% result$term)
})


## ============================================================================
## SECTION 13: Confidence Level
## ============================================================================

test_that("m2dt respects conf_level parameter", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result_95 <- m2dt(data = clintrial, model = model, conf_level = 0.95)
    result_90 <- m2dt(data = clintrial, model = model, conf_level = 0.90)
    
    ## 90% CI should be narrower than 95% CI
    ## Column names are ci_lower/ci_upper (lowercase)
    ci_width_95 <- result_95$ci_upper[1] - result_95$ci_lower[1]
    ci_width_90 <- result_90$ci_upper[1] - result_90$ci_lower[1]
    
    expect_true(ci_width_90 < ci_width_95)
})


## ============================================================================
## SECTION 14: Significance Indicators
## ============================================================================

test_that("m2dt adds significance indicators", {
    
    model <- glm(response ~ age + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_true("sig" %in% names(result))
    expect_true("sig_binary" %in% names(result))
})


test_that("m2dt sig_binary is correct", {
    
    model <- glm(response ~ age + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    ## sig_binary should be TRUE where p < 0.05
    result_check <- result[!is.na(p_value)]
    expect_equal(result_check$sig_binary, result_check$p_value < 0.05)
})


test_that("m2dt sig markers are correct", {
    
    model <- glm(response ~ age + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    ## Check significance markers
    for (i in seq_len(nrow(result))) {
        p <- result$p_value[i]
        sig <- result$sig[i]
        
        if (is.na(p)) {
            expect_equal(sig, "")
        } else if (p < 0.001) {
            expect_equal(sig, "***")
        } else if (p < 0.01) {
            expect_equal(sig, "**")
        } else if (p < 0.05) {
            expect_equal(sig, "*")
        } else if (p < 0.1) {
            expect_equal(sig, ".")
        } else {
            expect_equal(sig, "")
        }
    }
})


## ============================================================================
## SECTION 15: Variable and Group Parsing
## ============================================================================

test_that("m2dt parses factor variables correctly", {
    
    model <- glm(response ~ stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, 
                   include_intercept = FALSE,
                   reference_rows = TRUE)
    
    ## All stage terms should have variable = "stage"
    stage_terms <- result[variable == "stage"]
    expect_true(nrow(stage_terms) > 1)  # Multiple levels
    
    ## Group should contain level names
    expect_true(all(stage_terms$group %in% c("I", "II", "III", "IV")))
})


test_that("m2dt parses continuous variables correctly", {
    
    model <- glm(response ~ age + bmi, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    ## Continuous variables should have variable = term name
    age_row <- result[variable == "age"]
    expect_equal(nrow(age_row), 1)
    expect_equal(age_row$group, "")
})


## ============================================================================
## SECTION 16: n and Events Columns
## ============================================================================

test_that("m2dt includes n column", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_true("n" %in% names(result))
    expect_true(all(!is.na(result$n)))
})


test_that("m2dt includes n_group for factor variables", {
    
    model <- glm(response ~ stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, reference_rows = TRUE)
    
    expect_true("n_group" %in% names(result))
})


test_that("m2dt events_group for factor variables in survival model", {
    
    model <- survival::coxph(
                           survival::Surv(os_months, os_status) ~ stage,
                           data = clintrial
                       )
    
    result <- m2dt(data = clintrial, model = model, reference_rows = TRUE)
    
    expect_true("events_group" %in% names(result))
})


## ============================================================================
## SECTION 17: Edge Cases
## ============================================================================

test_that("m2dt handles single predictor", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
})


test_that("m2dt handles many predictors", {
    
    model <- glm(response ~ age + sex + smoking + diabetes + hypertension + stage,
                 data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
})


test_that("m2dt handles interaction terms", {
    
    model <- glm(response ~ age + sex + age:sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_m2dt_result(result)
    
    ## Should have interaction term
    expect_true(any(grepl(":", result$term)))
})


test_that("m2dt handles factor-factor interaction", {
    
    model <- glm(response ~ sex + stage + sex:stage, data = clintrial, family = binomial)
    
    ## Suppress Hoslem test warning about bins
    result <- suppressWarnings(m2dt(data = clintrial, model = model))
    
    expect_m2dt_result(result)
})


test_that("m2dt requires data parameter", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    expect_error(
        m2dt(model = model),
        regexp = "data parameter is required"
    )
})


test_that("m2dt works with data.frame input", {
    
    df <- as.data.frame(clintrial)
    model <- glm(response ~ age + sex, data = df, family = binomial)
    
    result <- m2dt(data = df, model = model)
    
    expect_m2dt_result(result)
})


## ============================================================================
## SECTION 18: Attributes
## ============================================================================

test_that("m2dt sets model_class attribute", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_equal(attr(result, "model_class"), "glm")
})


test_that("m2dt sets formula_str attribute", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    formula_str <- attr(result, "formula_str")
    expect_true(grepl("response", formula_str))
    expect_true(grepl("age", formula_str))
})


test_that("m2dt sets family attributes for GLM", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_equal(attr(result, "model_family"), "binomial")
    expect_equal(attr(result, "model_link"), "logit")
})


## ============================================================================
## SECTION 19: Coefficient and CI Columns
## ============================================================================

test_that("m2dt includes both raw and exponentiated coefficients", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    ## Should have raw coefficients
    expect_true("coefficient" %in% names(result))
    expect_true("coef" %in% names(result))
    expect_true("coef_lower" %in% names(result))
    expect_true("coef_upper" %in% names(result))
    
    ## Should have exponentiated coefficients
    expect_true("exp_coef" %in% names(result))
    expect_true("exp_lower" %in% names(result))
    expect_true("exp_upper" %in% names(result))
})


test_that("m2dt OR equals exp(coefficient) for logistic", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    expect_equal(result$OR, exp(result$coefficient), tolerance = 1e-10)
})


test_that("m2dt HR equals exp(coefficient) for Cox", {
    
    model <- survival::coxph(
                           survival::Surv(os_months, os_status) ~ age,
                           data = clintrial
                       )
    
    result <- m2dt(data = clintrial, model = model)
    
    expect_equal(result$HR, exp(result$coefficient), tolerance = 1e-10)
})


## ============================================================================
## SECTION 20: Consistency Tests
## ============================================================================

test_that("m2dt produces consistent results across runs", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    result1 <- m2dt(data = clintrial, model = model)
    result2 <- m2dt(data = clintrial, model = model)
    
    expect_equal(result1$coefficient, result2$coefficient)
    expect_equal(result1$p_value, result2$p_value)
})


test_that("m2dt coefficients match summary(model)", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = TRUE)
    
    model_coefs <- coef(model)
    
    for (term_name in names(model_coefs)) {
        result_coef <- result[term == term_name]$coefficient
        expected_coef <- unname(model_coefs[term_name])
        expect_equal(result_coef, expected_coef, 
                     tolerance = 1e-10,
                     info = paste("Coefficient mismatch for", term_name))
    }
})


test_that("m2dt p-values match summary(model)", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = TRUE)
    
    model_summary <- summary(model)$coefficients
    
    for (term_name in rownames(model_summary)) {
        result_p <- result[term == term_name]$p_value
        summary_p <- unname(model_summary[term_name, "Pr(>|z|)"])
        expect_equal(result_p, summary_p, 
                     tolerance = 1e-10,
                     info = paste("P-value mismatch for", term_name))
    }
})


## ============================================================================
## SECTION 21: Confidence Interval Methods
## ============================================================================

test_that("m2dt GLM CIs use profile likelihood (differ from Wald)", {
    
    model <- glm(response ~ age + sex + stage, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    ## Compute Wald CIs for comparison
    coefs <- coef(model)[-1]  # exclude intercept
    ses <- summary(model)$coefficients[-1, "Std. Error"]
    z <- qnorm(0.975)
    wald_lower <- exp(coefs - z * ses)
    wald_upper <- exp(coefs + z * ses)
    
    ## Profile CIs should differ from Wald (at least slightly)
    ## Get non-reference rows only
    non_ref <- result[is.na(reference) | reference == ""]
    
    ## Allow for the possibility that differences are very small for
    ## well-behaved models, but at least one should differ
    any_lower_differs <- any(abs(non_ref$ci_lower - wald_lower) > 1e-6)
    any_upper_differs <- any(abs(non_ref$ci_upper - wald_upper) > 1e-6)
    
    expect_true(any_lower_differs || any_upper_differs,
                info = "Profile CIs should differ from Wald for at least one term")
})


test_that("m2dt lm CIs use exact t-distribution (differ from normal)", {
    
    model <- lm(los_days ~ age + sex, data = clintrial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    ## Compute normal-approximation (Wald) CIs for comparison
    coefs <- coef(model)[-1]
    ses <- summary(model)$coefficients[-1, "Std. Error"]
    z <- qnorm(0.975)
    wald_lower <- coefs - z * ses
    
    ## Exact t CIs use qt() which gives wider intervals than qnorm()
    non_ref <- result[is.na(reference) | reference == ""]
    
    ## t-distribution CIs should be slightly wider than normal CIs
    ## (because t quantile > z quantile for finite df)
    expect_true(all(non_ref$ci_lower <= wald_lower + 1e-10),
                info = "t-distribution lower bounds should be at most equal to normal bounds")
})


test_that("m2dt quasi-family CIs use Wald (match confint.default)", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = quasibinomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    ## Compute Wald CIs explicitly
    coefs <- coef(model)[-1]
    ses <- summary(model)$coefficients[-1, "Std. Error"]
    z <- qnorm(0.975)
    wald_lower <- exp(coefs - z * ses)
    wald_upper <- exp(coefs + z * ses)
    
    non_ref <- result[is.na(reference) | reference == ""]
    
    ## Quasi-family CIs should match Wald exactly
    expect_equal(unname(non_ref$ci_lower), unname(wald_lower), tolerance = 1e-10,
                 info = "Quasi-family should use Wald CIs")
    expect_equal(unname(non_ref$ci_upper), unname(wald_upper), tolerance = 1e-10,
                 info = "Quasi-family should use Wald CIs")
})


test_that("m2dt caches confint result as attribute", {
    
    model <- glm(response ~ age + sex, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model)
    
    ## Should have cached confint attribute
    cached <- attr(result, "cached_confint")
    expect_true(!is.null(cached))
    expect_true(is.matrix(cached))
    
    ## Cached level should match
    expect_equal(attr(result, "cached_confint_level"), 0.95)
})


test_that("m2dt confint cache respects conf_level", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result_95 <- m2dt(data = clintrial, model = model, conf_level = 0.95)
    result_90 <- m2dt(data = clintrial, model = model, conf_level = 0.90)
    
    expect_equal(attr(result_95, "cached_confint_level"), 0.95)
    expect_equal(attr(result_90, "cached_confint_level"), 0.90)
    
    ## CI bounds should differ between levels
    cached_95 <- attr(result_95, "cached_confint")
    cached_90 <- attr(result_90, "cached_confint")
    expect_false(identical(cached_95, cached_90))
})


test_that("m2dt skips intercept profiling when include_intercept=FALSE", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    ## With intercept excluded, profiling should skip it
    result <- m2dt(data = clintrial, model = model, include_intercept = FALSE)
    
    ## Should still produce valid CIs for non-intercept terms
    expect_true(all(!is.na(result$ci_lower)))
    expect_true(all(!is.na(result$ci_upper)))
    
    ## No intercept row should be present
    expect_false("(Intercept)" %in% result$term)
})


test_that("m2dt profiles intercept when include_intercept=TRUE", {
    
    model <- glm(response ~ age, data = clintrial, family = binomial)
    
    result <- m2dt(data = clintrial, model = model, include_intercept = TRUE)
    
    ## Intercept should have valid CIs (not NA)
    intercept_row <- result[term == "(Intercept)"]
    expect_equal(nrow(intercept_row), 1)
    expect_true(!is.na(intercept_row$ci_lower))
    expect_true(!is.na(intercept_row$ci_upper))
})

## Group-level sample sizes must be counted over the observations that entered
## the model. Fitted objects that retain no model frame (survival::coxph() by
## default, coxme, and the memory-conserving model = FALSE fits used internally)
## previously fell back to the unrestricted data, so categorical predictors
## reported totals over every supplied row while continuous predictors reported
## complete cases only.

## Sample sizes for a single variable, including its reference level.
##
## Indexing is performed on extracted vectors rather than through the data.table
## '[' method: an argument named for a column is otherwise resolved against that
## column rather than against the calling environment.
group_sizes <- function(raw, var_name) {
    raw[["n"]][raw[["variable"]] == var_name]
}

## Event counts for a single variable, including its reference level.
group_events <- function(raw, var_name) {
    raw[["events"]][raw[["variable"]] == var_name]
}

## Count events using the same convention as m2dt(): for factor outcomes the
## second level is the event.
count_events <- function(x) {
    if (is.factor(x)) {
        return(sum(as.integer(x) == 2L, na.rm = TRUE))
    }
    return(sum(x, na.rm = TRUE))
}


test_that("uniscreen group sizes exclude observations with a missing response", {

    dt <- with_missing_outcome("readmission_30d", 100L)
    n_complete <- sum(!is.na(dt$readmission_30d))

    result <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex", "race", "ethnicity"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    raw <- attr(result, "raw_data")

    ## Continuous predictor: n is the fitted sample size
    expect_equal(unique(group_sizes(raw, "age")), n_complete)

    ## Categorical predictors: group sizes partition the fitted sample size
    for (pred in c("sex", "race", "ethnicity")) {
        expect_equal(sum(group_sizes(raw, pred)), n_complete)
    }
})


test_that("uniscreen event counts remain consistent with group sizes", {

    dt <- with_missing_outcome("readmission_30d", 100L)
    total_events <- count_events(dt$readmission_30d)

    result <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("sex", "race"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    raw <- attr(result, "raw_data")

    for (pred in c("sex", "race")) {
        expect_equal(sum(group_events(raw, pred)), total_events)
        expect_true(all(group_events(raw, pred) <= group_sizes(raw, pred)))
    }
})


test_that("multivariable fit group sizes match the fitted sample size", {

    dt <- with_missing_outcome("readmission_30d", 100L)

    result <- fit(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("age", "sex", "race"),
        model_type = "glm",
        family = "binomial"
    )

    raw <- attr(result, "raw_data")
    n_model <- stats::nobs(attr(result, "model"))

    for (pred in c("sex", "race")) {
        expect_equal(sum(group_sizes(raw, pred)), n_model)
    }
    expect_equal(unique(group_sizes(raw, "age")), n_model)
})


test_that("Cox models count only the observations used in fitting", {

    dt <- with_missing_outcome("os_status", 100L)

    result <- fit(
        data = dt,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "race"),
        model_type = "coxph"
    )

    raw <- attr(result, "raw_data")
    model <- attr(result, "model")

    for (pred in c("sex", "race")) {
        expect_equal(sum(group_sizes(raw, pred)), model$n)
        expect_equal(sum(group_events(raw, pred)), model$nevent)
    }
})


test_that("group counts do not depend on whether the model frame was retained", {

    dt <- with_missing_outcome("readmission_30d", 100L)

    with_frame <- stats::glm(readmission_30d ~ age + sex + race,
                             data = dt, family = stats::binomial,
                             model = TRUE)
    without_frame <- stats::glm(readmission_30d ~ age + sex + race,
                                data = dt, family = stats::binomial,
                                model = FALSE, y = TRUE)

    kept <- m2dt(dt, with_frame, keep_qc_stats = FALSE)
    dropped <- m2dt(dt, without_frame, keep_qc_stats = FALSE)

    expect_equal(kept[["n"]], dropped[["n"]])
    expect_equal(kept[["events"]], dropped[["events"]])
})


test_that("factor levels with no analysis rows report zero, not the model total", {

    ## Every observation in one race category has a missing event indicator, so
    ## that level contributes no analysis rows while remaining in the factor.
    ## Unlike stats::glm(), survival::coxph() retains the singular coefficient,
    ## so the emptied level still appears as a row in the output.
    df <- fresh_clintrial()
    df$os_status[df$race == "Other"] <- NA
    dt <- data.table::as.data.table(df)

    result <- fit(
        data = dt,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "race"),
        model_type = "coxph"
    )

    raw <- attr(result, "raw_data")
    model <- attr(result, "model")

    ## Group sizes still partition the fitted sample size
    expect_equal(sum(group_sizes(raw, "race")), model$n)
    expect_equal(sum(group_events(raw, "race")), model$nevent)

    ## The emptied level reports zero rather than inheriting the total
    empty_row <- raw[["variable"]] == "race" & raw[["group"]] == "Other"

    if (any(empty_row)) {
        expect_equal(raw[["n"]][empty_row], 0)
        expect_equal(raw[["events"]][empty_row], 0)
    }
})


test_that("complete data is unaffected by the analysis-row restriction", {

    dt <- data.table::as.data.table(fresh_clintrial())
    n_total <- nrow(dt)

    result <- uniscreen(
        data = dt,
        outcome = "readmission_30d",
        predictors = c("sex", "race"),
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )

    raw <- attr(result, "raw_data")

    for (pred in c("sex", "race")) {
        expect_equal(sum(group_sizes(raw, pred)), n_total)
    }
})

### * Main function

#' Create Forest Plot with Automatic Model Detection
#'
#' A convenience wrapper function that automatically detects the input type and
#' routes to the appropriate specialized forest plot function. This eliminates
#' the need to remember which forest function to call for different model types
#' or analysis objects, making it ideal for exploratory analysis and rapid prototyping.
#'
#' @param x One of the following:
#'   \itemize{
#'     \item A fitted model object: \code{glm}, \code{lm}, \code{coxph}, \emph{etc.}
#'     \item A \code{fit_result} object from \code{fit()}
#'     \item A \code{fullfit_result} object from \code{fullfit()}
#'     \item A \code{uniscreen_result} object from \code{uniscreen()}
#'     \item A \code{multifit_result} object from \code{multifit()}
#'   }
#'
#' @param data Data frame or data.table containing the original data. Required 
#'   when \code{x} is a raw model object. Ignored when \code{x} is a result object
#'   from \code{fit()}, \code{fullfit()}, \code{uniscreen()}, or \code{multifit()}
#'   since these contain embedded data.
#'
#' @param title Character string for plot title. If \code{NULL} (default), an 
#'   appropriate title is generated based on the detected model type:
#'   \itemize{
#'     \item Cox models: "Cox Proportional Hazards Model"
#'     \item Logistic regression: "Logistic Regression Model"
#'     \item Poisson regression: "Poisson Regression Model"
#'     \item Linear regression: "Linear Regression Model"
#'     \item Uniscreen results: "Univariable [Type] Screening"
#'     \item Multifit results: "Multivariate [Type] Analysis"
#'   }
#'
#' @param ... Additional arguments passed to the specific forest plot function.
#'   Common arguments include:
#'   \describe{
#'     \item{labels}{Named character vector for variable labels}
#'     \item{digits}{Number of decimal places for estimates (default 2)}
#'     \item{p_digits}{Number of decimal places for \emph{p}-values (default 3)}
#'     \item{conf_level}{Confidence level for intervals (default 0.95)}
#'     \item{show_n}{Logical, show sample sizes (default \code{TRUE})}
#'     \item{show_events}{Logical, show event counts (default \code{TRUE} for survival/binomial)}
#'     \item{qc_footnote}{Logical, show model QC stats in footer (default \code{TRUE})}
#'     \item{zebra_stripes}{Logical, alternating row shading (default \code{TRUE})}
#'     \item{indent_groups}{Logical, indent factor levels (default \code{FALSE})}
#'     \item{color}{Color for point estimates}
#'     \item{table_width}{Proportion of width for table (default 0.6)}
#'     \item{plot_width, plot_height}{Explicit dimensions}
#'     \item{units}{Dimension units: \code{"in"}, \code{"cm"}, or \code{"mm"}}
#'   }
#'   See the documentation for the specific forest function for all available options.
#'
#' @return A \code{ggplot} object containing the complete forest plot. The plot 
#'   can be:
#'   \itemize{
#'     \item Displayed directly: \code{print(plot)}
#'     \item Saved to file: \code{forestsave(plot, "forest.pdf")}
#'     \item Further customized with \pkg{ggplot2} functions
#'   }
#'   
#'   The returned object includes an attribute \code{"rec_dims"} 
#'   accessible via \code{attr(plot, "rec_dims")}, which is a list 
#'   containing:
#'   \describe{
#'     \item{width}{Numeric. Recommended plot width in specified units}
#'     \item{height}{Numeric. Recommended plot height in specified units}
#'   }
#'   
#'   These recommendations are automatically calculated based on the number of 
#'   variables, text sizes, and layout parameters, and are printed to console 
#'   if \code{plot_width} or \code{plot_height} are not specified.
#'
#' @details
#'
#' This function provides a convenient wrapper around the specialized forest
#' plot functions, automatically routing to the appropriate function based on
#' the model class or result type. All parameters are passed through to the
#' underlying function, so the full range of options remains available.
#' 
#' For model-specific advanced features, individual forest functions may be
#' called directly.
#'
#' \strong{Automatic Detection Logic:}
#' 
#' The function uses the following priority order for detection:
#' \enumerate{
#'   \item \strong{uniscreen results}: Detected by class \code{"uniscreen_result"} or
#'     presence of attributes \code{outcome}, \code{predictors}, \code{model_type},
#'     and \code{model_scope = "Univariable"}. Routes to \code{uniforest()}.
#'   \item \strong{multifit results}: Detected by presence of attributes 
#'     \code{predictor}, \code{outcomes}, \code{model_type}, and \code{raw_data}.
#'     Routes to \code{multiforest()}.
#'   \item \strong{Cox models}: Classes \code{coxph} or \code{clogit}. Routes to 
#'     \code{coxforest()}.
#'   \item \strong{GLM models}: Class \code{glm}. Routes to \code{glmforest()}.
#'   \item \strong{Linear models}: Class \code{lm} (but not \code{glm}). Routes to 
#'     \code{lmforest()}.
#' }
#' 
#' @seealso 
#' \code{\link{glmforest}} for GLM forest plots,
#' \code{\link{coxforest}} for Cox model forest plots,
#' \code{\link{lmforest}} for linear model forest plots,
#' \code{\link{uniforest}} for univariable screening forest plots,
#' \code{\link{multiforest}} for multi-outcome forest plots,
#' \code{\link{fit}} for single-model regression,
#' \code{\link{fullfit}} for combined univariable/multivariable regression,
#' \code{\link{uniscreen}} for univariable screening,
#' \code{\link{multifit}} for multi-outcome analysis,
#'   \code{\link{forestsave}} for saving with recommended dimensions
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#' library(survival)
#'
#' # Create example model
#' glm_model <- glm(surgery ~ age + sex + bmi + smoking,
#'                  family = binomial, data = clintrial)
#'
#' # Example 1: Logistic regression model
#' p <- autoforest(glm_model, data = clintrial)
#' # Automatically detects GLM and routes to glmforest()
#'
#' \donttest{
#'
#' # Example 2: Cox proportional hazards model
#' cox_model <- coxph(Surv(os_months, os_status) ~ age + sex + treatment + stage,
#'                    data = clintrial)
#' 
#' plot2 <- autoforest(cox_model, data = clintrial)
#' # Automatically detects coxph and routes to coxforest()
#' 
#' # Example 3: Linear regression model
#' lm_model <- lm(biomarker_x ~ age + sex + bmi + treatment, data = clintrial)
#' 
#' plot3 <- autoforest(lm_model, data = clintrial)
#' # Automatically detects lm and routes to lmforest()
#' 
#' # Example 4: With custom labels and formatting options
#' plot4 <- autoforest(
#'     cox_model,
#'     data = clintrial,
#'     labels = clintrial_labels,
#'     title = "Prognostic Factors for Overall Survival",
#'     zebra_stripes = TRUE,
#'     indent_groups = TRUE
#' )
#' 
#' # Example 5: From fit() result - data and labels extracted automatically
#' fit_result <- fit(
#'     data = clintrial,
#'     outcome = "surgery",
#'     predictors = c("age", "sex", "bmi", "treatment"),
#'     labels = clintrial_labels
#' )
#' 
#' plot5 <- autoforest(fit_result)
#' 
#' # Save with recommended dimensions
#' forestsave(plot5, file.path(tempdir(), "forest.pdf"))
#'
#' }
#' @family visualization functions
#' @export
autoforest <- function(x, data = NULL, title = NULL, ...) {
    
    ## First check if this is a fit_result or fullfit_result
    ## These contain a model object that should be extracted and routed
    if (inherits(x, "fit_result") || inherits(x, "fullfit_result")) {
        ## Extract the model from the result
        model <- attr(x, "model")
        
        if (is.null(model)) {
            stop("The fit_result/fullfit_result does not contain a model.\n",
                 "This may occur if fullfit() was run with columns='uni' only.")
        }
        
        ## Extract data if not provided
        if (is.null(data)) {
            data <- attr(x, "data")
        }
        
        ## Extract labels from the result's ... or attributes
        dots <- list(...)
        if (is.null(dots$labels)) {
            result_labels <- attr(x, "labels")
            if (!is.null(result_labels)) {
                dots$labels <- result_labels
            }
        }
        
        ## Determine model type and route to appropriate forest function
        if (inherits(model, "coxph") || inherits(model, "clogit") || inherits(model, "coxme")) {
            if (is.null(title)) {
                if (inherits(model, "coxme")) {
                    title <- "Mixed-Effects Cox Proportional Hazards"
                } else if (inherits(model, "clogit")) {
                    title <- "Conditional Logistic Regression"
                } else {
                    title <- "Cox Proportional Hazards Model"
                }
            }
            return(do.call(coxforest, c(list(x = x, data = data, title = title), dots)))
            
        } else if (inherits(model, "glmerMod")) {
            family_name <- model@resp$family$family
            if (is.null(title)) {
                if (family_name == "binomial") {
                    title <- "Mixed-Effects Logistic Regression"
                } else if (family_name == "poisson") {
                    title <- "Mixed-Effects Poisson Regression"
                } else {
                    title <- "Mixed-Effects Generalized Linear Model"
                }
            }
            return(do.call(glmforest, c(list(x = x, data = data, title = title), dots)))
            
        } else if (inherits(model, "lmerMod")) {
            if (is.null(title)) title <- "Mixed-Effects Linear Regression"
            return(do.call(lmforest, c(list(x = x, data = data, title = title), dots)))
            
        } else if (inherits(model, "glm")) {
            if (is.null(title)) {
                if (model$family$family == "binomial") {
                    title <- "Logistic Regression Model"
                } else if (model$family$family == "poisson") {
                    title <- "Poisson Regression Model"
                } else {
                    title <- "Generalized Linear Model"
                }
            }
            return(do.call(glmforest, c(list(x = x, data = data, title = title), dots)))
            
        } else if (inherits(model, "lm")) {
            if (is.null(title)) title <- "Linear Regression Model"
            return(do.call(lmforest, c(list(x = x, data = data, title = title), dots)))
            
        } else {
            stop("Model type in fit_result/fullfit_result is not supported for forest plots.")
        }
    }
    
    ## Check if this is a uniscreen result
    if (is_uniscreen_result(x)) {
        ## Generate appropriate title if not provided
        if (is.null(title)) {
            model_type <- attr(x, "model_type")
            title <- switch(model_type,
                            "glm" = "Univariable Logistic Regression Screening",
                            "coxph" = "Univariable Survival Analysis Screening",
                            "lm" = "Univariable Linear Regression Screening",
                            "glmer" = "Univariable Mixed-Effects Logistic Screening",
                            "lmer" = "Univariable Mixed-Effects Linear Screening",
                            "coxme" = "Univariable Mixed-Effects Survival Screening",
                            "Univariable Screening"
                            )
        }
        
        return(uniforest(x = x, title = title, ...))
    }
    
    ## Check if this is a multifit result
    if (is_multifit_result(x)) {
        ## Generate appropriate title if not provided
        if (is.null(title)) {
            model_type <- attr(x, "model_type")
            title <- switch(model_type,
                            "glm" = "Multivariate Logistic Regression",
                            "coxph" = "Multivariate Survival Analysis",
                            "lm" = "Multivariate Linear Regression",
                            "glmer" = "Multivariate Mixed-Effects Logistic Regression",
                            "lmer" = "Multivariate Mixed-Effects Linear Regression",
                            "coxme" = "Multivariate Mixed-Effects Survival Analysis",
                            "Multivariate Analysis"
                            )
        }
        
        return(multiforest(x = x, title = title, ...))
    }
    
    ## Otherwise, treat as a model object
    model <- x
    model_class <- class(model)[1]
    
    ## Check for mixed-effects model classes
    is_glmer <- inherits(model, c("glmerMod", "glmerMod"))
    is_lmer <- inherits(model, "lmerMod") && !is_glmer
    is_coxme <- inherits(model, "coxme")
    
    ## Generate appropriate title if not provided
    if (is.null(title)) {
        if (is_glmer) {
            ## Check family for glmer
            family_name <- model@resp$family$family
            if (family_name == "binomial") {
                title <- "Mixed-Effects Logistic Regression"
            } else if (family_name == "poisson") {
                title <- "Mixed-Effects Poisson Regression"
            } else {
                title <- "Mixed-Effects Generalized Linear Model"
            }
        } else if (is_lmer) {
            title <- "Mixed-Effects Linear Regression"
        } else if (is_coxme) {
            title <- "Mixed-Effects Cox Proportional Hazards"
        } else {
            title <- switch(model_class,
                            "coxph" = "Cox Proportional Hazards Model",
                            "clogit" = "Conditional Logistic Regression",
                            "glm" = {
                                if (model$family$family == "binomial") {
                                    "Logistic Regression Model"
                                } else if (model$family$family == "poisson") {
                                    "Poisson Regression Model" 
                                } else {
                                    "Generalized Linear Model"
                                }
                            },
                            "lm" = "Linear Regression Model",
                            "Model Results"  # Generic fallback
                            )
        }
    }
    
    ## Route to appropriate function
    ## Mixed-effects models first
    if (is_glmer) {
        glmforest(x = model, data = data, title = title, ...)
        
    } else if (is_lmer) {
        lmforest(x = model, data = data, title = title, ...)
        
    } else if (is_coxme) {
        coxforest(x = model, data = data, title = title, ...)
        
    } else if (model_class %in% c("coxph", "clogit")) {
        coxforest(x = model, data = data, title = title, ...)
        
    } else if (model_class == "glm") {
        glmforest(x = model, data = data, title = title, ...)
        
    } else if (model_class == "lm") {
        ## Ensure not actually a GLM
        if (inherits(model, "glm")) {
            glmforest(x = model, data = data, title = title, ...)
        } else {
            lmforest(x = model, data = data, title = title, ...)
        }
        
    } else {
        ## Check if it might be a supported class with different name
        if (inherits(model, "coxph")) {
            coxforest(x = model, data = data, title = title, ...)
        } else if (inherits(model, "glm")) {
            glmforest(x = model, data = data, title = title, ...)
        } else if (inherits(model, "lm")) {
            lmforest(x = model, data = data, title = title, ...)
        } else {
            stop(paste("Input class", model_class, 
                       "is not supported. Supported classes are:",
                       "lm, glm, coxph, clogit, glmerMod, lmerMod, coxme,",
                       "fit_result, fullfit_result, uniscreen_result, multifit_result"))
        }
    }
}


### * Helper functions

#' Check if object is a multifit result
#' 
#' Internal helper to detect multifit output objects.
#' 
#' @param x Object to check.
#' @return Logical indicating if x is a multifit result.
#' @keywords internal
is_multifit_result <- function(x) {
    ## Check for data.table with multifit-specific attributes
    if (!data.table::is.data.table(x)) {
        return(FALSE)
    }
    
    ## Check for required multifit attributes
    required_attrs <- c("predictor", "outcomes", "model_type", "raw_data")
    has_attrs <- vapply(required_attrs, function(a) !is.null(attr(x, a)), logical(1))
    
    all(has_attrs)
}


#' Check if object is a uniscreen result
#' 
#' Internal helper to detect uniscreen output objects.
#' 
#' @param x Object to check.
#' @return Logical indicating if x is a uniscreen result.
#' @keywords internal
is_uniscreen_result <- function(x) {
    ## Check for uniscreen_result class
    if ("uniscreen_result" %in% class(x)) {
        return(TRUE)
    }
    
    ## Also check for data.table with uniscreen-specific attributes
    if (!data.table::is.data.table(x)) {
        return(FALSE)
    }
    
    ## Check for required uniscreen attributes
    required_attrs <- c("outcome", "predictors", "model_type", "model_scope")
    has_attrs <- vapply(required_attrs, function(a) !is.null(attr(x, a)), logical(1))
    
    ## Also verify model_scope is "Univariable"
    if (all(has_attrs)) {
        return(attr(x, "model_scope") == "Univariable")
    }
    
    FALSE
}

### * Main function

#' Save a Forest Plot to File
#'
#' Writes a forest plot to file, sized by default from the recommendations the
#' plotting function computed from the plot's own content, and rendered through
#' a graphics device chosen to reproduce the fonts seen on screen.
#'
#' @param plot A forest plot produced by \code{lmforest()}, \code{glmforest()},
#'   \code{coxforest()}, \code{uniforest()}, \code{multiforest()}, or
#'   \code{autoforest()}. Any \pkg{ggplot2} object is accepted, but one that
#'   carries no recommended dimensions requires \code{width} and \code{height}
#'   to be supplied.
#' @param file Character string giving the output path. The file extension
#'   determines the format: \code{.pdf}, \code{.png}, \code{.tiff},
#'   \code{.jpeg}, \code{.svg}, \code{.eps}, and the other formats
#'   \code{ggplot2::ggsave()} supports.
#' @param width,height Numeric plot dimensions. Taken from the plot's
#'   recommended dimensions when not supplied, and converted where these differ
#'   from \code{units}.
#' @param units Character string giving the units for \code{width} and
#'   \code{height}: \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Defaults
#'   to the units the recommended dimensions were expressed in, and to
#'   \code{"in"} where none were recorded.
#' @param dpi Numeric resolution in dots per inch for raster formats. Ignored
#'   for vector formats. Default is \code{300}.
#' @param device Graphics device. Selected automatically when \code{NULL}: see
#'   Details. Supply \code{"cairo"} to use the Cairo device for the output
#'   format, which embeds fonts, or any device function or name recognized by
#'   \code{ggplot2::ggsave()}.
#' @param embed_fonts Logical. Embed the fonts in a PDF after it is written,
#'   using \code{grDevices::embedFonts()}. Requires Ghostscript, and applies
#'   only to PDF output. Default is \code{FALSE}.
#' @param quiet Logical. Suppress the message reporting the dimensions used.
#'   Default is \code{FALSE}.
#' @param ... Additional arguments passed to \code{ggplot2::ggsave()}, such as
#'   \code{bg} or \code{limitsize}.
#'
#' @return Invisibly returns \code{file}.
#'
#' @details
#' Forest plots are sized by their content rather than by the page: a plot with
#' more variables, longer labels, or more factor levels requires a larger
#' canvas, and the plotting functions compute one and attach it to the returned
#' object as a \code{"rec_dims"} attribute. \code{forestsave()} reads that
#' attribute, so the following are equivalent:
#'
#' \preformatted{
#' dims <- attr(p, "rec_dims")
#' ggplot2::ggsave("forest.pdf", p, width = dims$width,
#'                 height = dims$height, units = dims$units)
#'
#' forestsave(p, "forest.pdf")
#' }
#'
#' \strong{Device selection.} Fonts are measured differently by different
#' graphics devices, so a figure written through one device may not reproduce
#' the spacing seen on screen or in the package documentation. Unless
#' \code{device} is supplied, \code{forestsave()} selects:
#'
#' \describe{
#'   \item{PDF}{R's internal PDF device, \code{grDevices::pdf()}, reached by
#'     deferring to \code{ggplot2::ggsave()}. Cairo is not used by default:
#'     although it embeds fonts, its face selection can substitute an
#'     incorrect italic, which matters here because forest plot footers and
#'     statistical notation are set in italic. Pass \code{device = "cairo"}
#'     to use it where font embedding is required.}
#'   \item{PNG, TIFF, JPEG}{The corresponding \pkg{ragg} device where that
#'     package is installed, whose text metrics match those used to render the
#'     package vignettes. Falls back to the standard raster devices
#'     otherwise.}
#'   \item{Other formats}{Left to \code{ggplot2::ggsave()}.}
#' }
#'
#' \strong{Font embedding.} R's PDF device draws on the standard fonts that every
#' PDF reader is required to provide, so its output is portable without
#' embedding. Embedding matters where the plot uses a font outside that set, and
#' where a publisher requires it. Setting \code{embed_fonts = TRUE} embeds the
#' fonts after the file is written, through \code{grDevices::embedFonts()}, which
#' requires a Ghostscript installation. This is offered as an alternative to
#' \code{device = "cairo"}, which embeds fonts as it draws but selects its own
#' italic face.
#'
#' \code{device = "cairo"} is a shorthand of this package's own, resolving to
#' \code{grDevices::cairo_pdf()}, \code{grDevices::cairo_ps()}, or
#' \code{grDevices::svg()} according to the output format:
#' \preformatted{
#' forestsave(p, "forest.pdf", device = "cairo")
#' }
#' It applies to vector formats only, since raster output already uses the
#' \pkg{ragg} devices, which supersede Cairo's raster backends. Cairo is
#' absent from some installations even where \code{capabilities("cairo")}
#' reports otherwise, so the device is opened once before use and its
#' unavailability reported. Any other value of \code{device} is passed to
#' \code{ggplot2::ggsave()} unchanged.
#'
#' The dimensions used are reported through a \code{message()} unless
#' \code{quiet = TRUE}, so that a figure written at an unexpected size is
#' apparent at the point it is written.
#'
#' @seealso \code{\link{autoforest}}, \code{\link{coxforest}},
#'   \code{\link{glmforest}}, \code{\link{lmforest}}, \code{\link{uniforest}},
#'   \code{\link{multiforest}} for producing forest plots;
#'   \code{\link{tablesave}} for exporting formatted tables
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' model <- stats::glm(readmission_30d ~ age + sex + stage,
#'                     data = clintrial, family = stats::binomial)
#'
#' p <- glmforest(model, data = clintrial, labels = clintrial_labels)
#'
#' # Example 1: Save at the recommended dimensions
#' forestsave(p, file.path(tempdir(), "forest.pdf"))
#'
#' \donttest{
#' 
#' # Example 2: Raster output at publication resolution
#' forestsave(p, file.path(tempdir(), "forest.png"), dpi = 600)
#'
#' # Example 3: Dimensions given explicitly, in any supported units
#' forestsave(p, file.path(tempdir(), "forest_sized.pdf"),
#'            width = 24, height = 16, units = "cm")
#'
#' # Example 4: Embed fonts after writing, where Ghostscript is available
#' forestsave(p, file.path(tempdir(), "forest_embedded.pdf"),
#'            embed_fonts = TRUE)
#' 
#' }
#'
#' @family visualization functions
#' @export
forestsave <- function(plot, file, width = NULL, height = NULL,
                       units = NULL, dpi = 300, device = NULL,
                       embed_fonts = FALSE, quiet = FALSE, ...) {

    if (!inherits(plot, "ggplot")) {
        stop("'plot' must be a forest plot or other ggplot2 object.",
             call. = FALSE)
    }

    if (!is.character(file) || length(file) != 1 || !nzchar(file)) {
        stop("'file' must be a single non-empty character string.",
             call. = FALSE)
    }

    dims <- attr(plot, "rec_dims")

    ## The units in which the recommended dimensions are expressed
    dims_units <- if (!is.null(dims$units)) dims$units else "in"

    ## Output units default to those of the recommendation
    if (is.null(units)) {
        units <- if (!is.null(dims)) dims_units else "in"
    }

    valid_units <- c("in", "cm", "mm", "px", "pt")

    if (!units %in% valid_units) {
        stop("'units' must be one of: ",
             paste(sQuote(valid_units), collapse = ", "), ".",
             call. = FALSE)
    }

    ## Fill either dimension from the recommendation
    if (is.null(width) && !is.null(dims$width)) {
        width <- convert_units(dims$width, from = dims_units, to = units)
    }

    if (is.null(height) && !is.null(dims$height)) {
        height <- convert_units(dims$height, from = dims_units, to = units)
    }

    ## Case for a plot carrying no recommendation and given no dimensions
    if (is.null(width) || is.null(height)) {
        stop("'plot' carries no recommended dimensions, so 'width' and ",
             "'height' must be supplied.", call. = FALSE)
    }

    if (is.null(device)) {
        device <- select_forest_device(file)
    } else if (identical(device, "cairo")) {
        device <- cairo_device(file)
    }

    if (!quiet) {
        ## Phrased to match the table export functions, with the dimensions
        ## appended because an unexpected size is the failure this reports
        message(sprintf(
            "Forest plot saved to %s (width = %.1f %s, height = %.1f %s)",
            file, width, units, height, units))
    }

    ## ggsave() accepts NULL for device, in which case it infers one from the
    ## file extension
    ggplot2::ggsave(filename = file, plot = plot,
                    width = width, height = height, units = units,
                    dpi = dpi, device = device, ...)

    ## Case when a graphics device cannot be opened
    if (!file.exists(file)) {
        stop("The graphics device wrote no file to '", file, "'.",
             if (!is.null(device)) {
                 paste0(" The device supplied through 'device' may be ",
                        "unavailable on this system.")
             } else {
                 ""
             }, call. = FALSE)
    }

    if (isTRUE(embed_fonts)) {
        embed_plot_fonts(file, quiet = quiet)
    }

    invisible(file)
}


### * Graphics device selection

#' Select a graphics device for forest plot output
#'
#' Chooses a device whose text metrics reproduce those used to render the
#' package documentation, falling back to \code{ggplot2::ggsave()}'s own
#' selection where the preferred device is unavailable.
#'
#' @param file Character string giving the output path.
#' @return A device function, or \code{NULL} to defer to
#'   \code{ggplot2::ggsave()}.
#' @keywords internal
select_forest_device <- function(file) {

    ext <- tolower(tools::file_ext(file))

    has_ragg <- requireNamespace("ragg", quietly = TRUE)

    device <- switch(
        ext,
        ## PDF defers to ggsave(), and so to R's internal device
        ## Cairo embeds fonts but does not always correctly render italics
        pdf  = NULL,
        png  = if (has_ragg) ragg::agg_png else NULL,
        tif  = ,
        tiff = if (has_ragg) ragg::agg_tiff else NULL,
        jpg  = ,
        jpeg = if (has_ragg) ragg::agg_jpeg else NULL,
        NULL
    )

    return(device)
}


#' Resolve the Cairo device for an output format
#'
#' Maps the \code{"cairo"} shorthand accepted by \code{forestsave()} onto the
#' Cairo-backed device for the requested format. Cairo's raster backends are
#' superseded by \pkg{ragg} and are not offered, so the shorthand applies to
#' vector formats only.
#'
#' @param file Character string giving the output path.
#' @return A device function.
#' @keywords internal
cairo_device <- function(file) {

    ext <- tolower(tools::file_ext(file))

    device <- switch(
        ext,
        pdf = grDevices::cairo_pdf,
        ps  = ,
        eps = grDevices::cairo_ps,
        svg = grDevices::svg,
        NULL
    )

    if (is.null(device)) {
        stop("device = \"cairo\" applies to vector formats ('.pdf', '.ps', ",
             "'.eps', '.svg'), not '", ext, "'. Raster output already uses the ",
             "ragg devices where available.", call. = FALSE)
    }

    probe <- tempfile(fileext = paste0(".", ext))
    baseline <- grDevices::dev.cur()

    opened <- tryCatch({
        suppressWarnings(device(probe))
        TRUE
    }, error = function(e) FALSE)

    ## Closed whether or not the probe succeeded, so that a partially opened
    ## device is not left current
    if (!identical(grDevices::dev.cur(), baseline)) {
        grDevices::dev.off()
    }

    usable <- isTRUE(opened) && file.exists(probe) && file.size(probe) > 0
    unlink(probe)

    if (!usable) {
        stop("device = \"cairo\" requires a working Cairo installation. The ",
             "Cairo device could not be opened on this system, which happens ",
             "where the supporting libraries are absent even though ",
             "capabilities(\"cairo\") reports otherwise. Leaving 'device' ",
             "unset uses the default device for the format.", call. = FALSE)
    }

    return(device)
}


### * Font embedding

#' Embed fonts in a PDF after it is written
#'
#' Post-processes a PDF so that its fonts are embedded, through
#' \code{grDevices::embedFonts()}. This is offered as an alternative to drawing
#' with a Cairo device, which embeds fonts but renders italics unreliably.
#' If Cairo is not selected, embedding is performed by Ghostscript (external).
#'
#' @param file Character string giving the path to the written file.
#' @param quiet Logical. Suppress the confirmation message.
#' @return Invisibly returns \code{TRUE} where the fonts were embedded and
#'   \code{FALSE} otherwise.
#' @keywords internal
embed_plot_fonts <- function(file, quiet = FALSE) {

    if (!identical(tolower(tools::file_ext(file)), "pdf")) {
        warning("'embed_fonts' applies to PDF output only; '",
                tools::file_ext(file), "' left unchanged.", call. = FALSE)
        return(invisible(FALSE))
    }

    ## embedFonts() calls Ghostscript (external)
    ## find_gs_cmd() consults R_GSCMD and the PATH, and returns an empty string
    ## when nothing is found
    if (!nzchar(tools::find_gs_cmd())) {
        warning("Font embedding requires Ghostscript, which was not found. ",
                "The file was written without embedded fonts.", call. = FALSE)
        return(invisible(FALSE))
    }

    ## The plot is already on disk, so a failure here leaves a usable file
    ok <- tryCatch({
        grDevices::embedFonts(file)
        TRUE
    }, error = function(e) {
        warning("Font embedding failed: ", conditionMessage(e),
                ". The file was written without embedded fonts.",
                call. = FALSE)
        FALSE
    })

    if (ok && !quiet) {
        message("Fonts embedded in ", file)
    }

    return(invisible(ok))
}

## tablesave() and forestsave() are counterparts and validate, report, and
## return on the same terms.

tablesave_fixture <- function() {
    dt <- data.table::as.data.table(fresh_clintrial())
    desctable(dt, by = "treatment", variables = c("age", "sex"))
}


test_that("tablesave writes delimited formats without further dependencies", {

    tbl <- tablesave_fixture()

    csv <- tempfile(fileext = ".csv")
    expect_message(tablesave(tbl, csv), "Table saved to")
    expect_true(file.exists(csv))
    expect_gt(nrow(data.table::fread(csv)), 0)

    tsv <- tempfile(fileext = ".tsv")
    tablesave(tbl, tsv, quiet = TRUE)
    expect_true(file.exists(tsv))

    ## The tab separator is applied, not merely the extension
    expect_true(any(grepl("\t", readLines(tsv, n = 1), fixed = TRUE)))
})


test_that("tablesave returns its path invisibly and can be silenced", {

    tbl <- tablesave_fixture()
    out <- tempfile(fileext = ".csv")

    expect_no_message(result <- tablesave(tbl, out, quiet = TRUE))
    expect_identical(result, out)
})


test_that("tablesave validates its arguments as forestsave does", {

    tbl <- tablesave_fixture()

    expect_error(tablesave("not a table", tempfile(fileext = ".csv")),
                 "must be a data frame")
    expect_error(tablesave(tbl, character(0)), "non-empty character")
    expect_error(tablesave(tbl, ""), "non-empty character")
    expect_error(tablesave(tbl, tempfile()), "no extension")
    expect_error(tablesave(tbl, tempfile(fileext = ".xyz")),
                 "Unsupported file extension")
})


test_that("the two save functions agree on their contract", {

    tbl <- tablesave_fixture()
    dt <- data.table::as.data.table(fresh_clintrial())
    model <- stats::glm(readmission_30d ~ age + sex,
                        data = dt, family = stats::binomial)
    p <- suppressMessages(suppressWarnings(glmforest(model, data = dt)))

    ## Both are quiet on request, and both return the path they were given
    t_out <- tempfile(fileext = ".csv")
    f_out <- tempfile(fileext = ".pdf")

    expect_no_message(t_result <- tablesave(tbl, t_out, quiet = TRUE))
    expect_no_message(f_result <- forestsave(p, f_out, quiet = TRUE))

    expect_identical(t_result, t_out)
    expect_identical(f_result, f_out)

    ## Both name the file in their message
    expect_message(tablesave(tbl, tempfile(fileext = ".csv")), "saved to")
    expect_message(forestsave(p, tempfile(fileext = ".pdf")), "saved to")
})

## forestsave() writes a forest plot at the dimensions the plotting function
## recommended, so that a figure is not silently sized to the device default.

forest_fixture <- function() {
    dt <- data.table::as.data.table(fresh_clintrial())
    model <- stats::glm(readmission_30d ~ age + sex + stage,
                        data = dt, family = stats::binomial)
    suppressMessages(suppressWarnings(glmforest(model, data = dt)))
}


test_that("forest plots record the units their dimensions are expressed in", {

    dt <- data.table::as.data.table(fresh_clintrial())
    model <- stats::glm(readmission_30d ~ age + sex,
                        data = dt, family = stats::binomial)

    p_in <- suppressMessages(suppressWarnings(
        glmforest(model, data = dt, units = "in")))
    p_cm <- suppressMessages(suppressWarnings(
        glmforest(model, data = dt, units = "cm")))

    expect_equal(attr(p_in, "rec_dims")$units, "in")
    expect_equal(attr(p_cm, "rec_dims")$units, "cm")

    ## The centimetre figures are the inch figures scaled, so a consumer that
    ## assumed inches would mis-size the output by the conversion factor
    expect_equal(attr(p_cm, "rec_dims")$width,
                 attr(p_in, "rec_dims")$width * 2.54,
                 tolerance = 1e-6)
})


test_that("forestsave writes a file at the recommended dimensions", {

    p <- forest_fixture()
    dims <- attr(p, "rec_dims")
    out <- tempfile(fileext = ".pdf")

    expect_message(forestsave(p, out),
                   sprintf("width = %.1f", dims$width))
    expect_true(file.exists(out))
    expect_gt(file.size(out), 0)
})


test_that("forestsave returns its path invisibly and can be silenced", {

    p <- forest_fixture()
    out <- tempfile(fileext = ".pdf")

    expect_no_message(result <- forestsave(p, out, quiet = TRUE))
    expect_identical(result, out)
})


test_that("forestsave converts when the requested units differ", {

    dt <- data.table::as.data.table(fresh_clintrial())
    model <- stats::glm(readmission_30d ~ age + sex,
                        data = dt, family = stats::binomial)

    ## Recommended in inches, requested in centimetres
    p <- suppressMessages(suppressWarnings(
        glmforest(model, data = dt, units = "in")))
    expected <- attr(p, "rec_dims")$width * 2.54

    expect_message(forestsave(p, tempfile(fileext = ".pdf"), units = "cm"),
                   sprintf("width = %.1f cm", expected))
})


test_that("forestsave requires dimensions when none were recommended", {

    plain <- ggplot2::ggplot(data.frame(x = 1, y = 1), ggplot2::aes(x, y)) +
        ggplot2::geom_point()

    ## Writing at the device default is the mis-sizing the function prevents
    expect_error(forestsave(plain, tempfile(fileext = ".pdf")),
                 "no recommended dimensions")

    ## Explicit dimensions are accepted
    out <- tempfile(fileext = ".pdf")
    expect_silent(forestsave(plain, out, width = 6, height = 4, quiet = TRUE))
    expect_true(file.exists(out))
})


test_that("forestsave validates its arguments", {

    p <- forest_fixture()

    expect_error(forestsave("not a plot", tempfile(fileext = ".pdf")),
                 "must be a forest plot")
    expect_error(forestsave(p, character(0)), "non-empty character")
    expect_error(forestsave(p, tempfile(fileext = ".pdf"), units = "furlongs"),
                 "'units' must be one of")
})


test_that("PDF output defers to R's internal device rather than Cairo", {

    ## Cairo embeds fonts but can substitute an incorrect italic face, which
    ## the italic footer would expose. It remains available as an override.
    expect_null(summata:::select_forest_device("x.pdf"))
})


test_that("raster output prefers ragg where available", {

    skip_if_not_installed("ragg")

    expect_identical(summata:::select_forest_device("x.png"), ragg::agg_png)
    expect_identical(summata:::select_forest_device("x.tiff"), ragg::agg_tiff)
    expect_identical(summata:::select_forest_device("x.jpeg"), ragg::agg_jpeg)
})


test_that("formats with no preferred device defer to ggsave", {

    expect_null(summata:::select_forest_device("x.svg"))
    expect_null(summata:::select_forest_device("x"))
})


## capabilities("cairo") reports that the library was compiled in, not that the
## device can be opened. On a headless macOS runner the two disagree: the
## capability is TRUE while cairo_pdf() fails for want of the supporting
## infrastructure. The device is therefore probed rather than asked.
cairo_pdf_works <- function() {
    if (!isTRUE(unname(capabilities("cairo")))) {
        return(FALSE)
    }
    probe <- tempfile(fileext = ".pdf")
    ok <- tryCatch({
        grDevices::cairo_pdf(filename = probe)
        grDevices::dev.off()
        file.exists(probe) && file.size(probe) > 0
    }, error = function(e) FALSE, warning = function(w) FALSE)
    unlink(probe)
    return(isTRUE(ok))
}


test_that("an explicit device overrides the automatic selection", {

    skip_if_not(cairo_pdf_works(), "Cairo PDF device is not usable here")

    p <- forest_fixture()

    ## Both the device function and the shorthand reach the same device
    out_fn <- tempfile(fileext = ".pdf")
    expect_silent(forestsave(p, out_fn, device = grDevices::cairo_pdf,
                             quiet = TRUE))
    expect_true(file.exists(out_fn))

    out_str <- tempfile(fileext = ".pdf")
    expect_silent(forestsave(p, out_str, device = "cairo", quiet = TRUE))
    expect_true(file.exists(out_str))
})


test_that("the cairo shorthand rejects raster formats", {

    ## The format is checked before the device is probed, so this assertion
    ## holds whether or not Cairo is usable here. ragg supersedes Cairo's
    ## raster backends, so the shorthand does not silently fall through to
    ## something else.
    expect_error(summata:::cairo_device("x.png"), "vector formats")
    expect_error(summata:::cairo_device("x.tiff"), "vector formats")
})


test_that("the cairo shorthand resolves per vector format", {

    ## cairo_device() opens the device to confirm it is usable, so this needs
    ## a working Cairo installation rather than merely a compiled-in one.
    skip_if_not(cairo_pdf_works(), "Cairo PDF device is not usable here")

    expect_identical(summata:::cairo_device("x.pdf"), grDevices::cairo_pdf)
    expect_identical(summata:::cairo_device("x.eps"), grDevices::cairo_ps)
    expect_identical(summata:::cairo_device("x.svg"), grDevices::svg)
})


test_that("the cairo shorthand reports an unusable installation", {

    skip_if(cairo_pdf_works(), "Cairo PDF device is usable here")

    ## Where Cairo cannot be opened, the shorthand says so rather than
    ## returning a device that fails when it is used
    expect_error(summata:::cairo_device("x.pdf"), "working Cairo installation")
})


test_that("embed_fonts is rejected for non-PDF output", {

    p <- forest_fixture()
    out <- tempfile(fileext = ".png")

    expect_warning(forestsave(p, out, embed_fonts = TRUE, quiet = TRUE),
                   "PDF output only")
    ## The plot is still written; only the embedding step is skipped
    expect_true(file.exists(out))
})


test_that("embed_fonts degrades to a warning when Ghostscript is absent", {

    p <- forest_fixture()
    out <- tempfile(fileext = ".pdf")

    if (nzchar(tools::find_gs_cmd())) {
        expect_silent(forestsave(p, out, embed_fonts = TRUE, quiet = TRUE))
    } else {
        expect_warning(forestsave(p, out, embed_fonts = TRUE, quiet = TRUE),
                       "Ghostscript")
    }

    ## Either way the figure exists, since embedding runs after the write
    expect_true(file.exists(out))
    expect_gt(file.size(out), 0)
})

### * Table formatting

#' Calculate table layout for forest plots
#' 
#' Computes column widths and positions for the table portion of a forest plot.
#' Determines spacing based on content width, font size, and desired table/forest
#' proportion. Returns positions in log-scale units for plot coordinate system.
#' 
#' @param to_show_exp_clean Data.table with formatted display data for the plot.
#' @param show_n Logical whether to include sample size column.
#' @param show_events Logical whether to include events column.
#' @param indent_groups Logical whether groups are indented (affects level column).
#' @param condense_table Logical whether table is condensed (affects level column).
#' @param effect_label Character string describing effect measure type.
#' @param ref_label Character string label for reference categories.
#' @param font_size Numeric font size for width calculations.
#' @param table_width Numeric proportion of total width for table (0-1).
#' @param rangeb Numeric vector of length 2 with plot x-axis range.
#' @param center_padding Numeric additional padding for effect column.
#' @return List with table_width, forest_width, positions, rangeplot_start,
#'   total_width, and effect_abbrev components.
#' @keywords internal
calculate_forest_layout <- function(to_show_exp_clean, show_n, show_events, 
                                    indent_groups, condense_table, 
                                    effect_label, ref_label, font_size, 
                                    table_width = 0.6, rangeb, center_padding) {
    
    char_to_inch <- 0.08 * font_size
    margin <- 0.3  # inches between columns
    
    ## Build list of active columns with their widths
    columns <- list()
    
    ## Always have variable column
    var_width_chars <- max(nchar(to_show_exp_clean$var_display), nchar("Variable"), na.rm = TRUE)
    columns$var <- var_width_chars * char_to_inch
    
    ## Conditionally add level column
    if (!(indent_groups || condense_table)) {
        level_width_chars <- max(nchar(to_show_exp_clean$level), nchar("Group"), na.rm = TRUE)
        columns$level <- level_width_chars * char_to_inch
    }
    
    ## Conditionally add n column
    if (show_n) {
        n_width_chars <- max(nchar(to_show_exp_clean$n_formatted), nchar("____"), na.rm = TRUE)
        columns$n <- n_width_chars * char_to_inch
    }
    
    ## Conditionally add events column
    if (show_events) {
        events_width_chars <- max(nchar(to_show_exp_clean$events_formatted), nchar("___"), na.rm = TRUE)
        columns$events <- events_width_chars * char_to_inch
    }
    
    ## Always have effect column
    ## Create abbreviation from effect_label - handle common cases and custom labels
    ## For multivariable models, use adjusted abbreviations
    effect_abbrev <- if(effect_label == "Odds Ratio") "aOR" 
                     else if(effect_label == "Risk Ratio") "aRR" 
                     else if(effect_label == "Rate Ratio") "aRR"
                     else if(effect_label == "Hazard Ratio") "aHR"
                     else if(effect_label == "Coefficient") "Adj. Coefficient"
                     else if(effect_label == "Exp(Coefficient)") "Exp(Adj. Coef)"
                     else effect_label  ## Use custom label as-is for the header
    
    effect_header <- paste0(effect_abbrev, " (95% CI); p-value")
    
    ## Calculate max effect string length (without expression parsing)
    effect_lengths <- nchar(data.table::fifelse(
                                            is.na(to_show_exp_clean$estimate),
                                            ref_label,
                                            paste0(to_show_exp_clean$effect_formatted, " (",
                                                   to_show_exp_clean$conf_low_formatted, "-",
                                                   to_show_exp_clean$conf_high_formatted, "); p = ",
                                                   to_show_exp_clean$p_formatted)
                                        )) + center_padding
    
    effect_width_chars <- max(effect_lengths, nchar(effect_header), na.rm = TRUE)
    columns$effect <- effect_width_chars * char_to_inch
    
    ## Calculate total table width
    calc_table_width <- sum(unlist(columns)) + length(columns) * margin * 2
    
    ## Calculate forest width based on table_width proportion
    calc_forest_width <- calc_table_width * ((1 - table_width) / table_width)
    
    ## Convert table width to log scale units
    calc_range_width <- diff(rangeb)
    calc_table_width_log_units <- (calc_table_width / calc_forest_width) * calc_range_width
    
    ## Calculate positions in log scale
    ## Start from the extended left edge
    rangeplot_start <- rangeb[1] - calc_table_width_log_units
    
    ## Convert inch measurements to log scale units
    inch_to_log <- calc_range_width / calc_forest_width
    
    positions <- list()
    current_pos <- rangeplot_start
    
    for (name in names(columns)) {
        current_pos <- current_pos + margin * inch_to_log
        positions[[name]] <- current_pos
        current_pos <- current_pos + columns[[name]] * inch_to_log + margin * inch_to_log
    }
    
    return(list(
        calc_table_width = calc_table_width,
        calc_forest_width = calc_forest_width,
        positions = positions,
        rangeplot_start = rangeplot_start,
        total_width = calc_table_width + calc_forest_width,
        effect_abbrev = effect_abbrev
    ))
}


#' Find non-reference row for binary variable condensing
#' 
#' Identifies the non-reference row in a binary categorical variable by checking
#' for NA estimates (reference rows have NA). This is more robust than assuming
#' row position or matching specific strings like "Yes" or "Positive".
#' 
#' @param var_rows Data.table containing rows for a single variable.
#' @param estimate_col Character string naming the estimate column 
#'   (\emph{e.g.,} "estimate", "coef"). Default is "estimate".
#' @return Integer index of the non-reference row within var_rows, or \code{NULL} if
#'   cannot be determined (\emph{e.g.,} no NA estimates found, or multiple non-NA rows).
#' @keywords internal
find_non_reference_row <- function(var_rows, estimate_col = "estimate") {
    if (!estimate_col %in% names(var_rows)) {
        ## Fallback: try common column names
        possible_cols <- c("estimate", "coef", "coefficient", "hr", "or", "rr")
        estimate_col <- intersect(tolower(names(var_rows)), possible_cols)[1]
        if (is.na(estimate_col)) return(NULL)
        ## Match actual case
        estimate_col <- names(var_rows)[tolower(names(var_rows)) == estimate_col][1]
    }
    
    estimates <- var_rows[[estimate_col]]
    
    ## Reference row has NA estimate; non-reference has actual value
    non_ref_idx <- which(!is.na(estimates))
    
    if (length(non_ref_idx) == 1) {
        return(non_ref_idx)
    } else if (length(non_ref_idx) == 0) {
        ## All NA - shouldn't happen for binary, but fallback to row 2
        return(2L)
    } else {
        ## Multiple non-NA values - not a simple binary reference pattern
        ## Return NULL to signal condensing should not occur
        return(NULL)
    }
}


#' Check if category name is a standard reference/negative value
#' 
#' Determines whether a category name represents a standard reference or negative
#' value that indicates absence. Used to suppress redundant category names when
#' condensing binary variables.
#' 
#' @param category Character string with the category name.
#' @param label Optional character string with the variable label. If provided,
#'   checks if category is "No [label]" or similar patterns.
#' @param norm_category Optional pre-normalized category (lowercase, trimmed).
#'   If provided, skips normalization for performance.
#' @param norm_label Optional pre-normalized label (lowercase, trimmed).
#'   If provided, skips normalization for performance.
#' @return Logical indicating whether category is a reference/negative value.
#' @keywords internal
is_reference_category <- function(category, label = NULL, 
                                   norm_category = NULL, norm_label = NULL) {
    if (is.null(category) || is.na(category) || category == "") {
        return(FALSE)
    }
    
    ## Use pre-normalized value if provided, otherwise normalize
    if (is.null(norm_category)) {
        norm_category <- tolower(trimws(category))
    }
    
    ## Fast check against standard reference values using switch for common cases
    if (norm_category == "no" || norm_category == "none" || 
        norm_category == "0" || norm_category == "false" ||
        norm_category == "absent" || norm_category == "negative" ||
        norm_category == "normal" || norm_category == "-") {
        return(TRUE)
    }
    
    ## Check prefix patterns with startsWith (faster than regex for simple prefixes)
    if (startsWith(norm_category, "no ") || 
        startsWith(norm_category, "non-") || 
        startsWith(norm_category, "non ") ||
        startsWith(norm_category, "without ")) {
        return(TRUE)
    }
    
    ## Check suffix patterns with endsWith (faster than regex)
    if (endsWith(norm_category, " absent") || endsWith(norm_category, " negative")) {
        return(TRUE)
    }
    
    ## If label provided, check if category is "No [label]" pattern
    if (!is.null(label) && !is.na(label) && label != "") {
        if (is.null(norm_label)) {
            norm_label <- tolower(trimws(label))
        }
        
        ## "No [label]" exact pattern
        if (norm_category == paste0("no ", norm_label)) {
            return(TRUE)
        }
        
        ## "No [partial label]" pattern - only check if starts with "no "
        if (startsWith(norm_category, "no ")) {
            category_suffix <- substring(norm_category, 4L)  # Faster than sub()
            if (grepl(category_suffix, norm_label, fixed = TRUE) ||
                grepl(norm_label, category_suffix, fixed = TRUE)) {
                return(TRUE)
            }
        }
    }
    
    return(FALSE)
}


#' Check if category name should be suppressed in condensed label
#' 
#' Determines whether a category name should be suppressed when condensing
#' binary variables. Returns \code{TRUE} for standard affirmative values (\emph{e.g.,} "Yes", 
#' "1", "Positive"), standard reference values (\emph{e.g.,} "No", "Absent", "None"),
#' or when the category name essentially matches the variable label 
#' (case-insensitive comparison).
#' 
#' @param category Character string with the category name.
#' @param label Optional character string with the variable label. If provided,
#'   returns \code{TRUE} when category is a case-insensitive match or substring.
#' @param norm_category Optional pre-normalized category (lowercase, trimmed).
#'   If provided, skips normalization for performance.
#' @param norm_label Optional pre-normalized label (lowercase, trimmed).
#'   If provided, skips normalization for performance.
#' @return Logical indicating whether category should be suppressed.
#' @keywords internal
is_affirmative_category <- function(category, label = NULL,
                                     norm_category = NULL, norm_label = NULL) {
    if (is.null(category) || is.na(category) || category == "") {
        return(FALSE)
    }
    
    ## Use pre-normalized value if provided, otherwise normalize once
    if (is.null(norm_category)) {
        norm_category <- tolower(trimws(category))
    }
    
    ## Fast check against standard affirmative values
    if (norm_category == "yes" || norm_category == "1" || 
        norm_category == "true" || norm_category == "present" ||
        norm_category == "positive" || norm_category == "+") {
        return(TRUE)
    }
    
    ## Pre-normalize label if needed (do this before calling is_reference_category)
    if (!is.null(label) && !is.na(label) && label != "" && is.null(norm_label)) {
        norm_label <- tolower(trimws(label))
    }
    
    ## Check if it's a reference category (pass pre-normalized values)
    if (is_reference_category(category, label, norm_category, norm_label)) {
        return(TRUE)
    }
    
    ## Check if category matches or is contained in the label (case-insensitive)
    if (!is.null(norm_label)) {
        ## Check for exact match
        if (norm_category == norm_label) {
            return(TRUE)
        }
        
        ## Check if category is a substantial substring of label
        nc_len <- nchar(norm_category)
        if (nc_len >= 3L && grepl(norm_category, norm_label, fixed = TRUE)) {
            return(TRUE)
        }
        
        ## Check reverse: label is substring of category
        nl_len <- nchar(norm_label)
        if (nl_len >= 3L && grepl(norm_label, norm_category, fixed = TRUE)) {
            return(TRUE)
        }
    }
    
    return(FALSE)
}


#' Check if a binary variable should be condensed without category suffix
#' 
#' Uses a greedy/liberal approach to determine if a binary variable's condensed
#' display should omit the category name. Returns \code{TRUE} if EITHER level of the
#' binary variable is a standard reference/affirmative value, OR if either level
#' matches/contains the variable label.
#' 
#' This function is designed for binary (2-level) categorical variables where
#' one level is a reference and one is the "event" or "condition" level.
#' 
#' @param ref_category Character string with the reference category name
#'   (the level with NA estimate).
#' @param non_ref_category Character string with the non-reference category name
#'   (the level with the actual estimate).
#' @param label Optional character string with the variable label. Used for
#'   intelligent matching (\emph{e.g.,} "30-Day Readmission" label with 
#'   "30-day readmission" / "No 30-day readmission" levels).
#' @return Logical indicating whether the binary variable should be condensed
#'   without appending the category name.
#' @keywords internal
should_condense_binary <- function(ref_category, non_ref_category, label = NULL) {
    ## Pre-normalize all strings once to avoid redundant normalization
    norm_ref <- if (!is.null(ref_category) && !is.na(ref_category) && ref_category != "") {
                    tolower(trimws(ref_category))
                } else {
                    NULL
                }
    
    norm_non_ref <- if (!is.null(non_ref_category) && !is.na(non_ref_category) && non_ref_category != "") {
                        tolower(trimws(non_ref_category))
                    } else {
                        NULL
                    }
    
    norm_label <- if (!is.null(label) && !is.na(label) && label != "") {
                      tolower(trimws(label))
                  } else {
                      NULL
                  }
    
    ## Check reference category first (more likely to be "No", "0", etc.)
    if (!is.null(norm_ref)) {
        if (is_reference_category(ref_category, label, norm_ref, norm_label)) {
            return(TRUE)
        }
    }
    
    ## Check non-reference category
    if (!is.null(norm_non_ref)) {
        if (is_affirmative_category(non_ref_category, label, norm_non_ref, norm_label)) {
            return(TRUE)
        }
    }
    
    return(FALSE)
}


### * Text formatting

#' Format numeric value with fixed decimal places
#' 
#' Formats a numeric value to a specified number of decimal places, removing
#' leading/trailing whitespace and fixing negative zero display (\emph{e.g.,} "-0.00"
#' becomes "0.00"). When \code{marks} is supplied, applies locale-appropriate
#' decimal mark substitution.
#' 
#' @param x Numeric value to format.
#' @param digits Integer number of decimal places.
#' @param marks Optional list with \code{big.mark} and \code{decimal.mark} as
#'   returned by \code{\link{resolve_number_marks}}.
#' @return Character string with formatted value.
#' @keywords internal
format_number <- function(x, digits, marks = NULL) {
    result <- trimws(format(round(x, digits), nsmall = digits))
    if (!is.null(marks)) {
        return(apply_decimal_mark(result, marks))
    }
    ## Fallback: fix negative zero only (US locale)
    gsub("(?<![0-9])-0(\\.0+)(?![0-9])", "0\\1", result, perl = TRUE)
}


#' Determine CI separator for forest plot text annotations
#'
#' Returns the appropriate separator string between CI lower and upper bounds
#' in forest plot annotations. Considers whether values may be negative and
#' the current locale's decimal mark.
#'
#' @param has_negatives Logical indicating whether any CI bounds are negative.
#' @param marks Optional list with \code{big.mark} and \code{decimal.mark} as
#'   returned by \code{\link{resolve_number_marks}}.
#' @return Character string separator.
#' @keywords internal
forest_ci_separator <- function(has_negatives, marks = NULL) {
    if (has_negatives) return(", ")
    if (!is.null(marks) && marks$decimal.mark == ",") return("\u2013")
    "-"
}


#' Format a p-value for forest plot annotations
#'
#' Returns a formatted p-value string suitable for forest plot display,
#' using locale-aware decimal marks when \code{marks} is provided.
#'
#' @param p Numeric p-value.
#' @param p_digits Integer decimal places.
#' @param marks Optional list with \code{big.mark} and \code{decimal.mark} as
#'   returned by \code{\link{resolve_number_marks}}.
#' @return Character string.
#' @keywords internal
format_p_forest <- function(p, p_digits, marks = NULL) {
    threshold <- 10^(-p_digits)
    if (p < threshold) {
        if (!is.null(marks)) {
            return(paste0("< 0", marks$decimal.mark,
                          strrep("0", p_digits - 1), "1"))
        }
        return(paste0("< ", format(threshold, scientific = FALSE)))
    }
    format_number(p, p_digits, marks)
}


### * CI caching

#' Retrieve confidence intervals with cache support
#'
#' Returns confidence intervals for a fitted model, using a cached result
#' when available from upstream \code{fit()} or \code{m2dt()} calls to avoid
#' redundant computation. This is particularly beneficial for GLM and negative
#' binomial models where \code{confint()} performs profile likelihood profiling,
#' which can be expensive for models with many parameters.
#'
#' @param model Fitted model object. If the model carries a
#'   \code{"cached_confint"} attribute (set by \code{fit()} during table
#'   generation) and the cached confidence level matches \code{conf_level},
#'   the cached result is returned directly.
#' @param conf_level Numeric confidence level. Must match the cached level
#'   for the cache to be used.
#' @return A matrix with one row per model coefficient and two columns
#'   (lower and upper bounds), as returned by \code{stats::confint()}.
#' @keywords internal
get_cached_confint <- function(model, conf_level = 0.95) {

    ## Check for cached result from upstream fit()/m2dt()
    cached <- attr(model, "cached_confint")
    if (!is.null(cached) &&
        isTRUE(attr(model, "cached_confint_level") == conf_level)) {
        return(cached)
    }

    ## Compute fresh; suppress MASS profiling messages
    suppressMessages(suppressWarnings(
        stats::confint(model, level = conf_level)
    ))
}


### * Font selection

#' Detect available sans-serif font for plots
#' 
#' Checks for commonly available sans-serif fonts in order of preference
#' (Helvetica, Arial, Helvetica Neue) and returns the first available one.
#' Falls back to "sans" if none are found or if systemfonts is unavailable.
#' 
#' When ragg is being used as the graphics device (detected via options or
#' knitr settings), font detection works in non-interactive sessions since
#' ragg handles font rendering independently of the R graphics system.
#' 
#' @return Character string with the font family name to use.
#' @keywords internal
detect_plot_font <- function() {
    
    ## Check if ragg is being used - if so, can detect fonts even in
    ## non-interactive sessions since ragg handles fonts via systemfonts
    ragg_in_use <- isTRUE(getOption("summata.use_ragg")) ||
        identical(knitr::opts_chunk$get("dev"), "ragg_png") ||
        identical(knitr::opts_chunk$get("dev"), "agg_png")
    
    ## Only fall back to "sans" in non-interactive sessions when ragg is not in use
    if (!interactive() && !ragg_in_use) {
        return("sans")
    }
    
    if (requireNamespace("systemfonts", quietly = TRUE)) {
        available_fonts <- systemfonts::system_fonts()$family
        ## Check fonts in order of preference
        if ("Helvetica" %in% available_fonts) return("Helvetica")
        if ("Nimbus Sans L" %in% available_fonts) return("Nimbus Sans L")
        if ("Nimbus Sans" %in% available_fonts) return("Nimbus Sans")
        if ("Liberation Sans" %in% available_fonts) return("Liberation Sans")
        if ("Arial" %in% available_fonts) return("Arial")
        if ("Helvetica Neue" %in% available_fonts) return("Helvetica Neue")
    }
    ## Fallback to generic sans-serif
    return("sans")
}


### * Unit conversion

#' Convert between units
#' 
#' Converts measurements between different unit systems commonly used in
#' graphics (inches, centimeters, millimeters, pixels, points).
#' 
#' @param value Numeric value to convert.
#' @param from Character string specifying source unit ("in", "cm", "mm", "px", "pt").
#' @param to Character string specifying target unit ("in", "cm", "mm", "px", "pt").
#' @param dpi Integer dots per inch for pixel conversions (default 96).
#' @return Numeric value in target units.
#' @keywords internal
convert_units <- function(value, from = "in", to = "in", dpi = 96) {
    to_inches <- c(
        "in" = 1,
        "cm" = 1/2.54,    # 1 cm = 1/2.54 inches
        "mm" = 1/25.4,    # 1 mm = 1/25.4 inches
        "px" = 1/dpi,     # 1 px = 1/dpi inches
        "pt" = 1/72       # 1 pt = 1/72 inches
    )
    
    value_in_inches <- value * to_inches[[from]]
    value_in_target <- value_in_inches / to_inches[[to]]
    
    return(value_in_target)
}

### * Main function

#' Create Publication-Ready Survival Summary Tables
#'
#' Generates comprehensive survival summary tables with survival probabilities
#' at specified time points, median survival times, and optional group
#' comparisons with statistical testing. Designed for creating survival
#' summaries commonly used in clinical and epidemiological research
#' publications.
#'
#' @param data Data frame or data.table containing the survival dataset.
#'   Automatically converted to a data.table for efficient processing.
#'
#' @param outcome Character string or character vector specifying one or more
#'   survival outcomes using \code{Surv()} syntax (\emph{e.g.,}
#'   \code{"Surv(os_months, os_status)"}). When multiple outcomes are
#'   provided, results are stacked into a single table with outcome labels
#'   as row headers.
#'
#' @param by Character string specifying the column name of the stratifying
#'   variable for group comparisons (\emph{e.g.,} treatment arm, risk group). When
#'   \code{NULL} (default), produces overall survival summaries only.
#'
#' @param times Numeric vector of time points at which to estimate survival
#'   probabilities. For example, \code{c(12, 24, 36)} for 1-, 2-, and 3-year
#'   survival when time is measured in months. Default is \code{NULL}.
#'
#' @param probs Numeric vector of survival probabilities for which to estimate
#'   corresponding survival times (quantiles). Values must be between 0 and 1.
#'   For example, \code{c(0.5)} returns median survival time,
#'   \code{c(0.25, 0.5, 0.75)} returns quartiles. Default is \code{0.5}
#'   (median only).
#'
#' @param stats Character vector specifying which statistics to display:
#'   \itemize{
#'     \item \code{"survival"} - Survival probability at specified times
#'     \item \code{"ci"} - Confidence interval for survival probability
#'     \item \code{"n_risk"} - Number at risk at each time point
#'     \item \code{"n_event"} - Cumulative number of events by each time
#'       point
#'   }
#'   Default is \code{c("survival", "ci")}.
#'
#' @param type Character string specifying the type of probability to report:
#'   \itemize{
#'     \item \code{"survival"} - Survival probability S(t) [default]
#'     \item \code{"risk"} - Cumulative incidence/risk 1 - S(t)
#'     \item \code{"cumhaz"} - Cumulative hazard -log(S(t))
#'   }
#'
#' @param conf_level Numeric confidence level for confidence intervals. Must
#'   be between 0 and 1. Default is 0.95 (95\% confidence intervals).
#'
#' @param conf_type Character string specifying the confidence interval type
#'   for survival estimates:
#'   \itemize{
#'     \item \code{"log"} - Log transformation (default, recommended)
#'     \item \code{"log-log"} - Log-log transformation
#'     \item \code{"plain"} - Linear/identity (can produce CIs outside
#'       [0, 1])
#'     \item \code{"logit"} - Logit transformation
#'     \item \code{"arcsin"} - Arcsin square root transformation
#'   }
#'
#' @param digits Integer specifying the number of decimal places for survival
#'   probabilities (as percentages). Default is 0 (whole percentages).
#'
#' @param time_digits Integer specifying the number of decimal places for
#'   survival time estimates (median, quantiles). Default is 1.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param percent Logical. If \code{TRUE} (default), displays survival
#'   probabilities as percentages (\emph{e.g.,} \code{"85\%"}). If \code{FALSE},
#'   displays as proportions (\emph{e.g.,} \code{"0.85"}).
#'
#' @param test Logical. If \code{TRUE} (default), performs a survival curve
#'   comparison test and adds a \emph{p}-value column. Requires \code{by}
#'   to be specified.
#'
#' @param test_type Character string specifying the statistical test for
#'   comparing survival curves:
#'   \itemize{
#'     \item \code{"logrank"} - Log-rank test (default)
#'     \item \code{"wilcoxon"} - Wilcoxon (Breslow) test
#'     \item \code{"tarone"} - Tarone-Ware test
#'     \item \code{"petopeto"} - Peto-Peto test
#'   }
#'
#' @param total Logical or character string controlling the total/overall
#'   column:
#'   \itemize{
#'     \item \code{TRUE} or \code{"first"} - Include total column first
#'       [default]
#'     \item \code{"last"} - Include total column last (before
#'       \emph{p}-value)
#'     \item \code{FALSE} - Exclude total column
#'   }
#'
#' @param total_label Character string for the total/overall row label.
#'   Default is \code{"Total"}.
#'
#' @param time_unit Character string specifying the time unit for display
#'   in column headers and labels (\emph{e.g.,} \code{"months"}, \code{"days"},
#'   \code{"years"}). When specified, time column headers become
#'   "\{time\} \{time_unit\}" (\emph{e.g.,} "12 months"). Default is \code{NULL}
#'   (no unit shown).
#'
#' @param time_label Character string template for time column headers when
#'   \code{times} is specified. Use \code{"\{time\}"} as placeholder for
#'   the time value and \code{"\{unit\}"} for the time unit. Default is
#'   \code{"\{time\} \{unit\}"} when \code{time_unit} is specified,
#'   otherwise just \code{"\{time\}"}.
#'
#' @param median_label Character string for the median survival row label.
#'   Default is \code{NULL}, which auto-constructs from \code{conf_level}
#'   (\emph{e.g.,} \code{"Median (95\% CI)"} for \code{conf_level = 0.95}).
#'
#' @param labels Named character vector or list providing custom display
#'   labels. For stratified analyses, names should match levels of the
#'   \code{by} variable. For multiple outcomes, names should match the
#'   \code{Surv()} expressions. Default is \code{NULL}.
#'
#' @param by_label Character string providing a custom label for the
#'   stratifying variable (used in output attributes and headers).
#'   Default is \code{NULL} (uses variable name).
#'
#' @param na_rm Logical. If \code{TRUE} (default), observations with missing
#'   values in time, status, or the stratifying variable are excluded.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param ... Additional arguments passed to
#'   \code{\link[survival]{survfit}}.
#'
#' @return A data.table with S3 class \code{"survtable"} containing formatted
#'   survival statistics. The table structure depends on parameters:
#'
#'   \strong{When \code{times} is specified (survival at time points):}
#'   \describe{
#'     \item{Variable/Group}{Row identifier -- stratifying variable levels}
#'     \item{Time columns}{Survival statistics at each requested time point}
#'     \item{\emph{p}-value}{Test \emph{p}-value (if \code{test = TRUE} and
#'       \code{by} specified)}
#'   }
#'
#'   \strong{When only \code{probs} is specified (survival quantiles):}
#'   \describe{
#'     \item{Variable/Group}{Row identifier -- stratifying variable levels}
#'     \item{Quantile columns}{Time to reach each survival probability}
#'     \item{\emph{p}-value}{Test \emph{p}-value (if \code{test = TRUE} and
#'       \code{by} specified)}
#'   }
#'
#'   All numeric output (probabilities, times, counts, \emph{p}-values)
#'   respects the \code{number_format} setting for locale-appropriate
#'   formatting.
#'
#'   The returned object includes the following attributes:
#'   \describe{
#'     \item{raw_data}{Data.table with unformatted numeric values}
#'     \item{survfit_objects}{List of survfit objects for each stratum}
#'     \item{by_variable}{The stratifying variable name}
#'     \item{times}{The time points requested}
#'     \item{probs}{The probability quantiles requested}
#'     \item{test_result}{Full test result object (if test performed)}
#'   }
#'
#' @details
#' \strong{Survival Probability Estimation:}
#'
#' Survival probabilities are estimated using the Kaplan-Meier method via
#' \code{\link[survival]{survfit}}. At each specified time point, the function
#' reports the estimated probability of surviving beyond that time.
#'
#' \strong{Confidence Intervals:}
#'
#' The default \code{"log"} transformation for confidence intervals is
#' recommended as it ensures intervals remain within [0, 1] and has good
#' statistical properties. The \code{"log-log"} transformation is also
#' commonly used and may perform better in the tails.
#'
#' \strong{Statistical Testing:}
#'
#' The log-rank test (default) tests the null hypothesis that survival curves
#' are identical across groups. Alternative tests weight different parts of
#' the survival curve:
#' \itemize{
#'   \item Log-rank: Equal weights (best for proportional hazards)
#'   \item Wilcoxon: Weights by number at risk (sensitive to early
#'     differences)
#'   \item Tarone-Ware: Weights by square root of number at risk
#'   \item Peto-Peto: Modified Wilcoxon weights
#' }
#'
#' \strong{Formatting:}
#'
#' All numeric output respects the \code{number_format} parameter.
#' Separators within confidence intervals adapt automatically to avoid
#' ambiguity:
#' \itemize{
#'   \item Survival probabilities: \code{"85\% (80\%-89\%)"} (US) or
#'     \code{"85\% (80\%-89\%)"} (EU, en-dash separator)
#'   \item Median survival: \code{"24.5 (21.2-28.9)"} (US) or
#'     \code{"24,5 (21,2-28,9)"} (EU)
#'   \item Counts \eqn{\ge} 1000: \code{"1,234"} (US) or
#'     \code{"1.234"} (EU)
#'   \item \emph{p}-values: \code{"< 0.001"} (US) or
#'     \code{"< 0,001"} (EU)
#' }
#'
#' @seealso
#' \code{\link{desctable}} for baseline characteristics tables,
#' \code{\link{fit}} for regression analysis,
#' \code{\link{tablesave}} for export in any supported format,
#' \code{\link{table2pdf}} for PDF export,
#' \code{\link{table2docx}} for Word export,
#' \code{\link[survival]{survfit}} for underlying survival estimation,
#' \code{\link[survival]{survdiff}} for survival curve comparison tests
#'
#' @examples
#' # Load example data
#' data(clintrial)
#'
#' # Example 1: Survival at specific time points by treatment
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24, 36),
#'     time_unit = "months",
#'     labels = clintrial_labels
#' )
#'
#' \donttest{
#' 
#' # Example 2: Median survival only
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = NULL,
#'     probs = 0.5,
#'     labels = clintrial_labels
#' )
#'
#' # Example 3: Multiple quantiles (quartiles)
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "stage",
#'     times = NULL,
#'     probs = c(0.25, 0.5, 0.75),
#'     labels = clintrial_labels
#' )
#'
#' # Example 4: Both time points and median
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     probs = 0.5,
#'     time_unit = "months",
#'     labels = clintrial_labels
#' )
#'
#' # Example 5: Cumulative incidence (1 - survival)
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     type = "risk",
#'     labels = clintrial_labels
#' )
#'
#' # Example 6: Include number at risk
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     stats = c("survival", "ci", "n_risk"),
#'     labels = clintrial_labels
#' )
#'
#' # Example 7: Overall survival without stratification
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     times = c(12, 24, 36, 48),
#'     labels = clintrial_labels
#' )
#'
#' # Example 8: Without total row
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     total = FALSE,
#'     labels = clintrial_labels
#' )
#'
#' # Example 9: Custom labels
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     labels = c("Drug A" = "Treatment A", "Drug B" = "Treatment B"),
#'     time_unit = "months"
#' )
#'
#' # Example 10: Different confidence interval type
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     conf_type = "log-log",
#'     labels = clintrial_labels
#' )
#'
#' # Example 11: Wilcoxon test instead of log-rank
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     test_type = "wilcoxon",
#'     labels = clintrial_labels
#' )
#'
#' # Example 12: Access raw data for custom analysis
#' result <- survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24)
#' )
#' raw <- attr(result, "raw_data")
#' print(raw)
#'
#' # Example 13: Access survfit objects for plotting
#' fits <- attr(result, "survfit_objects")
#' plot(fits$overall)  # Plot overall survival curve
#'
#' # Example 14: Multiple survival outcomes stacked
#' survtable(
#'     data = clintrial,
#'     outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
#'     by = "treatment",
#'     times = c(12, 24),
#'     probs = 0.5,
#'     time_unit = "months",
#'     total = FALSE,
#'     labels = c(
#'         "Surv(pfs_months, pfs_status)" = "Progression-Free Survival",
#'         "Surv(os_months, os_status)" = "Overall Survival"
#'     )
#' )
#'
#' # Example 15: European number formatting
#' survtable(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     by = "treatment",
#'     times = c(12, 24),
#'     number_format = "eu",
#'     labels = clintrial_labels
#' )
#'
#' }
#' 
#' @family descriptive functions
#' @export
survtable <- function(data,
                      outcome,
                      by = NULL,
                      times = NULL,
                      probs = 0.5,
                      stats = c("survival", "ci"),
                      type = "survival",
                      conf_level = 0.95,
                      conf_type = "log",
                      digits = 0,
                      time_digits = 1,
                      p_digits = 3,
                      percent = TRUE,
                      test = TRUE,
                      test_type = "logrank",
                      total = TRUE,
                      total_label = "Total",
                      time_unit = NULL,
                      time_label = NULL,
                      median_label = NULL,
                      labels = NULL,
                      by_label = NULL,
                      na_rm = TRUE,
                      number_format = NULL,
                      ...) {

    ## Validate inputs
    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }

    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)

    if (is.null(times) && is.null(probs)) {
        stop("At least one of 'times' or 'probs' must be specified")
    }

    if (!is.null(probs)) {
        if (any(probs <= 0 | probs >= 1)) {
            stop("'probs' must contain values between 0 and 1 (exclusive)")
        }
    }

    ## Build time_label if not provided
    if (is.null(time_label)) {
        if (!is.null(time_unit)) {
            time_label <- "{time} {unit}"
        } else {
            time_label <- "{time}"
        }
    }

    type <- match.arg(type, c("survival", "risk", "cumhaz"))
    conf_type <- match.arg(conf_type, c("log", "log-log", "plain", "logit", "arcsin"))
    test_type <- match.arg(test_type, c("logrank", "wilcoxon", "tarone", "petopeto"))
    stats <- match.arg(stats, c("survival", "ci", "n_risk", "n_event"), several.ok = TRUE)

    if (!requireNamespace("survival", quietly = TRUE)) {
        stop("Package 'survival' required for survival analysis")
    }

    ## Auto-construct median_label from conf_level if not provided
    if (is.null(median_label)) {
        ci_pct <- round(conf_level * 100)
        median_label <- paste0("Median (", ci_pct, "% CI)")
    }

    ## Handle multiple outcomes
    if (length(outcome) > 1) {
        ## Process each outcome and stack results
        all_results <- vector("list", length(outcome))
        all_raw <- vector("list", length(outcome))
        all_survfit <- list()
        all_tests <- list()

        for (i in seq_along(outcome)) {
            ## Get label for this outcome
            outcome_label <- if (!is.null(labels) && outcome[i] %in% names(labels)) {
                                 labels[[outcome[i]]]
                             } else {
                                 outcome[i]
                             }

            ## Process single outcome
            single_result <- process_single_outcome(
                data = data,
                outcome = outcome[i],
                outcome_label = outcome_label,
                by = by,
                times = times,
                probs = probs,
                stats = stats,
                type = type,
                conf_level = conf_level,
                conf_type = conf_type,
                digits = digits,
                time_digits = time_digits,
                percent = percent,
                test = test,
                test_type = test_type,
                total = total,
                total_label = total_label,
                time_unit = time_unit,
                time_label = time_label,
                median_label = median_label,
                labels = labels,
                na_rm = na_rm,
                marks = marks,
                ...
            )

            all_results[[i]] <- single_result$formatted
            all_raw[[i]] <- single_result$raw
            all_survfit[[outcome[i]]] <- single_result$survfit_objects
            all_tests[[outcome[i]]] <- single_result$test_result
        }

        ## Stack all results
        result <- data.table::rbindlist(all_results, fill = TRUE)
        raw_result <- data.table::rbindlist(all_raw, fill = TRUE)

        ## Set attributes for multiple outcomes
        data.table::setattr(result, "raw_data", raw_result)
        data.table::setattr(result, "survfit_objects", all_survfit)
        data.table::setattr(result, "outcome", outcome)
        data.table::setattr(result, "by_variable", by)
        data.table::setattr(result, "by_label", by_label)
        data.table::setattr(result, "times", times)
        data.table::setattr(result, "probs", probs)
        data.table::setattr(result, "time_unit", time_unit)
        data.table::setattr(result, "test_result", all_tests)
        data.table::setattr(result, "type", type)

        class(result) <- c("survtable", class(result))
        return(result)
    }

    ## Single outcome - process directly
    single_result <- process_single_outcome(
        data = data,
        outcome = outcome,
        outcome_label = NULL,  
        by = by,
        times = times,
        probs = probs,
        stats = stats,
        type = type,
        conf_level = conf_level,
        conf_type = conf_type,
        digits = digits,
        time_digits = time_digits,
        percent = percent,
        test = test,
        test_type = test_type,
        total = total,
        total_label = total_label,
        time_unit = time_unit,
        time_label = time_label,
        median_label = median_label,
        labels = labels,
        na_rm = na_rm,
        marks = marks,
        ...
    )

    result <- single_result$formatted
    raw_result <- single_result$raw
    
    ## For single outcome, rename Group to Variable for consistency with desctable
    ## (Group column contains stratum labels which become the row identifiers)
    if ("Group" %in% names(result) && !"Variable" %in% names(result)) {
        data.table::setnames(result, "Group", "Variable")
        data.table::setnames(raw_result, "Group", "Variable")
    }

    ## Set attributes
    data.table::setattr(result, "raw_data", raw_result)
    data.table::setattr(result, "survfit_objects", single_result$survfit_objects)
    data.table::setattr(result, "outcome", outcome)
    data.table::setattr(result, "by_variable", by)
    data.table::setattr(result, "by_label", by_label)
    data.table::setattr(result, "times", times)
    data.table::setattr(result, "probs", probs)
    data.table::setattr(result, "time_unit", time_unit)
    data.table::setattr(result, "test_result", single_result$test_result)
    data.table::setattr(result, "type", type)

    class(result) <- c("survtable", class(result))

    return(result)
}


### * Print function

#' Print method for survtable
#'
#' @param x Object of class \code{survtable}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @family descriptive functions
#' @export
#' @keywords internal
print.survtable <- function(x, ...) {
    outcome <- attr(x, "outcome")
    by_var <- attr(x, "by_variable")
    by_label <- attr(x, "by_label")
    times <- attr(x, "times")
    probs <- attr(x, "probs")
    time_unit <- attr(x, "time_unit")
    type <- attr(x, "type")
    test_result <- attr(x, "test_result")

    cat("\nSurvival Summary Table\n")
    
    if (!is.null(outcome)) {
        if (length(outcome) > 1) {
            cat("Outcomes: ", length(outcome), "\n", sep = "")
        } else {
            cat("Outcome: ", outcome, "\n", sep = "")
        }
    }

    if (!is.null(by_var)) {
        display_by <- if (!is.null(by_label)) by_label else by_var
        cat("Stratified by: ", display_by, "\n", sep = "")
    }

    if (!is.null(times)) {
        times_str <- paste(times, collapse = ", ")
        if (!is.null(time_unit)) {
            times_str <- paste(times_str, time_unit)
        }
        cat("Time points: ", times_str, "\n", sep = "")
    }
    if (!is.null(probs)) {
        cat("Quantiles: ", paste(probs * 100, "%", sep = "", collapse = ", "), "\n", sep = "")
    }

    type_label <- switch(type,
                         "survival" = "Survival probability",
                         "risk" = "Cumulative incidence",
                         "cumhaz" = "Cumulative hazard")
    cat("Statistic: ", type_label, "\n", sep = "")

    ## Handle test result - could be single or list for multiple outcomes
    if (!is.null(test_result)) {
        if (is.list(test_result) && !is.null(test_result$test_type)) {
            ## Single test result
            test_name <- switch(test_result$test_type,
                                "logrank" = "Log-rank",
                                "wilcoxon" = "Wilcoxon",
                                "tarone" = "Tarone-Ware",
                                "petopeto" = "Peto-Peto")
            cat("Test: ", test_name, " (p = ", format_pvalue_survtable(test_result$p_value, 3), ")\n", sep = "")
        } else if (is.list(test_result) && length(test_result) > 0) {
            ## Multiple test results - show first one's type
            first_test <- test_result[[1]]
            if (!is.null(first_test)) {
                test_name <- switch(first_test$test_type,
                                    "logrank" = "Log-rank",
                                    "wilcoxon" = "Wilcoxon",
                                    "tarone" = "Tarone-Ware",
                                    "petopeto" = "Peto-Peto")
                cat("Test: ", test_name, "\n", sep = "")
            }
        }
    }

    cat("\n")
    NextMethod("print", x)
    invisible(x)
}


### * Table assembly

#' Process a single survival outcome
#'
#' Parses a Surv() expression, fits the survival models, performs the group
#' comparison test where requested, and assembles the formatted and raw
#' tables for one outcome. Called once per outcome by survtable().
#'
#' @param data Data.table with the source data.
#' @param outcome Character string giving a Surv() expression.
#' @param outcome_label Character label for the outcome, or NULL for a
#'   single-outcome table.
#' @param by Character name of stratifying variable.
#' @param times Numeric vector of time points.
#' @param probs Numeric vector of probabilities.
#' @param stats Character vector of statistics to include.
#' @param type Character string specifying probability type.
#' @param conf_level Numeric confidence level for interval estimates.
#' @param conf_type Character string specifying confidence interval type.
#' @param digits Integer decimal places for percentages.
#' @param time_digits Integer decimal places for time values.
#' @param percent Logical whether to display as percentages.
#' @param test Logical whether to perform a group comparison test.
#' @param test_type Character string specifying test type.
#' @param total Logical or character controlling total column.
#' @param total_label Character label for total column.
#' @param time_unit Character time unit for column headers.
#' @param time_label Character template for time column headers.
#' @param median_label Character label for median row.
#' @param labels Named character vector of display labels for group levels.
#' @param na_rm Logical whether to drop rows with missing time, status, or
#'   stratifying values.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @param ... Additional arguments passed to \code{survival::survfit()}.
#' @return List with the formatted table, the raw table, the survfit objects,
#'   and the test result.
#' @keywords internal
process_single_outcome <- function(data,
                                   outcome,
                                   outcome_label,
                                   by,
                                   times,
                                   probs,
                                   stats,
                                   type,
                                   conf_level,
                                   conf_type,
                                   digits,
                                   time_digits,
                                   percent,
                                   test,
                                   test_type,
                                   total,
                                   total_label,
                                   time_unit,
                                   time_label,
                                   median_label,
                                   labels,
                                   na_rm,
                                   marks = NULL,
                                   ...) {

    ## Parse Surv() expression
    surv_match <- regexec("Surv\\(([^,]+),\\s*([^)]+)\\)", outcome)
    surv_parts <- regmatches(outcome, surv_match)[[1]]

    if (length(surv_parts) < 3) {
        stop("Invalid Surv() syntax: ", outcome, 
             "\nExpected format: Surv(time_var, status_var)")
    }

    time <- trimws(surv_parts[2])
    status <- trimws(surv_parts[3])

    if (!time %in% names(data)) {
        stop("Time variable '", time, "' not found in data")
    }
    if (!status %in% names(data)) {
        stop("Status variable '", status, "' not found in data")
    }
    if (!is.null(by) && !by %in% names(data)) {
        stop("Stratifying variable '", by, "' not found in data")
    }

    ## Handle missing data
    if (na_rm) {
        complete_vars <- c(time, status)
        if (!is.null(by)) complete_vars <- c(complete_vars, by)
        complete_idx <- complete.cases(data[, ..complete_vars])
        data <- data[complete_idx]
    }

    ## Create survival object
    surv_obj <- survival::Surv(data[[time]], data[[status]])

    ## Store survfit objects
    survfit_objects <- list()

    ## Get group information
    groups <- NULL
    group_labels <- NULL
    if (!is.null(by)) {
        grp_vec <- data[[by]]
        if (is.factor(grp_vec)) {
            groups <- levels(grp_vec)
        } else {
            groups <- sort(unique(grp_vec))
            groups <- groups[!is.na(groups)]
        }

        ## Apply labels to groups
        group_labels <- as.character(groups)
        if (!is.null(labels)) {
            for (i in seq_along(groups)) {
                grp_char <- as.character(groups[i])
                if (grp_char %in% names(labels)) {
                    group_labels[i] <- labels[[grp_char]]
                }
            }
        }
    }

    ## Fit survival models
    if (!is.null(by)) {
        ## Stratified analysis
        fit_formula <- stats::as.formula(paste("surv_obj ~", by))
        fit_stratified <- survival::survfit(fit_formula, data = data,
                                            conf.int = conf_level,
                                            conf.type = conf_type, ...)
        survfit_objects$stratified <- fit_stratified

        ## Also fit overall
        if (!isFALSE(total)) {
            fit_overall <- survival::survfit(surv_obj ~ 1, data = data,
                                             conf.int = conf_level,
                                             conf.type = conf_type, ...)
            survfit_objects$overall <- fit_overall
        }
    } else {
        ## Overall only
        fit_overall <- survival::survfit(surv_obj ~ 1, data = data,
                                         conf.int = conf_level,
                                         conf.type = conf_type, ...)
        survfit_objects$overall <- fit_overall
    }

    ## Perform statistical test
    test_result <- NULL
    p_value <- NULL
    if (test && !is.null(by) && length(groups) >= 2) {
        test_result <- perform_survival_test(surv_obj, data[[by]], test_type)
        p_value <- test_result$p_value
    }

    ## Build results table
    result_list <- list()
    raw_list <- list()

    ## Process time points (survival probabilities)
    if (!is.null(times)) {
        time_results <- process_survival_times(
            survfit_objects = survfit_objects,
            times = times,
            groups = groups,
            group_labels = group_labels,
            stats = stats,
            type = type,
            digits = digits,
            percent = percent,
            total = total,
            total_label = total_label,
            time_label = time_label,
            time_unit = time_unit,
            by = by,
            data = data,
            marks = marks
        )
        result_list$times <- time_results$formatted
        raw_list$times <- time_results$raw
    }

    ## Process probability quantiles (median, quartiles, etc.)
    if (!is.null(probs)) {
        prob_results <- process_survival_probs(
            survfit_objects = survfit_objects,
            probs = probs,
            groups = groups,
            group_labels = group_labels,
            time_digits = time_digits,
            total = total,
            total_label = total_label,
            median_label = median_label,
            by = by,
            data = data,
            conf_level = conf_level,
            marks = marks
        )
        result_list$probs <- prob_results$formatted
        raw_list$probs <- prob_results$raw
    }

    ## Combine results - merge times and probs by Group
    if (length(result_list) == 1) {
        result <- result_list[[1]]
        raw_result <- raw_list[[1]]
    } else {
        ## Both times and probs - merge by Group column
        result <- merge(result_list$times, result_list$probs, 
                        by = "Group", all = TRUE, sort = FALSE)
        raw_result <- merge(raw_list$times, raw_list$probs,
                            by = "Group", all = TRUE, sort = FALSE)
        
        ## Restore original row order (Total first if present, then groups)
        if (!isFALSE(total) && total_label %in% result$Group) {
            total_row <- result[Group == total_label]
            other_rows <- result[Group != total_label]
            if (isTRUE(total) || identical(total, "first")) {
                result <- rbind(total_row, other_rows)
            } else {
                result <- rbind(other_rows, total_row)
            }
            
            ## Same for raw
            total_row_raw <- raw_result[Group == total_label]
            other_rows_raw <- raw_result[Group != total_label]
            if (isTRUE(total) || identical(total, "first")) {
                raw_result <- rbind(total_row_raw, other_rows_raw)
            } else {
                raw_result <- rbind(other_rows_raw, total_row_raw)
            }
        }
    }

    ## Add p-value column if test performed
    if (!is.null(p_value)) {
        result <- add_pvalue_column(result, p_value, 3, marks)  # p_digits handled in main
    }

    ## Rename Group to match desctable pattern when stacking multiple outcomes
    ## For multi-outcome tables: Variable = outcome label, Group = stratum label
    if (!is.null(outcome_label)) {
        ## Create Variable column with outcome label only in first row
        n_rows <- nrow(result)
        result[, Variable := c(outcome_label, rep("", n_rows - 1))]
        raw_result[, Variable := c(outcome_label, rep("", n_rows - 1))]
        
        ## Rename existing Group column to preserve stratum labels
        ## (Group column already exists from process_survival_times/probs)
        
        ## Reorder columns: Variable first, then Group, then rest
        other_cols <- setdiff(names(result), c("Variable", "Group"))
        data.table::setcolorder(result, c("Variable", "Group", other_cols))
        
        other_cols_raw <- setdiff(names(raw_result), c("Variable", "Group"))
        data.table::setcolorder(raw_result, c("Variable", "Group", other_cols_raw))
    }

    list(
        formatted = result,
        raw = raw_result,
        survfit_objects = survfit_objects,
        test_result = test_result
    )
}

#' Test Suite for survtable()
#' 
#' Comprehensive tests covering survival summary tables, time points,
#' quantiles, statistical tests, formatting options, and edge cases.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-survtable.R")

library(data.table)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Helper function to check survtable structure
expect_survtable_result <- function(result) {
    expect_s3_class(result, "survtable")
    expect_s3_class(result, "data.table")
    
    ## Required columns - either Variable (single outcome) or Variable+Group (multi)
    expect_true("Variable" %in% names(result) || "Group" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "raw_data")))
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "type")))
}


## Helper to check for specific time column
expect_time_column <- function(result, time, unit = NULL) {
    if (!is.null(unit)) {
        col_name <- paste(time, unit)
    } else {
        col_name <- as.character(time)
    }
    expect_true(col_name %in% names(result) || 
                any(grepl(as.character(time), names(result))))
}


## ============================================================================
## SECTION 1: Basic Functionality - No Grouping
## ============================================================================

test_that("survtable works without grouping variable", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12, 24)
    )
    
    expect_survtable_result(result)
    
    ## Should NOT have p-value column (no groups to compare)
    expect_false("p-value" %in% names(result))
})


test_that("survtable ungrouped returns single row", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        total = FALSE
    )
    
    ## Should have just one row for overall
    expect_equal(nrow(result), 1)
})


test_that("survtable outcome attribute is set", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12)
    )
    
    expect_equal(attr(result, "outcome"), "Surv(os_months, os_status)")
})


## ============================================================================
## SECTION 2: Basic Functionality - With Grouping
## ============================================================================

test_that("survtable works with grouping variable", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24)
    )
    
    expect_survtable_result(result)
    
    ## Should have p-value column by default
    expect_pvalue_column(result)
})


test_that("survtable by_variable attribute is set", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12)
    )
    
    expect_equal(attr(result, "by_variable"), "treatment")
})


test_that("survtable includes all group levels", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        total = FALSE
    )
    
    ## Should have rows for each treatment level
    treatment_levels <- levels(clintrial$treatment)
    group_col <- if ("Group" %in% names(result)) result$Group else result$Variable
    
    for (lvl in treatment_levels) {
        expect_true(lvl %in% group_col)
    }
})


## ============================================================================
## SECTION 3: Time Points
## ============================================================================

test_that("survtable handles single time point", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = 12
    )
    
    expect_survtable_result(result)
    expect_time_column(result, 12)
})


test_that("survtable handles multiple time points", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(6, 12, 24, 36)
    )
    
    expect_survtable_result(result)
    
    ## Check times attribute
    expect_equal(attr(result, "times"), c(6, 12, 24, 36))
})


test_that("survtable time_unit parameter works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12, 24),
        time_unit = "months"
    )
    
    ## Column names should include unit
    expect_true("12 months" %in% names(result))
    expect_true("24 months" %in% names(result))
    
    ## Attribute should be set
    expect_equal(attr(result, "time_unit"), "months")
})


test_that("survtable times attribute is stored", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12, 24, 36)
    )
    
    expect_equal(attr(result, "times"), c(12, 24, 36))
})


## ============================================================================
## SECTION 4: Probability Quantiles (Median, etc.)
## ============================================================================

test_that("survtable handles median (probs = 0.5)", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        probs = 0.5
    )
    
    expect_survtable_result(result)
    
    ## Should have median column
    expect_true(any(grepl("Median", names(result), ignore.case = TRUE)))
})


test_that("survtable handles multiple quantiles", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        probs = c(0.25, 0.5, 0.75)
    )
    
    expect_survtable_result(result)
    
    ## Check probs attribute
    expect_equal(attr(result, "probs"), c(0.25, 0.5, 0.75))
})


test_that("survtable probs attribute is stored", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        probs = c(0.25, 0.5)
    )
    
    expect_equal(attr(result, "probs"), c(0.25, 0.5))
})


test_that("survtable combining times and probs works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24),
        probs = 0.5
    )
    
    expect_survtable_result(result)
    
    ## Should have both time columns and median column
    expect_true(any(grepl("12", names(result))))
    expect_true(any(grepl("Median", names(result), ignore.case = TRUE)))
})


## ============================================================================
## SECTION 5: Type Parameter (survival, risk, cumhaz)
## ============================================================================

test_that("survtable type='survival' works (default)", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        type = "survival"
    )
    
    expect_survtable_result(result)
    expect_equal(attr(result, "type"), "survival")
})


test_that("survtable type='risk' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        type = "risk"
    )
    
    expect_survtable_result(result)
    expect_equal(attr(result, "type"), "risk")
})


test_that("survtable type='cumhaz' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        type = "cumhaz"
    )
    
    expect_survtable_result(result)
    expect_equal(attr(result, "type"), "cumhaz")
})


test_that("survtable risk values are 1 - survival", {
    
    result_surv <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        type = "survival",
        percent = FALSE
    )
    
    result_risk <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        type = "risk",
        percent = FALSE
    )
    
    ## Raw values should be complements
    raw_surv <- attr(result_surv, "raw_data")
    raw_risk <- attr(result_risk, "raw_data")
    
    expect_true(!is.null(raw_surv))
    expect_true(!is.null(raw_risk))
})


## ============================================================================
## SECTION 6: Statistical Tests
## ============================================================================

test_that("survtable test=TRUE adds p-value column", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE
    )
    
    expect_pvalue_column(result)
})


test_that("survtable test=FALSE excludes p-value column", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = FALSE
    )
    
    expect_false("p-value" %in% names(result))
})


test_that("survtable test_type='logrank' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE,
        test_type = "logrank"
    )
    
    expect_pvalue_column(result)
    
    test_result <- attr(result, "test_result")
    expect_equal(test_result$test_type, "logrank")
})


test_that("survtable test_type='wilcoxon' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE,
        test_type = "wilcoxon"
    )
    
    expect_pvalue_column(result)
    
    test_result <- attr(result, "test_result")
    expect_equal(test_result$test_type, "wilcoxon")
})


test_that("survtable test_type='tarone' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE,
        test_type = "tarone"
    )
    
    expect_pvalue_column(result)
    
    test_result <- attr(result, "test_result")
    expect_equal(test_result$test_type, "tarone")
})


test_that("survtable test_type='petopeto' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE,
        test_type = "petopeto"
    )
    
    expect_pvalue_column(result)
    
    test_result <- attr(result, "test_result")
    expect_equal(test_result$test_type, "petopeto")
})


test_that("survtable test_result attribute contains valid data", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE
    )
    
    test_result <- attr(result, "test_result")
    
    expect_true(!is.null(test_result))
    expect_true("p_value" %in% names(test_result))
    expect_true("test_type" %in% names(test_result))
    expect_true(is.numeric(test_result$p_value))
})


## ============================================================================
## SECTION 7: Total Row
## ============================================================================

test_that("survtable total=TRUE includes total row first", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        total = TRUE
    )
    
    ## First row should be Total
    first_group <- if ("Group" %in% names(result)) result$Group[1] else result$Variable[1]
    expect_equal(first_group, "Total")
})


test_that("survtable total='first' includes total row first", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        total = "first"
    )
    
    first_group <- if ("Group" %in% names(result)) result$Group[1] else result$Variable[1]
    expect_equal(first_group, "Total")
})


test_that("survtable total='last' includes total row last", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        total = "last"
    )
    
    last_group <- if ("Group" %in% names(result)) result$Group[nrow(result)] else result$Variable[nrow(result)]
    expect_equal(last_group, "Total")
})


test_that("survtable total=FALSE excludes total row", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        total = FALSE
    )
    
    group_col <- if ("Group" %in% names(result)) result$Group else result$Variable
    expect_false("Total" %in% group_col)
})


test_that("survtable total_label parameter works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        total = TRUE,
        total_label = "Overall"
    )
    
    group_col <- if ("Group" %in% names(result)) result$Group else result$Variable
    expect_true("Overall" %in% group_col)
})


## ============================================================================
## SECTION 8: Formatting Options
## ============================================================================

test_that("survtable percent=TRUE displays percentages", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        percent = TRUE
    )
    
    ## Values should contain %
    time_col <- names(result)[grepl("12", names(result))][1]
    expect_true(any(grepl("%", result[[time_col]])))
})


test_that("survtable percent=FALSE displays proportions", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        percent = FALSE
    )
    
    ## Values should NOT contain % (or have decimal values)
    time_col <- names(result)[grepl("12", names(result))][1]
    ## Check that values don't have %
    expect_false(any(grepl("%", result[[time_col]])))
})


test_that("survtable digits parameter affects survival precision", {
    
    result_0 <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        digits = 0
    )
    
    result_2 <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        digits = 2
    )
    
    ## Both should work
    expect_survtable_result(result_0)
    expect_survtable_result(result_2)
})


test_that("survtable time_digits parameter affects quantile precision", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        probs = 0.5,
        time_digits = 2
    )
    
    expect_survtable_result(result)
})


test_that("survtable p_digits parameter affects p-value precision", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = TRUE,
        p_digits = 4
    )
    
    expect_survtable_result(result)
    expect_pvalue_column(result)
})


## ============================================================================
## SECTION 9: Stats Parameter
## ============================================================================

test_that("survtable stats='survival' shows survival only", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        stats = "survival"
    )
    
    expect_survtable_result(result)
})


test_that("survtable stats includes 'ci' shows confidence intervals", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        stats = c("survival", "ci")
    )
    
    ## Values should contain parentheses for CI
    time_col <- names(result)[grepl("12", names(result))][1]
    expect_true(any(grepl("\\(", result[[time_col]])))
})


test_that("survtable stats includes 'n_risk' adds n at risk", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        stats = c("survival", "ci", "n_risk")
    )
    
    expect_survtable_result(result)
    
    ## Should contain "n =" in the output
    time_col <- names(result)[grepl("12", names(result))][1]
    expect_true(any(grepl("n =", result[[time_col]])))
})


test_that("survtable stats includes 'n_event' adds event count", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        stats = c("survival", "ci", "n_event")
    )
    
    expect_survtable_result(result)
    
    ## Should contain "e =" in the output
    time_col <- names(result)[grepl("12", names(result))][1]
    expect_true(any(grepl("e =", result[[time_col]])))
})


## ============================================================================
## SECTION 10: Confidence Interval Options
## ============================================================================

test_that("survtable conf_level=0.95 works (default)", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        conf_level = 0.95
    )
    
    expect_survtable_result(result)
})


test_that("survtable conf_level=0.90 works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        conf_level = 0.90
    )
    
    expect_survtable_result(result)
})


test_that("survtable conf_type='log' works (default)", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        conf_type = "log"
    )
    
    expect_survtable_result(result)
})


test_that("survtable conf_type='log-log' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        conf_type = "log-log"
    )
    
    expect_survtable_result(result)
})


test_that("survtable conf_type='plain' works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        conf_type = "plain"
    )
    
    expect_survtable_result(result)
})


## ============================================================================
## SECTION 11: Multiple Outcomes
## ============================================================================

test_that("survtable handles multiple outcomes", {
    
    result <- survtable(
        data = clintrial,
        outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
        by = "treatment",
        times = c(12),
        total = FALSE
    )
    
    expect_survtable_result(result)
    
    ## Should have Variable column for outcome names
    expect_true("Variable" %in% names(result))
    
    ## Outcome attribute should be vector
    expect_equal(length(attr(result, "outcome")), 2)
})


test_that("survtable multiple outcomes with labels works", {
    
    result <- survtable(
        data = clintrial,
        outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
        by = "treatment",
        times = c(12),
        total = FALSE,
        labels = c(
            "Surv(pfs_months, pfs_status)" = "PFS",
            "Surv(os_months, os_status)" = "OS"
        )
    )
    
    expect_survtable_result(result)
    
    ## Variable column should contain labels
    expect_true("PFS" %in% result$Variable)
    expect_true("OS" %in% result$Variable)
})


test_that("survtable multiple outcomes stacks correctly", {
    
    result <- survtable(
        data = clintrial,
        outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
        by = "treatment",
        times = c(12),
        total = FALSE
    )
    
    ## Number of rows should be 2 * number of treatment levels
    n_treatments <- length(levels(clintrial$treatment))
    expected_rows <- 2 * n_treatments
    expect_equal(nrow(result), expected_rows)
})


test_that("survtable multiple outcomes each have test results", {
    
    result <- survtable(
        data = clintrial,
        outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
        by = "treatment",
        times = c(12),
        test = TRUE,
        total = FALSE
    )
    
    ## test_result should be a list with both outcomes
    test_result <- attr(result, "test_result")
    expect_true(is.list(test_result))
    expect_equal(length(test_result), 2)
})


## ============================================================================
## SECTION 12: Labels Parameter
## ============================================================================

test_that("survtable labels parameter works for by variable", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "stage",
        times = c(12),
        labels = clintrial_labels
    )
    
    expect_survtable_result(result)
})


## ============================================================================
## SECTION 13: Missing Data
## ============================================================================

test_that("survtable handles missing values with na_rm=TRUE", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        na_rm = TRUE
    )
    
    expect_survtable_result(result)
})


test_that("survtable handles missing values with na_rm=FALSE", {
    
    ## This may produce warnings but should not error
    result <- tryCatch(
        survtable(
            data = clintrial,
            outcome = "Surv(os_months, os_status)",
            by = "treatment",
            times = c(12),
            na_rm = FALSE
        ),
        warning = function(w) {
            ## Capture but continue
            suppressWarnings(
                survtable(
                    data = clintrial,
                    outcome = "Surv(os_months, os_status)",
                    by = "treatment",
                    times = c(12),
                    na_rm = FALSE
                )
            )
        }
    )
    
    expect_survtable_result(result)
})


## ============================================================================
## SECTION 14: Edge Cases
## ============================================================================

test_that("survtable handles times=NULL with probs only", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = NULL,
        probs = c(0.25, 0.5, 0.75)
    )
    
    expect_survtable_result(result)
    
    ## Should not have time columns, only quantile columns
    expect_true(any(grepl("Median", names(result), ignore.case = TRUE)))
})


test_that("survtable handles probs=NULL with times only", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12, 24),
        probs = NULL
    )
    
    expect_survtable_result(result)
    
    ## Should have time columns, no median column
    expect_true(any(grepl("12", names(result))))
    expect_false(any(grepl("Median", names(result), ignore.case = TRUE)))
})


test_that("survtable handles single group", {
    
    ## Create single-group data
    single_group <- clintrial[clintrial$treatment == "Control", ]
    
    result <- survtable(
        data = single_group,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12),
        test = FALSE  # Can't test with single group
    )
    
    expect_survtable_result(result)
})


test_that("survtable handles two groups", {
    
    ## Use sex as binary grouping variable
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "sex",
        times = c(12),
        test = TRUE
    )
    
    expect_survtable_result(result)
    expect_pvalue_column(result)
})


test_that("survtable handles many groups", {
    
    ## Use stage with 4 levels
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "stage",
        times = c(12),
        test = TRUE
    )
    
    expect_survtable_result(result)
    expect_pvalue_column(result)
})


test_that("survtable handles long follow-up times", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(60, 120),  # Long follow-up
        probs = 0.5
    )
    
    expect_survtable_result(result)
})


test_that("survtable handles short follow-up times", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(1, 3, 6),  # Short follow-up
        probs = 0.5
    )
    
    expect_survtable_result(result)
})


## ============================================================================
## SECTION 15: Raw Data Attribute
## ============================================================================

test_that("survtable raw_data attribute contains numeric values", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24),
        probs = 0.5
    )
    
    raw <- attr(result, "raw_data")
    
    expect_true(!is.null(raw))
    expect_s3_class(raw, "data.table")
    
    ## Should have estimate columns
    est_cols <- grep("_estimate$", names(raw), value = TRUE)
    expect_true(length(est_cols) > 0)
})


test_that("survtable survfit_objects attribute is stored", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12)
    )
    
    survfit_objs <- attr(result, "survfit_objects")
    
    expect_true(!is.null(survfit_objs))
    expect_true(is.list(survfit_objs))
})


## ============================================================================
## SECTION 16: Data Input Types
## ============================================================================

test_that("survtable works with data.frame input", {
    
    df <- as.data.frame(clintrial)
    
    result <- survtable(
        data = df,
        outcome = "Surv(os_months, os_status)",
        times = c(12)
    )
    
    expect_survtable_result(result)
})


test_that("survtable works with data.table input", {
    
    result <- survtable(
        data = clintrial,  # Already data.table
        outcome = "Surv(os_months, os_status)",
        times = c(12)
    )
    
    expect_survtable_result(result)
})


## ============================================================================
## SECTION 17: Consistency Tests
## ============================================================================

test_that("survtable produces consistent results across runs", {
    
    result1 <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24)
    )
    
    result2 <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24)
    )
    
    expect_equal(result1, result2)
})


test_that("survtable survival probabilities are in valid range", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        times = c(12),
        probs = NULL,  # No quantiles, only time points
        type = "survival",
        percent = FALSE
    )
    
    raw <- attr(result, "raw_data")
    
    ## Only check columns that are survival probability estimates (time point columns)
    ## Exclude median/quantile columns which contain time values, not probabilities
    est_cols <- grep("^12.*_estimate$", names(raw), value = TRUE)
    
    for (col in est_cols) {
        vals <- raw[[col]]
        vals <- vals[!is.na(vals)]
        if (length(vals) > 0) {
            expect_true(all(vals >= 0 & vals <= 1),
                        info = paste("Column", col, "has values outside [0,1]:", 
                                     paste(vals[vals < 0 | vals > 1], collapse = ", ")))
        }
    }
})


## ============================================================================
## SECTION 18: Print Method
## ============================================================================

test_that("survtable print method works", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24),
        probs = 0.5
    )
    
    ## Print should not error
    expect_output(print(result))
})


test_that("survtable print method shows correct header info", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24),
        time_unit = "months"
    )
    
    ## Capture print output
    output <- capture.output(print(result))
    
    ## Should mention survival
    expect_true(any(grepl("Survival", output, ignore.case = TRUE)))
})


## ============================================================================
## SECTION 19: Integration with Export Functions
## ============================================================================

test_that("survtable output is suitable for export", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24),
        probs = 0.5
    )
    
    ## Should be a data.table
    expect_s3_class(result, "data.table")
    
    ## All columns should be character (for display)
    for (col in names(result)) {
        expect_true(is.character(result[[col]]))
    }
})


## ============================================================================
## SECTION 20: Error Handling
## ============================================================================

test_that("survtable errors on invalid Surv() syntax", {
    
    expect_error(
        survtable(
            data = clintrial,
            outcome = "invalid_outcome",
            times = c(12)
        ),
        regexp = "Surv"
    )
})


test_that("survtable errors on missing time variable", {
    
    expect_error(
        survtable(
            data = clintrial,
            outcome = "Surv(nonexistent_time, os_status)",
            times = c(12)
        ),
        regexp = "not found"
    )
})


test_that("survtable errors on missing status variable", {
    
    expect_error(
        survtable(
            data = clintrial,
            outcome = "Surv(os_months, nonexistent_status)",
            times = c(12)
        ),
        regexp = "not found"
    )
})


test_that("survtable errors on missing by variable", {
    
    expect_error(
        survtable(
            data = clintrial,
            outcome = "Surv(os_months, os_status)",
            by = "nonexistent_var",
            times = c(12)
        ),
        regexp = "not found"
    )
})


test_that("survtable errors when both times and probs are NULL", {
    
    expect_error(
        survtable(
            data = clintrial,
            outcome = "Surv(os_months, os_status)",
            times = NULL,
            probs = NULL
        )
    )
})


## ============================================================================
## SECTION 21: Complete Workflow
## ============================================================================

test_that("survtable creates complete survival summary", {
    
    result <- survtable(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        by = "treatment",
        times = c(12, 24, 36),
        probs = c(0.25, 0.5),
        time_unit = "months",
        stats = c("survival", "ci"),
        type = "survival",
        conf_level = 0.95,
        test = TRUE,
        test_type = "logrank",
        total = TRUE,
        digits = 0,
        time_digits = 1,
        p_digits = 3
    )
    
    expect_survtable_result(result)
    expect_pvalue_column(result)
    
    ## Check all expected columns
    expect_true("12 months" %in% names(result))
    expect_true("24 months" %in% names(result))
    expect_true("36 months" %in% names(result))
    expect_true(any(grepl("Median", names(result), ignore.case = TRUE)))
})


test_that("survtable PFS/OS workflow works", {
    
    result <- survtable(
        data = clintrial,
        outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
        by = "treatment",
        times = c(12, 24),
        probs = 0.5,
        time_unit = "months",
        total = FALSE,
        test = TRUE,
        labels = c(
            "Surv(pfs_months, pfs_status)" = "Progression-Free Survival",
            "Surv(os_months, os_status)" = "Overall Survival"
        )
    )
    
    expect_survtable_result(result)
    
    ## Should have labeled outcomes
    expect_true("Progression-Free Survival" %in% result$Variable)
    expect_true("Overall Survival" %in% result$Variable)
    
    ## Should have p-values for both
    expect_pvalue_column(result)
})

### * Main function

#' Complete Regression Analysis Workflow
#'
#' Executes a comprehensive regression analysis pipeline that combines univariable 
#' screening, automatic/manual variable selection, and multivariable modeling 
#' in a single function call. This function is designed to streamline the complete 
#' analytical workflow from initial exploration to final adjusted models, with 
#' publication-ready formatted output showing both univariable and multivariable 
#' results side-by-side if desired.
#'
#' @param data Data frame or data.table containing the analysis dataset. The 
#'   function automatically converts data frames to data.tables for efficient
#'   processing.
#'   
#' @param outcome Character string specifying the outcome variable name. For 
#'   time-to-event analysis, use \code{Surv()} syntax for the outcome variable
#'   (\emph{e.g.,} \code{"Surv(os_months, os_status)"}).
#'   
#' @param predictors Character vector of predictor variable names to analyze. 
#'   All predictors are tested in univariable models. The subset included in 
#'   the multivariable model depends on the \code{method} parameter.
#'   
#' @param method Character string specifying the variable selection strategy:
#'   \itemize{
#'     \item \code{"screen"} - Automatic selection based on univariable \emph{p}-value 
#'       threshold. Only predictors with p \eqn{\le} \code{p_threshold} in univariable 
#'       analysis are included in the multivariable model [default]
#'     \item \code{"all"} - Include all predictors in both univariable and 
#'       multivariable analyses (no selection)
#'     \item \code{"custom"} - Manual selection. All predictors in univariable 
#'       analysis, but only those specified in \code{multi_predictors} are 
#'       included in multivariable model
#'   }
#'   
#' @param multi_predictors Character vector of predictors to include in the 
#'   multivariable model when \code{method = "custom"}. Required when using 
#'   custom selection. Ignored for other methods. Default is \code{NULL}.
#'   
#' @param p_threshold Numeric \emph{p}-value threshold for automatic variable
#'   selection when \code{method = "screen"}. Predictors with univariable
#'   \emph{p}-value less than or equal to the threshold are included in
#'   multivariable model. Common values: 0.05 (strict), 0.10 (moderate),
#'   0.20 (liberal screening). Default is 0.05. Ignored for other methods.
#'   
#' @param columns Character string specifying which result columns to display:
#'   \itemize{
#'     \item \code{"both"} - Show both univariable and multivariable results 
#'       side-by-side [default]
#'     \item \code{"uni"} - Show only univariable results
#'     \item \code{"multi"} - Show only multivariable results
#'   }
#'   
#' @param model_type Character string specifying the regression model type:
#'   \itemize{
#'     \item \code{"glm"} - Generalized linear model (default). Supports multiple 
#'       distributions via the \code{family} parameter including logistic, Poisson, 
#'       Gamma, Gaussian, and quasi-likelihood models.
#'     \item \code{"lm"} - Linear regression for continuous outcomes with normally 
#'       distributed errors.
#'     \item \code{"coxph"} - Cox proportional hazards model for time-to-event 
#'       survival analysis. Requires \code{Surv()} outcome syntax.
#'     \item \code{"clogit"} - Conditional logistic regression for matched 
#'       case-control studies.
#'     \item \code{"negbin"} - Negative binomial regression for overdispersed count 
#'       data (requires MASS package). Estimates an additional dispersion parameter 
#'       compared to Poisson regression.
#'     \item \code{"glmer"} - Generalized linear mixed-effects model for hierarchical 
#'       or clustered data with non-normal outcomes (requires \pkg{lme4} package and 
#'       \code{random} parameter).
#'     \item \code{"lmer"} - Linear mixed-effects model for hierarchical or clustered 
#'       data with continuous outcomes (requires \pkg{lme4} package and \code{random} 
#'       parameter).
#'     \item \code{"coxme"} - Cox mixed-effects model for clustered survival data 
#'       (requires coxme package and \code{random} parameter).
#'   }
#'   
#' @param family For GLM and GLMER models, specifies the error distribution and link 
#'   function. Can be a character string, a family function, or a family object.
#'   Ignored for non-GLM/GLMER models.
#'   
#'   \strong{Binary/Binomial outcomes:}
#'   \itemize{
#'     \item \code{"binomial"} or \code{binomial()} - Logistic regression for binary 
#'       outcomes (0/1, TRUE/FALSE). Returns odds ratios (OR). Default.
#'     \item \code{"quasibinomial"} or \code{quasibinomial()} - Logistic regression 
#'       with overdispersion. Use when residual deviance >> residual df.
#'     \item \code{binomial(link = "probit")} - Probit regression (normal CDF link).
#'     \item \code{binomial(link = "cloglog")} - Complementary log-log link for 
#'       asymmetric binary outcomes.
#'   }
#'   
#'   \strong{Count outcomes:}
#'   \itemize{
#'     \item \code{"poisson"} or \code{poisson()} - Poisson regression for count 
#'       data. Returns rate ratios (RR). Assumes mean = variance.
#'     \item \code{"quasipoisson"} or \code{quasipoisson()} - Poisson regression 
#'       with overdispersion. Use when variance > mean.
#'   }
#'   
#'   \strong{Continuous outcomes:}
#'   \itemize{
#'     \item \code{"gaussian"} or \code{gaussian()} - Normal/Gaussian distribution 
#'       for continuous outcomes. Equivalent to linear regression.
#'     \item \code{gaussian(link = "log")} - Log-linear model for positive continuous 
#'       outcomes. Returns multiplicative effects.
#'   }
#'   
#'   \strong{Positive continuous outcomes:}
#'   \itemize{
#'     \item \code{"Gamma"} or \code{Gamma()} - Gamma distribution for positive, 
#'       right-skewed continuous data (\emph{e.g.,} costs, lengths of stay). When
#'       passed as a string, resolves to log link for interpretable multiplicative
#'       effects.
#'     \item \code{Gamma(link = "inverse")} - Gamma with inverse (canonical) link.
#'     \item \code{Gamma(link = "identity")} - Gamma with identity link for additive 
#'       effects on positive outcomes.
#'     \item \code{"inverse.gaussian"} or \code{inverse.gaussian()} - Inverse Gaussian 
#'       for positive, highly right-skewed data.
#'   }
#'   
#'   For negative binomial regression (overdispersed counts), use 
#'   \code{model_type = "negbin"} instead of the \code{family} parameter.
#'   
#'   See \code{\link[stats]{family}} for additional details and options.
#'
#' @param random Character string specifying the random-effects formula for 
#'   mixed-effects models (\code{glmer}, \code{lmer}, \code{coxme}). Use standard
#'   \pkg{lme4}/\pkg{coxme} syntax, \emph{e.g.,} \code{"(1|site)"} for random
#'   intercepts by site, \code{"(1|site/patient)"} for nested random effects.
#'   Required when \code{model_type} is a mixed-effects model type unless random
#'   effects are included in the \code{predictors} vector. Alternatively, random
#'   effects can be included directly in the \code{predictors} vector using the same 
#'   syntax (\emph{e.g.,} \code{predictors = c("age", "sex", "(1|site)")}), though 
#'   they will not be screened as predictors. Default is \code{NULL}.
#'   
#' @param conf_level Numeric confidence level for confidence intervals. Must be 
#'   between 0 and 1. Default is 0.95 (95\% CI).
#'   
#' @param reference_rows Logical. If \code{TRUE}, adds rows for reference 
#'   categories of factor variables with baseline values. Default is \code{TRUE}.
#'   
#' @param show_n Logical. If \code{TRUE}, includes sample size columns. 
#'   Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes events columns (survival 
#'   and logistic models). Default is \code{TRUE}.
#'   
#' @param digits Integer specifying decimal places for effect estimates. 
#'   Default is 2.
#'   
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param labels Named character vector or list providing custom display
#'   labels for variables. Names should match variable names, values are
#'   display labels. Default is \code{NULL}.
#'   
#' @param metrics Character specification for which statistics to display:
#'   \itemize{
#'     \item \code{"both"} - Show effect estimates with CI and \emph{p}-values [default]
#'     \item \code{"effect"} - Show only effect estimates with CI
#'     \item \code{"p"} - Show only \emph{p}-values
#'   }
#'   Can also be a character vector: \code{c("effect", "p")} is equivalent to 
#'   \code{"both"}.
#'   
#' @param return_type Character string specifying what to return:
#'   \itemize{
#'     \item \code{"table"} - Return formatted results table only [default]
#'     \item \code{"model"} - Return multivariable model object only
#'     \item \code{"both"} - Return list with both table and model
#'   }
#'   
#' @param keep_models Logical. If \code{TRUE}, stores univariable model objects 
#'   in the output. Can consume significant memory for many predictors. 
#'   Default is \code{FALSE}.
#'   
#' @param exponentiate Logical. Whether to exponentiate coefficients. Default 
#'   is \code{NULL}, which automatically exponentiates for logistic, Poisson,
#'   and Cox models, and displays raw coefficients for linear models.
#'
#' @param conf_method Character string controlling the confidence interval method.
#'   If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
#'   \itemize{
#'     \item \code{"profile"} - Profile likelihood intervals for GLM and negative
#'       binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
#'       intervals for linear models. Falls back to Wald on profiling failure.
#'       Quasi-likelihood families always use Wald (no true likelihood).
#'     \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
#'       \eqn{\times} SE) for all model types. Faster but less accurate near
#'       boundary conditions or with small subgroups.
#'   }
#'   Cox and mixed-effects models use Wald intervals regardless of this setting.
#'   Set globally with \code{options(summata.conf_method = "wald")} to use Wald
#'   throughout a session. Note: when \code{method = "screen"} and
#'   \code{columns = "multi"}, the internal screening pass always uses Wald
#'   since only \emph{p}-values are needed for variable selection.
#'
#' @param parallel Logical. If \code{TRUE} (default), fits univariable models
#'   in parallel using multiple CPU cores for improved performance.
#' 
#' @param n_cores Integer specifying the number of CPU cores to use for
#'   parallel processing. Default is \code{NULL} (auto-detect: uses
#'   \code{detectCores() - 1}). Ignored when \code{parallel = FALSE}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param verbose Logical. If \code{TRUE}, displays model fitting warnings
#'   (\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
#'   routine fitting messages are suppressed while unexpected warnings are
#'   preserved. When \code{NULL}, uses
#'   \code{getOption("summata.verbose", FALSE)}.
#'
#' @param ... Additional arguments passed to model fitting functions (\emph{e.g.,} 
#'   \code{weights}, \code{subset}, \code{na.action}).
#'
#' @return Depends on \code{return_type} parameter:
#'   
#'   When \code{return_type = "table"} (default): A data.table with S3 class 
#'   \code{"fullfit_result"} containing:
#'   \describe{
#'     \item{Variable}{Character. Predictor name or custom label}
#'     \item{Group}{Character. Category level for factors, empty for continuous}
#'     \item{n/n_group}{Integer. Sample sizes (if \code{show_n = TRUE}). For
#'       variables included in the multivariable model, reflects the
#'       complete-case sample size from the fitted model (listwise deletion
#'       across all included predictors). For variables not selected into the
#'       multivariable model, reflects the per-variable sample size from the
#'       univariable analysis. This follows STROBE guideline item 12,
#'       which recommends reporting the number of participants included at
#'       each stage of analysis.}
#'     \item{events/events_group}{Integer. Event counts (if \code{show_events = TRUE}).
#'       Same complete-case convention as \code{n}: multivariable rows show
#'       events from the fitted model, univariable-only rows show
#'       per-variable counts.}
#'     \item{OR/HR/RR/Coefficient (95\% CI)}{Character. Unadjusted effect 
#'       (if \code{columns} includes "uni" and \code{metrics} includes "effect")}
#'     \item{Uni p}{Character. Univariable \emph{p}-value (if \code{columns} includes 
#'       "uni" and \code{metrics} includes "p")}
#'     \item{aOR/aHR/aRR/Adj. Coefficient (95\% CI)}{Character. Adjusted effect 
#'       (if \code{columns} includes "multi" and \code{metrics} includes "effect")}
#'     \item{Multi p}{Character. Multivariable \emph{p}-value (if \code{columns} 
#'       includes "multi" and \code{metrics} includes "p")}
#'   }
#'   
#'   When \code{return_type = "model"}: The fitted multivariable model object 
#'   (glm, lm, coxph, \emph{etc.}).
#'   
#'   When \code{return_type = "both"}: A list with two elements:
#'   \describe{
#'     \item{table}{The formatted results data.table}
#'     \item{model}{The fitted multivariable model object}
#'   }
#'   
#'   The table includes the following attributes:
#'   \describe{
#'     \item{outcome}{Character. The outcome variable name}
#'     \item{model_type}{Character. The regression model type}
#'     \item{method}{Character. The variable selection method used}
#'     \item{columns}{Character. Which columns were displayed}
#'     \item{model}{The multivariable model object (if fitted)}
#'     \item{uni_results}{The complete univariable screening results}
#'     \item{n_multi}{Integer. Number of predictors in multivariable model}
#'     \item{screened}{Character vector. Names of predictors that passed 
#'       univariable screening at the specified \emph{p}-value threshold}
#'     \item{significant}{Character vector. Names of variables with p < 0.05
#'       in the multivariable model (or univariable if multivariable was not
#'       fitted)}
#'     \item{analysis_counts}{List. How much of the supplied data reached the
#'       models, with elements \code{n_supplied}, \code{n_analyzed},
#'       \code{n_missing_outcome} and \code{n_missing_predictor}, plus
#'       \code{events_supplied} and \code{events_analyzed} where the model
#'       type carries an event count. The screening models and the
#'       multivariable model rest on different samples whenever a predictor
#'       carries missing values, so \code{n_analyzed} holds one value per
#'       model and the reported sample may be a range. The \code{print()}
#'       method always reports the analyzed sample}
#'   }
#'
#' @details
#' \strong{Analysis Workflow:}
#' 
#' The function implements a complete regression analysis pipeline:
#' \enumerate{
#'   \item \strong{Univariable screening}: Fits separate models for each 
#'     predictor (outcome ~ predictor). Each predictor is tested independently 
#'     to understand crude associations.
#'   \item \strong{Variable selection}: Based on the \code{method} parameter:
#'     \itemize{
#'       \item \code{"screen"}: Automatically selects predictors with univariable 
#'         p \eqn{\le} \code{p_threshold}
#'       \item \code{"all"}: Includes all predictors (no selection)
#'       \item \code{"custom"}: Uses predictors specified in \code{multi_predictors}
#'     }
#'   \item \strong{Multivariable modeling}: Fits a single model with selected 
#'     predictors (outcome ~ predictor1 + predictor2 + ...). Estimates are 
#'     adjusted for all other variables in the model.
#'   \item \strong{Output formatting}: Combines results into publication-ready 
#'     table with appropriate effect measures and formatting.
#' }
#' 
#' \strong{Variable Selection Strategies:}
#' 
#' \emph{"Screen" Method} (\code{method = "screen"}):
#' \itemize{
#'   \item Uses \emph{p}-value threshold for automatic selection
#'   \item Liberal thresholds (\emph{e.g.,} 0.20) cast a wide net to avoid missing 
#'     important predictors
#'   \item Stricter thresholds (\emph{e.g.,} 0.05) focus on strongly associated predictors
#'   \item Helps reduce overfitting and multicollinearity
#'   \item Common in exploratory analyses and when sample size is limited
#' }
#' 
#' \emph{"All" Method} (\code{method = "all"}):
#' \itemize{
#'   \item No variable selection; includes all predictors
#'   \item Appropriate when all variables are theoretically important
#'   \item Risk of overfitting with many predictors relative to sample size
#'   \item Useful for confirmatory analyses with pre-specified models
#' }
#' 
#' \emph{"Custom" Method} (\code{method = "custom"}):
#' \itemize{
#'   \item Manual selection based on subject matter knowledge
#'   \item Runs univariable analysis for all predictors (for comparison)
#'   \item Includes only specified predictors in multivariable model
#'   \item Ideal for theory-driven model building
#'   \item Allows comparison of unadjusted vs adjusted effects for all variables
#' }
#' 
#' \strong{Interpreting Results:}
#' 
#' When \code{columns = "both"} (default), tables show:
#' \itemize{
#'   \item \strong{Univariable columns}: Crude associations, unadjusted for 
#'     other variables. Labeled as "OR/HR/RR/Coefficient (95\% CI)" and "Uni \emph{p}"
#'   \item \strong{Multivariable columns}: Adjusted associations, accounting 
#'     for all other predictors in the model. Labeled as "aOR/aHR/aRR/Adj. Coefficient 
#'     (95\% CI)" and "Multi \emph{p}" ("a" = adjusted)
#'   \item Variables not meeting selection criteria show "-" in multivariable columns
#' }
#' 
#' Comparing univariable and multivariable results helps identify:
#' \itemize{
#'   \item \strong{Confounding}: Large changes in effect estimates
#'   \item \strong{Independent effects}: Similar univariable and multivariable 
#'     estimates
#'   \item \strong{Mediation}: Attenuated effects in multivariable model
#'   \item \strong{Suppression}: Effects that emerge only after adjustment
#' }
#' 
#' \strong{Sample Size Considerations:}
#' 
#' Rule of thumb for multivariable models:
#' \itemize{
#'   \item \strong{Logistic regression}: \eqn{\ge} 10 events per predictor variable
#'   \item \strong{Cox regression}:  \eqn{\ge} 10 events per predictor variable
#'   \item \strong{Linear regression}: \eqn{\ge} 10-20 observations per predictor
#' }
#' 
#' Use screening methods to reduce predictor count when these ratios are not met.
#'
#' @seealso 
#' \code{\link{uniscreen}} for univariable screening only,
#' \code{\link{fit}} for fitting a single multivariable model,
#' \code{\link{compfit}} for comparing multiple models,
#' \code{\link{desctable}} for descriptive statistics,
#' \code{\link{autoforest}} for forest plots of the multivariable models,
#' \code{\link{forestsave}} for exporting forest plots,
#' \code{\link{tablesave}} for exporting tables
#'
#' @examples
#' # Load example data
#' data(clintrial)
#' data(clintrial_labels)
#' 
#' # Example 1: Basic screening with p < 0.05 threshold
#' result1 <- fullfit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "smoking",
#'                    "hypertension", "diabetes",
#'                    "treatment", "stage"),
#'     method = "screen",
#'     p_threshold = 0.05,
#'     labels = clintrial_labels
#' )
#' print(result1)
#'
#' \donttest{
#' 
#' # Example 2: Include all predictors (no selection)
#' result2 <- fullfit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     method = "all",
#'     labels = clintrial_labels
#' )
#' print(result2)
#' 
#' # Example 3: Custom variable selection
#' result3 <- fullfit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "smoking", "treatment", "stage"),
#'     method = "custom",
#'     multi_predictors = c("age", "treatment", "stage"),
#'     labels = clintrial_labels
#' )
#' print(result3)
#' 
#' # Example 4: Cox regression with screening
#' library(survival)
#' cox_result <- fullfit(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     model_type = "coxph",
#'     method = "screen",
#'     p_threshold = 0.10,
#'     labels = clintrial_labels
#' )
#' print(cox_result)
#' 
#' # Example 5: Linear regression without screening
#' linear_result <- fullfit(
#'     data = clintrial,
#'     outcome = "bmi",
#'     predictors = c("age", "sex", "smoking", "creatinine"),
#'     model_type = "lm",
#'     method = "all",
#'     labels = clintrial_labels
#' )
#' print(linear_result)
#' 
#' # Example 6: Poisson regression for count outcomes
#' poisson_result <- fullfit(
#'     data = clintrial,
#'     outcome = "fu_count",
#'     predictors = c("age", "stage", "treatment", "surgery"),
#'     model_type = "glm",
#'     family = "poisson",
#'     method = "all",
#'     labels = clintrial_labels
#' )
#' print(poisson_result)
#' 
#' # Example 7: Show only multivariable results
#' multi_only <- fullfit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     method = "all",
#'     columns = "multi",
#'     labels = clintrial_labels
#' )
#' print(multi_only)
#' 
#' # Example 8: Return both table and model object
#' both <- fullfit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     method = "all",
#'     return_type = "both",
#'     labels = clintrial_labels
#' )
#' print(both$table)
#' summary(both$model)
#' 
#' # Example 9: Keep univariable models for diagnostics
#' with_models <- fullfit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "bmi", "creatinine"),
#'     keep_models = TRUE
#' )
#' uni_results <- attr(with_models, "uni_results")
#' uni_models <- attr(uni_results, "models")
#' summary(uni_models[["age"]])
#' 
#' # Example 10: Linear mixed effects with site clustering
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     lmer_result <- fullfit(
#'         data = clintrial,
#'         outcome = "los_days",
#'         predictors = c("age", "treatment", "surgery", "stage"),
#'         random = "(1|site)",
#'         model_type = "lmer",
#'         method = "all",
#'         labels = clintrial_labels
#'     )
#'     print(lmer_result)
#' }
#'
#' }
#' 
#' @family regression functions
#' @export
fullfit <- function(data,
                    outcome, 
                    predictors,
                    method = "screen",
                    multi_predictors = NULL,
                    p_threshold = 0.05,
                    columns = "both",
                    model_type = "glm",
                    family = "binomial",
                    random = NULL,
                    conf_level = 0.95,
                    reference_rows = TRUE,
                    show_n = TRUE,
                    show_events = TRUE,
                    digits = 2,
                    p_digits = 3,
                    labels = NULL,
                    metrics = "both",
                    return_type = "table",
                    keep_models = FALSE,
                    exponentiate = NULL,
                    conf_method = NULL,
                    parallel = TRUE,
                    n_cores = NULL,
                    number_format = NULL,
                    verbose = NULL,
                    ...) {
    
    ## Input validation
    method <- match.arg(method, c("screen", "all", "custom"))
    columns <- match.arg(columns, c("both", "uni", "multi"))
    return_type <- match.arg(return_type, c("table", "model", "both"))

    if (method == "custom" && is.null(multi_predictors)) {
        stop("multi_predictors must be specified when method='custom'")
    }

    ## Convert metrics to standardized format
    if (length(metrics) == 1 && metrics == "both") {
        metrics <- c("effect", "p")
    }

    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    
    ## Resolve verbose setting
    verbose <- if (is.null(verbose)) getOption("summata.verbose", FALSE) else verbose
    
    ## Validate inputs and auto-correct model type if needed
    validation <- validate_fit_inputs(
        data = data,
        outcome = outcome,
        predictors = predictors,
        model_type = model_type,
        family = if (model_type %in% c("glm", "glmer")) family else NULL,
        conf_level = conf_level,
        digits = digits,
        p_digits = p_digits,
        p_threshold = p_threshold,
        auto_correct_model = TRUE
    )
    
    ## Apply any auto-corrections
    if (validation$auto_corrected) {
        model_type <- validation$model_type
    }
    
    ## Handle random effects - support both explicit parameter and embedded in predictors
    mixed_types <- c("glmer", "lmer", "coxme")
    has_random_in_predictors <- any(grepl("\\|", predictors))
    
    if (model_type %in% mixed_types && has_random_in_predictors) {
        random_mask <- grepl("\\|", predictors)
        random_in_preds <- predictors[random_mask]
        predictors <- predictors[!random_mask]
        
        ## Combine explicit random with embedded random (if both provided)
        if (!is.null(random)) {
            message("Note: Random effects specified in both 'random' parameter and 'predictors'. ",
                    "Combining: ", random, " + ", paste(random_in_preds, collapse = " + "))
            random <- paste(c(random, random_in_preds), collapse = " + ")
        } else {
            random <- paste(random_in_preds, collapse = " + ")
        }
        
        if (length(predictors) == 0) {
            stop("No fixed-effect predictors specified. Random effects should be specified ",
                 "via the 'random' parameter or alongside fixed-effect predictors.")
        }
    }
    
    ## Validate random is provided for mixed-effects models
    if (model_type %in% mixed_types && is.null(random)) {
        stop("'random' parameter is required for mixed-effects models (", model_type, ").\n",
             "Example: random = \"(1|site)\" for random intercepts by site.\n",
             "Alternatively, include random effects in 'predictors' using the same syntax.")
    }
    if (!is.null(random) && !model_type %in% mixed_types) {
        warning("'random' parameter is ignored for non-mixed-effects models (", model_type, ").")
        random <- NULL
    }

    ## Step 1: Univariable analysis (if needed)
    uni_results <- NULL
    uni_raw <- NULL
    
    if (columns %in% c("both", "uni")) {
        message("Running univariable analysis...")
        uni_results <- uniscreen(
            data = data,
            outcome = outcome,
            predictors = predictors,
            model_type = model_type,
            family = family,
            random = random,
            conf_level = conf_level,
            reference_rows = reference_rows,
            show_n = show_n,
            show_events = show_events,
            digits = digits,
            p_digits = p_digits,
            labels = labels,
            keep_models = keep_models,
            exponentiate = exponentiate,
            conf_method = conf_method,
            parallel = parallel,
            n_cores = n_cores,
            number_format = number_format,
            verbose = verbose,
            ...
        )
        ## Extract raw data for variable selection
        uni_raw <- attr(uni_results, "raw_data")
    }

    ## Step 2: Determine predictors for multivariable model
    multi_vars <- NULL
    multi_model <- NULL
    multi_results <- NULL
    multi_raw <- NULL

    if (columns %in% c("both", "multi")) {
        if (method == "screen") {
            ## Screen based on p-value threshold using raw data
            if (is.null(uni_raw)) {
                ## Need to run univariable if not already done
                ## Only need p-values for screening, so skip counts for efficiency
                uni_temp <- uniscreen(data, outcome, predictors, model_type, 
                                      family, random = random,
                                      conf_level = conf_level,
                                      reference_rows = FALSE,
                                      show_n = FALSE,
                                      show_events = FALSE,
                                      conf_method = "wald",
                                      parallel = parallel, n_cores = n_cores,
                                      number_format = number_format,
                                      verbose = verbose,
                                      ...)
                uni_raw <- attr(uni_temp, "raw_data")
            }
            
            ## Use raw data for filtering
            multi_vars <- unique(uni_raw[p_value <= p_threshold]$predictor)
            
            if (length(multi_vars) == 0) {
                warning("No variables meet p <= ", p_threshold, " threshold")
                if (return_type == "model") return(NULL)
            }
            
        } else if (method == "all") {
            ## Use all predictors
            multi_vars <- predictors
            
        } else if (method == "custom") {
            ## Use specified predictors
            multi_vars <- multi_predictors
        }
        
        ## Fit the multivariable model where predictors remain
        if (length(multi_vars) > 0) {
            message(sprintf("Fitting multivariable model with %d predictors...", 
                            length(multi_vars)))
            
            multi_results <- fit(
                data = data,
                outcome = outcome,
                predictors = multi_vars,
                model_type = model_type,
                family = family,
                random = random,
                conf_level = conf_level,
                show_n = show_n,
                show_events = show_events,
                digits = digits,
                p_digits = p_digits,
                labels = labels,
                keep_qc_stats = FALSE,  # QC stats not needed for display
                reference_rows = reference_rows,
                exponentiate = exponentiate,
                conf_method = conf_method,
                number_format = number_format,
                verbose = verbose,
                ...
            )
            
            ## Extract model and raw data
            multi_model <- attr(multi_results, "model")
            multi_raw <- attr(multi_results, "raw_data")
        }
    }

    ## Step 3: Handle return types
    if (return_type == "model") {
        return(multi_model)
    }

    ## Step 4: Format combined output using the new formatted tables
    result <- format_fullfit_combined(
        uni_formatted = uni_results,
        multi_formatted = multi_results,
        uni_raw = uni_raw,
        multi_raw = multi_raw,
        predictors = predictors,
        columns = columns,
        metrics = metrics,
        show_n = show_n,
        show_events = show_events,
        labels = labels,
        exponentiate = exponentiate,
        conf_level = conf_level
    )

    ## Add attributes
    data.table::setattr(result, "outcome", outcome)
    data.table::setattr(result, "model_type", model_type)
    data.table::setattr(result, "method", method)
    data.table::setattr(result, "columns", columns)
    data.table::setattr(result, "uni_raw", uni_raw)
    data.table::setattr(result, "multi_raw", multi_raw)
    data.table::setattr(result, "p_threshold", p_threshold)

    ## The table shows univariable estimates alongside multivariable ones,
    ## and the two rest on different samples whenever a predictor carries
    ## missing values. The range therefore spans the screening models and
    ## the multivariable model together.
    ff_outcome_vars <- all.vars(stats::as.formula(paste("~", outcome)))
    ff_predictor_sets <- lapply(predictors, function(p) {
        all.vars(stats::as.formula(paste("~", p)))
    })
    if (!is.null(multi_vars) && length(multi_vars) > 0) {
        ff_predictor_sets <- c(ff_predictor_sets,
                               list(all.vars(stats::as.formula(
                                   paste("~", paste(multi_vars, collapse = "+"))))))
    }
    data.table::setattr(
        result, "analysis_counts",
        get_analysis_counts(data, ff_outcome_vars, ff_predictor_sets,
                            get_event_variable_for_counts(ff_outcome_vars,
                                                          model_type, family))
    )
    data.table::setattr(result, "number_marks",
                        resolve_number_marks(number_format))
    ## Variable labels travel with the result so that the forest plot
    ## functions can apply the same labels without being given them again.
    ## An explicit labels argument to those functions still takes precedence.
    data.table::setattr(result, "labels", labels)
    data.table::setattr(result, "n_screened", length(predictors))
    
    if (!is.null(random)) {
        data.table::setattr(result, "random", random)
    }

    if (!is.null(multi_model)) {
        data.table::setattr(result, "model", multi_model)
    }

    if (keep_models && !is.null(uni_results)) {
        data.table::setattr(result, "uni_results", uni_results)
    }

    if (columns != "uni" && length(multi_vars) > 0) {
        data.table::setattr(result, "n_multi", length(multi_vars))
        data.table::setattr(result, "screened", multi_vars)
    }

    ## Extract significant variable names from multivariable model
    if (!is.null(multi_raw) && nrow(multi_raw) > 0 &&
        "p_value" %in% names(multi_raw) && "variable" %in% names(multi_raw)) {
        sig_vars <- unique(
            multi_raw[!is.na(p_value) & p_value < 0.05]$variable
        )
        data.table::setattr(result, "significant", sig_vars)
    } else if (!is.null(uni_raw) && nrow(uni_raw) > 0 &&
               "p_value" %in% names(uni_raw) && "predictor" %in% names(uni_raw)) {
        ## Fall back to univariable if no multivariable results
        sig_vars <- unique(
            uni_raw[!is.na(p_value) & p_value < p_threshold]$predictor
        )
        data.table::setattr(result, "significant", sig_vars)
    }

    class(result) <- c("fullfit_result", class(result))

    if (return_type == "both") {
        return(list(table = result, model = multi_model))
    } else {
        return(result)
    }
}


### * Print function

#' Print method for fullfit results
#' 
#' Displays a summary header with outcome, model type, method, and number of
#' multivariable predictors before printing the results table.
#' 
#' @param x fullfit_result object.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @family regression functions
#' @export
print.fullfit_result <- function(x, ...) {
    cat("\nFullfit Analysis Results\n\n")
    cat("Outcome: ", attr(x, "outcome"), "\n", sep = "")
    cat("Model Type: ", attr(x, "model_type"), "\n", sep = "")
    
    method <- attr(x, "method")
    p_thresh <- attr(x, "p_threshold")
    if (!is.null(method) && method == "screen" && !is.null(p_thresh)) {
        cat("Method: screen (p < ", p_thresh, ")\n", sep = "")
    } else if (!is.null(method)) {
        cat("Method: ", method, "\n", sep = "")
    }
    
    n_screened <- attr(x, "n_screened")
    if (!is.null(n_screened)) {
        cat("Predictors Screened: ", n_screened, "\n", sep = "")
    }
    
    n_multi <- attr(x, "n_multi")
    if (!is.null(n_multi)) {
        cat("Multivariable Predictors: ", n_multi, "\n", sep = "")
    }
    
    ## Always reported. The screening models and the multivariable model may
    ## rest on different samples, so the analyzed sample may be a range.
    ff_counts <- attr(x, "analysis_counts")
    
    ff_marks <- attr(x, "number_marks")
    
    analyzed <- format_analysis_counts(ff_counts, marks = ff_marks)
    if (!is.null(analyzed)) {
        cat(analyzed, "\n", sep = "")
    }
    
    ## Events per retained variable is the constraint on a multivariable
    ## model, so events are reported alongside the observations
    events_line <- format_event_counts(ff_counts, marks = ff_marks)
    if (!is.null(events_line)) {
        cat(events_line, "\n", sep = "")
    }
    
    cat("\n")
    ## Print as data.table, skipping uniscreen_result print method
    print(as.data.table(x))
    invisible(x)
}


### * Table formatting

#' Format combined fullfit output from formatted tables
#' 
#' Merges univariable and multivariable results into a single publication-ready
#' table with side-by-side display. Uses vectorized merge instead of per-variable loops.
#' 
#' @param uni_formatted Formatted data.table from univariable screening.
#' @param multi_formatted Formatted data.table from multivariable model.
#' @param uni_raw Raw data.table with univariable coefficients.
#' @param multi_raw Raw data.table with multivariable coefficients.
#' @param predictors Character vector of all predictor names.
#' @param columns Character string specifying columns to show ("both", "uni", "multi").
#' @param metrics Character vector specifying metrics to show ("effect", "p").
#' @param show_n Logical whether to include sample size column.
#' @param show_events Logical whether to include events column.
#' @param labels Optional named character vector of variable labels.
#' @param exponentiate Optional logical for coefficient exponentiation.
#' @param conf_level Numeric confidence level for CI label.
#' @return Combined data.table with aligned univariable and multivariable results.
#' @keywords internal
format_fullfit_combined <- function(uni_formatted, multi_formatted, 
                                    uni_raw, multi_raw,
                                    predictors, columns, metrics, 
                                    show_n, show_events, labels,
                                    exponentiate = NULL,
                                    conf_level = 0.95) {
    
    ## Handle single-table cases quickly
    if (columns == "uni" && !is.null(uni_formatted)) {
        return(finalize_column_names(uni_formatted, uni_raw, multi_raw, 
                                     exponentiate, columns, metrics, conf_level))
    }
    
    if (columns == "multi" && !is.null(multi_formatted)) {
        return(finalize_column_names(multi_formatted, uni_raw, multi_raw, 
                                     exponentiate, columns, metrics, conf_level))
    }
    
    ## For "both" columns, use merge approach
    if (is.null(uni_formatted) && is.null(multi_formatted)) {
        return(data.table::data.table())
    }
    
    if (is.null(uni_formatted)) {
        return(finalize_column_names(multi_formatted, uni_raw, multi_raw, 
                                     exponentiate, "multi", metrics, conf_level))
    }
    
    if (is.null(multi_formatted)) {
        return(finalize_column_names(uni_formatted, uni_raw, multi_raw, 
                                     exponentiate, "uni", metrics, conf_level))
    }
    
    ## Events only apply to binomial GLM (logistic) and survival models
    has_events <- FALSE
    if (!is.null(uni_raw)) {
        has_events <- !all(is.na(uni_raw$events))
    } else if (!is.null(multi_raw)) {
        has_events <- !all(is.na(multi_raw$events))
    }
    if (!has_events) show_events <- FALSE
    
    ## Create row keys for merging
    ## Add row identifier within each variable group
    uni_copy <- data.table::copy(uni_formatted)
    multi_copy <- data.table::copy(multi_formatted)
    
    ## Create variable group identifier (propagate variable name to empty rows)
    ## Use cumsum approach since nafill doesn't work with character
    uni_copy[, .var_group_id := cumsum(Variable != "")]
    uni_copy[, .var_group := Variable[1], by = .var_group_id]
    uni_copy[, .row_in_var := seq_len(.N), by = .var_group_id]
    
    multi_copy[, .var_group_id := cumsum(Variable != "")]
    multi_copy[, .var_group := Variable[1], by = .var_group_id]
    multi_copy[, .row_in_var := seq_len(.N), by = .var_group_id]
    
    ## Identify effect and p-value columns (match any CI percentage)
    uni_effect_col <- grep("\\([0-9]+% CI\\)$", names(uni_copy), value = TRUE)[1]
    uni_p_col <- if ("p-value" %in% names(uni_copy)) "p-value" else NULL
    
    multi_effect_col <- grep("\\([0-9]+% CI\\)$", names(multi_copy), value = TRUE)[1]
    multi_p_col <- if ("p-value" %in% names(multi_copy)) "p-value" else NULL
    
    ## Rename columns for merge
    if (!is.na(uni_effect_col) && "effect" %in% metrics) {
        data.table::setnames(uni_copy, uni_effect_col, "uni_effect")
    }
    if (!is.null(uni_p_col) && "p" %in% metrics) {
        data.table::setnames(uni_copy, uni_p_col, "uni_p")
    }
    
    if (!is.na(multi_effect_col) && "effect" %in% metrics) {
        data.table::setnames(multi_copy, multi_effect_col, "multi_effect")
    }
    if (!is.null(multi_p_col) && "p" %in% metrics) {
        data.table::setnames(multi_copy, multi_p_col, "multi_p")
    }
    
    ## Select columns for merge
    uni_keep <- c(".var_group", ".row_in_var", "Variable", "Group")
    if (show_n && "n" %in% names(uni_copy)) uni_keep <- c(uni_keep, "n")
    if (show_events && "Events" %in% names(uni_copy)) uni_keep <- c(uni_keep, "Events")
    if ("uni_effect" %in% names(uni_copy)) uni_keep <- c(uni_keep, "uni_effect")
    if ("uni_p" %in% names(uni_copy)) uni_keep <- c(uni_keep, "uni_p")
    
    uni_subset <- uni_copy[, intersect(uni_keep, names(uni_copy)), with = FALSE]
    
    multi_keep <- c(".var_group", ".row_in_var")
    if ("multi_effect" %in% names(multi_copy)) multi_keep <- c(multi_keep, "multi_effect")
    if ("multi_p" %in% names(multi_copy)) multi_keep <- c(multi_keep, "multi_p")
    ## Also pull complete-case n/Events from multivariable model
    if (show_n && "n" %in% names(multi_copy)) {
        data.table::setnames(multi_copy, "n", ".multi_n")
        multi_keep <- c(multi_keep, ".multi_n")
    }
    if (show_events && "Events" %in% names(multi_copy)) {
        data.table::setnames(multi_copy, "Events", ".multi_Events")
        multi_keep <- c(multi_keep, ".multi_Events")
    }
    
    multi_subset <- multi_copy[, intersect(multi_keep, names(multi_copy)), with = FALSE]
    
    ## Merge on variable group and row position
    result <- merge(uni_subset, multi_subset, 
                    by = c(".var_group", ".row_in_var"), 
                    all.x = TRUE)
    
    ## Fill missing multi values with "-"
    if ("multi_effect" %in% names(result)) {
        result[is.na(multi_effect), multi_effect := "-"]
    }
    if ("multi_p" %in% names(result)) {
        result[is.na(multi_p), multi_p := "-"]
    }
    
    ## Replace per-variable n/Events with complete-case values for
    ## variables included in the multivariable model (STROBE item 12)
    if (".multi_n" %in% names(result)) {
        has_multi <- !is.na(result[[".multi_n"]])
        if (any(has_multi) && "n" %in% names(result)) {
            result[has_multi, n := .multi_n]
        }
        result[, .multi_n := NULL]
    }
    if (".multi_Events" %in% names(result)) {
        has_multi <- !is.na(result[[".multi_Events"]])
        if (any(has_multi) && "Events" %in% names(result)) {
            result[has_multi, Events := .multi_Events]
        }
        result[, .multi_Events := NULL]
    }
    
    ## Preserve original order from uni_formatted
    uni_copy[, .orig_order := .I]
    result[uni_copy, .orig_order := i..orig_order, on = c(".var_group", ".row_in_var")]
    data.table::setorder(result, .orig_order)
    
    ## Clean up temporary columns (only remove those that exist)
    temp_cols <- intersect(c(".var_group", ".var_group_id", ".row_in_var", ".orig_order"), names(result))
    if (length(temp_cols) > 0) {
        result[, (temp_cols) := NULL]
    }
    
    ## Rename columns for display
    result <- finalize_column_names(result, uni_raw, multi_raw, exponentiate, columns, metrics, conf_level)
    
    return(result)
}


#' Finalize Column Names for Display
#' 
#' Renames internal column names (uni_effect, uni_p, multi_effect, multi_p) to
#' publication-ready display names with appropriate effect measure labels
#' (OR, HR, RR, aOR, aHR, aRR, Coefficient, \emph{etc.}).
#' 
#' @param result Data.table with columns to rename. Expected to contain some
#'   combination of: uni_effect, uni_p, multi_effect, multi_p.
#' @param uni_raw Raw univariable data.table used to determine effect type
#'   by checking for presence of OR, HR, RR, or Coefficient columns.
#' @param multi_raw Raw multivariable data.table used to determine adjusted
#'   effect type.
#' @param exponentiate Logical or \code{NULL}. If \code{TRUE}, forces 
#'   exponentiated labels (OR, HR, RR). If \code{FALSE}, uses "Coefficient".
#'   If \code{NULL}, auto-detects from raw data columns.
#' @param columns Character string indicating which columns are present
#'   (\code{"both"}, \code{"uni"}, or \code{"multi"}). Used for context but
#'   renaming is based on actual column presence.
#' @param metrics Character vector of metrics being displayed (\code{"effect"},
#'   \code{"p"}, or both). Used for context.
#' @return The input data.table with columns renamed
#' @keywords internal
finalize_column_names <- function(result, uni_raw, multi_raw, exponentiate, columns, metrics, conf_level = 0.95) {
    
    ## Format confidence level for display
    ci_pct <- round(conf_level * 100)
    ci_label <- paste0(ci_pct, "% CI")
    
    if ("uni_effect" %in% names(result)) {
        effect_type <- determine_effect_type(uni_raw, multi_raw, exponentiate, adjusted = FALSE)
        data.table::setnames(result, "uni_effect", paste0(effect_type, " (", ci_label, ")"))
    }
    
    if ("uni_p" %in% names(result)) {
        data.table::setnames(result, "uni_p", "Uni p")
    }
    
    if ("multi_effect" %in% names(result)) {
        effect_type <- determine_effect_type(uni_raw, multi_raw, exponentiate, adjusted = TRUE)
        data.table::setnames(result, "multi_effect", paste0(effect_type, " (", ci_label, ")"))
    }
    
    if ("multi_p" %in% names(result)) {
        data.table::setnames(result, "multi_p", "Multi p")
    }
    
    return(result)
}


#' Determine Effect Type Label
#' 
#' Identifies the appropriate effect measure label (OR, HR, RR, Coefficient,
#' aOR, aHR, aRR, Adj. Coefficient) based on model type, exponentiation setting,
#' and whether the estimate is adjusted (multivariable) or unadjusted (univariable).
#' 
#' @param uni_raw Raw univariable data.table containing coefficient columns.
#'   Used to detect effect type when \code{adjusted = FALSE}.
#' @param multi_raw Raw multivariable data.table containing coefficient columns.
#'   Used to detect effect type when \code{adjusted = TRUE}.
#' @param exponentiate Logical or \code{NULL} controlling label selection:
#'   \describe{
#'     \item{\code{TRUE}}{Force exponentiated labels (OR, HR, RR, Exp(Coef))}
#'     \item{\code{FALSE}}{Force coefficient labels (Coefficient, Adj. Coefficient)}
#'     \item{\code{NULL}}{Auto-detect from column names in raw data}
#'   }
#' @param adjusted Logical. If \code{TRUE}, returns adjusted effect labels
#'   (aOR, aHR, aRR, Adj. Coefficient) for multivariable results. If \code{FALSE},
#'   returns unadjusted labels (OR, HR, RR, Coefficient) for univariable results.
#' @return Character string with the effect measure label:
#' @keywords internal
determine_effect_type <- function(uni_raw, multi_raw, exponentiate, adjusted = FALSE) {
    raw_data <- if (adjusted) multi_raw else uni_raw
    
    if (!is.null(exponentiate)) {
        if (exponentiate) {
            if (!is.null(raw_data)) {
                if ("OR" %in% names(raw_data)) return(if (adjusted) "aOR" else "OR")
                if ("HR" %in% names(raw_data)) return(if (adjusted) "aHR" else "HR")
                if ("RR" %in% names(raw_data)) return(if (adjusted) "aRR" else "RR")
                return(if (adjusted) "Adj. Exp(Coef)" else "Exp(Coef)")
            }
        } else {
            return(if (adjusted) "Adj. Coefficient" else "Coefficient")
        }
    }
    
    if (!is.null(raw_data)) {
        if ("OR" %in% names(raw_data)) return(if (adjusted) "aOR" else "OR")
        if ("HR" %in% names(raw_data)) return(if (adjusted) "aHR" else "HR")
        if ("RR" %in% names(raw_data)) return(if (adjusted) "aRR" else "RR")
    }
    
    ## Fallback: use Coefficient (not Estimate) for linear models
    return(if (adjusted) "Adj. Coefficient" else "Coefficient")
}

### * Paper settings

#' Get paper size for PDF/LaTeX export
#' 
#' Returns paper dimensions and margin settings for the specified paper size.
#' 
#' @param paper Character string: "letter", "a4", or "auto".
#' @param margins Optional numeric vector of margins (length 1 or 4).
#' @return List with latex_paper, width, height, and margins components.
#' @keywords internal
get_paper_settings <- function (paper, margins = NULL) {
    paper <- match.arg(paper, c("letter", "a4", "auto"))
    settings <- switch(paper,
                       letter = list(latex_paper = "letterpaper", 
                                     width = 8.5,
                                     height = 11,
                                     default_margins = c(1, 1, 1, 1)),
                       a4 = list(latex_paper = "a4paper",
                                 width = 8.27, 
                                 height = 11.69,
                                 default_margins = c(1, 1, 1, 1)),
                       auto = list(latex_paper = "letterpaper", 
                                   width = NULL, height = NULL, default_margins = c(0.5, 0.5, 0.5, 0.5)))
    if (!is.null(margins)) {
        if (length(margins) == 1) {
            margins <- rep(margins, 4)
        }
        else if (length(margins) != 4) {
            stop("margins must be length 1 or 4")
        }
        settings$margins <- margins
    }
    else {
        settings$margins <- settings$default_margins
    }
    return(settings)
}


### * Cell padding

#' Add padding to exported table headers
#' 
#' Adds LaTeX vertical spacing rules to column headers for proper vertical
#' alignment in PDF/LaTeX exports.
#' 
#' @param col_names Character vector of column names.
#' @return Character vector with LaTeX padding rules added.
#' @keywords internal
add_header_padding <- function (col_names) {
    padded_names <- paste0("\\rule{0pt}{3ex}", col_names, "\\rule[-1.5ex]{0pt}{0pt}")
    return(as.character(padded_names))
}


#' Add padding to exported table variables
#' 
#' Inserts blank padding rows between variable groups in exported tables
#' for improved visual separation.
#' 
#' @param df Data.table with Variable column.
#' @return Data.table with padding rows inserted between variable groups.
#' @keywords internal
add_variable_padding <- function (df) {
    var_col <- if ("Variable" %chin% names(df)) 
                   "Variable"
               else if ("variable" %chin% names(df)) 
                   "variable"
               else NULL
    if (is.null(var_col)) 
        return(df)
    
    ## Find variable start rows - rows where Variable is not empty/NA and not indented
    ## Check for various indentation patterns: spaces, &nbsp;, \hspace (LaTeX)
    var_starts <- which(
        df[[var_col]] != "" & 
        !is.na(df[[var_col]]) &
        !grepl("^\\\\hspace", df[[var_col]]) &      # LaTeX indent
        !grepl("^\\s+", df[[var_col]]) &            # Space indent
        !grepl("^&nbsp;", df[[var_col]]) &          # HTML indent
        !grepl("^\\\\quad", df[[var_col]])          # LaTeX quad indent
    )
    
    if (length(var_starts) == 0) 
        return(df)
    
    new_df <- data.table()
    padding_row <- df[1, ]
    padding_row[1, ] <- ""
    new_df <- rbind(new_df, padding_row)
    for (i in seq_along(var_starts)) {
        if (i < length(var_starts)) {
            var_rows <- var_starts[i]:(var_starts[i + 1] - 1)
        }
        else {
            var_rows <- var_starts[i]:nrow(df)
        }
        new_df <- rbind(new_df, df[var_rows, ])
        padding_row <- df[1, ]
        padding_row[1, ] <- ""
        new_df <- rbind(new_df, padding_row)
    }
    return(new_df)
}


### * Cell alignment

#' Determine alignment for exported tables
#' 
#' Creates column alignment string for LaTeX tables. Variable and Group
#' columns are left-aligned; all others are centered.
#' 
#' @param df Data.frame or data.table to determine alignment for.
#' @return Character string with alignment codes (\emph{e.g.,} "rlcc").
#' @keywords internal
determine_alignment <- function(df) {
    align <- "r"  # Start with row numbers column (if any)
    for (col in names(df)) {
        if (col %chin% c("Variable", "Group")) {
            align <- paste0(align, "l")
        } else {
            align <- paste0(align, "c")
        }
    }
    return(align)
}


### * Table formatting

#' Apply formatting to column headers in exported tables (PDF/LaTeX)
#' 
#' Formats column headers for LaTeX output by escaping special characters,
#' italicizing 'n' and 'p', and optionally adding vertical spacing.
#' 
#' @param col_names Character vector of column names.
#' @param add_header_space Logical whether to add vertical padding.
#' @return Character vector with LaTeX-formatted column names.
#' @keywords internal
format_column_headers <- function (col_names, add_header_space = TRUE) {
    if (!is.character(col_names)) {
        col_names <- as.character(col_names)
    }
    col_names <- gsub("%", "\\\\%", col_names)
    col_names <- gsub("^n$", "\\\\textit{n}", col_names, ignore.case = TRUE)
    col_names <- gsub("p-value", "\\\\textit{p}-value", col_names, 
                      ignore.case = TRUE)
    col_names <- gsub("Uni p", "Uni \\\\textit{p}", col_names)
    col_names <- gsub("Multi p", "Multi \\\\textit{p}", col_names)
    if (add_header_space && !is.null(col_names)) {
        col_names <- add_header_padding(col_names)
    }
    return(as.character(col_names))
}

#' Apply formatting to column headers in exported tables (HTML)
#' 
#' Formats column headers for HTML output by italicizing 'n' and 'p'
#' using HTML tags.
#' 
#' @param col_names Character vector of column names.
#' @return Character vector with HTML-formatted column names.
#' @keywords internal
format_column_headers_html <- function (col_names) {
    col_names <- gsub("^n$", "<i>n</i>", col_names, ignore.case = TRUE)
    col_names <- gsub("p-value", "<i>p</i>-value", col_names, 
                      ignore.case = TRUE)
    return(col_names)
}

#' Apply formatting to indented groups
#' 
#' Transforms tables with Variable/Group columns into indented format where
#' group levels appear as indented rows under variable names. Handles both
#' regression and descriptive tables with appropriate \emph{p}-value placement.
#' 
#' @param df Data.table with Variable and Group columns.
#' @param indent_string Character string to use for indentation.
#' @return Data.table with Group column removed and levels indented under Variables.
#' @keywords internal
format_indented_groups <- function (df, indent_string = "    ") {
    if (!("Variable" %chin% names(df) && "Group" %chin% names(df))) {
        return(df)
    }
    
    var_rows <- which(df$Variable != "")
    p_cols <- grep("p-value|Uni p|Multi p", names(df), value = TRUE)
    effect_cols <- c("OR (95% CI)", "HR (95% CI)", "RR (95% CI)", "Estimate (95% CI)",
                     "Coefficient (95% CI)",  # ADD THIS
                     "Univariable OR (95% CI)", "Multivariable aOR (95% CI)", 
                     "Univariable HR (95% CI)", "Multivariable aHR (95% CI)",
                     "Univariable RR (95% CI)", "Multivariable aRR (95% CI)",
                     "Univariable Estimate (95% CI)", "Multivariable Estimate (95% CI)",
                     "Univariable Coefficient (95% CI)", "Multivariable Coefficient (95% CI)")
    is_regression_table <- any(effect_cols %chin% names(df))
    is_fullfit_table <- any(c("Uni p", "Multi p") %chin% names(df))
    new_df <- data.table()
    
    for (i in seq_along(var_rows)) {
        current <- var_rows[i]
        next_var <- if (i < length(var_rows)) 
                        var_rows[i + 1]
                    else nrow(df) + 1
        var_name <- df$Variable[current]
        has_group <- df$Group[current] != "" && !is.na(df$Group[current]) && 
            df$Group[current] != "-"
        
        if (has_group) {
            if (is_regression_table) {
                var_row <- df[current, ]
                var_row$Variable <- var_name
                var_row$Group <- ""
                data_cols <- setdiff(names(df), c("Variable", "Group", p_cols))
                for (col in data_cols) {
                    var_row[[col]] <- ""
                }
                new_df <- rbind(new_df, var_row)
                
                for (j in current:(next_var - 1)) {
                    group_row <- df[j, ]
                    group_row$Variable <- paste0(indent_string, 
                                                 df$Group[j])
                    if (is_fullfit_table) {
                        for (ec in effect_cols) {
                            if (ec %chin% names(group_row)) {
                                val <- group_row[[ec]]
                                if (!is.na(val) && (val == "-" || grepl("Reference", 
                                                                        val))) {
                                    if (grepl("Univariable", ec) && "Uni p" %chin% 
                                        names(group_row)) {
                                        group_row[["Uni p"]] <- "-"
                                    }
                                    else if (grepl("Multivariable", ec) && 
                                             "Multi p" %chin% names(group_row)) {
                                        group_row[["Multi p"]] <- "-"
                                    }
                                }
                            }
                        }
                    }
                    else {
                        is_reference <- FALSE
                        for (ec in effect_cols) {
                            if (ec %chin% names(group_row)) {
                                val <- group_row[[ec]]
                                if (!is.na(val) && (val == "-" || grepl("Reference", 
                                                                        val))) {
                                    is_reference <- TRUE
                                    break
                                }
                            }
                        }
                        if (is_reference) {
                            for (p_col in p_cols) {
                                if (p_col %chin% names(group_row)) {
                                    group_row[[p_col]] <- ""
                                }
                            }
                        }
                    }
                    new_df <- rbind(new_df, group_row)
                }
            }
            else {
                var_row <- df[current, ]
                var_row$Variable <- var_name
                data_cols <- setdiff(names(df), c("Variable", 
                                                  "Group", p_cols))
                for (col in data_cols) {
                    var_row[[col]] <- ""
                }
                new_df <- rbind(new_df, var_row)
                
                for (j in current:(next_var - 1)) {
                    group_row <- df[j, ]
                    group_row$Variable <- paste0(indent_string, 
                                                 df$Group[j])
                    for (p_col in p_cols) {
                        if (p_col %chin% names(group_row)) {
                            group_row[[p_col]] <- ""
                        }
                    }
                    new_df <- rbind(new_df, group_row)
                }
            }
        }
        else {
            new_df <- rbind(new_df, df[current, ])
        }
    }
    new_df$Group <- NULL
    rownames(new_df) <- NULL
    return(new_df)
}

#' Condense table rows for more compact display
#' 
#' Collapses multi-row variables into single rows for compact tables.
#' Continuous variables show only the first statistic row, binary categorical
#' variables show only the non-reference category, and survival variables
#' show only the median row.
#' 
#' @param df Data.table with Variable and Group columns.
#' @param indent_groups Logical whether indentation will be applied (affects processing).
#' @return Data.table with condensed rows.
#' @keywords internal
condense_table_rows <- function(df, indent_groups = TRUE) {
    
    ## Work with a copy
    result <- data.table::copy(as.data.table(df))
    
    ## Detect table type
    is_descriptive <- !any(grepl("(OR|HR|RR|Coefficient|Estimate).*\\(95% CI\\)", names(result)))
    
    ## Track rows to delete
    rows_to_delete <- integer()
    
    ## Process each variable
    vars_to_process <- which(result$Variable != "" & !is.na(result$Variable))
    
    for (i in seq_along(vars_to_process)) {
        var_start <- vars_to_process[i]
        var_end <- if (i < length(vars_to_process)) {
                       vars_to_process[i + 1] - 1
                   } else {
                       nrow(result)
                   }
        
        if (var_start %in% rows_to_delete) next
        
        var_name <- result$Variable[var_start]
        var_rows <- result[var_start:var_end]
        n_rows <- nrow(var_rows)
        
        if ("Group" %chin% names(var_rows)) {
            groups <- var_rows$Group
            
            ## Check for continuous variable
            is_continuous <- any(grepl("^(Mean|Median|Range|SD|IQR)", groups[1], ignore.case = TRUE))
            
            ## Check for binary categorical
            non_empty_groups <- groups[groups != "" & !is.na(groups)]
            is_binary <- length(non_empty_groups) == 2 && !is_continuous
            
            ## Check for survival
            is_survival <- any(grepl("Median.*\\(.*CI.*\\)", groups, ignore.case = TRUE))
            
            if (is_continuous || is_survival) {
                ## For continuous/survival, keep only first row
                if (is_descriptive) {
                    stat_type <- gsub("\\s*\\(.*\\)", "", groups[1])
                    stat_type <- gsub("\\s*\\+/-.*", "", stat_type)
                    if (stat_type != "" && !is.na(stat_type) && stat_type != "-") {
                        if (grepl("Mean \\+/- SD", groups[1])) {
                            result[var_start, Variable := paste0(var_name, ", mean +/- SD")]
                        } else if (grepl("Median", groups[1]) && grepl("IQR", groups[1])) {
                            result[var_start, Variable := paste0(var_name, ", median [IQR]")]
                        } else if (grepl("Median", groups[1]) && is_survival) {
                            result[var_start, Variable := paste0(var_name, ", median (95% CI)")]
                        } else {
                            result[var_start, Variable := paste0(var_name, ", ", tolower(stat_type))]
                        }
                    }
                    ## Clear Group column
                    result[var_start, Group := ""]
                }
                
                ## Mark extra rows for deletion
                if (n_rows > 1) {
                    rows_to_delete <- c(rows_to_delete, (var_start + 1):var_end)
                }
                
            } else if (is_binary) {
                
                ## For binary categorical
                data_cols <- setdiff(names(result), c("Variable", "Group", "p-value", "p.value"))
                
                if (length(data_cols) > 0) {
                    ## For regression tables, find the coefficient/effect column
                    ## This is more reliable than using the first data column (often "n")
                    effect_col <- NULL
                    effect_patterns <- c("Coefficient", "OR", "HR", "RR", "Estimate", "aOR", "aHR", "aRR")
                    for (col in data_cols) {
                        if (any(vapply(effect_patterns, function(p) grepl(p, col, ignore.case = TRUE), logical(1)))) {
                            effect_col <- col
                            break
                        }
                    }
                    
                    ## Use effect column if found, otherwise fall back to first data column
                    check_col <- if (!is.null(effect_col)) effect_col else data_cols[1]
                    
                    ## Find non-reference row by checking for actual data (not "-", "reference", etc.)
                    non_ref_idx <- which(!var_rows[[check_col]] %chin% c("-", "reference", "Reference", ""))
                    
                    if (length(non_ref_idx) > 1) {
                        ## Multiple non-ref rows - take the first one
                        non_ref_idx <- non_ref_idx[1]
                    } else if (length(non_ref_idx) == 0) {
                        ## All rows are reference-like - skip condensing
                        next
                    }
                    
                    if (non_ref_idx <= n_rows) {
                        non_ref_row <- var_start + non_ref_idx - 1
                        ref_idx <- setdiff(1:n_rows, non_ref_idx)[1]
                        ref_row <- var_start + ref_idx - 1
                        
                        non_ref_category <- result$Group[non_ref_row]
                        ref_category <- result$Group[ref_row]
                        
                        if (!is.na(non_ref_category) && non_ref_category != "") {
                            ## Use greedy helper function for condensing detection
                            ## Pass var_name as label for case-insensitive matching
                            if (should_condense_binary(ref_category, non_ref_category, var_name)) {
                                result[var_start, Variable := paste0(var_name)]
                            } else {
                                result[var_start, Variable := paste0(var_name, " (", non_ref_category, ")")]                                
                            }
                        }
                        
                        ## Copy statistics
                        for (col in data_cols) {
                            if (col %chin% names(result)) {
                                result[var_start, (col) := result[non_ref_row, get(col)]]
                            }
                        }
                        
                        ## Copy p-value
                        if ("p-value" %chin% names(result)) {
                            pval <- result[var_start:var_end, `p-value`]
                            pval <- pval[!is.na(pval) & pval != ""]
                            if (length(pval) > 0) {
                                result[var_start, `p-value` := pval[1]]
                            }
                        }
                        
                        ## Clear Group column
                        result[var_start, Group := ""]
                        
                        ## Mark other rows for deletion
                        rows_to_delete <- c(rows_to_delete, (var_start + 1):var_end)
                    }
                }
            } 
        }
    }
    
    ## Remove marked rows
    if (length(rows_to_delete) > 0) {
        rows_to_delete <- sort(unique(rows_to_delete))
        rows_to_delete <- rows_to_delete[rows_to_delete <= nrow(result)]
        if (length(rows_to_delete) > 0) {
            result <- result[-rows_to_delete]
        }
    }
    
    return(result)
}


#' Condense quantitative variable rows only
#' 
#' Collapses multi-row continuous and survival variables into single rows
#' while preserving all categorical variable rows (including binary).
#' Only applies to descriptive tables from desctable().
#' 
#' @param df Data.table or data frame
#' @param indent_groups Logical. Whether to apply indentation formatting.
#' @return A data.table with condensed continuous/survival rows
#' @keywords internal
condense_quantitative_rows <- function(df, indent_groups = TRUE) {
    
    ## Work with a copy
    result <- data.table::copy(as.data.table(df))
    
    ## Detect table type - only apply to descriptive tables
    is_descriptive <- !any(grepl("(OR|HR|RR|Coefficient|Estimate).*\\(95% CI\\)", names(result)))
    
    if (!is_descriptive) {
        warning("condense_quantitative only applies to descriptive tables. Returning unchanged.")
        return(result)
    }
    
    ## Track rows to delete
    rows_to_delete <- integer()
    
    ## Process each variable
    vars_to_process <- which(result$Variable != "" & !is.na(result$Variable))
    
    for (i in seq_along(vars_to_process)) {
        var_start <- vars_to_process[i]
        var_end <- if (i < length(vars_to_process)) {
                       vars_to_process[i + 1] - 1
                   } else {
                       nrow(result)
                   }
        
        if (var_start %in% rows_to_delete) next
        
        var_name <- result$Variable[var_start]
        var_rows <- result[var_start:var_end]
        n_rows <- nrow(var_rows)
        
        if ("Group" %chin% names(var_rows)) {
            groups <- var_rows$Group
            
            ## Check for continuous variable
            is_continuous <- any(grepl("^(Mean|Median|Range|SD|IQR)", groups[1], ignore.case = TRUE))
            
            ## Check for survival
            is_survival <- any(grepl("Median.*\\(.*CI.*\\)", groups, ignore.case = TRUE))
            
            ## Only condense continuous and survival - NOT categorical (including binary)
            if (is_continuous || is_survival) {
                ## For continuous/survival, keep only first row
                stat_type <- gsub("\\s*\\(.*\\)", "", groups[1])
                stat_type <- gsub("\\s*\\+/-.*", "", stat_type)
                if (stat_type != "" && !is.na(stat_type) && stat_type != "-") {
                    if (grepl("Mean \\+/- SD", groups[1])) {
                        result[var_start, Variable := paste0(var_name, ", mean +/- SD")]
                    } else if (grepl("Median", groups[1]) && grepl("IQR", groups[1])) {
                        result[var_start, Variable := paste0(var_name, ", median [IQR]")]
                    } else if (grepl("Median", groups[1]) && is_survival) {
                        result[var_start, Variable := paste0(var_name, ", median (95% CI)")]
                    } else {
                        result[var_start, Variable := paste0(var_name, ", ", tolower(stat_type))]
                    }
                }
                ## Clear Group column
                result[var_start, Group := ""]
                
                ## Mark extra rows for deletion
                if (n_rows > 1) {
                    rows_to_delete <- c(rows_to_delete, (var_start + 1):var_end)
                }
            }
            ## Note: Categorical variables (including binary) are not processed here
        }
    }
    
    ## Remove marked rows
    if (length(rows_to_delete) > 0) {
        rows_to_delete <- sort(unique(rows_to_delete))
        rows_to_delete <- rows_to_delete[rows_to_delete <= nrow(result)]
        if (length(rows_to_delete) > 0) {
            result <- result[-rows_to_delete]
        }
    }
    
    return(result)
}


#' Identify variable groups before indentation
#' 
#' Detects variable group boundaries by finding rows where Variable column
#' is non-empty. Returns row indices for each group for zebra stripe application.
#' 
#' @param df Data.table with Variable column.
#' @return List of integer vectors, each containing row indices for one variable group.
#' @keywords internal
identify_variable_groups <- function(df) {
    if (!"Variable" %in% names(df)) return(NULL)
    
    var_starts <- which(df$Variable != "" & !is.na(df$Variable))
    if (length(var_starts) == 0) return(NULL)
    
    ## Vectorized group creation
    var_ends <- c(var_starts[-1] - 1, nrow(df))
    groups <- mapply(seq, var_starts, var_ends, SIMPLIFY = FALSE)
    
    return(groups)
}


#' Replace empty cells with "-"
#' 
#' Converts empty strings and NA values to "-" for consistent display
#' in exported tables. Preserves Variable column values.
#' 
#' @param df Data.frame or data.table to process.
#' @return Data.table with empty cells replaced by "-".
#' @keywords internal
replace_empty_cells <- function(df) {
    ## Convert to data.table for efficient in-place modification
    dt <- data.table::as.data.table(df)
    
    ## Get columns to process (excluding Variable)
    cols_to_process <- setdiff(names(dt), "Variable")
    
    ## Vectorized replacement using data.table
    for (col in cols_to_process) {
        dt[is.na(get(col)) | get(col) == "", (col) := "-"]
    }
    
    return(data.table::as.data.table(dt))
}


#' Calculate table width based on paper size and orientation
#' 
#' Computes usable table width in inches based on paper dimensions and
#' orientation, accounting for standard 1-inch margins.
#' 
#' @param paper Character string paper size ("letter", "a4", "legal").
#' @param orientation Character string page orientation ("portrait", "landscape").
#' @return Numeric usable width in inches.
#' @keywords internal
calculate_table_width <- function(paper, orientation) {
    ## Define paper sizes (in inches) with margins
    paper_sizes <- list(
        letter = c(width = 8.5, height = 11),
        a4 = c(width = 8.27, height = 11.69),
        legal = c(width = 8.5, height = 14)
    )
    
    if (!paper %chin% names(paper_sizes)) {
        paper <- "letter"
    }
    
    dims <- paper_sizes[[paper]]
    
    ## Swap for landscape
    if (orientation == "landscape") {
        dims <- c(width = dims["height"], height = dims["width"])
    }
    
    ## Subtract margins (1 inch on each side)
    usable_width <- dims["width"] - 2
    
    return(as.numeric(usable_width))
}


### * TeX-specific functions

#' Check LaTeX installation
#' 
#' Verifies that a LaTeX distribution (pdflatex or xelatex) is available
#' on the system for PDF compilation.
#' 
#' @return Logical \code{TRUE} if LaTeX is available, \code{FALSE} otherwise.
#' @keywords internal
check_latex <- function () {
    pdflatex_check <- Sys.which("pdflatex")
    if (pdflatex_check != "") 
        return(TRUE)
    xelatex_check <- Sys.which("xelatex")
    if (xelatex_check != "") 
        return(TRUE)
    return(FALSE)
}

#' Sanitize certain symbols for LaTeX
#' 
#' Escapes special LaTeX characters (%, &, #, _, $, ^, ~, {, }) while
#' preserving existing LaTeX commands. Uses negative lookbehind to avoid
#' double-escaping already escaped characters.
#' 
#' @param x Character vector to sanitize.
#' @return Character vector with special characters escaped for LaTeX.
#' @keywords internal
sanitize_for_latex <- function(x) {
    if (is.null(x) || length(x) == 0) 
        return(x)
    
    ## Process each element
    result <- x
    
    for (i in seq_along(x)) {
        if (is.na(x[i])) next
        
        val <- x[i]
        
        ## Check for already-formatted LaTeX commands
        has_latex <- grepl("\\\\(text(bf|it|tt|sc|sl|rm)|hspace|vspace|rule|begin|end|[a-zA-Z]+\\{)", val)
        
        if (has_latex) {
            ## String has LaTeX commands - only escape unescaped special chars
            ## Escape % that is not already escaped (not preceded by \)
            val <- gsub("(?<!\\\\)%", "\\\\%", val, perl = TRUE)
            ## Escape & that is not already escaped
            val <- gsub("(?<!\\\\)&", "\\\\&", val, perl = TRUE)
            ## Escape # that is not already escaped
            val <- gsub("(?<!\\\\)#", "\\\\#", val, perl = TRUE)
            ## Escape _ that is not already escaped
            val <- gsub("(?<!\\\\)_", "\\\\_", val, perl = TRUE)
            ## Escape $ that is not already escaped
            val <- gsub("(?<!\\\\)\\$", "\\\\$", val, perl = TRUE)
        } else {
            ## No LaTeX commands - check if already escaped
            already_escaped <- grepl("\\\\[%&#_$]", val)
            
            if (!already_escaped) {
                ## Full sanitization needed
                val <- gsub("\\\\", "\\\\textbackslash{}", val)
                val <- gsub("%", "\\\\%", val)
                val <- gsub("&", "\\\\&", val)
                val <- gsub("#", "\\\\#", val)
                val <- gsub("_", "\\\\_", val)
                val <- gsub("\\$", "\\\\$", val)
                val <- gsub("\\^", "\\\\textasciicircum{}", val)
                val <- gsub("~", "\\\\textasciitilde{}", val)
                val <- gsub("\\{", "\\\\{", val)
                val <- gsub("\\}", "\\\\}", val)
            }
        }
        
        result[i] <- val
    }
    
    return(result)
}


#' Format column headers with n counts (TeX)
#' 
#' Creates LaTeX-formatted column headers with sample size counts displayed
#' below the column name in a stacked format.
#' 
#' @param col_names Character vector of column names.
#' @param n_row_data Named list or data.table row with n values for each column.
#' @return Character vector with LaTeX-formatted headers including n counts.
#' @keywords internal
format_column_headers_with_n_tex <- function(col_names, n_row_data) {
    new_names <- character(length(col_names))
    
    for (i in seq_along(col_names)) {
        col <- col_names[i]
        
        ## Sanitize the column name first
        col_sanitized <- sanitize_for_latex(col)
        
        has_n <- col %chin% names(n_row_data) && 
            !is.na(n_row_data[[col]]) && 
            n_row_data[[col]] != "" && 
            n_row_data[[col]] != "0"
        
        if (col %chin% c("Variable", "Group")) {
            new_names[i] <- format_column_headers(col_sanitized)
        } else if (col %chin% c("p-value", "p value")) {
            new_names[i] <- format_column_headers(col_sanitized)
        } else if (has_n) {
            n_value <- n_row_data[[col]]
            ## Build the complex structure with already-sanitized name
            new_names[i] <- paste0("\\begin{tabular}{@{}c@{}}\\rule{0pt}{2.5ex}", 
                                   col_sanitized,  # Use sanitized version
                                   "\\\\[-0ex] (\\textit{N} = ", 
                                   n_value, 
                                   ")\\rule[-1ex]{0pt}{0pt}\\end{tabular}")
        } else {
            new_names[i] <- format_column_headers(col_sanitized)
        }
    }
    
    return(new_names)
}


#' Format \emph{p}-values for exported tables (TeX)
#' 
#' Applies bold formatting to significant \emph{p}-values in LaTeX tables using
#' the textbf command.
#' 
#' @param df Data.table containing \emph{p}-value columns.
#' @param p_threshold Numeric threshold for significance (default 0.05).
#' @return Data.table with significant \emph{p}-values wrapped in LaTeX bold commands.
#' @keywords internal
format_pvalues_export_tex <- function (df, p_threshold = 0.05) {
    for (col in names(df)) {
        if (col == "p-value" || col == "Uni p" || col == "Multi p" || 
            grepl("p.value|pvalue", col, ignore.case = TRUE)) {
            for (i in seq_len(nrow(df))) {
                cell_value <- as.character(df[[col]][i])
                if (is.na(cell_value) || cell_value == "" || 
                    cell_value == "NA" || grepl("\\\\textbf", cell_value)) {
                    next
                }
                is_significant <- FALSE
                if (grepl("^<\\s*0\\.001", cell_value)) {
                    is_significant <- TRUE
                }
                else if (grepl("^[0-9]\\.[0-9]", cell_value) || 
                         grepl("^0\\.[0-9]", cell_value)) {
                    p_numeric <- suppressWarnings(as.numeric(gsub("[^0-9.]", 
                                                                  "", cell_value)))
                    if (!is.na(p_numeric) && p_numeric < p_threshold) {
                        is_significant <- TRUE
                    }
                }
                if (is_significant) {
                    df[[col]][i] <- paste0("\\textbf{", cell_value, 
                                           "}")
                }
            }
        }
    }
    return(df)
}


### * HTML-specific functions

#' Format column headers with n counts (HTML)
#' 
#' Creates HTML-formatted column headers with sample size counts displayed
#' below the column name using line breaks.
#' 
#' @param col_names Character vector of column names.
#' @param n_row_data Named list or data.table row with n values for each column.
#' @return Character vector with HTML-formatted headers including n counts.
#' @keywords internal
format_column_headers_with_n_html <- function(col_names, n_row_data) {
    new_names <- col_names
    for (i in seq_along(col_names)) {
        col <- col_names[i]
        
        ## Skip Variable, Group, and p-value columns
        if (col %chin% c("Variable", "Group", "p-value", "p value")) {
            new_names[i] <- format_column_headers_html(col)
            next
        }
        
        ## Get n value from the N row
        if (col %chin% names(n_row_data)) {
            n_value <- n_row_data[[col]]
            if (!is.na(n_value) && n_value != "" && n_value != "0") {
                ## Format header with only N italicized
                clean_col <- format_column_headers_html(col)
                new_names[i] <- paste0(clean_col, "<br>(<i>N</i> = ", n_value, ")")
            } else {
                new_names[i] <- format_column_headers_html(col)
            }
        } else {
            new_names[i] <- format_column_headers_html(col)
        }
    }
    return(new_names)
}


#' Format \emph{p}-values for exported tables (HTML)
#' 
#' Applies bold formatting to significant \emph{p}-values in HTML tables using
#' the b tag.
#' 
#' @param df Data.table containing \emph{p}-value columns.
#' @param p_threshold Numeric threshold for significance (default 0.05).
#' @return Data.table with significant \emph{p}-values wrapped in HTML bold tags.
#' @keywords internal
format_pvalues_export_html <- function (df, p_threshold = 0.05) {
    for (col in names(df)) {
        if (col == "p-value" || col == "Uni p" || col == "Multi p" || 
            grepl("p.value|pvalue", col, ignore.case = TRUE)) {
            for (i in seq_len(nrow(df))) {
                cell_value <- as.character(df[[col]][i])
                if (is.na(cell_value) || cell_value == "" || 
                    cell_value == "NA") {
                    next
                }
                is_significant <- FALSE
                if (grepl("^<\\s*0\\.001", cell_value)) {
                    is_significant <- TRUE
                }
                else if (grepl("^[0-9]\\.[0-9]", cell_value) || 
                         grepl("^0\\.[0-9]", cell_value)) {
                    p_numeric <- suppressWarnings(as.numeric(gsub("[^0-9.]", 
                                                                  "", cell_value)))
                    if (!is.na(p_numeric) && p_numeric < p_threshold) {
                        is_significant <- TRUE
                    }
                }
                if (is_significant) {
                    df[[col]][i] <- paste0("<b>", cell_value, "</b>")
                }
            }
        }
    }
    return(df)
}


### * Flextable-specific functions

#' Core flextable processing function
#' 
#' Central processing function for creating \pkg{flextable} objects from data tables.
#' Handles N row extraction, condensing, indentation, zebra stripes, formatting,
#' and styling. Used by table2docx, table2pptx, and table2rtf.
#' 
#' @param table Data.frame or data.table to process.
#' @param caption Optional character string for table caption.
#' @param font_size Numeric font size in points.
#' @param font_family Character string font family name.
#' @param format_headers Logical whether to format headers.
#' @param bold_significant Logical whether to bold significant \emph{p}-values.
#' @param p_threshold Numeric \emph{p}-value threshold for significance.
#' @param indent_groups Logical whether to indent group levels.
#' @param condense_table Logical whether to condense all variable types.
#' @param condense_quantitative Logical whether to condense only continuous/survival.
#' @param zebra_stripes Logical whether to apply alternating row shading.
#' @param dark_header Logical whether to use dark header style.
#' @param bold_variables Logical whether to bold variable names (non-indented rows).
#' @param paper Character string paper size.
#' @param orientation Character string page orientation.
#' @param width Optional numeric table width in inches.
#' @param align Optional alignment specification.
#' @return List with ft (\pkg{flextable} object) and caption components.
#' @keywords internal
process_table_for_flextable <- function(table,
                                        caption = NULL,
                                        font_size = 10,
                                        font_family = "Arial",
                                        format_headers = TRUE,
                                        bold_significant = TRUE,
                                        p_threshold = 0.05,
                                        indent_groups = FALSE,
                                        condense_table = FALSE,
                                        condense_quantitative = FALSE,
                                        zebra_stripes = FALSE,
                                        dark_header = FALSE,
                                        bold_variables = TRUE,
                                        paper = "letter",
                                        orientation = "portrait",
                                        width = NULL,
                                        align = NULL) {
    
    ## Convert to data.table
    df <- data.table::as.data.table(table)
    
    ## Handle N row if present
    has_n_row <- FALSE
    n_row_data <- NULL
    if (nrow(df) > 0 && "Variable" %chin% names(df) && 
        !is.na(df$Variable[1]) && df$Variable[1] == "N") {
        has_n_row <- TRUE
        n_row_data <- df[1, ]
        df <- df[-1, ]
    }
    
    ## Track variable groups BEFORE any transformation
    var_groups <- NULL
    if (zebra_stripes && "Variable" %chin% names(df)) {
        var_groups <- identify_variable_groups(df)
    }
    
    ## Apply condensing if requested
    if (condense_table) {
        indent_groups <- TRUE
        df <- condense_table_rows(df, indent_groups = indent_groups)
        
        ## Update variable groups after condensing
        if (zebra_stripes) {
            var_groups <- identify_variable_groups(df)
        }
        
        df <- format_indented_groups(df, indent_string = "    ")
    } else if (condense_quantitative) {
        ## Only condense continuous/survival variables (not categorical)
        ## Also set indent_groups = TRUE to avoid awkward empty Group column
        indent_groups <- TRUE
        df <- condense_quantitative_rows(df, indent_groups = indent_groups)
        
        ## Update variable groups after condensing
        if (zebra_stripes) {
            var_groups <- identify_variable_groups(df)
        }
        
        df <- format_indented_groups(df, indent_string = "    ")
    } else if (indent_groups) {
        df <- format_indented_groups(df, indent_string = "    ")
    }
    
    ## Replace empty cells with "-" for consistency
    df <- replace_empty_cells(df)
    
    ## Create flextable
    ft <- flextable::flextable(df)
    
    ## Set font
    ft <- flextable::font(ft, fontname = font_family, part = "all")
    ft <- flextable::fontsize(ft, size = font_size, part = "all")
    
    ## Format headers
    if (format_headers) {
        ft <- format_headers_ft(ft, has_n_row, n_row_data)
    }

    ## Apply dark header if requested
    if (dark_header) {
        ft <- flextable::bg(ft, bg = "#000000", part = "header")
        ft <- flextable::color(ft, color = "#FFFFFF", part = "header")
        ft <- flextable::bold(ft, bold = TRUE, part = "header")
    }
    
    ## Bold significant p-values
    if (bold_significant) {
        ft <- bold_pvalues_ft(ft, df, p_threshold)
    }
    
    ## Bold variable names (non-indented rows in Variable column)
    if (bold_variables && "Variable" %chin% names(df)) {
        ## Find rows where Variable is not empty and not indented (doesn't start with spaces)
        var_rows <- which(!is.na(df$Variable) & 
                          df$Variable != "" & 
                          df$Variable != "-" &
                          !grepl("^\\s+", df$Variable))
        if (length(var_rows) > 0) {
            ft <- flextable::bold(ft, i = var_rows, j = "Variable", part = "body")
        }
    }
    
    ## Set alignment
    if (is.null(align)) {
        for (col in names(df)) {
            if (col %chin% c("Variable", "Group")) {
                ft <- flextable::align(ft, j = col, align = "left", part = "all")
            } else {
                ft <- flextable::align(ft, j = col, align = "center", part = "all")
            }
        }
    } else {
        if (length(align) == 1) {
            ft <- flextable::align(ft, align = align, part = "all")
        } else if (length(align) == ncol(df)) {
            for (i in seq_along(align)) {
                ft <- flextable::align(ft, j = i, align = align[i], part = "all")
            }
        }
    }
    
    ## Add borders
    ft <- flextable::border_remove(ft)
    ft <- flextable::hline_top(ft, border = officer::fp_border(width = 2), part = "header")
    ft <- flextable::hline_bottom(ft, border = officer::fp_border(width = 1), part = "header")
    ft <- flextable::hline_bottom(ft, border = officer::fp_border(width = 2), part = "body")
    
    ## Reduce line spacing and padding
    ft <- flextable::line_spacing(ft, space = 1)
    ft <- flextable::padding(ft, i = NULL, j = NULL,
                             padding.top = 1, padding.bottom = 1,
                             padding.left = 1, padding.right = 1)
    
    ## Add zebra stripes by variable group if requested
    if (zebra_stripes && !is.null(var_groups)) {
        ft <- apply_zebra_stripes_ft(ft, df, var_groups)
    }
    
    ## Calculate width based on paper and orientation if not specified
    if (is.null(width)) {
        width <- calculate_table_width(paper, orientation)
    }
    
    ## Set width
    ft <- flextable::width(ft, width = width / ncol(df))
    
    return(list(ft = ft, caption = caption))
}


#' Format headers for flextable
#' 
#' Applies formatting to flextable headers including italicizing 'n',
#' adding sample size counts from N row data, and bolding all headers.
#' 
#' @param ft \pkg{flextable} object.
#' @param has_n_row Logical whether source data had an N row.
#' @param n_row_data Data from the N row for adding counts to headers.
#' @return Formatted \pkg{flextable} object.
#' @keywords internal
format_headers_ft <- function(ft, has_n_row, n_row_data) {
    col_names <- names(ft$body$dataset)
    
    for (i in seq_along(col_names)) {
        col <- col_names[i]
        
        ## Skip Variable column for N count addition
        if (col == "Variable") {
            ## Just keep the original label without adding (N = X)
            next
        }
        
        ## Italicize 'n' column header if present
        if (col == "n") {
            ft <- flextable::italic(ft, j = i, part = "header")
        }
        
        ## Add N counts for data columns only (not Variable)
        if (has_n_row && col %chin% names(n_row_data) && col != "Variable") {
            n_val <- n_row_data[[col]]
            if (!is.na(n_val) && n_val != "" && n_val != "0") {
                ## Use compose to italicize just the N
                ft <- flextable::compose(
                                     ft, 
                                     j = i, 
                                     part = "header",
                                     value = flextable::as_paragraph(
                                                            col, "\n(", 
                                                            flextable::as_i("N"), 
                                                            " = ", n_val, ")"
                                                        )
                                 )
            }
        }
    }
    
    ## Bold all headers
    ft <- flextable::bold(ft, part = "header")
    
    return(ft)
}


#' Bold significant \emph{p}-values in flextable outputs
#' 
#' Applies bold formatting to significant \emph{p}-values in \pkg{flextable} objects
#' by detecting values below threshold or "< 0.001" patterns.
#' 
#' @param ft \pkg{flextable} object.
#' @param df The source data.table.
#' @param p_threshold Numeric \emph{p}-value threshold for significance.
#' @return Flextable object with significant \emph{p}-values bolded.
#' @keywords internal
bold_pvalues_ft <- function(ft, df, p_threshold = 0.05) {
    p_cols <- grep("p-value|p value|Uni p|Multi p|pvalue", names(df), 
                   ignore.case = TRUE, value = TRUE)
    
    if (length(p_cols) == 0) return(ft)
    
    for (p_col in p_cols) {
        if (p_col %in% names(df)) {
            vals <- df[[p_col]]
            
            ## Significance check
            is_very_small <- grepl("^<\\s*0\\.001", vals)
            p_numeric <- suppressWarnings(as.numeric(gsub("[^0-9.]", "", vals)))
            is_small_numeric <- !is.na(p_numeric) & p_numeric < p_threshold
            
            sig_rows <- which((is_very_small | is_small_numeric) & 
                              vals != "" & !is.na(vals))
            
            ## Bulk bold operation
            if (length(sig_rows) > 0) {
                ft <- flextable::bold(ft, i = sig_rows, j = p_col, part = "body")
            }
        }
    }
    
    return(ft)
}


#' Apply zebra stripes with proper variable group detection for indented tables
#' 
#' Applies alternating background colors to variable groups in \pkg{flextable} objects.
#' Handles both indented tables (detects groups by leading whitespace) and
#' non-indented tables (uses pre-identified groups).
#' 
#' @param ft \pkg{flextable} object.
#' @param df The source data.table used to create the flextable.
#' @param var_groups List of row index vectors for variable groups.
#' @return Flextable object with zebra stripe formatting applied.
#' @keywords internal
apply_zebra_stripes_ft <- function(ft, df, var_groups) {
    ## Check if table has been indented (look for leading spaces in Variable column)
    is_indented <- any(grepl("^\\s{2,}", df$Variable))
    
    if (is_indented) {
        ## For indented tables, identify variable groups by non-indented rows
        var_starts <- which(!grepl("^\\s", df$Variable) & df$Variable != "")
        
        for (i in seq_along(var_starts)) {
            start_row <- var_starts[i]
            end_row <- if (i < length(var_starts)) {
                           var_starts[i + 1] - 1
                       } else {
                           nrow(df)
                       }
            
            if (i %% 2 == 1) {  ## Odd variable groups get gray shading
                ft <- flextable::bg(ft, i = start_row:end_row, 
                                    bg = "#EEEEEE", part = "body")
            } else {  ## Even variable groups get white background
                ft <- flextable::bg(ft, i = start_row:end_row, 
                                    bg = "#FFFFFF", part = "body")
            }
        }
    } else if (!is.null(var_groups)) {
        ## Use pre-identified groups for non-indented tables
        for (i in seq_along(var_groups)) {
            rows <- var_groups[[i]]
            rows <- rows[rows <= nrow(df)]
            if (length(rows) > 0) {
                if (i %% 2 == 1) {  ## Odd variable groups get gray shading
                    ft <- flextable::bg(ft, i = rows, bg = "#EEEEEE", part = "body")
                } else {  ## Even variable groups get white background
                    ft <- flextable::bg(ft, i = rows, bg = "#FFFFFF", part = "body")
                }
            }
        }
    } else {
        ## Fallback to row-based striping
        odd_rows <- seq(1, nrow(df), 2)
        even_rows <- seq(2, nrow(df), 2)
        ft <- flextable::bg(ft, i = odd_rows, bg = "#EEEEEE", part = "body")
        ft <- flextable::bg(ft, i = even_rows, bg = "#FFFFFF", part = "body")
    }
    
    return(ft)
}

### * Main function

#' Multivariate Regression Analysis
#'
#' Performs regression analyses of a single predictor (exposure) across multiple 
#' outcomes. This function is designed for studies where a single exposure variable 
#' is tested against multiple endpoints, such as complication screening, biomarker 
#' associations, or phenome-wide association studies. Returns publication-ready 
#' formatted results with optional covariate adjustment. Supports interactions,
#' mixed-effects models, stratification, and clustered standard errors.
#'
#' @param data Data frame or data.table containing the analysis dataset. The 
#'   function automatically converts data frames to data.tables for efficient 
#'   processing.
#'   
#' @param outcomes Character vector of outcome variable names to analyze. Each 
#'   outcome is tested in its own model with the predictor. For time-to-event
#'   analysis, use \code{Surv()} syntax for the outcome variable
#'   (\emph{e.g.,} \code{c("Surv(time1, status1)", "Surv(time2, status2)")}).
#'   
#' @param predictor Character string specifying the predictor (exposure) variable 
#'   name. This variable is tested against each outcome. Can be continuous or 
#'   categorical (factor).
#'   
#' @param covariates Optional character vector of covariate variable names to 
#'   include in adjusted models. When specified, models are fit as 
#'   \code{outcome ~ predictor + covariate1 + covariate2 + ...}, and only the 
#'   predictor effect is reported. Default is \code{NULL} (unadjusted models).
#'
#' @param interactions Optional character vector of interaction terms to include
#'   in adjusted models, using colon notation (\emph{e.g.,} \code{c("predictor:sex")}).
#'   Interactions involving the predictor will have their effects extracted and
#'   reported. Default is \code{NULL}.
#'
#' @param random Optional character string specifying random effects formula for
#'   mixed effects models (\emph{e.g.,} \code{"(1|hospital)"} or \code{"(1|site/patient)"}).
#'   Required when \code{model_type} is \code{"glmer"}, \code{"lmer"}, or 
#'   \code{"coxme"} unless random effects are included in the \code{covariates}
#'   vector. Alternatively, random effects can be included directly in the 
#'   \code{covariates} vector using the same syntax (\emph{e.g.,} 
#'   \code{covariates = c("age", "sex", "(1|site)")}). Default is \code{NULL}.
#'
#' @param strata Optional character string naming the stratification variable for
#'   Cox or conditional logistic models. Creates separate baseline hazards for 
#'   each stratum. Default is \code{NULL}.
#'
#' @param cluster Optional character string naming the clustering variable for
#'   Cox models. Computes robust clustered standard errors. Default is \code{NULL}.
#'   
#' @param model_type Character string specifying the type of regression model to 
#'   fit. Options include:
#'   \itemize{
#'     \item \code{"glm"} - Generalized linear model (default). Supports multiple 
#'       distributions via the \code{family} parameter including logistic, Poisson, 
#'       Gamma, Gaussian, and quasi-likelihood models.
#'     \item \code{"lm"} - Linear regression for continuous outcomes with normally 
#'       distributed errors.
#'     \item \code{"coxph"} - Cox proportional hazards model for time-to-event 
#'       survival analysis. Requires \code{Surv()} outcome syntax.
#'     \item \code{"clogit"} - Conditional logistic regression for matched 
#'       case-control studies.
#'     \item \code{"negbin"} - Negative binomial regression for overdispersed count 
#'       data (requires MASS package). Estimates an additional dispersion parameter 
#'       compared to Poisson regression.
#'     \item \code{"glmer"} - Generalized linear mixed-effects model for hierarchical 
#'       or clustered data with non-normal outcomes (requires \pkg{lme4} package and 
#'       \code{random} parameter).
#'     \item \code{"lmer"} - Linear mixed-effects model for hierarchical or clustered 
#'       data with continuous outcomes (requires \pkg{lme4} package and \code{random} 
#'       parameter).
#'     \item \code{"coxme"} - Cox mixed-effects model for clustered survival data 
#'       (requires coxme package and \code{random} parameter).
#'   }
#'   
#' @param family For GLM and GLMER models, specifies the error distribution and link 
#'   function. Can be a character string, a family function, or a family object.
#'   Ignored for non-GLM/GLMER models.
#'   
#'   \strong{Binary/Binomial outcomes:}
#'   \itemize{
#'     \item \code{"binomial"} or \code{binomial()} - Logistic regression for binary 
#'       outcomes (0/1, TRUE/FALSE). Returns odds ratios (OR). Default.
#'     \item \code{"quasibinomial"} or \code{quasibinomial()} - Logistic regression 
#'       with overdispersion. Use when residual deviance >> residual df.
#'     \item \code{binomial(link = "probit")} - Probit regression (normal CDF link).
#'     \item \code{binomial(link = "cloglog")} - Complementary log-log link for 
#'       asymmetric binary outcomes.
#'   }
#'   
#'   \strong{Count outcomes:}
#'   \itemize{
#'     \item \code{"poisson"} or \code{poisson()} - Poisson regression for count 
#'       data. Returns rate ratios (RR). Assumes mean = variance.
#'     \item \code{"quasipoisson"} or \code{quasipoisson()} - Poisson regression 
#'       with overdispersion. Use when variance > mean.
#'   }
#'   
#'   \strong{Continuous outcomes:}
#'   \itemize{
#'     \item \code{"gaussian"} or \code{gaussian()} - Normal/Gaussian distribution 
#'       for continuous outcomes. Equivalent to linear regression.
#'     \item \code{gaussian(link = "log")} - Log-linear model for positive continuous 
#'       outcomes. Returns multiplicative effects.
#'   }
#'   
#'   \strong{Positive continuous outcomes:}
#'   \itemize{
#'     \item \code{"Gamma"} or \code{Gamma()} - Gamma distribution for positive, 
#'       right-skewed continuous data (\emph{e.g.,} costs, lengths of stay). When
#'       passed as a string, resolves to log link for interpretable multiplicative
#'       effects.
#'     \item \code{Gamma(link = "inverse")} - Gamma with inverse (canonical) link.
#'     \item \code{Gamma(link = "identity")} - Gamma with identity link for additive 
#'       effects on positive outcomes.
#'     \item \code{"inverse.gaussian"} or \code{inverse.gaussian()} - Inverse Gaussian 
#'       for positive, highly right-skewed data.
#'   }
#'   
#'   For negative binomial regression (overdispersed counts), use 
#'   \code{model_type = "negbin"} instead of the \code{family} parameter.
#'   
#'   See \code{\link[stats]{family}} for additional details and options.
#'   
#' @param columns Character string specifying which result columns to display when 
#'   both unadjusted and adjusted models are fit (\emph{i.e.,} when \code{covariates} is 
#'   specified):
#'   \itemize{
#'     \item \code{"adjusted"} - Show only adjusted (covariate-controlled) results [default]
#'     \item \code{"unadjusted"} - Show only unadjusted (crude) results
#'     \item \code{"both"} - Show both unadjusted and adjusted results side-by-side
#'   }
#'   Ignored when \code{covariates = NULL}.
#'   
#' @param p_threshold Numeric value between 0 and 1 specifying a \emph{p}-value threshold 
#'   for filtering results. Only outcomes with \emph{p}-value less than or equal to the
#'   threshold are included in the output. Default is 1 (no filtering; all outcomes returned).
#'   
#' @param conf_level Numeric confidence level for confidence intervals. Must be 
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals).
#'   
#' @param show_n Logical. If \code{TRUE}, includes the sample size column in 
#'   the output table. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes the events column in the 
#'   output table (relevant for survival and logistic regression). Default is 
#'   \code{TRUE}.
#'   
#' @param digits Integer specifying the number of decimal places for effect 
#'   estimates (OR, HR, RR, coefficients). Default is 2.
#'   
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param labels Named character vector or list providing custom display 
#'   labels for variables. Can include labels for outcomes, predictors, and 
#'   covariates. Names should match variable names, values are the display labels.
#'   Labels are applied to: (1) outcome names in the Outcome column, (2) predictor
#'   variable name when displayed, and (3) variable names in formatted interaction
#'   terms. Variables not in \code{labels} use their original names. 
#'   Default is \code{NULL}.
#'   
#' @param predictor_label Optional character string providing a custom display 
#'   label for the predictor variable. Takes precedence over \code{labels} for
#'   the predictor. Default is \code{NULL} (uses label from \code{labels} or 
#'   original name).
#'
#' @param include_predictor Logical. If \code{TRUE} (default), includes the 
#'   predictor column in the output table showing which level of a factor 
#'   predictor is being compared. If \code{FALSE}, omits the predictor column,
#'   which may be useful when the predictor information will be explained in 
#'   a table caption or figure legend.
#'   
#' @param keep_models Logical. If \code{TRUE}, stores all fitted model objects 
#'   in the output as an attribute. Models are accessible via 
#'   \code{attr(result, "models")}. Default is \code{FALSE}.
#'   
#' @param exponentiate Logical. Whether to exponentiate coefficients (display
#'     OR/HR/RR instead of log odds/log hazards). Default is \code{NULL}, which
#'     automatically displays raw coefficients for linear models and
#'     exponentiates for logistic, Poisson, and Cox models.
#'   
#' @param conf_method Character string controlling the confidence interval method.
#'   If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
#'   \itemize{
#'     \item \code{"profile"} - Profile likelihood intervals for GLM and negative
#'       binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
#'       intervals for linear models. Falls back to Wald on profiling failure.
#'       Quasi-likelihood families always use Wald (no true likelihood).
#'     \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
#'       \eqn{\times} SE) for all model types. Faster but less accurate near
#'       boundary conditions or with small subgroups.
#'   }
#'   Cox and mixed-effects models use Wald intervals regardless of this setting.
#'   Set globally with \code{options(summata.conf_method = "wald")} to use Wald
#'   throughout a session.
#'   
#' @param parallel Logical. If \code{TRUE} (default), fits models in parallel
#'   for improved performance with many outcomes.
#'   
#' @param n_cores Integer specifying the number of CPU cores to use for
#'   parallel processing. Default is \code{NULL} (auto-detect: uses
#'   \code{detectCores() - 1}). Ignored when \code{parallel = FALSE}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param verbose Logical. If \code{TRUE}, displays model fitting warnings
#'   (\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
#'   routine fitting messages are suppressed while unexpected warnings are
#'   preserved. When \code{NULL}, uses
#'   \code{getOption("summata.verbose", FALSE)}.
#'
#' @param ... Additional arguments passed to the underlying model fitting functions.
#'
#' @return A data.table with S3 class \code{"multifit_result"} containing formatted 
#'   multivariate regression results. The table structure includes:
#'   \describe{
#'     \item{Outcome}{Character. Outcome variable name or custom label}
#'     \item{Predictor}{Character. For factor predictors: formatted as 
#'       "Variable (Level)" showing the level being compared to reference.
#'       For binary variables where the non-reference level is an affirmative
#'       value (Yes, 1, True, Present, Positive, +), shows just "Variable".
#'       For continuous predictors: the variable name. For interactions: the
#'       formatted interaction term (\emph{e.g.,} "Treatment (Drug A) × Sex (Male)")}
#'     \item{n}{Integer. Sample size used in fitting (if \code{show_n = TRUE}).
#'       When both unadjusted and adjusted estimates are shown, this is the
#'       adjusted model's sample, which is the one behind the adjusted
#'       estimate on the same row}
#'     \item{Events}{Integer. Number of events (if \code{show_events = TRUE})}
#'     \item{OR/HR/RR/Coefficient (95\% CI)}{Character. Unadjusted effect 
#'       estimate with CI (if \code{columns = "unadjusted"} or \code{"both"})}
#'     \item{aOR/aHR/aRR/Adj. Coefficient (95\% CI)}{Character. Adjusted 
#'       effect estimate with CI (if \code{columns = "adjusted"} or \code{"both"})}
#'     \item{Uni \emph{p} / Multi \emph{p} / \emph{p}-value}{Character. Formatted \emph{p}-value(s). Column 
#'       names depend on \code{columns} setting}
#'   }
#'   
#'   The returned object includes the following attributes accessible via \code{attr()}:
#'   \describe{
#'     \item{raw_data}{data.table. Unformatted numeric results with separate 
#'       columns for effect estimates, standard errors, confidence intervals, 
#'       and \emph{p}-values. Suitable for custom analysis or visualization}
#'     \item{models}{list (if \code{keep_models = TRUE}). Named list of fitted 
#'       model objects, with outcome names as list names. Each element contains
#'       \code{$unadjusted} and/or \code{$adjusted} models depending on settings}
#'     \item{predictor}{Character. The predictor variable name}
#'     \item{outcomes}{Character vector. The outcome variable names}
#'     \item{covariates}{Character vector or \code{NULL}. The covariate variable names}
#'     \item{interactions}{Character vector or \code{NULL}. The interaction terms}
#'     \item{random}{Character or \code{NULL}. The random effects formula}
#'     \item{strata}{Character or \code{NULL}. The stratification variable}
#'     \item{cluster}{Character or \code{NULL}. The clustering variable}
#'     \item{model_type}{Character. The regression model type used}
#'     \item{columns}{Character. Which columns were displayed}
#'     \item{analysis_type}{Character. \code{"multi_outcome"} to identify analysis type}
#'     \item{significant}{Character vector. Names of outcomes with \emph{p} < 0.05
#'       for the predictor (uses adjusted \emph{p}-values when available)}
#'     \item{analysis_counts}{List. How much of the supplied data reached the
#'       models, with elements \code{n_supplied}, \code{n_analyzed},
#'       \code{n_missing_outcome} and \code{n_missing_predictor}. One model is
#'       fitted per outcome, so \code{n_analyzed} holds one value per outcome
#'       and the reported sample may be a range. Event counts are not
#'       included: each outcome has its own event total, so a single reported
#'       figure could not describe them all. The \code{print()} method always
#'       reports the analyzed sample}
#'   }
#'
#' @details
#' \strong{Analysis Approach:}
#' 
#' The function implements a multivariate (multi-outcome) screening workflow that
#' inverts the typical regression paradigm:
#' \enumerate{
#'   \item For each outcome in \code{outcomes}, fits a separate model with the 
#'     predictor as the main exposure
#'   \item If \code{covariates} specified, fits adjusted model: 
#'     \code{outcome ~ predictor + covariates + interactions}
#'   \item Extracts only the predictor effect(s) from each model, ignoring 
#'     covariate coefficients
#'   \item Combines results into a single table for comparison across outcomes
#'   \item Optionally filters by \emph{p}-value threshold
#' }
#' 
#' This is conceptually opposite to \code{uniscreen()}, which tests multiple 
#' predictors against a single outcome. \code{multifit()} applies where there is one 
#' exposure of interest and want to screen across multiple endpoints.
#' 
#' \strong{When to Use Multivariate Regression Analysis:}
#' \itemize{
#'   \item \strong{Complication screening}: Test one exposure (\emph{e.g.,} operative time, 
#'     BMI, biomarker level) against multiple postoperative complications
#'   \item \strong{Treatment effects}: Test one treatment against multiple efficacy 
#'     and safety endpoints simultaneously
#'   \item \strong{Biomarker studies}: Test one biomarker against multiple clinical 
#'     outcomes to understand its prognostic value
#'   \item \strong{Phenome-wide association studies (PheWAS)}: Test genetic variants 
#'     or exposures against many phenotypes
#'   \item \strong{Risk factor profiling}: Understand how one risk factor relates 
#'     to a spectrum of outcomes
#' }
#' 
#' \strong{Handling Categorical Predictors:}
#' 
#' When the predictor is a factor variable with multiple levels:
#' \itemize{
#'   \item Each non-reference level gets its own row for each outcome
#'   \item Reference category is determined by factor level ordering
#'   \item The Predictor column shows "Variable (Level)" format 
#'     (\emph{e.g.,} "Treatment (Drug A)", "Treatment (Drug B)")
#'   \item For binary variables with affirmative non-reference levels 
#'     (Yes, 1, True, Present, Positive, +), shows just "Variable" 
#'     (\emph{e.g.,} "Diabetes" instead of "Diabetes (Yes)")
#'   \item Effect estimates compare each level to the reference
#' }
#' 
#' \strong{Adjusted vs. Unadjusted Results:}
#' 
#' When \code{covariates} is specified, the function fits both models but only 
#' extracts predictor effects:
#' \itemize{
#'   \item \code{columns = "adjusted"}: Reports only covariate-adjusted effects.
#'     Column labeled "aOR/aHR," \emph{etc.}
#'   \item \code{columns = "unadjusted"}: Reports only crude effects. Column 
#'     labeled "OR/HR," \emph{etc.}
#'   \item \code{columns = "both"}: Reports both side-by-side. Useful for 
#'     identifying confounding (large change in effect) or independent effects 
#'     (similar estimates)
#' }
#' 
#' \strong{Interaction Terms:}
#' 
#' When \code{interactions} includes terms involving the predictor:
#' \itemize{
#'   \item Main effect of predictor is always reported
#'   \item Interaction effects are extracted and displayed with formatted names
#'   \item Format: \code{Variable (Level) × Variable (Level)} using multiplication sign notation
#'   \item Useful for testing effect modification (\emph{e.g.,} does treatment effect 
#'     differ by sex?)
#' }
#' 
#' \strong{Mixed-Effects Models:}
#' 
#' For clustered or hierarchical data (\emph{e.g.,} patients within hospitals):
#' \itemize{
#'   \item Use \code{model_type = "glmer"} with \code{random = "(1|cluster)"} for 
#'     random intercept models
#'   \item Nested random effects: \code{random = "(1|site/patient)"}
#'   \item Crossed random effects: \code{random = "(1|site) + (1|doctor)"}
#'   \item For survival outcomes, use \code{model_type = "coxme"}
#' }
#' 
#' \strong{Stratification and Clustering (Cox models):}
#' 
#' For Cox proportional hazards models:
#' \itemize{
#'   \item \code{strata}: Creates separate baseline hazards for each stratum level.
#'     Use when hazards are non-proportional across strata but stratum effects do
#'     not need to be estimated
#'   \item \code{cluster}: Computes robust (sandwich) standard errors accounting 
#'     for within-cluster correlation. Alternative to mixed effects when only 
#'     robust SEs are needed
#' }
#' 
#' \strong{Filtering based on \emph{p}-value:}
#' 
#' The \code{p_threshold} parameter filters results after fitting all models:
#' \itemize{
#'   \item Only outcomes with \emph{p} less than or equal to the threshold are retained in output
#'   \item For factor predictors, outcome is kept if any level is significant
#'   \item Useful for focusing on significant associations in exploratory analyses
#'   \item Default is 1 (no filtering) - recommended for confirmatory analyses
#' }
#' 
#' \strong{Outcome Homogeneity:}
#' 
#' All outcomes in a single \code{multifit()} call should be of the same type
#' (all binary, all continuous, or all survival). Mixing outcome types produces
#' tables with incompatible effect measures (\emph{e.g.,} odds ratios alongside regression
#' coefficients), which can mislead readers. The function validates outcome
#' compatibility and issues a warning when mixed types are detected.
#' 
#' For analyses involving multiple outcome types, run separate \code{multifit()}
#' calls for each type:
#' 
#' \preformatted{
#' # Binary outcomes
#' binary_results <- multifit(data, outcomes = c("death", "readmission"),
#'                            predictor = "treatment", model_type = "glm")
#' 
#' # Continuous outcomes
#' continuous_results <- multifit(data, outcomes = c("los_days", "cost"),
#'                                predictor = "treatment", model_type = "lm")
#' }
#' 
#' \strong{Effect Measures by Model Type:}
#' \itemize{
#'   \item \strong{Logistic} (\code{model_type = "glm"}, \code{family = "binomial"}): 
#'     Odds ratios (OR/aOR)
#'   \item \strong{Cox} (\code{model_type = "coxph"}): Hazard ratios (HR/aHR)
#'   \item \strong{Poisson} (\code{model_type = "glm"}, \code{family = "poisson"}): 
#'     Rate ratios (RR/aRR)
#'   \item \strong{Linear} (\code{model_type = "lm"}): Coefficient estimates
#'   \item \strong{Mixed effects}: Same as fixed-effects counterparts
#' }
#' 
#' \strong{Memory and Performance:}
#' 
#' \itemize{
#'   \item \code{parallel = TRUE} (default) uses multiple cores for faster fitting
#'   \item \code{keep_models = FALSE} (default) discards model objects to save memory
#'   \item For many outcomes, parallel processing provides substantial speedup
#'   \item Set \code{keep_models = TRUE} only when model diagnostics are required
#' }
#'
#' @seealso 
#' \code{\link{uniscreen}} for screening multiple predictors against one outcome,
#' \code{\link{multiforest}} for creating forest plots from multifit results,
#' \code{\link{fit}} for single-outcome regression with full coefficient output,
#' \code{\link{fullfit}} for complete univariable-to-multivariable workflow,
#' \code{\link{forestsave}} for saving the forest plot,
#' \code{\link{tablesave}} for exporting tables
#'
#' @examples
#' # Load example data
#' data(clintrial)
#' data(clintrial_labels)
#' 
#' # Example 1: Basic multivariate analysis (unadjusted)
#' result1 <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result1)
#'
#' \donttest{
#' 
#' # Example 2: Adjusted analysis with covariates
#' result2 <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result2)
#' 
#' # Example 3: Compare unadjusted and adjusted results
#' result3 <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     columns = "both",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result3)
#' 
#' # Example 4: Continuous predictor across outcomes
#' result4 <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "age",
#'     covariates = c("sex", "treatment", "stage"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result4)
#' 
#' # Example 5: Cox regression for survival outcomes
#' library(survival)
#' cox_result <- multifit(
#'     data = clintrial,
#'     outcomes = c("Surv(pfs_months, pfs_status)", 
#'                  "Surv(os_months, os_status)"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     model_type = "coxph",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(cox_result)
#' 
#' # Example 6: Cox with stratification by site
#' cox_strat <- multifit(
#'     data = clintrial,
#'     outcomes = c("Surv(os_months, os_status)"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex"),
#'     strata = "site",
#'     model_type = "coxph",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(cox_strat)
#' 
#' # Example 7: Cox with clustered standard errors
#' cox_cluster <- multifit(
#'     data = clintrial,
#'     outcomes = c("Surv(os_months, os_status)"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     cluster = "site",
#'     model_type = "coxph",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(cox_cluster)
#' 
#' # Example 8: Interaction between predictor and covariate
#' result_int <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     interactions = c("treatment:sex"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result_int)
#' 
#' # Example 9: Linear model for continuous outcomes
#' linear_result <- multifit(
#'     data = clintrial,
#'     outcomes = c("los_days", "biomarker_x"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex"),
#'     model_type = "lm",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(linear_result)
#' 
#' # Example 10: Poisson regression for equidispersed count outcomes
#' # fu_count has variance ~= mean, appropriate for standard Poisson
#' poisson_result <- multifit(
#'     data = clintrial,
#'     outcomes = c("fu_count"),
#'     predictor = "treatment",
#'     covariates = c("age", "stage"),
#'     model_type = "glm",
#'     family = "poisson",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(poisson_result)
#' 
#' # Example 11: Filter to significant results only
#' sig_results <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "stage",
#'     p_threshold = 0.05,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(sig_results)
#' 
#' # Example 12: Custom outcome labels
#' result_labeled <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     labels = c(
#'         surgery = "Surgical Resection",
#'         pfs_status = "Disease Progression",
#'         os_status = "Death",
#'         treatment = "Treatment Group"
#'     ),
#'     parallel = FALSE
#' )
#' print(result_labeled)
#' 
#' # Example 13: Keep models for diagnostics
#' result_models <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex"),
#'     keep_models = TRUE,
#'     parallel = FALSE
#' )
#' 
#' # Access stored models
#' models <- attr(result_models, "models")
#' names(models)
#' 
#' # Get adjusted model for surgery outcome
#' surgery_model <- models$surgery$adjusted
#' summary(surgery_model)
#' 
#' # Example 14: Access raw numeric data
#' result <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "os_status"),
#'     predictor = "age",
#'     parallel = FALSE
#' )
#' 
#' # Get unformatted results for custom analysis
#' raw_data <- attr(result, "raw_data")
#' print(raw_data)
#' 
#' # Example 15: Hide sample size and event columns
#' result_minimal <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "os_status"),
#'     predictor = "treatment",
#'     show_n = FALSE,
#'     show_events = FALSE,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result_minimal)
#' 
#' # Example 16: Customize decimal places
#' result_digits <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "os_status"),
#'     predictor = "age",
#'     digits = 3,
#'     p_digits = 4,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result_digits)
#' 
#' # Example 17: Force coefficient display (no exponentiation)
#' result_coef <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery"),
#'     predictor = "age",
#'     exponentiate = FALSE,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(result_coef)
#' 
#' # Example 18: Complete publication workflow
#' final_table <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage", "grade"),
#'     columns = "both",
#'     labels = clintrial_labels,
#'     digits = 2,
#'     p_digits = 3,
#'     parallel = FALSE
#' )
#' print(final_table)
#' 
#' # Example 19: Gamma regression for positive continuous outcomes
#' gamma_result <- multifit(
#'     data = clintrial,
#'     outcomes = c("los_days", "recovery_days"),
#'     predictor = "treatment",
#'     covariates = c("age", "surgery"),
#'     model_type = "glm",
#'     family = Gamma(link = "log"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(gamma_result)
#' 
#' # Example 20: Quasipoisson for overdispersed counts
#' quasi_result <- multifit(
#'     data = clintrial,
#'     outcomes = c("ae_count"),
#'     predictor = "treatment",
#'     covariates = c("age", "diabetes"),
#'     model_type = "glm",
#'     family = "quasipoisson",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(quasi_result)
#' 
#' # Example 21: Generalized linear mixed effects (GLMER)
#' # Test treatment across outcomes with site clustering
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     glmer_result <- suppressWarnings(multifit(
#'         data = clintrial,
#'         outcomes = c("surgery", "pfs_status", "os_status"),
#'         predictor = "treatment",
#'         covariates = c("age", "sex"),
#'         random = "(1|site)",
#'         model_type = "glmer",
#'         family = "binomial",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     ))
#'     print(glmer_result)
#' }
#' 
#' # Example 22: Cox mixed effects with random site effects
#' if (requireNamespace("coxme", quietly = TRUE)) {
#'     coxme_result <- multifit(
#'         data = clintrial,
#'         outcomes = c("Surv(pfs_months, pfs_status)", 
#'                      "Surv(os_months, os_status)"),
#'         predictor = "treatment",
#'         covariates = c("age", "sex", "stage"),
#'         random = "(1|site)",
#'         model_type = "coxme",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     )
#'     print(coxme_result)
#' }
#' 
#' # Example 23: Multiple interactions across outcomes
#' multi_int <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     interactions = c("treatment:stage", "treatment:sex"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(multi_int)
#'
#' }
#' 
#' @family regression functions
#' @export
multifit <- function(data,
                     outcomes,
                     predictor,
                     covariates = NULL,
                     interactions = NULL,
                     random = NULL,
                     strata = NULL,
                     cluster = NULL,
                     model_type = "glm",
                     family = "binomial",
                     columns = "adjusted",
                     p_threshold = 1,
                     conf_level = 0.95,
                     show_n = TRUE,
                     show_events = TRUE,
                     digits = 2,
                     p_digits = 3,
                     labels = NULL,
                     predictor_label = NULL,
                     include_predictor = TRUE,
                     keep_models = FALSE,
                     exponentiate = NULL,
                     conf_method = NULL,
                     parallel = TRUE,
                     n_cores = NULL,
                     number_format = NULL,
                     verbose = NULL,
                     ...) {
    
    ## Validate inputs
    if (missing(data)) stop("'data' is required")
    if (missing(outcomes)) stop("'outcomes' is required")
    if (missing(predictor)) stop("'predictor' is required")
    if (length(predictor) != 1) stop("'predictor' must be a single variable name")
    if (length(outcomes) < 1) stop("'outcomes' must contain at least one outcome")
    
    ## Validate columns parameter
    columns <- match.arg(columns, c("adjusted", "unadjusted", "both"))
    
    ## Resolve verbose setting
    verbose <- if (is.null(verbose)) getOption("summata.verbose", FALSE) else verbose
    
    ## Resolve Gamma family string to log link (see fit.R for rationale)
    if (model_type %in% c("glm", "glmer") && is.character(family) &&
        tolower(family) == "gamma") {
        family <- stats::Gamma(link = "log")
    }

    ## Validate model_type for mixed effects
    mixed_types <- c("glmer", "lmer", "coxme")
    
    ## Check for random effects embedded in covariates
    has_random_in_covariates <- !is.null(covariates) && any(grepl("\\|", covariates))
    
    if (!is.null(random) && !(model_type %in% mixed_types)) {
        warning("'random' parameter is ignored for non-mixed-effects models (", model_type, ").")
        random <- NULL
    }
    
    ## Extract random effects from covariates if present (for mixed-effects models)
    random_in_covs <- NULL
    if (model_type %in% mixed_types && has_random_in_covariates) {
        random_mask <- grepl("\\|", covariates)
        random_in_covs <- covariates[random_mask]
        covariates <- covariates[!random_mask]
        if (length(covariates) == 0) covariates <- NULL
        
        ## Combine explicit random with embedded random (if both provided)
        if (!is.null(random)) {
            message("Note: Random effects specified in both 'random' parameter and 'covariates'. ",
                    "Combining: ", random, " + ", paste(random_in_covs, collapse = " + "))
            random <- paste(c(random, random_in_covs), collapse = " + ")
        } else {
            random <- paste(random_in_covs, collapse = " + ")
        }
    }
    
    ## Validate that random is provided for mixed-effects models
    if (model_type %in% mixed_types && is.null(random)) {
        stop("'random' parameter is required for mixed-effects models (", model_type, ").\n",
             "Example: random = \"(1|site)\" for random intercepts by site.\n",
             "Alternatively, include random effects in 'covariates' using the same syntax.")
    }
    
    ## Validate strata/cluster usage
    if (!is.null(strata) && !(model_type %in% c("coxph", "clogit"))) {
        stop("'strata' is only supported for coxph and clogit models")
    }
    if (!is.null(cluster) && model_type != "coxph") {
        stop("'cluster' is only supported for coxph models")
    }
    
    ## If no covariates/interactions/random, columns doesn't matter - treat as unadjusted
    has_adjustment <- !is.null(covariates) || !is.null(interactions) || !is.null(random)
    if (!has_adjustment) {
        columns <- "unadjusted"
    }
    
    ## Convert to data.table once at the start
    ## Use .data to avoid any potential conflicts with base::data
    .data <- if (data.table::is.data.table(data)) {
                 data.table::copy(data)
             } else {
                 data.table::as.data.table(data)
             }
    
    ## Validate outcome homogeneity
    validate_outcome_homogeneity(.data, outcomes, model_type, family)
    
    ## Determine if predictor is a factor (for handling multiple levels)
    is_factor_predictor <- is.factor(.data[[predictor]]) || 
        is.character(.data[[predictor]])
    
    ## Build the terms to extract from models
    ## Interactions involving the predictor are extracted as well
    terms_to_extract <- predictor
    if (!is.null(interactions)) {
        ## Find interactions that involve the predictor
        predictor_interactions <- interactions[grepl(predictor, interactions, fixed = TRUE)]
        terms_to_extract <- c(terms_to_extract, predictor_interactions)
    }
    
    ## Define model fitting function for a single outcome
    fit_one_outcome <- function(outcome) {
        
        results_list <- list()
        models_list <- list()
        
        ## Build unadjusted formula (predictor only, no interactions in univariable)
        unadj_formula_str <- paste(outcome, "~", predictor)
        
        ## Add strata for Cox models in unadjusted
        if (!is.null(strata) && model_type %in% c("coxph", "clogit")) {
            unadj_formula_str <- paste(unadj_formula_str, "+ strata(", strata, ")")
        }
        
        unadj_formula <- stats::as.formula(unadj_formula_str)
        
        ## Fit unadjusted model if needed
        if (columns %in% c("unadjusted", "both")) {
            unadj_model <- tryCatch({
                quiet_fit(quote(
                    switch(model_type,
                           "glm" = stats::glm(unadj_formula, data = .data, family = family,
                                              model = keep_models, x = FALSE, y = TRUE, ...),
                           "negbin" = {
                               if (!requireNamespace("MASS", quietly = TRUE))
                                   stop("Package 'MASS' required for negative binomial models")
                               MASS::glm.nb(unadj_formula, data = .data,
                                             model = keep_models, x = FALSE, y = TRUE, ...)
                           },
                           "lm" = stats::lm(unadj_formula, data = .data,
                                            model = keep_models, x = FALSE, y = TRUE, ...),
                           "coxph" = {
                               if (!requireNamespace("survival", quietly = TRUE)) 
                                   stop("Package 'survival' required for Cox models")
                               if (!is.null(cluster)) {
                                   survival::coxph(unadj_formula, data = .data, 
                                                   cluster = .data[[cluster]],
                                                   model = keep_models, x = FALSE, y = TRUE, ...)
                               } else {
                                   survival::coxph(unadj_formula, data = .data,
                                                   model = keep_models, x = FALSE, y = TRUE, ...)
                               }
                           },
                           "clogit" = {
                               if (!requireNamespace("survival", quietly = TRUE))
                                   stop("Package 'survival' required for conditional logistic regression")
                               survival::clogit(unadj_formula, data = .data, ...)
                           },
                           ## Mixed effects models - univariable still includes random effects
                           "glmer" = {
                               if (!requireNamespace("lme4", quietly = TRUE))
                                   stop("Package 'lme4' required for mixed effects models")
                               if (is.null(random)) stop("'random' is required for glmer models")
                               unadj_me_formula <- stats::as.formula(
                                                              paste(outcome, "~", predictor, "+", random)
                                                          )
                               lme4::glmer(unadj_me_formula, data = .data, family = family, ...)
                           },
                           "lmer" = {
                               if (!requireNamespace("lme4", quietly = TRUE))
                                   stop("Package 'lme4' required for mixed effects models")
                               if (is.null(random)) stop("'random' is required for lmer models")
                               unadj_me_formula <- stats::as.formula(
                                                              paste(outcome, "~", predictor, "+", random)
                                                          )
                               lme4::lmer(unadj_me_formula, data = .data, ...)
                           },
                           "coxme" = {
                               if (!requireNamespace("coxme", quietly = TRUE))
                                   stop("Package 'coxme' required for mixed effects Cox models")
                               if (is.null(random)) stop("'random' is required for coxme models")
                               unadj_me_formula <- stats::as.formula(
                                                              paste(outcome, "~", predictor, "+", random)
                                                          )
                               coxme::coxme(unadj_me_formula, data = .data, ...)
                           },
                           stop("Unsupported model type: ", model_type)
                           )
                ), verbose = verbose)
            }, error = function(e) {
                warning("Model failed for outcome '", outcome, "': ", e$message)
                return(NULL)
            })
            
            if (!is.null(unadj_model)) {
                ## Extract predictor rows from model
                unadj_raw <- extract_predictor_effects(
                    model = unadj_model,
                    predictor = predictor,
                    outcome = outcome,
                    conf_level = conf_level,
                    adjusted = FALSE,
                    terms_to_extract = predictor,
                    conf_method = conf_method
                )
                results_list$unadjusted <- unadj_raw
                
                if (keep_models) {
                    models_list$unadjusted <- unadj_model
                }
            }
        }
        
        ## Fit adjusted model if needed
        if (has_adjustment && columns %in% c("adjusted", "both")) {
            ## Build adjusted formula
            adj_terms <- predictor
            if (!is.null(covariates)) {
                adj_terms <- c(adj_terms, covariates)
            }
            if (!is.null(interactions)) {
                adj_terms <- c(adj_terms, interactions)
            }
            
            adj_formula_str <- paste(outcome, "~", paste(adj_terms, collapse = " + "))
            
            ## Add random effects for mixed models
            if (!is.null(random) && model_type %in% mixed_types) {
                adj_formula_str <- paste(adj_formula_str, "+", random)
            }
            
            ## Add strata for Cox models
            if (!is.null(strata) && model_type %in% c("coxph", "clogit")) {
                adj_formula_str <- paste(adj_formula_str, "+ strata(", strata, ")")
            }
            
            adj_formula <- stats::as.formula(adj_formula_str)
            
            adj_model <- tryCatch({
                quiet_fit(quote(
                    switch(model_type,
                           "glm" = stats::glm(adj_formula, data = .data, family = family,
                                              model = keep_models, x = FALSE, y = TRUE, ...),
                           "negbin" = {
                               if (!requireNamespace("MASS", quietly = TRUE))
                                   stop("Package 'MASS' required for negative binomial models")
                               MASS::glm.nb(adj_formula, data = .data,
                                             model = keep_models, x = FALSE, y = TRUE, ...)
                           },
                           "lm" = stats::lm(adj_formula, data = .data,
                                            model = keep_models, x = FALSE, y = TRUE, ...),
                           "coxph" = {
                               if (!requireNamespace("survival", quietly = TRUE)) 
                                   stop("Package 'survival' required for Cox models")
                               if (!is.null(cluster)) {
                                   survival::coxph(adj_formula, data = .data, 
                                                   cluster = .data[[cluster]],
                                                   model = keep_models, x = FALSE, y = TRUE, ...)
                               } else {
                                   survival::coxph(adj_formula, data = .data,
                                                   model = keep_models, x = FALSE, y = TRUE, ...)
                               }
                           },
                           "clogit" = {
                               if (!requireNamespace("survival", quietly = TRUE))
                                   stop("Package 'survival' required for conditional logistic regression")
                               survival::clogit(adj_formula, data = .data, ...)
                           },
                           "glmer" = {
                               if (!requireNamespace("lme4", quietly = TRUE))
                                   stop("Package 'lme4' required for mixed effects models")
                               lme4::glmer(adj_formula, data = .data, family = family, ...)
                           },
                           "lmer" = {
                               if (!requireNamespace("lme4", quietly = TRUE))
                                   stop("Package 'lme4' required for mixed effects models")
                               lme4::lmer(adj_formula, data = .data, ...)
                           },
                           "coxme" = {
                               if (!requireNamespace("coxme", quietly = TRUE))
                                   stop("Package 'coxme' required for mixed effects Cox models")
                               coxme::coxme(adj_formula, data = .data, ...)
                           },
                           stop("Unsupported model type: ", model_type)
                           )
                ), verbose = verbose)
            }, error = function(e) {
                warning("Adjusted model failed for outcome '", outcome, "': ", e$message)
                return(NULL)
            })
            
            if (!is.null(adj_model)) {
                adj_raw <- extract_predictor_effects(
                    model = adj_model,
                    predictor = predictor,
                    outcome = outcome,
                    conf_level = conf_level,
                    adjusted = TRUE,
                    terms_to_extract = terms_to_extract,
                    conf_method = conf_method
                )
                results_list$adjusted <- adj_raw
                
                if (keep_models) {
                    models_list$adjusted <- adj_model
                }
            }
        }
        
        list(
            results = results_list,
            models = if (keep_models) models_list else NULL
        )
    }
    
    ## Fit models (parallel or sequential)
    can_use_windows_parallel <- .Platform$OS.type == "windows" && 
        interactive() && 
        !nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", ""))
    
    use_parallel <- parallel && 
        length(outcomes) > 1 && 
        (.Platform$OS.type != "windows" || can_use_windows_parallel)
    
    if (use_parallel) {
        ## Determine number of cores
        ## Respect CRAN's 2-core limit during R CMD check
        if (is.null(n_cores)) {
            n_cores <- max(1L, parallel::detectCores() - 1L)
        }
        ## CRAN policy: respect _R_CHECK_LIMIT_CORES_ environment variable
        chk <- tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_", ""))
        if (nzchar(chk) && chk == "true") {
            n_cores <- min(n_cores, 2L)
        }
        
        if (.Platform$OS.type == "windows") {
            ## Windows: use parLapply with PSOCK cluster
            ## Only reaches here in interactive sessions with package installed
            cl <- parallel::makeCluster(n_cores)
            on.exit(parallel::stopCluster(cl), add = TRUE)
            
            ## Export required objects to workers
            parallel::clusterExport(cl, 
                                    varlist = c(".data", "predictor", "covariates", "interactions",
                                                "random", "strata", "cluster", "model_type", 
                                                "family", "conf_level", "columns", "keep_models",
                                                "has_adjustment", "mixed_types", "terms_to_extract",
                                                "extract_predictor_effects", "conf_method"),
                                    envir = environment()
                                    )
            
            ## Load required packages on workers
            parallel::clusterEvalQ(cl, {
                library(data.table)
                library(stats)
            })
            
            if (model_type %in% c("coxph", "clogit")) {
                parallel::clusterEvalQ(cl, library(survival))
            }
            if (model_type %in% c("glmer", "lmer")) {
                parallel::clusterEvalQ(cl, library(lme4))
            }
            if (model_type == "coxme") {
                parallel::clusterEvalQ(cl, library(coxme))
            }
            
            all_results <- parallel::parLapply(cl, outcomes, fit_one_outcome)
            
        } else {
            ## Unix/Mac: use mclapply (fork-based, shares memory)
            all_results <- parallel::mclapply(
                                         outcomes, 
                                         fit_one_outcome,
                                         mc.cores = n_cores
                                     )
        }
    } else {
        ## Sequential processing
        ## Used when: parallel=FALSE, single outcome, or Windows in non-interactive context
        all_results <- lapply(outcomes, fit_one_outcome)
    }
    
    names(all_results) <- outcomes
    
    ## Combine results based on columns parameter
    combined_raw <- combine_multifit_results(all_results, columns)
    
    ## Store models if requested
    if (keep_models) {
        models <- lapply(all_results, `[[`, "models")
        models <- models[!vapply(models, is.null, logical(1))]
    }
    
    ## Filter by p-value if requested
    if (p_threshold < 1 && nrow(combined_raw) > 0) {
        ## Determine which p-value column to use for filtering
        p_col <- if ("p_adj" %in% names(combined_raw)) "p_adj" else "p_value"
        combined_raw <- combined_raw[get(p_col) <= p_threshold | is.na(get(p_col))]
    }
    
    ## Format results for publication
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    formatted <- format_multifit_table(
        combined_raw,
        columns = columns,
        show_n = show_n,
        show_events = show_events,
        digits = digits,
        p_digits = p_digits,
        labels = labels,
        predictor_label = predictor_label,
        include_predictor = include_predictor,
        exponentiate = exponentiate,
        conf_level = conf_level,
        marks = marks
    )
    
    ## Attach attributes
    data.table::setattr(formatted, "raw_data", combined_raw)

    ## One adjusted model is fitted per outcome, so the analyzed sample is
    ## recorded per outcome and reported as a range where it varies.
    mf_adjusted_vars <- unique(c(predictor, covariates, random, strata, cluster))
    mf_adjusted_vars <- mf_adjusted_vars[!is.na(mf_adjusted_vars)]
    mf_counts <- lapply(outcomes, function(out) {
        get_analysis_counts(
            data,
            all.vars(stats::as.formula(paste("~", out))),
            all.vars(stats::as.formula(paste("~", paste(mf_adjusted_vars,
                                                        collapse = "+"))))
        )
    })
    mf_counts <- mf_counts[!vapply(mf_counts, is.null, logical(1))]
    if (length(mf_counts) > 0) {
        data.table::setattr(formatted, "analysis_counts", list(
            n_supplied = mf_counts[[1]]$n_supplied,
            n_analyzed = unlist(lapply(mf_counts, function(z) z$n_analyzed)),
            n_missing_outcome = unlist(lapply(mf_counts, function(z) z$n_missing_outcome)),
            n_missing_predictor = unlist(lapply(mf_counts, function(z) z$n_missing_predictor))
        ))
    }
    data.table::setattr(formatted, "number_marks", marks)
    ## Variable labels travel with the result so that the forest plot
    ## functions can apply the same labels without being given them again.
    ## An explicit labels argument to those functions still takes precedence.
    data.table::setattr(formatted, "labels", labels)
    
    if (keep_models) {
        data.table::setattr(formatted, "models", models)
    }
    
    data.table::setattr(formatted, "predictor", predictor)
    data.table::setattr(formatted, "outcomes", outcomes)
    data.table::setattr(formatted, "covariates", covariates)
    data.table::setattr(formatted, "interactions", interactions)
    data.table::setattr(formatted, "random", random)
    data.table::setattr(formatted, "strata", strata)
    data.table::setattr(formatted, "cluster", cluster)
    data.table::setattr(formatted, "model_type", model_type)
    data.table::setattr(formatted, "columns", columns)
    data.table::setattr(formatted, "include_predictor", include_predictor)
    data.table::setattr(formatted, "analysis_type", "multi_outcome")
    
    ## Extract significant outcome names (p < 0.05)
    if (nrow(combined_raw) > 0 && "outcome" %in% names(combined_raw)) {
        p_col <- if ("p_adj" %in% names(combined_raw)) "p_adj" else "p_value"
        if (p_col %in% names(combined_raw)) {
            sig_outcomes <- unique(
                combined_raw[!is.na(get(p_col)) & get(p_col) < 0.05]$outcome
            )
            data.table::setattr(formatted, "significant", sig_outcomes)
        }
    }
    
    class(formatted) <- c("multifit_result", class(formatted))
    
    return(formatted)
}


#' Print method for multifit results
#'
#' @param x Object of class \code{multifit_result}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @family regression functions
#' @export
print.multifit_result <- function(x, ...) {
    cat("\nMultivariate Regression Results\n\n")
    cat("Predictor: ", attr(x, "predictor"), "\n", sep = "")
    cat("Outcomes: ", length(attr(x, "outcomes")), "\n", sep = "")
    cat("Model Type: ", attr(x, "model_type"), "\n", sep = "")
    
    covariates <- attr(x, "covariates")
    if (!is.null(covariates)) {
        cat("Covariates: ", paste(covariates, collapse = ", "), "\n", sep = "")
    }
    
    interactions <- attr(x, "interactions")
    if (!is.null(interactions)) {
        cat("Interactions: ", paste(interactions, collapse = ", "), "\n", sep = "")
    }
    
    random <- attr(x, "random")
    if (!is.null(random)) {
        cat("Random Effects: ", random, "\n", sep = "")
    }
    
    strata <- attr(x, "strata")
    if (!is.null(strata)) {
        cat("Strata: ", strata, "\n", sep = "")
    }
    
    cluster <- attr(x, "cluster")
    if (!is.null(cluster)) {
        cat("Cluster: ", cluster, "\n", sep = "")
    }
    
    cat("Display: ", attr(x, "columns"), "\n", sep = "")
    
    ## Always reported, so that a complete sample is stated rather than left
    ## to be distinguished from an absent disclosure. One model is fitted per
    ## outcome, so the analyzed sample may be a range.
    analyzed <- format_analysis_counts(attr(x, "analysis_counts"),
                                       marks = attr(x, "number_marks"))
    if (!is.null(analyzed)) {
        cat(analyzed, "\n", sep = "")
    }
    
    if (!is.null(attr(x, "models"))) {
        cat("Models stored: Yes\n")
    }
    
    cat("\n")
    NextMethod("print", x)
    invisible(x)
}


### * Predictor effect extraction

#' Extract predictor effects from a fitted model
#' 
#' Internal helper function that extracts only the predictor variable's 
#' coefficient(s) from a fitted model, ignoring intercept and covariates.
#' Supports standard models (glm, lm, coxph) and mixed effects models
#' (glmer, lmer, coxme).
#' 
#' @param model Fitted model object.
#' @param predictor Character string of the predictor variable name.
#' @param outcome Character string of the outcome variable name.
#' @param conf_level Numeric confidence level.
#' @param adjusted Logical indicating if this is an adjusted model.
#' @param terms_to_extract Character vector of terms to extract (predictor and 
#'   optionally interaction terms involving the predictor).
#' @return data.table with predictor effect information.
#' @keywords internal
extract_predictor_effects <- function(model, predictor, outcome, 
                                      conf_level = 0.95, adjusted = FALSE,
                                      terms_to_extract = NULL,
                                      conf_method = NULL) {
    
    if (is.null(terms_to_extract)) {
        terms_to_extract <- predictor
    }
    
    model_classes <- class(model)
    model_class <- model_classes[1]
    
    ## Detect mixed effects models
    is_merMod <- inherits(model, "merMod") || any(c("glmerMod", "lmerMod") %in% model_classes)
    is_coxme <- inherits(model, "coxme")
    is_mixed <- is_merMod || is_coxme
    
    ## Get coefficient summary based on model type
    if (is_coxme) {
        ## coxme models
        coefs <- coxme::fixef(model)
        vcov_mat <- as.matrix(vcov(model))
        se_values <- sqrt(diag(vcov_mat))
        coef_names <- names(coefs)
        
        ## Build coefficient summary matrix
        z_vals <- coefs / se_values
        p_vals <- 2 * stats::pnorm(-abs(z_vals))
        coef_summary <- cbind(coef = coefs, `se(coef)` = se_values, 
                              z = z_vals, `Pr(>|z|)` = p_vals)
        rownames(coef_summary) <- coef_names
        
    } else if (is_merMod) {
        ## lme4 models (glmer, lmer)
        coef_summary <- summary(model)$coefficients
        coef_names <- rownames(coef_summary)
        
    } else {
        ## Standard models
        coef_summary <- summary(model)$coefficients
        coef_names <- rownames(coef_summary)
    }
    
    ## Find which coefficients belong to the terms being extracted
    ## Build pattern to match predictor and any interaction terms
    predictor_rows <- integer(0)
    for (term in terms_to_extract) {
        if (grepl(":", term, fixed = TRUE)) {
            ## Interaction term - match exactly or with factor level suffixes
            ## Split interaction components
            components <- strsplit(term, ":", fixed = TRUE)[[1]]
            ## Build regex that matches the interaction in either order
            ## e.g., "age:sexMale" or "sexMale:age"
            pattern <- paste0("(^|:)", components[1], ".*:", components[2], "|",
                              "(^|:)", components[2], ".*:", components[1])
            matches <- grep(pattern, coef_names)
        } else {
            ## Main effect - match term at start of coefficient name
            pattern <- paste0("^", term)
            matches <- grep(pattern, coef_names)
            ## Exclude matches that are actually interaction terms (contain ":")
            ## unless interactions are specifically sought
            matches <- matches[!grepl(":", coef_names[matches], fixed = TRUE)]
        }
        predictor_rows <- c(predictor_rows, matches)
    }
    predictor_rows <- unique(predictor_rows)
    
    if (length(predictor_rows) == 0) {
        warning("Predictor '", predictor, "' not found in model for outcome '", outcome, "'")
        return(NULL)
    }
    
    ## Extract predictor coefficients
    pred_coefs <- coef_summary[predictor_rows, , drop = FALSE]
    
    ## Get total sample size and events (will be replaced with group-specific values for factors)
    if (is_merMod) {
        n_obs <- nrow(model@frame)
    } else if (is_coxme) {
        n_obs <- model$n[1]
    } else {
        n_obs <- stats::nobs(model)
    }
    
    total_events <- NA_integer_
    if (model_class == "glm") {
        if (model$family$family %in% c("binomial", "quasibinomial")) {
            ## Binary outcome - count events (1s)
            if (!is.null(model$y)) {
                total_events <- sum(model$y, na.rm = TRUE)
            }
        } else if (model$family$family %in% c("poisson", "quasipoisson")) {
            ## Count outcome - sum total events
            if (!is.null(model$y)) {
                total_events <- sum(model$y, na.rm = TRUE)
            }
        }
    } else if (model_class == "negbin") {
        ## Negative binomial count outcome - sum total events
        if (!is.null(model$y)) {
            total_events <- sum(model$y, na.rm = TRUE)
        }
    } else if (model_class == "coxph") {
        total_events <- model$nevent
    } else if (is_coxme) {
        total_events <- sum(model$y[, "status"], na.rm = TRUE)
    } else if (inherits(model, "glmerMod")) {
        ## For glmer models - check family
        resp <- model@resp$y
        if (!is.null(resp)) {
            fam <- family(model)$family
            if (fam %in% c("binomial", "quasibinomial", "poisson", "quasipoisson")) {
                total_events <- sum(resp, na.rm = TRUE)
            }
        }
    }
    
    ## Build coefficient names (handle factor levels and interactions)
    term_names <- rownames(pred_coefs)
    group_names <- character(length(term_names))
    
    for (i in seq_along(term_names)) {
        term <- term_names[i]
        if (grepl(":", term, fixed = TRUE)) {
            ## Interaction term - store raw term, will be formatted later with labels
            group_names[i] <- term
        } else {
            ## Main effect - extract level name
            group_names[i] <- sub(paste0("^", predictor), "", term)
            if (group_names[i] == "") group_names[i] <- "-"  # Continuous predictor
        }
    }
    
    ## Calculate group-specific n and events for factor predictors
    n_values <- rep(n_obs, length(group_names))
    events_values <- rep(total_events, length(group_names))
    
    ## Check if this is a factor predictor (has non-"-" group names that aren't interaction terms)
    is_factor_predictor <- any(group_names != "-" & !grepl(":", group_names, fixed = TRUE))
    
    if (is_factor_predictor) {
        ## Get model data to calculate group-specific counts
        ## Use model$model or model.frame() which contains only complete cases used in fitting
        model_data <- NULL
        outcome_col <- NULL
        
        if (is_merMod) {
            model_data <- model@frame
            ## First column is the outcome
            outcome_col <- model_data[[1]]
        } else if (is_coxme) {
            ## For coxme, use the model's y component for events. Restricting on
            ## the predictor alone leaves observations with a missing response in
            ## place, which both inflates group sizes and misaligns model_data
            ## with outcome_col; complete cases across all model variables are
            ## required.
            if (!is.null(model$data)) {
                model_data <- get_analysis_data(model, "coxme", model$data)
            }
            if (!is.null(model$y)) {
                outcome_col <- model$y[, "status"]
            }
        } else if (model_class %in% c("coxph", "clogit")) {
            ## For coxph, model$model contains Surv object - extract status from model$y.
            ## survival::coxph() does not retain a model frame by default, so it
            ## is rebuilt to recover the complete cases used in fitting.
            model_data <- model$model
            if (is.null(model_data)) {
                model_data <- tryCatch(stats::model.frame(model),
                                       error = function(e) NULL)
            }
            if (!is.null(model$y)) {
                ## model$y is a Surv object - extract status column
                outcome_col <- model$y[, "status"]
            }
        } else if (!is.null(model$model)) {
            ## model$model contains the model frame with only complete cases
            model_data <- model$model
            ## First column is the outcome
            outcome_col <- model_data[[1]]
        } else {
            ## Fallback: rebuild the model frame, which contains only the
            ## complete cases used in fitting. Counting group sizes on the
            ## unrestricted data would inflate them whenever the response or a
            ## covariate is missing.
            model_data <- tryCatch(stats::model.frame(model),
                                   error = function(e) NULL)
            if (!is.null(model_data)) {
                outcome_col <- model_data[[1]]
            }
        }

        ## Group sizes are only meaningful when they can be counted over the
        ## observations used in fitting; falling back to the model-level total
        ## for every level would be misleading.
        if (!is.null(model_data) &&
            !is.null(outcome_col) &&
            nrow(model_data) != length(outcome_col)) {
            outcome_col <- NULL
        }
        
        if (!is.null(model_data) && predictor %in% names(model_data)) {
            pred_col <- model_data[[predictor]]
            
            ## Convert outcome to numeric if it's a factor (for binary outcomes stored as factors)
            if (!is.null(outcome_col) && is.factor(outcome_col)) {
                ## For factors, the second level is typically the "event" (1)
                ## Convert to 0/1 numeric
                outcome_col <- as.numeric(outcome_col) - 1
            }
            
            ## Handle Surv objects (shouldn't happen now but just in case)
            if (!is.null(outcome_col) && inherits(outcome_col, "Surv")) {
                outcome_col <- outcome_col[, "status"]
            }
            
            for (i in seq_along(group_names)) {
                grp <- group_names[i]
                if (grepl(":", grp, fixed = TRUE)) {
                    ## This is an interaction term - set n/Events to NA
                    ## (group-specific counts don't make sense for interactions)
                    n_values[i] <- NA_integer_
                    events_values[i] <- NA_integer_
                } else if (grp != "-") {
                    ## This is a factor level - calculate group-specific counts
                    grp_mask <- pred_col == grp
                    if (any(grp_mask, na.rm = TRUE)) {
                        n_values[i] <- sum(grp_mask, na.rm = TRUE)
                        
                        ## Calculate events for this group if outcome available
                        if (!is.null(outcome_col) && !is.na(total_events)) {
                            events_values[i] <- sum(outcome_col[grp_mask], na.rm = TRUE)
                        }
                    }
                }
            }
        }
    }
    
    ## Get appropriate column names based on model class
    if (model_class == "coxph" || is_coxme) {
        coef_col <- "coef"
        se_col <- "se(coef)"
        stat_col <- "z"
        p_col <- if ("Pr(>|z|)" %in% colnames(pred_coefs)) "Pr(>|z|)" else "p"
    } else if (model_class %in% c("glm", "negbin") || inherits(model, "glmerMod")) {
        coef_col <- "Estimate"
        se_col <- "Std. Error"
        stat_col <- "z value"
        p_col <- "Pr(>|z|)"
    } else if (model_class == "lm" || inherits(model, "lmerMod")) {
        coef_col <- "Estimate"
        se_col <- "Std. Error"
        stat_col <- if ("t value" %in% colnames(pred_coefs)) "t value" else "z value"
        p_col <- if ("Pr(>|t|)" %in% colnames(pred_coefs)) "Pr(>|t|)" else "Pr(>|z|)"
    } else {
        coef_col <- colnames(pred_coefs)[1]
        se_col <- colnames(pred_coefs)[2]
        stat_col <- colnames(pred_coefs)[3]
        p_col <- colnames(pred_coefs)[4]
    }
    
    ## Resolve CI method
    conf_method_resolved <- if (is.null(conf_method)) {
        getOption("summata.conf_method", "profile")
    } else {
        match.arg(conf_method, c("profile", "wald"))
    }

    ## Compute CIs: profile/exact-t when conf_method="profile", Wald otherwise
    coefficients <- pred_coefs[, coef_col]
    se_values <- pred_coefs[, se_col]

    use_confint <- FALSE
    if (conf_method_resolved == "profile") {
        if (inherits(model, c("glm", "negbin"))) {
            use_confint <- !(inherits(model, "glm") &&
                             grepl("^quasi", model$family$family))
        } else if (inherits(model, "lm")) {
            use_confint <- TRUE
        }
    }

    if (use_confint) {
        ## Profile likelihood CIs (GLM/negbin) or exact t CIs (lm)
        better_ci <- tryCatch(
            suppressMessages(suppressWarnings(
                stats::confint(model, level = conf_level)
            )),
            error = function(e) NULL
        )
        if (!is.null(better_ci)) {
            ## Remove intercept if present
            if ("(Intercept)" %in% rownames(better_ci)) {
                better_ci <- better_ci[rownames(better_ci) != "(Intercept)", ,
                                         drop = FALSE]
            }
            ## Match terms to confint rows
            term_match <- match(rownames(pred_coefs), rownames(better_ci))
            if (all(!is.na(term_match))) {
                ci_lower <- better_ci[term_match, 1L]
                ci_upper <- better_ci[term_match, 2L]
            } else {
                ## Partial match: use confint where available, Wald otherwise
                z_score <- stats::qnorm((1 + conf_level) / 2)
                ci_lower <- coefficients - z_score * se_values
                ci_upper <- coefficients + z_score * se_values
                matched <- !is.na(term_match)
                ci_lower[matched] <- better_ci[term_match[matched], 1L]
                ci_upper[matched] <- better_ci[term_match[matched], 2L]
            }
        } else {
            ## confint() failed; fall back to Wald
            z_score <- stats::qnorm((1 + conf_level) / 2)
            ci_lower <- coefficients - z_score * se_values
            ci_upper <- coefficients + z_score * se_values
        }
    } else {
        ## Wald CIs
        z_score <- stats::qnorm((1 + conf_level) / 2)
        ci_lower <- coefficients - z_score * se_values
        ci_upper <- coefficients + z_score * se_values
    }
    
    ## Determine effect type and calculate exponentiated values
    if (model_class == "coxph" || is_coxme) {
        effect_type <- "HR"
        exp_coef <- exp(coefficients)
        exp_lower <- exp(ci_lower)
        exp_upper <- exp(ci_upper)
    } else if (model_class == "negbin") {
        ## Negative binomial models produce rate ratios
        effect_type <- "RR"
        exp_coef <- exp(coefficients)
        exp_lower <- exp(ci_lower)
        exp_upper <- exp(ci_upper)
    } else if (model_class == "glm" || inherits(model, "glmerMod")) {
        family_name <- if (is_merMod) model@resp$family$family else model$family$family
        if (family_name %in% c("binomial", "quasibinomial")) {
            effect_type <- "OR"
            exp_coef <- exp(coefficients)
            exp_lower <- exp(ci_lower)
            exp_upper <- exp(ci_upper)
        } else if (family_name %in% c("poisson", "quasipoisson")) {
            effect_type <- "RR"
            exp_coef <- exp(coefficients)
            exp_lower <- exp(ci_lower)
            exp_upper <- exp(ci_upper)
        } else {
            effect_type <- "Coefficient"
            exp_coef <- coefficients
            exp_lower <- ci_lower
            exp_upper <- ci_upper
        }
    } else {
        effect_type <- "Coefficient"
        exp_coef <- coefficients
        exp_lower <- ci_lower
        exp_upper <- ci_upper
    }
    
    ## Handle p-values for lmer models (which may not have p-values)
    p_values <- if (p_col %in% colnames(pred_coefs)) {
                    pred_coefs[, p_col]
                } else {
                    rep(NA_real_, nrow(pred_coefs))
                }
    
    ## Handle statistics column
    stat_values <- if (stat_col %in% colnames(pred_coefs)) {
                       pred_coefs[, stat_col]
                   } else {
                       rep(NA_real_, nrow(pred_coefs))
                   }
    
    ## Build result data.table
    dt <- data.table::data.table(
                          outcome = outcome,
                          predictor = predictor,
                          group = group_names,
                          n = n_values,
                          events = events_values,
                          coefficient = coefficients,
                          se = se_values,
                          exp_coef = exp_coef,
                          ci_lower = exp_lower,
                          ci_upper = exp_upper,
                          statistic = stat_values,
                          p_value = p_values,
                          effect_type = effect_type,
                          adjusted = adjusted
                      )
    
    return(dt)
}


### * Table formatting

#' Combine multivariate results
#' 
#' Internal helper to combine unadjusted and/or adjusted results from 
#' multiple outcomes into a single data.table.
#' 
#' @param all_results List of results from fit_one_outcome calls.
#' @param columns Character specifying which columns to include.
#' @return Combined data.table.
#' @keywords internal
combine_multifit_results <- function(all_results, columns) {
    
    ## Extract results based on columns parameter
    if (columns == "unadjusted") {
        results_list <- lapply(all_results, function(x) x$results$unadjusted)
        results_list <- results_list[!vapply(results_list, is.null, logical(1))]
        
        if (length(results_list) == 0) return(data.table::data.table())
        
        combined <- data.table::rbindlist(results_list, fill = TRUE)
        
    } else if (columns == "adjusted") {
        results_list <- lapply(all_results, function(x) x$results$adjusted)
        results_list <- results_list[!vapply(results_list, is.null, logical(1))]
        
        if (length(results_list) == 0) return(data.table::data.table())
        
        combined <- data.table::rbindlist(results_list, fill = TRUE)
        
    } else {
        ## columns == "both" - merge unadjusted and adjusted
        unadj_list <- lapply(all_results, function(x) x$results$unadjusted)
        adj_list <- lapply(all_results, function(x) x$results$adjusted)
        
        unadj_list <- unadj_list[!vapply(unadj_list, is.null, logical(1))]
        adj_list <- adj_list[!vapply(adj_list, is.null, logical(1))]
        
        if (length(unadj_list) == 0 && length(adj_list) == 0) {
            return(data.table::data.table())
        }
        
        unadj_combined <- if (length(unadj_list) > 0) {
                              data.table::rbindlist(unadj_list, fill = TRUE)
                          } else {
                              data.table::data.table()
                          }
        
        adj_combined <- if (length(adj_list) > 0) {
                            data.table::rbindlist(adj_list, fill = TRUE)
                        } else {
                            data.table::data.table()
                        }
        
        ## Rename columns for merging
        if (nrow(unadj_combined) > 0) {
            data.table::setnames(unadj_combined, 
                                 c("exp_coef", "ci_lower", "ci_upper", "p_value"),
                                 c("exp_coef_unadj", "ci_lower_unadj", "ci_upper_unadj", "p_unadj"),
                                 skip_absent = TRUE
                                 )
        }
        
        if (nrow(adj_combined) > 0) {
            data.table::setnames(adj_combined,
                                 c("exp_coef", "ci_lower", "ci_upper", "p_value"),
                                 c("exp_coef_adj", "ci_lower_adj", "ci_upper_adj", "p_adj"),
                                 skip_absent = TRUE
                                 )
        }
        
        ## Merge on outcome, predictor, group
        if (nrow(unadj_combined) > 0 && nrow(adj_combined) > 0) {
            merge_cols <- c("outcome", "predictor", "group")
            keep_from_unadj <- c(merge_cols, "n", "events", "effect_type",
                                 "exp_coef_unadj", "ci_lower_unadj", "ci_upper_unadj", "p_unadj")
            keep_from_adj <- c(merge_cols, "n", "events",
                               "exp_coef_adj", "ci_lower_adj", "ci_upper_adj", "p_adj")
            
            unadj_subset <- unadj_combined[, intersect(keep_from_unadj, names(unadj_combined)), with = FALSE]
            adj_subset <- adj_combined[, intersect(keep_from_adj, names(adj_combined)), with = FALSE]
            
            ## Carry the adjusted counts under separate names so that they can
            ## replace the unadjusted ones after the merge
            data.table::setnames(adj_subset,
                                 c("n", "events"),
                                 c(".n_adj", ".events_adj"),
                                 skip_absent = TRUE)
            
            combined <- merge(unadj_subset, adj_subset, by = merge_cols, all = TRUE)
            
            ## A row carrying an adjusted estimate is described by the adjusted
            ## model's complete cases. The unadjusted sample is larger whenever a
            ## covariate carries missing values, and reporting it alongside the
            ## adjusted estimate would overstate the observations behind that
            ## estimate. Rows with no adjusted estimate keep their unadjusted
            ## counts. This matches the treatment in fullfit() (STROBE item 12).
            if (all(c(".n_adj", "n") %in% names(combined))) {
                rows <- which(!is.na(combined[[".n_adj"]]))
                if (length(rows) > 0) {
                    data.table::set(combined, i = rows, j = "n",
                                    value = combined[[".n_adj"]][rows])
                }
                data.table::set(combined, j = ".n_adj", value = NULL)
            }
            if (all(c(".events_adj", "events") %in% names(combined))) {
                rows <- which(!is.na(combined[[".events_adj"]]))
                if (length(rows) > 0) {
                    data.table::set(combined, i = rows, j = "events",
                                    value = combined[[".events_adj"]][rows])
                }
                data.table::set(combined, j = ".events_adj", value = NULL)
            }
        } else if (nrow(unadj_combined) > 0) {
            combined <- unadj_combined
        } else {
            combined <- adj_combined
        }
    }
    
    return(combined)
}


#' Format multifit results for publication
#' 
#' Internal helper that formats raw multivariate results into 
#' publication-ready table format.
#' 
#' @param data Raw combined data.table from combine_multifit_results.
#' @param columns Character specifying column layout.
#' @param show_n Logical for sample size column.
#' @param show_events Logical for events column.
#' @param digits Integer decimal places for effects.
#' @param p_digits Integer decimal places for p-values.
#' @param labels Named vector of labels for outcomes and predictors.
#' @param predictor_label Label for predictor variable.
#' @param include_predictor Logical for including predictor column.
#' @param exponentiate Logical or \code{NULL} for coefficient handling.
#' @return Formatted data.table.
#' @keywords internal
format_multifit_table <- function(data, 
                                  columns,
                                  show_n = TRUE,
                                  show_events = TRUE,
                                  digits = 2,
                                  p_digits = 3,
                                  labels = NULL,
                                  predictor_label = NULL,
                                  include_predictor = TRUE,
                                  exponentiate = NULL,
                                  conf_level = 0.95,
                                  marks = NULL) {
    
    if (nrow(data) == 0) {
        return(data.table::data.table())
    }
    
    result <- data.table::copy(data)
    
    ## Apply outcome labels
    if (!is.null(labels) && length(labels) > 0) {
        for (orig_name in names(labels)) {
            result[outcome == orig_name, outcome := labels[[orig_name]]]
        }
    }
    
    ## Rename outcome column
    data.table::setnames(result, "outcome", "Outcome")
    
    ## Handle predictor group display
    ## If only one group value ("-"), omit Predictor column
    has_multiple_groups <- length(unique(result$group)) > 1 || 
        !all(result$group == "-")
    
    if (has_multiple_groups) {
        ## Get predictor name for labeling
        pred_name <- result$predictor[1]
        pred_display <- if (!is.null(predictor_label)) {
                            predictor_label
                        } else if (!is.null(labels) && pred_name %in% names(labels)) {
                            labels[[pred_name]]
                        } else {
                            pred_name
                        }
        
        data.table::setnames(result, "group", "Predictor")
        
        ## Check if this is a binary variable with affirmative non-reference level
        ## Common affirmative values that do not need "(Level)" suffix
        affirmative_values <- c("Yes", "YES", "yes", "1", "True", "TRUE", "true", 
                                "Present", "Positive", "+")
        unique_levels <- unique(result$Predictor[result$Predictor != "-"])
        is_binary_affirmative <- length(unique_levels) == 1 && 
            unique_levels[1] %in% affirmative_values
        
        ## Format predictor values as "Variable (Level)" or just "Variable" for binary
        for (i in seq_len(nrow(result))) {
            pred_val <- result$Predictor[i]
            if (grepl(":", pred_val, fixed = TRUE)) {
                ## This is an interaction term - format nicely
                result[i, Predictor := format_interaction_term(pred_val, labels)]
            } else if (pred_val != "-") {
                if (is_binary_affirmative) {
                    ## Binary variable with affirmative level - just show variable name
                    result[i, Predictor := pred_display]
                } else {
                    ## Regular factor level - format as "Variable (Level)"
                    result[i, Predictor := paste0(pred_display, " (", pred_val, ")")]
                }
            }
        }
        
        ## For continuous predictors (marked with "-"), just show variable name
        result[Predictor == "-", Predictor := pred_display]
    }
    
    ## Determine effect type for column naming
    effect_type <- result$effect_type[1]
    if (is.na(effect_type)) effect_type <- "Coefficient"
    
    ## Disable show_events for linear models only
    ## Events make sense for binomial (OR), survival (HR), and count (RR) models
    if (effect_type == "Coefficient") {
        show_events <- FALSE
    }
    
    ## Format confidence level for display (e.g., 0.95 -> "95%", 0.90 -> "90%")
    ci_pct <- round(conf_level * 100)
    ci_label <- paste0(ci_pct, "% CI")
    
    ## Helper: format effect + CI columns using locale-aware marks
    format_effect_ci <- function(eff_vals, low_vals, hi_vals, digs, etype, mks) {
        fmt_s <- paste0("%.", digs, "f")
        e_fmt <- sprintf(fmt_s, eff_vals)
        l_fmt <- sprintf(fmt_s, low_vals)
        h_fmt <- sprintf(fmt_s, hi_vals)
        
        if (!is.null(mks)) {
            e_fmt <- apply_decimal_mark(e_fmt, mks)
            l_fmt <- apply_decimal_mark(l_fmt, mks)
            h_fmt <- apply_decimal_mark(h_fmt, mks)
            any_neg <- any(low_vals < 0 | hi_vals < 0, na.rm = TRUE)
            ci_sep <- if (any_neg) " to " else if (mks$decimal.mark == ",") "\u2013" else "-"
        } else {
            e_fmt <- fix_negative_zero(e_fmt)
            l_fmt <- fix_negative_zero(l_fmt)
            h_fmt <- fix_negative_zero(h_fmt)
            ci_sep <- if (etype %in% c("OR", "HR", "RR")) "-" else ", "
        }
        
        ## For non-exponentiated models without marks, use comma+space
        if (is.null(mks) && !etype %in% c("OR", "HR", "RR")) {
            paste0(e_fmt, " (", l_fmt, ", ", h_fmt, ")")
        } else {
            paste0(e_fmt, " (", l_fmt, ci_sep, h_fmt, ")")
        }
    }
    
    ## Format effect columns based on columns parameter
    if (columns == "both") {
        ## Effect type label is already correct (Coefficient for linear models)
        effect_display <- effect_type
        
        ## Format unadjusted effects - simple label without "Univariable" prefix
        if ("exp_coef_unadj" %in% names(result)) {
            unadj_label <- paste0(effect_display, " (", ci_label, ")")
            
            result[, (unadj_label) := format_effect_ci(
                exp_coef_unadj, ci_lower_unadj, ci_upper_unadj,
                digits, effect_type, marks)]
            result[is.na(exp_coef_unadj), (unadj_label) := "-"]
        }
        
        ## Format unadjusted p-values
        if ("p_unadj" %in% names(result)) {
            result[, `Uni p` := format_pvalues_multifit(p_unadj, p_digits, marks)]
        }
        
        ## Format adjusted effects - use aOR/aHR/aRR/Adj. Coefficient without "Multivariable" prefix
        if ("exp_coef_adj" %in% names(result)) {
            adj_effect <- switch(effect_type,
                                 "OR" = "aOR",
                                 "HR" = "aHR", 
                                 "RR" = "aRR",
                                 "Adj. Coefficient"
                                 )
            adj_label <- paste0(adj_effect, " (", ci_label, ")")
            
            result[, (adj_label) := format_effect_ci(
                exp_coef_adj, ci_lower_adj, ci_upper_adj,
                digits, effect_type, marks)]
            result[is.na(exp_coef_adj), (adj_label) := "-"]
        }
        
        ## Format adjusted p-values
        if ("p_adj" %in% names(result)) {
            result[, `Multi p` := format_pvalues_multifit(p_adj, p_digits, marks)]
        }
        
    } else {
        ## Single column mode (unadjusted or adjusted)
        ## Note: In single-column mode, columns are NOT renamed with _adj/_unadj suffix
        effect_col <- "exp_coef"
        ci_lower_col <- "ci_lower"
        ci_upper_col <- "ci_upper"
        p_col <- "p_value"
        
        ## Effect display name is already correct (Coefficient for linear models)
        effect_display <- effect_type
        
        ## Determine label based on adjustment status - no Univariable/Multivariable prefix
        if (columns == "adjusted") {
            adj_effect <- switch(effect_type,
                                 "OR" = "aOR",
                                 "HR" = "aHR",
                                 "RR" = "aRR",
                                 "Coefficient" = "Adj. Coefficient",
                                 "Adj. Coefficient"
                                 )
            effect_label <- paste0(adj_effect, " (", ci_label, ")")
        } else {
            effect_label <- paste0(effect_display, " (", ci_label, ")")
        }
        
        ## Format effect with CI
        if (effect_col %in% names(result)) {
            result[, (effect_label) := format_effect_ci(
                get(effect_col), get(ci_lower_col), get(ci_upper_col),
                digits, effect_type, marks)]
            result[is.na(get(effect_col)), (effect_label) := "-"]
        }
        
        ## Format p-value
        if (p_col %in% names(result)) {
            result[, `p-value` := format_pvalues_multifit(get(p_col), p_digits, marks)]
        }
    }
    
    ## Format n and events. format_count() is vectorized, applies the grouping
    ## mark only from four digits, and resolves the locale itself when marks are
    ## absent, so no threshold or fallback is needed here.
    if ("n" %in% names(result)) {
        result[, n := format_count(result$n, marks, na = "-")]
    }
    
    if ("events" %in% names(result)) {
        result[, events := format_count(result$events, marks, na = "-")]
        data.table::setnames(result, "events", "Events")
    }
    
    ## Select display columns
    display_cols <- "Outcome"
    if (has_multiple_groups && include_predictor) display_cols <- c(display_cols, "Predictor")
    if (show_n && "n" %in% names(result)) display_cols <- c(display_cols, "n")
    if (show_events && "Events" %in% names(result)) display_cols <- c(display_cols, "Events")
    
    ## Add effect and p-value columns based on layout
    if (columns == "both") {
        effect_display <- effect_type
        unadj_label <- paste0(effect_display, " (", ci_label, ")")
        adj_effect <- switch(effect_type, "OR" = "aOR", "HR" = "aHR", "RR" = "aRR", "Coefficient" = "Adj. Coefficient", "Adj. Coefficient")
        adj_label <- paste0(adj_effect, " (", ci_label, ")")
        
        if (unadj_label %in% names(result)) display_cols <- c(display_cols, unadj_label)
        if ("Uni p" %in% names(result)) display_cols <- c(display_cols, "Uni p")
        if (adj_label %in% names(result)) display_cols <- c(display_cols, adj_label)
        if ("Multi p" %in% names(result)) display_cols <- c(display_cols, "Multi p")
    } else {
        effect_display <- effect_type
        if (columns == "adjusted") {
            adj_effect <- switch(effect_type, "OR" = "aOR", "HR" = "aHR", "RR" = "aRR", "Coefficient" = "Adj. Coefficient", "Adj. Coefficient")
            effect_label <- paste0(adj_effect, " (", ci_label, ")")
        } else {
            effect_label <- paste0(effect_display, " (", ci_label, ")")
        }
        
        if (effect_label %in% names(result)) display_cols <- c(display_cols, effect_label)
        if ("p-value" %in% names(result)) display_cols <- c(display_cols, "p-value")
    }
    
    ## Select only display columns
    formatted <- result[, intersect(display_cols, names(result)), with = FALSE]
    
    return(formatted)
}


#' Format p-values for multifit display
#' 
#' Converts numeric p-values to formatted character strings. Values below the 
#' threshold (determined by digits parameter) display as "< 0.001" (for digits=3),
#' "< 0.0001" (for digits=4), \emph{etc.} NA values display as "-".
#' 
#' @param p Numeric vector of p-values.
#' @param digits Integer number of decimal places. Also determines the threshold
#'   for "less than" display: threshold = 10^(-digits). Default is 3.
#' @param marks Optional list with \code{big.mark} and \code{decimal.mark} as
#'   returned by \code{\link{resolve_number_marks}}.
#' @return Character vector of formatted p-values.
#' @keywords internal
format_pvalues_multifit <- function(p, digits = 3, marks = NULL) {
    ## Delegate to the shared p-value formatter in fit_utils
    format_pvalues_fit(p, digits, marks)
}


#' Format interaction term for display
#' 
#' Converts R's internal interaction term format (\emph{e.g.,} "treatmentDrug A:stageII")
#' to a more readable format (\emph{e.g.,} "Treatment (Drug A) × Stage (II)").
#' 
#' @param term Character string of the interaction term from model coefficients.
#' @param labels Optional named vector of labels for variable names.
#' @return Formatted interaction term string.
#' @keywords internal
format_interaction_term <- function(term, labels = NULL) {
    ## Split by colon
    parts <- strsplit(term, ":", fixed = TRUE)[[1]]
    
    formatted_parts <- vapply(parts, function(part) {
        ## Try to separate variable name from level
        ## Common patterns: varLevel, var.Level, varLevel1Level2
        
        ## First, check if this looks like a factor level (contains uppercase after lowercase)
        ## or ends with a number pattern
        
        ## Try to find where variable name ends and level begins
        ## Look for transition from lowercase to uppercase, or number after letter
        
        ## Strategy: find the longest matching variable name prefix
        ## "treatmentDrug A" yields "treatment" and "Drug A"
        ## "stageII" yields "stage" and "II"
        ## "age" (continuous in an interaction) yields "age" alone
        
        ## Check for common variable name patterns
        ## Match: lowercase letters, possibly followed by underscore/lowercase
        var_match <- regexpr("^[a-z][a-z0-9_]*", part, perl = TRUE)
        
        if (var_match > 0) {
            var_len <- attr(var_match, "match.length")
            var_name <- substr(part, 1, var_len)
            level <- substr(part, var_len + 1, nchar(part))
            
            ## Apply label if available, otherwise use raw variable name
            if (!is.null(labels) && var_name %in% names(labels)) {
                var_display <- labels[[var_name]]
            } else {
                ## Use raw variable name (consistent with fit() and other functions)
                var_display <- var_name
            }
            
            if (nchar(level) > 0) {
                ## Has a level - format as "Variable (Level)"
                paste0(var_display, " (", level, ")")
            } else {
                ## Continuous variable - just the name
                var_display
            }
        } else {
            ## Fallback - return as-is
            part
        }
    }, character(1))
    
    ## Join with multiplication sign (consistent with other fullfit functions)
    paste(formatted_parts, collapse = " \u00d7 ")
}


### * Input validation

#' Validate outcome homogeneity for multifit
#' 
#' Checks whether all outcomes in a multifit analysis are compatible with the
#' specified model type. Issues a warning when outcomes appear to be of mixed
#' types (\emph{e.g.,} binary and continuous outcomes in the same analysis), which
#' would produce tables with incompatible effect measures.
#' 
#' @param data Data.table containing the analysis data.
#' @param outcomes Character vector of outcome variable names.
#' @param model_type Character string specifying the model type.
#' @param family Character string specifying the GLM family (for glm/glmer).
#' @return Invisible \code{NULL}. Issues warnings if problems are detected.
#' @keywords internal
validate_outcome_homogeneity <- function(data, outcomes, model_type, family = "binomial") {
    
    ## Skip validation for survival outcomes (Surv() syntax)
    is_surv <- grepl("^Surv\\(", outcomes)
    if (all(is_surv)) return(invisible(NULL))
    if (any(is_surv) && !all(is_surv)) {
        warning("Mixed survival and non-survival outcomes detected. ",
                "Consider separate multifit() calls for each outcome type.",
                call. = FALSE)
        return(invisible(NULL))
    }
    
    ## Extract family name from family object/function/string
    family_name <- if (is.character(family)) {
        family
    } else if (is.function(family)) {
        tryCatch(family()$family, error = function(e) NULL)
    } else if (is.list(family) && "family" %in% names(family)) {
        family$family
    } else {
        NULL
    }
    
    ## Classify each outcome
    outcome_types <- vapply(outcomes, function(out) {
        if (grepl("^Surv\\(", out)) return("survival")
        
        vec <- data[[out]]
        if (is.null(vec)) return("unknown")
        
        if (is.factor(vec) || is.character(vec)) {
            n_levels <- length(unique(stats::na.omit(vec)))
            if (n_levels == 2) return("binary")
            return("categorical")
        }
        
        if (is.logical(vec)) return("binary")
        
        if (is.numeric(vec)) {
            unique_vals <- unique(stats::na.omit(vec))
            if (length(unique_vals) == 2 && all(unique_vals %in% c(0, 1))) {
                return("binary")
            }
            return("continuous")
        }
        
        return("unknown")
    }, character(1))
    
    ## Check for categorical outcomes with >2 levels (problematic for binomial GLM)
    if (model_type %in% c("glm", "glmer") && !is.null(family_name) && family_name == "binomial") {
        categorical_outcomes <- names(outcome_types)[outcome_types == "categorical"]
        if (length(categorical_outcomes) > 0) {
            warning("Categorical outcome(s) with more than 2 levels detected: ",
                    paste(categorical_outcomes, collapse = ", "), ". ",
                    "Binomial GLM will coerce these to binary (first level vs all others), ",
                    "which is likely not the intended analysis. ",
                    "Consider: (1) recoding to a true binary variable, ",
                    "(2) using multinomial regression (nnet::multinom), ",
                    "(3) using ordinal regression (MASS::polr) if levels are ordered, or ",
                    "(4) removing these outcomes from the analysis.",
                    call. = FALSE)
        }
    }
    
    ## Check for mixed types
    known_types <- outcome_types[outcome_types != "unknown"]
    if (length(unique(known_types)) > 1) {
        type_summary <- table(known_types)
        type_str <- paste(names(type_summary), type_summary, sep = ": ", collapse = ", ")
        
        warning("Outcomes appear to be of mixed types (", type_str, "). ",
                "This will produce a table with incompatible effect measures ",
                "(e.g., odds ratios alongside regression coefficients). ",
                "Consider separate multifit() calls for each outcome type:\n",
                "  - Binary outcomes: model_type = 'glm', family = 'binomial'\n",
                "  - Continuous outcomes: model_type = 'lm'\n",
                "  - Survival outcomes: model_type = 'coxph'",
                call. = FALSE)
    }
    
    ## Check for model/outcome mismatch
    if (model_type %in% c("glm", "glmer") && !is.null(family_name) && family_name == "binomial") {
        non_binary <- names(outcome_types)[outcome_types == "continuous"]
        if (length(non_binary) > 0) {
            warning("Continuous outcome(s) detected with binomial family: ",
                    paste(non_binary, collapse = ", "), ". ",
                    "Consider using model_type = 'lm' for continuous outcomes.",
                    call. = FALSE)
        }
    }
    
    if (model_type %in% c("lm", "lmer")) {
        binary_outcomes <- names(outcome_types)[outcome_types == "binary"]
        if (length(binary_outcomes) > 0) {
            warning("Binary outcome(s) detected with linear model: ",
                    paste(binary_outcomes, collapse = ", "), ". ",
                    "Consider using model_type = 'glm' with family = 'binomial' ",
                    "for binary outcomes.",
                    call. = FALSE)
        }
    }
    
    invisible(NULL)
}

#' Test Suite for multifit(), multiforest(), uniforest(), and uniscreen()
#' 
#' Comprehensive tests covering all model types, output formatting,
#' interactions, mixed effects, and visualization functions.
#' 
#' @details Run with testthat::test_file("tests/testthat/test_multivariate.R")

library(data.table)

## ============================================================================
## Setup: Load test data and define helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Create integer count outcome for Poisson/quasipoisson/negative binomial tests
set.seed(123)
clintrial$count_outcome <- as.integer(ceiling(clintrial$los_days))
clintrial_complete$count_outcome <- as.integer(ceiling(clintrial_complete$los_days))

## Define outcome sets for different model types
binary_outcomes <- c("surgery", "pfs_status", "os_status")
continuous_outcomes <- c("los_days", "biomarker_x")
surv_outcomes <- c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)")

## Standard predictor sets
standard_predictors <- c("age", "sex", "treatment", "stage")
surgery_covariates <- c("age", "sex", "ecog", "smoking_status")


## ----------------------------------------------------------------------------
## Helper: Check uniscreen result structure
## ----------------------------------------------------------------------------
expect_uniscreen_result <- function(result) {
    expect_s3_class(result, "uniscreen_result")
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("Variable" %in% names(result))
    expect_true("p-value" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "model_type")))
    expect_true(!is.null(attr(result, "model_scope")))
    expect_true(!is.null(attr(result, "raw_data")))
}


## ----------------------------------------------------------------------------
## Helper: Check multifit result structure
## ----------------------------------------------------------------------------
expect_multifit_result <- function(result) {
    expect_s3_class(result, "multifit_result")
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("Outcome" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "predictor")))
    expect_true(!is.null(attr(result, "outcomes")))
    expect_true(!is.null(attr(result, "model_type")))
    expect_true(!is.null(attr(result, "columns")))
    expect_true(!is.null(attr(result, "raw_data")))
}


## ----------------------------------------------------------------------------


## ----------------------------------------------------------------------------


## ----------------------------------------------------------------------------
## Helper: Check raw_data attribute structure for uniscreen
## ----------------------------------------------------------------------------
expect_uniscreen_raw_data <- function(result) {
    raw <- attr(result, "raw_data")
    expect_s3_class(raw, "data.table")
    
    ## Should have effect estimate columns
    has_effect <- any(c("OR", "HR", "RR", "Estimate", "Coefficient") %in% names(raw))
    expect_true(has_effect, info = "raw_data should have effect estimate column")
    
    ## Should have CI columns (lowercase)
    expect_true("ci_lower" %in% names(raw))
    expect_true("ci_upper" %in% names(raw))
}


## ----------------------------------------------------------------------------
## Helper: Check raw_data attribute structure for multifit
## ----------------------------------------------------------------------------
expect_multifit_raw_data <- function(result, columns = "adjusted") {
    raw <- attr(result, "raw_data")
    expect_s3_class(raw, "data.table")
    
    ## Required columns depend on columns parameter
    if (columns == "both") {
        expect_true("exp_coef_unadj" %in% names(raw) || "exp_coef_adj" %in% names(raw))
    } else {
        expect_true("exp_coef" %in% names(raw) || 
                    "exp_coef_unadj" %in% names(raw) || 
                    "exp_coef_adj" %in% names(raw))
    }
}


## ============================================================================
## SECTION 1: uniscreen() Basic Functionality Tests
## ============================================================================

test_that("uniscreen works with basic logistic regression", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = standard_predictors,
        model_type = "glm",
        family = "binomial"
    )
    
    expect_uniscreen_result(result)
    expect_effect_column(result, "OR")
    expect_pvalue_column(result)
    expect_uniscreen_raw_data(result)
    
    ## All predictors should be represented
    expect_true(any(grepl("age|Age", result$Variable)))
    expect_true(any(grepl("sex|Sex", result$Variable)))
})


test_that("uniscreen works with linear regression", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "los_days",
        predictors = standard_predictors,
        model_type = "lm"
    )
    
    expect_uniscreen_result(result)
    expect_effect_column(result, "Coefficient")
    expect_equal(attr(result, "model_scope"), "Univariable")
})


test_that("uniscreen works with Cox regression", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = standard_predictors,
        model_type = "coxph"
    )
    
    expect_uniscreen_result(result)
    expect_effect_column(result, "HR")
    
    ## model_type attribute uses display name
    expect_equal(attr(result, "model_type"), "Cox PH")
})


test_that("uniscreen p_threshold filtering works", {
    
    ## All results
    result_all <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment", "stage", "ecog"),
        model_type = "glm",
        p_threshold = 1
    )
    
    ## Filtered to p < 0.20
    result_filtered <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment", "stage", "ecog"),
        model_type = "glm",
        p_threshold = 0.20
    )
    
    expect_lte(nrow(result_filtered), nrow(result_all))
})


test_that("uniscreen applies labels correctly", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        labels = clintrial_labels
    )
    
    ## Labels should be applied to Variable column
    expect_true(any(grepl("Age|Sex|Treatment", result$Variable)))
})


test_that("uniscreen keep_models stores model objects", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex"),
        model_type = "glm",
        keep_models = TRUE
    )
    
    models <- attr(result, "models")
    expect_type(models, "list")
    expect_true(length(models) >= 2)
    expect_s3_class(models[[1]], "glm")
})


test_that("uniscreen exponentiate parameter works", {
    
    ## Force exponentiate = TRUE
    result_exp <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        exponentiate = TRUE
    )
    
    ## Force exponentiate = FALSE
    result_coef <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        exponentiate = FALSE
    )
    
    ## Column names should differ
    expect_true(any(grepl("OR", names(result_exp))))
    expect_true(any(grepl("Coefficient", names(result_coef))))
})


test_that("uniscreen show_n and show_events parameters work", {
    
    result_full <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "treatment"),
        model_type = "glm",
        show_n = TRUE,
        show_events = TRUE
    )
    
    expect_true("n" %in% names(result_full) || any(grepl("^n$", names(result_full))))
    
    result_minimal <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "treatment"),
        model_type = "glm",
        show_n = FALSE,
        show_events = FALSE
    )
    
    expect_false("n" %in% names(result_minimal))
})


test_that("uniscreen handles single predictor", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm"
    )
    
    expect_uniscreen_result(result)
    expect_true(nrow(result) >= 1)
})


test_that("uniscreen respects digits parameter", {
    
    result_2dig <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        digits = 2
    )
    
    result_4dig <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        digits = 4
    )
    
    ## 4-digit result should have more characters in effect column
    effect_col_2 <- grep("OR.*CI", names(result_2dig), value = TRUE)[1]
    effect_col_4 <- grep("OR.*CI", names(result_4dig), value = TRUE)[1]
    
    expect_true(nchar(result_4dig[[effect_col_4]][1]) > nchar(result_2dig[[effect_col_2]][1]))
})


## ============================================================================
## SECTION 1A: uniscreen() Additional GLM Family Tests
## ============================================================================

## ----------------------------------------------------------------------------
## Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("uniscreen works with gaussian family", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "los_days",
        predictors = standard_predictors,
        model_type = "glm",
        family = "gaussian"
    )
    
    expect_uniscreen_result(result)
    
    ## Should produce coefficients
    col_names <- names(result)
    expect_true(any(grepl("Coefficient.*CI|Estimate.*CI", col_names)),
                info = paste("Expected Coefficient column for gaussian, found:", 
                             paste(col_names, collapse = ", ")))
})


## ----------------------------------------------------------------------------
## Quasibinomial Family Tests
## ----------------------------------------------------------------------------

test_that("uniscreen works with quasibinomial family", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = standard_predictors,
        model_type = "glm",
        family = "quasibinomial"
    )
    
    expect_uniscreen_result(result)
    
    ## Should produce odds ratios
    col_names <- names(result)
    expect_true(any(grepl("OR.*CI", col_names)),
                info = paste("Expected OR column for quasibinomial, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("uniscreen quasibinomial point estimates match binomial", {
    
    result_quasi <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        family = "quasibinomial",
        keep_models = TRUE
    )
    
    result_binom <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        family = "binomial",
        keep_models = TRUE
    )
    
    ## Point estimates should match
    raw_quasi <- attr(result_quasi, "raw_data")
    raw_binom <- attr(result_binom, "raw_data")
    
    expect_equal(raw_quasi$OR[1], raw_binom$OR[1], tolerance = 1e-6)
})


## ----------------------------------------------------------------------------
## Quasipoisson Family Tests
## ----------------------------------------------------------------------------

test_that("uniscreen works with quasipoisson family", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = standard_predictors,
        model_type = "glm",
        family = "quasipoisson"
    )
    
    expect_uniscreen_result(result)
    
    ## Should produce rate ratios
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for quasipoisson, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("uniscreen quasipoisson point estimates match poisson", {
    
    result_quasi <- uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = "age",
        model_type = "glm",
        family = "quasipoisson",
        keep_models = TRUE
    )
    
    result_pois <- uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = "age",
        model_type = "glm",
        family = "poisson",
        keep_models = TRUE
    )
    
    ## Point estimates should match
    raw_quasi <- attr(result_quasi, "raw_data")
    raw_pois <- attr(result_pois, "raw_data")
    
    expect_equal(raw_quasi$RR[1], raw_pois$RR[1], tolerance = 1e-6)
})


## ----------------------------------------------------------------------------
## Gamma Family Tests
## ----------------------------------------------------------------------------

test_that("uniscreen works with Gamma family", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(uniscreen(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        family = Gamma(link = "log")
    ))
    
    expect_uniscreen_result(result)
})


## ----------------------------------------------------------------------------
## Inverse Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("uniscreen works with inverse.gaussian family", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(uniscreen(
        data = test_data,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        family = inverse.gaussian(link = "log")
    ))
    
    expect_uniscreen_result(result)
})


## ----------------------------------------------------------------------------
## Negative Binomial (negbin) Tests
## ----------------------------------------------------------------------------

test_that("uniscreen works with negative binomial regression (negbin)", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = standard_predictors,
        model_type = "negbin",
        parallel = FALSE
    ))
    
    expect_uniscreen_result(result)
    
    ## Should produce rate ratios
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for negbin, found:", 
                             paste(col_names, collapse = ", ")))
    
    ## All predictors should be represented
    expect_true(any(grepl("age|Age", result$Variable)))
    expect_true(any(grepl("sex|Sex", result$Variable)))
})


test_that("uniscreen negbin p_threshold filtering works", {
    
    skip_if_not_installed("MASS")
    
    ## All results
    result_all <- suppressWarnings(uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage", "ecog"),
        model_type = "negbin",
        p_threshold = 1,
        parallel = FALSE
    ))
    
    ## Filtered to p < 0.20
    result_filtered <- suppressWarnings(uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment", "stage", "ecog"),
        model_type = "negbin",
        p_threshold = 0.20,
        parallel = FALSE
    ))
    
    expect_lte(nrow(result_filtered), nrow(result_all))
})


test_that("uniscreen negbin with labels works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex", "treatment"),
        model_type = "negbin",
        labels = clintrial_labels,
        parallel = FALSE
    ))
    
    expect_uniscreen_result(result)
    expect_true(any(grepl("Age|Sex|Treatment", result$Variable)))
})


test_that("uniscreen negbin keep_models stores negbin objects", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(uniscreen(
        data = clintrial,
        outcome = "count_outcome",
        predictors = c("age", "sex"),
        model_type = "negbin",
        keep_models = TRUE,
        parallel = FALSE
    ))
    
    models <- attr(result, "models")
    expect_type(models, "list")
    expect_true(length(models) >= 2)
    expect_true(inherits(models[[1]], "negbin"))
})


test_that("uniscreen negbin coefficients match direct model", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(uniscreen(
        data = clintrial_complete,
        outcome = "count_outcome",
        predictors = "age",
        model_type = "negbin",
        keep_models = TRUE,
        parallel = FALSE
    ))
    
    ## Fit directly
    direct_model <- suppressWarnings(MASS::glm.nb(count_outcome ~ age, data = clintrial_complete))
    
    ## Get raw data
    raw <- attr(result, "raw_data")
    
    ## RR should match exp(coefficient)
    expected_rr <- exp(coef(direct_model)["age"])
    expect_equal(raw$RR[1], unname(expected_rr), tolerance = 1e-6)
})


## ============================================================================
## SECTION 1B: uniscreen() Input Validation Tests
## ============================================================================

test_that("uniscreen errors or auto-corrects Surv outcome with GLM model_type", {
    
    ## Should either auto-correct (with message) or error
    result_or_error <- tryCatch({
        suppressMessages(suppressWarnings(
            uniscreen(
                data = clintrial,
                outcome = "Surv(os_months, os_status)",
                predictors = c("age", "sex"),
                model_type = "glm"
            )
        ))
    }, error = function(e) e)
    
    ## Either produces valid result or an error (both acceptable)
    expect_true(
        inherits(result_or_error, "uniscreen_result") ||
        inherits(result_or_error, "error")
    )
})


test_that("uniscreen errors when coxph used without Surv outcome", {
    
    ## Should error - either from validation or from coxph itself
    expect_error(
        suppressWarnings(
            uniscreen(
                data = clintrial,
                outcome = "os_status",
                predictors = c("age", "sex"),
                model_type = "coxph"
            )
        )
    )
})


test_that("uniscreen errors with continuous outcome and binomial family", {
    
    ## Should error - either from validation or from glm itself
    expect_error(
        suppressWarnings(
            uniscreen(
                data = clintrial,
                outcome = "los_days",
                predictors = c("age", "sex"),
                model_type = "glm",
                family = "binomial"
            )
        )
    )
})


test_that("uniscreen errors when outcome variable not found", {
    
    expect_error(
        suppressWarnings(
            uniscreen(
                data = clintrial,
                outcome = "nonexistent_variable",
                predictors = c("age", "sex"),
                model_type = "glm"
            )
        )
    )
})


test_that("uniscreen errors when predictor variable not found", {
    
    expect_error(
        suppressWarnings(
            uniscreen(
                data = clintrial,
                outcome = "os_status",
                predictors = c("age", "nonexistent_predictor"),
                model_type = "glm"
            )
        )
    )
})


test_that("uniscreen handles edge case p_threshold values", {
    
    ## p_threshold = 1 should include all predictors
    result <- uniscreen(
        data = clintrial,
        outcome = "os_status",
        predictors = c("age", "sex"),
        model_type = "glm",
        p_threshold = 1.0
    )
    expect_uniscreen_result(result)
})


test_that("uniscreen errors with empty data", {
    
    empty_data <- clintrial[0, ]
    
    expect_error(
        suppressWarnings(
            uniscreen(
                data = empty_data,
                outcome = "os_status",
                predictors = c("age", "sex"),
                model_type = "glm"
            )
        )
    )
})


test_that("uniscreen handles binary outcome with lm model_type", {
    
    ## Linear probability model - should work (may warn)
    result <- suppressWarnings(
        uniscreen(
            data = clintrial,
            outcome = "os_status",
            predictors = c("age", "sex"),
            model_type = "lm"
        )
    )
    
    expect_uniscreen_result(result)
})


## ============================================================================
## SECTION 2: multifit() Basic Functionality Tests
## ============================================================================

test_that("multifit works with basic logistic regression - unadjusted", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "columns"), "unadjusted")
    expect_effect_column(result, "OR")
    expect_pvalue_column(result)
    
    ## Should have rows for each outcome
    expect_equal(length(unique(result$Outcome)), 3)
})


test_that("multifit works with basic logistic regression - adjusted", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "columns"), "adjusted")
    expect_effect_column(result, "aOR")
    
    ## Should store covariates attribute
    expect_equal(attr(result, "covariates"), c("age", "sex", "stage"))
})


test_that("multifit works with columns = 'both'", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        columns = "both",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "columns"), "both")
    
    ## Should have both p-value columns
    col_names <- names(result)
    expect_true("Uni p" %in% col_names)
    expect_true("Multi p" %in% col_names)
})


test_that("multifit handles continuous predictor", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "age",
        covariates = c("sex", "stage"),
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## One row per outcome for continuous predictor
    expect_equal(nrow(result), 3)
})


test_that("multifit handles factor predictor with multiple levels", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## Should have Predictor column with levels
    expect_true("Predictor" %in% names(result))
})


test_that("multifit show_n and show_events parameters work", {
    
    result_full <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        show_n = TRUE,
        show_events = TRUE,
        parallel = FALSE
    )
    
    expect_true("n" %in% names(result_full))
    expect_true("Events" %in% names(result_full))
    
    result_no_n <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        show_n = FALSE,
        show_events = FALSE,
        parallel = FALSE
    )
    
    expect_false("n" %in% names(result_no_n))
    expect_false("Events" %in% names(result_no_n))
})


test_that("multifit respects digits and p_digits parameters", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        digits = 3,
        p_digits = 4,
        parallel = FALSE
    )
    
    expect_multifit_result(result)
})


## ============================================================================
## SECTION 3: Cox Proportional Hazards Tests
## ============================================================================

test_that("uniscreen works with Cox model", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = standard_predictors,
        model_type = "coxph"
    )
    
    expect_uniscreen_result(result)
    expect_effect_column(result, "HR")
})


test_that("multifit works with Cox model", {
    
    result <- multifit(
        data = clintrial,
        outcomes = surv_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "coxph",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_effect_column(result, "HR")
})


test_that("multifit Cox with strata parameter", {
    
    result <- multifit(
        data = clintrial,
        outcomes = surv_outcomes[1],
        predictor = "treatment",
        covariates = c("age", "sex"),
        strata = "site",
        model_type = "coxph",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "strata"), "site")
})


test_that("multifit Cox with cluster parameter", {
    
    result <- multifit(
        data = clintrial,
        outcomes = surv_outcomes[1],
        predictor = "treatment",
        covariates = c("age", "sex"),
        cluster = "site",
        model_type = "coxph",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "cluster"), "site")
})


test_that("strata parameter validation for non-Cox models", {
    
    expect_error(
        multifit(clintrial, binary_outcomes, "treatment", 
                 strata = "site", model_type = "glm", parallel = FALSE),
        "'strata' is only supported for coxph"
    )
})


test_that("cluster parameter validation for non-Cox models", {
    
    expect_error(
        multifit(clintrial, binary_outcomes, "treatment", 
                 cluster = "site", model_type = "glm", parallel = FALSE),
        "'cluster' is only supported for coxph"
    )
})


## ============================================================================
## SECTION 4: Linear Model Tests
## ============================================================================

test_that("uniscreen works with linear model", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "los_days",
        predictors = standard_predictors,
        model_type = "lm"
    )
    
    expect_uniscreen_result(result)
    expect_effect_column(result, "Coefficient")
})


test_that("multifit works with linear model", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "los_days",
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        model_type = "lm",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_effect_column(result, "Coefficient")
    
    ## Events column should not be present for LM
    expect_false("Events" %in% names(result))
})


## ============================================================================
## SECTION 5: Poisson Regression Tests
## ============================================================================

test_that("multifit works with Poisson regression", {
    
    ## Create count variable
    clintrial$los_count <- ceiling(clintrial$los_days / 5)
    
    result <- multifit(
        data = clintrial,
        outcomes = "los_count",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = "poisson",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_effect_column(result, "RR")
})


## ----------------------------------------------------------------------------
## Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("multifit works with gaussian family", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "los_days",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = "gaussian",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## Should produce coefficients
    col_names <- names(result)
    expect_true(any(grepl("Coefficient.*CI|Estimate.*CI", col_names)),
                info = paste("Expected Coefficient column for gaussian, found:", 
                             paste(col_names, collapse = ", ")))
})


## ----------------------------------------------------------------------------
## Quasibinomial Family Tests
## ----------------------------------------------------------------------------

test_that("multifit works with quasibinomial family", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = "quasibinomial",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## Should produce odds ratios
    col_names <- names(result)
    expect_true(any(grepl("OR.*CI", col_names)),
                info = paste("Expected OR column for quasibinomial, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("multifit quasibinomial with columns='both' works", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = "quasibinomial",
        columns = "both",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## Should have both OR and aOR columns
    col_names <- names(result)
    expect_true(any(grepl("^OR", col_names)) && any(grepl("^aOR", col_names)))
})


## ----------------------------------------------------------------------------
## Quasipoisson Family Tests
## ----------------------------------------------------------------------------

test_that("multifit works with quasipoisson family", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = "quasipoisson",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## Should produce rate ratios
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for quasipoisson, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("multifit quasipoisson with columns='both' works", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = "quasipoisson",
        columns = "both",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    
    ## Should have both RR and aRR columns
    col_names <- names(result)
    expect_true(any(grepl("^RR", col_names)) && any(grepl("^aRR", col_names)))
})


## ----------------------------------------------------------------------------
## Gamma Family Tests
## ----------------------------------------------------------------------------

test_that("multifit works with Gamma family", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(multifit(
        data = test_data,
        outcomes = "los_days",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = Gamma(link = "log"),
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
})


## ----------------------------------------------------------------------------
## Inverse Gaussian Family Tests
## ----------------------------------------------------------------------------

test_that("multifit works with inverse.gaussian family", {
    
    ## Create positive outcome
    test_data <- data.table::as.data.table(clintrial)
    test_data <- test_data[los_days > 0]
    
    result <- suppressMessages(multifit(
        data = test_data,
        outcomes = "los_days",
        predictor = "treatment",
        covariates = c("age", "sex"),
        family = inverse.gaussian(link = "log"),
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
})


## ----------------------------------------------------------------------------
## Negative Binomial (negbin) Tests
## ----------------------------------------------------------------------------

test_that("multifit works with negative binomial regression (negbin)", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "negbin",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    
    ## Should produce rate ratios
    col_names <- names(result)
    expect_true(any(grepl("RR.*CI", col_names)),
                info = paste("Expected RR column for negbin, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("multifit negbin works with multiple outcomes", {
    
    skip_if_not_installed("MASS")
    
    ## Create second count variable
    clintrial$count_outcome2 <- as.integer(ceiling(clintrial$los_days / 5))
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = c("count_outcome", "count_outcome2"),
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "negbin",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    
    ## Should have rows for both outcomes
    expect_equal(length(unique(result$Outcome)), 2)
})


test_that("multifit negbin works with columns = 'both'", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "negbin",
        columns = "both",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    
    ## Should have unadjusted (RR) and adjusted (aRR) columns
    col_names <- names(result)
    expect_true(any(grepl("^RR", col_names)) && any(grepl("^aRR", col_names)),
                info = paste("Expected both RR and aRR columns, found:", 
                             paste(col_names, collapse = ", ")))
})


test_that("multifit negbin with interactions works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        interactions = c("treatment:stage"),
        model_type = "negbin",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    expect_equal(attr(result, "interactions"), "treatment:stage")
})


test_that("multifit negbin with labels works", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "negbin",
        labels = clintrial_labels,
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    
    ## Labels should be applied
    expect_true(any(grepl("Treatment", result$Predictor)))
})


test_that("multifit negbin keep_models stores negbin objects", {
    
    skip_if_not_installed("MASS")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "count_outcome",
        predictor = "treatment",
        covariates = c("age", "sex"),
        model_type = "negbin",
        keep_models = TRUE,
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    
    models <- attr(result, "models")
    expect_type(models, "list")
    
    ## Should have negbin model
    adj_model <- models$count_outcome$adjusted
    expect_true(inherits(adj_model, "negbin"))
})


## ============================================================================
## SECTION 6: Interaction Terms Tests
## ============================================================================

test_that("multifit works with factor * factor interaction", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        interactions = c("treatment:stage"),
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "interactions"), "treatment:stage")
    
    ## Interaction terms should contain * symbol
    expect_true(any(grepl("[:×*]", result$Predictor)))
})


test_that("multifit works with continuous * factor interaction", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "age",
        covariates = c("sex", "stage"),
        interactions = c("age:sex"),
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_true(any(grepl("[:×*]", result$Predictor)))
})


test_that("multifit works with multiple interactions", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        interactions = c("treatment:sex", "treatment:stage"),
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(attr(result, "interactions"), c("treatment:sex", "treatment:stage"))
})


test_that("Cox model with interaction works", {
    
    result <- multifit(
        data = clintrial,
        outcomes = surv_outcomes[1],
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        interactions = c("treatment:stage"),
        model_type = "coxph",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_true(any(grepl("[:×*]", result$Predictor)))
})


## ============================================================================
## SECTION 7: Mixed Effects Models Tests
## ============================================================================

test_that("multifit works with glmer", {
    skip_if_not_installed("lme4")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = binary_outcomes[1],
        predictor = "treatment",
        random = "(1|site)",
        model_type = "glmer",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
    expect_equal(attr(result, "random"), "(1|site)")
})


test_that("multifit works with lmer", {
    skip_if_not_installed("lme4")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "los_days",
        predictor = "treatment",
        random = "(1|site)",
        model_type = "lmer",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
})


test_that("multifit works with nested random effects", {
    skip_if_not_installed("lme4")
    
    ## Create nested structure
    clintrial$patient_id <- 1:nrow(clintrial)
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        random = "(1|site)",
        model_type = "glmer",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
})


test_that("multifit works with coxme", {
    skip_if_not_installed("coxme")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = surv_outcomes[1],
        predictor = "treatment",
        random = "(1|site)",
        model_type = "coxme",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
})


test_that("glmer with interaction and random effects works", {
    skip_if_not_installed("lme4")
    
    result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        covariates = c("age", "sex"),
        interactions = c("treatment:sex"),
        random = "(1|site)",
        model_type = "glmer",
        parallel = FALSE
    ))
    
    expect_multifit_result(result)
})


## ============================================================================
## SECTION 8: Labels Tests
## ============================================================================

test_that("uniscreen applies labels to Variable column", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        labels = clintrial_labels
    )
    
    ## Labels should appear in Variable column
    has_labels <- any(sapply(clintrial_labels, function(l) any(grepl(l, result$Variable))))
    expect_true(has_labels)
})


test_that("multifit applies labels to Outcome column", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        labels = c(
            surgery = "Surgical Resection",
            pfs_status = "Progression Event",
            os_status = "Death"
        ),
        parallel = FALSE
    )
    
    expect_true("Surgical Resection" %in% result$Outcome)
    expect_true("Progression Event" %in% result$Outcome)
    expect_true("Death" %in% result$Outcome)
})


test_that("multifit predictor_label parameter works", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "age",
        predictor_label = "Patient Age (years)",
        parallel = FALSE
    )
    
    ## predictor_label affects formatting, not necessarily stored as attribute
    ## Just verify the function runs without error
    expect_multifit_result(result)
})


## ============================================================================
## SECTION 9: P-value Filtering Tests
## ============================================================================

test_that("multifit p_threshold filtering works", {
    
    result_all <- multifit(
        data = clintrial,
        outcomes = c("surgery", "pfs_status", "os_status"),
        predictor = "age",
        covariates = c("sex"),
        p_threshold = 1,
        parallel = FALSE
    )
    
    result_filtered <- multifit(
        data = clintrial,
        outcomes = c("surgery", "pfs_status", "os_status"),
        predictor = "age",
        covariates = c("sex"),
        p_threshold = 0.05,
        parallel = FALSE
    )
    
    expect_lte(nrow(result_filtered), nrow(result_all))
})


## ============================================================================
## SECTION 10: Keep Models Tests
## ============================================================================

test_that("uniscreen keep_models stores all models", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm",
        keep_models = TRUE
    )
    
    models <- attr(result, "models")
    expect_type(models, "list")
    expect_true(length(models) >= 3)
})


test_that("multifit keep_models stores model objects", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes[1:2],
        predictor = "treatment",
        covariates = c("age", "sex"),
        keep_models = TRUE,
        parallel = FALSE
    )
    
    models <- attr(result, "models")
    expect_type(models, "list")
    expect_equal(length(models), 2)
})


test_that("stored models can be used for predictions", {
    
    result <- multifit(
        data = clintrial_complete,
        outcomes = "surgery",
        predictor = "treatment",
        covariates = c("age", "sex"),
        keep_models = TRUE,
        parallel = FALSE
    )
    
    models <- attr(result, "models")
    model <- models$surgery$adjusted
    
    preds <- predict(model, type = "response")
    expect_true(length(preds) > 0)
    expect_true(all(preds >= 0 & preds <= 1))
})


## ============================================================================
## SECTION 11: Raw Data Attribute Tests
## ============================================================================

test_that("uniscreen raw_data contains expected columns", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "treatment"),
        model_type = "glm"
    )
    
    expect_uniscreen_raw_data(result)
    
    raw <- attr(result, "raw_data")
    expect_true("p_value" %in% names(raw))
})


test_that("multifit raw_data contains expected columns", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "age",
        parallel = FALSE
    )
    
    expect_multifit_raw_data(result)
    
    raw <- attr(result, "raw_data")
    expect_true("outcome" %in% names(raw))
    expect_true("predictor" %in% names(raw))
})


test_that("multifit raw_data has correct structure for 'both' columns", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        columns = "both",
        parallel = FALSE
    )
    
    raw <- attr(result, "raw_data")
    
    ## Should have both adjusted and unadjusted columns
    expect_true("exp_coef_unadj" %in% names(raw))
    expect_true("exp_coef_adj" %in% names(raw))
})


## ============================================================================
## SECTION 12: uniforest() Tests
## ============================================================================

test_that("uniforest creates forest plot from uniscreen result", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm"
    )
    
    p <- uniforest(table, title = "Univariable Screening")
    
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "rec_dims")))
})


test_that("uniforest works with Cox model results", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "treatment", "stage"),
        model_type = "coxph"
    )
    
    p <- uniforest(table, title = "Survival Analysis")
    
    expect_s3_class(p, "ggplot")
})


test_that("uniforest works with linear model results", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "sex", "treatment"),
        model_type = "lm"
    )
    
    p <- uniforest(table, title = "Linear Model")
    
    expect_s3_class(p, "ggplot")
})


test_that("uniforest show_n and show_events parameters work", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    p1 <- uniforest(table, show_n = TRUE, show_events = TRUE)
    p2 <- uniforest(table, show_n = FALSE, show_events = FALSE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("uniforest indent_groups parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("treatment", "stage"),
        model_type = "glm"
    )
    
    p1 <- uniforest(table, indent_groups = FALSE)
    p2 <- uniforest(table, indent_groups = TRUE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("uniforest bold_variables parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    p1 <- uniforest(table, bold_variables = TRUE)
    p2 <- uniforest(table, bold_variables = FALSE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("uniforest zebra_stripes parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm"
    )
    
    p1 <- uniforest(table, zebra_stripes = TRUE)
    p2 <- uniforest(table, zebra_stripes = FALSE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("uniforest custom color works", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    p <- uniforest(table, color = "#FF5733")
    
    expect_s3_class(p, "ggplot")
})


test_that("uniforest labels parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm"
    )
    
    p <- uniforest(table, labels = clintrial_labels)
    
    expect_s3_class(p, "ggplot")
})


## ============================================================================
## SECTION 13: multiforest() Tests
## ============================================================================

test_that("multiforest creates forest plot from multifit result", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        parallel = FALSE
    )
    
    p <- multiforest(table, title = "Multi-Outcome Analysis")
    
    expect_s3_class(p, "ggplot")
    expect_true(!is.null(attr(p, "rec_dims")))
})


test_that("multiforest works with adjusted results", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        parallel = FALSE
    )
    
    p <- multiforest(table, column = "adjusted", covariates_footer = TRUE)
    
    expect_s3_class(p, "ggplot")
})


test_that("multiforest works with survival outcomes", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = surv_outcomes,
        predictor = "treatment",
        model_type = "coxph",
        parallel = FALSE
    )
    
    p <- multiforest(table)
    
    expect_s3_class(p, "ggplot")
})


test_that("multiforest works with linear model outcomes", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = c("los_days", "biomarker_x"),
        predictor = "treatment",
        model_type = "lm",
        parallel = FALSE
    )
    
    p <- multiforest(table)
    
    expect_s3_class(p, "ggplot")
})


test_that("multiforest bold_variables parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        parallel = FALSE
    )
    
    p1 <- multiforest(table, bold_variables = TRUE)
    p2 <- multiforest(table, bold_variables = FALSE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("multiforest indent_predictor parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes[1:2],
        predictor = "treatment",
        parallel = FALSE
    )
    
    p1 <- multiforest(table, indent_predictor = FALSE)
    p2 <- multiforest(table, indent_predictor = TRUE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("multiforest zebra_stripes parameter works", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        parallel = FALSE
    )
    
    p1 <- multiforest(table, zebra_stripes = TRUE)
    p2 <- multiforest(table, zebra_stripes = FALSE)
    
    expect_s3_class(p1, "ggplot")
    expect_s3_class(p2, "ggplot")
})


test_that("multiforest custom color works", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes[1:2],
        predictor = "treatment",
        parallel = FALSE
    )
    
    p <- multiforest(table, color = "#E74C3C")
    
    expect_s3_class(p, "ggplot")
})


test_that("multiforest with labels works", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        labels = clintrial_labels,
        parallel = FALSE
    )
    
    p <- multiforest(table, labels = clintrial_labels)
    
    expect_s3_class(p, "ggplot")
})


test_that("multiforest with mixed-effects model results works", {
    skip_if_not_installed("lme4")
    skip_if_not_installed("ggplot2")
    
    table <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = binary_outcomes[1:2],
        predictor = "treatment",
        random = "(1|site)",
        model_type = "glmer",
        parallel = FALSE
    ))
    
    p <- multiforest(table)
    
    expect_s3_class(p, "ggplot")
})


## ============================================================================
## SECTION 14: Input Validation Tests
## ============================================================================

test_that("uniscreen validates required inputs", {
    
    expect_error(
        uniscreen(predictors = c("age"), model_type = "glm"),
        "data.*required|missing"
    )
})


test_that("uniscreen errors on non-existent predictor", {
    
    expect_error(
        uniscreen(
            data = clintrial,
            outcome = "surgery",
            predictors = c("nonexistent_var"),
            model_type = "glm"
        )
    )
})


test_that("multifit validates required inputs", {
    
    expect_error(
        multifit(outcomes = binary_outcomes, predictor = "treatment", parallel = FALSE),
        "'data' is required"
    )
    
    expect_error(
        multifit(data = clintrial, predictor = "treatment", parallel = FALSE),
        "'outcomes' is required"
    )
    
    expect_error(
        multifit(data = clintrial, outcomes = binary_outcomes, parallel = FALSE),
        "'predictor' is required"
    )
})


test_that("multifit validates predictor length", {
    
    expect_error(
        multifit(clintrial, binary_outcomes, c("age", "sex"), parallel = FALSE),
        "'predictor' must be a single variable name"
    )
})


test_that("multifit errors on missing random for glmer", {
    skip_if_not_installed("lme4")
    
    expect_error(
        multifit(
            data = clintrial,
            outcomes = c("surgery"),
            predictor = "treatment",
            model_type = "glmer",
            parallel = FALSE
        ),
        regexp = "random.*required"
    )
})


test_that("multifit warns on non-existent outcome variable", {
    
    ## multifit uses tryCatch and warns instead of erroring
    expect_warning(
        multifit(
            data = clintrial,
            outcomes = c("nonexistent_var"),
            predictor = "treatment",
            model_type = "glm",
            parallel = FALSE
        ),
        regexp = "failed|not found"
    )
})


test_that("uniforest errors on non-uniscreen input", {
    skip_if_not_installed("ggplot2")
    
    expect_error(uniforest(data.frame(x = 1:5)))
})


test_that("multiforest errors on non-multifit input", {
    skip_if_not_installed("ggplot2")
    
    expect_error(multiforest(data.frame(x = 1:5)))
})


## ============================================================================
## SECTION 15: Edge Cases
## ============================================================================

test_that("uniscreen handles single predictor with labels", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm"
    )
    
    expect_uniscreen_result(result)
})


test_that("multifit handles single outcome", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        parallel = FALSE
    )
    
    expect_multifit_result(result)
    expect_equal(length(unique(result$Outcome)), 1)
})


test_that("uniscreen handles missing data appropriately", {
    
    dt_missing <- data.table::as.data.table(clintrial)
    set.seed(123)
    dt_missing[sample(1:.N, 20), age := NA]
    
    expect_no_error({
        result <- uniscreen(
            data = dt_missing,
            outcome = "surgery",
            predictors = c("age", "sex"),
            model_type = "glm"
        )
    })
})


## ============================================================================
## SECTION 16: Integration Tests
## ============================================================================

test_that("full pipeline: uniscreen -> uniforest", {
    skip_if_not_installed("ggplot2")
    
    ## Screen predictors
    screen_results <- uniscreen(
        data = clintrial,
        outcome = "Surv(os_months, os_status)",
        predictors = c("age", "sex", "treatment", "stage", "ecog"),
        model_type = "coxph",
        labels = clintrial_labels
    )
    
    ## Create forest plot
    p <- uniforest(
        screen_results,
        title = "Univariable Screening: Overall Survival",
        show_n = TRUE,
        show_events = TRUE,
        zebra_stripes = TRUE,
        bold_variables = TRUE
    )
    
    expect_uniscreen_result(screen_results)
    expect_s3_class(p, "ggplot")
})


test_that("full pipeline: multifit -> multiforest with covariates", {
    skip_if_not_installed("ggplot2")
    
    ## Multi-outcome analysis
    multi_results <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        labels = clintrial_labels,
        parallel = FALSE
    )
    
    ## Create forest plot
    p <- multiforest(
        multi_results,
        title = "Treatment Effects Across Outcomes",
        covariates_footer = TRUE,
        zebra_stripes = TRUE,
        bold_variables = TRUE
    )
    
    expect_multifit_result(multi_results)
    expect_s3_class(p, "ggplot")
})


test_that("full pipeline: multifit with interactions -> multiforest", {
    skip_if_not_installed("ggplot2")
    
    ## Multi-outcome analysis with interaction
    multi_results <- multifit(
        data = clintrial,
        outcomes = binary_outcomes[1:2],
        predictor = "treatment",
        covariates = c("age", "sex"),
        interactions = c("treatment:sex"),
        labels = clintrial_labels,
        parallel = FALSE
    )
    
    ## Create forest plot
    p <- multiforest(
        multi_results,
        title = "Treatment * Sex Interaction",
        zebra_stripes = TRUE
    )
    
    expect_multifit_result(multi_results)
    expect_s3_class(p, "ggplot")
})


test_that("full pipeline: multifit with mixed effects -> multiforest", {
    skip_if_not_installed("lme4")
    skip_if_not_installed("ggplot2")
    
    ## Multi-outcome analysis with random effects
    multi_results <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = binary_outcomes[1:2],
        predictor = "treatment",
        covariates = c("age", "sex"),
        random = "(1|site)",
        model_type = "glmer",
        labels = clintrial_labels,
        parallel = FALSE
    ))
    
    ## Create forest plot
    p <- multiforest(
        multi_results,
        title = "Treatment Effects with Random Effects",
        covariates_footer = TRUE,
        zebra_stripes = TRUE,
        bold_variables = TRUE
    )
    
    expect_multifit_result(multi_results)
    expect_s3_class(p, "ggplot")
})


## ============================================================================
## SECTION 17: Parallel Processing Tests
## ============================================================================

## test_that("parallel processing produces same results as sequential", {
##     skip_on_cran()
##     
##     set.seed(123)
##     result_seq <- multifit(
##         data = clintrial,
##         outcomes = binary_outcomes,
##         predictor = "treatment",
##         covariates = c("age", "sex"),
##         parallel = FALSE
##     )
##     
##     set.seed(123)
##     result_par <- multifit(
##         data = clintrial,
##         outcomes = binary_outcomes,
##         predictor = "treatment",
##         covariates = c("age", "sex"),
##         parallel = TRUE,
##         n_cores = 2
##     )
##     
##     ## Results should be identical (may differ in row order)
##     expect_equal(nrow(result_seq), nrow(result_par))
##     expect_equal(sort(result_seq$Outcome), sort(result_par$Outcome))
## })


## ============================================================================
## SECTION 18: Attribute Preservation Tests
## ============================================================================

test_that("uniscreen preserves all expected attributes", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    expect_true(!is.null(attr(result, "raw_data")))
    expect_true(!is.null(attr(result, "outcome")))
    expect_true(!is.null(attr(result, "model_type")))
    expect_true(!is.null(attr(result, "model_scope")))
})


test_that("multifit preserves all expected attributes", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        interactions = c("treatment:sex"),
        parallel = FALSE
    )
    
    expect_equal(attr(result, "predictor"), "treatment")
    expect_equal(attr(result, "outcomes"), binary_outcomes)
    expect_equal(attr(result, "covariates"), c("age", "sex"))
    expect_equal(attr(result, "interactions"), c("treatment:sex"))
    expect_equal(attr(result, "model_type"), "glm")
})


test_that("uniforest rec_dims attribute present", {
    skip_if_not_installed("ggplot2")
    
    table <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex"),
        model_type = "glm"
    )
    
    p <- uniforest(table)
    
    dims <- attr(p, "rec_dims")
    expect_true(!is.null(dims))
    expect_true("width" %in% names(dims))
    expect_true("height" %in% names(dims))
})


test_that("multiforest rec_dims attribute present", {
    skip_if_not_installed("ggplot2")
    
    table <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        parallel = FALSE
    )
    
    p <- multiforest(table)
    
    dims <- attr(p, "rec_dims")
    expect_true(!is.null(dims))
    expect_true("width" %in% names(dims))
    expect_true("height" %in% names(dims))
})


## ============================================================================
## SECTION 19: Print Method Tests
## ============================================================================

test_that("uniscreen print method displays correctly", {
    
    result <- uniscreen(
        data = clintrial,
        outcome = "surgery",
        predictors = c("age", "sex", "treatment"),
        model_type = "glm"
    )
    
    output <- capture.output(print(result))
    
    expect_true(any(grepl("Univariable", output)))
})


test_that("multifit print method displays correctly", {
    
    result <- multifit(
        data = clintrial,
        outcomes = binary_outcomes,
        predictor = "treatment",
        covariates = c("age", "sex"),
        parallel = FALSE
    )
    
    output <- capture.output(print(result))
    
    expect_true(any(grepl("Multivariate|Multi", output)))
    expect_true(any(grepl("Predictor", output)))
})


test_that("multifit print method shows interactions when present", {
    
    result <- multifit(
        data = clintrial,
        outcomes = "surgery",
        predictor = "treatment",
        covariates = c("stage"),
        interactions = c("treatment:stage"),
        parallel = FALSE
    )
    
    output <- capture.output(print(result))
    expect_true(any(grepl("Interaction", output)))
})


## ============================================================================
## SECTION 20: Coefficient Accuracy Tests
## ============================================================================

test_that("multifit coefficients match direct model fitting", {
    
    ## Fit using multifit
    result <- multifit(
        data = clintrial_complete,
        outcomes = "surgery",
        predictor = "treatment",
        covariates = c("age", "sex"),
        keep_models = TRUE,
        parallel = FALSE
    )
    
    ## Fit directly
    direct_model <- glm(surgery ~ treatment + age + sex, 
                        data = clintrial_complete, family = binomial)
    
    ## Get multifit model coefficients
    mf_model <- attr(result, "models")$surgery$adjusted
    mf_coefs <- coef(mf_model)
    direct_coefs <- coef(direct_model)
    
    ## Coefficients should match
    expect_equal(mf_coefs, direct_coefs, tolerance = 1e-10)
})


test_that("uniscreen coefficients match direct model fitting", {
    
    ## Fit using uniscreen
    result <- uniscreen(
        data = clintrial_complete,
        outcome = "surgery",
        predictors = "age",
        model_type = "glm",
        keep_models = TRUE
    )
    
    ## Fit directly
    direct_model <- glm(surgery ~ age, data = clintrial_complete, family = binomial)
    
    ## Get raw data
    raw <- attr(result, "raw_data")
    
    ## OR should match exp(coefficient)
    expected_or <- exp(coef(direct_model)["age"])
    expect_equal(raw$OR[1], unname(expected_or), tolerance = 1e-6)
})


## ============================================================================
## SECTION 23: Profile Likelihood CIs in Multifit
## ============================================================================

test_that("multifit GLM CIs use profile likelihood", {
    
    ## Fit using multifit (uses extract_predictor_results internally)
    result <- multifit(
        data = clintrial_complete,
        outcomes = "surgery",
        predictor = "age",
        model_type = "glm",
        keep_models = TRUE,
        parallel = FALSE
    )
    
    raw <- attr(result, "raw_data")
    
    ## Get the model for direct comparison
    model <- attr(result, "models")$surgery$unadjusted
    
    ## Compute Wald CIs
    coefs <- coef(model)
    ses <- summary(model)$coefficients[, "Std. Error"]
    z <- qnorm(0.975)
    wald_lower <- coefs["age"] - z * ses["age"]
    
    ## Profile CIs should differ from Wald
    profile_lower <- raw$coefficient[1] - (raw$exp_coef[1] - raw$ci_lower[1]) / raw$exp_coef[1] * raw$coefficient[1]
    
    ## Just verify CIs are present and valid
    expect_true(!is.na(raw$ci_lower[1]))
    expect_true(!is.na(raw$ci_upper[1]))
    expect_true(raw$ci_lower[1] < raw$ci_upper[1])
})


test_that("multifit quasi-family CIs differ from non-quasi CIs", {
    
    ## Quasibinomial should use Wald CIs
    result_quasi <- multifit(
        data = clintrial_complete,
        outcomes = "surgery",
        predictor = "age",
        model_type = "glm",
        family = "quasibinomial",
        parallel = FALSE
    )
    
    ## Binomial should use profile CIs
    result_binom <- multifit(
        data = clintrial_complete,
        outcomes = "surgery",
        predictor = "age",
        model_type = "glm",
        family = "binomial",
        parallel = FALSE
    )
    
    raw_quasi <- attr(result_quasi, "raw_data")
    raw_binom <- attr(result_binom, "raw_data")
    
    ## Point estimates should match
    expect_equal(raw_quasi$exp_coef[1], raw_binom$exp_coef[1], tolerance = 1e-6)
    
    ## CIs may differ (profile vs Wald) - just verify both are valid
    expect_true(!is.na(raw_quasi$ci_lower[1]))
    expect_true(!is.na(raw_binom$ci_lower[1]))
    expect_true(raw_quasi$ci_lower[1] < raw_quasi$ci_upper[1])
    expect_true(raw_binom$ci_lower[1] < raw_binom$ci_upper[1])
})

## Compound tables place unadjusted and adjusted estimates side by side under a
## single sample size column. That column must describe the observations behind
## the adjusted estimate, since the adjusted model is fitted to complete cases
## across every covariate and is therefore the smaller sample. Reporting the
## unadjusted sample alongside an adjusted estimate overstates it.

test_that("multifit both-column tables report the adjusted sample size", {

    ## Missingness confined to a covariate, so the adjusted model is fitted to
    ## fewer observations than the unadjusted one.
    df <- fresh_clintrial()
    df$bmi[seq_len(120)] <- NA
    dt <- data.table::as.data.table(df)

    result <- multifit(
        data = dt,
        outcomes = "readmission_30d",
        predictor = "sex",
        covariates = c("age", "bmi"),
        columns = "both",
        model_type = "glm",
        family = "binomial"
    )

    raw <- attr(result, "raw_data")

    ## multifit reports one row per non-reference level, so the count for the
    ## single row is the size of that level within the adjusted sample.
    adj_model <- stats::glm(readmission_30d ~ sex + age + bmi,
                            data = dt, family = stats::binomial)
    adj_frame <- stats::model.frame(adj_model)
    expected <- sum(adj_frame$sex == "Male")

    expect_equal(as.numeric(raw[["n"]][1]), as.numeric(expected))

    ## The unadjusted sample is strictly larger here, so the two are
    ## distinguishable and the assertion above is meaningful.
    unadj_model <- stats::glm(readmission_30d ~ sex,
                              data = dt, family = stats::binomial)
    expect_gt(stats::nobs(unadj_model), stats::nobs(adj_model))
})

#' Simulated Clinical Trial Dataset
#'
#' A simulated dataset from a hypothetical multi-center oncology clinical trial
#' comparing two experimental drugs against control. Designed to demonstrate
#' the full capabilities of descriptive and regression analysis functions.
#'
#' @format A data frame with 850 observations and 32 variables:
#' \describe{
#'   \item{patient_id}{Unique patient identifier (character)}
#'   \item{age}{Age at enrollment in years (numeric: 18-90)}
#'   \item{sex}{Biological sex (factor: Female, Male)}
#'   \item{race}{Self-reported race (factor: White, Black, Asian, Other)}
#'   \item{ethnicity}{Hispanic ethnicity (factor: Non-Hispanic, Hispanic)}
#'   \item{bmi}{Body mass index in kg/m\eqn{^2} (numeric)}
#'   \item{smoking}{Smoking history (factor: Never, Former, Current)}
#'   \item{hypertension}{Hypertension diagnosis (factor: No, Yes)}
#'   \item{diabetes}{Diabetes diagnosis (factor: No, Yes)}
#'   \item{ecog}{ECOG performance status (factor: 0, 1, 2, 3)}
#'   \item{creatinine}{Baseline creatinine in mg/dL (numeric)}
#'   \item{hemoglobin}{Baseline hemoglobin in g/dL (numeric)}
#'   \item{biomarker_x}{Serum biomarker A in ng/mL (numeric)}
#'   \item{biomarker_y}{Serum biomarker B in U/L (numeric)}
#'   \item{site}{Enrolling site (factor: Site Alpha through Site Kappa)}
#'   \item{grade}{Tumor grade (factor: Well/Moderately/Poorly differentiated)}
#'   \item{stage}{Disease stage at diagnosis (factor: I, II, III, IV)}
#'   \item{treatment}{Randomized treatment (factor: Control, Drug A, Drug B)}
#'   \item{surgery}{Surgical resection (factor: No, Yes)}
#'   \item{any_complication}{Any post-operative complication (factor: No, Yes)}
#'   \item{wound_infection}{Post-operative wound infection (factor: No, Yes)}
#'   \item{icu_admission}{ICU admission required (factor: No, Yes)}
#'   \item{readmission_30d}{Hospital readmission within 30 days (factor: No, Yes)}
#'   \item{pain_score}{Pain score at discharge (numeric: 0-10)}
#'   \item{recovery_days}{Days to functional recovery (numeric)}
#'   \item{los_days}{Hospital length of stay in days (numeric)}
#'   \item{ae_count}{Adverse event count (integer). Overdispersed count suitable
#'     for negative binomial or quasipoisson regression.}
#'   \item{fu_count}{Follow-up visit count (integer). Equidispersed count
#'     suitable for standard Poisson regression.}
#'   \item{pfs_months}{Progression-Free Survival Time (months)}
#'   \item{pfs_status}{Progression or Death Event}
#'   \item{os_months}{Overall survival time in months (numeric)}
#'   \item{os_status}{Death indicator (numeric: 0=censored, 1=death)}
#' }
#' 
#' @details
#' This dataset includes realistic correlations between variables:
#' - Survival is worse with higher stage, ECOG, age, and biomarker_x
#' - Treatment effects show Drug B > Drug A > Control
#' - \code{ae_count} is overdispersed (variance > mean) for negative binomial demos
#' - \code{fu_count} is equidispersed (variance \eqn{\approx} mean) for Poisson demos
#' - Approximately 2\% of values are missing at random
#' - Median follow-up is approximately 30 months
#' 
#' @source Simulated data for demonstration purposes
#' @family sample data
#' 
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#' 
#' # Descriptive statistics by treatment arm
#' desctable(clintrial,
#'         by = "treatment", 
#'         variables = c("age", "sex", "stage", "ecog", 
#'                      "biomarker_x", "Surv(os_months, os_status)"),
#'         labels = clintrial_labels)
#'
#' \donttest{
#' # Poisson regression for equidispersed counts
#' fit(clintrial,
#'     outcome = "fu_count",
#'     predictors = c("age", "stage", "treatment"),
#'     model_type = "glm",
#'     family = "poisson",
#'     labels = clintrial_labels)
#' 
#' # Negative binomial for overdispersed counts
#' fit(clintrial,
#'     outcome = "ae_count",
#'     predictors = c("age", "treatment", "diabetes"),
#'     model_type = "negbin",
#'     labels = clintrial_labels)
#' 
#' # Complete analysis pipeline
#' fullfit(clintrial,
#'         outcome = "Surv(os_months, os_status)",
#'         predictors = c("age", "sex", "stage", "grade", "ecog",
#'                       "smoking", "biomarker_x", "biomarker_y", "treatment"),
#'         method = "screen",
#'         p_threshold = 0.20,
#'         model_type = "coxph",
#'         labels = clintrial_labels)
#' }
#'         
"clintrial"

#' Variable Labels for Clinical Trial Dataset  
#'
#' A named character vector providing descriptive labels for all variables
#' in the clinical_trial dataset. Use with labels parameter in functions.
#'
#' @format Named character vector with 24 elements
#' @family sample data
#' @seealso \code{\link{clintrial}}
"clintrial_labels"

#' Global Variables for Package Check
#'
#' This file declares global variables used in non-standard evaluation (NSE)
#' contexts throughout the package, primarily in data.table and ggplot2 operations.
#' These declarations prevent R CMD check warnings about "no visible binding for
#' global variable".
#'
#' @importFrom stats aov fivenum
#' @keywords internal
#' @noRd

.datatable.aware <- TRUE

utils::globalVariables(c(
           
           ## data.table special symbols
           ".",
           "..",
           ".BY",
           ".GRP",
           ".I",
           ".N",
           ".NGRP",
           ".SD",

           ## data.table column selection with .. prefix
           "..available_cols",
           "..cols",
           "..cols_available",
           "..cols_needed",
           "..complete_vars",
           "..display_cols",
           "..keep_cols",
           "..key_cols",
           "..new_order",
           
           ## Column names used in statistical models and summaries
           "Estimate",
           "Events",
           "Freq",
           "Group",
           "HR",
           "Multi p",
           "OR",
           "Predictor",
           "R2",
           "RR",
           "N",
           "Composite Model Score (CMS)",
           "Uni p",
           "Variable",
           "p-value",
           
           ## Variables used in analysis functions
           "CMS",
           ".int_term",
           ".is_ref",
           ".group_order",
           ".multi_n",
           ".multi_Events",
           ".orig_order",
           ".row_in_var",
           ".temp_var_order",
           ".var_group",
           ".var_group_id",
           "age",
           "all_nonempty",
           "breaks",
           "brier",
           "ci_lower",
           "ci_lower_adj",
           "ci_lower_unadj",
           "ci_upper",
           "ci_upper_adj",
           "ci_upper_unadj",
           "c_statistic",
           "clean_level",
           "coef",
           "coef_lower",
           "coef_upper",
           "coefficient",
           "coef_names",
           "conf_high",
           "conf_high_formatted",
           "conf_low",
           "conf_low_formatted",
           "current_term",
           "deviance",
           "display_n",
           "display_events",
           "display_text",
           "effect",
           "effect_col",
           "effect_formatted",
           "effect_label",
           "effect_string_expr",
           "estimate",
           "event_data",
           "event_var",
           "events",
           "events_calc",
           "events_formatted",
           "events_group",
           "exp_coef",
           "exp_coef_adj",
           "exp_coef_unadj",
           "exp_lower",
           "exp_upper",
           "group",
           "group_var",
           "groups",
           "grp",
           "grp_data",
           "grp_denom",
           "hr",
           "hr_formatted",
           "hr_string_expr",
           "i..orig_order",
           "inds",
           "i.var_new",
           "i.events_group",
           "i.n_group",
           "i.Freq",
           "integrated_loglik",
           "is_continuous",
           "is_header",
           "is_reference",
           "label",
           "level",
           "level_display",
           "level_order",
           "level_row",
           "level_term",
           "matching",
           "multi_effect",
           "multi_p",
           "n",
           "n_events",
           "n_obs",
           "null_deviance",
           "n_formatted",
           "n_group",
           "n_miss",
           "orig_level",
           "orig_var",
           "outcome",
           "outcome_var",
           "p_adj",
           "p_formatted",
           "p_unadj",
           "p_value",
           "parent_var",
           "pos",
           "predictor",
           "pval",
           "ref_level",
           "reference",
           "reference_label",
           "result",
           "row_type",
           "rows",
           "rows_to_delete",
           "se",
           "sex",
           "shade_group",
           "should_bold",
           "sigma",
           "sort_key",
           "stage",
           "statistic",
           "status_var",
           "term",
           "theta",
           "time_var",
           "tjur_r2",
           "treatment",
           "uni_effect",
           "uni_p",
           "using",
           "var",
           "var_display",
           "var_new",
           "var_order",
           "var_orig",
           "var_rows",
           "variable",
           "variables",
           "x_pos",
           ".events_calc",

           ## ggplot2 aesthetics and coordinate variables
           "x",
           "y",
           "xend",
           "xmax",
           "xmin",
           "yend",
           "ymax",
           "ymin"
       ))

### * Main function

#' Fit Regression Model with Publication-Ready Output
#'
#' Provides a unified interface for fitting various types of regression models 
#' with automatic formatting of results for publication. Supports generalized 
#' linear models, linear models, survival models, and mixed-effects models with 
#' consistent syntax and output formatting. Handles both univariable and 
#' multivariable models automatically.
#' 
#' @param data Data frame or data.table containing the analysis dataset. 
#'   Required for formula-based workflow; optional for model-based workflow
#'   (extracted from model if not provided).
#'   
#' @param outcome Character string specifying the outcome variable name. For 
#'   survival analysis, use \code{Surv()} syntax from the \pkg{survival} package 
#'   (\emph{e.g.,} \code{"Surv(time, status)"} or \code{"Surv(os_months, os_status)"}).
#'   Required for formula-based workflow; ignored if \code{model} is provided.
#'   
#' @param predictors Character vector of predictor variable names to include in 
#'   the model. All predictors are included simultaneously (multivariable model). 
#'   For univariable models, provide a single predictor. Can include continuous, 
#'   categorical (factor), or binary variables. Required for formula-based
#'   workflow; ignored if \code{model} is provided.
#'
#' @param model Optional pre-fitted model object to format. When provided,
#'   \code{outcome} and \code{predictors} are ignored and the model is formatted
#'   directly. Supported model classes include:
#'   \itemize{
#'     \item \code{glm} - Generalized linear models
#'     \item \code{negbin} - Negative binomial models from \code{MASS::glm.nb()}
#'     \item \code{lm} - Linear models
#'     \item \code{coxph} - Cox proportional hazards models
#'     \item \code{lmerMod}, \code{glmerMod} - Mixed-effects models from \pkg{lme4}
#'     \item \code{coxme} - Mixed-effects Cox models
#'   }
#'   
#' @param model_type Character string specifying the type of regression model. 
#'   Ignored if \code{model} is provided. Options include:
#'   \itemize{
#'     \item \code{"glm"} - Generalized linear model (default). Supports multiple 
#'       distributions via the \code{family} parameter including logistic, Poisson, 
#'       Gamma, Gaussian, and quasi-likelihood models.
#'     \item \code{"lm"} - Linear regression for continuous outcomes with normally 
#'       distributed errors. Equivalent to \code{glm} with \code{family = "gaussian"}.
#'     \item \code{"coxph"} - Cox proportional hazards model for time-to-event 
#'       survival analysis. Requires \code{Surv()} outcome syntax.
#'     \item \code{"clogit"} - Conditional logistic regression for matched 
#'       case-control studies or stratified analyses.
#'     \item \code{"negbin"} - Negative binomial regression for overdispersed count 
#'       data (requires \pkg{MASS} package). Estimates an additional dispersion parameter 
#'       compared to Poisson regression.
#'     \item \code{"lmer"} - Linear mixed-effects model for hierarchical or clustered 
#'       data with continuous outcomes (requires \pkg{lme4} package).
#'     \item \code{"glmer"} - Generalized linear mixed-effects model for hierarchical 
#'       or clustered data with non-normal outcomes (requires \pkg{lme4} package).
#'     \item \code{"coxme"} - Cox mixed-effects model for clustered survival data 
#'       (requires coxme package).
#'   }
#'   
#' @param family For GLM and GLMER models, specifies the error distribution and link 
#'   function. Can be a character string, a family function, or a family object.
#'   Ignored for non-GLM/GLMER models.
#'   
#'   \strong{Binary/Binomial outcomes:}
#'   \itemize{
#'     \item \code{"binomial"} or \code{binomial()} - Logistic regression for binary 
#'       outcomes (0/1, TRUE/FALSE). Returns odds ratios (OR). Default.
#'     \item \code{"quasibinomial"} or \code{quasibinomial()} - Logistic regression 
#'       with overdispersion. Use when residual deviance >> residual df.
#'     \item \code{binomial(link = "probit")} - Probit regression (normal CDF link).
#'     \item \code{binomial(link = "cloglog")} - Complementary log-log link for 
#'       asymmetric binary outcomes.
#'   }
#'   
#'   \strong{Count outcomes:}
#'   \itemize{
#'     \item \code{"poisson"} or \code{poisson()} - Poisson regression for count 
#'       data. Returns rate ratios (RR). Assumes mean = variance.
#'     \item \code{"quasipoisson"} or \code{quasipoisson()} - Poisson regression 
#'       with overdispersion. Use when variance > mean.
#'   }
#'   
#'   \strong{Continuous outcomes:}
#'   \itemize{
#'     \item \code{"gaussian"} or \code{gaussian()} - Normal/Gaussian distribution 
#'       for continuous outcomes. Equivalent to linear regression.
#'     \item \code{gaussian(link = "log")} - Log-linear model for positive continuous 
#'       outcomes. Returns multiplicative effects.
#'     \item \code{gaussian(link = "inverse")} - Inverse link for specific applications.
#'   }
#'   
#'   \strong{Positive continuous outcomes:}
#'   \itemize{
#'     \item \code{"Gamma"} or \code{Gamma()} - Gamma distribution for positive, 
#'       right-skewed continuous data (\emph{e.g.,} costs, lengths of stay). When
#'       passed as a string, resolves to log link for interpretable multiplicative
#'       effects. The canonical inverse link can be specified explicitly with
#'       \code{Gamma(link = "inverse")}.
#'     \item \code{Gamma(link = "log")} - Gamma with log link (equivalent to
#'       \code{"Gamma"} string shorthand).
#'     \item \code{Gamma(link = "inverse")} - Gamma with inverse (canonical) link.
#'     \item \code{Gamma(link = "identity")} - Gamma with identity link for additive 
#'       effects on positive outcomes.
#'     \item \code{"inverse.gaussian"} or \code{inverse.gaussian()} - Inverse Gaussian 
#'       for positive, highly right-skewed data.
#'   }
#'   
#'   For negative binomial regression (overdispersed counts), use 
#'   \code{model_type = "negbin"} instead of the \code{family} parameter.
#'   
#'   See \code{\link[stats]{family}} for additional details and options.
#'
#' @param random Character string specifying the random-effects formula for 
#'   mixed-effects models (\code{glmer}, \code{lmer}, \code{coxme}). Use standard
#'   \pkg{lme4}/\pkg{coxme} syntax, \emph{e.g.,} \code{"(1|site)"} for random
#'   intercepts by site, \code{"(1|site/patient)"} for nested random effects, or 
#'   \code{"(1|site) + (1|doctor)"} for crossed random effects. Required when 
#'   \code{model_type} is a mixed-effects model type. Alternatively, random 
#'   effects can be included directly in the \code{predictors} vector using the
#'   same syntax. Default is \code{NULL}.
#'   
#' @param interactions Character vector of interaction terms using colon 
#'   notation (\emph{e.g.,} \code{c("age:sex", "treatment:stage")}). Interaction terms 
#'   are added to the model in addition to main effects. Default is \code{NULL} 
#'   (no interactions).
#'   
#' @param strata For Cox or conditional logistic models, character string naming 
#'   the stratification variable. Creates separate baseline hazards for each 
#'   stratum level without estimating stratum effects. Default is \code{NULL}.
#'   
#' @param cluster For Cox models, character string naming the variable for 
#'   robust clustered standard errors. Accounts for within-cluster correlation 
#'   (\emph{e.g.,} patients within hospitals). Default is \code{NULL}.
#'   
#' @param weights Character string naming the weights variable in \code{data}. 
#'   The specified column should contain numeric values for observation weights. 
#'   Used for weighted regression, survey data, or inverse probability weighting. 
#'   Default is \code{NULL}.
#'   
#' @param conf_level Numeric confidence level for confidence intervals. Must be 
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals).
#'   
#' @param reference_rows Logical. If \code{TRUE}, adds rows for reference 
#'   categories of factor variables with baseline values (OR/HR/RR = 1, 
#'   coefficient = 0). Default is \code{TRUE}.
#'   
#' @param show_n Logical. If \code{TRUE}, includes the sample size column in 
#'   the output. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes the events column in 
#'   the output (for survival and logistic regression). Default is \code{TRUE}.
#'   
#' @param digits Integer specifying the number of decimal places for effect 
#'   estimates (OR, HR, RR, coefficients). Default is 2.
#'   
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param labels Named character vector or list providing custom display 
#'   labels for variables. Names should match variable names, values are display 
#'   labels. Default is \code{NULL}.
#'   
#' @param keep_qc_stats Logical. If \code{TRUE}, includes model quality statistics 
#'   (AIC, BIC, R-squared, concordance, \emph{etc.}) in the raw data attribute for 
#'   model diagnostics and comparison. Default is \code{TRUE}.
#'   
#' @param exponentiate Logical. Whether to exponentiate coefficients. Default 
#'   is \code{NULL}, which automatically exponentiates for logistic, Poisson, 
#'   and Cox models, and displays raw coefficients for linear models. Set to 
#'   \code{TRUE} to force exponentiation or \code{FALSE} to force coefficients.
#'
#' @param conf_method Character string controlling the confidence interval method.
#'   If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
#'   \itemize{
#'     \item \code{"profile"} - Profile likelihood intervals for GLM and negative
#'       binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
#'       intervals for linear models. Falls back to Wald on profiling failure.
#'       Quasi-likelihood families always use Wald (no true likelihood).
#'     \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
#'       \eqn{\times} SE) for all model types. Faster but less accurate near
#'       boundary conditions or with small subgroups.
#'   }
#'   Cox and mixed-effects models use Wald intervals regardless of this setting.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param verbose Logical. If \code{TRUE}, displays model fitting warnings
#'   (\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
#'   routine fitting messages are suppressed while unexpected warnings are
#'   preserved. When \code{NULL}, uses
#'   \code{getOption("summata.verbose", FALSE)}.
#'
#' @param ... Additional arguments passed to the underlying model fitting 
#'   function (\code{\link[stats]{glm}}, \code{\link[stats]{lm}}, 
#'   \code{\link[survival]{coxph}}, \code{\link[lme4]{glmer}}, \emph{etc.}). Common 
#'   options include \code{subset}, \code{na.action}, and model-specific control 
#'   parameters.
#'
#' @return A data.table with S3 class \code{"fit_result"} containing formatted 
#'   regression results. The table structure includes:
#'   \describe{
#'     \item{Variable}{Character. Predictor name or custom label}
#'     \item{Group}{Character. For factor variables: category level. For 
#'       interactions: interaction term. For continuous: typically empty}
#'     \item{n}{Integer. Sample size used in fitting, that is complete cases
#'       across the model variables (if \code{show_n = TRUE})}
#'     \item{n_group}{Integer. Sample size for this factor level}
#'     \item{events}{Integer. Total number of events (if \code{show_events = TRUE})}
#'     \item{events_group}{Integer. Events for this factor level}
#'     \item{OR/HR/RR/Coefficient or aOR/aHR/aRR/Adj. Coefficient (95\% CI)}{Character. 
#'       Formatted effect estimate with confidence interval. Column name depends on 
#'       model type and scope. Univariable models use: OR, HR, RR, Coefficient.
#'       Multivariable models use adjusted notation: aOR, aHR, aRR, Adj. Coefficient}
#'     \item{\emph{p}-value}{Character. Formatted \emph{p}-value from Wald test}
#'   }
#'   
#'   The returned object includes the following attributes accessible via \code{attr()}:
#'   \describe{
#'     \item{model}{The fitted model object (glm, lm, coxph, \emph{etc.}). Access for 
#'       diagnostics, predictions, or further analysis}
#'     \item{raw_data}{data.table. Unformatted numeric results with columns for 
#'       coefficients, standard errors, confidence bounds, quality statistics, \emph{etc.}}
#'     \item{analysis_counts}{List. How much of the supplied data reached the
#'       model, with elements \code{n_supplied}, \code{n_analyzed},
#'       \code{n_missing_outcome} and \code{n_missing_predictor}, plus
#'       \code{events_supplied} and \code{events_analyzed} where the model
#'       type carries an event count. The counts describe the single
#'       fitted model, and the \code{print()} method always reports the
#'       analyzed sample}
#'     \item{outcome}{Character. The outcome variable name}
#'     \item{predictors}{Character vector. The predictor variable names}
#'     \item{formula_str}{Character. The complete model formula as a string}
#'     \item{model_scope}{Character. "Univariable" (one predictor) or 
#'       "Multivariable" (multiple predictors)}
#'     \item{model_type}{Character. The regression model type used}
#'     \item{interactions}{Character vector (if interactions specified). 
#'       The interaction terms included}
#'     \item{strata}{Character (if stratification used). The stratification variable}
#'     \item{cluster}{Character (if clustering used). The cluster variable}
#'     \item{weights}{Character (if weighting used). The weights variable}
#'     \item{significant}{Character vector. Names of predictors with \emph{p}-value
#'       below 0.05, suitable for downstream variable selection workflows}
#'   }
#'
#' @details
#' \strong{Model Scope Detection:}
#' 
#' The function automatically detects whether the model is:
#' \itemize{
#'   \item \strong{Univariable}: Single predictor (\emph{e.g.,} \code{predictors = "age"}). 
#'       Effect estimates are labeled as unadjusted ("OR", "HR", \emph{etc.}), representing
#'       crude (unadjusted) association
#'   \item \strong{Multivariable}: Multiple predictors (\emph{e.g.,} 
#'     \code{predictors = c("age", "sex", "treatment")}) 
#'       Effect estimates are labeled as adjusted ("aOR", "aHR", \emph{etc.}), representing
#'       associations adjusted for confounding
#' }
#' 
#' \strong{Interaction Terms:}
#' 
#' Interactions are specified using colon notation and added to the model:
#' \itemize{
#'   \item \code{interactions = c("age:treatment")} creates interaction 
#'     between age and treatment
#'   \item Main effects for both variables are automatically included
#'   \item Multiple interactions can be specified: 
#'     \code{c("age:sex", "treatment:stage")}
#'   \item For interactions between categorical variables, separate terms are 
#'     created for each combination of levels
#' }
#' 
#' \strong{Stratification (Cox/Conditional Logistic):}
#' 
#' The \code{strata} parameter creates separate baseline hazards:
#' \itemize{
#'   \item Allows baseline hazard to vary across strata without estimating 
#'     stratum effects
#'   \item Useful when proportional hazards assumption violated across strata
#'   \item Example: \code{strata = "center"} for multicenter studies
#'   \item Stratification variable is not included as a predictor
#' }
#' 
#' \strong{Clustering (Cox Models):}
#' 
#' The \code{cluster} parameter computes robust standard errors:
#' \itemize{
#'   \item Accounts for within-cluster correlation (\emph{e.g.,} multiple observations 
#'     per patient)
#'   \item Uses sandwich variance estimator
#'   \item Does not change point estimates, only standard errors and \emph{p}-values
#' }
#' 
#' \strong{Weighting:}
#' 
#' The \code{weights} parameter enables weighted regression:
#' \itemize{
#'   \item For survey data with sampling weights
#'   \item Inverse probability weighting for causal inference
#'   \item Frequency weights for aggregated data
#'   \item Weights should be in a column of \code{data}
#' }
#' 
#' \strong{Mixed-Effects Models (lmer/glmer/coxme):}
#' 
#' Mixed effects models handle hierarchical or clustered data:
#' \itemize{
#'   \item Use \code{model_type = "lmer"} for continuous/normal outcomes
#'   \item Use \code{model_type = "glmer"} with appropriate \code{family} for GLM outcomes
#'   \item Use \code{model_type = "coxme"} for survival outcomes with clustering
#'   \item Random effects are specified in predictors using lme4 syntax:
#'     \itemize{
#'       \item \code{"(1|site)"} - Random intercepts by site
#'       \item \code{"(treatment|site)"} - Random slopes for treatment by site
#'       \item \code{"(1 + treatment|site)"} - Both random intercepts and slopes
#'     }
#'   \item Include random effects as part of the predictors vector
#'   \item Example: \code{predictors = c("age", "treatment", "(1|site)")}
#' }
#' 
#' \strong{Effect Measures by Model Type:}
#' \itemize{
#'   \item \strong{Logistic} (\code{family = "binomial"/"quasibinomial"}): Odds ratios (OR/aOR)
#'   \item \strong{Cox} (\code{model_type = "coxph"}): Hazard ratios (HR/aHR)
#'   \item \strong{Poisson/Count} (\code{family = "poisson"/"quasipoisson"}): Rate ratios (RR/aRR)
#'   \item \strong{Negative binomial} (\code{model_type = "negbin"}): Rate ratios (RR/aRR)
#'   \item \strong{Gamma/Log-link}: Ratios (multiplicative effects)
#'   \item \strong{Linear/Gaussian}: Raw coefficient estimates (additive effects)
#' }
#' 
#' \strong{Confidence Intervals:}
#' 
#' Confidence interval computation is tailored to each model class using the
#' best available method:
#' \itemize{
#'   \item \strong{GLM and negative binomial}: Profile likelihood intervals via
#'     \code{MASS::confint.glm()}, which invert the profile deviance and account
#'     for asymmetry in the likelihood surface. More accurate than the Wald
#'     approximation when subgroup sizes are small or estimates are near boundary
#'     values. Quasi-likelihood families (\code{quasibinomial}, \code{quasipoisson})
#'     fall back to Wald intervals because they lack a true likelihood function.
#'   \item \strong{Linear models}: Exact \emph{t}-distribution intervals via
#'     \code{confint.lm()}, based on the known sampling distribution under
#'     normality.
#'   \item \strong{Cox proportional hazards}: Wald intervals (\emph{i.e.,}
#'     coefficient \eqn{\pm} \emph{z} \eqn{\times} SE), the standard approach in
#'     the survival analysis literature.
#'   \item \strong{Mixed-effects models} (lmer, glmer, coxme): Wald intervals.
#'     Profile likelihood is available for \pkg{lme4} models via
#'     \code{confint(model, method = "profile")} but can be prohibitively slow
#'     for complex random-effects structures and is not used by default.
#' }
#' 
#' If profile likelihood computation fails for any reason (\emph{e.g.,}
#' non-convergence during profiling), the function falls back silently to Wald
#' intervals.
#'
#' @seealso 
#' \code{\link{uniscreen}} for univariable screening of multiple predictors,
#' \code{\link{fullfit}} for complete univariable-to-multivariable workflow,
#' \code{\link{compfit}} for comparing multiple models,
#' \code{\link{m2dt}} for model-to-table conversion,
#' \code{\link{autoforest}} for forest plots of fitted models,
#' \code{\link{forestsave}} for exporting forest plots,
#' \code{\link{tablesave}} for exporting tables
#'
#' @examples
#' # Load example data
#' data(clintrial)
#' data(clintrial_labels)
#' library(survival)
#' 
#' # Example 1: Univariable logistic regression
#' uni_model <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = "age",
#'     labels = clintrial_labels
#' )
#' print(uni_model)
#'
#' \donttest{
#' 
#' # Example 2: Multivariable logistic regression
#' multi_model <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "treatment"),
#'     labels = clintrial_labels
#' )
#' print(multi_model)
#' 
#' # Example 3: Cox proportional hazards model
#' cox_model <- fit(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     model_type = "coxph",
#'     labels = clintrial_labels
#' )
#' print(cox_model)
#' 
#' # Example 4: Model with interaction terms
#' interact_model <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "treatment", "sex"),
#'     interactions = c("age:treatment"),
#'     labels = clintrial_labels
#' )
#' print(interact_model)
#' 
#' # Example 5: Cox model with stratification
#' strat_model <- fit(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "sex", "treatment"),
#'     model_type = "coxph",
#'     strata = "site",  # Separate baseline hazards by site
#'     labels = clintrial_labels
#' )
#' print(strat_model)
#' 
#' # Example 6: Cox model with clustering
#' cluster_model <- fit(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "treatment"),
#'     model_type = "coxph",
#'     cluster = "site",  # Robust SEs accounting for site clustering
#'     labels = clintrial_labels
#' )
#' print(cluster_model)
#' 
#' # Example 7: Linear regression
#' linear_model <- fit(
#'     data = clintrial,
#'     outcome = "bmi",
#'     predictors = c("age", "sex", "smoking"),
#'     model_type = "lm",
#'     labels = clintrial_labels
#' )
#' print(linear_model)
#' 
#' # Example 8: Poisson regression for equidispersed count data
#' # fu_count has variance ~= mean, appropriate for standard Poisson
#' poisson_model <- fit(
#'     data = clintrial,
#'     outcome = "fu_count",
#'     predictors = c("age", "stage", "treatment", "surgery"),
#'     model_type = "glm",
#'     family = "poisson",
#'     labels = clintrial_labels
#' )
#' print(poisson_model)
#' # Returns rate ratios (RR/aRR)
#' 
#' # Example 9: Negative binomial regression for overdispersed counts
#' # ae_count has variance > mean (overdispersed), use negbin or quasipoisson
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'     nb_result <- fit(
#'         data = clintrial,
#'         outcome = "ae_count",
#'         predictors = c("age", "treatment", "diabetes", "surgery"),
#'         model_type = "negbin",
#'         labels = clintrial_labels
#'     )
#'     print(nb_result)
#' }
#' 
#' # Example 10: Gamma regression for positive continuous outcomes
#' gamma_model <- fit(
#'     data = clintrial,
#'     outcome = "los_days",
#'     predictors = c("age", "treatment", "surgery"),
#'     model_type = "glm",
#'     family = Gamma(link = "log"),
#'     labels = clintrial_labels
#' )
#' print(gamma_model)
#' 
#' # Example 11: Access the underlying fitted model
#' result <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi")
#' )
#' 
#' # Get the model object
#' model_obj <- attr(result, "model")
#' summary(model_obj)
#' 
#' # Model diagnostics
#' plot(model_obj)
#' 
#' # Predictions
#' preds <- predict(model_obj, type = "response")
#' 
#' # Example 12: Access raw numeric data
#' raw_data <- attr(result, "raw_data")
#' print(raw_data)
#' # Contains unformatted coefficients, SEs, CIs, AIC, BIC, etc.
#' 
#' # Example 13: Multiple interactions
#' complex_model <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment", "bmi"),
#'     interactions = c("age:treatment", "sex:bmi"),
#'     labels = clintrial_labels
#' )
#' print(complex_model)
#' 
#' # Example 14: Customize output columns
#' minimal <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment"),
#'     show_n = FALSE,
#'     show_events = FALSE,
#'     reference_rows = FALSE,
#'     labels = clintrial_labels
#' )
#' print(minimal)
#' 
#' # Example 15: Different confidence levels
#' ci90 <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "treatment"),
#'     conf_level = 0.90,  # 90% confidence intervals
#'     labels = clintrial_labels
#' )
#' print(ci90)
#' 
#' # Example 16: Force coefficient display instead of OR
#' coef_model <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "bmi"),
#'     exponentiate = FALSE,  # Show log odds instead of OR
#'     labels = clintrial_labels
#' )
#' print(coef_model)
#' 
#' # Example 17: Confidence interval method
#' # Default: profile likelihood CIs for GLM (more accurate)
#' profile_result <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "treatment"),
#'     p_digits = 4,
#'     conf_method = "profile",
#'     labels = clintrial_labels
#' )
#' print(profile_result)
#'
#' # Wald CIs (faster, suitable for simulation or exploratory work)
#' wald_result <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "treatment"),
#'     p_digits = 4,
#'     conf_method = "wald",
#'     labels = clintrial_labels
#' )
#' print(wald_result)
#' 
#' # Example 18: Check model quality statistics
#' result <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment", "stage"),
#'     keep_qc_stats = TRUE
#' )
#' 
#' raw <- attr(result, "raw_data")
#' cat("AIC:", raw$AIC[1], "\n")
#' cat("BIC:", raw$BIC[1], "\n")
#' cat("C-statistic:", raw$c_statistic[1], "\n")
#' 
#' # Example 19: Interaction effects - treatment effect modified by stage
#' interaction_model <- fit(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "treatment", "stage"),
#'     interactions = c("treatment:stage"),
#'     model_type = "coxph",
#'     labels = clintrial_labels
#' )
#' print(interaction_model)
#' # Shows main effects plus all treatment×stage interaction terms
#' 
#' # Example 20: Multiple interactions in logistic regression
#' multi_interaction <- fit(
#'     data = clintrial,
#'     outcome = "readmission_30d",
#'     predictors = c("age", "sex", "surgery", "diabetes"),
#'     interactions = c("surgery:diabetes", "age:sex"),
#'     labels = clintrial_labels
#' )
#' print(multi_interaction)
#' 
#' # Example 21: Quasipoisson for overdispersed count data
#' # Alternative to negative binomial when MASS not available
#' quasi_model <- fit(
#'     data = clintrial,
#'     outcome = "ae_count",
#'     predictors = c("age", "treatment", "diabetes", "surgery"),
#'     model_type = "glm",
#'     family = "quasipoisson",
#'     labels = clintrial_labels
#' )
#' print(quasi_model)
#' # Adjusts standard errors for overdispersion
#' 
#' # Example 22: Quasibinomial for overdispersed binary data
#' quasi_logistic <- fit(
#'     data = clintrial,
#'     outcome = "any_complication",
#'     predictors = c("age", "bmi", "diabetes", "surgery"),
#'     model_type = "glm",
#'     family = "quasibinomial",
#'     labels = clintrial_labels
#' )
#' print(quasi_logistic)
#' 
#' # Example 23: Gamma regression with identity link for additive effects
#' gamma_identity <- fit(
#'     data = clintrial,
#'     outcome = "los_days",
#'     predictors = c("age", "treatment", "surgery", "any_complication"),
#'     model_type = "glm",
#'     family = Gamma(link = "identity"),
#'     labels = clintrial_labels
#' )
#' print(gamma_identity)
#' # Shows additive effects (coefficients) instead of multiplicative (ratios)
#' 
#' # Example 24: Inverse Gaussian regression for highly skewed data
#' inverse_gaussian <- fit(
#'     data = clintrial,
#'     outcome = "recovery_days",
#'     predictors = c("age", "surgery", "pain_score"),
#'     model_type = "glm",
#'     family = inverse.gaussian(link = "log"),
#'     labels = clintrial_labels
#' )
#' print(inverse_gaussian)
#' 
#' # Example 25: Linear mixed effects with random intercepts
#' # Accounts for clustering of patients within sites
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     lmer_model <- fit(
#'         data = clintrial,
#'         outcome = "los_days",
#'         predictors = c("age", "treatment", "stage", "(1|site)"),
#'         model_type = "lmer",
#'         labels = clintrial_labels
#'     )
#'     print(lmer_model)
#' }
#' 
#' # Example 26: Generalized linear mixed effects (logistic with random effects)
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     glmer_model <- fit(
#'         data = clintrial,
#'         outcome = "readmission_30d",
#'         predictors = c("age", "surgery", "los_days", "(1|site)"),
#'         model_type = "glmer",
#'         family = "binomial",
#'         labels = clintrial_labels
#'     )
#'     print(glmer_model)
#' }
#' 
#' # Example 27: Cox mixed effects for clustered survival data
#' if (requireNamespace("coxme", quietly = TRUE)) {
#'     coxme_model <- fit(
#'         data = clintrial,
#'         outcome = "Surv(os_months, os_status)",
#'         predictors = c("age", "treatment", "stage", "(1|site)"),
#'         model_type = "coxme",
#'         labels = clintrial_labels
#'     )
#'     print(coxme_model)
#' }
#' 
#' # Example 28: Random slopes - treatment effect varies by site
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     random_slopes <- fit(
#'         data = clintrial,
#'         outcome = "los_days",
#'         predictors = c("age", "treatment", "stage", "(treatment|site)"),
#'         model_type = "lmer",
#'         labels = clintrial_labels
#'     )
#'     print(random_slopes)
#' }
#' 
#' # Example 29: Format a pre-fitted model (model-based workflow)
#' # Useful for models fitted outside of fit()
#' pre_fitted <- glm(os_status ~ age + sex + treatment,
#'                   family = binomial, data = clintrial)
#' result <- fit(model = pre_fitted, 
#'               data = clintrial,
#'               labels = clintrial_labels)
#' print(result)
#'
#' }
#' 
#' @family regression functions
#' @export
fit <- function(data = NULL, 
                outcome = NULL,
                predictors = NULL,
                model = NULL,
                model_type = "glm",
                family = "binomial",
                random = NULL,
                interactions = NULL,
                strata = NULL,
                cluster = NULL,
                weights = NULL,
                conf_level = 0.95,
                reference_rows = TRUE,
                show_n = TRUE,
                show_events = TRUE,
                digits = 2,
                p_digits = 3,
                labels = NULL,
                keep_qc_stats = TRUE,
                exponentiate = NULL,
                conf_method = NULL,
                number_format = NULL,
                verbose = NULL,
                ...) {
    
    ## Determine whether the model-based workflow applies
    
    ## Resolve verbose setting
    verbose <- if (is.null(verbose)) getOption("summata.verbose", FALSE) else verbose
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    if (!is.null(model)) {
        ## Model-based workflow: format existing model
        
        ## Get data from model if not provided
        if (is.null(data)) {
            data <- get_model_data(model)
            if (is.null(data)) {
                stop("Cannot extract data from model. Please provide 'data' parameter.")
            }
        }
        
        ## Convert to data.table
        if (!data.table::is.data.table(data)) {
            data <- data.table::as.data.table(data)
        }
        
        ## Store data in model for m2dt
        if (isS4(model)) {
            attr(model, "data") <- data
        } else {
            model$data <- data
            attr(model, "data") <- data
        }
        
        ## Convert to readable format using m2dt
        raw_data <- m2dt(data = data,
                         model = model, 
                         conf_level = conf_level,
                         keep_qc_stats = keep_qc_stats,
                         include_intercept = FALSE,
                         reference_rows = reference_rows,
                         skip_counts = (!show_n && !show_events),
                         conf_method = conf_method)

        ## Propagate cached confint from m2dt to model for downstream reuse
        cached_ci <- attr(raw_data, "cached_confint")
        if (!is.null(cached_ci)) {
            attr(model, "cached_confint") <- cached_ci
            attr(model, "cached_confint_level") <- attr(raw_data, "cached_confint_level")
        }

        ## Format results for publication
        formatted <- format_model_table(raw_data,
                                        show_n = show_n,
                                        show_events = show_events,
                                        digits = digits,
                                        p_digits = p_digits,
                                        labels = labels,
                                        exponentiate = exponentiate,
                                        conf_level = conf_level,
                                        marks = marks)
        ## Extract formula and predictors from model
        formula_str <- deparse(stats::formula(model))
        
        ## Attach metadata
        data.table::setattr(formatted, "model", model)
        data.table::setattr(formatted, "raw_data", raw_data)
        data.table::setattr(formatted, "analysis_counts",
                            get_model_analysis_counts(model, class(model)[1], data))
        data.table::setattr(formatted, "number_marks", marks)
        ## Variable labels travel with the result so that the forest plot
        ## functions can apply the same labels without being given them again.
        ## An explicit labels argument to those functions still takes precedence.
        data.table::setattr(formatted, "labels", labels)
        data.table::setattr(formatted, "formula_str", formula_str)
        data.table::setattr(formatted, "model_scope", raw_data$model_scope[1])
        data.table::setattr(formatted, "model_type", raw_data$model_type[1])
        
        ## Extract significant predictors (p < 0.05)
        if ("p_value" %in% names(raw_data) && "variable" %in% names(raw_data)) {
            sig_vars <- unique(
                raw_data[!is.na(p_value) & p_value < 0.05]$variable
            )
            data.table::setattr(formatted, "significant", sig_vars)
        }
        
        class(formatted) <- c("fit_result", class(formatted))
        
        return(formatted)
    }
    
    ## Formula-based workflow: fit new model
    
    ## Validate required parameters for formula-based workflow
    if (is.null(data)) {
        stop("'data' parameter is required when not providing a pre-fitted model.")
    }
    if (is.null(outcome)) {
        stop("'outcome' parameter is required when not providing a pre-fitted model.")
    }
    if (is.null(predictors)) {
        stop("'predictors' parameter is required when not providing a pre-fitted model.")
    }
    
    ## Store original data name for reference
    data_name <- deparse(substitute(data))
    
    ## Convert to data.table once at the start
    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    
    ## Validate inputs and auto-correct model type if needed
    validation <- validate_fit_inputs(
        data = data,
        outcome = outcome,
        predictors = predictors,
        model_type = model_type,
        family = if (model_type %in% c("glm", "glmer")) family else NULL,
        conf_level = conf_level,
        digits = digits,
        p_digits = p_digits,
        auto_correct_model = TRUE
    )
    
    ## Apply any auto-corrections
    if (validation$auto_corrected) {
        model_type <- validation$model_type
    }

    ## Resolve Gamma family string to log link
    ## The canonical link for Gamma is inverse (1/μ), which produces
    ## uninterpretable coefficients. The log link yields exp(β) = ratio,
    ## consistent with the multiplicative effect interpretation used for
    ## OR, HR, and RR throughout summata.
    if (model_type %in% c("glm", "glmer") && is.character(family) &&
        tolower(family) == "gamma") {
        family <- stats::Gamma(link = "log")
    }
    
    ## Validate random effects for mixed-effects models
    ## Support both explicit 'random' parameter AND random effects embedded in predictors
    mixed_types <- c("glmer", "lmer", "coxme")
    has_random_in_predictors <- any(grepl("\\|", predictors))
    
    if (model_type %in% mixed_types && is.null(random) && !has_random_in_predictors) {
        stop("'random' parameter is required for mixed-effects models (", model_type, ").\n",
             "Example: random = \"(1|site)\" for random intercepts by site.\n",
             "Alternatively, include random effects in 'predictors' using the same syntax.")
    }
    if (!is.null(random) && !model_type %in% mixed_types) {
        warning("'random' parameter is ignored for non-mixed-effects models (", model_type, ").")
        random <- NULL
    }
    
    ## Extract random effects from predictors if present (for mixed-effects models)
    ## The random effects get added to the formula but aren't treated as regular predictors
    if (model_type %in% mixed_types && has_random_in_predictors) {
        random_mask <- grepl("\\|", predictors)
        random_in_preds <- predictors[random_mask]
        fixed_predictors <- predictors[!random_mask]
        
        ## Combine explicit random with embedded random (if both provided)
        if (!is.null(random)) {
            message("Note: Random effects specified in both 'random' parameter and 'predictors'. ",
                    "Combining: ", random, " + ", paste(random_in_preds, collapse = " + "))
            random <- paste(c(random, random_in_preds), collapse = " + ")
        } else {
            random <- paste(random_in_preds, collapse = " + ")
        }
        
        ## Update predictors to only include fixed effects
        predictors <- fixed_predictors
    }
    
    ## Build formula string
    if (!is.null(interactions)) {
        formula_str <- paste(outcome, "~", 
                             paste(c(predictors, interactions), collapse = " + "))
    } else {
        formula_str <- paste(outcome, "~", paste(predictors, collapse = " + "))
    }
    
    ## Add random effects if provided (for lmer/glmer - coxme uses same formula syntax)
    if (!is.null(random) && model_type %in% c("glmer", "lmer", "coxme")) {
        formula_str <- paste(formula_str, "+", random)
    }
    
    ## Add strata if provided (for Cox models)
    if (!is.null(strata) && model_type %in% c("coxph", "clogit")) {
        formula_str <- paste(formula_str, "+ strata(", strata, ")")
    }
    
    formula <- stats::as.formula(formula_str)
    
    ## Fit model based on type - use 'data' directly
    model <- quiet_fit(quote({
        if (model_type == "glm") {
            if (!is.null(weights)) {
                stats::glm(formula, data = data, family = family, 
                           weights = data[[weights]], ...)
            } else {
                stats::glm(formula, data = data, family = family, ...)
            }
            
        } else if (model_type == "negbin") {
            if (!requireNamespace("MASS", quietly = TRUE))
                stop("Package 'MASS' required for negative binomial models")
            if (!is.null(weights)) {
                MASS::glm.nb(formula, data = data, 
                              weights = data[[weights]], ...)
            } else {
                MASS::glm.nb(formula, data = data, ...)
            }
            
        } else if (model_type == "lm") {
            if (!is.null(weights)) {
                stats::lm(formula, data = data, weights = data[[weights]], ...)
            } else {
                stats::lm(formula, data = data, ...)
            }
            
        } else if (model_type == "coxph") {
            if (!requireNamespace("survival", quietly = TRUE)) 
                stop("Package 'survival' required for Cox models")
            
            if (!is.null(cluster)) {
                survival::coxph(formula, data = data, 
                                cluster = data[[cluster]], ...)
            } else {
                survival::coxph(formula, data = data, ...)
            }
            
        } else if (model_type == "clogit") {
            if (!requireNamespace("survival", quietly = TRUE)) 
                stop("Package 'survival' required for conditional logistic regression")
            survival::clogit(formula, data = data, ...)
            
        } else if (model_type == "coxme") {
            if (!requireNamespace("coxme", quietly = TRUE))
                stop("Package 'coxme' required for mixed effects Cox models")
            coxme::coxme(formula, data = data, ...)
            
        } else if (model_type == "lmer") {
            if (!requireNamespace("lme4", quietly = TRUE))
                stop("Package 'lme4' required for linear mixed effects models")
            lme4::lmer(formula, data = data, ...)
            
        } else if (model_type == "glmer") {
            if (!requireNamespace("lme4", quietly = TRUE))
                stop("Package 'lme4' required for mixed effects models")
            lme4::glmer(formula, data = data, family = family, ...)
            
        } else {
            stop("Unsupported model type: ", model_type)
        }
    }), verbose = verbose)
    
    ## Store the data directly in the model
    if (isS4(model)) {
        attr(model, "data") <- data
    } else {
        model$data <- data
        attr(model, "data") <- data
    }
    
    ## Attach metadata as attributes
    data.table::setattr(model, "formula_str", formula_str)
    data.table::setattr(model, "predictors", predictors)
    data.table::setattr(model, "model_type", model_type)
    data.table::setattr(model, "data_name", data_name)
    
    if (!is.null(interactions)) {
        data.table::setattr(model, "interactions", interactions)
    }
    if (!is.null(random)) {
        data.table::setattr(model, "random", random)
    }
    if (!is.null(strata)) {
        data.table::setattr(model, "strata", strata)
    }
    if (!is.null(cluster)) {
        data.table::setattr(model, "cluster", cluster)
    }
    if (!is.null(weights)) {
        data.table::setattr(model, "weights", weights)
    }
    
    ## Convert to readable format using m2dt
    raw_data <- m2dt(data = data,
                     model = model, 
                     conf_level = conf_level,
                     keep_qc_stats = keep_qc_stats,
                     include_intercept = FALSE,
                     reference_rows = reference_rows,
                     skip_counts = (!show_n && !show_events),
                     conf_method = conf_method)

    ## Propagate cached confint from m2dt to model for downstream reuse
    ## (avoids redundant profiling in forest plot functions)
    cached_ci <- attr(raw_data, "cached_confint")
    if (!is.null(cached_ci)) {
        attr(model, "cached_confint") <- cached_ci
        attr(model, "cached_confint_level") <- attr(raw_data, "cached_confint_level")
    }

    ## Format results for publication
    formatted <- format_model_table(raw_data,
                                    show_n = show_n,
                                    show_events = show_events,
                                    digits = digits,
                                    p_digits = p_digits,
                                    labels = labels,
                                    exponentiate = exponentiate,
                                    conf_level = conf_level,
                                    marks = marks)

    ## Attach metadata
    data.table::setattr(formatted, "model", model)
    data.table::setattr(formatted, "raw_data", raw_data)
    data.table::setattr(formatted, "analysis_counts",
                        get_model_analysis_counts(model, class(model)[1], data))
    data.table::setattr(formatted, "number_marks", marks)
    ## Variable labels travel with the result so that the forest plot
    ## functions can apply the same labels without being given them again.
    ## An explicit labels argument to those functions still takes precedence.
    data.table::setattr(formatted, "labels", labels)
    data.table::setattr(formatted, "outcome", outcome)
    data.table::setattr(formatted, "predictors", predictors)
    data.table::setattr(formatted, "formula_str", formula_str)
    data.table::setattr(formatted, "model_scope", raw_data$model_scope[1])
    data.table::setattr(formatted, "model_type", raw_data$model_type[1])

    ## Extract significant predictors (p < 0.05)
    if ("p_value" %in% names(raw_data) && "variable" %in% names(raw_data)) {
        sig_vars <- unique(
            raw_data[!is.na(p_value) & p_value < 0.05]$variable
        )
        data.table::setattr(formatted, "significant", sig_vars)
    }

    ## Add metadata from model fitting
    if (!is.null(interactions)) {
        data.table::setattr(formatted, "interactions", interactions)
    }
    if (!is.null(random)) {
        data.table::setattr(formatted, "random", random)
    }
    if (!is.null(strata)) {
        data.table::setattr(formatted, "strata", strata)
    }
    if (!is.null(cluster)) {
        data.table::setattr(formatted, "cluster", cluster)
    }
    if (!is.null(weights)) {
        data.table::setattr(formatted, "weights", weights)
    }

    class(formatted) <- c("fit_result", class(formatted))

    return(formatted)
}


### * Print function

#' Print method for fit results
#' 
#' Displays a summary header with model scope (Univariable/Multivariable),
#' model type, formula, sample size, and event count before printing the
#' formatted results table.
#' 
#' @param x fit_result object.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @family regression functions
#' @export
print.fit_result <- function(x, ...) {
    cat("\n", attr(x, "model_scope"), " ", attr(x, "model_type"), " Model\n\n", sep = "")
    cat("Formula: ", attr(x, "formula_str"), "\n", sep = "")
    
    raw <- attr(x, "raw_data")
    
    ## The analyzed count is the model's n, so a separate "n = " line would
    ## state the same figure twice. Always reported, so that a complete
    ## sample is stated rather than left to be distinguished from an absent
    ## disclosure.
    marks <- attr(x, "number_marks")
    if (is.null(marks)) {
        marks <- resolve_number_marks(NULL)
    }
    
    ## Used by the fallback branches below, which state a count without a
    ## denominator and so cannot go through the counts formatter
    bare_count <- function(value) format_count(value, marks)
    
    analyzed <- format_analysis_counts(attr(x, "analysis_counts"), marks = marks)
    if (!is.null(analyzed)) {
        cat(analyzed, "\n", sep = "")
    } else if (!is.null(raw)) {
        ## Counts unavailable, as when a model is supplied without its data
        cat("Observations analyzed: ", bare_count(raw$n[1]), "\n", sep = "")
    }
    
    ## Events are reported on the same footing, against the events present in
    ## the supplied data rather than against the observations. The same
    ## formatter is used so that the two lines cannot drift apart.
    events_line <- format_event_counts(attr(x, "analysis_counts"), marks = marks)
    if (!is.null(events_line)) {
        cat(events_line, "\n", sep = "")
    } else if (!is.null(raw) && !is.na(raw$events[1])) {
        ## Counts unavailable, as when a model is supplied without its data
        cat("Events analyzed: ", bare_count(raw$events[1]), "\n", sep = "")
    }
    
    cat("\n")
    
    NextMethod("print", x)
    invisible(x)
}

### * Main function

#' Univariable Screening for Multiple Predictors
#'
#' Performs comprehensive univariable (unadjusted) regression analyses by fitting 
#' separate models for each predictor against a single outcome. This function is 
#' designed for initial variable screening, hypothesis generation, and understanding 
#' crude associations before multivariable modeling. Returns publication-ready 
#' formatted results with optional \emph{p}-value filtering.
#'
#' @param data Data frame or data.table containing the analysis dataset. The 
#'   function automatically converts data frames to data.tables for efficient 
#'   processing.
#'   
#' @param outcome Character string specifying the outcome variable name. For 
#'   survival analysis, use \code{Surv()} syntax from the \pkg{survival} package 
#'   (\emph{e.g.,} \code{"Surv(time, status)"} or \code{"Surv(os_months, os_status)"}).
#'   
#' @param predictors Character vector of predictor variable names to screen. Each 
#'   predictor is tested independently in its own univariable model. Can include 
#'   continuous, categorical (factor), or binary variables.
#'   
#' @param model_type Character string specifying the type of regression model to 
#'   fit. Options include:
#'   \itemize{
#'     \item \code{"glm"} - Generalized linear model (default). Supports multiple 
#'       distributions via the \code{family} parameter including logistic, Poisson, 
#'       Gamma, Gaussian, and quasi-likelihood models.
#'     \item \code{"lm"} - Linear regression for continuous outcomes with normally 
#'       distributed errors. Equivalent to \code{glm} with \code{family = "gaussian"}.
#'     \item \code{"coxph"} - Cox proportional hazards model for time-to-event 
#'       survival analysis. Requires \code{Surv()} outcome syntax.
#'     \item \code{"clogit"} - Conditional logistic regression for matched 
#'       case-control studies or stratified analyses.
#'     \item \code{"negbin"} - Negative binomial regression for overdispersed count 
#'       data (requires MASS package). Estimates an additional dispersion parameter 
#'       compared to Poisson regression.
#'     \item \code{"glmer"} - Generalized linear mixed-effects model for hierarchical 
#'       or clustered data with non-normal outcomes (requires \pkg{lme4} package and 
#'       \code{random} parameter).
#'     \item \code{"lmer"} - Linear mixed-effects model for hierarchical or clustered 
#'       data with continuous outcomes (requires \pkg{lme4} package and \code{random} 
#'       parameter).
#'     \item \code{"coxme"} - Cox mixed-effects model for clustered survival data 
#'       (requires coxme package and \code{random} parameter).
#'   }
#'
#' @param random Character string specifying the random-effects formula for 
#'   mixed-effects models (\code{glmer}, \code{lmer}, \code{coxme}). Use standard
#'   \pkg{lme4}/\pkg{coxme} syntax, \emph{e.g.,} \code{"(1|site)"} for random
#'   intercepts by site, \code{"(1|site/patient)"} for nested random effects.
#'   Required when \code{model_type} is a mixed-effects model type unless random
#'   effects are included in the \code{predictors} vector. Alternatively, random
#'   effects can be included directly in the \code{predictors} vector using the same 
#'   syntax (\emph{e.g.,} \code{predictors = c("age", "sex", "(1|site)")}), though 
#'   they will not be iterated over as predictors. Default is \code{NULL}.
#'   
#' @param family For GLM and GLMER models, specifies the error distribution and link 
#'   function. Can be a character string, a family function, or a family object.
#'   Ignored for non-GLM/GLMER models.
#'   
#'   \strong{Binary/Binomial outcomes:}
#'   \itemize{
#'     \item \code{"binomial"} or \code{binomial()} - Logistic regression for binary 
#'       outcomes (0/1, TRUE/FALSE). Returns odds ratios (OR). Default.
#'     \item \code{"quasibinomial"} or \code{quasibinomial()} - Logistic regression 
#'       with overdispersion. Use when residual deviance >> residual df.
#'     \item \code{binomial(link = "probit")} - Probit regression (normal CDF link).
#'     \item \code{binomial(link = "cloglog")} - Complementary log-log link for 
#'       asymmetric binary outcomes.
#'   }
#'   
#'   \strong{Count outcomes:}
#'   \itemize{
#'     \item \code{"poisson"} or \code{poisson()} - Poisson regression for count 
#'       data. Returns rate ratios (RR). Assumes mean = variance.
#'     \item \code{"quasipoisson"} or \code{quasipoisson()} - Poisson regression 
#'       with overdispersion. Use when variance > mean.
#'   }
#'   
#'   \strong{Continuous outcomes:}
#'   \itemize{
#'     \item \code{"gaussian"} or \code{gaussian()} - Normal/Gaussian distribution 
#'       for continuous outcomes. Equivalent to linear regression.
#'     \item \code{gaussian(link = "log")} - Log-linear model for positive continuous 
#'       outcomes. Returns multiplicative effects.
#'     \item \code{gaussian(link = "inverse")} - Inverse link for specific applications.
#'   }
#'   
#'   \strong{Positive continuous outcomes:}
#'   \itemize{
#'     \item \code{"Gamma"} or \code{Gamma()} - Gamma distribution for positive, 
#'       right-skewed continuous data (\emph{e.g.,} costs, lengths of stay). When
#'       passed as a string, resolves to log link for interpretable multiplicative
#'       effects.
#'     \item \code{Gamma(link = "inverse")} - Gamma with inverse (canonical) link.
#'     \item \code{Gamma(link = "identity")} - Gamma with identity link for additive 
#'       effects on positive outcomes.
#'     \item \code{"inverse.gaussian"} or \code{inverse.gaussian()} - Inverse Gaussian 

#'       for positive, highly right-skewed data.
#'   }
#'   
#'   For negative binomial regression (overdispersed counts), use 
#'   \code{model_type = "negbin"} instead of the \code{family} parameter.
#'   
#'   See \code{\link[stats]{family}} for additional details and options.
#'   
#' @param p_threshold Numeric value between 0 and 1 specifying the \emph{p}-value threshold 
#'   used to count significant predictors in the printed summary. All predictors
#'   are always included in the output table. Default is 0.05.
#'   
#' @param conf_level Numeric confidence level for confidence intervals. Must be 
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals).
#'   
#' @param reference_rows Logical. If \code{TRUE}, adds rows for reference 
#'   categories of factor variables with baseline values (OR/HR/RR = 1, 
#'   coefficient = 0). Makes tables complete and easier to interpret. 
#'   Default is \code{TRUE}.
#'   
#' @param show_n Logical. If \code{TRUE}, includes the sample size column in 
#'   the output table. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes the events column in the 
#'   output table (relevant for survival and logistic regression). Default is 
#'   \code{TRUE}.
#'   
#' @param digits Integer specifying the number of decimal places for effect 
#'   estimates (OR, HR, RR, coefficients). Default is 2.
#'   
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param labels Named character vector or list providing custom display 
#'   labels for variables. Names should match predictor names, values are the 
#'   display labels. Predictors not in \code{labels} use their original names. 
#'   Default is \code{NULL}.
#'   
#' @param keep_models Logical. If \code{TRUE}, stores all fitted model objects 
#'   in the output as an attribute. This allows access to models for diagnostics, 
#'   predictions, or further analysis, but can consume significant memory for 
#'   large datasets or many predictors. Models are accessible via 
#'   \code{attr(result, "models")}. Default is \code{FALSE}.
#'   
#' @param exponentiate Logical. Whether to exponentiate coefficients (display 
#'   OR/HR/RR instead of log odds/log hazards). Default is \code{NULL}, which 
#'   automatically exponentiates for logistic, Poisson, and Cox models, and 
#'   displays raw coefficients for linear models and other GLM families. Set 
#'   to \code{TRUE} to force exponentiation or \code{FALSE} to force coefficients.
#'
#' @param conf_method Character string controlling the confidence interval method.
#'   If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
#'   \itemize{
#'     \item \code{"profile"} - Profile likelihood intervals for GLM and negative
#'       binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
#'       intervals for linear models. Falls back to Wald on profiling failure.
#'       Quasi-likelihood families always use Wald (no true likelihood).
#'     \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
#'       \eqn{\times} SE) for all model types. Faster but less accurate near
#'       boundary conditions or with small subgroups.
#'   }
#'   Cox and mixed-effects models use Wald intervals regardless of this setting.
#'   Set globally with \code{options(summata.conf_method = "wald")} to use Wald
#'   throughout a session.
#'
#' @param parallel Logical. If \code{TRUE} (default), fits models in parallel 
#'   using multiple CPU cores for improved performance with many predictors. 
#'   On Unix/Mac systems, uses fork-based parallelism via \code{mclapply}; 
#'   on Windows, uses socket clusters via \code{parLapply}. Set to \code{FALSE} 
#'   for sequential processing.
#'
#' @param n_cores Integer specifying the number of CPU cores to use for parallel 
#'   processing. Default is \code{NULL}, which automatically detects available 
#'   cores and uses \code{detectCores() - 1}. During R CMD check, the number 
#'   of cores is automatically limited to 2 per CRAN policy. Ignored when 
#'   \code{parallel = FALSE}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param verbose Logical. If \code{TRUE}, displays model fitting warnings
#'   (\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
#'   routine fitting messages are suppressed while unexpected warnings are
#'   preserved. When \code{NULL}, uses
#'   \code{getOption("summata.verbose", FALSE)}.
#'
#' @param ... Additional arguments passed to the underlying model fitting functions 
#'   (\code{\link[stats]{glm}}, \code{\link[stats]{lm}}, 
#'   \code{\link[survival]{coxph}}, \emph{etc.}). Common options include \code{weights}, 
#'   \code{subset}, \code{na.action}, and model-specific control parameters.
#'
#' @return A data.table with S3 class \code{"uniscreen_result"} containing formatted 
#'   univariable screening results. The table structure includes:
#'   \describe{
#'     \item{Variable}{Character. Predictor name or custom label (from \code{labels})}
#'     \item{Group}{Character. For factor variables: category level. For continuous 
#'       variables: typically empty or descriptive statistic label}
#'     \item{n}{Integer. Sample size used in the model (if \code{show_n = TRUE})}
#'     \item{n_group}{Integer. Sample size for this specific factor level 
#'       (factor variables only)}
#'     \item{events}{Integer. Total number of events in the model for survival 
#'       or logistic regression (if \code{show_events = TRUE})}
#'     \item{events_group}{Integer. Number of events for this specific factor 
#'       level (factor variables only)}
#'     \item{OR/HR/RR/Coefficient (95\% CI)}{Character. Formatted effect 
#'       estimate with confidence interval. Column name depends on model type:
#'       "OR (95\% CI)" for logistic, "HR (95\% CI)" for survival, 
#'       "RR (95\% CI)" for counts, "Coefficient (95\% CI)" for linear models}
#'     \item{\emph{p}-value}{Character. Formatted \emph{p}-value from the Wald test}
#'   }
#'   
#'   The returned object includes the following attributes accessible via \code{attr()}:
#'   \describe{
#'     \item{raw_data}{data.table. Unformatted numeric results with separate 
#'       columns for coefficients, standard errors, confidence interval bounds, 
#'       \emph{etc.} Suitable for further statistical analysis or custom formatting}
#'     \item{analysis_counts}{List. How much of the supplied data reached the
#'       model, with elements \code{n_supplied}, \code{n_analyzed},
#'       \code{n_missing_outcome} and \code{n_missing_predictor}, plus
#'       \code{events_supplied} and \code{events_analyzed} where the model
#'       type carries an event count. Each predictor is screened in its
#'       own model, so \code{n_analyzed} holds one value per predictor and
#'       the reported sample may be a range. The \code{print()} method always
#'       reports the analyzed sample}
#'     \item{models}{List (if \code{keep_models = TRUE}). Named list of fitted 
#'       model objects, with predictor names as list names. Access specific models 
#'       via \code{attr(result, "models")[["predictor_name"]]}}
#'     \item{outcome}{Character. The outcome variable name used}
#'     \item{model_type}{Character. The regression model type used}
#'     \item{model_scope}{Character. Always "Univariable" for screening results}
#'     \item{screening_type}{Character. Always "univariable" to identify the 
#'       analysis type}
#'     \item{p_threshold}{Numeric. The \emph{p}-value threshold used for significance}
#'     \item{significant}{Character vector. Names of predictors with \emph{p}-value 
#'       below the screening threshold, suitable for passing directly to 
#'       downstream modeling functions}
#'   }
#'
#' @details
#' \strong{Analysis Approach:}
#' 
#' The function implements a comprehensive univariable screening workflow:
#' \enumerate{
#'   \item For each predictor in \code{predictors}, fits a separate model: 
#'     \code{outcome ~ predictor}
#'   \item Extracts coefficients, confidence intervals, and \emph{p}-values from each model
#'   \item Combines results into a single table for easy comparison
#'   \item Formats output for publication with appropriate effect measures
#' }
#' 
#' Each predictor is tested \emph{independently} - these are crude (unadjusted) 
#' associations that do not account for confounding or interaction effects.
#' 
#' \strong{When to Use Univariable Screening:}
#' \itemize{
#'   \item \strong{Initial variable selection}: Identify predictors associated 
#'     with the outcome before building multivariable models
#'   \item \strong{Hypothesis generation}: Explore potential associations in 
#'     exploratory analyses
#'   \item \strong{Understanding crude associations}: Report unadjusted effects 
#'     alongside adjusted estimates
#'   \item \strong{Variable reduction}: Use \emph{p}-value thresholds (\emph{e.g.,}
#'     \emph{p} < 0.20) to identify candidates for multivariable modeling
#'   \item \strong{Checking multicollinearity}: Compare univariable and 
#'     multivariable effects to identify potential collinearity
#' }
#' 
#' \strong{Threshold for \emph{p}-values:}
#' 
#' The \code{p_threshold} parameter controls the significance threshold used 
#' in the printed summary to count how many predictors are significant. All
#' predictors are always included in the output table regardless of this setting.
#' 
#' \strong{Effect Measures by Model Type:}
#' \itemize{
#'   \item \strong{Logistic regression} (\code{model_type = "glm"}, 
#'     \code{family = "binomial"}): Odds ratios (OR)
#'   \item \strong{Cox regression} (\code{model_type = "coxph"}): Hazard ratios (HR)
#'   \item \strong{Poisson regression} (\code{model_type = "glm"}, 
#'     \code{family = "poisson"}): Rate/risk ratios (RR)
#'   \item \strong{Negative binomial} (\code{model_type = "negbin"}): Rate ratios (RR)
#'   \item \strong{Linear regression} (\code{model_type = "lm"} or GLM with 
#'     identity link): Raw coefficient estimates
#'   \item \strong{Gamma regression} (\code{model_type = "glm"}, 
#'     \code{family = "Gamma"}): Multiplicative effects (with default log link)
#' }
#' 
#' \strong{Memory Considerations:}
#' 
#' When \code{keep_models = FALSE} (default), fitted models are discarded after 
#' extracting results to conserve memory. Set \code{keep_models = TRUE} only when 
#' the following are needed:
#' \itemize{
#'   \item Model diagnostic plots
#'   \item Predictions from individual models
#'   \item Additional model statistics not extracted by default
#'   \item Further analysis of specific models
#' }
#'
#' @seealso 
#' \code{\link{fit}} for fitting a single multivariable model,
#' \code{\link{fullfit}} for complete univariable-to-multivariable workflow,
#' \code{\link{compfit}} for comparing multiple models,
#' \code{\link{m2dt}} for converting individual models to tables,
#' \code{\link{uniforest}} for forest plots of screening results,
#' \code{\link{forestsave}} for exporting forest plots,
#' \code{\link{tablesave}} for exporting tables
#'
#' @examples
#' # Load example data
#' data(clintrial)
#' data(clintrial_labels)
#' 
#' # Example 1: Basic logistic regression screening
#' screen1 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "smoking", "hypertension"),
#'     model_type = "glm",
#'     family = "binomial",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(screen1)
#'
#' \donttest{
#' 
#' # Example 2: With custom variable labels
#' screen2 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "treatment"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(screen2)
#' 
#' # Example 3: Filter by p-value threshold
#' # Only keep predictors with p < 0.20 (common for screening)
#' screen3 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "smoking", "hypertension", 
#'                   "diabetes", "stage"),
#'     p_threshold = 0.20,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(screen3)
#' # Only significant predictors are shown
#' 
#' # Example 4: Cox proportional hazards screening
#' library(survival)
#' cox_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "sex", "treatment", "stage", "grade"),
#'     model_type = "coxph",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(cox_screen)
#' # Returns hazard ratios (HR) instead of odds ratios
#' 
#' # Example 5: Keep models for diagnostics
#' screen5 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "bmi", "creatinine"),
#'     keep_models = TRUE,
#'     parallel = FALSE
#' )
#' 
#' # Access stored models
#' models <- attr(screen5, "models")
#' summary(models[["age"]])
#' plot(models[["age"]])  # Diagnostic plots
#' 
#' # Example 6: Linear regression screening
#' linear_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "bmi",
#'     predictors = c("age", "sex", "smoking", "creatinine", "hemoglobin"),
#'     model_type = "lm",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(linear_screen)
#' 
#' # Example 7: Poisson regression for equidispersed count outcomes
#' # fu_count has variance ~= mean, appropriate for standard Poisson
#' poisson_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "fu_count",
#'     predictors = c("age", "stage", "treatment", "surgery"),
#'     model_type = "glm",
#'     family = "poisson",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(poisson_screen)
#' # Returns rate ratios (RR)
#' 
#' # Example 8: Negative binomial for overdispersed counts
#' # ae_count has variance > mean (overdispersed), use negbin
#' if (requireNamespace("MASS", quietly = TRUE)) {
#'     nb_screen <- uniscreen(
#'         data = clintrial,
#'         outcome = "ae_count",
#'         predictors = c("age", "treatment", "diabetes", "surgery"),
#'         model_type = "negbin",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     )
#'     print(nb_screen)
#' }
#' 
#' # Example 9: Gamma regression for positive continuous outcomes (\emph{e.g.,} costs)
#' gamma_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "los_days",
#'     predictors = c("age", "sex", "treatment", "surgery"),
#'     model_type = "glm",
#'     family = Gamma(link = "log"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(gamma_screen)
#' 
#' # Example 10: Hide reference rows for factor variables
#' screen10 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("treatment", "stage", "grade"),
#'     reference_rows = FALSE,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(screen10)
#' 
#' # Example 11: Customize decimal places
#' screen11 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "bmi", "creatinine"),
#'     digits = 3,      # 3 decimal places for OR
#'     p_digits = 4,    # 4 decimal places for p-values
#'     labels = clintrial_labels
#' )
#' print(screen11)
#' 
#' # Example 12: Hide sample size and event columns
#' screen12 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi"),
#'     show_n = FALSE,
#'     show_events = FALSE,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(screen12)
#' 
#' # Example 13: Access raw numeric data
#' screen13 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "treatment"),
#'     parallel = FALSE
#' )
#' raw_data <- attr(screen13, "raw_data")
#' print(raw_data)
#' 
#' # Example 14: Force coefficient display instead of OR
#' screen14 <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "bmi"),
#'     model_type = "glm",
#'     family = "binomial",
#'     labels = clintrial_labels,
#'     parallel = FALSE,
#'     exponentiate = FALSE  # Show log odds instead of OR
#' )
#' print(screen14)
#' 
#' # Example 15: Screening with weights
#' screen15 <- uniscreen(
#'     data = clintrial,
#'     outcome = "Surv(os_months, os_status)",
#'     predictors = c("age", "sex", "bmi"),
#'     model_type = "coxph",
#'     weights = runif(nrow(clintrial), min = 0.5, max = 2),  # Random numbers for example
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' 
#' # Example 16: Strict significance filter (p < 0.05)
#' sig_only <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "smoking", "hypertension", 
#'                   "diabetes", "ecog", "treatment", "stage", "grade"),
#'     p_threshold = 0.05,
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' 
#' # Check how many predictors passed the filter
#' n_significant <- length(unique(sig_only$Variable[sig_only$Variable != ""]))
#' cat("Significant predictors:", n_significant, "\n")
#' 
#' # Example 17: Complete workflow - screen then use in multivariable
#' # Step 1: Screen with liberal threshold
#' candidates <- uniscreen(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = c("age", "sex", "bmi", "smoking", "hypertension",
#'                   "diabetes", "treatment", "stage", "grade"),
#'     p_threshold = 0.20,
#'     parallel = FALSE
#' )
#' 
#' # Step 2: Extract significant predictor names
#' sig_predictors <- attr(candidates, "significant")
#' 
#' # Step 3: Fit multivariable model with selected predictors
#' multi_model <- fit(
#'     data = clintrial,
#'     outcome = "os_status",
#'     predictors = sig_predictors,
#'     labels = clintrial_labels
#' )
#' print(multi_model)
#'
#' # Example 18: Mixed-effects logistic regression (glmer)
#' # Accounts for clustering by site
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     glmer_screen <- uniscreen(
#'         data = clintrial,
#'         outcome = "os_status",
#'         predictors = c("age", "sex", "treatment", "stage"),
#'         model_type = "glmer",
#'         random = "(1|site)",
#'         family = "binomial",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     )
#'     print(glmer_screen)
#' }
#'
#' # Example 19: Mixed-effects linear regression (lmer)
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     lmer_screen <- uniscreen(
#'         data = clintrial,
#'         outcome = "biomarker_x",
#'         predictors = c("age", "sex", "treatment", "smoking"),
#'         model_type = "lmer",
#'         random = "(1|site)",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     )
#'     print(lmer_screen)
#' }
#'
#' # Example 20: Mixed-effects Cox model (coxme)
#' if (requireNamespace("coxme", quietly = TRUE)) {
#'     coxme_screen <- uniscreen(
#'         data = clintrial,
#'         outcome = "Surv(os_months, os_status)",
#'         predictors = c("age", "sex", "treatment", "stage"),
#'         model_type = "coxme",
#'         random = "(1|site)",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     )
#'     print(coxme_screen)
#' }
#'
#' # Example 21: Mixed-effects with nested random effects
#' # Patients nested within sites
#' if (requireNamespace("lme4", quietly = TRUE)) {
#'     nested_screen <- uniscreen(
#'         data = clintrial,
#'         outcome = "os_status",
#'         predictors = c("age", "treatment"),
#'         model_type = "glmer",
#'         random = "(1|site/patient_id)",
#'         family = "binomial",
#'         labels = clintrial_labels,
#'         parallel = FALSE
#'     )
#' }
#' 
#' # Example 22: Quasipoisson for overdispersed count data
#' # Alternative to negative binomial when MASS not available
#' quasi_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "ae_count",
#'     predictors = c("age", "treatment", "diabetes", "surgery", "stage"),
#'     model_type = "glm",
#'     family = "quasipoisson",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(quasi_screen)
#' # Adjusts standard errors for overdispersion
#' 
#' # Example 23: Quasibinomial for overdispersed binary data
#' quasibin_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "any_complication",
#'     predictors = c("age", "bmi", "diabetes", "surgery", "ecog"),
#'     model_type = "glm",
#'     family = "quasibinomial",
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(quasibin_screen)
#' 
#' # Example 24: Inverse Gaussian for highly skewed positive data
#' invgauss_screen <- uniscreen(
#'     data = clintrial,
#'     outcome = "recovery_days",
#'     predictors = c("age", "surgery", "pain_score", "los_days"),
#'     model_type = "glm",
#'     family = inverse.gaussian(link = "log"),
#'     labels = clintrial_labels,
#'     parallel = FALSE
#' )
#' print(invgauss_screen)
#'
#' }
#' 
#' @family regression functions
#' @export
uniscreen <- function(data,
                      outcome,
                      predictors,
                      model_type = "glm",
                      family = "binomial",
                      random = NULL,
                      p_threshold = 0.05,
                      conf_level = 0.95,
                      reference_rows = TRUE,
                      show_n = TRUE,
                      show_events = TRUE,
                      digits = 2,
                      p_digits = 3,
                      labels = NULL,
                      keep_models = FALSE,
                      exponentiate = NULL,
                      conf_method = NULL,
                      parallel = TRUE,
                      n_cores = NULL,
                      number_format = NULL,
                      verbose = NULL,
                      ...) {
    
    ## Convert to data.table once at the start
    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    
    ## Resolve verbose setting
    verbose <- if (is.null(verbose)) getOption("summata.verbose", FALSE) else verbose
    
    ## Validate inputs and auto-correct model type if needed
    validation <- validate_fit_inputs(
        data = data,
        outcome = outcome,
        predictors = predictors,
        model_type = model_type,
        family = if (model_type %in% c("glm", "glmer")) family else NULL,
        conf_level = conf_level,
        digits = digits,
        p_digits = p_digits,
        p_threshold = p_threshold,
        auto_correct_model = TRUE
    )
    
    ## Apply any auto-corrections
    if (validation$auto_corrected) {
        model_type <- validation$model_type
    }
    
    ## Resolve Gamma family string to log link (see fit.R for rationale)
    if (model_type %in% c("glm", "glmer") && is.character(family) &&
        tolower(family) == "gamma") {
        family <- stats::Gamma(link = "log")
    }

    ## Validate random effects for mixed-effects models
    ## Support both explicit 'random' parameter AND random effects embedded in predictors
    mixed_types <- c("glmer", "lmer", "coxme")
    has_random_in_predictors <- any(grepl("\\|", predictors))
    
    if (model_type %in% mixed_types && is.null(random) && !has_random_in_predictors) {
        stop("'random' parameter is required for mixed-effects models (", model_type, ").\n",
             "Example: random = \"(1|site)\" for random intercepts by site.\n",
             "Alternatively, include random effects in 'predictors' using the same syntax.")
    }
    if (!is.null(random) && !model_type %in% mixed_types) {
        warning("'random' parameter is ignored for non-mixed-effects models (", model_type, ").")
        random <- NULL
    }
    
    ## Extract random effects from predictors if present (for mixed-effects models)
    ## These will be used in formula building but not iterated over as predictors
    random_in_preds <- NULL
    fixed_predictors <- predictors
    if (model_type %in% mixed_types && has_random_in_predictors) {
        random_mask <- grepl("\\|", predictors)
        random_in_preds <- predictors[random_mask]
        fixed_predictors <- predictors[!random_mask]
        
        ## Combine explicit random with embedded random (if both provided)
        if (!is.null(random)) {
            ## User provided both - combine them
            message("Note: Random effects specified in both 'random' parameter and 'predictors'. ",
                    "Combining: ", random, " + ", paste(random_in_preds, collapse = " + "))
            all_random <- paste(c(random, random_in_preds), collapse = " + ")
            random <- all_random
        } else {
            ## Only embedded random effects
            random <- paste(random_in_preds, collapse = " + ")
        }
        
        ## Update predictors to only include fixed effects for iteration
        predictors <- fixed_predictors
        
        if (length(predictors) == 0) {
            stop("No fixed-effect predictors specified. Random effects should be specified ",
                 "via the 'random' parameter or alongside fixed-effect predictors.")
        }
    }
    
    ## Define model fitting function (used by lapply/mclapply)
    fit_one_predictor <- function(pred) {
        ## Build formula - for mixed-effects, append random effects
        formula_str <- paste(outcome, "~", pred)
        if (model_type %in% c("glmer", "lmer")) {
            formula_str <- paste(formula_str, "+", random)
        }
        formula <- stats::as.formula(formula_str)
        
        ## Fit model based on type
        ## Use model=FALSE for glm/lm when not keeping models to save memory
        model <- quiet_fit(quote(
            switch(model_type,
                   "glm" = stats::glm(formula, data = data, family = family, 
                                      model = keep_models, x = FALSE, y = TRUE, ...),
                   "negbin" = {
                       if (!requireNamespace("MASS", quietly = TRUE))
                           stop("Package 'MASS' required for negative binomial models")
                       MASS::glm.nb(formula, data = data, 
                                     model = keep_models, x = FALSE, y = TRUE, ...)
                   },
                   "lm" = stats::lm(formula, data = data, 
                                    model = keep_models, x = FALSE, y = TRUE, ...),
                   "coxph" = {
                       if (!requireNamespace("survival", quietly = TRUE)) 
                           stop("Package 'survival' required for Cox models")
                       survival::coxph(formula, data = data, 
                                       model = keep_models, x = FALSE, y = TRUE, ...)
                   },
                   "clogit" = {
                       if (!requireNamespace("survival", quietly = TRUE))
                           stop("Package 'survival' required for conditional logistic regression")
                       survival::clogit(formula, data = data, ...)
                   },
                   "glmer" = {
                       if (!requireNamespace("lme4", quietly = TRUE))
                           stop("Package 'lme4' required for glmer models")
                       lme4::glmer(formula, data = data, family = family, ...)
                   },
                   "lmer" = {
                       if (!requireNamespace("lme4", quietly = TRUE))
                           stop("Package 'lme4' required for lmer models")
                       lme4::lmer(formula, data = data, ...)
                   },
                   "coxme" = {
                       if (!requireNamespace("coxme", quietly = TRUE))
                           stop("Package 'coxme' required for coxme models")
                       if (!requireNamespace("survival", quietly = TRUE))
                           stop("Package 'survival' required for coxme models")
                       ## coxme uses different formula syntax - random in formula
                       coxme_formula <- stats::as.formula(paste(outcome, "~", pred, "+", random))
                       coxme::coxme(coxme_formula, data = data, ...)
                   },
                   stop("Unsupported model type: ", model_type)
                   )
        ), verbose = verbose)
        
        raw_result <- m2dt(
            data = data,
            model = model,
            conf_level = conf_level,
            keep_qc_stats = FALSE,
            include_intercept = FALSE,
            reference_rows = reference_rows,
            skip_counts = (!show_n && !show_events),
            conf_method = conf_method
        )

        ## Add predictor name for tracking
        raw_result[, predictor := pred]
        
        ## Return both raw result and optionally the model
        list(
            raw = raw_result,
            model = if (keep_models) model else NULL
        )
    }
    
    ## Fit models (parallel or sequential)
    can_use_windows_parallel <- .Platform$OS.type == "windows" && 
        interactive() && 
        !nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_", ""))
    
    use_parallel <- parallel && 
        length(predictors) > 1 && 
        (.Platform$OS.type != "windows" || can_use_windows_parallel)
    
    if (use_parallel) {
        ## Determine number of cores
        if (is.null(n_cores)) {
            n_cores <- max(1L, parallel::detectCores() - 1L)
        }
        ## CRAN policy: respect _R_CHECK_LIMIT_CORES_ environment variable
        chk <- tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_", ""))
        if (nzchar(chk) && chk == "true") {
            n_cores <- min(n_cores, 2L)
        }
        
        if (.Platform$OS.type == "windows") {
            ## Windows: use parLapply() with PSOCK cluster
            ## Only reaches here in interactive sessions with package installed
            cl <- parallel::makeCluster(n_cores)
            on.exit(parallel::stopCluster(cl), add = TRUE)
            
            ## Export the fit_one_predictor closure and all required local objects
            parallel::clusterExport(cl, 
                                    varlist = c("fit_one_predictor", "data", "outcome", 
                                                "model_type", "family", "random", 
                                                "conf_level", "reference_rows", 
                                                "keep_models", "show_n", "show_events",
                                                "conf_method"),
                                    envir = environment()
                                    )
            
            ## Export m2dt from summata namespace
            parallel::clusterExport(cl, 
                                    varlist = "m2dt",
                                    envir = asNamespace("summata")
                                    )
            
            ## Load required packages on workers
            parallel::clusterEvalQ(cl, library(data.table))
            
            ## Load survival if needed
            if (model_type %in% c("coxph", "clogit", "coxme")) {
                parallel::clusterEvalQ(cl, library(survival))
            }
            
            ## Load lme4 if needed
            if (model_type %in% c("glmer", "lmer")) {
                parallel::clusterEvalQ(cl, library(lme4))
            }
            
            ## Load coxme if needed
            if (model_type == "coxme") {
                parallel::clusterEvalQ(cl, library(coxme))
            }
            
            results <- parallel::parLapply(cl, predictors, fit_one_predictor)
            
        } else {
            ## Unix/Mac: use mclapply (fork-based, shares memory so no export needed)
            results <- parallel::mclapply(
                                     predictors, 
                                     fit_one_predictor,
                                     mc.cores = n_cores
                                 )
        }
    } else {
        ## Sequential processing with lapply()
        ## Used when: parallel = FALSE, single predictor, or Windows in non-interactive context
        results <- lapply(predictors, fit_one_predictor)
    }
    
    ## Extract raw results and combine
    raw_results <- lapply(results, `[[`, "raw")
    combined_raw <- data.table::rbindlist(raw_results, fill = TRUE)
    
    ## Extract models if kept
    if (keep_models) {
        models <- lapply(results, `[[`, "model")
        names(models) <- predictors
        models <- models[!vapply(models, is.null, logical(1))]
    }
    
    ## Note: p_threshold is used for reporting "Significant" count, not filtering
    
    ## Format results
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    formatted <- format_model_table(
        combined_raw,
        show_n = show_n,
        show_events = show_events,
        digits = digits,
        p_digits = p_digits,
        labels = labels,
        exponentiate = exponentiate,
        conf_level = conf_level,
        marks = marks
    )
    
    ## Attach attributes
    data.table::setattr(formatted, "raw_data", combined_raw)

    ## Each predictor is screened in its own model, so the analyzed sample
    ## is recorded per predictor and reported as a range where it varies.
    us_outcome_vars <- all.vars(stats::as.formula(paste("~", outcome)))
    data.table::setattr(
        formatted, "analysis_counts",
        get_analysis_counts(
            data,
            us_outcome_vars,
            lapply(predictors, function(p) {
                all.vars(stats::as.formula(paste("~", p)))
            }),
            get_event_variable_for_counts(us_outcome_vars, model_type, family)
        )
    )
    data.table::setattr(formatted, "number_marks", marks)
    ## Variable labels travel with the result so that the forest plot
    ## functions can apply the same labels without being given them again.
    ## An explicit labels argument to those functions still takes precedence.
    data.table::setattr(formatted, "labels", labels)
    
    if (keep_models) {
        data.table::setattr(formatted, "models", models)
    }
    
    data.table::setattr(formatted, "outcome", outcome)
    data.table::setattr(formatted, "model_type", unique(combined_raw$model_type)[1])
    data.table::setattr(formatted, "model_scope", "Univariable")
    data.table::setattr(formatted, "screening_type", "univariable")
    data.table::setattr(formatted, "p_threshold", p_threshold)
    
    ## Extract significant predictor names
    if ("p_value" %in% names(combined_raw)) {
        sig_predictors <- unique(
            combined_raw[!is.na(p_value) & p_value < p_threshold]$predictor
        )
        data.table::setattr(formatted, "significant", sig_predictors)
    }
    
    class(formatted) <- c("uniscreen_result", class(formatted))
    
    return(formatted)
}


### * Print function

#' Print method for uniscreen results
#'
#' @param x Object of class \code{uniscreen_result}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @family regression functions
#' @export
print.uniscreen_result <- function(x, ...) {
    cat("\nUnivariable Screening Results\n\n")
    cat("Outcome: ", attr(x, "outcome"), "\n", sep = "")
    cat("Model Type: ", attr(x, "model_type"), "\n", sep = "")
    
    n_predictors <- length(unique(x$Variable[x$Variable != ""]))
    cat("Predictors Screened: ", n_predictors, "\n", sep = "")
    
    raw <- attr(x, "raw_data")
    p_thresh <- attr(x, "p_threshold")
    if (is.null(p_thresh)) p_thresh <- 0.05
    
    if (!is.null(raw) && "p_value" %in% names(raw)) {
        sig_predictors <- unique(raw[p_value < p_thresh]$predictor)
        n_sig <- length(sig_predictors)
        cat("Significant (p < ", p_thresh, "): ", n_sig, "\n", sep = "")
    }
    
    ## Always reported, so that a complete sample is stated rather than left
    ## to be distinguished from an absent disclosure. Each predictor is
    ## screened separately, so the analyzed sample may be a range.
    us_counts <- attr(x, "analysis_counts")
    
    us_marks <- attr(x, "number_marks")
    
    analyzed <- format_analysis_counts(us_counts, marks = us_marks)
    if (!is.null(analyzed)) {
        cat(analyzed, "\n", sep = "")
    }
    
    ## Events, not observations, constrain a screening workflow, so they are
    ## reported alongside where the model type carries them
    events_line <- format_event_counts(us_counts, marks = us_marks)
    if (!is.null(events_line)) {
        cat(events_line, "\n", sep = "")
    }
    
    if (!is.null(attr(x, "models"))) {
        cat("Models stored: Yes (", length(attr(x, "models")), ")\n", sep = "")
    }
    
    cat("\n")
    NextMethod("print", x)
    invisible(x)
}

### * Main function

#' Create Forest Plot for Generalized Linear Models
#'
#' Generates a publication-ready forest plot that combines a formatted data table 
#' with a graphical representation of effect estimates (odds ratios, risk ratios, 
#' or coefficients) from a generalized linear model. The plot integrates variable 
#' names, group levels, sample sizes, effect estimates with confidence intervals, 
#' \emph{p}-values, and model diagnostics in a single comprehensive visualization designed 
#' for manuscripts and presentations.
#'
#' @param x Either a fitted GLM object (class \code{glm} or \code{glmerMod}), 
#'   a \code{fit_result} object from \code{fit()}, or a \code{fullfit_result}
#'   object from \code{fullfit()}. When a \code{fit_result} or \code{fullfit_result}
#'   is provided, the model, data, and labels are automatically extracted.
#'   
#' @param data Data frame or data.table containing the original data used to 
#'   fit the model. If \code{NULL} (default) and \code{x} is a model, the function 
#'   attempts to extract data from the model object. If \code{x} is a \code{fit_result},
#'   data is extracted automatically. Providing data explicitly is recommended when
#'   passing a model directly.
#'   
#' @param title Character string specifying the plot title displayed at the top. 
#'   Default is \code{"Generalized Linear Model"}. Use descriptive titles like 
#'   "Risk Factors for Disease Outcome" for publication.
#'   
#' @param effect_label Character string for the effect measure label on the 
#'   forest plot axis. If \code{NULL} (default), automatically determined based 
#'   on model family and link function: "Odds Ratio" for logistic regression 
#'   (\code{family = binomial, link = logit}), "Risk Ratio" for log-link models, 
#'   "Exp(Coefficient)" for other exponential families, or "Coefficient" for 
#'   identity link.
#'   
#' @param digits Integer specifying the number of decimal places for effect 
#'   estimates and confidence intervals in the data table. Default is 2.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param conf_level Numeric confidence level for confidence intervals. Must be
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
#'   percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
#'   when \code{conf_level = 0.90}).
#'   
#' @param font_size Numeric multiplier controlling the base font size for all 
#'   text elements. Values > 1 increase all fonts proportionally, values < 1 
#'   decrease them. Default is 1.0. Useful for adjusting readability across 
#'   different output sizes.
#'   
#' @param annot_size Numeric value controlling the relative font size for 
#'   data annotations (variable names, values in table cells). Default is 3.88. 
#'   Adjust relative to \code{font_size}.
#'   
#' @param header_size Numeric value controlling the relative font size for 
#'   column headers ("Variable", "Group", "n", \emph{etc.}). Default is 5.82. Headers 
#'   are typically larger than annotations for hierarchy.
#'   
#' @param title_size Numeric value controlling the relative font size for the 
#'   main plot title. Default is 23.28. The title is typically the largest text 
#'   element.
#'   
#' @param table_width Numeric value between 0 and 1 specifying the proportion of 
#'   total plot width allocated to the data table (left side). The forest plot 
#'   occupies \code{1 - table_width}. Default is 0.6 (60\% table, 40\% forest). 
#'   Increase for longer variable names, decrease to emphasize the forest plot.
#'   
#' @param plot_width Numeric value specifying the intended output width in 
#'   specified \code{units}. Used for optimizing layout and text sizing. 
#'   Default is \code{NULL} (automatic). Recommended: 10-16 inches for standard 
#'   publications.
#'   
#' @param plot_height Numeric value specifying the intended output height in 
#'   specified \code{units}. Default is \code{NULL} (automatic based on number 
#'   of rows). The function provides recommendations if not specified.
#'   
#' @param show_n Logical. If \code{TRUE}, includes a column showing group-specific 
#'   sample sizes for categorical variables and the fitted sample size for
#'   continuous variables. Counts describe the observations used in fitting
#'   rather than every row supplied. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes a column showing the 
#'   number of events for each group. Relevant for logistic regression (number 
#'   of cases) and other binary outcomes. Default is \code{TRUE}.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels under 
#'   their parent variable name, creating a hierarchical visual structure. When 
#'   \code{TRUE}, the "Group" column is hidden. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses binary categorical 
#'   variables into single rows by showing only the non-reference category. 
#'   Automatically sets \code{indent_groups = TRUE}. Useful for tables with 
#'   many binary variables. Default is \code{FALSE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. If \code{FALSE} (default), variable names are displayed in plain
#'   text.
#'   
#' @param center_padding Numeric value specifying the horizontal spacing (in 
#'   character units) between the data table and forest plot. Increase for more 
#'   separation, decrease to fit more content. Default is 4.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating gray 
#'   background shading to different variables (not rows) to improve visual 
#'   grouping and readability. Default is \code{TRUE}.
#'   
#' @param ref_label Character string to display for reference categories of 
#'   factor variables. Typically shown in place of effect estimates. 
#'   Default is \code{"reference"}. Common alternatives: "ref", "1.00 (ref)".
#'   
#' @param labels Named character vector or list providing custom display 
#'   labels for variables. Names should match variable names in the model, 
#'   values are the labels to display. Example: 
#'   \code{c(age = "Age (years)", bmi = "Body Mass Index")}. Default is
#'   \code{NULL}, in which case the labels attached by the function that produced
#'   \code{x} are used, where \code{x} is a \pkg{summata} result rather than a
#'   model object. Original variable names are used where neither is available.
#' @param color Character string specifying the color for effect estimate point 
#'   markers in the forest plot. Use hex codes or R color names. Default is 
#'   \code{NULL}, which auto-selects based on effect type: \code{"#4BA6B6"} 
#'   (teal) for odds ratios (binomial/quasibinomial with logit link), 
#'   \code{"#3F87EE"} (blue) for rate/risk ratios (Poisson, Gamma, inverse 
#'
#'   Gaussian with log link), and \code{"#5A8F5A"} (green) for coefficients 
#'   (Gaussian/identity link). This scheme matches \code{uniforest()} and 
#'   \code{multiforest()}. Choose colors that contrast well with black error bars.
#'   
#' @param exponentiate Logical. If \code{TRUE}, exponentiates coefficients to 
#'   display odds ratios, risk ratios, \emph{etc.} If \code{FALSE}, shows raw 
#'   coefficients. Default is \code{NULL}, which automatically exponentiates 
#'   for logit, log, and cloglog links, and shows raw coefficients for identity 
#'   link.
#'
#' @param qc_footer Logical. If \code{TRUE}, displays model quality control
#'   statistics in the footer (observations analyzed, model family, deviance,
#'   pseudo-\emph{R}\eqn{^2}, AIC). Default is \code{TRUE}.
#'   
#' @param units Character string specifying the units for plot dimensions. 
#'   Options: \code{"in"} (inches), \code{"cm"} (centimeters), \code{"mm"} 
#'   (millimeters). Default is \code{"in"}. Affects interpretation of 
#'   \code{plot_width} and \code{plot_height}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @return A \code{ggplot} object containing the complete forest plot. The plot 
#'   can be:
#'   \itemize{
#'     \item Displayed directly: \code{print(plot)}
#'     \item Saved to file: \code{forestsave(plot, "forest.pdf")}
#'     \item Further customized with \pkg{ggplot2} functions
#'   }
#'   
#'   The returned object includes an attribute \code{"rec_dims"} 
#'   accessible via \code{attr(plot, "rec_dims")}, which is a list 
#'   containing:
#'   \describe{
#'     \item{width}{Numeric. Recommended plot width in specified units}
#'     \item{height}{Numeric. Recommended plot height in specified units}
#'   }
#'   
#'   These recommendations are automatically calculated based on the number of 
#'   variables, text sizes, and layout parameters, and are printed to console 
#'   if \code{plot_width} or \code{plot_height} are not specified. The list also 
#'   carries a \code{units} element recording the units the dimensions are 
#'   expressed in, matching the \code{units} argument. \code{forestsave()} reads 
#'   all three and requires no further handling.
#'
#'   The returned object also includes an attribute \code{"table_data"}
#'   accessible via \code{attr(plot, "table_data")}, a data.table holding the
#'   values drawn in the plot: one row per term in model order, with the
#'   variable, factor level, sample size, event count, estimate, confidence
#'   bounds and p-value. Sample sizes and event counts describe the
#'   observations used in fitting rather than every row supplied, so they sum
#'   to the model sample size within each variable.
#'
#' @details
#' \strong{Plot Components:}
#' 
#' The forest plot consists of several integrated components:
#' \enumerate{
#'   \item \strong{Title}: Centered at top, describes the analysis
#'   \item \strong{Data Table} (left side): Contains columns for:
#'     \itemize{
#'       \item Variable: Predictor names (or custom labels)
#'       \item Group: Factor levels (optional, hidden when indenting)
#'       \item n: Sample sizes by group (optional)
#'       \item Events: Event counts by group (optional)
#'       \item Effect (95\% CI); \emph{p}-value: Formatted estimates with \emph{p}-values
#'     }
#'   \item \strong{Forest Plot} (right side): Graphical display with:
#'     \itemize{
#'       \item Point estimates (squares sized by sample size)
#'       \item 95\% confidence intervals (error bars)
#'       \item Reference line (at OR/RR = 1 or coefficient = 0)
#'       \item Log scale for odds/risk ratios
#'       \item Labeled axis
#'     }
#'   \item \strong{Model Statistics} (footer): Summary of:
#'     \itemize{
#'       \item Observations analyzed (with total observations and percentage)
#'       \item Model family (Binomial, Poisson, \emph{etc.})
#'       \item Deviance statistics
#'       \item Pseudo-\emph{R}\eqn{^2} (McFadden)
#'       \item AIC
#'     }
#' }
#' 
#' \strong{Automatic Effect Measure Selection:}
#' 
#' When \code{effect_label = NULL} and \code{exponentiate = NULL}, the function 
#' intelligently selects the appropriate effect measure:
#' \itemize{
#'   \item \strong{Logistic regression} (\code{family = binomial(link = "logit")}): 
#'     Odds Ratios (OR)
#'   \item \strong{Log-link models} (\code{link = "log"}): Risk Ratios (RR) 
#'     or Rate Ratios
#'   \item \strong{Other exponential families}: exp(coefficient)
#'   \item \strong{Identity link}: Raw coefficients
#' }
#' 
#' \strong{Reference Categories:}
#' 
#' For factor variables, the first level (determined by factor ordering or 
#' alphabetically for character variables) serves as the reference category:
#' \itemize{
#'   \item Displayed with the \code{ref_label} instead of an estimate
#'   \item No confidence interval or \emph{p}-value shown
#'   \item Visually aligned with other categories
#'   \item When \code{condense_table = TRUE}, reference-only variables may be 
#'     omitted entirely
#' }
#' 
#' \strong{Layout Optimization:}
#' 
#' The function automatically optimizes layout based on content:
#' \itemize{
#'   \item Calculates appropriate axis ranges to accommodate all confidence intervals
#'   \item Selects meaningful tick marks on log or linear scales
#'   \item Sizes point markers proportional to sample size (larger = more data)
#'   \item Adjusts table width based on variable name lengths when \code{table_width = NULL}
#'   \item Recommends overall dimensions based on number of rows
#' }
#' 
#' \strong{Visual Grouping Options:}
#' 
#' Three display modes are available:
#' \enumerate{
#'   \item \strong{Standard} (\code{indent_groups = FALSE}, 
#'     \code{condense_table = FALSE}): 
#'     Separate "Variable" and "Group" columns, all categories shown
#'   \item \strong{Indented} (\code{indent_groups = TRUE}, 
#'     \code{condense_table = FALSE}): 
#'     Hierarchical display with groups indented under variables
#'   \item \strong{Condensed} (\code{condense_table = TRUE}): 
#'     Binary variables shown in single rows, automatically indented
#' }
#' 
#' \strong{Zebra Striping:}
#' 
#' When \code{zebra_stripes = TRUE}, alternating variables (not individual rows) 
#' receive light gray backgrounds. This helps visually group all levels of a 
#' factor variable together, making the plot easier to read especially with 
#' many multi-level factors.
#' 
#' \strong{Model Statistics Display:}
#' 
#' The footer shows key diagnostic information:
#' \itemize{
#'   \item \strong{Observations analyzed}: Observations used in fitting, the 
#'     total supplied, and the percentage retained (accounting for missing 
#'     values)
#'   \item \strong{Null/Residual Deviance}: Model fit improvement
#'   \item \strong{Pseudo-\emph{R}\eqn{^2}}: McFadden \emph{R}\eqn{^2} = 1 - (log L_1 / log L_2)
#'   \item \strong{AIC}: For model comparison (lower is better)
#' }
#' 
#' For logistic regression, concordance (C-statistic/AUC) may also be displayed 
#' if available.
#' 
#' \strong{Saving Plots:}
#' 
#' Use \code{forestsave()}, which applies the recommended dimensions and
#' selects a graphics device suited to the format:
#' \preformatted{
#'   p <- glmforest(model, data)
#'   forestsave(p, "forest.pdf")
#' }
#' 
#' Or specify custom dimensions:
#' \preformatted{
#' forestsave(p, "forest.png", width = 12, height = 8, dpi = 300)
#' }
#'
#' @seealso 
#' \code{\link{autoforest}} for automatic model detection,
#' \code{\link{coxforest}} for Cox proportional hazards forest plots,
#' \code{\link{lmforest}} for linear model forest plots,
#' \code{\link{uniforest}} for univariable screening forest plots,
#' \code{\link{multiforest}} for multi-outcome forest plots,
#' \code{\link[stats]{glm}} for fitting GLMs,
#' \code{\link{fit}} for regression modeling,
#'   \code{\link{forestsave}} for saving with recommended dimensions
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example model
#' model1 <- glm(os_status ~ age + sex + bmi + treatment,
#'               data = clintrial, family = binomial)
#'
#' # Example 1: Basic logistic regression forest plot
#' p <- glmforest(model1, data = clintrial)
#'
#' \donttest{
#' 
#' old_width <- options(width = 180)
#' 
#' # Example 2: With custom variable labels
#' plot2 <- glmforest(
#'     x = model1,
#'     data = clintrial,
#'     title = "Risk Factors for Mortality",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 3: Indented layout with formatting options
#' plot3 <- glmforest(
#'     x = model1,
#'     data = clintrial,
#'     indent_groups = TRUE,
#'     zebra_stripes = TRUE,
#'     color = "#D62728",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 4: Condensed layout for many binary variables
#' model4 <- glm(os_status ~ age + sex + smoking + hypertension + 
#'                   diabetes + surgery,
#'               data = clintrial,
#'               family = binomial)
#' 
#' plot4 <- glmforest(
#'     x = model4,
#'     data = clintrial,
#'     condense_table = TRUE,
#'     labels = clintrial_labels
#' )
#' # Binary variables shown in single rows
#' 
#' # Example 5: Poisson regression for count data
#' model5 <- glm(ae_count ~ age + treatment + diabetes + surgery,
#'                data = clintrial,
#'                family = poisson)
#' 
#' plot5 <- glmforest(
#'     x = model5,
#'     data = clintrial,
#'     title = "Rate Ratios for Adverse Events",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 6: Save with recommended dimensions
#' forestsave(plot5, file.path(tempdir(), "forest.pdf"))
#'
#' options(old_width)
#' 
#' }
#' @family visualization functions
#' @export
glmforest <- function(x, data = NULL,
                      title = "Generalized Linear Model",
                      effect_label = NULL,
                      digits = 2,
                      p_digits = 3,
                      conf_level = 0.95,
                      font_size = 1.0,
                      annot_size = 3.88,
                      header_size = 5.82,
                      title_size = 23.28,
                      plot_width = NULL,
                      plot_height = NULL,
                      table_width = 0.6,
                      show_n = TRUE,
                      show_events = TRUE,
                      indent_groups = FALSE,
                      condense_table = FALSE,
                      bold_variables = FALSE,
                      center_padding = 4,
                      zebra_stripes = TRUE,
                      ref_label = "reference",
                      labels = NULL,
                      color = NULL,
                      exponentiate = NULL,
                      qc_footer = TRUE,
                      units = "in",
                      number_format = NULL) {
    
    ## Check for required packages
    if (!requireNamespace("data.table", quietly = TRUE)) {
        stop("Package 'data.table' is required but not installed.")
    }
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required but not installed.")
    }
    if (!requireNamespace("grid", quietly = TRUE)) {
        stop("Package 'grid' is required but not installed.")
    }
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    ## Handle input: accept either a model object or a fit_result/fullfit_result from fit()/fullfit()
    if (inherits(x, "fit_result") || inherits(x, "fullfit_result")) {
        ## Extract model, data, and labels from fit_result or fullfit_result
        model <- attr(x, "model")
        
        if (is.null(model)) {
            stop("The fit_result/fullfit_result does not contain a model.\n",
                 "This may occur if fullfit() was run with columns='uni' only.")
        }
        
        ## Use data from fit_result if not provided
        if (is.null(data)) {
            ## Try to get data from model
            if (!is.null(model$data)) {
                data <- model$data
            } else if (!is.null(model$model)) {
                data <- model$model
            } else {
                tryCatch({
                    data <- eval(model$call$data)
                }, error = function(e) {
                    data <- NULL
                })
            }
        }
        
        ## Use labels from fit_result if not provided
        if (is.null(labels)) {
            labels <- attr(x, "labels")
        }
        
        ## Validate that the model is a GLM model
        if (!inherits(model, c("glm", "glmerMod", "merMod"))) {
            stop("fit_result does not contain a GLM model (glm or glmerMod).\n",
                 "Model type: ", class(model)[1], "\n",
                 "Use coxforest() for Cox models or lmforest() for linear models.")
        }
    } else if (inherits(x, c("glm", "glmerMod", "merMod"))) {
        ## Direct model object
        model <- x
    } else {
        stop("x must be either:\n",
             "  - A glm or glmerMod model object, or\n",
             "  - A fit_result from fit(), or\n",
             "  - A fullfit_result from fullfit()\n",
             "
Received class: ", paste(class(x), collapse = ", "))
    }
    
    ## Check model class - support both GLM and GLMER
    is_lme4 <- inherits(model, c("glmerMod", "merMod"))
    is_glm <- inherits(model, "glm")
    
    ## Internally work in inches
    if (!is.null(plot_width) && units != "in") {
        plot_width <- convert_units(plot_width, from = units, to = "in")
    }

    ## Determine whether to exponentiate, based on the link function
    if(is.null(exponentiate)) {
        if (is_lme4) {
            ## For lme4 models, extract family info differently
            family_obj <- model@resp$family
            link_func <- family_obj$link
        } else {
            ## For regular GLMs
            link_func <- model$family$link
        }
        exponentiate <- link_func %in% c("logit", "log", "cloglog")
    }
    
    ## Set effect label based on model family and link
    if(is.null(effect_label)) {
        if (is_lme4) {
            family_obj <- model@resp$family
            family_name <- family_obj$family
            link_func <- family_obj$link
        } else {
            family_name <- model$family$family
            link_func <- model$family$link
        }
        
        if(family_name %in% c("binomial", "quasibinomial") && link_func == "logit") {
            effect_label <- "Odds Ratio"
        } else if(family_name %in% c("poisson", "quasipoisson") && link_func == "log") {
            effect_label <- "Rate Ratio"
        } else if(family_name %in% c("Gamma", "inverse.gaussian") && link_func == "log") {
            effect_label <- "Ratio"
        } else if(link_func == "log") {
            effect_label <- "Risk Ratio"
        } else if(exponentiate) {
            effect_label <- "Exp(Coefficient)"
        } else {
            effect_label <- "Coefficient"
        }
    }
    
    ## Automatically disable show_events for continuous outcome families
    ## Events only make sense for binomial, poisson, quasibinomial, quasipoisson, negbin
    if (show_events) {
        if (is_lme4) {
            fam_for_events <- model@resp$family$family
        } else if (inherits(model, "negbin")) {
            fam_for_events <- "negbin"
        } else {
            fam_for_events <- model$family$family
        }
        
        event_families <- c("binomial", "quasibinomial", "poisson", "quasipoisson", "negbin")
        if (!fam_for_events %in% event_families) {
            show_events <- FALSE
        }
    }
    
    ## Set default color based on effect type (consistent with uniforest/multiforest)
    ## Color scheme based on what the effect measure represents:
    ##   - Odds ratios (OR): Teal #4BA6B6 - binomial/quasibinomial with logit link
    ##   - Rate/risk ratios (RR): Blue #3F87EE - poisson, Gamma, inverse.gaussian with log link
    ##   - Coefficients: Green #5A8F5A - gaussian, identity link models
    if (is.null(color)) {
        if (is_lme4) {
            family_name <- model@resp$family$family
            link_func <- model@resp$family$link
        } else {
            family_name <- model$family$family
            link_func <- model$family$link
        }
        
        if (family_name == "gaussian" || link_func == "identity") {
            color <- "#5A8F5A"  # Green for coefficient models
        } else if (family_name %in% c("binomial", "quasibinomial") && link_func == "logit") {
            color <- "#4BA6B6"  # Teal for odds ratio models
        } else {
            color <- "#3F87EE"  # Blue for all ratio models (Poisson, Gamma, \emph{etc.})
        }
    }
    
    ## Get both original and model data
    if (is_lme4) {
        ## For lme4 models, the model frame contains the complete cases
        model_data <- model@frame
    } else {
        ## For regular GLMs, use model$model which contains complete cases
        model_data <- model$model
    }
    
    ## Get the original data for reference (if provided)
    if(is.null(data)){
        warning("The `data` argument is not provided. Data will be extracted from model fit.")
        data <- model_data  # Use the model data as the base
    }
    
    ## A model fitted with model = FALSE retains no model frame. Group counts
    ## must still describe the observations used in fitting, so the supplied
    ## data is restricted to complete cases across the model variables rather
    ## than counted in full.
    if (is.null(model_data) && !is.null(data)) {
        model_data <- get_analysis_data(model, class(model)[1], data)
    }
    
    ## Convert to data.table if not already
    if(!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    if(!data.table::is.data.table(model_data)) {
        model_data <- data.table::as.data.table(model_data)
    }
    
    ## Extract terms based on model type
    if (is_lme4) {
        ## Get the fixed effects formula for lme4 models
        fixed_formula <- lme4::nobars(stats::formula(model))
        
        ## Get variable names from the fixed formula (excluding outcome)
        var_names <- all.vars(fixed_formula)[-1]  # Remove outcome variable
        
        ## Get the model frame for determining variable types
        model_frame <- model@frame
        
        ## Create terms object similar to GLM structure
        terms <- sapply(var_names, function(v) {
            if (v %in% names(model_frame)) {
                if (is.factor(model_frame[[v]])) "factor"
                else if (is.character(model_frame[[v]])) "character"
                else "numeric"
            } else if (v %in% names(data)) {
                if (is.factor(data[[v]])) "factor"
                else if (is.character(data[[v]])) "character"
                else "numeric"
            } else "numeric"
        }, USE.NAMES = TRUE)
        
    } else {
        ## For regular GLMs
        terms <- attr(model$terms, "dataClasses")[-1]
    }
    
    ## Filter out interaction terms (contain ":"), which are handled separately via coefficients
    terms <- terms[!grepl(":", names(terms), fixed = TRUE)]
    
    ## Extract coefficients and confidence intervals
    if (is_lme4) {
        ## For lme4 models
        coef_summary <- summary(model)$coefficients
        
        ## Calculate confidence intervals manually for lme4
        ## Using Wald confidence intervals with specified conf_level
        coef_vals <- lme4::fixef(model)
        se_vals <- sqrt(diag(as.matrix(vcov(model))))
        z_crit <- qnorm((1 + conf_level) / 2)
        
        conf_int <- cbind(
            lower = coef_vals - z_crit * se_vals,
            upper = coef_vals + z_crit * se_vals
        )
        rownames(conf_int) <- names(coef_vals)
        
        ## Extract z values; for lme4 these are calculated
        z_values <- coef_summary[, "Estimate"] / coef_summary[, "Std. Error"]
        p_values <- 2 * pnorm(abs(z_values), lower.tail = FALSE)
        
    } else {
        ## For regular GLMs (including glm.nb from MASS)
        coef_summary <- summary(model)$coefficients
        conf_int <- get_cached_confint(model, conf_level)
        
        ## Different GLM families use different column names for test statistics:
        ## - binomial, poisson, quasibinomial, quasipoisson, negbin: "z value", "Pr(>|z|)"
        ## - gaussian, Gamma, inverse.gaussian: "t value", "Pr(>|t|)"
        ## Handle both cases by checking which columns exist
        if ("z value" %in% colnames(coef_summary)) {
            z_values <- coef_summary[, "z value"]
            p_values <- coef_summary[, "Pr(>|z|)"]
        } else if ("t value" %in% colnames(coef_summary)) {
            z_values <- coef_summary[, "t value"]
            p_values <- coef_summary[, "Pr(>|t|)"]
        } else {
            ## Fallback: calculate z/t values manually from estimate and SE
            z_values <- coef_summary[, "Estimate"] / coef_summary[, "Std. Error"]
            ## Use normal approximation for \emph{p}-values
            p_values <- 2 * pnorm(abs(z_values), lower.tail = FALSE)
        }
    }
    
    ## Remove intercept if present
    if("(Intercept)" %in% rownames(coef_summary)) {
        intercept_idx <- which(rownames(coef_summary) == "(Intercept)")
        coef_summary <- coef_summary[-intercept_idx, , drop = FALSE]
        conf_int <- conf_int[-intercept_idx, , drop = FALSE]
        z_values <- z_values[-intercept_idx]
        p_values <- p_values[-intercept_idx]
    }
    
    ## Create coefficient data table
    coef <- data.table::data.table(
                            term = rownames(coef_summary),
                            estimate = coef_summary[, "Estimate"],
                            std_error = coef_summary[, "Std. Error"],
                            statistic = z_values,
                            p_value = p_values,
                            conf_low = conf_int[, 1],
                            conf_high = conf_int[, 2]
                        )
    
    ## Get model statistics
    if (is_lme4) {
        ## For lme4 models
        gmodel <- list(
            nobs = nrow(model@frame),
            null_deviance = NA,  # Not directly available in lme4
            residual_deviance = NA,  # Not directly available in lme4
            AIC = stats::AIC(model),
            family = paste0(model@resp$family$family, " (", model@resp$family$link, " link)")
        )
        
        ## Calculate deviances if possible
        tryCatch({
            ## Some lme4 models have deviance method
            gmodel$residual_deviance <- stats::deviance(model)
        }, error = function(e) {
            ## Keep as NA if not available
        })
        
    } else {
        ## For regular GLMs
        gmodel <- list(
            nobs = nobs(model),
            null_deviance = model$null.deviance,
            residual_deviance = model$deviance,
            AIC = stats::AIC(model),
            family = paste0(model$family$family, " (", model$family$link, " link)")
        )
    }
    
    ## Calculate total observations and percentage analyzed
    total_obs <- nrow(data)
    
    ## Events are reported alongside observations for the families that carry
    ## them. Events per predictor is the constraint on a logistic or count
    ## model, and a figure is read without the console output beside it.
    fp_event_families <- c("binomial", "quasibinomial", "poisson",
                           "quasipoisson", "negbin")
    fp_family <- if (is_lme4) {
                     model@resp$family$family
                 } else if (inherits(model, "negbin")) {
                     "negbin"
                 } else {
                     model$family$family
                 }

    gmodel$nevent <- NA_real_
    fp_total_events <- NA_real_

    if (fp_family %in% fp_event_families) {
        ## Coded as the modeling functions code it, matching the Events
        ## column built further below
        code_events <- function(y) {
            if (is.factor(y)) as.numeric(y) == 2 else y
        }
        fp_outcome_var <- all.vars(stats::formula(model))[1]

        if (is_lme4) {
            gmodel$nevent <- sum(code_events(model@resp$y), na.rm = TRUE)
        } else if (fp_outcome_var %in% names(model_data)) {
            gmodel$nevent <- sum(code_events(model_data[[fp_outcome_var]]),
                                 na.rm = TRUE)
        }

        if (fp_outcome_var %in% names(data)) {
            fp_total_events <- sum(code_events(data[[fp_outcome_var]]),
                                   na.rm = TRUE)
        }
    }

    ## Both lines are rendered by the formatter the print methods use, so that
    ## a figure and the table it accompanies disclose in identical terms
    gmodel$obs_line <- format_analysis_counts(
        list(n_supplied = total_obs, n_analyzed = gmodel$nobs),
        label = "Observations analyzed", marks = marks)

    gmodel$events_line <- format_analysis_counts(
        list(n_supplied = fp_total_events, n_analyzed = gmodel$nevent),
        label = "Events analyzed", marks = marks)

    ## Either line is dropped rather than shown without its denominator
    gmodel$counts_block <- paste(
        c(gmodel$obs_line, gmodel$events_line), collapse = "\n")
    aic_val <- format(round(gmodel$AIC, 2), big.mark = marks$big.mark, decimal.mark = marks$decimal.mark, scientific = FALSE, nsmall = 2)
    gmodel$AIC_formatted <- trimws(aic_val)
    
    ## Calculate pseudo R-squared (McFadden)
    if (!is.na(gmodel$null_deviance) && !is.na(gmodel$residual_deviance)) {
        gmodel$pseudo_r2 <- 1 - (gmodel$residual_deviance / gmodel$null_deviance)
    } else {
        gmodel$pseudo_r2 <- NA
    }
    
    ## Extract xlevels (factor levels) based on model type
    if (is_lme4) {
        ## For lme4 models, extract from the model frame
        frame_data <- model@frame
        xlevels <- list()
        
        for (var_name in names(terms)) {
            if (terms[var_name] %in% c("factor", "character")) {
                if (var_name %in% names(frame_data) && is.factor(frame_data[[var_name]])) {
                    xlevels[[var_name]] <- levels(frame_data[[var_name]])
                } else if (var_name %in% names(model_data) && is.factor(model_data[[var_name]])) {
                    xlevels[[var_name]] <- levels(model_data[[var_name]])
                }
            }
        }
        
        if (length(xlevels) == 0) xlevels <- NULL
        
    } else {
        ## For regular GLMs
        xlevels <- model$xlevels
    }
    
    ## Extract statistics for every variable, preserving order
    all_terms <- lapply(seq_along(terms), function(i){
        var <- names(terms)[i]
        
        if (terms[i] %in% c("factor", "character")) {
            ## Get the factor levels from xlevels (proper order)
            if(!is.null(xlevels) && var %in% names(xlevels)) {
                factor_levels <- xlevels[[var]]
                
                ## Create data table with proper levels FROM MODEL DATA
                level_counts <- model_data[!is.na(get(var)), .N, by = var]
                data.table::setnames(level_counts, c("level", "Freq"))
                
                ## Ensure all levels are present
                all_levels_dt <- data.table::data.table(
                                                 level = factor_levels,
                                                 Freq = 0
                                             )
                
                ## Update with actual counts
                all_levels_dt[level_counts, Freq := i.Freq, on = "level"]
                all_levels_dt[, var := var]
                all_levels_dt[, pos := .I]
                all_levels_dt[, var_order := i]
                return(all_levels_dt[, .(var, level, Freq, pos, var_order)])
            } else {
                ## Fallback for variables not in xlevels - use model data
                adf <- model_data[!is.na(get(var)), .N, by = var]
                data.table::setnames(adf, old = c(var, "N"), new = c("level", "Freq"))
                adf[, var := var]
                adf[, pos := .I]
                adf[, var_order := i]
                return(adf[, .(var, level, Freq, pos, var_order)])
            }
        }
        else if (terms[i] == "numeric") {
            ## For numeric variables, return a single row - use model data
            return(data.table::data.table(
                                   var = var, 
                                   level = "-", 
                                   Freq = sum(!is.na(model_data[[var]])), 
                                   pos = 1, 
                                   var_order = i
                               ))
        }
        else {
            ## Other cases
            vars = grep(paste0("^", var, "*."), coef$term, value=TRUE)
            return(data.table::data.table(
                                   var = vars, 
                                   level = "", 
                                   Freq = nrow(model_data),  # Use model_data row count
                                   pos = seq_along(vars), 
                                   var_order = i
                               ))
        }
    })
    
    ## Remove any NULL entries from the list
    all_terms <- all_terms[!sapply(all_terms, is.null)]
    
    ## Bind all terms together
    all_terms_df <- data.table::rbindlist(all_terms)
    data.table::setnames(all_terms_df, c("var", "level", "N", "pos", "var_order"))

    ## Add events for binomial, poisson, quasibinomial, quasipoisson, and negative binomial models
    ## NOT for gaussian, Gamma, inverse.gaussian (continuous outcomes)
    if (is_lme4) {
        family_name <- model@resp$family$family
    } else if (inherits(model, "negbin")) {
        family_name <- "negbin"
    } else {
        family_name <- model$family$family
    }

    ## Families where events are meaningful
    event_families <- c("binomial", "quasibinomial", "poisson", "quasipoisson", "negbin")
    
    if (family_name %in% event_families) {
        outcome_data <- NULL
        
        if (is_lme4) {
            ## For lme4 models, extract outcome from the response
            outcome_data <- model@resp$y
        } else {
            ## For regular GLMs and negbin, get outcome from model data
            outcome_var <- all.vars(stats::formula(model))[1]
            
            if (!outcome_var %in% names(model_data)) {
                warning(paste("Outcome variable", outcome_var, "not found in model data. Events column will not be created."))
            } else {
                outcome_data <- model_data[[outcome_var]]
                
                ## For binomial with factor outcome, convert to 0/1
                if (family_name %in% c("binomial", "quasibinomial") && is.factor(outcome_data)) {
                    outcome_data <- as.numeric(outcome_data) == 2
                }
            }
        }
        
        if (!is.null(outcome_data)) {
            all_terms_df[, Events := {
                if (level == "-") {
                    ## For continuous variables, total events
                    sum(outcome_data, na.rm = TRUE)
                } else {
                    ## For factor levels, events within that level
                    if (var %in% names(model_data)) {
                        sum(outcome_data[model_data[[var]] == level], na.rm = TRUE)
                    } else {
                        NA_integer_
                    }
                }
            }, by = seq_len(nrow(all_terms_df))]
        } else {
            all_terms_df[, Events := NA_integer_]
        }
    } else {
        all_terms_df[, Events := NA_integer_]
    }

    ## Interaction terms - use ":" as identifier
    interaction_terms <- coef$term[grepl(":", coef$term, fixed = TRUE)]
    
    if (length(interaction_terms) > 0) {
        ## Get the next var_order value
        next_var_order <- max(all_terms_df$var_order) + 1L
        
        ## Process each interaction term
        interaction_rows <- lapply(seq_along(interaction_terms), function(idx) {
            int_term <- interaction_terms[idx]
            
            ## Parse the interaction term to create a readable label
            ## \emph{e.g.,}, "treatmentDrug A:stageII" -> "Treatment Group (Drug A) × Disease Stage (II)"
            int_parts <- strsplit(int_term, ":", fixed = TRUE)[[1]]
            
            ## Build readable variable name from parts
            display_parts <- sapply(int_parts, function(part) {
                ## Try to match against known factor variables to extract var and level
                matched_result <- NULL
                for (var_name in names(terms)) {
                    if (startsWith(part, var_name)) {
                        level_str <- sub(paste0("^", var_name), "", part)
                        if (nchar(level_str) > 0) {
                            ## Apply label if available
                            var_label <- if (!is.null(labels) && var_name %in% names(labels)) {
                                             labels[[var_name]]
                                         } else {
                                             var_name
                                         }
                            matched_result <- paste0(var_label, " (", level_str, ")")
                            break
                        }
                    }
                }
                ## If no match found (continuous variable in interaction or exact match)
                if (is.null(matched_result)) {
                    var_label <- if (!is.null(labels) && part %in% names(labels)) {
                                     labels[[part]]
                                 } else {
                                     part
                                 }
                    matched_result <- var_label
                }
                matched_result
            })
            
            display_name <- paste(display_parts, collapse = " \u00d7 ")
            
            ## Calculate N for this interaction (observations where both conditions are met)
            int_n <- tryCatch({
                n_count <- NA_integer_
                
                ## Try to extract the subset count from the model data
                ## For each part of the interaction, build a condition
                conditions <- list()
                for (j in seq_along(int_parts)) {
                    part <- int_parts[j]
                    for (var_name in names(terms)) {
                        if (startsWith(part, var_name) && terms[var_name] %in% c("factor", "character")) {
                            level_str <- sub(paste0("^", var_name), "", part)
                            if (nchar(level_str) > 0 && var_name %in% names(model_data)) {
                                conditions[[var_name]] <- level_str
                            }
                            break
                        }
                    }
                }
                
                ## Where conditions were found, count matching rows
                if (length(conditions) > 0) {
                    ## Build subset expression safely
                    subset_dt <- data.table::copy(model_data)
                    for (cond_var in names(conditions)) {
                        subset_dt <- subset_dt[get(cond_var) == conditions[[cond_var]]]
                    }
                    n_count <- nrow(subset_dt)
                }
                
                n_count
            }, error = function(e) NA_integer_)
            
            ## Calculate Events for this interaction (for binomial models)
            int_events <- tryCatch({
                events_count <- NA_integer_
                
                if (family_name == "binomial" && !is.null(outcome_data) && length(conditions) > 0) {
                    ## Build subset for this interaction
                    subset_indices <- rep(TRUE, nrow(model_data))
                    for (cond_var in names(conditions)) {
                        subset_indices <- subset_indices & (model_data[[cond_var]] == conditions[[cond_var]])
                    }
                    events_count <- sum(outcome_data[subset_indices], na.rm = TRUE)
                }
                
                events_count
            }, error = function(e) NA_integer_)
            
            data.table::data.table(
                            var = display_name,
                            level = "-",
                            N = int_n,
                            pos = 1L,
                            var_order = next_var_order + idx - 1L,
                            Events = int_events,
                            .int_term = int_term  # Store raw term for later inds assignment
                        )
        })
        
        interaction_df <- data.table::rbindlist(interaction_rows)
        
        ## Bind interaction rows to main terms
        all_terms_df <- data.table::rbindlist(list(all_terms_df, interaction_df), 
                                              use.names = TRUE, fill = TRUE)
    }

    ## Apply condensing and indenting if requested
    if (condense_table || indent_groups) {
        if (condense_table) {
            indent_groups <- TRUE
        }
        
        ## For interaction rows, use the stored raw term; for others use var+level pattern
        if (".int_term" %in% names(all_terms_df)) {
            all_terms_df[, inds := data.table::fifelse(
                                                   !is.na(.int_term), 
                                                   .int_term,
                                                   data.table::fifelse(level == "-", var, paste0(var, level))
                                               )]
        } else {
            all_terms_df[, inds := data.table::fifelse(level == "-", var, paste0(var, level))]
        }
        orig_inds_map <- data.table::copy(all_terms_df[, .(var, level, inds, N, Events)])
        
        processed_rows <- list()
        row_counter <- 1
        unique_vars <- unique(all_terms_df[, var])
        
        for (v in unique_vars) {
            var_rows <- all_terms_df[var == v]
            
            if (nrow(var_rows) == 1) {
                processed_rows[[row_counter]] <- var_rows
                row_counter <- row_counter + 1
            } else {
                is_binary <- nrow(var_rows) == 2
                
                if (condense_table && is_binary) {
                    ## Detect reference row by checking if level is first in xlevels
                    ## (first level is always reference in R factor contrasts)
                    ref_level <- NULL
                    if (!is.null(xlevels) && v %in% names(xlevels)) {
                        ref_level <- xlevels[[v]][1]
                    }
                    
                    ## Find non-reference row
                    non_ref_idx <- NULL
                    if (!is.null(ref_level)) {
                        ref_idx <- which(var_rows$level == ref_level)
                        if (length(ref_idx) == 1) {
                            non_ref_idx <- setdiff(1:2, ref_idx)
                        }
                    }
                    
                    ## Fallback to estimate-based detection if available
                    if (is.null(non_ref_idx) && "estimate" %in% names(var_rows)) {
                        non_ref_idx <- find_non_reference_row(var_rows, "estimate")
                    }
                    
                    ## Final fallback: assume row 2 is non-reference
                    if (is.null(non_ref_idx) || length(non_ref_idx) != 1) {
                        non_ref_idx <- 2L
                        ref_idx <- 1L
                    } else {
                        ref_idx <- setdiff(1:2, non_ref_idx)
                    }
                    
                    non_ref_row <- var_rows[non_ref_idx]
                    ref_row <- var_rows[ref_idx]
                    
                    condensed_row <- data.table::copy(non_ref_row)
                    non_ref_category <- condensed_row$level
                    ref_category <- ref_row$level
                    
                    ## Look up label for smarter condensing detection
                    var_label <- if (!is.null(labels) && v %in% names(labels)) {
                                     labels[[v]]
                                 } else if (v %in% names(model_data) && 
                                            !is.null(attr(model_data[[v]], "label"))) {
                                     attr(model_data[[v]], "label")
                                 } else {
                                     v
                                 }
                    
                    ## Use greedy approach: condense if EITHER category is recognized
                    if (should_condense_binary(ref_category, non_ref_category, var_label)) {
                        condensed_row[, var := v]
                    } else {
                        condensed_row[, var := paste0(v, " (", non_ref_category, ")")]
                    }
                    condensed_row[, level := "-"]
                    processed_rows[[row_counter]] <- condensed_row
                    row_counter <- row_counter + 1
                    
                } else {
                    if (indent_groups) {
                        header_row <- data.table::data.table(
                                                      var = v,
                                                      level = "-",
                                                      N = NA_integer_,
                                                      Events = NA_integer_,
                                                      pos = var_rows$pos[1],
                                                      var_order = var_rows$var_order[1],
                                                      inds = NA_character_
                                                  )
                        processed_rows[[row_counter]] <- header_row
                        row_counter <- row_counter + 1
                        

                        for (i in seq_len(nrow(var_rows))) {
                            group_row <- data.table::copy(var_rows[i])
                            group_row[, var := paste0("    ", level)]
                            group_row[, level := "-"]
                            processed_rows[[row_counter]] <- group_row
                            row_counter <- row_counter + 1
                        }
                    } else {
                        for (i in seq_len(nrow(var_rows))) {
                            processed_rows[[row_counter]] <- var_rows[i]
                            row_counter <- row_counter + 1
                        }
                    }
                }
            }
        }
        
        all_terms_df <- data.table::rbindlist(processed_rows, fill = TRUE)
        
        ## Restore inds, N, and Events values for condensed/indented rows

        for (i in seq_len(nrow(all_terms_df))) {
            current_var <- all_terms_df$var[i]
            
            if (is.na(all_terms_df$inds[i]) && !grepl("^    ", current_var) && !grepl("\\(", current_var)) {
                next
            }
            
            if (grepl("\\(", current_var)) {
                orig_var <- gsub(" \\(.*\\)", "", current_var)
                orig_level <- gsub(".*\\((.*)\\)", "\\1", current_var)
                
                matching <- orig_inds_map[var == orig_var & level == orig_level]
                if (nrow(matching) > 0) {
                    all_terms_df[i, `:=`(inds = matching$inds[1],
                                         N = matching$N[1],
                                         Events = matching$Events[1])]
                }
            }
            else if (grepl("^    ", current_var)) {
                clean_level <- gsub("^    ", "", current_var)
                
                parent_var <- NA_character_
                for (j in (i-1):1) {
                    if (!grepl("^    ", all_terms_df$var[j]) && all_terms_df$var[j] != "") {
                        parent_var <- gsub(" \\(.*\\)", "", all_terms_df$var[j])
                        break
                    }
                }
                
                if (!is.na(parent_var)) {
                    matching <- orig_inds_map[var == parent_var & level == clean_level]
                    if (nrow(matching) > 0) {
                        all_terms_df[i, `:=`(inds = matching$inds[1],
                                             N = matching$N[1],
                                             Events = matching$Events[1])]
                    }
                }
            }
        }
    } else {
        ## For non-condensed/non-indented display, set inds for coefficient matching
        ## For interaction rows, use the stored raw term; for others use var+level pattern
        if (".int_term" %in% names(all_terms_df)) {
            all_terms_df[, inds := data.table::fifelse(
                                                   !is.na(.int_term), 
                                                   .int_term,
                                                   data.table::fifelse(level == "-", var, paste0(var, level))
                                               )]
        } else {
            all_terms_df[, inds := data.table::fifelse(level == "-", var, paste0(var, level))]
        }
    }
    
    ## Clean up temporary column if it exists
    if (".int_term" %in% names(all_terms_df)) {
        all_terms_df[, .int_term := NULL]
    }
    
    ## Process coefficients
    coef[, term := gsub(term, pattern = "`", replacement = "")]
    coef[, inds := term]
    
    ## Merge data
    to_show <- merge(all_terms_df, coef, by.x = "inds", by.y = "inds", all.x = TRUE, sort = FALSE)
    
    ## Sort by variable order first, then position within variable
    data.table::setorder(to_show, var_order, pos)
    
    ## Add variable-based shading indicator (zebra stripes)
    if (zebra_stripes) {
        to_show[, shade_group := var_order %% 2]
        shade_colors <- c("#FFFFFF", "#EEEEEE")
    } else {
        to_show[, shade_group := 0]
        shade_colors <- c("#FFFFFF", "#FFFFFF")
    }
    
    ## Select columns
    to_show <- to_show[, .(var, level, N, Events, p_value, estimate, conf_low, conf_high, pos, var_order, shade_group)]

    ## Format the values based on exponentiate setting
    to_show_exp_clean <- data.table::copy(to_show)

    ## Create formatted columns for display
    if(exponentiate) {
        to_show_exp_clean[, effect := data.table::fifelse(is.na(estimate), 
                                                          NA_real_,
                                                          exp(estimate))]
        to_show_exp_clean[, effect_formatted := data.table::fifelse(is.na(N) & is.na(estimate),
                                                                    "",
                                                                    data.table::fifelse(is.na(estimate), 
                                                                                        ref_label,
                                                                                        format_number(exp(estimate), digits, marks)))]
        to_show_exp_clean[, conf_low_formatted := data.table::fifelse(is.na(conf_low), 
                                                                      NA_character_,
                                                                      format_number(exp(conf_low), digits, marks))]
        to_show_exp_clean[, conf_high_formatted := data.table::fifelse(is.na(conf_high), 
                                                                       NA_character_,
                                                                       format_number(exp(conf_high), digits, marks))]
    } else {
        to_show_exp_clean[, effect := estimate]
        to_show_exp_clean[, effect_formatted := data.table::fifelse(is.na(N) & is.na(estimate),
                                                                    "",
                                                                    data.table::fifelse(is.na(estimate), 
                                                                                        ref_label,
                                                                                        format_number(estimate, digits, marks)))]
        to_show_exp_clean[, conf_low_formatted := data.table::fifelse(is.na(conf_low), 
                                                                      NA_character_,
                                                                      format_number(conf_low, digits, marks))]
        to_show_exp_clean[, conf_high_formatted := data.table::fifelse(is.na(conf_high), 
                                                                       NA_character_,
                                                                       format_number(conf_high, digits, marks))]
    }

    ## Format CI percentage for display in headers
    ci_pct <- round(conf_level * 100)

    ## Format \emph{p}-values using p_digits parameter
    p_threshold <- 10^(-p_digits)
    p_threshold_str <- if (!is.null(marks)) {
                           paste0("< 0", marks$decimal.mark, strrep("0", p_digits - 1), "1")
                       } else {
                           paste0("< ", format(p_threshold, scientific = FALSE))
                       }
    
    to_show_exp_clean[, p_formatted := data.table::fifelse(is.na(p_value), 
                                                           NA_character_,
                                                           data.table::fifelse(p_value < p_threshold, 
                                                                               p_threshold_str,
                                                                               format_number(p_value, p_digits, marks)))]

    ## Determine if ANY coefficient or CI bound is negative (only relevant when not exponentiating)
    ## If so, use comma notation throughout for consistency
    use_comma_notation <- !exponentiate && any(to_show_exp_clean$conf_low < 0 | to_show_exp_clean$conf_high < 0, na.rm = TRUE)
    ci_separator <- forest_ci_separator(use_comma_notation, marks)

    ## Create the combined effect string with expression for italic p
    ## n.b.: effect_abbrev will be recalculated by calculate_forest_layout for header

    to_show_exp_clean[, effect_string_expr := data.table::fifelse(
                                                              is.na(N) & is.na(estimate),
                                                              "''",
                                                              data.table::fcase(
                                                                  is.na(estimate), paste0("'", ref_label, "'"),
                                                                  
                                                                  p_value < p_threshold,
                                                                  paste0("'", effect_formatted, " (", conf_low_formatted, ci_separator, 
                                                                         conf_high_formatted, "); '*~italic(p)~'", p_threshold_str, "'"),
                                                                  
                                                                  default = paste0("'", effect_formatted, " (", conf_low_formatted, ci_separator, 
                                                                                   conf_high_formatted, "); '*~italic(p)~'= ", p_formatted, "'")
                                                              )
                                                          )]

    ## Format N and events with thousands separator
    to_show_exp_clean[, n_formatted := data.table::fifelse(is.na(N), "",
        format_count(N, marks, na = ""))]
    to_show_exp_clean[, events_formatted := data.table::fifelse(is.na(Events), "",
        format_count(Events, marks, na = ""))]

    ## Clean up variable names for display
    to_show_exp_clean[, var_display := as.character(var)]

    if (indent_groups || condense_table) {
        to_show_exp_clean[, var_display := var]
        
        for (v in unique(to_show_exp_clean$var)) {
            if (v != "" && !grepl("^    ", v)) {
                ## Skip interaction terms (already formatted) - identified by " × " in the name
                if (grepl(" \u00d7 ", v)) next
                
                clean_v <- gsub(" \\(.*\\)", "", v)
                
                label <- if (!is.null(labels) && clean_v %in% names(labels)) {
                             labels[clean_v]
                         } else if (clean_v %in% names(data) && !is.null(attr(data[[clean_v]], "label"))) {
                             attr(data[[clean_v]], "label")
                         } else {
                             NULL
                         }
                
                if (!is.null(label)) {
                    if (grepl("\\(", v)) {
                        category <- gsub(".*\\((.*)\\)", "\\1", v)
                        if (is_affirmative_category(category, label)) {
                            to_show_exp_clean[var == v, var_display := label]
                        } else {
                            to_show_exp_clean[var == v, var_display := paste0(label, " (", category, ")")]
                        }
                    } else {
                        to_show_exp_clean[var == v, var_display := label]
                    }
                }
            }
        }
        
        to_show_exp_clean[, level := ""]
        
    } else {
        
        for(v in unique(to_show_exp_clean$var)) {
            if(v %in% to_show_exp_clean$var) {
                ## Skip interaction terms (already formatted) - identified by " × " in the name
                if (grepl(" \u00d7 ", v)) next
                
                if(!is.null(labels) && v %in% names(labels)) {
                    to_show_exp_clean[var == v, var_display := labels[v]]
                }
                else if(v %in% names(data) && !is.null(attr(data[[v]], "label"))) {
                    to_show_exp_clean[var == v, var_display := attr(data[[v]], "label")]
                }
            }
        }
    }

    if (!indent_groups) {
        to_show_exp_clean[duplicated(var), var_display := ""]
    }

    ## Handle missing estimates for plotting
    if(exponentiate) {
        to_show_exp_clean[is.na(estimate), estimate := 0]
    } else {
        to_show_exp_clean[is.na(estimate), estimate := 0]
    }

    ## Reorder (flip), but maintain the variable grouping
    ## Retain the assembled table for return as an attribute. The copy is taken
    ## before the row order is reversed for plotting, so that the returned table
    ## follows the order of the model terms rather than the drawing order.
    table_data <- data.table::copy(to_show_exp_clean)
    
    to_show_exp_clean <- to_show_exp_clean[order(rev(seq_len(nrow(to_show_exp_clean))))]
    to_show_exp_clean[, x_pos := .I]

    ## Calculate plot ranges with better handling of extreme cases
    if(exponentiate) {
        rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
        
        min_ci <- min(to_show$conf_low, na.rm = TRUE)
        max_ci <- max(to_show$conf_high, na.rm = TRUE)
        
        is_one_sided <- (min_ci > 0) || (max_ci < 0)
        
        ## Intelligent tick selection to prevent overlap
        range_magnitude <- diff(rangeb)
        
        if (exp(min_ci) < 0.01 && exp(max_ci) > 2) {
            ## Very wide range
            breaks <- c(0.01, 0.1, 0.5, 1, 2, 5)
        } else if (range_magnitude > 3) {
            ## Wide range - thin out the ticks
            all_breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
            if (length(all_breaks) > 7) {
                ## Too many - keep key values only
                important <- c(1)
                if (min(all_breaks) < 0.5) important <- c(min(all_breaks), important)
                if (max(all_breaks) > 2) important <- c(important, max(all_breaks))
                
                other_breaks <- setdiff(all_breaks, important)
                if (length(other_breaks) > 3) {
                    keep_idx <- round(seq(1, length(other_breaks), length.out = 3))
                    other_breaks <- other_breaks[keep_idx]
                }
                breaks <- sort(unique(c(important, other_breaks)))
            } else {
                breaks <- all_breaks
            }
        } else {
            ## Normal range - use standard calculation
            breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
        }
        
        if (!1 %in% breaks) {
            breaks <- sort(unique(c(breaks, 1)))
        }
        
        if (min_ci > 0) {
            rangeb[1] <- log(0.9)
        } else if(max_ci < 0) {
            rangeb[2] <- log(1.1)
        }
        
        breaks <- breaks[breaks >= exp(rangeb[1]) & breaks <= exp(rangeb[2])]
        reference_value <- 1
        
    } else {
        
        rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
        breaks <- pretty(rangeb, n = 7)
        breaks <- breaks[breaks >= rangeb[1] & breaks <= rangeb[2]]
        
        is_one_sided <- (min(rangeb) > 0) || (max(rangeb) < 0)
        
        if(!0 %in% breaks) {
            breaks <- sort(unique(c(breaks, 0)))
        }
        
        if(min(rangeb) > 0) {
            rangeb[1] <- -0.1 * abs(max(rangeb))
        } else if(max(rangeb) < 0) {
            rangeb[2] <- 0.1 * abs(min(rangeb))
        }
        
        reference_value <- 0
    }

    ## Calculate layout using helper function
    layout <- calculate_forest_layout(
        to_show_exp_clean = to_show_exp_clean,
        show_n = show_n,
        show_events = show_events,
        indent_groups = indent_groups,
        condense_table = condense_table,
        effect_label = effect_label,
        ref_label = ref_label,
        font_size = font_size,
        table_width = data.table::fifelse(is.null(table_width), 0.6, table_width),
        rangeb = rangeb,
        center_padding = center_padding
    )

    ## Set up the extended range for plotting
    rangeplot <- c(layout$rangeplot_start, rangeb[2] + diff(rangeb) * 0.05)

    ## Extract positions
    y_variable <- layout$positions$var
    if (!(indent_groups || condense_table)) {
        y_level <- layout$positions$level
    }
    if (show_n) {
        y_n <- layout$positions$n
    }
    if (show_events) {
        y_events <- layout$positions$events
    }
    y_or <- layout$positions$effect

    ## Use the effect abbreviation from layout
    effect_abbrev <- layout$effect_abbrev

    ## Calculate recommended dimensions
    rec_height <- max(5, min(20, 3 + nrow(to_show_exp_clean) * 0.25))

    if(!is.null(plot_width)) {
        rec_width <- plot_width
        if(!is.null(plot_height)) {
            rec_height <- plot_height
        }
    } else {
        ## Use the calculated total width
        rec_width <- layout$total_width + 1.0  # Add margins
        rec_width <- max(10, min(20, rec_width))  # Apply reasonable bounds
    }

    ## Font sizes
    annot_font <- font_size * annot_size
    header_font <- font_size * header_size

    ## Custom ticks data
    ticks_df <- data.frame(
        x = -0.5,
        xend = -0.7,
        y = breaks,
        yend = breaks
    )

    ## Format deviance values
    null_dev_formatted <- format(round(gmodel$null_deviance, 2), nsmall = 2)
    resid_dev_formatted <- format(round(gmodel$residual_deviance, 2), nsmall = 2)
    pseudo_r2_formatted <- format(round(gmodel$pseudo_r2, 3), nsmall = 3)

    ## Create the plot
    if(exponentiate) {
        p <- ggplot2::ggplot(to_show_exp_clean, ggplot2::aes(x_pos, exp(estimate))) +
            
            ggplot2::geom_rect(ggplot2::aes(xmin = x_pos - .5, xmax = x_pos + .5,
                                            ymin = exp(rangeplot[1]), ymax = exp(rangeplot[2]),
                                            fill = ordered(shade_group))) +
            ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.10, 0.05))) +
            ggplot2::scale_size_area(max_size = 6, guide = "none") +
            ggplot2::scale_fill_manual(values = shade_colors, guide = "none") +
            
            ggplot2::geom_point(ggplot2::aes(size = N), pch = 22, color = "#000000", fill = color, na.rm = TRUE) +
            ggplot2::geom_errorbar(ggplot2::aes(ymin = exp(conf_low), ymax = exp(conf_high)), width = 0.15) +
            
            ggplot2::annotate(geom = "segment",
                              x = -0.5, xend = -0.5,
                              y = exp(rangeb[1]),
                              yend = exp(rangeb[2]),
                              color = "#000000", linewidth = 1) +
            
            ggplot2::annotate(geom = "segment", 
                              x = -0.5, xend = max(to_show_exp_clean$x_pos) + 0.5, 
                              y = reference_value, yend = reference_value, 
                              linetype = "longdash") +
            
            ggplot2::geom_segment(data = ticks_df,
                                  ggplot2::aes(x = x, xend = xend, y = y, yend = yend),
                                  inherit.aes = FALSE,
                                  color = "#000000",
                                  linewidth = 1) +
            ggplot2::geom_text(data = ticks_df,
                               ggplot2::aes(x = xend - 0.05, y = y, label = sprintf("%g", y)),
                               inherit.aes = FALSE,
                               hjust = 0.5,
                               vjust = 1.3,
                               size = annot_font * 1.5) +
            
            ggplot2::coord_flip(ylim = exp(rangeplot)) +
            ggplot2::ggtitle(title) +
            ggplot2::scale_y_log10(name = effect_label,
                                   labels = sprintf("%g", breaks),
                                   expand = c(0.02, 0.02),
                                   breaks = breaks) +
            ggplot2::theme_light(base_family = detect_plot_font()) +
            ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 0, b = 0, l = 0),
                           panel.grid.minor.y = ggplot2::element_blank(),
                           panel.grid.minor.x = ggplot2::element_blank(),
                           panel.grid.major.y = ggplot2::element_blank(),
                           panel.grid.major.x = ggplot2::element_blank(),
                           legend.position = "none",
                           panel.border = ggplot2::element_blank(),
                           axis.title.y = ggplot2::element_blank(),
                           axis.text.y = ggplot2::element_blank(),
                           axis.ticks.y = ggplot2::element_blank(),
                           axis.title.x = ggplot2::element_blank(),
                           axis.ticks.x = ggplot2::element_blank(),
                           axis.text.x = ggplot2::element_blank(),
                           plot.title = ggplot2::element_text(size = font_size * title_size, face = "bold", hjust = 0.5)) +
            ggplot2::xlab("") +
            
            ## Variable column
            ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_variable),
                              label = "Variable", fontface = "bold", hjust = 0,
                              size = header_font) +
            
            {if (indent_groups || condense_table) {
                 fontfaces <- if (bold_variables) {
                                  data.table::fifelse(grepl("^    ", to_show_exp_clean$var_display), "plain", "bold")
                              } else {
                                  rep("plain", nrow(to_show_exp_clean))
                              }
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_variable),
                                   label = to_show_exp_clean$var_display, 
                                   fontface = fontfaces, 
                                   hjust = 0,
                                   size = annot_font)
             } else {
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_variable),
                                   label = to_show_exp_clean$var_display, 
                                   fontface = if (bold_variables) "bold" else "plain", 
                                   hjust = 0,
                                   size = annot_font)
             }} +
            
            ## Group/level column
            {if (!(indent_groups || condense_table)) {
                 list(
                     ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_level),
                                       label = "Group", fontface = "bold", hjust = 0,
                                       size = header_font),
                     ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_level),
                                       label = to_show_exp_clean$level, hjust = 0,
                                       size = annot_font)
                 )
             }} +
            
            ## N column (conditional)
            {if (show_n) {
                 list(
                     ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_n),
                                       label = "n", fontface = "bold.italic", hjust = 0.5,
                                       size = header_font),
                     ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_n),
                                       label = to_show_exp_clean$n_formatted, hjust = 0.5,
                                       size = annot_font)
                 )
             }} +
            
            ## Events column (conditional)
            {if (show_events) {
                 list(
                     ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_events),
                                       label = "Events", fontface = "bold", hjust = 0.5,
                                       size = header_font),
                     ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_events),
                                       label = to_show_exp_clean$events_formatted, hjust = 0.5,
                                       size = annot_font)
                 )
             }} +
            
            ## Effect column - use dynamic CI percentage
            ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.4, y = exp(y_or),
                              label = paste0("bold('", effect_abbrev, " (", ci_pct, "% CI); '*bolditalic(p)*'-value')"),
                              hjust = 0, size = header_font, parse = TRUE) +
            
            ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_or),
                              label = to_show_exp_clean$effect_string_expr, hjust = 0,
                              size = annot_font, parse = TRUE) +
            
            ## X-axis label
            ggplot2::annotate(geom = "text", x = -1.5, y = reference_value,
                              label = effect_label, fontface = "bold",
                              hjust = 0.5, vjust = 2, size = annot_font * 1.5) +
            
            ## Model statistics footer (conditional)
            {if (qc_footer) {
                 ggplot2::annotate(geom = "text", x = 0.5, y = exp(y_variable),
                                   label = paste0(gmodel$counts_block,
                                                  "\nModel: ", gmodel$family,
                                                  "\nNull (Residual) Deviance: ", null_dev_formatted, " (", resid_dev_formatted, ")",
                                                  "\nPseudo R\u00b2: ", pseudo_r2_formatted,
                                                  "\nAIC: ", gmodel$AIC_formatted),
                                   size = annot_font * 0.65, hjust = 0, vjust = 1.2, fontface = "italic")
            }}
        
    } else {
        ## Non-exponentiated plot (linear scale)
        p <- ggplot2::ggplot(to_show_exp_clean, ggplot2::aes(x_pos, estimate)) +
            
            ggplot2::geom_rect(ggplot2::aes(xmin = x_pos - .5, xmax = x_pos + .5,
                                            ymin = rangeplot[1], ymax = rangeplot[2],
                                            fill = ordered(shade_group + 1))) +
            ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.10, 0.05))) +
            ggplot2::scale_size_area(max_size = 6, guide = "none") +
            ggplot2::scale_fill_manual(values = c("#FFFFFF", "#EEEEEE"), guide = "none") +
            
            ggplot2::geom_point(ggplot2::aes(size = N), pch = 22, color = "#000000", fill = color, na.rm = TRUE) +
            ggplot2::geom_errorbar(ggplot2::aes(ymin = conf_low, ymax = conf_high), width = 0.15) +
            
            ggplot2::annotate(geom = "segment",
                              x = -0.5, xend = -0.5,
                              y = rangeb[1],
                              yend = rangeb[2],
                              color = "#000000", linewidth = 1) +
            
            ggplot2::annotate(geom = "segment", 
                              x = -0.5, xend = max(to_show_exp_clean$x_pos) + 0.5, 
                              y = reference_value, yend = reference_value, 
                              linetype = "longdash") +
            
            ggplot2::geom_segment(data = ticks_df,
                                  ggplot2::aes(x = x, xend = xend, y = y, yend = yend),
                                  inherit.aes = FALSE,
                                  color = "#000000",
                                  linewidth = 1) +
            ggplot2::geom_text(data = ticks_df,
                               ggplot2::aes(x = xend - 0.05, y = y, label = sprintf("%g", y)),
                               inherit.aes = FALSE,
                               hjust = 0.5,
                               vjust = 1.3,
                               size = annot_font * 1.5) +
            
            ggplot2::coord_flip(ylim = rangeplot) +
            ggplot2::ggtitle(title) +
            ggplot2::scale_y_continuous(name = effect_label,
                                        labels = sprintf("%g", breaks),
                                        expand = c(0.02, 0.02),
                                        breaks = breaks) +
            ggplot2::theme_light(base_family = detect_plot_font()) +
            ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 0, b = 0, l = 0),
                           panel.grid.minor.y = ggplot2::element_blank(),
                           panel.grid.minor.x = ggplot2::element_blank(),
                           panel.grid.major.y = ggplot2::element_blank(),
                           panel.grid.major.x = ggplot2::element_blank(),
                           legend.position = "none",
                           panel.border = ggplot2::element_blank(),
                           axis.title.y = ggplot2::element_blank(),
                           axis.text.y = ggplot2::element_blank(),
                           axis.ticks.y = ggplot2::element_blank(),
                           axis.title.x = ggplot2::element_blank(),
                           axis.ticks.x = ggplot2::element_blank(),
                           axis.text.x = ggplot2::element_blank(),
                           plot.title = ggplot2::element_text(size = font_size * title_size, face = "bold", hjust = 0.5)) +
            ggplot2::xlab("") +
            
            ## Variable column
            ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_variable,
                              label = "Variable", fontface = "bold", hjust = 0,
                              size = header_font) +
            
            {if (indent_groups || condense_table) {
                 fontfaces <- if (bold_variables) {
                                  data.table::fifelse(grepl("^    ", to_show_exp_clean$var_display), "plain", "bold")
                              } else {
                                  rep("plain", nrow(to_show_exp_clean))
                              }
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_variable,
                                   label = to_show_exp_clean$var_display, 
                                   fontface = fontfaces, 
                                   hjust = 0,
                                   size = annot_font)
             } else {
                 ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_variable,
                                   label = to_show_exp_clean$var_display, 
                                   fontface = if (bold_variables) "bold" else "plain", 
                                   hjust = 0,
                                   size = annot_font)
             }} +
            
            ## Group/level column
            {if (!(indent_groups || condense_table)) {
                 list(
                     ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_level,
                                       label = "Group", fontface = "bold", hjust = 0,
                                       size = header_font),
                     ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_level,
                                       label = to_show_exp_clean$level, hjust = 0,
                                       size = annot_font)
                 )
             }} +
            
            ## N column (conditional)
            {if (show_n) {
                 list(
                     ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_n,
                                       label = "n", fontface = "bold.italic", hjust = 0.5,
                                       size = header_font),
                     ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_n,
                                       label = to_show_exp_clean$n_formatted, hjust = 0.5,
                                       size = annot_font)
                 )
             }} +
            
            ## Events column (conditional)
            {if (show_events) {
                 list(
                     ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = y_events,
                                       label = "Events", fontface = "bold", hjust = 0.5,
                                       size = header_font),
                     ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_events,
                                       label = to_show_exp_clean$events_formatted, hjust = 0.5,
                                       size = annot_font)
                 )
             }} +
            
            ## Effect column - use dynamic CI percentage
            ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.4, y = y_or,
                              label = paste0("bold('", effect_abbrev, " (", ci_pct, "% CI); '*bolditalic(p)*'-value')"),
                              hjust = 0, size = header_font, parse = TRUE) +
            
            ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = y_or,
                              label = to_show_exp_clean$effect_string_expr, hjust = 0,
                              size = annot_font, parse = TRUE) +
            
            ## X-axis label
            ggplot2::annotate(geom = "text", x = -1.5, y = reference_value,
                              label = effect_label, fontface = "bold",
                              hjust = 0.5, vjust = 2, size = annot_font * 1.5) +
            
            ## Model statistics at bottom (conditional)
            {if (qc_footer) {
                 ggplot2::annotate(geom = "text", x = 0.5, y = y_variable,
                                   label = paste0(gmodel$counts_block,
                                                  "\nModel: ", gmodel$family,
                                                  "\nNull (Residual) Deviance: ", null_dev_formatted, " (", resid_dev_formatted, ")",
                                                  "\nPseudo R\u00b2: ", pseudo_r2_formatted,
                                                  "\nAIC: ", gmodel$AIC_formatted),
                                   size = annot_font * 0.65, hjust = 0, vjust = 1.2, fontface = "italic")
            }}
    }

    ## Convert units back for output if needed
    if (units != "in") {
        rec_width <- convert_units(rec_width, from = "in", to = units)
        rec_height <- convert_units(rec_height, from = "in", to = units)
    }

    ## Provide dimension recommendations
    if(is.null(plot_width) || is.null(plot_height)) {
        message(sprintf("Recommended plot dimensions: width = %.1f %s, height = %.1f %s",
                        rec_width, units, rec_height, units))
    }

    ## Add recommended dimensions as an attribute
    ## The units are recorded alongside the dimensions. Both are converted to
    ## the requested units above, so a consumer that assumed inches would
    ## otherwise mis-size the output by the conversion factor.
    attr(p, "rec_dims") <- list(width = rec_width, height = rec_height,
                                units = units)

    ## Add the assembled table as an attribute
    attr(p, "table_data") <- table_data

    ## Return the plot
    return(p)
}

### * Main function

#' Create Publication-Ready Descriptive Statistics Tables
#'
#' Generates comprehensive descriptive statistics tables with automatic variable
#' type detection, group comparisons, and appropriate statistical testing. This
#' function is designed to create "Table 1"-style summaries commonly used in
#' clinical and epidemiological research, with full support for continuous,
#' categorical, and time-to-event variables.
#'
#' @param data Data frame or data.table containing the dataset to summarize.
#'   Automatically converted to a data.table for efficient processing.
#'
#' @param by Character string specifying the column name of the grouping
#'   variable for stratified analysis (\emph{e.g.}, treatment arm, exposure
#'   status). When \code{NULL} (default), produces overall summaries only
#'   without group comparisons or statistical tests.
#'
#' @param variables Character vector of variable names to summarize. Can
#'   include standard column names for continuous or categorical variables,
#'   and survival expressions using \code{Surv()} syntax (\emph{e.g.},
#'   \code{"Surv(os_months, os_status)"}). Variables are processed in the
#'   order provided.
#'
#' @param stats_continuous Character vector specifying which statistics to
#'   compute for continuous variables. Multiple values create separate rows
#'   for each variable. Options:
#'   \itemize{
#'     \item \code{"mean_sd"} - Mean \eqn{\pm} standard deviation
#'     \item \code{"median_iqr"} - Median [interquartile range]
#'     \item \code{"median_range"} - Median (minimum-maximum)
#'     \item \code{"range"} - Minimum-maximum only
#'   }
#'   Default is \code{"median_iqr"}.
#'
#' @param stats_categorical Character string specifying the format for
#'   categorical variable summaries:
#'   \itemize{
#'     \item \code{"n"} - Count only
#'     \item \code{"percent"} - Percentage only
#'     \item \code{"n_percent"} - Count (percentage) [default]
#'   }
#'
#' @param digits Integer specifying the number of decimal places for
#'   continuous statistics. Default is 1.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param conf_level Numeric confidence level for confidence intervals in
#'   survival variable summaries (median survival time with CI). Must be
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals).
#'
#' @param p_per_stat Logical. If \code{TRUE}, displays \emph{p}-values on each
#'   row (per statistic) rather than only on the first row of each variable.
#'   Useful when different statistics within a variable warrant separate
#'   significance testing. Default is \code{FALSE}.
#'
#' @param na_include Logical. If \code{TRUE}, missing values (NAs) are
#'   displayed as a separate category/row for each variable. If \code{FALSE},
#'   missing values are silently excluded from calculations. Default is
#'   \code{FALSE}.
#'
#' @param na_label Character string used to label the missing values row when
#'   \code{na_include = TRUE}. Default is \code{"Unknown"}.
#'
#' @param na_percent Logical. Controls how percentages are calculated for
#'   categorical variables when \code{na_include = TRUE}:
#'   \itemize{
#'     \item If \code{TRUE}, percentages include NAs in the denominator (all
#'       categories sum to 100\%)
#'     \item If \code{FALSE}, percentages exclude NAs from the denominator
#'       (non-missing categories sum to 100\%, missing shown separately)
#'   }
#'   Only affects categorical variables. Default is \code{FALSE}.
#'
#' @param test Logical. If \code{TRUE}, performs appropriate statistical tests
#'   for group comparisons and adds a \emph{p}-value column. Requires
#'   \code{by} to be specified. Tests are automatically selected based on
#'   variable type and test parameters. Default is \code{TRUE}.
#'
#' @param test_continuous Character string specifying the statistical test for
#'   continuous variables:
#'   \itemize{
#'     \item \code{"auto"} - Automatic selection: \emph{t}-test/ANOVA for means,
#'       Wilcoxon/Kruskal-Wallis for medians [default]
#'     \item \code{"t"} - Independent samples \emph{t}-test (2 groups only)
#'     \item \code{"aov"} - One-way ANOVA (2+ groups)
#'     \item \code{"wrs"} - Wilcoxon rank-sum test (2 groups only)
#'     \item \code{"kwt"} - Kruskal-Wallis test (2+ groups)
#'   }
#'
#' @param test_categorical Character string specifying the statistical test
#'   for categorical variables:
#'   \itemize{
#'     \item \code{"auto"} - Automatic selection: Fisher exact test if any
#'       expected cell frequency < 5, otherwise \eqn{\chi^2} test [default]
#'     \item \code{"fisher"} - Fisher exact test
#'     \item \code{"chisq"} - \eqn{\chi^2} test
#'   }
#'
#' @param total Logical or character string controlling the total column:
#'   \itemize{
#'     \item \code{TRUE} or \code{"first"} - Include total column as first
#'       column after Variable/Group [default]
#'     \item \code{"last"} - Include total column as last column before
#'       \emph{p}-value
#'     \item \code{FALSE} - Exclude total column
#'   }
#'
#' @param total_label Character string for the total column header.
#'   Default is \code{"Total"}.
#'
#' @param labels Named character vector or list providing custom display
#'   labels for variables. Names should match variable names (or \code{Surv()}
#'   expressions), values are the display labels. Variables not in
#'   \code{labels} use their original names. Can also label the grouping
#'   variable specified in \code{by}. Default is \code{NULL}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @param ... Additional arguments passed to the underlying statistical test
#'   functions (\emph{e.g.}, \code{var.equal = TRUE} for \emph{t}-tests,
#'   \code{simulate.p.value = TRUE} for Fisher test).
#'
#' @return A data.table with S3 class \code{"desctable"} containing formatted
#'   descriptive statistics. The table structure includes:
#'   \describe{
#'     \item{Variable}{Variable name or label (from \code{labels})}
#'     \item{Group}{For continuous variables: statistic type (\emph{e.g.},
#'       "Mean ± SD", "Median [IQR]"). For categorical variables:
#'       category level. Empty for variable name rows.}
#'     \item{Total}{Statistics for the total sample (if
#'       \code{total = TRUE})}
#'     \item{Group columns}{Statistics for each group level (when \code{by}
#'       is specified). Column names match group levels.}
#'     \item{\emph{p}-value}{Formatted \emph{p}-values from statistical tests
#'       (when \code{test = TRUE} and \code{by} is specified)}
#'   }
#'
#'   The first row always shows sample sizes for each column. All numeric
#'   output (counts, statistics, \emph{p}-values) respects the
#'   \code{number_format} setting for locale-appropriate formatting.
#'
#'   The returned object includes the following attributes accessible via
#'   \code{attr()}:
#'   \describe{
#'     \item{raw_data}{A data.table containing unformatted numeric values
#'       suitable for further statistical analysis or custom formatting.
#'       Includes additional columns for standard deviations, quartiles,
#'       etc.}
#'     \item{by_variable}{The grouping variable name used (value of
#'       \code{by})}
#'     \item{variables}{The variables analyzed (value of
#'       \code{variables})}
#'   }
#'
#' @details
#' \strong{Variable Type Detection:}
#'
#' The function automatically detects variable types and applies appropriate
#' summaries:
#' \itemize{
#'   \item \strong{Continuous}: Numeric variables (integer or double) receive
#'     statistics specified in \code{stats_continuous}
#'   \item \strong{Categorical}: Character, factor, or logical variables
#'     receive frequency counts and percentages
#'   \item \strong{Time-to-Event}: Variables specified as
#'     \code{Surv(time, event)} display median survival with confidence
#'     intervals (level controlled by \code{conf_level})
#' }
#'
#' \strong{Statistical Testing:}
#'
#' When \code{test = TRUE} and \code{by} is specified:
#' \itemize{
#'   \item \strong{Continuous with "auto"}: Parametric tests (\emph{t}-test, ANOVA)
#'     for mean-based statistics; non-parametric tests (Wilcoxon,
#'     Kruskal-Wallis) for median-based statistics
#'   \item \strong{Categorical with "auto"}: Fisher exact test when any
#'     expected cell frequency < 5; \eqn{\chi^2} test otherwise
#'   \item \strong{Survival}: Log-rank test for comparing survival curves
#'   \item \strong{Range statistics}: No \emph{p}-value computed (ranges
#'     are descriptive)
#' }
#'
#' \strong{Missing Data Handling:}
#'
#' Missing values are handled differently by variable type:
#' \itemize{
#'   \item \strong{Continuous}: NAs excluded from calculations; optionally
#'     shown as count when \code{na_include = TRUE}
#'   \item \strong{Categorical}: NAs can be included as a category when
#'     \code{na_include = TRUE}. The \code{na_percent} parameter controls
#'     whether percentages are calculated with or without NAs in the
#'     denominator
#'   \item \strong{Survival}: NAs in time or event excluded from analysis
#' }
#'
#' \strong{Formatting Conventions:}
#'
#' All numeric output respects the \code{number_format} parameter. Separators
#' within ranges and confidence intervals adapt automatically to avoid
#' ambiguity:
#' \itemize{
#'   \item Mean ± SD: \code{"45.2 ± 12.3"} (US) or
#'     \code{"45,2 ± 12,3"} (EU)
#'   \item Median [IQR]: \code{"38.0 [28.0-52.0]"} (US) or
#'     \code{"38,0 [28,0-52,0]"} (EU, en-dash separator)
#'   \item Range: \code{"18.0-75.0"} (positive, US),
#'     \code{"-5.0 to 10.0"} (when bounds are negative)
#'   \item Survival: \code{"24.5 (21.2-28.9)"} (US) or
#'     \code{"24,5 (21,2-28,9)"} (EU)
#'   \item Counts \eqn{\ge} 1000: \code{"1,234"} (US) or \code{"1.234"} (EU)
#'   \item \emph{p}-values: \code{"< 0.001"} (US) or \code{"< 0,001"} (EU)
#' }
#'
#' @seealso
#' \code{\link{survtable}} for detailed survival summary tables,
#' \code{\link{fit}} for regression modeling,
#' \code{\link{tablesave}} for export in any supported format,
#' \code{\link{table2pdf}} for PDF export,
#' \code{\link{table2docx}} for Word export,
#' \code{\link{table2html}} for HTML export
#'
#' @examples
#' # Load example clinical trial data
#' data(clintrial)
#'
#' # Example 1: Basic descriptive table without grouping
#' desctable(clintrial,
#'         variables = c("age", "sex", "bmi"),
#'         labels = clintrial_labels)
#'
#' \donttest{
#' 
#' # Example 2: Grouped comparison with default tests
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "sex", "race", "bmi"),
#'         labels = clintrial_labels)
#'
#' # Example 3: Customize continuous statistics
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "bmi", "creatinine"),
#'         stats_continuous = c("median_iqr", "range"),
#'         labels = clintrial_labels)
#'
#' # Example 4: Change categorical display format
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("sex", "race", "smoking"),
#'         stats_categorical = "n",  # Show counts only
#'         labels = clintrial_labels)
#'
#' # Example 5: Include missing values
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "smoking", "hypertension"),
#'         na_include = TRUE,
#'         na_label = "Missing",
#'         labels = clintrial_labels)
#'
#' # Example 6: Disable statistical testing
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "sex", "bmi"),
#'         test = FALSE,
#'         labels = clintrial_labels)
#'
#' # Example 7: Force specific tests
#' desctable(clintrial,
#'         by = "surgery",
#'         variables = c("age", "sex"),
#'         test_continuous = "t",        # t-test instead of auto
#'         test_categorical = "fisher",  # Fisher test instead of auto
#'         labels = clintrial_labels)
#'
#' # Example 8: Adjust decimal places
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "bmi"),
#'         digits = 2,    # 2 decimals for continuous
#'         p_digits = 4,  # 4 decimals for p-values
#'         labels = clintrial_labels)
#'
#' # Example 9: Custom variable labels
#' labels <- c(
#'     age = "Age (years)",
#'     sex = "Sex",
#'     bmi = "Body Mass Index (kg/m\u00b2)",
#'     treatment = "Treatment Arm"
#' )
#'
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "sex", "bmi"),
#'         labels = labels)
#'
#' # Example 10: Position total column last
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "sex"),
#'         total = "last",
#'         labels = clintrial_labels)
#'
#' # Example 11: Exclude total column
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "sex"),
#'         total = FALSE,
#'         labels = clintrial_labels)
#'
#' # Example 12: Survival analysis
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = "Surv(os_months, os_status)",
#'         labels = clintrial_labels)
#'
#' # Example 13: Multiple survival endpoints
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c(
#'             "Surv(pfs_months, pfs_status)",
#'             "Surv(os_months, os_status)"
#'         ),
#'         labels = c(
#'             "Surv(pfs_months, pfs_status)" = "Progression-Free Survival",
#'             "Surv(os_months, os_status)" = "Overall Survival"
#'         ))
#'
#' # Example 14: Mixed variable types
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c(
#'             "age", "sex", "race",           # Demographics
#'             "bmi", "creatinine",            # Labs
#'             "smoking", "hypertension",      # Risk factors
#'             "Surv(os_months, os_status)"    # Survival
#'         ),
#'         labels = clintrial_labels)
#'
#' # Example 15: Three or more groups
#' desctable(clintrial,
#'         by = "stage",  # Assuming stage has 3+ levels
#'         variables = c("age", "sex", "bmi"),
#'         labels = clintrial_labels)
#' # Automatically uses ANOVA/Kruskal-Wallis and chi-squared
#'
#' # Example 16: Access raw unformatted data
#' result <- desctable(clintrial,
#'                   by = "treatment",
#'                   variables = c("age", "bmi"))
#' raw_data <- attr(result, "raw_data")
#' print(raw_data)
#' # Raw data includes unformatted numbers, SDs, quartiles, etc.
#'
#' # Example 17: Check which grouping variable was used
#' result <- desctable(clintrial,
#'                   by = "treatment",
#'                   variables = c("age", "sex"))
#' attr(result, "by_variable")  # "treatment"
#'
#' # Example 18: NA percentage calculation options
#' # Include NAs in percentage denominator (all sum to 100%)
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = "smoking",
#'         na_include = TRUE,
#'         na_percent = TRUE,
#'         labels = clintrial_labels)
#'
#' # Exclude NAs from denominator (non-missing sum to 100%)
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = "smoking",
#'         na_include = TRUE,
#'         na_percent = FALSE,
#'         labels = clintrial_labels)
#'
#' # Example 19: Passing additional test arguments
#' # Equal variance t-test
#' desctable(clintrial,
#'         by = "sex",
#'         variables = "age",
#'         test_continuous = "t",
#'         var.equal = TRUE,
#'         labels = clintrial_labels)
#'
#' # Example 20: European number formatting
#' desctable(clintrial,
#'         by = "treatment",
#'         variables = c("age", "sex", "bmi"),
#'         number_format = "eu",
#'         labels = clintrial_labels)
#'
#' # Example 21: Complete Table 1 for publication
#' table1 <- desctable(
#'     data = clintrial,
#'     by = "treatment",
#'     variables = c(
#'         "age", "sex", "race", "ethnicity", "bmi",
#'         "smoking", "hypertension", "diabetes",
#'         "ecog", "creatinine", "hemoglobin",
#'         "site", "stage", "grade",
#'         "Surv(os_months, os_status)"
#'     ),
#'     labels = clintrial_labels,
#'     stats_continuous = c("median_iqr", "range"),
#'     total = TRUE,
#'     na_include = FALSE
#' )
#' print(table1)
#'
#' }
#' 
#' @family descriptive functions
#' @export
desctable <- function(data,
                      by = NULL,
                    variables,
                    stats_continuous = c("median_iqr"),
                    stats_categorical = "n_percent",
                    digits = 1,
                    p_digits = 3,
                    conf_level = 0.95,
                    p_per_stat = FALSE,
                    na_include = FALSE,
                    na_label = "Unknown",
                    na_percent = FALSE,
                    test = TRUE,
                    test_continuous = "auto",
                    test_categorical = "auto",
                    total = TRUE,
                    total_label = "Total",
                    labels = NULL,
                    number_format = NULL,
                    ...) {
    
    if (!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    ## Set group_var from 'by' parameter
    group_var <- by
    group_var_label <- NULL
    
    ## Apply label to group variable if provided
    if (!is.null(group_var) && !is.null(labels) && group_var %chin% names(labels)) {
        group_var_label <- labels[group_var]
    } else if (!is.null(group_var)) {
        group_var_label <- group_var
    }
    
    ## Variables are already provided as a vector
    vars <- variables
    
    ## Pre-allocate result lists
    n_vars <- length(vars)
    result_list <- vector("list", n_vars)
    raw_result_list <- vector("list", n_vars)
    
    ## Process each variable
    for (i in seq_along(vars)) {
        var_data <- process_variable(
            data = data,
            var = vars[i],
            group_var = group_var,
            stats_continuous = stats_continuous,
            stats_categorical = stats_categorical,
            digits = digits,
            p_digits = p_digits,
            conf_level = conf_level,
            p_per_stat = p_per_stat,
            na_include = na_include,
            na_label = na_label,
            na_percent = na_percent,
            test = test,
            test_continuous = test_continuous,
            test_categorical = test_categorical,
            total = total,
            total_label = total_label,
            labels = labels,
            marks = marks,
            ...
        )
        
        result_list[[i]] <- var_data$formatted
        raw_result_list[[i]] <- var_data$raw
    }
    
    ## Combine all results using rbindlist
    result <- data.table::rbindlist(result_list, fill = TRUE)
    raw_result <- data.table::rbindlist(raw_result_list, fill = TRUE)

    ## Standardize column names
    old_names <- c("variable", "level")
    new_names <- c("Variable", "Group")
    
    if ("variable" %chin% names(result)) {
        data.table::setnames(result, old_names, new_names, skip_absent = TRUE)
        data.table::setnames(raw_result, old_names, new_names, skip_absent = TRUE)
    }

    ## Add p-value column if tests requested (after standardization)
    if (test && !is.null(group_var)) {
        result <- format_pvalues_desctable(result, p_digits, marks)
    }

    ## Add N row as first row (for both grouped and ungrouped tables)
    if (!is.null(group_var)) {
        ## Get the group values in the correct order
        if (is.factor(data[[group_var]])) {
            ## Use factor levels for proper ordering
            groups <- levels(data[[group_var]])
        } else {
            ## Fall back to unique values for non-factors
            groups <- unique(data[[group_var]])
            groups <- groups[!is.na(groups)]
        }
        
        ## Create N row
        n_row <- data.table::data.table(
                                 Variable = "N",
                                 Group = ""
                             )
        
        ## Calculate and add total if present
        if (total_label %chin% names(result)) {
            n_total <- nrow(data)
            n_row[[total_label]] <- format_count(n_total, marks)
        }
        
        ## Calculate for each group in the correct order
        for (grp in groups) {
            grp_col <- as.character(grp)
            if (grp_col %chin% names(result)) {
                n_group <- sum(data[[group_var]] == grp, na.rm = TRUE)
                n_row[[grp_col]] <- format_count(n_group, marks)
            }
        }
        
        ## Add empty p-value column if it exists
        if ("p-value" %chin% names(result)) {
            n_row[["p-value"]] <- ""
        }
        
        ## Prepend N row
        result <- rbind(n_row, result, fill = TRUE)
        
    } else if (total && total_label %chin% names(result)) {
        ## Add N row for ungrouped tables with Total column
        n_total <- nrow(data)
        n_row <- data.table::data.table(
                                 Variable = "N",
                                 Group = ""
                             )
        n_row[[total_label]] <- format_count(n_total, marks)
        
        ## Prepend N row
        result <- rbind(n_row, result, fill = TRUE)
    }

    ## Reorder columns if total position specified
    if (!isFALSE(total) && !is.null(group_var)) {
        result <- reorder_total_column(result, total, total_label)
    }

    ## Attach raw data and metadata as attributes
    data.table::setattr(result, "raw_data", raw_result)
    data.table::setattr(result, "by_variable", group_var)
    data.table::setattr(result, "variables", variables)

    result[]
    return(result)
}

#' Test Suite for desctable()
#' 
#' Comprehensive tests covering descriptive statistics tables,
#' continuous/categorical/survival variables, statistical tests,
#' formatting options, and edge cases.
#' 
#' @details Run with testthat::test_file("tests/testthat/test-desctable.R")

library(data.table)

## ============================================================================
## Setup: Create test data and helper functions
## ============================================================================


## Create a complete-case subset for tests requiring no missing data
clintrial_complete <- na.omit(clintrial)

## Helper function to check desctable structure
expect_desctable_result <- function(result) {
    expect_s3_class(result, "data.table")
    
    ## Required columns
    expect_true("Variable" %in% names(result))
    expect_true("Group" %in% names(result))
    
    ## Required attributes
    expect_true(!is.null(attr(result, "raw_data")))
    expect_true(!is.null(attr(result, "variables")))
    
    ## First row should be N
    expect_equal(result$Variable[1], "N")
}

## Helper to check for Total column
expect_total_column <- function(result, label = "Total") {
    expect_true(label %in% names(result))
}



## ============================================================================
## SECTION 1: Basic Functionality - No Grouping
## ============================================================================

test_that("desctable works without grouping variable", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age", "sex", "stage")
    )
    
    expect_desctable_result(result)
    
    ## Should have Total column by default
    expect_total_column(result)
    
    ## Should NOT have p-value column (no groups to compare)
    expect_false("p-value" %in% names(result))
})


test_that("desctable ungrouped shows correct N", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age")
    )
    
    ## First row should show total N
    n_displayed <- gsub(",", "", result$Total[1])  # Remove commas
    expect_equal(as.integer(n_displayed), nrow(clintrial))
})


## ============================================================================
## SECTION 2: Basic Functionality - With Grouping
## ============================================================================

test_that("desctable works with grouping variable", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex", "stage")
    )
    
    expect_desctable_result(result)
    
    ## Should have columns for each treatment group
    treatment_levels <- unique(clintrial$treatment[!is.na(clintrial$treatment)])
    for (lvl in treatment_levels) {
        expect_true(as.character(lvl) %in% names(result))
    }
    
    ## Should have p-value column by default
    expect_pvalue_column(result)
})


test_that("desctable by_variable attribute is set", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex")
    )
    
    expect_equal(attr(result, "by_variable"), "treatment")
})


test_that("desctable group N values are correct", {
    
    result <- desctable(
        data = clintrial,
        by = "sex",
        variables = c("age")
    )
    
    ## Check N row has correct counts per group
    n_male <- sum(clintrial$sex == "Male", na.rm = TRUE)
    n_female <- sum(clintrial$sex == "Female", na.rm = TRUE)
    
    male_n_displayed <- gsub(",", "", result$Male[1])
    female_n_displayed <- gsub(",", "", result$Female[1])
    
    expect_equal(as.integer(male_n_displayed), n_male)
    expect_equal(as.integer(female_n_displayed), n_female)
})


## ============================================================================
## SECTION 3: Continuous Variables
## ============================================================================

test_that("desctable handles continuous variables correctly", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age", "bmi"),
        stats_continuous = c("mean_sd", "median_iqr")
    )
    
    expect_desctable_result(result)
    
    ## Should have rows for each stat type
    expect_true(any(result$Group == "Mean ± SD"))
    expect_true(any(result$Group == "Median [IQR]"))
})


test_that("desctable stats_continuous='mean_sd' works", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "mean_sd"
    )
    
    ## Should only have mean_sd row (plus N row)
    expect_true(any(result$Group == "Mean ± SD"))
    expect_false(any(result$Group == "Median [IQR]"))
})


test_that("desctable stats_continuous='median_iqr' works", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "median_iqr"
    )
    
    expect_true(any(result$Group == "Median [IQR]"))
})


test_that("desctable stats_continuous='median_range' works", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "median_range"
    )
    
    expect_true(any(result$Group == "Median (Range)"))
})


test_that("desctable stats_continuous='range' works", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "range"
    )
    
    expect_true(any(result$Group == "Range"))
})


test_that("desctable multiple stats_continuous creates multiple rows", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = c("mean_sd", "median_iqr", "range")
    )
    
    ## Age should have 3 rows (one for each stat)
    age_rows <- result[Variable == "age" | (Variable == "" & shift(Variable, fill = "age") == "age")]
    ## Count rows with stat labels
    stat_rows <- sum(result$Group %in% c("Mean ± SD", "Median [IQR]", "Range"))
    expect_equal(stat_rows, 3)
})


## ============================================================================
## SECTION 4: Categorical Variables
## ============================================================================

test_that("desctable handles categorical variables correctly", {
    
    result <- desctable(
        data = clintrial,
        variables = c("sex", "stage")
    )
    
    expect_desctable_result(result)
    
    ## Should have rows for each level
    sex_levels <- levels(clintrial$sex)
    for (lvl in sex_levels) {
        expect_true(any(result$Group == lvl))
    }
})


test_that("desctable stats_categorical='n_percent' shows count and percent", {
    
    result <- desctable(
        data = clintrial,
        variables = c("sex"),
        stats_categorical = "n_percent"
    )
    
    ## Values should contain both count and percentage
    sex_rows <- result[Group %in% c("Male", "Female")]
    
    ## Should have format like "123 (45.6%)"
    expect_true(all(grepl("\\d+.*\\(.*%\\)", sex_rows$Total)))
})


test_that("desctable stats_categorical='n' shows count only", {
    
    result <- desctable(
        data = clintrial,
        variables = c("sex"),
        stats_categorical = "n"
    )
    
    sex_rows <- result[Group %in% c("Male", "Female")]
    
    ## Should NOT contain percentage
    expect_false(any(grepl("%", sex_rows$Total)))
})


test_that("desctable stats_categorical='percent' shows percent only", {
    
    result <- desctable(
        data = clintrial,
        variables = c("sex"),
        stats_categorical = "percent"
    )
    
    sex_rows <- result[Group %in% c("Male", "Female")]
    
    ## Should contain percentage
    expect_true(all(grepl("%", sex_rows$Total)))
})


## ============================================================================
## SECTION 5: Survival Variables
## ============================================================================

test_that("desctable handles survival variables", {
    
    result <- desctable(
        data = clintrial,
        variables = c("Surv(os_months, os_status)")
    )
    
    expect_desctable_result(result)
    
    ## Should show median survival
    expect_true(any(grepl("Median|Survival", result$Group, ignore.case = TRUE)))
})


test_that("desctable survival with grouping shows log-rank test", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("Surv(os_months, os_status)"),
        test = TRUE
    )
    
    expect_pvalue_column(result)
    
    ## Should have a p-value for survival comparison
    surv_rows <- result[grepl("Surv|Median", Variable, ignore.case = TRUE) | 
                        grepl("Surv|Median", Group, ignore.case = TRUE)]
    
    ## At least one row should have a non-empty p-value
    if (nrow(surv_rows) > 0) {
        pvals <- surv_rows$`p-value`
        has_pval <- any(!is.na(pvals) & pvals != "")
        expect_true(has_pval)
    }
})


## ============================================================================
## SECTION 6: Statistical Tests
## ============================================================================

test_that("desctable test=TRUE adds p-value column", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex"),
        test = TRUE
    )
    
    expect_pvalue_column(result)
})


test_that("desctable test=FALSE excludes p-value column", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex"),
        test = FALSE
    )
    
    expect_false("p-value" %in% names(result))
})


test_that("desctable test_continuous='t' uses t-test", {
    
    result <- desctable(
        data = clintrial,
        by = "sex",  # Two groups
        variables = c("age"),
        stats_continuous = "mean_sd",
        test = TRUE,
        test_continuous = "t"
    )
    
    expect_pvalue_column(result)
    
    ## P-value should exist and be valid
    raw <- attr(result, "raw_data")
    expect_true(!is.null(raw$p_value))
})


test_that("desctable test_continuous='wrs' uses Wilcoxon test", {
    
    result <- suppressWarnings(
        desctable(
            data = clintrial,
            by = "sex",
            variables = c("age"),
            stats_continuous = "median_iqr",
            test = TRUE,
            test_continuous = "wrs"
        )
    )
    
    expect_pvalue_column(result)
})


test_that("desctable test_continuous='auto' selects appropriate test", {
    
    ## For mean_sd, should use parametric test
    result_mean <- desctable(
        data = clintrial,
        by = "sex",
        variables = c("age"),
        stats_continuous = "mean_sd",
        test = TRUE,
        test_continuous = "auto"
    )
    
    ## For median_iqr, should use non-parametric test
    result_median <- suppressWarnings(
        desctable(
            data = clintrial,
            by = "sex",
            variables = c("age"),
            stats_continuous = "median_iqr",
            test = TRUE,
            test_continuous = "auto"
        )
    )
    
    ## Both should have p-values
    expect_pvalue_column(result_mean)
    expect_pvalue_column(result_median)
})


test_that("desctable test_categorical='auto' selects appropriate test", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("sex"),
        test = TRUE,
        test_categorical = "auto"
    )
    
    expect_pvalue_column(result)
})


test_that("desctable test_categorical='chisq' uses chi-squared test", {
    
    result <- suppressWarnings(
        desctable(
            data = clintrial,
            by = "treatment",
            variables = c("sex"),
            test = TRUE,
            test_categorical = "chisq"
        )
    )
    
    expect_pvalue_column(result)
})


test_that("desctable test_categorical='fisher' uses Fisher's exact test", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("sex"),
        test = TRUE,
        test_categorical = "fisher"
    )
    
    expect_pvalue_column(result)
})


test_that("desctable range stat does not show p-value", {
    
    result <- desctable(
        data = clintrial,
        by = "sex",
        variables = c("age"),
        stats_continuous = "range",
        test = TRUE
    )
    
    ## Result should be valid
    expect_desctable_result(result)
    
    ## Range rows should have empty p-value
    range_rows <- result[Group == "Range"]
    expect_true(nrow(range_rows) > 0)  # Should have range rows
    
    if ("p-value" %in% names(result)) {
        expect_true(all(range_rows$`p-value` == "" | is.na(range_rows$`p-value`)))
    }
})


## ============================================================================
## SECTION 7: Total Column Options
## ============================================================================

test_that("desctable total=TRUE includes Total column first", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age"),
        total = TRUE
    )
    
    expect_total_column(result)
    
    ## Total should be before group columns
    col_idx <- which(names(result) == "Total")
    group_cols <- setdiff(names(result), c("Variable", "Group", "Total", "p-value"))
    
    if (length(group_cols) > 0) {
        first_group_idx <- min(which(names(result) %in% group_cols))
        expect_true(col_idx < first_group_idx)
    }
})


test_that("desctable total='last' puts Total column last", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age"),
        total = "last"
    )
    
    expect_total_column(result)
    
    ## Total should be after group columns but before p-value
    cols <- names(result)
    total_idx <- which(cols == "Total")
    
    if ("p-value" %in% cols) {
        pval_idx <- which(cols == "p-value")
        expect_true(total_idx < pval_idx)
    }
})


test_that("desctable total=FALSE excludes Total column", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age"),
        total = FALSE
    )
    
    expect_false("Total" %in% names(result))
})


test_that("desctable total_label customizes column name", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age"),
        total = TRUE,
        total_label = "Overall"
    )
    
    expect_true("Overall" %in% names(result))
    expect_false("Total" %in% names(result))
})


## ============================================================================
## SECTION 8: Missing Data Handling
## ============================================================================

test_that("desctable na_include=FALSE excludes missing data", {
    
    ## Use variable with known missing values
    result <- desctable(
        data = clintrial,
        variables = c("smoking"),  # Has NAs
        na_include = FALSE
    )
    
    ## Should NOT have "Unknown" or NA category
    expect_false(any(result$Group == "Unknown", na.rm = TRUE))
})


test_that("desctable na_include=TRUE shows missing category", {
    
    ## Ensure variable has missing values
    if (any(is.na(clintrial$smoking))) {
        result <- desctable(
            data = clintrial,
            variables = c("smoking"),
            na_include = TRUE,
            na_label = "Unknown"
        )
        
        ## Should have "Unknown" category
        expect_true(any(result$Group == "Unknown", na.rm = TRUE))
    }
})


test_that("desctable na_label customizes missing label", {
    
    if (any(is.na(clintrial$smoking))) {
        result <- desctable(
            data = clintrial,
            variables = c("smoking"),
            na_include = TRUE,
            na_label = "Missing Data"
        )
        
        expect_true(any(result$Group == "Missing Data", na.rm = TRUE))
    }
})


test_that("desctable na_include shows missing count for continuous", {
    
    ## Create data with missing continuous values
    test_data <- data.table::as.data.table(clintrial)
    test_data[1:10, age := NA]
    
    result <- desctable(
        data = test_data,
        variables = c("age"),
        na_include = TRUE,
        na_label = "Missing"
    )
    
    ## Should have a row showing missing count
    expect_true(any(result$Group == "Missing", na.rm = TRUE))
})


## ============================================================================
## SECTION 9: Formatting Options
## ============================================================================

test_that("desctable respects digits parameter", {
    
    result_1dig <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "mean_sd",
        digits = 1
    )
    
    result_2dig <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "mean_sd",
        digits = 2
    )
    
    ## Extract mean values and check decimal places
    mean_row_1 <- result_1dig[Group == "Mean ± SD"]$Total
    mean_row_2 <- result_2dig[Group == "Mean ± SD"]$Total
    
    ## Count decimals (rough check)
    if (length(mean_row_1) > 0 && length(mean_row_2) > 0) {
        ## 1 digit should have format like "45.2"
        ## 2 digits should have format like "45.23"
        expect_true(nchar(mean_row_1) <= nchar(mean_row_2) + 2)
    }
})


test_that("desctable formats large numbers with commas", {
    
    ## Create data with large values
    test_data <- data.table::as.data.table(clintrial)
    test_data[, large_var := age * 1000]
    
    result <- desctable(
        data = test_data,
        variables = c("large_var"),
        stats_continuous = "mean_sd"
    )
    
    ## Values >= 1000 should have commas
    mean_row <- result[Group == "Mean ± SD"]$Total
    if (length(mean_row) > 0) {
        expect_true(grepl(",", mean_row))
    }
})


test_that("desctable respects digits_p parameter", {
    
    result <- desctable(
        data = clintrial,
        by = "sex",
        variables = c("age"),
        test = TRUE,
        digits_p = 4
    )
    
    ## P-values should have up to 4 decimal places
    if ("p-value" %in% names(result)) {
        pvals <- result$`p-value`
        numeric_pvals <- pvals[grepl("^0\\.", pvals)]
        
        if (length(numeric_pvals) > 0) {
            ## Check decimals
            decimals <- sapply(numeric_pvals, function(p) {
                nchar(sub("0\\.", "", p))
            })
            expect_true(all(decimals <= 4))
        }
    }
})


## ============================================================================
## SECTION 10: Custom Labels
## ============================================================================

test_that("desctable applies custom labels to variables", {
    
    custom_labels <- c(
        age = "Age (years)",
        sex = "Sex",
        stage = "Cancer Stage"
    )
    
    result <- desctable(
        data = clintrial,
        variables = c("age", "sex", "stage"),
        labels = custom_labels
    )
    
    ## Check labels are applied
    expect_true(any(result$Variable == "Age (years)"))
    expect_true(any(result$Variable == "Sex"))
    expect_true(any(result$Variable == "Cancer Stage"))
})


test_that("desctable works with clintrial_labels", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age", "sex", "stage", "treatment"),
        labels = clintrial_labels
    )
    
    expect_desctable_result(result)
})


test_that("desctable labels work for grouping variable", {
    
    custom_labels <- c(
        treatment = "Treatment Arm"
    )
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age"),
        labels = custom_labels
    )
    
    ## The grouping variable label might appear in column names or elsewhere
    ## Just verify it doesn't error
    expect_desctable_result(result)
})


## ============================================================================
## SECTION 11: Raw Data Attribute
## ============================================================================

test_that("desctable raw_data attribute has correct structure", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex")
    )
    
    raw <- attr(result, "raw_data")
    
    expect_s3_class(raw, "data.table")
    expect_true("Variable" %in% names(raw) || "variable" %in% names(raw))
})


test_that("desctable raw_data contains numeric values", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age"),
        stats_continuous = "mean_sd"
    )
    
    raw <- attr(result, "raw_data")
    
    ## Should have numeric values for statistics
    ## Column names may vary, but Total or similar should be numeric
    total_col <- grep("Total|total", names(raw), value = TRUE)[1]
    if (!is.na(total_col)) {
        expect_true(is.numeric(raw[[total_col]]))
    }
})


## ============================================================================
## SECTION 12: Edge Cases
## ============================================================================

test_that("desctable handles single variable", {
    
    result <- desctable(
        data = clintrial,
        variables = "age"
    )
    
    expect_desctable_result(result)
})


test_that("desctable handles many variables", {
    
    many_vars <- c("age", "sex", "smoking", "diabetes", "hypertension",
                   "stage", "grade", "ecog", "treatment", "surgery")
    
    result <- desctable(
        data = clintrial,
        variables = many_vars
    )
    
    expect_desctable_result(result)
    expect_equal(attr(result, "variables"), many_vars)
})


test_that("desctable handles all continuous variables", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age", "bmi", "creatinine", "hemoglobin"),
        stats_continuous = "mean_sd"
    )
    
    expect_desctable_result(result)
})


test_that("desctable handles all categorical variables", {
    
    result <- desctable(
        data = clintrial,
        variables = c("sex", "stage", "grade", "treatment")
    )
    
    expect_desctable_result(result)
})


test_that("desctable handles mixed variable types", {
    
    result <- desctable(
        data = clintrial,
        variables = c("age", "sex", "bmi", "stage")
    )
    
    expect_desctable_result(result)
})


test_that("desctable works with data.frame input", {
    
    df <- as.data.frame(clintrial)
    
    result <- desctable(
        data = df,
        variables = c("age", "sex")
    )
    
    expect_desctable_result(result)
})


test_that("desctable handles groups with different factor levels", {
    
    result <- desctable(
        data = clintrial,
        by = "stage",  # Multiple levels
        variables = c("age", "sex")
    )
    
    expect_desctable_result(result)
    
    ## Should have columns for each stage level
    stage_levels <- levels(clintrial$stage)
    for (lvl in stage_levels) {
        expect_true(as.character(lvl) %in% names(result))
    }
})


## ============================================================================
## SECTION 13: Variable Order Preservation
## ============================================================================

test_that("desctable preserves variable order", {
    
    vars <- c("stage", "age", "sex")  # Specific order
    
    result <- desctable(
        data = clintrial,
        variables = vars
    )
    
    ## Variables should appear in order provided
    ## Get non-empty Variable values (these are the variable headers)
    var_col <- result$Variable[result$Variable != "" & result$Variable != "N"]
    
    ## Get unique variable names in order of first appearance
    seen <- c()
    for (v in var_col) {
        if (!(v %in% seen)) {
            seen <- c(seen, v)
        }
    }
    
    ## The order of seen variables should match the order of input vars
    ## (accounting for possible label transformations)
    ## Simply check that we have the right number and they appeared
    expect_equal(length(seen), length(vars))
})


## ============================================================================
## SECTION 14: Factor Level Ordering
## ============================================================================

test_that("desctable respects factor level ordering in categorical variables", {
    
    result <- desctable(
        data = clintrial,
        variables = c("stage")  # Ordered factor I, II, III, IV
    )
    
    ## Get stage rows
    stage_rows <- result[Variable == "stage" | (Variable == "" & Group %in% c("I", "II", "III", "IV"))]
    groups <- stage_rows$Group[stage_rows$Group != ""]
    
    ## Should be in factor level order
    expected_order <- levels(clintrial$stage)
    actual_order <- groups[groups %in% expected_order]
    
    if (length(actual_order) > 0) {
        expect_equal(actual_order, expected_order[expected_order %in% actual_order])
    }
})


test_that("desctable respects factor level ordering in grouping variable", {
    
    result <- desctable(
        data = clintrial,
        by = "stage",
        variables = c("age")
    )
    
    ## Column order should match factor levels
    cols <- names(result)
    stage_cols <- cols[cols %in% levels(clintrial$stage)]
    
    expected_order <- levels(clintrial$stage)[levels(clintrial$stage) %in% stage_cols]
    expect_equal(stage_cols, expected_order)
})


## ============================================================================
## SECTION 15: Consistency Tests
## ============================================================================

test_that("desctable produces consistent results across runs", {
    
    result1 <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex")
    )
    
    result2 <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex")
    )
    
    expect_equal(result1, result2)
})


test_that("desctable N values are consistent with data", {
    
    result <- desctable(
        data = clintrial,
        by = "sex",
        variables = c("age")
    )
    
    ## Total N should equal nrow
    total_n <- gsub(",", "", result$Total[1])
    expect_equal(as.integer(total_n), nrow(clintrial))
    
    ## Group Ns should sum to total
    male_n <- as.integer(gsub(",", "", result$Male[1]))
    female_n <- as.integer(gsub(",", "", result$Female[1]))
    
    expected_sum <- sum(clintrial$sex == "Male", na.rm = TRUE) + 
        sum(clintrial$sex == "Female", na.rm = TRUE)
    expect_equal(male_n + female_n, expected_sum)
})


## ============================================================================
## SECTION 16: Integration with Export Functions
## ============================================================================

test_that("desctable output is suitable for export", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c("age", "sex", "stage"),
        labels = clintrial_labels
    )
    
    ## Should be a data.table
    expect_s3_class(result, "data.table")
    
    ## All columns should be character (for display)
    for (col in names(result)) {
        expect_true(is.character(result[[col]]) || is.numeric(result[[col]]))
    }
})


## ============================================================================
## SECTION 17: Complete Table 1 Workflow
## ============================================================================

test_that("desctable creates complete Table 1", {
    
    result <- desctable(
        data = clintrial,
        by = "treatment",
        variables = c(
            "age", "sex", "bmi",
            "smoking", "diabetes",
            "stage", "grade",
            "Surv(os_months, os_status)"
        ),
        stats_continuous = c("median_iqr"),
        labels = clintrial_labels,
        total = TRUE,
        test = TRUE
    )
    
    expect_desctable_result(result)
    expect_total_column(result)
    expect_pvalue_column(result)
    
    ## Should have all the variables
    vars_attr <- attr(result, "variables")
    expect_equal(length(vars_attr), 8)
})


## ============================================================================
## SECTION 18: Two-group vs Multi-group Tests
## ============================================================================

test_that("desctable uses appropriate test for two groups", {
    
    ## Two groups - should work with t-test or Wilcoxon
    result <- desctable(
        data = clintrial,
        by = "sex",  # Two levels
        variables = c("age"),
        test = TRUE,
        test_continuous = "auto"
    )
    
    expect_pvalue_column(result)
})


test_that("desctable uses appropriate test for multiple groups", {
    
    ## Multiple groups - should use ANOVA or Kruskal-Wallis
    result <- desctable(
        data = clintrial,
        by = "stage",  # Four levels
        variables = c("age"),
        test = TRUE,
        test_continuous = "auto"
    )
    
    expect_pvalue_column(result)
})

### * Table formatting

#' Format model results for publication-ready display
#' 
#' Transforms raw model coefficient data into a formatted table suitable for
#' publication. Handles effect measure formatting (OR, HR, RR, Estimate),
#' confidence intervals, \emph{p}-values, sample sizes, and variable labels.
#' Supports interaction terms and mixed-effects models.
#' 
#' @param data Data.table containing raw model results with coefficient columns.
#' @param effect_col Optional character string specifying the effect column name.
#'   If \code{NULL}, auto-detects from OR, HR, RR, or Estimate columns.
#' @param digits Integer number of decimal places for effect estimates.
#' @param p_digits Integer number of decimal places for \emph{p}-values.
#' @param labels Optional named character vector mapping variable names to display labels.
#'   Supports automatic labeling of interaction terms.
#' @param show_n Logical whether to include sample size column.
#' @param show_events Logical whether to include events column (ignored for linear models).
#' @param reference_label Character string to display for reference categories.
#' @param exponentiate Optional logical to force exponentiated (TRUE) or raw (FALSE)
#'   coefficient display. If \code{NULL}, uses existing columns.
#' @return Formatted data.table with publication-ready columns.
#' @keywords internal
format_model_table <- function(data, 
                               effect_col = NULL,
                               digits = 2, 
                               p_digits = 3, 
                               labels = NULL,
                               show_n = TRUE,
                               show_events = TRUE,
                               reference_label = "reference",
                               exponentiate = NULL,
                               conf_level = 0.95,
                               marks = NULL) {
    
    ## Determine the required columns, to avoid copying the whole table
    ## Start with columns that will be in output
    needed_cols <- c("variable", "group", "n", "n_group", "events", "events_group",
                     "p_value", "ci_lower", "ci_upper", "reference", "model_type",
                     "model_scope", "OR", "HR", "RR", "Coefficient",
                     "coef", "coef_lower", "coef_upper", "exp_coef", "exp_lower", "exp_upper")
    
    ## Only copy columns that exist and are needed
    available_cols <- intersect(needed_cols, names(data))
    result <- data[, ..available_cols]
    
    ## Make a shallow copy to avoid modifying original
    result <- data.table::copy(result)

    ## Disallow "Events" column for models where events don't make sense
    ## This includes linear models and GLMs with continuous distributions
    if ("model_type" %in% names(result)) {
        model_type <- result$model_type[1]
        ## Linear, Gamma, Gaussian, and Inverse Gaussian models don't have meaningful events
        continuous_patterns <- c("Linear", "Gamma", "Gaussian", "Inverse")
        if (any(vapply(continuous_patterns, function(p) grepl(p, model_type, ignore.case = TRUE), logical(1)))) {
            show_events <- FALSE
        }
    }
    
    ## Also check if all events are NA - if so, don't show the column
    if (show_events && "events" %in% names(result)) {
        if (all(is.na(result$events))) {
            show_events <- FALSE
        }
    }
    
    ## Standardize column names
    if ("variable" %in% names(result)) {
        data.table::setnames(result, "variable", "Variable")
    }
    if ("group" %in% names(result)) {
        data.table::setnames(result, "group", "Group")
    }

    ## Handle the exponentiate parameter to choose which columns to use
    if (!is.null(exponentiate)) {
        if (exponentiate && "exp_coef" %in% names(result)) {
            ## Check model type
            if ("OR" %in% names(result)) {
                effect_col <- "OR"
            } else if ("HR" %in% names(result)) {
                effect_col <- "HR"
            } else if ("RR" %in% names(result)) {
                effect_col <- "RR"
            } else {
                ## Generic model - create OR/RR based on model type
                model_type <- result$model_type[1]
                if (grepl("Logistic", model_type)) {
                    result[, `:=`(
                        OR = exp_coef,
                        ci_lower = exp_lower,
                        ci_upper = exp_upper
                    )]
                    effect_col <- "OR"
                } else if (grepl("Poisson", model_type)) {
                    result[, `:=`(
                        RR = exp_coef,
                        ci_lower = exp_lower,
                        ci_upper = exp_upper
                    )]
                    effect_col <- "RR"
                } else {
                    result[, `:=`(
                        Coefficient = exp_coef,
                        ci_lower = exp_lower,
                        ci_upper = exp_upper
                    )]
                    effect_col <- "Coefficient"
                }
            }
        } else if (!exponentiate && "coef" %in% names(result)) {
            result[, `:=`(
                Coefficient = coef,
                ci_lower = coef_lower,
                ci_upper = coef_upper
            )]
            effect_col <- "Coefficient"
        }
    }
    
    ## Auto-detect effect column if not specified
    if (is.null(effect_col)) {
        effect_col <- intersect(c("OR", "HR", "RR", "Coefficient"), names(result))[1]
        if (length(effect_col) == 0) {
            stop("No effect measure column found (OR, HR, RR, or Coefficient)")
        }
    }
    
    ## Apply variable labels if provided
    if (!is.null(labels) && "Variable" %in% names(result) && length(labels) > 0) {
        
        ## Create lookup table for main effects
        label_dt <- data.table::data.table(
                                    var_orig = names(labels),
                                    var_new = unname(unlist(labels))
                                )
        
        ## Update main effect variable names using merge
        result[label_dt, Variable := i.var_new, on = .(Variable = var_orig)]
        
        ## Handle interaction terms (contain ":")
        interaction_mask <- grepl(":", result$Variable, fixed = TRUE)
        
        if (any(interaction_mask)) {
            ## Process all interaction terms at once
            interaction_vars <- result$Variable[interaction_mask]
            
            ## Pre-sort labels by length (longest first) once
            sorted_label_names <- names(labels)[order(-nchar(names(labels)))]
            
            ## Vectorized interaction labeling
            labeled_interactions <- vapply(interaction_vars, function(original_var) {
                ## Check for direct custom label first
                if (original_var %in% names(labels)) {
                    return(labels[[original_var]])
                }
                
                ## Split and process components
                components <- strsplit(original_var, ":", fixed = TRUE)[[1]]
                
                labeled_parts <- vapply(components, function(comp) {
                    ## Try to match against base variable names
                    for (base_var in sorted_label_names) {
                        if (startsWith(comp, base_var)) {
                            suffix <- substring(comp, nchar(base_var) + 1)
                            if (nchar(suffix) == 0) {
                                return(labels[[base_var]])
                            } else {
                                return(paste0(labels[[base_var]], " (", suffix, ")"))
                            }
                        }
                    }
                    return(comp)  # No match, keep original
                }, character(1))
                
                paste(labeled_parts, collapse = " \u00d7 ")
            }, character(1))
            
            result$Variable[interaction_mask] <- labeled_interactions
        }
    }
    
    ## Clean up Group display (handle empty groups for continuous vars)
    if ("Group" %in% names(result)) {
        result[Group == "", Group := "-"]
    }

    ## Handle n column
    if ("n" %in% names(result)) {
        has_n_group <- "n_group" %in% names(result)
        
        if (has_n_group) {
            ## Use n_group where available, fallback to n
            n_vals <- data.table::fifelse(!is.na(result$n_group), result$n_group, result$n)
        } else {
            n_vals <- result$n
        }
        
        ## Vectorized formatting
        result[, n := as.character(n_vals)]
        if (!is.null(marks)) {
            result[n_vals >= 1000, n := vapply(n_vals[n_vals >= 1000], format_count, character(1), marks = marks)]
        } else {
            result[n_vals >= 1000, n := format(n_vals[n_vals >= 1000], big.mark = ",")]
        }
        result[is.na(n_vals), n := NA_character_]
    }

    ## Handle events column
    if ("events" %in% names(result)) {
        has_events_group <- "events_group" %in% names(result)
        
        if (has_events_group) {
            ## Use events_group where available, fallback to events
            ## Ensure both are numeric for comparison
            events_grp <- as.numeric(result$events_group)
            events_main <- as.numeric(result$events)
            event_vals <- data.table::fifelse(!is.na(events_grp), events_grp, events_main)
        } else {
            event_vals <- as.numeric(result$events)
        }
        
        ## Vectorized formatting
        evt_str <- as.character(round(event_vals))
        big_idx <- which(!is.na(event_vals) & event_vals >= 1000)
        if (length(big_idx) > 0) {
            if (!is.null(marks)) {
                evt_str[big_idx] <- vapply(round(event_vals[big_idx]), format_count, character(1), marks = marks)
            } else {
                evt_str[big_idx] <- format(round(event_vals[big_idx]), big.mark = ",", scientific = FALSE)
            }
        }
        evt_str[is.na(event_vals)] <- NA_character_
        result[, events := evt_str]
    }
    
    ## Eliminate repeated variable names
    if ("Variable" %in% names(result) && nrow(result) > 1) {
        vars <- result$Variable
        result[, Variable := data.table::fifelse(
                                             c(TRUE, vars[-length(vars)] != vars[-1]),
                                             Variable,
                                             ""
                                         )]
    }
    
    ## Create effect column label based on model scope
    model_scope <- if ("model_scope" %in% names(result)) result$model_scope[1] else "Effect"
    
    ## Format confidence level for display (e.g., 0.95 -> "95%", 0.90 -> "90%")
    ci_pct <- round(conf_level * 100)
    ci_label <- paste0(ci_pct, "% CI")
    
    ## Create appropriate label
    if (model_scope == "Univariable") {
        ## Univariable: OR, HR, RR, Coefficient
        effect_label <- paste0(effect_col, " (", ci_label, ")")
    } else if (model_scope == "Multivariable") {
        ## Multivariable: aOR, aHR, aRR, Adj. Coefficient
        adjusted_col <- switch(effect_col,
                               "OR" = "aOR",
                               "HR" = "aHR", 
                               "RR" = "aRR",
                               "Coefficient" = "Adj. Coefficient",
                               paste0("Adj. ", effect_col)
                               )
        effect_label <- paste0(adjusted_col, " (", ci_label, ")")
    } else {
        effect_label <- paste0(effect_col, " (", ci_label, ")")
    }
    
    ## Format effect sizes with CI
    if ("ci_lower" %in% names(result) && "ci_upper" %in% names(result)) {
        is_reference <- if ("reference" %in% names(result)) {
                            !is.na(result$reference) & result$reference == reference_label
                        } else {
                            rep(FALSE, nrow(result))
                        }
        
        effect_vals <- result[[effect_col]]
        ci_lower_vals <- result$ci_lower
        ci_upper_vals <- result$ci_upper
        
        ## Determine CI separator based on locale and values
        if (!is.null(marks)) {
            any_negative <- any(ci_lower_vals < 0 | ci_upper_vals < 0, na.rm = TRUE)
            ci_sep <- if (any_negative) " to " else if (marks$decimal.mark == ",") "\u2013" else "-"
        } else {
            ci_sep <- if (effect_col %in% c("OR", "HR", "RR")) "-" else ", "
        }
        
        ## Pre-compute format strings for individual components
        fmt_single <- paste0("%.", digits, "f")
        
        ## Vectorized sprintf for components
        eff_fmt <- sprintf(fmt_single, effect_vals)
        low_fmt <- sprintf(fmt_single, ci_lower_vals)
        hi_fmt  <- sprintf(fmt_single, ci_upper_vals)
        
        ## Apply locale decimal mark and fix negative zeros
        if (!is.null(marks)) {
            eff_fmt <- apply_decimal_mark(eff_fmt, marks)
            low_fmt <- apply_decimal_mark(low_fmt, marks)
            hi_fmt  <- apply_decimal_mark(hi_fmt, marks)
        } else {
            eff_fmt <- fix_negative_zero(eff_fmt)
            low_fmt <- fix_negative_zero(low_fmt)
            hi_fmt  <- fix_negative_zero(hi_fmt)
        }
        
        ## Assemble: "effect (lower-upper)" or "effect (lower, upper)"
        ## For non-exponentiated models without marks, use comma+space as separator
        if (is.null(marks) && !effect_col %in% c("OR", "HR", "RR")) {
            formatted_effects <- paste0(eff_fmt, " (", low_fmt, ", ", hi_fmt, ")")
        } else {
            formatted_effects <- paste0(eff_fmt, " (", low_fmt, ci_sep, hi_fmt, ")")
        }
        formatted_effects[is.na(effect_vals)] <- ""
        formatted_effects[is_reference] <- reference_label
        
        result[, (effect_label) := formatted_effects]
    }
    
    ## Format p-values
    if ("p_value" %in% names(result)) {
        result[, `p-value` := format_pvalues_fit(p_value, p_digits, marks)]
        
        if ("reference" %in% names(result)) {
            result[!is.na(reference) & reference == reference_label, `p-value` := "-"]
        }
    }

    ## Select columns for final output
    display_cols <- c(
        if ("Variable" %in% names(result)) "Variable",
        if ("Group" %in% names(result)) "Group",
        if (show_n && "n" %in% names(result)) "n",
        if (show_events && "events" %in% names(result)) "events",
        if (effect_label %in% names(result)) effect_label,
        if ("p-value" %in% names(result)) "p-value"
    )
    
    formatted <- result[, ..display_cols]

    if ("events" %in% names(formatted)) {
        data.table::setnames(formatted, "events", "Events")
    }

    return(formatted)
}


### * Ancillary formatters

#' Fix negative zero in formatted strings
#' 
#' Corrects floating-point rounding artifacts that produce "-0.00" or similar
#' negative zero strings. Works on character vectors, replacing patterns like
#' "-0.00", "-0.000", \emph{etc.} with their positive equivalents, even when embedded
#' within larger strings (\emph{e.g.,} "(-0.00, 1.23)" becomes "(0.00, 1.23)").
#' 
#' @param x Character vector of formatted numbers.
#' @param marks Optional list with \code{big.mark} and \code{decimal.mark} as
#'   returned by \code{\link{resolve_number_marks}}. When \code{NULL}, uses
#'   the default US period decimal.
#' @return Character vector with negative zeros corrected.
#' @keywords internal
fix_negative_zero <- function(x, marks = NULL) {
    if (!is.null(marks)) {
        return(apply_decimal_mark(x, marks))
    }
    ## Fallback: US locale (period decimal)
    gsub("(?<![0-9])-0(\\.0+)(?![0-9])", "0\\1", x, perl = TRUE)
}


#' Format \emph{p}-values for display
#' 
#' Converts numeric \emph{p}-values to formatted character strings using vectorized
#' operations. Values below the threshold (determined by digits parameter) 
#' display as "< 0.001" (for digits=3), "< 0.0001" (for digits=4), \emph{etc.}
#' NA values display as "-".
#' 
#' @param p Numeric vector of \emph{p}-values.
#' @param digits Integer number of decimal places. Also determines the threshold
#'   for "less than" display: threshold = 10^(-digits). Default is 3.
#' @param marks Optional list with \code{big.mark} and \code{decimal.mark} as
#'   returned by \code{\link{resolve_number_marks}}.
#' @return Character vector of formatted \emph{p}-values.
#' @keywords internal
format_pvalues_fit <- function(p, digits = 3, marks = NULL) {
    ## Calculate threshold based on digits
    threshold <- 10^(-digits)
    
    if (!is.null(marks)) {
        less_than_string <- paste0("< 0", marks$decimal.mark,
                                   strrep("0", digits - 1), "1")
    } else {
        less_than_string <- paste0("< ", format(threshold, scientific = FALSE))
    }
    
    ## Pre-compute format string
    fmt_str <- paste0("%.", digits, "f")
    
    ## Vectorized formatting
    result <- sprintf(fmt_str, p)
    result <- fix_negative_zero(result, marks)
    result[is.na(p)] <- "-"
    result[!is.na(p) & p < threshold] <- less_than_string
    
    result
}


### * Fit validation

#' Check if outcome is a Surv() expression
#' 
#' Tests whether an outcome specification string represents a survival outcome
#' by checking for the Surv() function pattern. Used to route model fitting
#' to Cox proportional hazards methods.
#' 
#' @param outcome Character string of the outcome specification.
#' @return Logical \code{TRUE} if outcome starts with "Surv(", \code{FALSE} otherwise.
#' @keywords internal
is_surv_outcome <- function(outcome) {
    grepl("^Surv\\s*\\(", outcome)
}

#' Detect outcome type from data
#' 
#' Automatically determines whether an outcome variable is binary, continuous,
#' or count-based by examining the data values. Used for automatic model type
#' selection and validation. Binary outcomes have 2 unique values, continuous
#' have many values or non-integers, counts have integers >= 0.
#' 
#' @param data Data frame or data.table containing the outcome variable.
#' @param outcome Character string naming the outcome variable.
#' @return Character string: "binary", "continuous", "count", or "unknown".
#' @keywords internal
detect_outcome_type <- function(data, outcome) {
    if (!outcome %in% names(data)) return("unknown")
    
    y <- data[[outcome]]
    
    if (is.factor(y) && length(levels(y)) == 2) return("binary")
    
    if (is.numeric(y)) {
        unique_vals <- unique(y[!is.na(y)])
        if (length(unique_vals) == 2 && all(unique_vals %in% c(0, 1))) {
            return("binary")
        }
        if (all(y[!is.na(y)] >= 0) && 
            all(y[!is.na(y)] == floor(y[!is.na(y)])) &&
            max(y, na.rm = TRUE) > 1) {
            return("count")
        }
        return("continuous")
    }
    "unknown"
}

#' Validate model type matches outcome specification
#' 
#' Ensures consistency between the specified model type, outcome variable type,
#' and GLM family (if applicable). Detects common mismatches like using survival
#' outcomes with non-survival models or binary outcomes with linear models.
#' Can auto-correct fixable issues or raise informative errors.
#'
#' @param outcome Character string outcome specification (may include Surv()).
#' @param model_type Character string specified model type.
#' @param family GLM family object, function, or string if applicable.
#' @param data Data frame or data.table for outcome type detection.
#' @param auto_correct Logical whether to auto-correct fixable mismatches.
#' @return List with model_type, family, messages, auto_corrected flag.
#' @keywords internal
validate_model_outcome <- function(outcome, model_type, family = NULL, 
                                   data = NULL, auto_correct = TRUE) {
    
    corrected_type <- model_type
    is_survival <- is_surv_outcome(outcome)
    
    ## coxph and coxme require Surv() syntax
    ## clogit can work with binary outcome + strata, so don't require Surv()
    surv_required_models <- c("coxph", "coxme")
    non_survival_models <- c("glm", "lm", "lmer", "glmer")
    
    ## Surv() outcome with non-survival model
    if (is_survival && model_type %in% non_survival_models) {
        if (auto_correct) {
            corrected_type <- "coxph"
            message(sprintf(
                "Survival outcome detected but model_type='%s' specified. Switching to 'coxph'.",
                model_type))
        } else {
            stop(sprintf(
                "Survival outcome detected but model_type='%s' specified. Use 'coxph', 'clogit', or 'coxme'.",
                model_type), call. = FALSE)
        }
    }
    
    ## Non-Surv outcome with model that requires Surv()
    if (!is_survival && model_type %in% surv_required_models) {
        stop(sprintf(
            "model_type='%s' requires Surv() syntax.\nExample: outcome = \"Surv(time, status)\"\nGot: \"%s\"",
            model_type, outcome), call. = FALSE)
    }
    
    ## GLM family checks - only when family is a character string
    if (!is.null(data) && model_type == "glm" && !is.null(family) && !is_survival) {
        ## Convert family to string name if it's a family object
        family_name <- if (is.character(family)) {
            family
        } else if (is.function(family)) {
            ## family is a function like binomial, gaussian, etc.
            family()$family
        } else if (is.list(family) && "family" %in% names(family)) {
            ## family is already evaluated (e.g., Gamma(link="log"))
            family$family
        } else {
            NULL
        }
        
        if (!is.null(family_name)) {
            outcome_type <- detect_outcome_type(data, outcome)
            
            if (outcome_type == "categorical" && family_name == "binomial") {
                warning(sprintf(
                    "Categorical outcome '%s' has more than 2 levels. Binomial GLM will coerce this to binary (first level vs all others), which is likely not intended. Consider: (1) recoding to a true binary variable, (2) using multinomial regression (nnet::multinom), (3) using ordinal regression (MASS::polr or ordinal::clm) if levels are ordered, or (4) using a different outcome.",
                    outcome), call. = FALSE)
            }
            if (outcome_type == "continuous" && family_name == "binomial") {
                stop(sprintf(
                    "Continuous outcome '%s' with family='binomial'. Use model_type='lm' or family='gaussian'.",
                    outcome), call. = FALSE)
            }
            if (outcome_type == "binary" && family_name == "gaussian") {
                warning(sprintf(
                    "Binary outcome '%s' with family='gaussian'. Consider family='binomial'.",
                    outcome), call. = FALSE)
            }
        }
    }
    
    ## glmer family checks
    if (!is.null(data) && model_type == "glmer" && !is.null(family) && !is_survival) {
        family_name <- if (is.character(family)) {
            family
        } else if (is.function(family)) {
            family()$family
        } else if (is.list(family) && "family" %in% names(family)) {
            family$family
        } else {
            NULL
        }
        
        if (!is.null(family_name) && family_name == "binomial") {
            outcome_type <- detect_outcome_type(data, outcome)
            if (outcome_type == "categorical") {
                warning(sprintf(
                    "Categorical outcome '%s' has more than 2 levels. Binomial GLMER will coerce this to binary (first level vs all others), which is likely not intended. Consider recoding to a true binary variable or using a different outcome.",
                    outcome), call. = FALSE)
            }
        }
    }
    
    ## lm with binary outcome
    if (!is.null(data) && model_type == "lm" && !is_survival) {
        if (detect_outcome_type(data, outcome) == "binary") {
            warning(sprintf(
                "Binary outcome '%s' with model_type='lm'. Consider model_type='glm' with family='binomial'.",
                outcome), call. = FALSE)
        }
    }
    
    list(model_type = corrected_type, 
         family = family,
         auto_corrected = corrected_type != model_type)
}

#' Validate outcome exists in data
#' 
#' Checks that the specified outcome variable (or survival variables within
#' Surv() expression) exists in the dataset. Raises informative error if
#' variables are missing. Handles both simple outcomes and Surv() expressions.
#' 
#' @param data Data frame or data.table to check.
#' @param outcome Character string outcome specification (may include Surv()).
#' @return Invisible \code{TRUE} if validation passes, otherwise stops with error.
#' @keywords internal
validate_outcome_exists <- function(data, outcome) {
    if (is_surv_outcome(outcome)) {
        surv_content <- gsub("^Surv\\s*\\((.*)\\)$", "\\1", outcome)
        surv_vars <- trimws(unlist(strsplit(surv_content, ",")))
        missing <- surv_vars[!surv_vars %in% names(data)]
        if (length(missing) > 0) {
            stop("Survival variable(s) not found: ", paste(missing, collapse = ", "),
                 call. = FALSE)
        }
    } else if (!outcome %in% names(data)) {
        stop("Outcome '", outcome, "' not found in data.", call. = FALSE)
    }
    invisible(TRUE)
}

#' Validate predictors exist in data
#' 
#' Checks that all specified predictor variables exist in the dataset. Handles
#' interaction terms (splits on ":"), mixed-effects random effects (ignores
#' "|" syntax), and raises informative errors for missing variables.
#' 
#' @param data Data frame or data.table to check.
#' @param predictors Character vector of predictor variable names.
#' @return Invisible \code{TRUE} if validation passes, otherwise stops with error.
#' @keywords internal
validate_predictors_exist <- function(data, predictors) {
    ## Remove random effects and extract interaction components
    fixed <- predictors[!grepl("\\|", predictors)]
    if (any(grepl(":", fixed))) {
        interaction_vars <- unlist(strsplit(fixed[grepl(":", fixed)], ":"))
        fixed <- c(fixed[!grepl(":", fixed)], interaction_vars)
    }
    
    missing <- fixed[!fixed %in% names(data)]
    if (length(missing) > 0) {
        stop("Predictor(s) not found: ", paste(missing, collapse = ", "),
             call. = FALSE)
    }
    invisible(TRUE)
}

#' Complete input validation for fit functions
#'
#' Master validation function called by fit(), uniscreen(), fullfit(). Performs
#' comprehensive checks on data structure, variable existence, numeric parameter
#' ranges, and model-outcome consistency. Returns validated parameters with
#' auto-corrections applied when appropriate.
#'
#' @param data Data frame or data.table containing all variables.
#' @param outcome Character string outcome specification (may include Surv()).
#' @param predictors Character vector of predictor variable names.
#' @param model_type Character string model type to validate.
#' @param family GLM family object, function, or string if applicable.
#' @param conf_level Numeric confidence level (must be between 0 and 1).
#' @param digits Integer number of decimal places for effect estimates.
#' @param p_digits Integer number of decimal places for \emph{p}-values.
#' @param p_threshold Numeric \emph{p}-value threshold for significance highlighting.
#' @param auto_correct_model Logical whether to auto-correct model type mismatches.
#' @return List with validated model_type, family, auto_corrected flag.
#' @keywords internal
validate_fit_inputs <- function(data, outcome, predictors, model_type,
                                family = NULL, conf_level = 0.95,
                                digits = 2, p_digits = 3, p_threshold = NULL,
                                auto_correct_model = TRUE) {
    
    ## Basic data check
    if (is.null(data) || !is.data.frame(data) || nrow(data) == 0) {
        stop("'data' must be a non-empty data.frame.", call. = FALSE)
    }
    
    ## Check variables exist
    validate_outcome_exists(data, outcome)
    validate_predictors_exist(data, predictors)
    
    ## Numeric parameter checks
    if (conf_level <= 0 || conf_level >= 1) {
        stop("'conf_level' must be between 0 and 1.", call. = FALSE)
    }
    if (digits < 0 || digits != floor(digits)) {
        stop("'digits' must be a non-negative integer.", call. = FALSE)
    }
    if (p_digits < 0 || p_digits != floor(p_digits)) {
        stop("'p_digits' must be a non-negative integer.", call. = FALSE
)
    }
    if (!is.null(p_threshold) && (p_threshold < 0 || p_threshold > 1)) {
        stop("'p_threshold' must be between 0 and 1.", call. = FALSE)
    }
    
    ## Model-outcome validation
    validation <- validate_model_outcome(
        outcome = outcome,
        model_type = model_type,
        family = family,
        data = data,
        auto_correct = auto_correct_model
    )
    
    validation
}


### * Miscellaneous functions

#' Fit a model with selective warning suppression
#'
#' Wraps model fitting expressions to suppress routine warnings from mixed-effects
#' and GLM fitting (\emph{e.g.,} singular fits, convergence messages, separation
#' warnings) while allowing unexpected warnings through. When \code{verbose = TRUE},
#' all warnings are displayed.
#'
#' @param expr An unevaluated expression (model fitting call) to execute.
#' @param verbose Logical. If \code{TRUE}, all warnings are shown. If \code{FALSE},
#'   routine fitting warnings are suppressed. Default is \code{FALSE}.
#' @return The result of evaluating \code{expr}.
#' @keywords internal
quiet_fit <- function(expr, verbose = FALSE) {
    if (verbose) {
        return(eval.parent(expr))
    }

    ## Diagnostics that are expected during screening and are reported through
    ## the fitted result rather than the console
    fit_noise <- paste0("singular|convergence|step size|maxfun|",
                        "fitted probabilities numerically 0 or 1|",
                        "fitted rates numerically 0|",
                        "algorithm did not converge|",
                        "number of observations.*less than")

    ## Both condition classes are handled: lme4 raises a singular fit as a
    ## message rather than a warning, so a warning handler alone leaves
    ## "boundary (singular) fit" on the console.
    withCallingHandlers(
        eval.parent(expr),
        warning = function(w) {
            if (grepl(fit_noise, conditionMessage(w), ignore.case = TRUE)) {
                invokeRestart("muffleWarning")
            }
        },
        message = function(m) {
            if (grepl(fit_noise, conditionMessage(m), ignore.case = TRUE)) {
                invokeRestart("muffleMessage")
            }
        }
    )
}

### * Data extraction

#' Process survival probability quantiles
#'
#' Extracts survival time quantiles from survfit objects.
#'
#' @param survfit_objects List of survfit objects.
#' @param probs Numeric vector of probabilities.
#' @param groups Character vector of group names.
#' @param group_labels Character vector of group display labels.
#' @param time_digits Integer decimal places for time values.
#' @param total Logical or character controlling total column.
#' @param total_label Character label for total column.
#' @param median_label Character label for median row.
#' @param by Character name of stratifying variable.
#' @param data Data.table with the source data.
#' @return List with formatted and raw data.tables.
#' @keywords internal
process_survival_probs <- function(survfit_objects,
                                   probs,
                                   groups,
                                   group_labels,
                                   time_digits,
                                   total,
                                   total_label,
                                   median_label,
                                   by,
                                   data,
                                   conf_level = 0.95,
                                   marks = NULL) {

    ## Build column headers for quantiles
    ci_pct <- round(conf_level * 100)
    quantile_cols <- ifelse(
        probs == 0.5,
        median_label,
        sprintf("%d%% Survival Time (%d%% CI)", round((1 - probs) * 100), ci_pct)
    )

    n_probs <- length(probs)
    fmt_str <- paste0("%.", time_digits, "f")

    ## Extract all quantile data at once
    if (!is.null(by) && !is.null(groups) && "stratified" %in% names(survfit_objects)) {
        fit <- survfit_objects$stratified
        quant <- stats::quantile(fit, probs = probs)

        n_rows <- length(groups)
        row_labels <- group_labels

        ## quant$quantile is matrix: strata x probs
        if (is.matrix(quant$quantile)) {
            est_mat <- quant$quantile
            lower_mat <- quant$lower
            upper_mat <- quant$upper
        } else {
            ## Single prob, convert to matrix
            est_mat <- matrix(quant$quantile, ncol = 1)
            lower_mat <- matrix(quant$lower, ncol = 1)
            upper_mat <- matrix(quant$upper, ncol = 1)
        }
    } else {
        ## Overall only
        fit <- survfit_objects$overall
        quant <- stats::quantile(fit, probs = probs)

        n_rows <- 1L
        row_labels <- total_label

        est_mat <- matrix(quant$quantile, nrow = 1)
        lower_mat <- matrix(quant$lower, nrow = 1)
        upper_mat <- matrix(quant$upper, nrow = 1)
    }

    ## Build formatted output
    formatted_list <- vector("list", n_probs + 1L)
    raw_list <- vector("list", n_probs * 3L + 1L)

    formatted_list[[1L]] <- row_labels
    raw_list[[1L]] <- row_labels
    names(formatted_list)[1L] <- "Group"
    names(raw_list)[1L] <- "Group"

    ## Formatting for each quantile column
    for (j in seq_len(n_probs)) {
        col_name <- quantile_cols[j]

        est_vec <- est_mat[, j]
        lower_vec <- lower_mat[, j]
        upper_vec <- upper_mat[, j]

        ## Cell formatting
        formatted_list[[j + 1L]] <- format_quantile_cells(
            est_vec, lower_vec, upper_vec, fmt_str, marks
        )
        names(formatted_list)[j + 1L] <- col_name

        ## Raw values
        raw_idx <- (j - 1L) * 3L + 2L
        raw_list[[raw_idx]] <- est_vec
        raw_list[[raw_idx + 1L]] <- lower_vec
        raw_list[[raw_idx + 2L]] <- upper_vec
        names(raw_list)[raw_idx:(raw_idx + 2L)] <- paste0(col_name, c("_estimate", "_lower", "_upper"))
    }

    formatted <- data.table::as.data.table(formatted_list)
    raw <- data.table::as.data.table(raw_list)

    ## Add overall row if stratified and total requested
    if (!is.null(by) && !isFALSE(total) && "overall" %in% names(survfit_objects)) {
        fit_overall <- survfit_objects$overall
        quant_overall <- stats::quantile(fit_overall, probs = probs)

        est_overall <- as.numeric(quant_overall$quantile)
        lower_overall <- as.numeric(quant_overall$lower)
        upper_overall <- as.numeric(quant_overall$upper)

        ## Build overall row
        overall_formatted <- list(Group = total_label)
        overall_raw <- list(Group = total_label)

        for (j in seq_len(n_probs)) {
            col_name <- quantile_cols[j]
            overall_formatted[[col_name]] <- format_quantile_cells(
                est_overall[j], lower_overall[j], upper_overall[j], fmt_str, marks
            )

            overall_raw[[paste0(col_name, "_estimate")]] <- est_overall[j]
            overall_raw[[paste0(col_name, "_lower")]] <- lower_overall[j]
            overall_raw[[paste0(col_name, "_upper")]] <- upper_overall[j]
        }

        overall_formatted <- data.table::as.data.table(overall_formatted)
        overall_raw <- data.table::as.data.table(overall_raw)

        ## Position total row
        if (isTRUE(total) || identical(total, "first")) {
            formatted <- rbind(overall_formatted, formatted, fill = TRUE)
            raw <- rbind(overall_raw, raw, fill = TRUE)
        } else if (identical(total, "last")) {
            formatted <- rbind(formatted, overall_formatted, fill = TRUE)
            raw <- rbind(raw, overall_raw, fill = TRUE)
        }
    }

    list(formatted = formatted, raw = raw)
}


### * Statistical calculations

#' Perform survival comparison test
#'
#' Performs statistical test comparing survival curves across groups.
#'
#' @param surv_obj Survival object created by Surv().
#' @param group_var Vector of group assignments.
#' @param test_type Character string specifying test type.
#' @return List with test statistic, \emph{p}-value, and test type.
#' @keywords internal
perform_survival_test <- function(surv_obj, group_var, test_type) {
    ## Map test type to survdiff rho parameter
    rho <- switch(test_type,
                  "logrank" = 0,
                  "wilcoxon" = 1,
                  "petopeto" = 1,
                  "tarone" = 0.5,
                  0)

    tryCatch({
        test_obj <- survival::survdiff(surv_obj ~ group_var, rho = rho)
        list(
            statistic = test_obj$chisq,
            df = length(test_obj$n) - 1L,
            p_value = test_obj$pvalue,
            test_type = test_type,
            test_object = test_obj
        )
    }, error = function(e) {
        list(
            statistic = NA_real_,
            df = NA_integer_,
            p_value = NA_real_,
            test_type = test_type,
            test_object = NULL,
            error = e$message
        )
    })
}


#' Process survival at specified time points
#'
#' Extracts survival probabilities at specified time points from survfit objects.
#'
#' @param survfit_objects List of survfit objects.
#' @param times Numeric vector of time points.
#' @param groups Character vector of group names.
#' @param group_labels Character vector of group display labels.
#' @param stats Character vector of statistics to include.
#' @param type Character string specifying probability type.
#' @param digits Integer decimal places for percentages.
#' @param percent Logical whether to display as percentages.
#' @param total Logical or character controlling total column.
#' @param total_label Character label for total column.
#' @param time_label Character template for time column headers.
#' @param time_unit Character time unit for column headers.
#' @param by Character name of stratifying variable.
#' @param data Data.table with the source data.
#' @return List with formatted and raw data.tables.
#' @keywords internal
process_survival_times <- function(survfit_objects,
                                   times,
                                   groups,
                                   group_labels,
                                   stats,
                                   type,
                                   digits,
                                   percent,
                                   total,
                                   total_label,
                                   time_label,
                                   time_unit,
                                   by,
                                   data,
                                   marks = NULL) {

    ## Build column headers for times
    time_cols <- vapply(times, function(t) {
        label <- gsub("\\{time\\}", t, time_label)
        if (!is.null(time_unit)) {
            label <- gsub("\\{unit\\}", time_unit, label)
        } else {
            label <- gsub("\\s*\\{unit\\}", "", label)
        }
        label
    }, character(1))

    n_times <- length(times)

    ## Pre-compute format strings
    if (percent) {
        fmt_est <- paste0("%.", digits, "f%%")
        fmt_ci_lower <- paste0("%.", digits, "f")
        fmt_ci_upper <- paste0("%.", digits, "f%%")
    } else {
        fmt_est <- paste0("%.", digits + 2L, "f")
        fmt_ci_lower <- paste0("%.", digits + 2L, "f")
        fmt_ci_upper <- paste0("%.", digits + 2L, "f")
    }

    ## Extract all survival data at once from stratified fit
    if (!is.null(by) && !is.null(groups) && "stratified" %in% names(survfit_objects)) {
        fit <- survfit_objects$stratified
        summ <- summary(fit, times = times, extend = TRUE)

        ## Build lookup for strata indices
        strata_vec <- as.character(summ$strata)
        strata_lookup <- split(seq_along(strata_vec), strata_vec)

        n_rows <- length(groups)
        row_labels <- group_labels

        ## Pre-allocate result matrices
        est_mat <- matrix(NA_real_, nrow = n_rows, ncol = n_times)
        lower_mat <- matrix(NA_real_, nrow = n_rows, ncol = n_times)
        upper_mat <- matrix(NA_real_, nrow = n_rows, ncol = n_times)
        n_risk_mat <- matrix(NA_integer_, nrow = n_rows, ncol = n_times)
        n_event_mat <- matrix(NA_integer_, nrow = n_rows, ncol = n_times)

        ## Extract data for each stratum
        for (i in seq_along(groups)) {
            strata_name <- paste0(by, "=", groups[i])
            idx <- strata_lookup[[strata_name]]

            if (!is.null(idx) && length(idx) > 0) {
                est_mat[i, ] <- summ$surv[idx]
                lower_mat[i, ] <- summ$lower[idx]
                upper_mat[i, ] <- summ$upper[idx]
                n_risk_mat[i, ] <- summ$n.risk[idx]
                n_event_mat[i, ] <- summ$n.event[idx]
            }
        }

        ## Apply type transformation
        if (type == "risk") {
            est_mat <- 1 - est_mat
            temp <- 1 - lower_mat
            lower_mat <- 1 - upper_mat
            upper_mat <- temp
        } else if (type == "cumhaz") {
            est_mat <- -log(est_mat)
            temp <- -log(lower_mat)
            lower_mat <- -log(upper_mat)
            upper_mat <- temp
        }

    } else {
        ## Overall only
        fit <- survfit_objects$overall
        summ <- summary(fit, times = times, extend = TRUE)

        n_rows <- 1L
        row_labels <- total_label

        est_mat <- matrix(summ$surv, nrow = 1)
        lower_mat <- matrix(summ$lower, nrow = 1)
        upper_mat <- matrix(summ$upper, nrow = 1)
        n_risk_mat <- matrix(summ$n.risk, nrow = 1)
        n_event_mat <- matrix(summ$n.event, nrow = 1)

        ## Apply type transformation
        if (type == "risk") {
            est_mat <- 1 - est_mat
            temp <- 1 - lower_mat
            lower_mat <- 1 - upper_mat
            upper_mat <- temp
        } else if (type == "cumhaz") {
            est_mat <- -log(est_mat)
            temp <- -log(lower_mat)
            lower_mat <- -log(upper_mat)
            upper_mat <- temp
        }
    }

    ## Build formatted output
    formatted_list <- vector("list", n_times + 1L)
    raw_list <- vector("list", n_times * 5L + 1L)

    formatted_list[[1L]] <- row_labels
    raw_list[[1L]] <- row_labels
    names(formatted_list)[1L] <- "Group"
    names(raw_list)[1L] <- "Group"

    ## Formatting for each time column
    for (j in seq_len(n_times)) {
        col_name <- time_cols[j]

        est_vec <- est_mat[, j]
        lower_vec <- lower_mat[, j]
        upper_vec <- upper_mat[, j]
        n_risk_vec <- n_risk_mat[, j]
        n_event_vec <- n_event_mat[, j]

        ## Cell formatting
        formatted_list[[j + 1L]] <- format_survival_cells(
            est_vec, lower_vec, upper_vec, n_risk_vec, n_event_vec,
            stats, fmt_est, fmt_ci_lower, fmt_ci_upper, percent, marks
        )
        names(formatted_list)[j + 1L] <- col_name

        ## Raw values
        raw_idx <- (j - 1L) * 5L + 2L
        raw_list[[raw_idx]] <- est_vec
        raw_list[[raw_idx + 1L]] <- lower_vec
        raw_list[[raw_idx + 2L]] <- upper_vec
        raw_list[[raw_idx + 3L]] <- n_risk_vec
        raw_list[[raw_idx + 4L]] <- n_event_vec
        names(raw_list)[raw_idx:(raw_idx + 4L)] <- paste0(col_name, c("_estimate", "_lower", "_upper", "_n_risk", "_n_event"))
    }

    formatted <- data.table::as.data.table(formatted_list)
    raw <- data.table::as.data.table(raw_list)

    ## Add overall row if stratified and total requested
    if (!is.null(by) && !isFALSE(total) && "overall" %in% names(survfit_objects)) {
        fit_overall <- survfit_objects$overall
        summ_overall <- summary(fit_overall, times = times, extend = TRUE)

        est_overall <- summ_overall$surv
        lower_overall <- summ_overall$lower
        upper_overall <- summ_overall$upper
        n_risk_overall <- summ_overall$n.risk
        n_event_overall <- summ_overall$n.event

        ## Apply type transformation
        if (type == "risk") {
            est_overall <- 1 - est_overall
            temp <- 1 - lower_overall
            lower_overall <- 1 - upper_overall
            upper_overall <- temp
        } else if (type == "cumhaz") {
            est_overall <- -log(est_overall)
            temp <- -log(lower_overall)
            lower_overall <- -log(upper_overall)
            upper_overall <- temp
        }

        ## Build overall row
        overall_formatted <- list(Group = total_label)
        overall_raw <- list(Group = total_label)

        for (j in seq_len(n_times)) {
            col_name <- time_cols[j]
            overall_formatted[[col_name]] <- format_survival_cells(
                est_overall[j], lower_overall[j], upper_overall[j],
                n_risk_overall[j], n_event_overall[j],
                stats, fmt_est, fmt_ci_lower, fmt_ci_upper, percent, marks
            )

            overall_raw[[paste0(col_name, "_estimate")]] <- est_overall[j]
            overall_raw[[paste0(col_name, "_lower")]] <- lower_overall[j]
            overall_raw[[paste0(col_name, "_upper")]] <- upper_overall[j]
            overall_raw[[paste0(col_name, "_n_risk")]] <- n_risk_overall[j]
            overall_raw[[paste0(col_name, "_n_event")]] <- n_event_overall[j]
        }

        overall_formatted <- data.table::as.data.table(overall_formatted)
        overall_raw <- data.table::as.data.table(overall_raw)

        ## Position total row
        if (isTRUE(total) || identical(total, "first")) {
            formatted <- rbind(overall_formatted, formatted, fill = TRUE)
            raw <- rbind(overall_raw, raw, fill = TRUE)
        } else if (identical(total, "last")) {
            formatted <- rbind(formatted, overall_formatted, fill = TRUE)
            raw <- rbind(raw, overall_raw, fill = TRUE)
        }
    }

    list(formatted = formatted, raw = raw)
}


### * Table formatting

#' Survival cell formatting
#'
#' Formats survival probability cells for multiple rows at once. Uses
#' locale-aware decimal marks and safe CI separators that avoid ambiguity
#' with negative values or decimal commas.
#'
#' @param est Numeric vector of estimates.
#' @param lower Numeric vector of lower CI bounds.
#' @param upper Numeric vector of upper CI bounds.
#' @param n_risk Integer vector of numbers at risk.
#' @param n_event Integer vector of event counts.
#' @param stats Character vector of statistics to include.
#' @param fmt_est Format string for estimate.
#' @param fmt_ci_lower Format string for lower CI bound.
#' @param fmt_ci_upper Format string for upper CI bound.
#' @param percent Logical whether percentages.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character vector of formatted cells.
#' @keywords internal
format_survival_cells <- function(est, lower, upper, n_risk, n_event,
                                             stats, fmt_est, fmt_ci_lower,
                                             fmt_ci_upper, percent,
                                             marks = NULL) {
    n <- length(est)
    result <- character(n)

    ## Handle NA values
    na_mask <- is.na(est)

    if (all(na_mask)) {
        return(rep("-", n))
    }

    ## Format non-NA values
    valid_idx <- which(!na_mask)

    if (percent) {
        est_vals <- est[valid_idx] * 100
        lower_vals <- lower[valid_idx] * 100
        upper_vals <- upper[valid_idx] * 100
    } else {
        est_vals <- est[valid_idx]
        lower_vals <- lower[valid_idx]
        upper_vals <- upper[valid_idx]
    }

    ## Format individual components
    est_fmt <- sprintf(fmt_est, est_vals)
    lower_fmt <- sprintf(fmt_ci_lower, lower_vals)
    upper_fmt <- sprintf(fmt_ci_upper, upper_vals)

    ## Apply locale decimal mark and fix negative zeros
    if (!is.null(marks)) {
        est_fmt <- apply_decimal_mark(est_fmt, marks)
        lower_fmt <- apply_decimal_mark(lower_fmt, marks)
        upper_fmt <- apply_decimal_mark(upper_fmt, marks)
    }

    ## Determine CI separator (consistent within column)
    any_negative <- any(lower_vals < 0, upper_vals < 0, na.rm = TRUE)
    if (any_negative) {
        sep <- " to "
    } else if (!is.null(marks) && marks$decimal.mark == ",") {
        sep <- "\u2013"
    } else {
        sep <- "-"
    }

    ## Build formatted strings
    if ("survival" %in% stats && "ci" %in% stats) {
        formatted <- paste0(est_fmt, " (", lower_fmt, sep, upper_fmt, ")")
    } else if ("survival" %in% stats) {
        formatted <- est_fmt
    } else if ("ci" %in% stats) {
        formatted <- paste0("(", lower_fmt, sep, upper_fmt, ")")
    } else {
        formatted <- est_fmt
    }

    ## Add n_risk if requested
    if ("n_risk" %in% stats) {
        n_risk_fmt <- if (!is.null(marks)) {
            vapply(n_risk[valid_idx], format_count, character(1), marks = marks)
        } else {
            as.character(n_risk[valid_idx])
        }
        formatted <- paste0(formatted, " [n = ", n_risk_fmt, "]")
    }

    ## Add n_event if requested
    if ("n_event" %in% stats) {
        n_event_fmt <- if (!is.null(marks)) {
            vapply(n_event[valid_idx], format_count, character(1), marks = marks)
        } else {
            as.character(n_event[valid_idx])
        }
        formatted <- paste0(formatted, " [e = ", n_event_fmt, "]")
    }

    result[valid_idx] <- formatted
    result[na_mask] <- "-"

    result
}


#' Quantile cell formatting
#'
#' Formats survival quantile cells for multiple rows at once.
#'
#' @param est Numeric vector of estimates.
#' @param lower Numeric vector of lower CI bounds.
#' @param upper Numeric vector of upper CI bounds.
#' @param fmt_str Format string for numeric values.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character vector of formatted cells.
#' @keywords internal
format_quantile_cells <- function(est, lower, upper, fmt_str,
                                             marks = NULL) {
    n <- length(est)
    result <- character(n)

    ## Handle NA (not reached) values
    na_mask <- is.na(est)

    if (all(na_mask)) {
        return(rep("NR", n))
    }

    ## Format non-NA values
    valid_idx <- which(!na_mask)

    est_fmt <- sprintf(fmt_str, est[valid_idx])
    lower_fmt <- ifelse(is.na(lower[valid_idx]), "NR", sprintf(fmt_str, lower[valid_idx]))
    upper_fmt <- ifelse(is.na(upper[valid_idx]), "NR", sprintf(fmt_str, upper[valid_idx]))

    if (!is.null(marks)) {
        ## Apply locale decimal mark and fix negative zeros
        est_fmt <- apply_decimal_mark(est_fmt, marks)
        lower_fmt <- ifelse(lower_fmt == "NR", "NR",
                            apply_decimal_mark(lower_fmt, marks))
        upper_fmt <- ifelse(upper_fmt == "NR", "NR",
                            apply_decimal_mark(upper_fmt, marks))

        ## Determine CI separator
        ## Use worst-case across all valid elements for consistency within a column
        any_negative <- any(lower[valid_idx] < 0, upper[valid_idx] < 0, na.rm = TRUE)
        if (any_negative) {
            sep <- " to "
        } else if (marks$decimal.mark == ",") {
            sep <- "\u2013"
        } else {
            sep <- "-"
        }
    } else {
        ## Fallback: check for negatives only
        any_negative <- any(lower[valid_idx] < 0, upper[valid_idx] < 0, na.rm = TRUE)
        sep <- if (any_negative) " to " else "-"
    }

    result[valid_idx] <- paste0(est_fmt, " (", lower_fmt, sep, upper_fmt, ")")
    result[na_mask] <- "NR"

    result
}


#' Add \emph{p}-value column to result table
#'
#' Adds formatted \emph{p}-value column to the survtable result.
#'
#' @param result Data.table result.
#' @param p_value Numeric \emph{p}-value.
#' @param p_digits Integer decimal places for \emph{p}-value.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Data.table with \emph{p}-value column added.
#' @keywords internal
add_pvalue_column <- function(result, p_value, p_digits, marks = NULL) {
    n_rows <- nrow(result)
    p_col <- c(format_pvalue_survtable(p_value, p_digits, marks), rep("", n_rows - 1L))
    result[, `p-value` := p_col]
    result
}


#' Format \emph{p}-value for survtable
#'
#' Provides \emph{p}-value formatting to the survtable result.
#'
#' @param p Numeric \emph{p}-value.
#' @param digits Integer decimal places.
#' @param marks List with \code{big.mark} and \code{decimal.mark} as returned
#'   by \code{\link{resolve_number_marks}}.
#' @return Character-formatted \emph{p}-value.
#' @keywords internal
format_pvalue_survtable <- function(p, digits, marks = NULL) {
    if (is.na(p)) {
        return("-")
    }

    ## Use marks-aware formatter if available
    if (!is.null(marks)) {
        return(format_pvalue(p, digits, marks))
    }

    ## Fallback (e.g., from print method without marks)
    threshold <- 10^(-digits)
    if (p < threshold) {
        return(paste0("< ", format(threshold, scientific = FALSE)))
    }

    sprintf(paste0("%.", digits, "f"), p)
}

### * Main function

#' Export Table to HTML Format
#'
#' Converts a data frame, data.table, or matrix to HTML format with optional CSS 
#' styling for web display, HTML documents, or embedding in web applications. 
#' Generates clean, standards-compliant HTML with professional styling options 
#' including responsive design support, color schemes, and interactive features.
#' Requires \pkg{xtable} for export.
#'
#' @param table Data frame, data.table, or matrix to export. Can be output from 
#'   \code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
#'   \code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.
#'   
#' @param file Character string specifying the output HTML filename. Must have 
#'   \code{.html} or \code{.htm} extension. Example: \code{"results.html"}.
#'   
#' @param caption Character string. Optional caption displayed below the table. 
#'   Supports basic HTML formatting. Default is \code{NULL}.
#'   
#' @param format_headers Logical. If \code{TRUE}, formats column headers by 
#'   converting underscores to spaces and applying proper casing. Wraps statistical 
#'   notation ("\emph{n}", "\emph{p}") in \code{<i>} tags. Default is \code{TRUE}.
#'   
#' @param variable_padding Logical. If \code{TRUE}, adds vertical spacing around 
#'   variable groups for improved readability. Particularly useful for multi-row 
#'   factor variables. Default is \code{FALSE}.
#'   
#' @param bold_significant Logical. If \code{TRUE}, wraps significant \emph{p}-values 
#'   in \code{<b>} tags for bold display. Makes important results stand out 
#'   visually. Default is \code{TRUE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. Default is \code{FALSE}.
#'   
#' @param p_threshold Numeric. Threshold for bold \emph{p}-value formatting. Only 
#'   used when \code{bold_significant = TRUE}. Default is 0.05.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents grouped rows using 
#'   non-breaking spaces (\code{&nbsp;}) for hierarchical display. Useful for 
#'   factor variables in regression output. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses table by showing 
#'   only essential rows. Automatically sets \code{indent_groups = TRUE}. 
#'   Default is \code{FALSE}.
#'   
#' @param condense_quantitative Logical. If \code{TRUE}, condenses continuous 
#'   and survival variables into single rows while preserving all categorical 
#'   variable rows (including binary). Only applies to descriptive tables from 
#'   \code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
#'   \code{condense_table}, this does not collapse binary categorical variables. 
#'   Default is \code{FALSE}.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating background 
#'   shading to different variables (not individual rows) for visual grouping. 
#'   Default is \code{FALSE}.
#'   
#' @param stripe_color Character string. HTML color specification for zebra 
#'   stripes. Can use hex codes (\code{"#EEEEEE"}), RGB (\code{"rgb(238,238,238)"}), 
#'   or color names (\code{"lightgray"}). Default is \code{"#EEEEEE"}.
#'   
#' @param dark_header Logical. If \code{TRUE}, creates black background with 
#'   white text for the header row. Provides strong visual contrast. 
#'   Default is \code{FALSE}.
#'   
#' @param include_css Logical. If \code{TRUE}, includes embedded CSS styling in 
#'   the output file for standalone HTML. Set to \code{FALSE} when embedding 
#'   in existing HTML with its own stylesheet. Default is \code{TRUE}.
#'   
#' @param ... Additional arguments passed to \code{\link[xtable]{xtable}}.
#'
#' @param quiet Logical. Suppress progress and confirmation messages, such
#'   as the notice reporting the file written. Errors and warnings are
#'   unaffected. Default is \code{FALSE}.
#'
#' @return Invisibly returns the file path, matching \code{tablesave()} and
#'   \code{forestsave()}. Called for its side effect of creating an HTML file at
#'   the specified location, which can be opened in web browsers or embedded in
#'   HTML documents.
#'
#' @details
#' \strong{Output Format:}
#' 
#' The function generates standards-compliant HTML5 markup with:
#' \itemize{
#'   \item Semantic \code{<table>} structure
#'   \item Proper \code{<thead>} and \code{<tbody>} sections
#'   \item Accessible header cells (\code{<th>})
#'   \item Clean, readable markup
#'   \item Optional embedded CSS styling
#' }
#' 
#' \strong{Standalone vs. Embedded:}
#' 
#' \emph{Standalone HTML (\code{include_css = TRUE}):}
#' \itemize{
#'   \item Can be opened directly in web browsers
#'   \item Includes all necessary styling
#'   \item Self-contained, portable
#'   \item Suitable for sharing via email or web hosting
#' }
#' 
#' \emph{Embedded HTML (\code{include_css = FALSE}):}
#' \itemize{
#'   \item For inclusion in existing HTML documents
#'   \item No CSS included (use parent document's styles)
#'   \item Smaller file size
#'   \item Integrates with web frameworks (Shiny, R Markdown, Quarto)
#' }
#' 
#' \strong{CSS Styling:}
#' 
#' When \code{include_css = TRUE}, the function applies professional styling:
#' \itemize{
#'   \item \strong{Table:} Border-collapse, sans-serif font (Arial), 20px margin
#'   \item \strong{Cells:} 8px vertical × 12px horizontal padding, left-aligned text
#'   \item \strong{Borders:} 1px solid \code{#DDD} (light gray)
#'   \item \strong{Headers:} Bold text, light gray background (\code{#F2F2F2})
#'   \item \strong{Numeric columns:} Center-aligned (auto-detected)
#'   \item \strong{Caption:} Bold, 1.1em font, positioned below table
#' }
#' 
#' \emph{With \code{dark_header = TRUE}:}
#' \itemize{
#'   \item Header background: Black (\code{#000000})
#'   \item Header text: White (\code{#FFFFFF})
#'   \item Creates high contrast, modern appearance
#' }
#' 
#' \emph{With \code{zebra_stripes = TRUE}:}
#' \itemize{
#'   \item Alternating variable groups receive background color
#'   \item Default color: \code{#EEEEEE} (light gray)
#'   \item Applied via CSS class \code{.zebra-stripe}
#'   \item Groups entire variable (all factor levels together)
#' }
#' 
#' \strong{Hierarchical Display:}
#' 
#' The \code{indent_groups} option creates visual hierarchy using HTML 
#' non-breaking spaces:
#' \preformatted{
#' <td><b>Treatment</b></td>  <!-- Variable name -->
#' <td>&nbsp;&nbsp;&nbsp;&nbsp;Control</td>  <!-- Indented level -->
#' <td>&nbsp;&nbsp;&nbsp;&nbsp;Active</td>   <!-- Indented level -->
#' }
#' 
#' \strong{Integration with R Markdown/Quarto:}
#' 
#' For R Markdown or Quarto documents:
#' \preformatted{
#' # Generate HTML fragment (no CSS)
#' table2html(results, "table.html", include_css = FALSE)
#' }
#' 
#' Then include in the document chunk with \code{results='asis'}:
#' \preformatted{
#' cat(readLines("table.html"), sep = "\n")
#' }
#' 
#' Or directly render without file:
#' \preformatted{
#' # For inline display
#' htmltools::HTML(
#'   capture.output(
#'     print(xtable::xtable(results), type = "html")
#'   )
#' )
#' }
#' 
#' \strong{Integration with Shiny:}
#' 
#' For Shiny applications:
#' \preformatted{
#' # In server function
#' output$results_table <- renderUI(\{
#'   table2html(results_data(), "temp.html", include_css = FALSE)
#'   HTML(readLines("temp.html"))
#' \})
#' 
#' # Or use directly with DT package for interactive tables
#' output$interactive_table <- DT::renderDT(\{
#'   results_data()
#' \})
#' }
#' 
#' \strong{Accessibility:}
#' 
#' The generated HTML follows accessibility best practices:
#' \itemize{
#'   \item Semantic table structure
#'   \item Proper header cells (\code{<th>}) with scope attributes
#'   \item Clear visual hierarchy
#'   \item Adequate color contrast (when using default styles)
#'   \item Screen reader friendly markup
#' }
#' 
#' @seealso
#' \code{\link{tablesave}} for saving in the format given by the file extension,
#' \code{\link{table2pdf}} for PDF output,
#' \code{\link{table2tex}} for LaTeX output,
#' \code{\link{table2docx}} for Word documents,
#' \code{\link{table2pptx}} for PowerPoint,
#' \code{\link{table2rtf}} for Rich Text Format,
#' \code{\link{fit}} for regression tables,
#' \code{\link{desctable}} for descriptive tables
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example table
#' results <- fit(
#'      data = clintrial,
#'      outcome = "os_status",
#'      predictors = c("age", "sex", "treatment", "stage"),
#'      labels = clintrial_labels
#' )
#'
#' # Example 1: Basic HTML export (standalone)
#' if (requireNamespace("xtable", quietly = TRUE)) {
#'   table2html(results, file.path(tempdir(), "results.html"))
#' }
#'
#' \donttest{
#' # Example 2: With caption
#' table2html(results, file.path(tempdir(), "captioned.html"),
#'           caption = "Table 1: Multivariable Logistic Regression Results")
#' 
#' # Example 3: For embedding (no CSS)
#' table2html(results, file.path(tempdir(), "embed.html"),
#'           include_css = FALSE)
#' # Include in the HTML document
#' 
#' # Example 4: Hierarchical display
#' table2html(results, file.path(tempdir(), "indented.html"),
#'           indent_groups = TRUE)
#' 
#' # Example 5: Condensed table
#' table2html(results, file.path(tempdir(), "condensed.html"),
#'           condense_table = TRUE)
#' 
#' # Example 6: With zebra stripes
#' table2html(results, file.path(tempdir(), "striped.html"),
#'           zebra_stripes = TRUE,
#'           stripe_color = "#F0F0F0")
#' 
#' # Example 7: Dark header style
#' table2html(results, file.path(tempdir(), "dark.html"),
#'           dark_header = TRUE)
#' 
#' # Example 8: Combination styling
#' table2html(results, file.path(tempdir(), "styled.html"),
#'           zebra_stripes = TRUE,
#'           dark_header = TRUE,
#'           bold_significant = TRUE)
#' 
#' # Example 9: Custom stripe color
#' table2html(results, file.path(tempdir(), "blue_stripes.html"),
#'           zebra_stripes = TRUE,
#'           stripe_color = "#E3F2FD")  # Light blue
#' 
#' # Example 10: Disable significance bolding
#' table2html(results, file.path(tempdir(), "no_bold.html"),
#'           bold_significant = FALSE)
#' 
#' # Example 11: Stricter significance threshold
#' table2html(results, file.path(tempdir(), "strict.html"),
#'           bold_significant = TRUE,
#'           p_threshold = 0.01)
#' 
#' # Example 12: No header formatting
#' table2html(results, file.path(tempdir(), "raw_headers.html"),
#'           format_headers = FALSE)
#' 
#' # Example 13: Descriptive statistics table
#' desc_table <- desctable(clintrial, by = "treatment",
#'      variables = c("age", "sex", "bmi"), labels = clintrial_labels)
#' 
#' table2html(desc_table, file.path(tempdir(), "baseline.html"),
#'           caption = "Table 1: Baseline Characteristics by Treatment Group")
#' 
#' # Example 14: For R Markdown (no CSS, for inline display)
#' table2html(results, file.path(tempdir(), "rmd_table.html"),
#'           include_css = FALSE,
#'           indent_groups = TRUE)
#' 
#' # Then in R Markdown, use a chunk with results='asis' to display inline:
#' cat(readLines(file.path(tempdir(), "rmd_table.html")), sep = "\n")
#' 
#' # Example 15: Email-friendly version
#' table2html(results, file.path(tempdir(), "email.html"),
#'           include_css = TRUE,  # Self-contained
#'           zebra_stripes = TRUE,
#'           caption = "Regression Results - See Attached")
#' # Can be directly included in HTML emails
#' 
#' # Example 16: Publication-ready web version
#' table2html(results, file.path(tempdir(), "publication.html"),
#'           caption = "Table 2: Multivariable Analysis of Risk Factors",
#'           indent_groups = TRUE,
#'           zebra_stripes = FALSE,  # Clean look
#'           bold_significant = TRUE,
#'           dark_header = FALSE)
#' 
#' # Example 17: Modern dark theme
#' table2html(results, file.path(tempdir(), "dark_theme.html"),
#'           dark_header = TRUE,
#'           stripe_color = "#2A2A2A",  # Dark gray stripes
#'           zebra_stripes = TRUE)
#' 
#' # Example 18: Minimal styling for custom CSS
#' table2html(results, file.path(tempdir(), "minimal.html"),
#'           include_css = FALSE,
#'           format_headers = FALSE,
#'           bold_significant = FALSE)
#' # Apply personal CSS classes and styling
#' 
#' # Example 19: Model comparison table
#' models <- list(
#'      base = c("age", "sex"),
#'      full = c("age", "sex", "treatment", "stage")
#' )
#' 
#' # Information criteria assume a common sample, so the candidate
#' # predictors are restricted to complete cases before comparison
#' comparison_data <- na.omit(clintrial[, c("os_status", "age", "sex", "treatment", "stage")])
#' 
#' comparison <- compfit(
#'      data = comparison_data,
#'      outcome = "os_status",
#'      model_list = models
#' )
#' 
#' table2html(comparison, file.path(tempdir(), "comparison.html"),
#'           caption = "Model Comparison Statistics")
#' 
#' }
#' @family export functions
#' @export
table2html <- function(table,
                       file,
                     caption = NULL,
                     format_headers = TRUE,
                     variable_padding = FALSE, 
                     bold_significant = TRUE,
                     bold_variables = FALSE,
                     p_threshold = 0.05,
                     indent_groups = FALSE,
                     condense_table = FALSE,
                     condense_quantitative = FALSE,
                     zebra_stripes = FALSE,
                     stripe_color = "#EEEEEE",
                     dark_header = FALSE,
                     include_css = TRUE,
                     quiet = FALSE,
                     ...) {
    
    if (!requireNamespace("xtable", quietly = TRUE)) {
        stop("Package 'xtable' required. Install with: install.packages('xtable')")
    }
    
    if (!grepl("\\.(html|htm)$", tolower(file))) {
        stop("File must have .html or .htm extension")
    }
    
    df <- as.data.frame(table)

    has_n_row <- FALSE
    n_row_data <- NULL
    if (nrow(df) > 0 && "Variable" %in% names(df) && df$Variable[1] == "N") {
        has_n_row <- TRUE
        n_row_data <- df[1, ]
        df <- df[-1, ]
    }

    ## Apply condense/indent transformations first
    if (condense_table) {
        indent_groups <- TRUE
        df <- condense_table_rows(df, indent_groups = indent_groups)
        df <- format_indented_groups(df, indent_string = "&nbsp;&nbsp;&nbsp;&nbsp;")
    } else if (condense_quantitative) {
        ## Only condense continuous/survival variables (not categorical)
        ## Also set indent_groups = TRUE to avoid awkward empty Group column
        indent_groups <- TRUE
        df <- condense_quantitative_rows(df, indent_groups = indent_groups)
        df <- format_indented_groups(df, indent_string = "&nbsp;&nbsp;&nbsp;&nbsp;")
    } else if (indent_groups) {
        df <- format_indented_groups(df, indent_string = "&nbsp;&nbsp;&nbsp;&nbsp;")
    }

    ## Apply variable padding AFTER all other transformations
    if (variable_padding && ("Variable" %in% names(df) || "variable" %in% names(df))) {
        df <- add_variable_padding(df)
    }

    ## Bold variable names (non-indented rows in Variable column)
    if (bold_variables && "Variable" %in% names(df)) {
        ## Find rows where Variable is not empty and not indented (doesn't start with &nbsp;)
        var_rows <- which(!is.na(df$Variable) & 
                          df$Variable != "" & 
                          df$Variable != "-" &
                          !grepl("^&nbsp;", df$Variable))
        if (length(var_rows) > 0) {
            df$Variable[var_rows] <- paste0("<b>", df$Variable[var_rows], "</b>")
        }
    }

    if (bold_significant) {
        df <- format_pvalues_export_html(df, p_threshold)
    }
    
    ## Calculate var_groups on the FINAL data frame structure
    var_groups <- NULL
    if (zebra_stripes && "Variable" %in% names(df)) {
        ## Identify padding rows (all cells empty) - these should not be striped
        is_padding_row <- apply(df, 1, function(row) {
            all(is.na(row) | row == "" | row == " ")
        })
        
        ## For indented tables, non-indented rows mark variable starts
        if (indent_groups || condense_table || condense_quantitative) {
            var_starts <- which(!grepl("^&nbsp;", df$Variable) & 
                                !grepl("^<b>&nbsp;", df$Variable) &
                                df$Variable != "" & 
                                !is.na(df$Variable) &
                                !is_padding_row)
        } else {
            ## For regular tables, any non-empty Variable marks a start
            var_starts <- which(df$Variable != "" & !is.na(df$Variable) & !is_padding_row)
        }
        
        if (length(var_starts) > 0) {
            var_groups <- integer(nrow(df))
            for (i in seq_along(var_starts)) {
                start_idx <- var_starts[i]
                end_idx <- if (i < length(var_starts)) {
                               var_starts[i + 1] - 1
                           } else {
                               nrow(df)
                           }
                var_groups[start_idx:end_idx] <- i
            }
            
            ## Mark padding rows with 0 so they don't get striped
            var_groups[is_padding_row] <- 0L
            
            ## Apply zebra stripes by tracking groups
            df$.zebra_group <- var_groups
        }
    }
    
    if (format_headers) {
        if (has_n_row) {
            names(df) <- format_column_headers_with_n_html(names(df), n_row_data)
        } else {
            names(df) <- format_column_headers_html(names(df))
        }
    }
    
    ## Remove zebra tracking column from display
    display_df <- df
    if (zebra_stripes && ".zebra_group" %in% names(display_df)) {
        zebra_groups <- display_df$.zebra_group
        display_df$.zebra_group <- NULL
    } else {
        zebra_groups <- NULL
    }
    
    xt <- xtable::xtable(display_df, caption = caption, ...)
    
    ## Build CSS (unchanged)
    if (include_css) {
        css <- paste0("<style>\n",
                      "table { \n",
                      "border-collapse: collapse; \n",
                      "font-family: Arial, sans-serif;\n",
                      "margin: 20px;\n",
                      "}\n",
                      "th, td { \n",
                      "padding: 8px 12px; \n",
                      "text-align: left; \n",
                      "border: 1px solid #ddd;\n",
                      "}\n")
        
        if (indent_groups) {
            css <- paste0(css,
                          "th:not(:first-child), \n",
                          "td:not(:first-child) { \n",
                          "text-align: center; \n",
                          "}\n")
        } else {
            css <- paste0(css,
                          "th:not(:nth-child(1)):not(:nth-child(2)), \n",
                          "td:not(:nth-child(1)):not(:nth-child(2)) { \n",
                          "text-align: center; \n",
                          "}\n")
        }
        
        ## Header styling - dark or light
        if (dark_header) {
            css <- paste0(css,
                          "th { \n",
                          "background-color: #000000; \n",
                          "color: #FFFFFF; \n",
                          "font-weight: bold;\n",
                          "}\n")
        } else {
            css <- paste0(css,
                          "th { \n",
                          "background-color: #f2f2f2; \n",
                          "font-weight: bold;\n",
                          "}\n")
        }
        
        ## Zebra stripe styling
        if (zebra_stripes) {
            css <- paste0(css,
                          "tr.zebra-stripe { \n",
                          "background-color: ", stripe_color, ";\n",
                          "}\n")
        }
        
        css <- paste0(css,
                      "caption {\n",
                      "text-align: left;\n",
                      "margin-top: 10px;\n",
                      "margin-bottom: 10px;\n",
                      "font-weight: bold;\n",
                      "font-size: 1.1em;\n",
                      "}\n",
                      "</style>\n")
    }
    
    ## Generate HTML table
    if (zebra_stripes && !is.null(zebra_groups)) {
        ## Capture the HTML output
        html_output <- capture.output(
            print(xt,
                  type = "html",
                  include.rownames = FALSE,
                  sanitize.text.function = identity,
                  sanitize.rownames.function = identity,
                  sanitize.colnames.function = identity,
                  ...)
        )
        
        ## Add classes to TR elements based on zebra groups
        tr_count <- 0
        for (i in seq_along(html_output)) {
            if (grepl("^  <tr>", html_output[i])) {
                tr_count <- tr_count + 1
                if (tr_count <= length(zebra_groups)) {
                    group_num <- zebra_groups[tr_count]
                    if (!is.na(group_num) && group_num %% 2 == 1) {
                        html_output[i] <- gsub("  <tr>", '  <tr class="zebra-stripe">', html_output[i])
                    }
                }
            }
        }
        
        ## Write everything to file
        if (include_css) {
            writeLines(c(css, html_output), file)
        } else {
            writeLines(html_output, file)
        }
    } else {
        ## Standard output without zebra stripes
        if (include_css) {
            cat(css, file = file)
            print(xt,
                  type = "html",
                  file = file,
                  append = TRUE,
                  include.rownames = FALSE,
                  sanitize.text.function = identity,
                  sanitize.rownames.function = identity,
                  sanitize.colnames.function = identity,
                  ...)
        } else {
            print(xt,
                  type = "html",
                  file = file,
                  append = FALSE,
                  include.rownames = FALSE,
                  sanitize.text.function = identity,
                  sanitize.rownames.function = identity,
                  sanitize.colnames.function = identity,
                  ...)
        }
    }
    
    if (!quiet) message(sprintf("Table saved to %s", file))
    
    invisible(file)
}

################################################################################
##                    summata Performance Benchmarks
## 
## Comparing summata functions against popular alternatives:
##   - Descriptive tables: desctable() vs tableby, tbl_summary, CreateTableOne,
##                         summary_factorlist
##   - Regression output: fit() vs tbl_regression, broom::tidy, glmmulti+fit2df
##   - Univariable screening: uniscreen() vs tbl_uvregression, modelsum, 
##                            glmuni+fit2df
##   - Complete workflows: fullfit() vs finalfit(), gtsummary multi-step
##   - Forest plots: glmforest/coxforest() vs ggforest, manual ggplot2
##   - Mixed-effects models: fit() with lmer/glmer/coxme vs alternatives
##
## Author: Paul H. McClelland
## Date: 2025-12-20
################################################################################

## =============================================================================
## SETUP
## =============================================================================

required_packages <- c(
    "summata", "finalfit", "arsenal", "gtsummary", "tableone",
    "broom", "broom.mixed", "survminer", "data.table", "microbenchmark",
    "ggplot2", "survival", "lme4", "coxme"
)

install_if_missing <- function(pkg) {
    if (!requireNamespace(pkg, quietly = TRUE)) {
        message("Installing package: ", pkg)
        install.packages(pkg, repos = "https://cloud.r-project.org")
    }
}
invisible(lapply(required_packages, install_if_missing))

library(summata)
library(finalfit)
library(arsenal)
library(gtsummary)
library(tableone)
library(broom)
library(broom.mixed)
library(survminer)
library(data.table)
library(microbenchmark)
library(ggplot2)
library(survival)
library(lme4)
library(coxme)
library(ragg)

set.seed(71)
setDTthreads(0)

## =============================================================================
## GENERATE BENCHMARK DATASETS
## =============================================================================

generate_benchmark_data <- function(n, seed = 71) {
    set.seed(seed)
    
    data.table(
        id = 1:n,
        site = factor(sample(paste0("Site_", 1:10), n, replace = TRUE)),
        age = round(rnorm(n, mean = 55, sd = 12)),
        sex = factor(sample(c("Male", "Female"), n, replace = TRUE)),
        race = factor(sample(c("White", "Black", "Asian", "Other"), n, 
                             replace = TRUE, prob = c(0.6, 0.2, 0.15, 0.05)),
                      levels = c("White", "Black", "Asian", "Other")),
        bmi = round(rnorm(n, mean = 27, sd = 5), 1),
        smoking = factor(sample(c("Never", "Former", "Current"), n, 
                                replace = TRUE, prob = c(0.4, 0.35, 0.25)),
                         levels = c("Never", "Former", "Current")),
        hypertension = factor(sample(0:1, n, replace = TRUE, prob = c(0.7, 0.3))),
        diabetes = factor(sample(0:1, n, replace = TRUE, prob = c(0.85, 0.15))),
        treatment = factor(sample(c("Placebo", "Drug A", "Drug B"), n, replace = TRUE),
                           levels = c("Placebo", "Drug A", "Drug B")),
        stage = factor(sample(c("I", "II", "III", "IV"), n, 
                              replace = TRUE, prob = c(0.2, 0.3, 0.35, 0.15)),
                       levels = c("I", "II", "III", "IV")),
        grade = factor(sample(1:3, n, replace = TRUE), 
                       levels = 1:3, labels = c("Low", "Moderate", "High")),
        biomarker1 = round(rlnorm(n, meanlog = 2, sdlog = 0.8), 2),
        biomarker2 = round(rnorm(n, mean = 100, sd = 25), 1),
        creatinine = round(rnorm(n, mean = 1.1, sd = 0.3), 2),
        hemoglobin = round(rnorm(n, mean = 13.5, sd = 1.8), 1),
        los_days = round(rexp(n, rate = 0.1) + 1, 1),
        event_count = rpois(n, lambda = 2),
        response = rbinom(n, 1, prob = 0.3),
        os_time = round(rexp(n, rate = 0.02) + 1, 1),
        os_status = rbinom(n, 1, prob = 0.4)
    )
}

cat("Generating benchmark datasets...\n")
data_50k <- generate_benchmark_data(50000)

data_500 <- data_50k[1:500]
data_1k <- data_50k[1:1000]
data_5k <- data_50k[1:5000]
data_10k <- data_50k[1:10000]

df_500 <- as.data.frame(data_500)
df_1k <- as.data.frame(data_1k)
df_5k <- as.data.frame(data_5k)
df_10k <- as.data.frame(data_10k)
df_50k <- as.data.frame(data_50k)

vars_continuous <- c("age", "bmi", "biomarker1", "biomarker2", "creatinine", "hemoglobin")
vars_categorical <- c("sex", "race", "smoking", "hypertension", "diabetes", "treatment", "stage", "grade")
vars_all <- c(vars_continuous, vars_categorical)

cat("Datasets ready: 500, 1k, 5k, 10k, 50k observations\n\n")

## =============================================================================
## COMMON PLOTTING THEME AND COLOR PALETTE
## =============================================================================

## Define consistent colors for all packages
pkg_colors <- c(
    "summata" = "#FC8D62",
    "summata_minimal" = "#FCB362",
    "finalfit" = "#5597DF",
    "gtsummary" = "#66C2A5",
    "arsenal" = "#A6D854",
    "tableone" = "#8DA0CB",
    "broom" = "#FFD92F",
    "broom_loop" = "#E5C494",
    "manual" = "#E78AC3",
    "survminer" = "#FB9A99"
)

## Helper function to reorder factor levels by median time (fastest to slowest)
order_by_speed <- function(dt, time_col = "time_ms", group_col = "expr") {
    ## Calculate median time for each package
    medians <- dt[, .(med_time = median(get(time_col))), by = group_col]
    setorder(medians, med_time)
    
    ## Reorder factor levels
    dt[, (group_col) := factor(get(group_col), levels = medians[[group_col]])]
    return(dt)
}

## Font setup for consistent rendering across devices
if (requireNamespace("ragg", quietly = TRUE)) {
    ## Use ragg for high-quality PNG output with proper font rendering
    knitr::opts_chunk$set(dev = "ragg_png") # For knitr/rmarkdown
    
    ## For ggsave, we'll use the device argument
    png_device <- ragg::agg_png
    cat("Using ragg for PNG rendering (recommended)\n")
} else if (capabilities("cairo")) {
    png_device <- function(...) grDevices::png(..., type = "cairo")
    cat("Using cairo for PNG rendering\n")
} else {
    png_device <- grDevices::png
    cat("Using default PNG device (fonts may differ from PDF)\n")
    message("For consistent fonts, install the 'ragg' package: install.packages('ragg')")
}

## Custom ggsave wrapper that uses appropriate device for each format
save_plot <- function(filename, plot, width = 10, height = 5, dpi = 150, ...) {
    ext <- tools::file_ext(filename)
    
    if (ext == "png" && exists("png_device")) {
        ggsave(filename, plot, width = width, height = height, dpi = dpi,
               device = png_device, ...)
    } else {
        ggsave(filename, plot, width = width, height = height, dpi = dpi, ...)
    }
    cat("Saved:", filename, "\n")
}

theme_benchmark <- function() {
    theme_minimal(base_family = "Helvetica") +
        theme(
            plot.title = element_text(face = "bold", size = 14),
            plot.subtitle = element_text(size = 10, color = "gray40"),
            axis.text.x = element_text(angle = 45, hjust = 1),
            legend.position = "bottom",
            panel.grid.minor = element_blank(),
            strip.background = element_rect(fill = "gray95", color = NA),
            strip.text = element_text(face = "bold")
        )
}

## =============================================================================
## BENCHMARK 1: DESCRIPTIVE STATISTICS TABLES
## =============================================================================

cat(strrep("=", 70), "\n")
cat("BENCHMARK 1: DESCRIPTIVE STATISTICS TABLES\n")
cat(strrep("=", 70), "\n\n")

benchmark_desctable <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    
    vars_for_finalfit <- setdiff(vars_all, "treatment")
    
    microbenchmark(
        summata = desctable(data, by = "treatment", variables = vars_all,
                            stats_continuous = c("mean_sd", "median_iqr"), test = TRUE),
        finalfit = summary_factorlist(df, dependent = "treatment",
                                      explanatory = vars_for_finalfit, p = TRUE, cont = "mean"),
        arsenal = {
            tab <- tableby(treatment ~ age + bmi + biomarker1 + biomarker2 + creatinine + 
                               hemoglobin + sex + race + smoking + hypertension + diabetes + stage + grade,
                           data = df, test = TRUE, numeric.stats = c("meansd", "medianq1q3"))
            as.data.frame(summary(tab, text = TRUE))
        },
        gtsummary = add_p(tbl_summary(df, by = treatment, include = all_of(vars_all),
                                      statistic = list(all_continuous() ~ "{mean} ({sd}), {median} ({p25}, {p75})",
                                                       all_categorical() ~ "{n} ({p}%)"), missing = "no")),
        tableone = CreateTableOne(vars = vars_all, strata = "treatment", data = df, test = TRUE, smd = FALSE),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Descriptive Table Benchmarks ---\n\n")
bench_desc_1k <- benchmark_desctable(data_1k, df_1k, n_times = 15)
bench_desc_5k <- benchmark_desctable(data_5k, df_5k, n_times = 10)
bench_desc_10k <- benchmark_desctable(data_10k, df_10k, n_times = 5)

bench_desc_all <- rbindlist(list(
    as.data.table(bench_desc_1k)[, n := 1000],
    as.data.table(bench_desc_5k)[, n := 5000],
    as.data.table(bench_desc_10k)[, n := 10000]
))
bench_desc_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest (based on overall median)
bench_desc_all <- order_by_speed(bench_desc_all)

p_desc_box <- ggplot(bench_desc_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(1000, 5000, 10000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Descriptive Tables: desctable() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_desctable.png", p_desc_box, width = 10, height = 5)

desc_scaling <- bench_desc_all[, .(median_time = median(time_ms)), by = .(expr, n)]
p_desc_scale <- ggplot(desc_scaling, aes(x = n, y = median_time, color = expr)) +
    geom_line(linewidth = 1) + geom_point(size = 3) +
    scale_x_log10(labels = scales::comma) + scale_y_log10() +
    scale_color_manual(values = pkg_colors) +
    labs(title = "Descriptive Tables: Scaling Behavior",
         subtitle = "Median execution time vs dataset size (log-log scale)",
         x = "Number of Observations", y = "Median Time (ms)", color = "Package") +
    theme_benchmark()
save_plot("benchmark_desctable_scaling.png", p_desc_scale, width = 8, height = 5)
cat("Saved: benchmark_desctable.png, benchmark_desctable_scaling.png\n")

## =============================================================================
## BENCHMARK 1B: SURVIVAL SUMMARY TABLES
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 1B: SURVIVAL SUMMARY TABLES\n")
cat(strrep("=", 70), "\n\n")

benchmark_survtable <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    
    microbenchmark(
        summata = survtable(data, outcome = "Surv(os_time, os_status)", by = "treatment",
                            times = c(12, 24, 36), probs = 0.5),
        ## Note: add_p() removed due to gtsummary 2.5.0 compatibility issue
        ## where it cannot extract data from survfit objects created within microbenchmark
        gtsummary = {
            fit <- survfit(Surv(os_time, os_status) ~ treatment, data = df)
            tbl_survfit(fit, times = c(12, 24, 36), label_header = "**{time}**")
        },
        manual = {
            fit <- survfit(Surv(os_time, os_status) ~ treatment, data = df)
            summ <- summary(fit, times = c(12, 24, 36))
            quant <- quantile(fit, probs = 0.5)
            list(survival = summ, median = quant)
        },
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Survival Table Benchmarks ---\n\n")
bench_surv_1k <- benchmark_survtable(data_1k, df_1k, n_times = 15)
bench_surv_5k <- benchmark_survtable(data_5k, df_5k, n_times = 10)
bench_surv_10k <- benchmark_survtable(data_10k, df_10k, n_times = 5)

bench_surv_all <- rbindlist(list(
    as.data.table(bench_surv_1k)[, n := 1000],
    as.data.table(bench_surv_5k)[, n := 5000],
    as.data.table(bench_surv_10k)[, n := 10000]
))
bench_surv_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_surv_all <- order_by_speed(bench_surv_all)

p_surv_box <- ggplot(bench_surv_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(1000, 5000, 10000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Survival Tables: survtable() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_survtable.png", p_surv_box, width = 10, height = 5)
cat("Saved: benchmark_survtable.png\n")

## =============================================================================
## BENCHMARK 2: LOGISTIC REGRESSION OUTPUT
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 2: LOGISTIC REGRESSION OUTPUT\n")
cat(strrep("=", 70), "\n\n")

benchmark_logistic <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    predictors <- c("age", "sex", "bmi", "smoking", "hypertension", "diabetes", "treatment", "stage")
    
    microbenchmark(
        summata = fit(data, outcome = "response", predictors = predictors,
                      model_type = "glm", family = "binomial"),
        summata_minimal = fit(data, outcome = "response", predictors = predictors,
                              model_type = "glm", family = "binomial",
                              show_n = FALSE, show_events = FALSE, reference_rows = FALSE,
                              conf_method = "wald", keep_qc_stats = FALSE),
        finalfit = fit2df(glmmulti(df, "response", predictors), estimate_suffix = " (multivariable)"),
        gtsummary = tbl_regression(glm(response ~ age + sex + bmi + smoking + hypertension + 
                                           diabetes + treatment + stage, data = df, family = binomial), exponentiate = TRUE),
        broom = tidy(glm(response ~ age + sex + bmi + smoking + hypertension + 
                             diabetes + treatment + stage, data = df, family = binomial), conf.int = TRUE, exponentiate = TRUE),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Logistic Regression Benchmarks ---\n\n")
bench_log_500 <- benchmark_logistic(data_500, df_500, n_times = 20)
bench_log_1k <- benchmark_logistic(data_1k, df_1k, n_times = 15)
bench_log_5k <- benchmark_logistic(data_5k, df_5k, n_times = 10)
bench_log_10k <- benchmark_logistic(data_10k, df_10k, n_times = 5)

bench_log_all <- rbindlist(list(
    as.data.table(bench_log_500)[, n := 500], as.data.table(bench_log_1k)[, n := 1000],
    as.data.table(bench_log_5k)[, n := 5000], as.data.table(bench_log_10k)[, n := 10000]
))
bench_log_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_log_all <- order_by_speed(bench_log_all)

p_log_box <- ggplot(bench_log_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000, 10000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Logistic Regression: fit() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_logistic.png", p_log_box, width = 10, height = 5)
cat("Saved: benchmark_logistic.png\n")

## =============================================================================
## BENCHMARK 3: LINEAR REGRESSION OUTPUT
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 3: LINEAR REGRESSION OUTPUT\n")
cat(strrep("=", 70), "\n\n")

benchmark_linear <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    predictors <- c("age", "sex", "bmi", "smoking", "hypertension", "diabetes", "treatment", "stage")
    
    microbenchmark(
        summata = fit(data, outcome = "los_days", predictors = predictors, model_type = "lm"),
        summata_minimal = fit(data, outcome = "los_days", predictors = predictors, model_type = "lm",
                              show_n = FALSE, reference_rows = FALSE,
                              conf_method = "wald", keep_qc_stats = FALSE),
        finalfit = fit2df(lmmulti(df, "los_days", predictors)),
        gtsummary = tbl_regression(lm(los_days ~ age + sex + bmi + smoking + hypertension + 
                                          diabetes + treatment + stage, data = df)),
        broom = tidy(lm(los_days ~ age + sex + bmi + smoking + hypertension + 
                            diabetes + treatment + stage, data = df), conf.int = TRUE),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Linear Regression Benchmarks ---\n\n")
bench_lm_500 <- benchmark_linear(data_500, df_500, n_times = 20)
bench_lm_1k <- benchmark_linear(data_1k, df_1k, n_times = 15)
bench_lm_5k <- benchmark_linear(data_5k, df_5k, n_times = 10)
bench_lm_10k <- benchmark_linear(data_10k, df_10k, n_times = 5)

bench_lm_all <- rbindlist(list(
    as.data.table(bench_lm_500)[, n := 500], as.data.table(bench_lm_1k)[, n := 1000],
    as.data.table(bench_lm_5k)[, n := 5000], as.data.table(bench_lm_10k)[, n := 10000]
))
bench_lm_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_lm_all <- order_by_speed(bench_lm_all)

p_lm_box <- ggplot(bench_lm_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000, 10000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Linear Regression: fit() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_linear.png", p_lm_box, width = 10, height = 5)
cat("Saved: benchmark_linear.png\n")

## =============================================================================
## BENCHMARK 4: POISSON REGRESSION OUTPUT
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 4: POISSON REGRESSION OUTPUT\n")
cat(strrep("=", 70), "\n\n")

benchmark_poisson <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    predictors <- c("age", "sex", "bmi", "smoking", "treatment", "stage")
    
    microbenchmark(
        summata = fit(data, outcome = "event_count", predictors = predictors,
                      model_type = "glm", family = "poisson"),
        summata_minimal = fit(data, outcome = "event_count", predictors = predictors,
                              model_type = "glm", family = "poisson",
                              show_n = FALSE, show_events = FALSE, reference_rows = FALSE,
                              conf_method = "wald", keep_qc_stats = FALSE),
        finalfit = fit2df(glm(event_count ~ age + sex + bmi + smoking + treatment + stage,
                              data = df, family = poisson)),
        gtsummary = tbl_regression(glm(event_count ~ age + sex + bmi + smoking + treatment + stage,
                                       data = df, family = poisson), exponentiate = TRUE),
        broom = tidy(glm(event_count ~ age + sex + bmi + smoking + treatment + stage,
                         data = df, family = poisson), conf.int = TRUE, exponentiate = TRUE),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Poisson Regression Benchmarks ---\n\n")
bench_pois_500 <- benchmark_poisson(data_500, df_500, n_times = 20)
bench_pois_1k <- benchmark_poisson(data_1k, df_1k, n_times = 15)
bench_pois_5k <- benchmark_poisson(data_5k, df_5k, n_times = 10)
bench_pois_10k <- benchmark_poisson(data_10k, df_10k, n_times = 5)

bench_pois_all <- rbindlist(list(
    as.data.table(bench_pois_500)[, n := 500], as.data.table(bench_pois_1k)[, n := 1000],
    as.data.table(bench_pois_5k)[, n := 5000], as.data.table(bench_pois_10k)[, n := 10000]
))
bench_pois_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_pois_all <- order_by_speed(bench_pois_all)

p_pois_box <- ggplot(bench_pois_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000, 10000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Poisson Regression: fit() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_poisson.png", p_pois_box, width = 10, height = 5)
cat("Saved: benchmark_poisson.png\n")

## =============================================================================
## BENCHMARK 5: COX REGRESSION
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 5: COX PROPORTIONAL HAZARDS REGRESSION\n")
cat(strrep("=", 70), "\n\n")

benchmark_cox <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    predictors <- c("age", "sex", "bmi", "treatment", "stage")
    
    microbenchmark(
        summata = fit(data, outcome = "Surv(os_time, os_status)", predictors = predictors, model_type = "coxph"),
        summata_minimal = fit(data, outcome = "Surv(os_time, os_status)", predictors = predictors, model_type = "coxph",
                              show_n = FALSE, show_events = FALSE, reference_rows = FALSE,
                              conf_method = "wald", keep_qc_stats = FALSE),
        finalfit = fit2df(coxphmulti(df, "Surv(os_time, os_status)", predictors)),
        gtsummary = tbl_regression(coxph(Surv(os_time, os_status) ~ age + sex + bmi + treatment + stage,
                                         data = df), exponentiate = TRUE),
        broom = tidy(coxph(Surv(os_time, os_status) ~ age + sex + bmi + treatment + stage,
                           data = df), conf.int = TRUE, exponentiate = TRUE),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Cox Regression Benchmarks ---\n\n")
bench_cox_500 <- benchmark_cox(data_500, df_500, n_times = 15)
bench_cox_1k <- benchmark_cox(data_1k, df_1k, n_times = 10)
bench_cox_5k <- benchmark_cox(data_5k, df_5k, n_times = 5)

bench_cox_all <- rbindlist(list(
    as.data.table(bench_cox_500)[, n := 500], as.data.table(bench_cox_1k)[, n := 1000],
    as.data.table(bench_cox_5k)[, n := 5000]
))
bench_cox_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_cox_all <- order_by_speed(bench_cox_all)

p_cox_box <- ggplot(bench_cox_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Cox Regression: fit() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_cox.png", p_cox_box, width = 9, height = 5)
cat("Saved: benchmark_cox.png\n")

## =============================================================================
## BENCHMARK 6: MIXED-EFFECTS MODELS
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 6: MIXED-EFFECTS MODELS\n")
cat(strrep("=", 70), "\n\n")

benchmark_mixed <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    
    microbenchmark(
        summata = fit(data, outcome = "los_days", predictors = c("age", "sex", "treatment", "(1|site)"),
                      model_type = "lmer"),
        summata_minimal = fit(data, outcome = "los_days", predictors = c("age", "sex", "treatment", "(1|site)"),
                              model_type = "lmer", show_n = FALSE, show_events = FALSE, reference_rows = FALSE,
                              conf_method = "wald", keep_qc_stats = FALSE),
        finalfit = fit2df(lmmixed(df, "los_days", c("age", "sex", "treatment"), "(1|site)")),
        gtsummary = tbl_regression(lmer(los_days ~ age + sex + treatment + (1|site), data = df)),
        broom = tidy(lmer(los_days ~ age + sex + treatment + (1|site), data = df), conf.int = TRUE),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Mixed-Effects Benchmarks ---\n\n")
bench_mixed_500 <- benchmark_mixed(data_500, df_500, n_times = 15)
bench_mixed_1k <- benchmark_mixed(data_1k, df_1k, n_times = 10)
bench_mixed_5k <- benchmark_mixed(data_5k, df_5k, n_times = 5)

bench_mixed_all <- rbindlist(list(
    as.data.table(bench_mixed_500)[, n := 500], as.data.table(bench_mixed_1k)[, n := 1000],
    as.data.table(bench_mixed_5k)[, n := 5000]
))
bench_mixed_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_mixed_all <- order_by_speed(bench_mixed_all)

p_mixed_box <- ggplot(bench_mixed_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Mixed-Effects Models: fit() with lmer vs Alternatives",
         subtitle = "Linear mixed model with random intercepts (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_mixed.png", p_mixed_box, width = 9, height = 5)
cat("Saved: benchmark_mixed.png\n")

## =============================================================================
## BENCHMARK 7: UNIVARIABLE SCREENING
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 7: UNIVARIABLE SCREENING (Multiple Predictors)\n")
cat(strrep("=", 70), "\n\n")

benchmark_uniscreen <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    predictors <- c("age", "sex", "bmi", "race", "smoking", "hypertension", 
                    "diabetes", "treatment", "stage", "grade", 
                    "biomarker1", "biomarker2", "creatinine", "hemoglobin")
    
    microbenchmark(
        summata = uniscreen(data, outcome = "response", predictors = predictors,
                            model_type = "glm", family = "binomial"),
        summata_minimal = uniscreen(data, outcome = "response", predictors = predictors,
                                    model_type = "glm", family = "binomial",
                                    show_n = FALSE, show_events = FALSE, reference_rows = FALSE,
                                    conf_method = "wald"),
        finalfit = fit2df(glmuni(df, "response", predictors), estimate_suffix = " (univariable)"),
        gtsummary = tbl_uvregression(df[, c("response", predictors)], method = glm, y = response,
                                     method.args = list(family = binomial), exponentiate = TRUE),
        arsenal = modelsum(response ~ age + sex + bmi + race + smoking + hypertension + 
                               diabetes + treatment + stage + grade + biomarker1 + biomarker2 + creatinine + hemoglobin,
                           data = df, family = binomial),
        broom_loop = rbindlist(lapply(predictors, function(pred) {
            tidy(glm(as.formula(paste("response ~", pred)), data = df, family = binomial),
                 conf.int = TRUE, exponentiate = TRUE)
        })),
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Univariable Screening Benchmarks ---\n\n")
bench_uni_500 <- benchmark_uniscreen(data_500, df_500, n_times = 15)
bench_uni_1k <- benchmark_uniscreen(data_1k, df_1k, n_times = 10)
bench_uni_5k <- benchmark_uniscreen(data_5k, df_5k, n_times = 5)

bench_uni_all <- rbindlist(list(
    as.data.table(bench_uni_500)[, n := 500], as.data.table(bench_uni_1k)[, n := 1000],
    as.data.table(bench_uni_5k)[, n := 5000]
))
bench_uni_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_uni_all <- order_by_speed(bench_uni_all)

p_uni_box <- ggplot(bench_uni_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Univariable Screening: uniscreen() vs Alternatives",
         subtitle = "Screening 14 predictors (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_uniscreen.png", p_uni_box, width = 10, height = 5)
cat("Saved: benchmark_uniscreen.png\n")

## =============================================================================
## BENCHMARK 8: COMPLETE WORKFLOW (Univariable + Multivariable)
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 8: COMPLETE ANALYSIS WORKFLOW\n")
cat(strrep("=", 70), "\n\n")

benchmark_workflow <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    predictors <- c("age", "sex", "bmi", "smoking", "hypertension", "diabetes", "treatment", "stage")
    
    microbenchmark(
        summata = fullfit(data, outcome = "response", predictors = predictors, method = "all",
                          model_type = "glm", family = "binomial", columns = "both"),
        summata_minimal = fullfit(data, outcome = "response", predictors = predictors, method = "all",
                                  model_type = "glm", family = "binomial", columns = "both",
                                  show_n = FALSE, show_events = FALSE, reference_rows = FALSE,
                                  conf_method = "wald"),
        finalfit = finalfit(df, dependent = "response", explanatory = predictors),
        gtsummary = {
            uni_table <- tbl_uvregression(df[, c("response", predictors)], method = glm, y = response,
                                          method.args = list(family = binomial), exponentiate = TRUE)
            multi_model <- glm(response ~ age + sex + bmi + smoking + hypertension + 
                                   diabetes + treatment + stage, data = df, family = binomial)
            multi_table <- tbl_regression(multi_model, exponentiate = TRUE)
            tbl_merge(tbls = list(uni_table, multi_table), tab_spanner = c("Univariable", "Multivariable"))
        },
        manual = {
            uni_dt <- rbindlist(lapply(predictors, function(pred) {
                res <- tidy(glm(as.formula(paste("response ~", pred)), data = df, family = binomial),
                            conf.int = TRUE, exponentiate = TRUE)
                res$predictor <- pred
                res
            }))
            multi_formula <- as.formula(paste("response ~", paste(predictors, collapse = " + ")))
            multi_dt <- as.data.table(tidy(glm(multi_formula, data = df, family = binomial),
                                           conf.int = TRUE, exponentiate = TRUE))
            list(univariable = uni_dt, multivariable = multi_dt)
        },
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Complete Workflow Benchmarks ---\n\n")
bench_wf_500 <- benchmark_workflow(data_500, df_500, n_times = 15)
bench_wf_1k <- benchmark_workflow(data_1k, df_1k, n_times = 10)
bench_wf_5k <- benchmark_workflow(data_5k, df_5k, n_times = 5)

bench_wf_all <- rbindlist(list(
    as.data.table(bench_wf_500)[, n := 500], as.data.table(bench_wf_1k)[, n := 1000],
    as.data.table(bench_wf_5k)[, n := 5000]
))
bench_wf_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_wf_all <- order_by_speed(bench_wf_all)

p_wf_box <- ggplot(bench_wf_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Complete Workflow: fullfit() vs Alternatives",
         subtitle = "Combined univariable + multivariable analysis (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_workflow.png", p_wf_box, width = 10, height = 5)
cat("Saved: benchmark_workflow.png\n")

## =============================================================================
## BENCHMARK 9: FOREST PLOTS
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("BENCHMARK 9: FOREST PLOT GENERATION\n")
cat(strrep("=", 70), "\n\n")

benchmark_forest <- function(data, df, n_times = 10) {
    cat(sprintf("Testing with %s observations...\n", format(nrow(data), big.mark = ",")))
    cox_model <- coxph(Surv(os_time, os_status) ~ age + sex + bmi + treatment + stage, data = df)
    
    microbenchmark(
        summata = coxforest(x = cox_model, data = df, title = "Cox Regression Results"),
        survminer = ggforest(cox_model, data = df),
        manual = {
            coef_data <- as.data.table(tidy(cox_model, conf.int = TRUE, exponentiate = TRUE))
            coef_data <- coef_data[term != "(Intercept)"]
            ggplot(coef_data, aes(x = estimate, y = term)) +
                geom_point() +
                geom_errorbarh(aes(xmin = conf.low, xmax = conf.high), height = 0.2) +
                geom_vline(xintercept = 1, linetype = "dashed") +
                scale_x_log10() + theme_minimal()
        },
        times = n_times, unit = "ms"
    )
}

cat("\n--- Running Forest Plot Benchmarks ---\n\n")
bench_forest_500 <- benchmark_forest(data_500, df_500, n_times = 15)
bench_forest_1k <- benchmark_forest(data_1k, df_1k, n_times = 10)
bench_forest_5k <- benchmark_forest(data_5k, df_5k, n_times = 5)

bench_forest_all <- rbindlist(list(
    as.data.table(bench_forest_500)[, n := 500], as.data.table(bench_forest_1k)[, n := 1000],
    as.data.table(bench_forest_5k)[, n := 5000]
))
bench_forest_all[, time_ms := time / 1e6]

## Order packages from fastest to slowest
bench_forest_all <- order_by_speed(bench_forest_all)

p_forest_box <- ggplot(bench_forest_all, aes(x = expr, y = time_ms, fill = expr)) +
    geom_boxplot(alpha = 0.7, outlier.size = 1) +
    facet_wrap(~ factor(n, labels = paste0("n = ", scales::comma(c(500, 1000, 5000)))), 
               scales = "free_y", nrow = 1) +
    scale_y_log10() + scale_fill_manual(values = pkg_colors) +
    labs(title = "Forest Plots: coxforest() vs Alternatives",
         subtitle = "Execution time in milliseconds (log scale)",
         x = NULL, y = "Time (ms)", fill = "Package") +
    theme_benchmark() + guides(fill = guide_legend(nrow = 1))
save_plot("benchmark_forest.png", p_forest_box, width = 9, height = 5)
cat("Saved: benchmark_forest.png\n")

## =============================================================================
## SUMMARY VISUALIZATION: RELATIVE PERFORMANCE
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("GENERATING SUMMARY VISUALIZATIONS\n")
cat(strrep("=", 70), "\n\n")

## Combine all benchmarks with labels in vignette order:
## Descriptive tables, Survival tables, Univariable screen, Logistic regression,
## Poisson regression, Linear regression, Cox regression, Mixed effects,
## Complete workflow, Forest plots
all_benchmarks <- rbindlist(list(
    bench_desc_all[, benchmark := "Descriptive Tables"],
    bench_surv_all[, benchmark := "Survival Tables"],
    bench_uni_all[, benchmark := "Univariable Screening"],
    bench_log_all[, benchmark := "Logistic Regression"],
    bench_pois_all[, benchmark := "Poisson Regression"],
    bench_lm_all[, benchmark := "Linear Regression"],
    bench_cox_all[, benchmark := "Cox Regression"],
    bench_mixed_all[, benchmark := "Mixed-Effects"],
    bench_wf_all[, benchmark := "Complete Workflow"],
    bench_forest_all[, benchmark := "Forest Plots"]
), fill = TRUE)

## Set benchmark factor order to match vignette
benchmark_order <- c("Descriptive Tables", "Survival Tables", "Univariable Screening",
                     "Logistic Regression", "Poisson Regression", "Linear Regression",
                     "Cox Regression", "Mixed-Effects", "Complete Workflow", "Forest Plots")
all_benchmarks[, benchmark := factor(benchmark, levels = benchmark_order)]

## Exclude summata_minimal from speedup calculations
speedup_data <- all_benchmarks[!expr %in% c("summata", "summata_minimal")]
summata_times <- all_benchmarks[expr == "summata", .(summata_time = median(time_ms)), by = .(benchmark, n)]
speedup_summary <- speedup_data[, .(median_ms = median(time_ms)), by = .(benchmark, n, expr)]
speedup_summary <- speedup_summary[summata_times, on = .(benchmark, n)]
speedup_summary[, speedup := median_ms / summata_time]

p_speedup <- ggplot(speedup_summary, aes(x = factor(n), y = speedup, fill = expr)) +
    geom_col(position = "dodge", alpha = 0.8) +
    geom_hline(yintercept = 1, linetype = "dashed", color = "red", linewidth = 0.5) +
    facet_wrap(~ benchmark, scales = "free_y", ncol = 2) +
    scale_fill_manual(values = pkg_colors) +
    labs(title = "Relative Performance: Time Relative to summata",
         subtitle = "Values > 1 indicate summata is faster (red line = equal performance)",
         x = "Dataset Size (n)", y = "Relative Time", fill = "Package") +
    theme_benchmark() +
    theme(legend.position = "bottom") +
    guides(fill = guide_legend(nrow = 2))
save_plot("benchmark_speedup.png", p_speedup, width = 10, height = 14)
cat("Saved: benchmark_speedup.png\n")

## Speedup relative to summata_minimal (Wald CIs, no QC stats, no n/Events)
## This shows how alternatives compare against summata's fastest configuration
speedup_min_data <- all_benchmarks[!expr %in% c("summata", "summata_minimal")]
summata_min_times <- all_benchmarks[expr == "summata_minimal", 
                                    .(summata_min_time = median(time_ms)), by = .(benchmark, n)]
speedup_min_summary <- speedup_min_data[, .(median_ms = median(time_ms)), by = .(benchmark, n, expr)]
speedup_min_summary <- speedup_min_summary[summata_min_times, on = .(benchmark, n)]
speedup_min_summary <- speedup_min_summary[!is.na(summata_min_time) & !is.na(median_ms)]
speedup_min_summary[, speedup := median_ms / summata_min_time]

p_speedup_min <- ggplot(speedup_min_summary, aes(x = factor(n), y = speedup, fill = expr)) +
    geom_col(position = "dodge", alpha = 0.8) +
    geom_hline(yintercept = 1, linetype = "dashed", color = "red", linewidth = 0.5) +
    facet_wrap(~ benchmark, scales = "free_y", ncol = 2) +
    scale_fill_manual(values = pkg_colors) +
    labs(title = "Relative Performance: Time Relative to summata_minimal",
         subtitle = "summata_minimal uses Wald CIs, no QC stats, no n/Events (red line = equal)",
         x = "Dataset Size (n)", y = "Relative Time", fill = "Package") +
    theme_benchmark() +
    theme(legend.position = "bottom") +
    guides(fill = guide_legend(nrow = 2))
save_plot("benchmark_speedup_minimal.png", p_speedup_min, width = 10, height = 14)
cat("Saved: benchmark_speedup_minimal.png\n")

ff_comparison <- all_benchmarks[expr %in% c("summata", "finalfit"), 
                                .(median_time = median(time_ms)), by = .(benchmark, expr, n)]

p_ff <- ggplot(ff_comparison, aes(x = factor(n), y = median_time, fill = expr)) +
    geom_col(position = "dodge", alpha = 0.8) +
    facet_wrap(~ benchmark, scales = "free_y", ncol = 2) +
    scale_fill_manual(values = c("summata" = "#FC8D62", "finalfit" = "#5597DF")) +
    labs(title = "summata vs finalfit: Direct Comparison",
         subtitle = "summata builds upon finalfit with performance optimizations",
         x = "Dataset Size (n)", y = "Median Time (ms)", fill = "Package") +
    theme_benchmark() +
    theme(legend.position = "bottom")
save_plot("benchmark_summata_vs_finalfit.png", p_ff, width = 10, height = 14)
cat("Saved: benchmark_summata_vs_finalfit.png\n")

## =============================================================================
## SUMMARY TABLES
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("FINAL SUMMARY TABLES\n")
cat(strrep("=", 70), "\n\n")

final_summary <- dcast(
    all_benchmarks[, .(median_ms = round(median(time_ms), 2)), by = .(benchmark, n, expr)],
    benchmark + n ~ expr, value.var = "median_ms"
)
setorder(final_summary, benchmark, n)
cat("Median execution time in milliseconds:\n\n")
print(final_summary)

## Exclude summata_minimal from summata speedup table
speedup_for_table <- all_benchmarks[!expr %in% c("summata_minimal")]
speedup_table <- dcast(
    speedup_for_table[, .(median_ms = median(time_ms)), by = .(benchmark, n, expr)][
      , summata_time := median_ms[expr == "summata"], by = .(benchmark, n)][
      , speedup := round(median_ms / summata_time, 2)],
    benchmark + n ~ expr, value.var = "speedup"
)
setorder(speedup_table, benchmark, n)
cat("\n\nSpeedup factors (alternative / summata, >1 means summata is faster):\n\n")
print(speedup_table)

## Speedup relative to summata_minimal
speedup_min_for_table <- all_benchmarks[!expr %in% c("summata", "summata_minimal")]
summata_min_for_table <- all_benchmarks[expr == "summata_minimal",
                                        .(summata_min_time = median(time_ms)), by = .(benchmark, n)]
speedup_min_table_data <- speedup_min_for_table[, .(median_ms = median(time_ms)), by = .(benchmark, n, expr)]
speedup_min_table_data <- speedup_min_table_data[summata_min_for_table, on = .(benchmark, n)]
speedup_min_table_data <- speedup_min_table_data[!is.na(summata_min_time) & !is.na(median_ms)]
speedup_min_table_data[, speedup := round(median_ms / summata_min_time, 2)]

## Also include summata (default) vs summata_minimal
summata_vs_min <- all_benchmarks[expr == "summata",
                                 .(median_ms = median(time_ms)), by = .(benchmark, n, expr)]
summata_vs_min <- summata_vs_min[summata_min_for_table, on = .(benchmark, n)]
summata_vs_min <- summata_vs_min[!is.na(summata_min_time) & !is.na(median_ms)]
summata_vs_min[, speedup := round(median_ms / summata_min_time, 2)]
speedup_min_table_data <- rbindlist(list(summata_vs_min, speedup_min_table_data), use.names = TRUE)

speedup_min_table <- dcast(speedup_min_table_data,
                           benchmark + n ~ expr, value.var = "speedup")
setorder(speedup_min_table, benchmark, n)
cat("\n\nSpeedup factors (alternative / summata_minimal, >1 means summata_minimal is faster):\n\n")
print(speedup_min_table)

fwrite(final_summary, "benchmark_summary_times.csv")
fwrite(speedup_table, "benchmark_summary_speedup.csv")
fwrite(speedup_min_table, "benchmark_summary_speedup_minimal.csv")
cat("\nSaved: benchmark_summary_times.csv, benchmark_summary_speedup.csv, benchmark_summary_speedup_minimal.csv\n")

## =============================================================================
## SESSION INFO
## =============================================================================

cat("\n", strrep("=", 70), "\n")
cat("SESSION INFORMATION\n")
cat(strrep("=", 70), "\n\n")
sessionInfo()

cat("\n\nBenchmarking complete!\n\nOutput files:\n")
cat(" - benchmark_desctable.png\n - benchmark_desctable_scaling.png\n")
cat(" - benchmark_survtable.png\n")
cat(" - benchmark_logistic.png\n - benchmark_linear.png\n - benchmark_poisson.png\n")
cat(" - benchmark_cox.png\n - benchmark_mixed.png\n - benchmark_uniscreen.png\n")
cat(" - benchmark_workflow.png\n - benchmark_forest.png\n")
cat(" - benchmark_speedup.png\n - benchmark_speedup_minimal.png\n")
cat(" - benchmark_summata_vs_finalfit.png\n")
cat(" - benchmark_summary_times.csv\n - benchmark_summary_speedup.csv\n")
cat(" - benchmark_summary_speedup_minimal.csv\n")

### * Main function

#' Export Table to Rich Text Format (RTF)
#'
#' Converts a data frame, data.table, or matrix to a Rich Text Format (\code{.rtf}) 
#' document using the \pkg{flextable} and \pkg{officer} packages. Creates 
#' widely compatible tables with extensive formatting options. RTF files can be 
#' opened and edited in Microsoft Word, LibreOffice, WordPad, and many other word 
#' processors. Particularly useful for regulatory submissions, cross-platform 
#' compatibility, and when maximum editability is required.
#'
#' @param table Data frame, data.table, or matrix to export. Can be output from 
#'   \code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
#'   \code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.
#'   
#' @param file Character string specifying the output RTF filename. Must have 
#'   \code{.rtf} extension. Example: \code{"results.rtf"}, \code{"Table1.rtf"}.
#'   
#' @param caption Character string. Optional caption displayed above the table 
#'   in the RTF document. Default is \code{NULL}.
#'   
#' @param font_size Numeric. Base font size in points for table content. 
#'   Default is 8. Typical range: 8-12 points. Headers use slightly larger size.
#'   
#' @param font_family Character string. Font family name for the table. Must be 
#'   a font installed on the system. Default is \code{"Arial"}. Common options: 
#'   \code{"Times New Roman"}, \code{"Calibri"}, \code{"Courier New"}.
#'   
#' @param format_headers Logical. If \code{TRUE}, formats column headers by 
#'   italicizing statistical notation ("\emph{n}", "\emph{p}"), converting underscores 
#'   to spaces, and improving readability. Default is \code{TRUE}.
#'   
#' @param bold_significant Logical. If \code{TRUE}, applies bold formatting to 
#'   \emph{p}-values below the significance threshold. Makes significant results stand 
#'   out. Default is \code{TRUE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. Default is \code{FALSE}.
#'   
#' @param p_threshold Numeric. Threshold for bold \emph{p}-value formatting. Only 
#'   used when \code{bold_significant = TRUE}. Default is 0.05.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels under 
#'   their parent variable using horizontal spacing, creating hierarchical display. 
#'   Useful for categorical variables in regression tables. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses table by showing 
#'   only essential rows (single row for continuous, non-reference for binary). 
#'   Automatically sets \code{indent_groups = TRUE}. Significantly reduces table 
#'   height. Default is \code{FALSE}.
#'   
#' @param condense_quantitative Logical. If \code{TRUE}, condenses continuous 
#'   and survival variables into single rows while preserving all categorical 
#'   variable rows (including binary). Only applies to descriptive tables from 
#'   \code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
#'   \code{condense_table}, this does not collapse binary categorical variables. 
#'   Default is \code{FALSE}.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating row shading 
#'   to different variables (not individual rows) for visual grouping. 
#'   Default is \code{FALSE}.
#'   
#' @param dark_header Logical. If \code{TRUE}, creates a dark background with 
#'   light text for the header row, providing strong visual contrast. 
#'   Default is \code{FALSE}.
#'   
#' @param paper Character string specifying paper size:
#'   \itemize{
#'     \item \code{"letter"} - US Letter (8.5" × 11") [default]
#'     \item \code{"a4"} - A4 (210 mm × 297 mm)
#'     \item \code{"legal"} - US Legal (8.5" × 14")
#'   }
#'   
#' @param orientation Character string specifying page orientation:
#'   \itemize{
#'     \item \code{"portrait"} - Vertical [default]
#'     \item \code{"landscape"} - Horizontal (for wide tables)
#'   }
#'   
#' @param width Numeric. Table width in inches. If \code{NULL} (default), 
#'   automatically fits to content and page width. Specify to control exactly.
#'   
#' @param align Character vector specifying column alignment for each column. 
#'   Options: \code{"left"}, \code{"center"}, or \code{"right"}. If \code{NULL} 
#'   (default), automatically determines based on content (text left, numbers right).
#'   Example: \code{c("left", "left", "center", "right", "right")}.
#'   
#' @param return_ft Logical. If \code{TRUE}, returns the \pkg{flextable} object 
#'   directly for further customization. If \code{FALSE} (default), returns 
#'   invisibly with flextable as attribute. See Details for usage. 
#'   Default is \code{FALSE}.
#'   
#' @param ... Additional arguments (currently unused, reserved for future extensions).
#'
#' @param quiet Logical. Suppress progress and confirmation messages, such
#'   as the notice reporting the file written. Errors and warnings are
#'   unaffected. Default is \code{FALSE}.
#'
#' @return Behavior depends on \code{return_ft}:
#'   \describe{
#'     \item{\code{return_ft = FALSE}}{Invisibly returns a list with components:
#'       \itemize{
#'         \item \code{file} - Path to created file
#'         \item \code{caption} - Caption text (if provided)
#'       }
#'       The \pkg{flextable} object is accessible via \code{attr(result, "flextable")}
#'     }
#'     \item{\code{return_ft = TRUE}}{Directly returns the \pkg{flextable} object for 
#'       immediate further customization}
#'   }
#'   
#'   In both cases, creates a .rtf file at the specified location.
#'
#' @details
#' \strong{Package Requirements:}
#' 
#' This function requires:
#' \itemize{
#'   \item \strong{\pkg{flextable}} - For creating formatted tables
#'   \item \strong{\pkg{officer}} - For RTF document generation
#' }
#' 
#' Install if needed:
#' \preformatted{
#' install.packages(c("flextable", "officer"))
#' }
#' 
#' \strong{RTF Format Advantages:}
#' 
#' RTF (Rich Text Format) is a universal document format with several advantages:
#' \itemize{
#'   \item \strong{Maximum compatibility} - Opens in virtually all word processors
#'   \item \strong{Cross-platform} - Works on Windows, Mac, Linux without conversion
#'   \item \strong{Fully editable} - Native text format, not embedded objects
#'   \item \strong{Lightweight} - Smaller file sizes than DOCX
#'   \item \strong{Regulatory compliance} - Widely accepted for submissions (FDA, EMA)
#'   \item \strong{Long-term accessibility} - Simple text-based format
#'   \item \strong{Version control friendly} - Text-based, works with diff tools
#' }
#' 
#' Applications that can open RTF files:
#' \itemize{
#'   \item Microsoft Word (Windows, Mac)
#'   \item LibreOffice Writer
#'   \item Apache OpenOffice Writer
#'   \item WordPad (Windows built-in)
#'   \item TextEdit (Mac built-in)
#'   \item Google Docs (with import)
#'   \item Pages (Mac)
#'   \item Many other word processors
#' }
#' 
#' \strong{Output Features:}
#' 
#' The generated RTF document contains:
#' \itemize{
#'   \item Fully editable table (native RTF table, not image)
#'   \item Professional typography and spacing
#'   \item Proper page setup (size, orientation, margins)
#'   \item Caption (if provided) as separate paragraph above table
#'   \item All formatting preserved but editable
#'   \item Compatible with RTF 1.5 specification
#' }
#' 
#' \strong{Further Customization:}
#' 
#' For programmatic customization beyond the built-in options, access the 
#' \code{flextable} object:
#' 
#' \emph{Method 1: Via attribute (default)}
#' \preformatted{
#' result <- table2rtf(table, "output.rtf")
#' ft <- attr(result, "flextable")
#' 
#' # Customize flextable
#' ft <- flextable::bold(ft, i = 1, j = 1, part = "body")
#' ft <- flextable::color(ft, i = 2, j = 3, color = "red")
#' 
#' # Re-save if needed
#' flextable::save_as_rtf(ft, path = "customized.rtf")
#' }
#' 
#' \emph{Method 2: Direct return}
#' \preformatted{
#' ft <- table2rtf(table, "output.rtf", return_ft = TRUE)
#' 
#' # Customize immediately
#' ft <- flextable::bg(ft, bg = "yellow", part = "header")
#' ft <- flextable::autofit(ft)
#' 
#' # Save to new file
#' flextable::save_as_rtf(ft, path = "custom.rtf")
#' }
#' 
#' \strong{Page Layout:}
#' 
#' The function automatically sets up the RTF document with:
#' \itemize{
#'   \item Specified paper size and orientation
#'   \item Standard margins (1 inch by default)
#'   \item Table positioned at document start
#'   \item Left-aligned table placement
#' }
#' 
#' For landscape orientation:
#' \itemize{
#'   \item Automatically swaps page dimensions
#'   \item Applies landscape property
#'   \item Useful for wide tables with many columns
#' }
#' 
#' \strong{Table Width Management:}
#' 
#' Width behavior:
#' \itemize{
#'   \item \code{width = NULL} - Auto-fits to content and page width
#'   \item \code{width = 6} - Exactly 6 inches wide
#'   \item Width distributed evenly across columns by default
#'   \item Can adjust individual column widths in word processor after creation
#' }
#' 
#' For very wide tables:
#' \enumerate{
#'   \item Use \code{orientation = "landscape"}
#'   \item Use \code{paper = "legal"} for extra width
#'   \item Reduce \code{font_size}
#'   \item Use \code{condense_table = TRUE}
#'   \item Consider breaking across multiple tables
#' }
#' 
#' \strong{Typography:}
#' 
#' The function applies professional typography:
#' \itemize{
#'   \item Column headers: Bold, slightly larger font
#'   \item Body text: Regular weight, specified font size
#'   \item Numbers: Right-aligned for easy comparison
#'   \item Text: Left-aligned for readability
#'   \item Consistent spacing: Adequate padding in cells
#'   \item Statistical notation: Italicized appropriately
#' }
#' 
#' @seealso
#' \code{\link{tablesave}} for saving in the format given by the file extension,
#' \code{\link{table2docx}} for Word documents,
#' \code{\link{table2pptx}} for PowerPoint slides,
#' \code{\link{table2pdf}} for PDF output,
#' \code{\link{table2html}} for HTML tables,
#' \code{\link{table2tex}} for LaTeX output,
#' \code{\link[flextable]{flextable}} for the underlying table object,
#' \code{\link[flextable]{save_as_rtf}} for direct RTF export
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#'
#' # Create example table
#' results <- fit(
#'    data = clintrial,
#'    outcome = "os_status",
#'    predictors = c("age", "sex", "treatment", "stage"),
#'    labels = clintrial_labels
#' )
#'
#' # Example 1: Basic RTF export
#' if (requireNamespace("flextable", quietly = TRUE)) {
#'   table2rtf(results, file.path(tempdir(), "results.rtf"))
#' }
#'
#' \donttest{
#' old_width <- options(width = 180)
#' # Example 2: With caption
#' table2rtf(results, file.path(tempdir(), "captioned.rtf"),
#'        caption = "Table 1: Multivariable Logistic Regression Results")
#' 
#' # Example 3: Landscape orientation for wide tables
#' table2rtf(results, file.path(tempdir(), "wide.rtf"),
#'        orientation = "landscape")
#' 
#' # Example 4: Custom font and size
#' table2rtf(results, file.path(tempdir(), "custom_font.rtf"),
#'        font_family = "Times New Roman",
#'        font_size = 11)
#' 
#' # Example 5: Hierarchical display
#' table2rtf(results, file.path(tempdir(), "indented.rtf"),
#'        indent_groups = TRUE)
#' 
#' # Example 6: Condensed table
#' table2rtf(results, file.path(tempdir(), "condensed.rtf"),
#'        condense_table = TRUE)
#' 
#' # Example 7: With zebra stripes
#' table2rtf(results, file.path(tempdir(), "striped.rtf"),
#'        zebra_stripes = TRUE)
#' 
#' # Example 8: Dark header style
#' table2rtf(results, file.path(tempdir(), "dark.rtf"),
#'        dark_header = TRUE)
#' 
#' # Example 9: A4 paper for international submissions
#' table2rtf(results, file.path(tempdir(), "a4.rtf"),
#'        paper = "a4")
#' 
#' # Example 10: Get flextable for customization
#' result <- table2rtf(results, file.path(tempdir(), "base.rtf"))
#' ft <- attr(result, "flextable")
#' 
#' # Customize the flextable
#' ft <- flextable::bold(ft, i = 1, part = "body")
#' ft <- flextable::color(ft, j = "p-value", color = "blue")
#' 
#' # Re-save
#' flextable::save_as_rtf(ft, path = file.path(tempdir(), "customized.rtf"))
#' 
#' # Example 11: Direct flextable return
#' ft <- table2rtf(results, file.path(tempdir(), "direct.rtf"), return_ft = TRUE)
#' ft <- flextable::bg(ft, bg = "yellow", part = "header")
#' 
#' # Example 12: Regulatory submission table
#' table2rtf(results, file.path(tempdir(), "submission.rtf"),
#'        caption = "Table 2: Adjusted Odds Ratios for Mortality",
#'        font_family = "Times New Roman",
#'        font_size = 10,
#'        indent_groups = TRUE,
#'        zebra_stripes = FALSE,
#'        bold_significant = TRUE)
#' 
#' # Example 13: Custom column alignment
#' table2rtf(results, file.path(tempdir(), "aligned.rtf"),
#'        align = c("left", "left", "center", "right", "right"))
#' 
#' # Example 14: Disable significance bolding
#' table2rtf(results, file.path(tempdir(), "no_bold.rtf"),
#'        bold_significant = FALSE)
#' 
#' # Example 15: Stricter significance threshold
#' table2rtf(results, file.path(tempdir(), "strict.rtf"),
#'        bold_significant = TRUE,
#'        p_threshold = 0.01)
#' 
#' # Example 16: Descriptive statistics for baseline characteristics
#' desc <- desctable(clintrial, by = "treatment",
#'    variables = c("age", "sex", "bmi", "stage"), labels = clintrial_labels)
#' 
#' table2rtf(desc, file.path(tempdir(), "baseline.rtf"),
#'        caption = "Table 1: Baseline Patient Characteristics",
#'        zebra_stripes = TRUE)
#' 
#' # Example 17: Clinical trial efficacy table
#' table2rtf(results, file.path(tempdir(), "efficacy.rtf"),
#'        caption = "Table 3: Primary Efficacy Analysis - Intent to Treat Population",
#'        font_family = "Courier New",  # Monospace for alignment
#'        paper = "letter",
#'        orientation = "landscape",
#'        condense_table = TRUE)
#'
#' options(old_width)
#' }
#' @family export functions
#' @export
table2rtf <- function(table,
                      file,
                    caption = NULL,
                    font_size = 8,
                    font_family = "Arial",
                    format_headers = TRUE,
                    bold_significant = TRUE,
                    bold_variables = FALSE,
                    p_threshold = 0.05,
                    indent_groups = FALSE,
                    condense_table = FALSE,
                    condense_quantitative = FALSE,
                    zebra_stripes = FALSE,
                    dark_header = FALSE,
                    paper = "letter",
                    orientation = "portrait",
                    width = NULL,
                    align = NULL,
                    return_ft = FALSE,
                    quiet = FALSE,
                    ...) {
    
    ## Warn about unused arguments
    dots <- list(...)
    if (length(dots) > 0) {
        warning("Unknown arguments ignored: ", paste(names(dots), collapse = ", "), 
                call. = FALSE)
    }
    
    if (!requireNamespace("flextable", quietly = TRUE)) {
        stop("Package 'flextable' required. Install with: install.packages('flextable')")
    }
    if (!requireNamespace("officer", quietly = TRUE)) {
        stop("Package 'officer' required. Install with: install.packages('officer')")
    }
    
    if (!grepl("\\.rtf$", tolower(file))) {
        stop("File must have .rtf extension")
    }
    
    ## Validate paper and orientation
    paper <- match.arg(paper, c("letter", "a4", "legal"))
    orientation <- match.arg(orientation, c("portrait", "landscape"))
    
    ## Process table using shared function
    result <- process_table_for_flextable(
        table = table,
        caption = caption,
        font_size = font_size,
        font_family = font_family,
        format_headers = format_headers,
        bold_significant = bold_significant,
        p_threshold = p_threshold,
        indent_groups = indent_groups,
        condense_table = condense_table,
        condense_quantitative = condense_quantitative,
        zebra_stripes = zebra_stripes,
        dark_header = dark_header,
        bold_variables = bold_variables,
        paper = paper,
        orientation = orientation,
        width = width,
        align = align
    )
    
    ## Extract flextable for easy access
    ft <- result$ft
    
    ## Define page dimensions based on paper size
    page_width <- if(paper == "a4") 8.27 else 8.5
    page_height <- if(paper == "a4") 11.69 else if(paper == "legal") 14 else 11
    
    if (orientation == "landscape") {
        temp <- page_width
        page_width <- page_height
        page_height <- temp
    }
    
    ## Set page size properties for the flextable
    ## RTF uses page_size() from officer
    ps <- officer::page_size(width = page_width, 
                             height = page_height, 
                             orient = orientation)
    
    pm <- officer::page_mar(bottom = 1, top = 1, right = 1, left = 1)
    
    ## Create section properties for RTF
    sect_properties <- officer::prop_section(
                                    page_size = ps,
                                    page_margins = pm,
                                    type = "continuous"
                                )
    
    ## Add section properties to flextable
    ## This helps maintain consistent page layout in RTF
    ft <- flextable::set_table_properties(ft, layout = "autofit")
    
    ## Save as RTF with caption if provided
    if (!is.null(result$caption)) {
        ## Save flextable to temporary RTF to get complete, valid RTF structure
        temp_rtf <- tempfile(fileext = ".rtf")
        flextable::save_as_rtf(ft, path = temp_rtf)
        
        ## Read the entire RTF content
        rtf_text <- paste(readLines(temp_rtf, warn = FALSE), collapse = "\n")
        
        ## Find the first \trowd (table row definition)
        ## Insert the caption before this position
        trowd_pos <- regexpr("\\\\trowd", rtf_text)
        
        if (trowd_pos[1] > 0) {
            ## Split at the table start
            before_table <- substr(rtf_text, 1, trowd_pos[1] - 1)
            table_and_after <- substr(rtf_text, trowd_pos[1], nchar(rtf_text))
            
            ## Create caption paragraph with proper RTF formatting
            ## Use \qc for centered, or \ql for left-aligned
            caption_rtf <- sprintf(
                "\\pard\\ql\\f0\\fs%d %s\\par\n\\par\n",
                font_size * 2,  ## RTF uses half-points
                result$caption
            )
            
            ## Combine: before_table + caption + table_and_after
            rtf_final <- paste0(before_table, caption_rtf, table_and_after)
            
        } else {
            ## Fallback if \trowd does not exist
            ## Try to insert after the last closing brace of header tables
            warning("Could not find table start, attempting fallback caption insertion")
            
            ## Find the position after {\colortable...} and {\fonttable...}
            ## Look for the last } before \viewkind or similar
            insert_pattern <- "\\}\\s*\\\\viewkind"
            insert_pos <- regexpr(insert_pattern, rtf_text)
            
            if (insert_pos[1] > 0) {
                ## Insert after the }
                pos <- insert_pos[1]
                caption_rtf <- sprintf(
                    "\n\\pard\\ql\\f0\\fs%d %s\\par\n\\par\n",
                    font_size * 2,
                    result$caption
                )
                rtf_final <- paste0(
                    substr(rtf_text, 1, pos),
                    caption_rtf,
                    substr(rtf_text, pos + 1, nchar(rtf_text))
                )
            } else {
                ## Last resort: just prepend to the whole thing after header
                warning("Caption insertion failed, table will be generated without caption")
                rtf_final <- rtf_text
            }
        }
        
        ## Write final RTF
        writeLines(rtf_final, file)
        
        ## Clean up temp file
        unlink(temp_rtf)
        
        ## Write final RTF
        writeLines(rtf_final, file)
        
        ## Clean up temp file
        unlink(temp_rtf)
        
    } else {
        ## No caption - direct RTF export
        flextable::save_as_rtf(ft, path = file)
    }
    
    if (!quiet) message(sprintf("Table saved to %s", file))
    
    ## Return based on user preference
    if (return_ft) {
        ## Return flextable directly for easy access
        return(ft)
    } else {
        ## Create a proper list to hold attributes
        ## Return invisibly with flextable as attribute for backward compatibility
        result_obj <- list(
            file = file,
            caption = result$caption
        )
        attr(result_obj, "flextable") <- ft
        class(result_obj) <- c("table2rtf_result", "list")
        return(invisible(result_obj))
    }
}


### * Print function

#' Print method for table2rtf results
#'
#' @param x Object of class \code{table2rtf_result}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @export
print.table2rtf_result <- function(x, ...) {
    cat("Table saved to:", x$file, "\n")
    if (!is.null(x$caption)) {
        cat("Caption:", x$caption, "\n")
    }
    cat("Flextable object available via: attr(result, 'flextable')\n")
    invisible(x)
}

### * Main function

#' Create Forest Plot for Cox Proportional Hazards Models
#'
#' Generates a publication-ready forest plot that combines a formatted data table 
#' with a graphical representation of hazard ratios from a Cox proportional hazards 
#' survival model. The plot integrates variable names, group levels, sample sizes, 
#' event counts, hazard ratios with confidence intervals, \emph{p}-values, and model 
#' diagnostics in a single comprehensive visualization designed for manuscripts 
#' and presentations.
#'
#' @param x Either a fitted Cox model object (class \code{coxph} or \code{coxme}), 
#'   a \code{fit_result} object from \code{fit()}, or a \code{fullfit_result}
#'   object from \code{fullfit()}. When a \code{fit_result} or \code{fullfit_result}
#'   is provided, the model, data, and labels are automatically extracted.
#'   
#' @param data Data frame or data.table containing the original data used to 
#'   fit the model. If \code{NULL} (default) and \code{x} is a model, the function 
#'   attempts to extract data from the model object. If \code{x} is a \code{fit_result},
#'   data is extracted automatically. Providing data explicitly is recommended when
#'   passing a model directly.
#'   
#' @param title Character string specifying the plot title displayed at the top. 
#'   Default is \code{"Cox Proportional Hazards Model"}. Use descriptive titles 
#'   for publication.
#'   
#' @param effect_label Character string for the effect measure label on the 
#'   forest plot axis. Default is \code{"Hazard Ratio"}. Alternatives might include 
#'   "HR" for space-constrained plots.
#'   
#' @param digits Integer specifying the number of decimal places for hazard ratios 
#'   and confidence intervals. Default is 2.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param conf_level Numeric confidence level for confidence intervals. Must be
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
#'   percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
#'   when \code{conf_level = 0.90}).
#'   
#' @param font_size Numeric multiplier controlling the base font size for all 
#'   text elements. Default is 1.0.
#'   
#' @param annot_size Numeric value controlling the relative font size for data 
#'   annotations. Default is 3.88.
#'   
#' @param header_size Numeric value controlling the relative font size for column 
#'   headers. Default is 5.82.
#'   
#' @param title_size Numeric value controlling the relative font size for the 
#'   main plot title. Default is 23.28.
#'   
#' @param table_width Numeric value between 0 and 1 specifying the proportion of 
#'   total plot width allocated to the data table. Default is 0.6 (60\% table, 
#'   40\% forest plot).
#'   
#' @param plot_width Numeric value specifying the intended output width in 
#'   specified \code{units}. Used for optimizing layout. Default is \code{NULL} 
#'   (automatic). Recommended: 10-16 inches.
#'   
#' @param plot_height Numeric value specifying the intended output height in 
#'   specified \code{units}. Default is \code{NULL} (automatic based on number 
#'   of rows).
#'   
#' @param show_n Logical. If \code{TRUE}, includes a column showing group-specific 
#'   sample sizes.
#'   Counts describe the observations used in fitting rather than every row
#'   supplied. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes a column showing the 
#'   number of events (deaths, failures) for each group. Critical for survival 
#'   analysis interpretation. Default is \code{TRUE}.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels under 
#'   their parent variable, creating hierarchical structure. The "Group" column 
#'   is hidden when \code{TRUE}. Default is \code{FALSE}.
#'
#' @param condense_table Logical. If \code{TRUE}, condenses binary variables to 
#'   show only the non-reference level (\emph{e.g.,} "Yes" for Yes/No variables), with 
#'   the variable name indicating the displayed level. This creates a more compact 
#'   table for models with many binary predictors. Default is \code{FALSE}.
#'   
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. If \code{FALSE} (default), variable names are displayed in plain
#'   text.
#'   
#' @param center_padding Numeric value specifying horizontal spacing between 
#'   table and forest plot. Default is 4.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating gray 
#'   background shading to different variables for improved readability. 
#'   Default is \code{TRUE}.
#'   
#' @param ref_label Character string to display for reference categories. 
#'   Default is \code{"reference"}.
#'   
#' @param labels Named character vector providing custom display labels for 
#'   variables. Example: \code{c(age = "Age (years)", stage = "Disease Stage")}. 
#'   Default is \code{NULL}, in which case the labels attached by the function
#'   that produced \code{x} are used, where \code{x} is a \pkg{summata} result
#'   rather than a model object. Original variable names are used where neither
#'   is available.
#' @param color Character string specifying the color for hazard ratio point 
#'   estimates in the forest plot. Default is \code{"#8A61D8"} (purple). Use 
#'   hex codes or R color names.
#'
#' @param qc_footer Logical. If \code{TRUE}, displays model quality control
#'   statistics in the footer (events analyzed, global log-rank \emph{p}-value,
#'   concordance index, AIC). Default is \code{TRUE}.
#'   
#' @param units Character string specifying units for plot dimensions: 
#'   \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @return A \code{ggplot} object containing the complete forest plot. The plot 
#'   can be:
#'   \itemize{
#'     \item Displayed directly: \code{print(plot)}
#'     \item Saved to file: \code{forestsave(plot, "forest.pdf")}
#'     \item Further customized with \pkg{ggplot2} functions
#'   }
#'   
#'   The returned object includes an attribute \code{"rec_dims"} 
#'   accessible via \code{attr(plot, "rec_dims")}, which is a list 
#'   containing:
#'   \describe{
#'     \item{width}{Numeric. Recommended plot width in specified units}
#'     \item{height}{Numeric. Recommended plot height in specified units}
#'   }
#'   
#'   These recommendations are automatically calculated based on the number of 
#'   variables, text sizes, and layout parameters, and are printed to console 
#'   if \code{plot_width} or \code{plot_height} are not specified. The list also 
#'   carries a \code{units} element recording the units the dimensions are 
#'   expressed in, matching the \code{units} argument. \code{forestsave()} reads 
#'   all three and requires no further handling.
#'
#'   The returned object also includes an attribute \code{"table_data"}
#'   accessible via \code{attr(plot, "table_data")}, a data.table holding the
#'   values drawn in the plot: one row per term in model order, with the
#'   variable, factor level, sample size, event count, estimate, confidence
#'   bounds and p-value. Sample sizes and event counts describe the
#'   observations used in fitting rather than every row supplied, so they sum
#'   to the model sample size within each variable.
#'
#' @details
#' \strong{Survival-Specific Features:}
#' 
#' The Cox forest plot includes several survival analysis-specific components:
#' \itemize{
#'   \item \strong{Event counts}: Number of events (deaths, failures) shown for 
#'     each predictor category, critical for assessing statistical power
#'   \item \strong{Hazard ratios}: Always exponentiated coefficients (never raw), 
#'     interpreted as the multiplicative change in hazard
#'   \item \strong{Log scale}: Forest plot uses log scale for HR (reference line at 1)
#'   \item \strong{Model diagnostics}: Includes concordance (C-index), global 
#'     log-rank test \emph{p}-value, and AIC
#' }
#' 
#' \strong{Plot Components:}
#' 
#' \enumerate{
#'   \item \strong{Title}: Centered at top
#'   \item \strong{Data Table} (left): Contains:
#'     \itemize{
#'       \item Variable and Group columns
#'       \item n: Sample sizes by group
#'       \item Events: Event counts by group (critical for survival)
#'       \item aHR (95\% CI); \emph{p}-value: Adjusted hazard ratios with CIs and \emph{p}-values
#'     }
#'   \item \strong{Forest Plot} (right):
#'     \itemize{
#'       \item Point estimates (squares sized by sample size)
#'       \item 95\% confidence intervals
#'       \item Reference line at HR = 1
#'       \item Log scale for hazard ratios
#'     }
#'   \item \strong{Model Statistics} (footer):
#'     \itemize{
#'       \item Events analyzed (with total events and percentage)
#'       \item Global log-rank test \emph{p}-value
#'       \item Concordance (C-index) with standard error
#'       \item AIC
#'     }
#' }
#' 
#' \strong{Interpreting Hazard Ratios:}
#' 
#' \itemize{
#'   \item \strong{HR = 1}: No effect on hazard (reference)
#'   \item \strong{HR > 1}: Increased hazard (worse survival)
#'   \item \strong{HR < 1}: Decreased hazard (better survival)
#'   \item Example: HR = 2.0 means twice the hazard of the event at any time
#' }
#' 
#' \strong{Event Counts:}
#' 
#' The "Events" column is particularly important in survival analysis:
#' \itemize{
#'   \item Indicates the number of actual events (not censored observations) 
#'     in each group
#'   \item Essential for assessing statistical power
#'   \item Categories with very few events may have unreliable HR estimates
#'   \item The footer shows events analyzed, total events, and percentage 
#'     in the original data
#' }
#' 
#' \strong{Concordance (C-index):}
#' 
#' The concordance statistic displayed in the footer indicates discrimination:
#' \itemize{
#'   \item Range: 0.5 to 1.0
#'   \item 0.5 = random prediction (coin flip)
#'   \item 0.7-0.8 = acceptable discrimination
#'   \item > 0.8 = excellent discrimination
#'   \item Standard error provided for confidence interval calculation
#' }
#' 
#' \strong{Global Log-Rank Test:}
#' 
#' The global \emph{p}-value tests the null hypothesis that all coefficients are zero:
#' \itemize{
#'   \item Significant \emph{p}-value (< 0.05) indicates the model as a whole predicts survival
#'   \item Non-significant global test doesn't preclude significant individual 
#'     predictors
#'   \item Based on the score (log-rank) test
#' }
#' 
#' \strong{Stratification and Clustering:}
#' 
#' If the model includes stratification (\code{strata()}) or clustering 
#' (\code{cluster()}):
#' \itemize{
#'   \item Stratified variables are not shown in the forest plot (they don't 
#'     have HRs)
#'   \item Clustering affects standard errors but not point estimates
#'   \item Both are handled automatically by the function
#' }
#' 
#' \strong{Proportional Hazards Assumption:}
#' 
#' The forest plot assumes proportional hazards (constant HR over time). Users 
#' should verify this assumption using:
#' \itemize{
#'   \item \code{cox.zph(model)} for testing
#'   \item Stratification for variables violating the assumption
#'   \item Time-dependent coefficients if needed
#' }
#'
#' @seealso 
#' \code{\link{autoforest}} for automatic model detection,
#' \code{\link{glmforest}} for logistic/GLM forest plots,
#' \code{\link{lmforest}} for linear model forest plots,
#' \code{\link{uniforest}} for univariable screening forest plots,
#' \code{\link{multiforest}} for multi-outcome forest plots,
#' \code{\link[survival]{coxph}} for fitting Cox models,
#' \code{\link{fit}} for regression modeling,
#'   \code{\link{forestsave}} for saving with recommended dimensions
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#' library(survival)
#'
#' # Create example model
#' model1 <- coxph(
#'     survival::Surv(os_months, os_status) ~ age + sex + treatment,
#'     data = clintrial)
#'
#' # Example 1: Basic Cox model forest plot
#' p <- coxforest(model1, data = clintrial)
#'
#' \donttest{
#' 
#' old_width <- options(width = 180)
#' 
#' # Example 2: With custom labels and title
#' plot2 <- coxforest(
#'     x = model1,
#'     data = clintrial,
#'     title = "Prognostic Factors for Overall Survival",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 3: Comprehensive model with indented layout
#' model3 <- coxph(
#'     Surv(os_months, os_status) ~ age + sex + bmi + smoking + 
#'         treatment + stage + grade,
#'     data = clintrial
#' )
#' 
#' plot3 <- coxforest(
#'     x = model3,
#'     data = clintrial,
#'     labels = clintrial_labels,
#'     indent_groups = TRUE,
#'     zebra_stripes = TRUE
#' )
#' 
#' # Example 4: Condensed layout for many binary predictors
#' model4 <- coxph(
#'     Surv(os_months, os_status) ~ age + sex + smoking + 
#'         hypertension + diabetes + surgery,
#'     data = clintrial
#' )
#' 
#' plot4 <- coxforest(
#'     x = model4,
#'     data = clintrial,
#'     condense_table = TRUE,
#'     labels = clintrial_labels
#' )
#' 
#' # Example 5: Stratified Cox model
#' model5 <- coxph(
#'     Surv(os_months, os_status) ~ age + sex + treatment + strata(site),
#'     data = clintrial
#' )
#' 
#' plot5 <- coxforest(
#'     x = model5,
#'     data = clintrial,
#'     title = "Stratified by Study Site",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 6: Save with recommended dimensions
#' forestsave(plot5, file.path(tempdir(), "survival_forest.pdf"))
#'
#' options(old_width)
#' 
#' }
#' @family visualization functions
#' @export
coxforest <- function(x, data = NULL,
                      title = "Cox Proportional Hazards Model",
                      effect_label = "Hazard Ratio",
                      digits = 2,
                      p_digits = 3,
                      conf_level = 0.95,
                      font_size = 1.0,
                      annot_size = 3.88,
                      header_size = 5.82,
                      title_size = 23.28,
                      plot_width = NULL,
                      plot_height = NULL,
                      table_width = 0.6,
                      show_n = TRUE,
                      show_events = TRUE,
                      indent_groups = FALSE,
                      condense_table = FALSE,
                      bold_variables = FALSE,
                      center_padding = 4,
                      zebra_stripes = TRUE,
                      ref_label = "reference",
                      labels = NULL,
                      color = "#8A61D8",
                      qc_footer = TRUE,
                      units = "in",
                      number_format = NULL) {

    ## Check for required packages
    if (!requireNamespace("data.table", quietly = TRUE)) {
        stop("Package 'data.table' is required but not installed.")
    }
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required but not installed.")
    }
    if (!requireNamespace("grid", quietly = TRUE)) {
        stop("Package 'grid' is required but not installed.")
    }
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    ## Handle input: accept either a model object or a fit_result/fullfit_result from fit()/fullfit()
    if (inherits(x, "fit_result") || inherits(x, "fullfit_result")) {
        ## Extract model, data, and labels from fit_result or fullfit_result
        model <- attr(x, "model")
        
        if (is.null(model)) {
            stop("The fit_result/fullfit_result does not contain a model.\n",
                 "This may occur if fullfit() was run with columns='uni' only.")
        }
        
        ## Use data from fit_result if not provided
        if (is.null(data)) {
            ## Try to get data from model first
            if (!is.null(model$data)) {
                data <- model$data
            } else {
                ## Try to evaluate the call
                tryCatch({
                    data <- eval(model$call$data)
                }, error = function(e) {
                    data <- NULL
                })
            }
        }
        
        ## Use labels from fit_result if not provided
        if (is.null(labels)) {
            labels <- attr(x, "labels")
        }
        
        ## Validate that the model is a Cox model
        if (!inherits(model, c("coxph", "coxme"))) {
            stop("fit_result does not contain a Cox model (coxph or coxme).\n",
                 "Model type: ", class(model)[1], "\n",
                 "Use glmforest() for GLM models or lmforest() for linear models.")
        }
    } else if (inherits(x, c("coxph", "coxme"))) {
        ## Direct model object
        model <- x
    } else {
        stop("x must be either:\n",
             "  - A coxph or coxme model object, or\n",
             "  - A fit_result from fit(), or\n
",
"  - A fullfit_result from fullfit()\n",
"
Received class: ", paste(class(x), collapse = ", "))
    }

    ## Internally work in inches
    if (!is.null(plot_width) && units != "in") {
        plot_width <- convert_units(plot_width, from = units, to = "in")
    }

    ## Get data
    if(is.null(data)){
        warning("The `data` argument is not provided. Data will be extracted from model fit.")
        
        ## First try to get data from model$data (stored by mmodel)
        if (!is.null(model$data)) {
            data <- model$data
        } else {
            ## Try to evaluate the call
            tryCatch({
                data <- eval(model$call$data)
            }, error = function(e) {
                data <- NULL
            })
            
            ## Try model$model as fallback
            if (is.null(data))
                data <- model$model
        }
        
        if (is.null(data))
            stop("The `data` argument should be provided either to coxforest or stored in the model.")
    }
    
    ## Convert to data.table if not already
    if(!data.table::is.data.table(data)) {
        data <- data.table::as.data.table(data)
    }
    
    ## Counting frame: the observations that entered the model.
    ##
    ## survival::coxph() does not retain a model frame by default and coxme
    ## models never do, so the supplied data is restricted to complete cases
    ## across the model variables. Counting the unrestricted data would report
    ## group sizes over every supplied row while the model-level statistics
    ## describe complete cases only.
    ##
    ## The restriction is deliberately not applied to 'data', which remains the
    ## denominator for the "percentage analyzed" figures reported below.
    model_data <- get_analysis_data(model, class(model)[1], data)
    
    terms <- attr(model$terms, "dataClasses")[-1]

    ## Filter out strata() and cluster() terms - these are not predictors to plot
    terms <- terms[!grepl("strata|cluster", names(terms))]
    
    ## Filter out interaction terms (contain ":"), which are handled separately via coefficients
    terms <- terms[!grepl(":", names(terms), fixed = TRUE)]

    ## Extract coefficients based on model type
    model_class <- class(model)[1]

    ## For coxme models, data is mandatory
    if (model_class == "coxme" && is.null(data)) {
        stop("The 'data' argument is required for coxme models.\n",
             "Usage: coxforest(model, data = your_data, ...)")
    }

    ## Initialize coef variables
    coef_summary <- NULL
    conf_int <- NULL
    
    if (model_class == "coxme") {
        ## Special handling for coxme models
        
        ## Extract fixed effects coefficients
        fixed_effects <- coxme::fixef(model)
        
        ## Extract standard errors from variance matrix
        n_fixed <- length(fixed_effects)
        se_values <- sqrt(diag(as.matrix(model$variance))[1:n_fixed])
        
        ## Calculate z-statistics and \emph{p}-values
        z_values <- fixed_effects / se_values
        p_values <- 2 * pnorm(abs(z_values), lower.tail = FALSE)
        
        ## Calculate confidence intervals using conf_level
        z_mult <- qnorm((1 + conf_level) / 2)
        conf_low <- fixed_effects - z_mult * se_values
        conf_high <- fixed_effects + z_mult * se_values
        
        ## Create coefficient summary in expected format
        coef_summary <- cbind(
            coef = fixed_effects,
            "se(coef)" = se_values,
            z = z_values,
            "Pr(>|z|)" = p_values
        )
        rownames(coef_summary) <- names(fixed_effects)
        
        ## Create conf_int in expected format
        ci_pct_low <- paste0(round((1 - conf_level) / 2 * 100, 1), " %")
        ci_pct_high <- paste0(round((1 + conf_level) / 2 * 100, 1), " %")
        conf_int <- cbind(conf_low, conf_high)
        colnames(conf_int) <- c(ci_pct_low, ci_pct_high)
        rownames(conf_int) <- names(fixed_effects)
        
        ## Also need to set nevent and create xlevels as before
        model$nevent <- model$n[1]

        ## For coxme, systematically extract from formula
        ## Parse formula to get predictor variables
        formula_chr <- as.character(model$formulaList$fixed)
        rhs_formula <- formula_chr[3]

        ## Split by + to get individual terms
        predictor_terms <- trimws(strsplit(rhs_formula, "\\+")[[1]])

        ## Remove random effects terms like (1|site)
        predictor_vars <- predictor_terms[!grepl("\\(.*\\|.*\\)", predictor_terms)]

        ## Create terms
        terms <- sapply(predictor_vars, function(v) {
            if (v %in% names(data)) {
                if (is.factor(data[[v]])) "factor"
                else if (is.numeric(data[[v]])) "numeric"
                else class(data[[v]])[1]
            } else {
                "numeric"  # Default
            }
        })
        
        ## Filter out interaction terms (contain ":")
        terms <- terms[!grepl(":", names(terms), fixed = TRUE)]

        ## Construct xlevels for the model
        if (is.null(model$xlevels)) {
            model$xlevels <- list()
            for (var in predictor_vars) {
                if (var %in% names(data) && is.factor(data[[var]])) {
                    model$xlevels[[var]] <- levels(data[[var]])
                }
            }
        }
        
    } else {
        ## Standard coxph extraction
        coef_summary <- summary(model)$coefficients
        conf_int <- get_cached_confint(model, conf_level)
    }    

    
    coef <- data.table::data.table(
                            term = rownames(coef_summary),
                            estimate = coef_summary[, "coef"],
                            std_error = coef_summary[, "se(coef)"],
                            statistic = coef_summary[, "z"],
                            p_value = coef_summary[, "Pr(>|z|)"],
                            conf_low = conf_int[, 1],
                            conf_high = conf_int[, 2]
                        )
    
    ## Get model statistics
    if (model_class == "coxme") {
        ## Calculate AIC for coxme
        ## AIC = -2 * loglik + 2 * df
        coxme_aic <- if (!is.null(model$loglik) && length(model$loglik) >= 3) {
                         -2 * model$loglik[3] + 2 * model$df[2]  # Using penalized loglik
                     } else {
                         NA
                     }
        
        ## Calculate likelihood ratio test \emph{p}-value
        p_value_log <- if (!is.null(model$loglik) && length(model$loglik) >= 2) {
                           lr_stat <- 2 * (model$loglik[2] - model$loglik[1])
                           lr_df <- model$df[1]
                           pchisq(lr_stat, df = lr_df, lower.tail = FALSE)
                       } else {
                           NA
                       }
        
        gmodel <- list(
            nevent = model$n[1],
            p_value_log = p_value_log,
            AIC = coxme_aic,
            concordance = NA,  # Not easily available for coxme
            concordance.se = NA
        )
    } else {
        ## Standard coxph statistics
        conc_values <- summary(model)$concordance
        gmodel <- list(
            nevent = model$nevent,
            p_value_log = as.numeric(summary(model)$sctest["pvalue"]),
            AIC = stats::AIC(model),
            concordance = as.numeric(conc_values["C"]),
            concordance.se = as.numeric(conc_values["se(C)"])
        )
    }

    ## Observations are reported alongside events. The two answer different
    ## questions, attrition from the cohort and loss of statistical
    ## information, and a figure is read without the console output beside it.
    gmodel$nobs <- get_model_nobs(model, class(model)[1])

    ## Calculate total events in original data and percentage analyzed
    formula_terms <- all.vars(model$formula)
    if (length(formula_terms) >= 2) {
        event_var <- formula_terms[2]
        event_data <- data[[event_var]]
        
        ## Handle factor event indicators (rare but possible)
        if (is.factor(event_data)) {
            event_binary <- as.numeric(event_data) == 2
        } else {
            event_binary <- event_data
        }
        
        total_events <- sum(event_binary, na.rm = TRUE)
    } else {
        total_events <- NA
    }
    
    ## Both lines are rendered by the formatter the print methods use, so that
    ## a figure and the table it accompanies disclose in identical terms
    gmodel$obs_line <- format_analysis_counts(
        list(n_supplied = nrow(data), n_analyzed = gmodel$nobs),
        label = "Observations analyzed", marks = marks)

    gmodel$events_line <- format_analysis_counts(
        list(n_supplied = total_events, n_analyzed = gmodel$nevent),
        label = "Events analyzed", marks = marks)

    ## Either line is dropped rather than shown without its denominator
    gmodel$counts_block <- paste(
        c(gmodel$obs_line, gmodel$events_line), collapse = "\n")
    aic_val <- format(round(gmodel$AIC, 2), big.mark = marks$big.mark, decimal.mark = marks$decimal.mark, scientific = FALSE, nsmall = 2)
    gmodel$AIC_formatted <- trimws(aic_val)

    ## Calculate CI for concordance if both concordance and SE are available
    z_crit <- qnorm((1 + conf_level) / 2)
    if(!is.null(gmodel$concordance) && !is.na(gmodel$concordance) &&
       !is.null(gmodel$concordance.se) && !is.na(gmodel$concordance.se)) {
        gmodel$concordance.lower <- gmodel$concordance - z_crit * gmodel$concordance.se
        gmodel$concordance.upper <- gmodel$concordance + z_crit * gmodel$concordance.se
    } else {
        gmodel$concordance.lower <- NA
        gmodel$concordance.upper <- NA
    }

    ## Format the global \emph{p}-value for display using p_digits
    global_p_threshold <- 10^(-p_digits)
    global_p_threshold_str <- if (!is.null(marks)) {
                                  paste0("< 0", marks$decimal.mark, strrep("0", p_digits - 1), "1")
                              } else {
                                  paste0("< ", format(global_p_threshold, scientific = FALSE))
                              }
    global_p_formatted <- if(as.numeric(gmodel$p_value_log) < global_p_threshold) {
                              global_p_threshold_str
                          } else {
                              format_number(as.numeric(gmodel$p_value_log), p_digits, marks)
                          }
    
    ## Format CI percentage for display
    ci_pct <- round(conf_level * 100)
    
    ## Build concordance string based on whether CI is available
    concordance_string <- if(!is.null(gmodel$concordance) && !is.na(gmodel$concordance)) {
                              if(!is.null(gmodel$concordance.lower) && !is.na(gmodel$concordance.lower) && 
                                 !is.null(gmodel$concordance.upper) && !is.na(gmodel$concordance.upper)) {
                                  paste0("Concordance Index: ", round(gmodel$concordance, 2), 
                                         " (", ci_pct, "% CI ", round(gmodel$concordance.lower, 2), "-", 
                                         round(gmodel$concordance.upper, 2), ")")
                              } else {
                                  paste0("Concordance Index: ", round(gmodel$concordance, 2))
                              }
                          } else {
                              "Concordance Index: Not available"
                          }

    ## Extract statistics for every variable - preserving order
    all_terms <- lapply(seq_along(terms), function(i){
        var <- names(terms)[i]

        if (terms[i] %in% c("factor", "character")) {
            ## Get the factor levels from the model's xlevels (proper order)
            if(var %in% names(model$xlevels)) {
                factor_levels <- model$xlevels[[var]]
                
                ## Create data table with proper levels
                level_counts <- model_data[!is.na(get(var)), .N, by = var]
                data.table::setnames(level_counts, c("level", "Freq"))
                
                ## Ensure all levels are present
                all_levels_dt <- data.table::data.table(
                                                 level = factor_levels,
                                                 Freq = 0
                                             )
                
                ## Update with actual counts
                all_levels_dt[level_counts, Freq := i.Freq, on = "level"]
                all_levels_dt[, var := var]
                all_levels_dt[, pos := .I]
                all_levels_dt[, var_order := i]
                all_levels_dt[, .(var, level, Freq, pos, var_order)]
            } else {
                ## Fallback for variables not in xlevels
                adf <- model_data[!is.na(get(var)), .N, by = var]
                data.table::setnames(adf, old = c(var, "N"), new = c("level", "Freq"))
                adf[, var := var]
                adf[, pos := .I]
                adf[, var_order := i]
                adf[, .(var, level, Freq, pos, var_order)]
            }
        }
        else if (terms[i] == "numeric") {
            data.table::data.table(var = var, level = "-", Freq = nrow(model_data), pos = 1, var_order = i)
        }
        else {
            vars = grep(paste0("^", var, "*."), coef$term, value=TRUE)
            data.table::data.table(var = vars, level = "-", Freq = nrow(model_data),
                                   pos = seq_along(vars), var_order = i)
        }
    })
    
    all_terms_df <- data.table::rbindlist(all_terms)

    data.table::setnames(all_terms_df, c("var", "level", "N", "pos", "var_order"))

    ## Add events
    if (model_class == "coxme") {
        formula_terms <- all.vars(model$formulaList$fixed)
    } else {
        formula_terms <- all.vars(model$formula)
    }
    
    if (length(formula_terms) >= 2) {
        ## Assuming standard Surv(time, status) format
        event_var <- formula_terms[2]
        event_data <- model_data[[event_var]]
        
        ## Handle factor event indicators (rare but possible)
        if (is.factor(event_data)) {
            ## Convert to binary (assuming second level is the event)
            event_binary <- as.numeric(event_data) == 2
        } else {
            ## Already numeric
            event_binary <- event_data
        }
        
        ## Calculate events for each row
        all_terms_df[, Events := {
            if (level == "-") {
                ## Continuous variable - total events
                sum(event_binary, na.rm = TRUE)
            } else {
                ## Factor level - events in that group
                if (var %in% names(model_data)) {
                    sum(event_binary[model_data[[var]] == level], na.rm = TRUE)
                } else {
                    NA_integer_
                }
            }
        }, by = seq_len(nrow(all_terms_df))]
    } else {
        ## No events column if can't extract event variable
        all_terms_df[, Events := NA_integer_]
    }

    ## ----- INTERACTION TERMS HANDLING -----
    ## Detect and add interaction terms that aren't in all_terms_df
    ## Interaction terms have ":" in their names
    interaction_terms <- coef$term[grepl(":", coef$term, fixed = TRUE)]
    
    if (length(interaction_terms) > 0) {
        ## Get the next var_order value
        next_var_order <- max(all_terms_df$var_order) + 1L
        
        ## Get event variable for calculating events in interactions
        event_var <- if (length(formula_terms) >= 2) formula_terms[2] else NULL
        
        ## Process each interaction term
        interaction_rows <- lapply(seq_along(interaction_terms), function(idx) {
            int_term <- interaction_terms[idx]
            
            ## Parse the interaction term to create a readable label
            ## \emph{e.g.,}, "treatmentDrug A:stageII" -> "Treatment Group (Drug A) × Disease Stage (II)"
            int_parts <- strsplit(int_term, ":", fixed = TRUE)[[1]]
            
            ## Build readable variable name from parts
            display_parts <- sapply(int_parts, function(part) {
                ## Try to match against known factor variables to extract var and level
                matched_result <- NULL
                for (var_name in names(terms)) {
                    if (startsWith(part, var_name)) {
                        level_str <- sub(paste0("^", var_name), "", part)
                        if (nchar(level_str) > 0) {
                            ## Apply label if available
                            var_label <- if (!is.null(labels) && var_name %in% names(labels)) {
                                             labels[[var_name]]
                                         } else {
                                             var_name
                                         }
                            matched_result <- paste0(var_label, " (", level_str, ")")
                            break
                        }
                    }
                }
                ## If no match found (continuous variable in interaction or exact match)
                if (is.null(matched_result)) {
                    var_label <- if (!is.null(labels) && part %in% names(labels)) {
                                     labels[[part]]
                                 } else {
                                     part
                                 }
                    matched_result <- var_label
                }
                matched_result
            })
            
            display_name <- paste(display_parts, collapse = " \u00d7 ")
            
            ## Calculate N and Events for this interaction
            int_n <- NA_integer_
            int_events <- NA_integer_
            
            tryCatch({
                ## For each part of the interaction, build conditions
                conditions <- list()
                for (j in seq_along(int_parts)) {
                    part <- int_parts[j]
                    for (var_name in names(terms)) {
                        if (startsWith(part, var_name) && terms[var_name] %in% c("factor", "character")) {
                            level_str <- sub(paste0("^", var_name), "", part)
                            if (nchar(level_str) > 0 && var_name %in% names(data)) {
                                conditions[[var_name]] <- level_str
                            }
                            break
                        }
                    }
                }
                
                ## Where conditions were found, count matching rows and events
                if (length(conditions) > 0) {
                    ## Build subset indices
                    subset_indices <- rep(TRUE, nrow(model_data))
                    for (cond_var in names(conditions)) {
                        subset_indices <- subset_indices & (model_data[[cond_var]] == conditions[[cond_var]])
                    }
                    int_n <- sum(subset_indices, na.rm = TRUE)
                    
                    ## Calculate events if event variable is available
                    if (!is.null(event_var) && event_var %in% names(model_data)) {
                        event_data <- model_data[[event_var]]
                        if (is.factor(event_data)) {
                            event_binary <- as.numeric(event_data) == 2
                        } else {
                            event_binary <- event_data
                        }
                        int_events <- sum(event_binary[subset_indices], na.rm = TRUE)
                    }
                }
            }, error = function(e) {
                ## Keep as NA
            })
            
            data.table::data.table(
                            var = display_name,
                            level = "-",
                            N = int_n,
                            pos = 1L,
                            var_order = next_var_order + idx - 1L,
                            Events = int_events,
                            .int_term = int_term  # Store raw term for later inds assignment
                        )
        })
        
        interaction_df <- data.table::rbindlist(interaction_rows)
        
        ## Bind interaction rows to main terms
        all_terms_df <- data.table::rbindlist(list(all_terms_df, interaction_df), 
                                              use.names = TRUE, fill = TRUE)
    }

    ## Apply condensing and indenting if requested
    if (condense_table || indent_groups) {
        if (condense_table) {
            indent_groups <- TRUE
        }
        
        ## For interaction rows, use the stored raw term; for others use var+level pattern
        if (".int_term" %in% names(all_terms_df)) {
            all_terms_df[, inds := data.table::fifelse(
                                                   !is.na(.int_term), 
                                                   .int_term,
                                                   data.table::fifelse(level == "-", var, paste0(var, level))
                                               )]
        } else {
            all_terms_df[, inds := data.table::fifelse(level == "-", var, paste0(var, level))]
        }
        orig_inds_map <- data.table::copy(all_terms_df[, .(var, level, inds, N, Events)])
        
        processed_rows <- list()
        row_counter <- 1
        unique_vars <- unique(all_terms_df[, var])
        
        for (v in unique_vars) {
            var_rows <- all_terms_df[var == v]
            
            if (nrow(var_rows) == 1) {
                processed_rows[[row_counter]] <- var_rows
                row_counter <- row_counter + 1
            } else {
                is_binary <- nrow(var_rows) == 2
                
                if (condense_table && is_binary) {
                    ## Detect reference row by checking if level is first in model$xlevels
                    ## (first level is always reference in R factor contrasts)
                    ref_level <- NULL
                    if (!is.null(model$xlevels) && v %in% names(model$xlevels)) {
                        ref_level <- model$xlevels[[v]][1]
                    }
                    
                    ## Find non-reference row
                    non_ref_idx <- NULL
                    if (!is.null(ref_level)) {
                        ref_idx <- which(var_rows$level == ref_level)
                        if (length(ref_idx) == 1) {
                            non_ref_idx <- setdiff(1:2, ref_idx)
                        }
                    }
                    
                    ## Fallback to estimate-based detection if available
                    if (is.null(non_ref_idx) && "estimate" %in% names(var_rows)) {
                        non_ref_idx <- find_non_reference_row(var_rows, "estimate")
                    }
                    
                    ## Final fallback: assume row 2 is non-reference
                    if (is.null(non_ref_idx) || length(non_ref_idx) != 1) {
                        non_ref_idx <- 2L
                        ref_idx <- 1L
                    } else {
                        ref_idx <- setdiff(1:2, non_ref_idx)
                    }
                    
                    non_ref_row <- var_rows[non_ref_idx]
                    ref_row <- var_rows[ref_idx]
                    
                    condensed_row <- data.table::copy(non_ref_row)
                    non_ref_category <- condensed_row$level
                    ref_category <- ref_row$level
                    
                    ## Look up label for smarter condensing detection
                    var_label <- if (!is.null(labels) && v %in% names(labels)) {
                                     labels[[v]]
                                 } else if (v %in% names(data) && 
                                            !is.null(attr(data[[v]], "label"))) {
                                     attr(data[[v]], "label")
                                 } else {
                                     v
                                 }
                    
                    ## Use greedy approach: condense if EITHER category is recognized
                    if (should_condense_binary(ref_category, non_ref_category, var_label)) {
                        condensed_row[, var := v]
                    } else {
                        condensed_row[, var := paste0(v, " (", non_ref_category, ")")]
                    }
                    condensed_row[, level := "-"]
                    processed_rows[[row_counter]] <- condensed_row
                    row_counter <- row_counter + 1
                    
                } else {
                    if (indent_groups) {
                        header_row <- data.table::data.table(
                                                      var = v,
                                                      level = "-",
                                                      N = NA_integer_,
                                                      Events = NA_integer_,
                                                      pos = var_rows$pos[1],
                                                      var_order = var_rows$var_order[1],
                                                      inds = NA_character_
                                                  )
                        processed_rows[[row_counter]] <- header_row
                        row_counter <- row_counter + 1
                        
                        for (i in seq_len(nrow(var_rows))) {
                            group_row <- data.table::copy(var_rows[i])
                            group_row[, var := paste0("    ", level)]
                            group_row[, level := "-"]
                            processed_rows[[row_counter]] <- group_row
                            row_counter <- row_counter + 1
                        }
                    } else {
                        for (i in seq_len(nrow(var_rows))) {
                            processed_rows[[row_counter]] <- var_rows[i]
                            row_counter <- row_counter + 1
                        }
                    }
                }
            }
        }
        
        all_terms_df <- data.table::rbindlist(processed_rows, fill = TRUE)
        
        for (i in seq_len(nrow(all_terms_df))) {
            current_var <- all_terms_df$var[i]
            
            if (is.na(all_terms_df$inds[i]) && !grepl("^    ", current_var) && !grepl("\\(", current_var)) {
                next
            }
            
            if (grepl("\\(", current_var)) {
                orig_var <- gsub(" \\(.*\\)", "", current_var)
                orig_level <- gsub(".*\\((.*)\\)", "\\1", current_var)
                
                matching <- orig_inds_map[var == orig_var & level == orig_level]
                if (nrow(matching) > 0) {
                    all_terms_df[i, `:=`(inds = matching$inds[1],
                                         N = matching$N[1],
                                         Events = matching$Events[1])]
                }
            }
            else if (grepl("^    ", current_var)) {
                clean_level <- gsub("^    ", "", current_var)
                
                parent_var <- NA_character_
                for (j in (i-1):1) {
                    if (!grepl("^    ", all_terms_df$var[j]) && all_terms_df$var[j] != "") {
                        parent_var <- gsub(" \\(.*\\)", "", all_terms_df$var[j])
                        break
                    }
                }
                
                if (!is.na(parent_var)) {
                    matching <- orig_inds_map[var == parent_var & level == clean_level]
                    if (nrow(matching) > 0) {
                        all_terms_df[i, `:=`(inds = matching$inds[1],
                                             N = matching$N[1],
                                             Events = matching$Events[1])]
                    }
                }
            }
        }
    } else {
        ## For non-condensed/non-indented display, set inds for coefficient matching
        ## For interaction rows, use the stored raw term; for others use var+level pattern
        if (".int_term" %in% names(all_terms_df)) {
            all_terms_df[, inds := data.table::fifelse(
                                                   !is.na(.int_term), 
                                                   .int_term,
                                                   data.table::fifelse(level == "-", var, paste0(var, level))
                                               )]
        } else {
            all_terms_df[, inds := data.table::fifelse(level == "-", var, paste0(var, level))]
        }
    }
    
    ## Clean up temporary column if it exists
    if (".int_term" %in% names(all_terms_df)) {
        all_terms_df[, .int_term := NULL]
    }
    
    ## Process coefficients
    coef[, term := gsub(term, pattern = "`", replacement = "-")]
    coef[, inds := term]
    
    ## Merge data
    to_show <- merge(all_terms_df, coef, by.x = "inds", by.y = "inds", all.x = TRUE, sort = FALSE)
    
    ## Sort by variable order first, then position within variable
    data.table::setorder(to_show, var_order, pos)
    
    ## Add variable-based shading indicator (zebra stripes)
    if (zebra_stripes) {
        to_show[, shade_group := var_order %% 2]
        shade_colors <- c("#FFFFFF", "#EEEEEE")
    } else {
        to_show[, shade_group := 0]
        shade_colors <- c("#FFFFFF", "#FFFFFF")
    }
    
    ## Select columns
    to_show <- to_show[, .(var, level, N, Events, p_value, estimate, conf_low, conf_high, pos, var_order, shade_group)]
    
    ## Format the exponential values
    to_show_exp_clean <- data.table::copy(to_show)
    
    ## Create formatted columns for display
    to_show_exp_clean[, hr := data.table::fifelse(is.na(estimate), 
                                                  NA_real_,
                                                  exp(estimate))]

    ## For header rows (with NA N values), show empty strings instead of "reference"
    to_show_exp_clean[, hr_formatted := data.table::fifelse(is.na(N) & is.na(estimate),
                                                            "",  # Empty for header rows
                                                            data.table::fifelse(is.na(estimate), 
                                                                                ref_label,
                                                                                format_number(exp(estimate), digits, marks)))]

    to_show_exp_clean[, conf_low_formatted := data.table::fifelse(is.na(conf_low), 
                                                                  NA_character_,
                                                                  format_number(exp(conf_low), digits, marks))]
    to_show_exp_clean[, conf_high_formatted := data.table::fifelse(is.na(conf_high), 
                                                                   NA_character_,
                                                                   format_number(exp(conf_high), digits, marks))]

    ## Format \emph{p}-values using p_digits parameter
    p_threshold <- 10^(-p_digits)
    p_threshold_str <- if (!is.null(marks)) {
                           paste0("< 0", marks$decimal.mark, strrep("0", p_digits - 1), "1")
                       } else {
                           paste0("< ", format(p_threshold, scientific = FALSE))
                       }
    
    to_show_exp_clean[, p_formatted := data.table::fifelse(is.na(p_value), 
                                                           NA_character_,
                                                           data.table::fifelse(p_value < p_threshold, 
                                                                               p_threshold_str,
                                                                               format_number(p_value, p_digits, marks)))]

    ## Determine CI separator (HRs are always positive, but handle EU comma)
    ci_separator <- forest_ci_separator(FALSE, marks)

    ## Create the combined HR string with expression for italic p
    to_show_exp_clean[, hr_string_expr := data.table::fifelse(
                                                          is.na(N) & is.na(estimate),
                                                          "''",  # Empty string for header rows
                                                          data.table::fifelse(
                                                                          is.na(estimate),
                                                                          paste0("'", ref_label, "'"),
                                                                          data.table::fifelse(p_value < p_threshold,
                                                                                              paste0("'", hr_formatted, " (", conf_low_formatted, ci_separator, 
                                                                                                     conf_high_formatted, "); '*~italic(p)~'", p_threshold_str, "'"),
                                                                                              paste0("'", hr_formatted, " (", conf_low_formatted, ci_separator, 
                                                                                                     conf_high_formatted, "); '*~italic(p)~'= ", p_formatted, "'"))
                                                                      )
                                                      )]

    ## Format N and events with locale-aware thousands separator
    to_show_exp_clean[, n_formatted := data.table::fifelse(is.na(N), "",
        format_count(N, marks, na = ""))]
    to_show_exp_clean[, events_formatted := data.table::fifelse(is.na(Events), "",
        format_count(Events, marks, na = ""))]
    
    ## Clean up variable names for display
    to_show_exp_clean[, var_display := as.character(var)]

    if (indent_groups || condense_table) {
        to_show_exp_clean[, var_display := var]
        
        for (v in unique(to_show_exp_clean$var)) {
            if (v != "" && !grepl("^    ", v)) {
                ## Skip interaction terms (already formatted) - identified by " × " in the name
                if (grepl(" \u00d7 ", v)) next
                
                clean_v <- gsub(" \\(.*\\)", "", v)

                label <- if (!is.null(labels) && clean_v %in% names(labels)) {
                             labels[clean_v]
                         } else if (clean_v %in% names(data) && !is.null(attr(data[[clean_v]], "label"))) {
                             attr(data[[clean_v]], "label")
                         } else {
                             NULL
                         }

                if (!is.null(label)) {
                    if (grepl("\\(", v)) {
                        category <- gsub(".*\\((.*)\\)", "\\1", v)
                        if (is_affirmative_category(category, label)) {
                            to_show_exp_clean[var == v, var_display := label]
                        } else {
                            to_show_exp_clean[var == v, var_display := paste0(label, " (", category, ")")]
                        }
                    } else {
                        to_show_exp_clean[var == v, var_display := label]
                    }
                }
            }
        }
        
        to_show_exp_clean[, level := ""]
        
    } else {
        
        for(v in unique(to_show_exp_clean$var)) {
            if(v %in% to_show_exp_clean$var) {
                ## Skip interaction terms (already formatted) - identified by " × " in the name
                if (grepl(" \u00d7 ", v)) next
                
                if(!is.null(labels) && v %in% names(labels)) {
                    to_show_exp_clean[var == v, var_display := labels[v]]
                }
                else if(v %in% names(data) && !is.null(attr(data[[v]], "label"))) {
                    to_show_exp_clean[var == v, var_display := attr(data[[v]], "label")]
                }
            }
        }
    }

    if (!indent_groups) {
        to_show_exp_clean[duplicated(var), var_display := ""]
    }
    
    ## Handle missing estimates for plotting
    to_show_exp_clean[is.na(estimate), estimate := 0]
    
    ## Reorder (flip) - but maintain the variable grouping
    ## Retain the assembled table for return as an attribute. The copy is taken
    ## before the row order is reversed for plotting, so that the returned table
    ## follows the order of the model terms rather than the drawing order.
    table_data <- data.table::copy(to_show_exp_clean)
    
    to_show_exp_clean <- to_show_exp_clean[order(rev(seq_len(nrow(to_show_exp_clean))))]
    to_show_exp_clean[, x_pos := .I]
    
    ## Calculate plot ranges with better handling of extreme cases
    rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
    
    min_ci <- min(to_show$conf_low, na.rm = TRUE)
    max_ci <- max(to_show$conf_high, na.rm = TRUE)
    
    is_one_sided <- (min_ci > 0) || (max_ci < 0)
    
    ## Intelligent tick selection to prevent overlap
    range_magnitude <- diff(rangeb)
    
    if (exp(min_ci) < 0.01 && exp(max_ci) > 2) {
        ## Very wide range
        breaks <- c(0.01, 0.1, 0.5, 1, 2, 5)
    } else if (range_magnitude > 3) {
        ## Wide range - thin out the ticks
        all_breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
        if (length(all_breaks) > 7) {
            ## Too many - keep key values only
            important <- c(1)
            if (min(all_breaks) < 0.5) important <- c(min(all_breaks), important)
            if (max(all_breaks) > 2) important <- c(important, max(all_breaks))
            
            other_breaks <- setdiff(all_breaks, important)
            if (length(other_breaks) > 3) {
                keep_idx <- round(seq(1, length(other_breaks), length.out = 3))
                other_breaks <- other_breaks[keep_idx]
            }
            breaks <- sort(unique(c(important, other_breaks)))
        } else {
            breaks <- all_breaks
        }
    } else {
        ## Normal range - use standard calculation
        breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
    }
    
    if (!1 %in% breaks) {
        breaks <- sort(unique(c(breaks, 1)))
    }
    
    if (min_ci > 0) {
        rangeb[1] <- log(0.9)
    } else if(max_ci < 0) {
        rangeb[2] <- log(1.1)
    }
    
    breaks <- breaks[breaks >= exp(rangeb[1]) & breaks <= exp(rangeb[2])]
    reference_value <- 1

    ## Calculate layout using helper function
    layout <- calculate_forest_layout(
        to_show_exp_clean = to_show_exp_clean,
        show_n = show_n,
        show_events = show_events,
        indent_groups = indent_groups,
        condense_table = condense_table,
        effect_label = effect_label,
        ref_label = ref_label,
        font_size = font_size,
        table_width = data.table::fifelse(is.null(table_width), 0.6, table_width),
        rangeb = rangeb,
        center_padding = center_padding
    )

    ## Set up the extended range for plotting
    rangeplot <- c(layout$rangeplot_start, rangeb[2] + diff(rangeb) * 0.05)

    ## Extract positions
    y_variable <- layout$positions$var
    if (!(indent_groups || condense_table)) {
        y_level <- layout$positions$level
    }
    if (show_n) {
        y_n <- layout$positions$n
    }
    if (show_events) {
        y_events <- layout$positions$events
    }
    y_hr <- layout$positions$effect

    ## Use the effect abbreviation from layout
    effect_abbrev <- layout$effect_abbrev

    ## Calculate recommended dimensions
    rec_height <- max(5, min(20, 3 + nrow(to_show_exp_clean) * 0.25))

    if(!is.null(plot_width)) {
        rec_width <- plot_width
        if(!is.null(plot_height)) {
            rec_height <- plot_height
        }
    } else {
        ## Use the calculated total width
        rec_width <- layout$total_width + 1.0  # Add margins
        rec_width <- max(10, min(20, rec_width))  # Apply reasonable bounds
    }

    ## Font sizes
    annot_font <- font_size * annot_size
    header_font <- font_size * header_size
    
    ## Custom ticks data
    ticks_df <- data.frame(
        x = -0.5,
        xend = -0.7,
        y = breaks,
        yend = breaks
    )

    ## Create the plot
    p <- ggplot2::ggplot(to_show_exp_clean, ggplot2::aes(x_pos, exp(estimate))) +
        
        ## Shading rectangles - extend to cover table area too
        ggplot2::geom_rect(ggplot2::aes(xmin = x_pos - .5, xmax = x_pos + .5,
                                        ymin = exp(rangeplot[1]), ymax = exp(rangeplot[2]),
                                        fill = ordered(shade_group))) +
        ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.10, 0.05))) +
        ggplot2::scale_size_area(max_size = 6, guide = "none") +
        ggplot2::scale_fill_manual(values = shade_colors, guide = "none") +
        
        ## Forest plot elements
        ggplot2::geom_point(ggplot2::aes(size = N), pch = 22, color = "#000000", fill = color, na.rm = TRUE) +
        ggplot2::geom_errorbar(ggplot2::aes(ymin = exp(conf_low), ymax = exp(conf_high)), width = 0.15) +
        
        ## Y-axis for forest plot
        ggplot2::annotate(geom = "segment",
                          x = -0.5, xend = -0.5,
                          y = exp(rangeb[1]),
                          yend = exp(rangeb[2]),
                          color = "#000000", linewidth = 1) +
        
        ## Reference line at HR = 1
        ggplot2::annotate(geom = "segment", 
                          x = -0.5, xend = max(to_show_exp_clean$x_pos) + 0.5, 
                          y = 1, yend = 1, 
                          linetype = "longdash") +
        
        ## Ticks
        ggplot2::geom_segment(data = ticks_df,
                              ggplot2::aes(x = x, xend = xend, y = y, yend = yend),
                              inherit.aes = FALSE,
                              color = "#000000",
                              linewidth = 1) +
        ggplot2::geom_text(data = ticks_df,
                           ggplot2::aes(x = xend - 0.05, y = y, label = sprintf("%g", y)),
                           inherit.aes = FALSE,
                           hjust = 0.5,
                           vjust = 1.3,
                           size = annot_font * 1.5) +
        
        ## Set coordinate system with extended limits
        ggplot2::coord_flip(ylim = exp(rangeplot)) +
        ggplot2::ggtitle(title) +
        ggplot2::scale_y_log10(name = "Hazard Ratio",
                               labels = sprintf("%g", breaks),
                               expand = c(0.02, 0.02),
                               breaks = breaks) +
        ggplot2::theme_light(base_family = detect_plot_font()) +
        ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 0, b = 0, l = 0),
                       panel.grid.minor.y = ggplot2::element_blank(),
                       panel.grid.minor.x = ggplot2::element_blank(),
                       panel.grid.major.y = ggplot2::element_blank(),
                       panel.grid.major.x = ggplot2::element_blank(),
                       legend.position = "none",
                       panel.border = ggplot2::element_blank(),
                       axis.title.y = ggplot2::element_blank(),
                       axis.text.y = ggplot2::element_blank(),
                       axis.ticks.y = ggplot2::element_blank(),
                       axis.title.x = ggplot2::element_blank(),
                       axis.ticks.x = ggplot2::element_blank(),
                       axis.text.x = ggplot2::element_blank(),
                       plot.title = ggplot2::element_text(size = font_size * title_size, face = "bold", hjust = 0.5)) +
        ggplot2::xlab("") +
        
        ## Variable column
        ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_variable),
                          label = "Variable", fontface = "bold", hjust = 0,
                          size = header_font) +

    {if (indent_groups || condense_table) {
         ## When indented/condensed, use conditional formatting
         fontfaces <- if (bold_variables) {
             data.table::fifelse(grepl("^    ", to_show_exp_clean$var_display), "plain", "bold")
         } else {
             rep("plain", nrow(to_show_exp_clean))
         }
         ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_variable),
                           label = to_show_exp_clean$var_display, 
                           fontface = fontfaces, 
                           hjust = 0,
                           size = annot_font)
     } else {
         ## Non-indented: bold all variable names if bold_variables is TRUE
         ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_variable),
                           label = to_show_exp_clean$var_display, 
                           fontface = if (bold_variables) "bold" else "plain", 
                           hjust = 0,
                           size = annot_font)
     }} +
    
    ## Group/level column
    {if (!(indent_groups || condense_table)) {
         list(
             ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_level),
                               label = "Group", fontface = "bold", hjust = 0,
                               size = header_font),
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_level),
                               label = to_show_exp_clean$level, hjust = 0,
                               size = annot_font)
         )
     }} +
    
    ## N column (conditional)
    {if (show_n) {
         list(
             ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_n),
                               label = "n", fontface = "bold.italic", hjust = 0.5,
                               size = header_font),
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_n),
                               label = to_show_exp_clean$n_formatted, hjust = 0.5,
                               size = annot_font)
         )
     }} +
    
    ## Events column (conditional)
    {if (show_events) {
         list(
             ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = exp(y_events),
                               label = "Events", fontface = "bold", hjust = 0.5,
                               size = header_font),
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_events),
                               label = to_show_exp_clean$events_formatted, hjust = 0.5,
                               size = annot_font)
         )
     }} +
    
    ## Effect column - use dynamic CI percentage
    ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.4, y = exp(y_hr),
                      label = paste0("bold('", effect_abbrev, " (", ci_pct, "% CI); '*bolditalic(p)*'-value')"),
                      hjust = 0, size = header_font, parse = TRUE) +
    
    ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = exp(y_hr),
                      label = to_show_exp_clean$hr_string_expr, hjust = 0,
                      size = annot_font, parse = TRUE) +
    
    ## X-axis label
    ggplot2::annotate(geom = "text", x = -1.5, y = 1,
                      label = effect_label, fontface = "bold",
                      hjust = 0.5, vjust = 2, size = annot_font * 1.5) +
    
    ## Model statistics footer (conditional)
    {if (qc_footer) {
         ggplot2::annotate(geom = "text", x = 0.5, y = exp(y_variable),
                           label = paste0(gmodel$counts_block,
                                          "\nGlobal log-rank p: ", global_p_formatted,
                                          "\n", concordance_string,
                                          "\nAIC: ", gmodel$AIC_formatted),
                           size = annot_font * 0.65, hjust = 0, vjust = 1.2, fontface = "italic")
    }}
    
    ## Convert units back for output if needed
    if (units != "in") {
        rec_width <- convert_units(rec_width, from = "in", to = units)
        rec_height <- convert_units(rec_height, from = "in", to = units)
    }

    ## Provide dimension recommendations
    if(is.null(plot_width) || is.null(plot_height)) {
        message(sprintf("Recommended plot dimensions: width = %.1f %s, height = %.1f %s",
                        rec_width, units, rec_height, units))
    }
    
    ## Add recommended dimensions as an attribute
    ## The units are recorded alongside the dimensions. Both are converted to
    ## the requested units above, so a consumer that assumed inches would
    ## otherwise mis-size the output by the conversion factor.
    attr(p, "rec_dims") <- list(width = rec_width, height = rec_height,
                                units = units)

    ## Add the assembled table as an attribute
    attr(p, "table_data") <- table_data

    ## Return the plot
    return(p)
}

### * Main function

#' Create Forest Plot for Multivariate Regression
#'
#' Generates a publication-ready forest plot from a \code{multifit()} output
#' object. The plot displays effect estimates (OR, HR, RR, or coefficients) with
#' confidence intervals across multiple outcomes, organized by outcome with the
#' predictor levels shown for each.
#'
#' @param x Multifit result object (data.table with class attributes from 
#'   \code{multifit()}).
#'   
#' @param title Character string specifying the plot title. Default is 
#'   \code{"Multivariate Analysis"}. Use descriptive titles for publication.
#'   
#' @param effect_label Character string for the effect measure label on the 
#'   forest plot axis. Default is \code{NULL}, which auto-detects based on 
#'   model type (\emph{e.g.,} "Odds Ratio", "Hazard Ratio", "Rate Ratio", "Estimate").
#'   
#' @param column Character string specifying which results to plot when
#'   \code{multifit()} was called with \code{columns = "both"}. Options are
#'   \code{"adjusted"} (default) or \code{"unadjusted"}. Ignored when the
#'   \code{multifit} result contains only one column type.
#'   
#' @param digits Integer specifying the number of decimal places for effect 
#'   estimates and confidence intervals. Default is 2.
#'
#' @param p_digits Integer specifying the number of decimal places for
#'   \emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
#'   as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
#'   \code{p_digits = 4}), etc. Default is 3.
#'
#' @param conf_level Numeric confidence level for confidence intervals. Must be
#'   between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
#'   percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
#'   when \code{conf_level = 0.90}).
#'   
#' @param font_size Numeric multiplier controlling the base font size for all 
#'   text elements. Default is 1.0.
#'   
#' @param annot_size Numeric value controlling the relative font size for data 
#'   annotations. Default is 3.88.
#'   
#' @param header_size Numeric value controlling the relative font size for column 
#'   headers. Default is 5.82.
#'   
#' @param title_size Numeric value controlling the relative font size for the 
#'   main plot title. Default is 23.28.
#'   
#' @param table_width Numeric value between 0 and 1 specifying the proportion of 
#'   total plot width allocated to the data table. Default is 0.6 (60\% table, 
#'   40\% forest plot).
#'   
#' @param plot_width Numeric value specifying the intended output width in 
#'   specified \code{units}. Used for optimizing layout. Default is \code{NULL} 
#'   (automatic). Recommended: 10-16 inches.
#'   
#' @param plot_height Numeric value specifying the intended output height in 
#'   specified \code{units}. Default is \code{NULL} (automatic based on number 
#'   of rows).
#'   
#' @param show_n Logical. If \code{TRUE}, includes a column showing sample sizes.
#'   Counts describe the observations used in fitting rather than every row
#'   supplied. Default is \code{TRUE}.
#'   
#' @param show_events Logical. If \code{TRUE}, includes a column showing the 
#'   number of events for each row. Default is \code{TRUE} for binomial and 
#'   survival models, \code{FALSE} for linear models.
#'
#' @param show_predictor Logical. If \code{TRUE}, includes the "Predictor" column 
#'   showing which level of a factor predictor is being compared. If \code{FALSE},
#'   omits the column (useful when predictor info is in the caption). Default is 
#'   \code{NULL}, which uses the \code{include_predictor} setting from
#'   \code{multifit()} if available, otherwise \code{TRUE}.
#'
#' @param covariates_footer Logical. If \code{TRUE} (default), displays a footer
#'   listing the covariates used in adjusted models. Covariate names are formatted
#'   using the \code{labels} parameter if provided. Only shown when displaying
#'   adjusted results.
#'
#' @param indent_predictor Logical. If \code{TRUE}, indents predictor levels
#'   under outcome names, creating a hierarchical display. If \code{FALSE} 
#'   (default), shows outcome and predictor level in separate columns.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. If \code{FALSE} (default), variable names are displayed in plain
#'   text.
#'   
#' @param center_padding Numeric value specifying horizontal spacing between 
#'   table and forest plot. Default is 4.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating gray 
#'   background shading to different outcomes for improved readability. 
#'   Default is \code{TRUE}.
#'   
#' @param color Character string specifying the color for point estimates in 
#'   the forest plot. Default is \code{NULL}, which auto-selects based on
#'   effect type: purple (\code{"#8A61D8"}) for hazard ratios (Cox), teal 
#'   (\code{"#4BA6B6"}) for odds ratios (logistic), blue (\code{"#3F87EE"}) for 
#'   rate/risk ratios (Poisson, Gamma, \emph{etc.}), and green (\code{"#5A8F5A"}) for 
#'   coefficients (linear models).
#'   Use hex codes or R color names for custom colors.
#'   
#' @param null_line Numeric value for the reference line position. Default is 
#'   \code{NULL}, which uses 1 for ratio measures (OR, HR, RR) and 0 for 
#'   coefficients.
#'   
#' @param log_scale Logical. If \code{TRUE}, uses log scale for the x-axis.
#'   Default is \code{NULL}, which auto-detects (\code{TRUE} for OR/HR/RR, \code{FALSE} for
#'   coefficients).
#'   
#' @param labels Named character vector providing custom display labels for 
#'   outcomes and variables. Applied to outcome names in the plot.
#'   Default is \code{NULL}, in which case the labels attached by
#'   \code{multifit()} are used. Original names are used where neither is
#'   available.
#' @param units Character string specifying units for plot dimensions: 
#'   \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.
#'
#' @param number_format Character string or two-element character vector
#'   controlling thousand and decimal separators in formatted output. Named
#'   presets:
#'   \itemize{
#'     \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
#'     \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
#'     \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
#'       (SI/ISO 31-0)
#'     \item \code{"none"} - No thousands separator: \code{1234.56}
#'   }
#'   Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
#'   \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.
#'
#'   When \code{NULL} (default), uses
#'   \code{getOption("summata.number_format", "us")}. Set the global option
#'   once per session to avoid passing this argument repeatedly:
#'   \preformatted{
#'     options(summata.number_format = "eu")
#'   }
#'
#' @return A \code{ggplot} object containing the complete forest plot. The plot 
#'   can be:
#'   \itemize{
#'     \item Displayed directly: \code{print(plot)}
#'     \item Saved to file: \code{forestsave(plot, "forest.pdf")}
#'     \item Further customized with \pkg{ggplot2} functions
#'   }
#'   
#'   The returned object includes an attribute \code{"rec_dims"} 
#'   accessible via \code{attr(plot, "rec_dims")}, which is a list 
#'   containing:
#'   \describe{
#'     \item{width}{Numeric. Recommended plot width in specified units}
#'     \item{height}{Numeric. Recommended plot height in specified units}
#'     \item{units}{Character. The units the dimensions are expressed in,
#'       matching the \code{units} argument}
#'   }
#'   
#'   These recommendations are automatically calculated based on the number of 
#'   variables, text sizes, and layout parameters, and are printed to console 
#'   if \code{plot_width} or \code{plot_height} are not specified.
#'   \code{forestsave()} reads all three and requires no further handling.
#'
#'   The returned object also includes an attribute \code{"table_data"}
#'   accessible via \code{attr(plot, "table_data")}, a data.table holding the
#'   values drawn in the plot, in the order of the model terms. Sample sizes
#'   and event counts describe the observations used in fitting rather than
#'   every row supplied.
#'
#' @details
#' \strong{Plot Layout:}
#' 
#' The forest plot is organized with outcomes as grouping headers and predictor
#' levels (or interaction terms) as rows within each outcome. This provides a
#' clear visual comparison of how a single predictor affects multiple outcomes.
#' 
#' \enumerate{
#'   \item \strong{Title}: Centered at top
#'   \item \strong{Data Table} (left): Contains:
#'     \itemize{
#'       \item Outcome column (or grouped headers)
#'       \item Predictor/Group column
#'       \item n: Sample sizes (optional)
#'       \item Events: Event counts (optional, for applicable models)
#'       \item Effect (95\% CI); \emph{p}-value
#'     }
#'   \item \strong{Forest Plot} (right):
#'     \itemize{
#'       \item Point estimates (squares)
#'       \item 95\% confidence intervals
#'       \item Reference line at null value (1 or 0)
#'       \item Log scale for ratio measures
#'     }
#' }
#' 
#' \strong{Data Source:}
#' 
#' The function extracts effect estimates directly from the multifit output
#' object's \code{raw_data} attribute, which contains the numeric values
#' needed for plotting. This approach is efficient and ensures consistency
#' with the formatted table output.
#'
#' @seealso 
#' \code{\link{autoforest}} for automatic model detection,
#' \code{\link{multifit}} for multi-outcome regression analysis,
#' \code{\link{glmforest}} for single GLM forest plots,
#' \code{\link{coxforest}} for single Cox model forest plots,
#' \code{\link{lmforest}} for single linear model forest plots,
#' \code{\link{uniforest}} for univariable screening forest plots,
#'   \code{\link{forestsave}} for saving with recommended dimensions
#'
#' @examples
#' data(clintrial)
#' data(clintrial_labels)
#' library(survival)
#'
#' # Create example multifit result
#' result <- multifit(
#'     data = clintrial,
#'     outcomes = c("surgery", "pfs_status", "os_status"),
#'     predictor = "treatment",
#'     covariates = c("age", "sex", "stage"),
#'     parallel = FALSE
#' )
#'
#' # Example 1: Basic multivariate forest plot
#' p <- multiforest(result)
#'
#' \donttest{
#' 
#' old_width <- options(width = 180)
#' 
#' # Example 2: With custom title and labels
#' plot2 <- multiforest(
#'     result,
#'     title = "Treatment Effects Across Clinical Outcomes",
#'     labels = clintrial_labels
#' )
#' 
#' # Example 3: Customize appearance
#' plot3 <- multiforest(
#'     result,
#'     color = "#E74C3C",
#'     zebra_stripes = TRUE,
#'     labels = clintrial_labels
#' )
#' 
#' # Example 4: Save with recommended dimensions
#' forestsave(plot3, file.path(tempdir(), "multioutcome_forest.pdf"))
#'
#' options(old_width)
#' 
#' }
#' @family visualization functions
#' @export
multiforest <- function(x,
                        title = "Multivariate Analysis",
                        effect_label = NULL,
                        column = "adjusted",
                        digits = 2,
                        p_digits = 3,
                        conf_level = 0.95,
                        font_size = 1.0,
                        annot_size = 3.88,
                        header_size = 5.82,
                        title_size = 23.28,
                        plot_width = NULL,
                        plot_height = NULL,
                        table_width = 0.6,
                        show_n = TRUE,
                        show_events = NULL,
                        show_predictor = NULL,
                        covariates_footer = TRUE,
                        indent_predictor = FALSE,
                        bold_variables = TRUE,
                        center_padding = 4,
                        zebra_stripes = TRUE,
                        color = NULL,
                        null_line = NULL,
                        log_scale = NULL,
                        labels = NULL,
                        units = "in",
                        number_format = NULL) {
    
    ## Input validation
    if (!requireNamespace("data.table", quietly = TRUE)) {
        stop("Package 'data.table' is required but not installed.")
    }
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required but not installed.")
    }
    
    ## Resolve number formatting marks
    validate_number_format(number_format)
    marks <- resolve_number_marks(number_format)
    
    ## Check if user passed a model object instead of a multifit result
    ## Common mistake since other forest functions (coxforest, glmforest) take models
    if (inherits(x, c("coxph", "glm", "lm", "glmerMod", "lmerMod", "coxme"))) {
        stop("multiforest() expects a multifit() result, not a model object.\n",
             "\nFor single-model forest plots, use:\n",
             "  - coxforest() for Cox models\n",
             "  - glmforest() for GLM models\n",
             "  - lmforest() for linear models\n",
             "\nFor multi-outcome analysis, first run multifit(), then pass the result to multiforest().")
    }
    
    if (!data.table::is.data.table(x)) {
        stop("multiforest() expects a multifit() result (data.table with raw_data attribute).\n",
             "\nInput is of class: ", paste(class(x), collapse = ", "))
    }
    
    ## Labels attached by the producing function are used when none are given
    ## here, so that a labeled result does not need them supplied twice. An
    ## explicit argument takes precedence.
    if (is.null(labels)) {
        labels <- attr(x, "labels")
    }

    raw_data <- attr(x, "raw_data")
    if (is.null(raw_data)) {
        stop("Input does not appear to be a multifit() result (missing raw_data attribute).\n",
             "\nmultiforest() requires output from multifit(). For single-model forest plots, use:\n",
             "  - coxforest() for Cox models\n",
             "  - glmforest() for GLM models\n",
             "  - lmforest() for linear models")
    }
    
    ## Extract multifit attributes
    mf_predictor <- attr(x, "predictor")
    mf_outcomes <- attr(x, "outcomes")
    mf_model_type <- attr(x, "model_type")
    mf_columns <- attr(x, "columns")
    
    ## Resolve show_predictor: use multifit attribute if not specified
    if (is.null(show_predictor)) {
        mf_include_predictor <- attr(x, "include_predictor")
        show_predictor <- if (!is.null(mf_include_predictor)) mf_include_predictor else TRUE
    }
    
    ## Validate column parameter
    column <- match.arg(column, c("adjusted", "unadjusted"))
    
    ## If multifit only has one column type, use that
    if (mf_columns %in% c("adjusted", "unadjusted")) {
        column <- mf_columns
    }
    
    ## Determine effect type and defaults
    effect_type <- raw_data$effect_type[1]
    if (is.na(effect_type)) effect_type <- "Estimate"
    
    if (is.null(null_line)) {
        null_line <- if (effect_type %in% c("OR", "HR", "RR")) 1 else 0
    }
    
    if (is.null(log_scale)) {
        log_scale <- effect_type %in% c("OR", "HR", "RR")
    }
    
    ## Determine adjustment prefix based on column parameter
    ## "adjusted" -> "a" prefix for OR/HR/RR, "Adj." for Coefficient
    ## "unadjusted" -> no prefix
    is_adjusted <- (column == "adjusted") || (mf_columns == "adjusted")
    
    if (effect_type %in% c("OR", "HR", "RR")) {
        adj_prefix <- if (is_adjusted) "a" else ""
        effect_abbrev <- paste0(adj_prefix, effect_type)  # \emph{e.g.,}, "aOR", "aHR", "HR", "OR"
    } else {
        adj_prefix <- if (is_adjusted) "Adj. " else ""
        effect_abbrev <- paste0(adj_prefix, "Coefficient")  # \emph{e.g.,}, "Adj. Coefficient" or "Coefficient"
    }
    
    ## Set effect_label for axis (full name)
    if (is.null(effect_label)) {
        effect_label <- switch(effect_type,
                               "OR" = "Odds Ratio",
                               "HR" = "Hazard Ratio",
                               "RR" = "Rate Ratio",
                               "Coefficient")
    }
    
    ## Format CI percentage for display in headers
    ci_pct <- round(conf_level * 100)
    
    ## Create header label for the effect column with italic p (\emph{e.g.,} "aOR (95% CI); \emph{p}-value")
    ## Format: bold('aOR (95% CI); '*bolditalic(p)*'-value')
    effect_header_expr <- paste0("bold('", effect_abbrev, " (", ci_pct, "% CI); '*bolditalic(p)*'-value')")
    
    if (is.null(show_events)) {
        show_events <- effect_type %in% c("OR", "HR", "RR")
    }
    
    ## Set default color based on effect type to match other forest plot conventions
    if (is.null(color)) {
        color <- switch(effect_type,
                        "HR" = "#8A61D8",
                        "OR" = "#4BA6B6",
                        "RR" = "#3F87EE",
                        "#5A8F5A")  # Default for Estimate/Coefficient
    }
    
    ## Prepare data - select appropriate columns for "both" multifit results
    dt <- data.table::copy(raw_data)
    
    ## For multifit results with both column types, select based on `column` param
    if ("exp_coef_adj" %in% names(dt)) {
        if (column == "adjusted") {
            dt[, `:=`(
                exp_coef = exp_coef_adj,
                ci_lower = ci_lower_adj,
                ci_upper = ci_upper_adj,
                p_value = p_adj
            )]
        } else {
            dt[, `:=`(
                exp_coef = exp_coef_unadj,
                ci_lower = ci_lower_unadj,
                ci_upper = ci_upper_unadj,
                p_value = p_unadj
            )]
        }
    }
    
    ## Apply labels to outcomes if provided
    if (!is.null(labels)) {
        for (orig_name in names(labels)) {
            dt[outcome == orig_name, outcome := labels[[orig_name]]]
        }
    }
    
    ## Convert to log scale using conf_level for CI calculation
    if ("coefficient" %in% names(dt)) {
        dt[, estimate := coefficient]
        if ("se" %in% names(dt)) {
            z_crit <- qnorm((1 + conf_level) / 2)
            dt[, conf_low := coefficient - z_crit * se]
            dt[, conf_high := coefficient + z_crit * se]
        } else {
            dt[, conf_low := log(ci_lower)]
            dt[, conf_high := log(ci_upper)]
        }
    } else {
        dt[, estimate := log(exp_coef)]
        dt[, conf_low := log(ci_lower)]
        dt[, conf_high := log(ci_upper)]
    }
    
    ## Rename columns to match glmforest format
    ## var = outcome (the "variable" in multifit context is the outcome)
    dt[, var := outcome]
    
    ## level = group (predictor level like "Drug A", "Drug B")
    dt[, level := data.table::fifelse(is.na(group) | group == "-", "-", group)]
    
    ## N and Events (uppercase to match glmforest)
    if ("n" %in% names(dt)) {
        data.table::setnames(dt, "n", "N")
    }
    if ("events" %in% names(dt)) {
        data.table::setnames(dt, "events", "Events")
    }
    
    ## Assign row positions (pos, var_order)
    outcomes <- unique(dt$var)
    
    rows_list <- list()
    row_counter <- 0
    
    for (i in seq_along(outcomes)) {
        outcome_name <- outcomes[i]
        outcome_rows <- dt[var == outcome_name]
        
        for (j in seq_len(nrow(outcome_rows))) {
            row_counter <- row_counter + 1
            data_row <- data.table::copy(outcome_rows[j])
            data_row[, `:=`(
                pos = j,
                var_order = i
            )]
            rows_list[[row_counter]] <- data_row
        }
    }
    
    dt <- data.table::rbindlist(rows_list, fill = TRUE)
    
    ## Add zebra stripe shading
    if (zebra_stripes) {
        ## Use (var_order + 1) %% 2 so first group (var_order=1) gets shade_group=0 (gray)
        dt[, shade_group := (var_order + 1) %% 2]
        shade_colors <- c("#EEEEEE", "#FFFFFF")  # 0 = gray, 1 = white
    } else {
        dt[, shade_group := 0]
        shade_colors <- c("#FFFFFF", "#FFFFFF")
    }
    
    ## Select final columns and sort
    cols_to_keep <- c("var", "level", "N", "Events", "p_value", "estimate", "conf_low", "conf_high", "pos", "var_order", "shade_group")
    cols_available <- intersect(cols_to_keep, names(dt))
    
    to_show <- dt[, ..cols_available]
    
    data.table::setorder(to_show, var_order, pos)
    
    ## Format the values for display
    to_show_exp_clean <- data.table::copy(to_show)
    
    ## Create formatted columns for display
    ## For log_scale = TRUE (OR, HR, RR): exponentiate values
    ## For log_scale = FALSE (linear models): use raw coefficients
    if (log_scale) {
        to_show_exp_clean[, hr := data.table::fifelse(is.na(estimate), 
                                                      NA_real_,
                                                      exp(estimate))]
        
        ## Format effect estimate (exponentiated)
        to_show_exp_clean[, hr_formatted := data.table::fifelse(is.na(estimate),
                                                                "",
                                                                format_number(exp(estimate), digits, marks))]
        
        to_show_exp_clean[, conf_low_formatted := data.table::fifelse(is.na(conf_low), 
                                                                      "",
                                                                      format_number(exp(conf_low), digits, marks))]
        to_show_exp_clean[, conf_high_formatted := data.table::fifelse(is.na(conf_high), 
                                                                       "",
                                                                       format_number(exp(conf_high), digits, marks))]
    } else {
        ## Linear scale - use raw coefficients
        to_show_exp_clean[, hr := data.table::fifelse(is.na(estimate), 
                                                      NA_real_,
                                                      estimate)]
        
        ## Format effect estimate (raw coefficient)
        to_show_exp_clean[, hr_formatted := data.table::fifelse(is.na(estimate),
                                                                "",
                                                                format_number(estimate, digits, marks))]
        
        to_show_exp_clean[, conf_low_formatted := data.table::fifelse(is.na(conf_low), 
                                                                      "",
                                                                      format_number(conf_low, digits, marks))]
        to_show_exp_clean[, conf_high_formatted := data.table::fifelse(is.na(conf_high), 
                                                                       "",
                                                                       format_number(conf_high, digits, marks))]
    }

    ## Format \emph{p}-values using p_digits parameter
    p_threshold <- 10^(-p_digits)
    p_threshold_str <- if (!is.null(marks)) {
                           paste0("< 0", marks$decimal.mark, strrep("0", p_digits - 1), "1")
                       } else {
                           paste0("< ", format(p_threshold, scientific = FALSE))
                       }
    
    to_show_exp_clean[, p_formatted := data.table::fifelse(is.na(p_value), 
                                                           "",
                                                           data.table::fifelse(p_value < p_threshold, 
                                                                               p_threshold_str,
                                                                               format_number(p_value, p_digits, marks)))]

    ## Determine if ANY coefficient or CI bound is negative (only relevant for linear scale)
    ## If so, use comma notation throughout for consistency
    use_comma_notation <- !log_scale && any(to_show_exp_clean$conf_low < 0 | to_show_exp_clean$conf_high < 0, na.rm = TRUE)
    ci_separator <- forest_ci_separator(use_comma_notation, marks)

    ## Create the combined effect string with expression for italic p
    to_show_exp_clean[, hr_string_expr := data.table::fifelse(
                                                          is.na(estimate),
                                                          "''",  # Empty string for header rows
                                                          data.table::fcase(
                                                              p_value < p_threshold,
                                                              paste0("'", hr_formatted, " (", conf_low_formatted, ci_separator, 
                                                                     conf_high_formatted, "); '*~italic(p)~'", p_threshold_str, "'"),
                                                              
                                                              default = paste0("'", hr_formatted, " (", conf_low_formatted, ci_separator, 
                                                                               conf_high_formatted, "); '*~italic(p)~'= ", p_formatted, "'")
                                                          )
                                                      )]

    ## Format N and events with locale-aware thousands separator
    to_show_exp_clean[, n_formatted := format_count(N, marks, na = "")]
    to_show_exp_clean[, events_formatted := format_count(as.integer(Events), marks,
                                                         na = "")]
    
    ## Clean up variable names for display
    to_show_exp_clean[, var_display := as.character(var)]

    if (indent_predictor) {
        ## Apply labels to outcome names
        for (v in unique(to_show_exp_clean$var)) {
            if (v != "" && !grepl("^    ", v)) {
                if (!is.null(labels) && v %in% names(labels)) {
                    to_show_exp_clean[var == v, var_display := labels[v]]
                }
            }
        }
        
        ## Create indented display: outcome header rows + indented predictor rows
        outcomes <- unique(to_show_exp_clean$var)
        new_rows <- list()
        row_idx <- 0
        
        for (i in seq_along(outcomes)) {
            outcome_name <- outcomes[i]
            outcome_rows <- to_show_exp_clean[var == outcome_name]
            outcome_label <- outcome_rows$var_display[1]
            
            ## shade_group: first outcome (i = 1) should be 0 (gray)
            current_shade <- (i + 1) %% 2
            
            ## Add header row for outcome (bold, no estimate)
            row_idx <- row_idx + 1
            header_row <- data.table::data.table(
                                          var = outcome_name,
                                          level = "",
                                          N = NA_integer_,
                                          Events = NA_real_,
                                          p_value = NA_real_,
                                          estimate = NA_real_,
                                          conf_low = NA_real_,
                                          conf_high = NA_real_,
                                          pos = 0L,
                                          var_order = as.integer(i),
                                          shade_group = current_shade,
                                          var_display = outcome_label,
                                          is_header = TRUE,
                                          hr_formatted = "",
                                          conf_low_formatted = "",
                                          conf_high_formatted = "",
                                          p_formatted = "",
                                          hr_string_expr = "''",
                                          n_formatted = "",
                                          events_formatted = ""
                                      )
            new_rows[[row_idx]] <- header_row
            
            ## Add indented predictor level rows
            for (j in seq_len(nrow(outcome_rows))) {
                row_idx <- row_idx + 1
                data_row <- data.table::copy(outcome_rows[j])
                data_row[, `:=`(
                    var_display = paste0("    ", level),  # Indent predictor level
                    is_header = FALSE,
                    shade_group = current_shade  # Match header row shading
                )]
                new_rows[[row_idx]] <- data_row
            }
        }
        
        to_show_exp_clean <- data.table::rbindlist(new_rows, fill = TRUE)
        to_show_exp_clean[, level := ""]  # Clear level column since it's now in var_display
        
    } else {
        ## Without indent_predictor: show outcome and predictor in separate columns
        ## Apply labels to outcome names
        for (v in unique(to_show_exp_clean$var)) {
            if (v %in% to_show_exp_clean$var) {
                if (!is.null(labels) && v %in% names(labels)) {
                    to_show_exp_clean[var == v, var_display := labels[v]]
                }
            }
        }
        
        ## For non-indented display, avoid repeating outcome name for same outcome
        to_show_exp_clean[duplicated(var), var_display := ""]
        to_show_exp_clean[, is_header := FALSE]
    }
    
    ## Handle missing estimates for plotting
    to_show_exp_clean[is.na(estimate), estimate := 0]
    
    ## Reorder (flip) - but maintain the variable grouping
    ## Retain the assembled table for return as an attribute. The copy is taken
    ## before the row order is reversed for plotting, so that the returned table
    ## follows the order of the model terms rather than the drawing order.
    table_data <- data.table::copy(to_show_exp_clean)
    
    to_show_exp_clean <- to_show_exp_clean[order(rev(seq_len(nrow(to_show_exp_clean))))]
    to_show_exp_clean[, x_pos := .I]
    
    ## Calculate plot ranges - different for log vs linear scale
    if (log_scale) {
        ## Log scale (OR, HR, RR)
        rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
        
        min_ci <- min(to_show$conf_low, na.rm = TRUE)
        max_ci <- max(to_show$conf_high, na.rm = TRUE)
        
        ## Intelligent tick selection for log scale
        range_magnitude <- diff(rangeb)
        
        if (exp(min_ci) < 0.01 && exp(max_ci) > 2) {
            breaks <- c(0.01, 0.1, 0.5, 1, 2, 5)
        } else if (range_magnitude > 3) {
            all_breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
            if (length(all_breaks) > 7) {
                important <- c(1)
                if (min(all_breaks) < 0.5) important <- c(min(all_breaks), important)
                if (max(all_breaks) > 2) important <- c(important, max(all_breaks))
                
                other_breaks <- setdiff(all_breaks, important)
                if (length(other_breaks) > 3) {
                    keep_idx <- round(seq(1, length(other_breaks), length.out = 3))
                    other_breaks <- other_breaks[keep_idx]
                }
                breaks <- sort(unique(c(important, other_breaks)))
            } else {
                breaks <- all_breaks
            }
        } else {
            breaks <- grDevices::axisTicks(rangeb/2, log = TRUE, nint = 7)
        }
        
        if (!1 %in% breaks) {
            breaks <- sort(unique(c(breaks, 1)))
        }
        
        if (min_ci > 0) {
            rangeb[1] <- log(0.9)
        } else if(max_ci < 0) {
            rangeb[2] <- log(1.1)
        }
        
        breaks <- breaks[breaks >= exp(rangeb[1]) & breaks <= exp(rangeb[2])]
        reference_value <- 1
    } else {
        ## Linear scale (coefficients)
        rangeb <- range(to_show$conf_low, to_show$conf_high, na.rm = TRUE)
        
        ## Add some padding
        range_width <- diff(rangeb)
        rangeb[1] <- rangeb[1] - range_width * 0.05
        rangeb[2] <- rangeb[2] + range_width * 0.05
        
        ## Ensure 0 is included in the range for reference line
        if (rangeb[1] > 0) rangeb[1] <- -0.1 * abs(rangeb[2])
        if (rangeb[2] < 0) rangeb[2] <- 0.1 * abs(rangeb[1])
        
        ## Generate breaks for linear scale
        breaks <- pretty(rangeb, n = 7)
        breaks <- breaks[breaks >= rangeb[1] & breaks <= rangeb[2]]
        
        if (!0 %in% breaks) {
            breaks <- sort(unique(c(breaks, 0)))
        }
        
        reference_value <- 0
    }

    ## Calculate layout
    layout <- calculate_multiforest_layout(
        to_show_exp_clean = to_show_exp_clean,
        show_n = show_n,
        show_events = show_events,
        indent_predictor = indent_predictor,
        show_predictor = show_predictor,
        effect_abbrev = effect_abbrev,
        font_size = font_size,
        table_width = data.table::fifelse(is.null(table_width), 0.6, table_width),
        rangeb = rangeb,
        center_padding = center_padding,
        ci_pct = ci_pct
    )

    ## Set up the extended range for plotting
    rangeplot <- c(layout$rangeplot_start, rangeb[2] + diff(rangeb) * 0.05)

    ## Extract positions
    y_outcome <- layout$positions$outcome
    if (!indent_predictor && show_predictor) {
        y_predictor <- layout$positions$predictor
    }
    if (show_n) {
        y_n <- layout$positions$n
    }
    if (show_events) {
        y_events <- layout$positions$events
    }
    y_effect <- layout$positions$effect

    ## Calculate recommended dimensions
    rec_height <- max(5, min(20, 3 + nrow(to_show_exp_clean) * 0.25))

    if(!is.null(plot_width)) {
        rec_width <- plot_width
        if(!is.null(plot_height)) {
            rec_height <- plot_height
        }
    } else {
        rec_width <- layout$total_width + 1.0
        rec_width <- max(10, min(20, rec_width))
    }

    ## Font sizes
    annot_font <- font_size * annot_size
    header_font <- font_size * header_size
    
    ## Custom ticks data
    ticks_df <- data.frame(
        x = -0.5,
        xend = -0.7,
        y = breaks,
        yend = breaks
    )

    ## Create filtered data for forest plot elements (exclude header rows)
    plot_data <- if ("is_header" %in% names(to_show_exp_clean)) {
                     to_show_exp_clean[is_header == FALSE | is.na(is_header)]
                 } else {
                     to_show_exp_clean
                 }

    ## Define coordinate transformation based on scale type
    ## For log scale: use exp() to transform from log to original scale
    ## For linear scale: use identity (no transformation)
    tfm <- if (log_scale) exp else identity

    ## Create the plot
    p <- ggplot2::ggplot(to_show_exp_clean, ggplot2::aes(x_pos, tfm(estimate))) +
        
        ## Shading rectangles
        ggplot2::geom_rect(ggplot2::aes(xmin = x_pos - .5, xmax = x_pos + .5,
                                        ymin = tfm(rangeplot[1]), ymax = tfm(rangeplot[2]),
                                        fill = ordered(shade_group))) +
        ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.10, 0.05))) +
        ## Size scaling: use sqrt transformation with reasonable limits
        ## This ensures points are visible even when N values are similar
        ggplot2::scale_size_area(max_size = 6, guide = "none") +
        ggplot2::scale_fill_manual(values = shade_colors, guide = "none") +
        
        ## Forest plot elements (use filtered data without header rows)
        ggplot2::geom_point(data = plot_data, ggplot2::aes(size = N), pch = 22, color = "#000000", fill = color, na.rm = TRUE) +
        ggplot2::geom_errorbar(data = plot_data, ggplot2::aes(ymin = tfm(conf_low), ymax = tfm(conf_high)), width = 0.15, na.rm = TRUE) +
        
        ## Y-axis for forest plot
        ggplot2::annotate(geom = "segment",
                          x = -0.5, xend = -0.5,
                          y = tfm(rangeb[1]),
                          yend = tfm(rangeb[2]),
                          color = "#000000", linewidth = 1) +
        
        ## Reference line (1 for log scale, 0 for linear)
        ggplot2::annotate(geom = "segment", 
                          x = -0.5, xend = max(to_show_exp_clean$x_pos) + 0.5, 
                          y = reference_value, yend = reference_value, 
                          linetype = "longdash") +
        
        ## Ticks
        ggplot2::geom_segment(data = ticks_df,
                              ggplot2::aes(x = x, xend = xend, y = y, yend = yend),
                              inherit.aes = FALSE,
                              color = "#000000",
                              linewidth = 1) +
        ggplot2::geom_text(data = ticks_df,
                           ggplot2::aes(x = xend - 0.05, y = y, label = sprintf("%g", y)),
                           inherit.aes = FALSE,
                           hjust = 0.5,
                           vjust = 1.3,
                           size = annot_font * 1.5) +
        
        ## Set coordinate system with extended limits
        ggplot2::coord_flip(ylim = tfm(rangeplot)) +
        ggplot2::ggtitle(title) +
        
        ## Y-axis scale (log10 or continuous)
        {if (log_scale) {
             ggplot2::scale_y_log10(name = effect_label,
                                    labels = sprintf("%g", breaks),
                                    expand = c(0.02, 0.02),
                                    breaks = breaks)
         } else {
             ggplot2::scale_y_continuous(name = effect_label,
                                         labels = sprintf("%g", breaks),
                                         expand = c(0.02, 0.02),
                                         breaks = breaks)
         }} +
        
        ggplot2::theme_light(base_family = detect_plot_font()) +
        ggplot2::theme(plot.margin = ggplot2::margin(t = 10, r = 0, b = 0, l = 0),
                       panel.grid.minor.y = ggplot2::element_blank(),
                       panel.grid.minor.x = ggplot2::element_blank(),
                       panel.grid.major.y = ggplot2::element_blank(),
                       panel.grid.major.x = ggplot2::element_blank(),
                       legend.position = "none",
                       panel.border = ggplot2::element_blank(),
                       axis.title.y = ggplot2::element_blank(),
                       axis.text.y = ggplot2::element_blank(),
                       axis.ticks.y = ggplot2::element_blank(),
                       axis.title.x = ggplot2::element_blank(),
                       axis.ticks.x = ggplot2::element_blank(),
                       axis.text.x = ggplot2::element_blank(),
                       plot.title = ggplot2::element_text(size = font_size * title_size, face = "bold", hjust = 0.5)) +
        ggplot2::xlab("") +
        
        ## Outcome column header
        ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_outcome),
                          label = "Outcome", fontface = "bold", hjust = 0,
                          size = header_font) +

    {if (indent_predictor) {
         ## When indented, use conditional formatting for headers vs data rows
         fontfaces <- if (bold_variables) {
                          data.table::fifelse(to_show_exp_clean$is_header, "bold", "plain")
                      } else {
                          rep("plain", nrow(to_show_exp_clean))
                      }
         ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_outcome),
                           label = to_show_exp_clean$var_display, 
                           fontface = fontfaces, 
                           hjust = 0,
                           size = annot_font)
     } else {
         ## Non-indented: outcome names are bold if bold_variables is TRUE
         ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_outcome),
                           label = to_show_exp_clean$var_display, 
                           fontface = if (bold_variables) "bold" else "plain", 
                           hjust = 0,
                           size = annot_font)
     }} +
    
    ## Predictor level column (only shown when not indented AND show_predictor is TRUE)
    {if (!indent_predictor && show_predictor) {
         list(
             ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_predictor),
                               label = "Predictor", fontface = "bold", hjust = 0,
                               size = header_font),
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_predictor),
                               label = to_show_exp_clean$level, hjust = 0,
                               size = annot_font)
         )
     }} +
    
    ## N column (conditional)
    {if (show_n) {
         list(
             ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_n),
                               label = "n", fontface = "bold.italic", hjust = 0.5,
                               size = header_font),
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_n),
                               label = to_show_exp_clean$n_formatted, hjust = 0.5,
                               size = annot_font)
         )
     }} +
    
    ## Events column (conditional)
    {if (show_events) {
         list(
             ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_events),
                               label = "Events", fontface = "bold", hjust = 0.5,
                               size = header_font),
             ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_events),
                               label = to_show_exp_clean$events_formatted, hjust = 0.5,
                               size = annot_font)
         )
     }} +
    
    ## Effect column header (with italic p)
    ggplot2::annotate(geom = "text", x = max(to_show_exp_clean$x_pos) + 1.5, y = tfm(y_effect),
                      label = effect_header_expr,
                      hjust = 0, size = header_font, parse = TRUE) +
    
    ggplot2::annotate(geom = "text", x = to_show_exp_clean$x_pos, y = tfm(y_effect),
                      label = to_show_exp_clean$hr_string_expr, hjust = 0,
                      size = annot_font, parse = TRUE) +
    
    ## X-axis label
    ggplot2::annotate(geom = "text", x = -1.5, y = reference_value,
                      label = effect_label, fontface = "bold",
                      hjust = 0.5, vjust = 2, size = annot_font * 1.5) +
    
    ## Covariates footer (if adjusted model with covariates and covariates_footer = TRUE)
    {
        mf_covariates <- attr(x, "covariates")
        if (covariates_footer && !is.null(mf_covariates) && length(mf_covariates) > 0 && is_adjusted) {
            ## Apply labels to covariate names if available
            covar_display <- sapply(mf_covariates, function(cv) {
                if (!is.null(labels) && cv %in% names(labels)) {
                    labels[[cv]]
                } else {
                    cv
                }
            })
            
            ## Format covariates list, truncating if very long
            if (length(covar_display) > 8) {
                covar_text <- paste0("Covariates: ", 
                                     paste(covar_display[1:8], collapse = ", "),
                                     ", ... (", length(covar_display), " total)")
            } else {
                covar_text <- paste0("Covariates: ", paste(covar_display, collapse = ", "))
            }
            ggplot2::annotate(geom = "text", x = 0.25, y = tfm(y_outcome),
                              label = covar_text,
                              size = annot_font, hjust = 0, vjust = 1.2, fontface = "italic")
        }
    }
    
    ## Convert units back for output if needed
    if (units != "in") {
        rec_width <- convert_units(rec_width, from = "in", to = units)
        rec_height <- convert_units(rec_height, from = "in", to = units)
    }

    ## Provide dimension recommendations
    if(is.null(plot_width) || is.null(plot_height)) {
        message(sprintf("Recommended plot dimensions: width = %.1f %s, height = %.1f %s",
                        rec_width, units, rec_height, units))
    }
    
    ## Add recommended dimensions as an attribute
    ## The units are recorded alongside the dimensions. Both are converted to
    ## the requested units above, so a consumer that assumed inches would
    ## otherwise mis-size the output by the conversion factor.
    attr(p, "rec_dims") <- list(width = rec_width, height = rec_height,
                                units = units)

    ## Add the assembled table as an attribute
    attr(p, "table_data") <- table_data

    return(p)
}


### * Layout formatting

#' Calculate table layout for multiforest plots
#' 
#' Computes column widths and positions for the table portion of a multiforest plot.
#' 
#' @keywords internal
calculate_multiforest_layout <- function(to_show_exp_clean, show_n, show_events, 
                                         indent_predictor, show_predictor = TRUE,
                                         effect_abbrev,
                                         font_size, table_width = 0.6, rangeb, 
                                         center_padding, ci_pct = 95) {
    
    char_to_inch <- 0.08 * font_size
    margin <- 0.3  # inches between columns
    
    ## Build list of active columns with their widths
    columns <- list()
    
    ## Always have outcome column
    outcome_width_chars <- max(nchar(to_show_exp_clean$var_display), nchar("Outcome"), na.rm = TRUE)
    columns$outcome <- outcome_width_chars * char_to_inch
    
    ## Conditionally add predictor column (only when not indented AND show_predictor is TRUE)
    if (!indent_predictor && show_predictor) {
        predictor_width_chars <- max(nchar(to_show_exp_clean$level), nchar("Predictor"), na.rm = TRUE)
        columns$predictor <- predictor_width_chars * char_to_inch + 0.2
    }
    
    ## Conditionally add n column
    if (show_n) {
        n_width_chars <- max(nchar(to_show_exp_clean$n_formatted), nchar("____"), na.rm = TRUE)
        columns$n <- n_width_chars * char_to_inch
    }
    
    ## Conditionally add events column
    if (show_events) {
        events_width_chars <- max(nchar(to_show_exp_clean$events_formatted), nchar("Events"), na.rm = TRUE)
        columns$events <- events_width_chars * char_to_inch
    }
    
    ## Always have effect column - use dynamic CI percentage
    effect_header <- paste0(effect_abbrev, " (", ci_pct, "% CI); p-value")
    
    ## Calculate max effect string length
    effect_lengths <- nchar(paste0(
        to_show_exp_clean$hr_formatted, " (",
        to_show_exp_clean$conf_low_formatted, "-",
        to_show_exp_clean$conf_high_formatted, "); p = ",
        to_show_exp_clean$p_formatted
    ))
    
    effect_width_chars <- max(effect_lengths, nchar(effect_header), na.rm = TRUE) + center_padding
    columns$effect <- effect_width_chars * char_to_inch
    
    ## Calculate total table width
    calc_table_width <- sum(unlist(columns)) + length(columns) * margin * 2
    
    ## Calculate forest width based on table_width proportion
    calc_forest_width <- calc_table_width * ((1 - table_width) / table_width)
    
    ## Convert table width to log scale units
    calc_range_width <- diff(rangeb)
    calc_table_width_log_units <- (calc_table_width / calc_forest_width) * calc_range_width
    
    ## Calculate positions in log scale
    rangeplot_start <- rangeb[1] - calc_table_width_log_units
    
    ## Convert inch measurements to log scale units
    inch_to_log <- calc_range_width / calc_forest_width
    
    positions <- list()
    current_pos <- rangeplot_start
    
    for (name in names(columns)) {
        current_pos <- current_pos + margin * inch_to_log
        positions[[name]] <- current_pos
        current_pos <- current_pos + columns[[name]] * inch_to_log + margin * inch_to_log
    }
    
    return(list(
        calc_table_width = calc_table_width,
        calc_forest_width = calc_forest_width,
        positions = positions,
        rangeplot_start = rangeplot_start,
        total_width = calc_table_width + calc_forest_width
    ))
}

### * Main function

#' Export Table to Microsoft PowerPoint Format (PPTX)
#'
#' Converts a data frame, data.table, or matrix to a Microsoft PowerPoint slide 
#' (\code{.pptx}) with a formatted table using the \pkg{flextable} and \pkg{officer}
#' packages. Creates presentation-ready slides with extensive control over table
#' formatting, positioning, and layout. Tables can be further edited in PowerPoint
#' after creation. Ideal for creating data-driven presentations and conference talks.
#'
#' @param table Data frame, data.table, or matrix to export. Can be output from 
#'   \code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
#'   \code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.
#'   
#' @param file Character string specifying the output PPTX filename. Must have 
#'   \code{.pptx} extension. Example: \code{"results.pptx"}, \code{"Slide1.pptx"}.
#'   
#' @param caption Character string. Optional title displayed in the slide's title 
#'   placeholder or as text box above the table. Default is \code{NULL}.
#'   
#' @param font_size Numeric. Base font size in points for table content. 
#'   Default is 10. Typical range for presentations: 10-14 points. Larger than 
#'   print documents for visibility at distance.
#'   
#' @param font_family Character string. Font family name for the table. Must be 
#'   installed on the system. Default is \code{"Arial"}. Common presentation 
#'   fonts: \code{"Calibri"}, \code{"Helvetica"}, \code{"Arial"}.
#'   
#' @param format_headers Logical. If \code{TRUE}, formats column headers by 
#'   italicizing statistical notation ("\emph{n}", "\emph{p}") and improving readability. 
#'   Default is \code{TRUE}.
#'   
#' @param bold_significant Logical. If \code{TRUE}, applies bold formatting to 
#'   \emph{p}-values below the significance threshold. Makes important results stand 
#'   out in presentations. Default is \code{TRUE}.
#'
#' @param bold_variables Logical. If \code{TRUE}, variable names are displayed
#'   in bold. Default is \code{FALSE}.
#'   
#' @param p_threshold Numeric. Threshold for bold \emph{p}-value formatting. Only 
#'   used when \code{bold_significant = TRUE}. Default is 0.05.
#'   
#' @param indent_groups Logical. If \code{TRUE}, indents factor levels under 
#'   their parent variable, creating hierarchical display. Useful for categorical 
#'   variables. Default is \code{FALSE}.
#'   
#' @param condense_table Logical. If \code{TRUE}, condenses table to essential 
#'   rows only. Automatically sets \code{indent_groups = TRUE}. Crucial for 
#'   fitting content on slides. Default is \code{FALSE}.
#'   
#' @param condense_quantitative Logical. If \code{TRUE}, condenses continuous 
#'   and survival variables into single rows while preserving all categorical 
#'   variable rows (including binary). Only applies to descriptive tables from 
#'   \code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
#'   \code{condense_table}, this does not collapse binary categorical variables. 
#'   Default is \code{FALSE}.
#'   
#' @param zebra_stripes Logical. If \code{TRUE}, applies alternating row shading 
#'   to different variables for visual grouping. Improves readability during 
#'   presentations. Default is \code{FALSE}.
#'   
#' @param dark_header Logical. If \code{TRUE}, creates dark background with 
#'   light text for header row. Provides strong contrast visible from distance. 
#'   Default is \code{FALSE}.
#'   
#' @param width Numeric. Table width in inches. If \code{NULL} (default), 
#'   auto-fits to slide width (approximately 9 inches for standard 10-inch slide 
#'   with margins). Specify for exact control.
#'   
#' @param align Character vector specifying column alignment. Options: 
#'   \code{"left"}, \code{"center"}, or \code{"right"}. If \code{NULL} (default), 
#'   automatically determines based on content.
#'   
#' @param template Character string. Path to custom PPTX template file. If 
#'   \code{NULL} (default), uses officer's default blank template. Use to match 
#'   corporate branding or conference themes.
#'   
#' @param layout Character string. Name of slide layout to use from template. 
#'   Default is \code{"Title and Content"}. Common layouts:
#'   \itemize{
#'     \item \code{"Title and Content"} - Standard layout [default]
#'     \item \code{"Blank"} - Empty slide for maximum control
#'     \item \code{"Title Only"} - Title area only
#'     \item \code{"Two Content"} - Title with two content areas
#'   }
#'   
#' @param master Character string. Name of slide master to use. Default is 
#'   \code{"Office Theme"}. Varies by template. Check template for available masters.
#'   
#' @param left Numeric. Horizontal position from left edge of slide in inches. 
#'   Default is 0.5. Standard slide is 10 inches wide.
#'   
#' @param top Numeric. Vertical position from top edge of slide in inches. 
#'   Default is 1.5 (leaves room for title). Standard slide is 7.5 inches tall. 
#'   Adjust based on table size and layout.
#'   
#' @param return_ft Logical. If \code{TRUE}, returns the \pkg{flextable} object 
#'   directly for further customization. If \code{FALSE} (default), returns 
#'   invisibly with \pkg{flextable} object as attribute. See Details for usage. 
#'   Default is \code{FALSE}.
#'   
#' @param ... Additional arguments passed to \code{\link[officer]{read_pptx}}.
#'
#' @param quiet Logical. Suppress progress and confirmation messages, such
#'   as the notice reporting the file written. Errors and warnings are
#'   unaffected. Default is \code{FALSE}.
#'
#' @return Behavior depends on \code{return_ft}:
#'   \describe{
#'     \item{\code{return_ft = FALSE}}{Invisibly returns a list with:
#'       \itemize{
#'         \item \code{file} - Path to created file
#'         \item \code{caption} - Caption/title text
#'         \item \code{layout} - Layout name used
#'         \item \code{master} - Master name used
#'         \item \code{template} - Template path (if provided)
#'         \item \code{position} - List with \code{left} and \code{top} coordinates
#'       }
#'       Flextable accessible via \code{attr(result, "flextable")}
#'     }
#'     \item{\code{return_ft = TRUE}}{Directly returns the \pkg{flextable} object}
#'   }
#'   
#'   Always creates a \code{.pptx} file at the specified location.
#'
#' @details
#' \strong{Package Requirements:}
#' 
#' Requires:
#' \itemize{
#'   \item \strong{\pkg{flextable}} - Table creation and formatting
#'   \item \strong{\pkg{officer}} - PowerPoint manipulation
#' }
#' 
#' Install: \code{install.packages(c("flextable", "officer"))}
#' 
#' \strong{Slide Dimensions:}
#' 
#' Standard PowerPoint slide:
#' \itemize{
#'   \item Width: 10 inches (25.4 cm)
#'   \item Height: 7.5 inches (19.05 cm)
#'   \item Aspect ratio: 4:3 (standard) or 16:9 (widescreen)
#' }
#' 
#' Safe content area (with margins):
#' \itemize{
#'   \item Width: ~9 inches
#'   \item Height: ~6 inches (accounting for title)
#' }
#' 
#' \strong{Positioning:}
#' 
#' The \code{left} and \code{top} parameters control table placement:
#' \itemize{
#'   \item (0, 0) = Top-left corner of slide
#'   \item Default (0.5, 1.5) = Standard position with title room
#'   \item Center: \code{left = (10 - table_width) / 2}
#' }
#' 
#' When caption is provided:
#' \itemize{
#'   \item Attempts to use title placeholder (if layout supports)
#'   \item Falls back to text box above table
#'   \item Automatically adjusts table position downward
#' }
#' 
#' \strong{Slide Layouts:}
#' 
#' Different layouts serve different purposes:
#' 
#' \emph{Title and Content (default):}
#' \itemize{
#'   \item Has title and content placeholders
#'   \item Caption goes in title area
#'   \item Table in content area
#'   \item Most common for data slides
#' }
#' 
#' \emph{Blank:}
#' \itemize{
#'   \item No predefined areas
#'   \item Maximum flexibility
#'   \item Use absolute positioning (\code{left}, \code{top})
#'   \item Good for custom layouts
#' }
#' 
#' \emph{Title-Only:}
#' \itemize{
#'   \item Title area only
#'   \item Large space for table
#'   \item Good for data-heavy slides
#' }
#' 
#' \strong{Custom Templates:}
#' 
#' Use organizational or conference templates:
#' \preformatted{
#' table2pptx(table, "branded.pptx",
#'          template = "company_template.pptx",
#'          layout = "Content Layout",  # Name from template
#'          master = "Company Theme")   # Name from template
#' }
#' 
#' To find layout and master names in template:
#' \preformatted{
#' pres <- officer::read_pptx("template.pptx")
#' officer::layout_summary(pres)
#' }
#' 
#' \strong{Multiple Slides:}
#' 
#' Creating presentations with multiple tables:
#' \preformatted{
#' # Each call creates new presentation - combine after
#' table2pptx(table1, "slide1.pptx", caption = "Results Part 1")
#' table2pptx(table2, "slide2.pptx", caption = "Results Part 2")
#' 
#' # Then manually combine in PowerPoint, or:
#' # Use officer to create multi-slide presentation
#' pres <- officer::read_pptx()
#' 
#' # Add first table
#' ft1 <- table2pptx(table1, "temp1.pptx", return_ft = TRUE)
#' pres <- officer::add_slide(pres)
#' pres <- officer::ph_with(pres, ft1, 
#'                          location = officer::ph_location(left = 0.5, top = 1.5))
#' 
#' # Add second table
#' ft2 <- table2pptx(table2, "temp2.pptx", return_ft = TRUE)
#' pres <- officer::add_slide(pres)
#' pres <- officer::ph_with(pres, ft2,
#'                          location = officer::ph_location(left = 0.5, top = 1.5))
#' 
#' print(pres, target = "combined.pptx")
#' }
#' 
#' \strong{Further Customization:}
#' 
#' Access the \code{flextable} object for advanced formatting:
#' \preformatted{
#' ft <- table2pptx(table, "base.pptx", return_ft = TRUE)
#' 
#' # Customize
#' ft <- flextable::color(ft, j = "p-value", color = "red")
#' ft <- flextable::bg(ft, i = 1, bg = "yellow")
#' ft <- flextable::bold(ft, i = ~ estimate > 0, j = "estimate")
#' 
#' # Save to new slide
#' pres <- officer::read_pptx()
#' pres <- officer::add_slide(pres)
#' pres <- officer::ph_with(pres, ft, 
#'                          location = officer::ph_location(left = 0.5, top = 1.5))
#' print(pres, target = "custom.pptx")
#' }
#' 
#' @seealso
#' \code{\link{tablesave}} for saving in the format given by the file extension,
#' \code{\link{table2docx}} for Word documents,
#' \code{\link{table2pdf}} for PDF output,
#' \code{\link{table2html}} for HTML tables,
#' \code{\link{table2rtf}} for Rich Text Format,
#' \code{\link{table2tex}} for LaTeX output,
#' \code{\link[flextable]{flextable}} for table customization,
#' \code{\link[officer]{read_pptx}} for PowerPoint manipulation
#'
#' @examples
#' # Create example data
#' data(clintrial)
#' data(clintrial_labels)
#' tbl <- desctable(clintrial, by = "treatment",
#'     variables = c("age", "sex"), labels = clintrial_labels)
#'
#' # Basic PowerPoint export
#' if (requireNamespace("flextable", quietly = TRUE) &&
#'     requireNamespace("officer", quietly = TRUE)) {
#'   table2pptx(tbl, file.path(tempdir(), "example.pptx"))
#' }
#'
#' \donttest{
#' old_width <- options(width = 180)
#' # Load data
#' data(clintrial)
#' data(clintrial_labels)
#' 
#' # Create regression table
#' results <- fit(
#'    data = clintrial,
#'    outcome = "os_status",
#'    predictors = c("age", "sex", "treatment"),
#'    labels = clintrial_labels
#' )
#' 
#' # Example 1: Basic PowerPoint slide
#' table2pptx(results, file.path(tempdir(), "results.pptx"))
#' 
#' # Example 2: With title
#' table2pptx(results, file.path(tempdir(), "titled.pptx"),
#'         caption = "Multivariable Regression Results")
#' 
#' # Example 3: Larger font for visibility
#' table2pptx(results, file.path(tempdir(), "large_font.pptx"),
#'         font_size = 12,
#'         caption = "Main Findings")
#' 
#' # Example 4: Condensed for slide space
#' table2pptx(results, file.path(tempdir(), "condensed.pptx"),
#'         condense_table = TRUE,
#'         caption = "Key Results")
#' 
#' # Example 5: Dark header for emphasis
#' table2pptx(results, file.path(tempdir(), "dark.pptx"),
#'         dark_header = TRUE,
#'         caption = "Risk Factors")
#' 
#' # Example 6: With zebra stripes
#' table2pptx(results, file.path(tempdir(), "striped.pptx"),
#'         zebra_stripes = TRUE)
#' 
#' # Example 7: Blank layout with custom positioning
#' table2pptx(results, file.path(tempdir(), "blank.pptx"),
#'         layout = "Blank",
#'         left = 1,
#'         top = 1.5,
#'         width = 8)
#' 
#' # Example 8: Get flextable for customization
#' ft <- table2pptx(results, file.path(tempdir(), "base.pptx"), return_ft = TRUE)
#' 
#' # Customize the returned flextable object
#' ft <- flextable::color(ft, j = "p-value", color = "darkred")
#' 
#' # Example 9: Presentation-optimized table
#' table2pptx(results, file.path(tempdir(), "presentation.pptx"),
#'         caption = "Main Analysis Results",
#'         font_size = 11,
#'         condense_table = TRUE,
#'         zebra_stripes = TRUE,
#'         dark_header = TRUE,
#'         bold_significant = TRUE)
#' 
#' # Example 10: Descriptive statistics slide
#' desc <- desctable(
#'    data = clintrial,
#'    by = "treatment",
#'    variables = c("age", "sex", "bmi"),
#'    labels = clintrial_labels
#' )
#' 
#' table2pptx(desc, file.path(tempdir(), "baseline.pptx"),
#'         caption = "Baseline Characteristics",
#'         font_size = 10)
#' 
#' # Example 11: Conference presentation style
#' table2pptx(results, file.path(tempdir(), "conference.pptx"),
#'         caption = "Study Outcomes",
#'         font_family = "Calibri",
#'         font_size = 14,  # Large for big rooms
#'         dark_header = TRUE,
#'         condense_table = TRUE)
#'
#' options(old_width)
#' }
#' @family export functions
#' @export
table2pptx <- function(table,
                       file,
                     caption = NULL,
                     font_size = 10,
                     font_family = "Arial",
                     format_headers = TRUE,
                     bold_significant = TRUE,
                     bold_variables = FALSE,
                     p_threshold = 0.05,
                     indent_groups = FALSE,
                     condense_table = FALSE,
                     condense_quantitative = FALSE,
                     zebra_stripes = FALSE,
                     dark_header = FALSE,
                     width = NULL,
                     align = NULL,
                     template = NULL,
                     layout = "Title and Content",
                     master = "Office Theme",
                     left = 0.5,
                     top = 1.5,
                     return_ft = FALSE,
                     quiet = FALSE,
                     ...) {
    
    if (!requireNamespace("flextable", quietly = TRUE)) {
        stop("Package 'flextable' required. Install with: install.packages('flextable')")
    }
    if (!requireNamespace("officer", quietly = TRUE)) {
        stop("Package 'officer' required. Install with: install.packages('officer')")
    }
    
    if (!grepl("\\.pptx$", tolower(file))) {
        stop("File must have .pptx extension")
    }
    
    ## Process table using shared function
    ## For PowerPoint, default to landscape-style dimensions
    result <- process_table_for_flextable(
        table = table,
        caption = caption,
        font_size = font_size,
        font_family = font_family,
        format_headers = format_headers,
        bold_significant = bold_significant,
        p_threshold = p_threshold,
        indent_groups = indent_groups,
        condense_table = condense_table,
        condense_quantitative = condense_quantitative,
        zebra_stripes = zebra_stripes,
        dark_header = dark_header,
        bold_variables = bold_variables,
        paper = "letter",
        orientation = "landscape",  # Usually better for slides
        width = width,
        align = align
    )
    
    ## Extract flextable
    ft <- result$ft
    
    ## Adjust width for PowerPoint if not specified
    if (is.null(width)) {
        ## Standard slide is 10 inches wide, leave margins
        slide_width <- 9  # Leave 0.5 inch margins on each side
        ft <- flextable::width(ft, width = slide_width / ncol(table))
    }
    
    ## Create or read PowerPoint presentation
    if (!is.null(template) && file.exists(template)) {
        pres <- officer::read_pptx(template, ...)
    } else {
        pres <- officer::read_pptx(...)
    }
    
    ## Add a new slide
    pres <- officer::add_slide(pres, layout = layout, master = master)
    
    ## Track actual position used (may be adjusted if caption is added)
    actual_top <- top
    
    ## Add title if provided and layout supports it
    if (!is.null(caption)) {
        if (layout %in% c("Title and Content", "Title Only", "Two Content")) {
            tryCatch({
                pres <- officer::ph_with(pres, value = caption, 
                                         location = officer::ph_location_type(type = "title"))
            }, error = function(e) {
                ## If title placeholder not available, add as text box
                pres <- officer::ph_with(pres, value = caption,
                                         location = officer::ph_location(left = left, top = top - 0.5,
                                                                         width = 9, height = 0.5))
                ## Adjust table position down
                actual_top <- top + 0.5
            })
        } else {
            ## For layouts without title placeholder, add as text box above table
            pres <- officer::ph_with(pres, value = caption,
                                     location = officer::ph_location(left = left, top = top - 0.5,
                                                                     width = 9, height = 0.5))
            ## Adjust table position down
            actual_top <- top + 0.5
        }
    }
    
    ## Add the table to the slide
    if (layout == "Title and Content" && is.null(caption)) {
        ## Try to use content placeholder if available
        tryCatch({
            pres <- officer::ph_with(pres, value = ft,
                                     location = officer::ph_location_type(type = "body"))
        }, error = function(e) {
            ## Fall back to absolute positioning
            pres <- officer::ph_with(pres, value = ft,
                                     location = officer::ph_location(left = left, top = actual_top))
        })
    } else {
        ## Use absolute positioning
        pres <- officer::ph_with(pres, value = ft,
                                 location = officer::ph_location(left = left, top = actual_top))
    }
    
    ## Save presentation
    print(pres, target = file)
    
    if (!quiet) message(sprintf("Table saved to %s", file))
    
    ## Return based on user preference
    if (return_ft) {
        ## Return flextable directly for easy access
        return(ft)
    } else {
        ## Create a proper list to hold attributes
        ## Return invisibly with flextable and metadata as attributes
        result_obj <- list(
            file = file,
            caption = caption,
            layout = layout,
            master = master,
            template = template,
            position = list(left = left, top = actual_top)
        )
        attr(result_obj, "flextable") <- ft
        class(result_obj) <- c("table2pptx_result", "list")
        return(invisible(result_obj))
    }
}


### * Print function

#' Print method for table2pptx results
#'
#' @param x Object of class \code{table2pptx_result}.
#' @param ... Additional arguments passed to print methods.
#' @return Invisibly returns the input object \code{x}. Called for its
#'   side effect of printing a formatted summary to the console.
#' @keywords internal
#' @export
print.table2pptx_result <- function(x, ...) {
    cat("Table saved to:", x$file, "\n")
    if (!is.null(x$caption)) {
        cat("Caption:", x$caption, "\n")
    }
    cat("Layout:", x$layout, "\n")
    cat("Master:", x$master, "\n")
    if (!is.null(x$template)) {
        cat("Template:", x$template, "\n")
    }
    cat("Position: left =", x$position$left, ", top =", x$position$top, "\n")
    cat("Flextable object available via: attr(result, 'flextable')\n")
    invisible(x)
}

---
title: "Installation and Setup"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Installation and Setup}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

This document provides instructions for installing the `summata` package and configuring the dependencies required for complete functionality. The package requires R ≥ 4.2.0 and several additional components for document generation.

---

# Package Installation

## From Repository

Install the stable release from CRAN:

```{r}
install.packages("summata")
```

Alternatively, install the stable release directly from GitHub:
 
```{r}
install.packages("remotes")
remotes::install_github("phmcc/summata")
```

For experimental builds, the development version is available from Codeberg:

```{r}
remotes::install_git("https://codeberg.org/phmcc/summata.git")
```

## Verification

Confirm successful installation by loading the package:

```{r}
library(summata)
packageVersion("summata")
```

---

# Dependencies

## Required Dependencies

The following packages are installed automatically as dependencies:

| Package | Purpose |
|:--------|:--------|
| `data.table` | High-performance data manipulation and core data operations |
| `survival` | Survival and time-to-event analysis |
| `ggplot2` | Visualization system for forest plots and graphics |
| `stats` | Statistical functions and model fitting |
| `grDevices` | Graphics devices and color palettes |

## Optional Dependencies

Additional packages extend functionality for specific use-cases:

| Package | Purpose | Required For |
|:--------|:--------|:-------------|
| `lme4` | Linear mixed-effects models | `lmer()`, `glmer()` model types |
| `coxme` | Mixed-effects Cox models | `coxme()` model type |
| `xtable` | LaTeX table generation | `table2pdf()`, `table2tex()`, `table2html()` |
| `flextable` | Office document tables | `table2docx()`, `table2pptx()`, `table2rtf()` |
| `officer` | Office document creation | `table2docx()`, `table2pptx()` |
| `knitr` | Dynamic document generation | R Markdown integration, vignettes |
| `rmarkdown` | R Markdown documents | Vignette rendering |
| `ragg` | High-quality graphics | Enhanced PNG rendering with better fonts |
| `systemfonts` | Font management | Font detection for graphics |
| `tinytex` | R-integrated LaTeX distribution | PDF table export via `table2pdf()` |
| `MASS` | Statistical methods | Model diagnostics and testing |
| `MuMIn` | Multi-model inference | Model selection and averaging |
| `pROC` | ROC curve analysis | Diagnostic performance evaluation |
| `ResourceSelection` | Goodness-of-fit tests | Hosmer-Lemeshow tests for logistic models |
| `stringr` | String manipulation | Text processing utilities |
| `utils` | Utility functions | Data import/export helpers |
| `withr` | Temporary state changes | Testing and safe state modifications |

Install optional dependencies as needed:
```r
# For mixed-effects models
install.packages(c("lme4", "coxme"))

# For table export (all formats)
install.packages(c("xtable", "flextable", "officer", "knitr", "tinytex"))

# For enhanced graphics
install.packages(c("ragg", "systemfonts"))

# For model diagnostics and selection
install.packages(c("MASS", "MuMIn", "pROC", "ResourceSelection"))
```

Or install all suggested packages at once:
```r
install.packages("summata", dependencies = TRUE)
```

---

# LaTeX Configuration

PDF and LaTeX export through `table2pdf()` and `table2tex()` require a LaTeX distribution.

## TinyTeX (Recommended)
 
TinyTeX provides a lightweight, R-integrated LaTeX distribution:

```{r}
install.packages("tinytex")
tinytex::install_tinytex()
```

Verify the installation:

```{r}
tinytex::is_tinytex()
tinytex::tlmgr_version()
```

TinyTeX installs required LaTeX packages automatically on first use.

## Alternative LaTeX Distributions

Full LaTeX distributions may be installed system-wide:

| Distribution | Platform | URL |
|:-------------|:---------|:----|
| TeX Live | Cross-platform | https://www.tug.org/texlive/ |
| MiKTeX | Windows | https://miktex.org/ |
| MacTeX | macOS | https://www.tug.org/mactex/ |

For Debian/Ubuntu systems:

```{bash}
sudo apt install texlive-latex-base texlive-latex-extra texlive-fonts-recommended
```

## Required LaTeX Packages

The export functions utilize the following LaTeX packages:

| Category | Packages |
|:---------|:---------|
| Typography | fontenc, inputenc, helvet |
| Tables | array, booktabs, longtable, colortbl |
| Layout | geometry, pdflscape, lscape |
| Graphics | graphicx, xcolor |
| Specialized | standalone, varwidth, float, caption |

With TinyTeX, these packages install automatically. For standard distributions:

```{bash}
tlmgr install fontenc inputenc array booktabs longtable graphicx geometry \
  pdflscape lscape helvet standalone varwidth float caption xcolor colortbl
```

---

# Microsoft Office Export

Export to Word, PowerPoint, and RTF formats requires:

```{r}
install.packages(c("flextable", "officer"))
```

These packages enable creation of `.docx`, `.pptx`, and `.rtf` files that can be opened and edited in Microsoft Office or compatible applications.

---

# Export Verification Procedure

Execute the following to verify complete functionality:

```{r}
library(summata)

# Create test data
test_data <- data.frame(
  Variable = c("Sample Size", "Mean", "Standard Deviation"),
  Value = c("150", "45.3", "12.1")
)

# Test Word export (requires flextable, officer)
table2docx(test_data, file = file.path(tempdir(), "verification_test.docx"))

# Test PDF export (requires xtable, LaTeX)
table2pdf(test_data, file = file.path(tempdir(), "verification_test.pdf"))

# Clean up
file.remove(
  file.path(tempdir(), "verification_test.docx"),
  file.path(tempdir(), "verification_test.pdf")
)
```

Successful execution confirms proper configuration of all export-related dependencies.

---

# Troubleshooting

## LaTeX Not Found

If PDF generation fails, verify LaTeX accessibility:

```{r}
Sys.which("pdflatex")
```
 
An empty return value indicates LaTeX is not in the system PATH. Solutions include:

1. Restart the R session after LaTeX installation
2. Add the LaTeX binary directory to PATH manually
3. Reinstall TinyTeX: `tinytex::reinstall_tinytex()`

## Missing LaTeX Packages

For TinyTeX, install missing packages directly:

```{r}
tinytex::tlmgr_install("package_name")
```

For compilation errors, examine the log file:

```{r}
table2pdf(test_data, file = file.path(tempdir(), "debug.pdf"), show_logs = TRUE)
```

## Package Installation Failures

Ensure repository access is configured:

```{r}
options(repos = c(CRAN = "https://cloud.r-project.org"))
```

For packages requiring system libraries (Linux):

```{bash}
sudo apt install r-base-dev libcurl4-openssl-dev libssl-dev libxml2-dev
```

## Write Permission Errors

Verify write access to the target directory:

```{r}
getwd()
file.access(getwd(), mode = 2)  # Returns 0 if writable
```

---

# System Requirements

| Component | Requirement |
|:----------|:------------|
| R | ≥ 4.2.0 |
| Operating System | Windows, macOS, or Linux |
| RAM | 4 GB minimum; 8 GB recommended |
| Disk Space | Base package (< 5 MB)<br>TinyTeX (~150 MB)<br>Full LaTeX distribution (2–6 GB) |

---
title: "Forest Plots"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Forest Plots}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
## Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  old_opts <- options(summata.use_ragg = TRUE, width = 180)
  knitr::opts_chunk$set(
    dev = "ragg_png",
    fig.retina = 1,
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "100%"
  )
} else {
  old_opts <- options(width = 180)
  knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "100%"
  )
}

## Dynamic figure sizing: queue_plot() stashes rec_dims from a plot object,
## and the opts_hook on the NEXT chunk (with use_rec_dims = TRUE) applies them
## before knitr opens the graphics device. Plots render via ragg (dev = "ragg_png"
## set above) and knitr captures them natively. No files written to disk.
.plot_dims <- new.env(parent = emptyenv())
.plot_dims$width <- NULL
.plot_dims$height <- NULL

knitr::opts_hooks$set(use_rec_dims = function(options) {
  if (isTRUE(options$use_rec_dims)) {
    if (!is.null(.plot_dims$width))  options$fig.width  <- .plot_dims$width
    if (!is.null(.plot_dims$height)) options$fig.height <- .plot_dims$height
    .plot_dims$width <- NULL
    .plot_dims$height <- NULL
  }
  options
})

## Call at the end of a plot-creation chunk to stash dimensions for the next chunk.
queue_plot <- function(plot) {
  dims <- attr(plot, "rec_dims")
  if (!is.null(dims)) {
    .plot_dims$width  <- dims$width
    .plot_dims$height <- dims$height
  }
  invisible(plot)
}
```

Forest plots provide a graphical representation of effect estimates and their precision. Each row displays a point estimate (typically an odds ratio, hazard ratio, or regression coefficient) with a confidence interval, enabling rapid visual assessment of effect magnitude, direction, and statistical significance. The format is standard in published research and regulatory submissions.

The `summata` package provides specialized forest plot functions for each model type, plus an automatic detection function:
 
| Function | Model Type | Effect Measure |
|:---------|:-----------|:---------------|
| `lmforest()` | Linear regression | Coefficient (*β*) |
| `glmforest()` | Logistic/Poisson | Odds ratio / Rate ratio |
| `coxforest()` | Cox regression | Hazard ratio |
| `uniforest()` | Univariable screening | Model-dependent |
| `multiforest()` | Multi-outcome analysis | Model-dependent |
| `autoforest()` | Auto-detect | Auto-detect |

These functions follow a standard syntax when called:

``` r
forest_plot <- autoforest(x, data, ...)
```

where `x` is either a model or a `summata` fitted output (e.g., from `uniscreen()`, `fit()`, `fullfit()`, or `multifit()`), and `data` is the name of the dataset used. The `data` argument is optional and is primarily used to derive *n* and Events counts for various groups/subgroups.

All forest plot functions produce `ggplot2` objects that can be further customized. This vignette demonstrates the various capabilities of these functions using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)
library(ggplot2)

data(clintrial)
data(clintrial_labels)
```

> *n.b.:* To ensure correct font rendering and figure sizing, the forest plots below are displayed using a helper function (`queue_plot()`) that applies each plot's recommended dimensions (stored in the `"rec_dims"` attribute) via the [`ragg`](https://ragg.r-lib.org/) graphics device. In practice, replace `queue_plot()` with `ggplot2::ggsave()` using recommended plot dimensions for equivalent results:
> 
> ```{r, eval = FALSE}
> p <- glmforest(model, data = mydata)
> dims <- attr(p, "rec_dims")
> ggplot2::ggsave("forest_plot.png", p,
>                 width = dims$width, 
>                 height = dims$height)
> ```
> 
> This ensures that the figure size is always large enough to accommodate the constituent plot text and graphics, and it is generally the preferred method for saving forest plot outputs in `summata`.

---

# Creating Forest Plots from Model Objects

Forest plots can be created from standard R model objects or from `summata` function output.

## **Example 1:** Logistic Regression

Fit a model using base R, then create a forest plot:

```{r}
logistic_model <- glm(
  surgery ~ age + sex + stage + treatment + ecog,
  data = clintrial,
  family = binomial
)

example1 <- glmforest(
  x = logistic_model,
  data = clintrial,
  title = "Logistic Regression: Predictors of Outcome",
  labels = clintrial_labels
)
queue_plot(example1)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example1)
```

## **Example 2:** Linear Regression

For continuous outcomes, use `lmforest()`:

```{r}
linear_model <- lm(
  los_days ~ age + sex + stage + surgery + ecog,
  data = clintrial
)

example2 <- lmforest(
  x = linear_model,
  data = clintrial,
  title = "Linear Regression: Length of Stay",
  labels = clintrial_labels
)
queue_plot(example2)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example2)
```

## **Example 3:** Cox Regression

For survival models, use `coxforest()`:

```{r}
cox_model <- coxph(
  Surv(os_months, os_status) ~ age + sex + stage + treatment + ecog,
  data = clintrial
)

example3 <- coxforest(
  x = cox_model,
  data = clintrial,
  title = "Cox Regression: Survival Analysis",
  labels = clintrial_labels
)
queue_plot(example3)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example3)
```

## **Example 4:** Automatic Model Detection

The `autoforest()` function detects the model type automatically:

```{r}
example4 <- autoforest(
  x = cox_model,
  data = clintrial,
  labels = clintrial_labels
)
queue_plot(example4)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example4)
```

---

# Creating Forest Plots from *summata* Output

Forest plots integrate seamlessly with `fit()` and `fullfit()` output by extracting the attached model object.

## **Example 5:** Direct Extraction from Fit Output

The same approach works for logistic models:

```{r}
table_logistic <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "stage", "treatment", "ecog"),
  model_type = "glm",
  labels = clintrial_labels
)

example5 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Predictors of Surgical Intervention",
  labels = clintrial_labels,
  zebra_stripes = TRUE
)
queue_plot(example5)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example5)
```

## **Example 6:** Model Attribute from Fit Output

And for Cox models:

```{r}
table_cox <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "stage", "treatment", "ecog"),
  model_type = "coxph",
  labels = clintrial_labels
)

example6 <- coxforest(
  x = attr(table_cox, "model"),
  title = "Predictors of Overall Survival",
  labels = clintrial_labels,
  zebra_stripes = TRUE
)
queue_plot(example6)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example6)
```

---

# Display Options

## **Example 7:** Indenting Factor Levels

The `indent_groups` parameter creates hierarchical display for a more compact aesthetic:

```{r}
example7 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Indented Factor Levels",
  labels = clintrial_labels,
  indent_groups = TRUE
)
queue_plot(example7)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example7)
```

## **Example 8:** Condensing Binary Variables

The `condense_table` parameter displays binary variables on single rows. Group indenting is applied automatically:

```{r}
example8 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Condensed Display",
  labels = clintrial_labels,
  condense_table = TRUE
)
queue_plot(example8)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example8)
```

## **Example 9:** Toggle Zebra Striping

Zebra striping is enabled by default. It can be disabled via `zebra_stripes = FALSE`:

```{r}
example9 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Without Zebra Striping",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = FALSE
)
queue_plot(example9)
```

```{r, echo = FALSE, out.width = "90%", use_rec_dims = TRUE}
print(example9)
```

## **Example 10:** Sample Size and Event Columns

Control display of sample size (*n*) and event counts:

```{r}
# Show both n and events
example10a <- coxforest(
  x = attr(table_cox, "model"),
  title = "With Sample Size and Events",
  labels = clintrial_labels,
  show_n = TRUE,
  show_events = TRUE,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
queue_plot(example10a)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example10a)
```

```{r}
# Minimal display
example10b <- coxforest(
  x = attr(table_cox, "model"),
  title = "Minimal Display",
  labels = clintrial_labels,
  show_n = FALSE,
  show_events = FALSE,
  indent_groups = TRUE
)
queue_plot(example10b)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example10b)
```

---

# Formatting Options

## **Example 11:** Adjusting Numeric Precision

The `digits` parameter controls decimal places for effect estimates and confidence intervals:

```{r}
example11 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Custom Precision (3 decimal places)",
  labels = clintrial_labels,
  digits = 3,
  indent_groups = TRUE
)
queue_plot(example11)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example11)
```

## **Example 12:** Custom Reference Label

Customize the label shown for reference categories:

```{r}
example12 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Custom Reference Label",
  labels = clintrial_labels,
  ref_label = "ref",
  indent_groups = TRUE
)
queue_plot(example12)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example12)
```

## **Example 13:** Custom Effect Measure Label

Change the column header for the effect measure:

```{r}
example13 <- coxforest(
  x = attr(table_cox, "model"),
  title = "Custom Effect Label",
  labels = clintrial_labels,
  effect_label = "Effect (95% CI)",
  indent_groups = TRUE
)
queue_plot(example13)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example13)
```

## **Example 14:** Custom Colors

The `color` parameter changes the point and line color:

```{r}
example14 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Custom Color",
  labels = clintrial_labels,
  color = "#E41A1C",
  indent_groups = TRUE
)
queue_plot(example14)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example14)
```

## **Example 15:** Change Font Sizes

Adjust text size with the `font_size` multiplier:

```{r}
example15 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Larger Font (1.5×)",
  labels = clintrial_labels,
  font_size = 1.5,
  indent_groups = TRUE
)
queue_plot(example15)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example15)
```

## **Example 16:** Table Width

The `table_width` parameter adjusts the proportion of space allocated to the table vs. the forest plot:

```{r}
# Wide table (for long variable names)
example16a <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Wide Table (75%)",
  labels = clintrial_labels,
  table_width = 0.75
)
queue_plot(example16a)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example16a)
```

```{r}
# Narrow table (emphasizes forest plot)
example16b <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Narrow Table (50%)",
  labels = clintrial_labels,
  table_width = 0.50
)
queue_plot(example16b)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example16b)
```

---

# Saving Forest Plots

Forest plots include a `rec_dims` attribute for optimal sizing when saving to files.

## **Example 17:** Recommended Dimensions

Use the recommended dimensions for optimal output:

```{r, eval = FALSE}
p <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Publication-Ready Plot",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)

# Get recommended dimensions
dims <- attr(p, "rec_dims")
cat("Width:", dims$width, "inches\n")
cat("Height:", dims$height, "inches\n")

# Save with recommended dimensions
ggsave(
  filename = file.path(tempdir(), "forest_plot.pdf"),
  plot = p,
  width = dims$width,
  height = dims$height,
  units = "in"
)
```

## **Example 18:** Multiple Formats

Export to different formats as needed:

```{r, eval = FALSE}
p <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Forest Plot",
  labels = clintrial_labels
)

dims <- attr(p, "rec_dims")

# PDF (vector, best for publications)
ggsave("forest.pdf", p, width = dims$width, height = dims$height)

# PNG (raster, good for presentations)
ggsave("forest.png", p, width = dims$width, height = dims$height, dpi = 300)

# TIFF (high-quality raster, often required by journals)
ggsave("forest.tiff", p, width = dims$width, height = dims$height, dpi = 300)

# SVG (vector, good for web)
ggsave("forest.svg", p, width = dims$width, height = dims$height)
```

---

# Advanced Customization

Forest plots support extensive customization for publication requirements.

## **Example 19:** Combined Options

Combine multiple options for publication-ready output:

```{r}
example19 <- coxforest(
  x = attr(table_cox, "model"),
  title = "Comprehensive Survival Analysis",
  labels = clintrial_labels,
  effect_label = "Hazard Ratio",
  digits = 2,
  show_n = TRUE,
  show_events = TRUE,
  indent_groups = TRUE,
  condense_table = FALSE,
  zebra_stripes = TRUE,
  ref_label = "reference",
  font_size = 1.0,
  table_width = 0.62,
  color = "#8A61D8"
)
queue_plot(example19)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example19)
```

## **Example 20:** Extensions with ggplot2

Forest plots are `ggplot2` objects and can be modified further:

```{r}
example20 <- glmforest(
  x = attr(table_logistic, "model"),
  title = "Extended with ggplot2",
  labels = clintrial_labels,
  indent_groups = TRUE
)

example20_modified <- example20 +
  theme(
    plot.title = element_text(face = "italic", color = "#A72727"),
    plot.background = element_rect(fill = "white", color = NA)
  )
queue_plot(example20_modified)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example20_modified)
```

---

# Additional GLM Families

The `glmforest()` function supports all GLM families. These can be plotted in a similar fashion to standard logistic regression forest plots. See [Regression Modeling](regression_modeling.html) for the full list of supported model types.

## **Example 21:** Poisson Regression

For equidispersed count outcomes (variance ≈ mean), use Poisson regression:

```{r}
poisson_model <- glm(
  fu_count ~ age + stage + treatment + surgery,
  data = clintrial,
  family = poisson
)

example21 <- glmforest(
  x = poisson_model,
  data = clintrial,
  title = "Poisson Regression: Follow-Up Visits",
  labels = clintrial_labels
)
queue_plot(example21)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example21)
```

## **Example 22:** Negative Binomial Regression

For overdispersed count outcomes (variance > mean), negative binomial regression is preferred. Using `fit()` ensures proper handling:

```{r}
nb_result <- fit(
  data = clintrial,
  outcome = "ae_count",
  predictors = c("age", "treatment", "diabetes", "surgery"),
  model_type = "negbin",
  labels = clintrial_labels
)

example22 <- glmforest(
  x = nb_result,
  title = "Negative Binomial: Adverse Events"
)
queue_plot(example22)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example22)
```

---

# Function Parameter Summary

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `x` | Model object or model from `fit()` output | Required |
| `data` | Data frame (required for model objects) | `NULL` |
| `title` | Plot title | `NULL` |
| `labels` | Named vector for variable labels | `NULL` |
| `indent_groups` | Indent factor levels under variable names | `FALSE` |
| `condense_table` | Show binary variables on single row | `FALSE` |
| `zebra_stripes` | Alternating row shading | `TRUE` |
| `show_n` | Display sample size column | `TRUE` |
| `show_events` | Display events column (Cox models) | `TRUE` |
| `digits` | Decimal places for estimates | `2` |
| `ref_label` | Label for reference categories | `"reference"` |
| `effect_label` | Column header for effect measure | Model-dependent |
| `color` | Color for points and lines | Effect-type dependent |
| `font_size` | Text size multiplier | `1.0` |
| `table_width` | Proportion of width for table | `0.55` |

---

# Best Practices

## Model Preparation

1. Ensure all factor levels are properly defined before fitting
2. Use meaningful reference categories
3. Consider centering continuous variables for interpretability
4. Check model convergence before plotting

## Visual Design

1. Use `indent_groups = TRUE` for cleaner presentation of categorical variables
2. Match `table_width` to variable name lengths
3. Consider `condense_table = TRUE` for binary predictors
4. Use consistent colors across related figures

## Publication Preparation

1. Use `rec_dims` attribute for optimal sizing
2. Save as PDF for vector graphics in print
3. Use 300+ DPI for raster formats
4. Check that reference lines and confidence intervals are clearly visible

---

# Common Issues

## Long Variable Names

If variable names are truncated, increase `table_width`:

```{r, eval = FALSE}
p <- glmforest(model, table_width = 0.75)
```

## Overlapping Text

Reduce font size or increase figure dimensions:
  
```{r, eval = FALSE}
p <- glmforest(model, font_size = 0.9)
ggsave(file.path(tempdir(), "plot.pdf"), p, width = 14, height = 8)
```

## Missing Labels

Ensure labels vector includes all variable names:

```{r, eval = FALSE}
labels <- c(
  age = "Age (years)",
  sex = "Sex",
  stage = "Disease Stage"
)
p <- glmforest(model, labels = labels)
```

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Survival Tables](survival_tables.html): `survtable()` for time-to-event summaries
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Multivariate Regression"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Multivariate Regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
## Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  old_opts <- options(summata.use_ragg = TRUE, width = 180)
  knitr::opts_chunk$set(
    dev = "ragg_png",
    fig.retina = 1,
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "80%"
  )
} else {
  old_opts <- options(width = 180)
  knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "80%"
  )
}

## Dynamic figure sizing: queue_plot() stashes rec_dims from a plot object,
## and the opts_hook on the NEXT chunk (with use_rec_dims = TRUE) applies them
## before knitr opens the graphics device. Plots render via ragg (dev = "ragg_png"
## set above) and knitr captures them natively. No files written to disk.
.plot_dims <- new.env(parent = emptyenv())
.plot_dims$width <- NULL
.plot_dims$height <- NULL

knitr::opts_hooks$set(use_rec_dims = function(options) {
  if (isTRUE(options$use_rec_dims)) {
    if (!is.null(.plot_dims$width))  options$fig.width  <- .plot_dims$width
    if (!is.null(.plot_dims$height)) options$fig.height <- .plot_dims$height
    .plot_dims$width <- NULL
    .plot_dims$height <- NULL
  }
  options
})

## Call at the end of a plot-creation chunk to stash dimensions for the next chunk.
queue_plot <- function(plot) {
  dims <- attr(plot, "rec_dims")
  if (!is.null(dims)) {
    .plot_dims$width  <- dims$width
    .plot_dims$height <- dims$height
  }
  invisible(plot)
}
```

Multivariate regression refers to the simultaneous examination of a single independent predictor across multiple dependent variables. This approach inverts the typical regression paradigm: rather than testing multiple predictors against one outcome (as in `uniscreen()`, `fit()`, and `fullfit()`), multivariate regression tests the predictive value of a individual factor against multiple outcomes, with or without multivariable adjustment for other independent factors. This design is particularly useful when a key exposure or intervention must be evaluated against several endpoints simultaneously.

The `multifit()` function implements this workflow, supporting all model types available in `summata` with optional covariate adjustment, interaction terms, and mixed-effects specifications. Results can be visualized using `multiforest()` and exported using standard table export functions.

As with other `summata` functions, this function adheres to the standard calling convention:

```{r, eval = FALSE}
multifit(data, outcomes, predictor, covariates, model_type, ...)
```

where `data` is the dataset, `outcomes` is a vector of screened outcome variables, `predictor` is the predicting variable under examination, `covariates` is a vector of simultaneous independent predicting variables (i.e., potential confounders), and `model_type` is the type of model to be implemented. This vignette demonstrates the various capabilities of this function using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)
library(ggplot2)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset includes multiple outcome types suitable for multivariate analysis:
 
| Outcome Type | Variables | Model |
|:-------------|:----------|:------|
| Continuous | `los_days`, `pain_score`, `recovery_days` | `lm` |
| Binary | `any_complication`, `wound_infection`, `readmission_30d`, `icu_admission` | `glm` (binomial) |
| Count (equidispersed) | `fu_count` | `glm` (poisson) |
| Count (overdispersed) | `ae_count` | `negbin` or `quasipoisson` |
| Time-to-event | `Surv(pfs_months, pfs_status)`, `Surv(os_months, os_status)` | `coxph` |

> *n.b.:* To ensure correct font rendering and figure sizing, the forest plots below are displayed using a helper function (`queue_plot()`) that applies each plot's recommended dimensions (stored in the `"rec_dims"` attribute) via the [`ragg`](https://ragg.r-lib.org/) graphics device. In practice, replace `queue_plot()` with `ggplot2::ggsave()` using recommended plot dimensions for equivalent results:
> 
> ```{r, eval = FALSE}
> p <- glmforest(model, data = mydata)
> dims <- attr(p, "rec_dims")
> ggplot2::ggsave("forest_plot.png", p,
>                 width = dims$width, 
>                 height = dims$height)
> ```
> 
> This ensures that the figure size is always large enough to accommodate the constituent plot text and graphics, and it is generally the preferred method for saving forest plot outputs in `summata`.

---

# Basic Usage

## **Example 1:** Unadjusted Analysis

Test a single predictor against multiple binary outcomes:

```{r}
example1 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection",
               "readmission_30d", "icu_admission"),
  predictor = "surgery",
  labels = clintrial_labels,
  parallel = FALSE
)

example1
```

Each row shows the effect of the predictor on a specific outcome. For categorical predictors, each non-reference level appears separately.

## **Example 2:** Adjusted Analysis

Include covariates for confounding control:

```{r}
example2 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection",
               "readmission_30d", "icu_admission"),
  predictor = "surgery",
  covariates = c("age", "sex", "smoking", "diabetes"),
  labels = clintrial_labels,
  parallel = FALSE
)

example2
```

## **Example 3:** Unadjusted and Adjusted Comparison

Use `columns = "both"` to display unadjusted and adjusted results side-by-side:

```{r}
example3 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection",
               "readmission_30d", "icu_admission"),
  predictor = "surgery",
  covariates = c("age", "sex", "diabetes", "surgery"),
  columns = "both",
  labels = clintrial_labels,
  parallel = FALSE
)

example3
```

Comparing columns reveals confounding (large differences) or robust associations (similar estimates).

---

# Predictor Types

## **Example 4:** Continuous Predictors

For continuous predictors, one row appears per outcome:

```{r}
example4 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection", "icu_admission"),
  predictor = "age",
  covariates = c("sex", "treatment", "surgery"),
  labels = clintrial_labels,
  parallel = FALSE
)

example4
```

The effect estimate represents the change in log-odds per one-unit increase in the predictor.

## **Example 5:** Multilevel Categorical Predictors

For categorical predictors with multiple levels, the display is expanded:

```{r}
example5 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection",
               "readmission_30d", "icu_admission"),
  predictor = "treatment",
  covariates = c("age", "sex", "surgery"),
  labels = clintrial_labels,
  parallel = FALSE
)

example5
```

---

# Model Types

## **Example 6:** Cox Regression

For time-to-event outcomes, use `model_type = "coxph"`:

```{r}
example6 <- multifit(
  data = clintrial,
  outcomes = c("Surv(pfs_months, pfs_status)",
               "Surv(os_months, os_status)"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage"),
  model_type = "coxph",
  labels = clintrial_labels,
  parallel = FALSE
)

example6
```

## **Example 7:** Linear Regression

For continuous outcomes, use `model_type = "lm"`:

```{r}
example7 <- multifit(
  data = clintrial,
  outcomes = c("los_days", "pain_score", "recovery_days"),
  predictor = "treatment",
  covariates = c("age", "sex", "surgery"),
  model_type = "lm",
  labels = clintrial_labels,
  parallel = FALSE
)

example7
```

## **Example 8:** Mixed-Effects Models

Account for clustering using random effects:

```{r}
example8 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  random = "(1|site)",
  model_type = "glmer",
  labels = clintrial_labels,
  parallel = FALSE
)

example8
```

---

# Advanced Features

## **Example 9:** Interaction Terms

Interaction terms can be added to test effect modification:

```{r}
example9 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  interactions = c("treatment:sex"),
  labels = clintrial_labels,
  parallel = FALSE
)

example9
```

## **Example 10:** Filtering by p-value

Outputs can be filtered to retain only significant associations:

```{r}
example10 <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection",
               "readmission_30d", "icu_admission"),
  predictor = "treatment",
  covariates = c("age", "sex", "surgery"),
  p_threshold = 0.01,
  labels = clintrial_labels,
  parallel = FALSE
)

example10
```

## **Example 11:** Accessing Model Objects

Underlying model objects are stored as attributes for additional analysis:

```{r}
result <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  labels = clintrial_labels,
  keep_models = TRUE,
  parallel = FALSE
)

# Access individual models
models <- attr(result, "models")
names(models)

# Examine a specific model
summary(models[["any_complication"]])
```

---

# Forest Plot Visualization

The `multiforest()` function creates forest plots from `multifit()` results.

## **Example 12:** Forest Plot for Logistic (Binary) Outcomes

Results from `multifit()` can be directly inserted into the `multiforest()` function to generate a forest plot:

```{r}
result <- multifit(
  data = clintrial,
  outcomes = c("any_complication", "wound_infection",
               "readmission_30d", "icu_admission"),
  predictor = "treatment",
  covariates = c("age", "sex", "diabetes", "surgery"),
  labels = clintrial_labels,
  parallel = FALSE
)

example12 <- multiforest(
  result,
  title = "Treatment Effects Across Outcomes",
  indent_predictor = TRUE,
  zebra_stripes = TRUE
)
queue_plot(example12)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example12)
```

## **Example 13:** Customization Options

Customize the appearance of the forest plot by adjusting function parameters:

```{r}
example13 <- multiforest(
  result,
  title = "Effect Estimates",
  column = "adjusted",
  show_predictor = FALSE,
  covariates_footer = TRUE,
  table_width = 0.65,
  color = "#4BA6B6"
)
queue_plot(example13)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example13)
```

## **Example 14:** Forest Plot for Continuous Outcomes

Create forest plots for continuous outcomes:

```{r}
lm_result <- multifit(
  data = clintrial,
  outcomes = c("pain_score", "recovery_days", "los_days"),
  predictor = "treatment",
  covariates = c("age", "sex", "surgery"),
  model_type = "lm",
  parallel = FALSE
)

example14 <- multiforest(
  lm_result,
  title = "Treatment Effects on Recovery Metrics",
  show_predictor = FALSE,
  covariates_footer = TRUE,
  labels = clintrial_labels
)
queue_plot(example14)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example14)
```

## **Example 15:** Forest Plot for Survival Outcomes

The `multiforest()` function supports multiple `model_type` outputs from `multifit()`. In addition, variable labels can be applied to `multiforest()` outputs similarly to other forest plot functions:

```{r}
cox_result <- multifit(
  data = clintrial,
  outcomes = c("Surv(pfs_months, pfs_status)",
               "Surv(os_months, os_status)"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage"),
  model_type = "coxph",
  parallel = FALSE
)

example15 <- multiforest(
  cox_result,
  title = "Treatment Effects on Survival Outcomes",
  indent_predictor = TRUE,
  zebra_stripes = TRUE,
  labels = clintrial_labels
)
queue_plot(example15)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example15)
```

---

# Exporting Results

## Tables

Export tables using standard export functions:

```{r, eval = FALSE}
table2docx(
  table = result,
  file = file.path(tempdir(), "multioutcome_analysis.docx"),
  caption = "Treatment Effects Across Outcomes"
)

table2pdf(
  table = result,
  file = file.path(tempdir(), "multioutcome_analysis.pdf"),
  caption = "Treatment Effects Across Outcomes"
)
```

## Forest Plots

Save forest plots using `ggsave()`:

```{r, eval = FALSE}
p <- multiforest(result, title = "Effect Estimates")
dims <- attr(p, "rec_dims")

ggsave(file.path(tempdir(), "multioutcome_forest.pdf"), p,
       width = attr(result, "rec_dims")$width,
       height = attr(result, "rec_dims")$height, 
       units = "in")
```

---

## Complete Workflow Example

The following demonstrates a complete multi-outcome analysis workflow:

```{r}
## Define outcomes by type
binary_outcomes <- c("any_complication", "wound_infection",
                     "readmission_30d", "icu_admission")
survival_outcomes <- c("Surv(pfs_months, pfs_status)",
                       "Surv(os_months, os_status)")

## Unadjusted screening
unadjusted <- multifit(
  data = clintrial,
  outcomes = binary_outcomes,
  predictor = "treatment",
  labels = clintrial_labels,
  parallel = FALSE
)

unadjusted

## Adjusted analysis with comparison
adjusted <- multifit(
  data = clintrial,
  outcomes = binary_outcomes,
  predictor = "treatment",
  covariates = c("age", "sex", "diabetes", "surgery"),
  columns = "both",
  labels = clintrial_labels,
  parallel = FALSE
)

adjusted

## Forest plot visualization
forest_plot <- multiforest(
  adjusted,
  title = "Treatment Effect Estimates",
  column = "adjusted",
  indent_predictor = TRUE,
  zebra_stripes = TRUE,
  table_width = 0.65,
  labels = clintrial_labels
)
queue_plot(forest_plot)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(forest_plot)
```

---

## Parameter Reference

| Parameter | Description |
|:----------|:------------|
| `outcomes` | Character vector of outcome variable names |
| `predictor` | Single predictor variable name |
| `covariates` | Optional adjustment covariates |
| `interactions` | Interaction terms (colon notation) |
| `random` | Random effects formula for mixed models |
| `strata` | Stratification variable (Cox models) |
| `cluster` | Clustering variable for robust standard errors |
| `model_type` | `"glm"`, `"lm"`, `"coxph"`, `"glmer"`, `"lmer"`, `"coxme"` |
| `family` | GLM family (default: `"binomial"`) |
| `columns` | `"adjusted"`, `"unadjusted"`, or `"both"` |
| `p_threshold` | Filter results by *p*-value |
| `labels` | Custom variable labels |
| `parallel` | Enable parallel processing |

---

# Best Practices

## Outcome Selection

1. Use compatible outcome types within a single analysis
2. Group conceptually related outcomes
3. Consider multiple testing adjustments when testing many outcomes

## Adjustment Strategy

1. Pre-specify covariates based on domain knowledge
2. Use `columns = "both"` to assess confounding
3. Apply consistent covariates across outcomes for comparability

## Interpretation

1. Focus on effect magnitude and precision, not only *p*-values
2. Look for consistent patterns across related outcomes
3. Consider practical significance alongside statistical significance

---

# Common Issues

## Empty Results

If results are empty, verify:

- Predictor variable exists and has variation
- Outcomes are correctly specified (`Surv()` syntax for survival)
- Model type matches outcome type

## Convergence Warnings

For mixed-effects models, simplify the random effects structure:

```{r, eval = FALSE}
## Start with random intercepts only
multifit(data, outcomes, predictor,
         random = "(1|site)",
         model_type = "glmer")
```

## Many Factor Levels

For predictors with many levels, consider collapsing categories:

```{r, eval = FALSE}
clintrial$treatment_binary <- ifelse(clintrial$treatment == "Control", 
                                      "Control", "Active")
```

---

# Other Considerations

## Multivariate Regression vs. Univariable Screening

The distinction between `multifit()` and `uniscreen()` is important:

| Function | Tests | Use Case |
|:---------|:------|:---------|
| `uniscreen()` | Multiple predictors → One outcome | Variable screening, risk factor identification |
| `multifit()` | One predictor → Multiple outcomes | Exposure effects, intervention evaluation |

All outcomes in a single `multifit()` call should be of the same type (all binary, all continuous, or all survival). Mixing outcome types produces tables with incompatible effect measures. The function validates outcome compatibility and issues a warning when mixed types are detected.

## Categorical Outcomes with More Than Two Levels

`multifit()` supports binary outcomes (via logistic regression) but not multinomial or ordinal outcomes. If a categorical outcome with three or more levels is included (e.g., treatment group with "Control", "Drug A", "Drug B"), the function will issue a warning because binomial GLM coerces such variables to binary (first level *vs.* all others).

For multilevel categorical outcomes, use dedicated packages outside of `summata`:

```{r, eval = FALSE}
## Multinomial regression (unordered categories)
library(nnet)
model <- multinom(treatment ~ age + sex + stage, data = clintrial)

## Ordinal regression (ordered categories)
library(MASS)
model <- polr(grade ~ age + sex + stage, data = clintrial, Hess = TRUE)
```

```{r, include = FALSE}
options(old_opts)
```

---

## Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Advanced Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Advanced Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
## Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  old_opts <- options(summata.use_ragg = TRUE, width = 180)
  knitr::opts_chunk$set(
    dev = "ragg_png",
    fig.retina = 1,
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "100%"
  )
} else {
  old_opts <- options(width = 180)
  knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "100%"
  )
}

## Dynamic figure sizing: queue_plot() stashes rec_dims from a plot object,
## and the opts_hook on the NEXT chunk (with use_rec_dims = TRUE) applies them
## before knitr opens the graphics device. Plots render via ragg (dev = "ragg_png"
## set above) and knitr captures them natively. No files written to disk.
.plot_dims <- new.env(parent = emptyenv())
.plot_dims$width <- NULL
.plot_dims$height <- NULL

knitr::opts_hooks$set(use_rec_dims = function(options) {
  if (isTRUE(options$use_rec_dims)) {
    if (!is.null(.plot_dims$width))  options$fig.width  <- .plot_dims$width
    if (!is.null(.plot_dims$height)) options$fig.height <- .plot_dims$height
    .plot_dims$width <- NULL
    .plot_dims$height <- NULL
  }
  options
})

## Call at the end of a plot-creation chunk to stash dimensions for the next chunk.
queue_plot <- function(plot) {
  dims <- attr(plot, "rec_dims")
  if (!is.null(dims)) {
    .plot_dims$width  <- dims$width
    .plot_dims$height <- dims$height
  }
  invisible(plot)
}
```

Standard regression analysis assumes independent observations and constant effects across subgroups. In practice, these assumptions often fail: observations may be clustered (e.g., subjects within study sites), effects may vary by subgroup (interaction), or the baseline hazard may differ across strata. This vignette demonstrates advanced analytical techniques to address these complexities.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)
library(ggplot2)

data(clintrial)
data(clintrial_labels)

# Examine the clustering structure
table(clintrial$site)
```

The `clintrial` dataset includes 10 study sites, providing a natural clustering variable for hierarchical analysis.

> *n.b.:* To ensure correct font rendering and figure sizing, the forest plots below are displayed using a helper function (`queue_plot()`) that applies each plot's recommended dimensions (stored in the `"rec_dims"` attribute) via the [`ragg`](https://ragg.r-lib.org/) graphics device. In practice, replace `queue_plot()` with `ggplot2::ggsave()` using recommended plot dimensions for equivalent results:
> 
> ```{r, eval = FALSE}
> p <- glmforest(model, data = mydata)
> dims <- attr(p, "rec_dims")
> ggplot2::ggsave("forest_plot.png", p,
>                 width = dims$width, 
>                 height = dims$height)
> ```
> 
> This ensures that the figure size is always large enough to accommodate the constituent plot text and graphics, and it is generally the preferred method for saving forest plot outputs in `summata`.

---

# Interaction Effects

Interaction analysis tests whether the association between a predictor and outcome varies across levels of another variable. Interactions are specified using colon notation in the `interactions` parameter; main effects are automatically included alongside the interaction term.

## **Example 1:** Two-Way Interaction

Specify interactions using colon notation in the `interactions` parameter:

```{r}
example1 <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  interactions = c("sex:treatment"),
  model_type = "glm",
  labels = clintrial_labels
)

example1
```

## **Example 2:** Multiple Interactions

Test multiple effect modifiers simultaneously:

```{r}
example2 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage"),
  interactions = c("sex:treatment", "stage:treatment"),
  model_type = "coxph",
  labels = clintrial_labels
)

example2
```

## **Example 3:** Continuous × Categorical Interaction

Test whether effects of a continuous variable vary by group:

```{r}
example3 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "surgery"),
  interactions = c("age:treatment"),
  model_type = "lm",
  labels = clintrial_labels
)

example3
```

## **Example 4:** Interaction in fullfit()

Include interaction terms in combined univariable/multivariable analysis:

```{r}
example4 <- fullfit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage", "sex:treatment"),
  model_type = "glm",
  method = "all",
  labels = clintrial_labels
)

example4
```

## **Example 5:** Testing Interaction Significance

Use `compfit()` to formally evaluate whether interactions improve model fit (*see* [Model Comparison](model_comparison.html)):

```{r}
example5 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Main Effects" = c("age", "sex", "treatment", "stage"),
    "Sex × Treatment" = c("age", "sex", "treatment", "stage"),
    "Stage × Treatment" = c("age", "sex", "treatment", "stage"),
    "Both Interactions" = c("age", "sex", "treatment", "stage")
  ),
  interactions_list = list(
    NULL,
    c("sex:treatment"),
    c("stage:treatment"),
    c("sex:treatment", "stage:treatment")
  ),
  model_type = "glm",
  labels = clintrial_labels
)

example5
```

## **Example 6:** Forest Plot with Interactions

Visualize models including interaction terms:

```{r}
interaction_model <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  interactions = c("sex:treatment"),
  model_type = "glm",
  labels = clintrial_labels
)

example6 <- glmforest(
  x = attr(interaction_model, "model"),
  title = "Logistic Regression with Interaction",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
queue_plot(example6)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example6)
```

---

# Mixed-Effects Models

Mixed-effects models (multilevel or hierarchical models) account for correlation within clusters by incorporating random effects. The `summata` package supports linear mixed models (`lmer`), generalized linear mixed models (`glmer`), and Cox mixed models (`coxme`).

Random effects are specified using pipe notation within either the `predictors` parameter or in a separate `random` parameter:

| Syntax | Meaning |
|:-------|:--------|
| `(1\|group)` | Random intercepts by group |
| `(x\|group)` | Random intercepts and slopes for *x* |
| `(1\|g1) + (1\|g2)` | Crossed random effects |

## **Example 7:** Random Intercepts (Linear Mixed)

The `(1|site)` syntax specifies a random intercept for each site:

```{r}
example7 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "lmer",
  labels = clintrial_labels
)

example7
```

## **Example 8:** Random Intercepts (Logistic Mixed)

For binary outcomes with clustering (note the use of an independent `random` parameter):

```{r}
example8 <- fit(
    data = clintrial,
    outcome = "surgery",
    predictors = c("age", "sex", "treatment", "stage"),
    random = "(1|site)",
    model_type = "glmer",
    labels = clintrial_labels
)

example8
```

## **Example 9:** Random Slopes

Allow effects to vary by cluster:

```{r}
example9 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "(1 + treatment|site)"),
  model_type = "lmer",
  labels = clintrial_labels
)

example9
```

## **Example 10:** Cox Mixed-Effects Model

For survival outcomes with clustering:

```{r}
example10 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "coxme",
  labels = clintrial_labels
)

example10
```

## **Example 11:** Forest Plot from Mixed Model

Visualize fixed effects from mixed-effects models:

```{r}
example11 <- glmforest(
  x = attr(example8, "model"),
  title = "Logistic Mixed Model (Fixed Effects)",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
queue_plot(example11)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example11)
```

## **Example 12:** Comparing Random-Effects Specifications

Use `compfit()` to compare different random-effects structures (*see* [Model Comparison](model_comparison.html)):

```{r}
example12 <- compfit(
  data = clintrial,
  outcome = "los_days",
  model_list = list(
    "Random Intercepts" = c("age", "sex", "treatment", "stage", "(1|site)"),
    "Random Slopes" = c("age", "sex", "treatment", "stage", "(1 + treatment|site)")
  ),
  model_type = "lmer",
  labels = clintrial_labels
)

example12
```

---

# Stratification and Clustering

In addition to mixed-effects models, Cox models support stratification (separate baseline hazards) and clustered standard errors (robust variance estimation).

| Approach | When to Use |
|:---------|:------------|
| Mixed-effects | Model cluster-specific effects; prediction at cluster level |
| Stratification | Proportional hazards assumption violated for a variable |
| Clustered SE | Correlation within clusters; robust inference |

## **Example 13:** Stratified Cox Model

Stratification allows nonproportional hazards across strata without estimating stratum effects. Use the `strata` parameter:

```{r}
example13 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment"),
  strata = "site",
  model_type = "coxph",
  labels = clintrial_labels
)

example13
```

## **Example 14:** Cluster-Robust SEs

For robust inference with correlated observations, cluster-robust standard errors account for within-cluster correlation. Use the `cluster` parameter:

```{r}
example14 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage"),
  cluster = "site",
  model_type = "coxph",
  labels = clintrial_labels
)

example14
```

---

# Weighted Regression

For survey data or inverse probability weighting, use the `weights` parameter. Weights can account for sampling design, non-response, or confounding adjustment.

## **Example 15:** Weighted Analysis

```{r}
# Create example weights
clintrial$analysis_weight <- runif(nrow(clintrial), 0.5, 2.0)

example15 <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  weights = "analysis_weight",
  model_type = "glm",
  labels = clintrial_labels
)

example15
```

---

# Advanced Univariable Screening Features

The `uniscreen()` function supports advanced model specifications including random effects and stratification.

## **Example 16:** Univariable Screening with Random Effects

Screen multiple predictors while accounting for clustering:

```{r}
example16 <- uniscreen(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  model_type = "glmer",
  random = "(1|site)",
  labels = clintrial_labels
)

example16
```

## **Example 17:** Cox Screening with Mixed Effects

For survival outcomes with clustering:

```{r}
example17 <- uniscreen(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog"),
  model_type = "coxme",
  random = "(1|site)",
  labels = clintrial_labels
)

example17
```

## **Example 18:** Forest Plot from Univariable Screening

The `uniforest()` function visualizes univariable screening results:

```{r}
uni_results <- uniscreen(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog", "grade"),
  model_type = "coxph",
  labels = clintrial_labels
)

example18 <- uniforest(
  uni_results,
  title = "Univariable Screening Results",
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
queue_plot(example18)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(example18)
```

---

# Advanced Multivariate Regression Features

The `multifit()` function supports interactions and mixed-effects models when testing a predictor across multiple outcomes.

## **Example 19:** Multi-Outcome with Interactions

Test effect modification across multiple outcomes:

```{r}
example19 <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status", "os_status"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage"),
  interactions = c("treatment:sex"),
  labels = clintrial_labels,
  parallel = FALSE
)

example19
```

## **Example 20:** Multi-Outcome with Mixed Effects

Account for clustering when testing effects across outcomes:

```{r}
example20 <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  random = "(1|site)",
  model_type = "glmer",
  labels = clintrial_labels,
  parallel = FALSE
)

example20
```

## **Example 21:** Survival Multi-Outcome with Mixed Effects

For multiple survival outcomes with clustering:

```{r}
example21 <- multifit(
  data = clintrial,
  outcomes = c("Surv(pfs_months, pfs_status)",
               "Surv(os_months, os_status)"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  random = "(1|site)",
  model_type = "coxme",
  labels = clintrial_labels,
  parallel = FALSE
)

example21
```

---

# Complete Workflow Example

The following demonstrates a complete advanced analysis workflow:

```{r, fig.width = 12, fig.height = 8}
# Step 1: Screen risk factors for primary outcome
risk_screening <- uniscreen(
  data = clintrial,
  outcome = "os_status",
  predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                 "hypertension", "stage", "ecog", "treatment"),
  model_type = "glm",
  p_threshold = 0.20,
  labels = clintrial_labels
)

risk_screening

# Step 2: Test key exposure across multiple outcomes
effects <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status", "os_status"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage"),
  columns = "both",
  labels = clintrial_labels,
  parallel = FALSE
)

effects

# Step 3: Visualize effects
forest_plot <- multiforest(
  effects,
  title = "Effects Across Outcomes",
  indent_predictor = TRUE,
  zebra_stripes = TRUE
)
queue_plot(forest_plot)
```

```{r, echo = FALSE, out.width = "100%", use_rec_dims = TRUE}
print(forest_plot)
```

---

# Best Practices

## Interaction Analysis

1. Include main effects for all variables in interactions
2. Test formally using interaction *p*-values
3. Non-significant interactions do not prove effect homogeneity
4. Consider domain plausibility when interpreting

## Mixed-Effects Models

1. Use when data exhibits natural clustering
2. Start with random intercepts; add random slopes if justified
3. Monitor convergence; simplify if necessary
4. Compare to fixed-effects models using `compfit()`

## Method Selection

| Scenario | Recommended Approach |
|:---------|:---------------------|
| Clustered observations | Mixed-effects models |
| Robust inference needed | Clustered standard errors |
| PH assumption violated | Stratification |
| Effect modification | Interaction terms |
| Survey data | Weighted regression |

## Sample Size Considerations

- **Interactions**: Require larger samples than main effects models
- **Mixed models**: Need sufficient clusters (≥ 20-30 recommended)
- **Stratification**: Each stratum needs adequate events

---

# Common Issues

## Convergence Problems in Mixed Models

Simplify the random effects structure if convergence fails:

```{r, eval = FALSE}
# Start with random intercepts only
fit(data, outcome, c(predictors, "(1|site)"), model_type = "lmer")
```

Check for common problems such as few observations per cluster, near-zero variance in random effects, or highly correlated predictors.

## Interpreting Interactions

For categorical × categorical interactions, the coefficient represents the additional effect beyond the sum of main effects:

```{r, eval = FALSE}
# Access model for detailed interpretation
model <- attr(result, "model")
summary(model)
```

## Forest Plots with Interactions

Forest plots display all terms including interactions. For complex interaction patterns, consider stratified analyses or effect plots.

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis

---
title: "Survival Tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Survival Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Survival analysis requires specialized summary tables that report time-to-event outcomes in formats appropriate for longitudinal research. While `desctable()` includes basic survival summaries (median with 95% CI), detailed survival analysis often requires more comprehensive reporting: survival probabilities at specified time points, multiple quantiles, and group comparisons with appropriate statistical tests.
 
The `survtable()` function generates publication-ready survival tables with flexible output options. It uses the familiar `Surv()` syntax for specifying survival outcomes and adheres to the standard `summata` calling convention:

```{r, eval = FALSE}
survtable(data, outcome, by, times, probs, ...)
```

where `data` is the dataset, `outcome` specifies the survival endpoint using `Surv()` notation, `by` defines the grouping variable, `times` specifies landmark time points, and `probs` specifies survival quantiles to report.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset contains `r nrow(clintrial)` observations with time-to-event variables suitable for demonstrating survival summaries.

---

# Landmark Survival Estimates

Landmark survival estimates report the probability of survival at specific time points (*e.g.*, 1-year, 2-year survival rates).

## **Example 1:** Basic Landmark Analysis

The following example reports survival probabilities at 12, 24, and 36 months (with the default median survival reporting included):

```{r}
example1 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24, 36),
  time_unit = "months"
)

example1
```

## **Example 2:** Remove Median Survival Column

By default, a column showing median survival is displayed (`probs = 0.5`). To remove it, set `probs = NULL`.

```{r}
example2 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24),
  probs = NULL,
  time_unit = "months",
  total = FALSE
)

example2
```

---

# Survival Quantiles

Survival quantiles report the time at which a specified proportion of subjects have experienced the event. The median survival time (50th percentile) is the most common, but other quantiles provide additional context.

## **Example 3:** Survival Quartiles

Report multiple survival quantiles by specifying the `probs` parameter:

```{r}
example3 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "stage",
  times = NULL,
  probs = c(0.25, 0.5, 0.75),
  labels = clintrial_labels
)

example3
```

---

# Multiple Endpoints

Studies often include multiple time-to-event outcomes, such as progression-free survival (PFS) and overall survival (OS). The `survtable()` function handles multiple endpoints in a single call.

## **Example 4:** PFS and OS Comparison

Pass a vector of outcomes to compare multiple survival endpoints:

```{r}
example4 <- survtable(
  data = clintrial,
  outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
  by = "treatment",
  times = c(12, 24),
  probs = 0.5,
  time_unit = "months",
  total = FALSE,
  labels = c(
    "Surv(pfs_months, pfs_status)" = "Progression-Free Survival",
    "Surv(os_months, os_status)" = "Overall Survival"
  )
)

example4
```

---

# Output Variations

## **Example 5:** Cumulative Event Rates

For competing risks analyses or when reporting event rates rather than survival probabilities, use `type = "risk"` to display cumulative incidence (1 − survival):

```{r}
example5 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24, 36),
  type = "risk",
  time_unit = "months"
)

example5
```

## **Example 6:** Including At-Risk Counts

The number at risk at each time point provides context for the precision of survival estimates:

```{r}
example6 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24),
  stats = c("survival", "ci", "n_risk"),
  time_unit = "months",
  total = FALSE
)

example6
```

---

# Pairing with Kaplan-Meier Curves

Survival tables are often presented alongside Kaplan-Meier curves in publications. While `summata` focuses on tabular output, the `survminer` and `ggsurvfit` packages provide excellent options for survival curves.

## **Example 7:** Coordinated Figure-Table Output

The following workflow produces a matched figure-table pair suitable for publication:

```{r, eval = FALSE}
library(ggsurvfit)
library(survival)

# Fit the survival model
km_fit <- survfit(Surv(os_months, os_status) ~ treatment, data = clintrial)

# Create Kaplan-Meier plot with risk table
ggsurvfit(km_fit) +
  add_confidence_interval() +
  add_risktable() +
  add_quantile(y_value = 0.5, linetype = "dashed") +
  scale_ggsurvfit() +
  labs(
    title = "Overall Survival by Treatment",
    x = "Time (months)",
    y = "Survival Probability"
  ) +
  theme_minimal()

# Generate the companion table
surv_table <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24, 36),
  probs = 0.5,
  time_unit = "months"
)

# Export both for publication
ggsave(file.path(tempdir(), "km_curve.pdf"), width = 8, height = 6)
table2pdf(surv_table, file.path(tempdir(), "survival_table.pdf"), 
          caption = "Table 2. Survival Estimates by Treatment Group")
```

This workflow ensures that the Kaplan-Meier curve and survival table report consistent time points and groupings.

---

# Exporting Tables

Survival tables can be exported to various formats using the standard `summata` export functions. See the [Table Export](table_export.html) vignette for comprehensive documentation.

```{r, eval = FALSE}
# Microsoft Word
table2docx(
  table = example1,
  file = file.path(tempdir(), "SurvivalTable.docx"),
  caption = "Table 2. Survival Estimates by Treatment Group"
)

# PDF (requires LaTeX)
table2pdf(
  table = example1,
  file = file.path(tempdir(), "SurvivalTable.pdf"),
  caption = "Table 2. Survival Estimates by Treatment Group"
)
```

---

# Best Practices

## Time Point Selection

When selecting landmark time points, consider the following:

1. Use clinically meaningful intervals (e.g., 1-year, 2-year for oncology)
2. Ensure adequate follow-up at selected time points
3. Match time points across treatment arms for comparability
4. Consider the median follow-up when selecting the latest time point

## Reporting Recommendations

1. Always report confidence intervals alongside point estimates
2. Include the number at risk when space permits
3. Specify the time unit in table headers or footnotes
4. Use consistent decimal precision across estimates

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Regression Modeling"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Regression Modeling}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 12,
  fig.height = 6,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Regression analysis quantifies the relationship between a response variable and one or more predictors while accounting for potential confounding. The standard analytical workflow proceeds in two phases: *univariable* (unadjusted) analysis, in which each predictor is examined independently; and *multivariable* (adjusted) analysis, in which effects conditional on other predictors are examined collectively. Comparing these estimates found in these analyses reveals the extent of confounding and identifies independent predictors of a given outcome.

The `summata` package provides three principal functions for regression analysis:

| Function | Purpose |
|:---------|:--------|
| `uniscreen()` | Univariable screening of multiple predictors |
| `fit()` | Single univariable/multivariable model |
| `fullfit()` | Combined univariable and multivariable analysis |

All functions support a wide range of model types including linear models, generalized linear models (logistic, Poisson, Gaussian, Gamma, negative binomial, *etc.*), Cox proportional hazards models, and mixed-effects models, with consistent syntax and formatted output.

As with other `summata` functions, these functions adhere to the standard calling convention:

```{r, eval = FALSE}
fullfit(data, outcome, predictors, model_type, ...)
```

where `data` is the dataset, `outcome` is the outcome variable, `predictors` is a vector of predicting variables, and `model_type` is the type of model to be implemented. This vignette demonstrates the various capabilities of these functions using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset simulates a multisite oncology clinical trial with realistic outcomes and site-level clustering. Variables suitable for different regression approaches include:

| Variable | Description | Type | Application |
|:---------|:------------|:-----|:------------|
| `pain_score` | Postoperative Pain Score (0-10) | Continuous | Linear regression |
| `readmission_30d` | 30-Day Readmission | Binary | Logistic regression |
| `any_complication` | Any Complication | Binary | Logistic regression |
| `los_days` | Length of Hospital Stay (days) | Continuous (positive) | Linear or Gamma regression |
| `recovery_days` | Days to Functional Recovery | Continuous (positive) | Linear or Gamma regression |
| `ae_count` | Adverse Event Count | Count (overdispersed) | Negative binomial / quasipoisson |
| `fu_count` | Follow-Up Visit Count | Count (equidispersed) | Poisson regression |
| `pfs_months`, `pfs_status` | Progression-Free Survival Time (months) | Time-to-event | Cox regression |
| `os_months`, `os_status` | Overall Survival Time (months) | Time-to-event | Cox regression |
| `site` | Study Site | Clustering variable | Mixed-effects models |

---

# Supported Model Types

The following table summarizes all supported model types (`model_type`) and families/links (`family`). For most estimates, effect measures are displayed by default; set `exponentiate = FALSE` to display raw coefficients.

| `model_type` | `family` | Outcome | Effect Measure | Package |
|:-------------|:------------|:--------|:---------------|:--------|
| `"lm"` | — | Continuous | *β* coefficient | `stats` |
| `"glm"` | `"binomial"` | Binary | Odds ratio | `stats` |
| `"glm"` | `"quasibinomial"` | Binary (overdispersed) | Odds ratio | `stats` |
| `"glm"` | `"poisson"` | Count | Rate ratio | `stats` |
| `"glm"` | `"quasipoisson"` | Count (overdispersed) | Rate ratio | `stats` |
| `"glm"` | `"gaussian"` | Continuous | *β* coefficient | `stats` |
| `"glm"` | `"Gamma"` | Positive continuous | Ratio (with log link) | `stats` |
| `"glm"` | `"inverse.gaussian"` | Positive continuous | Coefficient | `stats` |
| `"negbin"` | — | Count (overdispersed) | Rate ratio | `MASS` |
| `"coxph"` | — | Time-to-event | Hazard ratio | `survival` |
| `"clogit"` | — | Matched binary | Odds ratio | `survival` |
| `"lmer"` | — | Continuous (clustered) | *β* coefficient | `lme4` |
| `"glmer"` | `"binomial"` | Binary (clustered) | Odds ratio | `lme4` |
| `"glmer"` | `"poisson"` | Count (clustered) | Rate ratio | `lme4` |
| `"coxme"` | — | Time-to-event (clustered) | Hazard ratio | `coxme` |

---

# Univariable Screening

The `uniscreen()` function fits separate regression models for each predictor, combining results into a single table. This approach identifies candidate predictors for multivariable modeling.

## **Example 1:** Binary Outcome Screen (Logistic Regression)

For binary outcomes, use `model_type = "glm"` (logistic regression is the default family). The following example examines predictors of 30-day hospital readmission:

```{r}
screening_vars <- c("age", "sex", "race", "bmi", "smoking", 
                    "diabetes", "stage", "ecog", "treatment")

example1 <- uniscreen(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = screening_vars,
  model_type = "glm",
  labels = clintrial_labels
)

example1
```

Each entry represents a separate univariable model. For categorical predictors, each non-reference level appears as a separate row.

## **Example 2:** Filtering Screen by *p*-value

The `p_threshold` parameter retains only predictors meeting a significance criterion:

```{r}
example2 <- uniscreen(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = screening_vars,
  model_type = "glm",
  p_threshold = 0.01,
  labels = clintrial_labels
)

example2
```

## **Example 3:** Continuous Outcome Screen (Linear Regression)

For continuous outcomes, specify `model_type = "lm"`:

```{r}
example4 <- uniscreen(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "stage", "diabetes", "ecog"),
  model_type = "lm",
  labels = clintrial_labels
)

example4
```

## **Example 4:** Time-to-Event Outcome Screen (Cox Regression)

For survival outcomes, specify the outcome using `Surv()` notation and set `model_type = "coxph"`:

```{r}
example3 <- uniscreen(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog"),
  model_type = "coxph",
  labels = clintrial_labels
)

example3
```

## **Example 5:** Retaining Model Objects

Setting `keep_models = TRUE` stores the fitted model objects for diagnostics:

```{r}
example5 <- uniscreen(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  labels = clintrial_labels,
  model_type = "glm",
  keep_models = TRUE
)

# Access individual models
models <- attr(example5, "models")
names(models)

# Examine a specific model
summary(models[["age"]])
```

---

# Multivariable Modeling

The `fit()` function estimates a single regression model with multiple predictors, producing adjusted effect estimates. It is effectively a wrapper for existing model functions in R and other supported packages that enforces `summata` syntax for more consistent function calling.

## **Example 6:** Logistic Regression

For binary outcomes with multiple predictors:

```{r}
example6 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "treatment", "stage", "diabetes"),
  model_type = "glm",
  labels = clintrial_labels
)

example6
```

## **Example 7:** Linear Regression

For continuous outcomes:

```{r}
example8 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "stage", "ecog"),
  model_type = "lm",
  labels = clintrial_labels
)

example8
```

## **Example 8:** Cox Regression

For time-to-event outcomes:

```{r}
example7 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage"),
  model_type = "coxph",
  labels = clintrial_labels
)

example7
```

## **Example 9:** Poisson Regression

For count outcomes where variance approximately equals the mean, use `model_type = "glm"` with `family = "poisson"`. The `fu_count` variable represents the number of follow-up clinic visits, which is equidispersed (suitable for standard Poisson):
  
```{r}
example9 <- fit(
  data = clintrial,
  outcome = "fu_count",
  predictors = c("age", "stage", "treatment", "surgery"),
  model_type = "glm",
  family = "poisson",
  labels = clintrial_labels
)

example9
```

Output displays rate ratios (RR). A rate ratio of 1.10 indicates a 10% higher event rate compared to the reference group. For overdispersed counts (variance > mean), consider negative binomial or quasipoisson regression instead (see Example 18).

## **Example 10:** Toggling Reference Rows for Factors

By default, reference rows are shown in output tables (`reference_rows = TRUE`). Setting this parameter to `FALSE` removes these reference rows:

```{r}
example10 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("sex", "stage", "treatment"),
  model_type = "glm",
  reference_rows = FALSE,
  labels = clintrial_labels
)

example10
```

## **Example 11:** Confidence Level

The `conf_level` parameter adjusts the confidence interval width:

```{r}
example11 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  labels = clintrial_labels,
  model_type = "glm",
  conf_level = 0.90
)

example11
```

## **Example 12:** Confidence Interval Method

By default, `summata` computes confidence intervals based on per-model best practices:

| Model Class | Default CI Method | Description |
|:------------|:------------------|:------------|
| GLM (binomial, poisson) | Profile likelihood | Via `MASS::confint.glm()`. More accurate near boundary estimates |
| Negative binomial | Profile likelihood | Via `MASS::confint.glm()` |
| Quasi-likelihood | Wald | No true likelihood; Wald is the only option |
| Linear model (`lm`) | Exact *t*-distribution | Via `confint.lm()`. Accounts for estimated residual variance |
| Cox PH (`coxph`) | Wald | Standard in survival analysis literature |
| Mixed-effects (`lmer`, `glmer`, `coxme`) | Wald | Profile too slow for routine use |

The `conf_method` parameter controls this behavior. For faster modeling, Wald (normal approximation) intervals can be substituted; this is computationally simpler but may be less accurate in edge cases. To set Wald intervals as the default method globally, use `options(summata.conf_method = "wald")`.

```{r}
# Default CIs (profile likelihood for GLM, slower but more accurate)
example12a <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  labels = clintrial_labels,
  model_type = "glm",
  conf_method = "profile"
)

example12a

# Wald CIs (faster, suitable for simulation studies or exploratory work)
example12b <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  labels = clintrial_labels,
  model_type = "glm",
  conf_method = "wald"
)

example12b
```

## **Example 13:** Raw Coefficients

For logistic and Cox models, set `exponentiate = FALSE` to display log-scale coefficients (*β* rather than *e^β^*):

```{r}
example13 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  exponentiate = FALSE
)

example13
```

---

# Combined Analysis

The `fullfit()` function integrates univariable screening and multivariable modeling into a single workflow, producing a combined table with both unadjusted and adjusted estimates.

This function extends `fit()` with an additional parameter (`method`) for variable selection:

```{r, eval = FALSE}
fullfit(data, outcome, predictors, model_type, method, ...)
```

For the `method` parameter, the following options are available. Setting `method` to one of the following options controls the predictors entering the multivariable model:

| Method | Description |
|:-------|:------------|
| `"screen"` | Only predictors meeting the *p*-threshold in univariable analysis are used in the multivariable model (default) |
| `"all"` | All predictors are used in both univariable and multivariable analyses |
| `"custom"` | Multivariable predictors are explicitly specified |

## **Example 14:** Screening-Based Selection

The default `method = "screen"` approach specifies that only predictors with univariable *p*-value below `p_threshold` are subsequently used in the multivariable analysis:

```{r}
example14 <- fullfit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                 "stage", "treatment"),
  model_type = "glm",
  method = "screen",
  p_threshold = 0.05,
  labels = clintrial_labels
)

example14
```

## **Example 15:** All Predictors

The `method = "all"` approach includes the same predictors in both univariable and multivariable analysis:

```{r}
example15 <- fullfit(
  data = clintrial,
  outcome = "any_complication",
  predictors = c("age", "sex", "treatment", "stage"),
  model_type = "glm",
  method = "all",
  labels = clintrial_labels
)

example15
```

## **Example 16:** Custom Selection

The `method = "custom"` approach allows explicit specification of multivariable predictors via the `multi_predictors` argument:

```{r}
example16 <- fullfit(
  data = clintrial,
  outcome = "icu_admission",
  predictors = c("age", "sex", "bmi", "smoking", "stage", "treatment"),
  model_type = "glm",
  method = "custom",
  multi_predictors = c("age", "sex", "stage", "treatment"),
  labels = clintrial_labels
)

example16
```

## **Example 17:** Controlling Output Columns

The `columns` parameter controls which results are displayed:

```{r}
# Univariable only
example17a <- fullfit(
  data = clintrial,
  outcome = "wound_infection",
  predictors = c("age", "sex", "stage"),
  labels = clintrial_labels,
  model_type = "glm",
  columns = "uni"
)

example17a

# Multivariable only
example17b <- fullfit(
  data = clintrial,
  outcome = "wound_infection",
  predictors = c("age", "sex", "stage"),
  labels = clintrial_labels,
  model_type = "glm",
  columns = "multi"
)

example17b
```

## **Example 18:** Survival Analysis

Output tables can use Cox regression for survival outcomes by specifying `Surv()` notation and `model_type = "coxph"`:

```{r}
example18 <- fullfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog"),
  model_type = "coxph",
  method = "screen",
  p_threshold = 0.10,
  labels = clintrial_labels
)

example18
```

---

# Extended Model Types

The `summata` package supports several additional model types for specialized analyses. These require external packages that are suggested dependencies.

## **Example 19:** Negative Binomial

Negative binomial models are appropriate for overdispersed count data where the variance exceeds the mean—a common violation of the Poisson assumption. This model type requires the `MASS` package. The `ae_count` variable in `clintrial` is generated with overdispersion, making it ideal for this demonstration.

```{r}
example19 <- fit(
  data = clintrial,
  outcome = "ae_count",
  predictors = c("age", "sex", "diabetes", "treatment"),
  model_type = "negbin",
  labels = clintrial_labels
)

example19
```

## **Example 20:** Gamma Regression

Gamma regression is appropriate for positive, continuous, right-skewed outcomes. In the `clintrial` dataset, length of hospital stay (`los_days`) fits this description. Note that R's canonical link for the Gamma family is the *inverse* link (1/μ), which produces coefficients on a difficult-to-interpret scale. The log link is generally preferred for interpretability—exponentiated coefficients then represent multiplicative effects on the mean. When `family = "Gamma"` is passed as a string, `summata` defaults to the log link:

```{r}
# String shorthand: resolves to Gamma(link = "log")
example20 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "ecog", "stage", "treatment"),
  model_type = "glm",
  family = "Gamma",
  labels = clintrial_labels
)

example20
```

## **Example 21:** Random-Intercept Model

Linear mixed-effects models (LMMs) account for hierarchical or clustered data structures. The `clintrial` dataset includes a `site` variable representing the study site, making it suitable for demonstrating random effects. This model type requires the `lme4` package.

Include random intercepts for study site using `(1|site)` notation in the predictors:

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
example21 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "lmer",
  labels = clintrial_labels
)

example21
```

## **Example 22:** GLMM for Binary Outcome

Generalized linear mixed-effects models (GLMMs) extend mixed-effects models to non-normal outcomes (binary, count). This model type also requires the `lme4` package.

Model 30-day readmission with site-level random effects:

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
example22 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "diabetes", "treatment", "(1|site)"),
  model_type = "glmer",
  family = "binomial",
  labels = clintrial_labels
)

example22
```

## **Example 23:** GLMM for Count Outcome

For count outcomes with clustering, use `fu_count` (equidispersed) with site-level random effects. Standard Poisson GLMMs assume equidispersion:

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
example23 <- fit(
  data = clintrial,
  outcome = "fu_count",
  predictors = c("age", "stage", "treatment", "(1|site)"),
  model_type = "glmer",
  family = "poisson",
  labels = clintrial_labels
)

example23
```

## **Example 24:** Cox Mixed-Effects Model

Cox mixed-effects models account for within-cluster correlation in survival outcomes. This is useful when patients are nested within sites or physicians and outcomes may be correlated within clusters. This model type requires the `coxme` package. Model overall survival with site-level random effects:
  
```{r, eval = requireNamespace("coxme", quietly = TRUE)}
example24 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "coxme",
  labels = clintrial_labels
)

example24
```

## **Example 25:** Quasibinomial for Overdispersed Binary Data

When binary or count data exhibit overdispersion (residual deviance >> residual degrees of freedom), quasi-likelihood models provide more appropriate standard errors.

```{r}
example25 <- fit(
  data = clintrial,
  outcome = "any_complication",
  predictors = c("age", "sex", "diabetes", "stage"),
  model_type = "glm",
  family = "quasibinomial",
  labels = clintrial_labels
)

example25
```

## **Example 26:** Conditional Logistic Model for Matched Data

Conditional logistic regression is used for matched case-control studies where cases and controls are matched within strata. The stratification variable should be specified using the `strata` parameter. This model type requires the `survival` package.

```{r}
# Matched case-control dataset (100 pairs)
set.seed(456)
n_pairs <- 100
matched_data <- do.call(rbind, lapply(1:n_pairs, function(i) {
  base_bmi <- rnorm(1, 27, 4)
  data.frame(
    match_id = i,
    case = c(1, 0),
    smoking = factor(c(rbinom(1, 1, 0.45), rbinom(1, 1, 0.30)), 
                     levels = c(0, 1), labels = c("No", "Yes")),
    diabetes = factor(c(rbinom(1, 1, 0.30), rbinom(1, 1, 0.20)), 
                      levels = c(0, 1), labels = c("No", "Yes")),
    bmi = c(base_bmi + rnorm(1, 1.5, 2), base_bmi + rnorm(1, -0.5, 2))
  )
}))

example26 <- fit(
  data = matched_data,
  outcome = "case",
  predictors = c("smoking", "diabetes", "bmi"),
  model_type = "clogit",
  strata = "match_id"
)

example26
```

The output shows odds ratios conditional on the matching strata.

---

# Exporting Results

Regression tables can be exported to various formats using the `tablesave()` dispatcher or the various per-format export functions:

```{r, eval = FALSE}
# Microsoft Word
table2docx(
  table = example13,
  file = "table2_regression.docx",
  caption = "Table 2. Univariable and Multivariable Analysis"
)

# PDF
table2pdf(
  table = example13,
  file = "table2_regression.pdf",
  caption = "Table 2. Regression Results"
)
```

See the [Table Export](table_export.html) vignette for comprehensive documentation.

---

# Best Practices

## Variable Selection Strategy

1. **Exploratory phase**: Use `uniscreen()` with liberal *p*-threshold (e.g., 0.20)
2. **Model building**: Use `fullfit()` with `method = "screen"` or `method = "custom"`
3. **Confirmatory analysis**: Use `fit()` with pre-specified predictors
4. **Sensitivity analysis**: Compare specifications with `compfit()` (*see* [Model Comparison](model_comparison.html))

## Choosing the Right Model

| Outcome Type | Characteristics | Recommended Model |
|:-------------|:----------------|:------------------|
| Binary | Independent observations | `glm` with `binomial` |
| Binary | Overdispersed | `glm` with `quasibinomial` |
| Binary | Clustered/hierarchical | `glmer` with `binomial` |
| Binary | Matched case-control | `clogit` |
| Count | Mean ≈ variance | `glm` with `poisson` |
| Count | Variance > mean | `negbin` or `quasipoisson` |
| Count | Clustered | `glmer` with `poisson` |
| Continuous | Symmetric, normal errors | `lm` |
| Continuous | Positive, right-skewed | `glm` with `Gamma` |
| Continuous | Clustered | `lmer` |
| Time-to-event | Independent | `coxph` |
| Time-to-event | Clustered | `coxme` |

## Interpreting Results

The comparison between univariable and multivariable estimates reveals confounding:

- **Large differences**: Substantial confounding present
- **Similar estimates**: Association robust to adjustment
- **Sign reversal**: Simpson's paradox; investigate carefully

## Sample Size Considerations

Adequate events per predictor are required for stable estimation:

- Logistic/Cox models: ≥ 10 events per predictor (rule of thumb)
- Linear models: ≥ 10–20 observations per predictor
- Mixed-effects models: ≥ 5–10 clusters recommended
- Use `method = "screen"` to reduce predictors when sample size is limited

---

# Common Issues

## Missing Data

Regression functions use complete-case analysis by default:

```{r, eval = FALSE}
# Check missingness
sapply(clintrial[, c("age", "sex", "stage")], function(x) sum(is.na(x)))

# Create complete-case dataset explicitly
complete_data <- na.omit(clintrial[, c("readmission_30d", "age", "sex", "stage")])
```

Printed output reports the analyzed sample against the data supplied, and, for model types that carry an event count, the analyzed events on the same basis:

```
Observations analyzed: 750 of 850 (88.2%)
Events analyzed: 382 of 382 (100.0%)
```

The event denominator counts only events among observations with an observed outcome, since an observation whose outcome is missing cannot be classified as an event or a non-event. Where missingness is confined to the outcome, every identifiable event is therefore analyzed and that line reads 100%; a figure below 100% indicates events lost to missing covariates.

## Factor Reference Levels

Ensure reference categories are set appropriately:

```{r, eval = FALSE}
# Set specific reference level
clintrial$stage <- relevel(factor(clintrial$stage), ref = "I")
```

## Convergence Issues

For models that fail to converge:

```{r, eval = FALSE}
# Access model for diagnostics
result <- fit(data, outcome, predictors, model_type = "glm")
model <- attr(result, "model")

# Check convergence
model$converged

# Large coefficients may indicate separation
coef(model)
```

## Overdispersion Diagnostics

To check for overdispersion in Poisson or binomial models:

```{r, eval = FALSE}
# Fit Poisson model
result <- fit(data, outcome, predictors, model_type = "glm", family = "poisson")
model <- attr(result, "model")

# Dispersion estimate (should be ~1 for no overdispersion)
sum(residuals(model, type = "pearson")^2) / model$df.residual
```

If the dispersion is substantially greater than 1, consider using `quasipoisson`, `quasibinomial`, or `negbin` instead.

## Mixed-Effects Model Convergence

Mixed-effects models can be sensitive to starting values and optimization:

```{r, eval = FALSE}
# Increase iterations
result <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "treatment", "(1|site)"),
  model_type = "glmer",
  family = "binomial",
  control = lme4::glmerControl(optimizer = "bobyqa", optCtrl = list(maxfun = 100000))
)
```

## Multinomial and Ordinal Outcomes

The `summata` package currently supports binary outcomes (via logistic regression) but not multinomial or ordinal outcomes with more than two categories. For categorical outcomes with more than two levels, use dedicated packages such as `nnet::multinom()` for multinomial regression or `MASS::polr()` for ordinal regression.

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Model Comparison"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Model Comparison}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Model selection is a fundamental problem in applied statistics. Given a set of candidate models, it is important to determine which specification best balances goodness-of-fit against parsimony. Traditional approaches rely on information criteria (AIC, BIC), discrimination metrics (C-statistic), and hypothesis tests for nested models.

The `compfit()` function synthesizes these metrics into a weighted Composite Model Score (CMS) to facilitate systematic comparison between models. Like other functions in this package, it follows the standard `summata` input paradigm, with an additional parameter to allow for multiple models:

```{r, eval = FALSE}
compfit(data, outcome, model_list, model_type, ...)
```

where `data` is the dataset, `outcome` is the common endpoint variable, `model_list` is a named list of predictor vectors (each representing a different model to be compared), and `model_type` is the type of model to be implemented. This vignette demonstrates the various capabilities of this function using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)

data(clintrial)
data(clintrial_labels)
```

---

# Quality Metrics and the Composite Model Score

The `compfit()` function evaluates models using several quality metrics, then combines them into a single Composite Model Score (CMS) ranging from 0 to 100 for rapid comparison. The metrics available depend on the model type.

## Available Metrics

| Metric | Used by | Interpretation | Better |
|:-------|:--------|:---------------|:-------|
| AIC | All | Information criterion balancing fit and complexity | Lower |
| BIC | Fallback | Information criterion with stronger complexity penalty | Lower |
| Concordance | GLM, Cox, GLMER, Cox ME | Discrimination ability (0.5 = random, 1.0 = perfect) | Higher |
| Pseudo-*R*² | GLM, Cox ME | Proportion of variation explained (McFadden's) | Higher |
| *R*² | LM | Proportion of variance explained | Higher |
| Brier Score | GLM | Prediction accuracy for binary outcomes (0 = perfect) | Lower |
| Global *p* | Cox | Omnibus likelihood ratio test *p*-value | Lower |
| RMSE | LM | Root mean squared error of residuals | Lower |
| Marginal *R*² | LMER, GLMER | Variance explained by fixed effects alone | Higher |
| Conditional *R*² | LMER | Variance explained by fixed + random effects | Higher |
| ICC | LMER, GLMER, Cox ME | Intraclass correlation; proportion of variance from clustering | Moderate |
| Convergence | All | Whether the model converged successfully | Yes |

## CMS Interpretation

| Range | Interpretation |
|:------|:---------------|
| 85–100 | Excellent |
| 75–84 | Very Good |
| 65–74 | Good |
| 55–64 | Fair |
| < 55 | Poor |

## Default CMS Weights by Model Type

The score components and their default weights vary by model type.

| Component | LM | GLM | Cox | LMER | GLMER | Cox ME |
|:----------|:--:|:---:|:---:|:----:|:-----:|:------:|
| Convergence | 15% | 15% | 15% | 20% | 15% | 15% |
| AIC | 25% | 25% | 30% | 25% | 25% | 30% |
| *R*² | 45% | | | | | |
| Pseudo-*R*² | | 15% | | | | 10% |
| Concordance | | 40% | 40% | | 30% | 35% |
| Brier Score | | 5% | | | | |
| Global *p* | | | 15% | | | |
| RMSE | 15% | | | | | |
| Marginal *R*² | | | | 25% | 15% | |
| Conditional *R*² | | | | 15% | | |
| ICC | | | | 15% | 15% | 10% |

---

# Basic Usage

## **Example 1:** Nested Model Comparison

Compare models with increasing complexity:

```{r}
example1 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Demographics" = c("age", "sex"),
    "Plus Stage" = c("age", "sex", "stage", "ecog"),
    "Full Model" = c("age", "sex", "stage", "ecog", "treatment", "smoking")
  ),
  model_type = "glm"
)

example1
```

## **Example 2:** Linear Regression Models

For continuous outcomes, use `model_type = "lm"`:

```{r}
example2 <- compfit(
  data = clintrial,
  outcome = "los_days",
  model_list = list(
    "Simple" = c("age", "sex"),
    "Disease" = c("age", "sex", "stage", "ecog"),
    "Treatment" = c("age", "sex", "stage", "ecog", "surgery", "treatment")
  ),
  model_type = "lm"
)

example2
```

## **Example 3:** Cox Regression Models

For time-to-event outcomes, use `model_type = "coxph"`:

```{r}
example3 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Unadjusted" = c("treatment"),
    "Demographics" = c("treatment", "age", "sex"),
    "Full" = c("treatment", "age", "sex", "stage", "ecog")
  ),
  model_type = "coxph"
)

example3
```

## **Example 4:** Count Models

For count outcomes, use `model_type = "glm"` with a count model family (e.g., `family = "poisson"`):

```{r}
example4 <- compfit(
  data = clintrial,
  outcome = "fu_count",
  model_list = list(
    "Minimal" = c("age", "treatment"),
    "Clinical" = c("age", "treatment", "stage", "ecog"),
    "Full" = c("age", "treatment", "stage", "ecog", "surgery", "diabetes")
  ),
  model_type = "glm",
  family = "poisson",
  labels = clintrial_labels
)

example4
```

---

# Interaction Testing

A key application of `compfit()` is testing whether interaction terms improve model fit.

## **Example 5:** Single Interaction

Compare a main-effects model to one with an interaction term:

```{r}
example5 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Main Effects" = c("age", "treatment", "sex"),
    "With Interaction" = c("age", "treatment", "sex")
  ),
  interactions_list = list(
    NULL,
    c("sex:treatment")
  ),
  model_type = "glm"
)

example5
```

## **Example 6:** Multiple Interactions

Compare different interaction hypotheses:

```{r}
example6 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Main Effects" = c("age", "treatment", "sex", "stage"),
    "Age × Treatment" = c("age", "treatment", "sex", "stage"),
    "Sex × Treatment" = c("age", "treatment", "sex", "stage"),
    "Both" = c("age", "treatment", "sex", "stage")
  ),
  interactions_list = list(
    NULL,
    c("age:treatment"),
    c("sex:treatment"),
    c("age:treatment", "sex:treatment")
  ),
  model_type = "coxph"
)

example6
```

---

# Accessing Detailed Results

## **Example 7:** Coefficient Comparison

Setting `include_coefficients = TRUE` generates a table comparing coefficients across models:

```{r}
example7 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Model A" = c("age", "sex"),
    "Model B" = c("age", "sex", "stage"),
    "Model C" = c("age", "sex", "stage", "treatment")
  ),
  model_type = "glm",
  include_coefficients = TRUE,
  labels = clintrial_labels
)

# Main comparison
example7

# Coefficient table
coef_table <- attr(example7, "coefficients")
coef_table
```

## **Example 8:** Fitted Model Objects

The underlying model objects are stored as attributes for further analysis:

```{r}
models <- attr(example7, "models")
names(models)

# Examine a specific model
summary(models[["Model C"]])
```

## **Example 9:** Recommended Model

The best-performing model is identified automatically based on the Composite Model Score (CMS):

```{r}
example9 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Minimal" = c("treatment"),
    "Standard" = c("treatment", "age", "sex", "stage"),
    "Extended" = c("treatment", "age", "sex", "stage", "ecog", "grade")
  ),
  model_type = "coxph"
)

recommended <- attr(example9, "best_model")
cat("Recommended model:", recommended, "\n")
```

---

# Custom CMS Weights

## **Example 10:** Custom Weights

Modify the CMS scoring weights to emphasize specific metrics:

```{r}
example10 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Simple" = c("age", "sex"),
    "Standard" = c("age", "sex", "stage"),
    "Full" = c("age", "sex", "stage", "treatment", "ecog")
  ),
  model_type = "glm",
  scoring_weights = list(
    convergence = 0.05,
    aic = 0.20,
    concordance = 0.60,
    pseudo_r2 = 0.10,
    brier = 0.05
  )
)

example10
```

---

# Application Scenarios

## **Scenario 1:** Confounder Assessment

Evaluate the impact of covariate adjustment on effect estimates:

```{r}
scenario1 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Crude" = c("treatment"),
    "Age-Sex Adjusted" = c("treatment", "age", "sex"),
    "Fully Adjusted" = c("treatment", "age", "sex", "stage", "ecog")
  ),
  model_type = "coxph",
  include_coefficients = TRUE,
  labels = clintrial_labels
)

scenario1

# Compare treatment effect across models
attr(scenario1, "coefficients")
```

## **Scenario 2:** Variable Selection Validation

Compare automated versus theory-driven selection:

```{r}
# Identify candidates via screening
screening <- uniscreen(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                 "hypertension", "stage", "ecog", "treatment"),
  model_type = "glm",
  p_threshold = 0.10
)

# Extract significant predictors
sig_vars <- attr(screening, "significant")

scenario2 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Theory-Driven" = c("age", "sex", "stage", "treatment"),
    "Data-Driven" = sig_vars,
    "Combined" = unique(c("age", "sex", "stage", "treatment", sig_vars))
  ),
  model_type = "glm"
)

scenario2
```

## **Scenario 3:** Parsimony Assessment

Test whether additional predictors meaningfully improve fit:

```{r}
scenario3 <- compfit(
  data = clintrial,
  outcome = "los_days",
  model_list = list(
    "3 Predictors" = c("age", "surgery", "ecog"),
    "5 Predictors" = c("age", "surgery", "ecog", "stage", "treatment"),
    "8 Predictors" = c("age", "surgery", "ecog", "stage", "treatment",
                       "sex", "smoking", "diabetes")
  ),
  model_type = "lm",
  labels = clintrial_labels
)

scenario3
```

---

# Exporting Results

Comparison tables can be exported to various formats:

```{r, eval = FALSE}
# Main comparison table
table2docx(
  table = example1,
  file = file.path(tempdir(), "Model_Comparison.docx"),
  caption = "Table 3. Model Comparison Results"
)

# Coefficient table
table2docx(
  table = attr(example6, "coefficients"),
  file = file.path(tempdir(), "Coefficient_Comparison.docx"),
  caption = "Table S1. Coefficient Estimates Across Models"
)

# PDF with landscape orientation for wide tables
table2pdf(
  table = example1,
  file = file.path(tempdir(), "Model_Comparison.pdf"),
  caption = "Model Comparison",
  orientation = "landscape"
)
```

---

# Best Practices

## When to Compare Models

1. **Covariate selection**: Determine appropriate adjustment level
2. **Interaction testing**: Evaluate effect modification
3. **Parsimony**: Balance complexity against fit
4. **Sensitivity analysis**: Compare different specifications

## Interpreting Results

1. **Multiple metrics**: The Composite Model Score (CMS) provides a summary, but examine individual metrics
2. **Practical significance**: Statistical improvement may not translate to meaningful differences
3. **Sample size**: Complex models require larger samples
4. **Convergence**: Non-converged models should be interpreted cautiously

## Limitations

1. The Composite Model Score (CMS) is a heuristic, not a formal statistical test
2. Comparisons assume models are fit on identical observations
3. Information criteria are most meaningful for nested models
4. Small score differences may not be practically significant

---

# Common Issues

## Non-Convergence

Check convergence status and consider simplifying non-converging models:

```{r, eval = FALSE}
# Check convergence status
comparison[, .(Model, Converged)]

# For non-converging models:
# 1. Reduce complexity
# 2. Check for separation (logistic)
# 3. Examine predictor correlations
```

When encountering non-converging models, consider reducing complexity, examining predictor correlations, and checking for perfect separation (primarily in logistic models).

## Differing Sample Sizes

Models with missing data may have different sample sizes, which complicates comparison:

```{r, eval = FALSE}
# Check sample sizes
comparison[, .(Model, N, Events)]

# Use complete cases for fair comparison
complete_data <- na.omit(data[, relevant_vars, with = FALSE])
```

## Interpreting Close Scores

When scores are similar, prefer parsimony and consider domain knowledge:

```{r, eval = FALSE}
# Examine individual metrics
comparison[, .(Model, `Composite Model Score (CMS)`, AIC, Concordance)]

# Prefer parsimony when scores are close
# Consider interpretability
```

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Table Export"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Table Export}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  eval = FALSE
)
old_opts <- options(width = 180)
```

Publication requires tables in specific formats: PDF for static outputs, Word documents for collaborative drafting, LaTeX for manuscript preparation, HTML for web supplements, PowerPoint for presentations, and RTF for broad compatability. The `summata` package provides export functions for each format, with consistent parameters across formats and format-specific options where needed.

| Function | Format | Dependencies |
|:---------|:-------|:-------------|
| `data.table::fwrite()` | CSV / TSV | `data.table` |
| `table2pdf()` | PDF | `xtable`, LaTeX distribution |
| `table2docx()` | Microsoft Word (.docx) | `flextable`, `officer` |
| `table2html()` | HTML | `xtable` |
| `table2pptx()` | PowerPoint (.pptx) | `flextable`, `officer` |
| `table2tex()` | LaTeX source (.tex) | `xtable` |
| `table2rtf()` | Rich Text Format (.rtf) | `flextable` |
| `autotable()` | Auto-detect from extension | Varies |

For delimited text formats (CSV, TSV), tables can be written directly with `data.table::fwrite()` since all `summata` output tables are `data.table` objects. For formatted document output, these functions follow a standard syntax when called:

``` r
result <- autotable(data, "output.pdf", ...)
```

where `data` is the dataset, and `"output.pdf"` is the name of the destination file. This vignette demonstrates the various capabilities of these functions using the included sample dataset.

---

# Preliminaries

The examples in this vignette create sample tables using `summata` functions:

```{r setup, eval = TRUE}
library(summata)

data(clintrial)
data(clintrial_labels)

# Create sample tables for export
table1 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "sex", "bmi", "smoking", "stage", "ecog"),
  labels = clintrial_labels
)

table2 <- fullfit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "stage", "treatment", "ecog"),
  model_type = "glm",
  labels = clintrial_labels
)

table3 <- fullfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "stage", "treatment", "ecog"),
  model_type = "coxph",
  labels = clintrial_labels
)

table4 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Demographics" = c("age", "sex"),
    "Clinical" = c("age", "sex", "stage", "ecog"),
    "Full" = c("age", "sex", "stage", "ecog", "treatment")
  ),
  model_type = "glm"
)
```

---

# CSV / TSV Export

All `summata` output tables are `data.table` objects, so they can be written to delimited text files directly with `data.table::fwrite()`. This is the simplest export path and requires no additional dependencies beyond `data.table`, which `summata` already imports.

## **Example 1:** CSV Export

```{r}
data.table::fwrite(table1, file.path(tempdir(), "Table1.csv"))
```

## **Example 2:** TSV Export

Specify a tab delimiter for TSV output:

```{r}
data.table::fwrite(table1, file.path(tempdir(), "Table1.tsv"), sep = "\t")
```

## **Example 3:** Semicolon-Delimited (European Locale)

Use a semicolon delimiter for compatibility with locales where the comma is the decimal separator:

```{r}
data.table::fwrite(table1, file.path(tempdir(), "Table1.csv"), sep = ";")
```

CSV and TSV files are useful for archiving raw table output, importing into spreadsheet software, or downstream processing in other statistical environments. For formatted, publication-ready output, use the export functions described in the sections that follow.

---

# PDF Export

The `table2pdf()` function creates PDF documents using LaTeX typesetting via the `xtable` package. A LaTeX distribution (TinyTeX, TeX Live, MiKTeX, or MacTeX) is required.

## **Example 4:** Basic Export

The simplest usage requires only a table and filename:

```{r}
table2pdf(
  table = table1,
  file = file.path(tempdir(), "Table1.pdf")
)
```

## **Example 5:** With Caption

Add a caption using the `caption` parameter:

```{r}
table2pdf(
  table = table1,
  file = file.path(tempdir(), "Table1.pdf"),
  caption = "Table 1. Baseline Characteristics by Group"
)
```

## **Example 6:** With Formatting Options

Multiple formatting options can be combined:

```{r}
table2pdf(
  table = table2,
  file = file.path(tempdir(), "Table2.pdf"),
  caption = "Table 2. Regression Analysis",
  font_size = 8,
  bold_significant = TRUE,
  p_threshold = 0.05,
  indent_groups = TRUE,
  variable_padding = TRUE,
  zebra_stripes = TRUE
)
```

## **Example 7:** Page Layout

Control page size and orientation:

```{r}
table2pdf(
  table = table2,
  file = file.path(tempdir(), "Table2_Landscape.pdf"),
  caption = "Table 2. Regression Results",
  paper = "letter",
  orientation = "landscape"
)
```

## **Example 8:** Custom Margins

Adjust page margins (in inches):

```{r}
table2pdf(
  table = table2,
  file = file.path(tempdir(), "Table2_Margins.pdf"),
  margins = c(0.5, 0.5, 0.5, 0.5)  # top, right, bottom, left
)
```

## **Example 9:** Fit to Page Width

Scale tables to fit the page width:

```{r}
table2pdf(
  table = table2,
  file = file.path(tempdir(), "Table2_Fitted.pdf"),
  fit_to_page = TRUE
)
```

## **Example 10:** Standalone Table

Standalone settings are useful for embedding tables in larger documents:

```{r}
table2pdf(
  table = table2,
  file = file.path(tempdir(), "Table2_Auto.pdf"),
  paper = "auto"
)
```

---

# Microsoft Word Export

The `table2docx()` function creates editable Word documents using the `flextable` and `officer` packages.

## **Example 11:** Basic Export

The simplest usage requires only a table and filename:

```{r}
table2docx(
  table = table1,
  file = file.path(tempdir(), "Table1.docx")
)
```

## **Example 12:** With Caption

Add a caption using the `caption` parameter:

```{r}
table2docx(
  table = table1,
  file = file.path(tempdir(), "Table1.docx"),
  caption = "Table 1. Baseline Characteristics by Group"
)
```

## **Example 13:** With Formatting Options

Multiple formatting options can be combined:

```{r}
table2docx(
  table = table2,
  file = file.path(tempdir(), "Table2.docx"),
  caption = "Table 2. Regression Analysis",
  font_size = 9,
  font_family = "Times New Roman",
  bold_significant = TRUE,
  p_threshold = 0.05,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

## **Example 14:** Page Layout

Control page size and orientation for wide tables:

```{r}
table2docx(
  table = table2,
  file = file.path(tempdir(), "Table2_Landscape.docx"),
  caption = "Table 2. Regression Results",
  paper = "letter",
  orientation = "landscape"
)
```

## **Example 15:** Dark Header Style

Use a dark header background for visual emphasis:

```{r}
table2docx(
  table = table1,
  file = file.path(tempdir(), "Table1_DarkHeader.docx"),
  caption = "Table 1. Baseline Characteristics",
  dark_header = TRUE,
  zebra_stripes = TRUE
)
```

---

# HTML Export

The `table2html()` function exports tables to HTML file output for web use via the `xtable` package.

## **Example 16:** Basic Export

The simplest usage requires only a table and filename:

```{r}
table2html(
  table = table1,
  file = file.path(tempdir(), "Table1.html")
)
```

## **Example 17:** With Caption

Add a caption using the `caption` parameter:

```{r}
table2html(
  table = table1,
  file = file.path(tempdir(), "Table1.html"),
  caption = "Table 1. Baseline Characteristics"
)
```

## **Example 18:** With Formatting Options

Multiple formatting options can be combined:

```{r}
table2html(
  table = table2,
  file = file.path(tempdir(), "Table2.html"),
  caption = "Table 2. Regression Analysis",
  bold_significant = TRUE,
  indent_groups = TRUE,
  zebra_stripes = TRUE,
  stripe_color = "#f5f5f5"
)
```

---

# PowerPoint Export

The `table2pptx()` function creates PowerPoint presentations with tables using the `flextable` and `officer` packages.

## **Example 19:** Basic Export

The simplest usage creates a single-slide presentation:

```{r}
table2pptx(
  table = table1,
  file = file.path(tempdir(), "Table1.pptx")
)
```

## **Example 20:** With Slide Title

Add a slide title using the `slide_title` parameter:

```{r}
table2pptx(
  table = table1,
  file = file.path(tempdir(), "Table1.pptx"),
  slide_title = "Baseline Characteristics"
)
```

## **Example 21:** With Formatting Options

Multiple formatting options can be combined:

```{r}
table2pptx(
  table = table2,
  file = file.path(tempdir(), "Table2.pptx"),
  slide_title = "Regression Analysis",
  font_size = 10,
  font_family = "Arial",
  bold_significant = TRUE,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

## **Example 22:** Custom Template

Use an existing PowerPoint template:

```{r}
table2pptx(
  table = table1,
  file = file.path(tempdir(), "Table1_Custom.pptx"),
  template = "my_template.pptx",
  layout = "Title and Content",
  master = "Office Theme"
)
```

---

# LaTeX Export

The `table2tex()` function creates LaTeX source files for inclusion in manuscripts via the `xtable` package.

## **Example 23:** Basic Export

The simplest usage requires only a table and filename:

```{r}
table2tex(
  table = table1,
  file = file.path(tempdir(), "Table1.tex")
)
```

## **Example 24:** With Caption and Label

Add caption and cross-reference label:

```{r}
table2tex(
  table = table1,
  file = file.path(tempdir(), "Table1.tex"),
  caption = "Baseline Characteristics by Group",
  label = "tab:demographics"
)
```

## **Example 25:** With Booktabs Styling

Use the booktabs package for professional formatting:

```{r}
table2tex(
  table = table2,
  file = file.path(tempdir(), "Table2.tex"),
  caption = "Regression Analysis",
  label = "tab:regression",
  booktabs = TRUE,
  bold_significant = TRUE,
  indent_groups = TRUE
)
```

---

# RTF Export

The `table2rtf()` function creates Rich Text Format files using the `flextable` and `officer` packages.

## **Example 26:** Basic Export

The simplest usage requires only a table and filename:

```{r}
table2rtf(
  table = table1,
  file = file.path(tempdir(), "Table1.rtf")
)
```

## **Example 27:** With Formatting Options

Multiple formatting options can be combined:

```{r}
table2rtf(
  table = table2,
  file = file.path(tempdir(), "Table2.rtf"),
  caption = "Table 2. Regression Analysis",
  font_size = 9,
  font_family = "Times New Roman",
  bold_significant = TRUE,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

## **Example 28:** Page Layout

Control page size and orientation:

```{r}
table2rtf(
  table = table2,
  file = file.path(tempdir(), "Table2_Landscape.rtf"),
  paper = "letter",
  orientation = "landscape"
)
```

---

# Automatic Format Detection

The `autotable()` function detects the output format from the file extension, simplifying the export workflow:

```{r}
autotable(table1, file.path(tempdir(), "Table1.csv"))   # CSV output
autotable(table1, file.path(tempdir(), "Table1.tsv"))   # TSV output
autotable(table1, file.path(tempdir(), "Table1.pdf"))   # PDF output
autotable(table1, file.path(tempdir(), "Table1.docx"))  # DOCX output
autotable(table1, file.path(tempdir(), "Table1.html"))  # HTML output
autotable(table1, file.path(tempdir(), "Table1.pptx"))  # PPTX output
autotable(table1, file.path(tempdir(), "Table1.tex"))   # TeX output
autotable(table1, file.path(tempdir(), "Table1.rtf"))   # RTF output
```

Format-specific parameters pass through to the underlying function:

```{r}
autotable(
  table = table2,
  file = file.path(tempdir(), "Table2.pdf"),
  caption = "Table 2. Regression Analysis",
  orientation = "landscape",
  font_size = 8,
  bold_significant = TRUE,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

---

# Common Formatting Parameters

All export functions share common parameters for consistent results:

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `caption` | Table caption/title | `NULL` |
| `font_size` | Base font size (points) | 8–10 |
| `format_headers` | Auto-format column headers | `TRUE` |
| `bold_significant` | Bold significant *p*-values | `TRUE` |
| `p_threshold` | Significance threshold | 0.05 |
| `indent_groups` | Indent factor levels | `FALSE` |
| `condense_table` | Show essential rows only | `FALSE` |
| `condense_quantitative` | Condense continuous/survival only (`desctable()` only) | `FALSE` |
| `zebra_stripes` | Alternating row shading | `FALSE` |
| `dark_header` | Dark header background | `FALSE` |

## Condensing Options

The `condense_table` and `condense_quantitative` parameters reduce table height:

- `condense_table = TRUE`: Condenses continuous, survival, and binary categorical variables
- `condense_quantitative = TRUE`: Condenses only continuous and survival variables (`desctable()` outputs only)

```{r}
# Full display
table2docx(table1, file.path(tempdir(), "Table1_Full.docx"))

# Condense all variable types
table2docx(
  table = table1,
  file = file.path(tempdir(), "Table1_Condensed.docx"),
  condense_table = TRUE,
  zebra_stripes = TRUE
)

# Condense only continuous/survival (descriptive tables only)
table2docx(
  table = table1,
  file = file.path(tempdir(), "Table1_CondenseQuant.docx"),
  condense_quantitative = TRUE,
  zebra_stripes = TRUE
)
```

---

# Format-Specific Parameters

## PDF

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `margins` | Page margins `c(top, right, bottom, left)` | `c(1, 1, 1, 1)` |
| `fit_to_page` | Scale table to fit page width | `TRUE` |
| `cell_padding` | Vertical padding ("none", "normal", "relaxed", "loose") | `"normal"` |
| `variable_padding` | Add spacing between variables | `FALSE` |
| `show_logs` | Keep LaTeX log files | `FALSE` |

## Word/RTF/PowerPoint

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `font_family` | Font name | `Arial` |
| `width` | Table width (inches) | `NULL` |
| `align` | Column alignment vector | `NULL` |
| `return_ft` | Return flextable object | `FALSE` |

## HTML

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `include_css` | Include embedded CSS styling | `FALSE` |
| `stripe_color` | Zebra stripe color (hex or name) | `"#EEEEEE"` |

## LaTeX

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `booktabs` | Use booktabs package styling | `FALSE` |
| `label` | LaTeX cross-reference label | `NULL` |
| `cell_padding` | Arraystretch value | `"normal"` |

## PowerPoint (additional)

| Parameter | Description | Default |
|:----------|:------------|:--------|
| `template` | Custom PPTX template file | `NULL` |
| `layout` | Slide layout name | `"Title and Content"` |
| `master` | Slide master name | `"Office Theme"` |
| `left`, `top` | Table position (inches) | `0.5`, `1.5` |

---

# Best Practices

## Format Selection

| Use-Case | Recommended Format |
|:---------|:-------------------|
| Data archival, further processing | CSV (.csv) or TSV (.tsv) |
| Journal submission | PDF (.pdf) or LaTeX (.tex) |
| Manuscript drafts, collaboration | Word (.docx) |
| Web/R Markdown | HTML (.html) |
| Presentations | PowerPoint (.pptx) |
| Maximum compatibility | RTF (.rtf) |

## Page Layout Guidelines

1. **Portrait**: Tables with ≤ 6 columns
2. **Landscape**: Tables with > 6 columns
3. **Auto-sizing**: Use `paper = "auto"` for embedding

## Consistent Multi-Format Export

Export to multiple formats with consistent formatting:

```{r}
common_opts <- list(
  caption = "Table 2. Regression Analysis",
  bold_significant = TRUE,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)

formats <- c("csv", "pdf", "docx", "html", "pptx", "rtf", "tex")

for (fmt in formats) {
  autotable(
    table = table2,
    file = file.path(tempdir(), paste0("Table2.", fmt)),
    caption = common_opts$caption,
    bold_significant = common_opts$bold_significant,
    indent_groups = common_opts$indent_groups,
    zebra_stripes = common_opts$zebra_stripes
  )
}
```

---

# Troubleshooting

## PDF Compilation Fails

Verify LaTeX installation and consider keeping log files for debugging:

```{r}
# Check LaTeX installation
Sys.which("pdflatex")

# Install TinyTeX if needed
# tinytex::install_tinytex()

# Keep log files for debugging
table2pdf(table, file.path(tempdir(), "debug.pdf"), show_logs = TRUE)
```

See [Installation and Setup](installation_setup.html) for LaTeX configuration details.

## Table Too Wide

Several options address wide tables:

```{r}
# Use landscape orientation
table2pdf(table, file.path(tempdir(), "wide.pdf"), orientation = "landscape")

# Enable fit-to-page scaling
table2pdf(table, file.path(tempdir(), "wide.pdf"), fit_to_page = TRUE)

# Reduce font size
table2pdf(table, file.path(tempdir(), "wide.pdf"), font_size = 7)
```

## Customizing flextable Output

Get the flextable object for further customization:

```{r}
ft <- table2docx(table1, file.path(tempdir(), "Table1.docx"), return_ft = TRUE)

library(flextable)
ft <- ft %>%
  bold(i = 1, part = "header") %>%
  color(i = 1, color = "navy", part = "header")

save_as_docx(ft, path = file.path(tempdir(), "Table1_Custom.docx"))
```

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Descriptive Tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Descriptive Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Descriptive statistics provide the foundation for any quantitative analysis. Before estimating relationships between variables, it is essential to first characterize the distribution of each variable and assess balance across comparison groups. A well-constructed descriptive table—often designated "Table 1" in published research—accomplishes three objectives: it summarizes the central tendency and dispersion of continuous variables, tabulates the frequency distribution of categorical variables, and tests for systematic differences between groups.

The `desctable()` function generates publication-ready descriptive tables with automatic detection of variable types, appropriate summary statistics, and optional hypothesis testing.  It adheres to the to the standard `summata` calling convention:

```{r, eval = FALSE}
desctable(data, by, variables, ...)
```

where `data` is the dataset, `by` specifies the grouping variable (optional), and `variables` lists the variables to summarize. This vignette demonstrates the function’s capabilities using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset contains `r nrow(clintrial)` observations with continuous, categorical, and time-to-event variables suitable for demonstrating descriptive statistics. The `clintrial_labels` vector provides human-readable labels for display.

---

# Summary Statistics and Tests

The `desctable()` function automatically selects appropriate summary statistics and hypothesis tests based on variable type:

| Variable Type | Summary Statistic | Two Groups | Three+ Groups |
|:--------------|:------------------|:-----------|:--------------|
| Continuous (parametric) | Mean ± SD | *t*-test | ANOVA |
| Continuous (nonparametric) | Median [IQR] | Wilcoxon | Kruskal–Wallis |
| Categorical | *n* (%) | χ² or Fisher | χ² or Fisher |
| Time-to-event | Median (95% CI) | Log-rank | Log-rank |

For categorical variables, Fisher exact test is used when any expected cell count falls below 5. For continuous variables, the test selection follows the displayed statistic: parametric tests are used with mean-based statistics, nonparametric tests with median-based statistics.

---

# Basic Usage

## **Example 1:** Grouped Descriptive Table

The most common use-case for descriptive tables is comparing characteristics across groups:

```{r}
desc_vars <- c("age", "sex", "race", "bmi", "stage", "ecog", 
               "Surv(os_months, os_status)")

example1 <- desctable(
  data = clintrial, 
  by = "treatment", 
  variables = desc_vars,
  labels = clintrial_labels
)

example1
```

The output includes a "Total" column by default, showing overall statistics alongside group-specific values.

## **Example 2:** Ungrouped Summary Statistics

Omitting the `by` argument produces overall summary statistics without group comparisons:

```{r}
example2 <- desctable(
  data = clintrial,
  variables = c("age", "bmi", "sex", "stage"),
  labels = clintrial_labels
)

example2
```

---

# Customizing Summary Statistics

The default summary statistics can be customized for both continuous and categorical variables.

## **Example 3:** Continuous Variables

The `stats_continuous` parameter controls how continuous variables are summarized:

| Value | Output Format |
|:------|:--------------|
| `"mean_sd"` | Mean ± SD |
| `"median_iqr"` | Median [Q1–Q3] (default) |
| `"median_range"` | Median (min–max) |

```{r}
example3 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "los_days"),
  stats_continuous = c("mean_sd", "median_iqr", "median_range"),
  labels = clintrial_labels
)

example3
```

## **Example 4:** Categorical Variables

The `stats_categorical` parameter controls categorical variable display:

| Value | Output Format |
|:------|:--------------|
| `"n_percent"` | *n* (%) (default) |
| `"n"` | *n* only |
| `"percent"` | % only |

```{r}
example4 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("sex", "stage", "ecog"),
  stats_categorical = "percent",
  labels = clintrial_labels
)

example4
```

## **Example 5:** Numeric Precision

Control decimal places with `digits` (for statistics) and `p_digits` (for *p*-values):

```{r}
example5 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "sex"),
  digits = 2,
  p_digits = 4,
  test = TRUE,
  labels = clintrial_labels
)

example5
```

---

# Statistical Testing

When comparing groups, hypothesis tests assess whether observed differences are statistically significant.

## **Example 6:** Disabling Automatic Test Selection

By default, automatic hypothesis testing based on the summary statistic is displayed (`test = TRUE`). Setting `test = FALSE` disables this functionality:

```{r}
example6 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "sex", "stage"),
  test = FALSE,
  labels = clintrial_labels
)

example6
```

## **Example 7:** Specifying Tests Manually

Override automatic selection with `test_continuous` and `test_categorical`. Available test specifications include:

**Continuous** (`test_continuous`):

- `"auto"`: Automatic selection (default)
- `"t"`: Student *t*-test
- `"wrs"`: Wilcoxon rank-sum test
- `"aov"`: One-way ANOVA
- `"kwt"`: Kruskal–Wallis test

**Categorical** (`test_categorical`):

- `"auto"`: Automatic selection (default)
- `"chisq"`: Pearson χ² test
- `"fisher"`: Fisher exact test

The following example forces parametric tests for continuous variables:

```{r}
example7a <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "los_days"),
  test = TRUE,
  test_continuous = "aov",  # ANOVA
  labels = clintrial_labels
)

example7a
```

This example forces the Fisher exact test for categorical variables:

```{r}
example7b <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("sex", "stage"),
  test = TRUE,
  test_categorical = "fisher",
  labels = clintrial_labels
)

example7b
```

---

# Handling Missing Data

Missing values require special consideration in descriptive tables. Options control whether missing values are displayed and how percentages are calculated.

## **Example 8:** Including Missing Values

By default, missing values are excluded from calculations. Set `na_include = TRUE` to display them as a separate category:

```{r}
example8 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking", "diabetes"),
  na_include = TRUE,
  labels = clintrial_labels
)

example8
```

## **Example 9:** Missing Value Denominators

The `na_percent` parameter controls whether missing values are included in percentage calculations:

```{r}
# Percentages exclude missing (denominator = non-missing)
example9a <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking"),
  na_include = TRUE,
  na_percent = FALSE,
  labels = clintrial_labels
)

example9a

# Percentages include missing (denominator = total)
example9b <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking"),
  na_include = TRUE,
  na_percent = TRUE,
  labels = clintrial_labels
)

example9b
```

## **Example 10:** Custom Missing Value Label

The label for missing values can be customized using the `na_label` parameter:

```{r}
example10 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking"),
  na_include = TRUE,
  na_label = "Not Reported",
  labels = clintrial_labels
)

example10
```

---

# Total Column Options

The total column provides overall statistics alongside group-specific values.

## **Example 11:** Total Column Configuration

The `total` parameter controls the presence and position of the total column:

| Value | Effect |
|:------|:-------|
| `TRUE`, `"first"` | Total column first (default) |
| `"last"` | Total column last |
| `FALSE` | No total column |

```{r}
# Total column in last position
example11a <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "sex", "stage"),
  total = "last",
  labels = clintrial_labels
)

example11a

# No total column
example11b <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "sex", "stage"),
  total = FALSE,
  labels = clintrial_labels
)

example11b
```

---

# Complete Example

The following demonstrates a comprehensive descriptive table suitable for publication:

```{r}
table1 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c(
    "age", "sex", "race", "ethnicity", "bmi",
    "smoking", "diabetes", "hypertension",
    "stage", "grade", "ecog",
    "Surv(os_months, os_status)"
  ),
  labels = clintrial_labels,
  stats_continuous = "mean_sd",
  stats_categorical = "n_percent",
  test = TRUE,
  total = TRUE,
  digits = 1,
  p_digits = 3
)

table1
```

## Accessing Raw Data

The underlying numeric values are stored as an attribute for programmatic access:

```{r}
raw_data <- attr(table1, "raw_data")
head(raw_data)
```

---

# Survival Summary Tables

For detailed survival analysis—including landmark survival estimates, survival quantiles, and multiple endpoints—see the dedicated [Survival Tables](survival_tables.html) vignette. The `survtable()` function provides comprehensive options for reporting time-to-event outcomes.

---

# Exporting Tables

Descriptive tables can be exported to various formats. See the [Table Export](table_export.html) vignette for comprehensive documentation.

```{r, eval = FALSE}
# Microsoft Word
table2docx(
  table = table1,
  file = file.path(tempdir(), "Table1.docx"),
  caption = "Table 1. Baseline Characteristics by Group"
)

# PDF (requires LaTeX)
table2pdf(
  table = table1,
  file = file.path(tempdir(), "Table1.pdf"),
  caption = "Table 1. Baseline Characteristics by Group"
)

# HTML
table2html(
  table = table1,
  file = file.path(tempdir(), "Table1.html"),
  caption = "Table 1. Baseline Characteristics by Group"
)
```

---

# Best Practices

## Variable Selection

1. Include all relevant baseline characteristics
2. Order variables logically (typically chronologically and by domain)
3. Exclude the grouping variable from the variables list

## Statistical Considerations

1. Use automatic test selection unless there is specific justification otherwise
2. Report exact *p*-values when possible; very small values display as "< 0.001" (or to preferred degree of precision)
3. Consider multiple comparison adjustments when testing many variables
4. For skewed or non-normally distributed continuous variables, report using nonparametric statistical procedures (e.g., median with IQR) rather than parametric ones (e.g., mean with SD)

## Formatting Recommendations

1. Use consistent decimal precision within variable types
2. Include units in variable labels (e.g., "Age (years)")
3. Include a total column for context
4. Use landscape orientation for tables with many columns

---

# Common Issues

## Empty Cells in Categorical Variables

When a factor level has zero observations in a group, ensure all levels are explicitly defined:

```{r, eval = FALSE}
data$stage <- factor(data$stage, levels = c("I", "II", "III", "IV"))
```

## Skewed Continuous Variables

For highly skewed distributions, use median and IQR:

```{r, eval = FALSE}
desctable(data, by, variables, stats_continuous = "median_iqr")
```

## Large Tables

For tables with many variables, consider splitting by category or using landscape orientation for export:

```{r, eval = FALSE}
table2pdf(table, file.path(tempdir(), "table1.pdf"), orientation = "landscape", font_size = 8)
```

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Survival Tables](survival_tables.html): `survtable()` for time-to-event summaries
- [Regression Modeling](regression_modeling.html): `fit()`, `uniscreen()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Regression Modeling"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Regression Modeling}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  fig.width = 12,
  fig.height = 6,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Regression analysis quantifies the relationship between a response variable and one or more predictors while accounting for potential confounding. The standard analytical workflow proceeds in two phases: *univariable* (unadjusted) analysis, in which each predictor is examined independently; and *multivariable* (adjusted) analysis, in which effects conditional on other predictors are examined collectively. Comparing these estimates found in these analyses reveals the extent of confounding and identifies independent predictors of a given outcome.

The `summata` package provides three principal functions for regression analysis:

| Function | Purpose |
|:---------|:--------|
| `uniscreen()` | Univariable screening of multiple predictors |
| `fit()` | Single univariable/multivariable model |
| `fullfit()` | Combined univariable and multivariable analysis |

All functions support a wide range of model types including linear models, generalized linear models (logistic, Poisson, Gaussian, Gamma, negative binomial), Cox proportional hazards models, and mixed-effects models, with consistent syntax and formatted output.

As with other `summata` functions, these functions adhere to the standard calling convention:

```{r, eval = FALSE}
fullfit(data, outcome, predictors, model_type, ...)
```

where `data` is the dataset, `outcome` is the outcome variable, `predictors` is a vector of predicting variables, and `model_type` is the type of model to be implemented. This vignette demonstrates the various capabilities of these functions using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset simulates a multisite oncology clinical trial with realistic outcomes and site-level clustering. Variables suitable for different regression approaches include:

| Variable | Description | Type | Application |
|:---------|:------------|:-----|:------------|
| `pain_score` | Postoperative Pain Score (0-10) | Continuous | Linear regression |
| `readmission_30d` | 30-Day Readmission | Binary | Logistic regression |
| `any_complication` | Any Complication | Binary | Logistic regression |
| `los_days` | Length of Hospital Stay (days) | Continuous (positive) | Linear or Gamma regression |
| `recovery_days` | Days to Functional Recovery | Continuous (positive) | Linear or Gamma regression |
| `ae_count` | Adverse Event Count | Count (overdispersed) | Negative binomial / quasipoisson |
| `fu_count` | Follow-Up Visit Count | Count (equidispersed) | Poisson regression |
| `pfs_months`, `pfs_status` | Progression-Free Survival Time (months) | Time-to-event | Cox regression |
| `os_months`, `os_status` | Overall Survival Time (months) | Time-to-event | Cox regression |
| `site` | Study Site | Clustering variable | Mixed-effects models |

---

# Supported Model Types

The following table summarizes all supported model types (`model_type`) and families/links (`family`). For most estimates, effect measures are displayed by default; set `exponentiate = FALSE` to display raw coefficients.

| `model_type` | `family` | Outcome | Effect Measure | Package |
|:-------------|:------------|:--------|:---------------|:--------|
| `"lm"` | — | Continuous | *β* coefficient | `stats` |
| `"glm"` | `"binomial"` | Binary | Odds ratio | `stats` |
| `"glm"` | `"quasibinomial"` | Binary (overdispersed) | Odds ratio | `stats` |
| `"glm"` | `"poisson"` | Count | Rate ratio | `stats` |
| `"glm"` | `"quasipoisson"` | Count (overdispersed) | Rate ratio | `stats` |
| `"glm"` | `"gaussian"` | Continuous | *β* coefficient | `stats` |
| `"glm"` | `"Gamma"` | Positive continuous | Ratio (with log link) | `stats` |
| `"glm"` | `"inverse.gaussian"` | Positive continuous | Coefficient | `stats` |
| `"negbin"` | — | Count (overdispersed) | Rate ratio | `MASS` |
| `"coxph"` | — | Time-to-event | Hazard ratio | `survival` |
| `"clogit"` | — | Matched binary | Odds ratio | `survival` |
| `"lmer"` | — | Continuous (clustered) | *β* coefficient | `lme4` |
| `"glmer"` | `"binomial"` | Binary (clustered) | Odds ratio | `lme4` |
| `"glmer"` | `"poisson"` | Count (clustered) | Rate ratio | `lme4` |
| `"coxme"` | — | Time-to-event (clustered) | Hazard ratio | `coxme` |

---

# Univariable Screening

The `uniscreen()` function fits separate regression models for each predictor, combining results into a single table. This approach identifies candidate predictors for multivariable modeling.

## **Example 1:** Binary Outcome Screen (Logistic Regression)

For binary outcomes, use `model_type = "glm"` (logistic regression is the default family). Here we examine predictors of 30-day hospital readmission:

```{r}
screening_vars <- c("age", "sex", "race", "bmi", "smoking", 
                    "diabetes", "stage", "ecog", "treatment")

example1 <- uniscreen(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = screening_vars,
  model_type = "glm",
  labels = clintrial_labels
)

example1
```

Each entry represents a separate univariable model. For categorical predictors, each non-reference level appears as a separate row.

## **Example 2:** Filtering Screen by *p*-value

The `p_threshold` parameter retains only predictors meeting a significance criterion:

```{r}
example2 <- uniscreen(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = screening_vars,
  model_type = "glm",
  p_threshold = 0.01,
  labels = clintrial_labels
)

example2
```

## **Example 3:** Continuous Outcome Screen (Linear Regression)

For continuous outcomes, specify `model_type = "lm"`:

```{r}
example4 <- uniscreen(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "stage", "diabetes", "ecog"),
  model_type = "lm",
  labels = clintrial_labels
)

example4
```

## **Example 4:** Time-to-Event Outcome Screen (Cox Regression)

For survival outcomes, specify the outcome using `Surv()` notation and set `model_type = "coxph"`:

```{r}
example3 <- uniscreen(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog"),
  model_type = "coxph",
  labels = clintrial_labels
)

example3
```

## **Example 5:** Retaining Model Objects

Setting `keep_models = TRUE` stores the fitted model objects for diagnostics:

```{r}
example5 <- uniscreen(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  keep_models = TRUE
)

# Access individual models
models <- attr(example5, "models")
names(models)

# Examine a specific model
summary(models[["age"]])
```

---

# Multivariable Modeling

The `fit()` function estimates a single regression model with multiple predictors, producing adjusted effect estimates. It is effectively a wrapper for existing model functions in R and other supported packages that enforces `summata` syntax for more consistent function calling.

## **Example 6:** Logistic Regression

For binary outcomes with multiple predictors:

```{r}
example6 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "treatment", "stage", "diabetes"),
  model_type = "glm",
  labels = clintrial_labels
)

example6
```

## **Example 7:** Linear Regression

For continuous outcomes:

```{r}
example8 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "stage", "ecog"),
  model_type = "lm",
  labels = clintrial_labels
)

example8
```

## **Example 8:** Cox Regression

For time-to-event outcomes:

```{r}
example7 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage"),
  model_type = "coxph",
  labels = clintrial_labels
)

example7
```

## **Example 9:** Poisson Regression

For count outcomes where variance approximately equals the mean, use `model_type = "glm"` with `family = "poisson"`. The `fu_count` variable represents the number of follow-up clinic visits, which is equidispersed (suitable for standard Poisson):
  
```{r}
example9 <- fit(
  data = clintrial,
  outcome = "fu_count",
  predictors = c("age", "stage", "treatment", "surgery"),
  model_type = "glm",
  family = "poisson",
  labels = clintrial_labels
)

example9
```

Output displays rate ratios (RR). A rate ratio of 1.10 indicates a 10% higher event rate compared to the reference group. For overdispersed counts (variance > mean), consider negative binomial or quasipoisson regression instead (see Example 18).

## **Example 10:** Toggling Reference Rows for Factors

By default, reference rows are shown in output tables (`reference_rows = TRUE`). Setting this parameter to `FALSE` removes these reference rows:

```{r}
example10 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("sex", "stage", "treatment"),
  model_type = "glm",
  reference_rows = FALSE,
  labels = clintrial_labels
)

example10
```

## **Example 11:** Confidence Level

The `conf_level` parameter adjusts the confidence interval width:

```{r}
example11 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  conf_level = 0.90
)

example11
```

## **Example 12:** Confidence Interval Method

By default, `summata` computes confidence intervals based on per-model best practices:

| Model Class | Default CI Method | Description |
|:------------|:------------------|:------------|
| GLM (binomial, poisson) | Profile likelihood | Via `MASS::confint.glm()`. More accurate near boundary estimates |
| Negative binomial | Profile likelihood | Via `MASS::confint.glm()` |
| Quasi-likelihood | Wald | No true likelihood; Wald is the only option |
| Linear model (`lm`) | Exact *t*-distribution | Via `confint.lm()`. Accounts for estimated residual variance |
| Cox PH (`coxph`) | Wald | Standard in survival analysis literature |
| Mixed-effects (`lmer`, `glmer`, `coxme`) | Wald | Profile too slow for routine use |

The `conf_method` parameter controls this behavior. For faster modeling, Wald (normal approximation) intervals can be substituted; this is computationally simpler but may be less accurate in edge cases. To set Wald intervals as the default method globally, use `options(summata.conf_method = "wald")`.

```{r}
# Default CIs (profile likelihood for GLM, slower but more accurate)
example12a <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  conf_method = "profile"
)

example12a

# Wald CIs (faster, suitable for simulation studies or exploratory work)
example12b <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  conf_method = "wald"
)

example12b
```

## **Example 13:** Raw Coefficients

For logistic and Cox models, set `exponentiate = FALSE` to display log-scale coefficients (*β* rather than *e^β^*):

```{r}
example13 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  exponentiate = FALSE
)

example13
```

---

# Combined Analysis

The `fullfit()` function integrates univariable screening and multivariable modeling into a single workflow, producing a combined table with both unadjusted and adjusted estimates.

This function extends `fit()` with an additional parameter (`method`) for variable selection:

```{r, eval = FALSE}
fullfit(data, outcome, predictors, model_type, method, ...)
```

For the `method` parameter, the following options are available. Setting `method` to one of the following options controls the predictors entering the multivariable model:

| Method | Description |
|:-------|:------------|
| `"screen"` | Only predictors meeting the *p*-threshold in univariable analysis are used in the multivariable model (default) |
| `"all"` | All predictors are used in both univariable and multivariable analyses |
| `"custom"` | Multivariable predictors are explicitly specified |

## **Example 14:** Screening-Based Selection

The default `method = "screen"` approach specifies that only predictors with univariable *p*-value below `p_threshold` are subsequently used in the multivariable analysis:

```{r}
example14 <- fullfit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                 "stage", "treatment"),
  model_type = "glm",
  method = "screen",
  p_threshold = 0.05,
  labels = clintrial_labels
)

example14
```

## **Example 15:** All Predictors

The `method = "all"` approach includes the same predictors in both univariable and multivariable analysis:

```{r}
example15 <- fullfit(
  data = clintrial,
  outcome = "any_complication",
  predictors = c("age", "sex", "treatment", "stage"),
  model_type = "glm",
  method = "all",
  labels = clintrial_labels
)

example15
```

## **Example 16:** Custom Selection

The `method = "custom"` approach allows explicit specification of multivariable predictors via the `multi_predictors` argument:

```{r}
example16 <- fullfit(
  data = clintrial,
  outcome = "icu_admission",
  predictors = c("age", "sex", "bmi", "smoking", "stage", "treatment"),
  model_type = "glm",
  method = "custom",
  multi_predictors = c("age", "sex", "stage", "treatment"),
  labels = clintrial_labels
)

example16
```

## **Example 17:** Controlling Output Columns

The `columns` parameter controls which results are displayed:

```{r}
# Univariable only
example17a <- fullfit(
  data = clintrial,
  outcome = "wound_infection",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  columns = "uni"
)

example17a

# Multivariable only
example17b <- fullfit(
  data = clintrial,
  outcome = "wound_infection",
  predictors = c("age", "sex", "stage"),
  model_type = "glm",
  columns = "multi"
)

example17b
```

## **Example 18:** Survival Analysis

Output tables can use Cox regression for survival outcomes by specifying `Surv()` notation and `model_type = "coxph"`:

```{r}
example18 <- fullfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog"),
  model_type = "coxph",
  method = "screen",
  p_threshold = 0.10,
  labels = clintrial_labels
)

example18
```

---

# Extended Model Types

The `summata` package supports several additional model types for specialized analyses. These require external packages that are suggested dependencies.

## **Example 19:** Negative Binomial

Negative binomial models are appropriate for overdispersed count data where the variance exceeds the mean—a common violation of the Poisson assumption. This model type requires the `MASS` package. The `ae_count` variable in `clintrial` is generated with overdispersion, making it ideal for this demonstration.

```{r}
example19 <- fit(
  data = clintrial,
  outcome = "ae_count",
  predictors = c("age", "sex", "diabetes", "treatment"),
  model_type = "negbin",
  labels = clintrial_labels
)

example19
```

## **Example 20:** Gamma Regression

Gamma regression is appropriate for positive, continuous, right-skewed outcomes. In the `clintrial` dataset, length of hospital stay (`los_days`) fits this description. Note that R's canonical link for the Gamma family is the *inverse* link (1/μ), which produces coefficients on a difficult-to-interpret scale. The log link is generally preferred for interpretability—exponentiated coefficients then represent multiplicative effects on the mean. When `family = "Gamma"` is passed as a string, `summata` defaults to the log link:

```{r}
# String shorthand: resolves to Gamma(link = "log")
example20 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "ecog", "stage", "treatment"),
  model_type = "glm",
  family = "Gamma",
  labels = clintrial_labels
)

example20
```

## **Example 21:** Random-Intercept Model

Linear mixed-effects models (LMMs) account for hierarchical or clustered data structures. The `clintrial` dataset includes a `site` variable representing the study site, making it suitable for demonstrating random effects. This model type requires the `lme4` package.

Include random intercepts for study site using `(1|site)` notation in the predictors:

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
example21 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "lmer",
  labels = clintrial_labels
)

example21
```

## **Example 22:** GLMM for Binary Outcome

Generalized linear mixed-effects models (GLMMs) extend mixed-effects models to non-normal outcomes (binary, count). This model type also requires the `lme4` package.

Model 30-day readmission with site-level random effects:

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
example22 <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "sex", "diabetes", "treatment", "(1|site)"),
  model_type = "glmer",
  family = "binomial",
  labels = clintrial_labels
)

example22
```

## **Example 23:** GLMM for Count Outcome

For count outcomes with clustering, use `fu_count` (equidispersed) with site-level random effects. Standard Poisson GLMMs assume equidispersion:

```{r, eval = requireNamespace("lme4", quietly = TRUE)}
example23 <- fit(
  data = clintrial,
  outcome = "fu_count",
  predictors = c("age", "stage", "treatment", "(1|site)"),
  model_type = "glmer",
  family = "poisson",
  labels = clintrial_labels
)

example23
```

## **Example 24:** Cox Mixed-Effects Model

Cox mixed-effects models account for within-cluster correlation in survival outcomes. This is useful when patients are nested within sites or physicians and outcomes may be correlated within clusters. This model type requires the `coxme` package. Model overall survival with site-level random effects:
  
```{r, eval = requireNamespace("coxme", quietly = TRUE)}
example24 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "coxme",
  labels = clintrial_labels
)

example24
```

## **Example 25:** Quasibinomial for Overdispersed Binary Data

When binary or count data exhibit overdispersion (residual deviance >> residual degrees of freedom), quasi-likelihood models provide more appropriate standard errors.

```{r}
example25 <- fit(
  data = clintrial,
  outcome = "any_complication",
  predictors = c("age", "sex", "diabetes", "stage"),
  model_type = "glm",
  family = "quasibinomial",
  labels = clintrial_labels
)

example25
```

## **Example 26:** Conditional Logistic Model for Matched Data

Conditional logistic regression is used for matched case-control studies where cases and controls are matched within strata. The stratification variable should be specified using the `strata` parameter. This model type requires the `survival` package.

```{r}
# Matched case-control dataset (100 pairs)
set.seed(456)
n_pairs <- 100
matched_data <- do.call(rbind, lapply(1:n_pairs, function(i) {
  base_bmi <- rnorm(1, 27, 4)
  data.frame(
    match_id = i,
    case = c(1, 0),
    smoking = factor(c(rbinom(1, 1, 0.45), rbinom(1, 1, 0.30)), 
                     levels = c(0, 1), labels = c("No", "Yes")),
    diabetes = factor(c(rbinom(1, 1, 0.30), rbinom(1, 1, 0.20)), 
                      levels = c(0, 1), labels = c("No", "Yes")),
    bmi = c(base_bmi + rnorm(1, 1.5, 2), base_bmi + rnorm(1, -0.5, 2))
  )
}))

example26 <- fit(
  data = matched_data,
  outcome = "case",
  predictors = c("smoking", "diabetes", "bmi"),
  model_type = "clogit",
  strata = "match_id"
)

example26
```

The output shows odds ratios conditional on the matching strata.

---

# Exporting Results

Regression tables can be exported to various formats:

```{r, eval = FALSE}
# Microsoft Word
table2docx(
  table = example13,
  file = file.path(tempdir(), "Table2_Regression.docx"),
  caption = "Table 2. Univariable and Multivariable Analysis"
)

# PDF
table2pdf(
  table = example13,
  file = file.path(tempdir(), "Table2_Regression.pdf"),
  caption = "Table 2. Regression Results"
)
```

See the [Table Export](table_export.html) vignette for comprehensive documentation.

---

# Best Practices

## Variable Selection Strategy

1. **Exploratory phase**: Use `uniscreen()` with liberal *p*-threshold (e.g., 0.20)
2. **Model building**: Use `fullfit()` with `method = "screen"` or `method = "custom"`
3. **Confirmatory analysis**: Use `fit()` with pre-specified predictors
4. **Sensitivity analysis**: Compare specifications with `compfit()` (*see* [Model Comparison](model_comparison.html))

## Choosing the Right Model

| Outcome Type | Characteristics | Recommended Model |
|:-------------|:----------------|:------------------|
| Binary | Independent observations | `glm` with `binomial` |
| Binary | Overdispersed | `glm` with `quasibinomial` |
| Binary | Clustered/hierarchical | `glmer` with `binomial` |
| Binary | Matched case-control | `clogit` |
| Count | Mean ≈ variance | `glm` with `poisson` |
| Count | Variance > mean | `negbin` or `quasipoisson` |
| Count | Clustered | `glmer` with `poisson` |
| Continuous | Symmetric, normal errors | `lm` |
| Continuous | Positive, right-skewed | `glm` with `Gamma` |
| Continuous | Clustered | `lmer` |
| Time-to-event | Independent | `coxph` |
| Time-to-event | Clustered | `coxme` |

## Interpreting Results

The comparison between univariable and multivariable estimates reveals confounding:

- **Large differences**: Substantial confounding present
- **Similar estimates**: Association robust to adjustment
- **Sign reversal**: Simpson's paradox; investigate carefully

## Sample Size Considerations

Adequate events per predictor are required for stable estimation:

- Logistic/Cox models: ≥10 events per predictor (rule of thumb)
- Linear models: ≥10–20 observations per predictor
- Mixed-effects models: ≥5–10 clusters recommended
- Use `method = "screen"` to reduce predictors when sample size is limited

---

# Common Issues

## Missing Data

Regression functions use complete-case analysis by default:

```{r, eval = FALSE}
# Check missingness
sapply(clintrial[, c("age", "sex", "stage")], function(x) sum(is.na(x)))

# Create complete-case dataset explicitly
complete_data <- na.omit(clintrial[, c("readmission_30d", "age", "sex", "stage")])
```

## Factor Reference Levels

Ensure reference categories are set appropriately:

```{r, eval = FALSE}
# Set specific reference level
clintrial$stage <- relevel(factor(clintrial$stage), ref = "I")
```

## Convergence Issues

For models that fail to converge:

```{r, eval = FALSE}
# Access model for diagnostics
result <- fit(data, outcome, predictors, model_type = "glm")
model <- attr(result, "model")

# Check convergence
model$converged

# Large coefficients may indicate separation
coef(model)
```

## Overdispersion Diagnostics

To check for overdispersion in Poisson or binomial models:

```{r, eval = FALSE}
# Fit Poisson model
result <- fit(data, outcome, predictors, model_type = "glm", family = "poisson")
model <- attr(result, "model")

# Dispersion estimate (should be ~1 for no overdispersion)
sum(residuals(model, type = "pearson")^2) / model$df.residual
```

If the dispersion is substantially greater than 1, consider using `quasipoisson`, `quasibinomial`, or `negbin` instead.

## Mixed-Effects Model Convergence

Mixed-effects models can be sensitive to starting values and optimization:

```{r, eval = FALSE}
# Increase iterations
result <- fit(
  data = clintrial,
  outcome = "readmission_30d",
  predictors = c("age", "treatment", "(1|site)"),
  model_type = "glmer",
  family = "binomial",
  control = lme4::glmerControl(optimizer = "bobyqa", optCtrl = list(maxfun = 100000))
)
```

## Multinomial and Ordinal Outcomes

The `summata` package currently supports binary outcomes (via logistic regression) but not multinomial or ordinal outcomes with more than two categories. For categorical outcomes with more than two levels, use dedicated packages such as `nnet::multinom()` for multinomial regression or `MASS::polr()` for ordinal regression.

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Development Roadmap"
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

This article outlines planned features for future versions of `summata`, informed by capabilities available in related packages and user feedback. The roadmap is tentative and subject to change based on development priorities and community input.

---

# Planned Features

The following features are under consideration for future releases, organized by priority level.

## Higher Priority

Features expected in near-term releases:

| Feature | Description | Reference |
|:--------|:------------|:----------|
| Paired/longitudinal tables | Summary tables for paired measurements across time points | `arsenal` |
| Subgroup analysis | Stratified analyses with syntax similar to `tbl_strata()` | `gtsummary` |
| Missing data visualization | Visual patterns of missingness for data quality assessment | `finalfit` |

## Medium Priority

Features planned for subsequent releases:

| Feature | Description | Reference |
|:--------|:------------|:----------|
| Inline text reporting | Extract statistics for R Markdown inline reporting | `gtsummary` |
| Survey-weighted analysis | Support for `svyglm()`, `svycoxph()` from the survey package | `gtsummary`, `survey` |
| Competing risks regression | Fine-Gray models via the `cmprsk` package | `finalfit` |
| Multinomial regression | Support for `nnet::multinom()` with multi-level categorical outcomes | `gtsummary`, `nnet` |
| Ordinal regression | Support for `MASS::polr()` and `ordinal::clm()` for ordered categorical outcomes | `MASS`, `ordinal` |
| Custom statistics | User-defined summary statistics functions for `desctable()` | `gtsummary` |

## Lower Priority

Features under consideration for future releases:

| Feature | Description | Reference |
|:--------|:------------|:----------|
| Bootstrap CIs | Bootstrapped confidence intervals for model predictions | `finalfit` |
| GAM models | Generalized additive model support via `mgcv` | `gtsummary` |
| Bayesian models | Support for `brmsfit` and `stanreg` model objects | `gtsummary` |

---

# Contributing

Contributions to `summata` are welcome. If interested in implementing a roadmap feature or proposing a new one, please consult the contributing guidelines in the [package repository](https://codeberg.org/phmcc/summata).

## Bug Reports

Bug reports and feature requests may be submitted via the issue tracker, either on [Codeberg](https://codeberg.org/phmcc/summata/issues) or [GitHub](https://github.com/phmcc/summata/issues).

## Development Repository

- **Primary development**: [codeberg.org/phmcc/summata](https://codeberg.org/phmcc/summata)
- **GitHub mirror**: [github.com/phmcc/summata](https://github.com/phmcc/summata)

---

# Version History

See the [Changelog](news/index.html) for a detailed history of changes in each release.

---

# Additional Resources

- [Feature Comparison](feature_comparison.html): Comparison with related packages
- [Benchmarks](benchmarks.html): Performance comparisons
- [gtsummary documentation](https://www.danieldsjoberg.com/gtsummary/)
- [finalfit documentation](https://finalfit.org/)
- [arsenal documentation](https://mayoverse.github.io/arsenal/)

---
title: "Model Comparison"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Model Comparison}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Model selection is a fundamental problem in applied statistics. Given a set of candidate models, it is important to determine which specification best balances goodness-of-fit against parsimony. Traditional approaches rely on information criteria (AIC, BIC), discrimination metrics (C-statistic), and hypothesis tests for nested models.

The `compfit()` function synthesizes these metrics into a weighted Composite Model Score (CMS) to facilitate systematic comparison between models. Like other functions in this package, it follows the standard `summata` input paradigm, with an additional parameter to allow for multiple models:

```{r, eval = FALSE}
compfit(data, outcome, model_list, model_type, ...)
```

where `data` is the dataset, `outcome` is the common endpoint variable, `model_list` is a named list of predictor vectors (each representing a different model to be compared), and `model_type` is the type of model to be implemented. This vignette demonstrates the various capabilities of this function using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)

data(clintrial)
data(clintrial_labels)
```

---

# Quality Metrics and the Composite Model Score

The `compfit()` function evaluates models using several quality metrics, then combines them into a single Composite Model Score (CMS) ranging from 0 to 100 for rapid comparison. The metrics available depend on the model type.

## Available Metrics

| Metric | Used by | Interpretation | Better |
|:-------|:--------|:---------------|:-------|
| AIC | All | Information criterion balancing fit and complexity | Lower |
| BIC | Fallback | Information criterion with stronger complexity penalty | Lower |
| Concordance | GLM, Cox, GLMER, Cox ME | Discrimination ability (0.5 = random, 1.0 = perfect) | Higher |
| Pseudo-*R*² | GLM, Cox ME | Proportion of variation explained (McFadden's) | Higher |
| *R*² | LM | Proportion of variance explained | Higher |
| Brier Score | GLM | Prediction accuracy for binary outcomes (0 = perfect) | Lower |
| Global *p* | Cox | Omnibus likelihood ratio test *p*-value | Lower |
| RMSE | LM | Root mean squared error of residuals | Lower |
| Marginal *R*² | LMER, GLMER | Variance explained by fixed effects alone | Higher |
| Conditional *R*² | LMER | Variance explained by fixed + random effects | Higher |
| ICC | LMER, GLMER, Cox ME | Intraclass correlation; proportion of variance from clustering | Moderate |
| Convergence | All | Whether the model converged successfully | Yes |

## CMS Interpretation

| Range | Interpretation |
|:------|:---------------|
| 85–100 | Excellent |
| 75–84 | Very Good |
| 65–74 | Good |
| 55–64 | Fair |
| < 55 | Poor |

## Default CMS Weights by Model Type

The score components and their default weights vary by model type.

| Component | LM | GLM | Cox | LMER | GLMER | Cox ME |
|:----------|:--:|:---:|:---:|:----:|:-----:|:------:|
| Convergence | 15% | 15% | 15% | 20% | 15% | 15% |
| AIC | 25% | 25% | 30% | 25% | 25% | 30% |
| *R*² | 45% | | | | | |
| Pseudo-*R*² | | 15% | | | | 10% |
| Concordance | | 40% | 40% | | 30% | 35% |
| Brier Score | | 5% | | | | |
| Global *p* | | | 15% | | | |
| RMSE | 15% | | | | | |
| Marginal *R*² | | | | 25% | 15% | |
| Conditional *R*² | | | | 15% | | |
| ICC | | | | 15% | 15% | 10% |

---

# Basic Usage

## **Example 1:** Nested Model Comparison

Compare models with increasing complexity:

```{r}
example1 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Demographics" = c("age", "sex"),
    "Plus Stage" = c("age", "sex", "stage", "ecog"),
    "Full Model" = c("age", "sex", "stage", "ecog", "treatment", "smoking")
  ),
  model_type = "glm"
)

example1
```

## **Example 2:** Linear Regression Models

For continuous outcomes, use `model_type = "lm"`:

```{r}
example2 <- compfit(
  data = clintrial,
  outcome = "los_days",
  model_list = list(
    "Simple" = c("age", "sex"),
    "Disease" = c("age", "sex", "stage", "ecog"),
    "Treatment" = c("age", "sex", "stage", "ecog", "surgery", "treatment")
  ),
  model_type = "lm"
)

example2
```

## **Example 3:** Cox Regression Models

For time-to-event outcomes, use `model_type = "coxph"`:

```{r}
example3 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Unadjusted" = c("treatment"),
    "Demographics" = c("treatment", "age", "sex"),
    "Full" = c("treatment", "age", "sex", "stage", "ecog")
  ),
  model_type = "coxph"
)

example3
```

## **Example 4:** Count Models

For count outcomes, use `model_type = "glm"` with a count model family (e.g., `family = "poisson"`):

```{r}
example4 <- compfit(
  data = clintrial,
  outcome = "fu_count",
  model_list = list(
    "Minimal" = c("age", "treatment"),
    "Clinical" = c("age", "treatment", "stage", "ecog"),
    "Full" = c("age", "treatment", "stage", "ecog", "surgery", "diabetes")
  ),
  model_type = "glm",
  family = "poisson",
  labels = clintrial_labels
)

example4
```

---

# Interaction Testing

A key application of `compfit()` is testing whether interaction terms improve model fit.

## **Example 5:** Single Interaction

Compare a main-effects model to one with an interaction term:

```{r}
example5 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Main Effects" = c("age", "treatment", "sex"),
    "With Interaction" = c("age", "treatment", "sex")
  ),
  interactions_list = list(
    NULL,
    c("sex:treatment")
  ),
  model_type = "glm"
)

example5
```

## **Example 6:** Multiple Interactions

Compare different interaction hypotheses:

```{r}
example6 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Main Effects" = c("age", "treatment", "sex", "stage"),
    "Age × Treatment" = c("age", "treatment", "sex", "stage"),
    "Sex × Treatment" = c("age", "treatment", "sex", "stage"),
    "Both" = c("age", "treatment", "sex", "stage")
  ),
  interactions_list = list(
    NULL,
    c("age:treatment"),
    c("sex:treatment"),
    c("age:treatment", "sex:treatment")
  ),
  model_type = "coxph"
)

example6
```

---

# Accessing Detailed Results

## **Example 7:** Coefficient Comparison

Setting `include_coefficients = TRUE` generates a table comparing coefficients across models:

```{r}
example7 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Model A" = c("age", "sex"),
    "Model B" = c("age", "sex", "stage"),
    "Model C" = c("age", "sex", "stage", "treatment")
  ),
  model_type = "glm",
  include_coefficients = TRUE,
  labels = clintrial_labels
)

# Main comparison
example7

# Coefficient table
coef_table <- attr(example7, "coefficients")
coef_table
```

## **Example 8:** Fitted Model Objects

The underlying model objects are stored as attributes for further analysis:

```{r}
models <- attr(example7, "models")
names(models)

# Examine a specific model
summary(models[["Model C"]])
```

## **Example 9:** Recommended Model

The best-performing model is identified automatically based on the Composite Model Score (CMS):

```{r}
example9 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Minimal" = c("treatment"),
    "Standard" = c("treatment", "age", "sex", "stage"),
    "Extended" = c("treatment", "age", "sex", "stage", "ecog", "grade")
  ),
  model_type = "coxph"
)

recommended <- attr(example9, "best_model")
cat("Recommended model:", recommended, "\n")
```

---

# Custom CMS Weights

## **Example 10:** Custom Weights

Modify the CMS scoring weights to emphasize specific metrics:

```{r}
example10 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Simple" = c("age", "sex"),
    "Standard" = c("age", "sex", "stage"),
    "Full" = c("age", "sex", "stage", "treatment", "ecog")
  ),
  model_type = "glm",
  scoring_weights = list(
    convergence = 0.05,
    aic = 0.20,
    concordance = 0.60,
    pseudo_r2 = 0.10,
    brier = 0.05
  )
)

example10
```

---

# Application Scenarios

## **Scenario 1:** Confounder Assessment

Evaluate the impact of covariate adjustment on effect estimates:

```{r}
scenario1 <- compfit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  model_list = list(
    "Crude" = c("treatment"),
    "Age-Sex Adjusted" = c("treatment", "age", "sex"),
    "Fully Adjusted" = c("treatment", "age", "sex", "stage", "ecog")
  ),
  model_type = "coxph",
  include_coefficients = TRUE,
  labels = clintrial_labels
)

scenario1

# Compare treatment effect across models
attr(scenario1, "coefficients")
```

## **Scenario 2:** Variable Selection Validation

Compare automated versus theory-driven selection:

```{r}
# Identify candidates via screening
screening <- uniscreen(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                 "hypertension", "stage", "ecog", "treatment"),
  model_type = "glm",
  p_threshold = 0.10
)

# Extract significant predictors
sig_vars <- attr(screening, "significant")

scenario2 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Theory-Driven" = c("age", "sex", "stage", "treatment"),
    "Data-Driven" = sig_vars,
    "Combined" = unique(c("age", "sex", "stage", "treatment", sig_vars))
  ),
  model_type = "glm"
)

scenario2
```

## **Scenario 3:** Parsimony Assessment

Test whether additional predictors meaningfully improve fit:

```{r}
scenario3 <- compfit(
  data = clintrial,
  outcome = "los_days",
  model_list = list(
    "3 Predictors" = c("age", "surgery", "ecog"),
    "5 Predictors" = c("age", "surgery", "ecog", "stage", "treatment"),
    "8 Predictors" = c("age", "surgery", "ecog", "stage", "treatment",
                       "sex", "smoking", "diabetes")
  ),
  model_type = "lm",
  labels = clintrial_labels
)

scenario3
```

---

# Exporting Results

Comparison tables can be exported to various formats using the `tablesave()` dispatcher or the various per-format export functions:

```{r, eval = FALSE}
# Main comparison table
table2docx(
  table = example1,
  file = "model_comparison.docx",
  caption = "Table 3. Model Comparison Results"
)

# Coefficient table
table2docx(
  table = attr(example6, "coefficients"),
  file = "coefficient_comparison.docx",
  caption = "Table S1. Coefficient Estimates Across Models"
)

# PDF with landscape orientation for wide tables
table2pdf(
  table = example1,
  file = "model_comparison.pdf",
  caption = "Model Comparison",
  orientation = "landscape"
)
```

---

# Best Practices

## When to Compare Models

1. **Covariate selection**: Determine appropriate adjustment level
2. **Interaction testing**: Evaluate effect modification
3. **Parsimony**: Balance complexity against fit
4. **Sensitivity analysis**: Compare different specifications

## Interpreting Results

1. **Multiple metrics**: The Composite Model Score (CMS) provides a summary, but examine individual metrics
2. **Practical significance**: Statistical improvement may not translate to meaningful differences
3. **Sample size**: Complex models require larger samples
4. **Convergence**: Non-converged models should be interpreted cautiously

## Limitations

1. The Composite Model Score (CMS) is a heuristic, not a formal statistical test
2. Comparisons assume models are fit on identical observations
3. Information criteria are most meaningful for nested models
4. Small score differences may not be practically significant

---

# Common Issues

## Non-Convergence

Check convergence status and consider simplifying non-converging models:

```{r, eval = FALSE}
# Check convergence status
comparison[, .(Model, Converged)]

# For non-converging models:
# 1. Reduce complexity
# 2. Check for separation (logistic)
# 3. Examine predictor correlations
```

When encountering non-converging models, consider reducing complexity, examining predictor correlations, and checking for perfect separation (primarily in logistic models).

## Differing Sample Sizes

Models with missing data may have different sample sizes, which complicates comparison:

```{r, eval = FALSE}
# Check sample sizes
comparison[, .(Model, N, Events)]

# Use complete cases for fair comparison
complete_data <- na.omit(data[, relevant_vars, with = FALSE])
```

## Interpreting Close Scores

When scores are similar, prefer parsimony and consider domain knowledge:

```{r, eval = FALSE}
# Examine individual metrics
comparison[, .(Model, `Composite Model Score (CMS)`, AIC, Concordance)]

# Prefer parsimony when scores are close
# Consider interpretability
```

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `uniscreen()`, `fit()`, and `fullfit()`
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Statistical Foundations"
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  eval = FALSE
)
```

This article provides a comprehensive reference for the statistical methods implemented in `summata`. It serves as a technical companion to the package vignettes, detailing the mathematical foundations, assumptions, and interpretation of each supported model class.

For practical guidance on using these methods, see the [Regression Modeling](regression_modeling.html) vignette.

---

# Notation and Conventions

The following notation is used throughout this article:

| Symbol | Description |
|:-------|:------------|
| *Y* | Response (outcome) variable |
| *X*₁, ..., *X*ₚ | Predictor (covariate) variables |
| *β*₀ | Intercept |
| *β*₁, ..., *β*ₚ | Regression coefficients |
| *n* | Sample size |
| *i* | Observation index (*i* = 1, ..., *n*) |
| *j* | Cluster index (for hierarchical data) |
| *t* | Time (for survival models) |
| *g*(·) | Link function |
| exp(·) | Exponential function |
| log(·) | Natural logarithm |

---

# Hypothesis Testing

## Wald Test

Tests H₀: *β* = 0 using:

$$z = \frac{\hat{\beta}}{\text{SE}(\hat{\beta})}$$

Under H₀, *z* ~ N(0, 1) for GLM and Cox models. For linear models, the analogous statistic follows a *t*-distribution with *n* − *p* − 1 degrees of freedom.

## Likelihood Ratio Test

Compares nested models:

$$\Lambda = -2[\log L_{\text{reduced}} - \log L_{\text{full}}]$$

Under H₀, Λ ~ χ²ₖ, where *k* is the difference in parameters.

## Global Test

Tests H₀: all *β*ⱼ = 0 (except intercept):

- **Linear models**: F-test
- **GLMs**: Likelihood ratio test
- **Cox models**: Likelihood ratio, Wald, or score test

---

# Confidence Intervals

## Wald Confidence Intervals

For coefficient *β* with standard error SE(*β*):

$$\beta \pm z_{1-\alpha/2} \cdot \text{SE}(\beta)$$

where *z*₁₋α/₂ is the standard normal quantile (1.96 for 95% CI). Wald intervals are computationally inexpensive but may be inaccurate when the likelihood surface is asymmetric, which can occur with small samples, sparse data, or estimates near boundary values.

## Exact *t*-Distribution Intervals

For linear models, confidence intervals use the *t*-distribution to account for estimation of the residual variance:

$$\beta \pm t_{n-p-1, \; 1-\alpha/2} \cdot \text{SE}(\beta)$$

where *t*ₙ₋ₚ₋₁ is the *t*-quantile with *n* − *p* − 1 degrees of freedom. This produces wider intervals than the normal approximation, particularly when *n* is small relative to *p*.

## Profile Likelihood Intervals

Profile likelihood intervals are based on inverting the likelihood ratio test. The confidence set for *β* is defined as:

$$\{\beta : 2[\log L(\hat{\beta}) - \log L(\beta)] \leq \chi^2_{1,1-\alpha}\}$$

Profile intervals are more accurate than Wald intervals for small samples or boundary estimates, because they respect the curvature of the likelihood surface rather than approximating it as quadratic. However, they require iterative computation (refitting the model for each parameter), which increases execution time.

## Confidence Intervals for Effect Measures

For exponentiated coefficients (OR, HR, RR), the CI is obtained by exponentiating the interval endpoints from the coefficient scale:

$$\left(\exp(\text{CI}_{\text{lower}}), \; \exp(\text{CI}_{\text{upper}})\right)$$

where CI_lower and CI_upper are the confidence limits for *β* on the log scale, computed using whichever method (Wald, exact *t*, or profile likelihood) is appropriate for the model class. Note that exponentiation preserves coverage probability because exp(·) is a monotone transformation.

## Implementation

The CI method is selected automatically based on model class, and can be overridden via the `conf_method` parameter:

| Model Class | Default (`conf_method = "profile"`) | With `conf_method = "wald"` |
|:------------|:----------------------------------|:--------------------------|
| GLM (binomial, poisson) | Profile likelihood via `MASS::confint.glm()` | Wald (normal approximation) |
| Negative binomial | Profile likelihood via `MASS::confint.glm()` | Wald (normal approximation) |
| Quasi-likelihood (quasibinomial, quasipoisson) | Wald (no true likelihood) | Wald (no true likelihood) |
| Linear model (`lm`) | Exact *t*-distribution via `confint.lm()` | Wald (normal approximation) |
| Cox PH (`coxph`) | Wald | Wald |
| Mixed-effects (`lmer`, `glmer`, `coxme`) | Wald | Wald |

Cox and mixed-effects models use Wald intervals regardless of the `conf_method` setting. For Cox models, Wald CIs are the standard convention in the survival analysis literature. For mixed-effects models, profile likelihood CIs are available in principle (via `lme4::confint.merMod(method = "profile")`), but are prohibitively slow for routine use and are not implemented in `summata`.

The `conf_method` can be set globally for a session:

```{r}
options(summata.conf_method = "profile")  # Profile/exact-t (default)
options(summata.conf_method = "wald")  # Wald CIs throughout
```

---

# Linear Regression

## Model Specification

Linear regression models the expected value of a continuous response as a linear combination of predictors:

$$E[Y_i | X_i] = \beta_0 + \beta_1 X_{i1} + \beta_2 X_{i2} + \cdots + \beta_p X_{ip}$$

Equivalently, in matrix notation:

$$\mathbf{Y} = \mathbf{X}\mathbf{\beta} + \mathbf{\varepsilon}$$

where $\mathbf{\varepsilon} \sim N(\mathbf{0}, \sigma^2\mathbf{I})$.

## Assumptions

1. **Linearity**: The relationship between predictors and response is linear
2. **Independence**: Observations are independent
3. **Homoscedasticity**: Constant error variance across all levels of predictors
4. **Normality**: Errors are normally distributed (for inference)
5. **No multicollinearity**: Predictors are not perfectly correlated

## Parameter Estimation

Coefficients are estimated by ordinary least squares (OLS):

$$\hat{\mathbf{\beta}} = (\mathbf{X}'\mathbf{X})^{-1}\mathbf{X}'\mathbf{Y}$$

## Interpretation

Each coefficient *β*ⱼ represents the expected change in *Y* for a one-unit increase in *X*ⱼ, holding all other predictors constant.

| Predictor Type | Interpretation of *β* |
|:---------------|:----------------------|
| Continuous | Change in *Y* per unit increase in *X* |
| Binary (0/1) | Difference in *Y* between groups |
| Categorical | Difference from reference category |

## Goodness of Fit

**Coefficient of determination (*R*²):**
$$R^2 = 1 - \frac{\sum_i (Y_i - \hat{Y}_i)^2}{\sum_i (Y_i - \bar{Y})^2}$$

**Adjusted *R*²:**
$$R^2_{\text{adj}} = 1 - \frac{(1 - R^2)(n - 1)}{n - p - 1}$$

**Root mean squared error (RMSE):**
$$\text{RMSE} = \sqrt{\frac{1}{n}\sum_i (Y_i - \hat{Y}_i)^2}$$

Note: this is the descriptive (biased) RMSE. The unbiased estimate of the residual standard deviation σ uses *n* − *p* − 1 in the denominator and is reported by `summary.lm()` as the "residual standard error."

## Implementation

In `summata`, linear regression is specified with `model_type = "lm"`:

```{r}
fit(data, outcome = "continuous_var", predictors, model_type = "lm")
```

---

# Generalized Linear Models

Generalized linear models (GLMs) extend linear regression to accommodate non-normal response distributions through three components:

1. **Random component**: Distribution of *Y* from the exponential family
2. **Systematic component**: Linear predictor $\eta = \mathbf{X}\mathbf{\beta}$
3. **Link function**: $g(\mu) = \eta$, where $\mu = E[Y]$

## Logistic Regression

### Model Specification

For binary outcomes *Y* ∈ {0, 1}, logistic regression models the log-odds of the event:

$$\log\left(\frac{P(Y_i = 1 | X_i)}{1 - P(Y_i = 1 | X_i)}\right) = \beta_0 + \beta_1 X_{i1} + \cdots + \beta_p X_{ip}$$

Equivalently:
$$P(Y_i = 1 | X_i) = \frac{\exp(\beta_0 + \beta_1 X_{i1} + \cdots + \beta_p X_{ip})}{1 + \exp(\beta_0 + \beta_1 X_{i1} + \cdots + \beta_p X_{ip})}$$

### Assumptions

1. **Binary outcome**: *Y* takes values 0 or 1
2. **Independence**: Observations are independent
3. **Linearity in log-odds**: Log-odds is a linear function of predictors
4. **No multicollinearity**: Predictors are not perfectly correlated
5. **Large sample**: Adequate events per predictor (≥10 rule of thumb)

### Interpretation: Odds Ratios

The exponentiated coefficient exp(*β*ⱼ) yields the **odds ratio (OR)**:

$$\text{OR} = \exp(\beta_j) = \frac{\text{Odds}(Y = 1 | X_j + 1)}{\text{Odds}(Y = 1 | X_j)}$$

| OR Value | Interpretation |
|:---------|:---------------|
| OR = 1 | No association |
| OR > 1 | Increased odds of outcome |
| OR < 1 | Decreased odds of outcome |

For a continuous predictor, the OR represents the multiplicative change in odds per one-unit increase. For a categorical predictor, the OR compares each level to the reference category.

### Implementation

```{r}
fit(data, outcome = "binary_var", predictors, model_type = "glm", family = "binomial")
```

## Poisson Regression

### Model Specification

For count outcomes *Y* ∈ {0, 1, 2, ...}, Poisson regression models the log of the expected count:

$$\log(E[Y_i | X_i]) = \beta_0 + \beta_1 X_{i1} + \cdots + \beta_p X_{ip}$$

### Assumptions

1. **Count outcome**: *Y* is a non-negative integer
2. **Independence**: Observations are independent
3. **Equidispersion**: Mean equals variance (E[*Y*] = Var(*Y*))
4. **Log-linear relationship**: Log of expected count is linear in predictors

### Interpretation: Rate Ratios

The exponentiated coefficient exp(*β*ⱼ) yields the **rate ratio (RR)**:

$$\text{RR} = \exp(\beta_j) = \frac{E[Y | X_j + 1]}{E[Y | X_j]}$$

### Overdispersion

When Var(*Y*) > E[*Y*] (overdispersion), standard Poisson SEs are underestimated. Solutions include:

- **Quasipoisson**: Estimates dispersion parameter *φ*
- **Negative binomial**: Models additional variance component

### Implementation

```{r}
# Standard Poisson
fit(data, outcome = "count_var", predictors, model_type = "glm", family = "poisson")

# Overdispersed counts
fit(data, outcome = "count_var", predictors, model_type = "glm", family = "quasipoisson")
```

## Negative Binomial Regression

### Model Specification

Negative binomial regression extends Poisson regression with an additional dispersion parameter *θ*:

$$Y_i \sim \text{NegBin}(\mu_i, \theta)$$
$$\log(\mu_i) = \beta_0 + \beta_1 X_{i1} + \cdots + \beta_p X_{ip}$$

The variance is:
$$\text{Var}(Y_i) = \mu_i + \frac{\mu_i^2}{\theta}$$

As *θ* → ∞, the negative binomial converges to Poisson.

### When to Use

- Residual deviance >> residual degrees of freedom in Poisson model
- Variance substantially exceeds mean
- Overdispersion test is significant

### Implementation

```{r}
fit(data, outcome = "count_var", predictors, model_type = "negbin")
```

Requires the **MASS** package.

## Gamma Regression

### Model Specification

For positive, continuous, right-skewed outcomes, Gamma regression models:

$$Y_i \sim \text{Gamma}(\mu_i, \phi)$$

The canonical link for the Gamma family is the inverse (1/μ). In practice, the log link is often preferred because it yields more interpretable coefficients (multiplicative effects). When `family = "Gamma"` is passed as a string, `summata` defaults to the log link. The link can also be specified explicitly:
$$\log(E[Y_i | X_i]) = \beta_0 + \beta_1 X_{i1} + \cdots + \beta_p X_{ip}$$

### Assumptions

1. **Positive outcome**: *Y* > 0
2. **Gamma distribution**: Right-skewed, variance proportional to mean squared
3. **Independence**: Observations are independent

### Interpretation

With log link, exp(*β*ⱼ) represents the **multiplicative effect** (ratio):

$$\frac{E[Y | X_j + 1]}{E[Y | X_j]} = \exp(\beta_j)$$

### Common Applications

- Healthcare costs
- Length of hospital stay
- Biomarker concentrations
- Time durations

### Implementation

```{r}
# String shorthand (resolves to log link)
fit(data, outcome = "positive_continuous", predictors, model_type = "glm", family = "Gamma")

# Explicit link specification
fit(data, outcome = "positive_continuous", predictors, model_type = "glm", family = Gamma(link = "log"))

# Gamma with inverse link (canonical)
fit(data, outcome = "positive_continuous", predictors, model_type = "glm", family = Gamma(link = "inverse"))
```

## Quasi-Likelihood Models

### Quasibinomial

For overdispersed binary data, quasibinomial regression estimates a dispersion parameter:

$$\text{Var}(Y_i) = \phi \cdot \mu_i(1 - \mu_i)$$

where *φ* > 1 indicates overdispersion. Coefficients are identical to binomial GLM, but standard errors are adjusted.

### Quasipoisson

For overdispersed count data:

$$\text{Var}(Y_i) = \phi \cdot \mu_i$$

### Implementation

```{r}
# Overdispersed binary
fit(data, outcome, predictors, model_type = "glm", family = "quasibinomial")

# Overdispersed counts
fit(data, outcome, predictors, model_type = "glm", family = "quasipoisson")
```

## Summary of GLM Families

| Family | Outcome Type | Link | Effect Measure |
|:-------|:-------------|:-----|:---------------|
| `binomial` | Binary | logit | Odds ratio |
| `quasibinomial` | Binary (overdispersed) | logit | Odds ratio |
| `poisson` | Count | log | Rate ratio |
| `quasipoisson` | Count (overdispersed) | log | Rate ratio |
| `gaussian` | Continuous | identity | Coefficient |
| `Gamma` | Positive continuous | inverse (canonical); log (in `summata`) | Ratio (with log link) |
| `inverse.gaussian` | Positive continuous | 1/μ² | — |

---

# Survival Analysis

## Cox Proportional Hazards Model

### Model Specification

The Cox model relates the hazard function to predictors:

$$h(t | X_i) = h_0(t) \exp(\beta_1 X_{i1} + \beta_2 X_{i2} + \cdots + \beta_p X_{ip})$$

where:

- *h*(*t* | *X*) is the hazard (instantaneous risk) at time *t*
- *h*₀(*t*) is the baseline hazard (unspecified)
- exp(*β*ⱼ) is the hazard ratio for predictor *j*

### Assumptions

1. **Proportional hazards**: Hazard ratios are constant over time
2. **Independence**: Observations are independent (or appropriately modeled)
3. **Censoring**: Non-informative censoring
4. **No multicollinearity**: Predictors are not perfectly correlated

### Interpretation: Hazard Ratios

The **hazard ratio (HR)** = exp(*β*ⱼ) represents the relative hazard:

$$\text{HR} = \frac{h(t | X_j + 1)}{h(t | X_j)} = \exp(\beta_j)$$

| HR Value | Interpretation |
|:---------|:---------------|
| HR = 1 | No association with survival |
| HR > 1 | Increased hazard (shorter survival) |
| HR < 1 | Decreased hazard (longer survival) |

### Goodness of Fit

**Concordance (Harrell's C-index):**
$$C = P(\hat{h}_i > \hat{h}_j | T_i < T_j)$$

Proportion of concordant pairs; C = 0.5 indicates no discrimination, C = 1 indicates perfect discrimination.

### Testing Proportional Hazards

The proportional hazards assumption can be tested using:

- Scaled Schoenfeld residuals
- Log-log survival plots
- Time-dependent covariates

### Implementation

```{r}
fit(data, outcome = "Surv(time, status)", predictors, model_type = "coxph")
```

Requires the **survival** package.

## Conditional Logistic Regression

### Model Specification

For matched case-control studies, conditional logistic regression conditions on the stratum:

$$\log\left(\frac{P(Y_i = 1 | X_i, \text{stratum})}{P(Y_i = 0 | X_i, \text{stratum})}\right) = \beta_1 X_{i1} + \cdots + \beta_p X_{ip}$$

Note: No intercept is estimated; it is absorbed by the stratum.

### When to Use

- 1:1 or 1:*m* matched case-control designs
- Stratified analyses where stratum-specific effects are nuisance parameters

### Implementation

```{r}
fit(data, outcome = "case_status", predictors, model_type = "clogit", strata = "match_id")
```

Requires the **survival** package.

---

# Mixed-Effects Models

Mixed-effects models (hierarchical or multilevel models) account for correlation within clusters by incorporating random effects.

## General Framework

For observation *i* within cluster *j*:

$$g(E[Y_{ij} | X_{ij}, u_j]) = \mathbf{X}_{ij}\mathbf{\beta} + \mathbf{Z}_{ij}\mathbf{u}_j$$

where:

- **β** are fixed effects (common across all clusters)
- **u**ⱼ ~ N(**0**, **Σ**) are random effects (cluster-specific deviations)
- **Z**ᵢⱼ is the random effects design matrix

## Linear Mixed-Effects Models

### Model Specification

For continuous outcomes:

$$Y_{ij} = \beta_0 + \beta_1 X_{1ij} + \cdots + \beta_p X_{pij} + u_{0j} + u_{1j}X_{1ij} + \varepsilon_{ij}$$

where:

- *u*₀ⱼ ~ N(0, σ²ᵤ₀) is the random intercept for cluster *j*
- *u*₁ⱼ ~ N(0, σ²ᵤ₁) is the random slope (if included)
- εᵢⱼ ~ N(0, σ²) is the residual error

### Random Effects Syntax

| Syntax | Meaning |
|:-------|:--------|
| `(1\|group)` | Random intercepts by group |
| `(x\|group)` | Random intercepts and slopes for *x* |
| `(1\|group1) + (1\|group2)` | Crossed random effects |
| `(1\|group1/group2)` | Nested random effects |

### Implementation

```{r}
fit(data, outcome, c(predictors, "(1|cluster)"), model_type = "lmer")
```

Requires the **lme4** package.

## Generalized Linear Mixed-Effects Models

### Model Specification

GLMMs extend mixed-effects models to non-normal outcomes:

$$g(E[Y_{ij} | X_{ij}, u_j]) = \beta_0 + \beta_1 X_{1ij} + \cdots + \beta_p X_{pij} + u_{0j}$$

Common families:

- **Binomial**: Clustered binary outcomes
- **Poisson**: Clustered count outcomes

### Interpretation

Fixed effects are interpreted as **conditional** (cluster-specific) effects. The population-averaged effect may differ due to the non-linear link function.

### Implementation

```{r}
# Binary outcome with clustering
fit(data, outcome, c(predictors, "(1|cluster)"), model_type = "glmer", family = "binomial")

# Count outcome with clustering
fit(data, outcome, c(predictors, "(1|cluster)"), model_type = "glmer", family = "poisson")
```

Requires the **lme4** package.

## Cox Mixed-Effects Models

### Model Specification

For clustered survival data:

$$h_{ij}(t) = h_0(t) \exp(\beta_1 X_{1ij} + \cdots + \beta_p X_{pij} + u_j)$$

where *u*ⱼ ~ N(0, σ²ᵤ) is the cluster-specific frailty term.

### When to Use

- Patients nested within hospitals or physicians
- Repeated events within subjects
- Multi-center clinical trials

### Implementation

```{r}
fit(data, outcome = "Surv(time, status)", c(predictors, "(1|cluster)"), model_type = "coxme")
```

Requires the **coxme** package.

## Sample Size Considerations

Mixed-effects models require adequate sample sizes at both levels:

| Level | Recommendation |
|:------|:---------------|
| Clusters | ≥ 20–30 for stable variance estimates |
| Observations per cluster | ≥ 5 for random intercepts |
| Total observations | Standard regression guidelines apply |

---

# Interaction Effects

## Model Specification

An interaction occurs when the effect of one predictor depends on the level of another. For predictors *X* and *Z*:

$$g(E[Y]) = \beta_0 + \beta_1 X + \beta_2 Z + \beta_3 (X \times Z)$$

where:

- *β*₁ is the effect of *X* when *Z* = 0
- *β*₂ is the effect of *Z* when *X* = 0
- *β*₃ is the interaction effect (modification of *X* effect by *Z*)

## Interpretation

### Continuous × Continuous

For continuous *X* and *Z*, the effect of *X* on *Y* is:

$$\frac{\partial E[Y]}{\partial X} = \beta_1 + \beta_3 Z$$

The effect of *X* varies linearly with *Z*.

### Continuous × Categorical

For continuous *X* and binary *Z* (0/1):

- When *Z* = 0: Effect of *X* is *β*₁
- When *Z* = 1: Effect of *X* is *β*₁ + *β*₃

### Categorical × Categorical

For two categorical variables, each combination of levels has a distinct effect. The interaction coefficient represents the additional effect beyond the sum of main effects.

## Testing Interactions

1. **Wald test**: Test H₀: *β*₃ = 0
2. **Likelihood ratio test**: Compare models with and without interaction
3. **Model comparison**: Use `compfit()` to compare AIC/BIC

## Implementation

```{r}
fit(data, outcome, predictors, interactions = c("X:Z"), model_type = "glm")
```

---

# Model Quality Metrics

## Information Criteria

### Akaike Information Criterion (AIC)

$$\text{AIC} = -2\log L + 2k$$

where *L* is the maximized likelihood and *k* is the number of parameters. AIC balances fit and complexity; lower values indicate better models.

### Bayesian Information Criterion (BIC)

$$\text{BIC} = -2\log L + k\log n$$

BIC penalizes complexity more heavily than AIC, especially for large samples.

### Interpretation

| Comparison | Guideline |
|:-----------|:----------|
| ΔAIC < 2 | Models essentially equivalent |
| ΔAIC 2–7 | Some support for lower-AIC model |
| ΔAIC > 10 | Strong support for lower-AIC model |

## Discrimination Metrics

### Concordance (C-statistic)

For binary outcomes or survival models:

$$C = P(\hat{p}_i > \hat{p}_j | Y_i = 1, Y_j = 0)$$

| C Value | Discrimination |
|:--------|:---------------|
| 0.5 | No discrimination (random) |
| 0.6–0.7 | Poor |
| 0.7–0.8 | Acceptable |
| 0.8–0.9 | Excellent |
| > 0.9 | Outstanding |

### Brier Score

For binary outcomes:

$$\text{Brier} = \frac{1}{n}\sum_i (Y_i - \hat{p}_i)^2$$

Lower values indicate better calibration; Brier = 0 is perfect.

## Explained Variation

### R² (Linear Models)

$$R^2 = 1 - \frac{SS_{\text{res}}}{SS_{\text{tot}}}$$

Proportion of variance explained.

### Pseudo-R² (GLMs)

**McFadden pseudo-R²:**
$$R^2_{\text{McF}} = 1 - \frac{\log L_{\text{full}}}{\log L_{\text{null}}}$$

**Nagelkerke pseudo-R²:**
$$R^2_N = \frac{1 - (L_{\text{null}}/L_{\text{full}})^{2/n}}{1 - L_{\text{null}}^{2/n}}$$

Values are not directly comparable to linear *R*²; 0.2–0.4 typically indicates good fit for logistic models.

---

# Composite Model Score (CMS)

## Rationale

When comparing multiple candidate models, researchers typically examine several quality metrics: information criteria (AIC, BIC), discrimination (concordance), calibration (Brier score), and explained variation (*R*² or pseudo-*R*²). Each metric captures a different aspect of model performance, and no single measure is universally optimal for model selection.

The **Composite Model Score (CMS)** addresses this challenge by combining multiple metrics into a single summary value, facilitating rapid comparison across candidate models. This approach:

1. Synthesizes complementary information from diverse metrics
2. Provides model-type-specific weighting based on statistical conventions
3. Normalizes metrics to a common 0–100 scale for interpretability
4. Accounts for convergence issues that may invalidate model results

The CMS is intended as a screening tool to identify promising models, not as a replacement for substantive evaluation. Final model selection should always consider domain knowledge, parsimony, and the specific research objectives.

## Methodology

### Normalization

Each component metric is transformed to a 0–100 scale where higher values indicate better performance:

**Information criteria** (AIC, BIC) — Lower is better:
$$S_{\text{AIC}} = 100 \times \left(1 - \frac{\text{AIC}_i - \text{AIC}_{\min}}{\text{AIC}_{\max} - \text{AIC}_{\min}}\right)$$

**Concordance** (*C*-statistic) — Higher is better, with 0.5 as floor:
$$S_C = \begin{cases} 
0 & C \leq 0.5 \\
200 \times (C - 0.5) & C > 0.5
\end{cases}$$

**Pseudo-*R*²** (McFadden) — Scaled recognizing typical range:
$$S_{R^2} = \min(100, \; 250 \times R^2_{\text{McF}})$$

This scaling reflects that McFadden's *R*² rarely exceeds 0.4 even for well-fitting models.

**Brier score** — Lower is better, with 0.25 as no-skill baseline:
$$S_{\text{Brier}} = 100 \times \left(1 - \frac{\text{Brier}}{0.25}\right)$$

**Convergence** — Categorical assessment:
$$S_{\text{conv}} = \begin{cases}
100 & \text{converged normally} \\
70 & \text{suspect (warnings)} \\
30 & \text{failed to converge}
\end{cases}$$

### Weighting

Component scores are combined using model-type-specific weights that reflect the relative importance of each metric for that model class:

| Model Type | Convergence | AIC | Concordance | Pseudo-*R*² | Brier | Global *p* |
|:-----------|:-----------:|:---:|:-----------:|:-----------:|:-----:|:----------:|
| GLM (logistic) | 0.15 | 0.25 | 0.40 | 0.15 | 0.05 | — |
| Cox PH | 0.15 | 0.30 | 0.40 | — | — | 0.15 |
| Linear | 0.15 | 0.25 | — | 0.45 | — | 0.15 |

For mixed-effects models, additional components include marginal *R*², conditional *R*², and ICC.

### Final Score

The CMS is computed as a weighted sum:

$$\text{CMS} = \sum_j w_j \times S_j$$

where *w*ⱼ are the weights and *S*ⱼ are the normalized component scores.

## Interpretation

| CMS Range | Interpretation |
|:----------|:---------------|
| 85–100 | Excellent model quality |
| 75–84 | Very good |
| 65–74 | Good |
| 55–64 | Fair |
| < 55 | Poor |

These thresholds provide general guidance. A model with CMS = 80 is not necessarily "better" than one with CMS = 78—small differences should prompt examination of individual metrics and substantive considerations.

## Limitations

1. **Relative ranking**: CMS values are relative to the set of models being compared; they do not indicate absolute model quality
2. **Weight sensitivity**: Results depend on the chosen weights; users with strong priors about metric importance can supply custom weights
3. **Metric availability**: Not all metrics are available for all model types; weights are renormalized when components are missing
4. **No substitute for judgment**: The CMS facilitates screening but does not replace careful model diagnostics and domain expertise

## Implementation

```{r}
# Compare candidate models
comparison <- compfit(
  data = mydata,
  outcome = "y",
  predictors = list(
    "Model 1" = c("x1", "x2"),
    "Model 2" = c("x1", "x2", "x3"),
    "Model 3" = c("x1", "x2", "x3", "x4")
  ),
  model_type = "logistic"
)

# Custom weights emphasizing discrimination
comparison_custom <- compfit(
  data = mydata,
  outcome = "y",
  predictors = model_list,
  model_type = "logistic",
  scoring_weights = list(
    convergence = 0.10,
    aic = 0.20,
    concordance = 0.50,
    pseudo_r2 = 0.15,
    brier = 0.05
  )
)
```

---

# References

## Methodological

- Agresti, A. (2013). *Categorical Data Analysis* (3rd ed.). Wiley.
- Harrell, F. E. (2015). *Regression Modeling Strategies* (2nd ed.). Springer.
- Hosmer, D. W., Lemeshow, S., & Sturdivant, R. X. (2013). *Applied Logistic Regression* (3rd ed.). Wiley.
- Therneau, T. M., & Grambsch, P. M. (2000). *Modeling Survival Data*. Springer.
- Gelman, A., & Hill, J. (2007). *Data Analysis Using Regression and Multilevel/Hierarchical Models*. Cambridge University Press.

## Software

- R Core Team (2025). *R: A Language and Environment for Statistical Computing*. https://www.R-project.org/
- Therneau, T. (2025). *survival: Survival Analysis*. R package. https://CRAN.R-project.org/package=survival
- Bates, D., Mächler, M., Bolker, B., & Walker, S. (2015). Fitting linear mixed-effects models using lme4. *Journal of Statistical Software*, 67(1), 1–48.
- Therneau, T. (2025). *coxme: Cox Mixed-Effects Models*. R package. https://CRAN.R-project.org/package=coxme
- Venables, W. N., & Ripley, B. D. (2002). *Modern Applied Statistics with S* (4th ed.). Springer. (MASS package)

---

# See Also

- [Regression Modeling](regression_modeling.html): Practical examples using `fit()`, `uniscreen()`, and `fullfit()`
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models
- [Model Comparison](model_comparison.html): Comparing alternative specifications with `compfit()`
- [Forest Plots](forest_plots.html): Visualization of regression results

---
title: "Survival Tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Survival Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Survival analysis requires specialized summary tables that report time-to-event outcomes in formats appropriate for longitudinal research. While `desctable()` includes basic survival summaries (median with 95% CI), detailed survival analysis often requires more comprehensive reporting: survival probabilities at specified time points, multiple quantiles, and group comparisons with appropriate statistical tests.
 
The `survtable()` function generates publication-ready survival tables with flexible output options. It uses the familiar `Surv()` syntax for specifying survival outcomes and adheres to the standard `summata` calling convention:

```{r, eval = FALSE}
survtable(data, outcome, by, times, probs, ...)
```

where `data` is the dataset, `outcome` specifies the survival endpoint using `Surv()` notation, `by` defines the grouping variable, `times` specifies landmark time points, and `probs` specifies survival quantiles to report.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset contains `r nrow(clintrial)` observations with time-to-event variables suitable for demonstrating survival summaries.

---

# Landmark Survival Estimates

Landmark survival estimates report the probability of survival at specific time points (*e.g.*, 1-year, 2-year survival rates).

## **Example 1:** Basic Landmark Analysis

The following example reports survival probabilities at 12, 24, and 36 months (with the default median survival reporting included):

```{r}
example1 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24, 36),
  time_unit = "months"
)

example1
```

## **Example 2:** Remove Median Survival Column

By default, a column showing median survival is displayed (`probs = 0.5`). To remove it, set `probs = NULL`.

```{r}
example2 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24),
  probs = NULL,
  time_unit = "months",
  total = FALSE
)

example2
```

---

# Survival Quantiles

Survival quantiles report the time at which a specified proportion of subjects have experienced the event. The median survival time (50th percentile) is the most common, but other quantiles provide additional context.

## **Example 3:** Survival Quartiles

Report multiple survival quantiles by specifying the `probs` parameter. Note that `labels` names levels of the `by` variable here, rather than variable names as in `desctable()` and the regression functions:

```{r}
example3 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "stage",
  times = NULL,
  probs = c(0.25, 0.5, 0.75),
  labels = c(
    "I" = "Stage I",
    "II" = "Stage II",
    "III" = "Stage III",
    "IV" = "Stage IV"
  )
)

example3
```

---

# Multiple Endpoints

Studies often include multiple time-to-event outcomes, such as progression-free survival (PFS) and overall survival (OS). The `survtable()` function handles multiple endpoints in a single call.

## **Example 4:** PFS and OS Comparison

Pass a vector of outcomes to compare multiple survival endpoints:

```{r}
example4 <- survtable(
  data = clintrial,
  outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
  by = "treatment",
  times = c(12, 24),
  probs = 0.5,
  time_unit = "months",
  total = FALSE,
  labels = c(
    "Surv(pfs_months, pfs_status)" = "Progression-Free Survival",
    "Surv(os_months, os_status)" = "Overall Survival"
  )
)

example4
```

---

# Output Variations

## **Example 5:** Cumulative Event Rates

For competing risks analyses or when reporting event rates rather than survival probabilities, use `type = "risk"` to display cumulative incidence (1 − survival):

```{r}
example5 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24, 36),
  type = "risk",
  time_unit = "months"
)

example5
```

## **Example 6:** Including At-Risk Counts

The number at risk at each time point provides context for the precision of survival estimates:

```{r}
example6 <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24),
  stats = c("survival", "ci", "n_risk"),
  time_unit = "months",
  total = FALSE
)

example6
```

---

# Pairing with Kaplan-Meier Curves

Survival tables are often presented alongside Kaplan-Meier curves in publications. While `summata` focuses on tabular output, the `survminer` and `ggsurvfit` packages provide excellent options for survival curves.

## **Example 7:** Coordinated Figure-Table Output

The following workflow produces a matched figure-table pair suitable for publication:

```{r, eval = FALSE}
library(ggsurvfit)
library(survival)

# Fit the survival model
km_fit <- survfit(Surv(os_months, os_status) ~ treatment, data = clintrial)

# Create Kaplan-Meier plot with risk table
ggsurvfit(km_fit) +
  add_confidence_interval() +
  add_risktable() +
  add_quantile(y_value = 0.5, linetype = "dashed") +
  scale_ggsurvfit() +
  labs(
    title = "Overall Survival by Treatment",
    x = "Time (months)",
    y = "Survival Probability"
  ) +
  theme_minimal()

# Generate the companion table
surv_table <- survtable(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  by = "treatment",
  times = c(12, 24, 36),
  probs = 0.5,
  time_unit = "months"
)

# Export both for publication
ggsave("km_curve.pdf", width = 8, height = 6)
table2pdf(surv_table, "survival_table.pdf", 
          caption = "Table 2. Survival Estimates by Treatment Group")
```

This workflow ensures that the Kaplan-Meier curve and survival table report consistent time points and groupings.

---

# Exporting Tables

Survival tables can be exported to various formats using the standard `summata` export functions. See the [Table Export](table_export.html) vignette for comprehensive documentation.

```{r, eval = FALSE}
# Microsoft Word
table2docx(
  table = example1,
  file = "survival_table.docx",
  caption = "Table 2. Survival Estimates by Treatment Group"
)

# PDF (requires LaTeX)
table2pdf(
  table = example1,
  file = "survival_table.pdf",
  caption = "Table 2. Survival Estimates by Treatment Group"
)
```

---

# Best Practices

## Time Point Selection

When selecting landmark time points, consider the following:

1. Use clinically meaningful intervals (e.g., 1-year, 2-year for oncology)
2. Ensure adequate follow-up at selected time points
3. Match time points across treatment arms for comparability
4. Consider the median follow-up when selecting the latest time point

## Reporting Recommendations

1. Always report confidence intervals alongside point estimates
2. Include the number at risk when space permits
3. Specify the time unit in table headers or footnotes
4. Use consistent decimal precision across estimates

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `uniscreen()`, `fit()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Descriptive Tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Descriptive Tables}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  dpi = 150
)

# Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  knitr::opts_chunk$set(dev = "ragg_png")
}

old_opts <- options(width = 180)
```

Descriptive statistics provide the foundation for any quantitative analysis. Before estimating relationships between variables, it is essential to first characterize the distribution of each variable and assess balance across comparison groups. A well-constructed descriptive table—often designated "Table 1" in published research—accomplishes three objectives: it summarizes the central tendency and dispersion of continuous variables, tabulates the frequency distribution of categorical variables, and tests for systematic differences between groups.

The `desctable()` function generates publication-ready descriptive tables with automatic detection of variable types, appropriate summary statistics, and optional hypothesis testing.  It adheres to the to the standard `summata` calling convention:

```{r, eval = FALSE}
desctable(data, by, variables, ...)
```

where `data` is the dataset, `by` specifies the grouping variable (optional), and `variables` lists the variables to summarize. This vignette demonstrates the function’s capabilities using the included sample dataset.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)

data(clintrial)
data(clintrial_labels)
```

The `clintrial` dataset contains `r nrow(clintrial)` observations with continuous, categorical, and time-to-event variables suitable for demonstrating descriptive statistics. The `clintrial_labels` vector provides human-readable labels for display.

---

# Summary Statistics and Tests

The `desctable()` function automatically selects appropriate summary statistics and hypothesis tests based on variable type:

| Variable Type | Summary Statistic | Two Groups | Three+ Groups |
|:--------------|:------------------|:-----------|:--------------|
| Continuous (parametric) | Mean ± SD | *t*-test | ANOVA |
| Continuous (nonparametric) | Median [IQR] | Wilcoxon | Kruskal–Wallis |
| Categorical | *n* (%) | χ² or Fisher | χ² or Fisher |
| Time-to-event | Median (95% CI) | Log-rank | Log-rank |

By default, Fisher exact test is used when any expected cell count falls below 5 for categorical variables. Similarly, the test selection for continuous variables will follow the displayed statistic unless otherwise specified: parametric tests are used with mean-based statistics, nonparametric tests with median-based statistics.

---

# Basic Usage

## **Example 1:** Grouped Descriptive Table

The most common use-case for descriptive tables is comparing characteristics across groups:

```{r}
desc_vars <- c("age", "sex", "race", "bmi", "stage", "ecog", 
               "Surv(os_months, os_status)")

example1 <- desctable(
  data = clintrial, 
  by = "treatment", 
  variables = desc_vars,
  labels = clintrial_labels
)

example1
```

The output includes a "Total" column by default, showing overall statistics alongside group-specific values.

## **Example 2:** Ungrouped Summary Statistics

Omitting the `by` argument produces overall summary statistics without group comparisons:

```{r}
example2 <- desctable(
  data = clintrial,
  variables = c("age", "bmi", "sex", "stage"),
  labels = clintrial_labels
)

example2
```

---

# Customizing Summary Statistics

The default summary statistics can be customized for both continuous and categorical variables.

## **Example 3:** Continuous Variables

The `stats_continuous` parameter controls how continuous variables are summarized:

| Value | Output Format |
|:------|:--------------|
| `"mean_sd"` | Mean ± SD |
| `"median_iqr"` | Median [Q1–Q3] (default) |
| `"median_range"` | Median (min–max) |

```{r}
example3 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "los_days"),
  stats_continuous = c("mean_sd", "median_iqr", "median_range"),
  labels = clintrial_labels
)

example3
```

## **Example 4:** Categorical Variables

The `stats_categorical` parameter controls categorical variable display:

| Value | Output Format |
|:------|:--------------|
| `"n_percent"` | *n* (%) (default) |
| `"n"` | *n* only |
| `"percent"` | % only |

```{r}
example4 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("sex", "stage", "ecog"),
  stats_categorical = "percent",
  labels = clintrial_labels
)

example4
```

## **Example 5:** Numeric Precision

Control decimal places with `digits` (for statistics) and `p_digits` (for *p*-values):

```{r}
example5 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "sex"),
  digits = 2,
  p_digits = 4,
  test = TRUE,
  labels = clintrial_labels
)

example5
```

---

# Statistical Testing

When comparing groups, hypothesis tests assess whether observed differences are statistically significant.

## **Example 6:** Disabling Automatic Test Selection

By default, automatic hypothesis testing based on the summary statistic is displayed (`test = TRUE`). Setting `test = FALSE` disables this functionality:

```{r}
example6 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "sex", "stage"),
  test = FALSE,
  labels = clintrial_labels
)

example6
```

## **Example 7:** Specifying Tests Manually

Override automatic selection with `test_continuous` and `test_categorical`. Available test specifications include:

**Continuous** (`test_continuous`):

- `"auto"`: Automatic selection (default)
- `"t"`: Student *t*-test
- `"wrs"`: Wilcoxon rank-sum test
- `"aov"`: One-way ANOVA
- `"kwt"`: Kruskal–Wallis test

**Categorical** (`test_categorical`):

- `"auto"`: Automatic selection (default)
- `"chisq"`: Pearson χ² test
- `"fisher"`: Fisher exact test

The following example forces parametric tests for continuous variables:

```{r}
example7a <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "bmi", "los_days"),
  test = TRUE,
  test_continuous = "aov",  # ANOVA
  labels = clintrial_labels
)

example7a
```

This example forces the Fisher exact test for categorical variables:

```{r}
example7b <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("sex", "stage"),
  test = TRUE,
  test_categorical = "fisher",
  labels = clintrial_labels
)

example7b
```

---

# Handling Missing Data

Missing values require special consideration in descriptive tables. Options control whether missing values are displayed and how percentages are calculated.

## **Example 8:** Including Missing Values

By default, missing values are excluded from calculations. Set `na_include = TRUE` to display them as a separate category:

```{r}
example8 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking", "diabetes"),
  na_include = TRUE,
  labels = clintrial_labels
)

example8
```

## **Example 9:** Missing Value Denominators

The `na_percent` parameter controls whether missing values are included in percentage calculations:

```{r}
# Percentages exclude missing (denominator = non-missing)
example9a <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking"),
  na_include = TRUE,
  na_percent = FALSE,
  labels = clintrial_labels
)

example9a

# Percentages include missing (denominator = total)
example9b <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking"),
  na_include = TRUE,
  na_percent = TRUE,
  labels = clintrial_labels
)

example9b
```

## **Example 10:** Custom Missing Value Label

The label for missing values can be customized using the `na_label` parameter:

```{r}
example10 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("smoking"),
  na_include = TRUE,
  na_label = "Not Reported",
  labels = clintrial_labels
)

example10
```

---

# Total Column Options

The total column provides overall statistics alongside group-specific values.

## **Example 11:** Total Column Configuration

The `total` parameter controls the presence and position of the total column:

| Value | Effect |
|:------|:-------|
| `TRUE`, `"first"` | Total column first (default) |
| `"last"` | Total column last |
| `FALSE` | No total column |

```{r}
# Total column in last position
example11a <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "sex", "stage"),
  total = "last",
  labels = clintrial_labels
)

example11a

# No total column
example11b <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c("age", "sex", "stage"),
  total = FALSE,
  labels = clintrial_labels
)

example11b
```

---

# Complete Example

The following demonstrates a comprehensive descriptive table suitable for publication:

```{r}
table1 <- desctable(
  data = clintrial,
  by = "treatment",
  variables = c(
    "age", "sex", "race", "ethnicity", "bmi",
    "smoking", "diabetes", "hypertension",
    "stage", "grade", "ecog",
    "Surv(os_months, os_status)"
  ),
  labels = clintrial_labels,
  stats_continuous = "mean_sd",
  stats_categorical = "n_percent",
  test = TRUE,
  total = TRUE,
  digits = 1,
  p_digits = 3
)

table1
```

## Accessing Raw Data

The underlying numeric values are stored as an attribute for programmatic access:

```{r}
raw_data <- attr(table1, "raw_data")
head(raw_data)
```

---

# Survival Summary Tables

For detailed survival analysis—including landmark survival estimates, survival quantiles, and multiple endpoints—see the dedicated [Survival Tables](survival_tables.html) vignette. The `survtable()` function provides comprehensive options for reporting time-to-event outcomes.

---

# Exporting Tables

Descriptive tables can be exported to various formats. See the [Table Export](table_export.html) vignette for comprehensive documentation.

```{r, eval = FALSE}
# Microsoft Word
table2docx(
  table = table1,
  file = "Table1.docx",
  caption = "Table 1. Baseline Characteristics by Group"
)

# PDF (requires LaTeX)
table2pdf(
  table = table1,
  file = "Table1.pdf",
  caption = "Table 1. Baseline Characteristics by Group"
)

# HTML
table2html(
  table = table1,
  file = "Table1.html",
  caption = "Table 1. Baseline Characteristics by Group"
)
```

---

# Best Practices

## Variable Selection

1. Include all relevant baseline characteristics
2. Order variables logically (typically chronologically and by domain)
3. Exclude the grouping variable from the variables list

## Statistical Considerations

1. Use automatic test selection unless there is specific justification otherwise
2. Report exact *p*-values when possible; very small values display as "< 0.001" (or to preferred degree of precision)
3. Consider multiple comparison adjustments when testing many variables
4. For skewed or non-normally distributed continuous variables, report using nonparametric statistical procedures (e.g., median with IQR) rather than parametric ones (e.g., mean with SD)

## Formatting Recommendations

1. Use consistent decimal precision within variable types
2. Include units in variable labels (e.g., "Age (years)")
3. Include a total column for context
4. Use landscape orientation for tables with many columns

---

# Common Issues

## Empty Cells in Categorical Variables

When a factor level has zero observations in a group, ensure all levels are explicitly defined:

```{r, eval = FALSE}
data$stage <- factor(data$stage, levels = c("I", "II", "III", "IV"))
```

## Skewed Continuous Variables

For highly skewed distributions, use median and IQR:

```{r, eval = FALSE}
desctable(data, by, variables, stats_continuous = "median_iqr")
```

## Large Tables

For tables with many variables, consider splitting by category or using landscape orientation for export:

```{r, eval = FALSE}
table2pdf(table, "table1.pdf", orientation = "landscape", font_size = 8)
```

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Survival Tables](survival_tables.html): `survtable()` for time-to-event summaries
- [Regression Modeling](regression_modeling.html): `uniscreen()`, `fit()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis
- [Advanced Workflows](advanced_workflows.html): Interactions and mixed-effects models

---
title: "Feature Comparison"
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = TRUE
)
```

This article compares `summata` with other R packages for regression table generation and summary statistics.

---

# Similar Packages

The R ecosystem includes several well-established packages for creating publication-ready regression tables. The following comparison identifies areas of overlap and distinction between `summata` and its alternatives.

| Package | Primary Focus |
|:--------|:--------------|
| **summata** | Fast regression workflows, forest plots, multivariate regression |
| **gtsummary** | Comprehensive table generation, `gt` ecosystem, maximum flexibility |
| **finalfit** | Clinical research, missing data handling, bootstrap simulations |
| **arsenal** | Large-scale summaries, SAS-like output |
| **stargazer** | Econometrics, LaTeX output, academic journal formatting |
| **tableone** | Simple Table 1 generation, SMD calculations |
| **compareGroups** | Bivariate analysis, clinical epidemiology |

---

## Core Feature Matrix

```{r feature-table, echo=FALSE, results='asis'}
# Define the feature matrix
features <- data.frame(
  Feature = c(
    "Descriptive Tables",
    "Stratified Summaries", 
    "Univariable Screening",
    "Multivariable Workflow",
    "Multivariate Regression",
    "Model Comparison",
    "Mixed-Effects Models",
    "Cox/Survival Models",
    "Interaction Formatting",
    "Forest Plots",
    "Table Merge/Stack",
    "Export (Word/PDF)",
    "Variable Labels"
  ),
  summata = c("Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "No", "Yes", "Yes"),
  gtsummary = c("Yes", "Yes", "Yes", "Yes", "No", "Yes", "Yes", "Yes", "Partial", "Partial", "Yes", "Yes", "Yes"),
  finalfit = c("Yes", "Yes", "Yes", "Yes", "No", "Partial", "Partial", "Yes", "Partial", "Yes", "Partial", "Yes", "Yes"),
  arsenal = c("Yes", "Yes", "Yes", "Yes", "No", "No", "No", "Yes", "Partial", "No", "No", "Yes", "Yes"),
  stargazer = c("Yes", "No", "No", "Yes", "No", "Yes", "Partial", "Yes", "No", "No", "No", "Yes", "Partial"),
  tableone = c("Yes", "Yes", "No", "No", "No", "No", "No", "No", "No", "No", "No", "Partial", "Yes"),
  compareGroups = c("Yes", "Yes", "Yes", "No", "No", "No", "No", "Yes", "No", "No", "No", "Yes", "Yes"),
  stringsAsFactors = FALSE
)

# Create HTML table with color coding
cat('<table class="table table-sm" style="font-size: 0.9em;">\n')
cat('<thead>\n<tr>\n')
cat('<th style="text-align: left;">Feature</th>\n')
cat('<th style="text-align: center;">summata</th>\n')
cat('<th style="text-align: center;">gtsummary</th>\n')
cat('<th style="text-align: center;">finalfit</th>\n')
cat('<th style="text-align: center;">arsenal</th>\n')
cat('<th style="text-align: center;">stargazer</th>\n')
cat('<th style="text-align: center;">tableone</th>\n')
cat('<th style="text-align: center;">compareGroups</th>\n')
cat('</tr>\n</thead>\n<tbody>\n')

for (i in 1:nrow(features)) {
  cat('<tr>\n')
  cat(sprintf('<td style="text-align: left;">%s</td>\n', features$Feature[i]))
  
  for (pkg in c("summata", "gtsummary", "finalfit", "arsenal", "stargazer", "tableone", "compareGroups")) {
    val <- features[[pkg]][i]
    if (val == "Yes") {
      style <- 'background-color: #d4edda; color: #155724; text-align: center;'
      symbol <- "✓"
    } else if (val == "Partial") {
      style <- 'background-color: #fff3cd; color: #856404; text-align: center;'
      symbol <- "◐"
    } else {
      style <- 'background-color: #f8d7da; color: #721c24; text-align: center;'
      symbol <- "—"
    }
    cat(sprintf('<td style="%s">%s</td>\n', style, symbol))
  }
  cat('</tr>\n')
}

cat('</tbody>\n</table>\n')
cat('<p style="font-size: 0.85em; color: #666;"><strong>Legend:</strong> ✓ Full support | ◐ Partial support | — Not available</p>\n')
```

---

## Feature Definitions

| Feature | Description |
|:--------|:------------|
| **Descriptive Tables** | Summary statistics tables (mean, SD, median, IQR, *n*, %) |
| **Stratified Summaries** | Table 1-style summaries stratified by group with *p*-values |
| **Univariable Screening** | Test multiple predictors against one outcome (crude associations) |
| **Multivariable Workflow** | Combined univariable + multivariable analysis in one table |
| **Multivariate Regression** | Test one predictor across multiple outcomes |
| **Model Comparison** | Compare multiple models side-by-side with fit statistics |
| **Mixed-Effects Models** | Support for `lmer`/`glmer`/`coxme` random effects models |
| **Cox/Survival Models** | Cox proportional hazards and survival analysis |
| **Interaction Formatting** | Native support for interaction terms with formatted output |
| **Forest Plots** | Integrated forest plot visualization from regression results |
| **Table Merge/Stack** | Combine separate tables horizontally or vertically |
| **Export (Word/PDF)** | Direct export to publication formats |
| **Variable Labels** | Apply custom labels to variables in output |

---

# Unique Strengths of *summata*

The following features distinguish `summata` from comparable packages:

## Multivariate Regression

The `multifit()` and `multiforest()` functions implement an inverted screening paradigm: testing a single predictor across multiple outcomes simultaneously. This workflow is common in epidemiological and clinical research but not directly supported by other packages.
 
```{r multifit-example, eval=FALSE}
# Test treatment effect across multiple outcomes
result <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status", "os_status"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage")
)
```

## Integrated Forest Plots

Customizable forest plots are generated directly from analysis results without intermediate steps:

```{r forest-example, eval=FALSE}
# From univariable screening
screen_result <- uniscreen(data, outcome, predictors)
uniforest(screen_result)

# From multivariate regression
multi_result <- multifit(data, outcomes, predictor)
multiforest(multi_result)
```

## Performance Optimization

Built on `data.table` for computational efficiency, `summata` provides competitive performance across all benchmarked workflows. With `conf_method = "wald"`, it is the fastest option tested for GLM-based regression tables and univariable screening. With the default profile likelihood CIs, performance is comparable to other packages that use the same CI method (`finalfit`, `broom`). See the [Benchmarks](benchmarks.html) article for detailed comparisons.

## Unified API

All modeling functions share consistent syntax across model types:

```{r api-example, eval=FALSE}
# Same interface for different model types
fit(data, outcome, predictors, model_type = "glm")
fit(data, outcome, predictors, model_type = "coxph")
fit(data, outcome, predictors, model_type = "lmer", random = "(1|site)")
```

## Complete Mixed-Effects Support

Full support for `coxme` (mixed-effects Cox models) alongside `lmer` and `glmer`, which is limited or absent in other packages.

---

# Additional Resources

- [Benchmarks](benchmarks.html) — Performance comparisons
- [gtsummary documentation](https://www.danieldsjoberg.com/gtsummary/)
- [finalfit documentation](https://finalfit.org/)
- [arsenal documentation](https://mayoverse.github.io/arsenal/)

---
title: "Performance Benchmarks"
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = FALSE,
  warning = FALSE,
  eval = TRUE
)
```

Execution speed is rarely the primary consideration when selecting statistical software—correctness, interpretability, and ease-of-use usually take precedence. However, computational efficiency becomes relevant when working with large datasets, conducting simulation studies, or iterating through model specifications during exploratory analysis.

This article documents the computational performance of `summata` relative to established alternatives. The benchmarks presented here are intended as a reference for users whose workflows involve performance-sensitive operations, and as a record of the design tradeoffs inherent in different implementation approaches.

---

# Methodology

All benchmarks were conducted using the `microbenchmark` package under the following conditions:

- **Iterations**: 5–20 per benchmark, adjusted for computational intensity
- **Dataset sizes**: 500 to 10,000 observations
- **Data structure**: Simulated clinical trial data with continuous, categorical, and time-to-event variables
- **Predictors**: 14 variables for screening benchmarks

Datasets were generated using a fixed random seed to ensure reproducibility. Timing measurements exclude package loading and data generation. All packages were tested using default parameters unless otherwise noted.

Two `summata` configurations are benchmarked throughout: the **default** configuration, which uses profile likelihood confidence intervals for GLM models and includes full formatting (QC statistics, sample sizes, reference rows); and a **minimal** configuration (`summata_minimal`), which uses Wald CIs and disables optional output features. This distinction is important because profile likelihood CIs dominate GLM execution time, and the minimal configuration provides a way to measure `summata`'s formatting overhead in isolation. Note that `finalfit` and `broom` also use profile likelihood CIs by default for GLM models.

```{r setup, eval = FALSE}
library(summata)
library(microbenchmark)
library(ggplot2)
```

---

# Descriptive Tables

Descriptive summary tables represent a common first step in data analysis. The following packages provide comparable functionality with differing implementation strategies.

| Package | Function | Implementation Notes |
|:--------|:---------|:---------------------|
| `summata` | `desctable()` | `data.table` operations |
| `arsenal` | `tableby()` | Formula-based interface |
| `tableone` | `CreateTableOne()` | Matrix-based computation |
| `finalfit` | `summary_factorlist()` | `tidyverse` ecosystem |
| `gtsummary` | `tbl_summary()` | `gt` table framework |

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_desctable.png")
```

| Dataset Size | `summata` | `arsenal` | `tableone` | `finalfit` | `gtsummary` |
|:-------------|--------:|--------:|---------:|---------:|----------:|
| *n* = 1,000    | 42 ms   | 64 ms   | 46 ms    | 429 ms   | 2,901 ms  |
| *n* = 5,000    | 57 ms   | 77 ms   | 79 ms    | 442 ms   | 2,929 ms  |
| *n* = 10,000   | 73 ms   | 98 ms   | 126 ms   | 464 ms   | 3,001 ms  |

The observed timing differences reflect underlying implementation choices. Packages built on `data.table` or base R matrix operations (`summata`, `tableone`, `arsenal`) exhibit lower overhead than those employing more extensive formatting pipelines (`gtsummary`). The `gtsummary` package prioritizes output flexibility and `gt` integration, which introduces additional computational cost.

---

# Survival Tables

Survival probability tables summarize Kaplan-Meier estimates at specified time points.

| Package | Function | Notes |
|:--------|:---------|:------|
| `summata` | `survtable()` | Formatted output |
| manual | `survival::survfit()` | Raw computation |
| `gtsummary` | `tbl_survfit()` | `gt` integration |

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_survtable.png")
```

| Dataset Size | `summata` | `gtsummary` | manual |
|:-------------|--------:|----------:|-------:|
| *n* = 1,000    | 21 ms   | 266 ms    | 6 ms   |
| *n* = 5,000    | 35 ms   | 271 ms    | 11 ms  |
| *n* = 10,000   | 52 ms   | 274 ms    | 14 ms  |

Direct `survfit()` computation provides a baseline for the minimum time required. The difference between raw computation and formatted output reflects the cost of table construction and presentation logic.

---

# Regression Output

The following benchmarks compare functions that extract and format regression coefficients. Each package produces tables suitable for publication, though with varying levels of default formatting. Compared functions are as follows:

| Package | Function | Notes |
|:--------|:---------|:------|
| `summata` | `fit()` | Profile likelihood CIs, QC stats, counts, and reference rows |
| `summata_minimal` | `fit(..., conf_method = "wald", show_n = FALSE, show_events = FALSE, reference_rows = FALSE, keep_qc_stats = FALSE)` | Wald CIs, reduced output |
| `finalfit` | `glmuni()` + `fit2df()` | Profile likelihood CIs (default) |
| `broom` | `tidy()` | Profile likelihood CIs via `confint()` dispatch |
| `gtsummary` | `tbl_regression()` | `gt` formatting |

## Logistic Regression

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_logistic.png")
```

| Dataset Size | `summata_minimal` | `summata` | `finalfit` | `broom` | `gtsummary` |
|:-------------|----------------:|--------:|---------:|------:|----------:|
| *n* = 500      | 18 ms   | 174 ms  | 147 ms   | 150 ms | 1,344 ms |
| *n* = 1,000    | 22 ms   | 234 ms  | 212 ms   | 214 ms | 1,399 ms |
| *n* = 5,000    | 37 ms   | 840 ms  | 749 ms   | 936 ms | 2,153 ms |
| *n* = 10,000   | 45 ms   | 1,532 ms | 1,562 ms | 1,564 ms | 2,756 ms |

The default `summata` configuration uses profile likelihood confidence intervals for GLM models, as do `finalfit` and `broom::tidy()`. The three packages show comparable performance for logistic regression because profile likelihood profiling dominates execution time for all of them. The `summata_minimal` configuration uses Wald CIs instead, skipping the profiling step entirely, and achieves the fastest extraction times at all sample sizes. At large *n*, profiling cost grows with the number of IRLS iterations, causing all profile-based packages to converge toward similar timings.

## Linear Regression

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_linear.png")
```

| Dataset Size | `summata_minimal` | `summata` | `finalfit` | `broom` | `gtsummary` |
|:-------------|----------------:|--------:|---------:|------:|----------:|
| *n* = 500      | 20 ms   | 35 ms   | 6 ms     | 6 ms  | 1,179 ms  |
| *n* = 1,000    | 21 ms   | 36 ms   | 7 ms     | 6 ms  | 1,193 ms  |
| *n* = 5,000    | 27 ms   | 43 ms   | 9 ms     | 9 ms  | 1,192 ms  |
| *n* = 10,000   | 38 ms   | 49 ms   | 13 ms    | 12 ms | 1,230 ms  |

For linear models, `broom::tidy()` and `finalfit` achieve faster coefficient extraction due to lower formatting overhead. All three packages use exact *t*-distribution CIs for `lm` objects (via `confint.lm()`), so the timing difference reflects formatting features (reference rows, QC statistics) rather than CI computation.

## Poisson Regression

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_poisson.png")
```

| Dataset Size | `summata_minimal` | `summata` | `finalfit` | `broom` | `gtsummary` |
|:-------------|----------------:|--------:|---------:|------:|----------:|
| *n* = 500      | 20 ms   | 155 ms  | 135 ms   | 144 ms | 1,293 ms |
| *n* = 1,000    | 24 ms   | 201 ms  | 181 ms   | 184 ms | 1,325 ms |
| *n* = 5,000    | 34 ms   | 595 ms  | 613 ms   | 612 ms | 1,868 ms |
| *n* = 10,000   | 47 ms   | 1,351 ms | 1,409 ms | 1,402 ms | 2,577 ms |

Poisson regression shows the same profile likelihood pattern as logistic regression: the default `summata`, `finalfit`, and `broom` all use profile CIs and show comparable performance. The `summata_minimal` configuration with Wald CIs is consistently the fastest option.

## Cox Regression

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_cox.png")
```

| Dataset Size | summata_minimal | summata | finalfit | broom | gtsummary |
|:-------------|----------------:|--------:|---------:|------:|----------:|
| *n* = 500      | 17 ms   | 34 ms   | 7 ms     | 12 ms | 1,149 ms  |
| *n* = 1,000    | 21 ms   | 38 ms   | 9 ms     | 14 ms | 1,161 ms  |
| *n* = 5,000    | 40 ms   | 61 ms   | 25 ms    | 30 ms | 1,227 ms  |

Cox models use Wald CIs regardless of the `conf_method` setting (the standard approach in survival analysis), so the timing difference between `summata` and `summata_minimal` reflects formatting overhead only. `finalfit` and `broom` achieve faster extraction with less formatting.

---

# Mixed-Effects Models

Mixed-effects models present a useful comparison case because the underlying model fitting (via `lme4`) dominates execution time regardless of the wrapper package.

| Package | Function | Notes |
|:--------|:---------|:------|
| `summata` | `fit(..., model_type = "lmer")` | Unified interface |
| `summata_minimal` | `fit(..., model_type = "lmer", conf_method = "wald", show_n = FALSE, show_events = FALSE, reference_rows = FALSE, keep_qc_stats = FALSE)` | Reduced output |
| `finalfit` | `lmmixed()` + `fit2df()` | Two-step process |
| `broom.mixed` | `tidy()` | Minimal extraction |
| `gtsummary` | `tbl_regression()` | gt formatting |

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_mixed.png")
```

| Dataset Size | `summata_minimal` | `summata` | `finalfit` | `broom` | `gtsummary` |
|:-------------|----------------:|--------:|---------:|------:|----------:|
| *n* = 500      | 35 ms   | 58 ms   | 26 ms    | 31 ms | 1,141 ms  |
| *n* = 1,000    | 37 ms   | 60 ms   | 28 ms    | 34 ms | 1,133 ms  |
| *n* = 5,000    | 52 ms   | 76 ms   | 43 ms    | 47 ms | 1,185 ms  |

The relatively narrow spread among `summata`, `finalfit`, and `broom` reflects the dominance of model fitting time. Differences in wrapper overhead become proportionally less significant as the underlying computation grows.

---

# Univariable Screening

Univariable screening—fitting separate models for each predictor—provides a test case for operations involving many repeated model fits.

| Package | Function | Notes |
|:--------|:---------|:------|
| `summata` | `uniscreen()` | Parallel-capable |
| `summata_minimal` | `uniscreen(..., conf_method = "wald", show_n = FALSE, show_events = FALSE, reference_rows = FALSE)` | Wald CIs, reduced output |
| `finalfit` | `glmuni()` + `fit2df()` | Sequential |
| `broom` | Loop + `tidy()` | Manual implementation |
| `arsenal` | `modelsum()` | Formula interface |
| `gtsummary` | `tbl_uvregression()` | gt formatting |

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_uniscreen.png")
```

Screening 14 predictors:

| Dataset Size | `summata_minimal` | `summata` | `finalfit` | `broom` | `arsenal` | `gtsummary` |
|:-------------|----------------:|--------:|---------:|------:|--------:|----------:|
| *n* = 500      | 117 ms  | 319 ms  | 360 ms   | 440 ms     | 877 ms   | 12,963 ms |
| *n* = 1,000    | 134 ms  | 351 ms  | 477 ms   | 558 ms     | 1,128 ms | 13,025 ms |
| *n* = 5,000    | 196 ms  | 624 ms  | 1,763 ms | 1,799 ms   | 3,401 ms | 14,001 ms |

The performance gap between `summata` (default) and `summata_minimal` is amplified during univariable screening because profile likelihood profiling is repeated for each of the 14 predictor models. All profile-based packages (`summata` default, `finalfit`, `broom`) show comparable performance, as profiling dominates their execution time. With Wald CIs, `summata_minimal` is the fastest option at all sample sizes, outperforming the next-fastest alternative by 2.6–9.0× due to `data.table` vectorization and parallel model fitting.

---

# Complete Workflow

The combined univariable screening and multivariable modeling workflow represents a common analytical pattern in statistical research.

| Package | Approach | Notes |
|:--------|:---------|:------|
| `summata` | `fullfit()` | Single function |
| `summata_minimal` | `fullfit(..., conf_method = "wald", show_n = FALSE, show_events = FALSE, reference_rows = FALSE)` | Wald CIs, reduced output |
| `finalfit` | `finalfit()` | Single function |
| manual | Loop + `glm()` + `broom::tidy()` + `rbind()` | Custom |
| `gtsummary` | `tbl_uvregression()` + `tbl_regression()` + `tbl_merge()` | Multi-step |

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_workflow.png")
```

| Dataset Size | `summata_minimal` | `summata` | `finalfit` | manual | `gtsummary` |
|:-------------|----------------:|--------:|---------:|-------:|----------:|
| *n* = 500      | 123 ms  | 450 ms  | 207 ms   | 407 ms | 9,655 ms  |
| *n* = 1,000    | 136 ms  | 549 ms  | 200 ms   | 541 ms | 9,726 ms |
| *n* = 5,000    | 196 ms  | 1,479 ms | 209 ms  | 1,889 ms | 11,092 ms |

The default `summata` and `finalfit` show comparable performance for GLM workflows because both use profile likelihood CIs. The difference between them reflects `summata`'s additional features (QC statistics, reference rows, complete-case sample sizes) versus `finalfit`'s inclusion of a descriptive statistics table. The `summata_minimal` configuration with Wald CIs is the fastest single-function option at small to moderate sample sizes, completing the combined analysis in roughly 60–70% of the time `finalfit` requires at *n* = 500–1,000.

---

# Forest Plots

Forest plot generation combines data extraction with graphical rendering.

| Package | Function | Notes |
|:--------|:---------|:------|
| `summata` | `coxforest()` | Integrated table and plot |
| `survminer` | `ggforest()` | Survival-focused |
| manual | Custom `ggplot2` | Maximum flexibility |

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_forest.png")
```

| Dataset Size | `summata` | `survminer` | manual |
|:-------------|--------:|----------:|-------:|
| *n* = 500      | 203 ms  | 345 ms    | 57 ms  |
| *n* = 1,000    | 198 ms  | 338 ms    | 54 ms  |
| *n* = 5,000    | 203 ms  | 329 ms    | 53 ms  |

The manual approach produces only the graphical element, while `summata` and `survminer` generate integrated displays with coefficient tables. The relatively constant timing across dataset sizes indicates that plot rendering, rather than data processing, dominates execution time. Also, there are significant cosmetic differences between the three graphical outputs, which predominates other factors when selecting a plotting function.

---

# Relative Performance

The following figures summarize timing ratios across benchmarks. Values greater than 1 indicate the comparison package requires more time than the baseline.

## Relative to `summata` (default, profile likelihood CIs)

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_speedup.png")
```

## Relative to `summata_minimal` (Wald CIs, no QC stats)

```{r, echo = FALSE, eval = TRUE, out.width = "100%"}
knitr::include_graphics("figures/benchmark_speedup_minimal.png")
```

## Summary of Ratios

Ratios relative to `summata` (default):

| Benchmark | `gtsummary` | `finalfit` | `arsenal` |
|:----------|----------:|---------:|--------:|
| Descriptive Tables | 41–70× | 6–10× | 1.4–1.5× |
| Survival Tables | 5–12× | — | — |
| Logistic Regression | 6–8× | 0.8–1.0× | — |
| Poisson Regression | 7–8× | 0.9–1.0× | — |
| Linear Regression | 25–34× | 0.2–0.3× | — |
| Cox Regression | 20–34× | 0.2–0.4× | — |
| Mixed-Effects | 16–20× | 0.5–0.6× | — |
| Univariable Screening | 22–41× | 1.1–2.8× | 2.8–5.5× |
| Complete Workflow | 8–21× | 0.1–0.5× | — |

Ratios relative to `summata_minimal` (Wald CIs):

| Benchmark | `gtsummary` | `finalfit` | `summata` (default) |
|:----------|----------:|---------:|--------:|
| Logistic Regression | 58–74× | 8–35× | 10–34× |
| Poisson Regression | 55–63× | 7–30× | 8–29× |
| Linear Regression | 32–59× | 0.3× | 1.3–1.7× |
| Cox Regression | 31–68× | 0.4–0.6× | 1.5–2.0× |
| Mixed-Effects | 23–33× | 0.8–0.9× | 1.5–1.7× |
| Univariable Screening | 71–111× | 3.1–9.0× | 2.6–3.2× |
| Complete Workflow | 57–78× | 1.1–1.7× | 3.7–7.5× |

For GLM models (logistic and Poisson), `summata_minimal` outperforms all alternatives by a wide margin: 8–35× faster than `finalfit`, 7–30× faster than `broom`. This is because `summata_minimal` is the only configuration that uses Wald CIs — `finalfit`, `broom`, and the default `summata` all use profile likelihood CIs, which accounts for their comparable timings.

For linear, Cox, and mixed-effects models, where all packages use the same CI method (exact *t*-distribution for lm, Wald for Cox and mixed-effects), the timing gap between `summata` and `summata_minimal` is narrow (1.3–2.0×) and reflects formatting overhead only.

---

# Scaling Characteristics

The relationship between dataset size and execution time provides insight into algorithmic complexity. Near-linear scaling (execution time proportional to *n*) indicates efficient implementation, while superlinear scaling may suggest operations with *O*(*n*²) complexity, such as repeated `rbind()` calls or element-wise data frame construction.

Observed scaling factors for `summata` (ratio of time at *n* = 10,000 to time at *n* = 1,000):

| Operation | Scaling Factor | Expected for *O*(*n*) |
|:----------|---------------:|----------------------:|
| Descriptive tables | 1.7× | 10× |
| Logistic regression | 6.5× | 10× |
| Univariable screening | 1.8× | 10× |

The sublinear scaling reflects that fixed overhead (package loading, object construction, profile likelihood profiling) constitutes a significant fraction of total time at smaller dataset sizes. Logistic regression shows nearer-to-linear scaling because profile likelihood profiling cost scales with the number of IRLS iterations, which grows with *n*.

---

# Implementation Notes

The performance characteristics documented here reflect specific implementation choices:

**summata**: Built on `data.table` for data manipulation, with coefficient extraction optimized for common model classes. Default configuration uses profile likelihood CIs for GLM/negbin models (matching `finalfit` and `broom`). The `conf_method = "wald"` option skips profiling entirely, producing a configuration faster than any alternative tested.
 
**gtsummary**: Prioritizes output flexibility through the `gt` table framework. The additional abstraction layers enable extensive customization but increase computational overhead.

**finalfit**: Balances functionality and performance with a tidyverse-compatible interface. Uses profile likelihood CIs by default for GLM models (`confint_type = "profile"`). The `finalfit()` function is particularly optimized for the combined workflow.

**arsenal**: Uses formula-based syntax familiar to SAS users. Performance varies by operation type.

**broom**: Provides minimal coefficient extraction with limited formatting. Uses profile likelihood CIs for GLM models via `stats::confint()` dispatch. Suitable as a building block for custom pipelines.

---

# Effect of Output Options

By default, `summata` regression functions compute profile likelihood confidence intervals (for GLM and negative binomial models), sample sizes, event counts, QC statistics, and reference rows for categorical variables. These features produce more complete and accurate output for publication but add computational overhead. For performance-sensitive applications, these options can be disabled.

The `summata_minimal` configuration shown in the benchmarks represents:

```{r, eval = FALSE}
fit(data, outcome, predictors, 
    conf_method = "wald",
    show_n = FALSE, 
    show_events = FALSE, 
    reference_rows = FALSE,
    keep_qc_stats = FALSE)
```

The `conf_method` parameter can also be set globally for an entire session:

```{r, eval = FALSE}
options(summata.conf_method = "wald")
```

The impact of each option varies by model type:

| Option | GLM/negbin models | Linear/Cox/mixed models |
|:-------|:-----------------|:-----------------------|
| `conf_method = "wald"` | Large effect (eliminates profile likelihood profiling) | Minimal effect (Wald already used for Cox/mixed; exact *t* is fast for lm) |
| `keep_qc_stats = FALSE` | Moderate effect (skips C-statistic, Hosmer-Lemeshow) | Small effect |
| `show_n/show_events = FALSE` | Small effect | Small effect |
| `reference_rows = FALSE` | Small effect | Small effect |

For logistic and Poisson models at *n* = 1,000, the minimal configuration is approximately 10× faster than the default (22 ms vs. 234 ms for logistic), with the majority of the difference attributable to `conf_method`. For linear and Cox models, the difference is roughly 1.5–2×, reflecting formatting overhead only.

The choice between configurations depends on the use case:

- **Publication tables**: Default settings provide profile likelihood CIs and complete output ready for manuscripts
- **Simulation studies**: `conf_method = "wald"` reduces per-iteration overhead substantially for GLM models
- **Exploratory analysis**: Either setting is appropriate; `conf_method = "wald"` is recommended when iterating through many model specifications

---

# Practical Considerations

The timing differences documented here range from negligible (tens of milliseconds) to substantial (several seconds). The practical significance depends on context:

- **Interactive analysis**: Differences under 500 ms are generally imperceptible
- **Batch processing**: Cumulative differences matter when processing many datasets
- **Simulation studies**: Per-iteration overhead compounds across thousands of replicates
- **Teaching and demonstration**: Faster feedback loops improve the interactive experience

Package selection should primarily reflect functional requirements, syntax preferences, and ecosystem compatibility. Performance considerations become relevant only when computational constraints are binding.

---

# Reproducibility

The benchmark script is available in the package repository at `inst/benchmarks/benchmarks.R`. Execution produces:

- Individual PNG figures for each benchmark category
- Summary figures (`benchmark_speedup.png`, `benchmark_speedup_minimal.png`)
- CSV files with detailed timing data

Results will vary across systems due to differences in hardware, R version, and package versions.

---

# Session Information

This benchmark was run under the following conditions:

```
R version 4.5.2 (2025-10-31)
Platform: x86_64-unknown-linux-gnu

Matrix products: default
BLAS/LAPACK: /usr/lib/libopenblasp-r0.3.30.so;  LAPACK version 3.12.0

Void Linux x86_64
Linux 6.12.63_1
Intel(R) Core(TM) i5-4670K (4) @ 3.80 GHz
NVIDIA GeForce GTX 970 [Discrete]
```

---
title: "Advanced Workflows"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Advanced Workflows}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
## Use ragg for better font rendering if available
if (requireNamespace("ragg", quietly = TRUE)) {
  old_opts <- options(summata.use_ragg = TRUE, width = 180)
  knitr::opts_chunk$set(
    dev = "ragg_png",
    fig.retina = 1,
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "100%"
  )
} else {
  old_opts <- options(width = 180)
  knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "##>",
    message = FALSE,
    warning = FALSE,
    fig.width = 8,
    fig.height = 5,
    out.width = "100%"
  )
}

## Dynamic figure sizing. A chunk that renders a forest plot names the object it
## is about to print, by giving the chunk option rec_dims_from = "example1".
##
## Chunk options are resolved before knitr opens the graphics device, so this
## hook reads the plot's recommended dimensions from its "rec_dims" attribute
## and sets fig.width and fig.height from them. The object was created in an
## earlier chunk and so already exists in the knit environment. Plots render via
## ragg (dev = "ragg_png" set above) and knitr captures them natively; no files
## are written to disk.
##
## A plot without recommended dimensions, or a name that does not resolve,
## falls through to the default figure size rather than failing the build.
knitr::opts_hooks$set(rec_dims_from = function(options) {
  plot <- get0(options$rec_dims_from, envir = knitr::knit_global(),
               ifnotfound = NULL)
  dims <- attr(plot, "rec_dims")

  if (!is.null(dims)) {
    options$fig.width  <- dims$width
    options$fig.height <- dims$height
  }

  options
})
```

Standard regression analysis assumes independent observations and constant effects across subgroups. In practice, these assumptions often fail: observations may be clustered (e.g., subjects within study sites), effects may vary by subgroup (interaction), or the baseline hazard may differ across strata. This vignette demonstrates advanced analytical techniques to address these complexities.

---

# Preliminaries

The examples in this vignette use the `clintrial` dataset included with `summata`:

```{r setup}
library(summata)
library(survival)
library(ggplot2)

data(clintrial)
data(clintrial_labels)

# Examine the clustering structure
table(clintrial$site)
```

The `clintrial` dataset includes 10 study sites, providing a natural clustering variable for hierarchical analysis.

> *n.b.:* The forest plots below are sized by this vignette from each plot's recommended dimensions, stored in its `"rec_dims"` attribute, and rendered via the [`ragg`](https://ragg.r-lib.org/) graphics device for correct font metrics. That sizing is applied behind the scenes and is not shown in the code. Writing a plot to file elsewhere is performed with `forestsave()`, which applies the same dimensions and selects a suitable graphics device:
> 
> ```{r, eval = FALSE}
> p <- glmforest(model, data = mydata)
> forestsave(p, "forest_plot.png")
> ```
> 
> This ensures that the figure size is always large enough to accommodate the constituent plot text and graphics, and it is the preferred method for saving forest plot outputs in `summata`.

---

# Interaction Effects

Interaction analysis tests whether the association between a predictor and outcome varies across levels of another variable. Interactions are specified using colon notation in the `interactions` parameter; main effects are automatically included alongside the interaction term.

## **Example 1:** Two-Way Interaction

Specify interactions using colon notation in the `interactions` parameter:

```{r}
example1 <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  interactions = c("sex:treatment"),
  model_type = "glm",
  labels = clintrial_labels
)

example1
```

## **Example 2:** Multiple Interactions

Test multiple effect modifiers simultaneously:

```{r}
example2 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage"),
  interactions = c("sex:treatment", "stage:treatment"),
  model_type = "coxph",
  labels = clintrial_labels
)

example2
```

## **Example 3:** Continuous × Categorical Interaction

Test whether effects of a continuous variable vary by group:

```{r}
example3 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "surgery"),
  interactions = c("age:treatment"),
  model_type = "lm",
  labels = clintrial_labels
)

example3
```

## **Example 4:** Interaction in fullfit()

Include interaction terms in combined univariable/multivariable analysis:

```{r}
example4 <- fullfit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage", "sex:treatment"),
  model_type = "glm",
  method = "all",
  labels = clintrial_labels
)

example4
```

## **Example 5:** Testing Interaction Significance

Use `compfit()` to formally evaluate whether interactions improve model fit (*see* [Model Comparison](model_comparison.html)):

```{r}
example5 <- compfit(
  data = clintrial,
  outcome = "surgery",
  model_list = list(
    "Main Effects" = c("age", "sex", "treatment", "stage"),
    "Sex × Treatment" = c("age", "sex", "treatment", "stage"),
    "Stage × Treatment" = c("age", "sex", "treatment", "stage"),
    "Both Interactions" = c("age", "sex", "treatment", "stage")
  ),
  interactions_list = list(
    NULL,
    c("sex:treatment"),
    c("stage:treatment"),
    c("sex:treatment", "stage:treatment")
  ),
  model_type = "glm",
  labels = clintrial_labels
)

example5
```

## **Example 6:** Forest Plot with Interactions

Visualize models including interaction terms:

```{r}
interaction_model <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  interactions = c("sex:treatment"),
  model_type = "glm",
  labels = clintrial_labels
)

example6 <- glmforest(
  x = attr(interaction_model, "model"),
  title = "Logistic Regression with Interaction",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

```{r, echo = FALSE, out.width = "100%", rec_dims_from = "example6"}
print(example6)
```

---

# Mixed-Effects Models

Mixed-effects models (multilevel or hierarchical models) account for correlation within clusters by incorporating random effects. The `summata` package supports linear mixed models (`lmer`), generalized linear mixed models (`glmer`), and Cox mixed models (`coxme`).

Random effects are specified using pipe notation within either the `predictors` parameter or in a separate `random` parameter:

| Syntax | Meaning |
|:-------|:--------|
| `(1\|group)` | Random intercepts by group |
| `(x\|group)` | Random intercepts and slopes for *x* |
| `(1\|g1) + (1\|g2)` | Crossed random effects |

## **Example 7:** Random Intercepts (Linear Mixed)

The `(1|site)` syntax specifies a random intercept for each site:

```{r}
example7 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "lmer",
  labels = clintrial_labels
)

example7
```

## **Example 8:** Random Intercepts (Logistic Mixed)

For binary outcomes with clustering (note the use of an independent `random` parameter):

```{r}
example8 <- fit(
    data = clintrial,
    outcome = "surgery",
    predictors = c("age", "sex", "treatment", "stage"),
    random = "(1|site)",
    model_type = "glmer",
    labels = clintrial_labels
)

example8
```

## **Example 9:** Random Slopes

Allow effects to vary by cluster:

```{r}
example9 <- fit(
  data = clintrial,
  outcome = "los_days",
  predictors = c("age", "sex", "treatment", "stage", "(1 + treatment|site)"),
  model_type = "lmer",
  labels = clintrial_labels
)

example9
```

## **Example 10:** Cox Mixed-Effects Model

For survival outcomes with clustering:

```{r}
example10 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "(1|site)"),
  model_type = "coxme",
  labels = clintrial_labels
)

example10
```

## **Example 11:** Forest Plot from Mixed Model

Visualize fixed effects from mixed-effects models:

```{r}
example11 <- glmforest(
  x = attr(example8, "model"),
  title = "Logistic Mixed Model (Fixed Effects)",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

```{r, echo = FALSE, out.width = "100%", rec_dims_from = "example11"}
print(example11)
```

## **Example 12:** Comparing Random-Effects Specifications

Use `compfit()` to compare different random-effects structures (*see* [Model Comparison](model_comparison.html)):

```{r}
example12 <- compfit(
  data = clintrial,
  outcome = "los_days",
  model_list = list(
    "Random Intercepts" = c("age", "sex", "treatment", "stage", "(1|site)"),
    "Random Slopes" = c("age", "sex", "treatment", "stage", "(1 + treatment|site)")
  ),
  model_type = "lmer",
  labels = clintrial_labels
)

example12
```

---

# Stratification and Clustering

In addition to mixed-effects models, Cox models support stratification (separate baseline hazards) and clustered standard errors (robust variance estimation).

| Approach | When to Use |
|:---------|:------------|
| Mixed-effects | Model cluster-specific effects; prediction at cluster level |
| Stratification | Proportional hazards assumption violated for a variable |
| Clustered SE | Correlation within clusters; robust inference |

## **Example 13:** Stratified Cox Model

Stratification allows nonproportional hazards across strata without estimating stratum effects. Use the `strata` parameter:

```{r}
example13 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment"),
  strata = "site",
  model_type = "coxph",
  labels = clintrial_labels
)

example13
```

## **Example 14:** Cluster-Robust SEs

For robust inference with correlated observations, cluster-robust standard errors account for within-cluster correlation. Use the `cluster` parameter:

```{r}
example14 <- fit(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage"),
  cluster = "site",
  model_type = "coxph",
  labels = clintrial_labels
)

example14
```

---

# Weighted Regression

For survey data or inverse probability weighting, use the `weights` parameter. Weights can account for sampling design, non-response, or confounding adjustment.

## **Example 15:** Weighted Analysis

```{r}
# Create example weights
clintrial$analysis_weight <- runif(nrow(clintrial), 0.5, 2.0)

example15 <- fit(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  weights = "analysis_weight",
  model_type = "glm",
  labels = clintrial_labels
)

example15
```

---

# Advanced Univariable Screening Features

The `uniscreen()` function supports advanced model specifications including random effects and stratification.

## **Example 16:** Univariable Screening with Random Effects

Screen multiple predictors while accounting for clustering:

```{r}
example16 <- uniscreen(
  data = clintrial,
  outcome = "surgery",
  predictors = c("age", "sex", "treatment", "stage"),
  model_type = "glmer",
  random = "(1|site)",
  labels = clintrial_labels
)

example16
```

## **Example 17:** Cox Screening with Mixed Effects

For survival outcomes with clustering:

```{r}
example17 <- uniscreen(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog"),
  model_type = "coxme",
  random = "(1|site)",
  labels = clintrial_labels
)

example17
```

## **Example 18:** Forest Plot from Univariable Screening

The `uniforest()` function visualizes univariable screening results. Labels supplied to `uniscreen()` are carried through automatically, and may also be given directly:

```{r}
uni_results <- uniscreen(
  data = clintrial,
  outcome = "Surv(os_months, os_status)",
  predictors = c("age", "sex", "treatment", "stage", "ecog", "grade"),
  model_type = "coxph",
  labels = clintrial_labels
)

example18 <- uniforest(
  uni_results,
  title = "Univariable Screening Results",
  labels = clintrial_labels,
  indent_groups = TRUE,
  zebra_stripes = TRUE
)
```

```{r, echo = FALSE, out.width = "100%", rec_dims_from = "example18"}
print(example18)
```

---

# Advanced Multivariate Regression Features

The `multifit()` function supports interactions and mixed-effects models when testing a predictor across multiple outcomes.

## **Example 19:** Multi-Outcome with Interactions

Test effect modification across multiple outcomes:

```{r}
example19 <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status", "os_status"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage"),
  interactions = c("treatment:sex"),
  labels = clintrial_labels,
  parallel = FALSE
)

example19
```

## **Example 20:** Multi-Outcome with Mixed Effects

Account for clustering when testing effects across outcomes:

```{r}
example20 <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  random = "(1|site)",
  model_type = "glmer",
  labels = clintrial_labels,
  parallel = FALSE
)

example20
```

## **Example 21:** Survival Multi-Outcome with Mixed Effects

For multiple survival outcomes with clustering:

```{r}
example21 <- multifit(
  data = clintrial,
  outcomes = c("Surv(pfs_months, pfs_status)",
               "Surv(os_months, os_status)"),
  predictor = "treatment",
  covariates = c("age", "sex"),
  random = "(1|site)",
  model_type = "coxme",
  labels = clintrial_labels,
  parallel = FALSE
)

example21
```

---

# Complete Workflow Example

The following demonstrates a complete advanced analysis workflow:

```{r, fig.width = 12, fig.height = 8}
# Step 1: Screen risk factors for primary outcome
risk_screening <- uniscreen(
  data = clintrial,
  outcome = "os_status",
  predictors = c("age", "sex", "bmi", "smoking", "diabetes",
                 "hypertension", "stage", "ecog", "treatment"),
  model_type = "glm",
  p_threshold = 0.20,
  labels = clintrial_labels
)

risk_screening

# Step 2: Test key exposure across multiple outcomes
effects <- multifit(
  data = clintrial,
  outcomes = c("surgery", "pfs_status", "os_status"),
  predictor = "treatment",
  covariates = c("age", "sex", "stage"),
  columns = "both",
  labels = clintrial_labels,
  parallel = FALSE
)

effects

# Step 3: Visualize effects
forest_plot <- multiforest(
  effects,
  title = "Effects Across Outcomes",
  labels = clintrial_labels,
  indent_predictor = TRUE,
  zebra_stripes = TRUE
)
```

```{r, echo = FALSE, out.width = "100%", rec_dims_from = "forest_plot"}
print(forest_plot)
```

---

# Best Practices

## Interaction Analysis

1. Include main effects for all variables in interactions
2. Test formally using interaction *p*-values
3. Non-significant interactions do not prove effect homogeneity
4. Consider domain plausibility when interpreting

## Mixed-Effects Models

1. Use when data exhibits natural clustering
2. Start with random intercepts; add random slopes if justified
3. Monitor convergence; simplify if necessary
4. Compare to fixed-effects models using `compfit()`

## Method Selection

| Scenario | Recommended Approach |
|:---------|:---------------------|
| Clustered observations | Mixed-effects models |
| Robust inference needed | Clustered standard errors |
| PH assumption violated | Stratification |
| Effect modification | Interaction terms |
| Survey data | Weighted regression |

## Sample Size Considerations

- **Interactions**: Require larger samples than main effects models
- **Mixed models**: Need sufficient clusters (≥ 20-30 recommended)
- **Stratification**: Each stratum needs adequate events

---

# Common Issues

## Convergence Problems in Mixed Models

Simplify the random effects structure if convergence fails:

```{r, eval = FALSE}
# Start with random intercepts only
fit(data, outcome, c(predictors, "(1|site)"), model_type = "lmer")
```

Check for common problems such as few observations per cluster, near-zero variance in random effects, or highly correlated predictors.

## Interpreting Interactions

For categorical × categorical interactions, the coefficient represents the additional effect beyond the sum of main effects:

```{r, eval = FALSE}
# Access model for detailed interpretation
model <- attr(result, "model")
summary(model)
```

## Forest Plots with Interactions

Forest plots display all terms including interactions. For complex interaction patterns, consider stratified analyses or effect plots.

```{r, include = FALSE}
options(old_opts)
```

---

# Further Reading

- [Descriptive Tables](descriptive_tables.html): `desctable()` for baseline characteristics
- [Regression Modeling](regression_modeling.html): `uniscreen()`, `fit()`, and `fullfit()`
- [Model Comparison](model_comparison.html): `compfit()` for comparing models
- [Table Export](table_export.html): Export to PDF, Word, and other formats
- [Forest Plots](forest_plots.html): Visualization of regression results
- [Multivariate Regression](multivariate_regression.html): `multifit()` for multi-outcome analysis

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{normalize_model_type}
\alias{normalize_model_type}
\title{Normalize model type names}
\usage{
normalize_model_type(model_type)
}
\arguments{
\item{model_type}{Character string of model type or class name.}
}
\value{
Normalized character string (\emph{e.g.,} "lmerMod" becomes "lmer").
}
\description{
Converts model class names to standardized model type strings.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{perform_categorical_test}
\alias{perform_categorical_test}
\title{Perform statistical tests for categorical variables}
\usage{
perform_categorical_test(tab, test_type)
}
\arguments{
\item{tab}{Contingency table (matrix or table object).}

\item{test_type}{Character string: "chisq" for chi-square, "fisher" for
Fisher's exact, or "auto" for automatic selection.}
}
\value{
Numeric \emph{p}-value from the hypothesis test.
}
\description{
Conducts chi-square or Fisher's exact tests for categorical variables
across groups. Automatically selects Fisher's exact test for small
expected frequencies.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{format_multifit_table}
\alias{format_multifit_table}
\title{Format multifit results for publication}
\usage{
format_multifit_table(
  data,
  columns,
  show_n = TRUE,
  show_events = TRUE,
  digits = 2,
  p_digits = 3,
  labels = NULL,
  predictor_label = NULL,
  include_predictor = TRUE,
  exponentiate = NULL,
  conf_level = 0.95,
  marks = NULL
)
}
\arguments{
\item{data}{Raw combined data.table from combine_multifit_results.}

\item{columns}{Character specifying column layout.}

\item{show_n}{Logical for sample size column.}

\item{show_events}{Logical for events column.}

\item{digits}{Integer decimal places for effects.}

\item{p_digits}{Integer decimal places for p-values.}

\item{labels}{Named vector of labels for outcomes and predictors.}

\item{predictor_label}{Label for predictor variable.}

\item{include_predictor}{Logical for including predictor column.}

\item{exponentiate}{Logical or \code{NULL} for coefficient handling.}
}
\value{
Formatted data.table.
}
\description{
Internal helper that formats raw multivariate results into 
publication-ready table format.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_model_type_name}
\alias{get_model_type_name}
\title{Get readable model type name}
\usage{
get_model_type_name(model)
}
\arguments{
\item{model}{Fitted model object.}
}
\value{
Character string with readable model type name.
}
\description{
Converts model class names to human-readable descriptions. For GLMs,
uses the family to provide specific names (\emph{e.g.,} "Logistic", "Poisson").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{format_survival_cells}
\alias{format_survival_cells}
\title{Survival cell formatting}
\usage{
format_survival_cells(
  est,
  lower,
  upper,
  n_risk,
  n_event,
  stats,
  fmt_est,
  fmt_ci_lower,
  fmt_ci_upper,
  percent,
  marks = NULL
)
}
\arguments{
\item{est}{Numeric vector of estimates.}

\item{lower}{Numeric vector of lower CI bounds.}

\item{upper}{Numeric vector of upper CI bounds.}

\item{n_risk}{Integer vector of numbers at risk.}

\item{n_event}{Integer vector of event counts.}

\item{stats}{Character vector of statistics to include.}

\item{fmt_est}{Format string for estimate.}

\item{fmt_ci_lower}{Format string for lower CI bound.}

\item{fmt_ci_upper}{Format string for upper CI bound.}

\item{percent}{Logical whether percentages.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character vector of formatted cells.
}
\description{
Formats survival probability cells for multiple rows at once. Uses
locale-aware decimal marks and safe CI separators that avoid ambiguity
with negative values or decimal commas.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniforest.R
\name{uniforest}
\alias{uniforest}
\title{Create Forest Plot for Univariable Screening}
\usage{
uniforest(
  x,
  title = "Univariable Screening",
  effect_label = NULL,
  digits = 2,
  p_digits = 3,
  conf_level = 0.95,
  font_size = 1,
  annot_size = 3.88,
  header_size = 5.82,
  title_size = 23.28,
  plot_width = NULL,
  plot_height = NULL,
  table_width = 0.6,
  show_n = TRUE,
  show_events = NULL,
  indent_groups = FALSE,
  condense_table = FALSE,
  bold_variables = FALSE,
  center_padding = 4,
  zebra_stripes = TRUE,
  color = NULL,
  null_line = NULL,
  log_scale = NULL,
  labels = NULL,
  show_footer = TRUE,
  units = "in",
  number_format = NULL
)
}
\arguments{
\item{x}{Univariable screen result object (data.table with class attributes
from \code{uniscreen()}).}

\item{title}{Character string specifying the plot title. Default is 
\code{"Univariable Screening"}. Use descriptive titles for publication.}

\item{effect_label}{Character string for the effect measure label on the 
forest plot axis. Default is \code{NULL}, which auto-detects based on 
model type (\emph{e.g.,} "Odds Ratio", "Hazard Ratio", "Rate Ratio", "Coefficient").}

\item{digits}{Integer specifying the number of decimal places for effect 
estimates and confidence intervals. Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be
between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
when \code{conf_level = 0.90}). Note: This parameter affects display only;
the underlying CIs come from the uniscreen result.}

\item{font_size}{Numeric multiplier controlling the base font size for all 
text elements. Default is 1.0.}

\item{annot_size}{Numeric value controlling the relative font size for data 
annotations. Default is 3.88.}

\item{header_size}{Numeric value controlling the relative font size for column 
headers. Default is 5.82.}

\item{title_size}{Numeric value controlling the relative font size for the 
main plot title. Default is 23.28.}

\item{plot_width}{Numeric value specifying the intended output width in 
specified \code{units}. Used for optimizing layout. Default is \code{NULL} 
(automatic). Recommended: 10-16 inches.}

\item{plot_height}{Numeric value specifying the intended output height in 
specified \code{units}. Default is \code{NULL} (automatic based on number 
of rows).}

\item{table_width}{Numeric value between 0 and 1 specifying the proportion of 
total plot width allocated to the data table. Default is 0.6 (60\% table, 
40\% forest plot).}

\item{show_n}{Logical. If \code{TRUE}, includes a column showing sample sizes.
Counts describe the observations used in fitting rather than every row
supplied. Default is \code{TRUE}.}

\item{show_events}{Logical. If \code{TRUE}, includes a column showing the 
number of events for each row. Default is \code{NULL}, which auto-detects
based on model type (\code{TRUE} for binomial/survival, \code{FALSE} for linear).}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels
under their parent variable name, creating a hierarchical display. If 
\code{FALSE} (default), shows variable and level in separate columns.}

\item{condense_table}{Logical. If \code{TRUE}, condenses binary categorical 
variables into single rows by showing only the non-reference category. 
Automatically sets \code{indent_groups = TRUE}. Useful for tables with 
many binary variables. Default is \code{FALSE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. If \code{FALSE} (default), variable names are displayed in plain
text.}

\item{center_padding}{Numeric value specifying horizontal spacing between 
table and forest plot. Default is 4.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating gray 
background shading to different variables for improved readability. 
Default is \code{TRUE}.}

\item{color}{Character string specifying the color for point estimates in 
the forest plot. Default is \code{NULL}, which auto-selects based on
effect type: purple (\code{"#8A61D8"}) for hazard ratios (Cox), teal 
(\code{"#4BA6B6"}) for odds ratios (logistic), blue (\code{"#3F87EE"}) for 
rate/risk ratios (Poisson, Gamma, \emph{etc.}), and green (\code{"#5A8F5A"}) for 
coefficients (linear models).
Use hex codes or R color names for custom colors.}

\item{null_line}{Numeric value for the reference line position. Default is 
\code{NULL}, which uses 1 for ratio measures (OR, HR, RR) and 0 for 
coefficients.}

\item{log_scale}{Logical. If \code{TRUE}, uses log scale for the x-axis.
Default is \code{NULL}, which auto-detects (\code{TRUE} for OR/HR/RR, \code{FALSE} for
coefficients).}

\item{labels}{Named character vector providing custom display labels for 
variables. Applied to predictor names in the plot.
Default is \code{NULL}, in which case the labels attached by the function
that produced \code{x} are used where \code{x} is a \pkg{summata} result
rather than a model object, so that labels supplied once are carried
through the workflow. Original variable names are used where neither is
available.}

\item{show_footer}{Logical. If \code{TRUE}, displays a footer with the
outcome variable name. Default is \code{TRUE}.}

\item{units}{Character string specifying units for plot dimensions: 
\code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}
}
\value{
A \code{ggplot} object containing the complete forest plot. The plot 
  can be:
  \itemize{
    \item Displayed directly: \code{print(plot)}
    \item Saved to file: \code{forestsave(plot, "forest.pdf")}
    \item Further customized with \pkg{ggplot2} functions
  }
  
  The returned object includes an attribute \code{"rec_dims"} 
  accessible via \code{attr(plot, "rec_dims")}, which is a list 
  containing:
  \describe{
    \item{width}{Numeric. Recommended plot width in specified units}
    \item{height}{Numeric. Recommended plot height in specified units}
    \item{units}{Character. The units the dimensions are expressed in,
      matching the \code{units} argument}
  }
  
  These recommendations are automatically calculated based on the number of 
  variables, text sizes, and layout parameters, and are printed to console 
  if \code{plot_width} or \code{plot_height} are not specified.
  \code{forestsave()} reads all three and requires no further handling.

  The returned object also includes an attribute \code{"table_data"}
  accessible via \code{attr(plot, "table_data")}, a data.table holding the
  values drawn in the plot, in the order of the model terms. Sample sizes
  and event counts describe the observations used in fitting rather than
  every row supplied.
}
\description{
Generates a publication-ready forest plot from a \code{uniscreen()} output
object. The plot displays effect estimates (OR, HR, RR, or coefficients) with
confidence intervals for each predictor tested in univariable analysis against
a single outcome.
}
\details{
The forest plot displays univariable (unadjusted) associations between each
predictor and the outcome. This is useful for:
\itemize{
  \item Visualizing results of initial variable screening
  \item Identifying potential predictors for multivariable modeling
  \item Presenting crude associations alongside adjusted results
  \item Quick visual assessment of effect sizes and significance
}

The plot automatically handles:
\itemize{
  \item Different effect types (OR, HR, RR, coefficients) with appropriate
    axis scaling (log vs linear)
  \item Factor variables with multiple levels (grouped under variable name)
  \item Continuous variables (single row per predictor)
  \item Reference categories for categorical variables
}
}
\examples{
data(clintrial)
data(clintrial_labels)

# Create example uniscreen result
uni_results <- uniscreen(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "smoking", "treatment", "stage"),
    labels = clintrial_labels,
    parallel = FALSE
)

# Example 1: Basic univariable forest plot
p <- uniforest(uni_results, title = "Univariable Associations with Mortality")

\donttest{

old_width <- options(width = 180)

# Example 2: Survival analysis
library(survival)
surv_results <- uniscreen(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    predictors = c("age", "sex", "treatment", "stage"),
    model_type = "coxph",
    labels = clintrial_labels,
    parallel = FALSE
)

# Labels attached by uniscreen() are applied automatically
p2 <- uniforest(surv_results, title = "Univariable Survival Analysis")

# Example 3: Linear regression
lm_results <- uniscreen(
    data = clintrial,
    outcome = "los_days",
    predictors = c("age", "sex", "surgery", "diabetes"),
    model_type = "lm",
    labels = clintrial_labels,
    parallel = FALSE
)

p3 <- uniforest(lm_results, title = "Predictors of Length of Stay")

# Example 4: Customize appearance
p4 <- uniforest(
    uni_results,
    title = "Crude Associations with Mortality",
    color = "#E74C3C",
    indent_groups = TRUE,
    zebra_stripes = TRUE,
    bold_variables = TRUE
)

# Example 5: Save with recommended dimensions
forestsave(p4, file.path(tempdir(), "univariable_forest.pdf"))

options(old_width)

}
}
\seealso{
\code{\link{autoforest}} for automatic model detection,
\code{\link{uniscreen}} for generating univariable screening results,
\code{\link{multiforest}} for multi-outcome forest plots,
\code{\link{coxforest}}, \code{\link{glmforest}}, \code{\link{lmforest}} for
single-model forest plots,
  \code{\link{forestsave}} for saving with recommended dimensions

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=coxforest]{coxforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=recdims]{recdims()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fullfit.R
\name{determine_effect_type}
\alias{determine_effect_type}
\title{Determine Effect Type Label}
\usage{
determine_effect_type(uni_raw, multi_raw, exponentiate, adjusted = FALSE)
}
\arguments{
\item{uni_raw}{Raw univariable data.table containing coefficient columns.
Used to detect effect type when \code{adjusted = FALSE}.}

\item{multi_raw}{Raw multivariable data.table containing coefficient columns.
Used to detect effect type when \code{adjusted = TRUE}.}

\item{exponentiate}{Logical or \code{NULL} controlling label selection:
\describe{
  \item{\code{TRUE}}{Force exponentiated labels (OR, HR, RR, Exp(Coef))}
  \item{\code{FALSE}}{Force coefficient labels (Coefficient, Adj. Coefficient)}
  \item{\code{NULL}}{Auto-detect from column names in raw data}
}}

\item{adjusted}{Logical. If \code{TRUE}, returns adjusted effect labels
(aOR, aHR, aRR, Adj. Coefficient) for multivariable results. If \code{FALSE},
returns unadjusted labels (OR, HR, RR, Coefficient) for univariable results.}
}
\value{
Character string with the effect measure label:
}
\description{
Identifies the appropriate effect measure label (OR, HR, RR, Coefficient,
aOR, aHR, aRR, Adj. Coefficient) based on model type, exponentiation setting,
and whether the estimate is adjusted (multivariable) or unadjusted (univariable).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2docx.R
\name{print.table2docx_result}
\alias{print.table2docx_result}
\title{Print method for table2docx results}
\usage{
\method{print}{table2docx_result}(x, ...)
}
\arguments{
\item{x}{Object of class \code{table2docx_result}.}

\item{...}{Additional arguments passed to print methods.}
}
\value{
Invisibly returns the input object \code{x}. Called for its
  side effect of printing a formatted summary to the console.
}
\description{
Print method for table2docx results
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{number_format}
\alias{number_format}
\title{Number Formatting Utilities}
\description{
Internal utilities for locale-aware number formatting across all summata
output functions. Supports preset locales (US, European, SI/ISO, plain) and
fully custom separator definitions.
}
\section{Global Option}{

The default number format can be set once per session:
\preformatted{
  options(summata.number_format = "eu")
}
This avoids passing \code{number_format} to every function call.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_column_headers_with_n_tex}
\alias{format_column_headers_with_n_tex}
\title{Format column headers with n counts (TeX)}
\usage{
format_column_headers_with_n_tex(col_names, n_row_data)
}
\arguments{
\item{col_names}{Character vector of column names.}

\item{n_row_data}{Named list or data.table row with n values for each column.}
}
\value{
Character vector with LaTeX-formatted headers including n counts.
}
\description{
Creates LaTeX-formatted column headers with sample size counts displayed
below the column name in a stacked format.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{validate_number_format}
\alias{validate_number_format}
\title{Validate number_format parameter}
\usage{
validate_number_format(number_format)
}
\arguments{
\item{number_format}{Value to validate.}
}
\value{
Invisibly returns \code{TRUE} if valid.
}
\description{
Checks that a \code{number_format} value is valid before use. Called early
in top-level functions to fail fast with a clear error message.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoforest.R
\name{autoforest}
\alias{autoforest}
\title{Create Forest Plot with Automatic Model Detection}
\usage{
autoforest(x, data = NULL, title = NULL, ...)
}
\arguments{
\item{x}{One of the following:
\itemize{
  \item A fitted model object: \code{glm}, \code{lm}, \code{coxph}, \emph{etc.}
  \item A \code{fit_result} object from \code{fit()}
  \item A \code{fullfit_result} object from \code{fullfit()}
  \item A \code{uniscreen_result} object from \code{uniscreen()}
  \item A \code{multifit_result} object from \code{multifit()}
}}

\item{data}{Data frame or data.table containing the original data. Required 
when \code{x} is a raw model object. Ignored when \code{x} is a result object
from \code{fit()}, \code{fullfit()}, \code{uniscreen()}, or \code{multifit()}
since these contain embedded data.}

\item{title}{Character string for plot title. If \code{NULL} (default), an 
appropriate title is generated based on the detected model type:
\itemize{
  \item Cox models: "Cox Proportional Hazards Model"
  \item Logistic regression: "Logistic Regression Model"
  \item Poisson regression: "Poisson Regression Model"
  \item Linear regression: "Linear Regression Model"
  \item Uniscreen results: "Univariable [Type] Screening"
  \item Multifit results: "Multivariate [Type] Analysis"
}}

\item{...}{Additional arguments passed to the specific forest plot function.
Common arguments include:
\describe{
  \item{labels}{Named character vector for variable labels}
  \item{digits}{Number of decimal places for estimates (default 2)}
  \item{p_digits}{Number of decimal places for \emph{p}-values (default 3)}
  \item{conf_level}{Confidence level for intervals (default 0.95)}
  \item{show_n}{Logical, show sample sizes (default \code{TRUE})}
  \item{show_events}{Logical, show event counts (default \code{TRUE} for survival/binomial)}
  \item{qc_footnote}{Logical, show model QC stats in footer (default \code{TRUE})}
  \item{zebra_stripes}{Logical, alternating row shading (default \code{TRUE})}
  \item{indent_groups}{Logical, indent factor levels (default \code{FALSE})}
  \item{color}{Color for point estimates}
  \item{table_width}{Proportion of width for table (default 0.6)}
  \item{plot_width, plot_height}{Explicit dimensions}
  \item{units}{Dimension units: \code{"in"}, \code{"cm"}, or \code{"mm"}}
}
See the documentation for the specific forest function for all available options.}
}
\value{
A \code{ggplot} object containing the complete forest plot. The plot 
  can be:
  \itemize{
    \item Displayed directly: \code{print(plot)}
    \item Saved to file: \code{forestsave(plot, "forest.pdf")}
    \item Further customized with \pkg{ggplot2} functions
  }
  
  The returned object includes an attribute \code{"rec_dims"} 
  accessible via \code{attr(plot, "rec_dims")}, which is a list 
  containing:
  \describe{
    \item{width}{Numeric. Recommended plot width in specified units}
    \item{height}{Numeric. Recommended plot height in specified units}
  }
  
  These recommendations are automatically calculated based on the number of 
  variables, text sizes, and layout parameters, and are printed to console 
  if \code{plot_width} or \code{plot_height} are not specified.
}
\description{
A convenience wrapper function that automatically detects the input type and
routes to the appropriate specialized forest plot function. This eliminates
the need to remember which forest function to call for different model types
or analysis objects, making it ideal for exploratory analysis and rapid prototyping.
}
\details{
This function provides a convenient wrapper around the specialized forest
plot functions, automatically routing to the appropriate function based on
the model class or result type. All parameters are passed through to the
underlying function, so the full range of options remains available.

For model-specific advanced features, individual forest functions may be
called directly.

\strong{Automatic Detection Logic:}

The function uses the following priority order for detection:
\enumerate{
  \item \strong{uniscreen results}: Detected by class \code{"uniscreen_result"} or
    presence of attributes \code{outcome}, \code{predictors}, \code{model_type},
    and \code{model_scope = "Univariable"}. Routes to \code{uniforest()}.
  \item \strong{multifit results}: Detected by presence of attributes 
    \code{predictor}, \code{outcomes}, \code{model_type}, and \code{raw_data}.
    Routes to \code{multiforest()}.
  \item \strong{Cox models}: Classes \code{coxph} or \code{clogit}. Routes to 
    \code{coxforest()}.
  \item \strong{GLM models}: Class \code{glm}. Routes to \code{glmforest()}.
  \item \strong{Linear models}: Class \code{lm} (but not \code{glm}). Routes to 
    \code{lmforest()}.
}
}
\examples{
data(clintrial)
data(clintrial_labels)
library(survival)

# Create example model
glm_model <- glm(surgery ~ age + sex + bmi + smoking,
                 family = binomial, data = clintrial)

# Example 1: Logistic regression model
p <- autoforest(glm_model, data = clintrial)
# Automatically detects GLM and routes to glmforest()

\donttest{

# Example 2: Cox proportional hazards model
cox_model <- coxph(Surv(os_months, os_status) ~ age + sex + treatment + stage,
                   data = clintrial)

plot2 <- autoforest(cox_model, data = clintrial)
# Automatically detects coxph and routes to coxforest()

# Example 3: Linear regression model
lm_model <- lm(biomarker_x ~ age + sex + bmi + treatment, data = clintrial)

plot3 <- autoforest(lm_model, data = clintrial)
# Automatically detects lm and routes to lmforest()

# Example 4: With custom labels and formatting options
plot4 <- autoforest(
    cox_model,
    data = clintrial,
    labels = clintrial_labels,
    title = "Prognostic Factors for Overall Survival",
    zebra_stripes = TRUE,
    indent_groups = TRUE
)

# Example 5: From fit() result - data and labels extracted automatically
fit_result <- fit(
    data = clintrial,
    outcome = "surgery",
    predictors = c("age", "sex", "bmi", "treatment"),
    labels = clintrial_labels
)

plot5 <- autoforest(fit_result)

# Save with recommended dimensions
forestsave(plot5, file.path(tempdir(), "forest.pdf"))

}
}
\seealso{
\code{\link{glmforest}} for GLM forest plots,
\code{\link{coxforest}} for Cox model forest plots,
\code{\link{lmforest}} for linear model forest plots,
\code{\link{uniforest}} for univariable screening forest plots,
\code{\link{multiforest}} for multi-outcome forest plots,
\code{\link{fit}} for single-model regression,
\code{\link{fullfit}} for combined univariable/multivariable regression,
\code{\link{uniscreen}} for univariable screening,
\code{\link{multifit}} for multi-outcome analysis,
  \code{\link{forestsave}} for saving with recommended dimensions

Other visualization functions:
\code{\link[=coxforest]{coxforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=recdims]{recdims()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{get_cached_confint}
\alias{get_cached_confint}
\title{Retrieve confidence intervals with cache support}
\usage{
get_cached_confint(model, conf_level = 0.95)
}
\arguments{
\item{model}{Fitted model object. If the model carries a
\code{"cached_confint"} attribute (set by \code{fit()} during table
generation) and the cached confidence level matches \code{conf_level},
the cached result is returned directly.}

\item{conf_level}{Numeric confidence level. Must match the cached level
for the cache to be used.}
}
\value{
A matrix with one row per model coefficient and two columns
  (lower and upper bounds), as returned by \code{stats::confint()}.
}
\description{
Returns confidence intervals for a fitted model, using a cached result
when available from upstream \code{fit()} or \code{m2dt()} calls to avoid
redundant computation. This is particularly beneficial for GLM and negative
binomial models where \code{confint()} performs profile likelihood profiling,
which can be expensive for models with many parameters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{format_survival_ci}
\alias{format_survival_ci}
\title{Format survival median with CI for display}
\usage{
format_survival_ci(median, lower, upper, fmt_str, marks)
}
\arguments{
\item{median}{Numeric median survival time.}

\item{lower}{Numeric lower CI bound.}

\item{upper}{Numeric upper CI bound.}

\item{fmt_str}{Character string \code{sprintf} format specification.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with formatted "median (lower-upper)".
}
\description{
Formats a survival median estimate with confidence interval using
locale-aware decimal marks and safe CI separators. Used by
\code{process_survival} in descriptive tables.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{format_quantile_cells}
\alias{format_quantile_cells}
\title{Quantile cell formatting}
\usage{
format_quantile_cells(est, lower, upper, fmt_str, marks = NULL)
}
\arguments{
\item{est}{Numeric vector of estimates.}

\item{lower}{Numeric vector of lower CI bounds.}

\item{upper}{Numeric vector of upper CI bounds.}

\item{fmt_str}{Format string for numeric values.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character vector of formatted cells.
}
\description{
Formats survival quantile cells for multiple rows at once.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fullfit.R
\name{fullfit}
\alias{fullfit}
\title{Complete Regression Analysis Workflow}
\usage{
fullfit(
  data,
  outcome,
  predictors,
  method = "screen",
  multi_predictors = NULL,
  p_threshold = 0.05,
  columns = "both",
  model_type = "glm",
  family = "binomial",
  random = NULL,
  conf_level = 0.95,
  reference_rows = TRUE,
  show_n = TRUE,
  show_events = TRUE,
  digits = 2,
  p_digits = 3,
  labels = NULL,
  metrics = "both",
  return_type = "table",
  keep_models = FALSE,
  exponentiate = NULL,
  conf_method = NULL,
  parallel = TRUE,
  n_cores = NULL,
  number_format = NULL,
  verbose = NULL,
  ...
)
}
\arguments{
\item{data}{Data frame or data.table containing the analysis dataset. The 
function automatically converts data frames to data.tables for efficient
processing.}

\item{outcome}{Character string specifying the outcome variable name. For 
time-to-event analysis, use \code{Surv()} syntax for the outcome variable
(\emph{e.g.,} \code{"Surv(os_months, os_status)"}).}

\item{predictors}{Character vector of predictor variable names to analyze. 
All predictors are tested in univariable models. The subset included in 
the multivariable model depends on the \code{method} parameter.}

\item{method}{Character string specifying the variable selection strategy:
\itemize{
  \item \code{"screen"} - Automatic selection based on univariable \emph{p}-value 
    threshold. Only predictors with p \eqn{\le} \code{p_threshold} in univariable 
    analysis are included in the multivariable model [default]
  \item \code{"all"} - Include all predictors in both univariable and 
    multivariable analyses (no selection)
  \item \code{"custom"} - Manual selection. All predictors in univariable 
    analysis, but only those specified in \code{multi_predictors} are 
    included in multivariable model
}}

\item{multi_predictors}{Character vector of predictors to include in the 
multivariable model when \code{method = "custom"}. Required when using 
custom selection. Ignored for other methods. Default is \code{NULL}.}

\item{p_threshold}{Numeric \emph{p}-value threshold for automatic variable
selection when \code{method = "screen"}. Predictors with univariable
\emph{p}-value less than or equal to the threshold are included in
multivariable model. Common values: 0.05 (strict), 0.10 (moderate),
0.20 (liberal screening). Default is 0.05. Ignored for other methods.}

\item{columns}{Character string specifying which result columns to display:
\itemize{
  \item \code{"both"} - Show both univariable and multivariable results 
    side-by-side [default]
  \item \code{"uni"} - Show only univariable results
  \item \code{"multi"} - Show only multivariable results
}}

\item{model_type}{Character string specifying the regression model type:
\itemize{
  \item \code{"glm"} - Generalized linear model (default). Supports multiple 
    distributions via the \code{family} parameter including logistic, Poisson, 
    Gamma, Gaussian, and quasi-likelihood models.
  \item \code{"lm"} - Linear regression for continuous outcomes with normally 
    distributed errors.
  \item \code{"coxph"} - Cox proportional hazards model for time-to-event 
    survival analysis. Requires \code{Surv()} outcome syntax.
  \item \code{"clogit"} - Conditional logistic regression for matched 
    case-control studies.
  \item \code{"negbin"} - Negative binomial regression for overdispersed count 
    data (requires MASS package). Estimates an additional dispersion parameter 
    compared to Poisson regression.
  \item \code{"glmer"} - Generalized linear mixed-effects model for hierarchical 
    or clustered data with non-normal outcomes (requires \pkg{lme4} package and 
    \code{random} parameter).
  \item \code{"lmer"} - Linear mixed-effects model for hierarchical or clustered 
    data with continuous outcomes (requires \pkg{lme4} package and \code{random} 
    parameter).
  \item \code{"coxme"} - Cox mixed-effects model for clustered survival data 
    (requires coxme package and \code{random} parameter).
}}

\item{family}{For GLM and GLMER models, specifies the error distribution and link 
function. Can be a character string, a family function, or a family object.
Ignored for non-GLM/GLMER models.

\strong{Binary/Binomial outcomes:}
\itemize{
  \item \code{"binomial"} or \code{binomial()} - Logistic regression for binary 
    outcomes (0/1, TRUE/FALSE). Returns odds ratios (OR). Default.
  \item \code{"quasibinomial"} or \code{quasibinomial()} - Logistic regression 
    with overdispersion. Use when residual deviance >> residual df.
  \item \code{binomial(link = "probit")} - Probit regression (normal CDF link).
  \item \code{binomial(link = "cloglog")} - Complementary log-log link for 
    asymmetric binary outcomes.
}

\strong{Count outcomes:}
\itemize{
  \item \code{"poisson"} or \code{poisson()} - Poisson regression for count 
    data. Returns rate ratios (RR). Assumes mean = variance.
  \item \code{"quasipoisson"} or \code{quasipoisson()} - Poisson regression 
    with overdispersion. Use when variance > mean.
}

\strong{Continuous outcomes:}
\itemize{
  \item \code{"gaussian"} or \code{gaussian()} - Normal/Gaussian distribution 
    for continuous outcomes. Equivalent to linear regression.
  \item \code{gaussian(link = "log")} - Log-linear model for positive continuous 
    outcomes. Returns multiplicative effects.
}

\strong{Positive continuous outcomes:}
\itemize{
  \item \code{"Gamma"} or \code{Gamma()} - Gamma distribution for positive, 
    right-skewed continuous data (\emph{e.g.,} costs, lengths of stay). When
    passed as a string, resolves to log link for interpretable multiplicative
    effects.
  \item \code{Gamma(link = "inverse")} - Gamma with inverse (canonical) link.
  \item \code{Gamma(link = "identity")} - Gamma with identity link for additive 
    effects on positive outcomes.
  \item \code{"inverse.gaussian"} or \code{inverse.gaussian()} - Inverse Gaussian 
    for positive, highly right-skewed data.
}

For negative binomial regression (overdispersed counts), use 
\code{model_type = "negbin"} instead of the \code{family} parameter.

See \code{\link[stats]{family}} for additional details and options.}

\item{random}{Character string specifying the random-effects formula for 
mixed-effects models (\code{glmer}, \code{lmer}, \code{coxme}). Use standard
\pkg{lme4}/\pkg{coxme} syntax, \emph{e.g.,} \code{"(1|site)"} for random
intercepts by site, \code{"(1|site/patient)"} for nested random effects.
Required when \code{model_type} is a mixed-effects model type unless random
effects are included in the \code{predictors} vector. Alternatively, random
effects can be included directly in the \code{predictors} vector using the same 
syntax (\emph{e.g.,} \code{predictors = c("age", "sex", "(1|site)")}), though 
they will not be screened as predictors. Default is \code{NULL}.}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be 
between 0 and 1. Default is 0.95 (95\% CI).}

\item{reference_rows}{Logical. If \code{TRUE}, adds rows for reference 
categories of factor variables with baseline values. Default is \code{TRUE}.}

\item{show_n}{Logical. If \code{TRUE}, includes sample size columns. 
Default is \code{TRUE}.}

\item{show_events}{Logical. If \code{TRUE}, includes events columns (survival 
and logistic models). Default is \code{TRUE}.}

\item{digits}{Integer specifying decimal places for effect estimates. 
Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{labels}{Named character vector or list providing custom display
labels for variables. Names should match variable names, values are
display labels. Default is \code{NULL}.}

\item{metrics}{Character specification for which statistics to display:
\itemize{
  \item \code{"both"} - Show effect estimates with CI and \emph{p}-values [default]
  \item \code{"effect"} - Show only effect estimates with CI
  \item \code{"p"} - Show only \emph{p}-values
}
Can also be a character vector: \code{c("effect", "p")} is equivalent to 
\code{"both"}.}

\item{return_type}{Character string specifying what to return:
\itemize{
  \item \code{"table"} - Return formatted results table only [default]
  \item \code{"model"} - Return multivariable model object only
  \item \code{"both"} - Return list with both table and model
}}

\item{keep_models}{Logical. If \code{TRUE}, stores univariable model objects 
in the output. Can consume significant memory for many predictors. 
Default is \code{FALSE}.}

\item{exponentiate}{Logical. Whether to exponentiate coefficients. Default 
is \code{NULL}, which automatically exponentiates for logistic, Poisson,
and Cox models, and displays raw coefficients for linear models.}

\item{conf_method}{Character string controlling the confidence interval method.
If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
\itemize{
  \item \code{"profile"} - Profile likelihood intervals for GLM and negative
    binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
    intervals for linear models. Falls back to Wald on profiling failure.
    Quasi-likelihood families always use Wald (no true likelihood).
  \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
    \eqn{\times} SE) for all model types. Faster but less accurate near
    boundary conditions or with small subgroups.
}
Cox and mixed-effects models use Wald intervals regardless of this setting.
Set globally with \code{options(summata.conf_method = "wald")} to use Wald
throughout a session. Note: when \code{method = "screen"} and
\code{columns = "multi"}, the internal screening pass always uses Wald
since only \emph{p}-values are needed for variable selection.}

\item{parallel}{Logical. If \code{TRUE} (default), fits univariable models
in parallel using multiple CPU cores for improved performance.}

\item{n_cores}{Integer specifying the number of CPU cores to use for
parallel processing. Default is \code{NULL} (auto-detect: uses
\code{detectCores() - 1}). Ignored when \code{parallel = FALSE}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}

\item{verbose}{Logical. If \code{TRUE}, displays model fitting warnings
(\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
routine fitting messages are suppressed while unexpected warnings are
preserved. When \code{NULL}, uses
\code{getOption("summata.verbose", FALSE)}.}

\item{...}{Additional arguments passed to model fitting functions (\emph{e.g.,} 
\code{weights}, \code{subset}, \code{na.action}).}
}
\value{
Depends on \code{return_type} parameter:
  
  When \code{return_type = "table"} (default): A data.table with S3 class 
  \code{"fullfit_result"} containing:
  \describe{
    \item{Variable}{Character. Predictor name or custom label}
    \item{Group}{Character. Category level for factors, empty for continuous}
    \item{n/n_group}{Integer. Sample sizes (if \code{show_n = TRUE}). For
      variables included in the multivariable model, reflects the
      complete-case sample size from the fitted model (listwise deletion
      across all included predictors). For variables not selected into the
      multivariable model, reflects the per-variable sample size from the
      univariable analysis. This follows STROBE guideline item 12,
      which recommends reporting the number of participants included at
      each stage of analysis.}
    \item{events/events_group}{Integer. Event counts (if \code{show_events = TRUE}).
      Same complete-case convention as \code{n}: multivariable rows show
      events from the fitted model, univariable-only rows show
      per-variable counts.}
    \item{OR/HR/RR/Coefficient (95\% CI)}{Character. Unadjusted effect 
      (if \code{columns} includes "uni" and \code{metrics} includes "effect")}
    \item{Uni p}{Character. Univariable \emph{p}-value (if \code{columns} includes 
      "uni" and \code{metrics} includes "p")}
    \item{aOR/aHR/aRR/Adj. Coefficient (95\% CI)}{Character. Adjusted effect 
      (if \code{columns} includes "multi" and \code{metrics} includes "effect")}
    \item{Multi p}{Character. Multivariable \emph{p}-value (if \code{columns} 
      includes "multi" and \code{metrics} includes "p")}
  }
  
  When \code{return_type = "model"}: The fitted multivariable model object 
  (glm, lm, coxph, \emph{etc.}).
  
  When \code{return_type = "both"}: A list with two elements:
  \describe{
    \item{table}{The formatted results data.table}
    \item{model}{The fitted multivariable model object}
  }
  
  The table includes the following attributes:
  \describe{
    \item{outcome}{Character. The outcome variable name}
    \item{model_type}{Character. The regression model type}
    \item{method}{Character. The variable selection method used}
    \item{columns}{Character. Which columns were displayed}
    \item{model}{The multivariable model object (if fitted)}
    \item{uni_results}{The complete univariable screening results}
    \item{n_multi}{Integer. Number of predictors in multivariable model}
    \item{screened}{Character vector. Names of predictors that passed 
      univariable screening at the specified \emph{p}-value threshold}
    \item{significant}{Character vector. Names of variables with p < 0.05
      in the multivariable model (or univariable if multivariable was not
      fitted)}
    \item{analysis_counts}{List. How much of the supplied data reached the
      models, with elements \code{n_supplied}, \code{n_analyzed},
      \code{n_missing_outcome} and \code{n_missing_predictor}, plus
      \code{events_supplied} and \code{events_analyzed} where the model
      type carries an event count. The screening models and the
      multivariable model rest on different samples whenever a predictor
      carries missing values, so \code{n_analyzed} holds one value per
      model and the reported sample may be a range. The \code{print()}
      method always reports the analyzed sample}
  }
}
\description{
Executes a comprehensive regression analysis pipeline that combines univariable 
screening, automatic/manual variable selection, and multivariable modeling 
in a single function call. This function is designed to streamline the complete 
analytical workflow from initial exploration to final adjusted models, with 
publication-ready formatted output showing both univariable and multivariable 
results side-by-side if desired.
}
\details{
\strong{Analysis Workflow:}

The function implements a complete regression analysis pipeline:
\enumerate{
  \item \strong{Univariable screening}: Fits separate models for each 
    predictor (outcome ~ predictor). Each predictor is tested independently 
    to understand crude associations.
  \item \strong{Variable selection}: Based on the \code{method} parameter:
    \itemize{
      \item \code{"screen"}: Automatically selects predictors with univariable 
        p \eqn{\le} \code{p_threshold}
      \item \code{"all"}: Includes all predictors (no selection)
      \item \code{"custom"}: Uses predictors specified in \code{multi_predictors}
    }
  \item \strong{Multivariable modeling}: Fits a single model with selected 
    predictors (outcome ~ predictor1 + predictor2 + ...). Estimates are 
    adjusted for all other variables in the model.
  \item \strong{Output formatting}: Combines results into publication-ready 
    table with appropriate effect measures and formatting.
}

\strong{Variable Selection Strategies:}

\emph{"Screen" Method} (\code{method = "screen"}):
\itemize{
  \item Uses \emph{p}-value threshold for automatic selection
  \item Liberal thresholds (\emph{e.g.,} 0.20) cast a wide net to avoid missing 
    important predictors
  \item Stricter thresholds (\emph{e.g.,} 0.05) focus on strongly associated predictors
  \item Helps reduce overfitting and multicollinearity
  \item Common in exploratory analyses and when sample size is limited
}

\emph{"All" Method} (\code{method = "all"}):
\itemize{
  \item No variable selection; includes all predictors
  \item Appropriate when all variables are theoretically important
  \item Risk of overfitting with many predictors relative to sample size
  \item Useful for confirmatory analyses with pre-specified models
}

\emph{"Custom" Method} (\code{method = "custom"}):
\itemize{
  \item Manual selection based on subject matter knowledge
  \item Runs univariable analysis for all predictors (for comparison)
  \item Includes only specified predictors in multivariable model
  \item Ideal for theory-driven model building
  \item Allows comparison of unadjusted vs adjusted effects for all variables
}

\strong{Interpreting Results:}

When \code{columns = "both"} (default), tables show:
\itemize{
  \item \strong{Univariable columns}: Crude associations, unadjusted for 
    other variables. Labeled as "OR/HR/RR/Coefficient (95\% CI)" and "Uni \emph{p}"
  \item \strong{Multivariable columns}: Adjusted associations, accounting 
    for all other predictors in the model. Labeled as "aOR/aHR/aRR/Adj. Coefficient 
    (95\% CI)" and "Multi \emph{p}" ("a" = adjusted)
  \item Variables not meeting selection criteria show "-" in multivariable columns
}

Comparing univariable and multivariable results helps identify:
\itemize{
  \item \strong{Confounding}: Large changes in effect estimates
  \item \strong{Independent effects}: Similar univariable and multivariable 
    estimates
  \item \strong{Mediation}: Attenuated effects in multivariable model
  \item \strong{Suppression}: Effects that emerge only after adjustment
}

\strong{Sample Size Considerations:}

Rule of thumb for multivariable models:
\itemize{
  \item \strong{Logistic regression}: \eqn{\ge} 10 events per predictor variable
  \item \strong{Cox regression}:  \eqn{\ge} 10 events per predictor variable
  \item \strong{Linear regression}: \eqn{\ge} 10-20 observations per predictor
}

Use screening methods to reduce predictor count when these ratios are not met.
}
\examples{
# Load example data
data(clintrial)
data(clintrial_labels)

# Example 1: Basic screening with p < 0.05 threshold
result1 <- fullfit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "bmi", "smoking",
                   "hypertension", "diabetes",
                   "treatment", "stage"),
    method = "screen",
    p_threshold = 0.05,
    labels = clintrial_labels
)
print(result1)

\donttest{

# Example 2: Include all predictors (no selection)
result2 <- fullfit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "treatment", "stage"),
    method = "all",
    labels = clintrial_labels
)
print(result2)

# Example 3: Custom variable selection
result3 <- fullfit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "bmi", "smoking", "treatment", "stage"),
    method = "custom",
    multi_predictors = c("age", "treatment", "stage"),
    labels = clintrial_labels
)
print(result3)

# Example 4: Cox regression with screening
library(survival)
cox_result <- fullfit(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    predictors = c("age", "sex", "treatment", "stage"),
    model_type = "coxph",
    method = "screen",
    p_threshold = 0.10,
    labels = clintrial_labels
)
print(cox_result)

# Example 5: Linear regression without screening
linear_result <- fullfit(
    data = clintrial,
    outcome = "bmi",
    predictors = c("age", "sex", "smoking", "creatinine"),
    model_type = "lm",
    method = "all",
    labels = clintrial_labels
)
print(linear_result)

# Example 6: Poisson regression for count outcomes
poisson_result <- fullfit(
    data = clintrial,
    outcome = "fu_count",
    predictors = c("age", "stage", "treatment", "surgery"),
    model_type = "glm",
    family = "poisson",
    method = "all",
    labels = clintrial_labels
)
print(poisson_result)

# Example 7: Show only multivariable results
multi_only <- fullfit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "treatment", "stage"),
    method = "all",
    columns = "multi",
    labels = clintrial_labels
)
print(multi_only)

# Example 8: Return both table and model object
both <- fullfit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "treatment", "stage"),
    method = "all",
    return_type = "both",
    labels = clintrial_labels
)
print(both$table)
summary(both$model)

# Example 9: Keep univariable models for diagnostics
with_models <- fullfit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "bmi", "creatinine"),
    keep_models = TRUE
)
uni_results <- attr(with_models, "uni_results")
uni_models <- attr(uni_results, "models")
summary(uni_models[["age"]])

# Example 10: Linear mixed effects with site clustering
if (requireNamespace("lme4", quietly = TRUE)) {
    lmer_result <- fullfit(
        data = clintrial,
        outcome = "los_days",
        predictors = c("age", "treatment", "surgery", "stage"),
        random = "(1|site)",
        model_type = "lmer",
        method = "all",
        labels = clintrial_labels
    )
    print(lmer_result)
}

}

}
\seealso{
\code{\link{uniscreen}} for univariable screening only,
\code{\link{fit}} for fitting a single multivariable model,
\code{\link{compfit}} for comparing multiple models,
\code{\link{desctable}} for descriptive statistics,
\code{\link{autoforest}} for forest plots of the multivariable models,
\code{\link{forestsave}} for exporting forest plots,
\code{\link{tablesave}} for exporting tables

Other regression functions:
\code{\link[=compfit]{compfit()}},
\code{\link[=fit]{fit()}},
\code{\link[=multifit]{multifit()}},
\code{\link[=print.compfit_result]{print.compfit_result()}},
\code{\link[=print.fit_result]{print.fit_result()}},
\code{\link[=print.fullfit_result]{print.fullfit_result()}},
\code{\link[=print.multifit_result]{print.multifit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fullfit.R
\name{format_fullfit_combined}
\alias{format_fullfit_combined}
\title{Format combined fullfit output from formatted tables}
\usage{
format_fullfit_combined(
  uni_formatted,
  multi_formatted,
  uni_raw,
  multi_raw,
  predictors,
  columns,
  metrics,
  show_n,
  show_events,
  labels,
  exponentiate = NULL,
  conf_level = 0.95
)
}
\arguments{
\item{uni_formatted}{Formatted data.table from univariable screening.}

\item{multi_formatted}{Formatted data.table from multivariable model.}

\item{uni_raw}{Raw data.table with univariable coefficients.}

\item{multi_raw}{Raw data.table with multivariable coefficients.}

\item{predictors}{Character vector of all predictor names.}

\item{columns}{Character string specifying columns to show ("both", "uni", "multi").}

\item{metrics}{Character vector specifying metrics to show ("effect", "p").}

\item{show_n}{Logical whether to include sample size column.}

\item{show_events}{Logical whether to include events column.}

\item{labels}{Optional named character vector of variable labels.}

\item{exponentiate}{Optional logical for coefficient exponentiation.}

\item{conf_level}{Numeric confidence level for CI label.}
}
\value{
Combined data.table with aligned univariable and multivariable results.
}
\description{
Merges univariable and multivariable results into a single publication-ready
table with side-by-side display. Uses vectorized merge instead of per-variable loops.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_event_variable}
\alias{get_event_variable}
\title{Extract event variable from survival model}
\usage{
get_event_variable(model, model_class)
}
\arguments{
\item{model}{Fitted survival model object.}

\item{model_class}{Character string of the model class.}
}
\value{
Character string naming the event variable, or \code{NULL} if not found.
}
\description{
Parses the Surv() expression in survival model formulas to extract
the event/status variable name. Works with coxph, clogit, and coxme models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{calculate_table_width}
\alias{calculate_table_width}
\title{Calculate table width based on paper size and orientation}
\usage{
calculate_table_width(paper, orientation)
}
\arguments{
\item{paper}{Character string paper size ("letter", "a4", "legal").}

\item{orientation}{Character string page orientation ("portrait", "landscape").}
}
\value{
Numeric usable width in inches.
}
\description{
Computes usable table width in inches based on paper dimensions and
orientation, accounting for standard 1-inch margins.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{reorder_total_column}
\alias{reorder_total_column}
\title{Reorder columns to position total column}
\usage{
reorder_total_column(result, total, total_label)
}
\arguments{
\item{result}{Data.table with columns to reorder.}

\item{total}{Logical or character: \code{TRUE}/"first" (total first), "last" (total last).}

\item{total_label}{Character string name of the total column.}
}
\value{
Modified data.table with reordered columns.
}
\description{
Rearranges data.table columns to place the total column in the specified
position (first, last, or default). Ensures proper ordering of Variable,
Group, total, group columns, and \emph{p}-value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{detect_outcome_type}
\alias{detect_outcome_type}
\title{Detect outcome type from data}
\usage{
detect_outcome_type(data, outcome)
}
\arguments{
\item{data}{Data frame or data.table containing the outcome variable.}

\item{outcome}{Character string naming the outcome variable.}
}
\value{
Character string: "binary", "continuous", "count", or "unknown".
}
\description{
Automatically determines whether an outcome variable is binary, continuous,
or count-based by examining the data values. Used for automatic model type
selection and validation. Binary outcomes have 2 unique values, continuous
have many values or non-integers, counts have integers >= 0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forestsave.R
\name{cairo_device}
\alias{cairo_device}
\title{Resolve the Cairo device for an output format}
\usage{
cairo_device(file)
}
\arguments{
\item{file}{Character string giving the output path.}
}
\value{
A device function.
}
\description{
Maps the \code{"cairo"} shorthand accepted by \code{forestsave()} onto the
Cairo-backed device for the requested format. Cairo's raster backends are
superseded by \pkg{ragg} and are not offered, so the shorthand applies to
vector formats only.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{multifit}
\alias{multifit}
\title{Multivariate Regression Analysis}
\usage{
multifit(
  data,
  outcomes,
  predictor,
  covariates = NULL,
  interactions = NULL,
  random = NULL,
  strata = NULL,
  cluster = NULL,
  model_type = "glm",
  family = "binomial",
  columns = "adjusted",
  p_threshold = 1,
  conf_level = 0.95,
  show_n = TRUE,
  show_events = TRUE,
  digits = 2,
  p_digits = 3,
  labels = NULL,
  predictor_label = NULL,
  include_predictor = TRUE,
  keep_models = FALSE,
  exponentiate = NULL,
  conf_method = NULL,
  parallel = TRUE,
  n_cores = NULL,
  number_format = NULL,
  verbose = NULL,
  ...
)
}
\arguments{
\item{data}{Data frame or data.table containing the analysis dataset. The 
function automatically converts data frames to data.tables for efficient 
processing.}

\item{outcomes}{Character vector of outcome variable names to analyze. Each 
outcome is tested in its own model with the predictor. For time-to-event
analysis, use \code{Surv()} syntax for the outcome variable
(\emph{e.g.,} \code{c("Surv(time1, status1)", "Surv(time2, status2)")}).}

\item{predictor}{Character string specifying the predictor (exposure) variable 
name. This variable is tested against each outcome. Can be continuous or 
categorical (factor).}

\item{covariates}{Optional character vector of covariate variable names to 
include in adjusted models. When specified, models are fit as 
\code{outcome ~ predictor + covariate1 + covariate2 + ...}, and only the 
predictor effect is reported. Default is \code{NULL} (unadjusted models).}

\item{interactions}{Optional character vector of interaction terms to include
in adjusted models, using colon notation (\emph{e.g.,} \code{c("predictor:sex")}).
Interactions involving the predictor will have their effects extracted and
reported. Default is \code{NULL}.}

\item{random}{Optional character string specifying random effects formula for
mixed effects models (\emph{e.g.,} \code{"(1|hospital)"} or \code{"(1|site/patient)"}).
Required when \code{model_type} is \code{"glmer"}, \code{"lmer"}, or 
\code{"coxme"} unless random effects are included in the \code{covariates}
vector. Alternatively, random effects can be included directly in the 
\code{covariates} vector using the same syntax (\emph{e.g.,} 
\code{covariates = c("age", "sex", "(1|site)")}). Default is \code{NULL}.}

\item{strata}{Optional character string naming the stratification variable for
Cox or conditional logistic models. Creates separate baseline hazards for 
each stratum. Default is \code{NULL}.}

\item{cluster}{Optional character string naming the clustering variable for
Cox models. Computes robust clustered standard errors. Default is \code{NULL}.}

\item{model_type}{Character string specifying the type of regression model to 
fit. Options include:
\itemize{
  \item \code{"glm"} - Generalized linear model (default). Supports multiple 
    distributions via the \code{family} parameter including logistic, Poisson, 
    Gamma, Gaussian, and quasi-likelihood models.
  \item \code{"lm"} - Linear regression for continuous outcomes with normally 
    distributed errors.
  \item \code{"coxph"} - Cox proportional hazards model for time-to-event 
    survival analysis. Requires \code{Surv()} outcome syntax.
  \item \code{"clogit"} - Conditional logistic regression for matched 
    case-control studies.
  \item \code{"negbin"} - Negative binomial regression for overdispersed count 
    data (requires MASS package). Estimates an additional dispersion parameter 
    compared to Poisson regression.
  \item \code{"glmer"} - Generalized linear mixed-effects model for hierarchical 
    or clustered data with non-normal outcomes (requires \pkg{lme4} package and 
    \code{random} parameter).
  \item \code{"lmer"} - Linear mixed-effects model for hierarchical or clustered 
    data with continuous outcomes (requires \pkg{lme4} package and \code{random} 
    parameter).
  \item \code{"coxme"} - Cox mixed-effects model for clustered survival data 
    (requires coxme package and \code{random} parameter).
}}

\item{family}{For GLM and GLMER models, specifies the error distribution and link 
function. Can be a character string, a family function, or a family object.
Ignored for non-GLM/GLMER models.

\strong{Binary/Binomial outcomes:}
\itemize{
  \item \code{"binomial"} or \code{binomial()} - Logistic regression for binary 
    outcomes (0/1, TRUE/FALSE). Returns odds ratios (OR). Default.
  \item \code{"quasibinomial"} or \code{quasibinomial()} - Logistic regression 
    with overdispersion. Use when residual deviance >> residual df.
  \item \code{binomial(link = "probit")} - Probit regression (normal CDF link).
  \item \code{binomial(link = "cloglog")} - Complementary log-log link for 
    asymmetric binary outcomes.
}

\strong{Count outcomes:}
\itemize{
  \item \code{"poisson"} or \code{poisson()} - Poisson regression for count 
    data. Returns rate ratios (RR). Assumes mean = variance.
  \item \code{"quasipoisson"} or \code{quasipoisson()} - Poisson regression 
    with overdispersion. Use when variance > mean.
}

\strong{Continuous outcomes:}
\itemize{
  \item \code{"gaussian"} or \code{gaussian()} - Normal/Gaussian distribution 
    for continuous outcomes. Equivalent to linear regression.
  \item \code{gaussian(link = "log")} - Log-linear model for positive continuous 
    outcomes. Returns multiplicative effects.
}

\strong{Positive continuous outcomes:}
\itemize{
  \item \code{"Gamma"} or \code{Gamma()} - Gamma distribution for positive, 
    right-skewed continuous data (\emph{e.g.,} costs, lengths of stay). When
    passed as a string, resolves to log link for interpretable multiplicative
    effects.
  \item \code{Gamma(link = "inverse")} - Gamma with inverse (canonical) link.
  \item \code{Gamma(link = "identity")} - Gamma with identity link for additive 
    effects on positive outcomes.
  \item \code{"inverse.gaussian"} or \code{inverse.gaussian()} - Inverse Gaussian 
    for positive, highly right-skewed data.
}

For negative binomial regression (overdispersed counts), use 
\code{model_type = "negbin"} instead of the \code{family} parameter.

See \code{\link[stats]{family}} for additional details and options.}

\item{columns}{Character string specifying which result columns to display when 
both unadjusted and adjusted models are fit (\emph{i.e.,} when \code{covariates} is 
specified):
\itemize{
  \item \code{"adjusted"} - Show only adjusted (covariate-controlled) results [default]
  \item \code{"unadjusted"} - Show only unadjusted (crude) results
  \item \code{"both"} - Show both unadjusted and adjusted results side-by-side
}
Ignored when \code{covariates = NULL}.}

\item{p_threshold}{Numeric value between 0 and 1 specifying a \emph{p}-value threshold 
for filtering results. Only outcomes with \emph{p}-value less than or equal to the
threshold are included in the output. Default is 1 (no filtering; all outcomes returned).}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be 
between 0 and 1. Default is 0.95 (95\% confidence intervals).}

\item{show_n}{Logical. If \code{TRUE}, includes the sample size column in 
the output table. Default is \code{TRUE}.}

\item{show_events}{Logical. If \code{TRUE}, includes the events column in the 
output table (relevant for survival and logistic regression). Default is 
\code{TRUE}.}

\item{digits}{Integer specifying the number of decimal places for effect 
estimates (OR, HR, RR, coefficients). Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{labels}{Named character vector or list providing custom display 
labels for variables. Can include labels for outcomes, predictors, and 
covariates. Names should match variable names, values are the display labels.
Labels are applied to: (1) outcome names in the Outcome column, (2) predictor
variable name when displayed, and (3) variable names in formatted interaction
terms. Variables not in \code{labels} use their original names. 
Default is \code{NULL}.}

\item{predictor_label}{Optional character string providing a custom display 
label for the predictor variable. Takes precedence over \code{labels} for
the predictor. Default is \code{NULL} (uses label from \code{labels} or 
original name).}

\item{include_predictor}{Logical. If \code{TRUE} (default), includes the 
predictor column in the output table showing which level of a factor 
predictor is being compared. If \code{FALSE}, omits the predictor column,
which may be useful when the predictor information will be explained in 
a table caption or figure legend.}

\item{keep_models}{Logical. If \code{TRUE}, stores all fitted model objects 
in the output as an attribute. Models are accessible via 
\code{attr(result, "models")}. Default is \code{FALSE}.}

\item{exponentiate}{Logical. Whether to exponentiate coefficients (display
OR/HR/RR instead of log odds/log hazards). Default is \code{NULL}, which
automatically displays raw coefficients for linear models and
exponentiates for logistic, Poisson, and Cox models.}

\item{conf_method}{Character string controlling the confidence interval method.
If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
\itemize{
  \item \code{"profile"} - Profile likelihood intervals for GLM and negative
    binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
    intervals for linear models. Falls back to Wald on profiling failure.
    Quasi-likelihood families always use Wald (no true likelihood).
  \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
    \eqn{\times} SE) for all model types. Faster but less accurate near
    boundary conditions or with small subgroups.
}
Cox and mixed-effects models use Wald intervals regardless of this setting.
Set globally with \code{options(summata.conf_method = "wald")} to use Wald
throughout a session.}

\item{parallel}{Logical. If \code{TRUE} (default), fits models in parallel
for improved performance with many outcomes.}

\item{n_cores}{Integer specifying the number of CPU cores to use for
parallel processing. Default is \code{NULL} (auto-detect: uses
\code{detectCores() - 1}). Ignored when \code{parallel = FALSE}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}

\item{verbose}{Logical. If \code{TRUE}, displays model fitting warnings
(\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
routine fitting messages are suppressed while unexpected warnings are
preserved. When \code{NULL}, uses
\code{getOption("summata.verbose", FALSE)}.}

\item{...}{Additional arguments passed to the underlying model fitting functions.}
}
\value{
A data.table with S3 class \code{"multifit_result"} containing formatted 
  multivariate regression results. The table structure includes:
  \describe{
    \item{Outcome}{Character. Outcome variable name or custom label}
    \item{Predictor}{Character. For factor predictors: formatted as 
      "Variable (Level)" showing the level being compared to reference.
      For binary variables where the non-reference level is an affirmative
      value (Yes, 1, True, Present, Positive, +), shows just "Variable".
      For continuous predictors: the variable name. For interactions: the
      formatted interaction term (\emph{e.g.,} "Treatment (Drug A) × Sex (Male)")}
    \item{n}{Integer. Sample size used in fitting (if \code{show_n = TRUE}).
      When both unadjusted and adjusted estimates are shown, this is the
      adjusted model's sample, which is the one behind the adjusted
      estimate on the same row}
    \item{Events}{Integer. Number of events (if \code{show_events = TRUE})}
    \item{OR/HR/RR/Coefficient (95\% CI)}{Character. Unadjusted effect 
      estimate with CI (if \code{columns = "unadjusted"} or \code{"both"})}
    \item{aOR/aHR/aRR/Adj. Coefficient (95\% CI)}{Character. Adjusted 
      effect estimate with CI (if \code{columns = "adjusted"} or \code{"both"})}
    \item{Uni \emph{p} / Multi \emph{p} / \emph{p}-value}{Character. Formatted \emph{p}-value(s). Column 
      names depend on \code{columns} setting}
  }
  
  The returned object includes the following attributes accessible via \code{attr()}:
  \describe{
    \item{raw_data}{data.table. Unformatted numeric results with separate 
      columns for effect estimates, standard errors, confidence intervals, 
      and \emph{p}-values. Suitable for custom analysis or visualization}
    \item{models}{list (if \code{keep_models = TRUE}). Named list of fitted 
      model objects, with outcome names as list names. Each element contains
      \code{$unadjusted} and/or \code{$adjusted} models depending on settings}
    \item{predictor}{Character. The predictor variable name}
    \item{outcomes}{Character vector. The outcome variable names}
    \item{covariates}{Character vector or \code{NULL}. The covariate variable names}
    \item{interactions}{Character vector or \code{NULL}. The interaction terms}
    \item{random}{Character or \code{NULL}. The random effects formula}
    \item{strata}{Character or \code{NULL}. The stratification variable}
    \item{cluster}{Character or \code{NULL}. The clustering variable}
    \item{model_type}{Character. The regression model type used}
    \item{columns}{Character. Which columns were displayed}
    \item{analysis_type}{Character. \code{"multi_outcome"} to identify analysis type}
    \item{significant}{Character vector. Names of outcomes with \emph{p} < 0.05
      for the predictor (uses adjusted \emph{p}-values when available)}
    \item{analysis_counts}{List. How much of the supplied data reached the
      models, with elements \code{n_supplied}, \code{n_analyzed},
      \code{n_missing_outcome} and \code{n_missing_predictor}. One model is
      fitted per outcome, so \code{n_analyzed} holds one value per outcome
      and the reported sample may be a range. Event counts are not
      included: each outcome has its own event total, so a single reported
      figure could not describe them all. The \code{print()} method always
      reports the analyzed sample}
  }
}
\description{
Performs regression analyses of a single predictor (exposure) across multiple 
outcomes. This function is designed for studies where a single exposure variable 
is tested against multiple endpoints, such as complication screening, biomarker 
associations, or phenome-wide association studies. Returns publication-ready 
formatted results with optional covariate adjustment. Supports interactions,
mixed-effects models, stratification, and clustered standard errors.
}
\details{
\strong{Analysis Approach:}

The function implements a multivariate (multi-outcome) screening workflow that
inverts the typical regression paradigm:
\enumerate{
  \item For each outcome in \code{outcomes}, fits a separate model with the 
    predictor as the main exposure
  \item If \code{covariates} specified, fits adjusted model: 
    \code{outcome ~ predictor + covariates + interactions}
  \item Extracts only the predictor effect(s) from each model, ignoring 
    covariate coefficients
  \item Combines results into a single table for comparison across outcomes
  \item Optionally filters by \emph{p}-value threshold
}

This is conceptually opposite to \code{uniscreen()}, which tests multiple 
predictors against a single outcome. \code{multifit()} applies where there is one 
exposure of interest and want to screen across multiple endpoints.

\strong{When to Use Multivariate Regression Analysis:}
\itemize{
  \item \strong{Complication screening}: Test one exposure (\emph{e.g.,} operative time, 
    BMI, biomarker level) against multiple postoperative complications
  \item \strong{Treatment effects}: Test one treatment against multiple efficacy 
    and safety endpoints simultaneously
  \item \strong{Biomarker studies}: Test one biomarker against multiple clinical 
    outcomes to understand its prognostic value
  \item \strong{Phenome-wide association studies (PheWAS)}: Test genetic variants 
    or exposures against many phenotypes
  \item \strong{Risk factor profiling}: Understand how one risk factor relates 
    to a spectrum of outcomes
}

\strong{Handling Categorical Predictors:}

When the predictor is a factor variable with multiple levels:
\itemize{
  \item Each non-reference level gets its own row for each outcome
  \item Reference category is determined by factor level ordering
  \item The Predictor column shows "Variable (Level)" format 
    (\emph{e.g.,} "Treatment (Drug A)", "Treatment (Drug B)")
  \item For binary variables with affirmative non-reference levels 
    (Yes, 1, True, Present, Positive, +), shows just "Variable" 
    (\emph{e.g.,} "Diabetes" instead of "Diabetes (Yes)")
  \item Effect estimates compare each level to the reference
}

\strong{Adjusted vs. Unadjusted Results:}

When \code{covariates} is specified, the function fits both models but only 
extracts predictor effects:
\itemize{
  \item \code{columns = "adjusted"}: Reports only covariate-adjusted effects.
    Column labeled "aOR/aHR," \emph{etc.}
  \item \code{columns = "unadjusted"}: Reports only crude effects. Column 
    labeled "OR/HR," \emph{etc.}
  \item \code{columns = "both"}: Reports both side-by-side. Useful for 
    identifying confounding (large change in effect) or independent effects 
    (similar estimates)
}

\strong{Interaction Terms:}

When \code{interactions} includes terms involving the predictor:
\itemize{
  \item Main effect of predictor is always reported
  \item Interaction effects are extracted and displayed with formatted names
  \item Format: \code{Variable (Level) × Variable (Level)} using multiplication sign notation
  \item Useful for testing effect modification (\emph{e.g.,} does treatment effect 
    differ by sex?)
}

\strong{Mixed-Effects Models:}

For clustered or hierarchical data (\emph{e.g.,} patients within hospitals):
\itemize{
  \item Use \code{model_type = "glmer"} with \code{random = "(1|cluster)"} for 
    random intercept models
  \item Nested random effects: \code{random = "(1|site/patient)"}
  \item Crossed random effects: \code{random = "(1|site) + (1|doctor)"}
  \item For survival outcomes, use \code{model_type = "coxme"}
}

\strong{Stratification and Clustering (Cox models):}

For Cox proportional hazards models:
\itemize{
  \item \code{strata}: Creates separate baseline hazards for each stratum level.
    Use when hazards are non-proportional across strata but stratum effects do
    not need to be estimated
  \item \code{cluster}: Computes robust (sandwich) standard errors accounting 
    for within-cluster correlation. Alternative to mixed effects when only 
    robust SEs are needed
}

\strong{Filtering based on \emph{p}-value:}

The \code{p_threshold} parameter filters results after fitting all models:
\itemize{
  \item Only outcomes with \emph{p} less than or equal to the threshold are retained in output
  \item For factor predictors, outcome is kept if any level is significant
  \item Useful for focusing on significant associations in exploratory analyses
  \item Default is 1 (no filtering) - recommended for confirmatory analyses
}

\strong{Outcome Homogeneity:}

All outcomes in a single \code{multifit()} call should be of the same type
(all binary, all continuous, or all survival). Mixing outcome types produces
tables with incompatible effect measures (\emph{e.g.,} odds ratios alongside regression
coefficients), which can mislead readers. The function validates outcome
compatibility and issues a warning when mixed types are detected.

For analyses involving multiple outcome types, run separate \code{multifit()}
calls for each type:

\preformatted{
# Binary outcomes
binary_results <- multifit(data, outcomes = c("death", "readmission"),
                           predictor = "treatment", model_type = "glm")

# Continuous outcomes
continuous_results <- multifit(data, outcomes = c("los_days", "cost"),
                               predictor = "treatment", model_type = "lm")
}

\strong{Effect Measures by Model Type:}
\itemize{
  \item \strong{Logistic} (\code{model_type = "glm"}, \code{family = "binomial"}): 
    Odds ratios (OR/aOR)
  \item \strong{Cox} (\code{model_type = "coxph"}): Hazard ratios (HR/aHR)
  \item \strong{Poisson} (\code{model_type = "glm"}, \code{family = "poisson"}): 
    Rate ratios (RR/aRR)
  \item \strong{Linear} (\code{model_type = "lm"}): Coefficient estimates
  \item \strong{Mixed effects}: Same as fixed-effects counterparts
}

\strong{Memory and Performance:}

\itemize{
  \item \code{parallel = TRUE} (default) uses multiple cores for faster fitting
  \item \code{keep_models = FALSE} (default) discards model objects to save memory
  \item For many outcomes, parallel processing provides substantial speedup
  \item Set \code{keep_models = TRUE} only when model diagnostics are required
}
}
\examples{
# Load example data
data(clintrial)
data(clintrial_labels)

# Example 1: Basic multivariate analysis (unadjusted)
result1 <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    labels = clintrial_labels,
    parallel = FALSE
)
print(result1)

\donttest{

# Example 2: Adjusted analysis with covariates
result2 <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    labels = clintrial_labels,
    parallel = FALSE
)
print(result2)

# Example 3: Compare unadjusted and adjusted results
result3 <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    columns = "both",
    labels = clintrial_labels,
    parallel = FALSE
)
print(result3)

# Example 4: Continuous predictor across outcomes
result4 <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "age",
    covariates = c("sex", "treatment", "stage"),
    labels = clintrial_labels,
    parallel = FALSE
)
print(result4)

# Example 5: Cox regression for survival outcomes
library(survival)
cox_result <- multifit(
    data = clintrial,
    outcomes = c("Surv(pfs_months, pfs_status)", 
                 "Surv(os_months, os_status)"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    model_type = "coxph",
    labels = clintrial_labels,
    parallel = FALSE
)
print(cox_result)

# Example 6: Cox with stratification by site
cox_strat <- multifit(
    data = clintrial,
    outcomes = c("Surv(os_months, os_status)"),
    predictor = "treatment",
    covariates = c("age", "sex"),
    strata = "site",
    model_type = "coxph",
    labels = clintrial_labels,
    parallel = FALSE
)
print(cox_strat)

# Example 7: Cox with clustered standard errors
cox_cluster <- multifit(
    data = clintrial,
    outcomes = c("Surv(os_months, os_status)"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    cluster = "site",
    model_type = "coxph",
    labels = clintrial_labels,
    parallel = FALSE
)
print(cox_cluster)

# Example 8: Interaction between predictor and covariate
result_int <- multifit(
    data = clintrial,
    outcomes = c("surgery", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    interactions = c("treatment:sex"),
    labels = clintrial_labels,
    parallel = FALSE
)
print(result_int)

# Example 9: Linear model for continuous outcomes
linear_result <- multifit(
    data = clintrial,
    outcomes = c("los_days", "biomarker_x"),
    predictor = "treatment",
    covariates = c("age", "sex"),
    model_type = "lm",
    labels = clintrial_labels,
    parallel = FALSE
)
print(linear_result)

# Example 10: Poisson regression for equidispersed count outcomes
# fu_count has variance ~= mean, appropriate for standard Poisson
poisson_result <- multifit(
    data = clintrial,
    outcomes = c("fu_count"),
    predictor = "treatment",
    covariates = c("age", "stage"),
    model_type = "glm",
    family = "poisson",
    labels = clintrial_labels,
    parallel = FALSE
)
print(poisson_result)

# Example 11: Filter to significant results only
sig_results <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "stage",
    p_threshold = 0.05,
    labels = clintrial_labels,
    parallel = FALSE
)
print(sig_results)

# Example 12: Custom outcome labels
result_labeled <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    labels = c(
        surgery = "Surgical Resection",
        pfs_status = "Disease Progression",
        os_status = "Death",
        treatment = "Treatment Group"
    ),
    parallel = FALSE
)
print(result_labeled)

# Example 13: Keep models for diagnostics
result_models <- multifit(
    data = clintrial,
    outcomes = c("surgery", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex"),
    keep_models = TRUE,
    parallel = FALSE
)

# Access stored models
models <- attr(result_models, "models")
names(models)

# Get adjusted model for surgery outcome
surgery_model <- models$surgery$adjusted
summary(surgery_model)

# Example 14: Access raw numeric data
result <- multifit(
    data = clintrial,
    outcomes = c("surgery", "os_status"),
    predictor = "age",
    parallel = FALSE
)

# Get unformatted results for custom analysis
raw_data <- attr(result, "raw_data")
print(raw_data)

# Example 15: Hide sample size and event columns
result_minimal <- multifit(
    data = clintrial,
    outcomes = c("surgery", "os_status"),
    predictor = "treatment",
    show_n = FALSE,
    show_events = FALSE,
    labels = clintrial_labels,
    parallel = FALSE
)
print(result_minimal)

# Example 16: Customize decimal places
result_digits <- multifit(
    data = clintrial,
    outcomes = c("surgery", "os_status"),
    predictor = "age",
    digits = 3,
    p_digits = 4,
    labels = clintrial_labels,
    parallel = FALSE
)
print(result_digits)

# Example 17: Force coefficient display (no exponentiation)
result_coef <- multifit(
    data = clintrial,
    outcomes = c("surgery"),
    predictor = "age",
    exponentiate = FALSE,
    labels = clintrial_labels,
    parallel = FALSE
)
print(result_coef)

# Example 18: Complete publication workflow
final_table <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage", "grade"),
    columns = "both",
    labels = clintrial_labels,
    digits = 2,
    p_digits = 3,
    parallel = FALSE
)
print(final_table)

# Example 19: Gamma regression for positive continuous outcomes
gamma_result <- multifit(
    data = clintrial,
    outcomes = c("los_days", "recovery_days"),
    predictor = "treatment",
    covariates = c("age", "surgery"),
    model_type = "glm",
    family = Gamma(link = "log"),
    labels = clintrial_labels,
    parallel = FALSE
)
print(gamma_result)

# Example 20: Quasipoisson for overdispersed counts
quasi_result <- multifit(
    data = clintrial,
    outcomes = c("ae_count"),
    predictor = "treatment",
    covariates = c("age", "diabetes"),
    model_type = "glm",
    family = "quasipoisson",
    labels = clintrial_labels,
    parallel = FALSE
)
print(quasi_result)

# Example 21: Generalized linear mixed effects (GLMER)
# Test treatment across outcomes with site clustering
if (requireNamespace("lme4", quietly = TRUE)) {
    glmer_result <- suppressWarnings(multifit(
        data = clintrial,
        outcomes = c("surgery", "pfs_status", "os_status"),
        predictor = "treatment",
        covariates = c("age", "sex"),
        random = "(1|site)",
        model_type = "glmer",
        family = "binomial",
        labels = clintrial_labels,
        parallel = FALSE
    ))
    print(glmer_result)
}

# Example 22: Cox mixed effects with random site effects
if (requireNamespace("coxme", quietly = TRUE)) {
    coxme_result <- multifit(
        data = clintrial,
        outcomes = c("Surv(pfs_months, pfs_status)", 
                     "Surv(os_months, os_status)"),
        predictor = "treatment",
        covariates = c("age", "sex", "stage"),
        random = "(1|site)",
        model_type = "coxme",
        labels = clintrial_labels,
        parallel = FALSE
    )
    print(coxme_result)
}

# Example 23: Multiple interactions across outcomes
multi_int <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    interactions = c("treatment:stage", "treatment:sex"),
    labels = clintrial_labels,
    parallel = FALSE
)
print(multi_int)

}

}
\seealso{
\code{\link{uniscreen}} for screening multiple predictors against one outcome,
\code{\link{multiforest}} for creating forest plots from multifit results,
\code{\link{fit}} for single-outcome regression with full coefficient output,
\code{\link{fullfit}} for complete univariable-to-multivariable workflow,
\code{\link{forestsave}} for saving the forest plot,
\code{\link{tablesave}} for exporting tables

Other regression functions:
\code{\link[=compfit]{compfit()}},
\code{\link[=fit]{fit()}},
\code{\link[=fullfit]{fullfit()}},
\code{\link[=print.compfit_result]{print.compfit_result()}},
\code{\link[=print.fit_result]{print.fit_result()}},
\code{\link[=print.fullfit_result]{print.fullfit_result()}},
\code{\link[=print.multifit_result]{print.multifit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{find_non_reference_row}
\alias{find_non_reference_row}
\title{Find non-reference row for binary variable condensing}
\usage{
find_non_reference_row(var_rows, estimate_col = "estimate")
}
\arguments{
\item{var_rows}{Data.table containing rows for a single variable.}

\item{estimate_col}{Character string naming the estimate column 
(\emph{e.g.,} "estimate", "coef"). Default is "estimate".}
}
\value{
Integer index of the non-reference row within var_rows, or \code{NULL} if
  cannot be determined (\emph{e.g.,} no NA estimates found, or multiple non-NA rows).
}
\description{
Identifies the non-reference row in a binary categorical variable by checking
for NA estimates (reference rows have NA). This is more robust than assuming
row position or matching specific strings like "Yes" or "Positive".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_column_headers_html}
\alias{format_column_headers_html}
\title{Apply formatting to column headers in exported tables (HTML)}
\usage{
format_column_headers_html(col_names)
}
\arguments{
\item{col_names}{Character vector of column names.}
}
\value{
Character vector with HTML-formatted column names.
}
\description{
Formats column headers for HTML output by italicizing 'n' and 'p'
using HTML tags.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{clintrial_labels}
\alias{clintrial_labels}
\title{Variable Labels for Clinical Trial Dataset}
\format{
Named character vector with 24 elements
}
\usage{
clintrial_labels
}
\description{
A named character vector providing descriptive labels for all variables
in the clinical_trial dataset. Use with labels parameter in functions.
}
\seealso{
\code{\link{clintrial}}

Other sample data:
\code{\link{clintrial}}
}
\concept{sample data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survtable.R
\name{survtable}
\alias{survtable}
\title{Create Publication-Ready Survival Summary Tables}
\usage{
survtable(
  data,
  outcome,
  by = NULL,
  times = NULL,
  probs = 0.5,
  stats = c("survival", "ci"),
  type = "survival",
  conf_level = 0.95,
  conf_type = "log",
  digits = 0,
  time_digits = 1,
  p_digits = 3,
  percent = TRUE,
  test = TRUE,
  test_type = "logrank",
  total = TRUE,
  total_label = "Total",
  time_unit = NULL,
  time_label = NULL,
  median_label = NULL,
  labels = NULL,
  by_label = NULL,
  na_rm = TRUE,
  number_format = NULL,
  ...
)
}
\arguments{
\item{data}{Data frame or data.table containing the survival dataset.
Automatically converted to a data.table for efficient processing.}

\item{outcome}{Character string or character vector specifying one or more
survival outcomes using \code{Surv()} syntax (\emph{e.g.,}
\code{"Surv(os_months, os_status)"}). When multiple outcomes are
provided, results are stacked into a single table with outcome labels
as row headers.}

\item{by}{Character string specifying the column name of the stratifying
variable for group comparisons (\emph{e.g.,} treatment arm, risk group). When
\code{NULL} (default), produces overall survival summaries only.}

\item{times}{Numeric vector of time points at which to estimate survival
probabilities. For example, \code{c(12, 24, 36)} for 1-, 2-, and 3-year
survival when time is measured in months. Default is \code{NULL}.}

\item{probs}{Numeric vector of survival probabilities for which to estimate
corresponding survival times (quantiles). Values must be between 0 and 1.
For example, \code{c(0.5)} returns median survival time,
\code{c(0.25, 0.5, 0.75)} returns quartiles. Default is \code{0.5}
(median only).}

\item{stats}{Character vector specifying which statistics to display:
\itemize{
  \item \code{"survival"} - Survival probability at specified times
  \item \code{"ci"} - Confidence interval for survival probability
  \item \code{"n_risk"} - Number at risk at each time point
  \item \code{"n_event"} - Cumulative number of events by each time
    point
}
Default is \code{c("survival", "ci")}.}

\item{type}{Character string specifying the type of probability to report:
\itemize{
  \item \code{"survival"} - Survival probability S(t) [default]
  \item \code{"risk"} - Cumulative incidence/risk 1 - S(t)
  \item \code{"cumhaz"} - Cumulative hazard -log(S(t))
}}

\item{conf_level}{Numeric confidence level for confidence intervals. Must
be between 0 and 1. Default is 0.95 (95\% confidence intervals).}

\item{conf_type}{Character string specifying the confidence interval type
for survival estimates:
\itemize{
  \item \code{"log"} - Log transformation (default, recommended)
  \item \code{"log-log"} - Log-log transformation
  \item \code{"plain"} - Linear/identity (can produce CIs outside
    [0, 1])
  \item \code{"logit"} - Logit transformation
  \item \code{"arcsin"} - Arcsin square root transformation
}}

\item{digits}{Integer specifying the number of decimal places for survival
probabilities (as percentages). Default is 0 (whole percentages).}

\item{time_digits}{Integer specifying the number of decimal places for
survival time estimates (median, quantiles). Default is 1.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{percent}{Logical. If \code{TRUE} (default), displays survival
probabilities as percentages (\emph{e.g.,} \code{"85\%"}). If \code{FALSE},
displays as proportions (\emph{e.g.,} \code{"0.85"}).}

\item{test}{Logical. If \code{TRUE} (default), performs a survival curve
comparison test and adds a \emph{p}-value column. Requires \code{by}
to be specified.}

\item{test_type}{Character string specifying the statistical test for
comparing survival curves:
\itemize{
  \item \code{"logrank"} - Log-rank test (default)
  \item \code{"wilcoxon"} - Wilcoxon (Breslow) test
  \item \code{"tarone"} - Tarone-Ware test
  \item \code{"petopeto"} - Peto-Peto test
}}

\item{total}{Logical or character string controlling the total/overall
column:
\itemize{
  \item \code{TRUE} or \code{"first"} - Include total column first
    [default]
  \item \code{"last"} - Include total column last (before
    \emph{p}-value)
  \item \code{FALSE} - Exclude total column
}}

\item{total_label}{Character string for the total/overall row label.
Default is \code{"Total"}.}

\item{time_unit}{Character string specifying the time unit for display
in column headers and labels (\emph{e.g.,} \code{"months"}, \code{"days"},
\code{"years"}). When specified, time column headers become
"\{time\} \{time_unit\}" (\emph{e.g.,} "12 months"). Default is \code{NULL}
(no unit shown).}

\item{time_label}{Character string template for time column headers when
\code{times} is specified. Use \code{"\{time\}"} as placeholder for
the time value and \code{"\{unit\}"} for the time unit. Default is
\code{"\{time\} \{unit\}"} when \code{time_unit} is specified,
otherwise just \code{"\{time\}"}.}

\item{median_label}{Character string for the median survival row label.
Default is \code{NULL}, which auto-constructs from \code{conf_level}
(\emph{e.g.,} \code{"Median (95\% CI)"} for \code{conf_level = 0.95}).}

\item{labels}{Named character vector or list providing custom display
labels. For stratified analyses, names should match levels of the
\code{by} variable. For multiple outcomes, names should match the
\code{Surv()} expressions. Default is \code{NULL}.}

\item{by_label}{Character string providing a custom label for the
stratifying variable (used in output attributes and headers).
Default is \code{NULL} (uses variable name).}

\item{na_rm}{Logical. If \code{TRUE} (default), observations with missing
values in time, status, or the stratifying variable are excluded.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}

\item{...}{Additional arguments passed to
\code{\link[survival]{survfit}}.}
}
\value{
A data.table with S3 class \code{"survtable"} containing formatted
  survival statistics. The table structure depends on parameters:

  \strong{When \code{times} is specified (survival at time points):}
  \describe{
    \item{Variable/Group}{Row identifier -- stratifying variable levels}
    \item{Time columns}{Survival statistics at each requested time point}
    \item{\emph{p}-value}{Test \emph{p}-value (if \code{test = TRUE} and
      \code{by} specified)}
  }

  \strong{When only \code{probs} is specified (survival quantiles):}
  \describe{
    \item{Variable/Group}{Row identifier -- stratifying variable levels}
    \item{Quantile columns}{Time to reach each survival probability}
    \item{\emph{p}-value}{Test \emph{p}-value (if \code{test = TRUE} and
      \code{by} specified)}
  }

  All numeric output (probabilities, times, counts, \emph{p}-values)
  respects the \code{number_format} setting for locale-appropriate
  formatting.

  The returned object includes the following attributes:
  \describe{
    \item{raw_data}{Data.table with unformatted numeric values}
    \item{survfit_objects}{List of survfit objects for each stratum}
    \item{by_variable}{The stratifying variable name}
    \item{times}{The time points requested}
    \item{probs}{The probability quantiles requested}
    \item{test_result}{Full test result object (if test performed)}
  }
}
\description{
Generates comprehensive survival summary tables with survival probabilities
at specified time points, median survival times, and optional group
comparisons with statistical testing. Designed for creating survival
summaries commonly used in clinical and epidemiological research
publications.
}
\details{
\strong{Survival Probability Estimation:}

Survival probabilities are estimated using the Kaplan-Meier method via
\code{\link[survival]{survfit}}. At each specified time point, the function
reports the estimated probability of surviving beyond that time.

\strong{Confidence Intervals:}

The default \code{"log"} transformation for confidence intervals is
recommended as it ensures intervals remain within [0, 1] and has good
statistical properties. The \code{"log-log"} transformation is also
commonly used and may perform better in the tails.

\strong{Statistical Testing:}

The log-rank test (default) tests the null hypothesis that survival curves
are identical across groups. Alternative tests weight different parts of
the survival curve:
\itemize{
  \item Log-rank: Equal weights (best for proportional hazards)
  \item Wilcoxon: Weights by number at risk (sensitive to early
    differences)
  \item Tarone-Ware: Weights by square root of number at risk
  \item Peto-Peto: Modified Wilcoxon weights
}

\strong{Formatting:}

All numeric output respects the \code{number_format} parameter.
Separators within confidence intervals adapt automatically to avoid
ambiguity:
\itemize{
  \item Survival probabilities: \code{"85\% (80\%-89\%)"} (US) or
    \code{"85\% (80\%-89\%)"} (EU, en-dash separator)
  \item Median survival: \code{"24.5 (21.2-28.9)"} (US) or
    \code{"24,5 (21,2-28,9)"} (EU)
  \item Counts \eqn{\ge} 1000: \code{"1,234"} (US) or
    \code{"1.234"} (EU)
  \item \emph{p}-values: \code{"< 0.001"} (US) or
    \code{"< 0,001"} (EU)
}
}
\examples{
# Load example data
data(clintrial)

# Example 1: Survival at specific time points by treatment
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24, 36),
    time_unit = "months",
    labels = clintrial_labels
)

\donttest{

# Example 2: Median survival only
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = NULL,
    probs = 0.5,
    labels = clintrial_labels
)

# Example 3: Multiple quantiles (quartiles)
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "stage",
    times = NULL,
    probs = c(0.25, 0.5, 0.75),
    labels = clintrial_labels
)

# Example 4: Both time points and median
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    probs = 0.5,
    time_unit = "months",
    labels = clintrial_labels
)

# Example 5: Cumulative incidence (1 - survival)
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    type = "risk",
    labels = clintrial_labels
)

# Example 6: Include number at risk
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    stats = c("survival", "ci", "n_risk"),
    labels = clintrial_labels
)

# Example 7: Overall survival without stratification
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    times = c(12, 24, 36, 48),
    labels = clintrial_labels
)

# Example 8: Without total row
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    total = FALSE,
    labels = clintrial_labels
)

# Example 9: Custom labels
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    labels = c("Drug A" = "Treatment A", "Drug B" = "Treatment B"),
    time_unit = "months"
)

# Example 10: Different confidence interval type
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    conf_type = "log-log",
    labels = clintrial_labels
)

# Example 11: Wilcoxon test instead of log-rank
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    test_type = "wilcoxon",
    labels = clintrial_labels
)

# Example 12: Access raw data for custom analysis
result <- survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24)
)
raw <- attr(result, "raw_data")
print(raw)

# Example 13: Access survfit objects for plotting
fits <- attr(result, "survfit_objects")
plot(fits$overall)  # Plot overall survival curve

# Example 14: Multiple survival outcomes stacked
survtable(
    data = clintrial,
    outcome = c("Surv(pfs_months, pfs_status)", "Surv(os_months, os_status)"),
    by = "treatment",
    times = c(12, 24),
    probs = 0.5,
    time_unit = "months",
    total = FALSE,
    labels = c(
        "Surv(pfs_months, pfs_status)" = "Progression-Free Survival",
        "Surv(os_months, os_status)" = "Overall Survival"
    )
)

# Example 15: European number formatting
survtable(
    data = clintrial,
    outcome = "Surv(os_months, os_status)",
    by = "treatment",
    times = c(12, 24),
    number_format = "eu",
    labels = clintrial_labels
)

}

}
\seealso{
\code{\link{desctable}} for baseline characteristics tables,
\code{\link{fit}} for regression analysis,
\code{\link{tablesave}} for export in any supported format,
\code{\link{table2pdf}} for PDF export,
\code{\link{table2docx}} for Word export,
\code{\link[survival]{survfit}} for underlying survival estimation,
\code{\link[survival]{survdiff}} for survival curve comparison tests

Other descriptive functions:
\code{\link[=desctable]{desctable()}},
\code{\link[=print.survtable]{print.survtable()}}
}
\concept{descriptive functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2rtf.R
\name{table2rtf}
\alias{table2rtf}
\title{Export Table to Rich Text Format (RTF)}
\usage{
table2rtf(
  table,
  file,
  caption = NULL,
  font_size = 8,
  font_family = "Arial",
  format_headers = TRUE,
  bold_significant = TRUE,
  bold_variables = FALSE,
  p_threshold = 0.05,
  indent_groups = FALSE,
  condense_table = FALSE,
  condense_quantitative = FALSE,
  zebra_stripes = FALSE,
  dark_header = FALSE,
  paper = "letter",
  orientation = "portrait",
  width = NULL,
  align = NULL,
  return_ft = FALSE,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{table}{Data frame, data.table, or matrix to export. Can be output from 
\code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
\code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.}

\item{file}{Character string specifying the output RTF filename. Must have 
\code{.rtf} extension. Example: \code{"results.rtf"}, \code{"Table1.rtf"}.}

\item{caption}{Character string. Optional caption displayed above the table 
in the RTF document. Default is \code{NULL}.}

\item{font_size}{Numeric. Base font size in points for table content. 
Default is 8. Typical range: 8-12 points. Headers use slightly larger size.}

\item{font_family}{Character string. Font family name for the table. Must be 
a font installed on the system. Default is \code{"Arial"}. Common options: 
\code{"Times New Roman"}, \code{"Calibri"}, \code{"Courier New"}.}

\item{format_headers}{Logical. If \code{TRUE}, formats column headers by 
italicizing statistical notation ("\emph{n}", "\emph{p}"), converting underscores 
to spaces, and improving readability. Default is \code{TRUE}.}

\item{bold_significant}{Logical. If \code{TRUE}, applies bold formatting to 
\emph{p}-values below the significance threshold. Makes significant results stand 
out. Default is \code{TRUE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. Default is \code{FALSE}.}

\item{p_threshold}{Numeric. Threshold for bold \emph{p}-value formatting. Only 
used when \code{bold_significant = TRUE}. Default is 0.05.}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels under 
their parent variable using horizontal spacing, creating hierarchical display. 
Useful for categorical variables in regression tables. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses table by showing 
only essential rows (single row for continuous, non-reference for binary). 
Automatically sets \code{indent_groups = TRUE}. Significantly reduces table 
height. Default is \code{FALSE}.}

\item{condense_quantitative}{Logical. If \code{TRUE}, condenses continuous 
and survival variables into single rows while preserving all categorical 
variable rows (including binary). Only applies to descriptive tables from 
\code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
\code{condense_table}, this does not collapse binary categorical variables. 
Default is \code{FALSE}.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating row shading 
to different variables (not individual rows) for visual grouping. 
Default is \code{FALSE}.}

\item{dark_header}{Logical. If \code{TRUE}, creates a dark background with 
light text for the header row, providing strong visual contrast. 
Default is \code{FALSE}.}

\item{paper}{Character string specifying paper size:
\itemize{
  \item \code{"letter"} - US Letter (8.5" × 11") [default]
  \item \code{"a4"} - A4 (210 mm × 297 mm)
  \item \code{"legal"} - US Legal (8.5" × 14")
}}

\item{orientation}{Character string specifying page orientation:
\itemize{
  \item \code{"portrait"} - Vertical [default]
  \item \code{"landscape"} - Horizontal (for wide tables)
}}

\item{width}{Numeric. Table width in inches. If \code{NULL} (default), 
automatically fits to content and page width. Specify to control exactly.}

\item{align}{Character vector specifying column alignment for each column. 
Options: \code{"left"}, \code{"center"}, or \code{"right"}. If \code{NULL} 
(default), automatically determines based on content (text left, numbers right).
Example: \code{c("left", "left", "center", "right", "right")}.}

\item{return_ft}{Logical. If \code{TRUE}, returns the \pkg{flextable} object 
directly for further customization. If \code{FALSE} (default), returns 
invisibly with flextable as attribute. See Details for usage. 
Default is \code{FALSE}.}

\item{quiet}{Logical. Suppress progress and confirmation messages, such
as the notice reporting the file written. Errors and warnings are
unaffected. Default is \code{FALSE}.}

\item{...}{Additional arguments (currently unused, reserved for future extensions).}
}
\value{
Behavior depends on \code{return_ft}:
  \describe{
    \item{\code{return_ft = FALSE}}{Invisibly returns a list with components:
      \itemize{
        \item \code{file} - Path to created file
        \item \code{caption} - Caption text (if provided)
      }
      The \pkg{flextable} object is accessible via \code{attr(result, "flextable")}
    }
    \item{\code{return_ft = TRUE}}{Directly returns the \pkg{flextable} object for 
      immediate further customization}
  }
  
  In both cases, creates a .rtf file at the specified location.
}
\description{
Converts a data frame, data.table, or matrix to a Rich Text Format (\code{.rtf}) 
document using the \pkg{flextable} and \pkg{officer} packages. Creates 
widely compatible tables with extensive formatting options. RTF files can be 
opened and edited in Microsoft Word, LibreOffice, WordPad, and many other word 
processors. Particularly useful for regulatory submissions, cross-platform 
compatibility, and when maximum editability is required.
}
\details{
\strong{Package Requirements:}

This function requires:
\itemize{
  \item \strong{\pkg{flextable}} - For creating formatted tables
  \item \strong{\pkg{officer}} - For RTF document generation
}

Install if needed:
\preformatted{
install.packages(c("flextable", "officer"))
}

\strong{RTF Format Advantages:}

RTF (Rich Text Format) is a universal document format with several advantages:
\itemize{
  \item \strong{Maximum compatibility} - Opens in virtually all word processors
  \item \strong{Cross-platform} - Works on Windows, Mac, Linux without conversion
  \item \strong{Fully editable} - Native text format, not embedded objects
  \item \strong{Lightweight} - Smaller file sizes than DOCX
  \item \strong{Regulatory compliance} - Widely accepted for submissions (FDA, EMA)
  \item \strong{Long-term accessibility} - Simple text-based format
  \item \strong{Version control friendly} - Text-based, works with diff tools
}

Applications that can open RTF files:
\itemize{
  \item Microsoft Word (Windows, Mac)
  \item LibreOffice Writer
  \item Apache OpenOffice Writer
  \item WordPad (Windows built-in)
  \item TextEdit (Mac built-in)
  \item Google Docs (with import)
  \item Pages (Mac)
  \item Many other word processors
}

\strong{Output Features:}

The generated RTF document contains:
\itemize{
  \item Fully editable table (native RTF table, not image)
  \item Professional typography and spacing
  \item Proper page setup (size, orientation, margins)
  \item Caption (if provided) as separate paragraph above table
  \item All formatting preserved but editable
  \item Compatible with RTF 1.5 specification
}

\strong{Further Customization:}

For programmatic customization beyond the built-in options, access the 
\code{flextable} object:

\emph{Method 1: Via attribute (default)}
\preformatted{
result <- table2rtf(table, "output.rtf")
ft <- attr(result, "flextable")

# Customize flextable
ft <- flextable::bold(ft, i = 1, j = 1, part = "body")
ft <- flextable::color(ft, i = 2, j = 3, color = "red")

# Re-save if needed
flextable::save_as_rtf(ft, path = "customized.rtf")
}

\emph{Method 2: Direct return}
\preformatted{
ft <- table2rtf(table, "output.rtf", return_ft = TRUE)

# Customize immediately
ft <- flextable::bg(ft, bg = "yellow", part = "header")
ft <- flextable::autofit(ft)

# Save to new file
flextable::save_as_rtf(ft, path = "custom.rtf")
}

\strong{Page Layout:}

The function automatically sets up the RTF document with:
\itemize{
  \item Specified paper size and orientation
  \item Standard margins (1 inch by default)
  \item Table positioned at document start
  \item Left-aligned table placement
}

For landscape orientation:
\itemize{
  \item Automatically swaps page dimensions
  \item Applies landscape property
  \item Useful for wide tables with many columns
}

\strong{Table Width Management:}

Width behavior:
\itemize{
  \item \code{width = NULL} - Auto-fits to content and page width
  \item \code{width = 6} - Exactly 6 inches wide
  \item Width distributed evenly across columns by default
  \item Can adjust individual column widths in word processor after creation
}

For very wide tables:
\enumerate{
  \item Use \code{orientation = "landscape"}
  \item Use \code{paper = "legal"} for extra width
  \item Reduce \code{font_size}
  \item Use \code{condense_table = TRUE}
  \item Consider breaking across multiple tables
}

\strong{Typography:}

The function applies professional typography:
\itemize{
  \item Column headers: Bold, slightly larger font
  \item Body text: Regular weight, specified font size
  \item Numbers: Right-aligned for easy comparison
  \item Text: Left-aligned for readability
  \item Consistent spacing: Adequate padding in cells
  \item Statistical notation: Italicized appropriately
}
}
\examples{
data(clintrial)
data(clintrial_labels)

# Create example table
results <- fit(
   data = clintrial,
   outcome = "os_status",
   predictors = c("age", "sex", "treatment", "stage"),
   labels = clintrial_labels
)

# Example 1: Basic RTF export
if (requireNamespace("flextable", quietly = TRUE)) {
  table2rtf(results, file.path(tempdir(), "results.rtf"))
}

\donttest{
old_width <- options(width = 180)
# Example 2: With caption
table2rtf(results, file.path(tempdir(), "captioned.rtf"),
       caption = "Table 1: Multivariable Logistic Regression Results")

# Example 3: Landscape orientation for wide tables
table2rtf(results, file.path(tempdir(), "wide.rtf"),
       orientation = "landscape")

# Example 4: Custom font and size
table2rtf(results, file.path(tempdir(), "custom_font.rtf"),
       font_family = "Times New Roman",
       font_size = 11)

# Example 5: Hierarchical display
table2rtf(results, file.path(tempdir(), "indented.rtf"),
       indent_groups = TRUE)

# Example 6: Condensed table
table2rtf(results, file.path(tempdir(), "condensed.rtf"),
       condense_table = TRUE)

# Example 7: With zebra stripes
table2rtf(results, file.path(tempdir(), "striped.rtf"),
       zebra_stripes = TRUE)

# Example 8: Dark header style
table2rtf(results, file.path(tempdir(), "dark.rtf"),
       dark_header = TRUE)

# Example 9: A4 paper for international submissions
table2rtf(results, file.path(tempdir(), "a4.rtf"),
       paper = "a4")

# Example 10: Get flextable for customization
result <- table2rtf(results, file.path(tempdir(), "base.rtf"))
ft <- attr(result, "flextable")

# Customize the flextable
ft <- flextable::bold(ft, i = 1, part = "body")
ft <- flextable::color(ft, j = "p-value", color = "blue")

# Re-save
flextable::save_as_rtf(ft, path = file.path(tempdir(), "customized.rtf"))

# Example 11: Direct flextable return
ft <- table2rtf(results, file.path(tempdir(), "direct.rtf"), return_ft = TRUE)
ft <- flextable::bg(ft, bg = "yellow", part = "header")

# Example 12: Regulatory submission table
table2rtf(results, file.path(tempdir(), "submission.rtf"),
       caption = "Table 2: Adjusted Odds Ratios for Mortality",
       font_family = "Times New Roman",
       font_size = 10,
       indent_groups = TRUE,
       zebra_stripes = FALSE,
       bold_significant = TRUE)

# Example 13: Custom column alignment
table2rtf(results, file.path(tempdir(), "aligned.rtf"),
       align = c("left", "left", "center", "right", "right"))

# Example 14: Disable significance bolding
table2rtf(results, file.path(tempdir(), "no_bold.rtf"),
       bold_significant = FALSE)

# Example 15: Stricter significance threshold
table2rtf(results, file.path(tempdir(), "strict.rtf"),
       bold_significant = TRUE,
       p_threshold = 0.01)

# Example 16: Descriptive statistics for baseline characteristics
desc <- desctable(clintrial, by = "treatment",
   variables = c("age", "sex", "bmi", "stage"), labels = clintrial_labels)

table2rtf(desc, file.path(tempdir(), "baseline.rtf"),
       caption = "Table 1: Baseline Patient Characteristics",
       zebra_stripes = TRUE)

# Example 17: Clinical trial efficacy table
table2rtf(results, file.path(tempdir(), "efficacy.rtf"),
       caption = "Table 3: Primary Efficacy Analysis - Intent to Treat Population",
       font_family = "Courier New",  # Monospace for alignment
       paper = "letter",
       orientation = "landscape",
       condense_table = TRUE)

options(old_width)
}
}
\seealso{
\code{\link{tablesave}} for saving in the format given by the file extension,
\code{\link{table2docx}} for Word documents,
\code{\link{table2pptx}} for PowerPoint slides,
\code{\link{table2pdf}} for PDF output,
\code{\link{table2html}} for HTML tables,
\code{\link{table2tex}} for LaTeX output,
\code{\link[flextable]{flextable}} for the underlying table object,
\code{\link[flextable]{save_as_rtf}} for direct RTF export

Other export functions:
\code{\link[=table2docx]{table2docx()}},
\code{\link[=table2html]{table2html()}},
\code{\link[=table2pdf]{table2pdf()}},
\code{\link[=table2pptx]{table2pptx()}},
\code{\link[=table2tex]{table2tex()}},
\code{\link[=tablesave]{tablesave()}}
}
\concept{export functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{replace_empty_cells}
\alias{replace_empty_cells}
\title{Replace empty cells with "-"}
\usage{
replace_empty_cells(df)
}
\arguments{
\item{df}{Data.frame or data.table to process.}
}
\value{
Data.table with empty cells replaced by "-".
}
\description{
Converts empty strings and NA values to "-" for consistent display
in exported tables. Preserves Variable column values.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_pvalues_export_tex}
\alias{format_pvalues_export_tex}
\title{Format \emph{p}-values for exported tables (TeX)}
\usage{
format_pvalues_export_tex(df, p_threshold = 0.05)
}
\arguments{
\item{df}{Data.table containing \emph{p}-value columns.}

\item{p_threshold}{Numeric threshold for significance (default 0.05).}
}
\value{
Data.table with significant \emph{p}-values wrapped in LaTeX bold commands.
}
\description{
Applies bold formatting to significant \emph{p}-values in LaTeX tables using
the textbf command.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{format_model_comparison}
\alias{format_model_comparison}
\title{Format model comparison table}
\usage{
format_model_comparison(comparison)
}
\arguments{
\item{comparison}{Data.table with comparison metrics.}
}
\value{
Data.table with properly rounded numeric columns.
}
\description{
Rounds numeric columns to appropriate precision based on metric type.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forestsave.R
\name{forestsave}
\alias{forestsave}
\title{Save a Forest Plot to File}
\usage{
forestsave(
  plot,
  file,
  width = NULL,
  height = NULL,
  units = NULL,
  dpi = 300,
  device = NULL,
  embed_fonts = FALSE,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{plot}{A forest plot produced by \code{lmforest()}, \code{glmforest()},
\code{coxforest()}, \code{uniforest()}, \code{multiforest()}, or
\code{autoforest()}. Any \pkg{ggplot2} object is accepted, but one that
carries no recommended dimensions requires \code{width} and \code{height}
to be supplied.}

\item{file}{Character string giving the output path. The file extension
determines the format: \code{.pdf}, \code{.png}, \code{.tiff},
\code{.jpeg}, \code{.svg}, \code{.eps}, and the other formats
\code{ggplot2::ggsave()} supports.}

\item{width, height}{Numeric plot dimensions. Taken from the plot's
recommended dimensions when not supplied, and converted where these differ
from \code{units}.}

\item{units}{Character string giving the units for \code{width} and
\code{height}: \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Defaults
to the units the recommended dimensions were expressed in, and to
\code{"in"} where none were recorded.}

\item{dpi}{Numeric resolution in dots per inch for raster formats. Ignored
for vector formats. Default is \code{300}.}

\item{device}{Graphics device. Selected automatically when \code{NULL}: see
Details. Supply \code{"cairo"} to use the Cairo device for the output
format, which embeds fonts, or any device function or name recognized by
\code{ggplot2::ggsave()}.}

\item{embed_fonts}{Logical. Embed the fonts in a PDF after it is written,
using \code{grDevices::embedFonts()}. Requires Ghostscript, and applies
only to PDF output. Default is \code{FALSE}.}

\item{quiet}{Logical. Suppress the message reporting the dimensions used.
Default is \code{FALSE}.}

\item{...}{Additional arguments passed to \code{ggplot2::ggsave()}, such as
\code{bg} or \code{limitsize}.}
}
\value{
Invisibly returns \code{file}.
}
\description{
Writes a forest plot to file, sized by default from the recommendations the
plotting function computed from the plot's own content, and rendered through
a graphics device chosen to reproduce the fonts seen on screen.
}
\details{
Forest plots are sized by their content rather than by the page: a plot with
more variables, longer labels, or more factor levels requires a larger
canvas, and the plotting functions compute one and attach it to the returned
object as a \code{"rec_dims"} attribute. \code{forestsave()} reads that
attribute, so the following are equivalent:

\preformatted{
dims <- attr(p, "rec_dims")
ggplot2::ggsave("forest.pdf", p, width = dims$width,
                height = dims$height, units = dims$units)

forestsave(p, "forest.pdf")
}

\strong{Device selection.} Fonts are measured differently by different
graphics devices, so a figure written through one device may not reproduce
the spacing seen on screen or in the package documentation. Unless
\code{device} is supplied, \code{forestsave()} selects:

\describe{
  \item{PDF}{R's internal PDF device, \code{grDevices::pdf()}, reached by
    deferring to \code{ggplot2::ggsave()}. Cairo is not used by default:
    although it embeds fonts, its face selection can substitute an
    incorrect italic, which matters here because forest plot footers and
    statistical notation are set in italic. Pass \code{device = "cairo"}
    to use it where font embedding is required.}
  \item{PNG, TIFF, JPEG}{The corresponding \pkg{ragg} device where that
    package is installed, whose text metrics match those used to render the
    package vignettes. Falls back to the standard raster devices
    otherwise.}
  \item{Other formats}{Left to \code{ggplot2::ggsave()}.}
}

\strong{Font embedding.} R's PDF device draws on the standard fonts that every
PDF reader is required to provide, so its output is portable without
embedding. Embedding matters where the plot uses a font outside that set, and
where a publisher requires it. Setting \code{embed_fonts = TRUE} embeds the
fonts after the file is written, through \code{grDevices::embedFonts()}, which
requires a Ghostscript installation. This is offered as an alternative to
\code{device = "cairo"}, which embeds fonts as it draws but selects its own
italic face.

\code{device = "cairo"} is a shorthand of this package's own, resolving to
\code{grDevices::cairo_pdf()}, \code{grDevices::cairo_ps()}, or
\code{grDevices::svg()} according to the output format:
\preformatted{
forestsave(p, "forest.pdf", device = "cairo")
}
It applies to vector formats only, since raster output already uses the
\pkg{ragg} devices, which supersede Cairo's raster backends. Cairo is
absent from some installations even where \code{capabilities("cairo")}
reports otherwise, so the device is opened once before use and its
unavailability reported. Any other value of \code{device} is passed to
\code{ggplot2::ggsave()} unchanged.

The dimensions used are reported through a \code{message()} unless
\code{quiet = TRUE}, so that a figure written at an unexpected size is
apparent at the point it is written.
}
\examples{
data(clintrial)
data(clintrial_labels)

model <- stats::glm(readmission_30d ~ age + sex + stage,
                    data = clintrial, family = stats::binomial)

p <- glmforest(model, data = clintrial, labels = clintrial_labels)

# Example 1: Save at the recommended dimensions
forestsave(p, file.path(tempdir(), "forest.pdf"))

\donttest{

# Example 2: Raster output at publication resolution
forestsave(p, file.path(tempdir(), "forest.png"), dpi = 600)

# Example 3: Dimensions given explicitly, in any supported units
forestsave(p, file.path(tempdir(), "forest_sized.pdf"),
           width = 24, height = 16, units = "cm")

# Example 4: Embed fonts after writing, where Ghostscript is available
forestsave(p, file.path(tempdir(), "forest_embedded.pdf"),
           embed_fonts = TRUE)

}

}
\seealso{
\code{\link{autoforest}}, \code{\link{coxforest}},
  \code{\link{glmforest}}, \code{\link{lmforest}}, \code{\link{uniforest}},
  \code{\link{multiforest}} for producing forest plots;
  \code{\link{tablesave}} for exporting formatted tables

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=coxforest]{coxforest()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=recdims]{recdims()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{format_event_counts}
\alias{format_event_counts}
\title{Describe the analyzed events for console output}
\usage{
format_event_counts(counts, label = "Events analyzed", marks = NULL)
}
\arguments{
\item{counts}{List as returned by \code{get_analysis_counts()}.}

\item{label}{Character label preceding the counts.}

\item{marks}{List of number marks as returned by
\code{resolve_number_marks()}.}
}
\value{
Character string, or \code{NULL} when the model carries no events.
}
\description{
Renders the event counts held in \code{get_analysis_counts()} using the same
formatter as the observation counts, so that the two lines cannot drift apart
in rounding, separators or range handling.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{check_required_packages}
\alias{check_required_packages}
\title{Check required packages for model type}
\usage{
check_required_packages(model_type)
}
\arguments{
\item{model_type}{Character string indicating model type.}
}
\value{
\code{NULL} (invisibly). Stops execution if packages missing.
}
\description{
Verifies that necessary packages are installed for the specified model type.
Stops with informative error if required packages are missing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiforest.R
\name{multiforest}
\alias{multiforest}
\title{Create Forest Plot for Multivariate Regression}
\usage{
multiforest(
  x,
  title = "Multivariate Analysis",
  effect_label = NULL,
  column = "adjusted",
  digits = 2,
  p_digits = 3,
  conf_level = 0.95,
  font_size = 1,
  annot_size = 3.88,
  header_size = 5.82,
  title_size = 23.28,
  plot_width = NULL,
  plot_height = NULL,
  table_width = 0.6,
  show_n = TRUE,
  show_events = NULL,
  show_predictor = NULL,
  covariates_footer = TRUE,
  indent_predictor = FALSE,
  bold_variables = TRUE,
  center_padding = 4,
  zebra_stripes = TRUE,
  color = NULL,
  null_line = NULL,
  log_scale = NULL,
  labels = NULL,
  units = "in",
  number_format = NULL
)
}
\arguments{
\item{x}{Multifit result object (data.table with class attributes from 
\code{multifit()}).}

\item{title}{Character string specifying the plot title. Default is 
\code{"Multivariate Analysis"}. Use descriptive titles for publication.}

\item{effect_label}{Character string for the effect measure label on the 
forest plot axis. Default is \code{NULL}, which auto-detects based on 
model type (\emph{e.g.,} "Odds Ratio", "Hazard Ratio", "Rate Ratio", "Estimate").}

\item{column}{Character string specifying which results to plot when
\code{multifit()} was called with \code{columns = "both"}. Options are
\code{"adjusted"} (default) or \code{"unadjusted"}. Ignored when the
\code{multifit} result contains only one column type.}

\item{digits}{Integer specifying the number of decimal places for effect 
estimates and confidence intervals. Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be
between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
when \code{conf_level = 0.90}).}

\item{font_size}{Numeric multiplier controlling the base font size for all 
text elements. Default is 1.0.}

\item{annot_size}{Numeric value controlling the relative font size for data 
annotations. Default is 3.88.}

\item{header_size}{Numeric value controlling the relative font size for column 
headers. Default is 5.82.}

\item{title_size}{Numeric value controlling the relative font size for the 
main plot title. Default is 23.28.}

\item{plot_width}{Numeric value specifying the intended output width in 
specified \code{units}. Used for optimizing layout. Default is \code{NULL} 
(automatic). Recommended: 10-16 inches.}

\item{plot_height}{Numeric value specifying the intended output height in 
specified \code{units}. Default is \code{NULL} (automatic based on number 
of rows).}

\item{table_width}{Numeric value between 0 and 1 specifying the proportion of 
total plot width allocated to the data table. Default is 0.6 (60\% table, 
40\% forest plot).}

\item{show_n}{Logical. If \code{TRUE}, includes a column showing sample sizes.
Counts describe the observations used in fitting rather than every row
supplied. Default is \code{TRUE}.}

\item{show_events}{Logical. If \code{TRUE}, includes a column showing the 
number of events for each row. Default is \code{TRUE} for binomial and 
survival models, \code{FALSE} for linear models.}

\item{show_predictor}{Logical. If \code{TRUE}, includes the "Predictor" column 
showing which level of a factor predictor is being compared. If \code{FALSE},
omits the column (useful when predictor info is in the caption). Default is 
\code{NULL}, which uses the \code{include_predictor} setting from
\code{multifit()} if available, otherwise \code{TRUE}.}

\item{covariates_footer}{Logical. If \code{TRUE} (default), displays a footer
listing the covariates used in adjusted models. Covariate names are formatted
using the \code{labels} parameter if provided. Only shown when displaying
adjusted results.}

\item{indent_predictor}{Logical. If \code{TRUE}, indents predictor levels
under outcome names, creating a hierarchical display. If \code{FALSE} 
(default), shows outcome and predictor level in separate columns.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. If \code{FALSE} (default), variable names are displayed in plain
text.}

\item{center_padding}{Numeric value specifying horizontal spacing between 
table and forest plot. Default is 4.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating gray 
background shading to different outcomes for improved readability. 
Default is \code{TRUE}.}

\item{color}{Character string specifying the color for point estimates in 
the forest plot. Default is \code{NULL}, which auto-selects based on
effect type: purple (\code{"#8A61D8"}) for hazard ratios (Cox), teal 
(\code{"#4BA6B6"}) for odds ratios (logistic), blue (\code{"#3F87EE"}) for 
rate/risk ratios (Poisson, Gamma, \emph{etc.}), and green (\code{"#5A8F5A"}) for 
coefficients (linear models).
Use hex codes or R color names for custom colors.}

\item{null_line}{Numeric value for the reference line position. Default is 
\code{NULL}, which uses 1 for ratio measures (OR, HR, RR) and 0 for 
coefficients.}

\item{log_scale}{Logical. If \code{TRUE}, uses log scale for the x-axis.
Default is \code{NULL}, which auto-detects (\code{TRUE} for OR/HR/RR, \code{FALSE} for
coefficients).}

\item{labels}{Named character vector providing custom display labels for 
outcomes and variables. Applied to outcome names in the plot.
Default is \code{NULL}, in which case the labels attached by
\code{multifit()} are used. Original names are used where neither is
available.}

\item{units}{Character string specifying units for plot dimensions: 
\code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}
}
\value{
A \code{ggplot} object containing the complete forest plot. The plot 
  can be:
  \itemize{
    \item Displayed directly: \code{print(plot)}
    \item Saved to file: \code{forestsave(plot, "forest.pdf")}
    \item Further customized with \pkg{ggplot2} functions
  }
  
  The returned object includes an attribute \code{"rec_dims"} 
  accessible via \code{attr(plot, "rec_dims")}, which is a list 
  containing:
  \describe{
    \item{width}{Numeric. Recommended plot width in specified units}
    \item{height}{Numeric. Recommended plot height in specified units}
    \item{units}{Character. The units the dimensions are expressed in,
      matching the \code{units} argument}
  }
  
  These recommendations are automatically calculated based on the number of 
  variables, text sizes, and layout parameters, and are printed to console 
  if \code{plot_width} or \code{plot_height} are not specified.
  \code{forestsave()} reads all three and requires no further handling.

  The returned object also includes an attribute \code{"table_data"}
  accessible via \code{attr(plot, "table_data")}, a data.table holding the
  values drawn in the plot, in the order of the model terms. Sample sizes
  and event counts describe the observations used in fitting rather than
  every row supplied.
}
\description{
Generates a publication-ready forest plot from a \code{multifit()} output
object. The plot displays effect estimates (OR, HR, RR, or coefficients) with
confidence intervals across multiple outcomes, organized by outcome with the
predictor levels shown for each.
}
\details{
\strong{Plot Layout:}

The forest plot is organized with outcomes as grouping headers and predictor
levels (or interaction terms) as rows within each outcome. This provides a
clear visual comparison of how a single predictor affects multiple outcomes.

\enumerate{
  \item \strong{Title}: Centered at top
  \item \strong{Data Table} (left): Contains:
    \itemize{
      \item Outcome column (or grouped headers)
      \item Predictor/Group column
      \item n: Sample sizes (optional)
      \item Events: Event counts (optional, for applicable models)
      \item Effect (95\% CI); \emph{p}-value
    }
  \item \strong{Forest Plot} (right):
    \itemize{
      \item Point estimates (squares)
      \item 95\% confidence intervals
      \item Reference line at null value (1 or 0)
      \item Log scale for ratio measures
    }
}

\strong{Data Source:}

The function extracts effect estimates directly from the multifit output
object's \code{raw_data} attribute, which contains the numeric values
needed for plotting. This approach is efficient and ensures consistency
with the formatted table output.
}
\examples{
data(clintrial)
data(clintrial_labels)
library(survival)

# Create example multifit result
result <- multifit(
    data = clintrial,
    outcomes = c("surgery", "pfs_status", "os_status"),
    predictor = "treatment",
    covariates = c("age", "sex", "stage"),
    parallel = FALSE
)

# Example 1: Basic multivariate forest plot
p <- multiforest(result)

\donttest{

old_width <- options(width = 180)

# Example 2: With custom title and labels
plot2 <- multiforest(
    result,
    title = "Treatment Effects Across Clinical Outcomes",
    labels = clintrial_labels
)

# Example 3: Customize appearance
plot3 <- multiforest(
    result,
    color = "#E74C3C",
    zebra_stripes = TRUE,
    labels = clintrial_labels
)

# Example 4: Save with recommended dimensions
forestsave(plot3, file.path(tempdir(), "multioutcome_forest.pdf"))

options(old_width)

}
}
\seealso{
\code{\link{autoforest}} for automatic model detection,
\code{\link{multifit}} for multi-outcome regression analysis,
\code{\link{glmforest}} for single GLM forest plots,
\code{\link{coxforest}} for single Cox model forest plots,
\code{\link{lmforest}} for single linear model forest plots,
\code{\link{uniforest}} for univariable screening forest plots,
  \code{\link{forestsave}} for saving with recommended dimensions

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=coxforest]{coxforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=recdims]{recdims()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{validate_predictors_exist}
\alias{validate_predictors_exist}
\title{Validate predictors exist in data}
\usage{
validate_predictors_exist(data, predictors)
}
\arguments{
\item{data}{Data frame or data.table to check.}

\item{predictors}{Character vector of predictor variable names.}
}
\value{
Invisible \code{TRUE} if validation passes, otherwise stops with error.
}
\description{
Checks that all specified predictor variables exist in the dataset. Handles
interaction terms (splits on ":"), mixed-effects random effects (ignores
"|" syntax), and raises informative errors for missing variables.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{process_categorical}
\alias{process_categorical}
\title{Process categorical variable}
\usage{
process_categorical(
  data,
  var,
  var_label,
  group_var,
  stats,
  na_include,
  na_label,
  test,
  test_type,
  total,
  total_label,
  na_percent,
  marks = NULL,
  ...
)
}
\arguments{
\item{data}{Data.table containing the variable.}

\item{var}{Character string naming the variable to process.}

\item{var_label}{Character string label for display.}

\item{group_var}{Optional character string naming the grouping variable.}

\item{stats}{Character vector of statistics to calculate.}

\item{na_include}{Logical whether to include missing values as a category.}

\item{na_label}{Character string label for missing values.}

\item{test}{Logical whether to perform statistical tests.}

\item{test_type}{Character string specifying test type.}

\item{total}{Logical or character controlling total column display.}

\item{total_label}{Character string label for total column.}

\item{na_percent}{Logical whether to include NA in percentage denominators.}

\item{...}{Additional arguments passed to test functions.}
}
\value{
List with 'formatted' and 'raw' data.table components.
}
\description{
Calculates frequency and percentage statistics for categorical variables,
with optional grouping and chi-square/Fisher's exact testing. Handles
factor levels, missing values, and custom labeling.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiforest.R
\name{calculate_multiforest_layout}
\alias{calculate_multiforest_layout}
\title{Calculate table layout for multiforest plots}
\usage{
calculate_multiforest_layout(
  to_show_exp_clean,
  show_n,
  show_events,
  indent_predictor,
  show_predictor = TRUE,
  effect_abbrev,
  font_size,
  table_width = 0.6,
  rangeb,
  center_padding,
  ci_pct = 95
)
}
\description{
Computes column widths and positions for the table portion of a multiforest plot.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recdims.R
\name{recdims}
\alias{recdims}
\title{Recommended Figure Dimensions}
\usage{
recdims(x, units = NULL)
}
\arguments{
\item{x}{A forest plot produced by \code{lmforest()}, \code{glmforest()},
\code{coxforest()}, \code{uniforest()}, \code{multiforest()}, or
\code{autoforest()}.}

\item{units}{Character string giving the units the dimensions are returned
in: \code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Defaults to the
units the plot was created in, so that the dimensions pass through
unconverted unless a change is requested.}
}
\value{
A named numeric vector with elements \code{width} and \code{height},
  expressed in \code{units}. The units are recorded on the result as a
  \code{"units"} attribute, so that a value carried between functions
  remains self-describing.
}
\description{
Returns the figure dimensions recommended for a forest plot, computed from
the plot's own content by the function that produced it.
}
\details{
Forest plots are sized by their content rather than by the page: a plot with
more variables, longer labels, or more factor levels requires a larger
canvas. Each plotting function computes a suitable size and attaches it to
the object it returns, and \code{recdims()} reports it.

The values returned are those \code{forestsave()} applies, and are reported
exactly as computed rather than rounded, so that a figure written with
\code{forestsave()} and one written with \code{ggplot2::ggsave()} at these
dimensions are identical. Explicit use is therefore needed only where the
dimensions themselves are wanted, as when sizing a \pkg{knitr} chunk or
arranging several plots on one page.

Where \code{units} differs from the units the plot was created in, the
dimensions are converted. A plot produced before the units were recorded
carries none, and is treated as having been created in inches.
}
\examples{
data(clintrial)
data(clintrial_labels)

model <- stats::glm(readmission_30d ~ age + sex + stage,
                    data = clintrial, family = stats::binomial)

p <- glmforest(model, data = clintrial, labels = clintrial_labels)

# Example 1: Dimensions in the units the plot was created in
recdims(p)

# Example 2: Journals commonly specify figure widths in millimeters
recdims(p, units = "mm")

# Example 3: Sizing an output canvas from the individual elements
dims <- recdims(p)
dims[["width"]]
dims[["height"]]
attr(dims, "units")

}
\seealso{
\code{\link{forestsave}} for saving at these dimensions,
  \code{\link{autoforest}} for producing the plot

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=coxforest]{coxforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{validate_outcome_exists}
\alias{validate_outcome_exists}
\title{Validate outcome exists in data}
\usage{
validate_outcome_exists(data, outcome)
}
\arguments{
\item{data}{Data frame or data.table to check.}

\item{outcome}{Character string outcome specification (may include Surv()).}
}
\value{
Invisible \code{TRUE} if validation passes, otherwise stops with error.
}
\description{
Checks that the specified outcome variable (or survival variables within
Surv() expression) exists in the dataset. Raises informative error if
variables are missing. Handles both simple outcomes and Surv() expressions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{print.fit_result}
\alias{print.fit_result}
\title{Print method for fit results}
\usage{
\method{print}{fit_result}(x, ...)
}
\arguments{
\item{x}{fit_result object.}

\item{...}{Additional arguments passed to print methods.}
}
\value{
Invisibly returns the input object \code{x}. Called for its
  side effect of printing a formatted summary to the console.
}
\description{
Displays a summary header with model scope (Univariable/Multivariable),
model type, formula, sample size, and event count before printing the
formatted results table.
}
\seealso{
Other regression functions:
\code{\link[=compfit]{compfit()}},
\code{\link[=fit]{fit()}},
\code{\link[=fullfit]{fullfit()}},
\code{\link[=multifit]{multifit()}},
\code{\link[=print.compfit_result]{print.compfit_result()}},
\code{\link[=print.fullfit_result]{print.fullfit_result()}},
\code{\link[=print.multifit_result]{print.multifit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{format_pvalues_multifit}
\alias{format_pvalues_multifit}
\title{Format p-values for multifit display}
\usage{
format_pvalues_multifit(p, digits = 3, marks = NULL)
}
\arguments{
\item{p}{Numeric vector of p-values.}

\item{digits}{Integer number of decimal places. Also determines the threshold
for "less than" display: threshold = 10^(-digits). Default is 3.}

\item{marks}{Optional list with \code{big.mark} and \code{decimal.mark} as
returned by \code{\link{resolve_number_marks}}.}
}
\value{
Character vector of formatted p-values.
}
\description{
Converts numeric p-values to formatted character strings. Values below the 
threshold (determined by digits parameter) display as "< 0.001" (for digits=3),
"< 0.0001" (for digits=4), \emph{etc.} NA values display as "-".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{sanitize_for_latex}
\alias{sanitize_for_latex}
\title{Sanitize certain symbols for LaTeX}
\usage{
sanitize_for_latex(x)
}
\arguments{
\item{x}{Character vector to sanitize.}
}
\value{
Character vector with special characters escaped for LaTeX.
}
\description{
Escapes special LaTeX characters (%, &, #, _, $, ^, ~, {, }) while
preserving existing LaTeX commands. Uses negative lookbehind to avoid
double-escaping already escaped characters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{extract_coxme_metrics}
\alias{extract_coxme_metrics}
\title{Extract metrics for mixed-effects Cox models (coxme)}
\usage{
extract_coxme_metrics(model, raw_data, metrics)
}
\arguments{
\item{model}{Fitted coxme object from coxme package.}

\item{raw_data}{Data.table with raw model information.}

\item{metrics}{Named list of initialized metrics to populate.}
}
\value{
Updated metrics list with coxme-specific values.
}
\description{
Extracts quality metrics specific to mixed-effects Cox proportional hazards
models including concordance, pseudo-R-squared, and ICC.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forestsave.R
\name{select_forest_device}
\alias{select_forest_device}
\title{Select a graphics device for forest plot output}
\usage{
select_forest_device(file)
}
\arguments{
\item{file}{Character string giving the output path.}
}
\value{
A device function, or \code{NULL} to defer to
  \code{ggplot2::ggsave()}.
}
\description{
Chooses a device whose text metrics reproduce those used to render the
package documentation, falling back to \code{ggplot2::ggsave()}'s own
selection where the preferred device is unavailable.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{resolve_separator}
\alias{resolve_separator}
\title{Resolve the CI or range separator}
\usage{
resolve_separator(lower, upper, marks)
}
\arguments{
\item{lower}{Numeric value of the lower bound (or minimum).}

\item{upper}{Numeric value of the upper bound (or maximum).}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string separator.
}
\description{
Determines the appropriate separator character between two numeric bounds
(\emph{e.g.,} CI lower-upper, range min-max) based on whether either bound is
negative and the current locale's decimal mark. This avoids ambiguous
output like \code{"(-5--3)"} or \code{"(1,2-3,4)"} with European commas.
}
\details{
Rules:
\enumerate{
  \item If either bound is negative, use \code{" to "} to avoid
    double-hyphen ambiguity (\emph{e.g.,} \code{"-5 to -3"} not \code{"-5--3"}).
  \item If the decimal mark is a comma (EU locale), use \code{"\u2013"}
    (en-dash) to avoid confusion between decimal commas and separating
    commas (\emph{e.g.,} \code{"1,2\u20133,4"} not \code{"1,2-3,4"}).
  \item Otherwise, use a plain hyphen \code{"-"}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{validate_model_outcome}
\alias{validate_model_outcome}
\title{Validate model type matches outcome specification}
\usage{
validate_model_outcome(
  outcome,
  model_type,
  family = NULL,
  data = NULL,
  auto_correct = TRUE
)
}
\arguments{
\item{outcome}{Character string outcome specification (may include Surv()).}

\item{model_type}{Character string specified model type.}

\item{family}{GLM family object, function, or string if applicable.}

\item{data}{Data frame or data.table for outcome type detection.}

\item{auto_correct}{Logical whether to auto-correct fixable mismatches.}
}
\value{
List with model_type, family, messages, auto_corrected flag.
}
\description{
Ensures consistency between the specified model type, outcome variable type,
and GLM family (if applicable). Detects common mismatches like using survival
outcomes with non-survival models or binary outcomes with linear models.
Can auto-correct fixable issues or raise informative errors.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2tex.R
\name{table2tex}
\alias{table2tex}
\title{Export Table to LaTeX Format}
\usage{
table2tex(
  table,
  file,
  format_headers = TRUE,
  variable_padding = FALSE,
  cell_padding = "normal",
  bold_significant = TRUE,
  bold_variables = FALSE,
  p_threshold = 0.05,
  align = NULL,
  indent_groups = FALSE,
  condense_table = FALSE,
  condense_quantitative = FALSE,
  booktabs = FALSE,
  zebra_stripes = FALSE,
  stripe_color = "gray!20",
  dark_header = FALSE,
  caption = NULL,
  caption_size = NULL,
  label = NULL,
  show_logs = FALSE,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{table}{Data frame, data.table, or matrix to export. Can be output from 
\code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
\code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.}

\item{file}{Character string specifying the output \code{.tex} filename. Must have 
\code{.tex} extension. Example: \code{"results.tex"}, \code{"table1.tex"}.}

\item{format_headers}{Logical. If \code{TRUE}, formats column headers by 
converting underscores to spaces, italicizing statistical notation ("\emph{n}", 
"\emph{p}"), and applying title case. Default is \code{TRUE}.}

\item{variable_padding}{Logical. If \code{TRUE}, adds vertical spacing around 
variable groups using \code{\\addlinespace} for improved readability. 
Default is \code{FALSE}.}

\item{cell_padding}{Character string or numeric. Vertical padding within cells:
\itemize{
  \item \code{"none"} - No extra padding
  \item \code{"normal"} - Standard padding [default]
  \item \code{"relaxed"} - Increased padding
  \item \code{"loose"} - Maximum padding
  \item Numeric - Custom \code{\\arraystretch} value
}}

\item{bold_significant}{Logical. If \code{TRUE}, wraps significant \emph{p}-values 
in \code{textbf} commands for bold display. Default is \code{TRUE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. Default is \code{FALSE}.}

\item{p_threshold}{Numeric. Threshold for bold \emph{p}-value formatting. Only 
used when \code{bold_significant = TRUE}. Default is 0.05.}

\item{align}{Character string or vector specifying column alignment:
\itemize{
  \item \code{"l"} - Left
  \item \code{"c"} - Center  
  \item \code{"r"} - Right
  \item Paragraph column with specified width (p-type)
}
If \code{NULL}, automatically determines based on content. Can specify 
per-column as vector. Default is \code{NULL}.}

\item{indent_groups}{Logical. If \code{TRUE}, uses hspace to 
indent grouped rows, creating hierarchical display. Useful for factor 
variables in regression tables. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses table by showing 
only essential rows (single row for continuous, non-reference for binary). 
Automatically sets \code{indent_groups = TRUE}. Default is \code{FALSE}.}

\item{condense_quantitative}{Logical. If \code{TRUE}, condenses continuous 
and survival variables into single rows while preserving all categorical 
variable rows (including binary). Only applies to descriptive tables from 
\code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
\code{condense_table}, this does not collapse binary categorical variables. 
Default is \code{FALSE}.}

\item{booktabs}{Logical. If \code{TRUE}, uses booktabs package commands 
(toprule, midrule, bottomrule) for professional 
table rules. Requires booktabs package in LaTeX preamble. 
Default is \code{FALSE}.}

\item{zebra_stripes}{Logical. If \code{TRUE}, adds alternating row colors 
for variable groups using rowcolor command. Requires 
\code{xcolor} package with table option in preamble. Default is \code{FALSE}.}

\item{stripe_color}{Character string. LaTeX color specification for zebra 
stripes (\emph{e.g.,} \code{"gray!20"}, \code{"blue!10"}). Only used when 
\code{zebra_stripes = TRUE}. Default is \code{"gray!20"}.}

\item{dark_header}{Logical. If \code{TRUE}, creates white text on black 
background for header row. Requires \code{xcolor} package with \code{table}
option. Default is \code{FALSE}.}

\item{caption}{Character string. Table caption for LaTeX caption command. 
Supports multi-line captions using double backslash. Default is \code{NULL}.}

\item{caption_size}{Numeric. Caption font size in points. If \code{NULL} (default), 
caption will use the document's default caption size (typically slightly smaller 
than body text). Set to a specific value (\emph{e.g.,} 6, 7, 8, 9) to control caption 
size explicitly. This generates a LaTeX comment for use when wrapping 
the table. Typical range: 6-10 points.}

\item{label}{Character string. LaTeX label for cross-references. 
Example: \code{"tab:regression"}. Default is \code{NULL}.}

\item{show_logs}{Logical. If \code{TRUE}, displays informational messages 
about required LaTeX packages and formatting options applied. If 
\code{FALSE}, suppresses these messages. Default is \code{FALSE}.}

\item{quiet}{Logical. Suppress progress and confirmation messages, such
as the notice reporting the file written. Errors and warnings are
unaffected. Default is \code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[xtable]{xtable}}.}
}
\value{
Invisibly returns the file path, matching \code{tablesave()} and
  \code{forestsave()}. Called for its side effect of creating a \code{.tex}
  file at the specified location containing a LaTeX tabular environment.
}
\description{
Converts a data frame, data.table, or matrix to LaTeX source code suitable for 
inclusion in LaTeX documents. Generates publication-quality table markup with 
extensive formatting options including booktabs styling, color schemes, and 
hierarchical displays. Output can be directly \code{\\input{}} or \code{\\include{}} 
into LaTeX manuscripts. Requires \pkg{xtable} for export.
}
\details{
\strong{Output Format:}

The function generates a standalone LaTeX tabular environment that can be:
\enumerate{
  \item Included in documents with \code{\\input} command
  \item Embedded in table/figure environments
  \item Used in manuscript classes (\code{article}, \code{report}, \emph{etc.})
}

The output includes:
\itemize{
  \item Complete tabular environment with proper alignment
  \item Horizontal rules (\code{\\hline} or \code{booktabs} rules)
  \item Column headers with optional formatting
  \item Data rows with automatic escaping of special characters
  \item Optional caption and label commands
}

\strong{Required LaTeX Packages:}

The following are required in the LaTeX document preamble:

\emph{Always required:}
\preformatted{
\\usepackage[T1]{fontenc}
\\usepackage[utf8]{inputenc}
\\usepackage{array}
\\usepackage{graphicx}  % If using resizebox
}

\emph{Optional (based on parameters):}
\preformatted{
\\usepackage{booktabs}  % For booktabs = TRUE
\\usepackage[table]{xcolor}  % For zebra_stripes or dark_header
}

\strong{Booktabs Style:}

When \code{booktabs = TRUE}, the table uses publication-quality rules:
\itemize{
  \item \code{\\toprule} - Heavy rule at top
  \item \code{\\midrule} - Medium rule below headers
  \item \code{\\bottomrule} - Heavy rule at bottom
  \item No vertical rules (\code{booktabs} style)
  \item Better spacing around rules
}

This is the preferred style for most academic journals.

\strong{Color Features:}

\emph{Zebra Stripes:}
Creates alternating background colors for visual grouping:
\preformatted{
zebra_stripes = TRUE
stripe_color = "gray!20"  # 20\% gray
stripe_color = "blue!10"  # 10\% blue  
}

\emph{Dark Header:}
Creates high-contrast header row:
\preformatted{
dark_header = TRUE  # Black background, white text
}

Both require the xcolor package with the table option in the document preamble.

\strong{Integration with LaTeX Documents:}

\emph{Basic inclusion:}
\preformatted{
\\begin{table}[htbp]
  \\centering
  \\caption{Regression Results}
  \\label{tab:regression}
  \\input{results.tex}
\\end{table}
}

\emph{With resizing:}
\preformatted{
\\begin{table}[htbp]
  \\centering
  \\caption{Results}
  \\resizebox{\\textwidth}{!}{\\input{results.tex}}
\\end{table}
}

\emph{Landscape orientation:}
\preformatted{
\\usepackage{pdflscape}
\\begin{landscape}
  \\begin{table}[htbp]
    \\centering
    \\input{wide_results.tex}
  \\end{table}
\\end{landscape}
}

\strong{Caption Formatting:}

Captions in the \code{caption} parameter are written as LaTeX comments in 
the output file for reference. For actual LaTeX captions, wrap the table 
in a table environment (see examples above).

\strong{Special Characters:}

The function automatically escapes LaTeX special characters in the data:
\itemize{
  \item Ampersand, percent, dollar sign, hash, underscore
  \item Left and right braces
  \item Tilde and caret (using \code{textasciitilde} and \code{textasciicircum})
}

Variable names and labels should not include these characters unless 
intentionally using LaTeX commands.
}
\examples{
data(clintrial)
data(clintrial_labels)

# Create example table
results <- fit(
   data = clintrial,
   outcome = "os_status",
   predictors = c("age", "sex", "treatment", "stage"),
   labels = clintrial_labels
)

# Example 1: Basic LaTeX export
if (requireNamespace("xtable", quietly = TRUE)) {
  table2tex(results, file.path(tempdir(), "basic.tex"))
}

\donttest{
# Example 2: With booktabs for publication
table2tex(results, file.path(tempdir(), "publication.tex"),
       booktabs = TRUE,
       caption = "Multivariable logistic regression results",
       label = "tab:regression")

# Example 3: Multi-line caption with abbreviations
table2tex(results, file.path(tempdir(), "detailed.tex"),
       booktabs = TRUE,
       caption = "Table 1: Risk Factors for Mortality\\\\\\\\
                 aOR = adjusted odds ratio; CI = confidence interval\\\\\\\\
                 Model adjusted for age, sex, treatment, and disease stage",
       label = "tab:mortality")

# Example 4: Hierarchical display with indentation
table2tex(results, file.path(tempdir(), "indented.tex"),
       indent_groups = TRUE,
       booktabs = TRUE)

# Example 5: Condensed table (reduced height)
table2tex(results, file.path(tempdir(), "condensed.tex"),
       condense_table = TRUE,
       booktabs = TRUE)

# Example 6: With zebra stripes
table2tex(results, file.path(tempdir(), "striped.tex"),
       zebra_stripes = TRUE,
       stripe_color = "gray!15",
       booktabs = TRUE)
# Remember to add \usepackage[table]{xcolor} to the LaTeX document

# Example 7: Dark header style
table2tex(results, file.path(tempdir(), "dark_header.tex"),
       dark_header = TRUE,
       booktabs = TRUE)
# Requires \usepackage[table]{xcolor}

# Example 8: Custom cell padding
table2tex(results, file.path(tempdir(), "relaxed.tex"),
       cell_padding = "relaxed",
       booktabs = TRUE)

# Example 9: Custom column alignment (auto-detected by default)
table2tex(results, file.path(tempdir(), "custom_align.tex"),
       align = c("c", "c", "c", "c", "c", "c", "c"))

# Example 10: No header formatting (keep original names)
table2tex(results, file.path(tempdir(), "raw_headers.tex"),
       format_headers = FALSE)

# Example 11: Disable significance bolding
table2tex(results, file.path(tempdir(), "no_bold.tex"),
       bold_significant = FALSE,
       booktabs = TRUE)

# Example 12: Stricter significance threshold
table2tex(results, file.path(tempdir(), "strict_sig.tex"),
       bold_significant = TRUE,
       p_threshold = 0.01,  # Bold only if p < 0.01
       booktabs = TRUE)

# Example 13: With caption size control
table2tex(results, file.path(tempdir(), "caption_size.tex"),
       caption_size = 6,
       caption = "Table 1 - Results with Compact Caption\\\\\\\\
                 Smaller caption fits better on constrained pages")

# Example 14: Complete publication-ready table
table2tex(results, file.path(tempdir(), "final_table1.tex"),
       booktabs = TRUE,
       caption = "Table 1: Multivariable Analysis of Mortality Risk Factors",
       label = "tab:main_results",
       indent_groups = TRUE,
       zebra_stripes = FALSE,  # Many journals prefer no stripes
       bold_significant = TRUE,
       cell_padding = "normal")

# Example 15: Descriptive statistics table
desc_table <- desctable(clintrial, by = "treatment",
   variables = c("age", "sex", "bmi"), labels = clintrial_labels)

table2tex(desc_table, file.path(tempdir(), "table1_descriptive.tex"),
       booktabs = TRUE,
       caption = "Table 1: Baseline Characteristics",
       label = "tab:baseline")

# Example 16: Model comparison table
models <- list(
   base = c("age", "sex"),
   full = c("age", "sex", "treatment", "stage")
)

# Information criteria assume a common sample, so the candidate
# predictors are restricted to complete cases before comparison
comparison_data <- na.omit(clintrial[, c("os_status", "age", "sex", "treatment", "stage")])

comparison <- compfit(
   data = comparison_data,
   outcome = "os_status",
   model_list = models
)

table2tex(comparison, file.path(tempdir(), "model_comparison.tex"),
       booktabs = TRUE,
       caption = "Model Comparison Statistics",
       label = "tab:models")

}
}
\seealso{
\code{\link{tablesave}} for saving in the format given by the file extension,
\code{\link{table2pdf}} for direct PDF output,
\code{\link{table2html}} for HTML tables,
\code{\link{table2docx}} for Word documents,
\code{\link{table2pptx}} for PowerPoint,
\code{\link{table2rtf}} for Rich Text Format,
\code{\link{fit}} for regression tables,
\code{\link{desctable}} for descriptive tables

Other export functions:
\code{\link[=table2docx]{table2docx()}},
\code{\link[=table2html]{table2html()}},
\code{\link[=table2pdf]{table2pdf()}},
\code{\link[=table2pptx]{table2pptx()}},
\code{\link[=table2rtf]{table2rtf()}},
\code{\link[=tablesave]{tablesave()}}
}
\concept{export functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_model_xlevels}
\alias{get_model_xlevels}
\title{Get factor levels from model (works with S3 and S4)}
\usage{
get_model_xlevels(model)
}
\arguments{
\item{model}{Fitted model object (S3 or S4).}
}
\value{
Named list of factor levels, or \code{NULL} if no factors present.
}
\description{
Extracts factor level information from fitted model objects. Handles both
S3 models (glm, lm, coxph) via xlevels slot and S4 models (lme4) via
the model frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{identify_variable_groups}
\alias{identify_variable_groups}
\title{Identify variable groups before indentation}
\usage{
identify_variable_groups(df)
}
\arguments{
\item{df}{Data.table with Variable column.}
}
\value{
List of integer vectors, each containing row indices for one variable group.
}
\description{
Detects variable group boundaries by finding rows where Variable column
is non-empty. Returns row indices for each group for zebra stripe application.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_column_headers}
\alias{format_column_headers}
\title{Apply formatting to column headers in exported tables (PDF/LaTeX)}
\usage{
format_column_headers(col_names, add_header_space = TRUE)
}
\arguments{
\item{col_names}{Character vector of column names.}

\item{add_header_space}{Logical whether to add vertical padding.}
}
\value{
Character vector with LaTeX-formatted column names.
}
\description{
Formats column headers for LaTeX output by escaping special characters,
italicizing 'n' and 'p', and optionally adding vertical spacing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{is_surv_outcome}
\alias{is_surv_outcome}
\title{Check if outcome is a Surv() expression}
\usage{
is_surv_outcome(outcome)
}
\arguments{
\item{outcome}{Character string of the outcome specification.}
}
\value{
Logical \code{TRUE} if outcome starts with "Surv(", \code{FALSE} otherwise.
}
\description{
Tests whether an outcome specification string represents a survival outcome
by checking for the Surv() function pattern. Used to route model fitting
to Cox proportional hazards methods.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{format_model_table}
\alias{format_model_table}
\title{Format model results for publication-ready display}
\usage{
format_model_table(
  data,
  effect_col = NULL,
  digits = 2,
  p_digits = 3,
  labels = NULL,
  show_n = TRUE,
  show_events = TRUE,
  reference_label = "reference",
  exponentiate = NULL,
  conf_level = 0.95,
  marks = NULL
)
}
\arguments{
\item{data}{Data.table containing raw model results with coefficient columns.}

\item{effect_col}{Optional character string specifying the effect column name.
If \code{NULL}, auto-detects from OR, HR, RR, or Estimate columns.}

\item{digits}{Integer number of decimal places for effect estimates.}

\item{p_digits}{Integer number of decimal places for \emph{p}-values.}

\item{labels}{Optional named character vector mapping variable names to display labels.
Supports automatic labeling of interaction terms.}

\item{show_n}{Logical whether to include sample size column.}

\item{show_events}{Logical whether to include events column (ignored for linear models).}

\item{reference_label}{Character string to display for reference categories.}

\item{exponentiate}{Optional logical to force exponentiated (TRUE) or raw (FALSE)
coefficient display. If \code{NULL}, uses existing columns.}
}
\value{
Formatted data.table with publication-ready columns.
}
\description{
Transforms raw model coefficient data into a formatted table suitable for
publication. Handles effect measure formatting (OR, HR, RR, Estimate),
confidence intervals, \emph{p}-values, sample sizes, and variable labels.
Supports interaction terms and mixed-effects models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{process_table_for_flextable}
\alias{process_table_for_flextable}
\title{Core flextable processing function}
\usage{
process_table_for_flextable(
  table,
  caption = NULL,
  font_size = 10,
  font_family = "Arial",
  format_headers = TRUE,
  bold_significant = TRUE,
  p_threshold = 0.05,
  indent_groups = FALSE,
  condense_table = FALSE,
  condense_quantitative = FALSE,
  zebra_stripes = FALSE,
  dark_header = FALSE,
  bold_variables = TRUE,
  paper = "letter",
  orientation = "portrait",
  width = NULL,
  align = NULL
)
}
\arguments{
\item{table}{Data.frame or data.table to process.}

\item{caption}{Optional character string for table caption.}

\item{font_size}{Numeric font size in points.}

\item{font_family}{Character string font family name.}

\item{format_headers}{Logical whether to format headers.}

\item{bold_significant}{Logical whether to bold significant \emph{p}-values.}

\item{p_threshold}{Numeric \emph{p}-value threshold for significance.}

\item{indent_groups}{Logical whether to indent group levels.}

\item{condense_table}{Logical whether to condense all variable types.}

\item{condense_quantitative}{Logical whether to condense only continuous/survival.}

\item{zebra_stripes}{Logical whether to apply alternating row shading.}

\item{dark_header}{Logical whether to use dark header style.}

\item{bold_variables}{Logical whether to bold variable names (non-indented rows).}

\item{paper}{Character string paper size.}

\item{orientation}{Character string page orientation.}

\item{width}{Optional numeric table width in inches.}

\item{align}{Optional alignment specification.}
}
\value{
List with ft (\pkg{flextable} object) and caption components.
}
\description{
Central processing function for creating \pkg{flextable} objects from data tables.
Handles N row extraction, condensing, indentation, zebra stripes, formatting,
and styling. Used by table2docx, table2pptx, and table2rtf.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2docx.R
\name{table2docx}
\alias{table2docx}
\title{Export Table to Microsoft Word Format (DOCX)}
\usage{
table2docx(
  table,
  file,
  caption = NULL,
  font_size = 8,
  font_family = "Arial",
  format_headers = TRUE,
  bold_significant = TRUE,
  bold_variables = FALSE,
  p_threshold = 0.05,
  indent_groups = FALSE,
  condense_table = FALSE,
  condense_quantitative = FALSE,
  zebra_stripes = FALSE,
  dark_header = FALSE,
  paper = "letter",
  orientation = "portrait",
  width = NULL,
  align = NULL,
  return_ft = FALSE,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{table}{Data frame, data.table, or matrix to export. Can be output from 
\code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
\code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.}

\item{file}{Character string specifying the output DOCX filename. Must have 
\code{.docx} extension. Example: \code{"results.docx"}, \code{"Table1.docx"}.}

\item{caption}{Character string. Optional caption displayed above the table 
in the Word document. Default is \code{NULL}.}

\item{font_size}{Numeric. Base font size in points for table content. 
Default is 8. Typical range: 8-12 points. Headers use slightly larger size.}

\item{font_family}{Character string. Font family name for the table. Must be 
a font installed on the system. Default is \code{"Arial"}. Common options: 
\code{"Times New Roman"}, \code{"Calibri"}, \code{"Helvetica"}.}

\item{format_headers}{Logical. If \code{TRUE}, formats column headers by 
italicizing statistical notation ("\emph{n}", "\emph{p}"), converting underscores 
to spaces, and improving readability. Default is \code{TRUE}.}

\item{bold_significant}{Logical. If \code{TRUE}, applies bold formatting to 
\emph{p}-values below the significance threshold. Makes significant results stand 
out. Default is \code{TRUE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. Default is \code{FALSE}.}

\item{p_threshold}{Numeric. Threshold for bold \emph{p}-value formatting. Only 
used when \code{bold_significant = TRUE}. Default is 0.05.}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels under 
their parent variable using horizontal spacing, creating hierarchical display. 
Useful for categorical variables in regression tables. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses table by showing 
only essential rows (single row for continuous, non-reference for binary). 
Automatically sets \code{indent_groups = TRUE}. Significantly reduces table 
height. Default is \code{FALSE}.}

\item{condense_quantitative}{Logical. If \code{TRUE}, condenses continuous 
and survival variables into single rows while preserving all categorical 
variable rows (including binary). Only applies to descriptive tables from 
\code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
\code{condense_table}, this does not collapse binary categorical variables. 
Default is \code{FALSE}.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating row shading 
to different variables (not individual rows) for visual grouping. 
Default is \code{FALSE}.}

\item{dark_header}{Logical. If \code{TRUE}, creates a dark background with 
light text for the header row, providing strong visual contrast. 
Default is \code{FALSE}.}

\item{paper}{Character string specifying paper size:
\itemize{
  \item \code{"letter"} - US Letter (8.5" × 11") [default]
  \item \code{"a4"} - A4 (210 mm × 297 mm)
  \item \code{"legal"} - US Legal (8.5" × 14")
}}

\item{orientation}{Character string specifying page orientation:
\itemize{
  \item \code{"portrait"} - Vertical [default]
  \item \code{"landscape"} - Horizontal (for wide tables)
}}

\item{width}{Numeric. Table width in inches. If \code{NULL} (default), 
automatically fits to content and page width. Specify to control exactly.}

\item{align}{Character vector specifying column alignment for each column. 
Options: \code{"left"}, \code{"center"}, or \code{"right"}. If \code{NULL} 
(default), automatically determines based on content (text left, numbers right).
Example: \code{c("left", "left", "center", "right", "right")}.}

\item{return_ft}{Logical. If \code{TRUE}, returns the \pkg{flextable} object 
directly for further customization. If \code{FALSE} (default), returns 
invisibly with \pkg{flextable} object as attribute. See Details for usage. 
Default is \code{FALSE}.}

\item{quiet}{Logical. Suppress progress and confirmation messages, such
as the notice reporting the file written. Errors and warnings are
unaffected. Default is \code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[officer]{read_docx}} 
for document initialization.}
}
\value{
Behavior depends on \code{return_ft}:
  \describe{
    \item{\code{return_ft = FALSE}}{Invisibly returns a list with components:
      \itemize{
        \item \code{file} - Path to created file
        \item \code{caption} - Caption text (if provided)
      }
      The \pkg{flextable} object is accessible via \code{attr(result, "flextable")}
    }
    \item{\code{return_ft = TRUE}}{Directly returns the \pkg{flextable} object for 
      immediate further customization}
  }
  
  In both cases, creates a \code{.docx} file at the specified location.
}
\description{
Converts a data frame, data.table, or matrix to a fully editable Microsoft Word 
document (\code{.docx}) using the \pkg{flextable} and \pkg{officer} packages.
Creates publication-ready tables with extensive formatting options including
typography, alignment, colors, and page layout. Tables can be further edited in
Microsoft Word after creation.
}
\details{
\strong{Package Requirements:}

This function requires:
\itemize{
  \item \strong{\pkg{flextable}} - For creating formatted tables
  \item \strong{\pkg{officer}} - For Word document manipulation
}

Install if needed:
\preformatted{
install.packages(c("flextable", "officer"))
}

\strong{Output Features:}

The generated Word document contains:
\itemize{
  \item Fully editable table (native Word table, not image)
  \item Professional typography and spacing
  \item Proper page setup (size, orientation, margins)
  \item Caption (if provided) as separate paragraph above table
  \item All formatting preserved but editable
  \item Compatible with Word 2007 and later
}

\strong{Further Customization:}

For programmatic customization beyond the built-in options, access the 
\code{flextable} object:

\emph{Method 1: Via attribute (default)}

\preformatted{
result <- table2docx(table, "output.docx")
ft <- attr(result, "flextable")

# Customize flextable
ft <- flextable::bold(ft, i = 1, j = 1, part = "body")
ft <- flextable::color(ft, i = 2, j = 3, color = "red")

# Re-save if needed
doc <- officer::read_docx()
doc <- flextable::body_add_flextable(doc, ft)
print(doc, target = "customized.docx")
}

\emph{Method 2: Direct return}

\preformatted{
ft <- table2docx(table, "output.docx", return_ft = TRUE)

# Customize immediately
ft <- flextable::bg(ft, bg = "yellow", part = "header")
ft <- flextable::autofit(ft)

# Save to new document
doc <- officer::read_docx()
doc <- flextable::body_add_flextable(doc, ft)
print(doc, target = "custom.docx")
}

\strong{Page Layout:}

The function automatically sets up the Word document with:
\itemize{
  \item Specified paper size and orientation
  \item Standard margins (1 inch by default)
  \item Continuous section (no page breaks before table)
  \item Left-aligned table placement
}

For landscape orientation:
\itemize{
  \item Automatically swaps page width and height
  \item Applies landscape property to section
  \item Useful for wide tables with many columns
}

\strong{Table Width Management:}

Width behavior:
\itemize{
  \item \code{width = NULL} - Auto-fits to content and page width
  \item \code{width = 6} - Exactly 6 inches wide
  \item Width distributed evenly across columns by default
  \item Can adjust individual column widths in Word after creation
}

For very wide tables:
\enumerate{
  \item Use \code{orientation = "landscape"}
  \item Use \code{paper = "legal"} for extra width
  \item Reduce \code{font_size}
  \item Use \code{condense_table = TRUE}
  \item Consider breaking across multiple tables
}

\strong{Typography:}

The function applies professional typography:
\itemize{
  \item Column headers: Bold, slightly larger font
  \item Body text: Regular weight, specified font size
  \item Numbers: Right-aligned for easy comparison
  \item Text: Left-aligned for readability
  \item Consistent spacing: Adequate padding in cells
}

Font family must be installed on the system where Word opens the document. 
Common cross-platform choices:
\itemize{
  \item Arial - Sans-serif, highly readable
  \item Times New Roman - Serif, traditional
  \item Calibri - Microsoft default, modern
  \item Helvetica - Sans-serif, professional
}

\strong{Zebra Striping:}

When \code{zebra_stripes = TRUE}:
\itemize{
  \item Alternating variables receive light gray background
  \item All rows of same variable share same shading
  \item Improves visual grouping
  \item Particularly useful for tables with many factor variables
  \item Color can be changed in Word after creation
}

\strong{Dark Header:}

When \code{dark_header = TRUE}:
\itemize{
  \item Header row: Dark gray/black background
  \item Header text: White for high contrast
  \item Modern, professional appearance
  \item Draws attention to column names
}

\strong{Integration with R Markdown/Quarto:}

For R Markdown/Quarto Word output:

\preformatted{
# Create flextable for inline display
ft <- table2docx(results, "temp.docx", return_ft = TRUE)

# Display in R Markdown chunk
ft  # Renders in Word output
}

Or use \code{flextable} directly in chunks:

\preformatted{
flextable::flextable(results)
}
}
\examples{
data(clintrial)
data(clintrial_labels)

# Create example table
results <- fit(
   data = clintrial,
   outcome = "os_status",
   predictors = c("age", "sex", "treatment", "stage"),
   labels = clintrial_labels
)

# Example 1: Basic Word export
if (requireNamespace("flextable", quietly = TRUE) &&
    requireNamespace("officer", quietly = TRUE)) {
  table2docx(results, file.path(tempdir(), "results.docx"))
}

\donttest{
old_width <- options(width = 180)
# Example 2: With caption
table2docx(results, file.path(tempdir(), "captioned.docx"),
        caption = "Table 1: Multivariable Logistic Regression Results")

# Example 3: Landscape orientation for wide tables
table2docx(results, file.path(tempdir(), "wide.docx"),
        orientation = "landscape")

# Example 4: Custom font and size
table2docx(results, file.path(tempdir(), "custom_font.docx"),
        font_family = "Times New Roman",
        font_size = 11)

# Example 5: Hierarchical display
table2docx(results, file.path(tempdir(), "indented.docx"),
        indent_groups = TRUE)

# Example 6: Condensed table
table2docx(results, file.path(tempdir(), "condensed.docx"),
        condense_table = TRUE)

# Example 7: With zebra stripes
table2docx(results, file.path(tempdir(), "striped.docx"),
        zebra_stripes = TRUE)

# Example 8: Dark header style
table2docx(results, file.path(tempdir(), "dark.docx"),
        dark_header = TRUE)

# Example 9: A4 paper for international journals
table2docx(results, file.path(tempdir(), "a4.docx"),
        paper = "a4")

# Example 10: Get flextable for customization
result <- table2docx(results, file.path(tempdir(), "base.docx"))
ft <- attr(result, "flextable")

# Customize the flextable
ft <- flextable::bold(ft, i = 1, part = "body")
ft <- flextable::color(ft, j = "p-value", color = "blue")

# Example 11: Direct flextable return
ft <- table2docx(results, file.path(tempdir(), "direct.docx"), return_ft = TRUE)
ft <- flextable::bg(ft, bg = "yellow", part = "header")

# Example 12: Publication-ready table
table2docx(results, file.path(tempdir(), "publication.docx"),
        caption = "Table 2: Adjusted Odds Ratios for Mortality",
        font_family = "Times New Roman",
        font_size = 10,
        indent_groups = TRUE,
        zebra_stripes = FALSE,
        bold_significant = TRUE)

# Example 13: Custom column alignment
table2docx(results, file.path(tempdir(), "aligned.docx"),
        align = c("left", "left", "center", "right", "right"))

# Example 14: Disable significance bolding
table2docx(results, file.path(tempdir(), "no_bold.docx"),
        bold_significant = FALSE)

# Example 15: Stricter significance threshold
table2docx(results, file.path(tempdir(), "strict.docx"),
        bold_significant = TRUE,
        p_threshold = 0.01)

options(old_width)
}
}
\seealso{
\code{\link{tablesave}} for saving in the format given by the file extension,
\code{\link{table2pptx}} for PowerPoint slides,
\code{\link{table2pdf}} for PDF output,
\code{\link{table2html}} for HTML tables,
\code{\link{table2rtf}} for Rich Text Format,
\code{\link{table2tex}} for LaTeX output,
\code{\link[flextable]{flextable}} for the underlying table object,
\code{\link[officer]{read_docx}} for Word document manipulation

Other export functions:
\code{\link[=table2html]{table2html()}},
\code{\link[=table2pdf]{table2pdf()}},
\code{\link[=table2pptx]{table2pptx()}},
\code{\link[=table2rtf]{table2rtf()}},
\code{\link[=table2tex]{table2tex()}},
\code{\link[=tablesave]{tablesave()}}
}
\concept{export functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tablesave.R
\name{tablesave}
\alias{tablesave}
\alias{autotable}
\title{Save a Table to File}
\usage{
tablesave(table, file, quiet = FALSE, ...)

autotable(table, file, quiet = FALSE, ...)
}
\arguments{
\item{table}{A table produced by \code{desctable()}, \code{survtable()},
\code{fit()}, \code{uniscreen()}, \code{fullfit()}, \code{compfit()}, or
\code{multifit()}. Any data frame, data.table, or matrix is accepted.}

\item{file}{Character string specifying the output filename. The file extension
determines the export format:
\itemize{
  \item \code{.csv} - Comma-separated values (uses \code{data.table::fwrite()})
  \item \code{.tsv} - Tab-separated values (uses \code{data.table::fwrite()})
  \item \code{.pdf} - PDF via LaTeX (uses \code{table2pdf()})
  \item \code{.docx} - Microsoft Word (uses \code{table2docx()})
  \item \code{.html} or \code{.htm} - HTML (uses \code{table2html()})
  \item \code{.pptx} - Microsoft PowerPoint (uses \code{table2pptx()})
  \item \code{.tex} - LaTeX source (uses \code{table2tex()})
  \item \code{.rtf} - Rich Text Format (uses \code{table2rtf()})
}}

\item{quiet}{Logical. Suppress progress and confirmation messages. The
setting is forwarded to the format-specific export function, so the
LaTeX compilation notices from \code{table2pdf()} are suppressed with
it. Default is \code{FALSE}.}

\item{...}{Additional arguments passed to the format-specific function. See
the documentation for individual functions for available parameters:
\describe{
  \item{PDF}{\code{table2pdf()} - \code{orientation}, \code{paper}, \code{margins},
             \code{fit_to_page}, \emph{etc.}}
  \item{DOCX}{\code{table2docx()} - \code{font_size}, \code{font_family},
             \code{caption}, \emph{etc.}}
  \item{HTML}{\code{table2html()} - \code{format_headers}, \code{zebra_stripes},
             \emph{etc.}}
  \item{PPTX}{\code{table2pptx()} - \code{font_size}, \code{font_family},
             \code{caption}, \emph{etc.}}
  \item{TEX}{\code{table2tex()} - \code{caption}, \code{format_headers},
             \code{align}, \emph{etc.}}
  \item{RTF}{\code{table2rtf()} - \code{font_size}, \code{font_family},
             \code{caption}, \emph{etc.}}
  \item{CSV, TSV}{\code{data.table::fwrite()} - \code{sep}, \code{quote},
             \code{na}, \emph{etc.}}
}

Common parameters across formats include:
\describe{
  \item{\code{caption}}{Table caption (supported by most formats)}
  \item{\code{font_size}}{Base font size in points (PDF, DOCX, PPTX, RTF)}
  \item{\code{format_headers}}{Format column headers (all formats)}
  \item{\code{bold_significant}}{Bold significant \emph{p}-values (all formats)}
  \item{\code{p_threshold}}{Threshold for \emph{p}-value bolding (all formats)}
  \item{\code{indent_groups}}{Indent factor levels (all formats)}
  \item{\code{condense_table}}{Condense to essential rows (all formats)}
  \item{\code{zebra_stripes}}{Alternating background colors (most formats)}
}}
}
\value{
Invisibly returns \code{file}.
}
\description{
Writes a table to file in the format indicated by the file extension,
dispatching to the appropriate specialized export function. Provides a
unified interface for table export across all supported formats, and is the
counterpart to \code{forestsave()} for forest plots.
}
\details{
This function provides a convenient wrapper around format-specific export 
functions, automatically routing to the appropriate function based on the 
file extension. All parameters are passed through to the underlying function,
so the full range of format-specific options remains available.

For format-specific advanced features, individual export functions may be
called directly:
\itemize{
  \item PDF exports support orientation, paper size, margins, and auto-sizing
  \item DOCX/PPTX/RTF support font customization and \pkg{flextable} formatting
  \item HTML supports CSS styling, responsive design, and custom themes
  \item TeX generates standalone LaTeX source with booktabs styling
}
}
\examples{
# Create example data
data(clintrial)
data(clintrial_labels)
tbl <- desctable(clintrial, by = "treatment",
    variables = c("age", "sex"), labels = clintrial_labels)

# Example 1: The format follows the file extension
if (requireNamespace("xtable", quietly = TRUE)) {
  tablesave(tbl, file.path(tempdir(), "example.html"))
}

# Example 2: Delimited output needs no additional packages
tablesave(tbl, file.path(tempdir(), "example.csv"))

# Example 3: Suppress the message reporting the file written
tablesave(tbl, file.path(tempdir(), "example.tsv"), quiet = TRUE)

\donttest{

# Load example data
data(clintrial)
data(clintrial_labels)

# Create a regression table
results <- fit(
    data = clintrial,
    outcome = "os_status",
    predictors = c("age", "sex", "treatment"),
    labels = clintrial_labels
)

# Test that LaTeX can actually compile (needed for PDF export)
has_latex <- local({
  if (!nzchar(Sys.which("pdflatex"))) return(FALSE)
  test_tex <- file.path(tempdir(), "summata_latex_test.tex")
  writeLines(c("\\\\documentclass{article}",
               "\\\\usepackage{booktabs}",
               "\\\\begin{document}", "test",
               "\\\\end{document}"), test_tex)
  result <- tryCatch(
    system2("pdflatex", c("-interaction=nonstopmode",
            paste0("-output-directory=", tempdir()), test_tex),
            stdout = FALSE, stderr = FALSE),
    error = function(e) 1L)
  result == 0L
})

# Example 4: The format is taken from the file extension
tablesave(results, file.path(tempdir(), "results.html"))  # Creates HTML file
tablesave(results, file.path(tempdir(), "results.docx"))  # Creates Word document
tablesave(results, file.path(tempdir(), "results.pptx"))  # Creates PowerPoint slide
tablesave(results, file.path(tempdir(), "results.tex"))   # Creates LaTeX source
tablesave(results, file.path(tempdir(), "results.rtf"))   # Creates RTF document
if (has_latex) {
  tablesave(results, file.path(tempdir(), "results.pdf")) # Creates PDF
}

# Example 5: Format-specific parameters are passed through
if (has_latex) {
  tablesave(results, file.path(tempdir(), "results.pdf"), 
             orientation = "landscape",
             paper = "a4",
             font_size = 10)
}

tablesave(results, file.path(tempdir(), "results.docx"),
           caption = "Table 1: Logistic Regression Results",
           font_family = "Times New Roman",
           condense_table = TRUE)

tablesave(results, file.path(tempdir(), "results.html"),
           zebra_stripes = TRUE,
           dark_header = TRUE,
           bold_significant = TRUE)

# Example 6: Any summata table may be saved
desc <- desctable(clintrial,
                  by = "treatment",
                  variables = c("age", "sex", "bmi"))
if (has_latex) {
  tablesave(desc, file.path(tempdir(), "demographics.pdf"))
}

# Example 7: Model comparison table
# Information criteria assume a common sample, so the candidate
# predictors are restricted to complete cases before comparison
comparison_data <- na.omit(
    clintrial[, c("os_status", "age", "sex", "treatment", "stage")]
)
comparison <- compfit(
    data = comparison_data,
    outcome = "os_status",
    model_list = list(
        base = c("age", "sex"),
        full = c("age", "sex", "treatment", "stage")
    )
)
tablesave(comparison, file.path(tempdir(), "model_comparison.docx"))

}

}
\seealso{
\code{\link{table2pdf}}, \code{\link{table2docx}}, \code{\link{table2pptx}},
\code{\link{table2html}}, \code{\link{table2rtf}}, \code{\link{table2tex}}

Other export functions:
\code{\link[=table2docx]{table2docx()}},
\code{\link[=table2html]{table2html()}},
\code{\link[=table2pdf]{table2pdf()}},
\code{\link[=table2pptx]{table2pptx()}},
\code{\link[=table2rtf]{table2rtf()}},
\code{\link[=table2tex]{table2tex()}}
}
\concept{export functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{combine_multifit_results}
\alias{combine_multifit_results}
\title{Combine multivariate results}
\usage{
combine_multifit_results(all_results, columns)
}
\arguments{
\item{all_results}{List of results from fit_one_outcome calls.}

\item{columns}{Character specifying which columns to include.}
}
\value{
Combined data.table.
}
\description{
Internal helper to combine unadjusted and/or adjusted results from 
multiple outcomes into a single data.table.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glmforest.R
\name{glmforest}
\alias{glmforest}
\title{Create Forest Plot for Generalized Linear Models}
\usage{
glmforest(
  x,
  data = NULL,
  title = "Generalized Linear Model",
  effect_label = NULL,
  digits = 2,
  p_digits = 3,
  conf_level = 0.95,
  font_size = 1,
  annot_size = 3.88,
  header_size = 5.82,
  title_size = 23.28,
  plot_width = NULL,
  plot_height = NULL,
  table_width = 0.6,
  show_n = TRUE,
  show_events = TRUE,
  indent_groups = FALSE,
  condense_table = FALSE,
  bold_variables = FALSE,
  center_padding = 4,
  zebra_stripes = TRUE,
  ref_label = "reference",
  labels = NULL,
  color = NULL,
  exponentiate = NULL,
  qc_footer = TRUE,
  units = "in",
  number_format = NULL
)
}
\arguments{
\item{x}{Either a fitted GLM object (class \code{glm} or \code{glmerMod}), 
a \code{fit_result} object from \code{fit()}, or a \code{fullfit_result}
object from \code{fullfit()}. When a \code{fit_result} or \code{fullfit_result}
is provided, the model, data, and labels are automatically extracted.}

\item{data}{Data frame or data.table containing the original data used to 
fit the model. If \code{NULL} (default) and \code{x} is a model, the function 
attempts to extract data from the model object. If \code{x} is a \code{fit_result},
data is extracted automatically. Providing data explicitly is recommended when
passing a model directly.}

\item{title}{Character string specifying the plot title displayed at the top. 
Default is \code{"Generalized Linear Model"}. Use descriptive titles like 
"Risk Factors for Disease Outcome" for publication.}

\item{effect_label}{Character string for the effect measure label on the 
forest plot axis. If \code{NULL} (default), automatically determined based 
on model family and link function: "Odds Ratio" for logistic regression 
(\code{family = binomial, link = logit}), "Risk Ratio" for log-link models, 
"Exp(Coefficient)" for other exponential families, or "Coefficient" for 
identity link.}

\item{digits}{Integer specifying the number of decimal places for effect 
estimates and confidence intervals in the data table. Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be
between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
when \code{conf_level = 0.90}).}

\item{font_size}{Numeric multiplier controlling the base font size for all 
text elements. Values > 1 increase all fonts proportionally, values < 1 
decrease them. Default is 1.0. Useful for adjusting readability across 
different output sizes.}

\item{annot_size}{Numeric value controlling the relative font size for 
data annotations (variable names, values in table cells). Default is 3.88. 
Adjust relative to \code{font_size}.}

\item{header_size}{Numeric value controlling the relative font size for 
column headers ("Variable", "Group", "n", \emph{etc.}). Default is 5.82. Headers 
are typically larger than annotations for hierarchy.}

\item{title_size}{Numeric value controlling the relative font size for the 
main plot title. Default is 23.28. The title is typically the largest text 
element.}

\item{plot_width}{Numeric value specifying the intended output width in 
specified \code{units}. Used for optimizing layout and text sizing. 
Default is \code{NULL} (automatic). Recommended: 10-16 inches for standard 
publications.}

\item{plot_height}{Numeric value specifying the intended output height in 
specified \code{units}. Default is \code{NULL} (automatic based on number 
of rows). The function provides recommendations if not specified.}

\item{table_width}{Numeric value between 0 and 1 specifying the proportion of 
total plot width allocated to the data table (left side). The forest plot 
occupies \code{1 - table_width}. Default is 0.6 (60\% table, 40\% forest). 
Increase for longer variable names, decrease to emphasize the forest plot.}

\item{show_n}{Logical. If \code{TRUE}, includes a column showing group-specific 
sample sizes for categorical variables and the fitted sample size for
continuous variables. Counts describe the observations used in fitting
rather than every row supplied. Default is \code{TRUE}.}

\item{show_events}{Logical. If \code{TRUE}, includes a column showing the 
number of events for each group. Relevant for logistic regression (number 
of cases) and other binary outcomes. Default is \code{TRUE}.}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels under 
their parent variable name, creating a hierarchical visual structure. When 
\code{TRUE}, the "Group" column is hidden. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses binary categorical 
variables into single rows by showing only the non-reference category. 
Automatically sets \code{indent_groups = TRUE}. Useful for tables with 
many binary variables. Default is \code{FALSE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. If \code{FALSE} (default), variable names are displayed in plain
text.}

\item{center_padding}{Numeric value specifying the horizontal spacing (in 
character units) between the data table and forest plot. Increase for more 
separation, decrease to fit more content. Default is 4.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating gray 
background shading to different variables (not rows) to improve visual 
grouping and readability. Default is \code{TRUE}.}

\item{ref_label}{Character string to display for reference categories of 
factor variables. Typically shown in place of effect estimates. 
Default is \code{"reference"}. Common alternatives: "ref", "1.00 (ref)".}

\item{labels}{Named character vector or list providing custom display 
labels for variables. Names should match variable names in the model, 
values are the labels to display. Example: 
\code{c(age = "Age (years)", bmi = "Body Mass Index")}. Default is
\code{NULL}, in which case the labels attached by the function that produced
\code{x} are used, where \code{x} is a \pkg{summata} result rather than a
model object. Original variable names are used where neither is available.}

\item{color}{Character string specifying the color for effect estimate point 
  markers in the forest plot. Use hex codes or R color names. Default is 
  \code{NULL}, which auto-selects based on effect type: \code{"#4BA6B6"} 
  (teal) for odds ratios (binomial/quasibinomial with logit link), 
  \code{"#3F87EE"} (blue) for rate/risk ratios (Poisson, Gamma, inverse 

  Gaussian with log link), and \code{"#5A8F5A"} (green) for coefficients 
  (Gaussian/identity link). This scheme matches \code{uniforest()} and 
  \code{multiforest()}. Choose colors that contrast well with black error bars.}

\item{exponentiate}{Logical. If \code{TRUE}, exponentiates coefficients to 
display odds ratios, risk ratios, \emph{etc.} If \code{FALSE}, shows raw 
coefficients. Default is \code{NULL}, which automatically exponentiates 
for logit, log, and cloglog links, and shows raw coefficients for identity 
link.}

\item{qc_footer}{Logical. If \code{TRUE}, displays model quality control
statistics in the footer (observations analyzed, model family, deviance,
pseudo-\emph{R}\eqn{^2}, AIC). Default is \code{TRUE}.}

\item{units}{Character string specifying the units for plot dimensions. 
Options: \code{"in"} (inches), \code{"cm"} (centimeters), \code{"mm"} 
(millimeters). Default is \code{"in"}. Affects interpretation of 
\code{plot_width} and \code{plot_height}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}
}
\value{
A \code{ggplot} object containing the complete forest plot. The plot 
  can be:
  \itemize{
    \item Displayed directly: \code{print(plot)}
    \item Saved to file: \code{forestsave(plot, "forest.pdf")}
    \item Further customized with \pkg{ggplot2} functions
  }
  
  The returned object includes an attribute \code{"rec_dims"} 
  accessible via \code{attr(plot, "rec_dims")}, which is a list 
  containing:
  \describe{
    \item{width}{Numeric. Recommended plot width in specified units}
    \item{height}{Numeric. Recommended plot height in specified units}
  }
  
  These recommendations are automatically calculated based on the number of 
  variables, text sizes, and layout parameters, and are printed to console 
  if \code{plot_width} or \code{plot_height} are not specified. The list also 
  carries a \code{units} element recording the units the dimensions are 
  expressed in, matching the \code{units} argument. \code{forestsave()} reads 
  all three and requires no further handling.

  The returned object also includes an attribute \code{"table_data"}
  accessible via \code{attr(plot, "table_data")}, a data.table holding the
  values drawn in the plot: one row per term in model order, with the
  variable, factor level, sample size, event count, estimate, confidence
  bounds and p-value. Sample sizes and event counts describe the
  observations used in fitting rather than every row supplied, so they sum
  to the model sample size within each variable.
}
\description{
Generates a publication-ready forest plot that combines a formatted data table 
with a graphical representation of effect estimates (odds ratios, risk ratios, 
or coefficients) from a generalized linear model. The plot integrates variable 
names, group levels, sample sizes, effect estimates with confidence intervals, 
\emph{p}-values, and model diagnostics in a single comprehensive visualization designed 
for manuscripts and presentations.
}
\details{
\strong{Plot Components:}

The forest plot consists of several integrated components:
\enumerate{
  \item \strong{Title}: Centered at top, describes the analysis
  \item \strong{Data Table} (left side): Contains columns for:
    \itemize{
      \item Variable: Predictor names (or custom labels)
      \item Group: Factor levels (optional, hidden when indenting)
      \item n: Sample sizes by group (optional)
      \item Events: Event counts by group (optional)
      \item Effect (95\% CI); \emph{p}-value: Formatted estimates with \emph{p}-values
    }
  \item \strong{Forest Plot} (right side): Graphical display with:
    \itemize{
      \item Point estimates (squares sized by sample size)
      \item 95\% confidence intervals (error bars)
      \item Reference line (at OR/RR = 1 or coefficient = 0)
      \item Log scale for odds/risk ratios
      \item Labeled axis
    }
  \item \strong{Model Statistics} (footer): Summary of:
    \itemize{
      \item Observations analyzed (with total observations and percentage)
      \item Model family (Binomial, Poisson, \emph{etc.})
      \item Deviance statistics
      \item Pseudo-\emph{R}\eqn{^2} (McFadden)
      \item AIC
    }
}

\strong{Automatic Effect Measure Selection:}

When \code{effect_label = NULL} and \code{exponentiate = NULL}, the function 
intelligently selects the appropriate effect measure:
\itemize{
  \item \strong{Logistic regression} (\code{family = binomial(link = "logit")}): 
    Odds Ratios (OR)
  \item \strong{Log-link models} (\code{link = "log"}): Risk Ratios (RR) 
    or Rate Ratios
  \item \strong{Other exponential families}: exp(coefficient)
  \item \strong{Identity link}: Raw coefficients
}

\strong{Reference Categories:}

For factor variables, the first level (determined by factor ordering or 
alphabetically for character variables) serves as the reference category:
\itemize{
  \item Displayed with the \code{ref_label} instead of an estimate
  \item No confidence interval or \emph{p}-value shown
  \item Visually aligned with other categories
  \item When \code{condense_table = TRUE}, reference-only variables may be 
    omitted entirely
}

\strong{Layout Optimization:}

The function automatically optimizes layout based on content:
\itemize{
  \item Calculates appropriate axis ranges to accommodate all confidence intervals
  \item Selects meaningful tick marks on log or linear scales
  \item Sizes point markers proportional to sample size (larger = more data)
  \item Adjusts table width based on variable name lengths when \code{table_width = NULL}
  \item Recommends overall dimensions based on number of rows
}

\strong{Visual Grouping Options:}

Three display modes are available:
\enumerate{
  \item \strong{Standard} (\code{indent_groups = FALSE}, 
    \code{condense_table = FALSE}): 
    Separate "Variable" and "Group" columns, all categories shown
  \item \strong{Indented} (\code{indent_groups = TRUE}, 
    \code{condense_table = FALSE}): 
    Hierarchical display with groups indented under variables
  \item \strong{Condensed} (\code{condense_table = TRUE}): 
    Binary variables shown in single rows, automatically indented
}

\strong{Zebra Striping:}

When \code{zebra_stripes = TRUE}, alternating variables (not individual rows) 
receive light gray backgrounds. This helps visually group all levels of a 
factor variable together, making the plot easier to read especially with 
many multi-level factors.

\strong{Model Statistics Display:}

The footer shows key diagnostic information:
\itemize{
  \item \strong{Observations analyzed}: Observations used in fitting, the 
    total supplied, and the percentage retained (accounting for missing 
    values)
  \item \strong{Null/Residual Deviance}: Model fit improvement
  \item \strong{Pseudo-\emph{R}\eqn{^2}}: McFadden \emph{R}\eqn{^2} = 1 - (log L_1 / log L_2)
  \item \strong{AIC}: For model comparison (lower is better)
}

For logistic regression, concordance (C-statistic/AUC) may also be displayed 
if available.

\strong{Saving Plots:}

Use \code{forestsave()}, which applies the recommended dimensions and
selects a graphics device suited to the format:
\preformatted{
  p <- glmforest(model, data)
  forestsave(p, "forest.pdf")
}

Or specify custom dimensions:
\preformatted{
forestsave(p, "forest.png", width = 12, height = 8, dpi = 300)
}
}
\examples{
data(clintrial)
data(clintrial_labels)

# Create example model
model1 <- glm(os_status ~ age + sex + bmi + treatment,
              data = clintrial, family = binomial)

# Example 1: Basic logistic regression forest plot
p <- glmforest(model1, data = clintrial)

\donttest{

old_width <- options(width = 180)

# Example 2: With custom variable labels
plot2 <- glmforest(
    x = model1,
    data = clintrial,
    title = "Risk Factors for Mortality",
    labels = clintrial_labels
)

# Example 3: Indented layout with formatting options
plot3 <- glmforest(
    x = model1,
    data = clintrial,
    indent_groups = TRUE,
    zebra_stripes = TRUE,
    color = "#D62728",
    labels = clintrial_labels
)

# Example 4: Condensed layout for many binary variables
model4 <- glm(os_status ~ age + sex + smoking + hypertension + 
                  diabetes + surgery,
              data = clintrial,
              family = binomial)

plot4 <- glmforest(
    x = model4,
    data = clintrial,
    condense_table = TRUE,
    labels = clintrial_labels
)
# Binary variables shown in single rows

# Example 5: Poisson regression for count data
model5 <- glm(ae_count ~ age + treatment + diabetes + surgery,
               data = clintrial,
               family = poisson)

plot5 <- glmforest(
    x = model5,
    data = clintrial,
    title = "Rate Ratios for Adverse Events",
    labels = clintrial_labels
)

# Example 6: Save with recommended dimensions
forestsave(plot5, file.path(tempdir(), "forest.pdf"))

options(old_width)

}
}
\seealso{
\code{\link{autoforest}} for automatic model detection,
\code{\link{coxforest}} for Cox proportional hazards forest plots,
\code{\link{lmforest}} for linear model forest plots,
\code{\link{uniforest}} for univariable screening forest plots,
\code{\link{multiforest}} for multi-outcome forest plots,
\code{\link[stats]{glm}} for fitting GLMs,
\code{\link{fit}} for regression modeling,
  \code{\link{forestsave}} for saving with recommended dimensions

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=coxforest]{coxforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=recdims]{recdims()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_outcome_variables}
\alias{get_outcome_variables}
\title{Identify the response variables of a fitted model}
\usage{
get_outcome_variables(model)
}
\arguments{
\item{model}{Fitted model object.}
}
\value{
Character vector of response variable names, or \code{character(0)}
  when the formula cannot be recovered.
}
\description{
Returns the variables appearing on the left-hand side of a model formula. A
survival response contributes both the time and status variables.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fullfit.R
\name{print.fullfit_result}
\alias{print.fullfit_result}
\title{Print method for fullfit results}
\usage{
\method{print}{fullfit_result}(x, ...)
}
\arguments{
\item{x}{fullfit_result object.}

\item{...}{Additional arguments passed to print methods.}
}
\value{
Invisibly returns the input object \code{x}. Called for its
  side effect of printing a formatted summary to the console.
}
\description{
Displays a summary header with outcome, model type, method, and number of
multivariable predictors before printing the results table.
}
\seealso{
Other regression functions:
\code{\link[=compfit]{compfit()}},
\code{\link[=fit]{fit()}},
\code{\link[=fullfit]{fullfit()}},
\code{\link[=multifit]{multifit()}},
\code{\link[=print.compfit_result]{print.compfit_result()}},
\code{\link[=print.fit_result]{print.fit_result()}},
\code{\link[=print.multifit_result]{print.multifit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{process_survival_probs}
\alias{process_survival_probs}
\title{Process survival probability quantiles}
\usage{
process_survival_probs(
  survfit_objects,
  probs,
  groups,
  group_labels,
  time_digits,
  total,
  total_label,
  median_label,
  by,
  data,
  conf_level = 0.95,
  marks = NULL
)
}
\arguments{
\item{survfit_objects}{List of survfit objects.}

\item{probs}{Numeric vector of probabilities.}

\item{groups}{Character vector of group names.}

\item{group_labels}{Character vector of group display labels.}

\item{time_digits}{Integer decimal places for time values.}

\item{total}{Logical or character controlling total column.}

\item{total_label}{Character label for total column.}

\item{median_label}{Character label for median row.}

\item{by}{Character name of stratifying variable.}

\item{data}{Data.table with the source data.}
}
\value{
List with formatted and raw data.tables.
}
\description{
Extracts survival time quantiles from survfit objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{quiet_fit}
\alias{quiet_fit}
\title{Fit a model with selective warning suppression}
\usage{
quiet_fit(expr, verbose = FALSE)
}
\arguments{
\item{expr}{An unevaluated expression (model fitting call) to execute.}

\item{verbose}{Logical. If \code{TRUE}, all warnings are shown. If \code{FALSE},
routine fitting warnings are suppressed. Default is \code{FALSE}.}
}
\value{
The result of evaluating \code{expr}.
}
\description{
Wraps model fitting expressions to suppress routine warnings from mixed-effects
and GLM fitting (\emph{e.g.,} singular fits, convergence messages, separation
warnings) while allowing unexpected warnings through. When \code{verbose = TRUE},
all warnings are displayed.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{process_continuous}
\alias{process_continuous}
\title{Process continuous variable}
\usage{
process_continuous(
  data,
  var,
  var_label,
  group_var,
  stats,
  digits,
  na_include,
  na_label,
  test,
  test_type,
  total,
  total_label,
  p_per_stat = FALSE,
  marks = NULL,
  ...
)
}
\arguments{
\item{data}{Data.table containing the variable.}

\item{var}{Character string naming the variable to process.}

\item{var_label}{Character string label for display.}

\item{group_var}{Optional character string naming the grouping variable.}

\item{stats}{Character vector of statistics to calculate.}

\item{digits}{Integer number of decimal places.}

\item{na_include}{Logical whether to include missing values.}

\item{na_label}{Character string label for missing values.}

\item{test}{Logical whether to perform statistical tests.}

\item{test_type}{Character string specifying test type.}

\item{total}{Logical or character controlling total column display.}

\item{total_label}{Character string label for total column.}

\item{p_per_stat}{Logical. If TRUE, calculate separate \emph{p}-values for each 
statistic type (\emph{e.g.,} t-test for means, Wilcoxon for medians). If \code{FALSE}
(default), calculate a single \emph{p}-value based on the first statistic type.}

\item{...}{Additional arguments passed to test functions.}
}
\value{
List with 'formatted' and 'raw' data.table components.
}
\description{
Calculates descriptive statistics for continuous numeric variables, with
optional grouping and statistical testing. Supports multiple summary
statistics (mean \eqn{\pm} SD, median [IQR], range) and various hypothesis tests.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{process_survival_times}
\alias{process_survival_times}
\title{Process survival at specified time points}
\usage{
process_survival_times(
  survfit_objects,
  times,
  groups,
  group_labels,
  stats,
  type,
  digits,
  percent,
  total,
  total_label,
  time_label,
  time_unit,
  by,
  data,
  marks = NULL
)
}
\arguments{
\item{survfit_objects}{List of survfit objects.}

\item{times}{Numeric vector of time points.}

\item{groups}{Character vector of group names.}

\item{group_labels}{Character vector of group display labels.}

\item{stats}{Character vector of statistics to include.}

\item{type}{Character string specifying probability type.}

\item{digits}{Integer decimal places for percentages.}

\item{percent}{Logical whether to display as percentages.}

\item{total}{Logical or character controlling total column.}

\item{total_label}{Character label for total column.}

\item{time_label}{Character template for time column headers.}

\item{time_unit}{Character time unit for column headers.}

\item{by}{Character name of stratifying variable.}

\item{data}{Data.table with the source data.}
}
\value{
List with formatted and raw data.tables.
}
\description{
Extracts survival probabilities at specified time points from survfit objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{format_count}
\alias{format_count}
\title{Format integer counts with locale-aware thousands separator}
\usage{
format_count(n, marks = NULL, na = NA_character_)
}
\arguments{
\item{n}{Numeric vector of counts.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}. Resolved from the global option
when not supplied, so that callers without a locale of their own still
produce output consistent with the rest of the package.}

\item{na}{Character string substituted for missing values. Contexts differ in
what reads correctly: tables conventionally show a dash, plot annotations
are better left blank, and the default preserves the missing value for
callers doing their own substitution.}
}
\value{
Character vector of formatted counts, the same length as \code{n}.
}
\description{
Formats integer values with a thousands separator for display in tables.
Values below 1000 are returned as plain character strings, since a grouping
mark only applies from four digits.
}
\details{
Vectorized over \code{n}. Each element is formatted independently, so counts
of differing magnitude are not padded to a common width.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{get_stat_label}
\alias{get_stat_label}
\title{Get display label for statistic type}
\usage{
get_stat_label(stat_type)
}
\arguments{
\item{stat_type}{Character string: "mean_sd", "median_iqr", "median_range",
"range", "n_miss", or custom type.}
}
\value{
Character string with formatted label (\emph{e.g.,} "Mean \eqn{\pm} SD").
}
\description{
Converts internal statistic type codes to formatted display labels for
table column headers.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{validate_outcome_homogeneity}
\alias{validate_outcome_homogeneity}
\title{Validate outcome homogeneity for multifit}
\usage{
validate_outcome_homogeneity(data, outcomes, model_type, family = "binomial")
}
\arguments{
\item{data}{Data.table containing the analysis data.}

\item{outcomes}{Character vector of outcome variable names.}

\item{model_type}{Character string specifying the model type.}

\item{family}{Character string specifying the GLM family (for glm/glmer).}
}
\value{
Invisible \code{NULL}. Issues warnings if problems are detected.
}
\description{
Checks whether all outcomes in a multifit analysis are compatible with the
specified model type. Issues a warning when outcomes appear to be of mixed
types (\emph{e.g.,} binary and continuous outcomes in the same analysis), which
would produce tables with incompatible effect measures.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoforest.R
\name{is_uniscreen_result}
\alias{is_uniscreen_result}
\title{Check if object is a uniscreen result}
\usage{
is_uniscreen_result(x)
}
\arguments{
\item{x}{Object to check.}
}
\value{
Logical indicating if x is a uniscreen result.
}
\description{
Internal helper to detect uniscreen output objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{check_latex}
\alias{check_latex}
\title{Check LaTeX installation}
\usage{
check_latex()
}
\value{
Logical \code{TRUE} if LaTeX is available, \code{FALSE} otherwise.
}
\description{
Verifies that a LaTeX distribution (pdflatex or xelatex) is available
on the system for PDF compilation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{format_analysis_counts}
\alias{format_analysis_counts}
\title{Describe the analyzed sample for console output}
\usage{
format_analysis_counts(counts, label = "Observations analyzed", marks = NULL)
}
\arguments{
\item{counts}{List as returned by \code{get_analysis_counts()}.}

\item{label}{Character label preceding the counts.}

\item{marks}{List of number marks as returned by
\code{resolve_number_marks()}. Counts and percentages are separated as
the accompanying table separates them, so that a locale setting applies
to the whole of the output rather than to the table alone. Resolved from
the global option when not supplied.}
}
\value{
Character string, or \code{NULL} when the counts are unavailable.
}
\description{
Renders the counts produced by \code{get_analysis_counts()} as a single line
for the \code{print()} methods. The line is produced whenever the counts are
available, including when every supplied observation entered the analysis:
a complete sample is itself worth stating, and reporting it only on loss
would leave the reader to infer the difference between no exclusions and no
disclosure.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{apply_zebra_stripes_ft}
\alias{apply_zebra_stripes_ft}
\title{Apply zebra stripes with proper variable group detection for indented tables}
\usage{
apply_zebra_stripes_ft(ft, df, var_groups)
}
\arguments{
\item{ft}{\pkg{flextable} object.}

\item{df}{The source data.table used to create the flextable.}

\item{var_groups}{List of row index vectors for variable groups.}
}
\value{
Flextable object with zebra stripe formatting applied.
}
\description{
Applies alternating background colors to variable groups in \pkg{flextable} objects.
Handles both indented tables (detects groups by leading whitespace) and
non-indented tables (uses pre-identified groups).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_headers_ft}
\alias{format_headers_ft}
\title{Format headers for flextable}
\usage{
format_headers_ft(ft, has_n_row, n_row_data)
}
\arguments{
\item{ft}{\pkg{flextable} object.}

\item{has_n_row}{Logical whether source data had an N row.}

\item{n_row_data}{Data from the N row for adding counts to headers.}
}
\value{
Formatted \pkg{flextable} object.
}
\description{
Applies formatting to flextable headers including italicizing 'n',
adding sample size counts from N row data, and bolding all headers.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{add_raw_stats}
\alias{add_raw_stats}
\title{Add raw statistics to row}
\usage{
add_raw_stats(row, col, stats, stat_type)
}
\arguments{
\item{row}{Data.table row to modify.}

\item{col}{Character string column name for the statistics.}

\item{stats}{Named list of numeric statistics (mean, sd, median, \emph{etc.}).}

\item{stat_type}{Character string indicating which statistic is displayed.}
}
\value{
Modified row (by reference).
}
\description{
Appends raw numeric statistics to a data.table row for downstream processing.
Used to preserve underlying values alongside formatted display strings.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{validate_fit_inputs}
\alias{validate_fit_inputs}
\title{Complete input validation for fit functions}
\usage{
validate_fit_inputs(
  data,
  outcome,
  predictors,
  model_type,
  family = NULL,
  conf_level = 0.95,
  digits = 2,
  p_digits = 3,
  p_threshold = NULL,
  auto_correct_model = TRUE
)
}
\arguments{
\item{data}{Data frame or data.table containing all variables.}

\item{outcome}{Character string outcome specification (may include Surv()).}

\item{predictors}{Character vector of predictor variable names.}

\item{model_type}{Character string model type to validate.}

\item{family}{GLM family object, function, or string if applicable.}

\item{conf_level}{Numeric confidence level (must be between 0 and 1).}

\item{digits}{Integer number of decimal places for effect estimates.}

\item{p_digits}{Integer number of decimal places for \emph{p}-values.}

\item{p_threshold}{Numeric \emph{p}-value threshold for significance highlighting.}

\item{auto_correct_model}{Logical whether to auto-correct model type mismatches.}
}
\value{
List with validated model_type, family, auto_corrected flag.
}
\description{
Master validation function called by fit(), uniscreen(), fullfit(). Performs
comprehensive checks on data structure, variable existence, numeric parameter
ranges, and model-outcome consistency. Returns validated parameters with
auto-corrections applied when appropriate.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{perform_continuous_test}
\alias{perform_continuous_test}
\title{Perform statistical tests for continuous variables}
\usage{
perform_continuous_test(var_vec, grp_vec, test_type, stat_type)
}
\arguments{
\item{var_vec}{Numeric vector of the continuous variable.}

\item{grp_vec}{Factor or character vector defining groups.}

\item{test_type}{Character string: "parametric", "nonparametric", or "auto".}

\item{stat_type}{Character string indicating primary statistic being tested.}
}
\value{
Numeric \emph{p}-value from the hypothesis test.
}
\description{
Conducts hypothesis tests comparing continuous variables across groups.
Supports t-tests, Wilcoxon tests, ANOVA, and Kruskal-Wallis tests with
automatic selection based on number of groups.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{format_p_forest}
\alias{format_p_forest}
\title{Format a p-value for forest plot annotations}
\usage{
format_p_forest(p, p_digits, marks = NULL)
}
\arguments{
\item{p}{Numeric p-value.}

\item{p_digits}{Integer decimal places.}

\item{marks}{Optional list with \code{big.mark} and \code{decimal.mark} as
returned by \code{\link{resolve_number_marks}}.}
}
\value{
Character string.
}
\description{
Returns a formatted p-value string suitable for forest plot display,
using locale-aware decimal marks when \code{marks} is provided.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_analysis_data}
\alias{get_analysis_data}
\title{Restrict data to the observations used in model fitting}
\usage{
get_analysis_data(model, model_class, data)
}
\arguments{
\item{model}{Fitted model object.}

\item{model_class}{Character string of the model class.}

\item{data}{Data frame or data.table supplied to the model call.}
}
\value{
Data.table restricted to the observations used in fitting. The input
  is returned unchanged when the analysis rows cannot be determined.
}
\description{
Group-level sample sizes must be counted over the observations that entered
the model, not over every row supplied by the user. Several fitted objects
carry no model frame from which those observations can be read directly:
\code{survival::coxph()} does not store one by default, \pkg{coxme} models
never store one, and the memory-conserving fits performed internally by
\code{uniscreen()} and \code{multifit()} pass \code{model = FALSE}. In these
cases the supplied data still contains the observations dropped during
fitting, and counting them inflates group sizes whenever the response or a
covariate is missing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoforest.R
\name{is_multifit_result}
\alias{is_multifit_result}
\title{Check if object is a multifit result}
\usage{
is_multifit_result(x)
}
\arguments{
\item{x}{Object to check.}
}
\value{
Logical indicating if x is a multifit result.
}
\description{
Internal helper to detect multifit output objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{extract_glmer_metrics}
\alias{extract_glmer_metrics}
\title{Extract metrics for generalized linear mixed-effects models (glmer)}
\usage{
extract_glmer_metrics(model, raw_data, metrics)
}
\arguments{
\item{model}{Fitted glmerMod object from lme4.}

\item{raw_data}{Data.table with raw model information.}

\item{metrics}{Named list of initialized metrics to populate.}
}
\value{
Updated metrics list with glmer-specific values.
}
\description{
Extracts quality metrics specific to generalized linear mixed-effects models
including concordance, R-squared measures, ICC, and Brier score for binomial.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniforest.R
\name{calculate_uniforest_layout}
\alias{calculate_uniforest_layout}
\title{Calculate table layout for uniforest plots}
\usage{
calculate_uniforest_layout(
  to_show_exp_clean,
  show_n,
  show_events,
  indent_groups,
  table_width,
  center_padding,
  effect_abbrev,
  font_size,
  log_scale,
  rangeb,
  ci_pct = 95
)
}
\arguments{
\item{to_show_exp_clean}{Data.table with formatted data for plotting.}

\item{show_n}{Logical whether to include n column.}

\item{show_events}{Logical whether to include events column.}

\item{indent_groups}{Logical whether levels are indented.}

\item{table_width}{Proportion of width for table.}

\item{center_padding}{Padding between table and forest.}

\item{effect_abbrev}{Effect type abbreviation.}

\item{font_size}{Font size multiplier.}

\item{log_scale}{Logical whether using log scale.}

\item{rangeb}{Numeric vector with plot range bounds (in data units).}
}
\value{
List with column positions, widths, and layout parameters.
}
\description{
Internal function to determine column positions and widths for forest plot
table section. Positions are calculated in the same units as the data
(log scale for OR/HR/RR, linear for coefficients).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{convert_units}
\alias{convert_units}
\title{Convert between units}
\usage{
convert_units(value, from = "in", to = "in", dpi = 96)
}
\arguments{
\item{value}{Numeric value to convert.}

\item{from}{Character string specifying source unit ("in", "cm", "mm", "px", "pt").}

\item{to}{Character string specifying target unit ("in", "cm", "mm", "px", "pt").}

\item{dpi}{Integer dots per inch for pixel conversions (default 96).}
}
\value{
Numeric value in target units.
}
\description{
Converts measurements between different unit systems commonly used in
graphics (inches, centimeters, millimeters, pixels, points).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{detect_plot_font}
\alias{detect_plot_font}
\title{Detect available sans-serif font for plots}
\usage{
detect_plot_font()
}
\value{
Character string with the font family name to use.
}
\description{
Checks for commonly available sans-serif fonts in order of preference
(Helvetica, Arial, Helvetica Neue) and returns the first available one.
Falls back to "sans" if none are found or if systemfonts is unavailable.
}
\details{
When ragg is being used as the graphics device (detected via options or
knitr settings), font detection works in non-interactive sessions since
ragg handles font rendering independently of the R graphics system.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{format_pvalues_desctable}
\alias{format_pvalues_desctable}
\title{Format \emph{p}-values for descriptive tables}
\usage{
format_pvalues_desctable(result, p_digits, marks)
}
\arguments{
\item{result}{Data.table with 'p_value' column to format.}

\item{p_digits}{Integer number of decimal places for p-values.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Modified data.table with 'p_value' column (formatted strings).
}
\description{
Converts numeric p-values to formatted strings with appropriate precision.
Handles very small p-values with threshold notation (\emph{e.g.,} "< 0.001").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_event_variable_for_counts}
\alias{get_event_variable_for_counts}
\title{Identify the event indicator among a model's response variables}
\usage{
get_event_variable_for_counts(outcome_vars, model_type = NULL, family = NULL)
}
\arguments{
\item{outcome_vars}{Character vector of response variable names.}

\item{model_type}{Character string of the model type or class.}

\item{family}{Model family, where applicable. Accepted as a name, a family
object, or a generator function, since callers resolve the family at
different points: \code{fit()} passes the name it was given, while
\code{uniscreen()} may already have resolved \code{"Gamma"} to
\code{Gamma(link = "log")}.}
}
\value{
Character name of the event indicator, or \code{NULL} when the model
  carries no event count.
}
\description{
Determines which response variable carries the event count, and whether the
model has one at all. Linear and Gaussian models do not; the families that
count events are those for which \code{m2dt()} reports an events figure, so
that a denominator is only offered where a numerator exists.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxforest.R
\name{coxforest}
\alias{coxforest}
\title{Create Forest Plot for Cox Proportional Hazards Models}
\usage{
coxforest(
  x,
  data = NULL,
  title = "Cox Proportional Hazards Model",
  effect_label = "Hazard Ratio",
  digits = 2,
  p_digits = 3,
  conf_level = 0.95,
  font_size = 1,
  annot_size = 3.88,
  header_size = 5.82,
  title_size = 23.28,
  plot_width = NULL,
  plot_height = NULL,
  table_width = 0.6,
  show_n = TRUE,
  show_events = TRUE,
  indent_groups = FALSE,
  condense_table = FALSE,
  bold_variables = FALSE,
  center_padding = 4,
  zebra_stripes = TRUE,
  ref_label = "reference",
  labels = NULL,
  color = "#8A61D8",
  qc_footer = TRUE,
  units = "in",
  number_format = NULL
)
}
\arguments{
\item{x}{Either a fitted Cox model object (class \code{coxph} or \code{coxme}), 
a \code{fit_result} object from \code{fit()}, or a \code{fullfit_result}
object from \code{fullfit()}. When a \code{fit_result} or \code{fullfit_result}
is provided, the model, data, and labels are automatically extracted.}

\item{data}{Data frame or data.table containing the original data used to 
fit the model. If \code{NULL} (default) and \code{x} is a model, the function 
attempts to extract data from the model object. If \code{x} is a \code{fit_result},
data is extracted automatically. Providing data explicitly is recommended when
passing a model directly.}

\item{title}{Character string specifying the plot title displayed at the top. 
Default is \code{"Cox Proportional Hazards Model"}. Use descriptive titles 
for publication.}

\item{effect_label}{Character string for the effect measure label on the 
forest plot axis. Default is \code{"Hazard Ratio"}. Alternatives might include 
"HR" for space-constrained plots.}

\item{digits}{Integer specifying the number of decimal places for hazard ratios 
and confidence intervals. Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be
between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
when \code{conf_level = 0.90}).}

\item{font_size}{Numeric multiplier controlling the base font size for all 
text elements. Default is 1.0.}

\item{annot_size}{Numeric value controlling the relative font size for data 
annotations. Default is 3.88.}

\item{header_size}{Numeric value controlling the relative font size for column 
headers. Default is 5.82.}

\item{title_size}{Numeric value controlling the relative font size for the 
main plot title. Default is 23.28.}

\item{plot_width}{Numeric value specifying the intended output width in 
specified \code{units}. Used for optimizing layout. Default is \code{NULL} 
(automatic). Recommended: 10-16 inches.}

\item{plot_height}{Numeric value specifying the intended output height in 
specified \code{units}. Default is \code{NULL} (automatic based on number 
of rows).}

\item{table_width}{Numeric value between 0 and 1 specifying the proportion of 
total plot width allocated to the data table. Default is 0.6 (60\% table, 
40\% forest plot).}

\item{show_n}{Logical. If \code{TRUE}, includes a column showing group-specific 
sample sizes.
Counts describe the observations used in fitting rather than every row
supplied. Default is \code{TRUE}.}

\item{show_events}{Logical. If \code{TRUE}, includes a column showing the 
number of events (deaths, failures) for each group. Critical for survival 
analysis interpretation. Default is \code{TRUE}.}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels under 
their parent variable, creating hierarchical structure. The "Group" column 
is hidden when \code{TRUE}. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses binary variables to 
show only the non-reference level (\emph{e.g.,} "Yes" for Yes/No variables), with 
the variable name indicating the displayed level. This creates a more compact 
table for models with many binary predictors. Default is \code{FALSE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. If \code{FALSE} (default), variable names are displayed in plain
text.}

\item{center_padding}{Numeric value specifying horizontal spacing between 
table and forest plot. Default is 4.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating gray 
background shading to different variables for improved readability. 
Default is \code{TRUE}.}

\item{ref_label}{Character string to display for reference categories. 
Default is \code{"reference"}.}

\item{labels}{Named character vector providing custom display labels for 
variables. Example: \code{c(age = "Age (years)", stage = "Disease Stage")}. 
Default is \code{NULL}, in which case the labels attached by the function
that produced \code{x} are used, where \code{x} is a \pkg{summata} result
rather than a model object. Original variable names are used where neither
is available.}

\item{color}{Character string specifying the color for hazard ratio point 
estimates in the forest plot. Default is \code{"#8A61D8"} (purple). Use 
hex codes or R color names.}

\item{qc_footer}{Logical. If \code{TRUE}, displays model quality control
statistics in the footer (events analyzed, global log-rank \emph{p}-value,
concordance index, AIC). Default is \code{TRUE}.}

\item{units}{Character string specifying units for plot dimensions: 
\code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}
}
\value{
A \code{ggplot} object containing the complete forest plot. The plot 
  can be:
  \itemize{
    \item Displayed directly: \code{print(plot)}
    \item Saved to file: \code{forestsave(plot, "forest.pdf")}
    \item Further customized with \pkg{ggplot2} functions
  }
  
  The returned object includes an attribute \code{"rec_dims"} 
  accessible via \code{attr(plot, "rec_dims")}, which is a list 
  containing:
  \describe{
    \item{width}{Numeric. Recommended plot width in specified units}
    \item{height}{Numeric. Recommended plot height in specified units}
  }
  
  These recommendations are automatically calculated based on the number of 
  variables, text sizes, and layout parameters, and are printed to console 
  if \code{plot_width} or \code{plot_height} are not specified. The list also 
  carries a \code{units} element recording the units the dimensions are 
  expressed in, matching the \code{units} argument. \code{forestsave()} reads 
  all three and requires no further handling.

  The returned object also includes an attribute \code{"table_data"}
  accessible via \code{attr(plot, "table_data")}, a data.table holding the
  values drawn in the plot: one row per term in model order, with the
  variable, factor level, sample size, event count, estimate, confidence
  bounds and p-value. Sample sizes and event counts describe the
  observations used in fitting rather than every row supplied, so they sum
  to the model sample size within each variable.
}
\description{
Generates a publication-ready forest plot that combines a formatted data table 
with a graphical representation of hazard ratios from a Cox proportional hazards 
survival model. The plot integrates variable names, group levels, sample sizes, 
event counts, hazard ratios with confidence intervals, \emph{p}-values, and model 
diagnostics in a single comprehensive visualization designed for manuscripts 
and presentations.
}
\details{
\strong{Survival-Specific Features:}

The Cox forest plot includes several survival analysis-specific components:
\itemize{
  \item \strong{Event counts}: Number of events (deaths, failures) shown for 
    each predictor category, critical for assessing statistical power
  \item \strong{Hazard ratios}: Always exponentiated coefficients (never raw), 
    interpreted as the multiplicative change in hazard
  \item \strong{Log scale}: Forest plot uses log scale for HR (reference line at 1)
  \item \strong{Model diagnostics}: Includes concordance (C-index), global 
    log-rank test \emph{p}-value, and AIC
}

\strong{Plot Components:}

\enumerate{
  \item \strong{Title}: Centered at top
  \item \strong{Data Table} (left): Contains:
    \itemize{
      \item Variable and Group columns
      \item n: Sample sizes by group
      \item Events: Event counts by group (critical for survival)
      \item aHR (95\% CI); \emph{p}-value: Adjusted hazard ratios with CIs and \emph{p}-values
    }
  \item \strong{Forest Plot} (right):
    \itemize{
      \item Point estimates (squares sized by sample size)
      \item 95\% confidence intervals
      \item Reference line at HR = 1
      \item Log scale for hazard ratios
    }
  \item \strong{Model Statistics} (footer):
    \itemize{
      \item Events analyzed (with total events and percentage)
      \item Global log-rank test \emph{p}-value
      \item Concordance (C-index) with standard error
      \item AIC
    }
}

\strong{Interpreting Hazard Ratios:}

\itemize{
  \item \strong{HR = 1}: No effect on hazard (reference)
  \item \strong{HR > 1}: Increased hazard (worse survival)
  \item \strong{HR < 1}: Decreased hazard (better survival)
  \item Example: HR = 2.0 means twice the hazard of the event at any time
}

\strong{Event Counts:}

The "Events" column is particularly important in survival analysis:
\itemize{
  \item Indicates the number of actual events (not censored observations) 
    in each group
  \item Essential for assessing statistical power
  \item Categories with very few events may have unreliable HR estimates
  \item The footer shows events analyzed, total events, and percentage 
    in the original data
}

\strong{Concordance (C-index):}

The concordance statistic displayed in the footer indicates discrimination:
\itemize{
  \item Range: 0.5 to 1.0
  \item 0.5 = random prediction (coin flip)
  \item 0.7-0.8 = acceptable discrimination
  \item > 0.8 = excellent discrimination
  \item Standard error provided for confidence interval calculation
}

\strong{Global Log-Rank Test:}

The global \emph{p}-value tests the null hypothesis that all coefficients are zero:
\itemize{
  \item Significant \emph{p}-value (< 0.05) indicates the model as a whole predicts survival
  \item Non-significant global test doesn't preclude significant individual 
    predictors
  \item Based on the score (log-rank) test
}

\strong{Stratification and Clustering:}

If the model includes stratification (\code{strata()}) or clustering 
(\code{cluster()}):
\itemize{
  \item Stratified variables are not shown in the forest plot (they don't 
    have HRs)
  \item Clustering affects standard errors but not point estimates
  \item Both are handled automatically by the function
}

\strong{Proportional Hazards Assumption:}

The forest plot assumes proportional hazards (constant HR over time). Users 
should verify this assumption using:
\itemize{
  \item \code{cox.zph(model)} for testing
  \item Stratification for variables violating the assumption
  \item Time-dependent coefficients if needed
}
}
\examples{
data(clintrial)
data(clintrial_labels)
library(survival)

# Create example model
model1 <- coxph(
    survival::Surv(os_months, os_status) ~ age + sex + treatment,
    data = clintrial)

# Example 1: Basic Cox model forest plot
p <- coxforest(model1, data = clintrial)

\donttest{

old_width <- options(width = 180)

# Example 2: With custom labels and title
plot2 <- coxforest(
    x = model1,
    data = clintrial,
    title = "Prognostic Factors for Overall Survival",
    labels = clintrial_labels
)

# Example 3: Comprehensive model with indented layout
model3 <- coxph(
    Surv(os_months, os_status) ~ age + sex + bmi + smoking + 
        treatment + stage + grade,
    data = clintrial
)

plot3 <- coxforest(
    x = model3,
    data = clintrial,
    labels = clintrial_labels,
    indent_groups = TRUE,
    zebra_stripes = TRUE
)

# Example 4: Condensed layout for many binary predictors
model4 <- coxph(
    Surv(os_months, os_status) ~ age + sex + smoking + 
        hypertension + diabetes + surgery,
    data = clintrial
)

plot4 <- coxforest(
    x = model4,
    data = clintrial,
    condense_table = TRUE,
    labels = clintrial_labels
)

# Example 5: Stratified Cox model
model5 <- coxph(
    Surv(os_months, os_status) ~ age + sex + treatment + strata(site),
    data = clintrial
)

plot5 <- coxforest(
    x = model5,
    data = clintrial,
    title = "Stratified by Study Site",
    labels = clintrial_labels
)

# Example 6: Save with recommended dimensions
forestsave(plot5, file.path(tempdir(), "survival_forest.pdf"))

options(old_width)

}
}
\seealso{
\code{\link{autoforest}} for automatic model detection,
\code{\link{glmforest}} for logistic/GLM forest plots,
\code{\link{lmforest}} for linear model forest plots,
\code{\link{uniforest}} for univariable screening forest plots,
\code{\link{multiforest}} for multi-outcome forest plots,
\code{\link[survival]{coxph}} for fitting Cox models,
\code{\link{fit}} for regression modeling,
  \code{\link{forestsave}} for saving with recommended dimensions

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=lmforest]{lmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=recdims]{recdims()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmforest.R
\name{lmforest}
\alias{lmforest}
\title{Create Forest Plot for Linear Models}
\usage{
lmforest(
  x,
  data = NULL,
  title = "Linear Model",
  effect_label = "Coefficient",
  digits = 2,
  p_digits = 3,
  conf_level = 0.95,
  font_size = 1,
  annot_size = 3.88,
  header_size = 5.82,
  title_size = 23.28,
  plot_width = NULL,
  plot_height = NULL,
  table_width = 0.6,
  show_n = TRUE,
  indent_groups = FALSE,
  condense_table = FALSE,
  bold_variables = FALSE,
  center_padding = 4,
  zebra_stripes = TRUE,
  ref_label = "reference",
  labels = NULL,
  units = "in",
  color = "#5A8F5A",
  qc_footer = TRUE,
  number_format = NULL
)
}
\arguments{
\item{x}{Either a fitted linear model object (class \code{lm} or \code{lmerMod}), 
a \code{fit_result} object from \code{fit()}, or a \code{fullfit_result}
object from \code{fullfit()}. When a \code{fit_result} or \code{fullfit_result}
is provided, the model, data, and labels are automatically extracted.}

\item{data}{Data frame or data.table containing the original data used to 
fit the model. If \code{NULL} (default) and \code{x} is a model, the function 
attempts to extract data from the model object. If \code{x} is a \code{fit_result},
data is extracted automatically. Providing data explicitly is recommended when
passing a model directly.}

\item{title}{Character string specifying the plot title displayed at the top. 
Default is \code{"Linear Model"}.}

\item{effect_label}{Character string for the effect measure label on the 
forest plot axis. Default is \code{"Coefficient"}.}

\item{digits}{Integer specifying the number of decimal places for coefficients 
and confidence intervals. Default is 2.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be
between 0 and 1. Default is 0.95 (95\% confidence intervals). The CI
percentage is automatically displayed in column headers (\emph{e.g.,} "90\% CI"
when \code{conf_level = 0.90}).}

\item{font_size}{Numeric multiplier controlling the base font size for all 
text elements. Default is 1.0.}

\item{annot_size}{Numeric value controlling the relative font size for data 
annotations. Default is 3.88.}

\item{header_size}{Numeric value controlling the relative font size for column 
headers. Default is 5.82.}

\item{title_size}{Numeric value controlling the relative font size for the 
main plot title. Default is 23.28.}

\item{plot_width}{Numeric value specifying the intended output width in 
specified \code{units}. Default is \code{NULL} (automatic).}

\item{plot_height}{Numeric value specifying the intended output height in 
specified \code{units}. Default is \code{NULL} (automatic).}

\item{table_width}{Numeric value between 0 and 1 specifying the proportion of 
total plot width allocated to the data table. Default is 0.6.}

\item{show_n}{Logical. If \code{TRUE}, includes a column showing group-specific 
sample sizes.
Counts describe the observations used in fitting rather than every row
supplied. Default is \code{TRUE}.}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels under 
their parent variable name, creating hierarchical structure. The "Group" 
column is hidden when \code{TRUE}. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses binary categorical 
variables into single rows. Automatically sets \code{indent_groups = TRUE}. 
Default is \code{FALSE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. If \code{FALSE} (default), variable names are displayed in plain
text.}

\item{center_padding}{Numeric value specifying horizontal spacing between 
table and forest plot. Default is 4.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating gray 
background shading to different variables. Default is \code{TRUE}.}

\item{ref_label}{Character string to display for reference categories of 
factor variables. Default is \code{"reference"}.}

\item{labels}{Named character vector providing custom display labels for 
variables. Example: \code{c(age = "Age (years)", height = "Height (cm)")}. 
Default is \code{NULL}, in which case the labels attached by the function
that produced \code{x} are used, where \code{x} is a \pkg{summata} result
rather than a model object. Original variable names are used where neither
is available.}

\item{units}{Character string specifying units for plot dimensions: 
\code{"in"} (inches), \code{"cm"}, or \code{"mm"}. Default is \code{"in"}.}

\item{color}{Character string specifying the color for coefficient point 
estimates in the forest plot. Default is \code{"#5A8F5A"} (green). Use 
hex codes or R color names.}

\item{qc_footer}{Logical. If \code{TRUE}, displays model quality control
statistics in the footer (observations analyzed, \emph{R}\eqn{^2}, adjusted
\emph{R}\eqn{^2}, \emph{F}-statistic, AIC). Default is \code{TRUE}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}
}
\value{
A \code{ggplot} object containing the complete forest plot. The plot 
  can be:
  \itemize{
    \item Displayed directly: \code{print(plot)}
    \item Saved to file: \code{forestsave(plot, "forest.pdf")}
    \item Further customized with \pkg{ggplot2} functions
  }
  
  The returned object includes an attribute \code{"rec_dims"} 
  accessible via \code{attr(plot, "rec_dims")}, which is a list 
  containing:
  \describe{
    \item{width}{Numeric. Recommended plot width in specified units}
    \item{height}{Numeric. Recommended plot height in specified units}
  }
  
  These recommendations are automatically calculated based on the number of 
  variables, text sizes, and layout parameters, and are printed to console 
  if \code{plot_width} or \code{plot_height} are not specified. The list also 
  carries a \code{units} element recording the units the dimensions are 
  expressed in, matching the \code{units} argument. \code{forestsave()} reads 
  all three and requires no further handling.

  The returned object also includes an attribute \code{"table_data"}
  accessible via \code{attr(plot, "table_data")}, a data.table holding the
  values drawn in the plot: one row per term in model order, with the
  variable, factor level, sample size, estimate, confidence bounds and
  p-value. Sample sizes describe the observations used in fitting rather
  than every row supplied, so they sum to the model sample size within each
  variable.
}
\description{
Generates a publication-ready forest plot that combines a formatted data table 
with a graphical representation of regression coefficients from a linear model. 
The plot integrates variable names, group levels, sample sizes, coefficients 
with confidence intervals, \emph{p}-values, and model diagnostics (\emph{R}\eqn{^2},
\emph{F}-statistic, AIC) in a single comprehensive visualization designed for
manuscripts and presentations.
}
\details{
\strong{Linear Model-Specific Features:}

The linear model forest plot differs from logistic and Cox plots in several ways:
\itemize{
  \item \strong{Coefficients}: Raw regression coefficients shown (not exponentiated)
  \item \strong{Reference line}: At coefficient = 0 (not at 1)
  \item \strong{Linear scale}: Forest plot uses linear scale (not log scale)
  \item \strong{No events column}: Only sample sizes shown (no event counts)
  \item \strong{\emph{R}\eqn{^2} statistics}: Model fit assessed by \emph{R}\eqn{^2} and adjusted \emph{R}\eqn{^2}
  \item \strong{\emph{F}-test}: Overall model significance from \emph{F}-statistic
}

\strong{Plot Components:}

\enumerate{
  \item \strong{Title}: Centered at top
  \item \strong{Data Table} (left): Contains:
    \itemize{
      \item Variable: Predictor names
      \item Group: Factor levels (if applicable)
      \item \emph{n}: Sample sizes by group
      \item Coefficient (95\% CI); \emph{p}-value: Raw coefficients with CIs and \emph{p}-values
    }
  \item \strong{Forest Plot} (right):
    \itemize{
      \item Point estimates (squares sized by sample size)
      \item 95\% confidence intervals (error bars)
      \item Reference line at coefficient = 0
      \item Linear scale
    }
  \item \strong{Model Statistics} (footer):
    \itemize{
      \item Observations analyzed (with total observations and percentage)
      \item \emph{R}\eqn{^2} and adjusted \emph{R}\eqn{^2}
      \item \emph{F}-statistic with degrees of freedom and \emph{p}-value
      \item AIC
    }
}

\strong{Interpreting Coefficients:}

Linear regression coefficients represent the change in the outcome variable 
for a one-unit change in the predictor:
\itemize{
  \item \strong{Continuous predictors}: Coefficient = change in Y per unit of X
  \item \strong{Binary predictors}: Coefficient = difference in Y between groups
  \item \strong{Factor predictors}: Coefficients = differences from reference 
    category
  \item \strong{Sign matters}: Positive = increase in Y, Negative = decrease in Y
  \item \strong{Zero crossing}: CI crossing zero suggests no significant effect
}

Example: If the coefficient for "age" is 0.50 when predicting BMI, 
BMI increases by 0.50 kg/m\eqn{^2} for each additional year of age.

\strong{Model Fit Statistics:}

The footer displays key diagnostics:
\itemize{
  \item \strong{\emph{R}\eqn{^2}}: Proportion of variance explained (0 to 1)
    \itemize{
      \item 0.0-0.3: Weak explanatory power
      \item 0.3-0.5: Moderate
      \item 0.5-0.7: Good
      \item > 0.7: Strong (rare in social/biological sciences)
    }
  \item \strong{Adjusted \emph{R}\eqn{^2}}: \emph{R}\eqn{^2} penalized for number of predictors
    \itemize{
      \item Always \eqn{\le} \emph{R}\eqn{^2}
      \item Preferred for model comparison
      \item Accounts for model complexity
    }
  \item \strong{\emph{F}-statistic}: Tests null hypothesis that all coefficients = 0
    \itemize{
      \item Degrees of freedom: df1 = # predictors, df2 = # observations - # predictors - 1
      \item Significant \emph{p}-value indicates model explains variance better than intercept-only
    }
  \item \strong{AIC}: For model comparison (lower is better)
}

\strong{Assumptions:}

Linear regression assumes:
\enumerate{
  \item Linearity of relationships
  \item Independence of observations
  \item Homoscedasticity (constant variance)
  \item Normality of residuals
  \item No multicollinearity
}

Check assumptions using:
\itemize{
  \item \code{plot(model)} for diagnostic plots
  \item \code{car::vif(model)} for multicollinearity
  \item \code{lmtest::bptest(model)} for heteroscedasticity
  \item \code{shapiro.test(residuals(model))} for normality
}

\strong{Reference Categories:}

For factor variables:
\itemize{
  \item First level is the reference (coefficient = 0)
  \item Other levels show difference from reference
  \item Reference displayed with \code{ref_label}
  \item Relevel factors before modeling if needed: 
    \code{factor(x, levels = c("desired_ref", ...))}
}

\strong{Sample Size Reporting:}

The "\emph{n}" column shows:
\itemize{
  \item For continuous variables: Total observations with non-missing data
  \item For factor variables: Number of observations in each category
  \item Footer shows observations used in fitting, the total supplied, and 
    the percentage retained (accounting for missing values)
}
}
\examples{
data(clintrial)
data(clintrial_labels)

# Create example model
model1 <- lm(bmi ~ age + sex + smoking, data = clintrial)

# Example 1: Basic linear model forest plot
p <- lmforest(model1, data = clintrial)

\donttest{

old_width <- options(width = 180)

# Example 2: With custom labels and title
plot2 <- lmforest(
    x = model1,
    data = clintrial,
    title = "Predictors of Body Mass Index",
    effect_label = "Change in BMI (kg/m^2)",
    labels = clintrial_labels
)

# Example 3: Comprehensive model with indented layout
model3 <- lm(
    bmi ~ age + sex + smoking + hypertension + diabetes + creatinine,
    data = clintrial
)

plot3 <- lmforest(
    x = model3,
    data = clintrial,
    labels = clintrial_labels,
    indent_groups = TRUE,
    zebra_stripes = TRUE
)

# Example 4: Condensed layout
plot4 <- lmforest(
    x = model3,
    data = clintrial,
    condense_table = TRUE,
    labels = clintrial_labels
)

# Example 5: Different outcome (hemoglobin)
model5 <- lm(
    hemoglobin ~ age + sex + bmi + smoking + creatinine,
    data = clintrial
)

plot5 <- lmforest(
    x = model5,
    data = clintrial,
    title = "Predictors of Baseline Hemoglobin",
    effect_label = "Change in Hemoglobin (g/dL)",
    labels = clintrial_labels
)

# Example 6: Save with recommended dimensions
forestsave(plot5, file.path(tempdir(), "linear_forest.pdf"))

options(old_width)

}
}
\seealso{
\code{\link{autoforest}} for automatic model detection,
\code{\link{glmforest}} for logistic/GLM forest plots,
\code{\link{coxforest}} for Cox model forest plots,
\code{\link{uniforest}} for univariable screening forest plots,
\code{\link{multiforest}} for multi-outcome forest plots,
\code{\link[stats]{lm}} for fitting linear models,
\code{\link{fit}} for regression modeling,
  \code{\link{forestsave}} for saving with recommended dimensions

Other visualization functions:
\code{\link[=autoforest]{autoforest()}},
\code{\link[=coxforest]{coxforest()}},
\code{\link[=forestsave]{forestsave()}},
\code{\link[=glmforest]{glmforest()}},
\code{\link[=multiforest]{multiforest()}},
\code{\link[=recdims]{recdims()}},
\code{\link[=uniforest]{uniforest()}}
}
\concept{visualization functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{print.multifit_result}
\alias{print.multifit_result}
\title{Print method for multifit results}
\usage{
\method{print}{multifit_result}(x, ...)
}
\arguments{
\item{x}{Object of class \code{multifit_result}.}

\item{...}{Additional arguments passed to print methods.}
}
\value{
Invisibly returns the input object \code{x}. Called for its
  side effect of printing a formatted summary to the console.
}
\description{
Print method for multifit results
}
\seealso{
Other regression functions:
\code{\link[=compfit]{compfit()}},
\code{\link[=fit]{fit()}},
\code{\link[=fullfit]{fullfit()}},
\code{\link[=multifit]{multifit()}},
\code{\link[=print.compfit_result]{print.compfit_result()}},
\code{\link[=print.fit_result]{print.fit_result()}},
\code{\link[=print.fullfit_result]{print.fullfit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{add_pvalue_column}
\alias{add_pvalue_column}
\title{Add \emph{p}-value column to result table}
\usage{
add_pvalue_column(result, p_value, p_digits, marks = NULL)
}
\arguments{
\item{result}{Data.table result.}

\item{p_value}{Numeric \emph{p}-value.}

\item{p_digits}{Integer decimal places for \emph{p}-value.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Data.table with \emph{p}-value column added.
}
\description{
Adds formatted \emph{p}-value column to the survtable result.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{calculate_model_scores}
\alias{calculate_model_scores}
\title{Calculate Composite Mean Scores (CMS) for model comparison}
\usage{
calculate_model_scores(comparison, model_type, scoring_weights = NULL)
}
\arguments{
\item{comparison}{Data.table with model comparison metrics.}

\item{model_type}{Character string indicating model type.}

\item{scoring_weights}{Optional named list of custom weights. If \code{NULL},
uses default weights for the model type.}
}
\value{
Data.table with CMS column added, sorted by score.
}
\description{
Computes composite Score based on weighted combination of model
quality metrics. Weights vary by model type to reflect academic consensus
on important metrics for each model class.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imports.R
\name{summata-imports}
\alias{summata-imports}
\title{Package Imports}
\description{
Declares the imports that are called without a namespace prefix, so that
R CMD check does not report them as undefined globals. Functions called as
\code{package::function()} elsewhere in the package need no entry here, but
their package must still appear under Imports in DESCRIPTION.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{condense_quantitative_rows}
\alias{condense_quantitative_rows}
\title{Condense quantitative variable rows only}
\usage{
condense_quantitative_rows(df, indent_groups = TRUE)
}
\arguments{
\item{df}{Data.table or data frame}

\item{indent_groups}{Logical. Whether to apply indentation formatting.}
}
\value{
A data.table with condensed continuous/survival rows
}
\description{
Collapses multi-row continuous and survival variables into single rows
while preserving all categorical variable rows (including binary).
Only applies to descriptive tables from desctable().
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{check_convergence}
\alias{check_convergence}
\title{Check model convergence}
\usage{
check_convergence(model)
}
\arguments{
\item{model}{Fitted model object.}
}
\value{
Character string: "Yes", "No", "Suspect", or "Failed"
}
\description{
Checks convergence status for various model types including standard
regression models and mixed-effects models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{format_pvalue}
\alias{format_pvalue}
\title{Format a \emph{p}-value with locale-aware decimal mark}
\usage{
format_pvalue(p, digits, marks)
}
\arguments{
\item{p}{Numeric \emph{p}-value.}

\item{digits}{Integer number of decimal places.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with the formatted \emph{p}-value.
}
\description{
Converts a numeric \emph{p}-value to a display string with the correct
decimal separator and threshold notation (\emph{e.g.,} "< 0.001" or "< 0,001").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{format_continuous_stat}
\alias{format_continuous_stat}
\title{Format continuous statistic for display}
\usage{
format_continuous_stat(stats, stat_type, fmt_str, marks)
}
\arguments{
\item{stats}{Named list of numeric statistics (mean, sd, median, q1, q3, min, max).}

\item{stat_type}{Character string: "mean_sd", "median_iqr", "median_range", or "range".}

\item{fmt_str}{Character string format specification for sprintf.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with formatted statistic.
}
\description{
Converts numeric summary statistics into formatted display strings following
standard conventions (mean \eqn{\pm} SD, median [IQR], range, \emph{etc.}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survtable.R
\name{process_single_outcome}
\alias{process_single_outcome}
\title{Process a single survival outcome}
\usage{
process_single_outcome(
  data,
  outcome,
  outcome_label,
  by,
  times,
  probs,
  stats,
  type,
  conf_level,
  conf_type,
  digits,
  time_digits,
  percent,
  test,
  test_type,
  total,
  total_label,
  time_unit,
  time_label,
  median_label,
  labels,
  na_rm,
  marks = NULL,
  ...
)
}
\arguments{
\item{data}{Data.table with the source data.}

\item{outcome}{Character string giving a Surv() expression.}

\item{outcome_label}{Character label for the outcome, or NULL for a
single-outcome table.}

\item{by}{Character name of stratifying variable.}

\item{times}{Numeric vector of time points.}

\item{probs}{Numeric vector of probabilities.}

\item{stats}{Character vector of statistics to include.}

\item{type}{Character string specifying probability type.}

\item{conf_level}{Numeric confidence level for interval estimates.}

\item{conf_type}{Character string specifying confidence interval type.}

\item{digits}{Integer decimal places for percentages.}

\item{time_digits}{Integer decimal places for time values.}

\item{percent}{Logical whether to display as percentages.}

\item{test}{Logical whether to perform a group comparison test.}

\item{test_type}{Character string specifying test type.}

\item{total}{Logical or character controlling total column.}

\item{total_label}{Character label for total column.}

\item{time_unit}{Character time unit for column headers.}

\item{time_label}{Character template for time column headers.}

\item{median_label}{Character label for median row.}

\item{labels}{Named character vector of display labels for group levels.}

\item{na_rm}{Logical whether to drop rows with missing time, status, or
stratifying values.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}

\item{...}{Additional arguments passed to \code{survival::survfit()}.}
}
\value{
List with the formatted table, the raw table, the survfit objects,
  and the test result.
}
\description{
Parses a Surv() expression, fits the survival models, performs the group
comparison test where requested, and assembles the formatted and raw
tables for one outcome. Called once per outcome by survtable().
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{format_categorical_stat}
\alias{format_categorical_stat}
\title{Format a categorical statistic for display}
\usage{
format_categorical_stat(n, total, stat_type, marks)
}
\arguments{
\item{n}{Integer count for the category.}

\item{total}{Integer total count for percentage calculation.}

\item{stat_type}{Character string: \code{"n"}, \code{"n_percent"}, or
\code{"percent"}.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with the formatted statistic.
}
\description{
Converts frequency counts into formatted display strings following
standard conventions (n, n (\%), \% only) with locale-aware decimal marks.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{add_variable_padding}
\alias{add_variable_padding}
\title{Add padding to exported table variables}
\usage{
add_variable_padding(df)
}
\arguments{
\item{df}{Data.table with Variable column.}
}
\value{
Data.table with padding rows inserted between variable groups.
}
\description{
Inserts blank padding rows between variable groups in exported tables
for improved visual separation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{order_comparison_columns}
\alias{order_comparison_columns}
\title{Order comparison columns based on model type}
\usage{
order_comparison_columns(comparison, model_type)
}
\arguments{
\item{comparison}{Data.table with comparison metrics.}

\item{model_type}{Character string indicating model type.}
}
\value{
Data.table with reordered columns.
}
\description{
Reorders columns in the comparison table to follow a logical sequence
appropriate for the model type.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{format_pvalues_fit}
\alias{format_pvalues_fit}
\title{Format \emph{p}-values for display}
\usage{
format_pvalues_fit(p, digits = 3, marks = NULL)
}
\arguments{
\item{p}{Numeric vector of \emph{p}-values.}

\item{digits}{Integer number of decimal places. Also determines the threshold
for "less than" display: threshold = 10^(-digits). Default is 3.}

\item{marks}{Optional list with \code{big.mark} and \code{decimal.mark} as
returned by \code{\link{resolve_number_marks}}.}
}
\value{
Character vector of formatted \emph{p}-values.
}
\description{
Converts numeric \emph{p}-values to formatted character strings using vectorized
operations. Values below the threshold (determined by digits parameter) 
display as "< 0.001" (for digits=3), "< 0.0001" (for digits=4), \emph{etc.}
NA values display as "-".
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{is_reference_category}
\alias{is_reference_category}
\title{Check if category name is a standard reference/negative value}
\usage{
is_reference_category(
  category,
  label = NULL,
  norm_category = NULL,
  norm_label = NULL
)
}
\arguments{
\item{category}{Character string with the category name.}

\item{label}{Optional character string with the variable label. If provided,
checks if category is "No [label]" or similar patterns.}

\item{norm_category}{Optional pre-normalized category (lowercase, trimmed).
If provided, skips normalization for performance.}

\item{norm_label}{Optional pre-normalized label (lowercase, trimmed).
If provided, skips normalization for performance.}
}
\value{
Logical indicating whether category is a reference/negative value.
}
\description{
Determines whether a category name represents a standard reference or negative
value that indicates absence. Used to suppress redundant category names when
condensing binary variables.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{extract_model_metrics}
\alias{extract_model_metrics}
\title{Extract comprehensive model metrics based on academic consensus}
\usage{
extract_model_metrics(model, raw_data, model_type)
}
\arguments{
\item{model}{Fitted model object.}

\item{raw_data}{Data.table with raw model information.}

\item{model_type}{Character string indicating model type.}
}
\value{
Named list of metrics.
}
\description{
Extracts quality control metrics from fitted models for comparison.
Supports GLM, Cox, linear, and mixed-effects models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{add_header_padding}
\alias{add_header_padding}
\title{Add padding to exported table headers}
\usage{
add_header_padding(col_names)
}
\arguments{
\item{col_names}{Character vector of column names.}
}
\value{
Character vector with LaTeX padding rules added.
}
\description{
Adds LaTeX vertical spacing rules to column headers for proper vertical
alignment in PDF/LaTeX exports.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compfit.R
\name{print.compfit_result}
\alias{print.compfit_result}
\title{Print method showing scoring methodology}
\usage{
\method{print}{compfit_result}(x, ...)
}
\arguments{
\item{x}{Object of class \code{compfit_result}.}

\item{...}{Additional arguments passed to print methods.}
}
\value{
Invisibly returns the input object \code{x}. Called for its
  side effect of printing a formatted summary to the console.
}
\description{
Print method showing scoring methodology
}
\seealso{
Other regression functions:
\code{\link[=compfit]{compfit()}},
\code{\link[=fit]{fit()}},
\code{\link[=fullfit]{fullfit()}},
\code{\link[=multifit]{multifit()}},
\code{\link[=print.fit_result]{print.fit_result()}},
\code{\link[=print.fullfit_result]{print.fullfit_result()}},
\code{\link[=print.multifit_result]{print.multifit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{safe_round}
\alias{safe_round}
\title{Safe rounding that handles \code{NULL} and NA}
\usage{
safe_round(x, digits)
}
\arguments{
\item{x}{Numeric value to round.}

\item{digits}{Integer number of decimal places.}
}
\value{
Rounded numeric value, or NA_real_ if input is \code{NULL}/NA/empty.
}
\description{
Rounds numeric values while gracefully handling \code{NULL}, empty, and NA inputs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{detect_model_type_auto}
\alias{detect_model_type_auto}
\title{Auto-detect model type based on outcome and random effects}
\usage{
detect_model_type_auto(data, outcome, has_random_effects, family = "binomial")
}
\arguments{
\item{data}{Data.frame or data.table containing the outcome variable.}

\item{outcome}{Character string specifying the outcome variable or Surv() expression.}

\item{has_random_effects}{Logical indicating if random effects are specified.}

\item{family}{Character string for GLM family (default "binomial").}
}
\value{
Character string indicating detected model type.
}
\description{
Determines the appropriate model type based on outcome variable
characteristics and presence of random effects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survtable.R
\name{print.survtable}
\alias{print.survtable}
\title{Print method for survtable}
\usage{
\method{print}{survtable}(x, ...)
}
\arguments{
\item{x}{Object of class \code{survtable}.}

\item{...}{Additional arguments passed to print methods.}
}
\value{
Invisibly returns the input object \code{x}. Called for its
  side effect of printing a formatted summary to the console.
}
\description{
Print method for survtable
}
\seealso{
Other descriptive functions:
\code{\link[=desctable]{desctable()}},
\code{\link[=survtable]{survtable()}}
}
\concept{descriptive functions}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{process_survival}
\alias{process_survival}
\title{Process time-to-event variable}
\usage{
process_survival(
  data,
  var,
  var_label,
  group_var,
  digits,
  conf_level = 0.95,
  na_include,
  na_label,
  test,
  total,
  total_label,
  marks = NULL,
  ...
)
}
\arguments{
\item{data}{Data.table containing the survival variables.}

\item{var}{Character string with Surv() expression (\emph{e.g.,} "Surv(time, status)").}

\item{var_label}{Character string label for display.}

\item{group_var}{Optional character string naming the grouping variable.}

\item{digits}{Integer number of decimal places.}

\item{conf_level}{Numeric confidence level for confidence intervals.}

\item{na_include}{Logical whether to include missing values.}

\item{na_label}{Character string label for missing values.}

\item{test}{Logical whether to perform log-rank test.}

\item{total}{Logical or character controlling total column display.}

\item{total_label}{Character string label for total column.}

\item{...}{Additional arguments (currently unused).}
}
\value{
List with 'formatted' and 'raw' data.table components.
}
\description{
Calculates survival statistics including median survival times with
confidence intervals, with optional grouping and log-rank testing. Parses
Surv() expressions and uses \pkg{survival} package functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_model_data}
\alias{get_model_data}
\title{Get data from model object (works with S3 and S4)}
\usage{
get_model_data(model)
}
\arguments{
\item{model}{Fitted model object (S3 or S4).}
}
\value{
Data frame or data.table used to fit the model, or \code{NULL} if unavailable.
}
\description{
Retrieves the original data used to fit a model. Checks multiple locations
including model attributes, $data slot, $model slot, and @frame for S4.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{determine_alignment}
\alias{determine_alignment}
\title{Determine alignment for exported tables}
\usage{
determine_alignment(df)
}
\arguments{
\item{df}{Data.frame or data.table to determine alignment for.}
}
\value{
Character string with alignment codes (\emph{e.g.,} "rlcc").
}
\description{
Creates column alignment string for LaTeX tables. Variable and Group
columns are left-aligned; all others are centered.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{bold_pvalues_ft}
\alias{bold_pvalues_ft}
\title{Bold significant \emph{p}-values in flextable outputs}
\usage{
bold_pvalues_ft(ft, df, p_threshold = 0.05)
}
\arguments{
\item{ft}{\pkg{flextable} object.}

\item{df}{The source data.table.}

\item{p_threshold}{Numeric \emph{p}-value threshold for significance.}
}
\value{
Flextable object with significant \emph{p}-values bolded.
}
\description{
Applies bold formatting to significant \emph{p}-values in \pkg{flextable} objects
by detecting values below threshold or "< 0.001" patterns.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_utils.R
\name{fix_negative_zero}
\alias{fix_negative_zero}
\title{Fix negative zero in formatted strings}
\usage{
fix_negative_zero(x, marks = NULL)
}
\arguments{
\item{x}{Character vector of formatted numbers.}

\item{marks}{Optional list with \code{big.mark} and \code{decimal.mark} as
returned by \code{\link{resolve_number_marks}}. When \code{NULL}, uses
the default US period decimal.}
}
\value{
Character vector with negative zeros corrected.
}
\description{
Corrects floating-point rounding artifacts that produce "-0.00" or similar
negative zero strings. Works on character vectors, replacing patterns like
"-0.00", "-0.000", \emph{etc.} with their positive equivalents, even when embedded
within larger strings (\emph{e.g.,} "(-0.00, 1.23)" becomes "(0.00, 1.23)").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{condense_table_rows}
\alias{condense_table_rows}
\title{Condense table rows for more compact display}
\usage{
condense_table_rows(df, indent_groups = TRUE)
}
\arguments{
\item{df}{Data.table with Variable and Group columns.}

\item{indent_groups}{Logical whether indentation will be applied (affects processing).}
}
\value{
Data.table with condensed rows.
}
\description{
Collapses multi-row variables into single rows for compact tables.
Continuous variables show only the first statistic row, binary categorical
variables show only the non-reference category, and survival variables
show only the median row.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_utils.R
\name{process_variable}
\alias{process_variable}
\title{Process variable wrapper}
\usage{
process_variable(
  data,
  var,
  group_var = NULL,
  stats_continuous,
  stats_categorical,
  digits,
  conf_level = 0.95,
  na_include,
  na_label,
  test,
  test_continuous,
  test_categorical,
  total,
  total_label,
  labels,
  na_percent,
  p_per_stat = FALSE,
  marks = NULL,
  ...
)
}
\arguments{
\item{data}{Data.table containing the variable.}

\item{var}{Character string naming the variable to process.}

\item{group_var}{Optional character string naming the grouping variable.}

\item{stats_continuous}{Character vector of statistics for continuous variables.}

\item{stats_categorical}{Character vector of statistics for categorical variables.}

\item{digits}{Integer number of decimal places for continuous statistics.}

\item{conf_level}{Numeric confidence level for survival confidence intervals.}

\item{na_include}{Logical whether to include missing values as a category.}

\item{na_label}{Character string label for missing values.}

\item{test}{Logical whether to perform statistical tests.}

\item{test_continuous}{Character string specifying test type for continuous variables.}

\item{test_categorical}{Character string specifying test type for categorical variables.}

\item{total}{Logical or character controlling total column display.}

\item{total_label}{Character string label for total column.}

\item{labels}{Named character vector of variable labels.}

\item{na_percent}{Logical whether to include NA in percentage denominators.}

\item{p_per_stat}{Logical whether to show separate \emph{p}-values per statistic for
continuous variables. Default \code{FALSE} for better performance.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}

\item{...}{Additional arguments passed to test functions.}
}
\value{
List with 'formatted' and 'raw' data.table components.
}
\description{
Routes variable processing to appropriate handler based on variable type
(continuous, categorical, or survival). Returns both formatted display
strings and raw numeric values.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{format_interaction_term}
\alias{format_interaction_term}
\title{Format interaction term for display}
\usage{
format_interaction_term(term, labels = NULL)
}
\arguments{
\item{term}{Character string of the interaction term from model coefficients.}

\item{labels}{Optional named vector of labels for variable names.}
}
\value{
Formatted interaction term string.
}
\description{
Converts R's internal interaction term format (\emph{e.g.,} "treatmentDrug A:stageII")
to a more readable format (\emph{e.g.,} "Treatment (Drug A) × Stage (II)").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{perform_survival_test}
\alias{perform_survival_test}
\title{Perform survival comparison test}
\usage{
perform_survival_test(surv_obj, group_var, test_type)
}
\arguments{
\item{surv_obj}{Survival object created by Surv().}

\item{group_var}{Vector of group assignments.}

\item{test_type}{Character string specifying test type.}
}
\value{
List with test statistic, \emph{p}-value, and test type.
}
\description{
Performs statistical test comparing survival curves across groups.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compfit.R
\name{compfit}
\alias{compfit}
\title{Compare Multiple Regression Models}
\usage{
compfit(
  data,
  outcome,
  model_list,
  model_names = NULL,
  interactions_list = NULL,
  random = NULL,
  model_type = "auto",
  family = "binomial",
  conf_level = 0.95,
  p_digits = 3,
  include_coefficients = FALSE,
  scoring_weights = NULL,
  labels = NULL,
  number_format = NULL,
  verbose = NULL,
  ...
)
}
\arguments{
\item{data}{Data frame or data.table containing the dataset.}

\item{outcome}{Character string specifying the outcome variable. For survival
analysis, use \code{Surv()} syntax (\emph{e.g.,} \code{"Surv(time, status)"}).}

\item{model_list}{List of character vectors, each containing predictor names
for one model. Can also be a single character vector to auto-generate nested models.}

\item{model_names}{Character vector of names for each model. If \code{NULL}, uses
"Model 1", "Model 2", \emph{etc.} Default is \code{NULL}.}

\item{interactions_list}{List of character vectors specifying interaction
terms for each model. Each element corresponds to one model in model_list.
Use \code{NULL} for models without interactions. Use colon notation for interactions
(\emph{e.g.,} \code{c("age:treatment")}). If \code{NULL}, no interactions are added to
any model. Default is \code{NULL}.}

\item{random}{Character string specifying the random-effects formula for 
mixed-effects models (\code{glmer}, \code{lmer}, \code{coxme}). Use standard
\code{lme4}/\code{coxme} syntax, \emph{e.g.,} \code{"(1|site)"} for random
intercepts by site. This random effects formula is applied to all models in the
comparison. Alternatively, random effects can be included directly in the predictor 
vectors within \code{model_list} using the same syntax, which allows
different random effects structures across models. Default is \code{NULL}.}

\item{model_type}{Character string specifying model type. If \code{"auto"}, detects
based on outcome. Options include:
\itemize{
  \item \code{"auto"} - Automatically detect based on outcome type (default)
  \item \code{"glm"} - Generalized linear model. Supports logistic, Poisson, 
    Gamma, Gaussian via \code{family} parameter.
  \item \code{"lm"} - Linear regression for continuous outcomes
  \item \code{"coxph"} - Cox proportional hazards for survival analysis
  \item \code{"negbin"} - Negative binomial regression for overdispersed counts 
    (requires MASS package)
  \item \code{"lmer"} - Mixed-effects linear regression for clustered continuous
    outcomes
  \item \code{"glmer"} - Mixed-effects logistic regression for clustered
    categorical outcomes
  \item \code{"coxme"} - Mixed-effects Cox regression for clustered time-to-event
    outcomes
}}

\item{family}{For GLM and GLMER models, specifies the error distribution and link function.
Common options include:
\itemize{
  \item \code{"binomial"} - Logistic regression for binary outcomes (default)
  \item \code{"poisson"} - Poisson regression for count data
  \item \code{"quasibinomial"} - Logistic with overdispersion
  \item \code{"quasipoisson"} - Poisson with overdispersion
  \item \code{"gaussian"} - Normal distribution (linear regression via GLM)
  \item \code{"Gamma"} - Gamma for positive continuous data
  \item \code{"inverse.gaussian"} - For positive, highly skewed data
}
For negative binomial, use \code{model_type = "negbin"} instead.
See \code{\link[stats]{family}} for all options.}

\item{conf_level}{Numeric confidence level for intervals. Default is 0.95.}

\item{p_digits}{Integer specifying the number of decimal places for
\emph{p}-values. Values smaller than \code{10^(-p_digits)} are displayed
as \code{"< 0.001"} (for \code{p_digits = 3}), \code{"< 0.0001"} (for
\code{p_digits = 4}), etc. Default is 3.}

\item{include_coefficients}{Logical. If TRUE, includes a second table with
coefficient estimates. Default is FALSE.}

\item{scoring_weights}{Named list of scoring weights. Each weight should be
between 0 and 1, and they should sum to 1. Available metrics depend on model
type. If \code{NULL}, uses sensible defaults. See Details for available metrics.}

\item{labels}{Named character vector providing custom display labels for
variables. Default is \code{NULL}.}

\item{number_format}{Character string or two-element character vector
  controlling thousand and decimal separators in formatted output. Named
  presets:
  \itemize{
    \item \code{"us"} - Comma thousands, period decimal: \code{1,234.56} [default]
    \item \code{"eu"} - Period thousands, comma decimal: \code{1.234,56}
    \item \code{"space"} - Thin-space thousands, period decimal: \code{1 234.56}
      (SI/ISO 31-0)
    \item \code{"none"} - No thousands separator: \code{1234.56}
  }
  Or provide a custom two-element vector \code{c(big.mark, decimal.mark)},
  \emph{e.g.}, \code{c("'", ".")} for Swiss-style: \verb{1'234.56}.

  When \code{NULL} (default), uses
  \code{getOption("summata.number_format", "us")}. Set the global option
  once per session to avoid passing this argument repeatedly:
  \preformatted{
    options(summata.number_format = "eu")
  }}

\item{verbose}{Logical. If \code{TRUE}, displays model fitting warnings
(\emph{e.g.,} singular fit, convergence issues). If \code{FALSE} (default),
routine fitting messages are suppressed while unexpected warnings are
preserved. When \code{NULL}, uses
\code{getOption("summata.verbose", FALSE)}.}

\item{...}{Additional arguments passed to model fitting functions.}
}
\value{
A data.table with class "compfit_result" containing:
  \describe{
    \item{Model}{Model name/identifier}
    \item{CMS}{Composite Model Score for model selection (higher is better)}
    \item{N}{Sample size}
    \item{Events}{Number of events (for survival/logistic)}
    \item{Predictors}{Number of predictors}
    \item{Converged}{Whether model converged properly}
    \item{AIC}{Akaike Information Criterion}
    \item{BIC}{Bayesian Information Criterion}
    \item{\emph{R}\eqn{^2} / Pseudo-\emph{R}\eqn{^2}}{McFadden pseudo-R-squared (GLM)}
    \item{Concordance}{C-statistic (logistic/survival)}
    \item{Brier Score}{Brier accuracy score (logistic)}
    \item{Global \emph{p}}{Overall model \emph{p}-value}
  }
  
  Attributes include:
  \describe{
    \item{models}{List of fitted model objects}
    \item{coefficients}{Coefficient comparison table (if requested)}
    \item{best_model}{Name of recommended model}
  }
}
\description{
Fits multiple regression models and provides a comprehensive comparison table
with model quality metrics, convergence diagnostics, and selection guidance.
Computes a composite score combining multiple quality metrics to facilitate 
rapid model comparison and selection.
}
\details{
This function fits all specified models and computes comprehensive quality
metrics for comparison. It generates a Composite Model Score (CMS) that 
combines multiple metrics: lower AIC/BIC (information criteria), higher 
concordance (discrimination), and model convergence status.

For GLMs, McFadden's pseudo-R-squared is calculated as 1 - (logLik/logLik_null).
For survival models, the global \emph{p}-value comes from the log-rank test.

Models that fail to converge are flagged and penalized in the composite score.

\strong{Interaction Terms:}

When \code{interactions_list} is provided, each element specifies the
interaction terms for the corresponding model in \code{model_list}. This is
particularly useful for testing whether adding interactions improves model fit:
\itemize{
  \item Use \code{NULL} for models without interactions
  \item Specify interactions using colon notation: \code{c("age:treatment", "sex:stage")}
  \item Main effects for all variables in interactions must be in the predictor list
  \item Common pattern: Compare main effects model vs model with interactions
}

Scoring weights can be customized based on model type:
\itemize{
  \item GLM: \code{"convergence"}, \code{"aic"}, \code{"concordance"}, \code{"pseudo_r2"}, \code{"brier"} 
  \item Cox: \code{"convergence"}, \code{"aic"}, \code{"concordance"}, \code{"global_p"}
  \item Linear: \code{"convergence"}, \code{"aic"}, \code{"pseudo_r2"}, \code{"rmse"}
}
Default weights emphasize discrimination (concordance) and model fit (AIC).

The composite score is designed as a tool to quickly rank models by their 
quality metrics. It should be used alongside traditional model selection 
criteria rather than as a definitive model selection method.
}
\examples{
# Load example data
data(clintrial)
data(clintrial_labels)

# Information criteria assume a common sample, so the candidate predictors
# are restricted to complete cases before any comparison
comparison_data <- na.omit(
    clintrial[, c("readmission_30d", "os_months", "os_status", "age", "sex",
                  "smoking", "diabetes", "stage", "ecog", "grade",
                  "treatment")]
)

# Example 1: Compare nested logistic regression models
models <- list(
    base = c("age", "sex"),
    clinical = c("age", "sex", "smoking", "diabetes"),
    full = c("age", "sex", "smoking", "diabetes", "stage", "ecog")
)

comparison <- compfit(
    data = comparison_data,
    outcome = "readmission_30d",
    model_list = models,
    model_names = c("Base", "Clinical", "Full")
)
comparison

\donttest{

# Example 2: Compare Cox survival models
surv_models <- list(
    simple = c("age", "sex"),
    clinical = c("age", "sex", "stage", "grade")
)

surv_comparison <- compfit(
    data = comparison_data,
    outcome = "Surv(os_months, os_status)",
    model_list = surv_models,
    model_type = "coxph"
)
surv_comparison

# Example 3: Test the effect of adding an interaction term.
# Treatment efficacy differs by disease stage in these data, so the
# interaction is expected to improve the fit.
interaction_models <- list(
    main = c("age", "treatment", "stage"),
    interact = c("age", "treatment", "stage")
)

interaction_comp <- compfit(
    data = comparison_data,
    outcome = "Surv(os_months, os_status)",
    model_list = interaction_models,
    model_type = "coxph",
    model_names = c("Main Effects", "With Interaction"),
    interactions_list = list(
        NULL,
        c("treatment:stage")
    )
)
interaction_comp

# Example 4: Include coefficient comparison table
detailed <- compfit(
    data = comparison_data,
    outcome = "readmission_30d",
    model_list = models,
    include_coefficients = TRUE,
    labels = clintrial_labels
)

# Access coefficient table
coef_table <- attr(detailed, "coefficients")
coef_table

# Example 5: Access fitted model objects
fitted_models <- attr(comparison, "models")
names(fitted_models)

# Example 6: Get best model recommendation
best <- attr(comparison, "best_model")
cat("Recommended model:", best, "\n")

}

}
\seealso{
\code{\link{fit}} for individual model fitting,
\code{\link{fullfit}} for automated variable selection,
\code{\link{tablesave}} for exporting tables

Other regression functions:
\code{\link[=fit]{fit()}},
\code{\link[=fullfit]{fullfit()}},
\code{\link[=multifit]{multifit()}},
\code{\link[=print.compfit_result]{print.compfit_result()}},
\code{\link[=print.fit_result]{print.fit_result()}},
\code{\link[=print.fullfit_result]{print.fullfit_result()}},
\code{\link[=print.multifit_result]{print.multifit_result()}},
\code{\link[=print.uniscreen_result]{print.uniscreen_result()}},
\code{\link[=uniscreen]{uniscreen()}}
}
\concept{regression functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{format_number}
\alias{format_number}
\title{Format numeric value with fixed decimal places}
\usage{
format_number(x, digits, marks = NULL)
}
\arguments{
\item{x}{Numeric value to format.}

\item{digits}{Integer number of decimal places.}

\item{marks}{Optional list with \code{big.mark} and \code{decimal.mark} as
returned by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with formatted value.
}
\description{
Formats a numeric value to a specified number of decimal places, removing
leading/trailing whitespace and fixing negative zero display (\emph{e.g.,} "-0.00"
becomes "0.00"). When \code{marks} is supplied, applies locale-appropriate
decimal mark substitution.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_analysis_counts}
\alias{get_analysis_counts}
\title{Count the observations available to an analysis}
\usage{
get_analysis_counts(
  data,
  outcome_vars,
  predictor_vars = NULL,
  event_var = NULL
)
}
\arguments{
\item{data}{Data frame or data.table supplied to the analysis.}

\item{outcome_vars}{Character vector of response variable names.}

\item{predictor_vars}{Character vector of predictor variable names, or a list
of such vectors when several models are described.}

\item{event_var}{Character name of the event indicator, or \code{NULL} for
models that carry no event count. Factors are coded as the modeling
functions code them, with any level beyond the first counting as an event.}
}
\value{
Named list with \code{n_supplied}, \code{n_analyzed},
  \code{n_missing_outcome} and \code{n_missing_predictor}, and, where an
  event indicator was supplied, \code{events_supplied} and
  \code{events_analyzed}. \code{NULL} when the counts cannot be determined.
}
\description{
Summarizes how many of the supplied observations could enter a model, and
separates those excluded for a missing response from those excluded for a
missing covariate. The distinction matters: a missing response and a missing
covariate raise different questions about whether a complete-case analysis is
appropriate.
}
\details{
Several models may be described at once by passing a list of predictor sets,
as when a univariable screen fits one model per predictor. The analyzed count
is then returned as one element per set.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{format_num}
\alias{format_num}
\title{Format a numeric value with locale-aware separators}
\usage{
format_num(x, fmt_str, marks)
}
\arguments{
\item{x}{Numeric value to format.}

\item{fmt_str}{Character string \code{sprintf} format specification
(\emph{e.g.,} \code{"\%.1f"}). Used only for |x| < 1000.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with the formatted number.
}
\description{
General-purpose number formatter used by all display functions. For values
\eqn{\ge} 1000 (in absolute value), inserts the appropriate
thousands separator. Fixes negative-zero display artefacts.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{calculate_forest_layout}
\alias{calculate_forest_layout}
\title{Calculate table layout for forest plots}
\usage{
calculate_forest_layout(
  to_show_exp_clean,
  show_n,
  show_events,
  indent_groups,
  condense_table,
  effect_label,
  ref_label,
  font_size,
  table_width = 0.6,
  rangeb,
  center_padding
)
}
\arguments{
\item{to_show_exp_clean}{Data.table with formatted display data for the plot.}

\item{show_n}{Logical whether to include sample size column.}

\item{show_events}{Logical whether to include events column.}

\item{indent_groups}{Logical whether groups are indented (affects level column).}

\item{condense_table}{Logical whether table is condensed (affects level column).}

\item{effect_label}{Character string describing effect measure type.}

\item{ref_label}{Character string label for reference categories.}

\item{font_size}{Numeric font size for width calculations.}

\item{table_width}{Numeric proportion of total width for table (0-1).}

\item{rangeb}{Numeric vector of length 2 with plot x-axis range.}

\item{center_padding}{Numeric additional padding for effect column.}
}
\value{
List with table_width, forest_width, positions, rangeplot_start,
  total_width, and effect_abbrev components.
}
\description{
Computes column widths and positions for the table portion of a forest plot.
Determines spacing based on content width, font size, and desired table/forest
proportion. Returns positions in log-scale units for plot coordinate system.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forestsave.R
\name{embed_plot_fonts}
\alias{embed_plot_fonts}
\title{Embed fonts in a PDF after it is written}
\usage{
embed_plot_fonts(file, quiet = FALSE)
}
\arguments{
\item{file}{Character string giving the path to the written file.}

\item{quiet}{Logical. Suppress the confirmation message.}
}
\value{
Invisibly returns \code{TRUE} where the fonts were embedded and
  \code{FALSE} otherwise.
}
\description{
Post-processes a PDF so that its fonts are embedded, through
\code{grDevices::embedFonts()}. This is offered as an alternative to drawing
with a Cairo device, which embeds fonts but renders italics unreliably.
If Cairo is not selected, embedding is performed by Ghostscript (external).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{should_condense_binary}
\alias{should_condense_binary}
\title{Check if a binary variable should be condensed without category suffix}
\usage{
should_condense_binary(ref_category, non_ref_category, label = NULL)
}
\arguments{
\item{ref_category}{Character string with the reference category name
(the level with NA estimate).}

\item{non_ref_category}{Character string with the non-reference category name
(the level with the actual estimate).}

\item{label}{Optional character string with the variable label. Used for
intelligent matching (\emph{e.g.,} "30-Day Readmission" label with 
"30-day readmission" / "No 30-day readmission" levels).}
}
\value{
Logical indicating whether the binary variable should be condensed
  without appending the category name.
}
\description{
Uses a greedy/liberal approach to determine if a binary variable's condensed
display should omit the category name. Returns \code{TRUE} if EITHER level of the
binary variable is a standard reference/affirmative value, OR if either level
matches/contains the variable label.
}
\details{
This function is designed for binary (2-level) categorical variables where
one level is a reference and one is the "event" or "condition" level.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{resolve_number_marks}
\alias{resolve_number_marks}
\title{Resolve number format marks}
\usage{
resolve_number_marks(number_format = NULL)
}
\arguments{
\item{number_format}{Character string specifying a named preset, a
  two-element character vector \code{c(big.mark, decimal.mark)}, or
  \code{NULL} to use the global option (falling back to \code{"us"}).

  Named presets:
  \describe{
    \item{\code{"us"}}{Comma thousands, period decimal: 1,234.56}
    \item{\code{"eu"}}{Period thousands, comma decimal: 1.234,56}
    \item{\code{"space"}}{Thin-space thousands, period decimal: 1 234.56
      (SI/ISO 31-0 standard)}
    \item{\code{"none"}}{No thousands separator, period decimal: 1234.56}
  }

  Custom vector: \code{c(",", ".")} or \code{c(".", ",")} \emph{etc.} The first
  element is \code{big.mark}, the second is \code{decimal.mark}.}
}
\value{
A list with components:
  \describe{
    \item{\code{big.mark}}{Character string for thousands separator.}
    \item{\code{decimal.mark}}{Character string for decimal separator.}
  }
}
\description{
Converts a \code{number_format} specification into a list of \code{big.mark}
and \code{decimal.mark} values used by all downstream formatting functions.
Supports named presets, custom two-element vectors, and the global
\code{summata.number_format} option.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surv_utils.R
\name{format_pvalue_survtable}
\alias{format_pvalue_survtable}
\title{Format \emph{p}-value for survtable}
\usage{
format_pvalue_survtable(p, digits, marks = NULL)
}
\arguments{
\item{p}{Numeric \emph{p}-value.}

\item{digits}{Integer decimal places.}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character-formatted \emph{p}-value.
}
\description{
Provides \emph{p}-value formatting to the survtable result.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{combine_coefficient_tables}
\alias{combine_coefficient_tables}
\title{Combine coefficient tables from multiple models}
\usage{
combine_coefficient_tables(coef_list, model_names)
}
\arguments{
\item{coef_list}{List of data.tables containing coefficient information.}

\item{model_names}{Character vector of model names corresponding to coef_list.}
}
\value{
Combined data.table with Model column, or \code{NULL} if empty.
}
\description{
Merges coefficient tables from multiple fitted models into a single
data.table with a Model identifier column.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{is_affirmative_category}
\alias{is_affirmative_category}
\title{Check if category name should be suppressed in condensed label}
\usage{
is_affirmative_category(
  category,
  label = NULL,
  norm_category = NULL,
  norm_label = NULL
)
}
\arguments{
\item{category}{Character string with the category name.}

\item{label}{Optional character string with the variable label. If provided,
returns \code{TRUE} when category is a case-insensitive match or substring.}

\item{norm_category}{Optional pre-normalized category (lowercase, trimmed).
If provided, skips normalization for performance.}

\item{norm_label}{Optional pre-normalized label (lowercase, trimmed).
If provided, skips normalization for performance.}
}
\value{
Logical indicating whether category should be suppressed.
}
\description{
Determines whether a category name should be suppressed when condensing
binary variables. Returns \code{TRUE} for standard affirmative values (\emph{e.g.,} "Yes", 
"1", "Positive"), standard reference values (\emph{e.g.,} "No", "Absent", "None"),
or when the category name essentially matches the variable label 
(case-insensitive comparison).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fullfit.R
\name{finalize_column_names}
\alias{finalize_column_names}
\title{Finalize Column Names for Display}
\usage{
finalize_column_names(
  result,
  uni_raw,
  multi_raw,
  exponentiate,
  columns,
  metrics,
  conf_level = 0.95
)
}
\arguments{
\item{result}{Data.table with columns to rename. Expected to contain some
combination of: uni_effect, uni_p, multi_effect, multi_p.}

\item{uni_raw}{Raw univariable data.table used to determine effect type
by checking for presence of OR, HR, RR, or Coefficient columns.}

\item{multi_raw}{Raw multivariable data.table used to determine adjusted
effect type.}

\item{exponentiate}{Logical or \code{NULL}. If \code{TRUE}, forces 
exponentiated labels (OR, HR, RR). If \code{FALSE}, uses "Coefficient".
If \code{NULL}, auto-detects from raw data columns.}

\item{columns}{Character string indicating which columns are present
(\code{"both"}, \code{"uni"}, or \code{"multi"}). Used for context but
renaming is based on actual column presence.}

\item{metrics}{Character vector of metrics being displayed (\code{"effect"},
\code{"p"}, or both). Used for context.}
}
\value{
The input data.table with columns renamed
}
\description{
Renames internal column names (uni_effect, uni_p, multi_effect, multi_p) to
publication-ready display names with appropriate effect measure labels
(OR, HR, RR, aOR, aHR, aRR, Coefficient, \emph{etc.}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest_utils.R
\name{forest_ci_separator}
\alias{forest_ci_separator}
\title{Determine CI separator for forest plot text annotations}
\usage{
forest_ci_separator(has_negatives, marks = NULL)
}
\arguments{
\item{has_negatives}{Logical indicating whether any CI bounds are negative.}

\item{marks}{Optional list with \code{big.mark} and \code{decimal.mark} as
returned by \code{\link{resolve_number_marks}}.}
}
\value{
Character string separator.
}
\description{
Returns the appropriate separator string between CI lower and upper bounds
in forest plot annotations. Considers whether values may be negative and
the current locale's decimal mark.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{build_model_row}
\alias{build_model_row}
\title{Build comparison row for successfully fitted model}
\usage{
build_model_row(
  model_name,
  n_predictors,
  converged,
  metrics,
  model_type,
  marks = NULL
)
}
\arguments{
\item{model_name}{Character string name of the model.}

\item{n_predictors}{Integer number of predictors in the model.}

\item{converged}{Character string convergence status.}

\item{metrics}{Named list of extracted model metrics.}

\item{model_type}{Character string indicating model type.}
}
\value{
Data.table with single row of formatted metrics.
}
\description{
Creates a comparison table row with extracted metrics for a successfully
fitted model.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{get_paper_settings}
\alias{get_paper_settings}
\title{Get paper size for PDF/LaTeX export}
\usage{
get_paper_settings(paper, margins = NULL)
}
\arguments{
\item{paper}{Character string: "letter", "a4", or "auto".}

\item{margins}{Optional numeric vector of margins (length 1 or 4).}
}
\value{
List with latex_paper, width, height, and margins components.
}
\description{
Returns paper dimensions and margin settings for the specified paper size.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_model_variables}
\alias{get_model_variables}
\title{Identify the variables required by a fitted model}
\usage{
get_model_variables(model, model_class)
}
\arguments{
\item{model}{Fitted model object.}

\item{model_class}{Character string of the model class.}
}
\value{
Character vector of variable names, or \code{character(0)} when the
  formula cannot be recovered.
}
\description{
Returns the names of every variable referenced by a model formula, including
the response, stratification factors, offsets and random-effect grouping
factors. Terms are expanded where possible, so a model specified with a dot
on the right-hand side (\emph{e.g.,} \code{y ~ .}) returns the complete set
of variables rather than the literal dot.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{calculate_lmer_scores}
\alias{calculate_lmer_scores}
\title{Calculate scores for linear mixed-effects models}
\usage{
calculate_lmer_scores(comparison, weights, scores, n_models)
}
\arguments{
\item{comparison}{Data.table with model comparison metrics.}

\item{weights}{Named list of scoring weights.}

\item{scores}{List of initialized score vectors.}

\item{n_models}{Integer number of models being compared.}
}
\value{
Updated scores list with calculated values and total.
}
\description{
Computes component scores for lmer models based on marginal R-squared,
conditional R-squared, and ICC metrics.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/number_utils.R
\name{apply_decimal_mark}
\alias{apply_decimal_mark}
\title{Apply locale decimal mark to a sprintf-formatted string}
\usage{
apply_decimal_mark(x, marks)
}
\arguments{
\item{x}{Character string (already formatted with \code{sprintf}).}

\item{marks}{List with \code{big.mark} and \code{decimal.mark} as returned
by \code{\link{resolve_number_marks}}.}
}
\value{
Character string with corrected decimal marks and no negative zeros.
}
\description{
Replaces the period decimal mark in a \code{sprintf}-formatted string with
the locale-appropriate decimal mark, and fixes negative-zero artefacts.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt.R
\name{m2dt}
\alias{m2dt}
\title{Convert Model to Data Table}
\usage{
m2dt(
  data,
  model,
  conf_level = 0.95,
  keep_qc_stats = TRUE,
  include_intercept = TRUE,
  terms_to_exclude = NULL,
  reference_rows = TRUE,
  reference_label = "reference",
  skip_counts = FALSE,
  conf_method = NULL
)
}
\arguments{
\item{data}{Data frame or data.table containing the dataset used to fit the
model. Required for computing group-level sample sizes and event counts.
Where the fitted object retains no model frame, the supplied data is
restricted to complete cases across the model variables so that group-level
sample sizes describe the observations actually used in fitting. Group
sample sizes therefore sum to the model-level \code{n}, and both exclude
observations with a missing response.}

\item{model}{Fitted model object. Supported classes include:
\itemize{
  \item \code{glm} - Generalized linear models (logistic, Poisson, \emph{etc.})
  \item \code{lm} - Linear models
  \item \code{coxph} - Cox proportional hazards models
  \item \code{clogit} - Conditional logistic regression
  \item \code{coxme} - Mixed effects Cox models
  \item \code{lmerMod} - Linear mixed effects models
  \item \code{glmerMod} - Generalized linear mixed effects models
}}

\item{conf_level}{Numeric confidence level for confidence intervals. Must be
between 0 and 1. Default is 0.95 (95\% CI).}

\item{keep_qc_stats}{Logical. If \code{TRUE}, includes model quality statistics 
such as AIC, BIC, \emph{R}\eqn{^2}, concordance, and model fit tests. These appear 
as additional columns in the output. Default is \code{TRUE}.}

\item{include_intercept}{Logical. If \code{TRUE}, includes the model intercept 
in output. If \code{FALSE}, removes the intercept row from results. Useful 
for creating cleaner presentation tables. Default is \code{TRUE}.}

\item{terms_to_exclude}{Character vector of term names to exclude from output.
Useful for removing specific unwanted parameters (\emph{e.g.,} nuisance variables,
spline terms). Default is \code{NULL}. Note: If \code{include_intercept = FALSE}, 
"(Intercept)" is automatically added to this list.}

\item{reference_rows}{Logical. If \code{TRUE}, adds rows for reference 
categories of factor variables with appropriate labels and baseline values 
(OR/HR = 1, Coefficient = 0). This makes tables more complete and easier to 
interpret. Default is \code{TRUE}.}

\item{reference_label}{Character string used to label reference category rows
in the output. Appears in the \code{reference} column. Default is \code{"reference"}.}

\item{skip_counts}{Logical. If \code{TRUE}, skips computation of group-level
sample sizes and event counts (faster but less informative). Default is 
\code{FALSE}.}

\item{conf_method}{Character string controlling the confidence interval method.
If \code{NULL} (default), uses \code{getOption("summata.conf_method", "profile")}.
\itemize{
  \item \code{"profile"} - Profile likelihood intervals for GLM and negative
    binomial models (via \code{MASS::confint.glm()}), exact \emph{t}-distribution
    intervals for linear models. Falls back to Wald on profiling failure.
    Quasi-likelihood families always use Wald (no true likelihood).
  \item \code{"wald"} - Wald intervals (coefficient \eqn{\pm} \emph{z}
    \eqn{\times} SE) for all model types. Faster but less accurate near
    boundary conditions or with small subgroups.
}
Cox and mixed-effects models use Wald intervals regardless of this setting.
Set globally with \code{options(summata.conf_method = "wald")} to use Wald
throughout a session.}
}
\value{
A \code{data.table} containing extracted model information with the 
  following standard columns:
  \describe{
    \item{model_scope}{Character. Either "Univariable" (unadjusted model with 
      single predictor) or "Multivariable" (adjusted model with multiple predictors)}
    \item{model_type}{Character. Type of regression (\emph{e.g.,} "Logistic", "Linear", 
      "Cox PH", "Poisson", \emph{etc.})}
    \item{variable}{Character. Variable name (for factor variables, the base 
      variable name without the level)}
    \item{group}{Character. Group/level name for factor variables; empty string 
      for continuous variables}
    \item{n}{Integer. Number of observations used in fitting the model, that
      is complete cases across the model variables}
    \item{n_group}{Integer. Number of those observations at this variable
      level (factor variables only). Group sizes sum to \code{n} within
      each variable}
    \item{events}{Integer. Number of events among the observations used in
      fitting (for survival and logistic models)}
    \item{events_group}{Integer. Number of events at this variable level
      (for survival and logistic models with factor variables). Group counts
      sum to \code{events} within each variable}
    \item{coefficient}{Numeric. Raw regression coefficient (log odds, log hazard, 
      \emph{etc.})}
    \item{se}{Numeric. Standard error of the coefficient}
    \item{OR/HR/RR/Coefficient}{Numeric. Effect estimate - column name depends on 
      model type:
      \itemize{
        \item \code{OR} for logistic regression (odds ratio)
        \item \code{HR} for Cox models (hazard ratio)
        \item \code{RR} for Poisson regression (rate/risk ratio)
        \item \code{Coefficient} for linear models or other GLMs
      }}
    \item{ci_lower}{Numeric. Lower bound of confidence interval for effect estimate}
    \item{ci_upper}{Numeric. Upper bound of confidence interval for effect estimate}
    \item{statistic}{Numeric. Test statistic (z-value for GLM/Cox, t-value for LM)}
    \item{p_value}{Numeric. \emph{p}-value for coefficient test}
    \item{sig}{Character. Significance markers: \code{***} (p < 0.001), \code{**}
      (\emph{p} < 0.01), \code{*} (\emph{p} < 0.05), \code{.} (\emph{p} < 0.10).}
    \item{sig_binary}{Logical. Binary indicator: \code{TRUE} if \emph{p} < 0.05, 
      \code{FALSE} otherwise}
    \item{reference}{Character. Contains \code{reference_label} for reference 
      category rows when \code{reference_rows = TRUE}, empty string otherwise}
  }
}
\description{
Extracts coefficients, confidence intervals, and comprehensive model statistics 
from fitted regression models and converts them to a standardized data.table 
format suitable for further analysis or publication. This is a core utility 
function frequently used internally by other \pkg{summata} regression functions,
although it can be used as a standalone function as well.
}
\details{
This function is the core extraction utility used by \code{fit()} and other
regression functions. It handles the complexities of different model classes
and provides a consistent output format suitable for tables and forest plots.

\strong{Model Type Detection:}
The function automatically detects model type and applies appropriate:
\itemize{
  \item Effect measure naming (OR, HR, RR, Coefficient)
  \item Confidence interval calculation (see below)
  \item Event counting for binary/survival outcomes
}

\strong{Confidence Interval Methods:}
The CI method is selected per model class using \code{stats::confint()}
dispatch:
\itemize{
  \item \strong{GLM/negative binomial}: Profile likelihood via
    \code{MASS::confint.glm()}, except quasi-families which use Wald
  \item \strong{Linear models}: Exact \emph{t}-distribution via
    \code{confint.lm()}
  \item \strong{Cox PH}: Wald intervals (coefficient \eqn{\pm}
    \emph{z} \eqn{\times} SE)
  \item \strong{Mixed-effects models}: Wald intervals
}
Falls back to Wald on profiling failure.

\strong{Mixed Effects Models:}
For \pkg{lme4} models (glmer, lmer), the function extracts fixed effects
only. Random effects variance components are not included in the output
table, as they represent clustering structure rather than predictor effects.
}
\examples{
# Load example data
data(clintrial)

# Example 1: Extract from logistic regression
glm_model <- glm(os_status ~ age + sex + treatment, 
                 data = clintrial, family = binomial)

glm_result <- m2dt(clintrial, glm_model)
glm_result

\donttest{

# Example 2: Extract from linear model
lm_model <- lm(los_days ~ age + sex + surgery, data = clintrial)

lm_result <- m2dt(clintrial, lm_model)
lm_result

# Example 3: Cox proportional hazards model
library(survival)
cox_model <- coxph(Surv(os_months, os_status) ~ age + sex + stage,
                   data = clintrial)

cox_result <- m2dt(clintrial, cox_model)
cox_result

# Example 4: Exclude intercept for cleaner tables
clean_result <- m2dt(clintrial, glm_model, include_intercept = FALSE)
clean_result

# Example 5: Change confidence level
result_90ci <- m2dt(clintrial, glm_model, conf_level = 0.90)
result_90ci

}

}
\seealso{
\code{\link{fit}} for the main regression interface,
\code{\link{glmforest}}, \code{\link{coxforest}}, \code{\link{lmforest}} for
forest plot visualization
}
\concept{utility functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_pvalues_export_html}
\alias{format_pvalues_export_html}
\title{Format \emph{p}-values for exported tables (HTML)}
\usage{
format_pvalues_export_html(df, p_threshold = 0.05)
}
\arguments{
\item{df}{Data.table containing \emph{p}-value columns.}

\item{p_threshold}{Numeric threshold for significance (default 0.05).}
}
\value{
Data.table with significant \emph{p}-values wrapped in HTML bold tags.
}
\description{
Applies bold formatting to significant \emph{p}-values in HTML tables using
the b tag.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_model_nobs}
\alias{get_model_nobs}
\title{Number of observations used to fit a model}
\usage{
get_model_nobs(model, model_class)
}
\arguments{
\item{model}{Fitted model object.}

\item{model_class}{Character string of the model class.}
}
\value{
Numeric scalar giving the number of observations used in fitting, or
  \code{NA_real_} when it cannot be determined.
}
\description{
Provides a single accessor for the fitted sample size across the model
classes supported by \pkg{summata}. Survival models record this quantity in
class-specific components rather than through \code{stats::nobs()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{get_model_analysis_counts}
\alias{get_model_analysis_counts}
\title{Count the observations available to a fitted model}
\usage{
get_model_analysis_counts(model, model_class, data)
}
\arguments{
\item{model}{Fitted model object.}

\item{model_class}{Character string of the model class.}

\item{data}{Data frame or data.table supplied to the model call.}
}
\value{
See \code{get_analysis_counts()}.
}
\description{
Convenience wrapper around \code{get_analysis_counts()} that derives the
response and predictor variables from the model itself.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_column_headers_with_n_html}
\alias{format_column_headers_with_n_html}
\title{Format column headers with n counts (HTML)}
\usage{
format_column_headers_with_n_html(col_names, n_row_data)
}
\arguments{
\item{col_names}{Character vector of column names.}

\item{n_row_data}{Named list or data.table row with n values for each column.}
}
\value{
Character vector with HTML-formatted headers including n counts.
}
\description{
Creates HTML-formatted column headers with sample size counts displayed
below the column name using line breaks.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{calculate_coxme_scores}
\alias{calculate_coxme_scores}
\title{Calculate scores for mixed-effects Cox models}
\usage{
calculate_coxme_scores(comparison, weights, scores, n_models)
}
\arguments{
\item{comparison}{Data.table with model comparison metrics.}

\item{weights}{Named list of scoring weights.}

\item{scores}{List of initialized score vectors.}

\item{n_models}{Integer number of models being compared.}
}
\value{
Updated scores list with calculated values and total.
}
\description{
Computes component scores for coxme models based on concordance,
pseudo-R-squared, and ICC metrics.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multifit.R
\name{extract_predictor_effects}
\alias{extract_predictor_effects}
\title{Extract predictor effects from a fitted model}
\usage{
extract_predictor_effects(
  model,
  predictor,
  outcome,
  conf_level = 0.95,
  adjusted = FALSE,
  terms_to_extract = NULL,
  conf_method = NULL
)
}
\arguments{
\item{model}{Fitted model object.}

\item{predictor}{Character string of the predictor variable name.}

\item{outcome}{Character string of the outcome variable name.}

\item{conf_level}{Numeric confidence level.}

\item{adjusted}{Logical indicating if this is an adjusted model.}

\item{terms_to_extract}{Character vector of terms to extract (predictor and 
optionally interaction terms involving the predictor).}
}
\value{
data.table with predictor effect information.
}
\description{
Internal helper function that extracts only the predictor variable's 
coefficient(s) from a fitted model, ignoring intercept and covariates.
Supports standard models (glm, lm, coxph) and mixed effects models
(glmer, lmer, coxme).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{extract_lmer_metrics}
\alias{extract_lmer_metrics}
\title{Extract metrics for linear mixed-effects models (lmer)}
\usage{
extract_lmer_metrics(model, raw_data, metrics)
}
\arguments{
\item{model}{Fitted lmerMod object from lme4.}

\item{raw_data}{Data.table with raw model information.}

\item{metrics}{Named list of initialized metrics to populate.}
}
\value{
Updated metrics list with lmer-specific values.
}
\description{
Extracts quality metrics specific to linear mixed-effects models including
R-squared measures, ICC, and global significance tests.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{detect_model_type}
\alias{detect_model_type}
\title{Detect if model is univariable or multivariable}
\usage{
detect_model_type(model)
}
\arguments{
\item{model}{Fitted model object.}
}
\value{
Character string: "Univariable" or "Multivariable".
}
\description{
Determines whether a model contains one predictor (univariable) or multiple
predictors (multivariable) by analyzing coefficient names and factor structure.
Handles interactions and random effects appropriately.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/m2dt_utils.R
\name{parse_term}
\alias{parse_term}
\title{Parse term into variable and group}
\usage{
parse_term(terms, xlevels = NULL, model = NULL)
}
\arguments{
\item{terms}{Character vector of coefficient term names.}

\item{xlevels}{Named list of factor levels from the model.}

\item{model}{Optional model object for extracting factor info from coxme models.}
}
\value{
Data.table with 'variable' and 'group' columns.
}
\description{
Splits coefficient term names into base variable names and factor levels.
For example, "sexMale" becomes variable="sex" and group="Male".
Handles interaction terms and continuous variables appropriately.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2_utils.R
\name{format_indented_groups}
\alias{format_indented_groups}
\title{Apply formatting to indented groups}
\usage{
format_indented_groups(df, indent_string = "    ")
}
\arguments{
\item{df}{Data.table with Variable and Group columns.}

\item{indent_string}{Character string to use for indentation.}
}
\value{
Data.table with Group column removed and levels indented under Variables.
}
\description{
Transforms tables with Variable/Group columns into indented format where
group levels appear as indented rows under variable names. Handles both
regression and descriptive tables with appropriate \emph{p}-value placement.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comp_utils.R
\name{build_failed_model_row}
\alias{build_failed_model_row}
\title{Build row for failed model}
\usage{
build_failed_model_row(model_name, n, n_predictors, model_type)
}
\arguments{
\item{model_name}{Character string name of the model.}

\item{n}{Integer sample size.}

\item{n_predictors}{Integer number of predictors attempted.}

\item{model_type}{Character string indicating model type.}
}
\value{
Data.table with single row of NA metrics and "Failed" convergence.
}
\description{
Creates a comparison table row with appropriate NA values for a model
that failed to fit.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table2pptx.R
\name{table2pptx}
\alias{table2pptx}
\title{Export Table to Microsoft PowerPoint Format (PPTX)}
\usage{
table2pptx(
  table,
  file,
  caption = NULL,
  font_size = 10,
  font_family = "Arial",
  format_headers = TRUE,
  bold_significant = TRUE,
  bold_variables = FALSE,
  p_threshold = 0.05,
  indent_groups = FALSE,
  condense_table = FALSE,
  condense_quantitative = FALSE,
  zebra_stripes = FALSE,
  dark_header = FALSE,
  width = NULL,
  align = NULL,
  template = NULL,
  layout = "Title and Content",
  master = "Office Theme",
  left = 0.5,
  top = 1.5,
  return_ft = FALSE,
  quiet = FALSE,
  ...
)
}
\arguments{
\item{table}{Data frame, data.table, or matrix to export. Can be output from 
\code{desctable()}, \code{survtable()}, \code{fit()}, \code{uniscreen()},
\code{fullfit()}, \code{compfit()}, \code{multifit()}, or any tabular data.}

\item{file}{Character string specifying the output PPTX filename. Must have 
\code{.pptx} extension. Example: \code{"results.pptx"}, \code{"Slide1.pptx"}.}

\item{caption}{Character string. Optional title displayed in the slide's title 
placeholder or as text box above the table. Default is \code{NULL}.}

\item{font_size}{Numeric. Base font size in points for table content. 
Default is 10. Typical range for presentations: 10-14 points. Larger than 
print documents for visibility at distance.}

\item{font_family}{Character string. Font family name for the table. Must be 
installed on the system. Default is \code{"Arial"}. Common presentation 
fonts: \code{"Calibri"}, \code{"Helvetica"}, \code{"Arial"}.}

\item{format_headers}{Logical. If \code{TRUE}, formats column headers by 
italicizing statistical notation ("\emph{n}", "\emph{p}") and improving readability. 
Default is \code{TRUE}.}

\item{bold_significant}{Logical. If \code{TRUE}, applies bold formatting to 
\emph{p}-values below the significance threshold. Makes important results stand 
out in presentations. Default is \code{TRUE}.}

\item{bold_variables}{Logical. If \code{TRUE}, variable names are displayed
in bold. Default is \code{FALSE}.}

\item{p_threshold}{Numeric. Threshold for bold \emph{p}-value formatting. Only 
used when \code{bold_significant = TRUE}. Default is 0.05.}

\item{indent_groups}{Logical. If \code{TRUE}, indents factor levels under 
their parent variable, creating hierarchical display. Useful for categorical 
variables. Default is \code{FALSE}.}

\item{condense_table}{Logical. If \code{TRUE}, condenses table to essential 
rows only. Automatically sets \code{indent_groups = TRUE}. Crucial for 
fitting content on slides. Default is \code{FALSE}.}

\item{condense_quantitative}{Logical. If \code{TRUE}, condenses continuous 
and survival variables into single rows while preserving all categorical 
variable rows (including binary). Only applies to descriptive tables from 
\code{desctable()}. Automatically sets \code{indent_groups = TRUE}. Unlike 
\code{condense_table}, this does not collapse binary categorical variables. 
Default is \code{FALSE}.}

\item{zebra_stripes}{Logical. If \code{TRUE}, applies alternating row shading 
to different variables for visual grouping. Improves readability during 
presentations. Default is \code{FALSE}.}

\item{dark_header}{Logical. If \code{TRUE}, creates dark background with 
light text for header row. Provides strong contrast visible from distance. 
Default is \code{FALSE}.}

\item{width}{Numeric. Table width in inches. If \code{NULL} (default), 
auto-fits to slide width (approximately 9 inches for standard 10-inch slide 
with margins). Specify for exact control.}

\item{align}{Character vector specifying column alignment. Options: 
\code{"left"}, \code{"center"}, or \code{"right"}. If \code{NULL} (default), 
automatically determines based on content.}

\item{template}{Character string. Path to custom PPTX template file. If 
\code{NULL} (default), uses officer's default blank template. Use to match 
corporate branding or conference themes.}

\item{layout}{Character string. Name of slide layout to use from template. 
Default is \code{"Title and Content"}. Common layouts:
\itemize{
  \item \code{"Title and Content"} - Standard layout [default]
  \item \code{"Blank"} - Empty slide for maximum control
  \item \code{"Title Only"} - Title area only
  \item \code{"Two Content"} - Title with two content areas
}}

\item{master}{Character string. Name of slide master to use. Default is 
\code{"Office Theme"}. Varies by template. Check template for available masters.}

\item{left}{Numeric. Horizontal position from left edge of slide in inches. 
Default is 0.5. Standard slide is 10 inches wide.}

\item{top}{Numeric. Vertical position from top edge of slide in inches. 
Default is 1.5 (leaves room for title). Standard slide is 7.5 inches tall. 
Adjust based on table size and layout.}

\item{return_ft}{Logical. If \code{TRUE}, returns the \pkg{flextable} object 
directly for further customization. If \code{FALSE} (default), returns 
invisibly with \pkg{flextable} object as attribute. See Details for usage. 
Default is \code{FALSE}.}

\item{quiet}{Logical. Suppress progress and confirmation messages, such
as the notice reporting the file written. Errors and warnings are
unaffected. Default is \code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[officer]{read_pptx}}.}
}
\value{
Behavior depends on \code{return_ft}:
  \describe{
    \item{\code{return_ft = FALSE}}{Invisibly returns a list with:
      \itemize{
        \item \code{file} - Path to created file
        \item \code{caption} - Caption/title text
        \item \code{layout} - Layout name used
        \item \code{master} - Master name used
        \item \code{template} - Template path (if provided)
        \item \code{position} - List with \code{left} and \code{top} coordinates
      }
      Flextable accessible via \code{attr(result, "flextable")}
    }
    \item{\code{return_ft = TRUE}}{Directly returns the \pkg{flextable} object}
  }
  
  Always creates a \code{.pptx} file at the specified location.
}
\description{
Converts a data frame, data.table, or matrix to a Microsoft PowerPoint slide 
(\code{.pptx}) with a formatted table using the \pkg{flextable} and \pkg{officer}
packages. Creates presentation-ready slides with extensive control over table
formatting, positioning, and layout. Tables can be further edited in PowerPoint
after creation. Ideal for creating data-driven presentations and conference talks.
}
\details{
\strong{Package Requirements:}

Requires:
\itemize{
  \item \strong{\pkg{flextable}} - Table creation and formatting
  \item \strong{\pkg{officer}} - PowerPoint manipulation
}

Install: \code{install.packages(c("flextable", "officer"))}

\strong{Slide Dimensions:}

Standard PowerPoint slide:
\itemize{
  \item Width: 10 inches (25.4 cm)
  \item Height: 7.5 inches (19.05 cm)
  \item Aspect ratio: 4:3 (standard) or 16:9 (widescreen)
}

Safe content area (with margins):
\itemize{
  \item Width: ~9 inches
  \item Height: ~6 inches (accounting for title)
}

\strong{Positioning:}

The \code{left} and \code{top} parameters control table placement:
\itemize{
  \item (0, 0) = Top-left corner of slide
  \item Default (0.5, 1.5) = Standard position with title room
  \item Center: \code{left = (10 - table_width) / 2}
}

When caption is provided:
\itemize{
  \item Attempts to use title placeholder (if layout supports)
  \item Falls back to text box above table
  \item Automatically adjusts table position downward
}

\strong{Slide Layouts:}

Different layouts serve different purposes:

\emph{Title and Content (default):}
\itemize{
  \item Has title and content placeholders
  \item Caption goes in title area
  \item Table in content area
  \item Most common for data slides
}

\emph{Blank:}
\itemize{
  \item No predefined areas
  \item Maximum flexibility
  \item Use absolute positioning (\code{left}, \code{top})
  \item Good for custom layouts
}

\emph{Title-Only:}
\itemize{
  \item Title area only
  \item Large space for table
  \item Good for data-heavy slides
}

\strong{Custom Templates:}

Use organizational or conference templates:
\preformatted{
table2pptx(table, "branded.pptx",
         template = "company_template.pptx",
         layout = "Content Layout",  # Name from template
         master = "Company Theme")   # Name from template
}

To find layout and master names in template:
\preformatted{
pres <- officer::read_pptx("template.pptx")
officer::layout_summary(pres)
}

\strong{Multiple Slides:}

Creating presentations with multiple tables:
\preformatted{
# Each call creates new presentation - combine after
table2pptx(table1, "slide1.pptx", caption = "Results Part 1")
table2pptx(table2, "slide2.pptx", caption = "Results Part 2")

# Then manually combine in PowerPoint, or:
# Use officer to create multi-slide presentation
pres <- officer::read_pptx()

# Add first table
ft1 <- table2pptx(table1, "temp1.pptx", return_ft = TRUE)
pres <- officer::add_slide(pres)
pres <- officer::ph_with(pres, ft1, 
                         location = officer::ph_location(left = 0.5, top = 1.5))

# Add second table
ft2 <- table2pptx(table2, "temp2.pptx", return_ft = TRUE)
pres <- officer::add_slide(pres)
pres <- officer::ph_with(pres, ft2,
                         location = officer::ph_location(left = 0.5, top = 1.5))

print(pres, target = "combined.pptx")
}

\strong{Further Customization:}

Access the \code{flextable} object for advanced formatting:
\preformatted{
ft <- table2pptx(table, "base.pptx", return_ft = TRUE)

# Customize
ft <- flextable::color(ft, j = "p-value", color = "red")
ft <- flextable::bg(ft, i = 1, bg = "yellow")
ft <- flextable::bold(ft, i = ~ estimate > 0, j = "estimate")

# Save to new slide
pres <- officer::read_pptx()
pres <- officer::add_slide(pres)
pres <- officer::ph_with(pres, ft, 
                         location = officer::ph_location(left = 0.5, top = 1.5))
print(pres, target = "custom.pptx")
}
}
\examples{
# Create example data
data(clintrial)
data(clintrial_labels)
tbl <- desctable(clintrial, by = "treatment",
    variables = c("age", "sex"), labels = clintrial_labels)

# Basic PowerPoint export
if (requireNamespace("flextable", quietly = TRUE) &&
    requireNamespace("officer", quietly = TRUE)) {
  table2pptx(tbl, file.path(tempdir(), "example.pptx"))
}

\donttest{
old_width <- options(width = 180)
# Load data
data(clintrial)
data(clintrial_labels)

# Create regression table
results <- fit(
   data = clintrial,
   outcome = "os_status",
   predictors = c("age", "sex", "treatment"),
   labels = clintrial_labels
)

# Example 1: Basic PowerPoint slide
table2pptx(results, file.path(tempdir(), "results.pptx"))

# Example 2: With title
table2pptx(results, file.path(tempdir(), "titled.pptx"),
        caption = "Multivariable Regression Results")

# Example 3: Larger font for visibility
table2pptx(results, file.path(tempdir(), "large_font.pptx"),
        font_size = 12,
        caption = "Main Findings")

# Example 4: Condensed for slide space
table2pptx(results, file.path(tempdir(), "condensed.pptx"),
        condense_table = TRUE,
        caption = "Key Results")

# Example 5: Dark header for emphasis
table2pptx(results, file.path(tempdir(), "dark.pptx"),
        dark_header = TRUE,
        caption = "Risk Factors")

# Example 6: With zebra stripes
table2pptx(results, file.path(tempdir(), "striped.pptx"),
        zebra_stripes = TRUE)

# Example 7: Blank layout with custom positioning
table2pptx(results, file.path(tempdir(), "blank.pptx"),
        layout = "Blank",
        left = 1,
        top = 1.5,
        width = 8)

# Example 8: Get flextable for customization
ft <- table2pptx(results, file.path(tempdir(), "base.pptx"), return_ft = TRUE)

# Customize the returned flextable object
ft <- flextable::color(ft, j = "p-value", color = "darkred")

# Example 9: Presentation-optimized table
table2pptx(results, file.path(tempdir(), "presentation.pptx"),
        caption = "Main Analysis Results",
        font_size = 11,
        condense_table = TRUE,
        zebra_stripes = TRUE,
        dark_header = TRUE,
        bold_significant = TRUE)

# Example 10: Descriptive statistics slide
desc <- desctable(
   data = clintrial,
   by = "treatment",
   variables = c("age", "sex", "bmi"),
   labels = clintrial_labels
)

table2pptx(desc, file.path(tempdir(), "baseline.pptx"),
        caption = "Baseline Characteristics",
        font_size = 10)

# Example 11: Conference presentation style
table2pptx(results, file.path(tempdir(), "conference.pptx"),
        caption = "Study Outcomes",
        font_family = "Calibri",
        font_size = 14,  # Large for big rooms
        dark_header = TRUE,
        condense_table = TRUE)

options(old_width)
}
}
\seealso{
\code{\link{tablesave}} for saving in the format given by the file extension,
\code{\link{table2docx}} for Word documents,
\code{\link{table2pdf}} for PDF output,
\code{\link{table2html}} for HTML tables,
\code{\link{table2rtf}} for Rich Text Format,
\code{\link{table2tex}} for LaTeX output,
\code{\link[flextable]{flextable}} for table customization,
\code{\link[officer]{read_pptx}} for PowerPoint manipulation

Other export functions:
\code{\link[=table2docx]{table2docx()}},
\code{\link[=table2html]{table2html()}},
\code{\link[=table2pdf]{table2pdf()}},
\code{\link[=table2rtf]{table2rtf()}},
\code{\link[=table2tex]{table2tex()}},
\code{\link[=tablesave]{tablesave()}}
}
\concept{export functions}
